eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define */\n\n/**\n * Defines hooks to assist with module initialization.\n *\n * This module defines 2 methods for client modules to attach callbacks:\n *    - htmlReady - When the main application template is rendered\n *    - appReady - When Brackets completes loading all modules and extensions\n *\n * These are *not* jQuery events. Each method is similar to $(document).ready\n * in that it will call the handler immediately if brackets is already done\n * loading.\n */\ndefine(\'utils/AppInit\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n    \n    // Fires when the index.html is loaded\n    var HTML_READY  = "htmlReady";\n\n    // Fires when all DOM (document object model) is called\n    var DOM_READY = "domReady";\n\n    // Fires when all extensions are loaded\n    var APP_READY   = "appReady";\n\n    var status      = { HTML_READY : false, DOM_READY : false, APP_READY : false },\n        callbacks   = {};\n\n    callbacks[HTML_READY]       = [];\n    callbacks[DOM_READY] = [];\n    callbacks[APP_READY]        = [];\n\n    function _callHandler(handler) {\n        try {\n            // TODO (issue 1034): We *could* use a $.Deferred for this, except deferred objects enter a broken\n            // state if any resolution callback throws an exception. Since third parties (e.g. extensions) may\n            // add callbacks to this, we need to be robust to exceptions\n            handler();\n        } catch (e) {\n            console.error("Exception when calling a \'brackets done loading\' handler:");\n            console.log(e.stack);\n        }\n    }\n\n    function _dispatchReady(type) {\n        var i,\n            myHandlers = callbacks[type];\n\n        // mark this status complete\n        status[type] = true;\n\n        for (i = 0; i < myHandlers.length; i++) {\n            _callHandler(myHandlers[i]);\n        }\n\n        // clear all callbacks after being called\n        callbacks[type] = [];\n    }\n\n    function _addListener(type, callback) {\n        if (status[type]) {\n            _callHandler(callback);\n        } else {\n            callbacks[type].push(callback);\n        }\n    }\n\n    /**\n     * Adds a callback for the htmlReady hook. Handlers are called after the\n     * index.html is load.\n     * @param {function} handler\n     */\n    function htmlReady(callback) {\n        _addListener(HTML_READY, callback);\n    }\n\n    /**\n     * Adds a callback for the domReady hook. Handlers are called after\n     * all the dynamic DOM construction is completed.\n     * @param {function} handler\n     */\n    function domReady(callback) {\n        _addListener(DOM_READY, callback);        \n    }    \n\n    /**\n     * Adds a callback for the ready hook. Handlers are called after\n     * htmlReady is done, the initial project is loaded, and all extensions are\n     * loaded.\n     * @param {function} handler\n     */\n    function appReady(callback) {\n        _addListener(APP_READY, callback);\n    }\n\n    exports.htmlReady = htmlReady;\n    exports.domReady  = domReady;\n    exports.appReady  = appReady;\n    \n    exports.HTML_READY = HTML_READY;\n    exports.DOM_READY  = DOM_READY;\n    exports.APP_READY  = APP_READY;\n\n    // internal use only\n    exports._dispatchReady = _dispatchReady;\n});\n\n//# sourceURL=/utils/AppInit.js'),eval("/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define*/\n\n/**\n * Global object.\n *\n * To get a global object, see the following code.\n *\n * ```\n * var global = app.getModule('core/Global').global;\n * ```\n */\ndefine('core/Global',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n\n    /*\n     * Define core app namespace if it isn't already defined\n     *\n     * We can't simply do 'app = {}' to define it in the global namespace because\n     * we're in \"use strict\" mode. Most likely, 'window' will always point to the global\n     * object when this code is running. However, in case it isn't (e.g. if we're running\n     * inside Node for CI testing) we use this trick to get the global object.\n     *\n     * Taken from:\n     *   http://stackoverflow.com/questions/3277182/how-to-get-the-global-object-in-javascript\n     *\n     * @private\n     */\n    var Fn = Function, _global = (new Fn(\"return this\"))();\n\n    // for meta object\n    _global.meta = {};\n\n    // for types\n    _global.type = {};\n\n    // for rules\n    _global.rules = {};\n\n    /**\n     * Array Extension\n     */\n\n    Array.prototype.indexOf = function (item) {\n        for (var i = 0, l = this.length; i < l; i++) {\n            if (i in this && this[i] === item) return i;\n        }\n        return -1;\n    };\n\n    Array.prototype.clear = function () {\n        this.length = 0;\n    };\n\n    Array.prototype.add = function (item) {\n        return this.push(item);\n    };\n\n    Array.prototype.remove = function (item) {\n        var index = this.indexOf(item);\n        if (index > -1) {\n            this.splice(index, 1);\n        }\n    };\n\n    Array.prototype.removeAt = function (index) {\n        return this.splice(index, 1);\n    };\n\n    Array.prototype.insert = function (index, item) {\n        return this.splice(index, 0, item);\n    };\n\n    Array.prototype.contains = function (item) {\n        return this.indexOf(item) > -1;\n    };\n\n    /**\n     * Math Extension\n     */\n\n    Math.square = function (x) {\n        return x * x;\n    };\n\n    Math.trunc = function (n) {\n        return ~~n;\n    };\n\n    exports.global = _global;\n\n});\n\n\n//# sourceURL=/core/Global.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n// TODO: KendoUI를 써서도 동적인 UI 개발이 가능하도록 할 수 있을듯. (아래 참고)\n/*\n * (플러그인 개발자는 KendoUI를 몰라도 가능해야 한다. HTML/CSS 수준으로 UI 확장 가능하도록 해야 함.)\n *\n * 1. 플러그인은 직접 HTML 코드를 삽입한다. CSS class를 지정한다.\n * 2. htmlReady 시에, jQuery selector를 사용해서 특정 CSS class를 모두 선택하여 kendoUI 위젯화 시킨다.\n *      e.g.) $(".ui-dropdown").kendoDropdown(..)\n * 3. 코드에서 이벤트를 받거나 하고 싶으면 id 값을 이용하도록 한다.\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global require, define, $, _, document, window, navigator, appshell, localStorage, Mustache */\n\ndefine(\'utils/Initializer\',[\'require\',\'exports\',\'module\',\'core/Global\',\'text!config.json\'],function (require, exports, module) {\n    "use strict";\n\n    var _global    = require("core/Global").global,\n        configJSON = require("text!config.json");\n\n    // Initialize global namespace\n    if (!_global.app) {\n        _global.app = {};\n        _global.staruml = _global.app; // Delete this in the future\n    }\n\n    // Determine OS/platform\n    if (_global.navigator.platform === "MacIntel" || _global.navigator.platform === "MacPPC") {\n        _global.app.platform = "mac";\n    } else if (_global.navigator.platform.indexOf("Linux") >= 0) {\n        _global.app.platform = "linux";\n    } else {\n        _global.app.platform = "win";\n    }\n\n    // Load config.json - Parse src/config.json\n    try {\n        _global.app.metadata = JSON.parse(configJSON);\n        _global.app.config = _global.app.metadata.config;\n    } catch (err) {\n        console.log(err);\n    }\n\n    // Are we in a desktop shell with a native menu bar?\n    _global.app.inBrowser = (!appshell);\n    _global.app.nativeMenus = (!_global.app.inBrowser && (_global.app.platform !== "linux"));\n\n    // Locale-related APIs\n    _global.app.isLocaleDefault = function () {\n        return !_global.localStorage.getItem("locale");\n    };\n\n    _global.app.getLocale = function () {\n        // By default use the locale that was determined in brackets.js\n        return _global.localStorage.getItem("locale") || _global.require.s.contexts._.config.locale;\n    };\n\n    _global.app.setLocale = function (locale) {\n        if (locale) {\n            _global.localStorage.setItem("locale", locale);\n        } else {\n            _global.localStorage.removeItem("locale");\n        }\n    };\n\n    // Require module for using in extension\n    _global.app.libRequire = _global.require;\n\n    // Require for loading modules in extensions\n    _global.app.getModule = require;\n\n    // App name\n    _global.app.name = \'staruml\';\n\n});\n\n//# sourceURL=/utils/Initializer.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, loopfunc: true */\n/*global define */\n\ndefine(\'core/IdGenerator\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n    /**\n     * Counter for IdGenerator\n     * @private\n     */\n    var idGeneratorCounter = Math.floor(Math.random() * 65536);\n\n    /**\n     * IdGenerator\n     * @private\n     */\n    var _idGenerator = null;\n\n    /**\n     * Table for base64\n     * @onst\n     */\n    var tableStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";\n    var table = tableStr.split("");\n\n    /**\n     * atob\n     * @private\n     */\n    function atob(base64) {\n        if (/(=[^=]+|={3,})$/.test(base64)) throw new Error("String contains an invalid character");\n        base64 = base64.replace(/=/g, "");\n        var n = base64.length & 3;\n        if (n === 1) throw new Error("String contains an invalid character");\n        for (var i = 0, j = 0, len = base64.length / 4, bin = []; i < len; ++i) {\n            var a = tableStr.indexOf(base64[j++] || "A"),\n                b = tableStr.indexOf(base64[j++] || "A");\n            var c = tableStr.indexOf(base64[j++] || "A"),\n                d = tableStr.indexOf(base64[j++] || "A");\n            if ((a | b | c | d) < 0) throw new Error("String contains an invalid character");\n            bin[bin.length] = ((a << 2) | (b >> 4)) & 255;\n            bin[bin.length] = ((b << 4) | (c >> 2)) & 255;\n            bin[bin.length] = ((c << 6) | d) & 255;\n        }\n        return String.fromCharCode.apply(null, bin).substr(0, bin.length + n - 4);\n    }\n\n    /**\n     * btoa\n     * @private\n     */\n    function btoa(bin) {\n        for (var i = 0, j = 0, len = bin.length / 3, base64 = []; i < len; ++i) {\n            var a = bin.charCodeAt(j++),\n                b = bin.charCodeAt(j++),\n                c = bin.charCodeAt(j++);\n            if ((a | b | c) > 255) throw new Error("String contains an invalid character");\n            base64[base64.length] = table[a >> 2] + table[((a << 4) & 63) | (b >> 4)] +\n                (isNaN(b) ? "=" : table[((b << 2) & 63) | (c >> 6)]) +\n                (isNaN(b + c) ? "=" : table[c & 63]);\n        }\n        return base64.join("");\n    }\n\n    /**\n     * hexToBase64\n     * @private\n     */\n    function hexToBase64(str) {\n        return btoa(String.fromCharCode.apply(null,\n            str.replace(/\\r|\\n/g, "").replace(/([\\da-fA-F]{2}) ?/g, "0x$1 ").replace(/ +$/, "").split(" ")));\n    }\n\n    /**\n     * base64ToHex\n     * @private\n     */\n    function base64ToHex(str) {\n        for (var i = 0, bin = atob(str.replace(/[ \\r\\n]+$/, "")), hex = []; i < bin.length; ++i) {\n            var tmp = bin.charCodeAt(i).toString(16);\n            if (tmp.length === 1) tmp = "0" + tmp;\n            hex[hex.length] = tmp;\n        }\n        return hex.join(" ");\n    }\n\n    /**\n     * IdGenerator\n     *\n     * @constructor\n     */\n    function IdGenerator() {\n    }\n\n    IdGenerator.prototype.setBaseHex = function (val) {\n        this.baseHex = val;\n    };\n\n    IdGenerator.prototype.toHex = function (digit, num) {\n        var fill, hex, r;\n        hex = num.toString(16);\n        if (hex.length < digit) {\n            r = digit - hex.length;\n            fill = new Array(r + 1).join("0");\n            hex = fill + hex;\n        }\n        return hex;\n    };\n\n    IdGenerator.prototype.generate = function () {\n        var base64, counter, counterHex, hex, random, randomHex, timestamp, timestampHex;\n        timestamp = (new Date()).getTime();\n        counter = idGeneratorCounter;\n        idGeneratorCounter++;\n        if (idGeneratorCounter > 65535) {\n            idGeneratorCounter = 0;\n        }\n        random = Math.floor(Math.random() * 65536);\n        timestampHex = this.toHex(16, timestamp);\n        counterHex = this.toHex(4, counter);\n        randomHex = this.toHex(4, random);\n        hex = this.baseHex + timestampHex + counterHex + randomHex;\n        base64 = hexToBase64(hex);\n        return base64;\n    };\n\n    /**\n     * Set GUID Base\n     */\n    function setGuidBase(baseHex) {\n        _idGenerator = new IdGenerator();\n        _idGenerator.setBaseHex(baseHex);\n    }\n\n    /**\n     * Set GUID Base as Number\n     */\n    function setGuidBaseNum(baseNum) {\n        _idGenerator = new IdGenerator();\n        _idGenerator.setBaseHex(_idGenerator.toHex(24, baseNum));\n    }\n\n    /**\n     * Generate GUID\n     */\n    function generateGuid() {\n        if (_idGenerator === null) {\n            _idGenerator = new IdGenerator("");\n        }\n        return _idGenerator.generate();\n    }\n\n    // Public API\n    exports.setGuidBase    = setGuidBase;\n    exports.setGuidBaseNum = setGuidBaseNum;\n    exports.generateGuid   = generateGuid;\n\n});\n\n\n//# sourceURL=/core/IdGenerator.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, _*/\n\ndefine(\'core/Graphics\',[\'require\',\'exports\',\'module\',\'core/Global\'],function (require, exports, module) {\n    "use strict";\n\n    var _global = require("core/Global").global;\n\n    /**\n     * Default Color Palette\n     */\n    var DEFAULT_COLOR_PALETTE = [\n        "#d8f2ff", "#d8e5ff", "#d8d8ff", "#ebd8ff", "#ffd8ff", "#ffd8eb", "#ffd8d8", "#ffe2d8", "#ffebd8", "#fff5d8", "#ffffd8", "#ebffd8", "#d8ffd8", "#d8ffeb", "#d8fff8", "#ffffff",\n        "#a3e0ff", "#adc8ff", "#adadff", "#d6adff", "#ffa3ff", "#ffadd6", "#ffadad", "#ffc1ad", "#ffd6ad", "#ffeaad", "#ffff91", "#d1ffa3", "#a3ffa3", "#a3ffd1", "#91ffec", "#e2e2e2",\n        "#6dceff", "#82abff", "#8282ff", "#c082ff", "#ff6dff", "#ff82c0", "#ff8282", "#ffa182", "#ffc082", "#ffdf82", "#ffff49", "#b6ff6d", "#6dff6d", "#6dffb6", "#49ffe0", "#c6c6c6",\n        "#38bcff", "#568eff", "#5656ff", "#aa56ff", "#ff38ff", "#ff56aa", "#ff5656", "#ff8056", "#ffaa56", "#ffd456", "#ffff00", "#9bff38", "#38ff38", "#38ff9b", "#00ffd4", "#aaaaaa",\n        "#00a9ff", "#2b71ff", "#2b2bff", "#952bff", "#ff00ff", "#ff2b95", "#ff2b2b", "#ff602b", "#ff952b", "#ffca2b", "#d8d800", "#7fff00", "#00ff00", "#00ff7f", "#00d8b4", "#8e8e8e",\n        "#008ed6", "#0055ff", "#0000ff", "#7f00ff", "#d600d6", "#ff007f", "#ff0000", "#ff3f00", "#ff7f00", "#ffbf00", "#b7b700", "#6cd800", "#00d600", "#00d369", "#00b799", "#707070",\n        "#0073ad", "#0044cc", "#0000d1", "#6600cc", "#ad00ad", "#cc0066", "#d10000", "#d63500", "#d16800", "#cc9900", "#969600", "#59b200", "#00ad00", "#00a854", "#00967d", "#545454",\n        "#005884", "#003399", "#0000a0", "#4c0099", "#840084", "#99004c", "#a00000", "#a82a00", "#a05000", "#997200", "#757500", "#468c00", "#008400", "#007c3e", "#007561", "#383838",\n        "#003d5b", "#002266", "#000070", "#330066", "#5b005b", "#660033", "#700000", "#7a1e00", "#703800", "#664c00", "#545400", "#336600", "#005b00", "#005128", "#005446", "#1c1c1c",\n        "#002133", "#001133", "#00003f", "#190033", "#330033", "#330019", "#3f0000", "#4c1300", "#3f1f00", "#332600", "#333300", "#1f3f00", "#003300", "#002613", "#00332a", "#000000"\n    ];\n\n    /**\n     * Alignment Kind\n     * @enum {number}\n     */\n    var AL_LEFT   = 0,\n        AL_RIGHT  = 1,\n        AL_CENTER = 2,\n        AL_TOP    = 3,\n        AL_BOTTOM = 4,\n        AL_MIDDLE = 5;\n\n    /**\n     * Font Styles\n     * @const {number}\n     */\n    var FS_NORMAL      = 0,\n        FS_BOLD        = 1,\n        FS_ITALIC      = 2,\n        FS_BOLD_ITALIC = 3;\n\n    /**\n     * Degree 값을 Radian 값으로 변환\n     * @param {number} degree\n     * @return {number} - radian\n     */\n    function toRadian(degree) {\n        return degree * Math.PI / 180;\n    }\n\n    /**\n     * Radian 값을 Degree 값으로 변환\n     * @param {number} radin\n     * @return {number} - degree\n     */\n    function toDegree(radian) {\n        return radian * (180 / Math.PI);\n    }\n\n\n    /**\n     * Point\n     * @constructor\n     */\n    function Point(x, y) {\n\n        /** @member {number} */\n        this.x = x;\n\n        /** @member {number} */\n        this.y = y;\n    }\n\n    /**\n     * Copy Point\n     * @return {Point} self-copied object\n     */\n    Point.prototype.copy = function () {\n        return new Point(this.x, this.y);\n    };\n\n    /**\n     * Add point\n     * @param {Point} p - add p.x to x and p.y to y\n     */\n    Point.prototype.add = function (p) {\n        this.x += p.x;\n        this.y += p.y;\n    };\n\n    /**\n     * Set point\'s coordinate x, y\n     * @param {number} x\n     * @param {number} y\n     */\n    Point.prototype.setPoint = function (x, y) {\n        this.x = x;\n        this.y = y;\n    };\n\n    /**\n     * Set point\'s coordinate x, y of p\n     * @param {Point} p\n     */\n    Point.prototype.setPoint2 = function (p) {\n        this.x = p.x;\n        this.y = p.y;\n    };\n\n    /**\n     * Quantize point\'s coordinate x, y\n     * @param {Point} p\n     */\n    Point.prototype.quantize = function () {\n        this.x = Math.round(this.x);\n        this.y = Math.round(this.y);\n    };\n\n\n    /**\n     * Rect Class\n     * @constructor\n     */\n    function Rect(x1, y1, x2, y2) {\n        this.x1 = x1;\n        this.y1 = y1;\n        this.x2 = x2;\n        this.y2 = y2;\n    }\n\n    /**\n     * Copy Rect\n     * @return {Rect} self-copied rect object\n     */\n    Rect.prototype.copy = function () {\n        return new Rect(this.x1, this.y1, this.x2, this.y2);\n    };\n\n    /**\n     * Add a point to Rect (same as move Rect as p.x and p.y)\n     * @param {Point} p\n     */\n    Rect.prototype.add = function (p) {\n        this.x1 += p.x;\n        this.y1 += p.y;\n        this.x2 += p.x;\n        this.y2 += p.y;\n    };\n\n    /**\n     * Set Rect\'s coordinates\n     * @param {number} x1\n     * @param {number} y1\n     * @param {number} x2\n     * @param {number} y2\n     */\n    Rect.prototype.setRect = function (x1, y1, x2, y2) {\n        this.x1 = x1;\n        this.y1 = y1;\n        this.x2 = x2;\n        this.y2 = y2;\n    };\n\n    /**\n     * Set Rect\'s coordinates from r\n     * @param {Rect} r\n     */\n    Rect.prototype.setRect2 = function (r) {\n        this.x1 = r.x1;\n        this.y1 = r.y1;\n        this.x2 = r.x2;\n        this.y2 = r.y2;\n    };\n\n    /**\n     * Set Rect\'s coordinates\n     * @param {Point} p1\n     * @param {Point} p2\n     */\n    Rect.prototype.setRect3 = function (p1, p2) {\n        this.x1 = p1.x;\n        this.y1 = p1.y;\n        this.x2 = p2.x;\n        this.y2 = p2.y;\n    };\n\n    /**\n     * Expand Rect as delta\n     * @param {number} delta\n     */\n    Rect.prototype.expand = function (delta) {\n        this.x1 = this.x1 - delta;\n        this.y1 = this.y1 - delta;\n        this.x2 = this.x2 + delta;\n        this.y2 = this.y2 + delta;\n    };\n\n    /**\n     * Union r into this rect. (this rect and r should be normalized)\n     * @param {Rect} r\n     */\n    Rect.prototype.union = function (r) {\n        if (r.x1 < this.x1) { this.x1 = r.x1; }\n        if (r.y1 < this.y1) { this.y1 = r.y1; }\n        if (r.x2 > this.x2) { this.x2 = r.x2; }\n        if (r.y2 > this.y2) { this.y2 = r.y2; }\n    };\n\n    /**\n     * Get Center point of Rect\n     * @return {Point}\n     */\n    Rect.prototype.getCenter = function () {\n        return new Point(Math.floor((this.x1 + this.x2) / 2), Math.floor((this.y1 + this.y2) / 2));\n    };\n\n    /**\n     * Get width of Rect\n     * @return {number}\n     */\n    Rect.prototype.getWidth = function () {\n        return Math.abs(this.x2 - this.x1);\n    };\n\n    /**\n     * Get height of Rect\n     * @return {number}\n     */\n    Rect.prototype.getHeight = function () {\n        return Math.abs(this.y2 - this.y1);\n    };\n\n    /**\n     * Get ratio percent : (width / height) * 100\n     * @return {number}\n     */\n    Rect.prototype.getRatioPercent = function () {\n        return (this.getWidth() * 100) / this.getHeight();\n    };\n\n    /**\n     * Quantize coordinates of rect\n     */\n    Rect.prototype.quantize = function () {\n        this.x1 = Math.round(this.x1);\n        this.y1 = Math.round(this.y1);\n        this.x2 = Math.round(this.x2);\n        this.y2 = Math.round(this.y2);\n    };\n\n    /**\n     * Color Object\n     */\n    var Color = {\n        BLACK:      "#000000",\n        WHITE:      "#FFFFFF",\n        DARK_GRAY:  "#404040",\n        LIGHT_GRAY: "#C0C0C0",\n        RED:        "#FF0000",\n        GREEN:      "#00FF00",\n        BLUE:       "#0000FF",\n        CYAN:       "#00FFFF",\n        MAGENTA:    "#FF00FF",\n        ORANGE:     "#FFC800",\n        PINK:       "#FFAFAF",\n        YELLOW:     "#FFFF00"\n    };\n\n\n    _global.meta.Font = {\n        kind: \'datatype\'\n    };\n\n    /**\n     * Font\n     * @constructor\n     */\n    function Font(face, size, style) {\n        this.face = (face ? face : "Arial");\n        this.size = (size ? size : 12);\n        this.style = (style ? style : FS_NORMAL);\n    }\n\n    /**\n     * Set Font\n     * @param {string} face\n     * @param {number} size\n     * @param {number} style\n     */\n    Font.prototype.set = function (face, size, style) {\n        this.face = face;\n        this.size = size;\n        this.style = style;\n    };\n\n    /**\n     * Assign Font\n     * @param {font}\n     */\n    Font.prototype.assign = function (font) {\n        this.set(font.face, font.size, font.style);\n    };\n\n    /**\n     * Copy Font\n     * @return {font} self-copied Font object\n     */\n    Font.prototype.copy = function () {\n        var font = new Font(this.face, this.size, this.style);\n        return font;\n    };\n\n    /**\n     * Define custom write method\n     * - Refer to Core.Writer.prototype.writeCustom\n     */\n    Font.prototype.__write = function () {\n        return this.face + ";" + this.size + ";" + this.style;\n    };\n\n    /**\n     * Define custom read method\n     * - Refer to Core.Reader.prototype.readCustom\n     */\n    Font.prototype.__read = function (value) {\n        var terms;\n        terms = value.split(";");\n        if (terms[0] !== "?") {\n            this.face = terms[0];\n        }\n        if (terms[1] !== "?") {\n            this.size = terms[1] * 1;\n        }\n        if (terms[2] !== "?") {\n            this.style = terms[2] * 1;\n        }\n    };\n\n    /**\n     * ZoomFactor\n     * @constructor\n     */\n    function ZoomFactor(numer, denom) {\n        this.numer = numer;\n        this.denom = denom;\n    }\n\n    /**\n     * Set Scale\n     * @param {number} value - scale value between 0..1\n     */\n    ZoomFactor.prototype.setScale = function (value) {\n        this.numer = value;\n        this.denom = 1;\n    };\n\n    /**\n     * Get Scale\n     * @return {number} scale value between 0..1\n     */\n    ZoomFactor.prototype.getScale = function () {\n        return this.numer / this.denom;\n    };\n\n\n    /**\n     * GridFactor\n     * @constructor\n     */\n    function GridFactor(width, height) {\n        this.width = width;\n        this.height = height;\n    }\n\n    GridFactor.NO_GRID = new GridFactor(1, 1);\n\n\n    _global.meta.Points = {\n        kind: \'datatype\'\n    };\n\n    /**\n     * Points\n     * @constructor\n     */\n    function Points() {\n        this.points = [];\n    }\n\n    /**\n     * Clear points\n     */\n    Points.prototype.clear = function () {\n        this.points.length = 0;\n    };\n\n    /**\n     * Copy points\n     * @return {Points}\n     */\n    Points.prototype.copy = function () {\n        var pts;\n        pts = new Points();\n        pts.assign(this);\n        return pts;\n    };\n\n    /**\n     * Assign given Points object to this points\n     * @param {Points} pts\n     */\n    Points.prototype.assign = function (pts) {\n        this.points.length = 0;\n        for (var i = 0, len = pts.points.length; i < len; i++) {\n            var p = pts.points[i];\n            this.points.add(p.copy());\n        }\n    };\n\n    /**\n     * Add a point\n     * @param {Point} p\n     */\n    Points.prototype.add = function (p) {\n        this.points.add(p);\n    };\n\n    /**\n     * Insert a point at a specific index\n     * @param {number} index\n     * @param {Point} p\n     */\n    Points.prototype.insert = function (index, p) {\n        this.points.insert(index, p);\n    };\n\n    /**\n     * Remove a point at index\n     * @param {number} index\n     */\n    Points.prototype.remove = function (index) {\n        this.points.removeAt(index);\n    };\n\n    /**\n     * Replace a point p at index\n     * @param {number} index\n     * @param {Point} p\n     */\n    Points.prototype.setPoint = function (index, p) {\n        this.points[index] = p;\n    };\n\n    /**\n     * Replace x, y of the point at index\n     * @param {number} index\n     * @param {number} x\n     * @param {number} y\n     */\n    Points.prototype.setPoint2 = function (index, x, y) {\n        this.points[index].x = x;\n        this.points[index].y = y;\n    };\n\n    /**\n     * Get Point at index\n     * @param {number} index\n     * @return {Point}\n     */\n    Points.prototype.getPoint = function (index) {\n        return this.points[index];\n    };\n\n    /**\n     * Return all x values of all points\n     * @return {Array.<number>}\n     */\n    Points.prototype.getXPoints = function () {\n        var p, xs, _i, _len, _ref;\n        xs = [];\n        _ref = this.points;\n        for (_i = 0, _len = _ref.length; _i < _len; _i++) {\n            p = _ref[_i];\n            xs.push(p.x);\n        }\n        return xs;\n    };\n\n    /**\n     * Return all y values of all points\n     * @return {Array.<number>}\n     */\n    Points.prototype.getYPoints = function () {\n        var p, ys, _i, _len, _ref;\n        ys = [];\n        _ref = this.points;\n        for (_i = 0, _len = _ref.length; _i < _len; _i++) {\n            p = _ref[_i];\n            ys.push(p.y);\n        }\n        return ys;\n    };\n\n    /**\n     * Return number of points\n     * @return {number}\n     */\n    Points.prototype.count = function () {\n        return this.points.length;\n    };\n\n    /**\n     * Reduce points for orthogonal line\n     */\n    Points.prototype.reduceOrthoLine = function () {\n        var i = 0;\n        while (i < this.points.length - 2) {\n            var p1 = this.getPoint(i).copy(),\n                p2 = this.getPoint(i + 1).copy(),\n                p3 = this.getPoint(i + 2).copy();\n            if ((p1.x === p2.x) && (p1.y === p2.y)) {\n                this.remove(i);\n            } else if (((p1.x === p2.x) && (p2.x === p3.x)) || ((p1.y === p2.y) && (p2.y === p3.y))) {\n                this.remove(i + 1);\n            } else {\n                i++;\n            }\n        }\n    };\n\n    /**\n     * Reduce points for line\n     */\n    Points.prototype.reduceLine = function () {\n        var i = 0;\n        while (i < this.points.length - 2) {\n            var p1 = this.getPoint(i).copy(),\n                p2 = this.getPoint(i + 1).copy(),\n                p3 = this.getPoint(i + 2).copy();\n            if (Coord.equalPt(p1, p2)) {\n                this.remove(i);\n            } else if (Coord.ptsInLine(p1, p2, p3)) {\n                this.remove(i + 1);\n            } else {\n                i++;\n            }\n        }\n    };\n\n    /**\n     * Return wether is this rectilinear line or not\n     */\n    Points.prototype.isRectilinear = function () {\n        for (var i = 1, len = this.points.length; i < len; i++) {\n            var p0 = this.points[i-1];\n            var p1 = this.points[i];\n            if ((p0.x !== p1.x) && (p0.y !== p1.y)) {\n                return false;\n            }\n        }\n        return true;\n    };\n\n    /**\n     * Convert oblique line to rectilinear line\n     */\n    Points.prototype.convObliqueToRectilinear = function () {\n        var i = 0;\n        while (i < this.count() - 1) {\n            this.insert(i + 1, Coord.makeOrthoPt(this.getPoint(i), this.getPoint(i + 1)));\n            i = i + 2;\n        }\n    };\n\n    /**\n     * Get bounding box\n     * @return {Rect}\n     */\n    Points.prototype.getBoundingRect = function () {\n        var p, r, _i, _len, _ref;\n        r = new Rect(this.getPoint(0).x, this.getPoint(0).y, this.getPoint(0).x, this.getPoint(0).y);\n        _ref = this.points;\n        for (_i = 0, _len = _ref.length; _i < _len; _i++) {\n            p = _ref[_i];\n            r.setRect2(Coord.unionRect(r, new Rect(p.x, p.y, p.x + 1, p.y + 1)));\n        }\n        return r;\n    };\n\n    /**\n     * Quantize\n     * @return {Rect}\n     */\n    Points.prototype.quantize = function () {\n        for (var i = 0, len = this.points.length; i < len; i++) {\n            var p = this.points[i];\n            p.x = Math.round(p.x);\n            p.y = Math.round(p.y);\n        }\n    };\n\n    /**\n     * Define custom write method\n     * - Refer to Core.Writer.prototype.writeCustom\n     */\n    Points.prototype.__write = function () {\n        var p, result, _i, _len, _ref;\n        result = [];\n        _ref = this.points;\n        for (_i = 0, _len = _ref.length; _i < _len; _i++) {\n            p = _ref[_i];\n            result.push(p.x + ":" + p.y);\n        }\n        return result.join(";");\n    };\n\n    /**\n     * Define custom read method\n     * - Refer to Core.Reader.prototype.readCustom\n     */\n    Points.prototype.__read = function (value) {\n        var p, t, terms, x, y, _i, _len;\n        this.points = [];\n        terms = value.split(";");\n        for (_i = 0, _len = terms.length; _i < _len; _i++) {\n            t = terms[_i];\n            p = t.split(":");\n            x = p[0] * 1;\n            y = p[1] * 1;\n            this.points.push(new Point(x, y));\n        }\n    };\n\n    /**\n     * Coordination utility object\n     * @namespace\n     */\n    var Coord = {};\n\n\n    /**\n     * transform value as a given zoonFactor\n     * @function\n     */\n    Coord.valueTransform = function (zf, value) {\n        var f;\n        f = zf.numer / zf.denom;\n        return f * value;\n    };\n\n    /**\n     * @function\n     */\n    Coord.coordTransform = function (zf, gf, p) {\n        var f, x, y;\n        x = p.x;\n        y = p.y;\n        x = x - (x % gf.width);\n        y = y - (y % gf.height);\n        f = zf.numer / zf.denom;\n        x = f * x;\n        y = f * y;\n        return p.setPoint(x, y);\n    };\n\n    /**\n     * @function\n     */\n    Coord.coordTransform2 = function (zf, gf, r) {\n        var p1, p2;\n        p1 = new Point(r.x1, r.y1);\n        p2 = new Point(r.x2, r.y2);\n        Coord.coordTransform(zf, gf, p1);\n        Coord.coordTransform(zf, gf, p2);\n        return r.setRect(p1.x, p1.y, p2.x, p2.y);\n    };\n\n    /**\n     * @function\n     */\n    Coord.coordRevTransform = function (zf, gf, p) {\n        var f, x, y;\n        x = p.x;\n        y = p.y;\n        f = zf.numer / zf.denom;\n        x = x / f;\n        y = y / f;\n        x = x - (x % gf.width);\n        y = y - (y % gf.height);\n        return p.setPoint(x, y);\n    };\n\n    /**\n     * @function\n     */\n    Coord.ptInRect = function (x, y, rect) {\n        Coord.normalizeRect(rect);\n        return ((rect.x1 <= x && x <= rect.x2)) && ((rect.y1 <= y && y <= rect.y2));\n    };\n\n    /**\n     * @function\n     */\n    Coord.ptInRect2 = function (p, rect) {\n        return Coord.ptInRect(p.x, p.y, rect);\n    };\n\n    /**\n     * @function\n     */\n    Coord.ptInLine = function (line, p) {\n        var a, b, bottom, c, left, ox, oy, r, result, right, s, top, tx, ty, x1, x2, y1, y2;\n        result = false;\n        left = Math.min(line.x1, line.x2) - 5;\n        right = Math.max(line.x1, line.x2) + 5;\n        top = Math.min(line.y1, line.y2) - 5;\n        bottom = Math.max(line.y1, line.y2) + 5;\n        if ((left <= p.x) && (right >= p.x) && (top <= p.y) && (bottom >= p.y)) {\n            a = line.x2 - line.x1;\n            b = line.y2 - line.y1;\n            r = Math.sqrt(a * a + b * b + 0.000001);\n            c = b / r;\n            s = a / r;\n            ox = p.x - line.x1;\n            oy = p.y - line.y1;\n            tx = c * ox - s * oy;\n            ty = s * ox + c * oy;\n            x1 = -5.0;\n            x2 = 5.0;\n            y1 = -5.0;\n            y2 = r + 5;\n            if ((x1 <= tx) && (x2 >= tx) && (y1 <= ty) && (y2 >= ty)) {\n                result = true;\n            } else {\n                result = false;\n            }\n        }\n        return result;\n    };\n\n    /**\n     * @function\n     */\n    Coord.ptsInLine = function (p1, p2, p3) {\n        return Coord.ptInLine(new Rect(p1.x, p1.y, p3.x, p3.y), p2) || Coord.ptInLine(new Rect(p1.x, p1.y, p2.x, p2.y), p3) || Coord.ptInLine(new Rect(p2.x, p2.y, p3.x, p3.y), p1);\n    };\n\n    /**\n     * @function\n     */\n    Coord.rectInRect = function (rect1, rect2) {\n        return !((rect1.x1 > rect2.x2) || (rect2.x1 > rect1.x2) || (rect1.y1 > rect2.y2) || (rect2.y1 > rect1.y2));\n    };\n\n    /**\n     * @function\n     */\n    Coord.getCenter = function (rect) {\n        return new Point(Math.floor((rect.x1 + rect.x2) / 2), Math.floor((rect.y1 + rect.y2) / 2));\n    };\n\n    /**\n     * @function\n     */\n    Coord.equalRect = function (r1, r2) {\n        return (r1.x1 === r2.x1) && (r1.y1 === r2.y1) && (r1.x2 === r2.x2) && (r1.y2 === r2.y2);\n    };\n\n    /**\n     * @function\n     */\n    Coord.equalPt = function (p1, p2) {\n        return Coord.equalPt2(p1, p2, 5.0);\n    };\n\n    /**\n     * @function\n     */\n    Coord.equalPt2 = function (p1, p2, d) {\n        return Coord.ptInRect(p2.x, p2.y, new Rect(p1.x - d, p1.y - d, p1.x + d, p1.y + d));\n    };\n\n    /**\n     * @function\n     */\n    Coord.normalizeRect = function (rect) {\n        var x1, x2, y1, y2;\n        if (rect.x1 < rect.x2) {\n            x1 = rect.x1;\n            x2 = rect.x2;\n        } else {\n            x1 = rect.x2;\n            x2 = rect.x1;\n        }\n        if (rect.y1 < rect.y2) {\n            y1 = rect.y1;\n            y2 = rect.y2;\n        } else {\n            y1 = rect.y2;\n            y2 = rect.y1;\n        }\n        return rect.setRect(x1, y1, x2, y2);\n    };\n\n    /**\n     * @function\n     */\n    Coord.normalizeRect2 = function (p1, p2) {\n        var r;\n        r = new Rect(p1.x, p1.y, p2.x, p2.y);\n        Coord.normalizeRect(r);\n        p1.setPoint(r.x1, r.y1);\n        return p2.setPoint(r.x2, r.y2);\n    };\n\n    /**\n     * @function\n     */\n    Coord.unionRect = function (r1, r2) {\n        var r;\n        r = new Rect(r1.x1, r1.y1, r1.x2, r1.y2);\n        if (r2.x1 < r.x1) {\n            r.x1 = r2.x1;\n        }\n        if (r2.y1 < r.y1) {\n            r.y1 = r2.y1;\n        }\n        if (r2.x2 > r.x2) {\n            r.x2 = r2.x2;\n        }\n        if (r2.y2 > r.y2) {\n            r.y2 = r2.y2;\n        }\n        return r;\n    };\n\n    /**\n     * @function\n     */\n    Coord.junction = function (r, pt) {\n        var c, i, lean, p, ps, _ref, _ref1, _ref2, _ref3;\n        p = new Point(pt.x, pt.y);\n        c = new Point(Math.floor((r.x1 + r.x2) / 2), Math.floor((r.y1 + r.y2) / 2));\n        if (c.x === p.x || c.y === p.y) {\n            return Coord.orthoJunction(r, p);\n        }\n        lean = (p.y - c.y) / (p.x - c.x);\n        ps = [0, 1, 2, 3, 4];\n        ps[0] = null;\n        ps[1] = new Point(r.x1, lean * (r.x1 - c.x) + c.y);\n        ps[2] = new Point(r.x2, lean * (r.x2 - c.x) + c.y);\n        ps[3] = new Point((r.y1 - c.y) / lean + c.x, r.y1);\n        ps[4] = new Point((r.y2 - c.y) / lean + c.x, r.y2);\n        Coord.normalizeRect2(c, p);\n        i = 0;\n        while (true) {\n            i++;\n            if (i > 4) {\n                break;\n            }\n            if ((((r.x1 <= (_ref = ps[i].x) && _ref <= r.x2)) && ((r.y1 <= (_ref1 = ps[i].y) && _ref1 <= r.y2)) && ((c.x <= (_ref2 = ps[i].x) && _ref2 <= p.x)) && ((c.y <= (_ref3 = ps[i].y) && _ref3 <= p.y))) || (i > 4)) {\n                break;\n            }\n        }\n        if (i > 4) {\n            return new Point(Math.floor((r.x1 + r.x2) / 2), Math.floor((r.y1 + r.y2) / 2));\n        } else {\n            return ps[i];\n        }\n    };\n\n    /**\n     * @function\n     */\n    Coord.orthoJunction = function (r, p) {\n        if ((r.x1 < p.x) && (p.x < r.x2)) {\n            if (r.y1 >= p.y) {\n                return new Point(p.x, r.y1);\n            } else {\n                return new Point(p.x, r.y2);\n            }\n        } else if ((r.y1 < p.y) && (p.y < r.y2)) {\n            if (r.x1 >= p.x) {\n                return new Point(r.x1, p.y);\n            } else {\n                return new Point(r.x2, p.y);\n            }\n        } else if ((r.x1 === p.x) || (r.x2 === p.x)) {\n            if (r.y1 >= p.y) {\n                return new Point(p.x, r.y1);\n            } else {\n                return new Point(p.x, r.y2);\n            }\n        } else if ((r.y1 === p.y) || (r.y2 === p.y)) {\n            if (r.x1 >= p.x) {\n                return new Point(r.x1, p.y);\n            } else {\n                return new Point(r.x2, p.y);\n            }\n        } else {\n            return new Point(-100, -100);\n        }\n    };\n\n    /**\n     * @function\n     */\n    Coord.makeOrthoPt = function (p1, p2) {\n        var result;\n        result = new Point(p1.x, Math.max(p1.y, p2.y));\n        if (result.y === p1.y) {\n            result.x = p2.x;\n        }\n        return result;\n    };\n\n    /**\n     * @function\n     */\n    Coord.getPointAwayLine = function (tailPoint, headPoint, alpha, distance) {\n        var a, b, th, x, y;\n        a = tailPoint.x - headPoint.x;\n        b = tailPoint.y - headPoint.y;\n        th = Math.atan(Math.abs(b) / (Math.abs(a) + 0.000001));\n        if (a > 0) {\n            if (b > 0) {\n                th = Math.PI - th;\n            } else {\n                th = th + Math.PI;\n            }\n        } else {\n            if (b < 0) {\n                th = 2 * Math.PI - th;\n            }\n        }\n        x = distance * Math.cos(th + alpha);\n        y = distance * Math.sin(th + alpha);\n        return new Point(Math.floor(x), Math.floor(-y));\n    };\n\n    /**\n     * @function\n     */\n    Coord.getPolar = function (line, p) {\n        var a, b, p1, p2, th1, th2;\n        p1 = new Point(line.x1, line.y1);\n        p2 = new Point(line.x2, line.y2);\n        a = p2.y - p1.y;\n        b = p2.x - p1.x + 0.00001;\n        th1 = Math.atan(a / b);\n        if (((a < 0) && (b < 0)) || ((a > 0) && (b < 0)) || ((a === 0) && (b < 0))) {\n            th1 = th1 + Math.PI;\n        }\n        a = p.y - p1.y;\n        b = p.x - p1.x + 0.00001;\n        th2 = Math.atan(a / b);\n        if (((a < 0) && (b < 0)) || ((a > 0) && (b < 0)) || ((a === 0) && (b < 0))) {\n            th2 = th2 + Math.PI;\n        }\n        return {\n            alpha: th1 - th2,\n            distance: Math.sqrt(Math.square(p1.x - p.x) + Math.square(p1.y - p.y))\n        };\n    };\n\n    /**\n     * @function\n     */\n    Coord.getAngle = function (x1, y1, x2, y2) {\n        var dx = x2 - x1,\n            dy = y2 - y1;\n        return Math.atan2(dy, dx);\n    };\n\n    /**\n     * Canvas\n     * @constructor\n     */\n    function Canvas(context) {\n        this.context = context;\n        this.stateStack = [];\n        this.color = Color.BLACK;\n        this.fillColor = Color.WHITE;\n        this.fontColor = Color.BLACK;\n        this.font = new Font("Arial", 12, FS_NORMAL);\n        this.lineWidth = 1;\n        this.alpha = 1.0;\n        this.origin = new Point(0.0, 0.0);\n        this.zoomFactor = new ZoomFactor(1, 1);\n        this.gridFactor = new GridFactor(1, 1);\n        this.coordTransformApplied = true;\n        this.ratio = 1;\n    }\n\n    Canvas.toHTML5Font = function (f) {\n        var style = "",\n            size = f.size;\n        switch (f.style) {\n        case FS_ITALIC:\n            style = "italic";\n            break;\n        case FS_BOLD:\n            style = "bold";\n            break;\n        case FS_BOLD_ITALIC:\n            style = "bold italic";\n        }\n        return "" + style + " " + size + "px " + f.face;\n    };\n\n    /**\n     * Canvas의 상태를 스택에 저장\n     */\n    Canvas.prototype.storeState = function () {\n        var state = {\n            color: this.color,\n            fillColor: this.fillColor,\n            fontColor: this.fontColor,\n            font: this.font.copy(),\n            lineWidth: this.lineWidth,\n            alpha: this.alpha\n        };\n        this.stateStack.push(state);\n    };\n\n    /**\n     * Canvas의 상태를 스택으로부터 복구\n     */\n    Canvas.prototype.restoreState = function () {\n        var state = this.stateStack.pop();\n        if (state !== null) {\n            this.color = state.color;\n            this.fillColor = state.fillColor;\n            this.fontColor = state.fontColor;\n            this.font = state.font;\n            this.lineWidth = state.lineWidth;\n            this.alpha = state.alpha;\n        }\n    };\n\n    /**\n     * Transform (Scale)\n     */\n    Canvas.prototype.transform = function () {\n        this.context.save();\n        if (this.coordTransformApplied === true) {\n            this.context.translate(this.origin.x, this.origin.y);\n            var scale = this.zoomFactor.getScale();\n            this.context.scale(scale * this.ratio, scale * this.ratio);\n        }\n    };\n\n    /**\n     * Restore Transform\n     */\n    Canvas.prototype.restoreTransform = function () {\n        this.context.restore();\n    };\n\n    /**\n     * Put a pixel\n     *\n     * @param {number} x\n     * @param {number} y\n     * @param {number} c - Color\n     */\n    Canvas.prototype.putPixel = function (x, y, c) {\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = c;\n        this.context.globalAlpha = this.alpha;\n        this.context.moveTo(x, y);\n        this.context.lineTo(x + 1, y + 1);\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw a line\n     *\n     * @param {number} x1\n     * @param {number} y1\n     * @param {number} x2\n     * @param {number} y2\n     * @param {?Array.<number>} dashPattern\n     */\n    Canvas.prototype.line = function (x1, y1, x2, y2, dashPattern) {\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        if (dashPattern) {\n            this.context.setLineDash(dashPattern);\n        }\n        this.context.moveTo(x1, y1);\n        this.context.lineTo(x2, y2);\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * rect\n     *\n     * @param {number} x1\n     * @param {number} y1\n     * @param {number} x2\n     * @param {number} y2\n     * @param {Array.<number>} dashPattern - e.g.) [3]\n     */\n    Canvas.prototype.rect = function (x1, y1, x2, y2, dashPattern) {\n        this.transform();\n        var x = (x1 < x2 ? x1 : x2),\n            y = (y1 < y2 ? y1 : y2),\n            w = Math.abs(x2 - x1),\n            h = Math.abs(y2 - y1);\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        if (dashPattern) {\n            this.context.setLineDash(dashPattern);\n        }\n        this.context.rect(x, y, w, h);\n        this.context.closePath();\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * fillRect\n     *\n     * @param {number} x1\n     * @param {number} y1\n     * @param {number} x2\n     * @param {number} y2\n     */\n    Canvas.prototype.fillRect = function (x1, y1, x2, y2) {\n        this.transform();\n        var x = (x1 < x2 ? x1 : x2),\n            y = (y1 < y2 ? y1 : y2),\n            w = Math.abs(x2 - x1),\n            h = Math.abs(y2 - y1);\n        this.context.fillStyle = this.fillColor;\n        this.context.globalAlpha = this.alpha;\n        this.context.fillRect(x, y, w, h);\n        this.restoreTransform();\n    };\n\n    /**\n     * roundRect\n     *\n     * @param {number} x1\n     * @param {number} y1\n     * @param {number} x2\n     * @param {number} y2\n     * @param {number} radius\n     */\n    Canvas.prototype.roundRect = function (x1, y1, x2, y2, radius, dashPattern) {\n        this.transform();\n        var x = (x1 < x2 ? x1 : x2),\n            y = (y1 < y2 ? y1 : y2),\n            w = Math.abs(x2 - x1),\n            h = Math.abs(y2 - y1);\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        if (dashPattern) {\n            this.context.setLineDash(dashPattern);\n        }\n        this.context.moveTo(x + radius, y);\n        this.context.lineTo(x + w - radius, y);\n        this.context.quadraticCurveTo(x + w, y, x + w, y + radius);\n        this.context.lineTo(x + w, y + h - radius);\n        this.context.quadraticCurveTo(x + w, y + h, x + w - radius, y + h);\n        this.context.lineTo(x + radius, y + h);\n        this.context.quadraticCurveTo(x, y + h, x, y + h - radius);\n        this.context.lineTo(x, y + radius);\n        this.context.quadraticCurveTo(x, y, x + radius, y);\n        this.context.closePath();\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * fillRoundRect\n     *\n     * @param {number} x1\n     * @param {number} y1\n     * @param {number} x2\n     * @param {number} y2\n     * @param {number} radius\n     */\n    Canvas.prototype.fillRoundRect = function (x1, y1, x2, y2, radius) {\n        this.transform();\n        var x = (x1 < x2 ? x1 : x2),\n            y = (y1 < y2 ? y1 : y2),\n            w = Math.abs(x2 - x1),\n            h = Math.abs(y2 - y1);\n        this.context.beginPath();\n        this.context.fillStyle = this.fillColor;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        this.context.moveTo(x + radius, y);\n        this.context.lineTo(x + w - radius, y);\n        this.context.quadraticCurveTo(x + w, y, x + w, y + radius);\n        this.context.lineTo(x + w, y + h - radius);\n        this.context.quadraticCurveTo(x + w, y + h, x + w - radius, y + h);\n        this.context.lineTo(x + radius, y + h);\n        this.context.quadraticCurveTo(x, y + h, x, y + h - radius);\n        this.context.lineTo(x, y + radius);\n        this.context.quadraticCurveTo(x, y, x + radius, y);\n        this.context.closePath();\n        this.context.fill();\n        this.restoreTransform();\n    };\n\n\n    /**\n     * ellipse\n     *\n     * @param {number} x1\n     * @param {number} y1\n     * @param {number} x2\n     * @param {number} y2\n     * @param {Array.<number>} dashPattern\n     */\n    Canvas.prototype.ellipse = function (x1, y1, x2, y2, dashPattern) {\n        this.transform();\n        var x = (x1 < x2 ? x1 : x2),\n            y = (y1 < y2 ? y1 : y2),\n            w = Math.abs(x2 - x1),\n            h = Math.abs(y2 - y1),\n            kappa = 0.5522848,\n            ox = (w / 2.0) * kappa,\n            oy = (h / 2.0) * kappa,\n            xe = x + w,\n            ye = y + h,\n            xm = x + w / 2.0,\n            ym = y + h / 2.0;\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        if (dashPattern) {\n            this.context.setLineDash(dashPattern);\n        }\n        this.context.moveTo(x, ym);\n        this.context.bezierCurveTo(x, ym - oy, xm - ox, y, xm, y);\n        this.context.bezierCurveTo(xm + ox, y, xe, ym - oy, xe, ym);\n        this.context.bezierCurveTo(xe, ym + oy, xm + ox, ye, xm, ye);\n        this.context.bezierCurveTo(xm - ox, ye, x, ym + oy, x, ym);\n        this.context.closePath();\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * fillEllipse\n     *\n     * @param {number} x1\n     * @param {number} y1\n     * @param {number} x2\n     * @param {number} y2\n     */\n    Canvas.prototype.fillEllipse = function (x1, y1, x2, y2) {\n        this.transform();\n        var x = (x1 < x2 ? x1 : x2),\n            y = (y1 < y2 ? y1 : y2),\n            w = Math.abs(x2 - x1),\n            h = Math.abs(y2 - y1),\n            kappa = 0.5522848,\n            ox = (w / 2.0) * kappa,\n            oy = (h / 2.0) * kappa,\n            xe = x + w,\n            ye = y + h,\n            xm = x + w / 2.0,\n            ym = y + h / 2.0;\n        this.context.beginPath();\n        this.context.fillStyle = this.fillColor;\n        this.context.globalAlpha = this.alpha;\n        this.context.moveTo(x, ym);\n        this.context.bezierCurveTo(x, ym - oy, xm - ox, y, xm, y);\n        this.context.bezierCurveTo(xm + ox, y, xe, ym - oy, xe, ym);\n        this.context.bezierCurveTo(xe, ym + oy, xm + ox, ye, xm, ye);\n        this.context.bezierCurveTo(xm - ox, ye, x, ym + oy, x, ym);\n        this.context.closePath();\n        this.context.fill();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw polyline\n     * @param {Array.<Point>} points\n     * @param {Array.<number>} dashPattern\n     */\n    Canvas.prototype.polyline = function (points, dashPattern) {\n        var i, len, p;\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        if (dashPattern) {\n            this.context.setLineDash(dashPattern);\n        }\n        for (i = 0, len = points.length; i < len; i++) {\n            p = points[i];\n            if (i === 0) {\n                this.context.moveTo(p.x, p.y);\n            } else {\n                this.context.lineTo(p.x, p.y);\n            }\n        }\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw roundRectLine\n     * @param {Array.<Point>} points\n     * @param {Array.<number>} dashPattern\n     */\n    Canvas.prototype.roundRectLine = function (points, dashPattern) {\n        var ROUND_RADIUS = 8;\n        var i, len;\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        if (dashPattern) {\n            this.context.setLineDash(dashPattern);\n        }\n        if (points.length > 0) {\n            var p, prev, next, pdir, ndir;\n            this.context.moveTo(points[0].x, points[0].y);\n            for (i = 1, len = points.length - 1; i < len; i++) {\n                p = points[i];\n                prev = points[i - 1];\n                next = points[i + 1];\n                // direction of previous line\n                if (prev.x === p.x) { // vertical\n                    pdir = (prev.y < p.y) ? \'VD\' : \'VU\'; // down or up\n                } else { // horizontal\n                    pdir = (prev.x < p.x) ? \'HR\' : \'HL\'; // right or left\n                }\n                // direction of next line\n                if (next.x === p.x) {\n                    ndir = (next.y < p.y) ? \'VU\' : \'VD\'; // vertical down or up\n                } else {\n                    ndir = (next.x < p.x) ? \'HL\' : \'HR\'; // horizontal right or left\n                }\n                // draw line\n                switch (pdir) {\n                case \'VD\':\n                    // this.context.moveTo(prev.x, prev.y + ROUND_RADIUS);\n                    this.context.lineTo(p.x, p.y - ROUND_RADIUS);\n                    break;\n                case \'VU\':\n                    // this.context.moveTo(prev.x, prev.y - ROUND_RADIUS);\n                    this.context.lineTo(p.x, p.y + ROUND_RADIUS);\n                    break;\n                case \'HR\':\n                    // this.context.moveTo(prev.x + ROUND_RADIUS, prev.y);\n                    this.context.lineTo(p.x - ROUND_RADIUS, p.y);\n                    break;\n                case \'HL\':\n                    // this.context.moveTo(prev.x - ROUND_RADIUS, prev.y);\n                    this.context.lineTo(p.x + ROUND_RADIUS, p.y);\n                    break;\n                }\n                // draw corner\n                switch (pdir) {\n                case \'VD\':\n                    if (ndir === \'HL\') {\n                        this.context.arc(p.x - ROUND_RADIUS, p.y - ROUND_RADIUS, ROUND_RADIUS, 0, 0.5 * Math.PI, false);\n                    } else { // HR\n                        this.context.arc(p.x + ROUND_RADIUS, p.y - ROUND_RADIUS, ROUND_RADIUS, Math.PI, 0.5 * Math.PI, true);\n                    }\n                    break;\n                case \'VU\':\n                    if (ndir === \'HL\') {\n                        this.context.arc(p.x - ROUND_RADIUS, p.y + ROUND_RADIUS, ROUND_RADIUS, 0, 1.5 * Math.PI, true);\n                    } else { // HR\n                        this.context.arc(p.x + ROUND_RADIUS, p.y + ROUND_RADIUS, ROUND_RADIUS, Math.PI, 1.5 * Math.PI, false);\n                    }\n                    break;\n                case \'HR\':\n                    if (ndir === \'VD\') {\n                        this.context.arc(p.x - ROUND_RADIUS, p.y + ROUND_RADIUS, ROUND_RADIUS, 1.5 * Math.PI, 0, false);\n                    } else { // VU\n                        this.context.arc(p.x - ROUND_RADIUS, p.y - ROUND_RADIUS, ROUND_RADIUS, 0.5 * Math.PI, 0, true);\n                    }\n                    break;\n                case \'HL\':\n                    if (ndir === \'VD\') {\n                        this.context.arc(p.x + ROUND_RADIUS, p.y + ROUND_RADIUS, ROUND_RADIUS, 1.5 * Math.PI, Math.PI, true);\n                    } else { // VU\n                        this.context.arc(p.x + ROUND_RADIUS, p.y - ROUND_RADIUS, ROUND_RADIUS, 0.5 * Math.PI, Math.PI, false);\n                    }\n                    break;\n                }\n            }\n            this.context.lineTo(points[points.length-1].x, points[points.length-1].y);\n        }\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw curveLine\n     * @param {Array.<Point>} points\n     * @param {Array.<number>} dashPattern\n     */\n    Canvas.prototype.curveLine = function (points, dashPattern) {\n        var i, len;\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        if (dashPattern) {\n            this.context.setLineDash(dashPattern);\n        }\n        this.context.moveTo(points[0].x, points[0].y);\n        if (points.length > 2) {\n            for (i = 1, len = points.length - 2; i < len; i++) {\n                var xc = (points[i].x + points[i + 1].x) / 2;\n                var yc = (points[i].y + points[i + 1].y) / 2;\n                this.context.quadraticCurveTo(points[i].x, points[i].y, xc, yc);\n            }\n            this.context.quadraticCurveTo(points[i].x, points[i].y, points[i+1].x,points[i+1].y);\n        } else {\n            this.context.lineTo(points[points.length-1].x, points[points.length-1].y);\n        }\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw polygon\n     * @param {Array.<Point>} points\n     */\n    Canvas.prototype.polygon = function (points) {\n        var i, len, p, start;\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        start = null;\n        for (i = 0, len = points.length; i < len; i++) {\n            p = points[i];\n            if (i === 0) {\n                start = p;\n                this.context.moveTo(p.x, p.y);\n            } else {\n                this.context.lineTo(p.x, p.y);\n            }\n        }\n        this.context.lineTo(start.x, start.y);\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw filled polygon\n     * @param {Points} points\n     */\n    Canvas.prototype.fillPolygon = function (points) {\n        var i, len, p;\n        this.transform();\n        this.context.beginPath();\n        this.context.fillStyle = this.fillColor;\n        this.context.globalAlpha = this.alpha;\n        for (i = 0, len = points.length; i < len; i++) {\n            p = points[i];\n            if (i === 0) {\n                this.context.moveTo(p.x, p.y);\n            } else {\n                this.context.lineTo(p.x, p.y);\n            }\n        }\n        this.context.fill();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw arc\n     *\n     * @param {number} x\n     * @param {number} y\n     * @param {number} r\n     * @param {number} startAngle\n     * @param {number} endAngle\n     * @param {boolean} counterClockwise\n     */\n    Canvas.prototype.arc = function (x, y, r, startAngle, endAngle, counterClockwise) {\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        this.context.arc(x, y, r, startAngle, endAngle, counterClockwise);\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw filled arc\n     *\n     * @param {number} x\n     * @param {number} y\n     * @param {number} r\n     * @param {number} startAngle\n     * @param {number} endAngle\n     * @param {boolean} counterClockwise\n     */\n    Canvas.prototype.fillArc = function (x, y, r, startAngle, endAngle, counterClockwise) {\n        this.transform();\n        this.context.beginPath();\n        this.context.fillStyle = this.fillColor;\n        this.context.globalAlpha = this.alpha;\n        this.context.arc(x, y, r, startAngle, endAngle, counterClockwise);\n        this.context.fill();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw a path.\n     * A path command is one of the followings:\n     *   [\'M\', x, y] - moveTo\n     *   [\'L\', x, y] - lineTo\n     *   [\'C\', x1, y1, x2, y2, x, y] - bezierCurveTo\n     *   [\'Q\', x1, y1, x, 2] - quadraticCurveTo\n     *   [\'Z\'] - closePath\n     *\n     * @param {Array.<Array>}\n     */\n    Canvas.prototype.path = function (commands) {\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        for (var i = 0, len = commands.length; i < len; i++) {\n            var comm = commands[i];\n            switch (comm[0]) {\n            case \'M\':\n                this.context.moveTo(comm[1], comm[2]);\n                break;\n            case \'L\':\n                this.context.lineTo(comm[1], comm[2]);\n                break;\n            case \'C\':\n                this.context.bezierCurveTo(comm[1], comm[2], comm[3], comm[4], comm[5], comm[6]);\n                break;\n            case \'Q\':\n                this.context.quadraticCurveTo(comm[1], comm[2], comm[3], comm[4]);\n                break;\n            case \'Z\':\n                this.context.closePath();\n                break;\n            }\n        }\n        this.context.stroke();\n        this.restoreTransform();\n    };\n\n    /**\n     * Draw a filled path\n     * A path command is one of the followings:\n     *   [\'M\', x, y] - moveTo\n     *   [\'L\', x, y] - lineTo\n     *   [\'C\', x1, y1, x2, y2, x, y] - bezierCurveTo\n     *   [\'Q\', x1, y1, x, 2] - quadraticCurveTo\n     *   [\'Z\'] - closePath\n     *\n     * @param {Array.<Array>}\n     * @param {boolean} doStroke\n     */\n    Canvas.prototype.fillPath = function (commands, doStroke) {\n        this.transform();\n        this.context.beginPath();\n        this.context.strokeStyle = this.color;\n        this.context.fillStyle = this.fillColor;\n        this.context.lineWidth = this.lineWidth;\n        this.context.globalAlpha = this.alpha;\n        for (var i = 0, len = commands.length; i < len; i++) {\n            var comm = commands[i];\n            switch (comm[0]) {\n            case \'M\':\n                this.context.moveTo(comm[1], comm[2]);\n                break;\n            case \'L\':\n                this.context.lineTo(comm[1], comm[2]);\n                break;\n            case \'C\':\n                this.context.bezierCurveTo(comm[1], comm[2], comm[3], comm[4], comm[5], comm[6]);\n                break;\n            case \'Q\':\n                this.context.quadraticCurveTo(comm[1], comm[2], comm[3], comm[4]);\n                break;\n            case \'Z\':\n                this.context.closePath();\n                break;\n            }\n        }\n        this.context.fill();\n        if (doStroke) {\n            this.context.stroke();\n        }\n        this.restoreTransform();\n    };\n\n    /**\n     * Word Wrap\n     * @param {string} text\n     * @param {number} width\n     * @return {Array.<string>}\n     */\n    Canvas.prototype.wordWrap = function (text, width) {\n        var lines  = text.split(" ");\n        if (lines.length > 0) {\n            var result = [],\n                term   = lines[0];\n            for (var i = 1, len = lines.length; i < len; i++) {\n                var t = term + " " + lines[i];\n                if (this.context.measureText(t).width <= width) {\n                    term = t;\n                } else {\n                    result.push(term);\n                    term = lines[i];\n                }\n            }\n            if (term.length > 0) {\n                result.push(term);\n            }\n            return result;\n        }\n        return [];\n    };\n\n    /**\n     * Draw text\n     *\n     * @param {number} x\n     * @param {number} y\n     * @param {text} text\n     * @param {boolean} rotate\n     * @param {boolean} wordWrap\n     * @param {boolean} underline\n     */\n    Canvas.prototype.textOut = function (x, y, text, rotation, wordWrap, underline) {\n        var r = new Rect(x, y, x, y);\n        rotation  = rotation ? rotation : 0;\n        wordWrap  = wordWrap ? wordWrap : false;\n        underline = underline ? underline : false;\n        this.textOut2(r, text, AL_LEFT, AL_TOP, rotation, wordWrap, underline);\n    };\n\n    /**\n     * Draw text underline\n     * @private\n     */\n    Canvas.prototype._textUnderline = function (baseX, baseY, text, horizontalAlignment, verticalAlignment) {\n        var x1, x2, y,\n            extent = this.textExtent(text);\n        switch (verticalAlignment) {\n        case AL_TOP:\n            y = baseY + extent.y;\n            break;\n        case AL_BOTTOM:\n            y = baseY;\n            break;\n        case AL_MIDDLE:\n            y = baseY + (extent.y / 2.0);\n            break;\n        }\n        switch (horizontalAlignment) {\n        case AL_LEFT:\n            x1 = baseX; x2 = x1 + extent.x;\n            break;\n        case AL_RIGHT:\n            x2 = baseX; x1 = x2 - extent.x;\n            break;\n        case AL_CENTER:\n            x1 = baseX + (extent.x / 2.0); x2 = baseX - (extent.x / 2.0);\n            break;\n        }\n        this.context.beginPath();\n        this.context.strokeStyle = this.fontColor;\n        this.context.moveTo(x1, y);\n        this.context.lineTo(x2, y);\n        this.context.stroke();\n    };\n\n    /**\n     * Draw text\n     *\n     * @param {Rect} rect\n     * @param {string} text\n     * @param {number} horizontalAlignment\n     * @param {number} verticalAlignment\n     * @param {boolean} rotate\n     * @param {boolean} wordWrap\n     * @param {boolean} underline\n     */\n    Canvas.prototype.textOut2 = function (rect, text, horizontalAlignment, verticalAlignment, rotate, wordWrap, underline) {\n        this.transform();\n        this.context.save();\n        this.context.beginPath();\n        this.context.fillStyle = this.fontColor;\n        this.context.globalAlpha = this.alpha;\n        this.context.font = Canvas.toHTML5Font(this.font);\n        var baseX = rect.x1,\n            baseY = rect.y1;\n        if (rotate === true) {\n            switch (verticalAlignment) {\n            case AL_TOP:\n                baseX = rect.x1;\n                this.context.textBaseline = "top";\n                break;\n            case AL_BOTTOM:\n                baseX = rect.x2;\n                this.context.textBaseline = "bottom";\n                break;\n            case AL_MIDDLE:\n                baseX = (rect.x1 + rect.x2) / 2.0;\n                this.context.textBaseline = "middle";\n            }\n            switch (horizontalAlignment) {\n            case AL_LEFT:\n                baseY = rect.y2;\n                this.context.textAlign = "left";\n                break;\n            case AL_RIGHT:\n                baseY = rect.y1;\n                this.context.textAlign = "right";\n                break;\n            case AL_CENTER:\n                baseY = (rect.y1 + rect.y2) / 2.0;\n                this.context.textAlign = "center";\n            }\n            this.context.translate(baseX, baseY);\n            this.context.rotate(toRadian(-90));\n            this.context.fillText(text, 0, 0);\n        } else {\n            switch (verticalAlignment) {\n            case AL_TOP:\n                baseY = rect.y1;\n                this.context.textBaseline = "top";\n                break;\n            case AL_BOTTOM:\n                baseY = rect.y2;\n                this.context.textBaseline = "bottom";\n                break;\n            case AL_MIDDLE:\n                baseY = (rect.y1 + rect.y2) / 2.0;\n                this.context.textBaseline = "middle";\n            }\n            switch (horizontalAlignment) {\n            case AL_LEFT:\n                baseX = rect.x1;\n                this.context.textAlign = "left";\n                break;\n            case AL_RIGHT:\n                baseX = rect.x2;\n                this.context.textAlign = "right";\n                break;\n            case AL_CENTER:\n                baseX = (rect.x1 + rect.x2) / 2.0;\n                this.context.textAlign = "center";\n            }\n\n            if (wordWrap) {\n                var lines = this.wordWrap(text, rect.getWidth()),\n                    _height = (lines.length * this.font.size);\n                switch (verticalAlignment) {\n                case AL_BOTTOM: baseY = rect.y2 - ((lines.length - 1) * this.font.size) ; break;\n                case AL_MIDDLE: baseY = rect.y1 + ((rect.getHeight() - _height) / 2) + (this.font.size / 2); break;\n                }\n                for (var i = 0, len = lines.length; i < len; i++) {\n                    this.context.fillText(lines[i], baseX, baseY);\n                    if (underline) {\n                        this._textUnderline(baseX, baseY, lines[i], horizontalAlignment, verticalAlignment);\n                    }\n                    baseY = baseY + this.font.size;\n                }\n            } else {\n                this.context.fillText(text, baseX, baseY);\n                if (underline) {\n                    this._textUnderline(baseX, baseY, text, horizontalAlignment, verticalAlignment);\n                }\n            }\n\n        }\n        this.context.restore();\n        this.restoreTransform();\n    };\n\n    /**\n     * Get Text Extent\n     * @param {string} text\n     * @param {number} wordWrapWidth\n     * @return {Point}\n     */\n    Canvas.prototype.textExtent = function (text, wordWrapWidth) {\n        var self = this;\n        this.context.font = Canvas.toHTML5Font(this.font);\n        var sz = new Point(0, 0);\n        if (wordWrapWidth) {\n            var lines = this.wordWrap(text, wordWrapWidth),\n                widths = _.map(lines, function (line) { return self.context.measureText(line).width; });\n            sz.x = _.max(widths);\n            sz.y = this.font.size * lines.length;\n        } else {\n            sz.x = this.context.measureText(text).width;\n            sz.y = this.font.size;\n        }\n        return sz;\n    };\n\n    /**\n     * Draw Image\n     *\n     * @param {Image} image\n     * @param {number} x\n     * @param {number} y\n     * @param {number} width\n     * @param {number} height\n     */\n    Canvas.prototype.drawImage = function (image, x, y, width, height) {\n        this.transform();\n        this.context.drawImage(image, x, y, width, height);\n        this.restoreTransform();\n    };\n\n    // Type Definitions\n    _global.type.Font   = Font;\n    _global.type.Points = Points;\n\n    exports.DEFAULT_COLOR_PALETTE = DEFAULT_COLOR_PALETTE;\n\n    exports.AL_LEFT   = AL_LEFT;\n    exports.AL_RIGHT  = AL_RIGHT;\n    exports.AL_CENTER = AL_CENTER;\n    exports.AL_TOP    = AL_TOP;\n    exports.AL_BOTTOM = AL_BOTTOM;\n    exports.AL_MIDDLE = AL_MIDDLE;\n\n    exports.FS_NORMAL      = FS_NORMAL;\n    exports.FS_BOLD        = FS_BOLD;\n    exports.FS_ITALIC      = FS_ITALIC;\n    exports.FS_BOLD_ITALIC = FS_BOLD_ITALIC;\n\n    // Public API\n    exports.toRadian   = toRadian;\n    exports.toDegree   = toDegree;\n\n    exports.Point      = Point;\n    exports.Rect       = Rect;\n    exports.Color      = Color;\n    exports.Font       = Font;\n    exports.ZoomFactor = ZoomFactor;\n    exports.GridFactor = GridFactor;\n    exports.Points     = Points;\n    exports.Coord      = Coord;\n    exports.Canvas     = Canvas;\n\n});\n\n\n//# sourceURL=/core/Graphics.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define*/\n\ndefine(\'core/Toolkit\',[\'require\',\'exports\',\'module\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\'],function (require, exports, module) {\n    "use strict";\n\n    var Point      = require("core/Graphics").Point,\n        Rect       = require("core/Graphics").Rect,\n        Color      = require("core/Graphics").Color,\n        Font       = require("core/Graphics").Font,\n        ZoomFactor = require("core/Graphics").ZoomFactor,\n        GridFactor = require("core/Graphics").GridFactor,\n        Points     = require("core/Graphics").Points,\n        Coord      = require("core/Graphics").Coord,\n        Canvas     = require("core/Graphics").Canvas;\n\n    /**************************************************************************\n     *                                                                        *\n     *                       CONSTANTS AND ENUMERATIONS                       *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * Selection Orientation (North/West/South/East)\n     * @const {number}\n     */\n    var NWSE_N = 0,\n        NWSE_S = 1,\n        NWSE_W = 2,\n        NWSE_E = 3;\n\n    /**\n     * Highlighter Position\n     * @const {number}\n     */\n    var CT_LT   = 0,\n        CT_LM   = 1,\n        CT_LB   = 2,\n        CT_MT   = 3,\n        CT_MB   = 4,\n        CT_RT   = 5,\n        CT_RM   = 6,\n        CT_RB   = 7,\n        CT_AREA = 8,\n        CT_ELSE = 9;\n\n    /**\n     * Constants\n     * @const\n     */\n    var HIGHLIGHTER_COLOR             = Color.BLACK,\n        SELECTION_COLOR               = "#4f99ff", //Color.GREEN,\n        SELECTION_BORDER_COLOR        = "#4f99ff", // Color.GREEN,\n        DEFAULT_HIGHLIGHTER_SIZE      = 8.0,\n        DEFAULT_HALF_HIGHLIGHTER_SIZE = 4.0,\n        DEFAULT_SELECTIONLINE_WIDTH   = DEFAULT_HIGHLIGHTER_SIZE - 1.0;\n\n\n    function drawEndPoint(canvas, x, y) {\n        var r = 5;\n        canvas.ellipse(x - r, y - r, x + r, y + r);\n    }\n\n    function drawRange(canvas, x1, y1, x2, y2, kind, showEnds) {\n        if (!kind) {\n            kind = "rect";\n        }\n        if (!showEnds) {\n            showEnds = false;\n        }\n\n        x1 = Math.round(x1);\n        y1 = Math.round(y1);\n        x2 = Math.round(x2);\n        y2 = Math.round(y2);\n\n        canvas.storeState();\n        if (canvas.ratio === 1) { // Non-retina\n            canvas.context.translate(0.5, 0.5);\n        }\n        canvas.color = SELECTION_COLOR;\n        switch (kind) {\n        case "rect":\n            canvas.rect(x1, y1, x2, y2);\n            if (showEnds) {\n                drawEndPoint(canvas, x1, y1);\n                drawEndPoint(canvas, x2, y2);\n            }\n            break;\n        case "line":\n            canvas.line(x1, y1, x2, y2);\n            if (showEnds) {\n                drawEndPoint(canvas, x1, y1);\n                drawEndPoint(canvas, x2, y2);\n            }\n            break;\n        case "point":\n            drawEndPoint(canvas, x2, y2);\n            break;\n        }\n        if (canvas.ratio === 1) { // Non-retina\n            canvas.context.translate(-0.5, -0.5);\n        }\n        canvas.restoreState();\n    }\n\n    function drawRangeBox(canvas, x1, y1, x2, y2) {\n        canvas.storeState();\n        canvas.color = SELECTION_COLOR;\n        canvas.rect(x1 - 2, y1 - 2, x2 + 3, y2 + 3);\n        canvas.restoreState();\n    }\n\n    function drawDottedLine(canvas, points) {\n        canvas.storeState();\n        if (canvas.ratio === 1) { // Non-retina\n            canvas.context.translate(0.5, 0.5);\n        }\n        canvas.color = SELECTION_COLOR;\n        canvas.polyline(points.points, [3]);\n        if (canvas.ratio === 1) { // Non-retina\n            canvas.context.translate(-0.5, -0.5);\n        }\n        canvas.restoreState();\n    }\n\n    function drawSelection(canvas, x1, y1, x2, y2) {\n        canvas.storeState();\n        if (canvas.ratio === 1) {\n            canvas.context.translate(0.5, 0.5);\n        }\n        canvas.fillColor = SELECTION_COLOR;\n        // canvas.lineWidth = 1.0 * canvas.ratio;\n        canvas.alpha = 0.1;\n        canvas.fillRoundRect(x1, y1, x2, y2, 2);\n        canvas.color = SELECTION_BORDER_COLOR;\n        canvas.alpha = 1;\n        canvas.roundRect(x1, y1, x2, y2, 2);\n        if (canvas.ratio === 1) {\n            canvas.context.translate(-0.5, -0.5);\n        }\n        canvas.restoreState();\n    }\n\n    function drawSelectionBox(canvas, x1, y1, x2, y2) {\n        canvas.storeState();\n        if (canvas.ratio === 1) {\n            canvas.context.translate(0.5, 0.5);\n        }\n        canvas.color = SELECTION_BORDER_COLOR;\n        canvas.alpha = 1;\n        // canvas.lineWidth = 1.0 * canvas.ratio;\n        canvas.rect(x1, y1, x2, y2);\n        if (canvas.ratio === 1) {\n            canvas.context.translate(-0.5, -0.5);\n        }\n        canvas.restoreState();\n    }\n\n    function drawSelectionLine(canvas, x1, y1, x2, y2, width, nwse, retouch) {\n        var ox, oy, r, r1, r2, rx1, rx2, ry1, ry2;\n        canvas.storeState();\n        ox = canvas.origin.x;\n        oy = canvas.origin.y;\n        r1 = new Point(x1, y1);\n        r2 = new Point(x2, y2);\n        Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, r1);\n        Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, r2);\n\n        // for Retina\n        canvas.lineWidth = 1.0 * canvas.ratio;\n        rx1 = r1.x * canvas.ratio;\n        ry1 = r1.y * canvas.ratio;\n        rx2 = r2.x * canvas.ratio;\n        ry2 = r2.y * canvas.ratio;\n        width = width * canvas.ratio;\n\n        if (retouch && (((rx1 + ox - width) % 2) === 1)) {\n            rx1 = rx1 + 1;\n        }\n        if (retouch && (((rx2 + ox) % 2) === 1)) {\n            rx2 = rx2 - 1;\n        }\n        r = new Rect(0, 0, 0, 0);\n        switch (nwse) {\n        case NWSE_N:\n            r.setRect(rx1 + ox, ry1 + oy - width, rx2 + ox, ry1 + oy);\n            break;\n        case NWSE_W:\n            r.setRect(rx1 + ox - width, ry1 + oy - width, rx1 + ox, ry2 + oy + width);\n            break;\n        case NWSE_S:\n            r.setRect(rx1 + ox, ry2 + oy, rx2 + ox, ry2 + oy + width);\n            break;\n        case NWSE_E:\n            r.setRect(rx2 + ox, ry1 + oy - width, rx2 + ox + width, ry2 + oy + width);\n        }\n        r.quantize();\n\n        canvas.fillColor = SELECTION_COLOR;\n        canvas.alpha = 0.1;\n        canvas.coordTransformApplied = false;\n\n        if (canvas.ratio === 1) {\n            canvas.context.translate(0.5, 0.5);\n        }\n\n        canvas.fillRect(r.x1, r.y1, r.x2, r.y2);\n\n        if (canvas.ratio === 1) {\n            canvas.context.translate(-0.5, -0.5);\n        }\n\n        canvas.coordTransformApplied = true;\n        canvas.restoreState();\n    }\n\n    function drawHighlighter(canvas, x, y, size, enabled, color) {\n        var o, p;\n        canvas.storeState();\n        canvas.color = SELECTION_BORDER_COLOR;\n        canvas.fillColor = Color.WHITE;\n        canvas.lineWidth = 1.0 * canvas.ratio;\n        p = new Point(x, y);\n\n        // for Retina\n        p.x = p.x * canvas.ratio;\n        p.y = p.y * canvas.ratio;\n        size = size * canvas.ratio;\n        p.quantize();\n\n        o = canvas.origin.copy();\n        Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, p);\n        canvas.coordTransformApplied = false;\n\n        if (canvas.ratio === 1) {\n            canvas.context.translate(0.5, 0.5);\n        }\n        canvas.fillRect(p.x + o.x - size, p.y + o.y - size, p.x + o.x + size, p.y + o.y + size);\n        canvas.rect(p.x + o.x - size, p.y + o.y - size, p.x + o.x + size, p.y + o.y + size);\n\n        if (canvas.ratio === 1) {\n            canvas.context.translate(-0.5, -0.5);\n        }\n        canvas.coordTransformApplied = true;\n        canvas.restoreState();\n    }\n\n    function drawHighlighter2(canvas, x1, y1, x2, y2, width, cornerType, enabled, color) {\n        var cx, cy, halfW, ox, oy, p, p1, p2;\n        canvas.storeState();\n        canvas.color = color;\n        ox = canvas.origin.x;\n        oy = canvas.origin.y;\n        p1 = new Point(x1, y1);\n        p2 = new Point(x2, y2);\n        Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, p1);\n        Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, p2);\n\n        // for Retina\n        x1 = p1.x * canvas.ratio;\n        y1 = p1.y * canvas.ratio;\n        x2 = p2.x * canvas.ratio;\n        y2 = p2.y * canvas.ratio;\n        width = width * canvas.ratio;\n\n        p = new Point(-100, -100);\n        cx = (x1 + x2) / 2;\n        cy = (y1 + y2) / 2;\n        halfW = width / 2;\n        switch (cornerType) {\n        case CT_LT:\n            p.setPoint(x1 - width, y1 - width);\n            break;\n        case CT_LM:\n            p.setPoint(x1 - width, cy - halfW);\n            break;\n        case CT_LB:\n            p.setPoint(x1 - width, y2);\n            break;\n        case CT_MT:\n            p.setPoint(cx - halfW, y1 - width);\n            break;\n        case CT_MB:\n            p.setPoint(cx - halfW, y2);\n            break;\n        case CT_RT:\n            p.setPoint(x2, y1 - width);\n            break;\n        case CT_RM:\n            p.setPoint(x2, cy - halfW);\n            break;\n        case CT_RB:\n            p.setPoint(x2, y2);\n        }\n        if (enabled) {\n            canvas.fillColor = Color.WHITE;\n        } else {\n            canvas.fillColor = "#D0D0D0";\n        }\n        canvas.lineWidth = 1.0 * canvas.ratio;\n\n        p.quantize();\n\n        canvas.color = "#4f99ff"; // Color.BLACK;\n        canvas.coordTransformApplied = false;\n\n        if (canvas.ratio === 1) {\n            canvas.context.translate(0.5, 0.5);\n        }\n\n        canvas.fillRect(p.x + ox, p.y + oy, p.x + ox + width, p.y + oy + width);\n        canvas.rect(p.x + ox, p.y + oy, p.x + ox + width, p.y + oy + width);\n\n        if (canvas.ratio === 1) {\n            canvas.context.translate(-0.5, -0.5);\n        }\n\n        canvas.coordTransformApplied = true;\n        canvas.restoreState();\n    }\n\n    function drawLineGuide(canvas, points) {\n        var o, p;\n        canvas.storeState();\n        canvas.color = SELECTION_BORDER_COLOR;\n        canvas.fillColor = Color.WHITE;\n        canvas.lineWidth = 1.0;\n\n        o = canvas.origin.copy();\n        canvas.coordTransformApplied = false;\n        canvas.context.translate(0.5, 0.5);\n        for (var i = 0, len = points.length; i < len; i++) {\n            p = new Point(points[i].x, points[i].y);\n            Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, p);\n            if (i === 0) {\n\n            } else {\n\n            }\n        }\n        canvas.context.translate(-0.5, -0.5);\n        canvas.coordTransformApplied = true;\n        canvas.restoreState();\n    }\n\n\n    // Constants\n    exports.NWSE_N = NWSE_N;\n    exports.NWSE_S = NWSE_S;\n    exports.NWSE_W = NWSE_W;\n    exports.NWSE_E = NWSE_E;\n\n    exports.CT_LT   = CT_LT;\n    exports.CT_LM   = CT_LM;\n    exports.CT_LB   = CT_LB;\n    exports.CT_MT   = CT_MT;\n    exports.CT_MB   = CT_MB;\n    exports.CT_RT   = CT_RT;\n    exports.CT_RM   = CT_RM;\n    exports.CT_RB   = CT_RB;\n    exports.CT_AREA = CT_AREA;\n    exports.CT_ELSE = CT_ELSE;\n\n    exports.HIGHLIGHTER_COLOR             = HIGHLIGHTER_COLOR;\n    exports.SELECTION_COLOR               = SELECTION_COLOR;\n    exports.DEFAULT_HIGHLIGHTER_SIZE      = DEFAULT_HIGHLIGHTER_SIZE;\n    exports.DEFAULT_HALF_HIGHLIGHTER_SIZE = DEFAULT_HALF_HIGHLIGHTER_SIZE;\n    exports.DEFAULT_SELECTIONLINE_WIDTH   = DEFAULT_SELECTIONLINE_WIDTH;\n\n    // Public API\n    exports.drawEndPoint      = drawEndPoint;\n    exports.drawRange         = drawRange;\n    exports.drawRangeBox      = drawRangeBox;\n    exports.drawDottedLine    = drawDottedLine;\n    exports.drawSelection     = drawSelection;\n    exports.drawSelectionBox  = drawSelectionBox;\n    exports.drawSelectionLine = drawSelectionLine;\n    exports.drawHighlighter   = drawHighlighter;\n    exports.drawHighlighter2  = drawHighlighter2;\n    exports.drawLineGuide     = drawLineGuide;\n\n});\n\n\n//# sourceURL=/core/Toolkit.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, _*/\n\ndefine(\'core/MetaModelManager\',[\'require\',\'exports\',\'module\',\'core/Global\'],function (require, exports, module) {\n    "use strict";\n\n    var _global    = require("core/Global").global;\n\n    /**\n     * Error Messages\n     */\n    var ERR_METATYPE_OBJECT      = \'"<%=name%>" should be an object\',\n        ERR_FIELD_TYPE           = \'"<%=metaType.name%>.<%=field%>" should be "<%=type%>" type\',\n        ERR_REQUIRE_FIELD        = \'"<%=metaType.name%>" requires field "<%=field%>"\',\n        ERR_METATYPE_KIND        = \'"<%=metaType.name%>.kind" should be "enum" or "class"\',\n        ERR_DUPLICATED_LITERAL   = \'"<%=metaType.name%>" has duplicated literal "<%=literal%>"\',\n        ERR_ENUM_HAS_LITERALS    = \'"<%=metaType.name%>" should have at least one or more literals\',\n        ERR_TYPE_NOT_FOUND       = \'"<%=type%>" is not found specified in "<%=metaType.name%>.<%=field%>"\',\n        ERR_UNNAMED_ATTRIBUTE    = \'"<%=metaType.name%>" has unnamed attribute\',\n        ERR_DUPLICATED_ATTRIBUTE = \'"<%=metaType.name%>" has duplicated attribute "<%=attribute.name%>"\',\n        ERR_ATTRIBUTE_KIND       = \'"<%=metaType.name%>.<%=attribute.name%>.kind" should be "prim", "enum", "var", "ref", "refs", "obj", "objs", or "custom"\',\n        ERR_ATTRIBUTE_PRIM_TYPE  = \'"<%=metaType.name%>.<%=attribute.name%>.type" should be "Integer", "String", "Boolean", or "Real"\';\n\n    /**\n     * Assertion\n     * @param {boolean} condition\n     * @param {string} message\n     */\n    function assert(condition, message, strings) {\n        if (!condition) {\n            var err = _.template(message);\n            throw "[MetaModelManager] " + err(strings);\n        }\n    }\n\n    /**\n     * Validate MetaType Definition\n     * @param {Object} metaType\n     */\n    function validateMetaType(name, metaType) {\n        assert(_.isObject(metaType), ERR_METATYPE_OBJECT, { name: name });\n        assert(metaType.name, ERR_REQUIRE_FIELD, { metaType: metaType, field: "name" });\n        assert(metaType.kind === "enum" || metaType.kind === "class", ERR_METATYPE_KIND, { metaType: metaType });\n\n        // Enum MetaType\n        if (metaType.kind === "enum") {\n            var lits = metaType.literals;\n            assert(Array.isArray(lits) && lits.length > 0, ERR_ENUM_HAS_LITERALS, { metaType: metaType });\n\n            // Check duplicated literal\n            _.each(lits, function (lit1) {\n                var len = _.filter(lits, function (lit2) { return lit2 === lit1; }).length;\n                assert(len === 1, ERR_DUPLICATED_LITERAL, { metaType: metaType, literal: lit1 });\n            });\n        }\n\n        // Class MetaType\n        if (metaType.kind === "class") {\n\n            // Check .super\n            if (metaType.super) {\n                assert(metaType.super && _global.meta[metaType.super], ERR_TYPE_NOT_FOUND, { metaType: metaType, type: metaType.super, field: "super" });\n            }\n\n            // Check Attributes\n            if (metaType.attributes) {\n                assert(Array.isArray(metaType.attributes), ERR_FIELD_TYPE, { metaType: metaType, field: "attributes", type: "Array" });\n                _.each(metaType.attributes, function (attribute) {\n\n                    // check \'name\'\n                    assert(attribute.name, ERR_UNNAMED_ATTRIBUTE, { metaType: metaType });\n\n                    // Check \'kind\'\n                    assert(_.contains(["prim", "enum", "var", "ref", "refs", "obj", "objs", "custom"], attribute.kind), ERR_ATTRIBUTE_KIND, { metaType: metaType, attribute: attribute });\n\n                    // Check \'type\'\n                    if (attribute.kind === "prim") {\n                        assert(_.contains(["Integer", "String", "Boolean", "Real"], attribute.type), ERR_ATTRIBUTE_PRIM_TYPE, { metaType: metaType, attribute: attribute });\n                    } else if (attribute.kind !== "custom") {\n                        assert(_global.meta[attribute.type], ERR_TYPE_NOT_FOUND, { metaType: metaType, type: attribute.type, field: attribute.name + ".type" });\n                    }\n\n                    // Check duplicated attribute in all inherited attributes\n                    var len = _.filter(getMetaAttributes(metaType.name), function (attr) { return attr.name === attribute.name; }).length;\n                    assert(len === 1, ERR_DUPLICATED_ATTRIBUTE, { metaType: metaType, attribute: attribute });\n                });\n            }\n        }\n    }\n\n    /**\n     * Register Metamodel by Object\n     * @param {Object} metamodel\n     */\n    function register(metamodel) {\n        var name, metaType;\n\n        // Registering MetaTypes to global.meta\n        for (name in metamodel) {\n            if (metamodel.hasOwnProperty(name)) {\n                metaType = metamodel[name];\n                metaType.name = name;\n\n                // Check duplicated MetaType\n                if (_global.meta[name]) {\n                    delete metamodel[name];\n                    console.error("[MetaModelManager] MetaType \'" + name + "\' is already exists.");\n                } else {\n                    _global.meta[name] = metaType;\n                }\n            }\n        }\n\n        // Validate MetaTypes\n        for (name in metamodel) {\n            if (metamodel.hasOwnProperty(name)) {\n                metaType = metamodel[name];\n                try {\n                    validateMetaType(name, metaType);\n                } catch (err) {\n                    console.error(err);\n                }\n            }\n        }\n    }\n\n    /**\n     * Return all meta-attributes\n     * @param {string} typeName\n     * @return {Array.<{name:string, kind:string, type:string}>}\n     */\n    function getMetaAttributes(typeName) {\n        var metaClass = _global.meta[typeName],\n            attrs     = [];\n        if (metaClass.super) {\n            attrs = getMetaAttributes(metaClass.super);\n        }\n        if (metaClass.attributes) {\n            var i, len, item;\n            for (i = 0, len = metaClass.attributes.length; i < len; i++) {\n                item = metaClass.attributes[i];\n                attrs.push(item);\n            }\n        }\n        return attrs;\n    }\n\n    /**\n     * Type test: is-kind-of\n     * @param {string} child\n     * @param {string} parent\n     * @return {boolean}\n     */\n    function isKindOf(child, parent) {\n        if (!_global.meta[child]) {\n            return false;\n        } else if (_global.meta[child] === _global.meta[parent]) {\n            return true;\n        } else {\n            return isKindOf(_global.meta[child].super, parent);\n        }\n    }\n\n\n    /**\n     * Return a corresponding view type of a given model type.\n     * @param {string} typeName\n     * @return {string}\n     */\n    function getViewTypeOf(typeName) {\n        var metaClass = _global.meta[typeName];\n        if (metaClass) {\n            return metaClass.view || null;\n        }\n        return null;\n    }\n\n    /**\n     * Return all available view types of a diagram type.\n     * @param {string} diagramTypeName\n     * @return {Array.<string}>}\n     */\n    function getAvailableViewTypes(diagramTypeName) {\n        var metaClass = _global.meta[diagramTypeName],\n            views     = [];\n        if (metaClass.super) {\n            views = getAvailableViewTypes(metaClass.super);\n        }\n        if (metaClass.views) {\n            var i, len, item;\n            for (i = 0, len = metaClass.views.length; i < len; i++) {\n                item = metaClass.views[i];\n                views.push(item);\n            }\n        }\n        return views;\n    }\n\n    exports.register              = register;\n    exports.getMetaAttributes     = getMetaAttributes;\n    exports.isKindOf              = isKindOf;\n    exports.getViewTypeOf         = getViewTypeOf;\n    exports.getAvailableViewTypes = getAvailableViewTypes;\n\n});\n\n\n//# sourceURL=/core/MetaModelManager.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, localStorage */\n\n/**\n * PreferenceManager only works in browser (it uses global localStorage object).\n */\ndefine(\'core/PreferenceManager\',[\'require\',\'exports\',\'module\',\'core/Global\'],function (require, exports, module) {\n    "use strict";\n\n    var global = require("core/Global").global;\n\n    /**\n     * Preference Schema Map\n     * @type {Object.<string,{name:string, schema:Object}>}\n     */\n    var _schemaMap = {};\n\n    /**\n     * Preference Item Map\n     * @type {Object.<string,Object>}\n     */\n    var _itemMap = {};\n\n    function getSchema(id) {\n        return _schemaMap[id].schema;\n    }\n\n    function getSchemaName(id) {\n        return _schemaMap[id].name;\n    }\n\n    function getSchemaIds() {\n        return _.keys(_schemaMap);\n    }\n\n    function getItem(key) {\n        return _itemMap[key];\n    }\n\n    /**\n     * Validate preference schema\n     * @param {Object} schema\n     * @return {boolean}\n     */\n    function validate(schema) {\n        var key, item;\n        for (key in schema) {\n            if (schema.hasOwnProperty(key)) {\n                item = schema[key];\n                if (!item.text) {\n                    console.error("[PreferenceManager] missing required field: \'text\' of \'" + key + "\'");\n                }\n                if (!item.type) {\n                    console.error("[PreferenceManager] missing required field: \'type\' of \'" + key + "\'");\n                }\n                if (item.type !== "Section" && typeof item[\'default\'] === "undefined") {\n                    console.error("[PreferenceManager] missing required field: \'default\' of \'" + key + "\'");\n                }\n                if (item.type === "Combo" || item.type === "Dropdown") {\n                    if (item.options && item.options.length > 0) {\n                        var i, len;\n                        for (i = 0, len = item.options.length; i < len; i++) {\n                            if (typeof item.options[i].value === "undefined") {\n                                console.error("[PreferenceManager] missing required field of option item: \'value\' of \'" + key + "\'");\n                            }\n                            if (typeof item.options[i].text === "undefined") {\n                                console.error("[PreferenceManager] missing required field of option item: \'text\' of \'" + key + "\'");\n                            }\n                        }\n                    } else {\n                        console.error("[PreferenceManager] missing required field or no items: \'options\' of \'" + key + "\'");\n                    }\n                }\n            }\n        }\n        return true;\n    }\n\n    /**\n     * Register preference schema\n     * @param {string} id\n     * @param {string} name\n     * @param {Object} schema\n     */\n    function register(id, name, schema) {\n        if (!id || !name || !schema) {\n            console.error("register(): missing required parameters: id, name, or schema");\n            return;\n        }\n\n        if (validate(schema)) {\n            _schemaMap[id] = {\n                id: id,\n                name: name,\n                schema: schema\n            };\n\n            // Build Preference Item Map\n            _.each(schema, function (item, key) {\n                if (item) {\n                    _itemMap[key] = item;\n                }\n            });\n        }\n    }\n\n    /**\n     * Return value of key\n     *\n     * @param {string} key\n     * @param {?} defaultValue\n     * @return {?}\n     */\n    function get(key, defaultValue) {\n        defaultValue = typeof defaultValue === "undefined" ? null : defaultValue;\n        if (global.localStorage) {\n            var _value = global.localStorage.getItem(key),\n                value = null;\n            if (_value) {\n                try {\n                    value = JSON.parse(_value);\n                } catch (e) {\n                    console.error("[PreferenceManager] Failed to read preference value of key: " + key);\n                }\n            } else {\n                // if not stored in localStorage, return default value from schema\n                if (_itemMap[key] && typeof _itemMap[key][\'default\'] !== "undefined") {\n                    value = _itemMap[key][\'default\'];\n                } else {\n                    value = defaultValue;\n                }\n            }\n            return value;\n        } else {\n            return defaultValue;\n        }\n    }\n\n    /**\n     * Change value of key\n     * @param {string} key\n     * @param {?} value\n     */\n    function set(key, value) {\n        if (global.localStorage) {\n            var _value;\n            try {\n                _value = JSON.stringify(value);\n                global.localStorage.setItem(key, _value);\n                $(exports).triggerHandler("change", [key, value]);\n            } catch (e) {\n                console.error("[PreferenceManager] Failed to write preference value of key: " + key);\n            }\n        }\n    }\n\n    /**\n     * Convenience function that gets a view state\n     *\n     * @param {string} id preference to get\n     * @param {?Object} context Optional additional information about the request\n     */\n    function getViewState(id, context) {\n        var state = get("_viewState." + id);\n        if (!state) {\n            state = {};\n        }\n        return state;\n    }\n\n    /**\n     * Convenience function that sets a view state and then saves the file\n     *\n     * @param {string} id preference to set\n     * @param {*} value new value for the preference\n     * @param {?Object} context Optional additional information about the request\n     * @param {boolean=} doNotSave If it is undefined or false, then save the\n     *      view state immediately.\n     */\n    function setViewState(id, value, context, doNotSave) {\n        set("_viewState." + id, value);\n    }\n\n    // Public API\n    exports.getSchema     = getSchema;\n    exports.getSchemaName = getSchemaName;\n    exports.getSchemaIds  = getSchemaIds;\n    exports.getItem       = getItem;\n    exports.register      = register;\n    exports.get           = get;\n    exports.set           = set;\n    exports.getViewState  = getViewState;\n    exports.setViewState  = setViewState;\n\n});\n\n//# sourceURL=/core/PreferenceManager.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*\n * Meta Class Definition\n * - kind: \'class\', \'enum\', or \'datatype\'\n * - super? : super meta class\n * - attributes : []\n * - constraints : {\n *     allowedParents: [ {type: \'UMLClassifier\', field: \'ownedElements\' }, ... ]\n *     correspondViews: [ {diagram: \'UMLClassDiagram\', viewType: \'UMLClass\' }, ... ]\n *   }\n *\n * Meta Attribute Definition\n * - name: 속성의 이름\n * - kind: 속성의 종류 (ATTR_KIND_*)\n * - type: 속성의 타입 (String등의 기본 타입에서 다른 클래스명) Allowed Primitive Types = String | Boolean | Integer | Real\n * - default: 속성의 기본 값 (optional)\n * - visible: 속성의 가시성 (PropertyEditor에 보일것인가를 결정) (optional)\n * - readOnly: 읽기전용 속성 (PropertyEditor에서 사용자에 의해 편집 불가) (optional)\n * - transient: 속성이 저장될것인가 (e.g. View\'s selected 속성은 저장되면 안됨) (optional)\n */\n\n/**\n * Core module\n * Meta Class Definition\n * - super? : super meta class\n * - attributes : []\n * - constraints : {\n *     allowedParents: [ {type: \'UMLClassifier\', field: \'ownedElements\' }, ... ]\n *     correspondViews: [ {diagram: \'UMLClassDiagram\', viewType: \'UMLClass\' }, ... ]\n *   }\n *\n * 메타 속성의 정의\n * - name: 속성의 이름\n * - kind: 속성의 종류 (ATTR_KIND_*)\n * - type: 속성의 타입 (String등의 기본 타입에서 다른 클래스명)\n * - default: 속성의 기본 값 (optional)\n * - visible: 속성의 가시성 (PropertyEditor에 보일것인가를 결정) (optional)\n * - readOnly: 읽기전용 속성 (PropertyEditor에서 사용자에 의해 편집 불가) (optional)\n * - transient: 속성이 저장될것인가 (e.g. View\'s selected 속성은 저장되면 안됨) (optional)\n * @exports core/Core\n */\n\n/**\n * NOTE:\n * Requires graphlib and dagre as global variables, which are used only in Diagram.layout().\n * If not exists, Diagram.layout() will not work with no errors.\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, loopfunc: true */\n/*global define, _, type, graphlib, dagre */\n\n/**\n * Core classes is defined in this module. We don\'t recommend to instantiate\n *   instance of classes defined in this module in your extension.\n *\n * (This module is not well documented)\n */\ndefine(\'core/Core\',[\'require\',\'exports\',\'module\',\'core/Global\',\'core/IdGenerator\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Toolkit\',\'core/MetaModelManager\',\'core/PreferenceManager\'],function (require, exports, module) {\n    "use strict";\n\n    var _global           = require("core/Global").global,\n        IdGenerator       = require("core/IdGenerator"),\n        Graphics          = require("core/Graphics"),\n        Point             = require("core/Graphics").Point,\n        Rect              = require("core/Graphics").Rect,\n        Color             = require("core/Graphics").Color,\n        Font              = require("core/Graphics").Font,\n        ZoomFactor        = require("core/Graphics").ZoomFactor,\n        GridFactor        = require("core/Graphics").GridFactor,\n        Points            = require("core/Graphics").Points,\n        Coord             = require("core/Graphics").Coord,\n        Canvas            = require("core/Graphics").Canvas,\n        Toolkit           = require("core/Toolkit"),\n        MetaModelManager  = require("core/MetaModelManager"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    /**\n     * Attribute Kind\n     * @const {string}\n     */\n    var ATTR_KIND_PRIM   = \'prim\',\n        ATTR_KIND_ENUM   = \'enum\',\n        ATTR_KIND_REF    = \'ref\',\n        ATTR_KIND_REFS   = \'refs\',\n        ATTR_KIND_OBJ    = \'obj\',\n        ATTR_KIND_OBJS   = \'objs\',\n        ATTR_KIND_VAR    = \'var\',\n        ATTR_KIND_CUSTOM = \'custom\';\n\n    /**\n     * Selectable Kind\n     * @enum {number}\n     */\n    var SK_NO        = 0,\n        SK_YES       = 1,\n        SK_PROPAGATE = 2;\n\n    /**\n     * Sizing Mode\n     * @enum {number}\n     */\n    var SZ_NONE  = 0,\n        SZ_HORZ  = 1,\n        SZ_VERT  = 2,\n        SZ_RATIO = 3,\n        SZ_FREE  = 4;\n\n    /**\n     * Move Mode\n     * @enum {number}\n     */\n    var MM_NONE = 0,\n        MM_HORZ = 1,\n        MM_VERT = 2,\n        MM_FREE = 3;\n\n    /**\n     * Line Mode\n     * @enum {number}\n     */\n    var LM_SOLID = 0,\n        LM_DOT   = 1;\n\n    /**\n     * Line Style\n     * @enum {number}\n     */\n    var LS_RECTILINEAR = 0,\n        LS_OBLIQUE     = 1,\n        LS_ROUNDRECT   = 2,\n        LS_CURVE       = 3;\n\n    /**\n     * Line End Style\n     * @enum {number}\n     */\n    var ES_FLAT                 = 0,\n        ES_STICK_ARROW          = 1,\n        ES_SOLID_ARROW          = 2,\n        ES_TRIANGLE             = 3,\n        ES_FILLED_TRIANGLE      = 4,\n        ES_DIAMOND              = 5,\n        ES_FILLED_DIAMOND       = 6,\n        ES_ARROW_DIAMOND        = 7,\n        ES_ARROW_FILLED_DIAMOND = 8,\n        ES_PLUS                 = 9,\n        ES_CIRCLE               = 10,\n        ES_CIRCLE_PLUS          = 11,\n        ES_CROWFOOT_ONE         = 12,\n        ES_CROWFOOT_MANY        = 13,\n        ES_CROWFOOT_ZERO_ONE    = 14,\n        ES_CROWFOOT_ZERO_MANY   = 15;\n\n    /**\n     * Edge Position\n     * @enum {number}\n     */\n    var EP_HEAD   = 0,\n        EP_MIDDLE = 1,\n        EP_TAIL   = 2;\n\n    /**\n     * Direction Kind\n     * @enum {number}\n     */\n    var DK_HORZ = 0,\n        DK_VERT = 1;\n\n\n    /**\n     * Diagram Layout Direction\n     */\n    var DIRECTION_TB = "TB",\n        DIRECTION_BT = "BT",\n        DIRECTION_LR = "LR",\n        DIRECTION_RL = "RL";\n\n\n    /**\n     * Constants for Layout\n     */\n    var LAYOUT_MARGIN_LEFT = 20,\n        LAYOUT_MARGIN_TOP  = 20,\n        NODE_SEPARATION    = 30,\n        EDGE_SEPARATION    = 30,\n        RANK_SEPARATION    = 30;\n\n    /**\n     * TagKind\n     * @enum\n     */\n    var TK_STRING    = \'string\',\n        TK_REFERENCE = \'reference\',\n        TK_BOOLEAN   = \'boolean\',\n        TK_NUMBER    = \'number\',\n        TK_HIDDEN    = \'hidden\';\n\n    /**\n     * Element\n     * @constructor\n     */\n    function Element() {\n\n        /**\n         * gloly unique identifier\n         * @member {string}\n         */\n        this._id = IdGenerator.generateGuid();\n\n        /**\n         * reference to its parent\n         * @member {object}\n         */\n        this._parent = null;\n    }\n\n    /**\n     * Return class name for display (e.g. "Class" rather than "UMLClass").\n     * @return {string}\n     */\n    Element.prototype.getDisplayClassName = function () {\n        return this.getClassName();\n    };\n\n    /**\n     * Return Class. (equivalent to `type[element.getClassName()])`\n     *\n     * @return {constructor}\n     */\n    Element.prototype.getClass = function () {\n        return _global.type[this.constructor.name];\n    };\n\n    /**\n     * Return Class name.\n     *\n     * @return {string}\n     */\n    Element.prototype.getClassName = function () {\n        return this.constructor.name;\n    };\n\n    /**\n     * Return Meta Class.\n     *\n     * @return {Object}\n     */\n    Element.prototype.getMetaClass = function () {\n        return _global.meta[this.constructor.name];\n    };\n\n    /**\n     * Return Meta Attributes.\n     *\n     * @return {Array.<{name:string, kind:string, type:string}>}\n     */\n    Element.prototype.getMetaAttributes = function () {\n        return MetaModelManager.getMetaAttributes(this.getClassName());\n    };\n\n    /**\n     * Return name of parent\'s field containing this object.\n     *\n     * @return {string} Field name (e.g. \'ownedElements\')\n     */\n    Element.prototype.getParentField = function () {\n        if (this._parent) {\n            for (var field in this._parent) {\n                var value = this._parent[field];\n                if (_.isArray(value)) {\n                    if (_.contains(value, this)) { return field; }\n                } else if (_.isObject(value)) {\n                    if (value === this) { return field; }\n                }\n            }\n            return null;\n        }\n        return null;\n    };\n\n    /**\n     * Return CSS class name of icon for this object. (Shown in Explorer)\n     *\n     * @return {string} - iconClass명 (e.g. \'icon-UMLClass\')\n     */\n    Element.prototype.getNodeIcon = function () {\n        return "icon-" + this.getClassName();\n    };\n\n    /**\n     * Return textual string for this object. (Shown in Explorer)\n     * @param {object} options\n     * @return {string} Text (e.g. \'<<stereotype>>Class1\')\n     */\n    Element.prototype.getNodeText = function (options) {\n        if (this.name && this.name.length > 0) {\n            return this.name;\n        } else {\n            return "(" + this.getDisplayClassName() + ")";\n        }\n    };\n\n    /**\n     * Return ordering priority number. (Lower value comes first in Explorer)\n     *\n     * @param {number} index\n     * @return {Number}\n     */\n    Element.prototype.getOrdering = function (index) {\n        var base      = 100000,\n            ordering  = 0,\n            metaClass = this.getMetaClass();\n        if (metaClass && metaClass.ordering) {\n            ordering = metaClass.ordering * base;\n        }\n        if (_.isNumber(index)) {\n            ordering = ordering + index;\n        }\n        return ordering;\n    };\n\n    /**\n     * Return all child nodes to be shown in Explorer.\n     *\n     * @param {boolean} sort\n     * @return {Array.<Element>}\n     */\n    Element.prototype.getChildNodes = function (sort) {\n        var children = [];\n        var self = this;\n\n        function push(elem) {\n            if (elem instanceof Model) {\n                children.push(elem);\n            }\n        }\n\n        _.forEach(this.getMetaAttributes(), function (attr) {\n            switch (attr.kind) {\n            case ATTR_KIND_OBJ:\n                if (self[attr.name]) {\n                    push(self[attr.name]);\n                }\n                break;\n            case ATTR_KIND_OBJS:\n                var items = self[attr.name];\n                if (items && items.length > 0) {\n                    _.forEach(items, function (item) {\n                        push(item);\n                    });\n                }\n                break;\n            }\n        });\n\n        if (sort) {\n            children = _.sortBy(children, function (child, idx) {\n                return child.getOrdering(idx);\n            });\n        }\n\n        return children;\n    };\n\n\n    /**\n     * Return all children.\n     *\n     * @return {Array.<Element>}\n     */\n    Element.prototype.getChildren = function () {\n        var children = [];\n        var self = this;\n        _.forEach(this.getMetaAttributes(), function (attr) {\n            switch (attr.kind) {\n            case ATTR_KIND_OBJ:\n                if (self[attr.name]) {\n                    children.push(self[attr.name]);\n                }\n                break;\n            case ATTR_KIND_OBJS:\n                var items = self[attr.name];\n                if (items && items.length > 0) {\n                    _.forEach(items, function (item) {\n                        children.push(item);\n                    });\n                }\n                break;\n            }\n        });\n        return children;\n    };\n\n    /**\n     * Traverse all nodes in the tree structure. (Breadth-First Traversal)\n     *\n     * @param {function(elem:Element)} fun\n     */\n    Element.prototype.traverse = function (fun) {\n        fun(this);\n        var attrs = this.getMetaAttributes();\n        for (var i = 0, len = attrs.length; i < len; i++) {\n            var attr = attrs[i];\n            if (this[attr.name] !== null) {\n                switch (attr.kind) {\n                case ATTR_KIND_OBJ:\n                    this[attr.name].traverse(fun);\n                    break;\n                case ATTR_KIND_OBJS:\n                    for (var j = 0, len1 = this[attr.name].length; j < len1; j++) {\n                        var obj = this[attr.name][j];\n                        obj.traverse(fun);\n                    }\n                    break;\n                }\n            }\n        }\n    };\n\n    /**\n     * Traverse all nodes in the tree structure. (Depth-First Traversal)\n     *\n     * @param {function(elem:Element)} fun\n     */\n    Element.prototype.traverseDepthFirst = function (fun) {\n        var attrs = this.getMetaAttributes();\n        for (var i = 0, len = attrs.length; i < len; i++) {\n            var attr = attrs[i];\n            if (this[attr.name] !== null) {\n                switch (attr.kind) {\n                case ATTR_KIND_OBJ:\n                    this[attr.name].traverseDepthFirst(fun);\n                    break;\n                case ATTR_KIND_OBJS:\n                    for (var j = 0, len1 = this[attr.name].length; j < len1; j++) {\n                        var obj = this[attr.name][j];\n                        obj.traverseDepthFirst(fun);\n                    }\n                    break;\n                }\n            }\n        }\n        fun(this);\n    };\n\n    /**\n     * Traverse a specific field chains. (Breadth-First Traversal)\n     *\n     * @param {string} field Field name to traverse (e.g. \'ownedElements\')\n     * @param {function} fun\n     */\n    Element.prototype.traverseField = function (field, fun) {\n        fun(this);\n        var ref = this[field];\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var v = ref[i];\n            v.traverseField(field, fun);\n        }\n    };\n\n    /**\n     * Traverse a specific field chains. (Depth-First Traversal)\n     *\n     * @param {string} field Field name to traverse (e.g. \'ownedElements\')\n     * @param {function} fun\n     */\n    Element.prototype.traverseFieldDepthFirst = function (field, fun) {\n        var ref = this[field];\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var v = ref[i];\n            v.traverseFieldDepthFirst(field, fun);\n        }\n        fun(this);\n    };\n\n    /**\n     * Traverse up along with the `_parent` chain.\n     * @param {function} fun\n     */\n    Element.prototype.traverseUp = function (fun) {\n        fun(this);\n        if (this._parent) {\n            this._parent.traverseUp(fun);\n        }\n    };\n\n    /**\n     * Read it\'s state data from Reader.\n     *\n     * @param {Reader} reader\n     */\n    Element.prototype.load = function (reader) {\n        var attrs = this.getMetaAttributes();\n        for (var i = 0, len = attrs.length; i < len; i++) {\n            var attr = attrs[i], val;\n            switch (attr.kind) {\n            case ATTR_KIND_PRIM:\n                val = reader.read(attr.name);\n                if (typeof val !== "undefined") {\n                    this[attr.name] = val;\n                }\n                break;\n            case ATTR_KIND_ENUM:\n                val = reader.read(attr.name);\n                if (typeof val !== "undefined") {\n                    this[attr.name] = val;\n                }\n                break;\n            case ATTR_KIND_REF:\n                val = reader.readRef(attr.name);\n                var prevRef = this[attr.name];\n                if (typeof val !== "undefined") {\n                    // Remove previous embedded reference\n                    if (prevRef && attr.embedded) {\n                        if (Array.isArray(this[attr.embedded])) {\n                            this[attr.embedded].remove(prevRef);\n                        }\n                    }\n                    // Assign new reference\n                    this[attr.name] = val;\n                } else {\n                    // Register previous embedded reference\n                    if (prevRef && attr.embedded) {\n                        reader.idMap[prevRef._id] = prevRef;\n                    }\n                }\n                break;\n            case ATTR_KIND_REFS:\n                val = reader.readRefArray(attr.name);\n                if (typeof val !== "undefined") {\n                    this[attr.name] = val;\n                }\n                break;\n            case ATTR_KIND_OBJ:\n                val = reader.readObj(attr.name);\n                if (typeof val !== "undefined") {\n                    this[attr.name] = val;\n                }\n                break;\n            case ATTR_KIND_OBJS:\n                val = reader.readObjArray(attr.name);\n                if (!Array.isArray(this[attr.name])) {\n                    this[attr.name] = [];\n                }\n                // Append loaded objects to the existing array\n                if (Array.isArray(val)) {\n                    for (var j = 0; j < val.length; j++) {\n                        this[attr.name].push(val[j]);\n                    }\n                }\n                break;\n            case ATTR_KIND_VAR:\n                val = reader.readVariant(attr.name);\n                if (typeof val !== "undefined") {\n                    this[attr.name] = val;\n                }\n                break;\n            case ATTR_KIND_CUSTOM:\n                val = reader.readCustom(attr.type, attr.name);\n                if (typeof val !== "undefined") {\n                    this[attr.name] = val;\n                }\n                break;\n            default:\n                void 0;\n            }\n        }\n    };\n\n    /**\n     * Write it\'s state data to writer.\n     *\n     * @param {Writer} writer\n     */\n    Element.prototype.save = function (writer) {\n        writer.write(\'_type\', this.getClassName());\n        var attrs = this.getMetaAttributes();\n        for (var i = 0, len = attrs.length; i < len; i++) {\n            var attr = attrs[i];\n            if (typeof this[attr.name] !== "undefined" && attr.transient !== true) {\n                switch (attr.kind) {\n                case ATTR_KIND_PRIM:\n                    switch (attr.type) {\n                    case "String":\n                        if (this[attr.name] !== "" && this[attr.name] !== null) {\n                            writer.write(attr.name, this[attr.name]);\n                        }\n                        break;\n                    case "Integer":\n                    case "Real":\n                        writer.write(attr.name, this[attr.name]);\n                        break;\n                    case "Boolean":\n                        writer.write(attr.name, this[attr.name]);\n                        break;\n                    default:\n                        writer.write(attr.name, this[attr.name]);\n                        break;\n                    }\n                    break;\n                case ATTR_KIND_ENUM:\n                    writer.write(attr.name, this[attr.name]);\n                    break;\n                case ATTR_KIND_REF:\n                    if (this[attr.name] !== null) {\n                        writer.writeRef(attr.name, this[attr.name]);\n                    }\n                    break;\n                case ATTR_KIND_REFS:\n                    if (_.isArray(this[attr.name]) && this[attr.name].length > 0) {\n                        writer.writeRefArray(attr.name, this[attr.name]);\n                    }\n                    break;\n                case ATTR_KIND_OBJ:\n                    if (this[attr.name] !== null) {\n                        writer.writeObj(attr.name, this[attr.name]);\n                    }\n                    break;\n                case ATTR_KIND_OBJS:\n                    if (_.isArray(this[attr.name]) && this[attr.name].length > 0) {\n                        writer.writeObjArray(attr.name, this[attr.name]);\n                    }\n                    break;\n                case ATTR_KIND_VAR:\n                    if (this[attr.name] !== null) {\n                        writer.writeVariant(attr.name, this[attr.name]);\n                    }\n                    break;\n                case ATTR_KIND_CUSTOM:\n                    if (this[attr.name] !== null) {\n                        writer.writeCustom(attr.name, this[attr.name]);\n                    }\n                    break;\n                }\n            }\n        }\n    };\n\n    /**\n     * Store it\'s state data to memento\n     *\n     * @param {object} memento\n     */\n    Element.prototype.assignTo = function (memento) {\n        var attrs = this.getMetaAttributes();\n        for (var i = 0, len = attrs.length; i < len; i++) {\n            var attr = attrs[i];\n            if (this[attr.name] !== null) {\n                switch (attr.kind) {\n                case ATTR_KIND_PRIM:\n                    memento[attr.name] = this[attr.name];\n                    break;\n                case ATTR_KIND_ENUM:\n                    memento[attr.name] = this[attr.name];\n                    break;\n                case ATTR_KIND_CUSTOM:\n                    memento[attr.name] = this[attr.name].__write();\n                    break;\n                }\n            }\n        }\n    };\n\n    /**\n     * Load it\'s state data from memento\n     *\n     * @param {object} memento\n     */\n    Element.prototype.assignFrom = function (memento) {\n        var attrs = this.getMetaAttributes();\n        for (var i = 0, len = attrs.length; i < len; i++) {\n            var attr = attrs[i];\n            switch (attr.kind) {\n            case ATTR_KIND_PRIM:\n                this[attr.name] = memento[attr.name];\n                break;\n            case ATTR_KIND_ENUM:\n                this[attr.name] = memento[attr.name];\n                break;\n            case ATTR_KIND_CUSTOM:\n                if (this[attr.name] !== null) {\n                    this[attr.name].__read(memento[attr.name]);\n                }\n                break;\n            }\n        }\n    };\n\n    /**\n     * Return differences between it\'s state data and data stored in memento.\n     *\n     * @param {object} memento\n     * @return {Array.<{elem:Element, f:string, n:?, o:?}>} `{f: Field name, n: New value, o: Old value}`\n     */\n    Element.prototype.diff = function (memento) {\n        var diffs = [];\n        var attrs = this.getMetaAttributes();\n        var newVal = null, oldVal = null;\n        for (var i = 0, len = attrs.length; i < len; i++) {\n            var attr = attrs[i];\n            if (this[attr.name] !== null) {\n                switch (attr.kind) {\n                case ATTR_KIND_PRIM:\n                    newVal = this[attr.name];\n                    oldVal = memento[attr.name];\n                    if (newVal !== oldVal) {\n                        diffs.push({\n                            elem: this,\n                            f: attr.name,\n                            n: newVal,\n                            o: oldVal\n                        });\n                    }\n                    break;\n                case ATTR_KIND_ENUM:\n                    newVal = this[attr.name];\n                    oldVal = memento[attr.name];\n                    if (newVal !== oldVal) {\n                        diffs.push({\n                            elem: this,\n                            f: attr.name,\n                            n: newVal,\n                            o: oldVal\n                        });\n                    }\n                    break;\n                case ATTR_KIND_CUSTOM:\n                    newVal = this[attr.name].__write();\n                    oldVal = memento[attr.name];\n                    if (newVal !== oldVal) {\n                        diffs.push({\n                            elem: this,\n                            f: attr.name,\n                            n: newVal,\n                            o: oldVal\n                        });\n                    }\n                }\n            }\n        }\n        return diffs;\n    };\n\n    /**\n     * Find an element by name. Find through `ownedElements` chain, and then lookup through `_parent` chain.\n     *\n     * @param {string} name\n     * @param {constructor} typeFilter\n     * @param {Element} namespace Lookup only inside of namespace\n     * @return {Element} Return the first matched element.\n     */\n    Element.prototype.lookup = function (name, typeFilter, namespace) {\n        var children = this.getChildren();\n        for (var i = 0, len = children.length; i < len; i++) {\n            var elem = children[i];\n            if (typeFilter) {\n                if ((elem instanceof typeFilter) && (elem.name === name)) {\n                    return elem;\n                }\n            } else {\n                if (elem.name === name) {\n                    return elem;\n                }\n            }\n        }\n        if (this !== namespace && this._parent !== null) {\n            return this._parent.lookup(name, typeFilter, namespace);\n        }\n        return null;\n    };\n\n    /**\n     * Find by name in child elements\n     *\n     * @param {string | Array.<string>} name\n     * @return {Element}\n     */\n    Element.prototype.findByName = function (name) {\n        var children = this.getChildren();\n        for (var i = 0, len = children.length; i < len; i++) {\n            var elem = children[i];\n            if (elem.name === name) {\n                return elem;\n            }\n        }\n        return null;\n    };\n\n    /**\n     * Look down an element along parent-children chains.\n     *\n     * @param {Array.<string>} pathName\n     * @return {Element}\n     */\n    Element.prototype.lookdown = function (pathName) {\n        if (_.isArray(pathName) && pathName.length > 0) {\n            var elem = this.findByName(pathName[0]);\n            if (pathName.length === 1) {\n                return elem;\n            } else if (elem) {\n                return elem.lookdown(_.rest(pathName));\n            }\n        }\n        return null;\n    };\n\n    /**\n     * Return true only if a given elem is one of the container.\n     *\n     * @param {Element} elem\n     * @return {boolean}\n     */\n    Element.prototype.isOneOfTheContainers = function(elem) {\n        if ((this._parent === null) || (elem === this)) {\n            return false;\n        } else if (elem === this._parent) {\n            return true;\n        } else {\n            return this._parent.isOneOfTheContainers(elem);\n        }\n    };\n\n    /**\n     * Return true only if it can contain the kind of elements\n     *\n     * @param {string} kind\n     * @return {Boolean}\n     */\n    Element.prototype.canContainKind = function(kind) {\n        var attrs = this.getMetaAttributes();\n        for (var i = 0, len = attrs.length; i < len; i++) {\n            var attr = attrs[i];\n            if (attr.kind === ATTR_KIND_OBJ || attr.kind === ATTR_KIND_OBJS) {\n                if (attr.type && _global.meta[kind] && (MetaModelManager.isKindOf(kind, attr.type))) {\n                    return true;\n                }\n            }\n        }\n        return false;\n    };\n\n    /**\n     * Return true only if it can contain a given element.\n     *\n     * @param {Element} elem\n     * @return {Boolean}\n     */\n    Element.prototype.canContain = function(elem) {\n        if (elem !== null) {\n            return this.canContainKind(elem.getClassName()) &&\n                   (elem !== this) &&\n                   (!this.isOneOfTheContainers(elem));\n        } else {\n            return false;\n        }\n    };\n\n    /**\n     * Return true only if it could be copied.\n     * @return {Boolean}\n     */\n    Element.prototype.canCopy = function () {\n        return true;\n    };\n\n    /**\n     * Return true only if it could be deleted alone.\n     * @return {Boolean}\n     */\n    Element.prototype.canDelete = function () {\n        return true;\n    };\n\n    /**\n     * Return true only if it can accomodate elements in clipboard\n     *     based on a given kind and copyContext.\n     *\n     * @param {string} kind\n     * @param {{field:string}} copyContext\n     * @return {Boolean}\n     */\n    Element.prototype.canPaste = function (kind, copyContext) {\n        return this.canContainKind(kind) && this[copyContext.field];\n    };\n\n    /**\n     * Model\n     *\n     * @constructor\n     * @extends Element\n     */\n    function Model() {\n        Element.apply(this, arguments);\n\n        /** @member {string} */\n        this.name = "";\n\n        /** @member {Array.<Model>} */\n        this.ownedElements = [];\n    }\n\n    // inherits from Element\n    Model.prototype = Object.create(Element.prototype);\n    Model.prototype.constructor = Model;\n\n    /**\n     * Get a corresponding view type\n     *\n     * @return {constructor}\n     */\n    Model.prototype.getViewType = function () {\n        var typeName = MetaModelManager.getViewTypeOf(this.getClassName()),\n            ViewType = typeName ? type[typeName] : null;\n        return ViewType;\n    };\n\n    /**\n     * Get path to from a given base\n     *\n     * @param {Model} base\n     * @return {Array.<string>}\n     */\n    Model.prototype.getPath = function(base) {\n        if (this === base) {\n            return [];\n        } else if (!this._parent) {\n            return [this];\n        } else if (this._parent) {\n            var result = this._parent.getPath(base);\n            result.push(this);\n            return result;\n        }\n    };\n\n    /**\n     * Return full pathname\n     *\n     * @return {string}\n     */\n    Model.prototype.getPathname = function() {\n        var _name = this.name;\n        if (this._parent && this._parent._parent) {\n            return this._parent.getPathname() + "::" + _name;\n        } else {\n            return _name;\n        }\n    };\n\n    /**\n     * Return true only if it can contain a given element.\n     *\n     * @override\n     * @param {Element} elem\n     * @return {Boolean}\n     */\n    Model.prototype.canContain = function(elem) {\n        if (elem !== null) {\n            if (elem instanceof Diagram) {\n                return this.canContainDiagram(elem);\n            } else {\n                return this.canContainKind(elem.getClassName()) &&\n                       (elem !== this) &&\n                       (!this.isOneOfTheContainers(elem));\n            }\n        } else {\n            return false;\n        }\n    };\n\n    /**\n     * Determines whether a given diagram type name can be contained or not.\n     *\n     * @param {string} kind\n     * @return {Boolean}\n     */\n    Model.prototype.canContainDiagramKind = function (kind) {\n        return false;\n    };\n\n    /**\n     * Determines whether a given diagram can be contained or not.\n     *\n     * @param {Diagram} diagram\n     * @return {Boolean}\n     */\n    Model.prototype.canContainDiagram = function (diagram) {\n        return this.canContainDiagramKind(diagram.getClassName());\n    };\n\n    /**\n     * Determines whether this object can be relocated to a given element.\n     *\n     * @param {Model} model\n     * @return {Boolean}\n     */\n    Model.prototype.canRelocateTo = function (model) {\n        return model.canContain(this);\n    };\n\n\n    /**\n     * Tag\n     *\n     * @constructor\n     * @extends Model\n     */\n    function Tag() {\n        Model.apply(this, arguments);\n\n        /** @member {TagKind} */\n        this.kind = TK_STRING;\n\n        /** @member {*} */\n        this.value = \'\';\n\n        /** @member {Model} */\n        this.reference = null;\n\n        /** @member {boolean} */\n        this.checked = false;\n\n        /** @member {number} */\n        this.number = 0;\n\n    }\n    // inherits from Model\n    Tag.prototype = Object.create(Model.prototype);\n    Tag.prototype.constructor = Tag;\n\n    /**\n     * Hyperlink\n     *\n     * @constructor\n     * @extends Model\n     */\n    function Hyperlink() {\n        Model.apply(this, arguments);\n\n        /** @member {Model} */\n        this.reference = null;\n\n        /** @member {*} */\n        this.url = \'\';\n    }\n    // inherits from Model\n    Hyperlink.prototype = Object.create(Model.prototype);\n    Hyperlink.prototype.constructor = Hyperlink;\n\n    Hyperlink.prototype.getNodeText = function (options) {\n        if (this.reference instanceof type.Model) {\n            return "(link to " + this.reference.name + ")";\n        } else if (this.url && this.url.length > 0) {\n            return "(link to " + this.url + ")";\n        } else {\n            return "(none)";\n        }\n    };\n\n\n    /**\n     * ExtensibleModel\n     *\n     * @constructor\n     * @extends Model\n     */\n    function ExtensibleModel() {\n        Model.apply(this, arguments);\n\n        /** @member {string} */\n        this.documentation = "";\n\n        /** @member {Array.<Tag>} */\n        this.tags = [];\n    }\n    // inherits from Element\n    ExtensibleModel.prototype = Object.create(Model.prototype);\n    ExtensibleModel.prototype.constructor = ExtensibleModel;\n\n\n    /**\n     * Return whether element has a specific tag\n     *\n     * @param {string} tagName\n     * @return {Boolean}\n     */\n    ExtensibleModel.prototype.hasTag = function (tagName) {\n        for (var i = 0, len = this.tags.length; i < len; i++) {\n            var tag = this.tags[i];\n            if (tag.name === tagName) {\n                return true;\n            }\n        }\n        return false;\n    };\n\n    /**\n     * Return a specific tag by name\n     *\n     * @param {string} tagName\n     * @return {Tag|undefined}\n     */\n    ExtensibleModel.prototype.getTag = function (tagName) {\n        for (var i = 0, len = this.tags.length; i < len; i++) {\n            var tag = this.tags[i];\n            if (tag.name === tagName) {\n                return tag;\n            }\n        }\n        return undefined;\n    };\n\n    /**\n     * Return value of a specific tag\n     *\n     * @param {string} tagName\n     * @return {Boolean|undefined}\n     */\n    ExtensibleModel.prototype.getTagValue = function (tagName) {\n        for (var i = 0, len = this.tags.length; i < len; i++) {\n            var tag = this.tags[i];\n            if (tag.name === tagName) {\n                return tag.value;\n            }\n        }\n        return undefined;\n    };\n\n\n    /**\n     * Relationship\n     *\n     * @constructor\n     * @extends ExtensibleModel\n     */\n    function Relationship() {\n        ExtensibleModel.apply(this, arguments);\n    }\n\n    // Relationship inherits ExtensibleModel\n    Relationship.prototype = Object.create(ExtensibleModel.prototype);\n    Relationship.prototype.constructor = Relationship;\n\n\n\n    /**\n     * DirectedRelationship\n     *\n     * @constructor\n     * @extends Relationship\n     */\n    function DirectedRelationship() {\n        Relationship.apply(this, arguments);\n\n        /** @member {Model} */\n        this.target = null;\n\n        /** @member {Model} */\n        this.source = null;\n    }\n    // DirectedRelationship inherits Relationship\n    DirectedRelationship.prototype = Object.create(Relationship.prototype);\n    DirectedRelationship.prototype.constructor = DirectedRelationship;\n\n    DirectedRelationship.prototype.getNodeText = function (options) {\n        if (!this.source || !this.target) {\n            console.error("source or target is not assigned");\n            console.log(this);\n            return "(?→?)";\n        }\n        var _text = "(" + this.source.name + "→" + this.target.name + ")";\n        if (this.name && this.name.length > 0) {\n            _text = this.name + " " + _text;\n        }\n        return _text;\n    };\n\n\n    /**\n     * RelationshipEnd\n     *\n     * @constructor\n     * @extends ExtensibleModel\n     */\n    function RelationshipEnd() {\n        ExtensibleModel.apply(this, arguments);\n\n        /** @member {Model} */\n        this.reference = null;\n    }\n\n    // RelationshipEnd inherits ExtensibleModel\n    RelationshipEnd.prototype = Object.create(ExtensibleModel.prototype);\n    RelationshipEnd.prototype.constructor = RelationshipEnd;\n\n    RelationshipEnd.prototype.canDelete = function () {\n        return false;\n    };\n\n    /**\n     * UndirectedRelationship\n     *\n     * @constructor\n     * @extends Relationship\n     */\n    function UndirectedRelationship() {\n        Relationship.apply(this, arguments);\n\n        /** @member {RelationshipEnd} */\n        this.end1 = null;\n\n        /** @member {RelationshipEnd} */\n        this.end2 = null;\n    }\n\n    // UndirectedRelationship inherits Relationship\n    UndirectedRelationship.prototype = Object.create(Relationship.prototype);\n    UndirectedRelationship.prototype.constructor = UndirectedRelationship;\n\n    UndirectedRelationship.prototype.getNodeText = function (options) {\n        return "(" + this.end1.reference.name + "—" + this.end2.reference.name + ")";\n    };\n\n\n    /**\n     * View\n     *\n     * @constructor\n     * @extends Element\n     */\n    function View() {\n        Element.apply(this, arguments);\n\n        /** @member {Model} */\n        this.model = null;\n\n        /** @member {Array.<View>} */\n        this.subViews = [];\n\n        /** @member {View} */\n        this.containerView = null;\n\n        /** @member {Array.<View>} */\n        this.containedViews = [];\n\n        /** @member {boolean} */\n        this.visible = true;\n\n        /** @member {boolean} */\n        this.enabled = true;\n\n        /** @member {boolean} */\n        this.selected = false;\n\n        /** @member {number} */\n        this.selectable = SK_YES;\n\n        /** @member {string} */\n        this.lineColor = PreferenceManager.get("view.lineColor", "#000000");\n\n        /** @member {string} */\n        this.fillColor = PreferenceManager.get("view.fillColor", "#ffffff");\n\n        /** @member {string} */\n        this.fontColor = PreferenceManager.get("view.fontColor", "#000000");\n\n        /** @member {Font} */\n        this.font = new Font(\n            PreferenceManager.get("view.font", "Arial"),\n            PreferenceManager.get("view.fontSize", 13),\n            Graphics.FS_NORMAL\n        );\n\n        /** @member {boolean} */\n        this.parentStyle = false;\n\n        /** @member {boolean} */\n        this.showShadow = PreferenceManager.get("view.showShadow", true);\n\n        /** @member {boolean} */\n        this.containerChangeable = false;\n\n        /** @member {boolean} */\n        this.containerExtending = false;\n\n        /** @member {number} */\n        this.zIndex = 0;\n    }\n    // View inherits Element\n    View.prototype = Object.create(Element.prototype);\n    View.prototype.constructor = View;\n\n    /**\n     * Traverse all sub views recursively (Breadth-First Traversal)\n     *\n     * @param {function} fun\n     */\n    View.prototype.traverse = function (fun) {\n        fun(this);\n        var ref = this.subViews;\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var v = ref[i];\n            v.traverse(fun);\n        }\n    };\n\n    /**\n     * Traverse all sub views recursively (Depth-First Traversal)\n     *\n     * @param {function} fun\n     */\n    View.prototype.traverseDepthFirst = function (fun) {\n        var ref = this.subViews;\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var v = ref[i];\n            v.traverseDepthFirst(fun);\n        }\n        fun(this);\n    };\n\n    View.prototype.find = function (predicate) {\n        if (predicate(this)) {\n            return this;\n        }\n        var ref = this.subViews;\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var v = ref[i];\n            var result = v.find(predicate);\n            if (result) {\n                return result;\n            }\n        }\n        return null;\n    };\n\n    View.prototype.findDepthFirst = function (predicate) {\n        var ref = this.subViews;\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var v = ref[i];\n            var result = v.findDepthFirst(predicate);\n            if (result) {\n                return result;\n            }\n        }\n        if (predicate(this)) {\n            return this;\n        }\n        return null;\n    };\n\n    /**\n     * Assign styles to canvas.\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.assignStyleToCanvas = function (canvas) {\n        canvas.color = this.lineColor;\n        canvas.fillColor = this.fillColor;\n        canvas.fontColor = this.fontColor;\n        canvas.font = this.font;\n    };\n\n    View.prototype.delimitContainingBoundary = function (canvas) {\n    };\n\n    /**\n     * Initialize view object.\n     *\n     * @abstract\n     * @param {Canvas} canvas\n     */\n    View.prototype.initialize = function (canvas, x1, y1, x2, y2) {};\n\n    /**\n     * Move view object including all sub views by (dx, dy)\n     *\n     * @param {Canvas} canvas\n     * @param {number} dx\n     * @param {number} dy\n     */\n    View.prototype.move = function (canvas, dx, dy) {\n        this.moveObject(canvas, dx, dy);\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var v = this.subViews[i];\n            v.move(canvas, dx, dy);\n        }\n    };\n\n    /**\n     * Move this view object by (dx, dy).\n     *\n     * @param {Canvas} canvas\n     * @param {number} dx\n     * @param {number} dy\n     */\n    View.prototype.moveObject = function (canvas, dx, dy) {};\n\n    /**\n     * Setup view object (styles and attributes)\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.setup = function (canvas) {\n        if (this.parentStyle && this._parent && this._parent.font) {\n            this.lineColor = this._parent.lineColor;\n            this.fillColor = this._parent.fillColor;\n            this.fontColor = this._parent.fontColor;\n            this.font.face = this._parent.font.face;\n            this.font.size = this._parent.font.size;\n            this.font.style = this._parent.font.style;\n        }\n        this.assignStyleToCanvas(canvas);\n        if (this._parent && this._parent.visible === false) {\n            this.visible = false;\n        }\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var v = this.subViews[i];\n            v.setup(canvas);\n        }\n    };\n\n    /**\n     * Update view object from a corresponding model object.\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.update = function (canvas) {\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var v = this.subViews[i];\n            v.update(canvas);\n        }\n    };\n\n    /**\n     * Compute it\'s size\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.size = function (canvas) {\n        this.assignStyleToCanvas(canvas);\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var v = this.subViews[i];\n            v.size(canvas);\n        }\n        this.sizeObject(canvas);\n    };\n\n    /**\n     * Compute it\'s size\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.sizeObject = function (canvas) {};\n\n    /**\n     * Arrange this view object.\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.arrange = function (canvas) {\n        this.assignStyleToCanvas(canvas);\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var v = this.subViews[i];\n            v.arrange(canvas);\n        }\n        this.delimitContainingBoundary(canvas);\n        this.arrangeObject(canvas);\n    };\n\n    /**\n     * Arrange this view object.\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.arrangeObject = function (canvas) {};\n\n    /**\n     * Draw shadow.\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.drawShadow = function (canvas) {\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var v = this.subViews[i];\n            if (v.visible) {\n                v.drawShadow(canvas);\n            }\n        }\n    };\n\n    /**\n     * Draw view object.\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.draw = function (canvas) {\n        this.assignStyleToCanvas(canvas);\n        this.drawObject(canvas);\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var v = this.subViews[i];\n            if (v.visible) {\n                v.draw(canvas);\n            }\n        }\n    };\n\n    /**\n     * Draw view object.\n     *\n     * @param {Canvas} canvas\n     */\n    View.prototype.drawObject = function (canvas) {};\n\n    /**\n     * Draw selection of this view object.\n     *\n     * @abstract\n     * @param {Canvas} canvas\n     */\n    View.prototype.drawSelection = function (canvas) {};\n\n    /**\n     * Return a sub view located at (x, y).\n     *\n     * @param {Canvas} canvas\n     * @param {number} x\n     * @param {number} y\n     */\n    View.prototype.getViewAt = function (canvas, x, y) {\n        for (var i = this.subViews.length - 1; i >= 0; i--) {\n            var v = this.subViews[i];\n            if (v.visible && (v.selectable != SK_NO)) {\n                var sub = v.getViewAt(canvas, x, y);\n                if (sub !== null) {\n                    return sub;\n                }\n            }\n        }\n        if (this.containsPoint(canvas, x, y) && (this.selectable == SK_YES)) {\n            return this;\n        }\n        return null;\n    };\n\n    /**\n     * Return a diagram containg this view object.\n     *\n     * @return {View}\n     */\n    View.prototype.getDiagram = function () {\n        if (this._parent instanceof Diagram) {\n            return this._parent;\n        } else if (this._parent !== null) {\n            return this._parent.getDiagram();\n        }\n        return null;\n    };\n\n    /**\n     * Return a bounding box.\n     *\n     * @param {Canvas} canvas\n     * @param {Rect}\n     */\n    View.prototype.getBoundingBox = function (canvas) {\n        return new Rect(-1, -1, 0, 0);\n    };\n\n    /**\n     * Determines whether this view contains a point (x, y)\n     *\n     * @param {Canvas} canvas\n     * @param {number} x\n     * @param {number} y\n     * @return {boolean}\n     */\n    View.prototype.containsPoint = function (canvas, x, y) {\n        var r = this.getBoundingBox(canvas);\n        if (this.selected) {\n            var zr = r,\n                zp = new Point(x, y);\n            Coord.coordTransform2(canvas.zoomFactor, GridFactor.NO_GRID, zr);\n            zr.setRect(zr.x1 - Toolkit.DEFAULT_HIGHLIGHTER_SIZE, zr.y1 - Toolkit.DEFAULT_HIGHLIGHTER_SIZE, zr.x2 + Toolkit.DEFAULT_HIGHLIGHTER_SIZE, zr.y2 + Toolkit.DEFAULT_HIGHLIGHTER_SIZE);\n            Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, zp);\n            return Coord.ptInRect(zp.x, zp.y, zr);\n        }\n        return Coord.ptInRect(x, y, r);\n    };\n\n    /**\n     * Determines whether this view overlaps a given rect\n     *\n     * @param {Canvas} canvas\n     * @param {Rect} rect\n     * @return {boolean}\n     */\n    View.prototype.overlapRect = function (canvas, rect) {\n        var bound = this.getBoundingBox(canvas);\n        if (this.selected) {\n            bound.setRect(bound.x1 - 5, bound.y1 - 5, bound.x2 + 5, bound.y2 + 5);\n        }\n        return Coord.rectInRect(rect, bound);\n    };\n\n    /**\n     * Add a sub view.\n     *\n     * @param {View} view\n     */\n    View.prototype.addSubView = function (view) {\n        this.subViews.add(view);\n        view._parent = this;\n    };\n\n    /**\n     * Remove a sub view\n     *\n     * @param {View} view\n     */\n    View.prototype.removeSubView = function (view) {\n        this.subViews.remove(view);\n        view._parent = null;\n    };\n\n    /**\n     * Add a contained view.\n     *\n     * @param {View} view\n     */\n    View.prototype.addContainedView = function (view) {\n        this.containedViews.add(view);\n        view.containerView = this;\n    };\n\n    /**\n     * Remove a contained view.\n     * @param {View} view\n     */\n    View.prototype.removeContainedView = function (view) {\n        this.containedViews.remove(view);\n        view.containerView = null;\n    };\n\n    /**\n     *\n     */\n    View.prototype.isOneOfTheContainerViews = function (view) {\n        if ((this.containerView === null) || (view == this)) {\n            return false;\n        } else if (this.containerView == view) {\n            return true;\n        } else {\n            return this.containerView.isOneOfTheContainerViews(view);\n        }\n    };\n\n    /**\n     *\n     */\n    View.prototype.canContainViewKind = function (kind) {\n        return false;\n    };\n\n    /**\n     *\n     */\n    View.prototype.canContainView = function (view) {\n        return (view !== null) &&\n               (view !== this) &&\n               (!this.isOneOfTheContainerViews(view)) &&\n               (this.canContainViewKind(view.getClassName()));\n    };\n\n    /**\n     *\n     */\n    View.prototype.drawContainingBox = function (canvas) {\n        var rect = this.getBoundingBox(canvas);\n        Toolkit.drawRangeBox(canvas, rect.x1 + 1, rect.y1 + 1, rect.x2 - 1, rect.y2 - 1);\n    };\n\n    /**\n     *\n     */\n    View.prototype.eraseContainingBox = function (canvas) {\n        // NOTHING TO DO\n    };\n\n\n    /**\n     * NodeView\n     * @constructor\n     * @extends View\n     */\n    function NodeView() {\n        View.apply(this, arguments);\n\n        /** @member {number} */\n        this.left = 0;\n\n        /** @member {number} */\n        this.top = 0;\n\n        /** @member {number} */\n        this.width = 0;\n\n        /** @member {number} */\n        this.height = 0;\n\n        /** @member {number} */\n        this.minWidth = 0;\n\n        /** @member {number} */\n        this.minHeight = 0;\n\n        /** @member {number} */\n        this.sizable = SZ_FREE;\n\n        /** @member {number} */\n        this.movable = MM_FREE;\n\n        /** @member {Boolean} */\n        this.autoResize = false;\n    }\n    // NodeView inherits View\n    NodeView.prototype = Object.create(View.prototype);\n    NodeView.prototype.constructor = NodeView;\n\n    /**\n     *\n     */\n    NodeView.prototype.getRight = function () {\n        return this.left + this.width - 1;\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.setRight = function (value) {\n        this.width = value - this.left + 1;\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.getBottom = function () {\n        return this.top + this.height - 1;\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.setBottom = function (value) {\n        this.height = value - this.top + 1;\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.getCenter = function () {\n        return new Point((this.left + this.getRight()) / 2, (this.top + this.getBottom()) / 2);\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.moveObject = function (canvas, dx, dy) {\n        this.left = this.left + dx;\n        this.top = this.top + dy;\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.initialize = function (canvas, x1, y1, x2, y2) {\n        var r = new Rect(x1, y1, x2, y2);\n        Coord.normalizeRect(r);\n        this.left = r.x1;\n        this.top = r.y1;\n        this.width = Math.max(this.minWidth, r.x2 - r.x1);\n        this.height = Math.max(this.minHeight, r.y2 - r.y1);\n    };\n\n    NodeView.prototype.sizeConstraints = function (canvas) {\n        if (this.autoResize) {\n            this.width = this.minWidth;\n            this.height = this.minHeight;\n        }\n        if (this.width < this.minWidth) {\n            this.width = this.minWidth;\n        }\n        if (this.height < this.minHeight) {\n            this.height = this.minHeight;\n        }\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.sizeObject = function (canvas) {\n        View.prototype.sizeObject.call(this, canvas);\n        this.sizeConstraints();\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.arrangeObject = function (canvas) {\n        View.prototype.arrangeObject.call(this, canvas);\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.drawObject = function (canvas) {\n        View.prototype.drawObject.call(this, canvas);\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.drawSelection = function (canvas) {\n        if (this.sizable === SZ_NONE) {\n            Toolkit.drawSelection(canvas, this.left-1, this.top-1, this.getRight()+1, this.getBottom()+1);\n        } else {\n            var retouch = !(this.sizable === SZ_FREE || this.sizable === SZ_RATIO);\n            var x1, x2, y1, y2;\n            x1 = this.left - 1;\n            y1 = this.top - 1;\n            x2 = this.getRight() + 1;\n            y2 = this.top - 1;\n            Toolkit.drawSelectionLine(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_SELECTIONLINE_WIDTH, Toolkit.NWSE_N, retouch);\n            x1 = this.left - 1;\n            y1 = this.top - 1;\n            x2 = this.left - 1;\n            y2 = this.getBottom() + 1;\n            Toolkit.drawSelectionLine(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_SELECTIONLINE_WIDTH, Toolkit.NWSE_W, retouch);\n            x1 = this.getRight() + 1;\n            y1 = this.top - 1;\n            x2 = this.getRight() + 1;\n            y2 = this.getBottom() + 1;\n            Toolkit.drawSelectionLine(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_SELECTIONLINE_WIDTH, Toolkit.NWSE_E, retouch);\n            x1 = this.left - 1;\n            y1 = this.getBottom() + 1;\n            x2 = this.getRight() + 1;\n            y2 = this.getBottom() + 1;\n            Toolkit.drawSelectionLine(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_SELECTIONLINE_WIDTH, Toolkit.NWSE_S, retouch);\n\n            x1 = this.left - 1;\n            y1 = this.top -1;\n            x2 = this.getRight() + 1;\n            y2 = this.getBottom() + 1;\n            Toolkit.drawSelectionBox(canvas, x1, y1, x2, y2);\n\n            if (this.sizable === SZ_FREE || this.sizable === SZ_RATIO) {\n                Toolkit.drawHighlighter2(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_HIGHLIGHTER_SIZE, Toolkit.CT_LT, true, Toolkit.HIGHLIGHTER_COLOR);\n                Toolkit.drawHighlighter2(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_HIGHLIGHTER_SIZE, Toolkit.CT_RT, true, Toolkit.HIGHLIGHTER_COLOR);\n                Toolkit.drawHighlighter2(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_HIGHLIGHTER_SIZE, Toolkit.CT_LB, true, Toolkit.HIGHLIGHTER_COLOR);\n                Toolkit.drawHighlighter2(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_HIGHLIGHTER_SIZE, Toolkit.CT_RB, true, Toolkit.HIGHLIGHTER_COLOR);\n            }\n            if (this.sizable === SZ_FREE || this.sizable === SZ_VERT) {\n                Toolkit.drawHighlighter2(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_HIGHLIGHTER_SIZE, Toolkit.CT_MT, true, Toolkit.HIGHLIGHTER_COLOR);\n                Toolkit.drawHighlighter2(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_HIGHLIGHTER_SIZE, Toolkit.CT_MB, true, Toolkit.HIGHLIGHTER_COLOR);\n            }\n            if (this.sizable === SZ_FREE || this.sizable === SZ_HORZ) {\n                Toolkit.drawHighlighter2(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_HIGHLIGHTER_SIZE, Toolkit.CT_LM, true, Toolkit.HIGHLIGHTER_COLOR);\n                Toolkit.drawHighlighter2(canvas, x1, y1, x2, y2, Toolkit.DEFAULT_HIGHLIGHTER_SIZE, Toolkit.CT_RM, true, Toolkit.HIGHLIGHTER_COLOR);\n            }\n        }\n    };\n\n    /**\n     *\n     */\n    NodeView.prototype.getBoundingBox = function (canvas) {\n        return new Rect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n\n    /**\n     * EdgeView\n     * @constructor\n     * @extends View\n     */\n    function EdgeView() {\n        View.apply(this, arguments);\n        this.BACKGROUND_COLOR = Color.WHITE;\n\n        /** @member {View} */\n        this.head = null;\n\n        /** @member {View} */\n        this.tail = null;\n\n        /** @member {number} */\n        this.lineStyle = PreferenceManager.get("view.lineStyle", LS_OBLIQUE);\n\n        this.lineMode = LM_SOLID;\n        this.points = new Points();\n        this.headEndStyle = ES_FLAT;\n        this.tailEndStyle = ES_FLAT;\n    }\n\n    // EdgeView inherits View\n    EdgeView.prototype = Object.create(View.prototype);\n    EdgeView.prototype.constructor = EdgeView;\n\n    /** EdgeView Constants */\n    EdgeView.SELF_EDGE_HORIZ_INTERVAL = 30;\n    EdgeView.SELF_EDGE_VERTI_INTERVAL = 20;\n\n    /**\n     *\n     */\n    EdgeView.prototype.reducePoints = function (canvas) {\n        var b, i, j, _i, _results;\n        if (!(this.tail instanceof EdgeView)) {\n            b = this.tail.getBoundingBox(canvas);\n            i = 1;\n            while (i < this.points.count() - 1) {\n                if (Coord.ptInRect2(this.points.getPoint(i), b)) {\n                    for (j = _i = 1; 1 <= i ? _i <= i : _i >= i; j = 1 <= i ? ++_i : --_i) {\n                        this.points.remove(1);\n                    }\n                    i = 1;\n                } else {\n                    i++;\n                }\n            }\n        }\n        if (!(this.head instanceof EdgeView)) {\n            b = this.head.getBoundingBox(canvas);\n            i = 1;\n            _results = [];\n            while (i < this.points.count() - 1) {\n                if (Coord.ptInRect2(this.points.getPoint(i), b)) {\n                    _results.push((function () {\n                        var _j, _ref, _results1;\n                        _results1 = [];\n                        for (j = _j = 1, _ref = this.points.count() - i - 1; 1 <= _ref ? _j <= _ref : _j >= _ref; j = 1 <= _ref ? ++_j : --_j) {\n                            _results1.push(this.points.remove(i));\n                        }\n                        return _results1;\n                    }).call(this));\n                } else {\n                    _results.push(i++);\n                }\n            }\n            return _results;\n        }\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.recalcOblique = function (canvas) {\n        var hb, headPoints, i, tailPoints, tb;\n        this.reducePoints(canvas);\n        if (!(this.tail instanceof EdgeView)) {\n            tb = this.tail.getBoundingBox(canvas);\n        } else {\n            tailPoints = this.tail.points;\n            i = Math.floor((tailPoints.count() - 1) / 2);\n            tb = new Rect(tailPoints.getPoint(i).x, tailPoints.getPoint(i).y, tailPoints.getPoint(i + 1).x, tailPoints.getPoint(i + 1).y);\n        }\n        this.points.setPoint(0, Coord.getCenter(tb));\n        if (!(this.head instanceof EdgeView)) {\n            hb = this.head.getBoundingBox(canvas);\n        } else {\n            headPoints = this.head.points;\n            i = Math.floor((headPoints.count() - 1) / 2);\n            hb = new Rect(headPoints.getPoint(i).x, headPoints.getPoint(i).y, headPoints.getPoint(i + 1).x, headPoints.getPoint(i + 1).y);\n        }\n        tb.expand(1);\n        hb.expand(1);\n        this.points.setPoint(this.points.count() - 1, Coord.getCenter(hb));\n        if (!(this.tail instanceof EdgeView)) {\n            this.points.setPoint(0, Coord.junction(tb, this.points.getPoint(1)));\n        }\n        if (!(this.head instanceof EdgeView)) {\n            this.points.setPoint(this.points.count() - 1, Coord.junction(hb, this.points.getPoint(this.points.count() - 2)));\n        }\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.recalcRectilinear = function (canvas) {\n        var bb, bh, bt, h, hps, i, p, tps, w;\n        if (this.head === this.tail) {\n            if (this.points.count() <= 3) {\n                this.points.clear();\n                bb = this.head.getBoundingBox(canvas);\n                w = bb.x2 - bb.x1;\n                h = bb.y2 - bb.y1;\n                this.points.add(Coord.getCenter(bb));\n                this.points.add(new Point(this.points.getPoint(0).x, Math.floor(this.points.getPoint(0).y - (h / 2) - EdgeView.SELF_EDGE_VERTI_INTERVAL)));\n                this.points.add(new Point(this.points.getPoint(0).x + Math.floor(w / 2) + EdgeView.SELF_EDGE_HORIZ_INTERVAL, this.points.getPoint(0).y - (h / 2) - EdgeView.SELF_EDGE_VERTI_INTERVAL));\n                this.points.add(new Point(this.points.getPoint(0).x + Math.floor(w / 2) + EdgeView.SELF_EDGE_HORIZ_INTERVAL, this.points.getPoint(0).y));\n                this.points.add(Coord.getCenter(bb));\n            }\n        }\n        if (!(this.tail instanceof EdgeView)) {\n            bt = this.tail.getBoundingBox(canvas);\n        } else {\n            tps = this.tail.points;\n            i = Math.floor((tps.count() - 1) / 2);\n            bt = new Rect(tps.getPoint(i).x, tps.getPoint(i).y, tps.getPoint(i + 1).x, tps.getPoint(i + 1).y);\n            bt.setRect3(Coord.getCenter(bt), Coord.getCenter(bt));\n        }\n        if (!(this.head instanceof EdgeView)) {\n            bh = this.head.getBoundingBox(canvas);\n        } else {\n            hps = this.head.points;\n            i = Math.floor((hps.count() - 1) / 2);\n            bh = new Rect(hps.getPoint(i).x, hps.getPoint(i).y, hps.getPoint(i + 1).x, hps.getPoint(i + 1).y);\n            bh.setRect3(Coord.getCenter(bh), Coord.getCenter(bh));\n            bt.expand(1);\n            bh.expand(1);\n        }\n\n        // Add new point, if have not enough points.\n        if (this.points.count() === 2) {\n            p = Coord.orthoJunction(bt, this.points.getPoint(1));\n            if ((p.x === -100) && (p.y === -100)) {\n                this.points.insert(0, Coord.orthoJunction(bt, this.points.getPoint(0)));\n            } else {\n                this.points.setPoint(0, p);\n            }\n            p = Coord.orthoJunction(bh, this.points.getPoint(this.points.count() - 2));\n            if ((p.x === -100) && (p.y === -100)) {\n                this.points.add(Coord.orthoJunction(bh, this.points.getPoint(this.points.count() - 1)));\n            } else {\n                this.points.setPoint(this.points.count() - 1, p);\n            }\n        }\n\n        // Replace 0-indexed point with junction point to TailView\n        p = Coord.orthoJunction(bt, this.points.getPoint(1));\n        if ((p.x === -100) && (p.y === -100)) {\n            if (this.points.getPoint(1).y === this.points.getPoint(2).y) {\n                this.points.setPoint(1, new Point(Coord.getCenter(bt).x, this.points.getPoint(1).y));\n            } else {\n                this.points.setPoint(1, new Point(this.points.getPoint(1).x, Coord.getCenter(bt).y));\n            }\n        }\n        this.points.setPoint(0, Coord.orthoJunction(bt, this.points.getPoint(1)));\n\n        // Replace highest-indexed point with junction point to HeadView\n        p = Coord.orthoJunction(bh, this.points.getPoint(this.points.count() - 2));\n        if ((p.x === -100) && (p.y === -100)) {\n            if (this.points.getPoint(this.points.count() - 2).y === this.points.getPoint(this.points.count() - 3).y) {\n                this.points.setPoint(this.points.count() - 2, new Point(Coord.getCenter(bh).x, this.points.getPoint(this.points.count() - 2).y));\n            } else {\n                this.points.setPoint(this.points.count() - 2, new Point(this.points.getPoint(this.points.count() - 2).x, Coord.getCenter(bh).y));\n            }\n        }\n        this.points.setPoint(this.points.count() - 1, Coord.orthoJunction(bh, this.points.getPoint(this.points.count() - 2)));\n\n        // Must be removed, and calculate this in another module (Handlers)\n        // FitToGrid(GraphicClasses.GridFactor(5, 5));\n        this.points.reduceOrthoLine();\n        this.reducePoints(canvas);\n\n        p = this.points.getPoint(0).copy();\n        this.points.setPoint(0, Coord.orthoJunction(bt, this.points.getPoint(1)));\n        if ((this.points.getPoint(0).x === -100) || (this.points.getPoint(0).y === -100)) {\n            this.points.setPoint(0, p);\n        }\n\n        p = this.points.getPoint(this.points.count() - 1).copy();\n        this.points.setPoint(this.points.count() - 1, Coord.orthoJunction(bh, this.points.getPoint(this.points.count() - 2)));\n        if ((this.points.getPoint(this.points.count() - 1).x === -100) || (this.points.getPoint(this.points.count() - 1).y === -100)) {\n            this.points.setPoint(this.points.count() - 1, p);\n        }\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.drawLineEnd = function (canvas, edgeEndStyle, isHead) {\n        if (edgeEndStyle !== ES_FLAT) {\n            var rt = new Rect(0, 0, 0, 0);\n            if (isHead) {\n                rt.setRect(this.points.getPoint(this.points.count() - 1).x, this.points.getPoint(this.points.count() - 1).y, this.points.getPoint(this.points.count() - 2).x, this.points.getPoint(this.points.count() - 2).y);\n            } else {\n                rt.setRect(this.points.getPoint(0).x, this.points.getPoint(0).y, this.points.getPoint(1).x, this.points.getPoint(1).y);\n            }\n            var a = rt.y2 - rt.y1,\n                b = rt.x2 - rt.x1,\n                th = Math.atan(a / b);\n            if ((a < 0 && b < 0) || (a > 0 && b < 0) || (a === 0 && b < 0)) {\n                th = th + Math.PI;\n            }\n            var th1 = th - Math.PI / 8,\n                th2 = th + Math.PI / 8,\n                th3 = th - Math.PI / 2,\n                th4 = th + Math.PI / 2,\n                th5 = th - Math.PI / 2,\n                th6 = th + Math.PI / 2,\n                c1 = 11.0,\n                c2 = c1 * 2.0,\n                c3 = 7,\n                p0 = new Point(rt.x1, rt.y1),\n                p1 = new Point((c1 * Math.cos(th1)) + rt.x1, (c1 * Math.sin(th1)) + rt.y1),\n                p2 = new Point((c1 * Math.cos(th2)) + rt.x1, (c1 * Math.sin(th2)) + rt.y1),\n                p3 = new Point((c2 * Math.cos(th)) + rt.x1, (c2 * Math.sin(th)) + rt.y1),\n                p4 = new Point((c2 * Math.cos(th1)) + rt.x1, (c2 * Math.sin(th1)) + rt.y1),\n                p5 = new Point((c2 * Math.cos(th2)) + rt.x1, (c2 * Math.sin(th2)) + rt.y1),\n                p6 = new Point((c1 * Math.cos(th1)) + p3.x, (c1 * Math.sin(th1)) + p3.y),\n                p7 = new Point((c1 * Math.cos(th2)) + p3.x, (c1 * Math.sin(th2)) + p3.y),\n                p8 = new Point((c3 * Math.cos(th)) + rt.x1, (c3 * Math.sin(th)) + rt.y1),\n                p9 = new Point((c3 * Math.cos(th3)) + p8.x, (c3 * Math.sin(th3)) + p8.y),\n                p10 = new Point((c3 * Math.cos(th4)) + p8.x, (c3 * Math.sin(th4)) + p8.y),\n                p11 = new Point(((c3*2) * Math.cos(th)) + rt.x1, ((c3*2) * Math.sin(th)) + rt.y1),\n                p12 = new Point(c3 * Math.cos(th5) + p0.x, c3 * Math.sin(th5) + p0.y),\n                p13 = new Point(c3 * Math.cos(th6) + p0.x, c3 * Math.sin(th6) + p0.y);\n            canvas.color = this.lineColor;\n            canvas.fillColor = Color.WHITE;\n            switch (edgeEndStyle) {\n            case ES_STICK_ARROW:\n                canvas.polyline([p1, p0, p2]);\n                break;\n            case ES_SOLID_ARROW:\n                canvas.fillColor = this.lineColor;\n                canvas.fillPolygon([p1, p0, p2]);\n                canvas.polygon([p1, p0, p2]);\n                break;\n            case ES_TRIANGLE:\n                canvas.fillPolygon([p4, p0, p5]);\n                canvas.polygon([p4, p0, p5]);\n                break;\n            case ES_FILLED_TRIANGLE:\n                canvas.fillColor = this.lineColor;\n                canvas.fillPolygon([p4, p0, p5]);\n                break;\n            case ES_DIAMOND:\n                canvas.fillPolygon([p1, p0, p2, p3]);\n                canvas.polygon([p1, p0, p2, p3]);\n                break;\n            case ES_FILLED_DIAMOND:\n                canvas.fillColor = this.lineColor;\n                canvas.fillPolygon([p1, p0, p2, p3]);\n                canvas.polygon([p1, p0, p2, p3]);\n                break;\n            case ES_ARROW_DIAMOND:\n                canvas.fillPolygon([p1, p0, p2, p3]);\n                canvas.polygon([p1, p0, p2, p3]);\n                canvas.polyline([p6, p3, p7]);\n                break;\n            case ES_ARROW_FILLED_DIAMOND:\n                canvas.fillColor = this.lineColor;\n                canvas.fillPolygon([p1, p0, p2, p3]);\n                canvas.polygon([p1, p0, p2, p3]);\n                canvas.polyline([p6, p3, p7]);\n                break;\n            case ES_PLUS:\n                canvas.line(p9.x, p9.y, p10.x, p10.y);\n                break;\n            case ES_CIRCLE:\n                canvas.fillEllipse(p8.x - c3, p8.y - c3, p8.x + c3, p8.y + c3);\n                canvas.ellipse(p8.x - c3, p8.y - c3, p8.x + c3, p8.y + c3);\n                break;\n            case ES_CIRCLE_PLUS:\n                canvas.fillEllipse(p8.x - c3, p8.y - c3, p8.x + c3, p8.y + c3);\n                canvas.ellipse(p8.x - c3, p8.y - c3, p8.x + c3, p8.y + c3);\n                canvas.line(p11.x, p11.y, p0.x, p0.y);\n                canvas.line(p9.x, p9.y, p10.x, p10.y);\n                break;\n            case ES_CROWFOOT_ONE:\n                /*\n                canvas.putPixel(p0.x, p0.y, Color.BLACK);\n\n                canvas.putPixel(p1.x, p1.y, Color.RED);\n                canvas.putPixel(p2.x, p2.y, Color.RED);\n                canvas.putPixel(p3.x, p3.y, Color.GREEN);\n                canvas.putPixel(p4.x, p4.y, Color.GREEN);\n                canvas.putPixel(p5.x, p5.y, Color.GREEN);\n                canvas.putPixel(p6.x, p6.y, Color.MAGENTA);\n                canvas.putPixel(p7.x, p7.y, Color.MAGENTA);\n                canvas.putPixel(p8.x, p8.y, Color.MAGENTA);\n\n                canvas.putPixel(p9.x, p9.y, Color.ORANGE);\n                canvas.putPixel(p10.x, p10.y, Color.ORANGE);\n                canvas.putPixel(p11.x, p11.y, Color.ORANGE);\n\n                canvas.putPixel(p12.x, p12.y, Color.CYAN);\n                canvas.putPixel(p13.x, p13.y, Color.CYAN);\n                */\n\n\n                // canvas.line(p11.x, p11.y, p12.x, p12.y);\n                // canvas.line(p11.x, p11.y, p13.x, p13.y);\n                // canvas.fillEllipse(p3.x - c3, p3.y - c3, p3.x + c3, p3.y + c3);\n                // canvas.ellipse(p3.x - c3, p3.y - c3, p3.x + c3, p3.y + c3);\n\n                canvas.line(p9.x, p9.y, p10.x, p10.y);\n\n\n                break;\n            case ES_CROWFOOT_MANY:\n                canvas.line(p11.x, p11.y, p12.x, p12.y);\n                canvas.line(p11.x, p11.y, p13.x, p13.y);\n                break;\n            case ES_CROWFOOT_ZERO_ONE:\n                canvas.fillEllipse(p3.x - c3, p3.y - c3, p3.x + c3, p3.y + c3);\n                canvas.ellipse(p3.x - c3, p3.y - c3, p3.x + c3, p3.y + c3);\n                canvas.line(p9.x, p9.y, p10.x, p10.y);\n                break;\n            case ES_CROWFOOT_ZERO_MANY:\n                canvas.line(p11.x, p11.y, p12.x, p12.y);\n                canvas.line(p11.x, p11.y, p13.x, p13.y);\n                canvas.fillEllipse(p3.x - c3, p3.y - c3, p3.x + c3, p3.y + c3);\n                canvas.ellipse(p3.x - c3, p3.y - c3, p3.x + c3, p3.y + c3);\n                break;\n\n            }\n        }\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.initialize = function (canvas, x1, y1, x2, y2) {\n        this.points.clear();\n        this.points.add(Coord.junction(this.tail.getBoundingBox(canvas), Coord.getCenter(this.head.getBoundingBox(canvas))));\n        this.points.add(Coord.junction(this.head.getBoundingBox(canvas), Coord.getCenter(this.tail.getBoundingBox(canvas))));\n        if (this.lineStyle === LS_RECTILINEAR || this.lineStyle === LS_ROUNDRECT) {\n            this.points.convObliqueToRectilinear();\n        }\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.moveObject = function (canvas, dx, dy) {\n        var ref = this.points.points;\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var p = ref[i];\n            p.setPoint(p.x + dx, p.y + dy);\n        }\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.recalcPoints = function (canvas) {\n        switch (this.lineStyle) {\n        case LS_RECTILINEAR:\n            if (!this.points.isRectilinear()) {\n                this.points.convObliqueToRectilinear();\n            }\n            this.recalcRectilinear(canvas);\n            break;\n        case LS_OBLIQUE:\n            this.recalcOblique(canvas);\n            break;\n        case LS_ROUNDRECT:\n            if (!this.points.isRectilinear()) {\n                this.points.convObliqueToRectilinear();\n            }\n            this.recalcRectilinear(canvas);\n            break;\n        case LS_CURVE:\n            this.recalcOblique(canvas);\n            break;\n        default:\n            if (!this.points.isRectilinear()) {\n                this.points.convObliqueToRectilinear();\n            }\n            this.recalcRectilinear(canvas);\n        }\n        this.points.quantize();\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.arrangeObject = function (canvas) {\n        if (this.head === this.tail) {\n            this.lineStyle = LS_RECTILINEAR;\n        }\n        this.recalcPoints(canvas);\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.drawObject = function (canvas) {\n        this.assignStyleToCanvas(canvas);\n        canvas.fillColor = this.BACKGROUND_COLOR;\n        var pattern = (this.lineMode === LM_SOLID) ? null : [3];\n        switch (this.lineStyle) {\n        case LS_RECTILINEAR:\n            canvas.polyline(this.points.points, pattern);\n            break;\n        case LS_OBLIQUE:\n            canvas.polyline(this.points.points, pattern);\n            break;\n        case LS_ROUNDRECT:\n            canvas.roundRectLine(this.points.points, pattern);\n            break;\n        case LS_CURVE:\n            canvas.curveLine(this.points.points, pattern);\n            break;\n        default:\n            canvas.polyline(this.points.points, pattern);\n        }\n        this.drawLineEnd(canvas, this.headEndStyle, true);\n        this.drawLineEnd(canvas, this.tailEndStyle, false);\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.drawSelection = function (canvas) {\n        Toolkit.drawDottedLine(canvas, this.points);\n        for (var i = 0, len = this.points.points.length; i < len; i++) {\n            var p = this.points.points[i];\n            Toolkit.drawHighlighter(canvas, p.x, p.y, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, true, Toolkit.HIGHLIGHTER_COLOR);\n        }\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.getBoundingBox = function (canvas) {\n        var r = this.points.getBoundingRect();\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var sv = this.subViews[i];\n            if (sv.visible) {\n                r.setRect2(Coord.unionRect(r, sv.getBoundingBox(canvas)));\n            }\n        }\n        return r;\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.containsPoint = function (canvas, x, y) {\n        return this.containedIndex(canvas, new Point(x, y)) > -1;\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.disToPoint = function (p1, p2) {\n        return Math.sqrt(Math.square(p1.x - p2.x) + Math.square(p1.y - p2.y));\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.disToOrthoLine = function (lh, lt, p) {\n        var l1, l2;\n        l1 = new Point(0, 0);\n        l2 = new Point(0, 0);\n        if (lh.y === lt.y) {\n            if (lh.x > lt.x) {\n                l1.setPoint(lt.x, lt.y);\n                l2.setPoint(lh.x, lh.y);\n            } else {\n                l1.setPoint(lh.x, lh.y);\n                l2.setPoint(lt.x, lt.y);\n            }\n            if (p.x > l2.x) {\n                return this.disToPoint(p, l2);\n            } else if (p.x < l1.x) {\n                return this.disToPoint(p, l1);\n            } else {\n                return Math.abs(p.y - l1.y);\n            }\n        } else {\n            if (lh.y > lt.y) {\n                l1.setPoint(lt.x, lt.y);\n                l2.setPoint(lh.x, lh.y);\n            } else {\n                l1.setPoint(lh.x, lh.y);\n                l2.setPoint(lt.x, lt.y);\n            }\n            if (p.y > l2.y) {\n                return this.disToPoint(p, l2);\n            } else if (p.y < l1.y) {\n                return this.disToPoint(p, l1);\n            } else {\n                return Math.abs(p.x - l1.x);\n            }\n        }\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.containedIndex = function (canvas, p) {\n        var RECOG_MIN_DIS, d, i, minDis, minDisIndex, ph, pt, result, _i, _j, _ref, _ref1;\n        result = -1;\n        pt = new Point(0, 0);\n        ph = new Point(0, 0);\n        if (this.lineStyle === LS_RECTILINEAR || this.lineStyle === LS_ROUNDRECT) {\n            RECOG_MIN_DIS = 5;\n            minDis = RECOG_MIN_DIS;\n            minDisIndex = -1;\n            for (i = _i = 0, _ref = this.points.count() - 2; 0 <= _ref ? _i <= _ref : _i >= _ref; i = 0 <= _ref ? ++_i : --_i) {\n                pt.setPoint2(this.points.getPoint(i));\n                ph.setPoint2(this.points.getPoint(i + 1));\n                d = this.disToOrthoLine(ph, pt, p);\n                if (d <= minDis) {\n                    minDis = d;\n                    minDisIndex = i;\n                }\n            }\n            result = minDisIndex;\n        } else {\n            result = -1;\n            for (i = _j = 0, _ref1 = this.points.count() - 2; 0 <= _ref1 ? _j <= _ref1 : _j >= _ref1; i = 0 <= _ref1 ? ++_j : --_j) {\n                pt.setPoint2(this.points.getPoint(i));\n                ph.setPoint2(this.points.getPoint(i + 1));\n                if (Coord.ptInLine(new Rect(pt.x, pt.y, ph.x, ph.y), p)) {\n                    result = i;\n                }\n            }\n        }\n        return result;\n    };\n\n    /**\n     *\n     */\n    EdgeView.prototype.selectedPoint = function (canvas, p) {\n        var result = -1;\n        for (var i = 0, len = this.points.points.length; i < len; i++) {\n            var pt = this.points.points[i];\n            if (Coord.equalPt(pt, p)) {\n                result = i;\n            }\n        }\n        return result;\n    };\n\n    /**\n     * Return true only if this edge can be connected the view\n     *\n     * @param {View} view\n     * @param {boolean} isTail Try to connect to tail-side or not\n     * @return {boolean}\n     */\n    EdgeView.prototype.canConnectTo = function (view, isTail) {\n        return true;\n    };\n\n\n    /**\n     * LabelView\n     * @constructor\n     * @extends NodeView\n     */\n    function LabelView() {\n        NodeView.apply(this, arguments);\n        this.parentStyle = true;\n        this.selectable = SK_NO;\n\n        /** @member {string} */\n        this.underline = false;\n\n        /** @member {string} */\n        this.text = "";\n\n        /** @member {number} */\n        this.horizontalAlignment = Graphics.AL_CENTER;\n\n        /** @member {number} */\n        this.verticalAlignment = Graphics.AL_MIDDLE;\n\n        /** @member {number} */\n        this.direction = DK_HORZ;\n\n        /** @member {boolean} */\n        this.wordWrap = false;\n\n    }\n    // LabelView inherits NodeView\n    LabelView.prototype = Object.create(NodeView.prototype);\n    LabelView.prototype.constructor = LabelView;\n\n    /**\n     *\n     */\n    LabelView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.assignStyleToCanvas(canvas);\n        var size, minW, minH, w, h;\n        if (this.wordWrap && this.direction === DK_HORZ) {\n            size = canvas.textExtent(this.text, 1);\n            minW = size.x;\n            minH = canvas.textExtent(this.text, this.width - 1).y;\n            size = canvas.textExtent(this.text, this.width - 1);\n            w = size.x;\n            h = size.y;\n        } else {\n            minW = canvas.textExtent(this.text).x;\n            minH = canvas.textExtent("^_").y;\n            w = minW;\n            h = minH;\n        }\n        if (this.direction === DK_HORZ) {\n            this.minWidth = minW;\n            this.minHeight = minH;\n            this.height = h;\n        } else {\n            this.minWidth = minH;\n            this.minHeight = minW;\n            this.width = w;\n        }\n        this.sizeConstraints();\n    };\n\n    /**\n     * Arrange LabelView\n     */\n    LabelView.prototype.arrange = function (canvas) {\n        NodeView.prototype.arrange.call(this, canvas);\n    };\n\n    /**\n     * Draw LabelView\n     */\n    LabelView.prototype.draw = function (canvas) {\n        this.assignStyleToCanvas(canvas);\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        if (this.direction === DK_HORZ) {\n            canvas.textOut2(r, this.text, this.horizontalAlignment, this.verticalAlignment, false, this.wordWrap, this.underline);\n        } else {\n            canvas.textOut2(r, this.text, this.horizontalAlignment, this.verticalAlignment, true, this.wordWrap, this.underline);\n        }\n        View.prototype.draw.call(this, canvas);\n    };\n\n    /**\n     * LabelView cannot be copied alone.\n     * @override\n     * @return {Boolean}\n     */\n    LabelView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * LabelView cannot be deleted alone.\n     * @override\n     * @return {Boolean}\n     */\n    LabelView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * ParasiticView\n     * @constructor\n     * @extends NodeView\n     */\n    function ParasiticView() {\n        NodeView.apply(this, arguments);\n        this.alpha = 0;\n        this.distance = 0;\n    }\n    // ParasiticView inherits NodeView\n    ParasiticView.prototype = Object.create(NodeView.prototype);\n    ParasiticView.prototype.constructor = ParasiticView;\n\n    /**\n     * ParasiticView cannot be copied alone.\n     * @override\n     * @return {Boolean}\n     */\n    ParasiticView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * ParasiticView cannot be deleted alone.\n     * @override\n     * @return {Boolean}\n     */\n    ParasiticView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * NodeParasiticView\n     * @constructor\n     * @extends ParasiticView\n     */\n    function NodeParasiticView() {\n        ParasiticView.apply(this, arguments);\n        this.hostNode = null;\n    }\n\n    // NodeParasiticView inherits ParasiticView\n    NodeParasiticView.prototype = Object.create(ParasiticView.prototype);\n    NodeParasiticView.prototype.constructor = NodeParasiticView;\n\n    NodeParasiticView.prototype.arrange = function (canvas) {\n        ParasiticView.prototype.arrange.call(this, canvas);\n        var node = null;\n        if (this.hostNode !== null) {\n            node = this.hostNode;\n        } else if (this._parent instanceof NodeView) {\n            node = this._parent;\n        } else {\n            return;\n        }\n        var p1 = new Point();\n        p1.x = (node.left + node.getRight()) / 2;\n        p1.y = (node.top + node.getBottom()) / 2;\n        var p = Coord.getPointAwayLine(p1, p1, this.alpha, this.distance);\n        this.left = p.x + p1.x - this.width / 2;\n        this.top = p.y + p1.y - this.height / 2;\n    };\n\n\n    /**\n     * EdgeParasiticView\n     * @constructor\n     * @extends ParasiticView\n     */\n    function EdgeParasiticView() {\n        ParasiticView.apply(this, arguments);\n        this.hostEdge = null;\n        this.edgePosition = EP_HEAD;\n    }\n\n    // EdgeParasiticView inherits ParasiticView\n    EdgeParasiticView.prototype = Object.create(ParasiticView.prototype);\n    EdgeParasiticView.prototype.constructor = EdgeParasiticView;\n\n    EdgeParasiticView.prototype.arrangeObject = function (canvas) {\n        ParasiticView.prototype.arrangeObject.call(this, canvas);\n        var edge, midPointIndex, p, p1, p2;\n        edge = null;\n        if (this.hostEdge !== null) {\n            edge = this.hostEdge;\n        } else if (this._parent instanceof EdgeView) {\n            edge = this._parent;\n        } else {\n            return;\n        }\n        switch (this.edgePosition) {\n        case EP_HEAD:\n            p1 = edge.points.getPoint(edge.points.count() - 1).copy();\n            p2 = edge.points.getPoint(edge.points.count() - 2).copy();\n            break;\n        case EP_TAIL:\n            p1 = edge.points.getPoint(0).copy();\n            p2 = edge.points.getPoint(1).copy();\n            break;\n        case EP_MIDDLE:\n            midPointIndex = Math.floor(edge.points.count() / 2);\n            if ((edge.points.count() % 2) === 0) {\n                midPointIndex--;\n            }\n            p1 = edge.points.getPoint(midPointIndex).copy();\n            p2 = edge.points.getPoint(midPointIndex + 1).copy();\n            if ((edge.points.count() % 2) === 0) {\n                p1.x = Math.floor((p1.x + p2.x) / 2);\n                p1.y = Math.floor((p1.y + p2.y) / 2);\n            }\n        }\n        p = Coord.getPointAwayLine(p1, p2, this.alpha, this.distance);\n        this.left = p.x + p1.x - Math.floor(this.width / 2);\n        this.top = p.y + p1.y - Math.floor(this.height / 2);\n    };\n\n\n    /**\n     * NodeLabelView\n     * @constructor\n     * @extends NodeParasiticView\n     */\n    function NodeLabelView() {\n        NodeParasiticView.apply(this, arguments);\n\n        /** @member {string} */\n        this.underline = false;\n\n        /** @member {boolean} */\n        this.wordWrap = false;\n\n        this.enabled = true;\n        this.movable = MM_FREE;\n        this.sizable = SZ_HORZ;\n        this.text = "";\n        this.horizontalAlignment = Graphics.AL_CENTER;\n        this.verticalAlignment = Graphics.AL_MIDDLE;\n    }\n\n    // NodeLabelView inherits NodeParasiticView\n    NodeLabelView.prototype = Object.create(NodeParasiticView.prototype);\n    NodeLabelView.prototype.constructor = NodeLabelView;\n\n    NodeLabelView.prototype.sizeObject = function (canvas) {\n        NodeParasiticView.prototype.sizeObject.call(this, canvas);\n        this.assignStyleToCanvas(canvas);\n        var size, minW, minH, w, h;\n        if (this.wordWrap) {\n            size = canvas.textExtent(this.text, 1);\n            minW = size.x;\n            minH = canvas.textExtent(this.text, this.width - 1).y;\n            size = canvas.textExtent(this.text, this.width - 1);\n            w = size.x;\n            h = size.y;\n        } else {\n            minW = canvas.textExtent(this.text).x;\n            minH = canvas.textExtent("^_").y;\n            w = minW;\n            h = minH;\n        }\n        this.minWidth = minW;\n        this.minHeight = minH;\n        this.height = h;\n        this.sizeConstraints();\n    };\n\n    NodeLabelView.prototype.arrange = function (canvas) {\n        NodeParasiticView.prototype.arrange.call(this, canvas);\n    };\n\n    NodeLabelView.prototype.draw = function (canvas) {\n        this.assignStyleToCanvas(canvas);\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.textOut2(r, this.text, this.horizontalAlignment, this.verticalAlignment, false, this.wordWrap, this.underline);\n    };\n\n\n    /**\n     * EdgeLabelView\n     * @constructor\n     * @extends EdgeParasiticView\n     */\n    function EdgeLabelView() {\n        EdgeParasiticView.apply(this, arguments);\n\n        /** @member {string} */\n        this.underline = false;\n\n        /** @member {boolean} */\n        this.wordWrap = false;\n\n        this.enabled = true;\n        this.movable = MM_FREE;\n        this.sizable = SZ_HORZ;\n        this.text = "";\n        this.horizontalAlignment = Graphics.AL_CENTER;\n        this.verticalAlignment = Graphics.AL_MIDDLE;\n    }\n    // EdgeLabelView inherits EdgeParasiticView\n    EdgeLabelView.prototype = Object.create(EdgeParasiticView.prototype);\n    EdgeLabelView.prototype.constructor = EdgeLabelView;\n\n    EdgeLabelView.prototype.sizeObject = function (canvas) {\n        EdgeParasiticView.prototype.sizeObject.call(this, canvas);\n        this.assignStyleToCanvas(canvas);\n        var size, minW, minH, w, h;\n        if (this.wordWrap) {\n            size = canvas.textExtent(this.text, 1);\n            minW = size.x;\n            minH = canvas.textExtent(this.text, this.width - 1).y;\n            size = canvas.textExtent(this.text, this.width - 1);\n            w = size.x;\n            h = size.y;\n        } else {\n            minW = canvas.textExtent(this.text).x;\n            minH = canvas.textExtent("^_").y;\n            w = minW;\n            h = minH;\n        }\n        this.minWidth = minW;\n        this.minHeight = minH;\n        this.height = h;\n        this.sizeConstraints();\n    };\n\n    EdgeLabelView.prototype.arrange = function (canvas) {\n        EdgeParasiticView.prototype.arrange.call(this, canvas);\n    };\n\n    EdgeLabelView.prototype.draw = function (canvas) {\n        this.assignStyleToCanvas(canvas);\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.textOut2(r, this.text, this.horizontalAlignment, this.verticalAlignment, false, this.wordWrap, this.underline);\n        EdgeParasiticView.prototype.draw.call(this, canvas);\n    };\n\n\n    /**\n     * NodeNodeView\n     * @constructor\n     * @extends NodeParasiticView\n     */\n    function NodeNodeView() {\n        NodeParasiticView.apply(this, arguments);\n    }\n    // NodeNodeView inherits NodeParasiticView\n    NodeNodeView.prototype = Object.create(NodeParasiticView.prototype);\n    NodeNodeView.prototype.constructor = NodeNodeView;\n\n\n    /**\n     * EdgeNodeView\n     * @constructor\n     * @extends EdgeParasiticView\n     */\n    function EdgeNodeView() {\n        EdgeParasiticView.apply(this, arguments);\n    }\n    // EdgeNodeView inherits EdgeParasiticView\n    EdgeNodeView.prototype = Object.create(EdgeParasiticView.prototype);\n    EdgeNodeView.prototype.constructor = EdgeNodeView;\n\n\n    /**\n     * Diagram\n     *\n     * @constructor\n     * @extends ExtensibleModel\n     */\n    function Diagram() {\n        ExtensibleModel.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.visible = true;\n\n        /** @member {boolean} */\n        this.defaultDiagram = false;\n\n        /** @member {Array.<View>} */\n        this.ownedViews = [];\n\n        /** @member {Array.<View>} */\n        this.selectedViews = [];\n    }\n\n    // Diagram inherits Model\n    Diagram.prototype = Object.create(ExtensibleModel.prototype);\n    Diagram.prototype.constructor = Diagram;\n\n    Diagram.prototype.traverse = function (fun) {\n        fun(this);\n        for (var i = 0, len = this.ownedViews.length; i < len; i++) {\n            var v = this.ownedViews[i];\n            v.traverse(fun);\n        }\n    };\n\n    Diagram.prototype.traverseDepthFirst = function (fun) {\n        for (var i = 0, len = this.ownedViews.length; i < len; i++) {\n            var v = this.ownedViews[i];\n            v.traverseDepthFirst(fun);\n        }\n        fun(this);\n    };\n\n\n    Diagram.prototype.find = function (predicate) {\n        if (predicate(this)) {\n            return this;\n        }\n        var ref = this.ownedViews;\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var v = ref[i];\n            var result = v.find(predicate);\n            if (result) {\n                return result;\n            }\n        }\n        return null;\n    };\n\n    Diagram.prototype.findDepthFirst = function (predicate) {\n        var ref = this.ownedViews;\n        for (var i = 0, len = ref.length; i < len; i++) {\n            var v = ref[i];\n            var result = v.findDepthFirst(predicate);\n            if (result) {\n                return result;\n            }\n        }\n        if (predicate(this)) {\n            return this;\n        }\n        return null;\n    };\n\n    Diagram.prototype.selectViewByRect = function (canvas, v, r) {\n        // EdgeLabelView들은 영역에 의해 선택가능하다. (parentStyle = false 이기 때문에)\n        // 하지만, LabelView, CompartmentView 들은 parentStyle = true이기 때문에 선택이 불가능하다.\n        if (v.visible === true && v.enabled === true && v.parentStyle === false && (v.selectable === SK_YES)) {\n            if (v.overlapRect(canvas, r)) {\n                v.selected = true;\n                if (!_.contains(this.selectedViews, v)) {\n                    this.selectedViews.push(v);\n                }\n            }\n        }\n        for (var i = 0, len = v.subViews.length; i < len; i++) {\n            var sub = v.subViews[i];\n            this.selectViewByRect(canvas, sub, r);\n        }\n    };\n\n    Diagram.prototype.selectView = function (v) {\n        if (v.visible && v.enabled && (v.selectable === SK_YES)) {\n            v.selected = true;\n            if (!_.contains(this.selectedViews, v)) {\n                this.selectedViews.push(v);\n            }\n        }\n    };\n\n    Diagram.prototype.deselectView = function (v) {\n        if (v.visible && v.enabled && (v.selectable === SK_YES)) {\n            v.selected = false;\n            if (_.contains(this.selectedViews, v)) {\n                this.selectedViews.remove(v);\n            }\n        }\n    };\n\n    /**\n     * Draw Diagram\n     * 1. setup all views (breadth-first)\n     * 2. update all views (breadth-first)\n     * 3. compute size of views (depth-first)\n     * 4. arrange views (breadth-first)\n     * 5. draw views (breadth-first)\n     *\n     * @param {Canvas} canvas\n     * @param {?boolean} drawSelection\n     */\n    Diagram.prototype.drawDiagram = function (canvas, drawSelection) {\n        var i, len;\n        // Sort views by zIndex\n        var sortedViews = _.sortBy(this.ownedViews, function (v) { return v.zIndex; });\n        // Draw views\n        for (i = 0, len = sortedViews.length; i < len; i++) {\n            var view = sortedViews[i];\n            try {\n                view.setup(canvas);\n                view.update(canvas);\n                view.size(canvas);\n                view.arrange(canvas);\n                if (view.showShadow) {\n                    view.drawShadow(canvas);\n                }\n                view.draw(canvas);\n            } catch (err) {\n                console.log(view);\n                console.error(err);\n            }\n        }\n        // Draw selection\n        if (drawSelection !== false) {\n            for (i = 0, len = this.selectedViews.length; i < len; i++) {\n                var v = this.selectedViews[i];\n                try {\n                    v.drawSelection(canvas);\n                } catch (err) {\n                    console.error(err);\n                }\n            }\n        }\n    };\n\n    Diagram.prototype.addOwnedView = function (view) {\n        this.ownedViews.add(view);\n        view._parent = this;\n    };\n\n    Diagram.prototype.removeOwnedView = function (view) {\n        this.ownedViews.remove(view);\n        view._parent = null;\n    };\n\n    Diagram.prototype.getOwnedViewById = function (id) {\n        for (var i = 0, len = this.ownedViews.length; i < len; i++) {\n            var v = this.ownedViews[i];\n            if (id === v._id) {\n                return v;\n            }\n        }\n        return null;\n    };\n\n    /**\n     * Find a view at specific position (x, y)\n     *\n     * @param {Canvas} canvas\n     * @param {number} x\n     * @param {number} y\n     * @param {boolean} shallow Find only in first level (don\'t find in subviews)\n     * @param {?constructor} viewType\n     * @return {View}\n     */\n    Diagram.prototype.getViewAt = function (canvas, x, y, shallow, viewType) {\n        // Sort views by zIndex\n        var sortedViews = _.sortBy(this.ownedViews, function (v) { return v.zIndex; });\n        for (var i = sortedViews.length - 1; i >= 0; i--) {\n            var v = sortedViews[i];\n            if (shallow === true) {\n                if (v.visible && (v.selectable === SK_YES) && v.containsPoint(canvas, x, y)) {\n                    if (viewType) {\n                        if (v instanceof viewType) {\n                            return v;\n                        }\n                    } else {\n                        return v;\n                    }\n                }\n            } else {\n                if (v.visible && (v.selectable !== SK_NO)) {\n                    var view = v.getViewAt(canvas, x, y);\n                    if (view !== null) {\n                        return view;\n                    }\n                }\n            }\n        }\n        if (this.containsPoint(canvas, x, y) && (this.selectable === SK_YES))  {\n            return this;\n        }\n        return null;\n    };\n\n    /**\n     * Find View at specific position (x, y) in depth-first manner\n     *\n     * @param {Canvas} canvas\n     * @param {number} x\n     * @param {number} y\n     * @return {View}\n     */\n    Diagram.prototype.getBottomViewAt = function (canvas, x, y) {\n        return this.findDepthFirst(function (v) {\n            return (v instanceof View && v.visible && (v.selectable !== SK_NO) && v.containsPoint(canvas, x, y));\n        });\n    };\n\n    /**\n     * Get a view of a specific model\n     *\n     * @param {Model} model\n     * @param {?constructor} viewType\n     * @return {View}\n     */\n    Diagram.prototype.getViewOf = function (model, viewType) {\n        return this.find(function (v) {\n            if (viewType) {\n                return (v.model === model) && (v instanceof viewType);\n            } else {\n                return (v.model === model);\n            }\n        });\n    };\n\n    Diagram.prototype.getSelectedBoundingBox = function (canvas) {\n        var r = new Rect(10000, 10000, -10000, -10000);\n        for (var i = 0, len = this.selectedViews.length; i < len; i++) {\n            var v = this.selectedViews[i];\n            r = Coord.unionRect(r, v.getBoundingBox(canvas));\n        }\n        return r;\n    };\n\n    Diagram.prototype.getBoundingBox = function (canvas) {\n        if (this.ownedViews.length > 0) {\n            var r = this.ownedViews[0].getBoundingBox(canvas);\n            for (var i = 1, len = this.ownedViews.length; i < len; i++) {\n                var v = this.ownedViews[i];\n                r.union(v.getBoundingBox(canvas));\n            }\n            return r;\n        }\n        return new Rect(0, 0, 0, 0);\n    };\n\n    Diagram.prototype.containsPoint = function (canvas, x, y) {\n        var r = this.getBoundingBox(canvas);\n        return Coord.ptInRect(x, y, r);\n    };\n\n    Diagram.prototype.selectArea = function (canvas, x1, y1, x2, y2) {\n        var r = new Rect(x1, y1, x2, y2);\n        Coord.normalizeRect(r);\n        for (var i = 0, len = this.ownedViews.length; i < len; i++) {\n            var v = this.ownedViews[i];\n            if (v.visible && v.enabled && (v.selectable === SK_YES) && v.overlapRect(canvas, r)) {\n                if (!_.contains(this.selectedViews, v)) {\n                    v.selected = true;\n                    this.selectedViews.push(v);\n                }\n            }\n            this.selectViewByRect(canvas, v, r);\n        }\n    };\n\n    Diagram.prototype.selectAll = function () {\n        var self = this;\n        for (var i = 0, len = this.ownedViews.length; i < len; i++) {\n            var v = this.ownedViews[i];\n            this.selectView(v);\n            v.traverse(function (sub) {\n                if (sub.visible === true && sub.enabled === true && sub.parentStyle === false && sub.selectable === SK_YES) {\n                    self.selectView(sub);\n                }\n            });\n        }\n    };\n\n    Diagram.prototype.deselectAll = function () {\n        for (var i = 0, len = this.selectedViews.length; i < len; i++) {\n            var v = this.selectedViews[i];\n            v.selected = false;\n        }\n        this.selectedViews.clear();\n    };\n\n    Diagram.prototype.canPaste = function (kind, copyContext) {\n        return false;\n    };\n\n    /**\n     * Return true only if all selected views could be copied.\n     *\n     * @return {Boolean}\n     */\n    Diagram.prototype.canCopyViews = function () {\n        for (var i = 0, len = this.selectedViews.length; i < len; i++) {\n            if (!this.selectedViews[i].canCopy()) {\n                return false;\n            }\n        }\n        return true;\n    };\n\n    /**\n     * Return true only if all selected views could be deleted.\n     *\n     * @return {Boolean}\n     */\n    Diagram.prototype.canDeleteViews = function () {\n        for (var i = 0, len = this.selectedViews.length; i < len; i++) {\n            if (!this.selectedViews[i].canDelete())  {\n                return false;\n            }\n        }\n        return true;\n    };\n\n    /**\n     * Determine whether views in clipboard can be pasted in this diagram.\n     *\n     * @param {Array.<View>} views\n     * @return {Boolean}\n     */\n    Diagram.prototype.canPasteViews = function (views) {\n        var viewTypes = MetaModelManager.getAvailableViewTypes(this.getClassName());\n        return _.all(views, function (v) {\n            return _.any(viewTypes, function (vt) {\n                return v instanceof type[vt];\n            });\n        });\n    };\n\n    /**\n     * Determine whether to accept to create a view of a given model drag-and-dropped from Explorer.\n     *\n     * @param {Model} model\n     * @return {Boolean}\n     */\n    Diagram.prototype.canAcceptModel = function (model) {\n        return false;\n    };\n\n    /**\n     * Layout diagram automatically\n     *\n     * @param {string} direction Rank Direction ( "TB" | "BT" | "LR" | "RL" )\n     * @param {{node:number, edge:number, rank:number}} separations\n     * @param {number} edgeLineStyle\n     */\n    Diagram.prototype.layout = function (direction, separations, edgeLineStyle) {\n        if (typeof graphlib !== "undefined" && typeof dagre !== "undefined") {\n            var i, len, v, g = new graphlib.Digraph();\n\n            // Make Graph\n            for (i = 0, len = this.ownedViews.length; i < len; i++) {\n                v = this.ownedViews[i];\n                if (v instanceof _global.type.NodeView) {\n                    g.addNode(v._id, { width: v.width, height: v.height });\n                }\n            }\n            for (i = 0, len = this.ownedViews.length; i < len; i++) {\n                v = this.ownedViews[i];\n                if (v instanceof _global.type.EdgeView && v.head instanceof type.NodeView && v.tail instanceof type.NodeView) {\n                    g.addEdge(v._id, v.head._id, v.tail._id);\n                }\n            }\n\n            // Layout Options\n            var nodeSeparation = NODE_SEPARATION,\n                edgeSeparation = EDGE_SEPARATION,\n                rankSeparation = RANK_SEPARATION,\n                rankDir        = DIRECTION_TB,\n                lineStyle      = LS_CURVE;\n            if (direction) {\n                rankDir = direction;\n            }\n            if (separations) {\n                if (typeof separations.node !== undefined) { nodeSeparation = separations.node; }\n                if (typeof separations.edge !== undefined) { edgeSeparation = separations.edge; }\n                if (typeof separations.rank !== undefined) { rankSeparation = separations.rank; }\n            }\n            if (_.isNumber(edgeLineStyle)) {\n                lineStyle = edgeLineStyle;\n            }\n\n            // Do Layout\n            var layout = dagre.layout()\n                .nodeSep(nodeSeparation)\n                .edgeSep(edgeSeparation)\n                .rankSep(rankSeparation)\n                .rankDir(rankDir)\n                .run(g);\n\n            // Apply Layout Results\n            var key;\n            for (key in layout._nodes) {\n                var node = layout._nodes[key];\n                var nodeView = this.getOwnedViewById(node.id);\n                nodeView.left = node.value.x - (node.value.width / 2) + LAYOUT_MARGIN_LEFT;\n                nodeView.top = node.value.y - (node.value.height / 2) + LAYOUT_MARGIN_TOP;\n            }\n            for (key in layout._edges) {\n                var edge = layout._edges[key];\n                var edgeView = this.getOwnedViewById(edge.id);\n                var headView = this.getOwnedViewById(edge.u);\n                var tailView = this.getOwnedViewById(edge.v);\n                edgeView.lineStyle = lineStyle;\n                edgeView.points.clear();\n                edgeView.points.add(tailView.getCenter());\n                for (var j = edge.value.points.length - 1; j >= 0; j--) {\n                    var p = edge.value.points[j];\n                    edgeView.points.add(new Graphics.Point(p.x + LAYOUT_MARGIN_LEFT, p.y + LAYOUT_MARGIN_TOP));\n                }\n                edgeView.points.add(headView.getCenter());\n            }\n        }\n    };\n\n\n    /**\n     * Project\n     *\n     * @constructor\n     * @extends ExtensibleModel\n     */\n    function Project() {\n        ExtensibleModel.apply(this, arguments);\n        this.name = "Untitled";\n        this.author = \'\';\n        this.company = \'\';\n        this.copyright = \'\';\n        this.version = \'\';\n    }\n\n    // Project inherits ExtensibleModel\n    Project.prototype = Object.create(ExtensibleModel.prototype);\n    Project.prototype.constructor = Project;\n\n    Project.prototype.canCopy = function () {\n        return false;\n    };\n\n    Project.prototype.canDelete = function () {\n        return false;\n    };\n\n    /**\n     * Writer Class for storing Elements into JSON data.\n     * @constructor\n     */\n    function Writer() {\n\n        /** @member {Object<string, ?>} - 현재 저장중인 데이터 */\n        this.current = {};\n    }\n\n    /**\n     * Write primitive value\n     *\n     * @param {string} name\n     * @param {string|number|boolean|null} value\n     */\n    Writer.prototype.write = function (name, value) {\n        if (!name) {\n            console.error("Writer.write(): missing required parameters: name");\n            return;\n        }\n        if (!_.isString(value) && !_.isNumber(value) && !_.isBoolean(value) && value !== null) {\n            console.error("Writer.write(): type of \'value\' parameter should be one of string, number or boolean.", name + " = ", value);\n            return;\n        }\n        this.current[name] = value;\n    };\n\n    /**\n     * Write an Object\n     *\n     * @param {string} name\n     * @param {Element} value\n     */\n    Writer.prototype.writeObj = function (name, value) {\n        if (!name) {\n            console.error("Writer.writeObj(): missing required parameters: name");\n            return;\n        }\n        if (value && value instanceof Element) {\n            var temp = this.current;\n            this.current[name] = {};\n            this.current = this.current[name];\n            value.save(this);\n            this.current = temp;\n        } else {\n            this.current[name] = null;\n        }\n    };\n\n    /**\n     * Write an Array of Objects\n     *\n     * @param {string} name\n     * @param {Array.<Element>} value\n     */\n    Writer.prototype.writeObjArray = function (name, value) {\n        if (!name) {\n            console.error("Writer.writeObjArray(): missing required parameters: name");\n            return;\n        }\n        if (!_.isArray(value)) {\n            console.error("Writer.writeObjArray(): type of \'value\' parameter should be an array.", name + " = ", value);\n            return;\n        }\n        var temp = this.current;\n        this.current[name] = [];\n        var i, len,\n            array = this.current[name];\n        for (i = 0, len = value.length; i < len; i++) {\n            var o = value[i];\n            if (o instanceof Element) {\n                this.current = {};\n                o.save(this);\n                array.push(this.current);\n            } else {\n                console.error("Writer.writeObjArray(): one of \'value\' array is not instanceof Core.Element.", o);\n                return;\n            }\n        }\n        this.current = temp;\n    };\n\n    /**\n     * Write a Reference\n     *\n     * @param {string} name\n     * @param {Element} value\n     */\n    Writer.prototype.writeRef = function (name, value) {\n        if (!name) {\n            console.error("Writer.writeRef(): missing required parameters: name");\n            return;\n        }\n        if (value) {\n            if (value instanceof Element) {\n                this.current[name] = { $ref: value._id };\n            } else {\n                console.error("Writer.writeRef(): \'value\' parameter is not instanceof Core.Element.", name + " = ", value);\n                return;\n            }\n        } else {\n            this.current[name] = null;\n        }\n    };\n\n    /**\n     * Write an Array of Reference\n     *\n     * @param {string} name\n     * @param {Array.<Element>} value\n     */\n    Writer.prototype.writeRefArray = function (name, value) {\n        if (!name) {\n            console.error("Writer.writeRefArray(): missing required parameters: name");\n            return;\n        }\n        if (!_.isArray(value)) {\n            console.error("Writer.writeRefArray(): type of \'value\' parameter should be an array.", name + " = ", value);\n            return;\n        }\n        var temp = this.current;\n        this.current[name] = [];\n        var i, len,\n            array = this.current[name];\n        for (i = 0, len = value.length; i < len; i++) {\n            var o = value[i];\n            if (o instanceof Element) {\n                var ref = { $ref: o._id };\n                array.push(ref);\n            } else {\n                console.error("Writer.writeRefArray(): one of \'value\' array is not instanceof Core.Element.", o);\n                return;\n            }\n        }\n        this.current = temp;\n    };\n\n    /**\n     * Write a Variant (Primitive or Reference)\n     *\n     * @param {string} name\n     * @param {number|string|boolean|Element} value\n     */\n    Writer.prototype.writeVariant = function (name, value) {\n        if (!name) {\n            console.error("Writer.writeVariant(): missing required parameters: name");\n            return;\n        }\n        if (value === null) {\n            this.current[name] = null;\n        } else if (value instanceof Element) {\n            this.current[name] = { $ref: value._id };\n        } else if (_.isString(value) || _.isNumber(value) || _.isBoolean(value)) {\n            this.current[name] = value;\n        } else {\n            console.error("Writer.writeVariant(): invalid type of parameter: value.", name + " = ", value);\n        }\n    };\n\n    /**\n     * Write a Custom Object (Custom Object have it\'s own method to store itself)\n     *\n     * @param {string} name\n     * @param {{__write: function()}} value\n     */\n    Writer.prototype.writeCustom = function (name, value) {\n        if (!name) {\n            console.error("Writer.writeCustom(): missing required parameters: name");\n            return;\n        }\n        if (value) {\n            if (_.isFunction(value.__write)) {\n                this.current[name] = value.__write();\n            } else {\n                console.error("Writer.writeCustom(): the \'value\' parameter should have \'__write\' function", name + " = ", value);\n                return;\n            }\n        } else {\n            console.error("Writer.writeCustom(): missing required parameters: value");\n            return;\n        }\n    };\n\n    /**\n     * Reader Class\n     * @constructor\n     * @param {Object} data\n     * @param {Object.<string, constructor>} ctors\n     */\n    function Reader(data, ctors) {\n        if (!_.isObject(data)) {\n            console.error("Reader constructor: \'data\' parameter should be JSON object");\n            return;\n        }\n        this.data = data;\n        this.ctors = ctors;\n        this.current = this.data;\n        this.idMap = {};\n    }\n\n    /**\n     * Read primitive type\n     * @param {string} name\n     * @return {number|string|boolean|null}\n     */\n    Reader.prototype.read = function (name) {\n        if (!name) {\n            console.error("Reader.read(): missing required parameters: name");\n            return;\n        }\n\n        var value = this.current[name];\n\n        if (typeof value !== "undefined") {\n            // Check type of data to be read\n            if (!_.isString(value) && !_.isNumber(value) && !_.isBoolean(value) && value !== null) {\n                console.error("Reader.read(): type of data to be read should be one of number, string, boolean, or null.", name + " = ", value);\n                return;\n            }\n            return this.current[name];\n        }\n        return undefined;\n    };\n\n    /**\n     * Read an Object\n     * @param {string} name\n     * @return {Element}\n     */\n    Reader.prototype.readObj = function (name) {\n        if (!name) {\n            console.error("Reader.readObj(): missing required parameters: name");\n            return;\n        }\n\n        var value = this.current[name];\n\n        if (typeof value !== "undefined") {\n            // Check type of data to be read\n            if (!_.isObject(value)) {\n                console.error("Reader.readObj(): type of data to be read should be an Object.", name + " = ", value);\n                return;\n            }\n            if (!value._type) {\n                console.error("Reader.readObj(): \'_type\' field is not found to instantiate an Object.", name + " = ", value);\n                return;\n            }\n            if (!_.isFunction(this.ctors[value._type])) {\n                console.error("Reader.readObj(): type." + value._type + " is not registered.");\n                return;\n            }\n\n            var obj = new this.ctors[value._type]();\n            var temp = this.current;\n            this.current = value;\n            obj.load(this);\n            this.current = temp;\n\n            // Register to idMap\n            this.idMap[obj._id] = obj;\n\n            return obj;\n        }\n        return undefined;\n    };\n\n    /**\n     * Read an Array of Object\n     * @param {string} name\n     * @return {Array.<Element>}\n     */\n    Reader.prototype.readObjArray = function (name) {\n        if (!name) {\n            console.error("Reader.readObjArray(): missing required parameters: name");\n            return;\n        }\n\n        var value = this.current[name];\n\n        if (typeof value !== "undefined") {\n            // Check type of data to be read\n            if (!_.isArray(value)) {\n                console.error("Reader.readObjArray(): type of data to be read should be an array.", name + " = ", value);\n                return;\n            }\n\n            var ref = value;\n            var i, len, array = [];\n            for (i = 0, len = ref.length; i < len; i++) {\n                var o = ref[i];\n\n                // Check type of an element of array\n                if (!_.isObject(o)) {\n                    console.error("Reader.readObjArray(): one of array elements is not Object.", o);\n                    continue;\n                }\n                if (!o._type) {\n                    console.error("Reader.readObjArray(): \'_type\' field is not found to instantiate an Object.", o);\n                    continue;\n                }\n                if (!_.isFunction(this.ctors[o._type])) {\n                    console.error("Reader.readObjArray(): type." + o._type + " is not registered.");\n                    continue;\n                }\n\n                var obj = new this.ctors[o._type]();\n                var temp = this.current;\n                this.current = o;\n                obj.load(this);\n                this.current = temp;\n\n                // Register to idMap\n                this.idMap[obj._id] = obj;\n                array.push(obj);\n            }\n            return array;\n        }\n        return undefined;\n    };\n\n    /**\n     * Read a reference to an Object. The returned {$ref} object should be resolved later.\n     * @param {string} name\n     * @return {{$ref:string}}\n     */\n    Reader.prototype.readRef = function (name) {\n        if (!name) {\n            console.error("Reader.readRef(): missing required parameters: name");\n            return;\n        }\n\n        var value = this.current[name];\n\n        if (typeof value !== "undefined") {\n            // Check type of data to be read\n            if (value && !_.isString(value.$ref)) {\n                console.error("Reader.readRef(): data is not a reference (\'$ref\' not found).", name + " = ", value);\n                return;\n            }\n\n            return value;\n        }\n        return undefined;\n    };\n\n    /**\n     * Read an array of reference.\n     * @param {string} name\n     * @return {Array.<{$ref:string}>}\n     */\n    Reader.prototype.readRefArray = function (name) {\n        if (!name) {\n            console.error("Reader.readRefArray(): missing required parameters: name");\n            return;\n        }\n\n        var value = this.current[name];\n\n        if (typeof value !== "undefined") {\n            // Check type of data to be read\n            if (!_.isArray(value)) {\n                console.error("Reader.readRefArray(): type of data to be read should be an array.", name + " = ", value);\n                return;\n            }\n\n            var array = [];\n            var i, len, ref = this.current[name];\n            for (i = 0, len = ref.length; i < len; i++) {\n                var _refObj = ref[i];\n\n                // Check type of an element of array\n                if (!_.isObject(_refObj)) {\n                    console.error("Reader.readRefArray(): one of array elements is not Object.", _refObj);\n                    return;\n                }\n                if (!_.isString(_refObj.$ref)) {\n                    console.error("Reader.readRefArray(): data is not a reference (\'$ref\' not found).", _refObj);\n                    return;\n                }\n\n                array.push(_refObj);\n            }\n            return array;\n        }\n        return undefined;\n    };\n\n    /**\n     * Read Variant\n     * @param {string} name\n     * @return {{$ref:string}|null|number|boolean|string}\n     */\n    Reader.prototype.readVariant = function (name) {\n        if (!name) {\n            console.error("Reader.readVariant(): missing required parameters: name");\n            return;\n        }\n\n        var value = this.current[name];\n\n        if (typeof value !== "undefined") {\n            // Check type of data to be read\n            if (!_.isString(value) && !_.isNumber(value) && !_.isBoolean(value) && value !== null && !_.isString(value.$ref)) {\n                console.error("Reader.readVariant(): type of data to be read should be one of number, string, boolean, null, or reference.", name + " = ", value);\n                return;\n            }\n\n            return this.current[name];\n        }\n        return undefined;\n    };\n\n    /**\n     * Read a custom object\n     * @param {string} typeName\n     * @param {string} name\n     * @return {{__read: function(Object)}}\n     */\n    Reader.prototype.readCustom = function (typeName, name) {\n        if (!name) {\n            console.error("Reader.readCustom(): missing required parameters: name");\n            return;\n        }\n\n        var value = this.current[name];\n\n        if (typeof value !== "undefined") {\n            if (!_.isFunction(this.ctors[typeName])) {\n                console.error("Reader.readCustom(): type." + typeName + " is not registered.");\n                return;\n            }\n\n            var custom = new this.ctors[typeName]();\n\n            if (!_.isFunction(custom.__read)) {\n                console.error("Reader.readCustom(): Object of type." + typeName + " should have \'__read\' function.", custom);\n                return;\n            }\n\n            custom.__read(value);\n            return custom;\n        }\n        return undefined;\n    };\n\n    /**\n     * Return a super type of a given type.\n     *\n     * @param {constructor} subType\n     * @return {constructor} - superType of subType\n     */\n    function getSuperType(subType) {\n        if (subType) {\n            return _global.type[_global.meta[subType.name].super];\n        } else {\n            return null;\n        }\n    }\n\n    /**\n     * Return a generalized type of given elements.\n     *\n     * @param {Array.<Element>} elems\n     * @return {constructor}\n     */\n    function getCommonType(elems) {\n        if (elems && elems.length > 0) {\n            var commonType = elems[0].getClass();\n            while (!_.every(elems, function (e) { return (e instanceof commonType); })) {\n                commonType = getSuperType(commonType);\n            }\n            return commonType;\n        } else {\n            return null;\n        }\n    }\n\n    /**\n     * Find element by name in a given array.\n     *\n     * @param {Array.<Element>} array\n     * @param {string} name\n     * @return {Element}\n     */\n    function findByName(array, name) {\n        if (array && array.length > 0) {\n            for (var i = 0, len = array.length; i < len; i++) {\n                var elem = array[i];\n                if (elem.name == name) {\n                    return elem;\n                }\n            }\n        }\n        return null;\n    }\n\n    /**\n     * Find an available name in a given array. (e.g. Class1, Class2, ... )\n     *\n     * @param {Array.<Element>} array\n     * @param {string} prefix Prefix for name\n     * @return {string} Return a new name\n     */\n    function getNewName(array, prefix) {\n        var num = 0, name = null;\n        do {\n            num++;\n            name = prefix + num;\n        } while (findByName(array, name));\n        return name;\n    }\n\n    // Type definitions\n    _global.type.Element                = Element;\n    _global.type.Model                  = Model;\n    _global.type.Tag                    = Tag;\n    _global.type.Hyperlink              = Hyperlink;\n    _global.type.ExtensibleModel        = ExtensibleModel;\n    _global.type.Relationship           = Relationship;\n    _global.type.DirectedRelationship   = DirectedRelationship;\n    _global.type.RelationshipEnd        = RelationshipEnd;\n    _global.type.UndirectedRelationship = UndirectedRelationship;\n    _global.type.View                   = View;\n    _global.type.NodeView               = NodeView;\n    _global.type.EdgeView               = EdgeView;\n    _global.type.LabelView              = LabelView;\n    _global.type.ParasiticView          = ParasiticView;\n    _global.type.NodeParasiticView      = NodeParasiticView;\n    _global.type.EdgeParasiticView      = EdgeParasiticView;\n    _global.type.NodeLabelView          = NodeLabelView;\n    _global.type.EdgeLabelView          = EdgeLabelView;\n    _global.type.NodeNodeView           = NodeNodeView;\n    _global.type.EdgeNodeView           = EdgeNodeView;\n    _global.type.Diagram                = Diagram;\n    _global.type.Project                = Project;\n\n    // Define public API\n\n    exports.ATTR_KIND_PRIM          = ATTR_KIND_PRIM;\n    exports.ATTR_KIND_ENUM          = ATTR_KIND_ENUM;\n    exports.ATTR_KIND_REF           = ATTR_KIND_REF;\n    exports.ATTR_KIND_REFS          = ATTR_KIND_REFS;\n    exports.ATTR_KIND_OBJ           = ATTR_KIND_OBJ;\n    exports.ATTR_KIND_OBJS          = ATTR_KIND_OBJS;\n    exports.ATTR_KIND_VAR           = ATTR_KIND_VAR;\n    exports.ATTR_KIND_CUSTOM        = ATTR_KIND_CUSTOM;\n\n    exports.SK_NO                   = SK_NO;\n    exports.SK_YES                  = SK_YES;\n    exports.SK_PROPAGATE            = SK_PROPAGATE;\n\n    exports.SZ_NONE                 = SZ_NONE;\n    exports.SZ_HORZ                 = SZ_HORZ;\n    exports.SZ_VERT                 = SZ_VERT;\n    exports.SZ_RATIO                = SZ_RATIO;\n    exports.SZ_FREE                 = SZ_FREE;\n\n    exports.MM_NONE                 = MM_NONE;\n    exports.MM_HORZ                 = MM_HORZ;\n    exports.MM_VERT                 = MM_VERT;\n    exports.MM_FREE                 = MM_FREE;\n\n    exports.LM_SOLID                = LM_SOLID;\n    exports.LM_DOT                  = LM_DOT;\n\n    exports.LS_RECTILINEAR          = LS_RECTILINEAR;\n    exports.LS_OBLIQUE              = LS_OBLIQUE;\n    exports.LS_ROUNDRECT            = LS_ROUNDRECT;\n    exports.LS_CURVE                = LS_CURVE;\n\n    exports.ES_FLAT                 = ES_FLAT;\n    exports.ES_STICK_ARROW          = ES_STICK_ARROW;\n    exports.ES_SOLID_ARROW          = ES_SOLID_ARROW;\n    exports.ES_TRIANGLE             = ES_TRIANGLE;\n    exports.ES_FILLED_TRIANGLE      = ES_FILLED_TRIANGLE;\n    exports.ES_DIAMOND              = ES_DIAMOND;\n    exports.ES_FILLED_DIAMOND       = ES_FILLED_DIAMOND;\n    exports.ES_ARROW_DIAMOND        = ES_ARROW_DIAMOND;\n    exports.ES_ARROW_FILLED_DIAMOND = ES_ARROW_FILLED_DIAMOND;\n    exports.ES_PLUS                 = ES_PLUS;\n    exports.ES_CIRCLE               = ES_CIRCLE;\n    exports.ES_CIRCLE_PLUS          = ES_CIRCLE_PLUS;\n    exports.ES_CROWFOOT_ONE         = ES_CROWFOOT_ONE;\n    exports.ES_CROWFOOT_MANY        = ES_CROWFOOT_MANY;\n    exports.ES_CROWFOOT_ZERO_ONE    = ES_CROWFOOT_ZERO_ONE;\n    exports.ES_CROWFOOT_ZERO_MANY   = ES_CROWFOOT_ZERO_MANY;\n\n\n    exports.EP_HEAD                 = EP_HEAD;\n    exports.EP_MIDDLE               = EP_MIDDLE;\n    exports.EP_TAIL                 = EP_TAIL;\n\n    exports.DK_HORZ                 = DK_HORZ;\n    exports.DK_VERT                 = DK_VERT;\n\n    exports.DIRECTION_TB            = DIRECTION_TB;\n    exports.DIRECTION_BT            = DIRECTION_BT;\n    exports.DIRECTION_LR            = DIRECTION_LR;\n    exports.DIRECTION_RL            = DIRECTION_RL;\n\n    exports.TK_STRING               = TK_STRING;\n    exports.TK_REFERENCE            = TK_REFERENCE;\n    exports.TK_BOOLEAN              = TK_BOOLEAN;\n    exports.TK_NUMBER               = TK_NUMBER;\n    exports.TK_HIDDEN               = TK_HIDDEN;\n\n\n    // Public Classes\n    exports.Element                 = Element;\n    exports.Model                   = Model;\n    exports.Tag                     = Tag;\n    exports.Hyperlink               = Hyperlink;\n    exports.ExtensibleModel         = ExtensibleModel;\n    exports.Relationship            = Relationship;\n    exports.DirectedRelationship    = DirectedRelationship;\n    exports.RelationshipEnd         = RelationshipEnd;\n    exports.UndirectedRelationship  = UndirectedRelationship;\n    exports.View                    = View;\n    exports.NodeView                = NodeView;\n    exports.EdgeView                = EdgeView;\n    exports.LabelView               = LabelView;\n    exports.ParasiticView           = ParasiticView;\n    exports.NodeParasiticView       = NodeParasiticView;\n    exports.EdgeParasiticView       = EdgeParasiticView;\n    exports.NodeLabelView           = NodeLabelView;\n    exports.EdgeLabelView           = EdgeLabelView;\n    exports.NodeNodeView            = NodeNodeView;\n    exports.EdgeNodeView            = EdgeNodeView;\n    exports.Diagram                 = Diagram;\n    exports.Project                 = Project;\n    exports.Reader                  = Reader;\n    exports.Writer                  = Writer;\n\n    // Public Functions\n    exports.getSuperType            = getSuperType;\n    exports.getCommonType           = getCommonType;\n    exports.findByName              = findByName;\n    exports.getNewName              = getNewName;\n});\n\n\n//# sourceURL=/core/Core.js'),eval("/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _ */\n\n/**\n *\n * This module dispatches these events:\n *     - insert:         (elem)\n *     - remove:         (elem)\n *     - fieldAssign:    (elem, field, val)\n *     - fieldInsert:    (elem, field, val)\n *     - fieldInsertAt:  (elem, field, val, pos)\n *     - fieldRemove:    (elem, field, val)\n *     - fieldRemoveAt:  (elem, field, val, pos)\n *     - fieldReorder:   (elem, field, val, newPos)\n *     - fieldRelocate:  (elem, field, oldParent, newParent)\n */\ndefine('core/OperationBuilder',['require','exports','module','core/Global','core/IdGenerator','core/Core'],function (require, exports, module) {\n    \"use strict\";\n\n    var global      = require(\"core/Global\").global,\n        IdGenerator = require(\"core/IdGenerator\"),\n        Writer      = require(\"core/Core\").Writer;\n\n    /**\n     * @const {string}\n     */\n    var OP_INSERT            = 'I',\n        OP_REMOVE            = 'R',\n        OP_FIELD_ASSIGN      = 'a',\n        OP_FIELD_INSERT      = 'i',\n        OP_FIELD_REMOVE      = 'r',\n        OP_FIELD_REORDER     = 'o',\n        OP_FIELD_RELOCATE    = 'l';\n\n    /**\n     * Current Operation\n     *\n     * @private\n     * @type {Object}\n     */\n    var _currentOperation = null;\n\n    /**\n     * Keep an array of array field of a particular element.\n     *\n     * @private\n     * @type {Object.<string,Array>}\n     */\n    var _currentArray = {};\n\n    /**\n     * Return timestamp\n     */\n    function getTimestamp() {\n        return (new Date()).getTime();\n    }\n\n    /**\n     * Make and return a base operation object.\n     *\n     * @private\n     * @param {string} name Operation name\n     * @return {Object} Base operation object\n     */\n    function _getBase(name) {\n        var operation = {\n            id: IdGenerator.generateGuid(),\n            time: getTimestamp(),\n            name: name,\n            bypass: false,\n            ops: []\n        };\n        return operation;\n    }\n\n    /**\n     * Return a copied array of array field of a particular element.\n     * It could be changed by `fieldInsert`, `fieldRemove`.\n     * The reason why using this is index value maybe incorrect when performing `fieldInsert` multiple times.\n     *\n     * @private\n     * @param {Element} elem An element\n     * @param {string} field Array field name\n     * @return {number}\n     */\n    function _getArray(elem, field) {\n        var f = elem._id + \".\" + field;\n        if (!_currentArray[f]) {\n            _currentArray[f] = _.clone(elem[field]);\n        }\n        return _currentArray[f];\n    }\n\n    /**\n     * Begin to make an operation.\n     *\n     * @param {string} opName Operation name\n     */\n    function begin(name, bypass) {\n        _currentOperation = _getBase(name);\n        if (bypass === true) {\n            _currentOperation.bypass = true;\n        }\n    }\n\n    /**\n     * Finish to make an operation.\n     */\n    function end() {\n        _currentArray = {};\n        return null;\n    }\n\n    /**\n     * Discard currently making operation.\n     */\n    function discard() {\n        _currentOperation = null;\n        _currentArray = {};\n    }\n\n    /**\n     * Return currently made operation.\n     *\n     * @return {Object}\n     */\n    function getOperation() {\n        return _currentOperation;\n    }\n\n    /**\n     * Insert an element.\n     *\n     * @param {Element} elem\n     */\n    function insert(elem) {\n        try {\n            $(exports).triggerHandler('insert', [elem]);\n        } catch (err) {\n            console.error(err);\n        }\n        var writer = new Writer();\n        elem.save(writer);\n        _currentOperation.ops.push({op: OP_INSERT, arg: writer.current});\n    }\n\n    /**\n     * Remove an element.\n     *\n     * @param {Element} elem\n     */\n    function remove(elem) {\n        try {\n            $(exports).triggerHandler('remove', [elem]);\n        } catch (err) {\n            console.error(err);\n        }\n        var writer = new Writer();\n        elem.save(writer);\n        _currentOperation.ops.push({op: OP_REMOVE, arg: writer.current});\n    }\n\n    /**\n     * Assign value to field.\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {?} val\n     */\n    function fieldAssign(elem, field, val) {\n        try {\n            $(exports).triggerHandler('fieldAssign', [elem, field, val]);\n        } catch (err) {\n            console.error(err);\n        }\n        var isCustomField = (elem[field] && elem[field].__read);\n        var oldVal;\n        if (isCustomField) {\n            oldVal = elem[field].__write();\n        } else {\n            oldVal = elem[field];\n        }\n        _currentOperation.ops.push({\n            op: OP_FIELD_ASSIGN,\n            arg: {\n                _id: elem._id,\n                f: field,\n                n: val,\n                o: oldVal\n            }\n        });\n    }\n\n    /**\n     * Insert an element to array field.\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {?} val\n     */\n    function fieldInsert(elem, field, val) {\n        try {\n            $(exports).triggerHandler('fieldInsert', [elem, field, val]);\n        } catch (err) {\n            console.error(err);\n        }\n        var array = _getArray(elem, field);\n        array.push(val);\n        _currentOperation.ops.push({\n            op: OP_FIELD_INSERT,\n            arg: {\n                _id: elem._id,\n                f: field,\n                e: val._id,\n                i: array.indexOf(val)\n            }\n        });\n    }\n\n    /**\n     * Insert a value to array field at a specific position (index).\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {?} val\n     * @param {number} pos\n     */\n    function fieldInsertAt(elem, field, val, pos) {\n        try {\n            $(exports).triggerHandler('fieldInsertAt', [elem, field, val, pos]);\n        } catch (err) {\n            console.error(err);\n        }\n        _currentOperation.ops.push({\n            op: OP_FIELD_INSERT,\n            arg: {\n                _id: elem._id,\n                f: field,\n                e: val._id,\n                i: pos\n            }\n        });\n    }\n\n    /**\n     * Remove a value from array field.\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {?} val\n     */\n    function fieldRemove(elem, field, val) {\n        try {\n            $(exports).triggerHandler('fieldRemove', [elem, field, val]);\n        } catch (err) {\n            console.error(err);\n        }\n        var array = _getArray(elem, field);\n        _currentOperation.ops.push({\n            op: OP_FIELD_REMOVE,\n            arg: {\n                _id: elem._id,\n                f: field,\n                e: val._id,\n                i: array.indexOf(val)\n            }\n        });\n        array.remove(val);\n    }\n\n    /**\n     * Remove a value from array field at a specific position.\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {?} value\n     * @param {number} pos\n     */\n    function fieldRemoveAt(elem, field, val, pos) {\n        try {\n            $(exports).triggerHandler('fieldRemoveAt', [elem, field, val, pos]);\n        } catch (err) {\n            console.error(err);\n        }\n        _currentOperation.ops.push({\n            op: OP_FIELD_REMOVE,\n            arg: {\n                _id: elem._id,\n                f: field,\n                e: val._id,\n                i: pos\n            }\n        });\n    }\n\n    /**\n     * Change order of a value in array field.\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {?} val\n     * @param {number} pos Position to be placed\n     */\n    function fieldReorder(elem, field, val, pos) {\n        try {\n            $(exports).triggerHandler('fieldReorder', [elem, field, val, pos]);\n        } catch (err) {\n            console.error(err);\n        }\n        _currentOperation.ops.push({\n            op: OP_FIELD_REORDER,\n            arg: {\n                _id: elem._id,\n                f: field,\n                e: val._id,\n                i: pos\n            }\n        });\n    }\n\n    /**\n     * Relocate an element to another parent.\n     *\n     * @param {Element} elem Element to be relocated\n     * @param {string} field Field name of parent\n     * @param {number} oldParent Current parent\n     * @param {number} newParent New parent to be located in\n     */\n    function fieldRelocate(elem, field, oldParent, newParent) {\n        try {\n            $(exports).triggerHandler('fieldRelocate', [elem, field, oldParent, newParent]);\n        } catch (err) {\n            console.error(err);\n        }\n        _currentOperation.ops.push({\n            op: OP_FIELD_RELOCATE,\n            arg: {\n                _id: elem._id,\n                f: field,\n                op: oldParent._id,\n                np: newParent._id\n            }\n        });\n    }\n\n    // constants\n    exports.OP_INSERT            = OP_INSERT;\n    exports.OP_REMOVE            = OP_REMOVE;\n    exports.OP_FIELD_ASSIGN      = OP_FIELD_ASSIGN;\n    exports.OP_FIELD_INSERT      = OP_FIELD_INSERT;\n    exports.OP_FIELD_REMOVE      = OP_FIELD_REMOVE;\n    exports.OP_FIELD_REORDER     = OP_FIELD_REORDER;\n    exports.OP_FIELD_RELOCATE    = OP_FIELD_RELOCATE;\n\n    // Define public API\n    exports.begin           = begin;\n    exports.end             = end;\n    exports.discard         = discard;\n    exports.getOperation    = getOperation;\n    exports.insert          = insert;\n    exports.remove          = remove;\n    exports.fieldAssign     = fieldAssign;\n    exports.fieldInsert     = fieldInsert;\n    exports.fieldInsertAt   = fieldInsertAt;\n    exports.fieldRemove     = fieldRemove;\n    exports.fieldRemoveAt   = fieldRemoveAt;\n    exports.fieldReorder    = fieldReorder;\n    exports.fieldRelocate   = fieldRelocate;\n\n});\n\n\n//# sourceURL=/core/OperationBuilder.js"),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, loopfunc: true */\n/*global define, $, _, type*/\n\n/**\n * Repository maintains a set of elements.\n *\n * Reponsibilities:\n * - Applies operations. (Undo/Redo)\n * - Query for retrieve elements.\n *\n * This module dispatches these events:\n * - created:                (elems)\n * - updated:                (elems)\n * - deleted:                (elems)\n * - reordered:              (elem)\n * - relocated:              (elem, field, oldParent, newParent)\n * - beforeExecuteOperation: (operation)\n * - operationExecuted:      (operation) - after doOperation, undo, redo\n * - beforeUndo:             (operation)\n * - beforeRedo:             (operation)\n */\ndefine(\'core/Repository\',[\'require\',\'exports\',\'module\',\'core/Global\',\'core/IdGenerator\',\'core/Core\',\'core/OperationBuilder\'],function (require, exports, module) {\n    "use strict";\n\n    var _global          = require("core/Global").global,\n        IdGenerator      = require("core/IdGenerator"),\n        Core             = require("core/Core"),\n        OperationBuilder = require("core/OperationBuilder");\n\n    /**\n     * Size-limited Stack\n     * @private\n     * @constructor\n     */\n    function Stack(maxSize) {\n        this.maxSize = maxSize;\n        this.stack = [];\n    }\n\n    /**\n     * Clear stack.\n     */\n    Stack.prototype.clear = function () {\n        this.stack = [];\n    };\n\n    /**\n     * Push an item\n     * @param {?} item Item to be stacked.\n     */\n    Stack.prototype.push = function (item) {\n        this.stack.push(item);\n        if (this.stack.length > this.maxSize) {\n            this.stack.splice(0, 1);\n        }\n    };\n\n    /**\n     * Pop an item from the top\n     * @return {?} Item on the top.\n     */\n    Stack.prototype.pop = function () {\n        return this.stack.pop();\n    };\n\n    /**\n     * Return size of stack\n     * @return {number} Size of stack.\n     */\n    Stack.prototype.size = function () {\n        return this.stack.length;\n    };\n\n\n    /**\n     * Maximum size of undo/redo stack\n     * @const\n     */\n    var MAX_STACK_SIZE = 100;\n\n    /**\n     * Map for id to Element.\n     * @private\n     * @type {Object.<string,Element>}\n     */\n    var _idMap = {};\n\n    /**\n     * Inverted index for referencing: {id, {id, refCount}}.\n     * @private\n     * @type {Object.<string,Object.<string,number>>}\n     */\n    var _refMap = {};\n\n    /**\n     * Indicate model is modified or not.\n     * @private\n     * @type {boolean}\n     */\n    var _modified = false;\n\n    /**\n     * Undo stack.\n     * @private\n     * @type {Stack}\n     */\n    var _undoStack = new Stack(MAX_STACK_SIZE);\n\n    /**\n     * Redo stack.\n     * @private\n     * @type {Stack}\n     */\n    var _redoStack = new Stack(MAX_STACK_SIZE);\n\n    /**\n     * Add a reference to the inverted index. (increase refCount by one)\n     *\n     * @private\n     * @param {Core.Element} refer Referencing element.\n     * @param {Core.Element} refee Referenced element.\n     */\n    function _addRef(refer, refee) {\n        var item = _refMap[refee._id];\n        if (!item) {\n            item = {};\n            _refMap[refee._id] = item;\n        }\n        var count = item[refer._id];\n        if (count) {\n            item[refer._id] = count + 1;\n        } else {\n            item[refer._id] = 1;\n        }\n    }\n\n    /**\n     * Remove a reference from the inverted index. (Decrease refCount by one. Remove the entry if refCount reached zero)\n     *\n     * @private\n     * @param {Core.Element} refer Referencing element.\n     * @param {Core.Element} refee Referenced element.\n     */\n    function _removeRef(refer, refee) {\n        var item = _refMap[refee._id];\n        if (item) {\n            var count = item[refer._id];\n            if (count && (count > 0)) {\n                item[refer._id] = count - 1;\n            } else {\n                item[refer._id] = 0;\n            }\n            if (item[refer._id] === 0) {\n                delete item[refer._id];\n            }\n        }\n    }\n\n    /**\n     * 요소의 모든 참조 필드들을 레퍼런스 인덱스에 추가\n     *\n     * @private\n     * @param {Core.Element} elem\n     */\n    function _addRefsOf(elem) {\n        _.forEach(elem.getMetaAttributes(), function (attr) {\n            var i;\n            switch (attr.kind) {\n            case Core.ATTR_KIND_OBJ:\n            case Core.ATTR_KIND_REF:\n                var ref = elem[attr.name];\n                if (ref && _idMap[ref._id]) {\n                    _addRef(elem, ref);\n                }\n                break;\n            case Core.ATTR_KIND_OBJS:\n            case Core.ATTR_KIND_REFS:\n                var refs = elem[attr.name];\n                if (refs && refs.length > 0) {\n                    for (i = 0; i < refs.length; i++) {\n                        if (refs[i] && refs[i]._id && _idMap[refs[i]._id]) {\n                            _addRef(elem, refs[i]);\n                        }\n                    }\n                }\n                break;\n            }\n        });\n    }\n\n    /**\n     * 요소의 모든 참조 필드들을 레퍼런스 인덱스에서 제거 (요소가 제거될 것이므로 본 요소가 가지는 모든 참조는 제거됨)\n     *\n     * @private\n     * @param {Core.Element} elem\n     */\n    function _removeRefsOf(elem) {\n        _.forEach(elem.getMetaAttributes(), function (attr) {\n            var i;\n            switch (attr.kind) {\n            case Core.ATTR_KIND_OBJ:\n            case Core.ATTR_KIND_REF:\n                var ref = elem[attr.name];\n                if (ref && _idMap[ref._id]) {\n                    _removeRef(elem, ref);\n                }\n                break;\n            case Core.ATTR_KIND_OBJS:\n            case Core.ATTR_KIND_REFS:\n                var refs = elem[attr.name];\n                if (refs && refs.length > 0) {\n                    for (i = 0; i < refs.length; i++) {\n                        if (refs[i]._id && _idMap[refs[i]._id]) {\n                            _removeRef(elem, refs[i]);\n                        }\n                    }\n                }\n                break;\n            }\n        });\n        // 인덱스에서 항목을 제거.\n        delete _refMap[elem._id];\n    }\n\n    /**\n     * Operation 수행하기.\n     *\n     * @private\n     * @param {Operation} operation\n     */\n    function _applyOperation(operation) {\n        var i,\n            j,\n            len,\n            op,\n            id,\n            name,\n            elem,\n            obj,\n            reader,\n            val;\n        var createdElems = [],\n            deletedElems = [],\n            updatedElems = [];\n        // 1) 요소들의 생성 및 인덱스 등록\n        for (i = 0, len = operation.ops.length; i < len; i++) {\n            op = operation.ops[i];\n            switch (op.op) {\n            case OperationBuilder.OP_INSERT:\n                reader = new Core.Reader({ data: op.arg }, _global.type);\n                elem = reader.readObj("data");\n                op._elem = elem;\n                op._idMap = reader.idMap;\n                // 인덱스에 등록하기\n                for (id in reader.idMap) {\n                    if (reader.idMap.hasOwnProperty(id)) {\n                        obj = reader.idMap[id];\n                        _idMap[obj._id] = obj;\n                    }\n                }\n                break;\n            case OperationBuilder.OP_REMOVE:\n                reader = new Core.Reader({ data: op.arg }, _global.type);\n                elem = reader.readObj("data");\n                op._elem = elem;\n                op._idMap = reader.idMap;\n                // 레퍼런스 인덱스에서 제거하기\n                for (id in op._idMap) {\n                    if (op._idMap.hasOwnProperty(id)) {\n                        if (!_.isUndefined(_idMap[id])) {\n                            _removeRefsOf(_idMap[id]);\n                        }\n                    }\n                }\n                break;\n            }\n        }\n        // 2) 요소들의 필드값 적용\n        for (i = 0, len = operation.ops.length; i < len; i++) {\n            op = operation.ops[i];\n            switch (op.op) {\n            case OperationBuilder.OP_INSERT:\n                elem = op._elem;\n                // 레퍼런스 값들을 복원하기.\n                for (id in op._idMap) {\n                    if (op._idMap.hasOwnProperty(id)) {\n                        obj = op._idMap[id];\n                        for (name in obj) {\n                            if (obj.hasOwnProperty(name)) {\n                                val = obj[name];\n                                // if val is a reference\n                                if (val && val.$ref) {\n                                    obj[name] = _idMap[val.$ref];\n                                // if val is an array reference\n                                } else if (_.isArray(val)) {\n                                    for (j = 0; j < val.length; j++) {\n                                        if (val[j] && val[j].$ref) {\n                                            val[j] = _idMap[val[j].$ref];\n                                        }\n                                    }\n                                }\n                            }\n                        }\n                        // obj의 참조들을 레퍼런스 인덱스에 추가하기\n                        _addRefsOf(obj);\n                    }\n                }\n                createdElems.push(elem);\n                break;\n            case OperationBuilder.OP_REMOVE:\n                elem = op._elem;\n                // 인덱스에서 제거하기\n                for (id in op._idMap) {\n                    if (op._idMap.hasOwnProperty(id)) {\n                        delete _idMap[id];\n                        delete _refMap[id];\n                    }\n                }\n                deletedElems.push(elem);\n                break;\n            case OperationBuilder.OP_FIELD_ASSIGN:\n                elem = _idMap[op.arg._id];\n                if (elem[op.arg.f] && elem[op.arg.f].__read) {\n                    elem[op.arg.f].__read(op.arg.n);\n                } else {\n                    if (op.arg.n && op.arg.n._id) {\n                        elem[op.arg.f] = _idMap[op.arg.n._id];\n                    } else {\n                        elem[op.arg.f] = op.arg.n;\n                    }\n                    // oldVal를 레퍼런스 인덱스에서 제거\n                    if (op.arg.o && op.arg.o._id) {\n                        _removeRef(elem, op.arg.o);\n                    }\n                    // newVal를 레퍼런스 인덱스에 추가\n                    if (op.arg.n && op.arg.n._id) {\n                        _addRef(elem, op.arg.n);\n                    }\n                }\n                updatedElems.push(elem);\n                break;\n            case OperationBuilder.OP_FIELD_INSERT:\n                elem = _idMap[op.arg._id];\n                val = _idMap[op.arg.e];\n                if (elem && val) {\n                    elem[op.arg.f].insert(op.arg.i, val);\n                    // val을 레퍼런스 인덱스에 추가\n                    if (val._id) {\n                        _addRef(elem, val);\n                    }\n                }\n                updatedElems.push(elem);\n                break;\n            case OperationBuilder.OP_FIELD_REMOVE:\n                elem = _idMap[op.arg._id];\n                val = _idMap[op.arg.e];\n                if (elem && val) {\n                    // 레퍼런스 인덱스에서 제거\n                    if (val._id) {\n                        _removeRef(elem, val);\n                    }\n                    elem[op.arg.f].remove(val);\n                }\n                updatedElems.push(elem);\n                break;\n            case OperationBuilder.OP_FIELD_REORDER:\n                elem = _idMap[op.arg._id];\n                val = _idMap[op.arg.e];\n                // Store old index of val in the array field of element.\n                op.arg.oi = _.indexOf(elem[op.arg.f], val);\n                if (elem && val) {\n                    elem[op.arg.f].remove(val);\n                    elem[op.arg.f].insert(op.arg.i, val);\n                }\n                // updatedElems.push(elem);\n                try {\n                    $(exports).triggerHandler(\'reordered\', [val]);\n                } catch (err) {\n                    console.error(err);\n                }\n                break;\n            case OperationBuilder.OP_FIELD_RELOCATE:\n                elem = _idMap[op.arg._id];\n                var oldParent  = _idMap[op.arg.op];\n                var newParent  = _idMap[op.arg.np];\n                if (elem && oldParent && newParent) {\n                    oldParent[op.arg.f].remove(elem);\n                    newParent[op.arg.f].push(elem);\n                    elem._parent = newParent;\n                    _removeRef(oldParent, elem);\n                    _addRef(newParent, elem);\n                }\n                try {\n                    $(exports).triggerHandler(\'relocated\', [elem, op.arg.f, oldParent, newParent]);\n                } catch (err) {\n                    console.error(err);\n                }\n                break;\n            }\n        }\n        // Bypass Operation가 아니면 이벤트를 발생한다.\n        if (operation.bypass !== true) {\n            if (createdElems.length > 0) {\n                try {\n                    $(exports).triggerHandler(\'created\', [createdElems]);\n                } catch (err) {\n                    console.error(err);\n                }\n            }\n            if (deletedElems.length > 0) {\n                try {\n                    $(exports).triggerHandler(\'deleted\', [deletedElems]);\n                } catch (err) {\n                    console.error(err);\n                }\n            }\n            if (updatedElems.length > 0) {\n                try {\n                    $(exports).triggerHandler(\'updated\', [updatedElems]);\n                } catch (err) {\n                    console.error(err);\n                }\n            }\n            setModified(true);\n        }\n    }\n\n    /**\n     * Operation 되돌리기.\n     *\n     * @private\n     * @param {Operation} operation\n     */\n    function _revertOperation(operation) {\n        var i,\n            j,\n            id,\n            name,\n            op,\n            obj,\n            elem,\n            reader,\n            val;\n        var createdElems = [],\n            deletedElems = [],\n            updatedElems = [];\n        // 1) 요소들의 복구 및 인덱스 등록\n        for (i = operation.ops.length - 1; i >= 0; i--) {\n            op = operation.ops[i];\n            switch (op.op) {\n            case OperationBuilder.OP_INSERT:\n                reader = new Core.Reader({ data: op.arg }, _global.type);\n                elem = reader.readObj("data");\n                op._elem = elem;\n                op._idMap = reader.idMap;\n                // 레퍼런스 인덱스에서 제거하기\n                for (id in op._idMap) {\n                    if (op._idMap.hasOwnProperty(id)) {\n                        _removeRefsOf(_idMap[id]);\n                    }\n                }\n                break;\n            case OperationBuilder.OP_REMOVE:\n                reader = new Core.Reader({ data: op.arg }, _global.type);\n                elem = reader.readObj("data");\n                op._elem = elem;\n                op._idMap = reader.idMap;\n                // 인덱스에 등록하기\n                for (id in reader.idMap) {\n                    if (reader.idMap.hasOwnProperty(id)) {\n                        obj = reader.idMap[id];\n                        _idMap[obj._id] = obj;\n                    }\n                }\n                break;\n            }\n        }\n        // 2) 요소들의 필드값 복구\n        for (i = operation.ops.length - 1; i >= 0; i--) {\n            op = operation.ops[i];\n            switch (op.op) {\n            case OperationBuilder.OP_INSERT:\n                elem = op._elem;\n                // 인덱스에서 제거하기\n                for (id in op._idMap) {\n                    if (op._idMap.hasOwnProperty(id)) {\n                        delete _idMap[id];\n                        delete _refMap[id];\n                    }\n                }\n                deletedElems.push(elem);\n                break;\n            case OperationBuilder.OP_REMOVE:\n                elem = op._elem;\n                // 레퍼런스 값들을 복원하기.\n                for (id in op._idMap) {\n                    if (op._idMap.hasOwnProperty(id)) {\n                        obj = op._idMap[id];\n                        for (name in obj) {\n                            if (obj.hasOwnProperty(name)) {\n                                val = obj[name];\n                                // if val is a reference\n                                if (val && val.$ref) {\n                                    obj[name] = _idMap[val.$ref];\n                                // if val is an array reference\n                                } else if (_.isArray(val)) {\n                                    for (j = 0; j < val.length; j++) {\n                                        if (val[j] && val[j].$ref) {\n                                            val[j] = _idMap[val[j].$ref];\n                                        }\n                                    }\n                                }\n                            }\n                        }\n                        // obj의 참조들을 레퍼런스 인덱스에 추가하기\n                        _addRefsOf(obj);\n                    }\n                }\n                createdElems.push(elem);\n                break;\n            case OperationBuilder.OP_FIELD_ASSIGN:\n                elem = _idMap[op.arg._id];\n                if (elem[op.arg.f] && elem[op.arg.f].__read) {\n                    elem[op.arg.f].__read(op.arg.o);\n                } else {\n                    if (op.arg.o && op.arg.o._id) {\n                        elem[op.arg.f] = _idMap[op.arg.o._id];\n                    } else {\n                        elem[op.arg.f] = op.arg.o;\n                    }\n                    // oldVal를 레퍼런스 인덱스에 추가\n                    if (op.arg.o && op.arg.o._id) {\n                        _addRef(elem, op.arg.o);\n                    }\n                    // newVal를 레퍼런스 인덱스에서 제거\n                    if (op.arg.n && op.arg.n._id) {\n                        _removeRef(elem, op.arg.n);\n                    }\n                }\n                updatedElems.push(elem);\n                break;\n            case OperationBuilder.OP_FIELD_INSERT:\n                elem = _idMap[op.arg._id];\n                val = _idMap[op.arg.e];\n                if (elem && val) {\n                    elem[op.arg.f].remove(val);\n                    // val 레퍼런스 인덱스에서 제거\n                    if (val._id) {\n                        _removeRef(elem, val);\n                    }\n                }\n                updatedElems.push(elem);\n                break;\n            case OperationBuilder.OP_FIELD_REMOVE:\n                elem = _idMap[op.arg._id];\n                val = _idMap[op.arg.e];\n                if (elem && val) {\n                    elem[op.arg.f].insert(op.arg.i, val);\n                    // val을 레퍼런스 인덱스에 추가\n                    if (val._id) {\n                        _addRef(elem, val);\n                    }\n                }\n                updatedElems.push(elem);\n                break;\n            case OperationBuilder.OP_FIELD_REORDER:\n                elem = _idMap[op.arg._id];\n                val = _idMap[op.arg.e];\n                if (elem && val) {\n                    elem[op.arg.f].remove(val);\n                    elem[op.arg.f].insert(op.arg.oi, val);\n                }\n                // updatedElems.push(elem);\n                try {\n                    $(exports).triggerHandler(\'reordered\', [elem]);\n                } catch (err) {\n                    console.error(err);\n                }\n                break;\n            case OperationBuilder.OP_FIELD_RELOCATE:\n                elem = _idMap[op.arg._id];\n                var oldParent  = _idMap[op.arg.op];\n                var newParent  = _idMap[op.arg.np];\n                if (elem && oldParent && newParent) {\n                    newParent[op.arg.f].remove(elem);\n                    oldParent[op.arg.f].push(elem);\n                    elem._parent = oldParent;\n                    _removeRef(newParent, elem);\n                    _addRef(oldParent, elem);\n                }\n                try {\n                    $(exports).triggerHandler(\'relocated\', [elem, op.arg.f, newParent, oldParent]);\n                } catch (err) {\n                    console.error(err);\n                }\n                break;\n            }\n        }\n        if (createdElems.length > 0) {\n            try {\n                $(exports).triggerHandler(\'created\', [createdElems]);\n            } catch (err) {\n                console.error(err);\n            }\n        }\n        if (deletedElems.length > 0) {\n            try {\n                $(exports).triggerHandler(\'deleted\', [deletedElems]);\n            } catch (err) {\n                console.error(err);\n            }\n        }\n        if (updatedElems.length > 0) {\n            try {\n                $(exports).triggerHandler(\'updated\', [updatedElems]);\n            } catch (err) {\n                console.error(err);\n            }\n        }\n        setModified(true);\n    }\n\n    /**\n     * Encode a given element to JSON data.\n     * @private\n     * @param {Core.Element} elem Element to be encoded\n     * @return {string} JSON-encoded data.\n     */\n    function writeObject(elem) {\n        var writer = new Core.Writer();\n        elem.save(writer);\n        var data = JSON.stringify(writer.current, null, "\\t");\n        return data;\n    }\n\n\n    /**\n     * Read object from JSON data\n     * @private\n     * @param {string|Object} data - Object or JSON-string\n     * @param {?boolean} replaceIds\n     * @return {Core.Element}\n     */\n    function readObject(data, replaceIds) {\n        var element = null,\n            id,\n            name,\n            obj,\n            val,\n            _replaceIdMap;\n\n        function _replaceRef(item) {\n            var newId = _replaceIdMap[item.$ref];\n            if (newId) {\n                item.$ref = newId;\n            }\n        }\n\n        function _resolveRef(item) {\n            return _idMap[item.$ref];\n        }\n\n        if (data) {\n            if (_.isString(data)) {\n                data = JSON.parse(data);\n            }\n            var reader = new Core.Reader({data: data}, _global.type);\n            element = reader.readObj("data");\n\n            // Replace all ids\n            if (replaceIds) {\n\n                // Map.<oldId, newId>\n                _replaceIdMap = {};\n\n                // Assign new ids\n                for (id in reader.idMap) {\n                    if (reader.idMap.hasOwnProperty(id)) {\n                        var newId = IdGenerator.generateGuid();\n                        obj = reader.idMap[id];\n                        _replaceIdMap[obj._id] = newId;\n                        delete reader.idMap[obj._id];\n                        obj._id = newId;\n                        reader.idMap[newId] = obj;\n                    }\n                }\n\n                // Fix all refs to refer to new ids\n                for (id in reader.idMap) {\n                    if (reader.idMap.hasOwnProperty(id)) {\n                        obj = reader.idMap[id];\n                        for (name in obj) {\n                            if (obj.hasOwnProperty(name)) {\n                                val = obj[name];\n                                // when val is reference\n                                if (val && val.$ref) {\n                                    _replaceRef(val);\n                                // when val is non-empty array of reference\n                                } else if (_.isArray(val) && val.length > 0 && val[0].$ref) {\n                                    _.each(val, _replaceRef);\n                                }\n                            }\n                        }\n\n                    }\n                }\n            }\n\n            // Resolve all references\n            _.extend(_idMap, reader.idMap);\n            for (id in reader.idMap) {\n                if (reader.idMap.hasOwnProperty(id)) {\n                    obj = _idMap[id];\n                    for (name in obj) {\n                        if (obj.hasOwnProperty(name)) {\n                            val = obj[name];\n                            // when val is reference\n                            if (val && val.$ref) {\n                                obj[name] = _idMap[val.$ref];\n                            // when val is non-empty array of reference\n                            } else if (_.isArray(val) && val.length > 0 && val[0].$ref) {\n                                var resolvedRefs = _.map(val, _resolveRef);\n                                obj[name] = resolvedRefs;\n                            }\n                        }\n                    }\n                    _addRefsOf(obj);\n                }\n            }\n\n            // Fix problems in the file\n            // TODO: Remove this enough time after\n            for (id in reader.idMap) {\n                if (reader.idMap.hasOwnProperty(id)) {\n                    obj = _idMap[id];\n\n                    // Fix _parent of Image owned by Stereotype\n                    if (obj instanceof type.UMLStereotype && obj.icon instanceof type.UMLImage) {\n                        obj.icon._parent = obj;\n                    }\n\n                    // Fix: remove disconnected UndirectedRelationships\n                    if (obj instanceof type.UndirectedRelationship) {\n                        if (!(obj.end1 instanceof type.RelationshipEnd &&\n                              obj.end1.reference instanceof type.Model &&\n                              obj.end2 instanceof type.RelationshipEnd &&\n                              obj.end2.reference instanceof type.Model)) {\n                            if (obj._parent && obj.getParentField()) {\n                                obj._parent[obj.getParentField()].remove(obj);\n                                obj._parent = null;\n                                obj.end1.reference = null;\n                                obj.end2.reference = null;\n                                delete reader.idMap[obj._id];\n                            }\n                        }\n                    }\n\n                    // Fix: remove disconnected DirectedRelationships\n                    if (obj instanceof type.DirectedRelationship) {\n                        if (!(obj.source instanceof type.Model &&\n                              obj.target instanceof type.Model)) {\n                            if (obj._parent && obj.getParentField()) {\n                                obj._parent[obj.getParentField()].remove(obj);\n                                obj._parent = null;\n                                obj.source = null;\n                                obj.target = null;\n                                delete reader.idMap[obj._id];\n                            }\n                        }\n                    }\n\n                    // Fix: Clear too large distance values\n                    if (obj instanceof type.ParasiticView) {\n                        if (obj.distance > 5000) {\n                            obj.distance = 10;\n                        }\n                    }\n\n                    // Fix: problems in diagram\n                    if (obj instanceof type.Diagram) {\n                        var diagram = obj;\n\n                        // 1) Communication Diagram에서 hostEdge가 null인 UMLCommMessageView를 모두 지움.\n                        if (diagram instanceof type.UMLCommunicationDiagram) {\n                            _.each(diagram.ownedViews, function (v) {\n                                if (v instanceof type.UMLCommMessageView && v.hostEdge === null) {\n                                    diagram.removeOwnedView(v);\n                                    delete reader.idMap[v._id];\n                                }\n                            });\n                        }\n\n                        for (var vi = 0, vlen = diagram.ownedViews.length; vi < vlen; vi++) {\n                            var v = diagram.ownedViews[vi];\n\n                            if (!v) { continue; }\n\n                            // 1) model이 없는 UMLGeneralNodeView, UMLGeneralEdgeView를 모두 삭제.\n                            if (!v.model && v instanceof type.UMLGeneralNodeView) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (!v.model && v instanceof type.UMLGeneralEdgeView) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n\n                            // 2) _parent가 없는 View들 모두 삭제\n                            if (!v._parent) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n\n                            // 3) head or tail이 없는 EdgeView는 삭제\n                            if (v instanceof type.EdgeView) {\n                                if (!v.head || !v.tail) {\n                                    diagram.removeOwnedView(v);\n                                    delete reader.idMap[v._id];\n                                }\n                            }\n\n                            // 4) end1.reference or end2.reference가 없는 UndirectedRelationship을 모두 삭제.\n                            if (v.model instanceof type.UndirectedRelationship) {\n                                if (!v.model.end1.reference || !v.model.end2.reference) {\n                                    var pf1 = v.model.getParentField();\n                                    if (v.model._parent && pf1) {\n                                        v.model._parent[pf1].remove(v.model);\n                                    }\n                                    diagram.removeOwnedView(v);\n                                    delete reader.idMap[v._id];\n                                }\n                            }\n\n                            // 5) source or target이 없는 DirectedRelationship을 모두 삭제.\n                            if (v.model instanceof type.DirectedRelationship) {\n                                if (!v.model.source || !v.model.target) {\n                                    var pf2 = v.model.getParentField();\n                                    if (v.model._parent && pf2) {\n                                        v.model._parent[pf2].remove(v.model);\n                                    }\n                                    diagram.removeOwnedView(v);\n                                    delete reader.idMap[v._id];\n                                }\n                            }\n\n                            // 5) nameLabel, stereotypeLabel, propertyLabel, RoleNameLabel, MultiplicityLabel, PropertyLabel, QualifiersCompartment\n                            if (v.nameLabel && !reader.idMap[v.nameLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.stereotypeLabel && !reader.idMap[v.stereotypeLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.propertyLabel && !reader.idMap[v.propertyLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.tailRoleNameLabel && !reader.idMap[v.tailRoleNameLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.tailPropertyLabel && !reader.idMap[v.tailPropertyLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.tailMultiplicityLabel && !reader.idMap[v.tailMultiplicityLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.tailQualifiersCompartment && !reader.idMap[v.tailQualifiersCompartment._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.headRoleNameLabel && !reader.idMap[v.headRoleNameLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.headPropertyLabel && !reader.idMap[v.headPropertyLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.headMultiplicityLabel && !reader.idMap[v.headMultiplicityLabel._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n                            if (v.headQualifiersCompartment && !reader.idMap[v.headQualifiersCompartment._id]) {\n                                diagram.removeOwnedView(v);\n                                delete reader.idMap[v._id];\n                            }\n\n                        }\n\n                    }\n\n                }\n            }\n        }\n        return element;\n    }\n\n\n    /**\n     * Extract changed elements from a given Operation\n     * @param {Operation} operation\n     * @return {Array.<Element>}\n     */\n    function extractChanged(operation) {\n        var i, len, op, elem, changed = [];\n        if (operation.ops.length > 0) {\n            for (i = 0, len = operation.ops.length; i < len; i++) {\n                op = operation.ops[i];\n                if (op._elem && op._elem._id) {\n                    elem = get(op._elem._id);\n                    if (elem && !_.contains(changed, elem)) {\n                        changed.push(elem);\n                    }\n                }\n                if (op.arg && op.arg._id) {\n                    elem = get(op.arg._id);\n                    if (elem && !_.contains(changed, elem)) {\n                        changed.push(elem);\n                    }\n                }\n            }\n        }\n        return changed;\n    }\n\n    /**\n     * Do an Operation\n     * If operation.bypass == true, the operation will not be pushed to UndoStack.\n     * @param {Operation} operation\n     */\n    function doOperation(operation) {\n        if (operation.ops.length > 0) {\n            try {\n                $(exports).triggerHandler(\'beforeExecuteOperation\', [operation]);\n                _applyOperation(operation);\n                if (operation.bypass !== true) {\n                    _undoStack.push(operation);\n                    _redoStack.clear();\n                    $(exports).triggerHandler(\'operationExecuted\', [operation]);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        }\n    }\n\n    /**\n     * Undo\n     */\n    function undo() {\n        if (_undoStack.size() > 0) {\n            try {\n                var operation = _undoStack.pop();\n                $(exports).triggerHandler(\'beforeUndo\', [operation]);\n                _revertOperation(operation);\n                _redoStack.push(operation);\n                $(exports).triggerHandler(\'undo\', [operation]);\n                $(exports).triggerHandler(\'operationExecuted\', [operation]);\n            } catch (err) {\n                console.error(err);\n            }\n        }\n    }\n\n    /**\n     * Redo\n     */\n    function redo() {\n        if (_redoStack.size() > 0) {\n            try {\n                var operation = _redoStack.pop();\n                $(exports).triggerHandler(\'beforeRedo\', [operation]);\n                _applyOperation(operation);\n                _undoStack.push(operation);\n                $(exports).triggerHandler(\'redo\', [operation]);\n                $(exports).triggerHandler(\'operationExecuted\', [operation]);\n            } catch (err) {\n                console.error(err);\n            }\n        }\n    }\n\n    /**\n     * Clear all maps and stacks\n     */\n    function clear() {\n        _idMap = {};\n        _refMap = {};\n        _undoStack.clear();\n        _redoStack.clear();\n        setModified(false);\n    }\n\n    /**\n     * Return whether project is modified or not.\n     * @return {boolean}\n     */\n    function isModified() {\n        return _modified;\n    }\n\n    /**\n     * Set model as modified. (A event will be triggered if modified state is changed.)\n     * @private\n     * @param {boolean} modified\n     */\n    function setModified(modified) {\n        _modified = modified;\n        if (_modified) {\n            try {\n                $(exports).triggerHandler(\'modified\');\n            } catch (err) {\n                console.error(err);\n            }\n        }\n    }\n\n    /**\n     * Return true if the given parameter is an element in this Repository.\n     * @param {Element} elem\n     * @return {boolean}\n     */\n    function isElement(elem) {\n        return (elem && elem._id && get(elem._id));\n    }\n\n    /**\n     * Return an array of elements selected by selector expression.\n     * This is a quite heavy operation, so you need to concern about performance.\n     *\n     * Selector expression\n     *     - Children selector\n     *       ex) Package1:: -- all children of Package1\n     *\n     *     - Type selector: "@<type>"\n     *       ex) Package1::@UMLClass\n     *\n     *     - Field selector: ".<field>"\n     *       ex) Class1.attributes, Package1.owendElements\n     *\n     *     - Value selector: "[field=value]"\n     *       ex) Class1.operations[isAbstract=false]\n     *\n     *     - Name selector: "<name>" (equivalent to "[name=<name>]")\n     *       ex) Class1, Class1::Attribute1\n     *\n     * Selector examples:\n     *     @UMLClass\n     *     Package1::Class1.attributes[type=String]\n     *     Package1::Model1::@UMLInterface.operations[isAbstract=false]\n     *\n     * @param {string} selector\n     * @return {Array.<Element>}\n     */\n    function select(selector) {\n        selector = selector || "";\n\n        // Parse selector into an array of terms\n        var interm = selector\n            .replace(/::/g, "\\n::\\n")\n            .replace(/@/g, "\\n@")\n            .replace(/\\./g, "\\n.")\n            .replace(/\\[/g, "\\n[");\n\n        var i, len,\n            sliced = interm.split("\\n"),\n            terms = [];\n\n        for (i = 0, len = sliced.length; i < len; i++) {\n            var item = sliced[i].trim(), arg;\n            // children selector\n            if (item === "::") {\n                terms.push({ op: "::" });\n            // type selector\n            } else if (item.charAt(0) === "@") {\n                arg = item.substring(1, item.length).trim();\n                if (arg.length === 0) {\n                    throw "[Selector] Type selector requires type name after \'@\'";\n                }\n                terms.push({ op: "@", type: arg });\n            // field selector\n            } else if (item.charAt(0) === ".") {\n                arg = item.substring(1, item.length).trim();\n                if (arg.length === 0) {\n                    throw "[Selector] Field selector requires field name after \'.\'";\n                }\n                terms.push({ op: ".", field: arg});\n            // value selector\n            } else if (item.charAt(0) === "[") {\n                arg = item.substring(1, item.length - 1);\n                var fv = arg.split("="), f = fv[0] || "", v = fv[1] || "";\n                if (!(item.charAt(item.length - 1) === "]" && fv.length === 2 && f.trim().length > 0 && v.trim().length > 0)) {\n                    throw "[Selector] Value selector should be format of \'[field=value]\'";\n                }\n                terms.push({ op: "[]", field: f.trim(), value: v.trim()});\n            // name selector\n            } else if (item.length > 0) {\n                terms.push({ op: "name", name: item });\n            }\n        }\n\n        // Process terms sequentially\n        var current = _.values(_idMap),\n            term,\n            elems;\n        for (i = 0, len = terms.length; i < len; i++) {\n            term = terms[i];\n            elems = [];\n            switch (term.op) {\n            case "::":\n                current.forEach(function (e) {\n                    elems = _.union(elems, e.getChildren());\n                });\n                current = elems;\n                break;\n            case "@":\n                current.forEach(function (e) {\n                    if (type[term.type] && e instanceof type[term.type]) {\n                        elems.push(e);\n                    }\n                });\n                current = elems;\n                break;\n            case ".":\n                current.forEach(function (e) {\n                    if (typeof e[term.field] !== "undefined") {\n                        var val = e[term.field];\n                        if (isElement(val)) {\n                            elems.push(val);\n                        }\n                        if (Array.isArray(val)) {\n                            val.forEach(function (e2) {\n                                if (isElement(e2)) {\n                                    elems.push(e2);\n                                }\n                            });\n                        }\n                    }\n                });\n                current = elems;\n                break;\n            case "[]":\n                current.forEach(function (e) {\n                    if (typeof e[term.field] !== "undefined") {\n                        var val = e[term.field];\n                        if (term.value == val) {\n                            elems.push(e);\n                        }\n                    }\n                });\n                current = elems;\n                break;\n            case "name":\n                current.forEach(function (e) {\n                    if (e.name === term.name) {\n                        elems.push(e);\n                    }\n                });\n                current = elems;\n                break;\n            }\n        }\n\n        return current;\n    }\n\n    /**\n     * Return element by id.\n     * @param {string} id Identifier of element.\n     * @return {Element} Element of id.\n     */\n    function get(id) {\n        return _idMap[id];\n    }\n\n    /**\n     * Return instances of a specified type name(s).\n     * @param {string|Array.<string>} _typeName Type name(s) of instances to be returned.\n     * @return {Array.<Element>} instances of the type name(s).\n     */\n    function getInstancesOf(_typeName) {\n        var _instances = [],\n            _typeNames = [];\n\n        if (_.isArray(_typeName)) {\n            _typeNames = _typeName;\n        } else if (_.isString(_typeName)) {\n            _typeNames.push(_typeName);\n        }\n\n        _.each(_idMap, function (elem) {\n            var i, len, _typeTest = false;\n\n            for (i = 0, len = _typeNames.length; i < len; i++) {\n                if (elem instanceof _global.type[_typeNames[i]]) {\n                    _typeTest = true;\n                    break;\n                }\n            }\n\n            if (_typeTest) {\n                _instances.push(elem);\n            }\n        });\n\n        return _instances;\n    }\n\n    /**\n     * Find the first matched element satisfying the predicate.\n     * @param {function(Element):boolean} predicate A function to filter elements.\n     * @return {Element} A matched element.\n     */\n    function find(predicate) {\n        var key, elem;\n        for (key in _idMap) {\n            if (_idMap.hasOwnProperty(key)) {\n                elem = _idMap[key];\n                if (predicate(elem)) {\n                    return elem;\n                }\n            }\n        }\n        return null;\n    }\n\n    /**\n     * Find all elements satisfying the predicate.\n     * @param {function(Element):boolean} predicate A function to filter elements.\n     * @return {Array.<Element>} All matched elements.\n     */\n    function findAll(predicate) {\n        var key,\n            elem,\n            result = [];\n        for (key in _idMap) {\n            if (_idMap.hasOwnProperty(key)) {\n                elem = _idMap[key];\n                if (predicate(elem)) {\n                    result.push(elem);\n                }\n            }\n        }\n        return result;\n    }\n\n\n    /**\n     * Search elements by keyword and type\n     * @param {string} keyword\n     * @param {constructor} typeFilter\n     * @return {Array.<Element>} elements\n     */\n    function search(keyword, typeFilter) {\n        keyword = keyword.toLowerCase();\n        typeFilter = typeFilter || type.Element;\n        var results = findAll(function (elem) {\n            var name = elem.name ? elem.name.toLowerCase() : "";\n            return (name.indexOf(keyword) > -1 && elem instanceof typeFilter);\n        });\n        return results;\n    }\n\n    /**\n     * Lookup an element and then find. (See `Element.prototype.lookup` and `find`).\n     * @param {!Element} namespace Element to start to lookup.\n     * @param {string} name Name of element to find\n     * @param {constructor} typeFilter Type filter. (e.g. `type.UMLClass`)\n     * @return {Element} A matched element.\n     */\n    function lookupAndFind(namespace, name, typeFilter) {\n        var ref = namespace.lookup(name, typeFilter);\n        if (ref === null) {\n            ref = find(function (elem) {\n                return ((elem instanceof typeFilter) && (elem.name === name));\n            });\n        }\n        return ref;\n    }\n\n\n    /**\n     * Return all elements referencing to the given element.\n     * @param {Core.Element} elem Element. (model element, view element, or diagram)\n     * @param {?function(Element):boolean} iterator if given, returns instances only satisfying iterator function.\n     * @return {Array<Core.Element>} Elements referencing to.\n     */\n    function getRefsTo(elem, iterator) {\n        var id,\n            ref,\n            obj,\n            list = [];\n        if (elem) {\n            obj = _refMap[elem._id];\n            if (obj) {\n                for (id in obj) {\n                    if (obj.hasOwnProperty(id)) {\n                        ref = _idMap[id];\n                        if (iterator) {\n                            if (iterator(ref)) { list.push(ref); }\n                        } else {\n                            list.push(ref);\n                        }\n                    }\n                }\n            }\n        }\n        return list;\n    }\n\n    /**\n     * Return all instances of Relationship connected to the given model element.\n     * @param {Core.Model} model Model element.\n     * @param {?function(Element):boolean} iterator if given, returns instances only satisfying iterator function.\n     * @return {Array<Core.Model>} Instances of Relationship.\n     */\n    function getRelationshipsOf(model, iterator) {\n        var i,\n            len,\n            ref,\n            refs = getRefsTo(model),\n            results = [];\n\n        function _add(rel) {\n            if (!_.contains(results, rel)) {\n                results.push(rel);\n            }\n        }\n\n        for (i = 0, len = refs.length; i < len; i++) {\n            ref = refs[i];\n\n            // for DirectedRelationship\n            if ((ref instanceof Core.DirectedRelationship) && (ref.source === model || ref.target === model)) {\n                if (iterator) {\n                    if (iterator(ref)) {\n                        _add(ref);\n                    }\n                } else {\n                    _add(ref);\n                }\n            }\n\n            // for UndirectedRelationship\n            if ((ref instanceof Core.RelationshipEnd) && (ref.reference === model)) {\n                if (iterator) {\n                    if (iterator(ref._parent)) {\n                        _add(ref._parent);\n                    }\n                } else {\n                    _add(ref._parent);\n                }\n            }\n        }\n        return results;\n    }\n\n    /**\n     * Return all views associated with the given model.\n     * @param {Core.Model} model Model element.\n     * @return {Array<Core.View>} View elements associated with.\n     */\n    function getViewsOf(model) {\n        return getRefsTo(model, function (ref) {\n            return (ref instanceof Core.View) && (ref.model === model);\n        });\n    }\n\n    /**\n     * Return all instances of EdgeView linked to the given view.\n     * @param {Core.View} view View element. Typically an instance of NodeView.\n     * @return {Array<Core.EdgeView>} Instances of EdgeView linked to.\n     */\n    function getEdgeViewsOf(view) {\n        return getRefsTo(view, function (ref) {\n            return (ref instanceof Core.EdgeView) &&\n                   (ref.head === view || ref.tail === view);\n        });\n    }\n\n    /**\n     * Insert an element to an array field. This will not insert an Operation into UndoStack (bypass = true). Use it carefully.\n     * @param {Core.Element} parent Field owner.\n     * @param {string} field Field name.\n     * @param {Core.Element} elem An element to be inserted.\n     */\n    function bypassInsert(parent, field, elem) {\n        OperationBuilder.begin("bypassInsert", true);\n        OperationBuilder.insert(elem);\n        OperationBuilder.fieldInsert(parent, field, elem);\n        OperationBuilder.end();\n        doOperation(OperationBuilder.getOperation());\n    }\n\n    /**\n     * Assign a value to a field. This will not insert an Operation into UndoStack (bypass = true). Use it carefully.\n     * @param {module:core/Core.Element} elem Field owner.\n     * @param {string} field Field name.\n     * @param {Core.Element} val Value to be assigned to.\n     */\n    function bypassFieldAssign(elem, field, val) {\n        OperationBuilder.begin("bypassFieldAssign", true);\n        OperationBuilder.fieldAssign(elem, field, val);\n        OperationBuilder.end();\n        doOperation(OperationBuilder.getOperation(), true);\n    }\n\n    // Define public API\n    exports.writeObject         = writeObject;\n    exports.readObject          = readObject;\n    exports.extractChanged      = extractChanged;\n    exports.doOperation         = doOperation;\n    exports.undo                = undo;\n    exports.redo                = redo;\n    exports.clear               = clear;\n    exports.isModified          = isModified;\n    exports.setModified         = setModified;\n    exports.isElement           = isElement;\n    exports.select              = select;\n    exports.get                 = get;\n    exports.getInstancesOf      = getInstancesOf;\n    exports.find                = find;\n    exports.findAll             = findAll;\n    exports.search              = search;\n    exports.lookupAndFind       = lookupAndFind;\n    exports.getRefsTo           = getRefsTo;\n    exports.getRelationshipsOf  = getRelationshipsOf;\n    exports.getViewsOf          = getViewsOf;\n    exports.getEdgeViewsOf      = getEdgeViewsOf;\n\n    exports.bypassInsert        = bypassInsert;\n    exports.bypassFieldAssign   = bypassFieldAssign;\n\n    exports.getIdMap  = function () { return _idMap; };  // for test\n    exports.getRefMap = function () { return _refMap; }; // for test\n\n    exports.Stack = Stack; // for test\n\n});\n\n//# sourceURL=/core/Repository.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define */\n\n/**\n * FileSystemError describes the errors that can occur when using the FileSystem, File,\n * and Directory modules.\n *\n * Error values are strings. Any "falsy" value: null, undefined or "" means "no error".\n */\ndefine(\'filesystem/FileSystemError\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n    module.exports = {\n        UNKNOWN                     : "Unknown",\n        INVALID_PARAMS              : "InvalidParams",\n        NOT_FOUND                   : "NotFound",\n        NOT_READABLE                : "NotReadable",\n        NOT_SUPPORTED               : "NotSupported",\n        NOT_WRITABLE                : "NotWritable",\n        OUT_OF_SPACE                : "OutOfSpace",\n        TOO_MANY_ENTRIES            : "TooManyEntries",\n        ALREADY_EXISTS              : "AlreadyExists",\n        CONTENTS_MODIFIED           : "ContentsModified",\n        ROOT_NOT_WATCHED            : "RootNotBeingWatched"\n        \n        // FUTURE: Add remote connection errors: timeout, not logged in, connection err, etc.\n    };\n});\n\n//# sourceURL=/filesystem/FileSystemError.js'),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define, $, brackets */\n\n/**\n *  Utilities functions related to string manipulation\n *\n */\ndefine(\'utils/StringUtils\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n    \n    // var _ = require("thirdparty/lodash");\n\n    /**\n     * Format a string by replacing placeholder symbols with passed in arguments.\n     *\n     * Example: var formatted = StringUtils.format("Hello {0}", "World");\n     *\n     * @param {string} str The base string\n     * @param {...} Arguments to be substituted into the string\n     *\n     * @return {string} Formatted string\n     */\n    function format(str) {\n        // arguments[0] is the base string, so we need to adjust index values here\n        var args = [].slice.call(arguments, 1);\n        return str.replace(/\\{(\\d+)\\}/g, function (match, num) {\n            return typeof args[num] !== "undefined" ? args[num] : match;\n        });\n    }\n\n    /**\n     * @deprecated\n     */\n    function htmlEscape(str) {\n        console.warn("StringUtils.htmlEscape is deprecated. Use _.escape instead.");\n        return _.escape(str);\n    }\n\n    function regexEscape(str) {\n        return str.replace(/([.?*+\\^$\\[\\]\\\\(){}|\\-])/g, "\\\\$1");\n    }\n\n    // Periods (aka "dots") are allowed in HTML identifiers, but jQuery interprets\n    // them as the start of a class selector, so they need to be escaped\n    function jQueryIdEscape(str) {\n        return str.replace(/\\./g, "\\\\.");\n    }\n\n    /**\n     * Splits the text by new line characters and returns an array of lines\n     * @param {string} text\n     * @return {Array.<string>} lines\n     */\n    function getLines(text) {\n        return text.split("\\n");\n    }\n\n    /**\n     * Returns a line number corresponding to an offset in some text. The text can\n     * be specified as a single string or as an array of strings that correspond to\n     * the lines of the string.\n     *\n     * Specify the text in lines when repeatedly calling the function on the same\n     * text in a loop. Use getLines() to divide the text into lines, then repeatedly call\n     * this function to compute a line number from the offset.\n     *\n     * @param {string | Array.<string>} textOrLines - string or array of lines from which\n     *      to compute the line number from the offset\n     * @param {number} offset\n     * @return {number} line number\n     */\n    function offsetToLineNum(textOrLines, offset) {\n        if (Array.isArray(textOrLines)) {\n            var lines = textOrLines,\n                total = 0,\n                line;\n            for (line = 0; line < lines.length; line++) {\n                if (total < offset) {\n                    // add 1 per line since /n were removed by splitting, but they needed to \n                    // contribute to the total offset count\n                    total += lines[line].length + 1;\n                } else if (total === offset) {\n                    return line;\n                } else {\n                    return line - 1;\n                }\n            }\n\n            // if offset is NOT over the total then offset is in the last line\n            if (offset <= total) {\n                return line - 1;\n            } else {\n                return undefined;\n            }\n        } else {\n            return textOrLines.substr(0, offset).split("\\n").length - 1;\n        }\n    }\n    \n    /**\n     * Returns true if the given string ends with the given suffix.\n     *\n     * @param {string} str\n     * @param {string} suffix\n     */\n    function endsWith(str, suffix) {\n        return str.indexOf(suffix, str.length - suffix.length) !== -1;\n    }\n\n    function urlSort(a, b) {\n        var a2, b2;\n        function isFile(s) {\n            return ((s.lastIndexOf("/") + 1) < s.length);\n        }\n\n        if (brackets.platform === "win") {\n            // Windows: prepend folder names with a \'0\' and file names with a \'1\' so folders are listed first\n            a2 = ((isFile(a)) ? "1" : "0") + a.toLowerCase();\n            b2 = ((isFile(b)) ? "1" : "0") + b.toLowerCase();\n        } else {\n            a2 = a.toLowerCase();\n            b2 = b.toLowerCase();\n        }\n\n        if (a2 === b2) {\n            return 0;\n        } else {\n            return (a2 > b2) ? 1 : -1;\n        }\n    }\n    \n    /**\n     * Return an escaped path or URL string that can be broken near path separators.\n     * @param {string} url the path or URL to format\n     * @return {string} the formatted path or URL\n     */\n    function breakableUrl(url) {\n        // This is for displaying in UI, so always want it escaped\n        var escUrl = _.escape(url);\n\n        // Inject zero-width space character (U+200B) near path separators (/) to allow line breaking there\n        return escUrl.replace(\n            new RegExp(regexEscape("/"), "g"),\n            "/" + "&#8203;"\n        );\n    }\n    \n    /**\n     * Convert number of bytes into human readable format. \n     * If param bytes is negative it returns the number without any changes.\n     *\n     * @param number bytes     Number of bytes to convert\n     * @param number precision Number of digits after the decimal separator\n     * @return string\n     */\n    function prettyPrintBytes(bytes, precision) {\n        var kilobyte = 1024,\n            megabyte = kilobyte * 1024,\n            gigabyte = megabyte * 1024,\n            terabyte = gigabyte * 1024,\n            returnVal = bytes;\n        \n        if ((bytes >= 0) && (bytes < kilobyte)) {\n            returnVal = bytes + " B";\n        } else if (bytes < megabyte) {\n            returnVal = (bytes / kilobyte).toFixed(precision) + " KB";\n        } else if (bytes < gigabyte) {\n            returnVal = (bytes / megabyte).toFixed(precision) + " MB";\n        } else if (bytes < terabyte) {\n            returnVal = (bytes / gigabyte).toFixed(precision) + " GB";\n        } else if (bytes >= terabyte) {\n            return (bytes / terabyte).toFixed(precision) + " TB";\n        }\n        \n        return returnVal;\n    }\n        \n\n    // Define public API\n    exports.format              = format;\n    exports.htmlEscape          = htmlEscape;\n    exports.regexEscape         = regexEscape;\n    exports.jQueryIdEscape      = jQueryIdEscape;\n    exports.getLines            = getLines;\n    exports.offsetToLineNum     = offsetToLineNum;\n    exports.urlSort             = urlSort;\n    exports.breakableUrl        = breakableUrl;\n    exports.endsWith            = endsWith;\n    exports.prettyPrintBytes    = prettyPrintBytes;\n});\n\n//# sourceURL=/utils/StringUtils.js'),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define, $, FileError, unescape, window, app */\n\n/**\n * Set of utilites for working with files and text content.\n */\ndefine(\'file/FileUtils\',[\'require\',\'exports\',\'module\',\'core/Global\',\'filesystem/FileSystemError\',\'utils/StringUtils\'],function (require, exports, module) {\n    "use strict";\n\n    require("core/Global");\n\n    var FileSystemError     = require("filesystem/FileSystemError"),\n        // PerfUtils           = require("utils/PerfUtils"),\n        // Dialogs             = require("dialogs/Dialogs"),\n        // DefaultDialogs      = require("widgets/DefaultDialogs"),\n        // Strings             = require("strings"),\n        StringUtils         = require("utils/StringUtils");\n\n\n    /**\n     * Asynchronously reads a file as UTF-8 encoded text.\n     * @param {!File} file File to read\n     * @return {$.Promise} a jQuery promise that will be resolved with the\n     *  file\'s text content plus its timestamp, or rejected with a FileSystemError string\n     *  constant if the file can not be read.\n     */\n    function readAsText(file) {\n        var result = new $.Deferred();\n\n        // Measure performance\n        /*\n        var perfTimerName = PerfUtils.markStart("readAsText:\\t" + file.fullPath);\n        result.always(function () {\n            PerfUtils.addMeasurement(perfTimerName);\n        });\n        */\n\n        // Read file\n        file.read(function (err, data, stat) {\n            if (!err) {\n                result.resolve(data, stat.mtime);\n            } else {\n                result.reject(err);\n            }\n        });\n\n        return result.promise();\n    }\n\n    /**\n     * Asynchronously writes a file as UTF-8 encoded text.\n     * @param {!File} file File to write\n     * @param {!string} text\n     * @param {boolean=} allowBlindWrite Indicates whether or not CONTENTS_MODIFIED\n     *      errors---which can be triggered if the actual file contents differ from\n     *      the FileSystem\'s last-known contents---should be ignored.\n     * @return {$.Promise} a jQuery promise that will be resolved when\n     * file writing completes, or rejected with a FileSystemError string constant.\n     */\n    function writeText(file, text, allowBlindWrite) {\n        var result = new $.Deferred(),\n            options = {};\n\n        if (allowBlindWrite) {\n            options.blind = true;\n        }\n\n        file.write(text, options, function (err) {\n            if (!err) {\n                result.resolve();\n            } else {\n                result.reject(err);\n            }\n        });\n\n        return result.promise();\n    }\n\n    /** @const */\n    var LINE_ENDINGS_CRLF = "CRLF";\n    /** @const */\n    var LINE_ENDINGS_LF = "LF";\n\n    /**\n     * Returns the standard line endings for the current platform\n     * @return {LINE_ENDINGS_CRLF|LINE_ENDINGS_LF}\n     */\n    function getPlatformLineEndings() {\n        return app.platform === "win" ? LINE_ENDINGS_CRLF : LINE_ENDINGS_LF;\n    }\n\n    /**\n     * Scans the first 1000 chars of the text to determine how it encodes line endings. Returns\n     * null if usage is mixed or if no line endings found.\n     * @param {!string} text\n     * @return {null|LINE_ENDINGS_CRLF|LINE_ENDINGS_LF}\n     */\n    function sniffLineEndings(text) {\n        var subset = text.substr(0, 1000);  // (length is clipped to text.length)\n        var hasCRLF = /\\r\\n/.test(subset);\n        var hasLF = /[^\\r]\\n/.test(subset);\n\n        if ((hasCRLF && hasLF) || (!hasCRLF && !hasLF)) {\n            return null;\n        } else {\n            return hasCRLF ? LINE_ENDINGS_CRLF : LINE_ENDINGS_LF;\n        }\n    }\n\n    /**\n     * Translates any line ending types in the given text to the be the single form specified\n     * @param {!string} text\n     * @param {null|LINE_ENDINGS_CRLF|LINE_ENDINGS_LF} lineEndings\n     * @return {string}\n     */\n    function translateLineEndings(text, lineEndings) {\n        if (lineEndings !== LINE_ENDINGS_CRLF && lineEndings !== LINE_ENDINGS_LF) {\n            lineEndings = getPlatformLineEndings();\n        }\n\n        var eolStr = (lineEndings === LINE_ENDINGS_CRLF ? "\\r\\n" : "\\n");\n        var findAnyEol = /\\r\\n|\\r|\\n/g;\n\n        return text.replace(findAnyEol, eolStr);\n    }\n\n    function getFileErrorString(name) {\n        // There are a few error codes that we have specific error messages for. The rest are\n        // displayed with a generic "(error N)" message.\n        var result;\n\n        if (name === FileSystemError.NOT_FOUND) {\n            result = "Strings.NOT_FOUND_ERR";\n        } else if (name === FileSystemError.NOT_READABLE) {\n            result = "Strings.NOT_READABLE_ERR";\n        } else if (name === FileSystemError.NOT_WRITABLE) {\n            result = "Strings.NO_MODIFICATION_ALLOWED_ERR_FILE";\n        } else if (name === FileSystemError.CONTENTS_MODIFIED) {\n            result = "Strings.CONTENTS_MODIFIED_ERR";\n        } else {\n            result = StringUtils.format("Strings.GENERIC_ERROR", name);\n        }\n\n        return result;\n    }\n\n    function showFileOpenError(name, path) {\n        /*\n        return Dialogs.showModalDialog(\n            DefaultDialogs.DIALOG_ID_ERROR,\n            Strings.ERROR_OPENING_FILE_TITLE,\n            StringUtils.format(\n                Strings.ERROR_OPENING_FILE,\n                StringUtils.breakableUrl(path),\n                getFileErrorString(name)\n            )\n        );\n        */\n    }\n\n    /**\n     * Convert a URI path to a native path.\n     * On both platforms, this unescapes the URI\n     * On windows, URI paths start with a "/", but have a drive letter ("C:"). In this\n     * case, remove the initial "/".\n     * @param {!string} path\n     * @return {string}\n     */\n    function convertToNativePath(path) {\n        path = unescape(path);\n        if (path.indexOf(":") !== -1 && path[0] === "/") {\n            return path.substr(1);\n        }\n\n        return path;\n    }\n\n    /**\n     * Convert a Windows-native path to use Unix style slashes.\n     * On Windows, this converts "C:\\foo\\bar\\baz.txt" to "C:/foo/bar/baz.txt".\n     * On Mac, this does nothing, since Mac paths are already in Unix syntax.\n     * (Note that this does not add an initial forward-slash. Internally, our\n     * APIs generally use the "C:/foo/bar/baz.txt" style for "native" paths.)\n     * @param {string} path A native-style path.\n     * @return {string} A Unix-style path.\n     */\n    function convertWindowsPathToUnixPath(path) {\n        if (app.platform === "win") {\n            path = path.replace(/\\\\/g, "/");\n        }\n        return path;\n    }\n\n    /**\n     * Removes the trailing slash from a path, if it has one.\n     * Warning: this differs from the format of most paths! Use paths ending in "/"\n     * normally, as this is the format used by Directory.fullPath.\n     *\n     * @param {string} path\n     * @return {string}\n     */\n    function stripTrailingSlash(path) {\n        if (path && path[path.length - 1] === "/") {\n            return path.slice(0, -1);\n        } else {\n            return path;\n        }\n    }\n\n    /**\n     * Although this app is generally standardized on folder paths with a trailing "/", some APIs here\n     * receive project paths without "/" due to legacy preference storage formats, etc.\n     * @param {!string} fullPath  Path that may or may not end in "/"\n     * @return {!string} Path that ends in "/"\n     */\n    function ensureTrailingSlash(fullPath) {\n        if (fullPath[fullPath.length - 1] !== "/") {\n            return fullPath + "/";\n        }\n        return fullPath;\n    }\n\n    /**\n     * Warning: Contrary to the name, this does NOT return a canonical path. The canonical format\n     * used by Directory.fullPath actually DOES include the trailing "/"\n     * @deprecated\n     *\n     * @param {string} path\n     * @return {string}\n     */\n    function canonicalizeFolderPath(path) {\n        console.error("Warning: FileUtils.canonicalizeFolderPath() is deprecated. Use paths ending in \'/\' if possible, like Directory.fullPath");\n        return stripTrailingSlash(path);\n    }\n\n    /**\n     * Get the name of a file or a directory, removing any preceding path.\n     * @param {string} fullPath full path to a file or directory\n     * @return {string} Returns the base name of a file or the name of a\n     * directory\n     */\n    function getBaseName(fullPath) {\n        var lastSlash = fullPath.lastIndexOf("/");\n        if (lastSlash === fullPath.length - 1) {  // directory: exclude trailing "/" too\n            return fullPath.slice(fullPath.lastIndexOf("/", fullPath.length - 2) + 1, -1);\n        } else {\n            return fullPath.slice(lastSlash + 1);\n        }\n    }\n\n    /**\n     * Returns a native absolute path to the application source directory.\n     * Note that this only works when run in /src/index.html, so it does\n     * not work for unit tests (which is run from /test/SpecRunner.html)\n     *\n     * WARNING: unlike most paths, this path EXCLUDES the trailing "/".\n     * @return {string}\n     */\n    function getNativeDirectoryPath() {\n        var pathname = decodeURI(window.location.pathname);\n        var directory = pathname.substr(0, pathname.lastIndexOf("/"));\n        return convertToNativePath(directory);\n    }\n\n    /**\n     * Returns the full path to the application directory.\n     * @return {!string} fullPath reference\n     */\n    function getApplicationDirectoryPath() {\n        var pathname = getNativeDirectoryPath();\n        pathname = pathname.substr(0, pathname.lastIndexOf("/"));\n        return ensureTrailingSlash(pathname);\n    }\n\n    /**\n     * Given the module object passed to JS module define function,\n     * convert the path to a native absolute path.\n     * Returns a native absolute path to the module folder.\n     *\n     * WARNING: unlike most paths, this path EXCLUDES the trailing "/".\n     * @return {string}\n     */\n    function getNativeModuleDirectoryPath(module) {\n        var path;\n\n        if (module && module.uri) {\n            path = decodeURI(module.uri);\n\n            // Remove module name and trailing slash from path.\n            path = path.substr(0, path.lastIndexOf("/"));\n        }\n        return path;\n    }\n\n    /**\n     * Get the file extension (excluding ".") given a path OR a bare filename.\n     * Returns "" for names with no extension. If the name starts with ".", the\n     * full remaining text is considered the extension.\n     *\n     * @param {string} fullPath full path to a file or directory\n     * @return {string} Returns the extension of a filename or empty string if\n     * the argument is a directory or a filename with no extension\n     */\n    function getFileExtension(fullPath) {\n        var baseName = getBaseName(fullPath),\n            idx      = baseName.lastIndexOf(".");\n\n        if (idx === -1) {\n            return "";\n        }\n\n        return baseName.substr(idx + 1);\n    }\n\n    /**\n     * Computes filename as relative to the basePath. For example:\n     * basePath: /foo/bar/, filename: /foo/bar/baz.txt\n     * returns: baz.txt\n     *\n     * The net effect is that the common prefix is stripped away. If basePath is not\n     * a prefix of filename, then undefined is returned.\n     *\n     * @param {string} basePath Path against which we\'re computing the relative path\n     * @param {string} filename Full path to the file for which we are computing a relative path\n     * @return {string} relative path\n     */\n    function getRelativeFilename(basePath, filename) {\n        if (!filename || filename.substr(0, basePath.length) !== basePath) {\n            return;\n        }\n\n        return filename.substr(basePath.length);\n    }\n\n    /** @const - hard-coded for now, but may want to make these preferences */\n    var _staticHtmlFileExts = ["htm", "html"],\n        _serverHtmlFileExts = ["php", "php3", "php4", "php5", "phtm", "phtml", "cfm", "cfml", "asp", "aspx", "jsp", "jspx", "shtm", "shtml"];\n\n    /**\n     * Determine if file extension is a static html file extension.\n     * @param {string} filePath could be a path, a file name or just a file extension\n     * @return {boolean} Returns true if fileExt is in the list\n     */\n    function isStaticHtmlFileExt(filePath) {\n        if (!filePath) {\n            return false;\n        }\n\n        return (_staticHtmlFileExts.indexOf(getFileExtension(filePath).toLowerCase()) !== -1);\n    }\n\n    /**\n     * Determine if file extension is a server html file extension.\n     * @param {string} filePath could be a path, a file name or just a file extension\n     * @return {boolean} Returns true if fileExt is in the list\n     */\n    function isServerHtmlFileExt(filePath) {\n        if (!filePath) {\n            return false;\n        }\n\n        return (_serverHtmlFileExts.indexOf(getFileExtension(filePath).toLowerCase()) !== -1);\n    }\n\n    /**\n     * Get the parent directory of a file. If a directory is passed in the directory is returned.\n     * @param {string} fullPath full path to a file or directory\n     * @return {string} Returns the path to the parent directory of a file or the path of a directory,\n     *                  including trailing "/"\n     */\n    function getDirectoryPath(fullPath) {\n        return fullPath.substr(0, fullPath.lastIndexOf("/") + 1);\n    }\n\n    /**\n     * @private\n     * Get the file name without the extension.\n     * @param {string} filename File name of a file or directory\n     * @return {string} Returns the file name without the extension\n     */\n    function _getFilenameWithoutExtension(filename) {\n        var index = filename.lastIndexOf(".");\n        return index === -1 ? filename : filename.slice(0, index);\n    }\n\n    /**\n     * Compares 2 filenames in lowercases. In Windows it compares the names without the\n     * extension first and then the extensions to fix issue #4409\n     * @param {string} filename1\n     * @param {string} filename2\n     * @param {boolean} extFirst If true it compares the extensions first and then the file names.\n     * @return {number} The result of the local compare function\n     */\n    function compareFilenames(filename1, filename2, extFirst) {\n        var ext1   = getFileExtension(filename1),\n            ext2   = getFileExtension(filename2),\n            cmpExt = ext1.toLocaleLowerCase().localeCompare(ext2.toLocaleLowerCase(), undefined, {numeric: true}),\n            cmpNames;\n\n        if (app.platform === "win") {\n            filename1 = _getFilenameWithoutExtension(filename1);\n            filename2 = _getFilenameWithoutExtension(filename2);\n        }\n        cmpNames = filename1.toLocaleLowerCase().localeCompare(filename2.toLocaleLowerCase(), undefined, {numeric: true});\n\n        return extFirst ? (cmpExt || cmpNames) : (cmpNames || cmpExt);\n    }\n    \n    /**\n     * Converts filename to be available in Windows\n     * @param {string} filename Filename to be converted available in Windows\n     * @return {string} forbidden chars were replaced by "_"\n     */\n    function convertToWindowsFilename(filename) {\n        return filename.replace(/[\\/\\*<>:\\\\\\"|?]/g, "_");\n    }\n\n\n    // Define public API\n    exports.LINE_ENDINGS_CRLF              = LINE_ENDINGS_CRLF;\n    exports.LINE_ENDINGS_LF                = LINE_ENDINGS_LF;\n    exports.getPlatformLineEndings         = getPlatformLineEndings;\n    exports.sniffLineEndings               = sniffLineEndings;\n    exports.translateLineEndings           = translateLineEndings;\n    exports.showFileOpenError              = showFileOpenError;\n    exports.getFileErrorString             = getFileErrorString;\n    exports.readAsText                     = readAsText;\n    exports.writeText                      = writeText;\n    exports.convertToNativePath            = convertToNativePath;\n    exports.convertWindowsPathToUnixPath   = convertWindowsPathToUnixPath;\n    exports.getNativeDirectoryPath         = getNativeDirectoryPath;\n    exports.getApplicationDirectoryPath    = getApplicationDirectoryPath;\n    exports.getNativeModuleDirectoryPath   = getNativeModuleDirectoryPath;\n    exports.canonicalizeFolderPath         = canonicalizeFolderPath;\n    exports.stripTrailingSlash             = stripTrailingSlash;\n    exports.isStaticHtmlFileExt            = isStaticHtmlFileExt;\n    exports.isServerHtmlFileExt            = isServerHtmlFileExt;\n    exports.getDirectoryPath               = getDirectoryPath;\n    exports.getBaseName                    = getBaseName;\n    exports.getRelativeFilename            = getRelativeFilename;\n    exports.getFileExtension               = getFileExtension;\n    exports.compareFilenames               = compareFilenames;\n    exports.convertToWindowsFilename       = convertToWindowsFilename;\n});\n\n//# sourceURL=/file/FileUtils.js'),eval("/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define */\n\ndefine('filesystem/WatchedRoot',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n    \n    /*\n     * @constructor\n     * Represents file or directory structure watched by the FileSystem. If the\n     * entry is a directory, all children (that pass the supplied filter function)\n     * are also watched. A WatchedRoot object begins and ends its life in the\n     * INACTIVE state. While in the process of starting up watchers, the WatchedRoot\n     * is in the STARTING state. When watchers are ready, the WatchedRoot enters\n     * the ACTIVE state.\n     *\n     * See the FileSystem class for more details.\n     *\n     * @param {File|Directory} entry \n     * @param {function(string, string):boolean} filter \n     */\n    function WatchedRoot(entry, filter) {\n        this.entry = entry;\n        this.filter = filter;\n    }\n\n    // Status constants\n    WatchedRoot.INACTIVE = 0;\n    WatchedRoot.STARTING = 1;\n    WatchedRoot.ACTIVE = 2;\n\n    /**\n     * @type {File|Directory}\n     */\n    WatchedRoot.prototype.entry = null;\n    \n    /**\n     * @type {function(string, string):boolean}\n     */\n    WatchedRoot.prototype.filter = null;\n\n    /**\n     * @type {number}\n     */\n    WatchedRoot.prototype.status = WatchedRoot.INACTIVE;\n    \n    \n    // Export this class\n    module.exports = WatchedRoot;\n});\n\n//# sourceURL=/filesystem/WatchedRoot.js"),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n/*\n * To ensure cache coherence, current and future asynchronous state-changing \n * operations of FileSystemEntry and its subclasses should implement the \n * following high-level sequence of steps:\n * \n * 1. Block external filesystem change events;\n * 2. Execute the low-level state-changing operation;\n * 3. Update the internal filesystem state, including caches;\n * 4. Apply the callback;\n * 5. Fire an appropriate internal change notification; and\n * 6. Unblock external change events.\n *\n * Note that because internal filesystem state is updated first, both the original \n * caller and the change notification listeners observe filesystem state that is\n * current w.r.t. the operation. Furthermore, because external change events are\n * blocked before the operation begins, listeners will only receive the internal\n * change event for the operation and not additional (or possibly inconsistent)\n * external change events.\n * \n * State-changing operations that block external filesystem change events must\n * take care to always subsequently unblock the external change events in all\n * control paths. It is safe to assume, however, that the underlying impl will\n * always apply the callback with some value.\n \n * Caches should be conservative. Consequently, the entry\'s cached data should\n * always be cleared if the underlying impl\'s operation fails. This is the case\n * event for read-only operations because an unexpected failure implies that the\n * system is in an unknown state. The entry should communicate this by failing\n * where appropriate, and should not use the cache to hide failure.\n * \n * Only watched entries should make use of cached data because change events are\n * only expected for such entries, and change events are used to granularly\n * invalidate out-of-date caches.\n *\n * By convention, callbacks are optional for asynchronous, state-changing\n * operations, but required for read-only operations. The first argument to the\n * callback should always be a nullable error string from FileSystemError.\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define */\n\ndefine(\'filesystem/FileSystemEntry\',[\'require\',\'exports\',\'module\',\'filesystem/FileSystemError\',\'filesystem/WatchedRoot\'],function (require, exports, module) {\n    "use strict";\n    \n    var FileSystemError = require("filesystem/FileSystemError"),\n        WatchedRoot     = require("filesystem/WatchedRoot");\n    \n    var VISIT_DEFAULT_MAX_DEPTH = 100,\n        VISIT_DEFAULT_MAX_ENTRIES = 30000;\n    \n    /* Counter to give every entry a unique id */\n    var nextId = 0;\n    \n    /**\n     * @constructor\n     * Model for a file system entry. This is the base class for File and Directory,\n     * and is never used directly.\n     *\n     * See the File, Directory, and FileSystem classes for more details.\n     *\n     * @param {string} path The path for this entry.\n     * @param {FileSystem} fileSystem The file system associated with this entry.\n     */\n    function FileSystemEntry(path, fileSystem) {\n        this._setPath(path);\n        this._fileSystem = fileSystem;\n        this._id = nextId++;\n    }\n    \n    // Add "fullPath", "name", "parent", "id", "isFile" and "isDirectory" getters\n    Object.defineProperties(FileSystemEntry.prototype, {\n        "fullPath": {\n            get: function () { return this._path; },\n            set: function () { throw new Error("Cannot set fullPath"); }\n        },\n        "name": {\n            get: function () { return this._name; },\n            set: function () { throw new Error("Cannot set name"); }\n        },\n        "parentPath": {\n            get: function () { return this._parentPath; },\n            set: function () { throw new Error("Cannot set parentPath"); }\n        },\n        "id": {\n            get: function () { return this._id; },\n            set: function () { throw new Error("Cannot set id"); }\n        },\n        "isFile": {\n            get: function () { return this._isFile; },\n            set: function () { throw new Error("Cannot set isFile"); }\n        },\n        "isDirectory": {\n            get: function () { return this._isDirectory; },\n            set: function () { throw new Error("Cannot set isDirectory"); }\n        },\n        "_impl": {\n            get: function () { return this._fileSystem._impl; },\n            set: function () { throw new Error("Cannot set _impl"); }\n        }\n    });\n    \n    /**\n     * Cached stat object for this file.\n     * @type {?FileSystemStats}\n     */\n    FileSystemEntry.prototype._stat = null;\n    \n    /**\n     * Parent file system.\n     * @type {!FileSystem}\n     */\n    FileSystemEntry.prototype._fileSystem = null;\n\n    /**\n     * The path of this entry.\n     * @type {string}\n     */\n    FileSystemEntry.prototype._path = null;\n    \n    /**\n     * The name of this entry.\n     * @type {string}\n     */\n    FileSystemEntry.prototype._name = null;\n\n    /**\n     * The parent of this entry.\n     * @type {string}\n     */\n    FileSystemEntry.prototype._parentPath = null;\n    \n    /**\n     * Whether or not the entry is a file\n     * @type {boolean}\n     */\n    FileSystemEntry.prototype._isFile = false;\n    \n    /**\n     * Whether or not the entry is a directory\n     * @type {boolean}\n     */\n    FileSystemEntry.prototype._isDirectory = false;\n    \n    /**\n     * Cached copy of this entry\'s watched root\n     * @type {entry: File|Directory, filter: function(FileSystemEntry):boolean, active: boolean}\n     */\n    FileSystemEntry.prototype._watchedRoot = null;\n\n    /**\n     * Cached result of _watchedRoot.filter(this.name, this.parentPath).\n     * @type {boolean}\n     */\n    FileSystemEntry.prototype._watchedRootFilterResult = false;\n    \n    /**\n     * Determines whether or not the entry is watched.\n     * @param {boolean=} relaxed If falsey, the method will only return true if\n     *      the watched root is fully active. If true, the method will return\n     *      true if the watched root is either starting up or fully active.\n     * @return {boolean}\n     */\n    FileSystemEntry.prototype._isWatched = function (relaxed) {\n        var watchedRoot = this._watchedRoot,\n            filterResult = this._watchedRootFilterResult;\n        \n        if (!watchedRoot) {\n            watchedRoot = this._fileSystem._findWatchedRootForPath(this._path);\n            \n            if (watchedRoot) {\n                this._watchedRoot = watchedRoot;\n                filterResult = watchedRoot.filter(this._name, this._parentPath);\n                this._watchedRootFilterResult = filterResult;\n            }\n        }\n        \n        if (watchedRoot) {\n            if (watchedRoot.status === WatchedRoot.ACTIVE ||\n                    (relaxed && watchedRoot.status === WatchedRoot.STARTING)) {\n                return filterResult;\n            } else {\n                // We had a watched root, but it\'s no longer active, so it must now be invalid.\n                this._watchedRoot = undefined;\n                this._watchedRootFilterResult = false;\n                this._clearCachedData();\n            }\n        }\n        return false;\n    };\n    \n    /**\n     * Update the path for this entry\n     * @private\n     * @param {String} newPath\n     */\n    FileSystemEntry.prototype._setPath = function (newPath) {\n        var parts = newPath.split("/");\n        if (this.isDirectory) {\n            parts.pop(); // Remove the empty string after last trailing "/"\n        }\n        this._name = parts[parts.length - 1];\n        parts.pop(); // Remove name\n        \n        if (parts.length > 0) {\n            this._parentPath = parts.join("/") + "/";\n        } else {\n            // root directories have no parent path\n            this._parentPath = null;\n        }\n        \n        this._path = newPath;\n        \n        var watchedRoot = this._watchedRoot;\n        if (watchedRoot) {\n            if (newPath.indexOf(watchedRoot.entry.fullPath) === 0) {\n                // Update watchedRootFilterResult\n                this._watchedRootFilterResult = watchedRoot.filter(this._name, this._parentPath);\n            } else {\n                // The entry was moved outside of the watched root\n                this._watchedRoot = null;\n                this._watchedRootFilterResult = false;\n            }\n        }\n    };\n    \n    /**\n     * Clear any cached data for this entry\n     * @private\n     */\n    FileSystemEntry.prototype._clearCachedData = function () {\n        this._stat = undefined;\n    };\n    \n    /**\n     * Helpful toString for debugging purposes\n     */\n    FileSystemEntry.prototype.toString = function () {\n        return "[" + (this.isDirectory ? "Directory " : "File ") + this._path + "]";\n    };\n    \n    /**\n     * Check to see if the entry exists on disk. Note that there will NOT be an\n     * error returned if the file does not exist on the disk; in that case the\n     * error parameter will be null and the boolean will be false. The error \n     * parameter will only be truthy when an unexpected error was encountered\n     * during the test, in which case the state of the entry should be considered\n     * unknown.\n     *\n     * @param {function (?string, boolean)} callback Callback with a FileSystemError\n     *      string or a boolean indicating whether or not the file exists.\n     */\n    FileSystemEntry.prototype.exists = function (callback) {\n        if (this._stat) {\n            callback(null, true);\n            return;\n        }\n        \n        this._impl.exists(this._path, function (err, exists) {\n            if (err) {\n                this._clearCachedData();\n                callback(err);\n                return;\n            }\n            \n            if (!exists) {\n                this._clearCachedData();\n            }\n            \n            callback(null, exists);\n        }.bind(this));\n    };\n    \n    /**\n     * Returns the stats for the entry.\n     *\n     * @param {function (?string, FileSystemStats=)} callback Callback with a\n     *      FileSystemError string or FileSystemStats object.\n     */\n    FileSystemEntry.prototype.stat = function (callback) {\n        if (this._stat) {\n            callback(null, this._stat);\n            return;\n        }\n        \n        this._impl.stat(this._path, function (err, stat) {\n            if (err) {\n                this._clearCachedData();\n                callback(err);\n                return;\n            }\n            \n            if (this._isWatched()) {\n                this._stat = stat;\n            }\n            \n            callback(null, stat);\n        }.bind(this));\n    };\n    \n    /**\n     * Rename this entry.\n     *\n     * @param {string} newFullPath New path & name for this entry.\n     * @param {function (?string)=} callback Callback with a single FileSystemError\n     *      string parameter.\n     */\n    FileSystemEntry.prototype.rename = function (newFullPath, callback) {\n        callback = callback || function () {};\n        \n        // Block external change events until after the write has finished\n        this._fileSystem._beginChange();\n        \n        this._impl.rename(this._path, newFullPath, function (err) {\n            var oldFullPath = this._path;\n            \n            try {\n                if (err) {\n                    this._clearCachedData();\n                    callback(err);\n                    return;\n                }\n                \n                // Update internal filesystem state\n                this._fileSystem._handleRename(this._path, newFullPath, this.isDirectory);\n                \n                try {\n                    // Notify the caller\n                    callback(null);\n                } finally {\n                    // Notify rename listeners\n                    this._fileSystem._fireRenameEvent(oldFullPath, newFullPath);\n                }\n            } finally {\n                // Unblock external change events\n                this._fileSystem._endChange();\n            }\n        }.bind(this));\n    };\n        \n    /**\n     * Permanently delete this entry. For Directories, this will delete the directory\n     * and all of its contents. For reversible delete, see moveToTrash().\n     *\n     * @param {function (?string)=} callback Callback with a single FileSystemError\n     *      string parameter.\n     */\n    FileSystemEntry.prototype.unlink = function (callback) {\n        callback = callback || function () {};\n        \n        // Block external change events until after the write has finished\n        this._fileSystem._beginChange();\n        \n        this._clearCachedData();\n        this._impl.unlink(this._path, function (err) {\n            var parent = this._fileSystem.getDirectoryForPath(this.parentPath);\n\n            // Update internal filesystem state\n            this._fileSystem._handleDirectoryChange(parent, function (added, removed) {\n                try {\n                    // Notify the caller \n                    callback(err);\n                } finally {\n                    // Notify change listeners\n                    this._fileSystem._fireChangeEvent(parent, added, removed);\n                    \n                    // Unblock external change events\n                    this._fileSystem._endChange();\n                }\n            }.bind(this));\n        }.bind(this));\n    };\n    \n    /**\n     * Move this entry to the trash. If the underlying file system doesn\'t support move\n     * to trash, the item is permanently deleted.\n     *\n     * @param {function (?string)=} callback Callback with a single FileSystemError\n     *      string parameter.\n     */\n    FileSystemEntry.prototype.moveToTrash = function (callback) {\n        if (!this._impl.moveToTrash) {\n            this.unlink(callback);\n            return;\n        }\n\n        callback = callback || function () {};\n\n        // Block external change events until after the write has finished\n        this._fileSystem._beginChange();\n        \n        this._clearCachedData();\n        this._impl.moveToTrash(this._path, function (err) {\n            var parent = this._fileSystem.getDirectoryForPath(this.parentPath);\n\n            // Update internal filesystem state\n            this._fileSystem._handleDirectoryChange(parent, function (added, removed) {\n                try {\n                    // Notify the caller\n                    callback(err);\n                } finally {\n                    // Notify change listeners\n                    this._fileSystem._fireChangeEvent(parent, added, removed);\n                    \n                    // Unblock external change events\n                    this._fileSystem._endChange();\n                }\n            }.bind(this));\n        }.bind(this));\n    };\n    \n    /**\n     * Private helper function for FileSystemEntry.visit that requires sanitized options.\n     *\n     * @private\n     * @param {FileSystemStats} stats - the stats for this entry\n     * @param {{string: boolean}} visitedPaths - the set of fullPaths that have already been visited\n     * @param {function(FileSystemEntry): boolean} visitor - A visitor function, which is\n     *      applied to descendent FileSystemEntry objects. If the function returns false for\n     *      a particular Directory entry, that directory\'s descendents will not be visited.\n     * @param {{maxDepth: number, maxEntriesCounter: {value: number}}} options\n     * @param {function(?string)=} callback Callback with single FileSystemError string parameter.\n     */\n    FileSystemEntry.prototype._visitHelper = function (stats, visitedPaths, visitor, options, callback) {\n        var maxDepth = options.maxDepth,\n            maxEntriesCounter = options.maxEntriesCounter;\n        \n        if (maxEntriesCounter.value-- <= 0 || maxDepth-- < 0) {\n            // The outer FileSystemEntry.visit call is responsible for applying\n            // the main callback to FileSystemError.TOO_MANY_FILES in this case\n            callback(null);\n            return;\n        }\n        \n        if (this.isDirectory) {\n            var visitedPath = stats.realPath || this.fullPath;\n    \n            if (visitedPaths.hasOwnProperty(visitedPath)) {\n                // Link cycle detected\n                callback(null);\n                return;\n            }\n            \n            visitedPaths[visitedPath] = true;\n        }\n        \n        if (!visitor(this) || this.isFile) {\n            callback(null);\n            return;\n        }\n        \n        this.getContents(function (err, entries, entriesStats) {\n            if (err) {\n                callback(err);\n                return;\n            }\n            \n            var counter = entries.length;\n            if (counter === 0) {\n                callback(null);\n                return;\n            }\n\n            function helperCallback(err) {\n                if (--counter === 0) {\n                    callback(null);\n                }\n            }\n            \n            var nextOptions = {\n                maxDepth: maxDepth,\n                maxEntriesCounter: maxEntriesCounter\n            };\n            \n            entries.forEach(function (entry, index) {\n                var stats = entriesStats[index];\n                entry._visitHelper(stats, visitedPaths, visitor, nextOptions, helperCallback);\n            });\n        }.bind(this));\n    };\n    \n    /**\n     * Visit this entry and its descendents with the supplied visitor function.\n     * Correctly handles symbolic link cycles and options can be provided to limit\n     * search depth and total number of entries visited. No particular traversal\n     * order is guaranteed; instead of relying on such an order, it is preferable\n     * to use the visit function to build a list of visited entries, sort those\n     * entries as desired, and then process them. Whenever possible, deep\n     * filesystem traversals should use this method. \n     *\n     * @param {function(FileSystemEntry): boolean} visitor - A visitor function, which is\n     *      applied to this entry and all descendent FileSystemEntry objects. If the function returns\n     *      false for a particular Directory entry, that directory\'s descendents will not be visited.\n     * @param {{maxDepth: number=, maxEntries: number=}=} options\n     * @param {function(?string)=} callback Callback with single FileSystemError string parameter.\n     */\n    FileSystemEntry.prototype.visit = function (visitor, options, callback) {\n        if (typeof options === "function") {\n            callback = options;\n            options = {};\n        } else {\n            if (options === undefined) {\n                options = {};\n            }\n            \n            callback = callback || function () {};\n        }\n        \n        if (options.maxDepth === undefined) {\n            options.maxDepth = VISIT_DEFAULT_MAX_DEPTH;\n        }\n        \n        if (options.maxEntries === undefined) {\n            options.maxEntries = VISIT_DEFAULT_MAX_ENTRIES;\n        }\n\n        options.maxEntriesCounter = { value: options.maxEntries };\n        \n        this.stat(function (err, stats) {\n            if (err) {\n                callback(err);\n                return;\n            }\n            \n            this._visitHelper(stats, {}, visitor, options, function (err) {\n                if (callback) {\n                    if (err) {\n                        callback(err);\n                        return;\n                    }\n                    \n                    if (options.maxEntriesCounter.value < 0) {\n                        callback(FileSystemError.TOO_MANY_ENTRIES);\n                        return;\n                    }\n                    \n                    callback(null);\n                }\n            }.bind(this));\n        }.bind(this));\n    };\n    \n    // Export this class\n    module.exports = FileSystemEntry;\n});\n\n//# sourceURL=/filesystem/FileSystemEntry.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define */\n\ndefine(\'filesystem/Directory\',[\'require\',\'exports\',\'module\',\'filesystem/FileSystemEntry\'],function (require, exports, module) {\n    "use strict";\n    \n    var FileSystemEntry = require("filesystem/FileSystemEntry");\n    \n    /*\n     * @constructor\n     * Model for a file system Directory.\n     *\n     * This class should *not* be instantiated directly. Use FileSystem.getDirectoryForPath,\n     * FileSystem.resolve, or Directory.getContents to create an instance of this class.\n     *\n     * Note: Directory.fullPath always has a trailing slash.\n     *\n     * See the FileSystem class for more details.\n     *\n     * @param {!string} fullPath The full path for this Directory.\n     * @param {!FileSystem} fileSystem The file system associated with this Directory.\n     */\n    function Directory(fullPath, fileSystem) {\n        this._isDirectory = true;\n        FileSystemEntry.call(this, fullPath, fileSystem);\n    }\n    \n    Directory.prototype = Object.create(FileSystemEntry.prototype);\n    Directory.prototype.constructor = Directory;\n    Directory.prototype.parentClass = FileSystemEntry.prototype;\n    \n    /**\n     * The contents of this directory. This "private" property is used by FileSystem.\n     * @type {Array<FileSystemEntry>}\n     */\n    Directory.prototype._contents = null;\n\n    /**\n     * The stats for the contents of this directory, such that this._contentsStats[i]\n     * corresponds to this._contents[i].\n     * @type {Array.<FileSystemStats>}\n     */\n    Directory.prototype._contentsStats = null;\n    \n    /**\n     * The stats errors for the contents of this directory.\n     * @type {object.<string: string>} fullPaths are mapped to FileSystemError strings\n     */\n    Directory.prototype._contentsStatsErrors = null;\n    \n    /**\n     * Clear any cached data for this directory. By default, we clear the contents\n     * of immediate children as well, because in some cases file watchers fail \n     * provide precise change notifications. (Sometimes, like after a "git\n     * checkout", they just report that some directory has changed when in fact\n     * many of the file within the directory have changed.\n     * \n     * @private\n     * @param {boolean=} preserveImmediateChildren\n     */\n    Directory.prototype._clearCachedData = function (preserveImmediateChildren) {\n        FileSystemEntry.prototype._clearCachedData.apply(this);\n        \n        if (!preserveImmediateChildren) {\n            if (this._contents) {\n                this._contents.forEach(function (child) {\n                    child._clearCachedData(true);\n                });\n            } else {\n                // No cached _contents, but child entries may still exist.\n                // Scan the full index to catch all of them.\n                var dirPath = this.fullPath;\n                this._fileSystem._index.visitAll(function (entry) {\n                    if (entry.parentPath === dirPath) {\n                        entry._clearCachedData(true);\n                    }\n                });\n            }\n        }\n        \n        this._contents = undefined;\n        this._contentsStats = undefined;\n        this._contentsStatsErrors = undefined;\n    };\n    \n    /**\n     * Apply each callback in a list to the provided arguments. Callbacks\n     * can throw without preventing other callbacks from being applied.\n     * \n     * @private\n     * @param {Array.<function>} callbacks The callbacks to apply\n     * @param {Array} args The arguments to which each callback is applied\n     */\n    function _applyAllCallbacks(callbacks, args) {\n        if (callbacks.length > 0) {\n            var callback = callbacks.pop();\n            try {\n                callback.apply(undefined, args);\n            } finally {\n                _applyAllCallbacks(callbacks, args);\n            }\n        }\n    }\n    \n    /**\n     * Read the contents of a Directory. \n     *\n     * @param {Directory} directory Directory whose contents you want to get\n     * @param {function (?string, Array.<FileSystemEntry>=, Array.<FileSystemStats>=, Object.<string, string>=)} callback\n     *          Callback that is passed an error code or the stat-able contents\n     *          of the directory along with the stats for these entries and a\n     *          fullPath-to-FileSystemError string map of unstat-able entries\n     *          and their stat errors. If there are no stat errors then the last\n     *          parameter shall remain undefined.\n     */\n    Directory.prototype.getContents = function (callback) {\n        if (this._contentsCallbacks) {\n            // There is already a pending call for this directory\'s contents.\n            // Push the new callback onto the stack and return.\n            this._contentsCallbacks.push(callback);\n            return;\n        }\n\n        // Return cached contents if the directory is watched\n        if (this._contents) {\n            callback(null, this._contents, this._contentsStats, this._contentsStatsErrors);\n            return;\n        }\n        \n        this._contentsCallbacks = [callback];\n        \n        this._impl.readdir(this.fullPath, function (err, names, stats) {\n            var contents = [],\n                contentsStats = [],\n                contentsStatsErrors;\n            \n            if (err) {\n                this._clearCachedData();\n            } else {\n                // Use the "relaxed" parameter to _isWatched because it\'s OK to\n                // cache data even while watchers are still starting up\n                var watched = this._isWatched(true);\n                \n                names.forEach(function (name, index) {\n                    var entryPath = this.fullPath + name;\n                    \n                    if (this._fileSystem._indexFilter(entryPath, name)) {\n                        var entryStats = stats[index],\n                            entry;\n                        \n                        // Note: not all entries necessarily have associated stats.\n                        if (typeof entryStats === "string") {\n                            // entryStats is an error string\n                            if (contentsStatsErrors === undefined) {\n                                contentsStatsErrors = {};\n                            }\n                            contentsStatsErrors[entryPath] = entryStats;\n                        } else {\n                            // entryStats is a FileSystemStats object\n                            if (entryStats.isFile) {\n                                entry = this._fileSystem.getFileForPath(entryPath);\n                            } else {\n                                entry = this._fileSystem.getDirectoryForPath(entryPath);\n                            }\n                            \n                            if (watched) {\n                                entry._stat = entryStats;\n                            }\n                            \n                            contents.push(entry);\n                            contentsStats.push(entryStats);\n                        }\n                    }\n                }, this);\n\n                if (watched) {\n                    this._contents = contents;\n                    this._contentsStats = contentsStats;\n                    this._contentsStatsErrors = contentsStatsErrors;\n                }\n            }\n            \n            // Reset the callback list before we begin calling back so that\n            // synchronous reentrant calls are handled correctly.\n            var currentCallbacks = this._contentsCallbacks;\n            \n            this._contentsCallbacks = null;\n            \n            // Invoke all saved callbacks\n            var callbackArgs = [err, contents, contentsStats, contentsStatsErrors];\n            _applyAllCallbacks(currentCallbacks, callbackArgs);\n        }.bind(this));\n    };\n    \n    /**\n     * Create a directory\n     *\n     * @param {function (?string, FileSystemStats=)=} callback Callback resolved with a\n     *      FileSystemError string or the stat object for the created directory.\n     */\n    Directory.prototype.create = function (callback) {\n        callback = callback || function () {};\n        \n        // Block external change events until after the write has finished\n        this._fileSystem._beginChange();\n        \n        this._impl.mkdir(this._path, function (err, stat) {\n            if (err) {\n                this._clearCachedData();\n                try {\n                    callback(err);\n                    return;\n                } finally {\n                    // Unblock external change events\n                    this._fileSystem._endChange();\n                }\n            }\n\n            var parent = this._fileSystem.getDirectoryForPath(this.parentPath);\n            \n            // Update internal filesystem state\n            if (this._isWatched()) {\n                this._stat = stat;\n            }\n            \n            this._fileSystem._handleDirectoryChange(parent, function (added, removed) {\n                try {\n                    callback(null, stat);\n                } finally {\n                    this._fileSystem._fireChangeEvent(parent, added, removed);\n                    // Unblock external change events\n                    this._fileSystem._endChange();\n                }\n            }.bind(this));\n        }.bind(this));\n    };\n    \n    // Export this class\n    module.exports = Directory;\n});\n\n//# sourceURL=/filesystem/Directory.js'),eval("/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define */\n\ndefine('filesystem/File',['require','exports','module','filesystem/FileSystemEntry','filesystem/FileSystemError'],function (require, exports, module) {\n    \"use strict\";\n    \n    var FileSystemEntry = require(\"filesystem/FileSystemEntry\"),\n        FileSystemError = require(\"filesystem/FileSystemError\");\n    \n    \n    /*\n     * @constructor\n     * Model for a File.\n     *\n     * This class should *not* be instantiated directly. Use FileSystem.getFileForPath,\n     * FileSystem.resolve, or Directory.getContents to create an instance of this class.\n     *\n     * See the FileSystem class for more details.\n     *\n     * @param {!string} fullPath The full path for this File.\n     * @param {!FileSystem} fileSystem The file system associated with this File.\n     */\n    function File(fullPath, fileSystem) {\n        this._isFile = true;\n        FileSystemEntry.call(this, fullPath, fileSystem);\n    }\n    \n    File.prototype = Object.create(FileSystemEntry.prototype);\n    File.prototype.constructor = File;\n    File.prototype.parentClass = FileSystemEntry.prototype;\n    \n    /**\n     * Cached contents of this file. This value is nullable but should NOT be undefined.\n     * @private\n     * @type {?string}\n     */\n    File.prototype._contents = null;\n    \n    /**\n     * Consistency hash for this file. Reads and writes update this value, and\n     * writes confirm the hash before overwriting existing files. The type of\n     * this object is dependent on the FileSystemImpl; the only constraint is\n     * that === can be used as an equality relation on hashes.\n     * @private\n     * @type {?object}\n     */\n    File.prototype._hash = null;\n    \n    /**\n     * Clear any cached data for this file. Note that this explicitly does NOT\n     * clear the file's hash.\n     * @private\n     */\n    File.prototype._clearCachedData = function () {\n        FileSystemEntry.prototype._clearCachedData.apply(this);\n        this._contents = null;\n    };\n    \n    /**\n     * Read a file.\n     *\n     * @param {Object=} options Currently unused.\n     * @param {function (?string, string=, FileSystemStats=)} callback Callback that is passed the\n     *              FileSystemError string or the file's contents and its stats.\n     */\n    File.prototype.read = function (options, callback) {\n        if (typeof (options) === \"function\") {\n            callback = options;\n            options = {};\n        }\n        \n        // We don't need to check isWatched() here because contents are only saved\n        // for watched files. Note that we need to explicitly test this._contents\n        // for a default value; otherwise it could be the empty string, which is\n        // falsey.\n        if (this._contents !== null && this._stat) {\n            callback(null, this._contents, this._stat);\n            return;\n        }\n        \n        var watched = this._isWatched();\n        if (watched) {\n            options.stat = this._stat;\n        }\n        \n        this._impl.readFile(this._path, options, function (err, data, stat) {\n            if (err) {\n                this._clearCachedData();\n                callback(err);\n                return;\n            }\n            \n            // Always store the hash\n            this._hash = stat._hash;\n            \n            // Only cache data for watched files\n            if (watched) {\n                this._stat = stat;\n                this._contents = data;\n            }\n            \n            callback(err, data, stat);\n        }.bind(this));\n    };\n    \n    /**\n     * Write a file.\n     *\n     * @param {string} data Data to write.\n     * @param {object=} options Currently unused.\n     * @param {function (?string, FileSystemStats=)=} callback Callback that is passed the\n     *              FileSystemError string or the file's new stats.\n     */\n    File.prototype.write = function (data, options, callback) {\n        if (typeof options === \"function\") {\n            callback = options;\n            options = {};\n        } else {\n            if (options === undefined) {\n                options = {};\n            }\n            \n            callback = callback || function () {};\n        }\n        \n        // Request a consistency check if the write is not blind\n        if (!options.blind) {\n            options.expectedHash = this._hash; \n            options.expectedContents = this._contents;\n        }\n        \n        // Block external change events until after the write has finished\n        this._fileSystem._beginChange();\n        \n        this._impl.writeFile(this._path, data, options, function (err, stat, created) {\n            if (err) {\n                this._clearCachedData();\n                try {\n                    callback(err);\n                    return;\n                } finally {\n                    // Always unblock external change events\n                    this._fileSystem._endChange();\n                }\n            }\n            \n            // Always store the hash\n            this._hash = stat._hash;\n            \n            // Only cache data for watched files\n            if (this._isWatched()) {\n                this._stat = stat;\n                this._contents = data;\n            }\n            \n            if (created) {\n                var parent = this._fileSystem.getDirectoryForPath(this.parentPath);\n                this._fileSystem._handleDirectoryChange(parent, function (added, removed) {\n                    try {\n                        // Notify the caller\n                        callback(null, stat);\n                    } finally {\n                        // If the write succeeded, fire a synthetic change event\n                        this._fileSystem._fireChangeEvent(parent, added, removed);\n                        \n                        // Always unblock external change events\n                        this._fileSystem._endChange();\n                    }\n                }.bind(this));\n            } else {\n                try {\n                    // Notify the caller\n                    callback(null, stat);\n                } finally {\n                    // existing file modified\n                    this._fileSystem._fireChangeEvent(this);\n                    \n                    // Always unblock external change events\n                    this._fileSystem._endChange();\n                }\n            }\n        }.bind(this));\n    };\n    \n    // Export this class\n    module.exports = File;\n});\n\n//# sourceURL=/filesystem/File.js"),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define */\n\n/**\n * FileIndex is an internal module used by FileSystem to maintain an index of all files and directories.\n *\n * This module is *only* used by FileSystem, and should not be called directly.\n */\ndefine(\'filesystem/FileIndex\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n    \n    /**\n     * @constructor\n     */\n    function FileIndex() {\n        this._index = {};\n    }\n    \n    /**\n     * Master index\n     * \n     * @type {Object.<string, File|Directory>} Maps a fullPath to a File or Directory object\n     */\n    FileIndex.prototype._index = null;\n    \n    /**\n     * Clear the file index cache.\n     */\n    FileIndex.prototype.clear = function () {\n        this._index = {};\n    };\n    \n    /**\n     * Visits every entry in the entire index; no stopping condition.\n     * @param {!function(FileSystemEntry, string):void} Called with an entry and its fullPath\n     */\n    FileIndex.prototype.visitAll = function (visitor) {\n        var path;\n        for (path in this._index) {\n            if (this._index.hasOwnProperty(path)) {\n                visitor(this._index[path], path);\n            }\n        }\n    };\n    \n    /**\n     * Add an entry.\n     *\n     * @param {FileSystemEntry} entry The entry to add.\n     */\n    FileIndex.prototype.addEntry = function (entry) {\n        this._index[entry.fullPath] = entry;\n    };\n    \n    /**\n     * Remove an entry.\n     * \n     * @param {FileSystemEntry} entry The entry to remove.\n     */\n    FileIndex.prototype.removeEntry = function (entry) {\n        var path = entry.fullPath,\n            property,\n            member;\n        \n        function replaceMember(property) {\n            var member = entry[property];\n            if (typeof member === "function") {\n                entry[property] = function () {\n                    console.warn("FileSystemEntry used after being removed from index: ", path);\n                    return member.apply(entry, arguments);\n                };\n            }\n        }\n        \n        delete this._index[path];\n        \n        for (property in entry) {\n            if (entry.hasOwnProperty(property)) {\n                replaceMember(property);\n            }\n        }\n    };\n    \n    /**\n     * Notify the index that an entry has been renamed. This updates\n     * all affected entries in the index.\n     *\n     * @param {string} oldPath\n     * @param {string} newPath\n     * @param {boolean} isDirectory\n     */\n    FileIndex.prototype.entryRenamed = function (oldPath, newPath, isDirectory) {\n        var path,\n            splitName = oldPath.split("/"),\n            finalPart = splitName.length - 1,\n            renameMap = {};\n        \n        // Find all entries affected by the rename and put into a separate map.\n        for (path in this._index) {\n            if (this._index.hasOwnProperty(path)) {\n                // See if we have a match. For directories, see if the path\n                // starts with the old name. This is safe since paths always end\n                // with \'/\'. For files, see if there is an exact match between\n                // the path and the old name.\n                if (isDirectory ? path.indexOf(oldPath) === 0 : path === oldPath) {\n                    renameMap[path] = newPath + path.substr(oldPath.length);\n                }\n            }\n        }\n        \n        // Do the rename. \n        for (path in renameMap) {\n            if (renameMap.hasOwnProperty(path)) {\n                var item = this._index[path];\n                \n                // Sanity check to make sure the item and path still match\n                console.assert(item.fullPath === path);\n                \n                delete this._index[path];\n                this._index[renameMap[path]] = item;\n                item._setPath(renameMap[path]);\n            }\n        }\n    };\n    \n    /**\n     * Returns the cached entry for the specified path, or undefined\n     * if the path has not been cached.\n     * \n     * @param {string} path The path of the entry to return.\n     * @return {File|Directory} The entry for the path, or undefined if it hasn\'t \n     *              been cached yet.\n     */\n    FileIndex.prototype.getEntry = function (path) {\n        return this._index[path];\n    };\n    \n    // Export public API\n    module.exports = FileIndex;\n});\n\n//# sourceURL=/filesystem/FileIndex.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define */\n\n/**\n * The FileSystemStats represents a particular FileSystemEntry\'s stats.\n */\ndefine(\'filesystem/FileSystemStats\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n    /**\n     * @constructor\n     * @param {{isFile: boolean, mtime: Date, size: Number, realPath: ?string, hash: object}} options\n     */\n    function FileSystemStats(options) {\n        var isFile = options.isFile;\n        \n        this._isFile = isFile;\n        this._isDirectory = !isFile;\n        this._mtime = options.mtime;\n        this._size = options.size;\n        this._hash = options.hash;\n        \n        var realPath = options.realPath;\n        if (realPath) {\n            if (!isFile && realPath[realPath.length - 1] !== "/") {\n                realPath += "/";\n            }\n        \n            this._realPath = realPath;\n        }\n    }\n    \n    // Add "isFile", "isDirectory", "mtime" and "size" getters\n    Object.defineProperties(FileSystemStats.prototype, {\n        "isFile": {\n            get: function () { return this._isFile; },\n            set: function () { throw new Error("Cannot set isFile"); }\n        },\n        "isDirectory": {\n            get: function () { return this._isDirectory; },\n            set: function () { throw new Error("Cannot set isDirectory"); }\n        },\n        "mtime": {\n            get: function () { return this._mtime; },\n            set: function () { throw new Error("Cannot set mtime"); }\n        },\n        "size": {\n            get: function () { return this._size; },\n            set: function () { throw new Error("Cannot set size"); }\n        },\n        "realPath": {\n            get: function () { return this._realPath; },\n            set: function () { throw new Error("Cannot set realPath"); }\n        }\n    });\n    \n    /**\n     * Whether or not this is a stats object for a file\n     * @type {boolean}\n     */\n    FileSystemStats.prototype._isFile = false;\n\n    /**\n     * Whether or not this is a stats object for a directory\n     * @type {boolean}\n     */\n    FileSystemStats.prototype._isDirectory = false;\n    \n    /**\n     * Modification time for a file\n     * @type {Date}\n     */\n    FileSystemStats.prototype._mtime = null;\n\n    /**\n     * Size in bytes of a file\n     * @type {Number}\n     */\n    FileSystemStats.prototype._size = null;\n\n    /**\n     * Consistency hash for a file\n     * @type {object}\n     */\n    FileSystemStats.prototype._hash = null;\n    \n    /**\n     * The canonical path of this file or directory ONLY if it is a symbolic link,\n     * and null otherwise.\n     * \n     * @type {?string}\n     */\n    FileSystemStats.prototype._realPath = null;\n\n    module.exports = FileSystemStats;\n});\n\n//# sourceURL=/filesystem/FileSystemStats.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4,\nmaxerr: 50, browser: true */\n/*global $, define, brackets, WebSocket, ArrayBuffer, Uint32Array */\n\ndefine(\'utils/NodeConnection\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n    /** @define{number} Connection attempts to make before failing */\n    var CONNECTION_ATTEMPTS = 10;\n\n    /**\n     * @define{number} Milliseconds to wait before a particular connection\n     *     attempt is considered failed.\n     *\n     * NOTE: It\'s okay for the connection timeout to be long because the\n     * expected behavior of WebSockets is to send a "close" event as soon\n     * as they realize they can\'t connect. So, we should rarely hit the\n     * connection timeout even if we try to connect to a port that isn\'t open.\n     */\n    var CONNECTION_TIMEOUT  = 10000; // 10 seconds\n\n    /** @define{number} Milliseconds to wait before retrying connecting */\n    var RETRY_DELAY         = 500;   // 1/2 second\n\n    /** @define {number} Maximum value of the command ID counter */\n    var MAX_COUNTER_VALUE = 4294967295; // 2^32 - 1\n\n    /**\n     * @private\n     * Helper function to auto-reject a deferred after a given amount of time.\n     * If the deferred is resolved/rejected manually, then the timeout is\n     * automatically cleared.\n     */\n    function setDeferredTimeout(deferred, delay) {\n        var timer = setTimeout(function () {\n            deferred.reject("timeout");\n        }, delay);\n        deferred.always(function () { clearTimeout(timer); });\n    }\n\n    /**\n     * @private\n     * Helper function to attempt a single connection to the node server\n     */\n    function attemptSingleConnect() {\n        var deferred = $.Deferred();\n        var port = null;\n        var ws = null;\n        setDeferredTimeout(deferred, CONNECTION_TIMEOUT);\n\n        appshell.app.getNodeState(function (err, nodePort) {\n            if (!err && nodePort && deferred.state() !== "rejected") {\n                port = nodePort;\n                ws = new WebSocket("ws://localhost:" + port);\n\n                // Expect ArrayBuffer objects from Node when receiving binary\n                // data instead of DOM Blobs, which are the default.\n                ws.binaryType = "arraybuffer";\n\n                // If the server port isn\'t open, we get a close event\n                // at some point in the future (and will not get an onopen\n                // event)\n                ws.onclose = function () {\n                    deferred.reject("WebSocket closed");\n                };\n\n                ws.onopen = function () {\n                    // If we successfully opened, remove the old onclose\n                    // handler (which was present to detect failure to\n                    // connect at all).\n                    ws.onclose = null;\n                    deferred.resolveWith(null, [ws, port]);\n                };\n            } else {\n                deferred.reject("appshell.app.getNodeState error: " + err);\n            }\n        });\n\n        return deferred.promise();\n    }\n\n    /**\n     * @constructor\n     * Provides an interface for interacting with the node server.\n     */\n    function NodeConnection() {\n        this.domains = {};\n        this._registeredModules = [];\n        this._pendingInterfaceRefreshDeferreds = [];\n        this._pendingCommandDeferreds = [];\n    }\n\n    /**\n     * @type{Object}\n     * Exposes the domains registered with the server. This object will\n     * have a property for each registered domain. Each of those properties\n     * will be an object containing properties for all the commands in that\n     * domain. So, myConnection.base.enableDebugger would point to the function\n     * to call to enable the debugger.\n     *\n     * This object is automatically replaced every time the API changes (based\n     * on the base:newDomains event from the server). Therefore, code that\n     * uses this object should not keep their own pointer to the domain property.\n     */\n    NodeConnection.prototype.domains = null;\n\n    /**\n     * @private\n     * @type{Array.<string>}\n     * List of module pathnames that should be re-registered if there is\n     * a disconnection/connection (i.e. if the server died).\n     */\n    NodeConnection.prototype._registeredModules = null;\n\n    /**\n     * @private\n     * @type{WebSocket}\n     * The connection to the server\n     */\n    NodeConnection.prototype._ws = null;\n\n    /**\n     * @private\n     * @type{?number}\n     * The port the WebSocket is currently connected to\n     */\n    NodeConnection.prototype._port = null;\n\n    /**\n     * @private\n     * @type{number}\n     * Unique ID for commands\n     */\n    NodeConnection.prototype._commandCount = 1;\n\n    /**\n     * @private\n     * @type{boolean}\n     * Whether to attempt reconnection if connection fails\n     */\n    NodeConnection.prototype._autoReconnect = false;\n\n    /**\n     * @private\n     * @type{Array.<jQuery.Deferred>}\n     * List of deferred objects that should be resolved pending\n     * a successful refresh of the API\n     */\n    NodeConnection.prototype._pendingInterfaceRefreshDeferreds = null;\n\n    /**\n     * @private\n     * @type{Array.<jQuery.Deferred>}\n     * Array (indexed on command ID) of deferred objects that should be\n     * resolved/rejected with the response of commands.\n     */\n    NodeConnection.prototype._pendingCommandDeferreds = null;\n\n    /**\n     * @private\n     * @return {number} The next command ID to use. Always representable as an\n     * unsigned 32-bit integer.\n     */\n    NodeConnection.prototype._getNextCommandID = function () {\n        var nextID;\n\n        if (this._commandCount > MAX_COUNTER_VALUE) {\n            nextID = this._commandCount = 0;\n        } else {\n            nextID = this._commandCount++;\n        }\n\n        return nextID;\n    };\n\n    /**\n     * @private\n     * Helper function to do cleanup work when a connection fails\n     */\n    NodeConnection.prototype._cleanup = function () {\n        // clear out the domains, since we may get different ones\n        // on the next connection\n        this.domains = {};\n\n        // shut down the old connection if there is one\n        if (this._ws && this._ws.readyState !== WebSocket.CLOSED) {\n            try {\n                this._ws.close();\n            } catch (e) { }\n        }\n        var failedDeferreds = this._pendingInterfaceRefreshDeferreds\n            .concat(this._pendingCommandDeferreds);\n        failedDeferreds.forEach(function (d) {\n            d.reject("cleanup");\n        });\n        this._pendingInterfaceRefreshDeferreds = [];\n        this._pendingCommandDeferreds = [];\n\n        this._ws = null;\n        this._port = null;\n    };\n\n    /**\n     * Connect to the node server. After connecting, the NodeConnection\n     * object will trigger a "close" event when the underlying socket\n     * is closed. If the connection is set to autoReconnect, then the\n     * event will also include a jQuery promise for the connection.\n     *\n     * @param {boolean} autoReconnect Whether to automatically try to\n     *    reconnect to the server if the connection succeeds and then\n     *    later disconnects. Note if this connection fails initially, the\n     *    autoReconnect flag is set to false. Future calls to connect()\n     *    can reset it to true\n     * @return {jQuery.Promise} Promise that resolves/rejects when the\n     *    connection succeeds/fails\n     */\n    NodeConnection.prototype.connect = function (autoReconnect) {\n        var self = this;\n        self._autoReconnect = autoReconnect;\n        var deferred = $.Deferred();\n        var attemptCount = 0;\n        var attemptTimestamp = null;\n\n        // Called after a successful connection to do final setup steps\n        function registerHandlersAndDomains(ws, port) {\n            // Called if we succeed at the final setup\n            function success() {\n                self._ws.onclose = function () {\n                    if (self._autoReconnect) {\n                        var $promise = self.connect(true);\n                        $(self).triggerHandler("close", [$promise]);\n                    } else {\n                        self._cleanup();\n                        $(self).triggerHandler("close");\n                    }\n                };\n                deferred.resolve();\n            }\n            // Called if we fail at the final setup\n            function fail(err) {\n                self._cleanup();\n                deferred.reject(err);\n            }\n\n            self._ws = ws;\n            self._port = port;\n            self._ws.onmessage = self._receive.bind(self);\n\n            // refresh the current domains, then re-register any\n            // "autoregister" modules\n            self._refreshInterface().then(\n                function () {\n                    if (self._registeredModules.length > 0) {\n                        self.loadDomains(self._registeredModules, false).then(\n                            success,\n                            fail\n                        );\n                    } else {\n                        success();\n                    }\n                },\n                fail\n            );\n        }\n\n        // Repeatedly tries to connect until we succeed or until we\'ve\n        // failed CONNECTION_ATTEMPT times. After each attempt, waits\n        // at least RETRY_DELAY before trying again.\n        function doConnect() {\n            attemptCount++;\n            attemptTimestamp = new Date();\n            attemptSingleConnect().then(\n                registerHandlersAndDomains, // succeded\n                function () { // failed this attempt, possibly try again\n                    if (attemptCount < CONNECTION_ATTEMPTS) { //try again\n                        // Calculate how long we should wait before trying again\n                        var now = new Date();\n                        var delay = Math.max(\n                            RETRY_DELAY - (now - attemptTimestamp),\n                            1\n                        );\n                        setTimeout(doConnect, delay);\n                    } else { // too many attempts, give up\n                        deferred.reject("Max connection attempts reached");\n                    }\n                }\n            );\n        }\n\n        // Start the connection process\n        self._cleanup();\n        doConnect();\n\n        return deferred.promise();\n    };\n\n    /**\n     * Determines whether the NodeConnection is currently connected\n     * @return{boolean} Whether the NodeConnection is connected.\n     */\n    NodeConnection.prototype.connected = function () {\n        return !!(this._ws && this._ws.readyState === WebSocket.OPEN);\n    };\n\n    /**\n     * Explicitly disconnects from the server. Note that even if\n     * autoReconnect was set to true at connection time, the connection\n     * will not reconnect after this call. Reconnection can be manually done\n     * by calling connect() again.\n     */\n    NodeConnection.prototype.disconnect = function () {\n        this._autoReconnect = false;\n        this._cleanup();\n    };\n\n    /**\n     * Load domains into the server by path\n     * @param {Array.<string>} List of absolute paths to load\n     * @param {boolean} autoReload Whether to auto-reload the domains if the server\n     *    fails and restarts. Note that the reload is initiated by the\n     *    client, so it will only happen after the client reconnects.\n     * @return {jQuery.Promise} Promise that resolves after the load has\n     *    succeeded and the new API is availale at NodeConnection.domains,\n     *    or that rejects on failure.\n     */\n    NodeConnection.prototype.loadDomains = function (paths, autoReload) {\n        var deferred = $.Deferred();\n        setDeferredTimeout(deferred, CONNECTION_TIMEOUT);\n        var pathArray = paths;\n        if (!Array.isArray(paths)) {\n            pathArray = [paths];\n        }\n\n        if (autoReload) {\n            Array.prototype.push.apply(this._registeredModules, pathArray);\n        }\n\n        if (this.domains.base && this.domains.base.loadDomainModulesFromPaths) {\n            this.domains.base.loadDomainModulesFromPaths(pathArray).then(\n                function (success) { // command call succeeded\n                    if (!success) {\n                        // response from commmand call was "false" so we know\n                        // the actual load failed.\n                        deferred.reject("loadDomainModulesFromPaths failed");\n                    }\n                    // if the load succeeded, we wait for the API refresh to\n                    // resolve the deferred.\n                },\n                function (reason) { // command call failed\n                    deferred.reject("Unable to load one of the modules: " + pathArray + (reason ? ", reason: " + reason : ""));\n                }\n            );\n\n            this._pendingInterfaceRefreshDeferreds.push(deferred);\n        } else {\n            deferred.reject("this.domains.base is undefined");\n        }\n\n        return deferred.promise();\n    };\n\n    /**\n     * @private\n     * Sends a message over the WebSocket. Automatically JSON.stringifys\n     * the message if necessary.\n     * @param {Object|string} m Object to send. Must be JSON.stringify-able.\n     */\n    NodeConnection.prototype._send = function (m) {\n        if (this.connected()) {\n\n            // Convert the message to a string\n            var messageString = null;\n            if (typeof m === "string") {\n                messageString = m;\n            } else {\n                try {\n                    messageString = JSON.stringify(m);\n                } catch (stringifyError) {\n                    console.error("[NodeConnection] Unable to stringify message in order to send: " + stringifyError.message);\n                }\n            }\n\n            // If we succeded in making a string, try to send it\n            if (messageString) {\n                try {\n                    this._ws.send(messageString);\n                } catch (sendError) {\n                    console.error("[NodeConnection] Error sending message: " + sendError.message);\n                }\n            }\n        } else {\n            console.error("[NodeConnection] Not connected to node, unable to send.");\n        }\n    };\n\n    /**\n     * @private\n     * Handler for receiving events on the WebSocket. Parses the message\n     * and dispatches it appropriately.\n     * @param {WebSocket.Message} message Message object from WebSocket\n     */\n    NodeConnection.prototype._receive = function (message) {\n        var responseDeferred = null;\n        var data = message.data;\n        var m;\n\n        if (message.data instanceof ArrayBuffer) {\n            // The first four bytes encode the command ID as an unsigned 32-bit integer\n            if (data.byteLength < 4) {\n                console.error("[NodeConnection] received malformed binary message");\n                return;\n            }\n\n            var header = data.slice(0, 4),\n                body = data.slice(4),\n                headerView = new Uint32Array(header),\n                id = headerView[0];\n\n            // Unpack the binary message into a commandResponse\n            m = {\n                type: "commandResponse",\n                message: {\n                    id: id,\n                    response: body\n                }\n            };\n        } else {\n            try {\n                m = JSON.parse(data);\n            } catch (e) {\n                console.error("[NodeConnection] received malformed message", message, e.message);\n                return;\n            }\n        }\n\n        switch (m.type) {\n        case "event":\n            var $this = $(this);\n\n            if (m.message.domain === "base" && m.message.event === "newDomains") {\n                this._refreshInterface();\n            }\n\n            // Event type for backwards compatibility for original design: "domain.event"\n            $this.triggerHandler(m.message.domain + "." + m.message.event,\n                                   m.message.parameters);\n\n            // Event type "domain:event"\n            $this.triggerHandler(m.message.domain + ":" + m.message.event,\n                                   m.message.parameters);\n            break;\n        case "commandResponse":\n            responseDeferred = this._pendingCommandDeferreds[m.message.id];\n            if (responseDeferred) {\n                responseDeferred.resolveWith(this, [m.message.response]);\n                delete this._pendingCommandDeferreds[m.message.id];\n            }\n            break;\n        case "commandError":\n            responseDeferred = this._pendingCommandDeferreds[m.message.id];\n            if (responseDeferred) {\n                responseDeferred.rejectWith(\n                    this,\n                    [m.message.message, m.message.stack]\n                );\n                delete this._pendingCommandDeferreds[m.message.id];\n            }\n            break;\n        case "error":\n            console.error("[NodeConnection] received error: " +\n                            m.message.message);\n            break;\n        default:\n            console.error("[NodeConnection] unknown event type: " + m.type);\n        }\n    };\n\n    /**\n     * @private\n     * Helper function for refreshing the interface in the "domain" property.\n     * Automatically called when the connection receives a base:newDomains\n     * event from the server, and also called at connection time.\n     */\n    NodeConnection.prototype._refreshInterface = function () {\n        var deferred = $.Deferred();\n        var self = this;\n\n        var pendingDeferreds = this._pendingInterfaceRefreshDeferreds;\n        this._pendingInterfaceRefreshDeferreds = [];\n        deferred.then(\n            function () {\n                pendingDeferreds.forEach(function (d) { d.resolve(); });\n            },\n            function (err) {\n                pendingDeferreds.forEach(function (d) { d.reject(err); });\n            }\n        );\n\n        function refreshInterfaceCallback(spec) {\n            function makeCommandFunction(domainName, commandSpec) {\n                return function () {\n                    var deferred = $.Deferred();\n                    var parameters = Array.prototype.slice.call(arguments, 0);\n                    var id = self._getNextCommandID();\n                    self._pendingCommandDeferreds[id] = deferred;\n                    self._send({id: id,\n                               domain: domainName,\n                               command: commandSpec.name,\n                               parameters: parameters\n                               });\n                    return deferred;\n                };\n            }\n\n            // TODO: Don\'t replace the domain object every time. Instead, merge.\n            self.domains = {};\n            self.domainEvents = {};\n            spec.forEach(function (domainSpec) {\n                self.domains[domainSpec.domain] = {};\n                domainSpec.commands.forEach(function (commandSpec) {\n                    self.domains[domainSpec.domain][commandSpec.name] =\n                        makeCommandFunction(domainSpec.domain, commandSpec);\n                });\n                self.domainEvents[domainSpec.domain] = {};\n                domainSpec.events.forEach(function (eventSpec) {\n                    var parameters = eventSpec.parameters;\n                    self.domainEvents[domainSpec.domain][eventSpec.name] = parameters;\n                });\n            });\n            deferred.resolve();\n        }\n\n        if (this.connected()) {\n            $.getJSON("http://localhost:" + this._port + "/api")\n                .done(refreshInterfaceCallback)\n                .fail(function (err) { deferred.reject(err); });\n        } else {\n            deferred.reject("Attempted to call _refreshInterface when not connected.");\n        }\n\n        return deferred.promise();\n    };\n\n    /**\n     * @private\n     * Get the default timeout value\n     * @return {number} Timeout value in milliseconds\n     */\n    NodeConnection._getConnectionTimeout = function () {\n        return CONNECTION_TIMEOUT;\n    };\n\n    module.exports = NodeConnection;\n\n});\n\n//# sourceURL=/utils/NodeConnection.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define, $ */\n\ndefine(\'utils/NodeDomain\',[\'require\',\'exports\',\'module\',\'utils/NodeConnection\'],function (require, exports, module) {\n    "use strict";\n    \n    var NodeConnection = require("utils/NodeConnection");\n    \n    // Used to remove all listeners at once when the connection drops\n    var EVENT_NAMESPACE = ".NodeDomainEvent";\n    \n    /**\n     * Provides a simple abstraction for executing the commands of a single\n     * domain loaded via a NodeConnection. Automatically handles connection\n     * management and domain loading, and exposes each command in the domain as\n     * a promise-returning method that can safely be called regardless of the\n     * current status of the underlying connection. Example usage:\n     * \n     * var myDomain = new NodeDomain("someDomain", "/path/to/SomeDomainDef.js"),\n     *     $result = myDomain.exec("someCommand", arg1, arg2);\n     * \n     * $result.done(function (value) {\n     *     // the command succeeded!\n     * });\n     * \n     * $result.fail(function (err) {\n     *     // the command failed; act accordingly!\n     * });\n     * \n     * To handle domain events, just listen for the event on the domain:\n     * \n     * $(myDomain).on("someEvent", someHandler);\n     * \n     * @constructor\n     * @param {string} domainName Name of the registered Node Domain\n     * @param {string} domainPath Full path of the JavaScript Node domain specification\n     */\n    function NodeDomain(domainName, domainPath) {\n        var connection = new NodeConnection();\n        \n        this.connection = connection;\n        this._domainName = domainName;\n        this._domainPath = domainPath;\n        this._domainLoaded = false;\n        this._load = this._load.bind(this);\n        this._connectionPromise = connection.connect(true)\n            .then(this._load);\n        \n        $(connection).on("close", function (event, promise) {\n            $(this.connection).off(EVENT_NAMESPACE);\n            this._domainLoaded = false;\n            this._connectionPromise = promise.then(this._load);\n        }.bind(this));\n    }\n    \n    /** \n     * The underlying Node connection object for this domain.\n     * \n     * @type {!NodeConnection} \n     */\n    NodeDomain.prototype.connection = null;\n\n    /**\n     * A promise that is resolved once the NodeConnection is connected and the\n     * domain has been loaded.\n     * \n     * @type {?jQuery.Promise}\n     * @private\n     */\n    NodeDomain.prototype._connectionPromise = null;\n    \n    /**\n     * The name of this domain.\n     * \n     * @type {string}\n     * @private\n     */\n    NodeDomain.prototype._domainName = null;\n    \n    /**\n     * The path at which the Node definition of this domain resides.\n     * \n     * @type {string}\n     * @private\n     */\n    NodeDomain.prototype._domainPath = null;\n    \n    /**\n     * Whether or not the domain has been successfully loaded.\n     * \n     * @type {boolean}\n     * @private\n     */\n    NodeDomain.prototype._domainLoaded = false;\n    \n    /**\n     * Loads the domain via the underlying connection object and exposes the\n     * domain\'s commands as methods on this object. Assumes the underlying\n     * connection has already been opened.\n     * \n     * @return {jQuery.Promise} Resolves once the domain is been loaded.\n     * @private\n     */\n    NodeDomain.prototype._load = function () {\n        var connection = this.connection;\n        return connection.loadDomains(this._domainPath, true)\n            .done(function () {\n                this._domainLoaded = true;\n                this._connectionPromise = null;\n                \n                var eventNames = Object.keys(connection.domainEvents[this._domainName]);\n                eventNames.forEach(function (domainEvent) {\n                    var connectionEvent = this._domainName + ":" + domainEvent + EVENT_NAMESPACE;\n                    \n                    $(connection).on(connectionEvent, function () {\n                        var params = Array.prototype.slice.call(arguments, 1);\n                        $(this).triggerHandler(domainEvent, params);\n                    }.bind(this));\n                }, this);\n            }.bind(this))\n            .fail(function (err) {\n                console.error("[NodeDomain] Error loading domain \\"" + this._domainName + "\\": " + err);\n            }.bind(this));\n    };\n    \n    /**\n     * Synchronously determine whether the domain is ready; i.e., whether the\n     * connection is open and the domain is loaded.\n     * \n     * @return {boolean} Whether or not the domain is currently ready.\n     */\n    NodeDomain.prototype.ready = function () {\n        return this._domainLoaded && this.connection.connected();\n    };\n    \n    /**\n     * Get a promise that resolves when the connection is open and the domain\n     * is loaded.\n     *\n     * @return {jQuery.Promise}\n     */\n    NodeDomain.prototype.promise = function () {\n        if (this._connectionPromise) {\n            return this._connectionPromise;\n        } else {\n            var deferred = new $.Deferred();\n            \n            if (this.ready()) {\n                deferred.resolve();\n            } else {\n                deferred.reject();\n            }\n            \n            return deferred.promise();\n        }\n    };\n    \n    /**\n     * Applies the named command from the domain to a list of parameters, which\n     * are passed as extra arguments to this method. If the connection is open\n     * and the domain is loaded, the function is applied immediately. Otherwise\n     * the function is applied as soon as the connection has been opened and the\n     * domain has finished loading.\n     * \n     * @param {string} name The name of the domain command to execute\n     * @return {jQuery.Promise} Resolves with the result of the command\n     */\n    NodeDomain.prototype.exec = function (name) {\n        var connection = this.connection,\n            params = Array.prototype.slice.call(arguments, 1),\n            execConnected = function () {\n                var domain  = connection.domains[this._domainName],\n                    fn      = domain && domain[name],\n                    execResult;\n        \n                if (fn) {\n                    execResult = fn.apply(domain, params);\n                } else {\n                    execResult = new $.Deferred().reject().promise();\n                }\n                return execResult;\n            }.bind(this);\n        \n        var result;\n        if (this.ready()) {\n            result = execConnected();\n        } else if (this._connectionPromise) {\n            result = this._connectionPromise.then(execConnected);\n        } else {\n            result = new $.Deferred.reject().promise();\n        }\n        return result;\n    };\n        \n    module.exports = NodeDomain;\n});\n\n//# sourceURL=/utils/NodeDomain.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define, appshell, $, window */\n\ndefine(\'fileSystemImpl\',[\'require\',\'exports\',\'module\',\'file/FileUtils\',\'filesystem/FileSystemStats\',\'filesystem/FileSystemError\',\'utils/NodeDomain\'],function (require, exports, module) {\n    "use strict";\n    \n    var FileUtils           = require("file/FileUtils"),\n        FileSystemStats     = require("filesystem/FileSystemStats"),\n        FileSystemError     = require("filesystem/FileSystemError"),\n        NodeDomain          = require("utils/NodeDomain");\n    \n    var FILE_WATCHER_BATCH_TIMEOUT = 200;   // 200ms - granularity of file watcher changes\n    \n    var _changeCallback,            // Callback to notify FileSystem of watcher changes\n        _offlineCallback,           // Callback to notify FileSystem that watchers are offline\n        _changeTimeout,             // Timeout used to batch up file watcher changes\n        _pendingChanges = {};       // Pending file watcher changes\n    \n    var _bracketsPath   = FileUtils.getNativeDirectoryPath(),\n        _modulePath     = FileUtils.getNativeModuleDirectoryPath(module),\n        _nodePath       = "node/FileWatcherDomain",\n        _domainPath     = [_bracketsPath, _modulePath, _nodePath].join("/"),\n        _nodeDomain     = new NodeDomain("fileWatcher", _domainPath);\n    \n    var _isRunningOnWindowsXP = navigator.userAgent.indexOf("Windows NT 5.") >= 0;\n    \n    // If the connection closes, notify the FileSystem that watchers have gone offline.\n    $(_nodeDomain.connection).on("close", function (event, promise) {\n        if (_offlineCallback) {\n            _offlineCallback();\n        }\n    });\n    \n    /**\n     * Enqueue a file change event for eventual reporting back to the FileSystem.\n     * \n     * @param {string} changedPath The path that was changed\n     * @param {boolean} needsStats Whether or not the eventual change event should include stats\n     * @private\n     */\n    function _enqueueChange(changedPath, needsStats) {\n        _pendingChanges[changedPath] = _pendingChanges[changedPath] || needsStats;\n\n        if (!_changeTimeout) {\n            _changeTimeout = window.setTimeout(function () {\n                if (_changeCallback) {\n                    Object.keys(_pendingChanges).forEach(function (path) {\n                        var needsStats = _pendingChanges[path];\n                        if (needsStats) {\n                            exports.stat(path, function (err, stats) {\n                                if (err) {\n                                    console.warn("Unable to stat changed path: ", path, err);\n                                    return;\n                                }\n                                _changeCallback(path, stats);\n                            });\n                        } else {\n                            _changeCallback(path);\n                        }\n                    });\n                }\n                \n                _changeTimeout = null;\n                _pendingChanges = {};\n            }, FILE_WATCHER_BATCH_TIMEOUT);\n        }\n    }\n    \n    /**\n     * Event handler for the Node fileWatcher domain\'s change event.\n     * \n     * @param {jQuery.Event} The underlying change event\n     * @param {string} path The path that is reported to have changed\n     * @param {string} event The type of the event: either "change" or "rename"\n     * @param {string=} filename The name of the file that changed.\n     * @private\n     */\n    function _fileWatcherChange(evt, path, event, filename) {\n        var change;\n\n        if (event === "change") {\n            // Only register change events if filename is passed\n            if (filename) {\n                // an existing file was created; stats are needed\n                change = path + filename;\n                _enqueueChange(change, true);\n            }\n        } else if (event === "rename") {\n            // a new file was created; no stats are needed\n            change = path;\n            _enqueueChange(change, false);\n        }\n    }\n\n    // Setup the change handler. This only needs to happen once.\n    $(_nodeDomain).on("change", _fileWatcherChange);\n\n    /**\n     * Convert appshell error codes to FileSystemError values.\n     * \n     * @param {?number} err An appshell error code\n     * @return {?string} A FileSystemError string, or null if there was no error code.\n     * @private\n     */\n    function _mapError(err) {\n        if (!err) {\n            return null;\n        }\n        \n        switch (err) {\n        case appshell.fs.ERR_INVALID_PARAMS:\n            return FileSystemError.INVALID_PARAMS;\n        case appshell.fs.ERR_NOT_FOUND:\n            return FileSystemError.NOT_FOUND;\n        case appshell.fs.ERR_CANT_READ:\n            return FileSystemError.NOT_READABLE;\n        case appshell.fs.ERR_CANT_WRITE:\n            return FileSystemError.NOT_WRITABLE;\n        case appshell.fs.ERR_UNSUPPORTED_ENCODING:\n            return FileSystemError.NOT_READABLE;\n        case appshell.fs.ERR_OUT_OF_SPACE:\n            return FileSystemError.OUT_OF_SPACE;\n        case appshell.fs.ERR_FILE_EXISTS:\n            return FileSystemError.ALREADY_EXISTS;\n        }\n        return FileSystemError.UNKNOWN;\n    }\n    \n    /**\n     * Convert a callback to one that transforms its first parameter from an\n     * appshell error code to a FileSystemError string.\n     * \n     * @param {function(?number)} cb A callback that expects an appshell error code\n     * @return {function(?string)} A callback that expects a FileSystemError string\n     * @private\n     */\n    function _wrap(cb) {\n        return function (err) {\n            var args = Array.prototype.slice.call(arguments);\n            args[0] = _mapError(args[0]);\n            cb.apply(null, args);\n        };\n    }\n    \n    /**\n     * Display an open-files dialog to the user and call back asynchronously with\n     * either a FileSystmError string or an array of path strings, which indicate\n     * the entry or entries selected.\n     * \n     * @param {boolean} allowMultipleSelection\n     * @param {boolean} chooseDirectories\n     * @param {string} title\n     * @param {string} initialPath\n     * @param {Array.<string>=} fileTypes\n     * @param {function(?string, Array.<string>=)} callback\n     */\n    function showOpenDialog(allowMultipleSelection, chooseDirectories, title, initialPath, fileTypes, callback) {\n        appshell.fs.showOpenDialog(allowMultipleSelection, chooseDirectories, title, initialPath, fileTypes, _wrap(callback));\n    }\n    \n    /**\n     * Display a save-file dialog and call back asynchronously with either a\n     * FileSystemError string or the path to which the user has chosen to save\n     * the file. If the dialog is cancelled, the path string will be empty.\n     * \n     * @param {string} title\n     * @param {string} initialPath\n     * @param {string} proposedNewFilename\n     * @param {function(?string, string=)} callback\n     */\n    function showSaveDialog(title, initialPath, proposedNewFilename, callback) {\n        appshell.fs.showSaveDialog(title, initialPath, proposedNewFilename, _wrap(callback));\n    }\n    \n    /**\n     * Stat the file or directory at the given path, calling back\n     * asynchronously with either a FileSystemError string or the entry\'s\n     * associated FileSystemStats object.\n     * \n     * @param {string} path\n     * @param {function(?string, FileSystemStats=)} callback\n     */\n    function stat(path, callback) {\n        appshell.fs.stat(path, function (err, stats) {\n            if (err) {\n                callback(_mapError(err));\n            } else {\n                var options = {\n                    isFile: stats.isFile(),\n                    mtime: stats.mtime,\n                    size: stats.size,\n                    realPath: stats.realPath,\n                    hash: stats.mtime.getTime()\n                };\n                \n                var fsStats = new FileSystemStats(options);\n                \n                callback(null, fsStats);\n            }\n        });\n    }\n    \n    /**\n     * Determine whether a file or directory exists at the given path by calling\n     * back asynchronously with either a FileSystemError string or a boolean,\n     * which is true if the file exists and false otherwise. The error will never\n     * be FileSystemError.NOT_FOUND; in that case, there will be no error and the\n     * boolean parameter will be false.\n     * \n     * @param {string} path\n     * @param {function(?string, boolean)} callback\n     */\n    function exists(path, callback) {\n        stat(path, function (err) {\n            if (err) {\n                if (err === FileSystemError.NOT_FOUND) {\n                    callback(null, false);\n                } else {\n                    callback(err);\n                }\n                return;\n            }\n\n            callback(null, true);\n        });\n    }\n    \n    /**\n     * Read the contents of the directory at the given path, calling back\n     * asynchronously either with a FileSystemError string or an array of \n     * FileSystemEntry objects along with another consistent array, each index\n     * of which either contains a FileSystemStats object for the corresponding\n     * FileSystemEntry object in the second parameter or a FileSystemError\n     * string describing a stat error.\n     * \n     * @param {string} path\n     * @param {function(?string, Array.<FileSystemEntry>=, Array.<string|FileSystemStats>=)} callback\n     */\n    function readdir(path, callback) {\n        appshell.fs.readdir(path, function (err, contents) {\n            if (err) {\n                callback(_mapError(err));\n                return;\n            }\n            \n            var count = contents.length;\n            if (!count) {\n                callback(null, [], []);\n                return;\n            }\n            \n            var stats = [];\n            contents.forEach(function (val, idx) {\n                stat(path + "/" + val, function (err, stat) {\n                    stats[idx] = err || stat;\n                    count--;\n                    if (count <= 0) {\n                        callback(null, contents, stats);\n                    }\n                });\n            });\n        });\n    }\n    \n    /**\n     * Create a directory at the given path, and call back asynchronously with\n     * either a FileSystemError string or a stats object for the newly created\n     * directory. The octal mode parameter is optional; if unspecified, the mode\n     * of the created directory is implementation dependent.\n     * \n     * @param {string} path\n     * @param {number=} mode The base-eight mode of the newly created directory.\n     * @param {function(?string, FileSystemStats=)=} callback\n     */\n    function mkdir(path, mode, callback) {\n        if (typeof mode === "function") {\n            callback = mode;\n            mode = parseInt("0755", 8);\n        }\n        appshell.fs.makedir(path, mode, function (err) {\n            if (err) {\n                callback(_mapError(err));\n            } else {\n                stat(path, function (err, stat) {\n                    callback(err, stat);\n                });\n            }\n        });\n    }\n    \n    /**\n     * Rename the file or directory at oldPath to newPath, and call back\n     * asynchronously with a possibly null FileSystemError string.\n     * \n     * @param {string} oldPath\n     * @param {string} newPath\n     * @param {function(?string)=} callback\n     */\n    function rename(oldPath, newPath, callback) {\n        appshell.fs.rename(oldPath, newPath, _wrap(callback));\n    }\n    \n    /**\n     * Read the contents of the file at the given path, calling back\n     * asynchronously with either a FileSystemError string, or with the data and\n     * the FileSystemStats object associated with the read file. The options\n     * parameter can be used to specify an encoding (default "utf8"), and also\n     * a cached stats object that the implementation is free to use in order\n     * to avoid an additional stat call.\n     * \n     * Note: if either the read or the stat call fails then neither the read data\n     * nor stat will be passed back, and the call should be considered to have failed.\n     * If both calls fail, the error from the read call is passed back.\n     * \n     * @param {string} path\n     * @param {{encoding: string=, stat: FileSystemStats=}} options\n     * @param {function(?string, string=, FileSystemStats=)} callback\n     */\n    function readFile(path, options, callback) {\n        var encoding = options.encoding || "utf8";\n        \n        // Execute the read and stat calls in parallel. Callback early if the\n        // read call completes first with an error; otherwise wait for both\n        // to finish.\n        var done = false, data, stat, err;\n\n        if (options.stat) {\n            done = true;\n            stat = options.stat;\n        } else {\n            exports.stat(path, function (_err, _stat) {\n                if (done) {\n                    callback(_err, _err ? null : data, _stat);\n                } else {\n                    done = true;\n                    stat = _stat;\n                    err = _err;\n                }\n            });\n        }\n        \n        appshell.fs.readFile(path, encoding, function (_err, _data) {\n            if (_err) {\n                callback(_mapError(_err));\n                return;\n            }\n            \n            if (done) {\n                callback(err, err ? null : _data, stat);\n            } else {\n                done = true;\n                data = _data;\n            }\n        });\n    }\n    \n    /**\n     * Write data to the file at the given path, calling back asynchronously with\n     * either a FileSystemError string or the FileSystemStats object associated\n     * with the written file and a boolean that indicates whether the file was\n     * created by the write (true) or not (false). If no file exists at the\n     * given path, a new file will be created. The options parameter can be used\n     * to specify an encoding (default "utf8"), an octal mode (default\n     * unspecified and implementation dependent), and a consistency hash, which\n     * is used to the current state of the file before overwriting it. If a\n     * consistency hash is provided but does not match the hash of the file on\n     * disk, a FileSystemError.CONTENTS_MODIFIED error is passed to the callback.\n     * \n     * @param {string} path\n     * @param {string} data\n     * @param {{encoding : string=, mode : number=, expectedHash : object=, expectedContents : string=}} options\n     * @param {function(?string, FileSystemStats=, boolean)} callback\n     */\n    function writeFile(path, data, options, callback) {\n        var encoding = options.encoding || "utf8";\n        \n        function _finishWrite(created) {\n            appshell.fs.writeFile(path, data, encoding, function (err) {\n                if (err) {\n                    callback(_mapError(err));\n                } else {\n                    stat(path, function (err, stat) {\n                        callback(err, stat, created);\n                    });\n                }\n            });\n        }\n        \n        stat(path, function (err, stats) {\n            if (err) {\n                switch (err) {\n                case FileSystemError.NOT_FOUND:\n                    _finishWrite(true);\n                    break;\n                default:\n                    callback(err);\n                }\n                return;\n            }\n            \n            if (options.hasOwnProperty("expectedHash") && options.expectedHash !== stats._hash) {\n                console.error("Blind write attempted: ", path, stats._hash, options.expectedHash);\n\n                if (options.hasOwnProperty("expectedContents")) {\n                    appshell.fs.readFile(path, encoding, function (_err, _data) {\n                        if (_err || _data !== options.expectedContents) {\n                            callback(FileSystemError.CONTENTS_MODIFIED);\n                            return;\n                        }\n                    \n                        _finishWrite(false);\n                    });\n                    return;\n                } else {\n                    callback(FileSystemError.CONTENTS_MODIFIED);\n                    return;\n                }\n            }\n            \n            _finishWrite(false);\n        });\n    }\n    \n    /**\n     * Unlink (i.e., permanently delete) the file or directory at the given path,\n     * calling back asynchronously with a possibly null FileSystemError string.\n     * Directories will be unlinked even when non-empty.\n     * \n     * @param {string} path\n     * @param {function(string)=} callback\n     */\n    function unlink(path, callback) {\n        appshell.fs.unlink(path, function (err) {\n            callback(_mapError(err));\n        });\n    }\n\n    /**\n     * Move the file or directory at the given path to a system dependent trash\n     * location, calling back asynchronously with a possibly null FileSystemError\n     * string. Directories will be moved even when non-empty.\n     * \n     * @param {string} path\n     * @param {function(string)=} callback\n     */\n    function moveToTrash(path, callback) {\n        appshell.fs.moveToTrash(path, function (err) {\n            callback(_mapError(err));\n        });\n    }\n    \n    /**\n     * Initialize file watching for this filesystem, using the supplied\n     * changeCallback to provide change notifications. The first parameter of\n     * changeCallback specifies the changed path (either a file or a directory);\n     * if this parameter is null, it indicates that the implementation cannot\n     * specify a particular changed path, and so the callers should consider all\n     * paths to have changed and to update their state accordingly. The second\n     * parameter to changeCallback is an optional FileSystemStats object that\n     * may be provided in case the changed path already exists and stats are\n     * readily available. The offlineCallback will be called in case watchers\n     * are no longer expected to function properly. All watched paths are\n     * cleared when the offlineCallback is called.\n     * \n     * @param {function(?string, FileSystemStats=)} changeCallback\n     * @param {function()=} offlineCallback\n     */\n    function initWatchers(changeCallback, offlineCallback) {\n        _changeCallback = changeCallback;\n        _offlineCallback = offlineCallback;\n        \n        if (_isRunningOnWindowsXP && _offlineCallback) {\n            _offlineCallback();\n        }\n    }\n    \n    /**\n     * Start providing change notifications for the file or directory at the\n     * given path, calling back asynchronously with a possibly null FileSystemError\n     * string when the initialization is complete. Notifications are provided\n     * using the changeCallback function provided by the initWatchers method.\n     * Note that change notifications are only provided recursively for directories\n     * when the recursiveWatch property of this module is true.\n     * \n     * @param {string} path\n     * @param {function(?string)=} callback\n     */\n    function watchPath(path, callback) {\n        if (_isRunningOnWindowsXP) {\n            callback(FileSystemError.NOT_SUPPORTED);\n            return;\n        }\n        appshell.fs.isNetworkDrive(path, function (err, isNetworkDrive) {\n            if (err || isNetworkDrive) {\n                callback(FileSystemError.UNKNOWN);\n                return;\n            }\n            \n            _nodeDomain.exec("watchPath", path)\n                .then(callback, callback);\n        });\n    }\n    \n    /**\n     * Stop providing change notifications for the file or directory at the\n     * given path, calling back asynchronously with a possibly null FileSystemError\n     * string when the operation is complete.\n     * \n     * @param {string} path\n     * @param {function(?string)=} callback\n     */\n    function unwatchPath(path, callback) {\n        _nodeDomain.exec("unwatchPath", path)\n            .then(callback, callback);\n    }\n    \n    /**\n     * Stop providing change notifications for all previously watched files and\n     * directories, optionally calling back asynchronously with a possibly null\n     * FileSystemError string when the operation is complete.\n     *\n     * @param {function(?string)=} callback\n     */\n    function unwatchAll(callback) {\n        _nodeDomain.exec("unwatchAll")\n            .then(callback, callback);\n    }\n    \n    // Export public API\n    exports.showOpenDialog  = showOpenDialog;\n    exports.showSaveDialog  = showSaveDialog;\n    exports.exists          = exists;\n    exports.readdir         = readdir;\n    exports.mkdir           = mkdir;\n    exports.rename          = rename;\n    exports.stat            = stat;\n    exports.readFile        = readFile;\n    exports.writeFile       = writeFile;\n    exports.unlink          = unlink;\n    exports.moveToTrash     = moveToTrash;\n    exports.initWatchers    = initWatchers;\n    exports.watchPath       = watchPath;\n    exports.unwatchPath     = unwatchPath;\n    exports.unwatchAll      = unwatchAll;\n    \n    /**\n     * Indicates whether or not recursive watching notifications are supported\n     * by the watchPath call. Currently, only Darwin supports recursive watching.\n     *\n     * @type {boolean}\n     */\n    exports.recursiveWatch = (appshell.platform === "mac" || appshell.platform === "win");\n    \n    /**\n     * Indicates whether or not the filesystem should expect and normalize UNC\n     * paths. If set, then //server/directory/ is a normalized path; otherwise the\n     * filesystem will normalize it to /server/directory. Currently, UNC path \n     * normalization only occurs on Windows.\n     *\n     * @type {boolean}\n     */\n    exports.normalizeUNCPaths = appshell.platform === "win";\n});\n\n//# sourceURL=/filesystem/impls/appshell/AppshellFileSystem.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define, $ */\n\n/**\n * FileSystem is a model object representing a complete file system. This object creates\n * and manages File and Directory instances, dispatches events when the file system changes,\n * and provides methods for showing \'open\' and \'save\' dialogs.\n *\n * FileSystem automatically initializes when loaded. It depends on a pluggable "impl" layer, which\n * it loads itself but must be designated in the require.config() that loads FileSystem.\n *\n * There are three ways to get File or Directory instances:\n *    * Use FileSystem.resolve() to convert a path to a File/Directory object. This will only\n *      succeed if the file/directory already exists.\n *    * Use FileSystem.getFileForPath()/FileSystem.getDirectoryForPath() if you know the\n *      file/directory already exists, or if you want to create a new entry.\n *    * Use Directory.getContents() to return all entries for the specified Directory.\n *\n * All paths passed *to* FileSystem APIs must be in the following format:\n *    * The path separator is "/" regardless of platform\n *    * Paths begin with "/" on Mac/Linux and "c:/" (or some other drive letter) on Windows\n *\n * All paths returned *from* FileSystem APIs additionally meet the following guarantees:\n *    * No ".." segments\n *    * No consecutive "/"s\n *    * Paths to a directory always end with a trailing "/"\n * (Because FileSystem normalizes paths automatically, paths passed *to* FileSystem do not need\n * to meet these requirements)\n *\n * FileSystem dispatches the following events:\n *    change - Sent whenever there is a change in the file system. The handler\n *          is passed up to three arguments: the changed entry and, if that changed entry\n *          is a Directory, a list of entries added to the directory and a list of entries\n *          removed from the Directory. The entry argument can be:\n *          *  a File - the contents of the file have changed, and should be reloaded.\n *          *  a Directory - an immediate child of the directory has been added, removed,\n *             or renamed/moved. Not triggered for "grandchildren".\n *               - If the added & removed arguments are null, we don\'t know what was added/removed:\n *                 clients should assume the whole subtree may have changed.\n *               - If the added & removed arguments are 0-length, there\'s no net change in the set\n *                 of files but a file may have been replaced: clients should assume the contents\n *                 of any immediate child file may have changed.\n *          *  null - a \'wholesale\' change happened, and you should assume everything may\n *             have changed.\n *          For changes made externally, there may be a significant delay before a "change" event\n *          is dispatched.\n *    rename - Sent whenever a File or Directory is renamed. All affected File and Directory\n *          objects have been updated to reflect the new path by the time this event is dispatched.\n *          This event should be used to trigger any UI updates that may need to occur when a path\n *          has changed. Note that these events will only be sent for rename operations that happen\n *          within the filesystem. If a file is renamed externally, a change event on the parent\n *          directory will be sent instead.\n *\n * FileSystem may perform caching. But it guarantees:\n *    * File contents & metadata - reads are guaranteed to be up to date (cached data is not used\n *      without first veryifying it is up to date).\n *    * Directory structure / file listing - reads may return cached data immediately, which may not\n *      reflect external changes made recently. (However, changes made via FileSystem itself are always\n *      reflected immediately, as soon as the change operation\'s callback signals success).\n *\n * The FileSystem doesn\'t directly read or write contents--this work is done by a low-level\n * implementation object. This allows client code to use the FileSystem API without having to\n * worry about the underlying storage, which could be a local filesystem or a remote server.\n */\ndefine(\'filesystem/FileSystem\',[\'require\',\'exports\',\'module\',\'filesystem/Directory\',\'filesystem/File\',\'filesystem/FileIndex\',\'filesystem/FileSystemError\',\'filesystem/WatchedRoot\',\'fileSystemImpl\'],function (require, exports, module) {\n    "use strict";\n\n    var Directory       = require("filesystem/Directory"),\n        File            = require("filesystem/File"),\n        FileIndex       = require("filesystem/FileIndex"),\n        FileSystemError = require("filesystem/FileSystemError"),\n        WatchedRoot     = require("filesystem/WatchedRoot");\n\n    /** Unique token used to indicate user-driven cancellation of Save As (as opposed to file IO error) */\n    var USER_CANCELED = { userCanceled: true };\n\n    /**\n     * @constructor\n     * The FileSystem is not usable until init() signals its callback.\n     */\n    function FileSystem() {\n        // Create a file index\n        this._index = new FileIndex();\n\n        // Initialize the set of watched roots\n        this._watchedRoots = {};\n\n        // Initialize the watch/unwatch request queue\n        this._watchRequests = [];\n\n        // Initialize the queue of pending external changes\n        this._externalChanges = [];\n    }\n\n    /**\n     * The low-level file system implementation used by this object.\n     * This is set in the init() function and cannot be changed.\n     */\n    FileSystem.prototype._impl = null;\n\n    /**\n     * The FileIndex used by this object. This is initialized in the constructor.\n     */\n    FileSystem.prototype._index = null;\n\n    /**\n     * Refcount of any pending filesystem mutation operations (e.g., writes,\n     * unlinks, etc.). Used to ensure that external change events aren\'t processed\n     * until after index fixups, operation-specific callbacks, and internal change\n     * events are complete. (This is important for distinguishing rename from\n     * an unrelated delete-add pair).\n     * @type {number}\n     */\n    FileSystem.prototype._activeChangeCount = 0;\n\n    // For unit testing only\n    FileSystem.prototype._getActiveChangeCount = function () {\n        return this._activeChangeCount;\n    };\n\n    /**\n     * Queue of arguments with which to invoke _handleExternalChanges(); triggered\n     * once _activeChangeCount drops to zero.\n     * @type {!Array.<{path:?string, stat:FileSystemStats=}>}\n     */\n    FileSystem.prototype._externalChanges = null;\n\n    /** Process all queued watcher results, by calling _handleExternalChange() on each */\n    FileSystem.prototype._triggerExternalChangesNow = function () {\n        this._externalChanges.forEach(function (info) {\n            this._handleExternalChange(info.path, info.stat);\n        }, this);\n        this._externalChanges.length = 0;\n    };\n\n    /**\n     * Receives a result from the impl\'s watcher callback, and either processes it\n     * immediately (if _activeChangeCount is 0) or otherwise stores it for later\n     * processing.\n     * @param {?string} path The fullPath of the changed entry\n     * @param {FileSystemStats=} stat An optional stat object for the changed entry\n     */\n    FileSystem.prototype._enqueueExternalChange = function (path, stat) {\n        this._externalChanges.push({path: path, stat: stat});\n        if (!this._activeChangeCount) {\n            this._triggerExternalChangesNow();\n        }\n    };\n\n\n    /**\n     * The queue of pending watch/unwatch requests.\n     * @type {Array.<{fn: function(), cb: function()}>}\n     */\n    FileSystem.prototype._watchRequests = null;\n\n    /**\n     * Dequeue and process all pending watch/unwatch requests\n     */\n    FileSystem.prototype._dequeueWatchRequest = function () {\n        if (this._watchRequests.length > 0) {\n            var request = this._watchRequests[0];\n\n            request.fn.call(null, function () {\n                // Apply the given callback\n                var callbackArgs = arguments;\n                try {\n                    request.cb.apply(null, callbackArgs);\n                } finally {\n                    // Process the remaining watch/unwatch requests\n                    this._watchRequests.shift();\n                    this._dequeueWatchRequest();\n                }\n            }.bind(this));\n        }\n    };\n\n    /**\n     * Enqueue a new watch/unwatch request.\n     *\n     * @param {function()} fn - The watch/unwatch request function.\n     * @param {callback()} cb - The callback for the provided watch/unwatch\n     *      request function.\n     */\n    FileSystem.prototype._enqueueWatchRequest = function (fn, cb) {\n        // Enqueue the given watch/unwatch request\n        this._watchRequests.push({fn: fn, cb: cb});\n\n        // Begin processing the queue if it is not already being processed\n        if (this._watchRequests.length === 1) {\n            this._dequeueWatchRequest();\n        }\n    };\n\n    /**\n     * The set of watched roots, encoded as a mapping from full paths to WatchedRoot\n     * objects which contain a file entry, filter function, and an indication of\n     * whether the watched root is inactive, starting up or fully active.\n     *\n     * @type{Object.<string, WatchedRoot>}\n     */\n    FileSystem.prototype._watchedRoots = null;\n\n    /**\n     * Finds a parent watched root for a given path, or returns null if a parent\n     * watched root does not exist.\n     *\n     * @param{string} fullPath The child path for which a parent watched root is to be found\n     * @return{?{entry: FileSystemEntry, filter: function(string) boolean}} The parent\n     *      watched root, if it exists, or null.\n     */\n    FileSystem.prototype._findWatchedRootForPath = function (fullPath) {\n        var watchedRoot = null;\n\n        Object.keys(this._watchedRoots).some(function (watchedPath) {\n            if (fullPath.indexOf(watchedPath) === 0) {\n                watchedRoot = this._watchedRoots[watchedPath];\n                return true;\n            }\n        }, this);\n\n        return watchedRoot;\n    };\n\n    /**\n     * Helper function to watch or unwatch a filesystem entry beneath a given\n     * watchedRoot.\n     *\n     * @private\n     * @param {FileSystemEntry} entry - The FileSystemEntry to watch. Must be a\n     *      non-strict descendent of watchedRoot.entry.\n     * @param {WatchedRoot} watchedRoot - See FileSystem._watchedRoots.\n     * @param {function(?string)} callback - A function that is called once the\n     *      watch is complete, possibly with a FileSystemError string.\n     * @param {boolean} shouldWatch - Whether the entry should be watched (true)\n     *      or unwatched (false).\n     */\n    FileSystem.prototype._watchOrUnwatchEntry = function (entry, watchedRoot, callback, shouldWatch) {\n        var impl = this._impl,\n            recursiveWatch = impl.recursiveWatch,\n            commandName = shouldWatch ? "watchPath" : "unwatchPath";\n\n        if (recursiveWatch) {\n            if (entry !== watchedRoot.entry) {\n                // Watch and unwatch calls to children of the watched root are\n                // no-ops if the impl supports recursiveWatch\n                callback(null);\n            } else {\n                // The impl will handle finding all subdirectories to watch.\n                this._enqueueWatchRequest(function (requestCb) {\n                    impl[commandName].call(impl, entry.fullPath, requestCb);\n                }.bind(this), callback);\n            }\n        } else if (shouldWatch) {\n            // The impl can\'t handle recursive watch requests, so it\'s up to the\n            // filesystem to recursively watch all subdirectories.\n            this._enqueueWatchRequest(function (requestCb) {\n                // First construct a list of entries to watch or unwatch\n                var entriesToWatch = [];\n\n                var visitor = function (child) {\n                    if (watchedRoot.filter(child.name, child.parentPath)) {\n                        if (child.isDirectory || child === watchedRoot.entry) {\n                            entriesToWatch.push(child);\n                        }\n                        return true;\n                    }\n                    return false;\n                };\n\n                entry.visit(visitor, function (err) {\n                    if (err) {\n                        // Unexpected error\n                        requestCb(err);\n                        return;\n                    }\n\n                    // Then watch or unwatched all these entries\n                    var count = entriesToWatch.length;\n                    if (count === 0) {\n                        requestCb(null);\n                        return;\n                    }\n\n                    var watchCallback = function () {\n                        if (--count === 0) {\n                            requestCb(null);\n                        }\n                    };\n\n                    entriesToWatch.forEach(function (entry) {\n                        impl.watchPath(entry.fullPath, watchCallback);\n                    });\n                });\n            }, callback);\n        } else {\n            this._enqueueWatchRequest(function (requestCb) {\n                impl.unwatchPath(entry.fullPath, requestCb);\n            }, callback);\n        }\n    };\n\n    /**\n     * Watch a filesystem entry beneath a given watchedRoot.\n     *\n     * @private\n     * @param {FileSystemEntry} entry - The FileSystemEntry to watch. Must be a\n     *      non-strict descendent of watchedRoot.entry.\n     * @param {WatchedRoot} watchedRoot - See FileSystem._watchedRoots.\n     * @param {function(?string)} callback - A function that is called once the\n     *      watch is complete, possibly with a FileSystemError string.\n     */\n    FileSystem.prototype._watchEntry = function (entry, watchedRoot, callback) {\n        this._watchOrUnwatchEntry(entry, watchedRoot, callback, true);\n    };\n\n    /**\n     * Unwatch a filesystem entry beneath a given watchedRoot.\n     *\n     * @private\n     * @param {FileSystemEntry} entry - The FileSystemEntry to watch. Must be a\n     *      non-strict descendent of watchedRoot.entry.\n     * @param {WatchedRoot} watchedRoot - See FileSystem._watchedRoots.\n     * @param {function(?string)} callback - A function that is called once the\n     *      watch is complete, possibly with a FileSystemError string.\n     */\n    FileSystem.prototype._unwatchEntry = function (entry, watchedRoot, callback) {\n        this._watchOrUnwatchEntry(entry, watchedRoot, function (err) {\n            // Make sure to clear cached data for all unwatched entries because\n            // entries always return cached data if it exists!\n            this._index.visitAll(function (child) {\n                if (child.fullPath.indexOf(entry.fullPath) === 0) {\n                    // \'true\' so entry doesn\'t try to clear its immediate childrens\' caches too. That would be redundant\n                    // with the visitAll() here, and could be slow if we\'ve already cleared its parent (#7150).\n                    child._clearCachedData(true);\n                }\n            }.bind(this));\n\n            callback(err);\n        }.bind(this), false);\n    };\n\n    /**\n     * Initialize this FileSystem instance.\n     *\n     * @param {FileSystemImpl} impl The back-end implementation for this\n     *      FileSystem instance.\n     */\n    FileSystem.prototype.init = function (impl) {\n        console.assert(!this._impl, "This FileSystem has already been initialized!");\n\n        var changeCallback = this._enqueueExternalChange.bind(this),\n            offlineCallback = this._unwatchAll.bind(this);\n\n        this._impl = impl;\n        this._impl.initWatchers(changeCallback, offlineCallback);\n    };\n\n    /**\n     * Close a file system. Clear all caches, indexes, and file watchers.\n     */\n    FileSystem.prototype.close = function () {\n        this._impl.unwatchAll();\n        this._index.clear();\n    };\n\n    /**\n     * Returns true if the given path should be automatically added to the index & watch list when one of its ancestors\n     * is a watch-root. (Files are added automatically when the watch-root is first established, or later when a new\n     * directory is created and its children enumerated).\n     *\n     * Entries explicitly created via FileSystem.getFile/DirectoryForPath() are *always* added to the index regardless\n     * of this filtering - but they will not be watched if the watch-root\'s filter excludes them.\n     *\n     * @param {string} path Full path\n     * @param {string} name Name portion of the path\n     */\n    FileSystem.prototype._indexFilter = function (path, name) {\n        var parentRoot = this._findWatchedRootForPath(path);\n\n        if (parentRoot) {\n            return parentRoot.filter(name, path);\n        }\n\n        // It might seem more sensible to return false (exclude) for files outside the watch roots, but\n        // that would break usage of appFileSystem for \'system\'-level things like enumerating extensions.\n        // (Or in general, Directory.getContents() for any Directory outside the watch roots).\n        return true;\n    };\n\n    /**\n     * Indicates that a filesystem-mutating operation has begun. As long as there\n     * are changes taking place, change events from the external watchers are\n     * blocked and queued, to be handled once changes have finished. This is done\n     * because for mutating operations that originate from within the filesystem,\n     * synthetic change events are fired that do not depend on external file\n     * watchers, and we prefer the former over the latter for the following\n     * reasons: 1) there is no delay; and 2) they may have higher fidelity ---\n     * e.g., a rename operation can be detected as such, instead of as a nearly\n     * simultaneous addition and deletion.\n     *\n     * All operations that mutate the file system MUST begin with a call to\n     * _beginChange and must end with a call to _endChange.\n     */\n    FileSystem.prototype._beginChange = function () {\n        this._activeChangeCount++;\n        //console.log("> beginChange  -> " + this._activeChangeCount);\n    };\n\n    /**\n     * Indicates that a filesystem-mutating operation has completed. See\n     * FileSystem._beginChange above.\n     */\n    FileSystem.prototype._endChange = function () {\n        this._activeChangeCount--;\n        //console.log("< endChange    -> " + this._activeChangeCount);\n\n        if (this._activeChangeCount < 0) {\n            console.error("FileSystem _activeChangeCount has fallen below zero!");\n        }\n\n        if (!this._activeChangeCount) {\n            this._triggerExternalChangesNow();\n        }\n    };\n\n    /**\n     * Determines whether or not the supplied path is absolute, as opposed to relative.\n     *\n     * @param {!string} fullPath\n     * @return {boolean} True if the fullPath is absolute and false otherwise.\n     */\n    FileSystem.isAbsolutePath = function (fullPath) {\n        return (fullPath[0] === "/" || (fullPath[1] === ":" && fullPath[2] === "/"));\n    };\n\n    function _ensureTrailingSlash(path) {\n        if (path[path.length - 1] !== "/") {\n            path += "/";\n        }\n\n        return path;\n    }\n\n    /*\n     * Matches continguous groups of forward slashes\n     * @const\n     */\n    var _DUPLICATED_SLASH_RE = /\\/{2,}/g;\n\n    /**\n     * Returns a canonical version of the path: no duplicated "/"es, no ".."s,\n     * and directories guaranteed to end in a trailing "/"\n     * @param {!string} path  Absolute path, using "/" as path separator\n     * @param {boolean=} isDirectory\n     * @return {!string}\n     */\n    FileSystem.prototype._normalizePath = function (path, isDirectory) {\n\n        if (!FileSystem.isAbsolutePath(path)) {\n            throw new Error("Paths must be absolute: \'" + path + "\'");  // expect only absolute paths\n        }\n\n        var isUNCPath = this._impl.normalizeUNCPaths && path.search(_DUPLICATED_SLASH_RE) === 0;\n\n        // Remove duplicated "/"es\n        path = path.replace(_DUPLICATED_SLASH_RE, "/");\n\n        // Remove ".." segments\n        if (path.indexOf("..") !== -1) {\n            var segments = path.split("/"),\n                i;\n            for (i = 1; i < segments.length; i++) {\n                if (segments[i] === "..") {\n                    if (i < 2) {\n                        throw new Error("Invalid absolute path: \'" + path + "\'");\n                    }\n                    segments.splice(i - 1, 2);\n                    i -= 2; // compensate so we start on the right index next iteration\n                }\n            }\n            path = segments.join("/");\n        }\n\n        if (isDirectory) {\n            // Make sure path DOES include trailing slash\n            path = _ensureTrailingSlash(path);\n        }\n\n        if (isUNCPath) {\n            // Restore the leading double slash that was removed previously\n            path = "/" + path;\n        }\n\n        return path;\n    };\n\n    /**\n     * Return a (strict subclass of a) FileSystemEntry object for the specified\n     * path using the provided constuctor. For now, the provided constructor\n     * should be either File or Directory.\n     *\n     * @private\n     * @param {function(string, FileSystem)} EntryConstructor Constructor with\n     *      which to initialize new FileSystemEntry objects.\n     * @param {string} path Absolute path of file.\n     * @return {File|Directory} The File or Directory object. This file may not\n     *      yet exist on disk.\n     */\n    FileSystem.prototype._getEntryForPath = function (EntryConstructor, path) {\n        var isDirectory = EntryConstructor === Directory;\n        path = this._normalizePath(path, isDirectory);\n        var entry = this._index.getEntry(path);\n\n        if (!entry) {\n            entry = new EntryConstructor(path, this);\n            this._index.addEntry(entry);\n        }\n\n        return entry;\n    };\n\n    /**\n     * Return a File object for the specified path.\n     *\n     * @param {string} path Absolute path of file.\n     *\n     * @return {File} The File object. This file may not yet exist on disk.\n     */\n    FileSystem.prototype.getFileForPath = function (path) {\n        return this._getEntryForPath(File, path);\n    };\n\n    /**\n     * Return a Directory object for the specified path.\n     *\n     * @param {string} path Absolute path of directory.\n     *\n     * @return {Directory} The Directory object. This directory may not yet exist on disk.\n     */\n    FileSystem.prototype.getDirectoryForPath = function (path) {\n        return this._getEntryForPath(Directory, path);\n    };\n\n    /**\n     * Resolve a path.\n     *\n     * @param {string} path The path to resolve\n     * @param {function (?string, FileSystemEntry=, FileSystemStats=)} callback Callback resolved\n     *      with a FileSystemError string or with the entry for the provided path.\n     */\n    FileSystem.prototype.resolve = function (path, callback) {\n        var normalizedPath = this._normalizePath(path, false),\n            item = this._index.getEntry(normalizedPath);\n\n        if (!item) {\n            normalizedPath = _ensureTrailingSlash(normalizedPath);\n            item = this._index.getEntry(normalizedPath);\n        }\n\n        if (item) {\n            item.stat(function (err, stat) {\n                if (err) {\n                    callback(err);\n                    return;\n                }\n\n                callback(null, item, stat);\n            });\n        } else {\n            this._impl.stat(path, function (err, stat) {\n                if (err) {\n                    callback(err);\n                    return;\n                }\n\n                if (stat.isFile) {\n                    item = this.getFileForPath(path);\n                } else {\n                    item = this.getDirectoryForPath(path);\n                }\n\n                if (item._isWatched()) {\n                    item._stat = stat;\n                }\n\n                callback(null, item, stat);\n            }.bind(this));\n        }\n    };\n\n    /**\n     * Show an "Open" dialog and return the file(s)/directories selected by the user.\n     *\n     * @param {boolean} allowMultipleSelection Allows selecting more than one file at a time\n     * @param {boolean} chooseDirectories Allows directories to be opened\n     * @param {string} title The title of the dialog\n     * @param {string} initialPath The folder opened inside the window initially. If initialPath\n     *                          is not set, or it doesn\'t exist, the window would show the last\n     *                          browsed folder depending on the OS preferences\n     * @param {?Array.<string>} fileTypes (Currently *ignored* except on Mac - https://trello.com/c/430aXkpq)\n     *                          List of extensions that are allowed to be opened, without leading ".".\n     *                          Null or empty array allows all files to be selected. Not applicable\n     *                          when chooseDirectories = true.\n     * @param {function (?string, Array.<string>=)} callback Callback resolved with a FileSystemError\n     *                          string or the selected file(s)/directories. If the user cancels the\n     *                          open dialog, the error will be falsy and the file/directory array will\n     *                          be empty.\n     */\n    FileSystem.prototype.showOpenDialog = function (allowMultipleSelection,\n                            chooseDirectories,\n                            title,\n                            initialPath,\n                            fileTypes,\n                            callback) {\n\n        this._impl.showOpenDialog(allowMultipleSelection, chooseDirectories, title, initialPath, fileTypes, callback);\n    };\n\n    /**\n     * Show a "Save" dialog and return the path of the file to save.\n     *\n     * @param {string} title The title of the dialog.\n     * @param {string} initialPath The folder opened inside the window initially. If initialPath\n     *                          is not set, or it doesn\'t exist, the window would show the last\n     *                          browsed folder depending on the OS preferences.\n     * @param {string} proposedNewFilename Provide a new file name for the user. This could be based on\n     *                          on the current file name plus an additional suffix\n     * @param {function (?string, string=)} callback Callback that is resolved with a FileSystemError\n     *                          string or the name of the file to save. If the user cancels the save,\n     *                          the error will be falsy and the name will be empty.\n     */\n    FileSystem.prototype.showSaveDialog = function (title, initialPath, proposedNewFilename, callback) {\n        this._impl.showSaveDialog(title, initialPath, proposedNewFilename, callback);\n    };\n\n    /**\n     * Fire a rename event. Clients listen for these events using FileSystem.on.\n     *\n     * @param {string} oldPath The entry\'s previous fullPath\n     * @param {string} newPath The entry\'s current fullPath\n     */\n    FileSystem.prototype._fireRenameEvent = function (oldPath, newPath) {\n        $(this).trigger("rename", [oldPath, newPath]);\n    };\n\n    /**\n     * Fire a change event. Clients listen for these events using FileSystem.on.\n     *\n     * @param {File|Directory} entry The entry that has changed\n     * @param {Array<File|Directory>=} added If the entry is a directory, this\n     *      is a set of new entries in the directory.\n     * @param {Array<File|Directory>=} removed If the entry is a directory, this\n     *      is a set of removed entries from the directory.\n     */\n    FileSystem.prototype._fireChangeEvent = function (entry, added, removed) {\n        $(this).trigger("change", [entry, added, removed]);\n    };\n\n    /**\n     * @private\n     * Notify the system when an entry name has changed.\n     *\n     * @param {string} oldFullPath\n     * @param {string} newFullPath\n     * @param {boolean} isDirectory\n     */\n    FileSystem.prototype._handleRename = function (oldFullPath, newFullPath, isDirectory) {\n        // Update all affected entries in the index\n        this._index.entryRenamed(oldFullPath, newFullPath, isDirectory);\n    };\n\n    /**\n     * Notify the filesystem that the given directory has changed. Updates the filesystem\'s\n     * internal state as a result of the change, and calls back with the set of added and\n     * removed entries. Mutating FileSystemEntry operations should call this method before\n     * applying the operation\'s callback, and pass along the resulting change sets in the\n     * internal change event.\n     *\n     * @param {Directory} directory The directory that has changed.\n     * @param {function(Array<File|Directory>=, Array<File|Directory>=)} callback\n     *      The callback that will be applied to a set of added and a set of removed\n     *      FileSystemEntry objects.\n     */\n    FileSystem.prototype._handleDirectoryChange = function (directory, callback) {\n        var oldContents = directory._contents;\n\n        directory._clearCachedData();\n        directory.getContents(function (err, contents) {\n            var addedEntries = oldContents && contents.filter(function (entry) {\n                return oldContents.indexOf(entry) === -1;\n            });\n\n            var removedEntries = oldContents && oldContents.filter(function (entry) {\n                return contents.indexOf(entry) === -1;\n            });\n\n            // If directory is not watched, clear children\'s caches manually.\n            var watchedRoot = this._findWatchedRootForPath(directory.fullPath);\n            if (!watchedRoot || !watchedRoot.filter(directory.name, directory.parentPath)) {\n                this._index.visitAll(function (entry) {\n                    if (entry.fullPath.indexOf(directory.fullPath) === 0) {\n                        // Passing \'true\' for a similar reason as in _unwatchEntry() - see #7150\n                        entry._clearCachedData(true);\n                    }\n                }.bind(this));\n\n                callback(addedEntries, removedEntries);\n                return;\n            }\n\n            var addedCounter = addedEntries ? addedEntries.length : 0,\n                removedCounter = removedEntries ? removedEntries.length : 0,\n                counter = addedCounter + removedCounter;\n\n            if (counter === 0) {\n                callback(addedEntries, removedEntries);\n                return;\n            }\n\n            var watchOrUnwatchCallback = function (err) {\n                if (err) {\n                    console.error("FileSystem error in _handleDirectoryChange after watch/unwatch entries: " + err);\n                }\n\n                if (--counter === 0) {\n                    callback(addedEntries, removedEntries);\n                }\n            };\n\n            if (addedEntries) {\n                addedEntries.forEach(function (entry) {\n                    this._watchEntry(entry, watchedRoot, watchOrUnwatchCallback);\n                }, this);\n            }\n\n            if (removedEntries) {\n                removedEntries.forEach(function (entry) {\n                    this._unwatchEntry(entry, watchedRoot, watchOrUnwatchCallback);\n                }, this);\n            }\n        }.bind(this));\n    };\n\n    /**\n     * @private\n     * Processes a result from the file/directory watchers. Watch results are sent from the low-level implementation\n     * whenever a directory or file is changed.\n     *\n     * @param {string} path The path that changed. This could be a file or a directory.\n     * @param {FileSystemStats=} stat Optional stat for the item that changed. This param is not always\n     *         passed.\n     */\n    FileSystem.prototype._handleExternalChange = function (path, stat) {\n\n        if (!path) {\n            // This is a "wholesale" change event; clear all caches\n            this._index.visitAll(function (entry) {\n                // Passing \'true\' for a similar reason as in _unwatchEntry() - see #7150\n                entry._clearCachedData(true);\n            });\n\n            this._fireChangeEvent(null);\n            return;\n        }\n\n        path = this._normalizePath(path, false);\n\n        var entry = this._index.getEntry(path);\n        if (entry) {\n            var oldStat = entry._stat;\n            if (entry.isFile) {\n                // Update stat and clear contents, but only if out of date\n                if (!(stat && oldStat && stat.mtime.getTime() === oldStat.mtime.getTime())) {\n                    entry._clearCachedData();\n                    entry._stat = stat;\n                    this._fireChangeEvent(entry);\n                }\n            } else {\n                this._handleDirectoryChange(entry, function (added, removed) {\n                    entry._stat = stat;\n\n                    // We send a change even if added & removed are both zero-length. Something may still have changed,\n                    // e.g. a file may have been quickly removed & re-added before we got a chance to reread the directory\n                    // listing.\n                    this._fireChangeEvent(entry, added, removed);\n                }.bind(this));\n            }\n        }\n    };\n\n    /**\n     * Start watching a filesystem root entry.\n     *\n     * @param {FileSystemEntry} entry - The root entry to watch. If entry is a directory,\n     *      all subdirectories that aren\'t explicitly filtered will also be watched.\n     * @param {function(string): boolean} filter - Returns true if a particular item should\n     *      be watched, given its name (not full path). Items that are ignored are also\n     *      filtered from Directory.getContents() results within this subtree.\n     * @param {function(?string)=} callback - A function that is called when the watch has\n     *      completed. If the watch fails, the function will have a non-null FileSystemError\n     *      string parametr.\n     */\n    FileSystem.prototype.watch = function (entry, filter, callback) {\n        var fullPath = entry.fullPath;\n\n        callback = callback || function () {};\n\n        var watchingParentRoot = this._findWatchedRootForPath(fullPath);\n        if (watchingParentRoot &&\n                (watchingParentRoot.status === WatchedRoot.STARTING ||\n                 watchingParentRoot.status === WatchedRoot.ACTIVE)) {\n            callback("A parent of this root is already watched");\n            return;\n        }\n\n        var watchingChildRoot = Object.keys(this._watchedRoots).some(function (path) {\n            var watchedRoot = this._watchedRoots[path],\n                watchedPath = watchedRoot.entry.fullPath;\n\n            return watchedPath.indexOf(fullPath) === 0;\n        }, this);\n\n        if (watchingChildRoot &&\n                (watchingChildRoot.status === WatchedRoot.STARTING ||\n                 watchingChildRoot.status === WatchedRoot.ACTIVE)) {\n            callback("A child of this root is already watched");\n            return;\n        }\n\n        var watchedRoot = new WatchedRoot(entry, filter);\n\n        this._watchedRoots[fullPath] = watchedRoot;\n\n        // Enter the STARTING state early to indiate that watched Directory\n        // objects may cache their contents. See FileSystemEntry._isWatched.\n        watchedRoot.status = WatchedRoot.STARTING;\n\n        this._watchEntry(entry, watchedRoot, function (err) {\n            if (err) {\n                console.warn("Failed to watch root: ", entry.fullPath, err);\n                delete this._watchedRoots[fullPath];\n                callback(err);\n                return;\n            }\n\n            watchedRoot.status = WatchedRoot.ACTIVE;\n\n            callback(null);\n        }.bind(this));\n    };\n\n    /**\n     * Stop watching a filesystem root entry.\n     *\n     * @param {FileSystemEntry} entry - The root entry to stop watching. The unwatch will\n     *      if the entry is not currently being watched.\n     * @param {function(?string)=} callback - A function that is called when the unwatch has\n     *      completed. If the unwatch fails, the function will have a non-null FileSystemError\n     *      string parameter.\n     */\n    FileSystem.prototype.unwatch = function (entry, callback) {\n        var fullPath = entry.fullPath,\n            watchedRoot = this._watchedRoots[fullPath];\n\n        callback = callback || function () {};\n\n        if (!watchedRoot) {\n            callback(FileSystemError.ROOT_NOT_WATCHED);\n            return;\n        }\n\n        // Mark this as inactive, but don\'t delete the entry until the unwatch is complete.\n        // This is useful for making sure we don\'t try to concurrently watch overlapping roots.\n        watchedRoot.status = WatchedRoot.INACTIVE;\n\n        this._unwatchEntry(entry, watchedRoot, function (err) {\n            delete this._watchedRoots[fullPath];\n\n            this._index.visitAll(function (child) {\n                if (child.fullPath.indexOf(entry.fullPath) === 0) {\n                    this._index.removeEntry(child);\n                }\n            }.bind(this));\n\n            if (err) {\n                console.warn("Failed to unwatch root: ", entry.fullPath, err);\n                callback(err);\n                return;\n            }\n\n            callback(null);\n        }.bind(this));\n    };\n\n    /**\n     * Unwatch all watched roots. Calls unwatch on the underlying impl for each\n     * watched root and ignores errors.\n     * @private\n     */\n    FileSystem.prototype._unwatchAll = function () {\n        console.warn("File watchers went offline!");\n\n        Object.keys(this._watchedRoots).forEach(function (path) {\n            var watchedRoot = this._watchedRoots[path];\n\n            watchedRoot.status = WatchedRoot.INACTIVE;\n            delete this._watchedRoots[path];\n            this._unwatchEntry(watchedRoot.entry, watchedRoot, function () {\n                console.warn("Watching disabled for", watchedRoot.entry.fullPath);\n            });\n        }, this);\n\n        // Fire a wholesale change event, clearing all caches and request that\n        // clients manually update their state.\n        this._handleExternalChange(null);\n    };\n\n\n    // The singleton instance\n    var _instance;\n\n    function _wrap(func) {\n        return function () {\n            return func.apply(_instance, arguments);\n        };\n    }\n\n\n    exports.USER_CANCELED = USER_CANCELED;\n\n    // Export public methods as proxies to the singleton instance\n    exports.init = _wrap(FileSystem.prototype.init);\n    exports.close = _wrap(FileSystem.prototype.close);\n    exports.shouldShow = _wrap(FileSystem.prototype.shouldShow);\n    exports.getFileForPath = _wrap(FileSystem.prototype.getFileForPath);\n    exports.getDirectoryForPath = _wrap(FileSystem.prototype.getDirectoryForPath);\n    exports.resolve = _wrap(FileSystem.prototype.resolve);\n    exports.showOpenDialog = _wrap(FileSystem.prototype.showOpenDialog);\n    exports.showSaveDialog = _wrap(FileSystem.prototype.showSaveDialog);\n    exports.watch = _wrap(FileSystem.prototype.watch);\n    exports.unwatch = _wrap(FileSystem.prototype.unwatch);\n\n    // Static public utility methods\n    exports.isAbsolutePath = FileSystem.isAbsolutePath;\n\n    // For testing only\n    exports._getActiveChangeCount = _wrap(FileSystem.prototype._getActiveChangeCount);\n\n    /**\n     * Add an event listener for a FileSystem event.\n     *\n     * @param {string} event The name of the event\n     * @param {function} handler The handler for the event\n     */\n    exports.on = function (event, handler) {\n        $(_instance).on(event, handler);\n    };\n\n    /**\n     * Remove an event listener for a FileSystem event.\n     *\n     * @param {string} event The name of the event\n     * @param {function} handler The handler for the event\n     */\n    exports.off = function (event, handler) {\n        $(_instance).off(event, handler);\n    };\n\n    // Export the FileSystem class as "private" for unit testing only.\n    exports._FileSystem = FileSystem;\n\n    // Create the singleton instance\n    _instance = new FileSystem();\n\n    // Initialize the singleton instance\n    _instance.init(require("fileSystemImpl"));\n});\n\n//# sourceURL=/filesystem/FileSystem.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _*/\n\n/**\n *\n * This module dispatches these events:\n * - projectCreated:         (project)\n * - projectSaved:           (filename, project)\n * - projectLoaded:          (filename, project)\n * - beforeProjectClose:     (filename, project);\n * - projectClosed:\n * - imported:               (filename, element)\n * - exported:               (filename, element)\n */\ndefine('engine/ProjectManager',['require','exports','module','core/Core','core/OperationBuilder','core/Repository','file/FileUtils','filesystem/FileSystem'],function (require, exports, module) {\n    \"use strict\";\n\n    var Core             = require(\"core/Core\"),\n        OperationBuilder = require(\"core/OperationBuilder\"),\n        Repository       = require(\"core/Repository\"),\n        FileUtils        = require(\"file/FileUtils\"),\n        FileSystem       = require(\"filesystem/FileSystem\");\n\n    /**\n     * Top-level project element.\n     * @private\n     * @type {Core.Project}\n     */\n    var _project = null;\n\n    /**\n     * 프로젝트 파일 이름\n     * @private\n     * @type {string}\n     */\n    var _filename = null;\n\n\n    /**\n     * Return project\n     * @return {Core.Project}\n     */\n    function getProject() {\n        return _project;\n    }\n\n    /**\n     * New project\n     * @return {Core.Project}\n     */\n    function newProject() {\n        try {\n            _project = new Core.Project();\n            _filename = null;\n            Repository.clear();\n            Repository.getIdMap()[_project._id] = _project;\n            $(exports).triggerHandler('projectCreated', [_project]);\n        } catch (err) {\n            console.error(err);\n        }\n        return _project;\n    }\n\n    /**\n     * Close project\n     */\n    function closeProject() {\n        try {\n            $(exports).triggerHandler('beforeProjectClose', [_filename, _project]);\n            _project = null;\n            _filename = null;\n            Repository.clear();\n            $(exports).triggerHandler('projectClosed');\n        } catch (err) {\n            console.error(err);\n        }\n    }\n\n    /**\n     * Save project to file\n     * @param {string} fullPath\n     * @return {$.Promise}\n     */\n    function save(fullPath) {\n        var file = FileSystem.getFileForPath(fullPath),\n            result = new $.Deferred(),\n            data = Repository.writeObject(_project);\n\n        FileUtils.writeText(file, data, true)\n            .done(function () {\n                try {\n                    _filename = fullPath;\n                    Repository.setModified(false);\n                    $(exports).triggerHandler('projectSaved', [_filename, _project]);\n                } catch (err) {\n                    console.error(err);\n                }\n                result.resolve(_filename);\n            })\n            .fail(function (err) {\n                result.reject(err);\n            });\n        return result.promise();\n    }\n\n    /**\n     * Load from a file\n     * @param {string} fullPath\n     * @return {$.Promise}\n     */\n    function load(fullPath) {\n        var result = new $.Deferred(),\n            file = FileSystem.getFileForPath(fullPath);\n\n        FileUtils.readAsText(file)\n            .done(function (data) {\n                if (data) {\n                    closeProject();\n                    try {\n                        _project = Repository.readObject(data);\n                        _filename = fullPath;\n                        Repository.setModified(false);\n                        $(exports).triggerHandler('projectLoaded', [_filename, _project]);\n                    } catch (err) {\n                        console.error(err);\n                    }\n                    result.resolve(_project);\n                }\n            })\n            .fail(function (err) {\n                result.reject(err);\n            });\n        return result.promise();\n    }\n\n\n    /**\n     * Load from a file as template\n     * @param {string} fullPath\n     * @return {$.Promise} JQuery Promise Object.\n     */\n    function loadAsTemplate(fullPath) {\n        var file = FileSystem.getFileForPath(fullPath),\n            result = new $.Deferred();\n        FileUtils.readAsText(file)\n            .done(function (data) {\n                if (data) {\n                    closeProject();\n                    try {\n                        _project = Repository.readObject(data);\n                        _filename = null;\n                        Repository.setModified(false);\n                        $(exports).triggerHandler('projectLoaded', [_filename, _project]);\n                    } catch (err) {\n                        console.error(err);\n                    }\n                    result.resolve(_project);\n                }\n            })\n            .fail(function (err) {\n                result.reject(err);\n            });\n        return result.promise();\n    }\n\n    /**\n     * Load project from JSON\n     * @param {Object} data JSON data.\n     * @param {Project} Loaded Project object.\n     */\n    function loadFromJson(data) {\n        if (data) {\n            closeProject();\n            try {\n                _project = Repository.readObject(data);\n                Repository.setModified(false);\n                $(exports).triggerHandler('projectLoaded', [null, _project]);\n            } catch (err) {\n                console.error(err);\n            }\n        }\n        return _project;\n    }\n\n    /**\n     * Import model fragment from a file.\n     * @param {Core.Element} parent Element to contain the imported element.\n     * @param {string} fullPath\n     * @return {$.Promise} JQuery Promise Object.\n     */\n    function importFromFile(parent, fullPath) {\n        var file = FileSystem.getFileForPath(fullPath),\n            result = new $.Deferred();\n        FileUtils.readAsText(file)\n            .done(function (data) {\n                var elem;\n                if (data) {\n                    try {\n                        elem = Repository.readObject(data, true);\n                        // Bypass Operation (for insert elem to parent)\n                        OperationBuilder.begin(\"\", true);\n                        OperationBuilder.fieldInsert(parent, \"ownedElements\", elem);\n                        OperationBuilder.fieldAssign(elem, \"_parent\", parent);\n                        OperationBuilder.end();\n                        Repository.doOperation(OperationBuilder.getOperation());\n                        Repository.setModified(true);\n                        $(exports).triggerHandler('imported', [fullPath, elem]);\n                        result.resolve(elem);\n                    } catch (err) {\n                        console.error(err);\n                        result.reject(err);\n                    }\n                }\n            })\n            .fail(function (err) {\n                result.reject(err);\n            });\n        return result.promise();\n    }\n\n    /**\n     * Import from JSON\n     * @param {Element} parent Element to contain the imported element.\n     * @param {Object} data JSON data\n     * @return {Element} Imported element.\n     */\n    function importFromJson(parent, data) {\n        var elem = null;\n        if (data) {\n            try {\n                elem = Repository.readObject(data);\n                // Bypass Operation (for insert elem to parent)\n                OperationBuilder.begin(\"\", true);\n                OperationBuilder.fieldInsert(parent, \"ownedElements\", elem);\n                OperationBuilder.fieldAssign(elem, \"_parent\", parent);\n                OperationBuilder.end();\n                Repository.doOperation(OperationBuilder.getOperation());\n                Repository.setModified(true);\n                $(exports).triggerHandler('imported', [null, elem]);\n            } catch (err) {\n                console.error(err);\n            }\n        }\n        return elem;\n    }\n\n    /**\n     * Export model fragment to a file.\n     * @param {Core.Element} elem Element to be exported.\n     * @param {string} fullPath\n     * @return {$.Promise} JQuerty Promise Object\n     */\n    function exportToFile(elem, fullPath) {\n        var file = FileSystem.getFileForPath(fullPath),\n            result = new $.Deferred(),\n            data = Repository.writeObject(elem);\n        FileUtils.writeText(file, data, true)\n            .done(function () {\n                try {\n                    $(exports).triggerHandler('exported', [fullPath, elem]);\n                } catch (err) {\n                    console.error(err);\n                }\n                result.resolve(fullPath);\n            })\n            .fail(function (err) {\n                result.reject(err);\n            });\n        return result.promise();\n    }\n\n    /**\n     * Return current filename.\n     * @return {string}\n     */\n    function getFilename() {\n        return _filename;\n    }\n\n    exports.getProject          = getProject;\n    exports.newProject          = newProject;\n    exports.closeProject        = closeProject;\n    exports.save                = save;\n    exports.load                = load;\n    exports.loadAsTemplate      = loadAsTemplate;\n    exports.loadFromJson        = loadFromJson;\n    exports.importFromFile      = importFromFile;\n    exports.importFromJson      = importFromJson;\n    exports.exportToFile        = exportToFile;\n    exports.getFilename         = getFilename;\n    exports._setFilename = function (fullPath) { _filename = fullPath; };  // for test\n\n});\n\n//# sourceURL=/engine/ProjectManager.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, _, $, window, btoa, atob*/\n\ndefine(\'utils/Helper\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n\n    /**\n     * 난수 생성을 위한 Counter\n     */\n    var idGeneratorCounter = Math.floor(Math.random() * 65536);\n\n\n    /**\n     * 난수 생성기\n     */\n    var _idGenerator = null;\n\n\n    /**\n     * Define atob, btoa functions\n     */\n    if (!window.atob) {\n        var tableStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";\n        var table = tableStr.split("");\n\n        window.atob = function (base64) {\n            if (/(=[^=]+|={3,})$/.test(base64)) throw new Error("String contains an invalid character");\n            base64 = base64.replace(/=/g, "");\n            var n = base64.length & 3;\n            if (n === 1) throw new Error("String contains an invalid character");\n            for (var i = 0, j = 0, len = base64.length / 4, bin = []; i < len; ++i) {\n                var a = tableStr.indexOf(base64[j++] || "A"),\n                    b = tableStr.indexOf(base64[j++] || "A");\n                var c = tableStr.indexOf(base64[j++] || "A"),\n                    d = tableStr.indexOf(base64[j++] || "A");\n                if ((a | b | c | d) < 0) throw new Error("String contains an invalid character");\n                bin[bin.length] = ((a << 2) | (b >> 4)) & 255;\n                bin[bin.length] = ((b << 4) | (c >> 2)) & 255;\n                bin[bin.length] = ((c << 6) | d) & 255;\n            }\n            return String.fromCharCode.apply(null, bin).substr(0, bin.length + n - 4);\n        };\n\n        window.btoa = function (bin) {\n            for (var i = 0, j = 0, len = bin.length / 3, base64 = []; i < len; ++i) {\n                var a = bin.charCodeAt(j++),\n                    b = bin.charCodeAt(j++),\n                    c = bin.charCodeAt(j++);\n                if ((a | b | c) > 255) throw new Error("String contains an invalid character");\n                base64[base64.length] = table[a >> 2] + table[((a << 4) & 63) | (b >> 4)] +\n                    (isNaN(b) ? "=" : table[((b << 2) & 63) | (c >> 6)]) +\n                    (isNaN(b + c) ? "=" : table[c & 63]);\n            }\n            return base64.join("");\n        };\n    }\n\n    /**\n     * hexToBase64\n     * @private\n     */\n    function hexToBase64(str) {\n        return btoa(String.fromCharCode.apply(null,\n            str.replace(/\\r|\\n/g, "").replace(/([\\da-fA-F]{2}) ?/g, "0x$1 ").replace(/ +$/, "").split(" ")));\n    }\n\n    /**\n     * base64ToHex\n     * @private\n     */\n    function base64ToHex(str) {\n        for (var i = 0, bin = atob(str.replace(/[ \\r\\n]+$/, "")), hex = []; i < bin.length; ++i) {\n            var tmp = bin.charCodeAt(i).toString(16);\n            if (tmp.length === 1) tmp = "0" + tmp;\n            hex[hex.length] = tmp;\n        }\n        return hex.join(" ");\n    }\n\n    /**\n     * IdGenerator\n     *\n     * @constructor\n     */\n    function IdGenerator() {\n\n    }\n\n    IdGenerator.prototype.setBaseHex = function (val) {\n        return this.baseHex = val;\n    };\n\n    IdGenerator.prototype.toHex = function (digit, num) {\n        var fill, hex, r;\n        hex = num.toString(16);\n        if (hex.length < digit) {\n            r = digit - hex.length;\n            fill = Array(r + 1).join("0");\n            hex = fill + hex;\n        }\n        return hex;\n    };\n\n    IdGenerator.prototype.generate = function () {\n        var base64, counter, counterHex, hex, random, randomHex, timestamp, timestampHex;\n        timestamp = (new Date()).getTime();\n        counter = idGeneratorCounter;\n        idGeneratorCounter++;\n        if (idGeneratorCounter > 65535) {\n            idGeneratorCounter = 0;\n        }\n        random = Math.floor(Math.random() * 65536);\n        timestampHex = this.toHex(16, timestamp);\n        counterHex = this.toHex(4, counter);\n        randomHex = this.toHex(4, random);\n        hex = this.baseHex + timestampHex + counterHex + randomHex;\n        base64 = hexToBase64(hex);\n        return base64;\n    };\n\n    /**\n     *\n     *\n     */\n    function isElement(elem) {\n        if (_.isObject(elem) && (elem._id)) {\n            return true;\n        } else {\n            return false;\n        }\n    }\n\n    /**\n     * append items to array\n     *\n     * @param {Array} array\n     * @param {Array} items\n     */\n    function append(array, items) {\n        var item, _i, _len, _results;\n        _results = [];\n        for (_i = 0, _len = items.length; _i < _len; _i++) {\n            item = items[_i];\n            _results.push(array.push(item));\n        }\n        return _results;\n    }\n\n    /**\n     * Traverse all elements\n     */\n    function traverse(elem, fun) {\n        var item, name, val, _i, _len, _results, _results1;\n        if (isElement(elem)) {\n            fun(elem);\n            _results = [];\n            for (name in elem) {\n                val = elem[name];\n                _results.push(this.traverse(val, fun));\n            }\n            return _results;\n        } else if (_.isArray(elem)) {\n            _results1 = [];\n            for (_i = 0, _len = elem.length; _i < _len; _i++) {\n                item = elem[_i];\n                _results1.push(this.traverse(item, fun));\n            }\n            return _results1;\n        }\n    }\n\n    /**\n     * Return timestamp\n     */\n    function getTimestamp() {\n        return (new Date()).getTime();\n    }\n\n    /**\n     * Set GUID Base\n     */\n    function setGuidBase(baseHex) {\n        _idGenerator = new IdGenerator();\n        _idGenerator.setBaseHex(baseHex);\n    }\n\n    /**\n     * Set GUID Base as Number\n     */\n    function setGuidBaseNum(baseNum) {\n        _idGenerator = new IdGenerator();\n        _idGenerator.setBaseHex(_idGenerator.toHex(24, baseNum));\n    }\n\n    /**\n     * Generate GUID\n     */\n    function generateGuid() {\n        if (_idGenerator === null) {\n            _idGenerator = new IdGenerator("");\n        }\n        return _idGenerator.generate();\n    }\n\n    /**\n     * 여러 요소의 필드 값을 조사하여 하나의 값으로 리턴.\n     * (모두 값은 값인 경우 그 값을 리턴하고, 아닌 경우에는 null을 리턴)\n     * @param {Array.<Element>} elems\n     * @param {string} field - 요소의 필드명\n     */\n    function reduceValue(elems, field) {\n        // Collect values only if elem has the field.\n        var values = [];\n        _.each(elems, function (e) {\n            if (typeof e[field] !== "undefined") {\n                values.push(e[field]);\n            }\n        });\n        // if all values are same, then return that value, otherwise return null;\n        if (values.length > 0) {\n            var value = values[0];\n            for (var i = 0, len = values.length; i < len; i++) {\n                if (value !== values[i]) {\n                    return null;\n                }\n            }\n            return value;\n        } else {\n            return null;\n        }\n    }\n\n    /**\n     * 모든 요소들이 필드를 가지고 있으면 true 아니면 false를 리턴.\n     * @param {Array.<Element>} elems\n     * @param {string} field - 요소의 필드명\n     */\n    function reduceField(elems, field) {\n        if (elems && elems.length > 0) {\n            for (var i = 0, len = elems.length; i < len; i++) {\n                if (_.isUndefined(elems[i][field])) {\n                    return false;\n                }\n            }\n            return true;\n        }\n        return false;\n    }\n\n    /**\n     * 부모 요소에 크기를 맞춤.\n     *\n     * @param {jQueryObject} $elem\n     */\n    function fillParent(selector) {\n        $(selector).each(function (index) {\n            var $elem          = $(this),\n                $parent        = $elem.parent(),\n                widthGap       = $elem.outerWidth(true) - $elem.innerWidth(),\n                heightGap      = $elem.outerHeight(true) - $elem.innerHeight(),\n                siblingsHeight = 0;\n            $elem.siblings().each(function (index) {\n                siblingsHeight += Math.ceil($(this).outerHeight(true));\n            });\n            $elem.width($parent.innerWidth() - widthGap);\n            $elem.height($parent.innerHeight() - heightGap - siblingsHeight);\n        });\n    }\n\n    /**\n     * 문자열을 jQuery용 id에 적합한 문자열로 변환하여 돌려줌.\n     * @param {string} idStr\n     * @return {string}\n     */\n    function convertId(idStr) {\n        return idStr.replace(/\\./gi, "-");\n    }\n\n    /**\n     * Degree 값을 Radian 값으로 변환\n     * @param {number} degree\n     * @return {number} - radian\n     */\n    function toRadian(degree) {\n        return degree * Math.PI / 180;\n    }\n\n    /**\n     * Radian 값을 Degree 값으로 변환\n     * @param {number} radin\n     * @return {number} - degree\n     */\n    function toDegree(radian) {\n        return radian * (180 / Math.PI);\n    }\n\n    /**\n     * Assert 함수\n     * @param {boolean} condition\n     * @param {string} message\n     * @throws exception\n     */\n    function assert(condition, message) {\n        if (!condition) {\n            throw message || "Assertion failed";\n        }\n    }\n\n    // Public API\n    exports.isElement      = isElement;\n    exports.append         = append;\n    exports.traverse       = traverse;\n    exports.getTimestamp   = getTimestamp;\n    exports.setGuidBase    = setGuidBase;\n    exports.setGuidBaseNum = setGuidBaseNum;\n    exports.generateGuid   = generateGuid;\n    exports.reduceValue    = reduceValue;\n    exports.reduceField    = reduceField;\n    exports.fillParent     = fillParent;\n    exports.convertId      = convertId;\n    exports.toRadian       = toRadian;\n    exports.toDegree       = toDegree;\n    exports.assert         = assert;\n\n});\n\n//# sourceURL=/utils/Helper.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, loopfunc: true */\n/*global define, $, _, window, appshell */\n\n/**\n * Engine\n */\ndefine(\'engine/Engine\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/Helper\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Repository\',\'core/OperationBuilder\'],function (require, exports, module) {\n    "use strict";\n\n    var global            = require("core/Global").global,\n        Helper            = require("utils/Helper"),\n        Core              = require("core/Core"),\n        Project           = require("core/Core").Project,\n        Reader            = require("core/Core").Reader,\n        Writer            = require("core/Core").Writer,\n        Const             = require("core/Core").Const,\n        Cursor            = require("core/Core").Cursor,\n        MouseEvent        = require("core/Core").MouseEvent,\n        NodeView          = require("core/Core").NodeView,\n        EdgeView          = require("core/Core").EdgeView,\n        LabelView         = require("core/Core").LabelView,\n        ParasiticView     = require("core/Core").ParasiticView,\n        NodeParasiticView = require("core/Core").NodeParasiticView,\n        EdgeParasiticView = require("core/Core").EdgeParasiticView,\n        EdgeLabelView     = require("core/Core").EdgeLabelView,\n        Repository        = require("core/Repository"),\n        OperationBuilder  = require("core/OperationBuilder");\n\n\n    /**\n     * Add a model element\n     *\n     * @param {Model} parent\n     * @param {string} field\n     * @param {Model} model - model element to be added\n     * @return {Model}\n     */\n    function addModel(parent, field, model) {\n        if (!parent || !field || !model) {\n            console.error("Engine.addModel(): missing required parameters: parent, field, or model.");\n            return null;\n        }\n        if (!parent[field]) {\n            console.error("Engine.addModel(): " + field + " is not defined in parent parameter.");\n            return null;\n        }\n        if (parent !== null) {\n            OperationBuilder.begin("add model");\n            model._parent = parent;\n            OperationBuilder.insert(model);\n            OperationBuilder.fieldInsert(parent, field, model);\n            OperationBuilder.end();\n            var cmd = OperationBuilder.getOperation();\n            Repository.doOperation(cmd);\n        }\n        if (model) {\n            return Repository.get(model._id);\n        }\n    }\n\n    /**\n     * Add view elements.\n     *\n     * @param {diagram} diagram Diagram where added views to be placed.\n     * @param {Array.<Core.View>} views Views to be added.\n     */\n    function addViews(diagram, views) {\n        if (!diagram || !views) {\n            console.error("Engine.addViews(): missing required parameters: diagram or views");\n            return null;\n        }\n\n        OperationBuilder.begin(\'add views\');\n        for (var i = 0, len = views.length; i < len; i++) {\n            var view = views[i];\n            view._parent = diagram;\n            OperationBuilder.insert(view);\n            OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n    /**\n     * Add a model element and a view element.\n     *\n     * @param {Diagram} diagram Diagram where added views to be placed.\n     * @param {Model} model Model element to be added.\n     * @param {View} view View element to be added.\n     * @param {Model} parent Parent element to contain the model element\n     * @param {?string} parentField Array field name of the parent (default = \'ownedElements\')\n     * @param {?View} containerView Container view where the view element to be contained (optional)\n     */\n    function addModelAndView(diagram, model, view, parent, parentField, containerView) {\n        if (!diagram || !model || !view || !parent) {\n            console.error("Engine.addModelAndView(): missing required parameters: diagram, model, view or parent");\n            return null;\n        }\n\n        var field = \'ownedElements\';\n        if (parentField) {\n            field = parentField;\n        }\n\n        if (!parent[field]) {\n            console.error("Engine.addModelAndView(): " + field + " is not defined in parent parameter.");\n            return null;\n        }\n\n        OperationBuilder.begin(\'add element\');\n        // model\n        model._parent = parent;\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, field, model);\n        // view\n        view.model = model;\n        view._parent = diagram;\n        if (containerView) {\n            view.containerView = containerView;\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n        if (containerView) {\n            OperationBuilder.fieldInsert(containerView, \'containedViews\', view);\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n\n        if (view) {\n            return Repository.get(view._id);\n        }\n    }\n\n    /**\n     * Set value to a property of an element.\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {?} value\n     */\n    function setProperty(elem, field, value) {\n        if (!elem || !field) {\n            console.error("Engine.setProperty(): missing required parameters: elem or field");\n            return null;\n        }\n\n        if (typeof elem[field] === "undefined") {\n            console.error("Engine.setProperty(): " + field + " is not defined in elem parameter.");\n            return null;\n        }\n\n        OperationBuilder.begin(\'change property\');\n        OperationBuilder.fieldAssign(elem, field, value);\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n    /**\n     * Set values to multiple properties of an elements.\n     *\n     * @param {Element} elem\n     * @param {Object.<{string,?}>} fieldValueMap\n     */\n    function setProperties(elem, fieldValueMap) {\n        if (!elem || !fieldValueMap) {\n            console.error("Engine.setProperties(): missing required parameters: elem or fieldValueMap");\n            return null;\n        }\n        OperationBuilder.begin(\'change properties\');\n        var field, value;\n        for (field in fieldValueMap) {\n            if (fieldValueMap.hasOwnProperty(field)) {\n                value = fieldValueMap[field];\n                OperationBuilder.fieldAssign(elem, field, value);\n            }\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n    /**\n     * Set value to a property of multiple elements\n     *\n     * @param {Array.<View>} views\n     * @param {string} field\n     * @param {?} value\n     */\n    function setElemsProperty(elems, field, value) {\n        if (!elems || !field) {\n            console.error("Engine.setElemsProperty(): missing required parameters: elems or field");\n            return null;\n        }\n\n        OperationBuilder.begin(\'change property\');\n        for (var i = 0, len = elems.length; i < len; i++) {\n            var elem = elems[i];\n            if (!_.isUndefined(elem[field])) {\n                OperationBuilder.fieldAssign(elem, field, value);\n            }\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n    /**\n     * Determine a set of elements to be deleted.\n     * @private\n     * @param {Array.<Model>} models A set of model elements to be deleted.\n     * @param {Array.<View>} views A set of view elements to be deleted.\n     */\n    function _determineDeletingElements(models, views) {\n\n        var changed = false;\n\n        // 1) exclude undeletable models\n        _.each(models, function (m) {\n            if (m instanceof Core.Project) {\n                models.remove(m);\n            }\n        });\n        do {\n            changed = false;\n            // 2) include models owned by\n            _.each(models, function (m) {\n                _.each(m.getChildren(), function (child) {\n                    if (!_.contains(models, child)) {\n                        models.push(child);\n                        changed = true;\n                    }\n                });\n            });\n            // 3) include relations of\n            _.each(models, function (m) {\n                _.each(Repository.getRelationshipsOf(m), function (rel) {\n                    if (!_.contains(models, rel)) {\n                        models.push(rel);\n                        changed = true;\n                    }\n                });\n            });\n            // 4) include views of\n            _.each(models, function (m) {\n                _.each(Repository.getViewsOf(m), function (v) {\n                    if (!_.contains(views, v)) {\n                        views.push(v);\n                        changed = true;\n                    }\n                });\n            });\n            // 5) include contained views of\n            _.each(views, function (v) {\n                _.each(v.containedViews, function (cv) {\n                    if (!_.contains(views, cv)) {\n                        views.push(cv);\n                        changed = true;\n                    }\n                });\n            });\n            // 6) include subviews of\n            _.each(views, function (v) {\n                _.each(v.subViews, function (sub) {\n                    if (!_.contains(views, sub)) {\n                        views.push(sub);\n                        changed = true;\n                    }\n                });\n            });\n            // 7) include edge views of\n            _.each(views, function (v) {\n                _.each(Repository.getEdgeViewsOf(v), function (edge) {\n                    if (!_.contains(views, edge)) {\n                        views.push(edge);\n                        changed = true;\n                    }\n                });\n            });\n            // 8) include parasitic views of\n            _.each(views, function (v) {\n                var _parasiticViews = Repository.getRefsTo(v, function (_refTo) {\n                    return ((_refTo instanceof type.NodeParasiticView) && (_refTo.hostNode === v)) ||\n                           ((_refTo instanceof type.EdgeParasiticView) && (_refTo.hostEdge === v));\n                });\n                _.each(_parasiticViews, function (parasiticView) {\n                    if (!_.contains(views, parasiticView)) {\n                        views.push(parasiticView);\n                        changed = true;\n                    }\n                });\n            });\n            // 9) customizable deleting models and views (plug-in dependent)\n            //    e.g) _includeRelatedViewsOf(data);\n            //    e.g.) _includeModelsThatShouldBeDeletedWithViews(data);\n        } while (changed);\n    }\n\n    /**\n     * Determine elements referencing to a particular set of elements.\n     *\n     * @private\n     * @param {Array.<Element>} elems A set of elements\n     * @return {Array.<Element>} Elements referencing to the set of elements\n     */\n    function _determineOutsideElements(elems) {\n        var outsides = [];\n        _.each(elems, function (e) {\n            _.each(Repository.getRefsTo(e), function (ref) {\n                if (!_.contains(outsides, ref) && !_.contains(elems, ref)) {\n                    outsides.push(ref);\n                }\n            });\n        });\n        return outsides;\n    }\n\n    /**\n     * Delete elements.\n     *\n     * @param {Array.<Model>} models Model elements to be deleted.\n     * @param {Array.<View>} views View elements to be deleted.\n     */\n    function deleteElements(models, views) {\n        if (!models || !views) {\n            console.error("Engine.deleteElements(): missing required parameters: models or views");\n            return null;\n        }\n\n        var ms = _.clone(models),\n            vs = _.clone(views);\n\n        // 1) determine deleting elements\n        _determineDeletingElements(ms, vs);\n\n        // 2) determine outside elements (which referencing deleting elements)\n        var all = _.union(ms, vs);\n        var outsides = _determineOutsideElements(all);\n\n        // 3) make delete operation and execute\n        OperationBuilder.begin(\'delete elements\');\n\n        // 3.1) - 외부로부터의 참조는 모두 제거하도록 한다.\n        _.each(outsides, function (elem) {\n            _.each(elem.getMetaAttributes(), function (attr) {\n                switch (attr.kind) {\n                case Core.ATTR_KIND_OBJ:\n                case Core.ATTR_KIND_REF:\n                    var ref = elem[attr.name];\n                    if (ref && _.contains(all, ref)) {\n                        OperationBuilder.fieldAssign(elem, attr.name, null);\n                    }\n                    break;\n                case Core.ATTR_KIND_OBJS:\n                case Core.ATTR_KIND_REFS:\n                    var refs = elem[attr.name];\n                    if (refs && refs.length > 0) {\n                        for (var i = 0; i < refs.length; i++) {\n                            var ref = refs[i];\n                            if (ref._id && _.contains(all, ref)) {\n                                OperationBuilder.fieldRemove(elem, attr.name, ref);\n                            }\n                        }\n                    }\n                    break;\n                }\n            });\n        });\n\n        // 3.2) - 중복 삭제 제거하기\n        // OperationBuilder의 remove는 모든 하위 요소들의 삭제들도 함께 제거하기 때문에\n        // 하위 요소들을 중복해서 삭제하지 않도록 하는것이 효율적이다.\n        var subElements = [];\n        _.each(all, function (e) {\n            var children = e.getChildren();\n            _.each(children, function (c) {\n                if (!_.contains(subElements, c)) {\n                    subElements.push(c);\n                }\n            });\n        });\n        for (var i = vs.length-1; i >= 0; i--) {\n            var v = vs[i];\n            if (_.contains(subElements, v)) {\n                vs.remove(v);\n            }\n        }\n        for (var i = ms.length-1; i >= 0; i--) {\n            var m = ms[i];\n            if (_.contains(subElements, m)) {\n                ms.remove(m);\n            }\n        }\n\n        // 3.3) - 뷰 요소들 삭제\n        _.each(vs, function (v) {\n            OperationBuilder.remove(v);\n        });\n\n        // 3.4) - 모델 요소들 삭제\n        _.each(ms, function (m) {\n            OperationBuilder.remove(m);\n        });\n        OperationBuilder.end();\n        Repository.doOperation(OperationBuilder.getOperation());\n    }\n\n    /**\n     * Add an item to a particular array field of an element.\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {Element} val\n     */\n    function addItem(elem, field, val) {\n        if (!elem || !field) {\n            console.error("Engine.addItem(): missing required parameters: elem or field");\n            return null;\n        }\n\n        if (_.isArray(elem[field])) {\n            OperationBuilder.begin("add item");\n            OperationBuilder.fieldInsert(elem, field, val);\n            OperationBuilder.end();\n            var cmd = OperationBuilder.getOperation();\n            Repository.doOperation(cmd);\n        }\n    }\n\n    /**\n     * Remove an item from a particular array field of an element.\n     *\n     * @param {Element} elem\n     * @param {string} field\n     * @param {Element} val\n     */\n    function removeItem(elem, field, val) {\n        if (!elem || !field) {\n            console.error("Engine.removeItem(): missing required parameters: elem or field");\n            return null;\n        }\n\n        if (_.isArray(elem[field])) {\n            OperationBuilder.begin("remove item");\n            OperationBuilder.fieldRemove(elem, field, val);\n            OperationBuilder.end();\n            var cmd = OperationBuilder.getOperation();\n            Repository.doOperation(cmd);\n        }\n    }\n\n    /**\n     * Reorder an item of array field of an element to index - 1.\n     *\n     * @param {Element} parent\n     * @param {string} field\n     * @param {Element} elem\n     */\n    function moveUp(parent, field, elem) {\n        if (!parent || !field) {\n            console.error("Engine.moveUp(): missing required parameters: parent or field");\n            return null;\n        }\n\n        // Find above index considering sort by type ordering.\n        var array = parent[field];\n        var sortedArray = _.sortBy(array, function (child, idx) {\n            return child.getOrdering(idx);\n        });\n        var indexInSortedArray = _.indexOf(sortedArray, elem);\n        var above, index = 0;\n        if (indexInSortedArray > 0) {\n            above = sortedArray[indexInSortedArray - 1];\n            index = _.indexOf(array, above);\n        }\n\n        // Make move up operation\n        if (index >= 0) {\n            OperationBuilder.begin("move up");\n            OperationBuilder.fieldReorder(parent, field, elem, index);\n            OperationBuilder.end();\n            var cmd = OperationBuilder.getOperation();\n            Repository.doOperation(cmd);\n        }\n    }\n\n    /**\n     * Reorder an item of array field of an element to index + 1.\n     *\n     * @param {Element} parent\n     * @param {string} field\n     * @param {Element} elem\n     */\n    function moveDown(parent, field, elem) {\n        if (!parent || !field) {\n            console.error("Engine.moveDown(): missing required parameters: parent or field");\n            return null;\n        }\n\n        // Find below index considering sort by type ordering.\n        var array = parent[field];\n        var sortedArray = _.sortBy(array, function (child, idx) {\n            return child.getOrdering(idx);\n        });\n        var indexInSortedArray = _.indexOf(sortedArray, elem);\n        var below, index = 0;\n        if (indexInSortedArray >= 0 && indexInSortedArray < sortedArray.length - 1) {\n            below = sortedArray[indexInSortedArray + 1];\n            index = _.indexOf(array, below);\n        }\n\n        // Make move down operation\n        if (index < array.length) {\n            OperationBuilder.begin("move down");\n            OperationBuilder.fieldReorder(parent, field, elem, index);\n            OperationBuilder.end();\n            var cmd = OperationBuilder.getOperation();\n            Repository.doOperation(cmd);\n        }\n    }\n\n    /**\n     * Relocate an element to a new owner (parent).\n     *\n     * @param {Element} elem\n     * @param {Element} newOwner\n     * @param {string} field\n     */\n    function relocate(elem, newOwner, field) {\n        if (!elem || !newOwner || !field) {\n            console.error("Engine.relocate(): missing required parameters: elem, newOwner or field");\n            return null;\n        }\n\n        var oldOwner = elem._parent;\n        if ((oldOwner !== newOwner) && oldOwner[field] && _.contains(oldOwner[field], elem) && newOwner[field]) {\n            OperationBuilder.begin("relocate");\n            OperationBuilder.fieldRelocate(elem, field, oldOwner, newOwner);\n            var cmd = OperationBuilder.getOperation();\n            Repository.doOperation(cmd);\n        }\n    }\n\n\n    /**\n     * Move views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {number} dx\n     * @param {number} dy\n     */\n    function moveViews(editor, views, dx, dy) {\n        if (!editor || !views) {\n            console.error("Engine.moveView(): missing required parameters: editor or views");\n            return null;\n        }\n\n        if (!_.isNumber(dx) || !_.isNumber(dy)) {\n            console.error("Engine.moveView(): dx and dy should be number type");\n            return null;\n        }\n\n        // 1. 이동할 뷰들을 결정\n        var moveViews = [], changed = false;\n        Helper.append(moveViews, views);\n        // 1.1) ContainedViews 포함하기\n        do {\n            changed = false;\n            _.each(moveViews, function (v) {\n                _.each(v.containedViews, function (v1) {\n                    if (!_.contains(moveViews, v1)) {\n                        moveViews.push(v1);\n                        changed = true;\n                    }\n                });\n                // subViews의 containedViews도 포함\n                _.each(v.subViews, function (v2) {\n                    _.each(v2.subViews, function (v3) {\n                        if (!_.contains(moveViews, v3)) {\n                            moveViews.push(v3);\n                            changed = true;\n                        }\n                    });\n                });\n            });\n        } while (changed);\n        // 1.2) Self-Linked EdgeViews들 포함하기\n        editor.diagram.traverse(function (v) {\n            if ((v instanceof EdgeView) && _.contains(moveViews, v.head) && (v.head === v.tail) && !_.contains(moveViews, v)) {\n                if (!_.contains(moveViews, v)) {\n                    moveViews.push(v);\n                }\n            }\n        });\n        // 1.3) Head 및 Tail 측이 모두 이동하는 경우, 해당 EdgeViews들 포함하기\n        editor.diagram.traverse(function (v) {\n            if ((v instanceof EdgeView) && _.contains(moveViews, v.head) && _.contains(moveViews, v.tail) && !_.contains(moveViews, v)) {\n                if (!_.contains(moveViews, v)) {\n                    moveViews.push(v);\n                }\n            }\n        });\n        // 2. 이동으로 인하여 변경될 수 있는 Views의 상태들 기억하기\n        var changes = [];\n        Helper.append(changes, moveViews);\n        editor.diagram.traverse(function (v) {\n            // 이동하는 Views들을 연결하는 Edge 뷰들도 변경 대상에 포함. (노드가 이동하면 엣지의 points들이 변경되기 때문)\n            if ((v instanceof EdgeView) && (_.contains(changes, v.head) || _.contains(changes, v.tail))) {\n                if (!_.contains(changes, v)) { changes.push(v); }\n                for (var i = 0, len = v.subViews.length; i < len; i++) {\n                    var sub = v.subViews[i];\n                    if (!_.contains(changes, v)) { changes.push(sub); }\n                }\n            }\n        });\n        // ContainedViews가 이동하면 ContainerView의 크기가 변한다.\n        for (var i = 0, len = moveViews.length; i < len; i++) {\n            var v = moveViews[i];\n            if ((!_.contains(changes, v.containerView)) && (v.containerView != null)) {\n                changes.push(v.containerView);\n            }\n        }\n        // 이동할 뷰들의 상태를 memento에 저장\n        var mementoArray = [];\n        for (var i = 0, len = changes.length; i < len; i++) {\n            var v = changes[i];\n            var memento = {};\n            v.assignTo(memento);\n            mementoArray.push(memento);\n        }\n        // 3. 뷰들을 이동\n        for (var i = 0, len = moveViews.length; i < len; i++) {\n            var v = moveViews[i];\n            v.move(editor.canvas, dx, dy);\n        }\n        // 모든 뷰들의 크기와 위치를 재계산함.\n        editor.repaint();\n        // 4. 이동하기전의 Views와 이동후의 Views의 diff를 수행하여 변경된 값들만 추출\n        var diffs = [];\n        for (var i = 0, len = changes.length; i < len; i++) {\n            var v = changes[i];\n            var ds = v.diff(mementoArray[i]);\n            Helper.append(diffs, ds);\n        }\n        // memento에 저장된 Views들의 상태를 복원\n        for (var i = 0, len = changes.length; i < len; i++) {\n            var v = changes[i];\n            v.assignFrom(mementoArray[i]);\n        }\n        // 5. diff의 결과를 바탕으로 operation를 구성하여 수행함.\n        OperationBuilder.begin(\'move views\');\n        for (var i = 0, len = diffs.length; i < len; i++) {\n            var d = diffs[i];\n            OperationBuilder.fieldAssign(d.elem, d.f, d.n);\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Move a parasitic view.\n     *\n     * @param {Editor} editor\n     * @param {ParasiticView} view\n     * @param {number} alpha\n     * @param {number} distance\n     */\n    function moveParasiticView(editor, view, alpha, distance) {\n        if (!editor || !view) {\n            console.error("Engine.moveParasiticView(): missing required parameters: editor or view");\n            return null;\n        }\n\n        if (!_.isNumber(alpha) || !_.isNumber(distance)) {\n            console.error("Engine.moveParasiticView(): alpha and distance should be number type");\n            return null;\n        }\n\n        var memento = {};\n        view.assignTo(memento);\n        view.alpha = alpha;\n        view.distance = distance;\n        editor.repaint();\n        var diffs = view.diff(memento);\n        OperationBuilder.begin(\'move parasitic view\');\n        for (var i = 0, len = diffs.length; i < len; i++) {\n            var d = diffs[i];\n            OperationBuilder.fieldAssign(d.elem, d.f, d.n);\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Resize a node view.\n     *\n     * @param {Editor} editor\n     * @param {NodeView} node\n     * @param {number} left\n     * @param {number} top\n     * @param {number} right\n     * @param {number} bottom\n     */\n    function resizeNode(editor, node, left, top, right, bottom) {\n        if (!editor || !node) {\n            console.error("Engine.resizeNode(): missing required parameters: editor or node");\n            return null;\n        }\n\n        if (!_.isNumber(left) || !_.isNumber(top) || !_.isNumber(right) || !_.isNumber(bottom)) {\n            console.error("Engine.resizeNode(): left, top, right and bottom should be number type");\n            return null;\n        }\n\n        var changes, cmd, d, diffs, ds, i, memento, mementoArray, v, _i, _j, _k, _l, _len, _len1, _len2, _len3;\n        changes = [node];\n        editor.diagram.traverse(function (v) {\n            var sub, _i, _len, _ref, _results;\n            if ((v instanceof EdgeView) && ((v.head === node) || (v.tail === node))) {\n                changes.push(v);\n                _ref = v.subViews;\n                _results = [];\n                for (_i = 0, _len = _ref.length; _i < _len; _i++) {\n                    sub = _ref[_i];\n                    _results.push(changes.push(sub));\n                }\n                return _results;\n            }\n        });\n        mementoArray = [];\n        for (_i = 0, _len = changes.length; _i < _len; _i++) {\n            v = changes[_i];\n            memento = {};\n            v.assignTo(memento);\n            mementoArray.push(memento);\n        }\n        node.left = left;\n        node.top = top;\n        node.setRight(right);\n        node.setBottom(bottom);\n        editor.repaint();\n        diffs = [];\n        for (i = _j = 0, _len1 = changes.length; _j < _len1; i = ++_j) {\n            v = changes[i];\n            ds = v.diff(mementoArray[i]);\n            Helper.append(diffs, ds);\n        }\n        for (i = _k = 0, _len2 = changes.length; _k < _len2; i = ++_k) {\n            v = changes[i];\n            v.assignFrom(mementoArray[i]);\n        }\n        OperationBuilder.begin(\'resize node\');\n        for (_l = 0, _len3 = diffs.length; _l < _len3; _l++) {\n            d = diffs[_l];\n            OperationBuilder.fieldAssign(d.elem, d.f, d.n);\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Modify points of an edge view.\n     *\n     * @param {Editor} editor\n     * @param {EdgeView} edge\n     * @param {Points} points\n     */\n    function modifyEdge(editor, edge, points) {\n        if (!editor || !edge || !points) {\n            console.error("Engine.modifyEdge(): missing required parameters: editor, edge, or points");\n            return null;\n        }\n\n        var changes = [edge];\n        for (var i = 0, len = edge.subViews.length; i < len; i++) {\n            var sub = edge.subViews[i];\n            changes.push(sub);\n        }\n        var mementoArray = [];\n        for (var j = 0, len1 = changes.length; j < len1; j++) {\n            var v = changes[j];\n            var memento = {};\n            v.assignTo(memento);\n            mementoArray.push(memento);\n        }\n        edge.points.assign(points);\n        editor.repaint();\n        var diffs = [];\n        for (var i = 0, len2 = changes.length; i < len2; i++) {\n            var v = changes[i];\n            var ds = v.diff(mementoArray[i]);\n            Helper.append(diffs, ds);\n        }\n        for (var i = 0, len3 = changes.length; i < len3; i++) {\n            var v = changes[i];\n            v.assignFrom(mementoArray[i]);\n        }\n        OperationBuilder.begin(\'modify edge\');\n        for (var m = 0, len4 = diffs.length; m < len4; m++) {\n            var d = diffs[m];\n            OperationBuilder.fieldAssign(d.elem, d.f, d.n);\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Reconnect an end (head-side or tail-side) of edge view to new element.\n     *\n     * @param {Editor} editor\n     * @param {EdgeView} edge\n     * @param {Points} points\n     * @param {NodeView} newParticipant\n     * @param {boolean} isTailSide\n     */\n    function reconnectEdge(editor, edge, points, newParticipant, isTailSide) {\n        if (!editor || !edge || !points || !newParticipant) {\n            console.error("Engine.reconnectEdge(): missing required parameters: editor, edge, points, or newParticipant");\n            return null;\n        }\n\n        var changes = [edge];\n        for (var i = 0, len = edge.subViews.length; i < len; i++) {\n            var sub = edge.subViews[i];\n            changes.push(sub);\n        }\n        var mementoArray = [];\n        for (var j = 0, len1 = changes.length; j < len1; j++) {\n            var v = changes[j];\n            var memento = {};\n            v.assignTo(memento);\n            mementoArray.push(memento);\n        }\n        edge.points.assign(points);\n        editor.repaint();\n        var diffs = [];\n        for (var i = 0, len2 = changes.length; i < len2; i++) {\n            var v = changes[i];\n            var ds = v.diff(mementoArray[i]);\n            Helper.append(diffs, ds);\n        }\n        for (var i = 0, len3 = changes.length; i < len3; i++) {\n            var v = changes[i];\n            v.assignFrom(mementoArray[i]);\n        }\n        OperationBuilder.begin(\'reconnect edge\');\n        for (var m = 0, len4 = diffs.length; m < len4; m++) {\n            var d = diffs[m];\n            OperationBuilder.fieldAssign(d.elem, d.f, d.n);\n        }\n        // reconnect model\'s field\n        if (isTailSide === true) {\n            OperationBuilder.fieldAssign(edge, \'tail\', newParticipant);\n        } else {\n            OperationBuilder.fieldAssign(edge, \'head\', newParticipant);\n        }\n        if (edge.model && newParticipant.model) {\n            if (edge.model instanceof Core.DirectedRelationship) {\n                if (isTailSide === true) {\n                    OperationBuilder.fieldAssign(edge.model, \'source\', newParticipant.model);\n                } else {\n                    OperationBuilder.fieldAssign(edge.model, \'target\', newParticipant.model);\n                }\n            } else if (edge.model instanceof Core.UndirectedRelationship) {\n                if (isTailSide === true) {\n                    OperationBuilder.fieldAssign(edge.model.end1, \'reference\', newParticipant.model);\n                } else {\n                    OperationBuilder.fieldAssign(edge.model.end2, \'reference\', newParticipant.model);\n                }\n            }\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n    /**\n     * Move views by changing their container view.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {number} dx\n     * @param {number} dy\n     * @param {View} containerView\n     */\n    function moveViewsChangingContainer(editor, views, dx, dy, containerView, containerModel) {\n        if (!editor || !views) {\n            console.error("Engine.moveViewsChangingContainer(): missing required parameters: editor, views");\n            return null;\n        }\n\n        if (!_.isNumber(dx) || !_.isNumber(dy)) {\n            console.error("Engine.moveViewsChangingContainer(): dx and dy should be number type");\n            return null;\n        }\n\n        // 1. 이동할 뷰들을 결정\n        var moveViews = [];\n        Helper.append(moveViews, views);\n        // 1.1) ContainedViews 포함하기\n        for (var i = 0, len = moveViews.length; i < len; i++) {\n            var v = moveViews[i];\n            v.traverseField(\'containedViews\', function (view) {\n                if (!_.contains(moveViews, view)) {\n                    moveViews.push(view);\n                }\n            });\n            // subViews의 containedViews도 포함\n            v.traverseField(\'subViews\', function (v2) {\n                v2.traverseField(\'containedViews\', function (v3) {\n                    if (!_.contains(moveViews, v3)) {\n                        moveViews.push(v3);\n                    }\n                });\n            });\n        }\n        // 1.2) Self-Linked EdgeViews들 포함하기\n        editor.diagram.traverse(function (v) {\n            if ((v instanceof EdgeView) && _.contains(moveViews, v.head) && (v.head === v.tail) && !_.contains(moveViews, v)) {\n                if (!_.contains(moveViews, v)) {\n                    moveViews.push(v);\n                }\n            }\n        });\n        // 2. 이동으로 인하여 변경될 수 있는 Views의 상태들 기억하기\n        var changes = [];\n        Helper.append(changes, moveViews);\n        editor.diagram.traverse(function (v) {\n            // 이동하는 Views들을 연결하는 Edge 뷰들도 변경 대상에 포함. (노드가 이동하면 엣지의 points들이 변경되기 때문)\n            if ((v instanceof EdgeView) && (_.contains(changes, v.head) || _.contains(changes, v.tail))) {\n                if (!_.contains(changes, v)) { changes.push(v); }\n                for (var i = 0, len = v.subViews.length; i < len; i++) {\n                    var sub = v.subViews[i];\n                    if (!_.contains(changes, v)) { changes.push(sub); }\n                }\n            }\n        });\n        // ContainedViews가 이동하면 ContainerView의 크기가 변한다.\n        for (var i = 0, len = moveViews.length; i < len; i++) {\n            var v = moveViews[i];\n            if ((!_.contains(changes, v.containerView)) && (v.containerView != null)) {\n                changes.push(v.containerView);\n            }\n        }\n        // 이동할 뷰들의 상태를 memento에 저장\n        var mementoArray = [];\n        for (var i = 0, len = changes.length; i < len; i++) {\n            var v = changes[i];\n            var memento = {};\n            v.assignTo(memento);\n            mementoArray.push(memento);\n        }\n        // 3. 뷰들을 이동\n        for (var i = 0, len = moveViews.length; i < len; i++) {\n            var v = moveViews[i];\n            v.move(editor.canvas, dx, dy);\n        }\n        // 모든 뷰들의 크기와 위치를 재계산함.\n        editor.repaint();\n        // 4. 이동하기전의 Views와 이동후의 Views의 diff를 수행하여 변경된 값들만 추출\n        var diffs = [];\n        for (var i = 0, len = changes.length; i < len; i++) {\n            var v = changes[i];\n            var ds = v.diff(mementoArray[i]);\n            Helper.append(diffs, ds);\n        }\n        // memento에 저장된 Views들의 상태를 복원\n        for (var i = 0, len = changes.length; i < len; i++) {\n            var v = changes[i];\n            v.assignFrom(mementoArray[i]);\n        }\n        // 5. diff의 결과를 바탕으로 operation를 구성하여 수행함.\n        OperationBuilder.begin(\'move views\');\n        for (var i = 0, len = diffs.length; i < len; i++) {\n            var d = diffs[i];\n            OperationBuilder.fieldAssign(d.elem, d.f, d.n);\n        }\n\n        // 6. containerView, _parent를 변경\n        for (var i = 0, len = views.length; i < len; i++) {\n            var v = views[i];\n            // v.containerView를 변경\n            if (v.containerView !== null) {\n                OperationBuilder.fieldRemove(v.containerView, \'containedViews\', v);\n            }\n            OperationBuilder.fieldAssign(v, \'containerView\', containerView);\n            if (containerView !== null) {\n                OperationBuilder.fieldInsert(containerView, \'containedViews\', v);\n            }\n            // v를 containerView보다 나중에 그려지도록 diagram의 가장 뒤로 index를 변경\n            var newIdx = v.getDiagram().ownedViews.length - 1;\n            OperationBuilder.fieldReorder(v.getDiagram(), \'ownedViews\', v, newIdx);\n            // v.model을 containerModel로 relocate\n            var m = v.model;\n            var field = m.getParentField();\n            if (field) {\n                OperationBuilder.fieldRelocate(m, field, m._parent, containerModel);\n            }\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change fill color of views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {string} color (color string in CSS style: e.g. "#ffffff")\n     */\n    function setFillColor(editor, views, color) {\n        if (!editor || !views || !color) {\n            console.error("Engine.setFillColor(): missing required parameters: editor, views, or color");\n            return null;\n        }\n\n        var cmd, v, _i, _len;\n        OperationBuilder.begin(\'change fill color\');\n        for (_i = 0, _len = views.length; _i < _len; _i++) {\n            v = views[_i];\n            OperationBuilder.fieldAssign(v, \'fillColor\', color);\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change line color of views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {string} color (color string in CSS style: e.g. "#ffffff")\n     */\n    function setLineColor(editor, views, color) {\n        if (!editor || !views || !color) {\n            console.error("Engine.setLineColor(): missing required parameters: editor, views, or color");\n            return null;\n        }\n\n        var cmd, v, _i, _len;\n        OperationBuilder.begin(\'change line color\');\n        for (_i = 0, _len = views.length; _i < _len; _i++) {\n            v = views[_i];\n            OperationBuilder.fieldAssign(v, \'lineColor\', color);\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change font of views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {string} face Font face (family)\n     * @param {number} size Font size (in pixel)\n     * @param {string} color Font color\n     */\n    function setFont(editor, views, face, size, color) {\n        if (!editor || !views || !face || !color) {\n            console.error("Engine.setFont(): missing required parameters: editor, views, face, or color");\n            return null;\n        }\n\n        if (!_.isNumber(size)) {\n            console.error("Engine.setFont(): size should be number type");\n            return null;\n        }\n\n        OperationBuilder.begin(\'change font\');\n        for (var i = 0, len = views.length; i < len; i++) {\n            var v = views[i];\n            if (face && size) {\n                OperationBuilder.fieldAssign(v, \'font\', face + ";" + size + ";?");\n            }\n            if (color) {\n                OperationBuilder.fieldAssign(v, \'fontColor\', color);\n            }\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change font color of views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {string} color Font color (color string in CSS style: e.g. "#ffffff")\n     */\n    function setFontColor(editor, views, color) {\n        if (!editor || !views || !color) {\n            console.error("Engine.setFontColor(): missing required parameters: editor, views, or color");\n            return null;\n        }\n\n        var cmd, v, _i, _len;\n        OperationBuilder.begin(\'change font color\');\n        for (_i = 0, _len = views.length; _i < _len; _i++) {\n            v = views[_i];\n            OperationBuilder.fieldAssign(v, \'fontColor\', color);\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change font face of views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {string} fontFace\n     */\n    function setFontFace(editor, views, fontFace) {\n        if (!editor || !views || !fontFace) {\n            console.error("Engine.setFontFace(): missing required parameters: editor, views, or fontFace");\n            return null;\n        }\n\n        var cmd, v, _i, _len;\n        OperationBuilder.begin(\'change font face\');\n        for (_i = 0, _len = views.length; _i < _len; _i++) {\n            v = views[_i];\n            OperationBuilder.fieldAssign(v, \'font\', fontFace + ";?;?");\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change font size of views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {number} fontSize\n     */\n    function setFontSize(editor, views, fontSize) {\n        if (!editor || !views) {\n            console.error("Engine.setFontSize(): missing required parameters: editor, or views");\n            return null;\n        }\n\n        if (!_.isNumber(fontSize)) {\n            console.error("Engine.setFontSize(): fontSize should be number type");\n            return null;\n        }\n\n        var cmd, v, _i, _len;\n        OperationBuilder.begin(\'change font size\');\n        for (_i = 0, _len = views.length; _i < _len; _i++) {\n            v = views[_i];\n            OperationBuilder.fieldAssign(v, \'font\', "?;" + fontSize + ";?");\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change stereotypeDisplay of views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {string} stereotypeDisplay\n     */\n    function setStereotypeDisplay(editor, views, stereotypeDisplay) {\n        if (!editor || !views || !stereotypeDisplay) {\n            console.error("Engine.setStereotypeDisplay(): missing required parameters: editor, views, or stereotypeDisplay");\n            return null;\n        }\n\n        var cmd, v, _i, _len;\n        OperationBuilder.begin(\'change stereotype display\');\n        for (_i = 0, _len = views.length; _i < _len; _i++) {\n            v = views[_i];\n            OperationBuilder.fieldAssign(v, \'stereotypeDisplay\', stereotypeDisplay);\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change lineStyle of edge views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {string} lineStyle\n     */\n    function setLineStyle(editor, views, lineStyle) {\n        if (!editor || !views) {\n            console.error("Engine.setLineStyle(): missing required parameters: editor, or views");\n            return null;\n        }\n\n        var changes, cmd, d, diffs, ds, i, memento, mementoArray, sub, v, _i, _j, _k, _l, _len, _len1, _len2, _len3, _len4, _len5, _len6, _m, _n, _o, _ref;\n        changes = [];\n        for (_i = 0, _len = views.length; _i < _len; _i++) {\n            v = views[_i];\n            if (v instanceof EdgeView) {\n                changes.push(v);\n                _ref = v.subViews;\n                for (_j = 0, _len1 = _ref.length; _j < _len1; _j++) {\n                    sub = _ref[_j];\n                    changes.push(sub);\n                }\n            }\n        }\n        mementoArray = [];\n        for (_k = 0, _len2 = changes.length; _k < _len2; _k++) {\n            v = changes[_k];\n            memento = {};\n            v.assignTo(memento);\n            mementoArray.push(memento);\n        }\n        for (_l = 0, _len3 = views.length; _l < _len3; _l++) {\n            v = views[_l];\n            if (v instanceof EdgeView) {\n                v.lineStyle = lineStyle;\n                v.arrange(editor.canvas);\n            }\n        }\n        editor.repaint();\n        diffs = [];\n        for (i = _m = 0, _len4 = changes.length; _m < _len4; i = ++_m) {\n            v = changes[i];\n            ds = v.diff(mementoArray[i]);\n            Helper.append(diffs, ds);\n        }\n        for (i = _n = 0, _len5 = changes.length; _n < _len5; i = ++_n) {\n            v = changes[i];\n            v.assignFrom(mementoArray[i]);\n        }\n        OperationBuilder.begin(\'change line style\');\n        for (_o = 0, _len6 = diffs.length; _o < _len6; _o++) {\n            d = diffs[_o];\n            OperationBuilder.fieldAssign(d.elem, d.f, d.n);\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Change autoResize of node views.\n     *\n     * @param {Editor} editor\n     * @param {Array.<View>} views\n     * @param {boolean} autoResize\n     */\n    function setAutoResize(editor, views, autoResize) {\n        if (!editor || !views) {\n            console.error("Engine.setAutoResize(): missing required parameters: editor, or views");\n            return null;\n        }\n\n        var cmd, v, _i, _len;\n        OperationBuilder.begin(\'change auto resize\');\n        for (_i = 0, _len = views.length; _i < _len; _i++) {\n            v = views[_i];\n            OperationBuilder.fieldAssign(v, \'autoResize\', autoResize);\n        }\n        OperationBuilder.end();\n        cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    };\n\n    /**\n     * Layout diagram\n     *\n     * @param {Editor} editor\n     * @param {Diagram} diagram\n     * @param {string} direction\n     * @param {{node:number, edge:number, rank:number}} separations\n     * @param {number} edgeLineStyle\n     */\n    function layoutDiagram(editor, diagram, direction, separations, edgeLineStyle) {\n        if (!editor || !diagram) {\n            console.error("Engine.layoutDiagram(): missing required parameters: editor, or diagram");\n            return null;\n        }\n\n        // store memento\n        var mementoArray = [];\n        for (var i = 0, len = diagram.ownedViews.length; i < len; i++) {\n            var v = diagram.ownedViews[i];\n            var memento = {};\n            v.assignTo(memento);\n            mementoArray.push(memento);\n        }\n        diagram.layout(direction, separations, edgeLineStyle);\n        editor.repaint();\n        // compute diffs\n        var diffs = [];\n        for (var i = 0, len = diagram.ownedViews.length; i < len; i++) {\n            var v = diagram.ownedViews[i];\n            var ds = v.diff(mementoArray[i]);\n            Helper.append(diffs, ds);\n            v.assignFrom(mementoArray[i]);\n        }\n        // execute as a operation\n        OperationBuilder.begin(\'layout diagram\');\n        for (var i = 0, len = diffs.length; i < len; i++) {\n            var d = diffs[i];\n            OperationBuilder.fieldAssign(d.elem, d.f, d.n);\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n    // Define public API\n    exports.addModel                   = addModel;\n    exports.addViews                   = addViews;\n    exports.addModelAndView            = addModelAndView;\n    exports.setProperty                = setProperty;\n    exports.setProperties              = setProperties;\n    exports.setElemsProperty           = setElemsProperty;\n    exports.deleteElements             = deleteElements;\n    exports.addItem                    = addItem;\n    exports.removeItem                 = removeItem;\n    exports.moveUp                     = moveUp;\n    exports.moveDown                   = moveDown;\n    exports.relocate                   = relocate;\n    exports.moveViews                  = moveViews;\n    exports.moveParasiticView          = moveParasiticView;\n    exports.resizeNode                 = resizeNode;\n    exports.modifyEdge                 = modifyEdge;\n    exports.reconnectEdge              = reconnectEdge;\n    exports.moveViewsChangingContainer = moveViewsChangingContainer;\n    exports.setFillColor               = setFillColor;\n    exports.setLineColor               = setLineColor;\n    exports.setFont                    = setFont;\n    exports.setFontColor               = setFontColor;\n    exports.setFontFace                = setFontFace;\n    exports.setFontSize                = setFontSize;\n    exports.setStereotypeDisplay       = setStereotypeDisplay;\n    exports.setLineStyle               = setLineStyle;\n    exports.setAutoResize              = setAutoResize;\n    exports.layoutDiagram              = layoutDiagram;\n\n});\n\n\n//# sourceURL=/engine/Engine.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _*/\n\n/**\n *\n * This module dispatches these events:\n * - selectionChanged: (selectedModels, selectedViews)\n */\ndefine('engine/SelectionManager',['require','exports','module','core/Global'],function (require, exports, module) {\n    \"use strict\";\n\n    var Global = require(\"core/Global\");\n\n    /**\n     * 선택된 모델 요소들\n     * @private\n     * @type {Array.<Model>}\n     */\n    var _selectedModels = [];\n\n    /**\n     * 선택된 뷰 요소들\n     * @private\n     * @type {Array.<View>}\n     */\n    var _selectedViews = [];\n\n    /**\n     * 활성화된 다이어그램\n     * @private\n     * @type {Diagram}\n     */\n    var _activeDiagram = null;\n\n    /**\n     * 요소 선택 이벤트 발생\n     * @private\n     */\n    function _triggerEvent() {\n        $(exports).triggerHandler('selectionChanged', [_selectedModels, _selectedViews]);\n    }\n\n    function _isChanged(models, views) {\n        models = models || [];\n        views = views || [];\n        var i, len, m, v;\n        for (i = 0, len = models.length; i < len; i++) {\n            m = models[i];\n            if (!_.contains(_selectedModels, m)) {\n                return true;\n            }\n        }\n        for (i = 0, len = _selectedModels.length; i < len; i++) {\n            m = _selectedModels[i];\n            if (!_.contains(models, m)) {\n                return true;\n            }\n        }\n        for (i = 0, len = views.length; i < len; i++) {\n            v = views[i];\n            if (!_.contains(_selectedViews, v)) {\n                return true;\n            }\n        }\n        for (i = 0, len = _selectedViews.length; i < len; i++) {\n            v = _selectedViews[i];\n            if (!_.contains(views, v)) {\n                return true;\n            }\n        }\n        return false;\n    }\n\n    /**\n     * Return selected model elements.\n     *\n     * @return {Array.<Model>} Array of selected model elements.\n     */\n    function getSelectedModels() {\n        return _selectedModels;\n    }\n\n    /**\n     * Return selected view elements.\n     *\n     * @return {Array.<View>} Array of selected view elements.\n     */\n    function getSelectedViews() {\n        return _selectedViews;\n    }\n\n    /**\n     * Deselect all elements.\n     */\n    function deselectAll() {\n        if (_isChanged([], [])) {\n            _selectedViews = [];\n            _selectedModels = [];\n            _triggerEvent();\n        }\n    }\n\n    /**\n     * Select elements (models and views)\n     * @param {Array.<Model>} models Model elements to be selected.\n     * @param {Array.<View>} views View elements to be selected.\n     */\n    function select(models, views) {\n        if (_isChanged(models, views)) {\n            _selectedModels = models;\n            _selectedViews = views;\n            if (!Array.isArray(_selectedModels)) {\n                _selectedModels = [];\n            }\n            if (!Array.isArray(_selectedViews)) {\n                _selectedViews = [];\n            }\n            _triggerEvent();\n        }\n    }\n\n    /**\n     * Select a model element\n     * @param {Model} model Model element to be selected.\n     */\n    function selectModel(model) {\n        if (_isChanged([model], [])) {\n            _selectedModels = [model];\n            _triggerEvent();\n        }\n    }\n\n    /**\n     * Select view elements\n     * @param {Array.<View>} views Array of view elements to be selected.\n     */\n    function selectViews(views) {\n        if ((views !== null) && views.length > 0) {\n            var _tempModels = [];\n            for (var i = 0, len = views.length; i < len; i++) {\n                var v = views[i];\n                if (v.model !== null) {\n                    _tempModels.push(v.model);\n                }\n            }\n            if (_isChanged(views, _tempModels)) {\n                _selectedViews = views;\n                _selectedModels = _tempModels;\n                _triggerEvent();\n            }\n        } else {\n            deselectAll();\n        }\n    }\n\n    /**\n     * Return the first selected model element.\n     */\n    function getSelected() {\n        return _selectedModels[0];\n    }\n\n    // Define public API\n    exports.getSelectedModels = getSelectedModels;\n    exports.getSelectedViews  = getSelectedViews;\n    exports.deselectAll       = deselectAll;\n    exports.select            = select;\n    exports.selectModel       = selectModel;\n    exports.selectViews       = selectViews;\n    exports.getSelected       = getSelected;\n\n});\n\n\n//# sourceURL=/engine/SelectionManager.js"),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, $ */\n\n /**\n  * Manages global application commands that can be called from menu items, key bindings, or subparts\n  * of the application.\n  *\n  * This module dispatches these event(s):\n  *    - commandRegistered  -- when a new command is registered\n  *    - beforeExecuteCommand -- before dispatching a command\n  */\ndefine(\'command/CommandManager\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n    /**\n     * Map of all registered global commands\n     * @type Object.<commandID: string, Command>\n     */\n    var _commands = {};\n\n    /**\n     * Temporary copy of commands map for restoring after testing\n     * TODO (issue #1039): implement separate require contexts for unit tests\n     * @type Object.<commandID: string, Command>\n     */\n    var _commandsOriginal = {};\n\n    /**\n     * @constructor\n     * @private\n     *\n     * @param {string} name - text that will be displayed in the UI to represent command\n     * @param {string} id\n     * @param {function} commandFn - the function that is called when the command is executed.\n     *\n     * TODO: where should this be triggered, The Command or Exports?\n     * Events:\n     *      enabledStateChange\n     *      checkedStateChange\n     *      visibleStateChange\n     *      keyBindingAdded\n     *      keyBindingRemoved\n     */\n    function Command(name, id, commandFn) {\n        this._name = name;\n        this._id = id;\n        this._commandFn = commandFn;\n        this._checked = undefined;\n        this._enabled = true;\n        this._visible = true;\n    }\n\n    /** @return {Command} */\n    Command.prototype.getID = function () {\n        return this._id;\n    };\n\n    /**\n     * Executes the command. Additional arguments are passed to the executing function\n     *\n     * @return {$.Promise} a jQuery promise that will be resolved when the command completes.\n     */\n    Command.prototype.execute = function () {\n        if (!this._enabled) {\n            return (new $.Deferred()).reject().promise();\n        }\n\n        var result = this._commandFn.apply(this, arguments);\n        if (!result) {\n            // If command does not return a promise, assume that it handled the\n            // command and return a resolved promise\n            return (new $.Deferred()).resolve().promise();\n        } else {\n            return result;\n        }\n    };\n\n    /** @return {boolean} */\n    Command.prototype.getEnabled = function () {\n        return this._enabled;\n    };\n\n    /**\n     * Sets enabled state of Command and dispatches "enabledStateChange"\n     * when the enabled state changes.\n     * @param {boolean} enabled\n     */\n    Command.prototype.setEnabled = function (enabled) {\n        var changed = this._enabled !== enabled;\n        this._enabled = enabled;\n\n        if (changed) {\n            $(this).triggerHandler("enabledStateChange");\n        }\n    };\n\n    /**\n     * Sets enabled state of Command and dispatches "checkedStateChange"\n     * when the enabled state changes.\n     * @param {boolean} checked\n     */\n    Command.prototype.setChecked = function (checked) {\n        var changed = this._checked !== checked;\n        this._checked = checked;\n\n        if (changed) {\n            $(this).triggerHandler("checkedStateChange");\n        }\n    };\n\n    /** @return {boolean} */\n    Command.prototype.getChecked = function () {\n        return this._checked;\n    };\n\n    /**\n     * Sets visible state of Command and dispatches "visibleStateChange"\n     * when the visible state changes.\n     * @param {boolean} visible\n     */\n    Command.prototype.setVisible = function (visible) {\n        var changed = this._visible !== visible;\n        this._visible = visible;\n\n        if (changed) {\n            $(this).triggerHandler("visibleStateChange");\n        }\n    };\n\n    /** @return {boolean} */\n    Command.prototype.getVisible = function () {\n        return this._visible;\n    };\n\n    /**\n     * Sets the name of the Command and dispatches "nameChange" so that\n     * UI that reflects the command name can update.\n     *\n     * Note, a Command name can appear in either HTML or native UI\n     * so HTML tags should not be used. To add a Unicode character,\n     * use \\uXXXX instead of an HTML entity.\n     *\n     * @param {string} name\n     */\n    Command.prototype.setName = function (name) {\n        var changed = this._name !== name;\n        this._name = name;\n\n        if (changed) {\n            $(this).triggerHandler("nameChange");\n        }\n    };\n\n    /** @return {string} */\n    Command.prototype.getName = function () {\n        return this._name;\n    };\n\n\n\n    /**\n     * Registers a global command.\n     * @param {string} name - text that will be displayed in the UI to represent command\n     * @param {string} id - unique identifier for command.\n     *      Core commands in app use a simple command title as an id, for example "open.file".\n     *      Extensions should use the following format: "author.myextension.mycommandname".\n     *      For example, "lschmitt.csswizard.format.css".\n     * @param {function(...)} commandFn - the function to call when the command is executed. Any arguments passed to\n     *     execute() (after the id) are passed as arguments to the function. If the function is asynchronous,\n     *     it must return a jQuery promise that is resolved when the command completes. Otherwise, the\n     *     CommandManager will assume it is synchronous, and return a promise that is already resolved.\n     * @return {?Command}\n     */\n    function register(name, id, commandFn) {\n        if (_commands[id]) {\n            console.log("Attempting to register an already-registered command: " + id);\n            return null;\n        }\n        if (!name || !id || !commandFn) {\n            console.error("Attempting to register a command with a missing name, id, or command function:" + name + " " + id);\n            return null;\n        }\n\n        var command = new Command(name, id, commandFn);\n        _commands[id] = command;\n\n        $(exports).triggerHandler("commandRegistered", [command]);\n\n        return command;\n    }\n\n    /**\n     * Registers a global internal only command.\n     * @param {string} id - unique identifier for command.\n     *      Core commands in app use a simple command title as an id, for example "app.abort_quit".\n     *      Extensions should use the following format: "author.myextension.mycommandname".\n     *      For example, "lschmitt.csswizard.format.css".\n     * @param {function(...)} commandFn - the function to call when the command is executed. Any arguments passed to\n     *     execute() (after the id) are passed as arguments to the function. If the function is asynchronous,\n     *     it must return a jQuery promise that is resolved when the command completes. Otherwise, the\n     *     CommandManager will assume it is synchronous, and return a promise that is already resolved.\n     * @return {?Command}\n     */\n    function registerInternal(id, commandFn) {\n        if (_commands[id]) {\n            console.log("Attempting to register an already-registered command: " + id);\n            return null;\n        }\n        if (!id || !commandFn) {\n            console.error("Attempting to register an internal command with a missing id, or command function: " + id);\n            return null;\n        }\n\n        var command = new Command(null, id, commandFn);\n        _commands[id] = command;\n\n        $(exports).triggerHandler("commandRegistered", [command]);\n\n        return command;\n    }\n\n    /**\n     * Clear all commands for unit testing, but first make copy of commands so that\n     * they can be restored afterward\n     */\n    function _testReset() {\n        _commandsOriginal = _commands;\n        _commands = {};\n    }\n\n    /**\n     * Restore original commands after test and release copy\n     */\n    function _testRestore() {\n        _commands = _commandsOriginal;\n        _commandsOriginal = {};\n    }\n\n    /**\n     * Retrieves a Command object by id\n     * @param {string} id\n     * @return {Command}\n     */\n    function get(id) {\n        return _commands[id];\n    }\n\n    /**\n     * Returns the ids of all registered commands\n     * @return {Array.<string>}\n     */\n    function getAll() {\n        return Object.keys(_commands);\n    }\n\n    /**\n     * Looks up and runs a global command. Additional arguments are passed to the command.\n     *\n     * @param {string} id The ID of the command to run.\n     * @return {$.Promise} a jQuery promise that will be resolved when the command completes.\n     */\n    function execute(id) {\n        var command = _commands[id];\n\n        if (command) {\n            try {\n                $(exports).triggerHandler("beforeExecuteCommand", id);\n            } catch (err) {\n                console.error(err);\n            }\n\n            return command.execute.apply(command, Array.prototype.slice.call(arguments, 1));\n        } else {\n            return (new $.Deferred()).reject().promise();\n        }\n    }\n\n    /**\n     * Blank handler\n     */\n    function doNothing() {\n    }\n\n    // Define public API\n    exports.register            = register;\n    exports.registerInternal    = registerInternal;\n    exports.execute             = execute;\n    exports.get                 = get;\n    exports.getAll              = getAll;\n    exports.doNothing           = doNothing;\n    exports._testReset          = _testReset;\n    exports._testRestore        = _testRestore;\n});\n\n//# sourceURL=/command/CommandManager.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*global define*/\n\n/**\n * List of common commands.\n *\n * ```\n *    exports.FILE                          = 'file';\n *    exports.FILE_NEW                      = 'file.new';\n *    exports.FILE_NEW_FROM_TEMPLATE        = 'file.newFromTemplate';\n *    exports.FILE_OPEN                     = 'file.open';\n *    exports.FILE_OPEN_RECENT              = 'file.openRecent';\n *    exports.FILE_SAVE                     = 'file.save';\n *    exports.FILE_SAVE_AS                  = 'file.saveAs';\n *    exports.FILE_CLOSE                    = 'file.close';\n *    exports.FILE_IMPORT                   = 'file.import';\n *    exports.FILE_IMPORT_FRAGMENT          = 'file.import.fragment';\n *    exports.FILE_EXPORT                   = 'file.export';\n *    exports.FILE_EXPORT_FRAGMENT          = 'file.export.fragment';\n *    exports.FILE_PREFERENCES              = 'file.preferences';\n *    exports.FILE_PRINT_TO_PDF             = 'file.printToPDF';\n *\n *    exports.FILE_CLOSE_WINDOW             = 'file.close_window';\n *    exports.FILE_QUIT                     = 'file.quit';\n *\n *    exports.EDIT                          = 'edit';\n *    exports.EDIT_UNDO                     = 'edit.undo';\n *    exports.EDIT_REDO                     = 'edit.redo';\n *    exports.EDIT_CUT                      = 'edit.cut';\n *    exports.EDIT_COPY                     = 'edit.copy';\n *    exports.EDIT_PASTE                    = 'edit.paste';\n *    exports.EDIT_SELECT_ALL               = 'edit.selectAll';\n *\n *    exports.EDIT_DELETE                   = 'edit.delete';\n *    exports.EDIT_DELETE_FROM_MODEL        = 'edit.deleteFromModel';\n *    exports.EDIT_MOVE_UP                  = 'edit.moveUp';\n *    exports.EDIT_MOVE_DOWN                = 'edit.moveDown';\n *    exports.EDIT_SELECT_IN_EXPLORER       = 'edit.selectInExplorer';\n *    exports.EDIT_SELECT_IN_DIAGRAM        = 'edit.selectInDiagram';\n *\n *    exports.FORMAT                        = 'format';\n *    exports.FORMAT_FONT                   = 'format.font';\n *    exports.FORMAT_FILL_COLOR             = 'format.fillColor';\n *    exports.FORMAT_LINE_COLOR             = 'format.lineColor';\n *    exports.FORMAT_LINE_STYLE             = 'format.linestyle';\n *    exports.FORMAT_LINE_STYLE_RECTILINEAR = 'format.linestyle.rectilinear';\n *    exports.FORMAT_LINE_STYLE_OBLIQUE     = 'format.linestyle.oblique';\n *    exports.FORMAT_LINE_STYLE_ROUNDRECT   = 'format.linestyle.roundRect';\n *    exports.FORMAT_LINE_STYLE_CURVE       = 'format.linestyle.curve';\n *    exports.FORMAT_AUTO_RESIZE            = 'format.autoResize';\n *    exports.FORMAT_SHOW_SHADOW            = 'format.showShadow';\n *\n *    exports.MODEL                         = 'model';\n *\n *    exports.TOOLS                         = 'tools';\n *    exports.TOOLS_EXTENSION_MANAGER       = 'tools.extensionManager';\n *\n *    exports.VIEW                          = 'view';\n *    exports.VIEW_CLOSE_DIAGRAM            = 'view.closeDiagram';\n *    exports.VIEW_CLOSE_OTHER_DIAGRAMS     = 'view.closeOtherDiagrams';\n *    exports.VIEW_CLOSE_ALL_DIAGRAMS       = 'view.closeAllDiagrams';\n *    exports.VIEW_NEXT_DIAGRAM             = 'view.nextDiagram';\n *    exports.VIEW_PREVIOUS_DIAGRAM         = 'view.previousDiagram';\n *    exports.VIEW_ZOOM_IN                  = 'view.zoomIn';\n *    exports.VIEW_ZOOM_OUT                 = 'view.zoomOut';\n *    exports.VIEW_ACTUAL_SIZE              = 'view.actualSize';\n *    exports.VIEW_FIT_TO_WINDOW            = 'view.fitToWindow';\n *    exports.VIEW_SHOW_GRID                = \"view.showGrid\";\n *    exports.VIEW_HIDE_SIDEBAR             = 'view.hideSidebar';\n *    exports.VIEW_HIDE_NAVIGATOR           = 'view.hideNavigator';\n *    exports.VIEW_HIDE_TOOLBAR             = 'view.hideToolbar';\n *    exports.VIEW_HIDE_STATUSBAR           = 'view.hideStatusBar';\n *    exports.VIEW_HIDE_TOOLBOX             = 'view.hideToolbox';\n *    exports.VIEW_HIDE_EDITORS             = 'view.hideEditors';\n *\n *    exports.HELP                          = 'help';\n *    exports.HELP_ABOUT                    = 'help.about';\n *    exports.HELP_CHECK_FOR_UPDATES        = 'help.checkForUpdates';\n *    exports.HELP_ENTER_LICENSE            = 'help.enterLicense';\n *    exports.HELP_DOCUMENTATION            = 'help.documentation';\n *    exports.HELP_FORUM                    = 'help.forum';\n *    exports.HELP_RELEASE_NOTE             = 'help.releaseNote';\n *    exports.HELP_REQUEST_FEATURE          = 'help.requestFeature';\n *\n *    exports.APP_RELOAD                    = 'app.reload';\n * ```\n */\ndefine('command/Commands',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n\n    // File\n    exports.FILE                          = 'file';\n    exports.FILE_NEW                      = 'file.new';\n    exports.FILE_NEW_FROM_TEMPLATE        = 'file.newFromTemplate';\n    exports.FILE_OPEN                     = 'file.open';\n    exports.FILE_OPEN_RECENT              = 'file.openRecent';\n    exports.FILE_SAVE                     = 'file.save';\n    exports.FILE_SAVE_AS                  = 'file.saveAs';\n    exports.FILE_CLOSE                    = 'file.close';\n    exports.FILE_IMPORT                   = 'file.import';\n    exports.FILE_IMPORT_FRAGMENT          = 'file.import.fragment';\n    exports.FILE_EXPORT                   = 'file.export';\n    exports.FILE_EXPORT_FRAGMENT          = 'file.export.fragment';\n    exports.FILE_PREFERENCES              = 'file.preferences';\n    exports.FILE_PRINT_TO_PDF             = 'file.printToPDF';\n\n    // File shell callbacks - string must MATCH string in native code (appshell/command_callbacks.h)\n    exports.FILE_CLOSE_WINDOW             = 'file.close_window';\n    exports.FILE_QUIT                     = 'file.quit';\n\n    // Edit\n    exports.EDIT                          = 'edit';\n    // File shell callbacks - string must MATCH string in native code (appshell/command_callbacks.h)\n    exports.EDIT_UNDO                     = 'edit.undo';\n    exports.EDIT_REDO                     = 'edit.redo';\n    exports.EDIT_CUT                      = 'edit.cut';\n    exports.EDIT_COPY                     = 'edit.copy';\n    exports.EDIT_PASTE                    = 'edit.paste';\n    exports.EDIT_SELECT_ALL               = 'edit.selectAll';\n\n    exports.EDIT_DELETE                   = 'edit.delete';\n    exports.EDIT_DELETE_FROM_MODEL        = 'edit.deleteFromModel';\n    exports.EDIT_MOVE_UP                  = 'edit.moveUp';\n    exports.EDIT_MOVE_DOWN                = 'edit.moveDown';\n    exports.EDIT_SELECT_IN_EXPLORER       = 'edit.selectInExplorer';\n    exports.EDIT_SELECT_IN_DIAGRAM        = 'edit.selectInDiagram';\n\n    // Format\n    exports.FORMAT                        = 'format';\n    exports.FORMAT_FONT                   = 'format.font';\n    exports.FORMAT_FILL_COLOR             = 'format.fillColor';\n    exports.FORMAT_LINE_COLOR             = 'format.lineColor';\n    exports.FORMAT_LINE_STYLE             = 'format.linestyle';\n    exports.FORMAT_LINE_STYLE_RECTILINEAR = 'format.linestyle.rectilinear';\n    exports.FORMAT_LINE_STYLE_OBLIQUE     = 'format.linestyle.oblique';\n    exports.FORMAT_LINE_STYLE_ROUNDRECT   = 'format.linestyle.roundRect';\n    exports.FORMAT_LINE_STYLE_CURVE       = 'format.linestyle.curve';\n    exports.FORMAT_AUTO_RESIZE            = 'format.autoResize';\n    exports.FORMAT_SHOW_SHADOW            = 'format.showShadow';\n\n    // Model\n    exports.MODEL                         = 'model';\n    exports.MODEL_ADD                     = \"model.add\";\n    exports.MODEL_ADD_DIAGRAM             = 'model.addDiagram';\n\n    // Tools\n    exports.TOOLS                         = 'tools';\n    exports.TOOLS_EXTENSION_MANAGER       = 'tools.extensionManager';\n\n    // Model\n    exports.VIEW                          = 'view';\n    exports.VIEW_CLOSE_DIAGRAM            = 'view.closeDiagram';\n    exports.VIEW_CLOSE_OTHER_DIAGRAMS     = 'view.closeOtherDiagrams';\n    exports.VIEW_CLOSE_ALL_DIAGRAMS       = 'view.closeAllDiagrams';\n    exports.VIEW_NEXT_DIAGRAM             = 'view.nextDiagram';\n    exports.VIEW_PREVIOUS_DIAGRAM         = 'view.previousDiagram';\n    exports.VIEW_ZOOM_IN                  = 'view.zoomIn';\n    exports.VIEW_ZOOM_OUT                 = 'view.zoomOut';\n    exports.VIEW_ACTUAL_SIZE              = 'view.actualSize';\n    exports.VIEW_FIT_TO_WINDOW            = 'view.fitToWindow';\n    exports.VIEW_SHOW_GRID                = \"view.showGrid\";\n    exports.VIEW_HIDE_SIDEBAR             = 'view.hideSidebar';\n    exports.VIEW_HIDE_NAVIGATOR           = 'view.hideNavigator';\n    exports.VIEW_HIDE_TOOLBAR             = 'view.hideToolbar';\n    exports.VIEW_HIDE_STATUSBAR           = 'view.hideStatusBar';\n    exports.VIEW_HIDE_TOOLBOX             = 'view.hideToolbox';\n    exports.VIEW_HIDE_EDITORS             = 'view.hideEditors';\n\n    // Help\n    exports.HELP                          = 'help';\n    exports.HELP_ABOUT                    = 'help.about';\n    exports.HELP_CHECK_FOR_UPDATES        = 'help.checkForUpdates';\n    exports.HELP_ENTER_LICENSE            = 'help.enterLicense';\n    exports.HELP_DOCUMENTATION            = 'help.documentation';\n    exports.HELP_FORUM                    = 'help.forum';\n    exports.HELP_RELEASE_NOTE             = 'help.releaseNote';\n    exports.HELP_REQUEST_FEATURE          = 'help.requestFeature';\n\n    // Internals\n    exports.APP_RELOAD                    = 'app.reload';\n\n});\n\n//# sourceURL=/command/Commands.js"),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define  */\n\n/**\n * Utilities module to provide constants for keyCodes\n *\n * ```\n *  DOM_VK_CANCEL: 3,\n *  DOM_VK_HELP: 6,\n *  DOM_VK_BACK_SPACE: 8,\n *  DOM_VK_TAB: 9,\n *  DOM_VK_CLEAR: 12,\n *  DOM_VK_RETURN: 13,\n *  DOM_VK_ENTER: 14,\n *  DOM_VK_SHIFT: 16,\n *  DOM_VK_CONTROL: 17,\n *  DOM_VK_ALT: 18,\n *  DOM_VK_PAUSE: 19,\n *  DOM_VK_CAPS_LOCK: 20,\n *  DOM_VK_ESCAPE: 27,\n *  DOM_VK_SPACE: 32,\n *  DOM_VK_PAGE_UP: 33,\n *  DOM_VK_PAGE_DOWN: 34,\n *  DOM_VK_END: 35,\n *  DOM_VK_HOME: 36,\n *  DOM_VK_LEFT: 37,\n *  DOM_VK_UP: 38,\n *  DOM_VK_RIGHT: 39,\n *  DOM_VK_DOWN: 40,\n *  DOM_VK_PRINTSCREEN: 44,\n *  DOM_VK_INSERT: 45,\n *  DOM_VK_DELETE: 46,\n *  DOM_VK_0: 48,\n *  DOM_VK_1: 49,\n *  DOM_VK_2: 50,\n *  DOM_VK_3: 51,\n *  DOM_VK_4: 52,\n *  DOM_VK_5: 53,\n *  DOM_VK_6: 54,\n *  DOM_VK_7: 55,\n *  DOM_VK_8: 56,\n *  DOM_VK_9: 57,\n *  DOM_VK_A: 65,\n *  DOM_VK_B: 66,\n *  DOM_VK_C: 67,\n *  DOM_VK_D: 68,\n *  DOM_VK_E: 69,\n *  DOM_VK_F: 70,\n *  DOM_VK_G: 71,\n *  DOM_VK_H: 72,\n *  DOM_VK_I: 73,\n *  DOM_VK_J: 74,\n *  DOM_VK_K: 75,\n *  DOM_VK_L: 76,\n *  DOM_VK_M: 77,\n *  DOM_VK_N: 78,\n *  DOM_VK_O: 79,\n *  DOM_VK_P: 80,\n *  DOM_VK_Q: 81,\n *  DOM_VK_R: 82,\n *  DOM_VK_S: 83,\n *  DOM_VK_T: 84,\n *  DOM_VK_U: 85,\n *  DOM_VK_V: 86,\n *  DOM_VK_W: 87,\n *  DOM_VK_X: 88,\n *  DOM_VK_Y: 89,\n *  DOM_VK_Z: 90,\n *  DOM_VK_CONTEXT_MENU: 93,\n *  DOM_VK_NUMPAD0: 96,\n *  DOM_VK_NUMPAD1: 97,\n *  DOM_VK_NUMPAD2: 98,\n *  DOM_VK_NUMPAD3: 99,\n *  DOM_VK_NUMPAD4: 100,\n *  DOM_VK_NUMPAD5: 101,\n *  DOM_VK_NUMPAD6: 102,\n *  DOM_VK_NUMPAD7: 103,\n *  DOM_VK_NUMPAD8: 104,\n *  DOM_VK_NUMPAD9: 105,\n *  DOM_VK_MULTIPLY: 106,\n *  DOM_VK_ADD: 107,\n *  DOM_VK_SEPARATOR: 108,\n *  DOM_VK_SUBTRACT: 109,\n *  DOM_VK_DECIMAL: 110,\n *  DOM_VK_DIVIDE: 111,\n *  DOM_VK_F1: 112,\n *  DOM_VK_F2: 113,\n *  DOM_VK_F3: 114,\n *  DOM_VK_F4: 115,\n *  DOM_VK_F5: 116,\n *  DOM_VK_F6: 117,\n *  DOM_VK_F7: 118,\n *  DOM_VK_F8: 119,\n *  DOM_VK_F9: 120,\n *  DOM_VK_F10: 121,\n *  DOM_VK_F11: 122,\n *  DOM_VK_F12: 123,\n *  DOM_VK_F13: 124,\n *  DOM_VK_F14: 125,\n *  DOM_VK_F15: 126,\n *  DOM_VK_F16: 127,\n *  DOM_VK_F17: 128,\n *  DOM_VK_F18: 129,\n *  DOM_VK_F19: 130,\n *  DOM_VK_F20: 131,\n *  DOM_VK_F21: 132,\n *  DOM_VK_F22: 133,\n *  DOM_VK_F23: 134,\n *  DOM_VK_F24: 135,\n *  DOM_VK_NUM_LOCK: 144,\n *  DOM_VK_SCROLL_LOCK: 145,\n *  DOM_VK_SEMICOLON: 186,\n *  DOM_VK_EQUALS: 187,\n *  DOM_VK_COMMA: 188,\n *  DOM_VK_DASH: 189,\n *  DOM_VK_PERIOD: 190,\n *  DOM_VK_SLASH: 191,\n *  DOM_VK_BACK_QUOTE: 192,\n *  DOM_VK_OPEN_BRACKET: 219,\n *  DOM_VK_BACK_SLASH: 220,\n *  DOM_VK_CLOSE_BRACKET: 221,\n *  DOM_VK_QUOTE: 222,\n *  DOM_VK_META: 224,\n *  DOM_VK_PLUS: 107, // in Number Keypad\n *  DOM_VK_MINUS: 109 // in Number Keypad\n * ```\n */\ndefine(\'utils/KeyEvent\',{\n    DOM_VK_CANCEL: 3,\n    DOM_VK_HELP: 6,\n    DOM_VK_BACK_SPACE: 8,\n    DOM_VK_TAB: 9,\n    DOM_VK_CLEAR: 12,\n    DOM_VK_RETURN: 13,\n    DOM_VK_ENTER: 14,\n    DOM_VK_SHIFT: 16,\n    DOM_VK_CONTROL: 17,\n    DOM_VK_ALT: 18,\n    DOM_VK_PAUSE: 19,\n    DOM_VK_CAPS_LOCK: 20,\n    DOM_VK_ESCAPE: 27,\n    DOM_VK_SPACE: 32,\n    DOM_VK_PAGE_UP: 33,\n    DOM_VK_PAGE_DOWN: 34,\n    DOM_VK_END: 35,\n    DOM_VK_HOME: 36,\n    DOM_VK_LEFT: 37,\n    DOM_VK_UP: 38,\n    DOM_VK_RIGHT: 39,\n    DOM_VK_DOWN: 40,\n    DOM_VK_PRINTSCREEN: 44,\n    DOM_VK_INSERT: 45,\n    DOM_VK_DELETE: 46,\n    DOM_VK_0: 48,\n    DOM_VK_1: 49,\n    DOM_VK_2: 50,\n    DOM_VK_3: 51,\n    DOM_VK_4: 52,\n    DOM_VK_5: 53,\n    DOM_VK_6: 54,\n    DOM_VK_7: 55,\n    DOM_VK_8: 56,\n    DOM_VK_9: 57,\n    DOM_VK_A: 65,\n    DOM_VK_B: 66,\n    DOM_VK_C: 67,\n    DOM_VK_D: 68,\n    DOM_VK_E: 69,\n    DOM_VK_F: 70,\n    DOM_VK_G: 71,\n    DOM_VK_H: 72,\n    DOM_VK_I: 73,\n    DOM_VK_J: 74,\n    DOM_VK_K: 75,\n    DOM_VK_L: 76,\n    DOM_VK_M: 77,\n    DOM_VK_N: 78,\n    DOM_VK_O: 79,\n    DOM_VK_P: 80,\n    DOM_VK_Q: 81,\n    DOM_VK_R: 82,\n    DOM_VK_S: 83,\n    DOM_VK_T: 84,\n    DOM_VK_U: 85,\n    DOM_VK_V: 86,\n    DOM_VK_W: 87,\n    DOM_VK_X: 88,\n    DOM_VK_Y: 89,\n    DOM_VK_Z: 90,\n    DOM_VK_CONTEXT_MENU: 93,\n    DOM_VK_NUMPAD0: 96,\n    DOM_VK_NUMPAD1: 97,\n    DOM_VK_NUMPAD2: 98,\n    DOM_VK_NUMPAD3: 99,\n    DOM_VK_NUMPAD4: 100,\n    DOM_VK_NUMPAD5: 101,\n    DOM_VK_NUMPAD6: 102,\n    DOM_VK_NUMPAD7: 103,\n    DOM_VK_NUMPAD8: 104,\n    DOM_VK_NUMPAD9: 105,\n    DOM_VK_MULTIPLY: 106,\n    DOM_VK_ADD: 107,\n    DOM_VK_SEPARATOR: 108,\n    DOM_VK_SUBTRACT: 109,\n    DOM_VK_DECIMAL: 110,\n    DOM_VK_DIVIDE: 111,\n    DOM_VK_F1: 112,\n    DOM_VK_F2: 113,\n    DOM_VK_F3: 114,\n    DOM_VK_F4: 115,\n    DOM_VK_F5: 116,\n    DOM_VK_F6: 117,\n    DOM_VK_F7: 118,\n    DOM_VK_F8: 119,\n    DOM_VK_F9: 120,\n    DOM_VK_F10: 121,\n    DOM_VK_F11: 122,\n    DOM_VK_F12: 123,\n    DOM_VK_F13: 124,\n    DOM_VK_F14: 125,\n    DOM_VK_F15: 126,\n    DOM_VK_F16: 127,\n    DOM_VK_F17: 128,\n    DOM_VK_F18: 129,\n    DOM_VK_F19: 130,\n    DOM_VK_F20: 131,\n    DOM_VK_F21: 132,\n    DOM_VK_F22: 133,\n    DOM_VK_F23: 134,\n    DOM_VK_F24: 135,\n    DOM_VK_NUM_LOCK: 144,\n    DOM_VK_SCROLL_LOCK: 145,\n    DOM_VK_SEMICOLON: 186,\n    DOM_VK_EQUALS: 187,\n    DOM_VK_COMMA: 188,\n    DOM_VK_DASH: 189,\n    DOM_VK_PERIOD: 190,\n    DOM_VK_SLASH: 191,\n    DOM_VK_BACK_QUOTE: 192,\n    DOM_VK_OPEN_BRACKET: 219,\n    DOM_VK_BACK_SLASH: 220,\n    DOM_VK_CLOSE_BRACKET: 221,\n    DOM_VK_QUOTE: 222,\n    DOM_VK_META: 224,\n    DOM_VK_PLUS: 107, // in Number Keypad\n    DOM_VK_MINUS: 109 // in Number Keypad\n\n});\n\n//# sourceURL=/utils/KeyEvent.js'),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define */\n\ndefine(\'strings\',{\n\n    // Menus\n\n    // Menus and Commands\n    // File\n    "CMD_FILE"                          : "File",\n    "CMD_FILE_NEW"                      : "New",\n    "CMD_FILE_NEW_FROM_TEMPLATE"        : "New From Template",\n    "CMD_FILE_OPEN"                     : "Open...",\n    "CMD_FILE_OPEN_RECENT"              : "Open Recent",\n    "CMD_FILE_SAVE"                     : "Save",\n    "CMD_FILE_SAVE_AS"                  : "Save As...",\n    "CMD_FILE_IMPORT"                   : "Import",\n    "CMD_FILE_IMPORT_FRAGMENT"          : "Fragment...",\n    "CMD_FILE_EXPORT"                   : "Export",\n    "CMD_FILE_EXPORT_FRAGMENT"          : "Fragment...",\n    "CMD_FILE_CLOSE"                    : "Close",\n    "CMD_FILE_PREFERENCES"              : "Preferences...",\n    "CMD_FILE_PRINT_TO_PDF"             : "Print to PDF...",\n    "CMD_QUIT"                          : "Quit",\n    "CMD_EXIT"                          : "Exit",\n    // Edit\n    "CMD_EDIT"                          : "Edit",\n    "CMD_EDIT_UNDO"                     : "Undo",\n    "CMD_EDIT_REDO"                     : "Redo",\n    "CMD_EDIT_CUT"                      : "Cut",\n    "CMD_EDIT_COPY"                     : "Copy",\n    "CMD_EDIT_PASTE"                    : "Paste",\n    "CMD_EDIT_DELETE"                   : "Delete",\n    "CMD_EDIT_DELETE_FROM_MODEL"        : "Delete from Model",\n    "CMD_EDIT_MOVE_UP"                  : "Move Up",\n    "CMD_EDIT_MOVE_DOWN"                : "Move Down",\n    "CMD_EDIT_SELECT_ALL"               : "Select All",\n    "CMD_EDIT_SELECT_IN_EXPLORER"       : "Select In Explorer",\n    "CMD_EDIT_SELECT_IN_DIAGRAM"        : "Select In Diagram",\n    // Format\n    "CMD_FORMAT"                        : "Format",\n    "CMD_FORMAT_FONT"                   : "Font...",\n    "CMD_FORMAT_FILL_COLOR"             : "Fill Color...",\n    "CMD_FORMAT_LINE_COLOR"             : "Line Color...",\n    "CMD_FORMAT_LINE_STYLE"             : "Line Style",\n    "CMD_FORMAT_LINE_STYLE_RECTILINEAR" : "Rectilinear",\n    "CMD_FORMAT_LINE_STYLE_OBLIQUE"     : "Oblique",\n    "CMD_FORMAT_LINE_STYLE_ROUNDRECT"   : "Rounded Rectilinear",\n    "CMD_FORMAT_LINE_STYLE_CURVE"       : "Curve",\n\n    "CMD_FORMAT_AUTO_RESIZE"            : "Auto Resize",\n    "CMD_FORMAT_SHOW_SHADOW"            : "Show Shadow",\n    // Model\n    "CMD_MODEL"                         : "Model",\n    // Tools\n    "CMD_TOOLS"                         : "Tools",\n    "CMD_TOOLS_EXTENSION_MANAGER"       : "Extension Manager...",\n    // View\n    "CMD_VIEW"                          : "View",\n    "CMD_VIEW_CLOSE_DIAGRAM"            : "Close Diagram",\n    "CMD_VIEW_CLOSE_OTHER_DIAGRAMS"     : "Close Other Diagrams",\n    "CMD_VIEW_CLOSE_ALL_DIAGRAMS"       : "Close All Diagrams",\n    "CMD_VIEW_NEXT_DIAGRAM"             : "Next Diagram",\n    "CMD_VIEW_PREVIOUS_DIAGRAM"         : "Previous Diagram",\n    "CMD_VIEW_ZOOM_IN"                  : "Zoom In",\n    "CMD_VIEW_ZOOM_OUT"                 : "Zoom Out",\n    "CMD_VIEW_ACTUAL_SIZE"              : "Actual Size",\n    "CMD_VIEW_FIT_TO_WINDOW"            : "Fit To Window",\n    "CMD_VIEW_SHOW_GRID"                : "Show Grid",\n    "CMD_VIEW_SHOW_SIDEBAR"             : "Show Sidebar",\n    "CMD_VIEW_HIDE_SIDEBAR"             : "Hide Sidebar",\n    "CMD_VIEW_SHOW_NAVIGATOR"           : "Show Navigator",\n    "CMD_VIEW_HIDE_NAVIGATOR"           : "Hide Navigator",\n    "CMD_VIEW_SHOW_TOOLBAR"             : "Show Toolbar",\n    "CMD_VIEW_HIDE_TOOLBAR"             : "Hide Toolbar",\n    "CMD_VIEW_SHOW_STATUSBAR"           : "Show Status Bar",\n    "CMD_VIEW_HIDE_STATUSBAR"           : "Hide Status Bar",\n    "CMD_VIEW_SHOW_TOOLBOX"             : "Show Toolbox",\n    "CMD_VIEW_HIDE_TOOLBOX"             : "Hide Toolbox",\n    "CMD_VIEW_SHOW_EDITORS"             : "Show Editors",\n    "CMD_VIEW_HIDE_EDITORS"             : "Hide Editors",\n    // Help\n    "CMD_HELP"                          : "Help",\n    "CMD_HELP_ABOUT"                    : "About StarUML",\n    "CMD_HELP_CHECK_FOR_UPDATES"        : "Check for Updates...",\n    "CMD_HELP_ENTER_LICENSE"            : "Enter License...",\n    "CMD_HELP_DOCUMENTATION"            : "Documentation",\n    "CMD_HELP_FORUM"                    : "Forum",\n    "CMD_HELP_RELEASE_NOTE"             : "Release Notes",\n    "CMD_HELP_REQUEST_FEATURE"          : "Request Feature",\n\n    // UML Commands\n    /* File: New From Template */\n    "CMD_FILE_NEW_FROM_TEMPLATE_UML_MINIMAL"      : "UML Minimal",\n    "CMD_FILE_NEW_FROM_TEMPLATE_UML_CONVENTIONAL" : "UML Conventional",\n    "CMD_FILE_NEW_FROM_TEMPLATE_4P1VIEWMODEL"     : "4+1 View Model",\n    "CMD_FILE_NEW_FROM_TEMPLATE_RATIONAL"         : "Rational",\n    // Model: Profiles\n    "CMD_MODEL_APPLY_PROFILE"                  : "Apply Profile",\n    "CMD_MODEL_APPLY_PROFILE_UML_STANDARD"     : "UML Standard Profile (v2)",\n    // Model Add\n    "CMD_MODEL_ADD"                            : "Add",\n    "CMD_MODEL_ADD_DIAGRAM"                    : "Add Diagram",\n    // Model: Packages\n    "CMD_MODEL_ADD_MODEL"                      : "Model",\n    "CMD_MODEL_ADD_SUBSYSTEM"                  : "Subsystem",\n    "CMD_MODEL_ADD_PACKAGE"                    : "Package",\n    "CMD_MODEL_ADD_PROFILE"                    : "Profile",\n    // Model: Classes\n    "CMD_MODEL_ADD_CLASS"                      : "Class",\n    "CMD_MODEL_ADD_INTERFACE"                  : "Interface",\n    "CMD_MODEL_ADD_SIGNAL"                     : "Signal",\n    "CMD_MODEL_ADD_DATATYPE"                   : "DataType",\n    "CMD_MODEL_ADD_PRIMITIVETYPE"              : "PrimitiveType",\n    "CMD_MODEL_ADD_ENUMERATION"                : "Enumeration",\n    "CMD_MODEL_ADD_ARTIFACT"                   : "Artifact",\n    "CMD_MODEL_ADD_COMPONENT"                  : "Component",\n    "CMD_MODEL_ADD_NODE"                       : "Node",\n    "CMD_MODEL_ADD_USECASE"                    : "UseCase",\n    "CMD_MODEL_ADD_ACTOR"                      : "Actor",\n    "CMD_MODEL_ADD_STEREOTYPE"                 : "Stereotype",\n    // Model: Instances\n    "CMD_MODEL_ADD_OBJECT"                     : "Object",\n    "CMD_MODEL_ADD_ARTIFACTINSTANCE"           : "Artifact Instance",\n    "CMD_MODEL_ADD_COMPONENTINSTANCE"          : "Component Instance",\n    "CMD_MODEL_ADD_NODEINSTANCE"               : "Node Instance",\n    // Model: Behaviors\n    "CMD_MODEL_ADD_COLLABORATION"              : "Collaboration",\n    "CMD_MODEL_ADD_INTERACTION"                : "Interaction",\n    "CMD_MODEL_ADD_STATEMACHINE"               : "State Machine",\n    "CMD_MODEL_ADD_ACTIVITY"                   : "Activity",\n    "CMD_MODEL_ADD_OPAQUEBEHAVIOR"             : "Opaque Behavior",\n    // Model: Features\n    "CMD_MODEL_ADD_TEMPLATEPARAMETER"          : "Template Parameter",\n    "CMD_MODEL_ADD_PARAMETER"                  : "Parameter",\n    "CMD_MODEL_ADD_ENUMERATIONLITERAL"         : "Enumeration Literal",\n    "CMD_MODEL_ADD_ATTRIBUTE"                  : "Attribute",\n    "CMD_MODEL_ADD_PORT"                       : "Port",\n    "CMD_MODEL_ADD_OPERATION"                  : "Operation",\n    "CMD_MODEL_ADD_RECEPTION"                  : "Reception",\n    "CMD_MODEL_ADD_EXTENSIONPOINT"             : "Extension Point",\n    "CMD_MODEL_ADD_SLOT"                       : "Slot",\n    // Model: States\n    "CMD_MODEL_ADD_STATE"                      : "State",\n    "CMD_MODEL_ADD_REGION"                     : "Region",\n    "CMD_MODEL_ADD_ENTRY_ACTIVITY"             : "Entry Activity",\n    "CMD_MODEL_ADD_DO_ACTIVITY"                : "Do Activity",\n    "CMD_MODEL_ADD_EXIT_ACTIVITY"              : "Exit Activity",\n    "CMD_MODEL_ADD_TRIGGER"                    : "Trigger",\n    "CMD_MODEL_ADD_EFFECT"                     : "Effect",\n    // Model: Actions\n    "CMD_MODEL_ADD_ACTION"                     : "Action",\n    // Model: Common\n    "CMD_MODEL_ADD_CONSTRAINT"                 : "Constraint",\n    "CMD_MODEL_ADD_TAG"                        : "Tag",\n    // Model: Diagrams\n    "CMD_MODEL_ADD_DIAGRAM_CLASS"              : "Class Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_PACKAGE"            : "Package Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_OBJECT"             : "Object Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_COMPOSITESTRUCTURE" : "Composite Structure Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_COMPONENT"          : "Component Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_DEPLOYMENT"         : "Deployment Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_USECASE"            : "Use Case Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_SEQUENCE"           : "Sequence Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_COMMUNICATION"      : "Communication Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_STATECHART"         : "Statechart Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_ACTIVITY"           : "Activity Diagram",\n    "CMD_MODEL_ADD_DIAGRAM_PROFILE"            : "Profile Diagram",\n    // Format\n    "CMD_FORMAT_STEREOTYPE"                    : "Stereotype Display",\n    "CMD_FORMAT_STEREOTYPE_NONE"               : "None",\n    "CMD_FORMAT_STEREOTYPE_LABEL"              : "Label",\n    "CMD_FORMAT_STEREOTYPE_DECORATION"         : "Decoration",\n    "CMD_FORMAT_STEREOTYPE_DECORATION_LABEL"   : "Decoration with Label",\n    "CMD_FORMAT_STEREOTYPE_ICON"               : "Icon",\n    "CMD_FORMAT_STEREOTYPE_ICON_LABEL"         : "Icon with Label",\n    "CMD_FORMAT_WORD_WRAP"                     : "Word Wrap",\n    "CMD_FORMAT_SHOW_VISIBILITY"               : "Show Visibility",\n    "CMD_FORMAT_SHOW_NAMESPACE"                : "Show Namespace",\n    "CMD_FORMAT_SHOW_PROPERTY"                 : "Show Property",\n    "CMD_FORMAT_SHOW_TYPE"                     : "Show Type",\n    "CMD_FORMAT_SHOW_MULTIPLICITY"             : "Show Multiplicity",\n    "CMD_FORMAT_SHOW_OPERATION_SIGNATURE"      : "Show Operation Signature",\n    "CMD_FORMAT_SUPPRESS_ATTRIBUTES"           : "Suppress Attributes",\n    "CMD_FORMAT_SUPPRESS_OPERATIONS"           : "Suppress Operations",\n    "CMD_FORMAT_SUPPRESS_RECEPTIONS"           : "Suppress Receptions",\n    "CMD_FORMAT_SUPPRESS_LITERALS"             : "Suppress Literals",\n\n    // Extension Manager\n    "EXTENSION_NOT_INSTALLED"              : "Couldn\'t remove extension {0} because it wasn\'t installed.",\n    "CANT_UPDATE"                          : "The update isn\'t compatible with this version of StarUML.",\n    "CANT_UPDATE_DEV"                      : "Extensions in the \\"dev\\" folder can\'t be updated automatically.",\n    "INSTALL"                              : "Install",\n    "UPDATE"                               : "Update",\n    "REMOVE"                               : "Remove",\n    "OVERWRITE"                            : "Overwrite",\n    "EXTENSION_MANAGER_ERROR_LOAD"         : "Unable to access the extension registry. Please try again later.",\n    "EXTENSION_INCOMPATIBLE_NEWER"         : "This extension requires a newer version of StarUML.",\n    "EXTENSION_INCOMPATIBLE_OLDER"         : "This extension currently only works with older versions of StarUML.",\n    "EXTENSION_LATEST_INCOMPATIBLE_NEWER"  : "Version {0} of this extension requires a newer version of StarUML. But you can install the earlier version {1}.",\n    "EXTENSION_LATEST_INCOMPATIBLE_OLDER"  : "Version {0} of this extension only works with older versions of StarUML. But you can install the earlier version {1}.",\n    "EXTENSION_NO_DESCRIPTION"             : "No description",\n    "EXTENSION_ISSUES"                     : "Issues",\n    "EXTENSION_MORE_INFO"                  : "More info...",\n    "EXTENSION_ERROR"                      : "Extension error",\n    "EXTENSION_KEYWORDS"                   : "Keywords",\n    "EXTENSION_INSTALLED"                  : "Installed",\n    "EXTENSION_UPDATE_INSTALLED"           : "This extension update has been downloaded and will be installed after StarUML reloads.",\n    "EXTENSION_MANAGER_REMOVE"             : "Remove Extension",\n    "EXTENSION_MANAGER_REMOVE_ERROR"       : "Unable to remove one or more extensions: {0}. StarUML will still reload.",\n    "EXTENSION_MANAGER_UPDATE"             : "Update Extension",\n    "EXTENSION_MANAGER_UPDATE_ERROR"       : "Unable to update one or more extensions: {0}. StarUML will still reload.",\n    "MARKED_FOR_REMOVAL"                   : "Marked for removal",\n    "UNDO_REMOVE"                          : "Undo",\n    "MARKED_FOR_UPDATE"                    : "Marked for update",\n    "UNDO_UPDATE"                          : "Undo",\n    "CHANGE_AND_RELOAD_TITLE"              : "Change Extensions",\n    "CHANGE_AND_RELOAD_MESSAGE"            : "To update or remove the marked extensions, StarUML will need to reload. You\'ll be prompted to save unsaved changes.",\n    "REMOVE_AND_RELOAD"                    : "Remove Extensions and Reload",\n    "CHANGE_AND_RELOAD"                    : "Change Extensions and Reload",\n    "UPDATE_AND_RELOAD"                    : "Update Extensions and Reload",\n    "PROCESSING_EXTENSIONS"                : "Processing extension changes\\u2026",\n    "NO_EXTENSIONS"                        : "No extensions installed yet.<br>Click on the Registry tab above to get started.",\n    "NO_EXTENSION_MATCHES"                 : "No extensions match your search.",\n    "REGISTRY_SANITY_CHECK_WARNING"        : "Be cautious when installing extensions from an unknown source.",\n    "UNKNOWN_ERROR"                        : "Unknown internal error.",\n\n    // Install Extension Dialog\n    "INSTALL_EXTENSION_TITLE"              : "Install Extension",\n    "UPDATE_EXTENSION_TITLE"               : "Update Extension",\n    "INSTALLING_FROM"                      : "Installing extension from {0}\\u2026",\n    "INSTALL_SUCCEEDED"                    : "Installation successful!",\n    "INSTALL_FAILED"                       : "Installation failed.",\n    "INSTALL_CANCELED"                     : "Installation canceled.",\n    "CANCELING_INSTALL"                    : "Canceling\\u2026",\n    "CANCELING_HUNG"                       : "Canceling the install is taking a long time. An internal error may have occurred.",\n\n    // Dialogs\n    "OK"                                   : "OK",\n    "CANCEL"                               : "Cancel",\n    "SAVE"                                 : "Save",\n    "DONTSAVE"                             : "Don\'t Save",\n    "CLOSE"                                : "Close",\n    "DOWNLOAD"                             : "Download",\n\n    // Settings for Explorer\n    "EXPLORER_SETTINGS_SORT_BY_ADDED"      : "Sort By Added",\n    "EXPLORER_SETTINGS_SORT_BY_NAME"       : "Sort By Name",\n    "EXPLORER_SETTINGS_SHOW_STEREOTYPE_TEXT" : "Show Stereotype Text",\n\n    /**\n     * Keyboard modifier names\n     */\n    "KEYBOARD_CTRL"   : "Ctrl",\n    "KEYBOARD_SHIFT"  : "Shift",\n    "KEYBOARD_SPACE"  : "Space"\n\n});\n\n//# sourceURL=/strings.js'),eval('define(\'text!base-config/keyboard.json\',[],function () { return \'{\\n    "file.new": [\\n        "Ctrl-N"\\n    ],\\n    "file.open": [\\n        "Ctrl-O"\\n    ],\\n    "file.save": [\\n        "Ctrl-S"\\n    ],\\n    "file.saveAs": [\\n        "Ctrl-Shift-S"\\n    ],\\n    "edit.undo": [\\n        "Ctrl-Z"\\n    ],\\n    "edit.redo": [\\n        "Ctrl-Y"\\n    ],\\n    "edit.cut": [\\n        "Ctrl-X"\\n    ],\\n    "edit.copy": [\\n        "Ctrl-C"\\n    ],\\n    "edit.paste": [\\n        "Ctrl-V"\\n    ],\\n    "edit.deleteFromModel": [\\n        {\\n            "key": "Ctrl-Delete"\\n        },\\n        {\\n            "key": "Cmd-Backspace",\\n            "platform": "mac"\\n        }\\n    ],\\n    "edit.moveUp": [\\n        "Ctrl-Shift-Up"\\n    ],\\n    "edit.moveDown": [\\n        "Ctrl-Shift-Down"\\n    ],\\n    "edit.selectAll": [\\n        "Ctrl-A"\\n    ],\\n    "edit.selectInExplorer": [\\n        "Ctrl-E"\\n    ],\\n    "edit.selectInDiagram": [\\n        "Ctrl-D"\\n    ],\\n    "format.font": [\\n        "Ctrl-Shift-F"\\n    ],\\n    "format.fillColor": [\\n        "Ctrl-Shift-I"\\n    ],\\n    "format.lineColor": [\\n        "Ctrl-Shift-L"\\n    ],\\n    "format.linestyle.rectilinear": [\\n        "Ctrl-L"\\n    ],\\n    "format.linestyle.oblique": [\\n        "Ctrl-B"\\n    ],\\n    "format.linestyle.roundRect": [\\n        "Ctrl-Alt-L"\\n    ],\\n    "format.linestyle.curve": [\\n        "Ctrl-Alt-B"\\n    ],\\n    "format.autoResize": [\\n        "Ctrl-Shift-R"\\n    ],\\n    "format.showShadow": [\\n        "Ctrl-Shift-H"\\n    ],\\n    "view.closeDiagram": [\\n        "F4"\\n    ],\\n    "view.closeOtherDiagrams": [\\n        "Ctrl-F4"\\n    ],\\n    "view.closeAllDiagrams": [\\n        "Shift-F4"\\n    ],\\n    "view.nextDiagram": [\\n        "Ctrl-Shift-]"\\n    ],\\n    "view.previousDiagram": [\\n        "Ctrl-Shift-["\\n    ],\\n    "view.zoomIn": [\\n        {\\n            "key": "Ctrl-=",\\n            "displayKey": "Cmd-+"\\n        },\\n        {\\n            "key": "Ctrl-+",\\n            "displayKey": "Cmd-+"\\n        }\\n    ],\\n    "view.zoomOut": [\\n        {\\n            "key": "Ctrl--",\\n            "displayKey": "Cmd-−"\\n        }\\n    ],\\n    "view.actualSize": [\\n        {\\n            "key": "Ctrl-0",\\n            "displayKey": "Cmd-0"\\n        }\\n    ],\\n    "view.fitToWindow": [\\n        {\\n            "key": "Ctrl-Alt-0",\\n            "displayKey": "Cmd-Alt-0"\\n        }\\n    ],\\n    "view.showGrid": [\\n        {\\n            "key": "Ctrl-G",\\n            "displayKey": "Cmd-G"\\n        }\\n    ],\\n    "view.hideSidebar": [\\n        {\\n            "key": "Ctrl-1",\\n            "displayKey": "Cmd-1"\\n        }\\n    ],\\n    "view.hideNavigator": [\\n        {\\n            "key": "Ctrl-2",\\n            "displayKey": "Cmd-2"\\n        }\\n    ],\\n    "view.hideToolbar": [\\n        {\\n            "key": "Ctrl-3",\\n            "displayKey": "Cmd-3"\\n        }\\n    ],\\n    "view.hideStatusBar": [\\n        {\\n            "key": "Ctrl-4",\\n            "displayKey": "Cmd-4"\\n        }\\n    ],\\n    "view.hideToolbox": [\\n        {\\n            "key": "Ctrl-5",\\n            "displayKey": "Cmd-5"\\n        }\\n    ],\\n    "view.hideEditors": [\\n        {\\n            "key": "Ctrl-6",\\n            "displayKey": "Cmd-6"\\n        }\\n    ],\\n    "tools.debug.reload": [\\n        "Ctrl-R"\\n    ],\\n    "format.stereotype.none": [\\n        "Ctrl-Shift-0"\\n    ],\\n    "format.stereotype.label": [\\n        "Ctrl-Shift-1"\\n    ],\\n    "format.stereotype.decoration": [\\n        "Ctrl-Shift-2"\\n    ],\\n    "format.stereotype.decorationLabel": [\\n        "Ctrl-Shift-3"\\n    ],\\n    "format.stereotype.icon": [\\n        "Ctrl-Shift-4"\\n    ],\\n    "format.stereotype.iconLabel": [\\n        "Ctrl-Shift-5"\\n    ],\\n    "format.wordWrap": [\\n        "Ctrl-Shift-W"\\n    ],\\n    "format.showVisibility": [\\n        "Ctrl-Shift-V"\\n    ],\\n    "format.showNamespace": [\\n        "Ctrl-Shift-N"\\n    ],\\n    "format.showProperty": [\\n        "Ctrl-Shift-P"\\n    ],\\n    "format.showType": [\\n        "Ctrl-Shift-Y"\\n    ],\\n    "format.showMultiplicity": [\\n        "Ctrl-Shift-M"\\n    ],\\n    "format.showOperationSignature": [\\n        "Ctrl-Shift-G"\\n    ],\\n    "format.suppressAttributes": [\\n        "Ctrl-Shift-A"\\n    ],\\n    "format.suppressOperations": [\\n        "Ctrl-Shift-O"\\n    ],\\n    "format.suppressReceptions": [\\n        "Ctrl-Shift-E"\\n    ],\\n    "format.suppressLiterals": [\\n        "Ctrl-Shift-T"\\n    ]\\n}\\n\';});\n\n//# sourceURL=/text!base-config/keyboard.json'),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, app, window */\n/*unittests: KeyBindingManager */\n\n/**\n * Manages the mapping of keyboard inputs to commands.\n */\ndefine(\'command/KeyBindingManager\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Global\',\'utils/Initializer\',\'command/CommandManager\',\'utils/KeyEvent\',\'strings\',\'text!base-config/keyboard.json\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit        = require("utils/AppInit"),\n        global         = require("core/Global").global,\n        Initializer    = require("utils/Initializer"),\n        CommandManager = require("command/CommandManager"),\n        KeyEvent       = require("utils/KeyEvent"),\n        Strings        = require("strings");\n\n    var KeyboardPrefs = JSON.parse(require("text!base-config/keyboard.json"));\n\n    /**\n     * @private\n     * Maps normalized shortcut descriptor to key binding info.\n     * @type {!Object.<string, {commandID: string, key: string, displayKey: string}>}\n     */\n    var _keyMap = {};\n\n    /**\n     * @private\n     * Maps commandID to the list of shortcuts that are bound to it.\n     * @type {!Object.<string, Array.<{key: string, displayKey: string}>>}\n     */\n    var _commandMap = {};\n\n    /**\n     * @private\n     * Allow clients to toggle key binding\n     * @type {boolean}\n     */\n    var _enabled = true;\n\n    /**\n     * @private\n     * Stack of registered global keydown hooks.\n     * @type {Array.<function(Event): boolean>}\n     */\n    var _globalKeydownHooks = [];\n\n    /**\n     * @private\n     */\n    function _reset() {\n        _keyMap = {};\n        _commandMap = {};\n        _globalKeydownHooks = [];\n    }\n\n    /**\n     * @private\n     * Initialize an empty keymap as the current keymap. It overwrites the current keymap if there is one.\n     * builds the keyDescriptor string from the given parts\n     * @param {boolean} hasCtrl Is Ctrl key enabled\n     * @param {boolean} hasAlt Is Alt key enabled\n     * @param {boolean} hasShift Is Shift key enabled\n     * @param {string} key The key that\'s pressed\n     * @return {string} The normalized key descriptor\n     */\n    function _buildKeyDescriptor(hasMacCtrl, hasCtrl, hasAlt, hasShift, key) {\n        if (!key) {\n            console.log("KeyBindingManager _buildKeyDescriptor() - No key provided!");\n            return "";\n        }\n\n        var keyDescriptor = [];\n\n        if (hasMacCtrl) {\n            keyDescriptor.push("Ctrl");\n        }\n        if (hasAlt) {\n            keyDescriptor.push("Alt");\n        }\n        if (hasShift) {\n            keyDescriptor.push("Shift");\n        }\n\n        if (hasCtrl) {\n            // Windows display Ctrl first, Mac displays Command symbol last\n            if (global.app.platform === "mac") {\n                keyDescriptor.push("Cmd");\n            } else {\n                keyDescriptor.unshift("Ctrl");\n            }\n        }\n\n        keyDescriptor.push(key);\n\n        return keyDescriptor.join("-");\n    }\n\n\n    /**\n     * normalizes the incoming key descriptor so the modifier keys are always specified in the correct order\n     * @param {string} The string for a key descriptor, can be in any order, the result will be Ctrl-Alt-Shift-<Key>\n     * @return {string} The normalized key descriptor or null if the descriptor invalid\n     */\n    function normalizeKeyDescriptorString(origDescriptor) {\n        var hasMacCtrl = false,\n            hasCtrl = false,\n            hasAlt = false,\n            hasShift = false,\n            key = "",\n            error = false;\n\n        function _compareModifierString(left, right) {\n            if (!left || !right) {\n                return false;\n            }\n            left = left.trim().toLowerCase();\n            right = right.trim().toLowerCase();\n\n            return (left.length > 0 && left === right);\n        }\n\n        origDescriptor.split("-").forEach(function parseDescriptor(ele, i, arr) {\n            if (_compareModifierString("ctrl", ele)) {\n                if (global.app.platform === "mac") {\n                    hasMacCtrl = true;\n                } else {\n                    hasCtrl = true;\n                }\n            } else if (_compareModifierString("cmd", ele)) {\n                hasCtrl = true;\n            } else if (_compareModifierString("alt", ele)) {\n                hasAlt = true;\n            } else if (_compareModifierString("opt", ele)) {\n                hasAlt = true;\n            } else if (_compareModifierString("shift", ele)) {\n                hasShift = true;\n            } else if (key.length > 0) {\n                console.log("KeyBindingManager normalizeKeyDescriptorString() - Multiple keys defined. Using key: " + key + " from: " + origDescriptor);\n                error = true;\n            } else {\n                key = ele;\n            }\n        });\n\n        if (error) {\n            return null;\n        }\n\n        // Check to see if the binding is for "-".\n        if (key === "" && origDescriptor.search(/^.+--$/) !== -1) {\n            key = "-";\n        }\n\n        // \'+\' char is valid if it\'s the only key. Keyboard shortcut strings should use\n        // unicode characters (unescaped). Keyboard shortcut display strings may use\n        // unicode escape sequences (e.g. \\u20AC euro sign)\n        if ((key.indexOf("+")) >= 0 && (key.length > 1)) {\n            return null;\n        }\n\n        return _buildKeyDescriptor(hasMacCtrl, hasCtrl, hasAlt, hasShift, key);\n    }\n\n    /**\n     * @private\n     * Looks for keycodes that have os-inconsistent keys and fixes them.\n     * @param {number} The keycode from the keyboard event.\n     * @param {string} The current best guess at what the key is.\n     * @return {string} If the key is OS-inconsistent, the correct key; otherwise, the original key.\n     **/\n    function _mapKeycodeToKey(keycode, key) {\n        // If keycode represents one of the digit keys (0-9), then return the corresponding digit\n        // by subtracting KeyEvent.DOM_VK_0 from keycode. ie. [48-57] --> [0-9]\n        if (keycode >= KeyEvent.DOM_VK_0 && keycode <= KeyEvent.DOM_VK_9) {\n            return String(keycode - KeyEvent.DOM_VK_0);\n        // Do the same with the numpad numbers\n        // by subtracting KeyEvent.DOM_VK_NUMPAD0 from keycode. ie. [96-105] --> [0-9]\n        } else if (keycode >= KeyEvent.DOM_VK_NUMPAD0 && keycode <= KeyEvent.DOM_VK_NUMPAD9) {\n            return String(keycode - KeyEvent.DOM_VK_NUMPAD0);\n        }\n\n\n        switch (keycode) {\n        case KeyEvent.DOM_VK_SEMICOLON:\n            return ";";\n        case KeyEvent.DOM_VK_EQUALS:\n            return "=";\n        case KeyEvent.DOM_VK_COMMA:\n            return ",";\n        case KeyEvent.DOM_VK_SUBTRACT:\n        case KeyEvent.DOM_VK_DASH:\n            return "-";\n        case KeyEvent.DOM_VK_ADD:\n            return "+";\n        case KeyEvent.DOM_VK_DECIMAL:\n        case KeyEvent.DOM_VK_PERIOD:\n            return ".";\n        case KeyEvent.DOM_VK_DIVIDE:\n        case KeyEvent.DOM_VK_SLASH:\n            return "/";\n        case KeyEvent.DOM_VK_BACK_QUOTE:\n            return "`";\n        case KeyEvent.DOM_VK_OPEN_BRACKET:\n            return "[";\n        case KeyEvent.DOM_VK_BACK_SLASH:\n            return "\\\\";\n        case KeyEvent.DOM_VK_CLOSE_BRACKET:\n            return "]";\n        case KeyEvent.DOM_VK_QUOTE:\n            return "\'";\n        default:\n            return key;\n        }\n    }\n\n    /**\n     * Takes a keyboard event and translates it into a key in a key map\n     */\n    function _translateKeyboardEvent(event) {\n        var hasMacCtrl = (global.app.platform === "mac") ? (event.ctrlKey) : false,\n            hasCtrl = (global.app.platform !== "mac") ? (event.ctrlKey) : (event.metaKey),\n            hasAlt = (event.altKey),\n            hasShift = (event.shiftKey),\n            key = String.fromCharCode(event.keyCode);\n\n        //From the W3C, if we can get the KeyboardEvent.keyIdentifier then look here\n        //As that will let us use keys like then function keys "F5" for commands. The\n        //full set of values we can use is here\n        //http://www.w3.org/TR/2007/WD-DOM-Level-3-Events-20071221/keyset.html#KeySet-Set\n        var ident = event.keyIdentifier;\n        if (ident) {\n            if (ident.charAt(0) === "U" && ident.charAt(1) === "+") {\n                //This is a unicode code point like "U+002A", get the 002A and use that\n                key = String.fromCharCode(parseInt(ident.substring(2), 16));\n            } else {\n                //This is some non-character key, just use the raw identifier\n                key = ident;\n            }\n        }\n\n        // Translate some keys to their common names\n        if (key === "\\t") {\n            key = "Tab";\n        } else if (key === " ") {\n            key = "Space";\n        } else {\n            key = _mapKeycodeToKey(event.keyCode, key);\n        }\n\n        return _buildKeyDescriptor(hasMacCtrl, hasCtrl, hasAlt, hasShift, key);\n    }\n\n    /**\n     * Convert normalized key representation to display appropriate for platform.\n     * @param {!string} descriptor Normalized key descriptor.\n     * @return {!string} Display/Operating system appropriate string\n     */\n    function formatKeyDescriptor(descriptor) {\n        var displayStr;\n\n        if (global.app.platform === "mac") {\n            displayStr = descriptor.replace(/-(?!$)/g, "");     // remove dashes\n            displayStr = displayStr.replace("Ctrl", "\\u2303");  // Ctrl > control symbol\n            displayStr = displayStr.replace("Cmd", "\\u2318");   // Cmd > command symbol\n            displayStr = displayStr.replace("Shift", "\\u21E7"); // Shift > shift symbol\n            displayStr = displayStr.replace("Alt", "\\u2325");   // Alt > option symbol\n            displayStr = displayStr.replace("Backspace", "⌫"); // Backspace > backspace symbol\n            displayStr = displayStr.replace("Delete", "⌦");    // Backspace > backspace symbol\n            displayStr = displayStr.replace("Up", "↑");         // Up > up arrow\n            displayStr = displayStr.replace("Down", "↓");       // Down >  down arrow\n        } else {\n            displayStr = descriptor.replace("Ctrl", Strings.KEYBOARD_CTRL);   // Ctrl\n            displayStr = displayStr.replace("Shift", Strings.KEYBOARD_SHIFT); // Shift > shift symbol\n            displayStr = displayStr.replace("Space", Strings.KEYBOARD_SPACE); // Alt > option symbol\n            displayStr = displayStr.replace(/-(?!$)/g, "+");\n        }\n\n        return displayStr;\n    }\n\n    /**\n     * @private\n     * @param {string} A normalized key-description string.\n     * @return {boolean} true if the key is already assigned, false otherwise.\n     */\n    function _isKeyAssigned(key) {\n        return (_keyMap[key] !== undefined);\n    }\n\n    /**\n     * Remove a key binding from _keymap\n     *\n     * @param {!string} key - a key-description string that may or may not be normalized.\n     * @param {?string} platform - OS from which to remove the binding (all platforms if unspecified)\n     */\n    function removeBinding(key, platform) {\n        if (!key || ((platform !== null) && (platform !== undefined) && (platform !== global.app.platform))) {\n            return;\n        }\n\n        var normalizedKey = normalizeKeyDescriptorString(key);\n\n        if (!normalizedKey) {\n            console.log("Fail to nomalize " + key);\n        } else if (_isKeyAssigned(normalizedKey)) {\n            var binding = _keyMap[normalizedKey],\n                command = CommandManager.get(binding.commandID),\n                bindings = _commandMap[binding.commandID];\n\n            // delete key binding record\n            delete _keyMap[normalizedKey];\n\n            if (bindings) {\n                // delete mapping from command to key binding\n                _commandMap[binding.commandID] = bindings.filter(function (b) {\n                    return (b.key !== normalizedKey);\n                });\n\n                if (command) {\n                    $(command).triggerHandler("keyBindingRemoved", [{key: normalizedKey, displayKey: binding.displayKey}]);\n                }\n            }\n        }\n    }\n\n    /**\n     * @private\n     *\n     * @param {string} commandID\n     * @param {string|{{key: string, displayKey: string}}} keyBinding - a single shortcut.\n     * @param {?string} platform\n     *     - "all" indicates all platforms, not overridable\n     *     - undefined indicates all platforms, overridden by platform-specific binding\n     * @return {?{key: string, displayKey:String}} Returns a record for valid key bindings.\n     *     Returns null when key binding platform does not match, binding does not normalize,\n     *     or is already assigned.\n     */\n    function _addBinding(commandID, keyBinding, platform) {\n        var key,\n            result = null,\n            normalized,\n            normalizedDisplay,\n            explicitPlatform = keyBinding.platform || platform,\n            targetPlatform,\n            command,\n            bindingsToDelete = [],\n            existing;\n\n        // For platform: "all", use explicit current plaform\n        if (explicitPlatform && explicitPlatform !== "all") {\n            targetPlatform = explicitPlatform;\n        } else {\n            targetPlatform = global.app.platform;\n        }\n\n        // if the request does not specify an explicit platform, and we\'re\n        // currently on a mac, then replace Ctrl with Cmd.\n        key = (keyBinding.key) || keyBinding;\n        if (global.app.platform === "mac" && (explicitPlatform === undefined || explicitPlatform === "all")) {\n            key = key.replace("Ctrl", "Cmd");\n            if (keyBinding.displayKey !== undefined) {\n                keyBinding.displayKey = keyBinding.displayKey.replace("Ctrl", "Cmd");\n            }\n        }\n        normalized = normalizeKeyDescriptorString(key);\n\n        // skip if the key binding is invalid\n        if (!normalized) {\n            console.error("Unable to parse key binding " + key + ". Permitted modifiers: Ctrl, Cmd, Alt, Opt, Shift; separated by \'-\' (not \'+\').");\n            return null;\n        }\n\n        // check for duplicate key bindings\n        existing = _keyMap[normalized];\n\n        // for cross-platform compatibility\n        if (exports.useWindowsCompatibleBindings) {\n            // windows-only key bindings are used as the default binding\n            // only if a default binding wasn\'t already defined\n            if (explicitPlatform === "win") {\n                // search for a generic or platform-specific binding if it\n                // already exists\n                if (existing && (!existing.explicitPlatform ||\n                                 existing.explicitPlatform === global.app.platform ||\n                                 existing.explicitPlatform === "all")) {\n                    // do not clobber existing binding with windows-only binding\n                    return null;\n                }\n\n                // target this windows binding for the current platform\n                targetPlatform = global.app.platform;\n            }\n        }\n\n        // skip if this binding doesn\'t match the current platform\n        if (targetPlatform !== global.app.platform) {\n            return null;\n        }\n\n        // skip if the key is already assigned\n        if (existing) {\n            if (!existing.explicitPlatform && explicitPlatform) {\n                // remove the the generic binding to replace with this new platform-specific binding\n                removeBinding(normalized);\n                existing = false;\n            }\n        }\n\n        // delete existing bindings when\n        // (1) replacing a windows-compatible binding with a generic or\n        //     platform-specific binding\n        // (2) replacing a generic binding with a platform-specific binding\n        var existingBindings = _commandMap[commandID] || [],\n            isWindowsCompatible,\n            isReplaceGeneric,\n            ignoreGeneric;\n\n        existingBindings.forEach(function (binding) {\n            // remove windows-only bindings in _commandMap\n            isWindowsCompatible = exports.useWindowsCompatibleBindings &&\n                binding.explicitPlatform === "win";\n\n            // remove existing generic binding\n            isReplaceGeneric = !binding.explicitPlatform &&\n                explicitPlatform;\n\n            if (isWindowsCompatible || isReplaceGeneric) {\n                bindingsToDelete.push(binding);\n            } else {\n                // existing binding is platform-specific and the requested binding is generic\n                ignoreGeneric = binding.explicitPlatform && !explicitPlatform;\n            }\n        });\n\n        if (ignoreGeneric) {\n            // explicit command binding overrides this one\n            return null;\n        }\n\n        if (existing) {\n            // do not re-assign a key binding\n            console.error("Cannot assign " + normalized + " to " + commandID + ". It is already assigned to " + _keyMap[normalized].commandID);\n            return null;\n        }\n\n        // remove generic or windows-compatible bindings\n        bindingsToDelete.forEach(function (binding) {\n            removeBinding(binding.key);\n        });\n\n        // optional display-friendly string (e.g. CMD-+ instead of CMD-=)\n        normalizedDisplay = (keyBinding.displayKey) ? normalizeKeyDescriptorString(keyBinding.displayKey) : normalized;\n\n        // 1-to-many commandID mapping to key binding\n        if (!_commandMap[commandID]) {\n            _commandMap[commandID] = [];\n        }\n\n        result = {\n            key                 : normalized,\n            displayKey          : normalizedDisplay,\n            explicitPlatform    : explicitPlatform\n        };\n\n        _commandMap[commandID].push(result);\n\n        // 1-to-1 key binding to commandID\n        _keyMap[normalized] = {\n            commandID           : commandID,\n            key                 : normalized,\n            displayKey          : normalizedDisplay,\n            explicitPlatform    : explicitPlatform\n        };\n\n        // notify listeners\n        command = CommandManager.get(commandID);\n\n        if (command) {\n            $(command).triggerHandler("keyBindingAdded", [result]);\n        }\n\n        return result;\n    }\n\n    /**\n     * Returns a copy of the keymap\n     * @returns {!Object.<string, {commandID: string, key: string, displayKey: string}>}\n     */\n    function getKeymap() {\n        return $.extend({}, _keyMap);\n    }\n\n    /**\n     * Process the keybinding for the current key.\n     *\n     * @param {string} A key-description string.\n     * @return {boolean} true if the key was processed, false otherwise\n     */\n    function _handleKey(key) {\n        if (_enabled && _keyMap[key]) {\n            // The execute() function returns a promise because some commands are async.\n            // Generally, commands decide whether they can run or not synchronously,\n            // and reject immediately, so we can test for that synchronously.\n            var promise = CommandManager.execute(_keyMap[key].commandID);\n            return (promise.state() !== "rejected");\n        }\n        return false;\n    }\n\n    // TODO (issue #414): Replace this temporary fix with a more robust solution to handle focus and modality\n    /**\n     * Enable or disable key bindings. Clients such as dialogs may wish to disable\n     * global key bindings temporarily.\n     *\n     * @param {string} A key-description string.\n     * @return {boolean} true if the key was processed, false otherwise\n     */\n    function setEnabled(value) {\n        _enabled = value;\n    }\n\n    /**\n     * @private\n     *\n     * Sort objects by platform property. Objects with a platform property come\n     * before objects without a platform property.\n     */\n    function _sortByPlatform(a, b) {\n        var a1 = (a.platform) ? 1 : 0,\n            b1 = (b.platform) ? 1 : 0;\n        return b1 - a1;\n    }\n\n    /**\n     * Add one or more key bindings to a particular Command.\n     *\n     * @param {!string | Command} command - A command ID or command object\n     * @param {?({key: string, displayKey: string} | Array.<{key: string, displayKey: string, platform: string}>)} keyBindings\n     *     a single key binding or an array of keybindings. Example:\n     *     "Shift-Cmd-F". Mac and Win key equivalents are automatically\n     *     mapped to each other. Use displayKey property to display a different\n     *     string (e.g. "CMD+" instead of "CMD=").\n     * @param {?string} platform - the target OS of the keyBindings either\n     *     "mac", "win" or "linux". If undefined, all platforms not explicitly\n     *     defined will use the key binding.\n     * @return {{key: string, displayKey:String}|Array.<{key: string, displayKey:String}>}\n     *     Returns record(s) for valid key binding(s)\n     */\n    function addBinding(command, keyBindings, platform) {\n        var commandID           = "",\n            results;\n\n        if (!command) {\n            console.error("addBinding(): missing required parameter: command");\n            return;\n        }\n\n        if (!keyBindings) { return; }\n\n        if (typeof (command) === "string") {\n            commandID = command;\n        } else {\n            commandID = command.getID();\n        }\n\n        if (Array.isArray(keyBindings)) {\n            var keyBinding;\n            results = [];\n\n            // process platform-specific bindings first\n            keyBindings.sort(_sortByPlatform);\n\n            keyBindings.forEach(function addSingleBinding(keyBindingRequest) {\n                // attempt to add keybinding\n                keyBinding = _addBinding(commandID, keyBindingRequest, keyBindingRequest.platform);\n\n                if (keyBinding) {\n                    results.push(keyBinding);\n                }\n            });\n        } else {\n            results = _addBinding(commandID, keyBindings, platform);\n        }\n\n        return results;\n    }\n\n    /**\n     * Retrieve key bindings currently associated with a command\n     *\n     * @param {!string | Command} command - A command ID or command object\n     * @return {!Array.<{{key: string, displayKey: string}}>} An array of associated key bindings.\n     */\n    function getKeyBindings(command) {\n        var bindings    = [],\n            commandID   = "";\n\n        if (!command) {\n            console.error("getKeyBindings(): missing required parameter: command");\n            return [];\n        }\n\n        if (typeof (command) === "string") {\n            commandID = command;\n        } else {\n            commandID = command.getID();\n        }\n\n        bindings = _commandMap[commandID];\n        return bindings || [];\n    }\n\n    /**\n     * Adds default key bindings when commands are registered to CommandManager\n     * @param {$.Event} event jQuery event\n     * @param {Command} command Newly registered command\n     */\n    function _handleCommandRegistered(event, command) {\n        var commandId   = command.getID(),\n            defaults    = KeyboardPrefs[commandId];\n        if (defaults) {\n            addBinding(commandId, defaults);\n        }\n    }\n\n    /**\n     * Adds a global keydown hook that gets first crack at keydown events\n     * before standard keybindings do. This is intended for use by modal or\n     * semi-modal UI elements like dialogs or the code hint list that should\n     * execute before normal command bindings are run.\n     *\n     * The hook is passed one parameter, the original keyboard event. If the\n     * hook handles the event (or wants to block other global hooks from\n     * handling the event), it should return true. Note that this will *only*\n     * stop other global hooks and KeyBindingManager from handling the\n     * event; to prevent further event propagation, you will need to call\n     * stopPropagation(), stopImmediatePropagation(), and/or preventDefault()\n     * as usual.\n     *\n     * Multiple keydown hooks can be registered, and are executed in order,\n     * most-recently-added first.\n     *\n     * (We have to have a special API for this because (1) handlers are normally\n     * called in least-recently-added order, and we want most-recently-added;\n     * (2) native DOM events don\'t have a way for us to find out if\n     * stopImmediatePropagation()/stopPropagation() has been called on the\n     * event, so we have to have some other way for one of the hooks to\n     * indicate that it wants to block the other hooks from running.)\n     *\n     * @param {function(Event): boolean} hook The global hook to add.\n     */\n    function addGlobalKeydownHook(hook) {\n        _globalKeydownHooks.push(hook);\n    }\n\n    /**\n     * Removes a global keydown hook added by `addGlobalKeydownHook`.\n     * Does not need to be the most recently added hook.\n     *\n     * @param {function(Event): boolean} hook The global hook to remove.\n     */\n    function removeGlobalKeydownHook(hook) {\n        var index = _globalKeydownHooks.indexOf(hook);\n        if (index !== -1) {\n            _globalKeydownHooks.splice(index, 1);\n        }\n    }\n\n\n\n    function _inModalDialog() {\n        return $(".modal.instance").length > 0;\n    }\n\n    function _inEditMode() {\n        return (document.activeElement.nodeName === "TEXTAREA" ||\n                document.activeElement.nodeName === "INPUT");\n    }\n\n    function _isEditKey(event) {\n        return ((event.ctrlKey && event.which === KeyEvent.DOM_VK_C) ||\n                (event.ctrlKey && event.which === KeyEvent.DOM_VK_X) ||\n                (event.ctrlKey && event.which === KeyEvent.DOM_VK_V) ||\n                (event.ctrlKey && event.which === KeyEvent.DOM_VK_Z) ||\n                (event.ctrlKey && event.which === KeyEvent.DOM_VK_Y) ||\n                (event.which === KeyEvent.DOM_VK_DELETE) ||\n                (event.which === KeyEvent.DOM_VK_BACK_SPACE));\n    }\n\n    /**\n     * Handles a given keydown event, checking global hooks first before\n     * deciding to handle it ourselves.\n     * @param {Event} The keydown event to handle.\n     */\n    function _handleKeyEvent(event) {\n        // ONLY FOR LINUX: Allow Ctrl+C/X/V/Z/Y and Delete for Input/TextArea or Modal Dialog\n        if (app.platform === "linux" && (_inModalDialog() || (_inEditMode() && _isEditKey(event)))) {\n            // Allow default browser\'s actions\n        } else {\n            var i, handled = false;\n            for (i = _globalKeydownHooks.length - 1; i >= 0; i--) {\n                if (_globalKeydownHooks[i](event)) {\n                    handled = true;\n                    break;\n                }\n            }\n            if (!handled && _handleKey(_translateKeyboardEvent(event))) {\n                event.stopPropagation();\n                event.preventDefault();\n            }\n        }\n    }\n\n    AppInit.htmlReady(function () {\n        // Install keydown event listener.\n        window.document.body.addEventListener(\n            "keydown",\n            _handleKeyEvent,\n            true\n        );\n\n        exports.useWindowsCompatibleBindings = (global.app.platform !== "mac") &&\n            (global.app.platform !== "win");\n    });\n\n    $(CommandManager).on("commandRegistered", _handleCommandRegistered);\n\n    // unit test only\n    exports._reset = _reset;\n\n    // Define public API\n    exports.getKeymap = getKeymap;\n    exports.setEnabled = setEnabled;\n    exports.addBinding = addBinding;\n    exports.removeBinding = removeBinding;\n    exports.formatKeyDescriptor = formatKeyDescriptor;\n    exports.getKeyBindings = getKeyBindings;\n    exports.addGlobalKeydownHook = addGlobalKeydownHook;\n    exports.removeGlobalKeydownHook = removeGlobalKeydownHook;\n\n    /**\n     * Use windows-specific bindings if no other are found (e.g. Linux).\n     * Core Brackets modules that use key bindings should always define at\n     * least a generic keybinding that is applied for all platforms. This\n     * setting effectively creates a compatibility mode for third party\n     * extensions that define explicit key bindings for Windows and Mac, but\n     * not Linux.\n     */\n    exports.useWindowsCompatibleBindings = false;\n\n    // For unit testing only\n    exports._handleKey = _handleKey;\n    exports._handleKeyEvent = _handleKeyEvent;\n});\n\n//# sourceURL=/command/KeyBindingManager.js'),eval('define(\'text!htmlContent/dialog-template.html\',[],function () { return \'<div class="{{dlgClass}} template dialog modal" data-title="{{title}}">\\n    <div class="dialog-body">\\n        <div class="dialog-message">{{{message}}}</div>\\n    </div>\\n    {{#hasButtons}}\\n    <div class="dialog-footer">\\n        {{#buttons}}\\n        <button class="k-button dialog-button {{className}}" data-button-id="{{id}}">{{{text}}}</button>\\n        {{/buttons}}\\n    </div>\\n    {{/hasButtons}}\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/dialog-template.html'),eval('define(\'text!htmlContent/input-dialog.html\',[],function () { return \'<div class="input-dialog dialog modal" data-title="Input">\\n    <div class="dialog-body">\\n        <div class="dialog-message">{{{message}}}</div>\\n        <input type=\\\'text\\\' class=\\\'k-input k-textbox input-box\\\'>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.OK}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/input-dialog.html'),eval('define(\'text!htmlContent/text-dialog.html\',[],function () { return \'<div class="text-dialog dialog modal" data-title="Edit Text">\\n    <div class="dialog-body">\\n        <div class="dialog-message">{{{message}}}</div>\\n        <textarea type=\\\'text\\\' class=\\\'k-textbox text-box primary\\\'>{{{text}}}</textarea>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button" data-button-id="ok">{{Strings.OK}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/text-dialog.html'),eval('define(\'text!htmlContent/select-radio-dialog.html\',[],function () { return \'<div class="select-radio-dialog dialog modal" data-title="Select One">\\n    <div class="dialog-body">\\n        <div class="dialog-message">{{{message}}}</div>\\n        <form class=\\\'select-options-form\\\'>\\n            {{#options}}\\n            <label class="k-radio-button">\\n                <input type="radio" name="select-option" value="{{value}}" {{#checked}}checked{{/checked}}>\\n                <div class="k-radio-button__checkmark"></div>\\n                {{text}}\\n            </label>\\n            <br>\\n            {{/options}}\\n        </form>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.OK}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/select-radio-dialog.html'),eval('define(\'text!htmlContent/select-dropdown-dialog.html\',[],function () { return \'<div class="select-radio-dialog dialog modal" data-title="Select One">\\n    <div class="dialog-body">\\n        <div class="dialog-message">{{{message}}}</div>\\n        <label class="k-select">\\n            <select class="select-options">\\n                {{#options}}\\n                <option value=\\\'{{value}}\\\'>{{text}}</option>\\n                {{/options}}\\n            </select>\\n        </label>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.OK}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/select-dropdown-dialog.html'),eval('define(\'text!htmlContent/color-dialog.html\',[],function () { return \'<div class="color-dialog dialog modal" data-title="Color">\\n    <div class="dialog-body">\\n        <table>\\n            <tr>\\n                <td><div class="color-picker"></div></td>\\n                <td><div class="color-palette"></div></td>\\n            </tr>\\n        </table>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.OK}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/color-dialog.html'),eval('define(\'text!htmlContent/font-dialog.html\',[],function () { return \'<div class="font-dialog dialog modal" data-title="Font">\\n    <div class="dialog-body">\\n        <table>\\n            <tr>\\n                <td>\\n                    <div class="item-label">Face:</div>\\n                    <label class="font-face k-combo">\\n                        <input type="text" style="width: 100%;">\\n                        <select tabindex="-1">\\n                            {{#faces}}\\n                            <option value="{{value}}">{{text}}</option>\\n                            {{/faces}}\\n                        </select>\\n                    </label>\\n                </td>\\n                <td>\\n                    <div class="item-label">Size:</div>\\n                    <label class="font-size k-combo">\\n                        <input type="text" style="width: 100%;">\\n                        <select tabindex="-1">\\n                            {{#sizes}}\\n                            <option value=\\\'{{.}}\\\'>{{.}}</option>\\n                            {{/sizes}}\\n                        </select>\\n                    </label>\\n                </td>\\n                <td>\\n                    <div class="item-label">Color:</div>\\n                    <input class="font-color" type="color">\\n                </td>\\n            </tr>\\n        </table>\\n        <table>\\n            <tr>\\n                <td>\\n                    <div class="item-label">Preview:</div>\\n                    <div class=\\\'font-preview\\\'>AaBb123</div>\\n                </td>\\n                <td>\\n                    <div class="color-palette"></div>\\n                </td>\\n            </tr>\\n        </table>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.OK}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/font-dialog.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, app, Mustache */\n\ndefine(\'dialogs/Dialogs\',[\'require\',\'exports\',\'module\',\'core/Graphics\',\'utils/KeyEvent\',\'command/KeyBindingManager\',\'strings\',\'text!htmlContent/dialog-template.html\',\'text!htmlContent/input-dialog.html\',\'text!htmlContent/text-dialog.html\',\'text!htmlContent/select-radio-dialog.html\',\'text!htmlContent/select-dropdown-dialog.html\',\'text!htmlContent/color-dialog.html\',\'text!htmlContent/font-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var Graphics          = require("core/Graphics"),\n        KeyEvent          = require("utils/KeyEvent"),\n        KeyBindingManager = require("command/KeyBindingManager"),\n        Strings           = require("strings");\n\n    /**\n     * Dialog HTML Templates\n     * @private\n     */\n    var dialogTemplate               = require("text!htmlContent/dialog-template.html"),\n        inputDialogTemplate          = require("text!htmlContent/input-dialog.html"),\n        textDialogTemplate           = require("text!htmlContent/text-dialog.html"),\n        selectRadioDialogTemplate    = require("text!htmlContent/select-radio-dialog.html"),\n        selectDropdownDialogTemplate = require("text!htmlContent/select-dropdown-dialog.html"),\n        colorDialogTemplate          = require("text!htmlContent/color-dialog.html"),\n        fontDialogTemplate           = require("text!htmlContent/font-dialog.html");\n\n    /**\n     * Dialog Modal Results\n     * @const\n     */\n    var DIALOG_BTN_OK       = \'ok\',\n        DIALOG_BTN_CANCEL   = \'cancel\',\n        DIALOG_BTN_SAVE     = \'save\',\n        DIALOG_BTN_DONTSAVE = \'dontsave\',\n        DIALOG_CANCELED     = "_canceled";\n\n    /**\n     * Dialog Buttons Class Names\n     * @const {string}\n     */\n    var DIALOG_BTN_CLASS_PRIMARY    = "primary",\n        DIALOG_BTN_CLASS_NORMAL     = "",\n        DIALOG_BTN_CLASS_LEFT       = "left";\n\n    /**\n     * @private\n     * Dismises a modal dialog\n     * @param {$.Element} $dlg\n     * @param {string} buttonId\n     */\n    function _dismissDialog($dlg, buttonId) {\n        $dlg.data("buttonId", buttonId);\n        $dlg.data("kendoWindow").close();\n    }\n\n    /**\n     * @private\n     * If autoDismiss is true, then dismisses the dialog. Otherwise just raises an event that the\n     * given button was clicked.\n     * @param {$.Element} $dlg The dialog element to be dismissed.\n     * @param {string} buttonId The ID of the button that was clicked.\n     * @param {boolean} autoDismiss Whether to autodismiss the dialog on a button click.\n     */\n    function _processButton($dlg, buttonId, autoDismiss) {\n        if (autoDismiss) {\n            _dismissDialog($dlg, buttonId);\n        } else {\n            $dlg.triggerHandler("buttonClick", buttonId);\n        }\n    }\n\n    /**\n     * @private\n     * Handles the use of Tab so that it stays inside the Dialog\n     * @param {$.Event} event\n     * @param {$.Element} $dlg\n     */\n    function _handleTab(event, $dlg) {\n        var $inputs = $(":input:enabled, a", $dlg).filter(":visible");\n\n        if ($(event.target).closest($dlg).length) {\n            // If it\'s the first or last tabbable element, focus the last/first element\n            if ((!event.shiftKey && event.target === $inputs[$inputs.length - 1]) ||\n                    (event.shiftKey && event.target === $inputs[0])) {\n                $inputs.filter(event.shiftKey ? ":last" : ":first").focus();\n                event.preventDefault();\n\n            // If there is no element to focus, don\'t let it focus outside of the dialog\n            } else if (!$inputs.length) {\n                event.preventDefault();\n            }\n\n        // If the focus left the dialog, focus the first element in the dialog\n        } else {\n            $inputs.first().focus();\n            event.preventDefault();\n        }\n    }\n\n    /**\n     * @private\n     * Returns true if the modal dialog has a button with the given ID\n     * @param {$.Element} $dlg\n     * @param {string} buttonId\n     * @return {boolean}\n     */\n    function _hasButton($dlg, buttonId) {\n        return ($dlg.find("[data-button-id=\'" + buttonId + "\']").length > 0);\n    }\n\n    /**\n     * Handles the keyDown event for the dialogs\n     * @param {$.Event} e\n     * @param {boolean} autoDismiss\n     * @return {boolean}\n     */\n    var _keydownHook = function (e, autoDismiss) {\n        var $primaryBtn     = this.find(".primary"),\n            buttonId        = null,\n            which           = String.fromCharCode(e.which),\n            $focusedElement = this.find(".dialog-button:focus, a:focus");\n\n        // There might be a textfield in the dialog\'s UI; don\'t want to mistake normal typing for dialog dismissal\n        var inTextArea    = (e.target.tagName === "TEXTAREA" || (e.target.tagName === "INPUT" && e.target.className.indexOf("use-enter-key") > -1)),\n            inTypingField = inTextArea || ($(e.target).filter(":text, :password").length > 0);\n\n        if (e.which === KeyEvent.DOM_VK_TAB) {\n            _handleTab(e, this);\n        } else if (e.which === KeyEvent.DOM_VK_ESCAPE) {\n            buttonId = DIALOG_BTN_CANCEL;\n        } else if (e.which === KeyEvent.DOM_VK_RETURN && (!inTextArea || e.ctrlKey)) {\n            // Enter key in single-line text input always dismisses; in text area, only Ctrl+Enter dismisses\n            // Click primary\n            $primaryBtn.click();\n        } else if (e.which === KeyEvent.DOM_VK_SPACE) {\n            // Space bar on focused button or link\n            $focusedElement.click();\n        } else {\n            // CMD+D Don\'t Save\n            if (e.metaKey && (which === "D")) {\n                if (_hasButton(this, DIALOG_BTN_DONTSAVE)) {\n                    buttonId = DIALOG_BTN_DONTSAVE;\n                }\n            // FIXME (issue #418) CMD+. Cancel swallowed by native shell\n            } else if (e.metaKey && (e.which === KeyEvent.DOM_VK_PERIOD)) {\n                buttonId = DIALOG_BTN_CANCEL;\n            // \'N\' Don\'t Save\n            } else if (which === "N" && !inTypingField) {\n                if (_hasButton(this, DIALOG_BTN_DONTSAVE)) {\n                    buttonId = DIALOG_BTN_DONTSAVE;\n                }\n            }\n        }\n\n        if (buttonId) {\n            _processButton(this, buttonId, autoDismiss);\n        }\n\n        // Stop any other global hooks from processing the event (but\n        // allow it to continue bubbling if we haven\'t otherwise stopped it).\n        return true;\n    };\n\n    /**\n     * @constructor\n     * @private\n     *\n     * @param {$.Element} $dlg The dialog jQuery element\n     * @param {$.Promise} promise A promise that will be resolved with the ID of the clicked button when the dialog\n     *     is dismissed. Never rejected.\n     */\n    function Dialog($dlg, promise) {\n        this._$dlg    = $dlg;\n        this._promise = promise;\n    }\n\n    /** @type {$.Element} The dialog jQuery element */\n    Dialog.prototype.getElement = function () {\n        return this._$dlg;\n    };\n\n    /** @type {$.Promise} The dialog promise */\n    Dialog.prototype.getPromise = function () {\n        return this._promise;\n    };\n\n    /**\n     * Closes the dialog if is visible\n     */\n    Dialog.prototype.close = function () {\n        _dismissDialog(this._$dlg, DIALOG_BTN_CANCEL);\n    };\n\n    /**\n     * Adds a done callback to the dialog promise\n     */\n    Dialog.prototype.done = function (callback) {\n        this._promise.done(callback);\n    };\n\n\n    /**\n     * Show Modal Dialog using Template\n     * @param {string} template\n     * @param {boolean} autoDismiss\n     * @param {function($.Element)} beforeClosing\n     * @return {Dialog}\n     */\n    function showModalDialogUsingTemplate(template, autoDismiss, beforeClosing) {\n        if (autoDismiss === undefined) {\n            autoDismiss = true;\n        }\n\n        var result  = $.Deferred(),\n            promise = result.promise(),\n            dialog,\n            $dlg;\n\n        $dlg = $(template);\n        $dlg.addClass("instance");\n        $(window.document.body).append($dlg);\n\n        var keydownHook = function (e) {\n            return _keydownHook.call($dlg, e, autoDismiss);\n        };\n\n        // Build kendo modal window\n        $dlg.kendoWindow({\n            title: $dlg.data("title"),\n            modal: true,\n            resizable: false,\n            pinned: true,\n            close: function () {\n                // Call before closing\n                if (beforeClosing) {\n                    beforeClosing($dlg);\n                }\n\n                // Set button Id\n                var buttonId = $dlg.data("buttonId");\n                if (!buttonId) {\n                    buttonId = DIALOG_BTN_CANCEL;\n                }\n\n                // Dialog\'s return value\n                var value = $dlg.data("returnValue");\n\n                // Remove from DOM\n                this.destroy();\n\n                // Remove our global keydown handler.\n                KeyBindingManager.removeGlobalKeydownHook(keydownHook);\n\n                // Call promise\n                _.defer(function () {\n                    result.resolve(buttonId, value);\n                });\n            }\n        });\n\n        // Open dialog at screen center\n        dialog = $dlg.data("kendoWindow");\n        dialog.center();\n\n        // Click handler for buttons\n        $dlg.one("click", ".dialog-button", function (e) {\n            _processButton($dlg, $(this).data("button-id"), autoDismiss);\n        });\n\n        // Set focus to the default button\n        var primaryBtn = $dlg.find(".primary");\n        if (primaryBtn) {\n            primaryBtn.focus();\n        }\n\n        // Push our global keydown handler onto the global stack of handlers.\n        KeyBindingManager.addGlobalKeydownHook(keydownHook);\n\n        return (new Dialog($dlg, promise));\n    }\n\n    /**\n     * Show typical modal dialog\n     *\n     * @param {string} dlgClass\n     * @param {string} title\n     * @param {string} message\n     * @param {Array.<{id:string, text:string, className:string}>} buttons\n     * @param {boolean} autoDismiss\n     * @return {Dialog}\n     */\n    function showModalDialog(dlgClass, title, message, buttons, autoDismiss) {\n        var templateVars = {\n            dlgClass: dlgClass,\n            title:    title   || false,\n            message:  message || "",\n            buttons:  buttons || [{ id: DIALOG_BTN_OK, text: Strings.OK, className: DIALOG_BTN_CLASS_PRIMARY }],\n            hasButtons: true\n        };\n        if (templateVars.buttons.length === 0) {\n            templateVars.hasButtons = false;\n        }\n        var template = Mustache.render(dialogTemplate, templateVars);\n        return showModalDialogUsingTemplate(template, autoDismiss);\n    }\n\n    /**\n     * Immediately closes any dialog instances with the given class. The dialog callback for each instance will\n     * be called with the special buttonId DIALOG_CANCELED (note: callback is run asynchronously).\n     * @param {string} dlgClass The class name identifier for the dialog.\n     * @param {string=} buttonId The button id to use when closing the dialog. Defaults to DIALOG_CANCELED\n     */\n    function cancelModalDialogIfOpen(dlgClass, buttonId) {\n        $("." + dlgClass + ".instance").each(function () {\n            if ($(this).is(":visible")) {   // Bootstrap breaks if try to hide dialog that\'s already hidden\n                _dismissDialog($(this), buttonId || DIALOG_CANCELED);\n            }\n        });\n    }\n\n    /**\n     * Show Save Confirmation Dialog\n     * @param {string} filename\n     * @return {Dialog}\n     */\n    function showSaveConfirmDialog(filename) {\n        var message = "Do you want to save the changes you made";\n        if (filename) {\n            message += " in<br>" + filename + "?";\n        } else {\n            message += "?";\n        }\n        return showModalDialog(\n            "save-change-dialog",\n            "Save Changes",\n            message,\n            [\n                { id: DIALOG_BTN_DONTSAVE, text: Strings.DONTSAVE, className: DIALOG_BTN_CLASS_LEFT },\n                { id: DIALOG_BTN_SAVE,     text: Strings.SAVE,     className: DIALOG_BTN_CLASS_PRIMARY },\n                { id: DIALOG_BTN_CANCEL,   text: Strings.CANCEL }\n            ]\n        );\n    }\n\n    /**\n     * Show Confirm Dialog\n     *\n     * @param {string} message\n     * @return {Dialog}\n     */\n    function showConfirmDialog(message) {\n        return showModalDialog(\n            "confirm-dialog",\n            "Confirmation",\n            message,\n            [\n                { id: DIALOG_BTN_OK,     text: Strings.OK, className: DIALOG_BTN_CLASS_PRIMARY },\n                { id: DIALOG_BTN_CANCEL, text: Strings.CANCEL }\n            ]\n        );\n    }\n\n    /**\n     * Show Error Dialog\n     * @param {string} message\n     * @return {Dialog}\n     */\n    function showErrorDialog(message) {\n        return showModalDialog("error-dialog", "Error", message);\n    }\n\n    /**\n     * Show Alert Dialog\n     * @param {string} message\n     * @return {Dialog}\n     */\n    function showAlertDialog(message) {\n        return showModalDialog("alert-dialog", "Alert", message);\n    }\n\n    /**\n     * Show Information Dialog\n     * @param {string} message\n     * @return {Dialog}\n     */\n    function showInfoDialog(message) {\n        return showModalDialog("information-dialog", "Information", message);\n    }\n\n    /**\n     * Show Input Dialog\n     *\n     * @param {string} message\n     * @return {Dialog}\n     */\n    function showInputDialog(message, initialValue) {\n        var context = {\n            Strings: Strings,\n            message: message\n        };\n        var dialog = showModalDialogUsingTemplate(Mustache.render(inputDialogTemplate, context), true, function ($dlg) {\n            var val = $dlg.find(".input-box").val();\n            $dlg.data("returnValue", val);\n        });\n        var $dlg = dialog.getElement(),\n            $input = $dlg.find(".input-box");\n\n        if (initialValue) {\n            $input.val(initialValue);\n            $input.select();\n        }\n\n        // Focus on input\n        $input.focus();\n\n        // Keydown Event\n        // TODO: This is temporal implementation.\n        // Please refer to KeyBindingManager.js of Brackets\' implementation\n        $input.keydown(function (event) {\n            switch (event.which) {\n            case KeyEvent.DOM_VK_RETURN:\n                $dlg.find(".primary").click();\n                break;\n            }\n        });\n\n        return dialog;\n    }\n\n    /**\n     * Show Text Dialog (Multiline Text)\n     * @param {string} message\n     * @param {string} text initial text\n     * @return {Dialog}\n     */\n    function showTextDialog(message, text) {\n        var context = {\n            Strings: Strings,\n            message: message,\n            text: text\n        };\n        var dialog = showModalDialogUsingTemplate(Mustache.render(textDialogTemplate, context), true, function ($dlg) {\n            var val = $dlg.find(".text-box").val();\n            $dlg.data("returnValue", val);\n        });\n        return dialog;\n    }\n\n    /**\n     * Show Select Dialog (Radio-type)\n     *\n     * @param {string} message\n     * @param {Array.<{text:string, value:string}>} items\n     * @return {Dialog}\n     */\n    function showSelectRadioDialog(message, options) {\n        var context = {\n            Strings: Strings,\n            message: message,\n            options: options\n        };\n        var dialog = showModalDialogUsingTemplate(Mustache.render(selectRadioDialogTemplate, context), true, function ($dlg) {\n            var val = $("input[name=select-option]:checked", $dlg).val();\n            $dlg.data("returnValue", val);\n        });\n        return dialog;\n    }\n\n    /**\n     * Show Select Dialog (Dropdown-type)\n     *\n     * @param {string} message\n     * @param {Array.<{text:string, value:string}>} items\n     * @return {Dialog}\n     */\n    function showSelectDropdownDialog(message, options) {\n        var context = {\n            Strings: Strings,\n            message: message,\n            options: options\n        };\n        var dialog = showModalDialogUsingTemplate(Mustache.render(selectDropdownDialogTemplate, context), true, function ($dlg) {\n            var $dropdown = $dlg.data("dropdown");\n            $dlg.data("returnValue", $dropdown.val());\n        });\n        // Setup Dropdown Widget\n        var $dlg = dialog.getElement(),\n            $dropdown = $dlg.find(".select-options");\n        $dlg.data("dropdown", $dropdown);\n        return dialog;\n    }\n\n    /**\n     * Show Color Dialog\n     *\n     * @param {string} color initial color\n     * @return {Dialog}\n     */\n    function showColorDialog(color) {\n        var context = { Strings: Strings };\n        var dialog = showModalDialogUsingTemplate(Mustache.render(colorDialogTemplate, context), true, function ($dlg) {\n            var colorPicker = $dlg.data("colorPicker");\n            $dlg.data("returnValue", colorPicker.value());\n        });\n        var $dlg = dialog.getElement(),\n            $picker = $dlg.find(".color-picker"),\n            $palette = $dlg.find(".color-palette"),\n            colorPicker;\n        // Setup Color Picker\n        $picker.kendoFlatColorPicker();\n        colorPicker = $picker.data("kendoFlatColorPicker");\n        colorPicker.value(color);\n        $dlg.data("colorPicker", colorPicker);\n        // Setup Color Palette\n        $palette.kendoColorPalette({\n            columns: 16,\n            tileSize: 8,\n            palette: Graphics.DEFAULT_COLOR_PALETTE,\n            change: function (e) {\n                colorPicker.value(e.value);\n            }\n        });\n        return dialog;\n    }\n\n    /**\n     * Show Font Dialog\n     *\n     * @param {Graphics.Font} font Initial font\n     * @return {Dialog}\n     */\n    function showFontDialog(font) {\n        var context = {\n            Strings: Strings,\n            faces: app.config.defaultFonts,\n            sizes: [ 8, 9, 10, 11, 12, 13, 15, 16, 18, 20, 22 ]\n        };\n\n        var dialog = showModalDialogUsingTemplate(Mustache.render(fontDialogTemplate, context), true, function ($dlg) {\n            var font = {\n                face: $dlg.data("fontFace").val(),\n                size: $dlg.data("fontSize").val(),\n                color: $dlg.data("fontColor").value()\n            };\n            $dlg.data("returnValue", font);\n        });\n\n        var $dlg = dialog.getElement(),\n            $fontFaceEdit   = $dlg.find(".font-face > input"),\n            $fontFaceSelect = $dlg.find(".font-face > select"),\n            $fontSizeEdit   = $dlg.find(".font-size > input"),\n            $fontSizeSelect = $dlg.find(".font-size > select"),\n            $fontColor = $dlg.find(".font-color"),\n            $colorPalette = $dlg.find(".color-palette"),\n            $fontPreview = $dlg.find(".font-preview");\n\n        // Setup Font Face\n        $fontFaceEdit.change(function () {\n            $fontPreview.css("font-family", $fontFaceEdit.val());\n        });\n        $fontFaceSelect.change(function () {\n            $fontFaceEdit.val($fontFaceSelect.val());\n            $fontFaceEdit.change();\n        });\n        $dlg.data("fontFace", $fontFaceEdit);\n\n        // Setup Font Size\n        $fontSizeEdit.change(function () {\n            if (_.isNumber(parseInt($fontSizeEdit.val()))) {\n                $fontPreview.css("font-size", $fontSizeEdit.val() + "px");\n            }\n        });\n        $fontSizeSelect.change(function () {\n            $fontSizeEdit.val($fontSizeSelect.val());\n            $fontSizeEdit.change();\n        });\n        $dlg.data("fontSize", $fontSizeEdit);\n\n        // Setup Font Color\n        $fontColor.kendoColorPicker({\n            change: function (e) {\n                $fontPreview.css("color", e.value);\n            }\n        });\n        $dlg.data("fontColor", $fontColor.data("kendoColorPicker"));\n\n        // Setup Color Palette\n        $colorPalette.kendoColorPalette({\n            columns: 16,\n            tileSize: 8,\n            palette: Graphics.DEFAULT_COLOR_PALETTE,\n            change: function (e) {\n                $fontColor.data("kendoColorPicker").value(e.value);\n                $fontPreview.css("color", e.value);\n            }\n        });\n\n        // Set initial font\n        if (font.face !== null) {\n            $fontFaceEdit.val(font.face);\n            $fontFaceSelect.val(font.face);\n        } else {\n            $fontFaceEdit.prop("placeholder", "—");\n            $fontFaceEdit.val("");\n            $fontFaceSelect.val(null);\n        }\n\n        if (_.isNumber(font.size)) {\n            $fontSizeEdit.val(font.size);\n            $fontSizeSelect.val(font.size);\n        } else {\n            $fontSizeEdit.prop("placeholder", "—");\n            $fontSizeEdit.val("");\n            $fontSizeSelect.val(null);\n        }\n\n        if (font.color) {\n            $fontColor.data("kendoColorPicker").value(font.color);\n        }\n\n        // Update Preview\n        function updatePreview() {\n            var face = $fontFaceEdit.val();\n            var size = $fontSizeEdit.val();\n            var color = $fontColor.data("kendoColorPicker").value();\n            $fontPreview.css("font-family", face);\n            $fontPreview.css("font-size", size + "px");\n            $fontPreview.css("color", color);\n        }\n        updatePreview();\n        return dialog;\n    }\n\n    /**\n     * Show Simple Dialog\n     * @param {string} message\n     * @return {Dialog}\n     */\n    function showSimpleDialog(message) {\n        return showModalDialog(\n            "simple-dialog",\n            false,\n            message,\n            []\n        );\n    }\n\n    // Public API\n\n    exports.DIALOG_BTN_OK            = DIALOG_BTN_OK;\n    exports.DIALOG_BTN_CANCEL        = DIALOG_BTN_CANCEL;\n    exports.DIALOG_BTN_SAVE          = DIALOG_BTN_SAVE;\n    exports.DIALOG_BTN_DONTSAVE      = DIALOG_BTN_DONTSAVE;\n\n    exports.DIALOG_BTN_CLASS_PRIMARY     = DIALOG_BTN_CLASS_PRIMARY;\n    exports.DIALOG_BTN_CLASS_NORMAL      = DIALOG_BTN_CLASS_NORMAL;\n    exports.DIALOG_BTN_CLASS_LEFT        = DIALOG_BTN_CLASS_LEFT;\n\n    exports.showModalDialogUsingTemplate = showModalDialogUsingTemplate;\n    exports.showModalDialog              = showModalDialog;\n    exports.cancelModalDialogIfOpen      = cancelModalDialogIfOpen;\n\n    exports.showSaveConfirmDialog        = showSaveConfirmDialog;\n    exports.showConfirmDialog            = showConfirmDialog;\n    exports.showErrorDialog              = showErrorDialog;\n    exports.showAlertDialog              = showAlertDialog;\n    exports.showInfoDialog               = showInfoDialog;\n    exports.showInputDialog              = showInputDialog;\n    exports.showTextDialog               = showTextDialog;\n    exports.showSelectRadioDialog        = showSelectRadioDialog;\n    exports.showSelectDropdownDialog     = showSelectDropdownDialog;\n    exports.showColorDialog              = showColorDialog;\n    exports.showFontDialog               = showFontDialog;\n    exports.showSimpleDialog             = showSimpleDialog;\n\n});\n\n//# sourceURL=/dialogs/Dialogs.js'),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, $, window */\n\ndefine(\'utils/ViewUtils\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n    var SCROLL_SHADOW_HEIGHT = 5;\n\n    /**\n     * @private\n     */\n    var _resizeHandlers = [];\n\n    /**\n     * Positions shadow background elements to indicate vertical scrolling.\n     * @param {!DOMElement} $displayElement the DOMElement that displays the shadow\n     * @param {!Object} $scrollElement the object that is scrolled\n     * @param {!DOMElement} $shadowTop div .scroller-shadow.top\n     * @param {!DOMElement} $shadowBottom div .scroller-shadow.bottom\n     * @param {boolean} isPositionFixed When using absolute position, top remains at 0.\n     */\n    function _updateScrollerShadow($displayElement, $scrollElement, $shadowTop, $shadowBottom, isPositionFixed) {\n        var offsetTop           = 0,\n            scrollElement       = $scrollElement.get(0),\n            scrollTop           = scrollElement.scrollTop,\n            topShadowOffset     = Math.min(scrollTop - SCROLL_SHADOW_HEIGHT, 0),\n            displayElementWidth = $displayElement.width();\n\n        if ($shadowTop) {\n            $shadowTop.css("background-position", "0px " + topShadowOffset + "px");\n\n            if (isPositionFixed) {\n                offsetTop = $displayElement.offset().top;\n                $shadowTop.css("top", offsetTop);\n            }\n\n            if (isPositionFixed) {\n                $shadowTop.css("width", displayElementWidth);\n            }\n        }\n\n        if ($shadowBottom) {\n            var clientHeight        = scrollElement.clientHeight,\n                outerHeight         = $displayElement.outerHeight(),\n                scrollHeight        = scrollElement.scrollHeight,\n                bottomOffset        = outerHeight - clientHeight,\n                bottomShadowOffset  = SCROLL_SHADOW_HEIGHT; // outside of shadow div viewport\n\n            if (scrollHeight > clientHeight) {\n                bottomShadowOffset -= Math.min(SCROLL_SHADOW_HEIGHT, (scrollHeight - (scrollTop + clientHeight)));\n            }\n\n            $shadowBottom.css("background-position", "0px " + bottomShadowOffset + "px");\n            $shadowBottom.css("top", offsetTop + outerHeight - SCROLL_SHADOW_HEIGHT);\n            $shadowBottom.css("width", displayElementWidth);\n        }\n    }\n\n    function getOrCreateShadow($displayElement, position, isPositionFixed) {\n        var $findShadow = $displayElement.find(".scroller-shadow." + position);\n\n        if ($findShadow.length === 0) {\n            $findShadow = $(window.document.createElement("div")).addClass("scroller-shadow " + position);\n            $displayElement.append($findShadow);\n        }\n\n        if (!isPositionFixed) {\n            // position is fixed by default\n            $findShadow.css("position", "absolute");\n            $findShadow.css(position, "0");\n        }\n\n        return $findShadow;\n    }\n\n    /**\n     * Installs event handlers for updatng shadow background elements to indicate vertical scrolling.\n     * @param {!DOMElement} displayElement the DOMElement that displays the shadow. Must fire\n     *  "contentChanged" events when the element is resized or repositioned.\n     * @param {?Object} scrollElement the object that is scrolled. Must fire "scroll" events\n     *  when the element is scrolled. If null, the displayElement is used.\n     * @param {?boolean} showBottom optionally show the bottom shadow\n     */\n    function addScrollerShadow(displayElement, scrollElement, showBottom) {\n        // use fixed positioning when the display and scroll elements are the same\n        var isPositionFixed = false;\n\n        if (!scrollElement) {\n            scrollElement = displayElement;\n            isPositionFixed = true;\n        }\n\n        // update shadows when the scrolling element is scrolled\n        var $displayElement = $(displayElement),\n            $scrollElement = $(scrollElement);\n\n        var $shadowTop = getOrCreateShadow($displayElement, "top", isPositionFixed);\n        var $shadowBottom = (showBottom) ? getOrCreateShadow($displayElement, "bottom", isPositionFixed) : null;\n\n        var doUpdate = function () {\n            _updateScrollerShadow($displayElement, $scrollElement, $shadowTop, $shadowBottom, isPositionFixed);\n        };\n\n        $scrollElement.on("scroll.scroller-shadow", doUpdate);\n        $displayElement.on("contentChanged.scroller-shadow", doUpdate);\n\n        // update immediately\n        doUpdate();\n    }\n\n    /**\n     * Remove scroller-shadow effect.\n     * @param {!DOMElement} displayElement the DOMElement that displays the shadow\n     * @param {?Object} scrollElement the object that is scrolled\n     */\n    function removeScrollerShadow(displayElement, scrollElement) {\n        if (!scrollElement) {\n            scrollElement = displayElement;\n        }\n\n        var $displayElement = $(displayElement),\n            $scrollElement = $(scrollElement);\n\n        // remove scrollerShadow elements from DOM\n        $displayElement.find(".scroller-shadow.top").remove();\n        $displayElement.find(".scroller-shadow.bottom").remove();\n\n        // remove event handlers\n        $scrollElement.off("scroll.scroller-shadow");\n        $displayElement.off("contentChanged.scroller-shadow");\n    }\n\n    /**\n     * Utility function to replace jQuery.toggleClass when used with the second argument, which needs to be a true boolean for jQuery\n     * @param {!jQueryObject} $domElement The jQueryObject to toggle the Class on\n     * @param {!string} className Class name or names (separated by spaces) to toggle\n     * @param {!boolean} addClass A truthy value to add the class and a falsy value to remove the class\n     */\n    function toggleClass($domElement, className, addClass) {\n        if (addClass) {\n            $domElement.addClass(className);\n        } else {\n            $domElement.removeClass(className);\n        }\n    }\n\n    /**\n     * Within a scrolling DOMElement, creates and positions a styled selection\n     * div to align a single selected list item from a ul list element.\n     *\n     * Assumptions:\n     * - scrollerElement is a child of the #sidebar div\n     * - ul list element fires a "selectionChanged" event after the\n     *   selectedClassName is assigned to a new list item\n     *\n     * @param {!DOMElement} scrollElement A DOMElement containing a ul list element\n     * @param {!string} selectedClassName A CSS class name on at most one list item in the contained list\n     */\n    function sidebarList($scrollerElement, selectedClassName, leafClassName) {\n        var $listElement = $scrollerElement.find("ul"),\n            $selectionMarker,\n            $selectionTriangle,\n            $sidebar = $("#sidebar"),\n            showTriangle = true;\n\n        // build selectionMarker and position absolute within the scroller\n        $selectionMarker = $(window.document.createElement("div")).addClass("sidebar-selection");\n        $scrollerElement.prepend($selectionMarker);\n\n        // enable scrolling\n        $scrollerElement.css("overflow", "auto");\n\n        // use relative postioning for clipping the selectionMarker within the scrollElement\n        $scrollerElement.css("position", "relative");\n\n        // build selectionTriangle and position fixed to the window\n        $selectionTriangle = $(window.document.createElement("div")).addClass("sidebar-selection-triangle");\n\n        $scrollerElement.append($selectionTriangle);\n\n        selectedClassName = "." + (selectedClassName || "selected");\n\n        var updateSelectionTriangle = function () {\n            var selectionMarkerHeight = $selectionMarker.height(),\n                selectionMarkerOffset = $selectionMarker.offset(),  // offset relative to *document*\n                scrollerOffset = $scrollerElement.offset(),\n                triangleHeight = $selectionTriangle.outerHeight(),\n                scrollerTop = scrollerOffset.top,\n                scrollerBottom = scrollerTop + $scrollerElement.outerHeight(),\n                scrollerLeft = scrollerOffset.left,\n                triangleTop = selectionMarkerOffset.top;\n\n            $selectionTriangle.css("top", triangleTop);\n            $selectionTriangle.css("left", $sidebar.width() - $selectionTriangle.outerWidth());\n            toggleClass($selectionTriangle, "triangle-visible", showTriangle);\n\n            var triangleClipOffsetYBy = Math.floor((selectionMarkerHeight - triangleHeight) / 2),\n                triangleBottom = triangleTop + triangleHeight + triangleClipOffsetYBy;\n\n            if (triangleTop < scrollerTop || triangleBottom > scrollerBottom) {\n                $selectionTriangle.css("clip", "rect(" + Math.max(scrollerTop - triangleTop - triangleClipOffsetYBy, 0) + "px, auto, " +\n                                           (triangleHeight - Math.max(triangleBottom - scrollerBottom, 0)) + "px, auto)");\n            } else {\n                $selectionTriangle.css("clip", "");\n            }\n        };\n\n        var updateSelectionMarker = function (event, reveal) {\n            // find the selected list item\n            var $listItem = $listElement.find(selectedClassName).closest("li");\n\n            if (leafClassName) {\n                showTriangle = $listItem.hasClass(leafClassName);\n            }\n\n            // always hide selection visuals first to force layout (issue #719)\n            $selectionTriangle.hide();\n            $selectionMarker.hide();\n\n            if ($listItem.length === 1) {\n                // list item position is relative to scroller\n                var selectionMarkerTop = $listItem.offset().top - $scrollerElement.offset().top + $scrollerElement.get(0).scrollTop;\n\n                // force selection width to match scroller\n                $selectionMarker.width($scrollerElement.get(0).scrollWidth);\n\n                // move the selectionMarker position to align with the list item\n                $selectionMarker.css("top", selectionMarkerTop);\n                $selectionMarker.show();\n\n                updateSelectionTriangle();\n                $selectionTriangle.show();\n\n                // fully scroll to the selectionMarker if it\'s not initially in the viewport\n                var scrollerElement = $scrollerElement.get(0),\n                    scrollerHeight = scrollerElement.clientHeight,\n                    selectionMarkerHeight = $selectionMarker.height(),\n                    selectionMarkerBottom = selectionMarkerTop + selectionMarkerHeight,\n                    currentScrollBottom = scrollerElement.scrollTop + scrollerHeight;\n\n                // update scrollTop to reveal the selected list item\n                if (reveal) {\n                    if (selectionMarkerTop >= currentScrollBottom) {\n                        $listItem.get(0).scrollIntoView(false);\n                    } else if (selectionMarkerBottom <= scrollerElement.scrollTop) {\n                        $listItem.get(0).scrollIntoView(true);\n                    }\n                }\n            }\n        };\n\n        $listElement.on("selectionChanged", updateSelectionMarker);\n        $scrollerElement.on("scroll", updateSelectionTriangle);\n        $scrollerElement.on("selectionRedraw", updateSelectionTriangle);\n\n        // update immediately\n        updateSelectionMarker();\n\n        // update clipping when the window resizes\n        _resizeHandlers.push(updateSelectionTriangle);\n    }\n\n    /**\n     * @private\n     */\n    function _handleResize() {\n        _resizeHandlers.forEach(function (f) {\n            f.apply();\n        });\n    }\n\n    /**\n     * Determine how much of an element rect is clipped in view.\n     *\n     * @param {!DOMElement} $view - A jQuery scrolling container\n     * @param {!{top: number, left: number, height: number, width: number}}\n     *          elementRect - rectangle of element\'s default position/size\n     * @return {{top: number, right: number, bottom: number, left: number}}\n     *          amount element rect is clipped in each direction\n     */\n    function getElementClipSize($view, elementRect) {\n        var delta,\n            clip = { top: 0, right: 0, bottom: 0, left: 0 },\n            viewOffset = $view.offset() || { top: 0, left: 0},\n            viewScroller = $view.get(0);\n\n        // Check if element extends below viewport\n        delta = (elementRect.top + elementRect.height) - (viewOffset.top + $view.height());\n        if (delta > 0) {\n            clip.bottom = delta;\n        }\n\n        // Check if element extends above viewport\n        delta = viewOffset.top - elementRect.top;\n        if (delta > 0) {\n            clip.top = delta;\n        }\n\n        // Check if element extends to the left of viewport\n        delta = viewOffset.left - elementRect.left;\n        if (delta > 0) {\n            clip.left = delta;\n        }\n\n        // Check if element extends to the right of viewport\n        delta = (elementRect.left + elementRect.width) - (viewOffset.left + $view.width());\n        if (delta > 0) {\n            clip.right = delta;\n        }\n\n        return clip;\n    }\n\n    /**\n     * Within a scrolling DOMElement, if necessary, scroll element into viewport.\n     *\n     * To Perform the minimum amount of scrolling necessary, cases should be handled as follows:\n     * - element already completely in view : no scrolling\n     * - element above    viewport          : scroll view so element is at top\n     * - element left of  viewport          : scroll view so element is at left\n     * - element below    viewport          : scroll view so element is at bottom\n     * - element right of viewport          : scroll view so element is at right\n     *\n     * Assumptions:\n     * - $view is a scrolling container\n     *\n     * @param {!DOMElement} $view - A jQuery scrolling container\n     * @param {!DOMElement} $element - A jQuery element\n     * @param {?boolean} scrollHorizontal - whether to also scroll horizontally\n     */\n    function scrollElementIntoView($view, $element, scrollHorizontal) {\n        var viewOffset = $view.offset(),\n            viewScroller = $view.get(0),\n            element = $element.get(0),\n            elementOffset = $element.offset();\n\n        // scroll minimum amount\n        var elementRect = {\n                top:    elementOffset.top,\n                left:   elementOffset.left,\n                height: $element.height(),\n                width:  $element.width()\n            },\n            clip = getElementClipSize($view, elementRect);\n\n        if (clip.bottom > 0) {\n            // below viewport\n            $view.scrollTop($view.scrollTop() + clip.bottom);\n        } else if (clip.top > 0) {\n            // above viewport\n            $view.scrollTop($view.scrollTop() - clip.top);\n        }\n\n        if (scrollHorizontal) {\n            if (clip.left > 0) {\n                $view.scrollLeft($view.scrollLeft() - clip.left);\n            } else if (clip.right > 0) {\n                $view.scrollLeft($view.scrollLeft() + clip.right);\n            }\n        }\n    }\n\n    /**\n     * HTML formats a file entry name  for display in the sidebar.\n     * @param {!File} entry File entry to display\n     * @return {string} HTML formatted string\n     */\n    function getFileEntryDisplay(entry) {\n        var name = entry.name,\n            i = name.lastIndexOf(".");\n\n        if (i >= 0) {\n            // Escape all HTML-sensitive characters in filename.\n            name = _.escape(name.substring(0, i)) + "<span class=\'extension\'>" + _.escape(name.substring(i)) + "</span>";\n        } else {\n            name = _.escape(name);\n        }\n\n        return name;\n    }\n\n    /**\n     * Determine the minimum directory path to distinguish duplicate file names\n     * for each file in list.\n     *\n     * @param {Array.<File>} files - list of Files with the same filename\n     * @return {Array.<string>} directory paths to match list of files\n     */\n    function getDirNamesForDuplicateFiles(files) {\n        // Must have at least two files in list for this to make sense\n        if (files.length <= 1) {\n            return [];\n        }\n\n        // First collect paths from the list of files and fill map with them\n        var map = {}, filePaths = [], displayPaths = [];\n        files.forEach(function (file, index) {\n            var fp = file.fullPath.split("/");\n            fp.pop(); // Remove the filename itself\n            displayPaths[index] = fp.pop();\n            filePaths[index] = fp;\n\n            if (!map[displayPaths[index]]) {\n                map[displayPaths[index]] = [index];\n            } else {\n                map[displayPaths[index]].push(index);\n            }\n        });\n\n        // This function is used to loop through map and resolve duplicate names\n        var processMap = function (map) {\n            var didSomething = false;\n            _.forEach(map, function (arr, key) {\n                // length > 1 means we have duplicates that need to be resolved\n                if (arr.length > 1) {\n                    arr.forEach(function (index) {\n                        if (filePaths[index].length !== 0) {\n                            displayPaths[index] = filePaths[index].pop() + "/" + displayPaths[index];\n                            didSomething = true;\n\n                            if (!map[displayPaths[index]]) {\n                                map[displayPaths[index]] = [index];\n                            } else {\n                                map[displayPaths[index]].push(index);\n                            }\n                        }\n                    });\n                }\n                delete map[key];\n            });\n            return didSomething;\n        };\n\n        var repeat;\n        do {\n            repeat = processMap(map);\n        } while (repeat);\n\n        return displayPaths;\n    }\n\n    // handle all resize handlers in a single listener\n    $(window).resize(_handleResize);\n\n    // Define public API\n    exports.SCROLL_SHADOW_HEIGHT         = SCROLL_SHADOW_HEIGHT;\n    exports.addScrollerShadow            = addScrollerShadow;\n    exports.removeScrollerShadow         = removeScrollerShadow;\n    exports.sidebarList                  = sidebarList;\n    exports.scrollElementIntoView        = scrollElementIntoView;\n    exports.getElementClipSize           = getElementClipSize;\n    exports.getFileEntryDisplay          = getFileEntryDisplay;\n    exports.toggleClass                  = toggleClass;\n    exports.getDirNamesForDuplicateFiles = getDirNamesForDuplicateFiles;\n});\n\n//# sourceURL=/utils/ViewUtils.js'),eval('define(\'text!htmlContent/element-picker-dialog.html\',[],function () { return \'<div class="element-picker dialog modal" data-title="{{title}}">\\n\\n    <div class="quick-search-wrapper"style="position: absolute; margin: 0; left: 15px; right: 15px; top: 4px;">\\n        <input class="quick-search use-enter-key" type="search" title="Quick Search">\\n    </div>\\n\\n    <div class="dialog-body no-padding">\\n        <div class="treeview-wrapper">\\n            <div class="treeview"></div>\\n        </div>\\n    </div>\\n    <div class="dialog-footer">\\n        <div class="left" style="text-align: left;">\\n            <label class="k-checkbox left">\\n                <input class="unspecified" type="checkbox">\\n                <div class="k-checkbox__checkmark"></div>\\n                <span style="padding-left: 7px;">Do not specify</span>\\n            </label>\\n        </div>\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.OK}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/element-picker-dialog.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache */\n\n\n// TODO: 키보드 처리 (ESC = Cancel, Enter = OK)\n// TODO: getChildren, getText, getIcon, getNode는 ModelExplorer와 중복 코드임. (중복제거요망)\n\n/**\n * Element Picker Dialog\n */\ndefine(\'dialogs/ElementPickerDialog\',[\'require\',\'exports\',\'module\',\'core/Repository\',\'engine/ProjectManager\',\'strings\',\'dialogs/Dialogs\',\'utils/ViewUtils\',\'text!htmlContent/element-picker-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var Repository     = require("core/Repository"),\n        ProjectManager = require("engine/ProjectManager"),\n        Strings        = require("strings"),\n        Dialogs        = require("dialogs/Dialogs"),\n        ViewUtils      = require("utils/ViewUtils");\n\n    var dialogTemplate = require("text!htmlContent/element-picker-dialog.html");\n\n    /**\n     * DataSource for TreeView\n     * @private\n     * @type {kendo.data.HierarchicalDataSource}\n     */\n    var _dataSource = null;\n\n    /**\n     * JQuery for Tree Wrapper\n     * @private\n     * @type {JQuery}\n     */\n    var $treeWrapper = null;\n\n    /**\n     * JQuery for Tree\n     * @private\n     * @type {JQuery}\n     */\n    var $tree = null;\n\n    /**\n     * KendoTreeView object\n     * @private\n     * @type {kendo.ui.TreeView}\n     */\n    var _tree = null;\n\n    /**\n     * JQuery for Unspecified Checkbox\n     * @private\n     * @type {JQuery}\n     */\n    var $unspecified = null;\n\n    /**\n     * Selected element\n     * @private\n     * @type {Element}\n     */\n    var _selectedElement = null;\n\n    /**\n     * Setup TreeView\n     * @private\n     */\n    function _setupTree() {\n        // DataSource for TreeView\n        _dataSource = new kendo.data.HierarchicalDataSource({\n            transport: {\n                read: function (options) {\n                    var nodes = [];\n                    var id = options.data._id;\n                    var elem = Repository.get(id);\n                    if (elem) {\n                        var elems = _sort(_getChildren(elem));\n                        _.each(elems, function (elem) {\n                            var node = _makeNode(elem);\n                            nodes.push(node);\n                        });\n                    } else {\n                        var node = _makeNode(ProjectManager.getProject());\n                        nodes.push(node);\n                    }\n                    options.success(nodes);\n                }\n            },\n            schema: {\n                model: {\n                    id: "_id",\n                    hasChildren: "hasChildren"\n                }\n            }\n        });\n\n        // Setup TreeView\n        _tree = $tree.kendoTreeView({\n            dataSource: _dataSource,\n            dataSpriteCssClassField: "sprite",\n            dataTextField: "text",\n            select: function (e) {\n                var item = this.dataItem(e.node);\n                var elem = Repository.get(item._id);\n                if (elem) {\n                    _selectedElement = elem;\n                    $unspecified.attr("checked", false);\n                }\n            }\n        }).data("kendoTreeView");\n    }\n\n    /**\n     * Return JSON object for tree node for a given element.\n     * @private\n     * @param {Core.Element} elem\n     * @return {{_id: string, text: string, sprite: string, hasChildren: boolean}}\n     */\n    function _makeNode(elem) {\n        return {\n            _id: elem._id,\n            text: elem.getNodeText(),\n            sprite: elem.getNodeIcon(elem),\n            hasChildren: (elem.getChildNodes().length > 0),\n            _name: elem.name,\n            _namespace: elem._parent ? elem._parent.name || "" : ""\n        };\n    }\n\n    /**\n     * Return child elements.\n     * @private\n     * @param {Element} elem\n     * @return {Array.<Element>}\n     */\n    function _getChildren(elem) {\n        return elem.getChildNodes();\n    }\n\n    /**\n     * Sort elements.\n     * @private\n     * @param {Array.<Element>} elements\n     * @return {Array.<Element>} - 정렬된 요소들의 배열\n     */\n    function _sort(elements) {\n        return _.sortBy(elements, function (child, idx) {\n            return child.getOrdering(idx);\n        });\n    }\n\n    /**\n     * Get a tree node of an element.\n     * @private\n     * @param {Element} elem\n     * @return {}\n     */\n    function _getNode(elem) {\n        var item = _dataSource.get(elem._id);\n        if (item && item.uid) {\n            var node = _tree.findByUid(item.uid);\n            return node;\n        }\n        return null;\n    }\n\n    /**\n     * Expand a tree node\n     *\n     * @param {Core.Model} elem Element to be expanded\n     * @param {?boolean} expandAllParents Whether expand all parents or not\n     */\n    function expand(elem, expandAllParents) {\n        if (elem) {\n            // Expand all parents\n            if ((expandAllParents === true) && elem._parent && elem._parent._id) {\n                expand(elem._parent, expandAllParents);\n            }\n            // Expand current node\n            var node = _getNode(elem);\n            if (node) {\n                _tree.expand(node);\n            }\n        }\n    }\n\n    /**\n     * Select an element\n     *\n     * @param {Core.Model} elem Element to be selected\n     * @param {?boolean} scrollTo Whether scroll to the element or not\n     */\n    function select(elem, scrollTo) {\n        if (elem) {\n            expand(elem._parent, true);\n            var node = _getNode(elem);\n            if (node) {\n                _tree.select(node);\n                // Scroll to node\n                if (scrollTo === true) {\n                    $treeWrapper.animate({\n                        scrollTop: node.offset().top - $treeWrapper.offset().top + $treeWrapper.scrollTop()\n                    }, 500);\n                }\n                $(exports).triggerHandler(\'selected\', [elem]);\n            }\n        } else {\n            _tree.select($());\n        }\n    }\n\n\n    function _setupQuickSearch($dlg) {\n        var searchDataSource =  new kendo.data.DataSource({\n            transport: {\n                read: function (options) {\n                    if (options.data.filter && options.data.filter.filters.length > 0) {\n                        var keyword = options.data.filter.filters[0].value;\n                        var results = Repository.search(keyword);\n                        options.success(_.map(results, _makeNode));\n                    } else {\n                        options.success([]);\n                    }\n                }\n            },\n            serverFiltering: true\n        });\n\n        $dlg.find(".quick-search").kendoAutoComplete({\n            dataTextField: "text",\n            minLength: 1,\n            filter: "contains",\n            select: function(e) {\n                var item = this.dataItem(e.item.index());\n                var elem = Repository.get(item._id);\n                if (elem) {\n                    _selectedElement = elem;\n                    $unspecified.attr("checked", false);\n                    select(elem, true);\n                }\n            },\n            template: "<div style=\'white-space: nowrap\'>" +\n                "<span class=\'k-sprite #:data.sprite#\'></span>" +\n                "<span style=\'margin-left: 5px\'>#:data.text#</span>" +\n                "#if (data._namespace.length > 0) {# <span style=\'margin-left: 5px; font-size: 11px; color: rgb(139,139,139);\'> — #:data._namespace#</span> #}#<div>",\n            dataSource: searchDataSource\n        });\n\n        $dlg.find(".quick-search").focus();\n    }\n\n    /**\n     * Show Element Picker Dialog\n     *\n     * @param {string} title Title of the dialog\n     * @param {?Element} selected Initial selected element when the dialog is shown\n     * @param {?constructor} type Type of selectable element\n     * @return {$.Promise}\n     */\n    function showDialog(title, selected, type) {\n        var context = {\n            Strings: Strings,\n            title: title\n        };\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(dialogTemplate, context), true, function ($dlg) {\n            if ((type !== null) && _.isObject(_selectedElement) && !(_selectedElement instanceof type)) {\n                $dlg.data("buttonId", Dialogs.DIALOG_BTN_CANCEL);\n                $dlg.data("returnValue", _selectedElement);\n                _.defer(function () {\n                    Dialogs.showAlertDialog("Selected element (" + _selectedElement.name + ") is not an instance of " + type.name);\n                });\n            } else {\n                $dlg.data("returnValue", _selectedElement);\n            }\n        });\n\n        var $dlg = dialog.getElement();\n        $unspecified = $dlg.find(".unspecified");\n        $treeWrapper = $dlg.find(".treeview-wrapper");\n        $tree = $dlg.find(".treeview");\n\n        var $wrapper = $dlg.find(".treeview-wrapper");\n        ViewUtils.addScrollerShadow($wrapper, null, true);\n\n        _setupTree();\n\n        // Expand the root node\n        var item = _dataSource.get(ProjectManager.getProject()._id);\n        var node = _tree.findByUid(item.uid);\n        _tree.expand(node);\n\n        // Setup Unspecified Checkbox\n        $unspecified.change(function () {\n            var checked = $unspecified.is(":checked");\n            if (checked) {\n                _tree.select($()); // clear selection\n                _selectedElement = null;\n            }\n        });\n\n        // Initial selection\n        if (selected) {\n            _selectedElement = selected;\n            select(_selectedElement, true);\n        } else {\n            _selectedElement = null;\n            $unspecified.attr("checked", true);\n        }\n\n        // Setup Quick Search Widget\n        _setupQuickSearch($dlg);\n\n        return dialog;\n    }\n\n    // Public API\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/dialogs/ElementPickerDialog.js'),eval("define('text!htmlContent/toast-template.html',[],function () { return '<div class=\"toast-item\">#=message#</div>\\n\\n';});\n\n//# sourceURL=/text!htmlContent/toast-template.html"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell */\n\n/**\n * Toast\n */\ndefine(\'ui/Toast\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'text!htmlContent/toast-template.html\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit       = require("utils/AppInit"),\n        toastTemplate = require("text!htmlContent/toast-template.html");\n\n    var toast;\n\n    /**\n     * Show information toast.\n     *\n     * @param {string} message\n     */\n    function info(message) {\n        toast.show({message: message}, "info");\n    }\n\n    /**\n     * Show warning toast.\n     *\n     * @param {string} message\n     */\n    function warning(message) {\n        toast.show({message: message}, "warning");\n    }\n\n    /**\n     * Show error toast.\n     *\n     * @param {string} message\n     */\n    function error(message) {\n        toast.show({message: message}, "error");\n    }\n\n    AppInit.htmlReady(function () {\n        toast = $("#toast").kendoNotification({\n            position: { pinned: true },\n            appendTo: "#toast-holder",\n            autoHideAfter: 5000,\n            stacking: "up",\n            templates: [\n                { type: "info",    template: toastTemplate },\n                { type: "warning", template: toastTemplate },\n                { type: "error",   template: toastTemplate }\n            ]\n        }).data("kendoNotification");\n    });\n\n    exports.info    = info;\n    exports.warning = warning;\n    exports.error   = error;\n\n});\n\n//# sourceURL=/ui/Toast.js'),eval("/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, $, window */\n\n/**\n * Utilities for working with Deferred, Promise, and other asynchronous processes.\n */\ndefine('utils/Async',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n    \n    // Further ideas for Async utilities...\n    //  - Utilities for blocking UI until a Promise completes?\n    //  - A \"SuperDeferred\" could feature some very useful enhancements:\n    //     - API for cancellation (non guaranteed, best attempt)\n    //     - Easier way to add a timeout clause (withTimeout() wrapper below is more verbose)\n    //     - Encapsulate the task kickoff code so you can start it later, e.g. superDeferred.start()\n    //  - Deferred/Promise are unable to do anything akin to a 'finally' block. It'd be nice if we\n    //    could harvest exceptions across all steps of an async process and pipe them to a handler,\n    //    so that we don't leave UI-blocking overlays up forever, etc. But this is hard: we'd have\n    //    wrap every async callback (including low-level native ones that don't use [Super]Deferred)\n    //    to catch exceptions, and then understand which Deferred(s) the code *would* have resolved/\n    //    rejected had it run to completion.\n    \n\n    /**\n     * Executes a series of tasks in parallel, returning a \"master\" Promise that is resolved once\n     * all the tasks have resolved. If one or more tasks fail, behavior depends on the failFast\n     * flag:\n     *   - If true, the master Promise is rejected as soon as the first task fails. The remaining\n     *     tasks continue to completion in the background.\n     *   - If false, the master Promise is rejected after all tasks have completed.\n     *\n     * If nothing fails:          (M = master promise; 1-4 = tasks; d = done; F = fail)\n     *  M  ------------d\n     *  1 >---d        .\n     *  2 >------d     .\n     *  3 >---------d  .\n     *  4 >------------d\n     *\n     * With failFast = false:\n     *  M  ------------F\n     *  1 >---d     .  .\n     *  2 >------d  .  .\n     *  3 >---------F  .\n     *  4 >------------d\n     *\n     * With failFast = true: -- equivalent to $.when()\n     *  M  ---------F\n     *  1 >---d     .\n     *  2 >------d  .\n     *  3 >---------F\n     *  4 >------------d   (#4 continues even though master Promise has failed)\n     * (Note: if tasks finish synchronously, the behavior is more like failFast=false because you\n     * won't get a chance to respond to the master Promise until after all items have been processed)\n     *\n     * To perform task-specific work after an individual task completes, attach handlers to each\n     * Promise before beginProcessItem() returns it.\n     *\n     * Note: don't use this if individual tasks (or their done/fail handlers) could ever show a user-\n     * visible dialog: because they run in parallel, you could show multiple dialogs atop each other.\n     *\n     * @param {!Array.<*>} items\n     * @param {!function(*, number):Promise} beginProcessItem\n     * @param {!boolean} failFast\n     * @return {$.Promise}\n     */\n    function doInParallel(items, beginProcessItem, failFast) {\n        var promises = [];\n        var masterDeferred = new $.Deferred();\n        \n        if (items.length === 0) {\n            masterDeferred.resolve();\n            \n        } else {\n            var numCompleted = 0;\n            var hasFailed = false;\n            \n            items.forEach(function (item, i) {\n                var itemPromise = beginProcessItem(item, i);\n                promises.push(itemPromise);\n                \n                itemPromise.fail(function () {\n                    if (failFast) {\n                        masterDeferred.reject();\n                    } else {\n                        hasFailed = true;\n                    }\n                });\n                itemPromise.always(function () {\n                    numCompleted++;\n                    if (numCompleted === items.length) {\n                        if (hasFailed) {\n                            masterDeferred.reject();\n                        } else {\n                            masterDeferred.resolve();\n                        }\n                    }\n                });\n            });\n            \n        }\n        \n        return masterDeferred.promise();\n    }\n    \n    /**\n     * Executes a series of tasks in serial (task N does not begin until task N-1 has completed).\n     * Returns a \"master\" Promise that is resolved once all the tasks have resolved. If one or more\n     * tasks fail, behavior depends on the failAndStopFast flag:\n     *   - If true, the master Promise is rejected as soon as the first task fails. The remaining\n     *     tasks are never started (the serial sequence is stopped).\n     *   - If false, the master Promise is rejected after all tasks have completed.\n     *\n     * If nothing fails:\n     *  M  ------------d\n     *  1 >---d        .\n     *  2     >--d     .\n     *  3        >--d  .\n     *  4           >--d\n     *\n     * With failAndStopFast = false:\n     *  M  ------------F\n     *  1 >---d     .  .\n     *  2     >--d  .  .\n     *  3        >--F  .\n     *  4           >--d\n     *\n     * With failAndStopFast = true:\n     *  M  ---------F\n     *  1 >---d     .\n     *  2     >--d  .\n     *  3        >--F\n     *  4          (#4 never runs)\n     *\n     * To perform task-specific work after an individual task completes, attach handlers to each\n     * Promise before beginProcessItem() returns it.\n     * \n     * @param {!Array.<*>} items\n     * @param {!function(*, number):Promise} beginProcessItem\n     * @param {!boolean} failAndStopFast\n     * @return {$.Promise}\n     */\n    function doSequentially(items, beginProcessItem, failAndStopFast) {\n\n        var masterDeferred = new $.Deferred(),\n            hasFailed = false;\n        \n        function doItem(i) {\n            if (i >= items.length) {\n                if (hasFailed) {\n                    masterDeferred.reject();\n                } else {\n                    masterDeferred.resolve();\n                }\n                return;\n            }\n            \n            var itemPromise = beginProcessItem(items[i], i);\n            \n            itemPromise.done(function () {\n                doItem(i + 1);\n            });\n            itemPromise.fail(function () {\n                if (failAndStopFast) {\n                    masterDeferred.reject();\n                    // note: we do NOT process any further items in this case\n                } else {\n                    hasFailed = true;\n                    doItem(i + 1);\n                }\n            });\n        }\n        \n        doItem(0);\n        \n        return masterDeferred.promise();\n    }\n    \n    /**\n     * Executes a series of synchronous tasks sequentially spread over time-slices less than maxBlockingTime.\n     * Processing yields by idleTime between time-slices.\n     * \n     * @param {!Array.<*>} items\n     * @param {!function(*, number)} fnProcessItem  Function that synchronously processes one item\n     * @param {number=} maxBlockingTime\n     * @param {number=} idleTime\n     * @return {$.Promise}\n     */\n    function doSequentiallyInBackground(items, fnProcessItem, maxBlockingTime, idleTime) {\n        \n        maxBlockingTime = maxBlockingTime || 15;\n        idleTime = idleTime || 30;\n        \n        var sliceStartTime = (new Date()).getTime();\n        \n        return doSequentially(items, function (item, i) {\n            var result = new $.Deferred();\n            \n            // process the next item\n            fnProcessItem(item, i);\n            \n            // if we've exhausted our maxBlockingTime\n            if ((new Date()).getTime() - sliceStartTime >= maxBlockingTime) {\n                //yield\n                window.setTimeout(function () {\n                    sliceStartTime = (new Date()).getTime();\n                    result.resolve();\n                }, idleTime);\n            } else {\n                //continue processing\n                result.resolve();\n            }\n\n            return result;\n        }, false);\n    }\n    \n    \n    /**\n     * Executes a series of tasks in parallel, saving up error info from any that fail along the way.\n     * Returns a Promise that is only resolved/rejected once all tasks are complete. This is\n     * essentially a wrapper around doInParallel(..., false).\n     *\n     * If one or more tasks failed, the entire \"master\" promise is rejected at the end - with one\n     * argument: an array objects, one per failed task. Each error object contains:\n     *  - item -- the entry in items whose task failed\n     *  - error -- the first argument passed to the fail() handler when the task failed\n     *\n     * @param {!Array.<*>} items\n     * @param {!function(*, number):Promise} beginProcessItem\n     * @return {$.Promise}\n     */\n    function doInParallel_aggregateErrors(items, beginProcessItem) {\n        var errors = [];\n        \n        var masterDeferred = new $.Deferred();\n        \n        var parallelResult = doInParallel(\n            items,\n            function (item, i) {\n                var itemResult = beginProcessItem(item, i);\n                itemResult.fail(function (error) {\n                    errors.push({ item: item, error: error });\n                });\n                return itemResult;\n            },\n            false\n        );\n        \n        parallelResult\n            .done(function () {\n                masterDeferred.resolve();\n            })\n            .fail(function () {\n                masterDeferred.reject(errors);\n            });\n        \n        return masterDeferred.promise();\n    }\n        \n    /** Value passed to fail() handlers that have been triggered due to withTimeout()'s timeout */\n    var ERROR_TIMEOUT = {};\n    \n    /**\n     * Adds timeout-driven failure to a Promise: returns a new Promise that is resolved/rejected when\n     * the given original Promise is resolved/rejected, OR is rejected after the given delay - whichever\n     * happens first.\n     * \n     * If the original Promise is resolved/rejected first, done()/fail() handlers receive arguments\n     * piped from the original Promise. If the timeout occurs first instead, fail() is called with the\n     * token Async.ERROR_TIMEOUT.\n     * \n     * @param {$.Promise} promise\n     * @param {number} timeout\n     * @return {$.Promise}\n     */\n    function withTimeout(promise, timeout) {\n        var wrapper = new $.Deferred();\n        \n        var timer = window.setTimeout(function () {\n            wrapper.reject(ERROR_TIMEOUT);\n        }, timeout);\n        promise.always(function () {\n            window.clearTimeout(timer);\n        });\n        \n        // If the wrapper was already rejected due to timeout, the Promise's calls to resolve/reject\n        // won't do anything\n        promise.then(wrapper.resolve, wrapper.reject);\n        \n        return wrapper.promise();\n    }\n    \n    /**\n     * Allows waiting for all the promises to be either resolved or rejected.\n     * Unlike $.when(), it does not call .fail() or .always() handlers on first\n     * reject. The caller should take all the precaution to make sure all the\n     * promises passed to this function are completed to avoid blocking.\n     * \n     * If failOnReject is set to true, promise returned by the function will be\n     * rejected if at least one of the promises was rejected. The default value\n     * is false, which will cause the call to this function to be always\n     * successfully resolved.\n     * \n     * If timeout is specified, the promise will be rejected on timeout as per\n     * Async.withTimeout.\n     * \n     * @param {!Array.<$.Promise>} promises Array of promises to wait for\n     * @param {boolean=} failOnReject       Whether to reject or not if one of the promises has been rejected.\n     * @param {number=} timeout             Number of milliseconds to wait until rejecting the promise\n     * \n     * @return {$.Promise} Promise which will be completed once al the \n     * \n     */\n    function waitForAll(promises, failOnReject, timeout) {\n        var masterDeferred = new $.Deferred(),\n            count = 0,\n            sawRejects = false;\n        \n        if (!promises || promises.length === 0) {\n            masterDeferred.resolve();\n            return masterDeferred.promise();\n        }\n        \n        // set defaults if needed\n        failOnReject = (failOnReject === undefined) ? false : true;\n        \n        if (timeout !== undefined) {\n            withTimeout(masterDeferred, timeout);\n        }\n        \n        promises.forEach(function (promise) {\n            promise\n                .fail(function (err) {\n                    sawRejects = true;\n                })\n                .always(function () {\n                    count++;\n                    if (count === promises.length) {\n                        if (failOnReject && sawRejects) {\n                            masterDeferred.reject();\n                        } else {\n                            masterDeferred.resolve();\n                        }\n                    }\n                });\n        });\n        \n        return masterDeferred.promise();\n    }\n    \n    /**\n     * Chains a series of synchronous and asynchronous (jQuery promise-returning) functions \n     * together, using the result of each successive function as the argument(s) to the next. \n     * A promise is returned that resolves with the result of the final call if all calls \n     * resolve or return normally. Otherwise, if any of the functions reject or throw, the \n     * computation is halted immediately and the promise is rejected with this halting error.\n     * \n     * @param {Array.<function(*)>} functions Functions to be chained\n     * @param {?Array} args Arguments to call the first function with\n     * @return {jQuery.Promise} A promise that resolves with the result of the final call, or\n     *      rejects with the first error.\n     */\n    function chain(functions, args) {\n        var deferred = $.Deferred();\n        \n        function chainHelper(index, args) {\n            if (functions.length === index) {\n                deferred.resolveWith(null, args);\n            } else {\n                var nextFunction = functions[index++];\n                try {\n                    var responseOrPromise = nextFunction.apply(null, args);\n                    if (responseOrPromise.hasOwnProperty(\"done\") &&\n                            responseOrPromise.hasOwnProperty(\"fail\")) {\n                        responseOrPromise.done(function () {\n                            chainHelper(index, arguments);\n                        });\n                        responseOrPromise.fail(function () {\n                            deferred.rejectWith(null, arguments);\n                        });\n                    } else {\n                        chainHelper(index, [responseOrPromise]);\n                    }\n                } catch (e) {\n                    deferred.reject(e);\n                }\n            }\n        }\n        \n        chainHelper(0, args || []);\n        \n        return deferred.promise();\n    }\n\n    /**\n     * @constructor\n     * Creates a queue of async operations that will be executed sequentially. Operations can be added to the\n     * queue at any time. If the queue is empty and nothing is currently executing when an operation is added, \n     * it will execute immediately. Otherwise, it will execute when the last operation currently in the queue \n     * has finished.\n     */\n    function PromiseQueue() {\n    }\n    \n    /**\n     * @private\n     * @type {Array.<function(): $.Promise>}\n     * The queue of operations to execute sequentially. Note that even if this array is empty, there might\n     * still be an operation we need to wait on; that operation's promise is stored in _curPromise.\n     */\n    PromiseQueue.prototype._queue = [];\n    \n    /**\n     * @private\n     * @type {$.Promise}\n     * The promise we're currently waiting on, or null if there's nothing currently executing.\n     */\n    PromiseQueue.prototype._curPromise = null;\n    \n    /**\n     * @type {number} The number of queued promises.\n     */\n    Object.defineProperties(PromiseQueue.prototype, {\n        \"length\": {\n            get: function () { return this._queue.length; },\n            set: function () { throw new Error(\"Cannot set length\"); }\n        }\n    });\n    \n    /**\n     * Adds an operation to the queue. If nothing is currently executing, it will execute immediately (and\n     * the next operation added to the queue will wait for it to complete). Otherwise, it will wait until\n     * the last operation in the queue (or the currently executing operation if nothing is in the queue) is\n     * finished. The operation must return a promise that will be resolved or rejected when it's finished;\n     * the queue will continue with the next operation regardless of whether the current operation's promise\n     * is resolved or rejected.\n     * @param {function(): $.Promise} op The operation to add to the queue.\n     */\n    PromiseQueue.prototype.add = function (op) {\n        this._queue.push(op);\n\n        // If something is currently executing, then _doNext() will get called when it's done. If nothing\n        // is executing (in which case the queue should have been empty), we need to call _doNext() to kickstart\n        // the queue.\n        if (!this._curPromise) {\n            this._doNext();\n        }\n    };\n    \n    /**\n     * Removes all pending promises from the queue.\n     */\n    PromiseQueue.prototype.removeAll = function () {\n        this._queue = [];\n    };\n    \n    /**\n     * @private\n     * Pulls the next operation off the queue and executes it.\n     */\n    PromiseQueue.prototype._doNext = function () {\n        var self = this;\n        if (this._queue.length) {\n            var op = this._queue.shift();\n            this._curPromise = op();\n            this._curPromise.done(function () {\n                self._curPromise = null;\n                self._doNext();\n            });\n        }\n    };\n    \n    // Define public API\n    exports.doInParallel   = doInParallel;\n    exports.doSequentially = doSequentially;\n    exports.doSequentiallyInBackground   = doSequentiallyInBackground;\n    exports.doInParallel_aggregateErrors = doInParallel_aggregateErrors;\n    exports.withTimeout    = withTimeout;\n    exports.waitForAll     = waitForAll;\n    exports.ERROR_TIMEOUT  = ERROR_TIMEOUT;\n    exports.chain          = chain;\n    exports.PromiseQueue   = PromiseQueue;\n});\n\n//# sourceURL=/utils/Async.js"),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, window */\n\ndefine(\'utils/UrlParams\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n    \n    // var _ = require("thirdparty/lodash");\n    \n    /**\n     * Convert between URL querystring and name/value pairs. Decodes and encodes URL parameters.\n     */\n    function UrlParams() {\n        this._store = {};\n    }\n    \n    /**\n     * Parse the window location by default. Optionally specify a URL to parse.\n     * @param {string} url\n     */\n    UrlParams.prototype.parse = function (url) {\n        var queryString = "",\n            urlParams,\n            p,\n            self = this;\n        \n        self._store = {};\n        \n        if (!url) {\n            queryString = window.document.location.search.substring(1);\n        } else if (url.indexOf("?") !== -1) {\n            queryString = url.substring(url.indexOf("?") + 1);\n        }\n        \n        queryString = queryString.trimRight();\n        \n        if (queryString) {\n            urlParams = queryString.split("&");\n            \n            urlParams.forEach(function (param) {\n                p = param.split("=");\n                p[1] = p[1] || "";\n                self._store[decodeURIComponent(p[0])] = decodeURIComponent(p[1]);\n            });\n        }\n    };\n    \n    /**\n     * Store a name/value string pair\n     * @param {!string} name\n     * @param {!string} value\n     */\n    UrlParams.prototype.put = function (name, value) {\n        this._store[name] = value;\n    };\n    \n    /**\n     * Retrieve a value by name\n     * @param {!string} name\n     * @return {string}\n     */\n    UrlParams.prototype.get = function (name) {\n        return this._store[name];\n    };\n    \n    /**\n     * Remove a name/value string pair\n     * @param {!string} name\n     */\n    UrlParams.prototype.remove = function (name) {\n        delete this._store[name];\n    };\n    \n    /**\n     * Returns true if the parameter list is empty, else returns false.\n     * @return {boolean}\n     */\n    UrlParams.prototype.isEmpty = function (name) {\n        return _.isEmpty(this._store);\n    };\n    \n    /**\n     * Encode name/value pairs as URI components.\n     * @return {string}\n     */\n    UrlParams.prototype.toString = function () {\n        var strs = [],\n            self = this;\n        \n        _.forEach(self._store, function (value, key) {\n            strs.push(encodeURIComponent(key) + "=" + encodeURIComponent(value));\n        });\n        \n        return strs.join("&");\n    };\n    \n    // Define public API\n    exports.UrlParams = UrlParams;\n});\n//# sourceURL=/utils/UrlParams.js'),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, $, brackets, window */\n\n/**\n * ExtensionLoader searches the filesystem for extensions, then creates a new context for each one and loads it.\n * This module dispatches the following events:\n *      "load" - when an extension is successfully loaded. The second argument is the file path to the\n *          extension root.\n *      "loadFailed" - when an extension load is unsuccessful. The second argument is the file path to the\n *          extension root.\n */\n\ndefine(\'utils/ExtensionLoader\',[\'require\',\'exports\',\'module\',\'utils/Initializer\',\'filesystem/FileSystem\',\'file/FileUtils\',\'utils/Async\',\'utils/UrlParams\'],function (require, exports, module) {\n    "use strict";\n\n    require("utils/Initializer");\n\n    var // _           = require("thirdparty/lodash"),\n        FileSystem  = require("filesystem/FileSystem"),\n        FileUtils   = require("file/FileUtils"),\n        Async       = require("utils/Async"),\n        UrlParams   = require("utils/UrlParams").UrlParams;\n\n    // default async initExtension timeout\n    var INIT_EXTENSION_TIMEOUT = 10000;\n\n    var _init       = false,\n        _extensions = {},\n        _initExtensionTimeout = INIT_EXTENSION_TIMEOUT,\n        /** @type {Object<string, Object>}  Stores require.js contexts of extensions */\n        contexts    = {},\n        srcPath     = FileUtils.getNativeDirectoryPath();\n\n    // The native directory path ends with either "test" or "src". We need "src" to\n    // load the text and i18n modules.\n    srcPath = srcPath.replace(/\\/test$/, "/src"); // convert from "test" to "src"\n\n    var globalConfig = {\n            "text" : srcPath + "/thirdparty/text/text"\n            /* "i18n" : srcPath + "/thirdparty/i18n/i18n" */\n        };\n\n    /**\n     * Returns the full path of the default user extensions directory. This is in the users\n     * application support directory, which is typically\n     * /Users/<user>/Application Support/StarUML/extensions/user on the mac, and\n     * C:\\Users\\<user>\\AppData\\Roaming\\StarUML\\extensions\\user on windows.\n     */\n    function getUserExtensionPath() {\n        if (appshell.app.getApplicationSupportDirectory) {\n            return appshell.app.getApplicationSupportDirectory() + "/extensions/user";\n        }\n\n        return null;\n    }\n\n    /**\n     * Returns the require.js require context used to load an extension\n     *\n     * @param {!string} name, used to identify the extension\n     * @return {!Object} A require.js require object used to load the extension, or undefined if\n     * there is no require object with that name\n     */\n    function getRequireContextForExtension(name) {\n        return contexts[name];\n    }\n\n    /**\n     * @private\n     * Get timeout value for rejecting an extension\'s async initExtension promise.\n     * @return {number} Timeout in milliseconds\n     */\n    function _getInitExtensionTimeout() {\n        return _initExtensionTimeout;\n    }\n\n    /**\n     * @private\n     * Set timeout for rejecting an extension\'s async initExtension promise.\n     * @param {number} value Timeout in milliseconds\n     */\n    function _setInitExtensionTimeout(value) {\n        _initExtensionTimeout = value;\n    }\n\n    /**\n     * @private\n     * Loads optional requirejs-config.json file for an extension\n     * @param {Object} baseConfig\n     * @return {$.Promise}\n     */\n    function _mergeConfig(baseConfig) {\n        var deferred = new $.Deferred(),\n            extensionConfigFile = FileSystem.getFileForPath(baseConfig.baseUrl + "/requirejs-config.json");\n\n        // Optional JSON config for require.js\n        FileUtils.readAsText(extensionConfigFile).done(function (text) {\n            try {\n                var extensionConfig = JSON.parse(text);\n\n                // baseConfig.paths properties will override any extension config paths\n                _.extend(extensionConfig.paths, baseConfig.paths);\n\n                // Overwrite baseUrl, context, locale (paths is already merged above)\n                _.extend(extensionConfig, _.omit(baseConfig, "paths"));\n\n                deferred.resolve(extensionConfig);\n            } catch (err) {\n                // Failed to parse requirejs-config.json\n                deferred.reject("failed to parse requirejs-config.json");\n            }\n        }).fail(function () {\n            // If requirejs-config.json isn\'t specified, resolve with the baseConfig only\n            deferred.resolve(baseConfig);\n        });\n\n        return deferred.promise();\n    }\n\n    /**\n     * Loads the extension that lives at baseUrl into its own Require.js context\n     *\n     * @param {!string} name, used to identify the extension\n     * @param {!{baseUrl: string}} config object with baseUrl property containing absolute path of extension\n     * @param {!string} entryPoint, name of the main js file to load\n     * @return {!$.Promise} A promise object that is resolved when the extension is loaded, or rejected\n     *              if the extension fails to load or throws an exception immediately when loaded.\n     *              (Note: if extension contains a JS syntax error, promise is resolved not rejected).\n     */\n    function loadExtension(name, config, entryPoint) {\n        var extensionConfig = {\n            context: name,\n            baseUrl: config.baseUrl,\n            /* FIXME (issue #1087): can we pass this from the global require context instead of hardcoding twice? */\n            paths: globalConfig,\n            locale: app.getLocale()\n        };\n\n        // Read optional requirejs-config.json\n        var promise = _mergeConfig(extensionConfig).then(function (mergedConfig) {\n            // Create new RequireJS context and load extension entry point\n            var extensionRequire = app.libRequire.config(mergedConfig),\n                extensionRequireDeferred = new $.Deferred();\n\n            contexts[name] = extensionRequire;\n            extensionRequire([entryPoint], extensionRequireDeferred.resolve, extensionRequireDeferred.reject);\n\n            return extensionRequireDeferred.promise();\n        }).then(function (module) {\n            // Extension loaded normally\n            var initPromise;\n\n            _extensions[name] = module;\n\n            // Optional sync/async initExtension\n            if (module && module.initExtension && (typeof module.initExtension === "function")) {\n                // optional async extension init\n                try {\n                    initPromise = Async.withTimeout(module.initExtension(), _getInitExtensionTimeout());\n                } catch (err) {\n                    // Synchronous error while initializing extension\n                    console.error("[Extension] Error -- error thrown during initExtension for " + name + ": " + err);\n                    return new $.Deferred().reject(err).promise();\n                }\n\n                // initExtension may be synchronous and may not return a promise\n                if (initPromise) {\n                    // WARNING: These calls to initPromise.fail() and initPromise.then(),\n                    // could also result in a runtime error if initPromise is not a valid\n                    // promise. Currently, the promise is wrapped via Async.withTimeout(),\n                    // so the call is safe as-is.\n                    initPromise.fail(function (err) {\n                        if (err === Async.ERROR_TIMEOUT) {\n                            console.error("[Extension] Error -- timeout during initExtension for " + name);\n                        } else {\n                            console.error("[Extension] Error -- failed initExtension for " + name + (err ? ": " + err : ""));\n                        }\n                    });\n\n                    return initPromise;\n                }\n            }\n        }, function errback(err) {\n            // Extension failed to load during the initial require() call\n            console.error("[Extension] failed to load " + config.baseUrl + " " + err);\n            if (err.requireType === "define") {\n                // This type has a useful stack (exception thrown by ext code or info on bad getModule() call)\n                console.log(err.stack);\n            }\n        }).then(function () {\n            $(exports).triggerHandler("load", config.baseUrl);\n        }, function (err) {\n            $(exports).triggerHandler("loadFailed", config.baseUrl);\n        });\n\n        return promise;\n    }\n\n    /**\n     * Runs unit tests for the extension that lives at baseUrl into its own Require.js context\n     *\n     * @param {!string} name, used to identify the extension\n     * @param {!{baseUrl: string}} config object with baseUrl property containing absolute path of extension\n     * @param {!string} entryPoint, name of the main js file to load\n     * @return {!$.Promise} A promise object that is resolved when all extensions complete loading.\n     */\n    function testExtension(name, config, entryPoint) {\n        var result = new $.Deferred(),\n            extensionPath = config.baseUrl + "/" + entryPoint + ".js";\n\n        FileSystem.resolve(extensionPath, function (err, entry) {\n            if (!err && entry.isFile) {\n                // unit test file exists\n                var extensionRequire = app.libRequire.config({\n                    context: name,\n                    baseUrl: config.baseUrl,\n                    paths: $.extend({}, config.paths, globalConfig)\n                });\n\n                extensionRequire([entryPoint], function () {\n                    result.resolve();\n                });\n            } else {\n                result.reject();\n            }\n        });\n\n        return result.promise();\n    }\n\n    /**\n     * @private\n     * Loads a file entryPoint from each extension folder within the baseUrl into its own Require.js context\n     *\n     * @param {!string} directory, an absolute native path that contains a directory of extensions.\n     *                  each subdirectory is interpreted as an independent extension\n     * @param {!{baseUrl: string}} config object with baseUrl property containing absolute path of extension folder\n     * @param {!string} entryPoint Module name to load (without .js suffix)\n     * @param {function} processExtension\n     * @return {!$.Promise} A promise object that is resolved when all extensions complete loading.\n     */\n    function _loadAll(directory, config, entryPoint, processExtension) {\n        var result = new $.Deferred();\n\n        FileSystem.getDirectoryForPath(directory).getContents(function (err, contents) {\n            if (!err) {\n                var i,\n                    extensions = [];\n\n                for (i = 0; i < contents.length; i++) {\n                    if (contents[i].isDirectory) {\n                        // FUTURE (JRB): read package.json instead of just using the entrypoint "main".\n                        // Also, load sub-extensions defined in package.json.\n                        extensions.push(contents[i].name);\n                    }\n                }\n\n                if (extensions.length === 0) {\n                    result.resolve();\n                    return;\n                }\n\n                Async.doInParallel(extensions, function (item) {\n                    var extConfig = {\n                        baseUrl: config.baseUrl + "/" + item,\n                        paths: config.paths\n                    };\n                    return processExtension(item, extConfig, entryPoint);\n                }).always(function () {\n                    // Always resolve the promise even if some extensions had errors\n                    result.resolve();\n                });\n            } else {\n                console.error("[Extension] Error -- could not read native directory: " + directory);\n                result.reject();\n            }\n        });\n\n        return result.promise();\n    }\n\n    /**\n     * Loads the extension that lives at baseUrl into its own Require.js context\n     *\n     * @param {!string} directory, an absolute native path that contains a directory of extensions.\n     *                  each subdirectory is interpreted as an independent extension\n     * @return {!$.Promise} A promise object that is resolved when all extensions complete loading.\n     */\n    function loadAllExtensionsInNativeDirectory(directory) {\n        return _loadAll(directory, {baseUrl: directory}, "main", loadExtension);\n    }\n\n    /**\n     * Runs unit test for the extension that lives at baseUrl into its own Require.js context\n     *\n     * @param {!string} directory, an absolute native path that contains a directory of extensions.\n     *                  each subdirectory is interpreted as an independent extension\n     * @return {!$.Promise} A promise object that is resolved when all extensions complete loading.\n     */\n    function testAllExtensionsInNativeDirectory(directory) {\n        var bracketsPath = FileUtils.getNativeDirectoryPath(),\n            config = {\n                baseUrl: directory\n            };\n\n        config.paths = {\n            "perf": bracketsPath + "/perf",\n            "spec": bracketsPath + "/spec"\n        };\n\n        return _loadAll(directory, config, "unittests", testExtension);\n    }\n\n    /**\n     * Load extensions.\n     *\n     * @param {?Array.<string>} A list containing references to extension source\n     *      location. A source location may be either (a) a folder name inside\n     *      src/extensions or (b) an absolute path.\n     * @return {!$.Promise} A promise object that is resolved when all extensions complete loading.\n     */\n    function init(paths) {\n        var params = new UrlParams();\n\n        if (_init) {\n            // Only init once. Return a resolved promise.\n            return new $.Deferred().resolve().promise();\n        }\n\n        if (!paths) {\n            params.parse();\n\n            if (params.get("reloadWithoutUserExts") === "true") {\n                paths = ["default"];\n            } else {\n                paths = ["default", "dev", getUserExtensionPath()];\n            }\n        }\n\n        // Load extensions before restoring the project\n\n        // Get a Directory for the user extension directory and create it if it doesn\'t exist.\n        // Note that this is an async call and there are no success or failure functions passed\n        // in. If the directory *doesn\'t* exist, it will be created. Extension loading may happen\n        // before the directory is finished being created, but that is okay, since the extension\n        // loading will work correctly without this directory.\n        // If the directory *does* exist, nothing else needs to be done. It will be scanned normally\n        // during extension loading.\n        var extensionPath = getUserExtensionPath();\n        FileSystem.getDirectoryForPath(extensionPath).create();\n\n        // Create the extensions/disabled directory, too.\n        var disabledExtensionPath = extensionPath.replace(/\\/user$/, "/disabled");\n        FileSystem.getDirectoryForPath(disabledExtensionPath).create();\n\n        var promise = Async.doSequentially(paths, function (item) {\n            var extensionPath = item;\n\n            // If the item has "/" in it, assume it is a full path. Otherwise, load\n            // from our source path + "/extensions/".\n            if (item.indexOf("/") === -1) {\n                extensionPath = FileUtils.getNativeDirectoryPath() + "/extensions/" + item;\n            }\n\n            return loadAllExtensionsInNativeDirectory(extensionPath);\n        }, false);\n\n        promise.always(function () {\n            _init = true;\n        });\n\n        return promise;\n    }\n\n    // unit tests\n    exports._setInitExtensionTimeout = _setInitExtensionTimeout;\n    exports._getInitExtensionTimeout = _getInitExtensionTimeout;\n\n    // public API\n    exports.init = init;\n    exports.getUserExtensionPath = getUserExtensionPath;\n    exports.getRequireContextForExtension = getRequireContextForExtension;\n    exports.loadExtension = loadExtension;\n    exports.testExtension = testExtension;\n    exports.loadAllExtensionsInNativeDirectory = loadAllExtensionsInNativeDirectory;\n    exports.testAllExtensionsInNativeDirectory = testAllExtensionsInNativeDirectory;\n});\n\n//# sourceURL=/utils/ExtensionLoader.js'),eval(" /*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, $, window, brackets */\n\ndefine('utils/DragAndDrop',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n\n    /**\n     * Returns true if the drag and drop items contains valid drop objects.\n     * @param {Array.<DataTransferItem>} items Array of items being dragged\n     * @return {boolean} True if one or more items can be dropped.\n     */\n    function isValidDrop(items) {\n        var i, len = items.length;\n\n        for (i = 0; i < len; i++) {\n            if (items[i].kind === \"file\") {\n                var entry = items[i].webkitGetAsEntry();\n\n                if (entry.isFile) {\n                    // If any files are being dropped, this is a valid drop\n                    return true;\n                } else if (len === 1) {\n                    // If exactly one folder is being dropped, this is a valid drop\n                    return true;\n                }\n            }\n        }\n\n        // No valid entries found\n        return false;\n    }\n\n    // Export public API\n    exports.isValidDrop         = isValidDrop;\n\n});\n//# sourceURL=/utils/DragAndDrop.js"),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell */\ndefine('utils/Constants',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n\n    exports.APP_EXT  = \"mdj\";\n    exports.FRAG_EXT = \"mfj\";\n\n});\n\n//# sourceURL=/utils/Constants.js"),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, $, brackets, less, PathUtils */\n\n/**\n * ExtensionUtils defines utility methods for implementing extensions.\n */\ndefine(\'utils/ExtensionUtils\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n    \n    /**\n     * Appends a <style> tag to the document\'s head.\n     *\n     * @param {!string} css CSS code to use as the tag\'s content\n     * @return {!HTMLStyleElement} The generated HTML node\n     **/\n    function addEmbeddedStyleSheet(css) {\n        return $("<style>").text(css).appendTo("head")[0];\n    }\n    \n    /**\n     * Appends a <link> tag to the document\'s head.\n     *\n     * @param {!string} url URL to a style sheet\n     * @param {$.Deferred=} deferred Optionally check for load and error events\n     * @return {!HTMLLinkElement} The generated HTML node\n     **/\n    function addLinkedStyleSheet(url, deferred) {\n        var attributes = {\n            type: "text/css",\n            rel:  "stylesheet",\n            href: url\n        };\n        \n        var $link = $("<link/>").attr(attributes);\n        \n        if (deferred) {\n            $link.on(\'load\', deferred.resolve).on(\'error\', deferred.reject);\n        }\n        \n        $link.appendTo("head");\n        \n        return $link[0];\n    }\n\n    /**\n     * Parses LESS code and returns a promise that resolves with plain CSS code.\n     *\n     * Pass the {@link url} argument to resolve relative URLs contained in the code.\n     * Make sure URLs in the code are wrapped in quotes, like so:\n     *     background-image: url("image.png");\n     *\n     * @param {!string} code LESS code to parse\n     * @param {?string} url URL to the file containing the code\n     * @return {!$.Promise} A promise object that is resolved with CSS code if the LESS code can be parsed\n     */\n    function parseLessCode(code, url) {\n        var result = new $.Deferred(),\n            options;\n        \n        if (url) {\n            var dir  = url.slice(0, url.lastIndexOf("/") + 1),\n                file = url.slice(dir.length);\n            \n            options = {\n                filename: file,\n                paths:    [dir],\n                rootpath: dir\n            };\n\n            if (PathUtils.isAbsoluteUrl(url)) {\n                options.currentFileInfo = {\n                    currentDirectory: dir,\n                    entryPath: dir,\n                    filename: url,\n                    rootFilename: url,\n                    rootpath: dir\n                };\n            }\n        }\n        \n        var parser = new less.Parser(options);\n        parser.parse(code, function onParse(err, tree) {\n            if (err) {\n                result.reject(err);\n            } else {\n                result.resolve(tree.toCSS());\n            }\n        });\n        \n        return result.promise();\n    }\n    \n    /**\n     * Returns a path to an extension module.\n     *\n     * @param {!module} module Module provided by RequireJS\n     * @param {?string} path Relative path from the extension folder to a file\n     * @return {!string} The path to the module\'s folder\n     **/\n    function getModulePath(module, path) {\n        var modulePath = module.uri.substr(0, module.uri.lastIndexOf("/") + 1);\n        if (path) {\n            modulePath += path;\n        }\n        \n        return modulePath;\n    }\n    \n    /**\n     * Returns a URL to an extension module.\n     *\n     * @param {!module} module Module provided by RequireJS\n     * @param {?string} path Relative path from the extension folder to a file\n     * @return {!string} The URL to the module\'s folder\n     **/\n    function getModuleUrl(module, path) {\n        var url = encodeURI(getModulePath(module, path));\n        \n        // On Windows, $.get() fails if the url is a full pathname. To work around this,\n        // prepend "file:///". On the Mac, $.get() works fine if the url is a full pathname,\n        // but *doesn\'t* work if it is prepended with "file://". Go figure.\n        // However, the prefix "file://localhost" does work.\n        if (brackets.platform === "win" && url.indexOf(":") !== -1) {\n            url = "file:///" + url;\n        }\n        \n        return url;\n    }\n    \n    /**\n     * Performs a GET request using a path relative to an extension module.\n     *\n     * The resulting URL can be retrieved in the resolve callback by accessing\n     * \n     * @param {!module} module Module provided by RequireJS\n     * @param {!string} path Relative path from the extension folder to a file\n     * @return {!$.Promise} A promise object that is resolved with the contents of the requested file\n     **/\n    function loadFile(module, path) {\n        var url     = PathUtils.isAbsoluteUrl(path) ? path : getModuleUrl(module, path),\n            promise = $.get(url);\n\n        return promise;\n    }\n    \n    /**\n     * Loads a style sheet (CSS or LESS) relative to the extension module.\n     *\n     * @param {!module} module Module provided by RequireJS\n     * @param {!string} path Relative path from the extension folder to a CSS or LESS file\n     * @return {!$.Promise} A promise object that is resolved with an HTML node if the file can be loaded.\n     */\n    function loadStyleSheet(module, path) {\n        var result = new $.Deferred();\n\n        loadFile(module, path)\n            .done(function (content) {\n                var url = this.url;\n                \n                if (url.slice(-5) === ".less") {\n                    parseLessCode(content, url)\n                        .done(function (css) {\n                            result.resolve(addEmbeddedStyleSheet(css));\n                        })\n                        .fail(result.reject);\n                } else {\n                    var deferred = new $.Deferred(),\n                        link = addLinkedStyleSheet(url, deferred);\n                    \n                    deferred\n                        .done(function () {\n                            result.resolve(link);\n                        })\n                        .fail(result.reject);\n                }\n            })\n            .fail(result.reject);\n        \n        return result.promise();\n    }\n    \n    exports.addEmbeddedStyleSheet = addEmbeddedStyleSheet;\n    exports.addLinkedStyleSheet   = addLinkedStyleSheet;\n    exports.parseLessCode         = parseLessCode;\n    exports.getModulePath         = getModulePath;\n    exports.getModuleUrl          = getModuleUrl;\n    exports.loadFile              = loadFile;\n    exports.loadStyleSheet        = loadStyleSheet;\n});\n\n//# sourceURL=/utils/ExtensionUtils.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, browser: true */\n/*global $, _, define, kendo, app, C2S */\n\ndefine(\'metadata-json/MetadataJson\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Repository\',\'engine/ProjectManager\',\'core/PreferenceManager\',\'utils/NodeDomain\',\'file/FileUtils\',\'filesystem/FileSystem\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit           = require("utils/AppInit"),\n        Repository        = require("core/Repository"),\n        ProjectManager    = require("engine/ProjectManager"),\n        PreferenceManager = require("core/PreferenceManager"),\n        NodeDomain        = require("utils/NodeDomain"),\n        FileUtils         = require("file/FileUtils"),\n        FileSystem        = require("filesystem/FileSystem");\n\n    // MetadataJson Domain\n    var domainPath = FileUtils.getNativeDirectoryPath() + "/" + FileUtils.getNativeModuleDirectoryPath(module) + "/node/MetadataJsonDomain";\n    var MetadataJsonDomain = new NodeDomain("MetadataJson", domainPath);\n\n    /**\n     * Validate a metadata file (.mdj)\n     * @param {string} fullPath\n     * @return {$.Promise}\n     */\n    function validate(fullPath) {\n        if (PreferenceManager.get("validation.validateModel")) {\n            return MetadataJsonDomain.exec("validate", fullPath);\n        } else {\n            return $.Deferred().resolve().promise();\n        }\n    }\n\n    /**\n     * Export diagram(s) to a PDF file\n     * @param {string} fullPath Model file (.mdj) to be exported\n     * @param {Array.<Diagram>} diagrams Array of diagram\'s id to be exported\n     * @param {string} filename Filename for an exported PDF\n     * @param {{layout:string, size:string, showName:boolean}} options\n     * @return {$.Promise}\n     */\n    function exportToPDF(fullPath, diagrams, filename, options) {\n        var ids = _.map(diagrams, function (d) { return d._id; });\n        return MetadataJsonDomain.exec("exportToPDF", fullPath, ids, filename, options);\n    }\n\n    /**\n     * Export project to HTML documents\n     * @param {string} fullPath Model file (.mdj) to be exported\n     * @param {string} targetDir Path for an exported HTML files\n     * @return {$.Promise}\n     */\n    function exportToHTML(fullPath, targetDir) {\n        return MetadataJsonDomain.exec("exportToHTML", fullPath, targetDir);\n    }\n\n    /**\n     * Register font\n     * @param {strong} path Path name containing .ttf font files and font.json\n     */\n    function registerFont(path) {\n        return MetadataJsonDomain.exec("registerFont", path);\n    }\n\n    exports.validate     = validate;\n    exports.exportToPDF  = exportToPDF;\n    exports.exportToHTML = exportToHTML;\n    exports.registerFont = registerFont;\n\n});\n\n//# sourceURL=/metadata-json/MetadataJson.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global require, define, $, _, app, document, appshell, Mustache*/\n\ndefine('utils/FontManager',['require','exports','module','file/FileUtils','filesystem/FileSystem','utils/ExtensionUtils','utils/Async','metadata-json/MetadataJson'],function (require, exports, module) {\n    \"use strict\";\n\n    var FileUtils      = require(\"file/FileUtils\"),\n        FileSystem     = require(\"filesystem/FileSystem\"),\n        ExtensionUtils = require(\"utils/ExtensionUtils\"),\n        Async          = require(\"utils/Async\"),\n        MetadataJson   = require(\"metadata-json/MetadataJson\");\n\n    \n    function loadFont(dir) {\n        var result = new $.Deferred();\n        var file = FileSystem.getFileForPath(dir + '/font.json');\n        FileUtils.readAsText(file)\n            .done(function (data) {\n                MetadataJson.registerFont(dir);\n                var fontArray = JSON.parse(data);\n                _.each(fontArray, function (font) {\n                    if (font.name !== \"default\") {\n                        font.dir = dir;\n                        var template =\n                            \"@font-face { font-family: '{{name}}'; src: url('{{{dir}}}/{{regular}}'); font-weight: normal; font-style: normal;}\" +\n                            \"@font-face { font-family: '{{name}}'; src: url('{{{dir}}}/{{italic}}'); font-weight: normal; font-style: italic;}\" +\n                            \"@font-face { font-family: '{{name}}'; src: url('{{{dir}}}/{{bold}}'); font-weight: bold; font-style: normal;}\" +\n                            \"@font-face { font-family: '{{name}}'; src: url('{{{dir}}}/{{boldItalic}}'); font-weight: bold; font-style: italic;}\";\n                        var css = Mustache.render(template, font);\n                        ExtensionUtils.addEmbeddedStyleSheet(css);\n                        if (!Array.isArray(app.config.defaultFonts)) {\n                            app.config.defaultFonts = [];\n                        }\n                        app.config.defaultFonts.push({ value: font.name, text: font.name });\n                    }\n                });\n            \n                // Sort fonts\n                var _sorted = _.sortBy(app.config.defaultFonts, function (f) { return f.value; });\n                app.config.defaultFonts = _sorted;\n                result.resolve();\n            })\n            .fail(function (err) {\n                result.reject(err);\n            });\n        return result.promise();\n    }\n\n    function loadDefaultFonts() {\n        var result = new $.Deferred();\n        var fontPath = FileUtils.getNativeDirectoryPath() + \"/styles/fonts\";\n        appshell.fs.readdir(fontPath, function (err, files) {\n            if (!err) {\n                Async.doSequentially(\n                    files,\n                    function (dir) {\n                        var _result = new $.Deferred();\n                        var path = fontPath + '/' + dir;\n                        appshell.fs.stat(path, function (err, stats) {\n                            if (!err) {\n                                if (stats.isDirectory()) {\n                                    loadFont(path).then(_result.resolve, _result.reject);\n                                } else {\n                                    _result.resolve();\n                                }\n                            } else {\n                                _result.reject(err);\n                            }\n                        });\n                        return _result.promise();\n                    },\n                    false\n                ).then(result.resolve, result.reject);\n            } else {\n                result.reject(err);\n            }\n        });\n        return result.promise();\n    }\n    \n    exports.loadFont = loadFont;\n    exports.loadDefaultFonts = loadDefaultFonts;\n    \n});\n\n//# sourceURL=/utils/FontManager.js"),eval('define(\'text!htmlContent/main-view.html\',[],function () { return \'<!-- Splash Screen -->\\n<div class="splash-screen"></div>\\n\\n<!-- Menu Bar (collapsed when nativeMenus) -->\\n<ul id="menu-bar"></ul>\\n\\n<!-- Main View -->\\n<div class="main-view">\\n\\n    <div id="sidebar" class="sidebar panel quiet-scrollbars horz-resizable right-resizer collapsible" data-minsize="0" data-forcesize=".content">\\n\\n        <!-- Working Diagrams -->\\n        <div id="working-diagrams" class="working-diagrams"></div>\\n\\n        <!-- Toolbox -->\\n        <div id="toolbox" class="vert-resizable top-resizer collapsible" data-minsize="60" data-forcesize=".working-diagrams">\\n        </div>\\n\\n    </div>\\n\\n    <div class="content">\\n\\n        <div class="content-holder">\\n\\n            <!-- Diagram Editor -->\\n            <div id="diagram-area-wrapper" style="position: relative">\\n                <div id="diagram-area">\\n                    <canvas id="diagram-canvas" tabindex="0" style="outline: none;"></canvas>\\n                    <canvas id="hidden-canvas"></canvas>\\n                </div>\\n            </div>\\n\\n            <!-- Bottom Panels are created here -->\\n        </div>\\n\\n        <!-- Toolbar -->\\n        <div id="toolbar">\\n            <div class="buttons">\\n                <a id="toolbar-extension-manager" href="#" title="Extension Manager"></a>\\n            </div>\\n            <div class="bottom-buttons">\\n            </div>\\n        </div>\\n\\n        <!-- Toast Holder -->\\n        <div id="toast-holder"></div>\\n\\n    </div>\\n\\n    <div id="navigator" class="navigator panel quiet-scrollbars horz-resizable left-resizer collapsible" data-minsize="0" data-forcesize=".content">\\n\\n        <!-- Model Explorer -->\\n        <div id="model-explorer" class="model-explorer"></div>\\n\\n        <!-- Editor -->\\n        <div id="editors" class="vert-resizable top-resizer collapsible" data-minsize="60" data-forcesize=".model-explorer"></div>\\n\\n    </div>\\n\\n</div>\\n\\n<!-- Status Bar -->\\n<div id="statusbar" class="statusbar"></div>\\n\\n<div id="toast" style="display: none"></div>\\n\';});\n\n//# sourceURL=/text!htmlContent/main-view.html'),eval("define('text!htmlContent/windows-stylesheet.html',[],function () { return '<style type=\"text/css\">\\n    ::-webkit-scrollbar {\\n        background: transparent;\\n        width: 6px;\\n        height: 6px;\\n    }\\n\\n    ::-webkit-scrollbar-corner {\\n        background: transparent;\\n    }\\n\\n    ::-webkit-scrollbar-track {\\n        padding: 2px;\\n    }\\n    \\n    ::-webkit-scrollbar-thumb:vertical {\\n        background-color: #262626;\\n    }\\n\\n    ::-webkit-scrollbar-thumb:horizontal {\\n        background-color: #262626;\\n    }\\n</style>';});\n\n//# sourceURL=/text!htmlContent/windows-stylesheet.html"),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, $, window */\n\n/**\n * Resizer is a Module utility to inject resizing capabilities to any element\n * inside Brackets.\n *\n * On initialization, Resizer discovers all nodes tagged as "vert-resizable"\n * and "horz-resizable" to add the resizer handler. Additionally, "top-resizer",\n * "bottom-resizer", "left-resizer" and "right-resizer" classes control the\n * position of the resizer on the element.\n *\n * An element can be made resizable at any time using the `makeResizable()` API.\n * Panel sizes are saved via preferences and restored when the DOM node becomes resizable\n * again in a subsequent launch.\n *\n * The resizable elements trigger a panelResizeStart, panelResizeUpdate and panelResizeEnd\n * event that can be used to create performance optimizations (such as hiding/showing elements\n * while resizing), custom layout logic, etc. See makeResizable() for details on the events.\n *\n * A resizable element can be collapsed/expanded using the `show`, `hide` and `toggle` APIs or\n * via user action. This triggers panelCollapsed/panelExpanded events - see makeResizable().\n */\ndefine(\'utils/Resizer\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/PreferenceManager\'],function (require, exports, module) {\n    "use strict";\n\n    var DIRECTION_VERTICAL = "vert";\n    var DIRECTION_HORIZONTAL = "horz";\n\n    var POSITION_TOP = "top";\n    var POSITION_BOTTOM = "bottom";\n    var POSITION_LEFT = "left";\n    var POSITION_RIGHT = "right";\n\n    // Minimum size (height or width) for autodiscovered resizable panels\n    var DEFAULT_MIN_SIZE = 100;\n\n    // Load dependent modules\n    var AppInit           = require("utils/AppInit"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    var $mainView;\n\n    var isResizing = false;\n\n    /**\n     * Shows a resizable element.\n     * @param {DOMNode} element Html element to show if possible\n     */\n    function show(element) {\n        var showFunc = $(element).data("show");\n        if (showFunc) {\n            showFunc.apply(element);\n        }\n    }\n\n    /**\n     * Hides a resizable element.\n     * @param {DOMNode} element Html element to hide if possible\n     */\n    function hide(element) {\n        var hideFunc = $(element).data("hide");\n        if (hideFunc) {\n            hideFunc.apply(element);\n        }\n    }\n\n    /**\n     * Changes the visibility state of a resizable element. The toggle\n     * functionality is added when an element is made resizable.\n     * @param {DOMNode} element Html element to toggle\n     */\n    function toggle(element) {\n        if ($(element).is(":visible")) {\n            hide(element);\n        } else {\n            show(element);\n        }\n    }\n\n    /**\n     * Returns the visibility state of a resizable element.\n     * @param {DOMNode} element Html element to toggle\n     * @return {boolean} true if element is visible, false if it is not visible\n     */\n    function isVisible(element) {\n        return $(element).is(":visible");\n    }\n\n    /**\n     * Adds resizing capabilities to a given html element.\n     *\n     * Resizing can be configured in two directions:\n     *  - Vertical ("vert"): Resizes the height of the element\n     *  - Horizontal ("horz"): Resizes the width of the element\n     *\n     * Resizer handlers can be positioned on the element at:\n     *  - Top ("top") or bottom ("bottom") for vertical resizing\n     *  - Left ("left") or right ("right") for horizontal resizing\n     *\n     * A resizable element triggers the following events while resizing:\n     *  - panelResizeStart: When the resize starts. Passed the new size.\n     *  - panelResizeUpdate: When the resize gets updated. Passed the new size.\n     *  - panelResizeEnd: When the resize ends. Passed the final size.\n     *  - panelCollapsed: When the panel gets collapsed (or hidden). Passed the last size\n     *      before collapse. May occur without any resize events.\n     *  - panelExpanded: When the panel gets expanded (or shown). Passed the initial size.\n     *      May occur without any resize events.\n     *\n     * @param {!DOMNode} element DOM element which should be made resizable.\n     * @param {!string} direction Direction of the resize action: one of the DIRECTION_* constants.\n     * @param {!string} position Which side of the element can be dragged: one of the POSITION_* constants\n     *                          (TOP/BOTTOM for vertical resizing or LEFT/RIGHT for horizontal).\n     * @param {?number} minSize Minimum size (width or height) of the element\'s outer dimensions, including\n     *                          border & padding. Defaults to DEFAULT_MIN_SIZE.\n     * @param {?boolean} collapsible Indicates the panel is collapsible on double click on the\n     *                          resizer. Defaults to false.\n     * @param {?string} forceSize CSS selector indicating element whose \'left,right,top,bottom\' should be locked to\n     *                          the resizable element\'s size (useful for siblings laid out to the right of\n     *                          the element). Must lie in element\'s parent\'s subtree.\n     * @param {?boolean} createdByPanelManager For internal use only\n     */\n    function makeResizable(element, direction, position, minSize, collapsible, forceSize, createdByPanelManager) {\n\n        var $resizer            = $(\'<div class="\' + direction + \'-resizer"></div>\'),\n            $element            = $(element),\n            $parent             = $element.parent(),\n            $resizableElement   = $($element.find(".resizable-content:first")[0]),\n            $body               = $(window.document.body),\n            elementID           = $element.attr("id"),\n            elementPrefs        = PreferenceManager.getViewState(elementID) || {},\n            animationRequest    = null,\n            directionProperty   = direction === DIRECTION_HORIZONTAL ? "clientX" : "clientY",\n            directionIncrement  = (position === POSITION_TOP || position === POSITION_LEFT) ? 1 : -1,\n            elementSizeFunction = direction === DIRECTION_HORIZONTAL ? $element.width : $element.height,\n            resizerCSSPosition  = direction === DIRECTION_HORIZONTAL ? "left" : "top",\n            contentSizeFunction = direction === DIRECTION_HORIZONTAL ? $resizableElement.width : $resizableElement.height;\n\n        if (minSize === undefined) {\n            minSize = DEFAULT_MIN_SIZE;\n        }\n\n        collapsible = collapsible || false;\n\n        // For left-resizer\n        if (position === POSITION_LEFT) {\n            resizerCSSPosition = "right";\n        }\n\n        $element.prepend($resizer);\n\n        // Important so min/max sizes behave predictably\n        $element.css("box-sizing", "border-box");\n\n        // Detect legacy cases where panels in the editor area are created without using PanelManager APIs\n        if ($parent[0] && $parent.is(".content") && !createdByPanelManager) {\n            console.warn("Deprecated: resizable panels should be created via PanelManager.createBottomPanel(). Using Resizer directly will stop working in the future. \\nElement:", element);\n            $(exports).triggerHandler("deprecatedPanelAdded", [$element]);\n        }\n\n        function adjustSibling(size) {\n            if (forceSize !== undefined) {\n                var forceCSS = "left"; // default\n                switch (position) {\n                case POSITION_LEFT:\n                    forceCSS = "right";\n                    break;\n                case POSITION_RIGHT:\n                    forceCSS = "left";\n                    break;\n                case POSITION_TOP:\n                    forceCSS = "bottom";\n                    break;\n                case POSITION_BOTTOM:\n                    forceCSS = "top";\n                    break;\n                }\n                $(forceSize, $parent).css(forceCSS, size);\n            }\n        }\n\n        function resizeElement(elementSize, contentSize) {\n            elementSizeFunction.apply($element, [elementSize]);\n\n            if ($resizableElement.length) {\n                contentSizeFunction.apply($resizableElement, [contentSize]);\n            }\n        }\n\n        $element.data("show", function () {\n            var elementOffset   = $element.offset(),\n                elementSize     = elementSizeFunction.apply($element) || elementPrefs.size,\n                contentSize     = contentSizeFunction.apply($resizableElement) || elementPrefs.contentSize,\n                resizerSize     = elementSizeFunction.apply($resizer);\n\n            // Resize the element before showing it again. If the panel was collapsed by dragging\n            // the resizer, the size of the element should be 0, so we restore size in preferences\n            resizeElement(elementSize, contentSize);\n\n            $element.show();\n            elementPrefs.visible = true;\n\n            if (collapsible) {\n                $element.prepend($resizer);\n\n                if (position === POSITION_TOP) {\n                    $resizer.css(resizerCSSPosition, "");\n                } else if (position === POSITION_LEFT) {\n                    $resizer.css(resizerCSSPosition, elementSize);\n                } else if (position === POSITION_RIGHT) {\n                    $resizer.css(resizerCSSPosition, elementOffset[resizerCSSPosition] + elementSize);\n                }\n            }\n\n            adjustSibling(elementSize);\n\n            $element.trigger("panelExpanded", [elementSize]);\n            PreferenceManager.setViewState(elementID, elementPrefs, null, isResizing);\n        });\n\n        $element.data("hide", function () {\n            var elementOffset   = $element.offset(),\n                elementSize     = elementSizeFunction.apply($element),\n                resizerSize     = elementSizeFunction.apply($resizer);\n\n            $element.hide();\n            elementPrefs.visible = false;\n            if (collapsible) {\n                $resizer.insertBefore($element);\n                if (position === POSITION_RIGHT || position === POSITION_LEFT) {\n                    $resizer.css(resizerCSSPosition, 0);\n                } else if (position === POSITION_TOP) {\n                    $resizer.css(resizerCSSPosition, elementOffset[resizerCSSPosition] + elementSize - resizerSize);\n                }\n            }\n\n            adjustSibling(0);\n\n            $element.trigger("panelCollapsed", [elementSize]);\n            PreferenceManager.setViewState(elementID, elementPrefs, null, isResizing);\n        });\n\n        // If the resizer is positioned right or bottom of the panel, we need to listen to\n        // reposition it if the element size changes externally\n        function repositionResizer(elementSize) {\n            var resizerPosition = elementSize || 1;\n            if (position === POSITION_RIGHT || position === POSITION_LEFT || position === POSITION_BOTTOM) {\n                $resizer.css(resizerCSSPosition, resizerPosition);\n            }\n        }\n\n        $resizer.on("mousedown", function (e) {\n            var $resizeShield   = $("<div class=\'resizing-container " + direction + "-resizing\' />"),\n                startPosition   = e[directionProperty],\n                startSize       = $element.is(":visible") ? elementSizeFunction.apply($element) : 0,\n                newSize         = startSize,\n                previousSize    = startSize,\n                baseSize        = 0,\n                resizeStarted   = false;\n\n            isResizing = true;\n            $body.append($resizeShield);\n\n            if ($resizableElement.length) {\n                $element.children().not(".horz-resizer, .vert-resizer, .resizable-content").each(function (index, child) {\n                    if (direction === DIRECTION_HORIZONTAL) {\n                        baseSize += $(child).outerWidth();\n                    } else {\n                        baseSize += $(child).outerHeight();\n                    }\n                });\n            }\n\n            function doRedraw() {\n                // only run this if the mouse is down so we don\'t constantly loop even\n                // after we\'re done resizing.\n                if (!isResizing) {\n                    return;\n                }\n\n                // Check for real size changes to avoid unnecessary resizing and events\n                if (newSize !== previousSize) {\n                    previousSize = newSize;\n\n                    if ($element.is(":visible")) {\n                        if (newSize < 10) {\n                            toggle($element);\n                            elementSizeFunction.apply($element, [0]);\n                        } else {\n                            // Trigger resizeStarted just before the first successful resize update\n                            if (!resizeStarted) {\n                                resizeStarted = true;\n                                $element.trigger("panelResizeStart", newSize);\n                            }\n\n                            // Resize the main element to the new size. If there is a content element,\n                            // its size is the new size minus the size of the non-resizable elements\n                            resizeElement(newSize, (newSize - baseSize));\n                            adjustSibling(newSize);\n\n                            $element.trigger("panelResizeUpdate", [newSize]);\n                        }\n                    } else if (newSize > 10) {\n                        elementSizeFunction.apply($element, [newSize]);\n                        toggle($element);\n\n                        // Trigger resizeStarted after expanding the element if it was previously collapsed\n                        if (!resizeStarted) {\n                            resizeStarted = true;\n                            $element.trigger("panelResizeStart", newSize);\n                        }\n                    }\n                }\n\n                animationRequest = window.requestAnimationFrame(doRedraw);\n            }\n\n            function onMouseMove(e) {\n                // calculate newSize adding to startSize the difference\n                // between starting and current position, capped at minSize\n                newSize = Math.max(startSize + directionIncrement * (startPosition - e[directionProperty]), minSize);\n\n                // respect max size if one provided (e.g. by PanelManager)\n                var maxSize = $element.data("maxsize");\n                if (maxSize !== undefined) {\n                    newSize = Math.min(newSize, maxSize);\n                }\n\n                // don\'t let larger than parent size\n                var parentSize = elementSizeFunction.apply($element.parent());\n                if (parentSize) {\n                    newSize = Math.min(newSize, parentSize);\n                }\n\n                e.preventDefault();\n\n                if (animationRequest === null) {\n                    animationRequest = window.requestAnimationFrame(doRedraw);\n                }\n            }\n\n            $(window.document).on("mousemove", onMouseMove);\n\n            // If the element is marked as collapsible, check for double click\n            // to toggle the element visibility\n            if (collapsible) {\n                $resizeShield.on("mousedown", function (e) {\n                    $(window.document).off("mousemove", onMouseMove);\n                    $resizeShield.off("mousedown");\n                    $resizeShield.remove();\n                    animationRequest = null;\n                    toggle($element);\n                });\n            }\n\n            function endResize(e) {\n                if (isResizing) {\n\n                    var elementSize    = elementSizeFunction.apply($element);\n                    if ($element.is(":visible")) {\n                        elementPrefs.size = elementSize;\n                        if ($resizableElement.length) {\n                            elementPrefs.contentSize = contentSizeFunction.apply($resizableElement);\n                        }\n                        PreferenceManager.setViewState(elementID, elementPrefs);\n                        repositionResizer(elementSize);\n                    }\n\n                    isResizing = false;\n\n                    if (resizeStarted) {\n                        $element.trigger("panelResizeEnd", [elementSize]);\n                    }\n\n                    // We wait 300ms to remove the resizer container to capture a mousedown\n                    // on the container that would account for double click\n                    window.setTimeout(function () {\n                        $(window.document).off("mousemove", onMouseMove);\n                        $resizeShield.off("mousedown");\n                        $resizeShield.remove();\n                        animationRequest = null;\n                    }, 300);\n                }\n            }\n\n            $(window.document).one("mouseup", endResize);\n\n            e.preventDefault();\n        });\n\n        // Panel preferences initialization\n        if (elementPrefs) {\n\n            if (elementPrefs.size !== undefined) {\n                elementSizeFunction.apply($element, [elementPrefs.size]);\n            }\n\n            if (elementPrefs.contentSize !== undefined) {\n                contentSizeFunction.apply($resizableElement, [elementPrefs.contentSize]);\n            }\n\n            if (elementPrefs.visible !== undefined && !elementPrefs.visible) {\n                hide($element);\n            } else {\n                adjustSibling(elementSizeFunction.apply($element));\n                repositionResizer(elementSizeFunction.apply($element));\n            }\n        }\n    }\n\n    // Scan DOM for horz-resizable and vert-resizable classes and make them resizable\n    AppInit.htmlReady(function () {\n        var minSize = DEFAULT_MIN_SIZE;\n\n        $mainView = $(".main-view");\n\n        $(".vert-resizable").each(function (index, element) {\n\n            if ($(element).data().minsize !== undefined) {\n                minSize = $(element).data().minsize;\n            }\n\n            if ($(element).hasClass("top-resizer")) {\n                makeResizable(element, DIRECTION_VERTICAL, POSITION_TOP, minSize, $(element).hasClass("collapsible"), $(element).data().forcesize);\n            }\n\n            if ($(element).hasClass("bottom-resizer")) {\n                makeResizable(element, DIRECTION_VERTICAL, POSITION_BOTTOM, minSize, $(element).hasClass("collapsible"), $(element).data().forcesize);\n            }\n        });\n\n        $(".horz-resizable").each(function (index, element) {\n\n            if ($(element).data().minsize !== undefined) {\n                minSize = $(element).data().minsize;\n            }\n\n            if ($(element).hasClass("left-resizer")) {\n                makeResizable(element, DIRECTION_HORIZONTAL, POSITION_LEFT, minSize, $(element).hasClass("collapsible"), $(element).data().forcesize);\n            }\n\n            if ($(element).hasClass("right-resizer")) {\n                makeResizable(element, DIRECTION_HORIZONTAL, POSITION_RIGHT, minSize, $(element).hasClass("collapsible"), $(element).data().forcesize);\n            }\n        });\n    });\n\n    /**\n     * @private\n     * Examine each preference key for migration of any panel state.\n     *\n     * @param {string} key The key of the preference to be examined\n     *      for migration of panel states.\n     * @return {?string} - the scope to which the preference is to be migrated\n     */\n    function _isPanelPreferences(key) {\n        if (key) {\n            return "user";\n        }\n\n        return null;\n    }\n\n    exports.makeResizable   = makeResizable;\n    exports.toggle          = toggle;\n    exports.show            = show;\n    exports.hide            = hide;\n    exports.isVisible       = isVisible;\n\n    //Resizer Constants\n    exports.DIRECTION_VERTICAL   = DIRECTION_VERTICAL;\n    exports.DIRECTION_HORIZONTAL = DIRECTION_HORIZONTAL;\n    exports.POSITION_TOP         = POSITION_TOP;\n    exports.POSITION_RIGHT       = POSITION_RIGHT;\n});\n\n//# sourceURL=/utils/Resizer.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, window, $, brackets */\n\n/**\n * Manages layout of panels surrounding the editor area, and size of the editor area (but not its contents).\n *\n * Updates panel sizes when the window is resized. Maintains the max resizing limits for panels, based on\n * currently available window size.\n *\n * Events:\n *    - editorAreaResize -- When editor-holder\'s size changes for any reason (including panel show/hide\n *              panel resize, or the window resize).\n *              The 2nd arg is the new editor-holder height.\n *              The 3rd arg is a refreshHint flag for internal EditorManager use.\n */\ndefine(\'utils/PanelManager\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'utils/Resizer\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit                 = require("utils/AppInit"),\n        Resizer                 = require("utils/Resizer");\n\n\n    /** @type {jQueryObject} The ".content" vertical stack (editor + all header/footer panels) */\n    var $windowContent;\n\n    /** @type {jQueryObject} The "#editor-holder": has only one visible child, the current CodeMirror\n        instance (or the no-editor placeholder) */\n    var $editorHolder;\n\n    /** @type {boolean} Have we already started listening for the end of the ongoing window resize? */\n    var windowResizing = false;\n\n\n    /**\n     * Calculates the available height for the full-size Editor (or the no-editor placeholder),\n     * accounting for the current size of all visible panels, toolbar, & status bar.\n     * @return {number}\n     */\n    function calcEditorHeight() {\n        var availableHt = $windowContent.height();\n\n        $editorHolder.siblings().each(function (i, elem) {\n            var $elem = $(elem);\n            if ($elem.css("display") !== "none" && $elem.css("position") !== "absolute") {\n                availableHt -= $elem.outerHeight();\n            }\n        });\n\n        // Clip value to 0 (it could be negative if a panel wants more space than we have)\n        return Math.max(availableHt, 0);\n    }\n\n    /** Updates panel resize limits to disallow making panels big enough to shrink editor area below 0 */\n    function updateResizeLimits() {\n        var editorAreaHeight = $editorHolder.height();\n\n        $editorHolder.siblings().each(function (i, elem) {\n            var $elem = $(elem);\n            if ($elem.css("display") === "none") {\n                $elem.data("maxsize", editorAreaHeight);\n            } else {\n                $elem.data("maxsize", editorAreaHeight + $elem.outerHeight());\n            }\n        });\n    }\n\n\n    /**\n     * Calculates a new size for editor-holder and resizes it accordingly, then and dispatches the "editorAreaResize"\n     * event. (The editors within are resized by EditorManager, in response to that event).\n     *\n     * @param {string=} refreshHint  One of "skip", "force", or undefined. See EditorManager docs.\n     */\n    function triggerEditorResize(refreshHint) {\n        // Find how much space is left for the editor\n        var editorAreaHeight = calcEditorHeight();\n\n        $editorHolder.height(editorAreaHeight);  // affects size of "not-editor" placeholder as well\n\n        // Resize editor to fill the space\n        $(exports).trigger("editorAreaResize", [editorAreaHeight, refreshHint]);\n    }\n\n\n    /** Trigger editor area resize whenever the window is resized */\n    function handleWindowResize() {\n        // These are not initialized in Jasmine Spec Runner window until a test\n        // is run that creates a mock document.\n        if (!$windowContent || !$editorHolder) {\n            return;\n        }\n\n        // Immediately adjust editor\'s height, but skip the refresh since CodeMirror will call refresh()\n        // itself when it sees the window resize event\n        // triggerEditorResize("skip");\n\n        // FIXME (issue #4564) Workaround https://github.com/marijnh/CodeMirror/issues/1787\n        triggerEditorResize();\n\n        if (!windowResizing) {\n            windowResizing = true;\n\n            // We don\'t need any fancy debouncing here - we just need to react before the user can start\n            // resizing any panels at the new window size. So just listen for first mousemove once the\n            // window resize releases mouse capture.\n            $(window.document).one("mousemove", function () {\n                windowResizing = false;\n                updateResizeLimits();\n            });\n        }\n    }\n\n    /** Trigger editor area resize whenever the given panel is shown/hidden/resized */\n    function listenToResize($panel) {\n        // Update editor height when shown/hidden, & continuously as panel is resized\n        $panel.on("panelCollapsed panelExpanded panelResizeUpdate", function () {\n            triggerEditorResize();\n        });\n        // Update max size of sibling panels when shown/hidden, & at *end* of resize gesture\n        $panel.on("panelCollapsed panelExpanded panelResizeEnd", function () {\n            updateResizeLimits();\n        });\n    }\n\n\n    /**\n     * Represents a panel below the editor area (a child of ".content").\n     *\n     * @param {!jQueryObject} $panel  The entire panel, including any chrome, already in the DOM.\n     * @param {number=} minSize  Minimum height of panel in px.\n     */\n    function Panel($panel, minSize) {\n        this.$panel = $panel;\n\n        Resizer.makeResizable($panel[0], Resizer.DIRECTION_VERTICAL, Resizer.POSITION_TOP, minSize, false, undefined, true);\n        listenToResize($panel);\n    }\n\n    /** @type {jQueryObject} */\n    Panel.prototype.$panel = null;\n\n    Panel.prototype.isVisible = function () {\n        return this.$panel.is(":visible");\n    };\n\n    Panel.prototype.show = function () {\n        Resizer.show(this.$panel[0]);\n    };\n    Panel.prototype.hide = function () {\n        Resizer.hide(this.$panel[0]);\n    };\n\n    Panel.prototype.setVisible = function (visible) {\n        if (visible) {\n            Resizer.show(this.$panel[0]);\n        } else {\n            Resizer.hide(this.$panel[0]);\n        }\n    };\n\n\n    /**\n     * Creates a new panel beneath the editor area and above the status bar footer. Panel is initially invisible.\n     *\n     * @param {!string} id  Unique id for this panel. Use package-style naming, e.g. "myextension.feature.panelname"\n     * @param {!jQueryObject} $panel  DOM content to use as the panel. Need not be in the document yet.\n     * @param {number=} minSize  Minimum height of panel in px.\n     * @return {!Panel}\n     */\n    function createBottomPanel(id, $panel, minSize) {\n        $panel.insertAfter("#diagram-area-wrapper");\n        $panel.hide();\n        updateResizeLimits();  // initialize panel\'s max size\n\n        return new Panel($panel, minSize);\n    }\n\n\n    /**\n     * Used by EditorManager to notify us of layout changes our normal panel/window listeners wouldn\'t detect.\n     * For internal use only: most code should call EditorManager.resizeEditor().\n     */\n    function _notifyLayoutChange(refreshHint) {\n        triggerEditorResize(refreshHint);\n        updateResizeLimits();\n    }\n\n\n    // Attach to key parts of the overall UI, once created\n    AppInit.htmlReady(function () {\n        $windowContent = $(".content");\n        $editorHolder = $("#diagram-area-wrapper");\n\n        // Sidebar is a special case: it isn\'t a Panel, and is not created dynamically. Need to explicitly\n        // listen for resize here.\n        listenToResize($("#sidebar"));\n        listenToResize($("#navigator"));\n        listenToResize($("#toolbar"));\n        listenToResize($("#statusbar"));\n    });\n\n    // Unit test only: allow passing in mock DOM notes, e.g. for use with SpecRunnerUtils.createMockEditor()\n    function _setMockDOM($mockWindowContent, $mockEditorHolder) {\n        $windowContent = $mockWindowContent;\n        $editorHolder = $mockEditorHolder;\n    }\n\n    // If someone adds a panel in the .content stack the old way, make sure we still listen for resize/show/hide\n    // (Resizer emits a deprecation warning for us - no need to log anything here)\n    $(Resizer).on("deprecatedPanelAdded", function (event, $panel) {\n        listenToResize($panel);\n    });\n\n    // Add this as a capture handler so we\'re guaranteed to run it before the editor does its own\n    // refresh on resize.\n    window.addEventListener("resize", handleWindowResize, true);\n\n\n    // Define public API\n    exports.createBottomPanel    = createBottomPanel;\n    exports._notifyLayoutChange  = _notifyLayoutChange;\n    exports._setMockDOM          = _setMockDOM;\n});\n\n//# sourceURL=/utils/PanelManager.js'),eval('define(\'text!htmlContent/preference-dialog.html\',[],function () { return \'<div class="preference-dialog dialog modal" data-title="Preferences">\\n    <div class="dialog-body no-padding">\\n        <div class="schema-section">\\n            <div class="listview"></div>\\n        </div>\\n        <div class="item-section">\\n            <table class="grid small"></table>\\n        </div>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button primary" data-button-id="close">{{Strings.CLOSE}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/preference-dialog.html'),eval("define('text!htmlContent/preference-dialog-section.html',[],function () { return '<tr>\\n    <td colspan=\"2\">\\n        <div class=\"preference-section\">{{text}}</div>\\n    </td>\\n</tr>';});\n\n//# sourceURL=/text!htmlContent/preference-dialog-section.html"),eval('define(\'text!htmlContent/preference-dialog-restore.html\',[],function () { return \'<tr>\\n    <td colspan="2" style="text-align: left; font-size: 13px;">\\n        <div class="preference-section" style="margin-bottom: 1em;">{{text}}</div>\\n        <button class="k-button">Restore Default Settings</button>\\n    </td>\\n</tr>\';});\n\n//# sourceURL=/text!htmlContent/preference-dialog-restore.html'),eval("define('text!htmlContent/preference-dialog-item-string.html',[],function () { return '<tr>\\n    <td>\\n        <input data-id=\\'{{key}}\\' type=\\'text\\' class=\\'widget k-input k-textbox small\\' value=\\'{{value}}\\'>\\n    </td>\\n    <td>\\n        <div class=\"name\">{{text}}</div>\\n        <div class=\"description\">{{description}}</div>\\n    </td>\\n</tr>\\n';});\n\n//# sourceURL=/text!htmlContent/preference-dialog-item-string.html"),eval('define(\'text!htmlContent/preference-dialog-item-combo.html\',[],function () { return \'<tr>\\n    <td>\\n        <label data-id="{{key}}" class="widget k-combo small" style="width: {{width}};">\\n            <input type="text" value="{{value}}">\\n            <select>\\n                {{#options}}\\n                <option value="{{value}}">{{text}}</option>\\n                {{/options}}\\n            </select>\\n        </label>\\n    </td>\\n    <td>\\n        <div class="name">{{text}}</div>\\n        <div class="description">{{description}}</div>\\n    </td>\\n</tr>\\n\';});\n\n//# sourceURL=/text!htmlContent/preference-dialog-item-combo.html'),eval('define(\'text!htmlContent/preference-dialog-item-check.html\',[],function () { return \'<tr>\\n    <td>\\n        <div class="widget" style="text-align: left; float: right;">\\n            <label class="k-switch small">\\n                <input data-id=\\\'{{key}}\\\' type="checkbox" class="k-switch__input" {{#value}}checked{{/value}}>\\n                <div class="k-switch__toggle"></div>\\n            </label>\\n        </div>\\n    </td>\\n    <td>\\n        <div class="name">{{text}}</div>\\n        <div class="description">{{description}}</div>\\n    </td>\\n</tr>\\n\';});\n\n//# sourceURL=/text!htmlContent/preference-dialog-item-check.html'),eval('define(\'text!htmlContent/preference-dialog-item-color.html\',[],function () { return \'<tr>\\n    <td>\\n        {{#defaultButton}}\\n            <button class="k-button small defaultButton">{{defaultButton.text}}</button>\\n        {{/defaultButton}}\\n        <input data-id="{{key}}" type="color" class="widget">\\n    </td>\\n    <td>\\n        <div class="name">{{text}}</div>\\n        <div class="description">{{description}}</div>\\n    </td>\\n</tr>\\n\';});\n\n//# sourceURL=/text!htmlContent/preference-dialog-item-color.html'),eval("define('text!htmlContent/preference-dialog-item-number.html',[],function () { return '<tr>\\n    <td>\\n        <input data-id=\\'{{key}}\\' type=\\'text\\' class=\\'widget k-input k-textbox small\\' value=\\'{{value}}\\' style=\"width: {{width}}\">\\n    </td>\\n    <td>\\n        <div class=\"name\">{{text}}</div>\\n        <div class=\"description\">{{description}}</div>\\n    </td>\\n</tr>\\n';});\n\n//# sourceURL=/text!htmlContent/preference-dialog-item-number.html"),eval('define(\'text!htmlContent/preference-dialog-item-dropdown.html\',[],function () { return \'<tr>\\n    <td>\\n        <label class="k-select small">\\n            <select data-id="{{key}}" class="widget">\\n            {{#options}}<option value="{{value}}">{{text}}</option>{{/options}}\\n            </select>\\n        </label>\\n    </td>\\n    <td>\\n        <div class="name">{{text}}</div>\\n        <div class="description">{{description}}</div>\\n    </td>\\n</tr>\\n\';});\n\n//# sourceURL=/text!htmlContent/preference-dialog-item-dropdown.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo */\n\n\n// TODO: 키보드 처리 (ESC = Cancel, Enter = OK)\n// TODO: getChildren, getText, getIcon, getNode는 ModelExplorer와 중복 코드임. (중복제거요망)\n\ndefine(\'preference/PreferenceDialog\',[\'require\',\'exports\',\'module\',\'utils/Helper\',\'core/PreferenceManager\',\'strings\',\'dialogs/Dialogs\',\'text!htmlContent/preference-dialog.html\',\'text!htmlContent/preference-dialog-section.html\',\'text!htmlContent/preference-dialog-restore.html\',\'text!htmlContent/preference-dialog-item-string.html\',\'text!htmlContent/preference-dialog-item-combo.html\',\'text!htmlContent/preference-dialog-item-check.html\',\'text!htmlContent/preference-dialog-item-color.html\',\'text!htmlContent/preference-dialog-item-number.html\',\'text!htmlContent/preference-dialog-item-dropdown.html\'],function (require, exports, module) {\n    "use strict";\n\n    var Helper               = require("utils/Helper"),\n        PreferenceManager    = require("core/PreferenceManager"),\n        Strings              = require("strings"),\n        Dialogs              = require("dialogs/Dialogs"),\n        dialogTemplate       = require("text!htmlContent/preference-dialog.html"),\n        sectionTemplate      = require("text!htmlContent/preference-dialog-section.html"),\n        restoreTemplate      = require("text!htmlContent/preference-dialog-restore.html"),\n        stringItemTemplate   = require("text!htmlContent/preference-dialog-item-string.html"),\n        comboItemTemplate    = require("text!htmlContent/preference-dialog-item-combo.html"),\n        checkItemTemplate    = require("text!htmlContent/preference-dialog-item-check.html"),\n        colorItemTemplate    = require("text!htmlContent/preference-dialog-item-color.html"),\n        numberItemTemplate   = require("text!htmlContent/preference-dialog-item-number.html"),\n        dropdownItemTemplate = require("text!htmlContent/preference-dialog-item-dropdown.html");\n\n    var $itemGrid = null;\n\n\n    function renderSection(item) {\n        $itemGrid.append(Mustache.render(sectionTemplate, item));\n    }\n\n    function renderStringItem(item) {\n        var $item = $(Mustache.render(stringItemTemplate, item)),\n            $edit = $item.find("input");\n\n        $itemGrid.append($item);\n\n        $edit.change(function () {\n            PreferenceManager.set(item.key, $edit.val());\n        });\n    }\n\n    function renderCheckItem(item) {\n        var $item = $(Mustache.render(checkItemTemplate, item)),\n            $check = $item.find("input");\n\n        $itemGrid.append($item);\n\n        $check.change(function () {\n            PreferenceManager.set(item.key, $check.is(":checked"));\n        });\n    }\n\n    function renderNumberItem(item) {\n        if (!item.width) {\n            item.width = "4em";\n        }\n\n        var $item = $(Mustache.render(numberItemTemplate, item)),\n            $edit = $item.find("input");\n\n        $itemGrid.append($item);\n\n        $edit.change(function () {\n            var val = parseInt($edit.val());\n            if (_.isNumber(val)) {\n                PreferenceManager.set(item.key, val);\n            }\n        });\n    }\n\n    function renderComboItem(item) {\n        item.width = (_.max(_.map(item.options, function (x) { return x.text.length; })) + 2.5) + "em";\n\n        var $item   = $(Mustache.render(comboItemTemplate, item)),\n            $edit   = $item.find(".k-combo > input"),\n            $select = $item.find(".k-combo > select");\n\n        $itemGrid.append($item);\n\n        $edit.change(function () {\n            var val = $edit.val();\n            if (_.isString(val) && _.isNumber(item.default)) {\n                val = parseInt(val);\n            }\n            PreferenceManager.set(item.key, val);\n        });\n\n        $select.val(item.value);\n        $select.change(function () {\n            $edit.val($select.val());\n            $edit.change();\n        });\n        // (for Windows) To prevent immediate collapse of dropdown in comboBox.\n        $select.click(function (e) {\n            $select.focus();\n            e.stopPropagation();\n            return false;\n        });\n\n    }\n\n    function renderDropdownItem(item) {\n        var $item     = $(Mustache.render(dropdownItemTemplate, item)),\n            $dropdown = $item.find("select");\n\n        $itemGrid.append($item);\n\n        $dropdown.val(item.value);\n        $dropdown.change(function () {\n            var val = $dropdown.val();\n            if (item.default && _.isNumber(item.default)) {\n                val = parseInt($dropdown.val());\n            }\n            PreferenceManager.set(item.key, val);\n        });\n    }\n\n    function renderColorItem(item) {\n        var $item  = $(Mustache.render(colorItemTemplate, item)),\n            $color = $item.find("input[type=\'color\']");\n\n        $itemGrid.append($item);\n\n        $color.kendoColorPicker({\n            change: function (e) {\n                PreferenceManager.set(item.key, e.value);\n            }\n        });\n        var colorPicker = $color.data("kendoColorPicker");\n        colorPicker.value(item.value);\n\n        if (item.defaultButton) {\n            var $defaultButton = $item.find("button.defaultButton");\n            $defaultButton.click(function () {\n                colorPicker.value(null);\n                PreferenceManager.set(item.key, null);\n            });\n        }\n\n    }\n\n    function renderRestoreDefaultSettings(schemaId) {\n        var $item   = $(Mustache.render(restoreTemplate, {})),\n            $button = $item.find("button");\n\n        $itemGrid.append($item);\n\n        $button.click(function () {\n            var schema = PreferenceManager.getSchema(schemaId);\n            var key, item;\n            for (key in schema) {\n                if (schema.hasOwnProperty(key)) {\n                    item = schema[key];\n                    if (typeof item.default !== "undefined") {\n                        PreferenceManager.set(key, item.default);\n                    }\n                }\n            }\n            _.defer(function () {\n                renderSchema(schemaId, schema);\n            });\n        });\n    }\n\n    /**\n     * Render Preference Schema\n     */\n    function renderSchema(id, schema) {\n        $itemGrid.empty();\n\n        var key, item;\n        for (key in schema) {\n            if (schema.hasOwnProperty(key)) {\n                item = schema[key];\n                item.key = key;\n                item.value = PreferenceManager.get(key);\n\n                switch (item.type) {\n                case "Section":\n                    renderSection(item);\n                    break;\n                case "String":\n                    renderStringItem(item);\n                    break;\n                case "Check":\n                    renderCheckItem(item);\n                    break;\n                case "Number":\n                    renderNumberItem(item);\n                    break;\n                case "Combo":\n                    renderComboItem(item);\n                    break;\n                case "Dropdown":\n                    renderDropdownItem(item);\n                    break;\n                case "Color":\n                    renderColorItem(item);\n                    break;\n                }\n            }\n        }\n\n        renderRestoreDefaultSettings(id);\n    }\n\n    function showDialog(preferenceId) {\n        var context = {\n            Strings: Strings\n        };\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(dialogTemplate, context), true, function ($dlg) {\n\n        });\n\n        var $dlg = dialog.getElement(),\n            $schemaList = $dlg.find(".schema-section .listview");\n\n        // Item Container\n        $itemGrid = $dlg.find(".item-section .grid");\n\n        // Preference Schema ListView\n        var _data = [];\n        _.forEach(PreferenceManager.getSchemaIds(), function (id) {\n            _data.push({\n                id: id,\n                text: PreferenceManager.getSchemaName(id)\n            });\n        });\n\n        var dataSource = new kendo.data.DataSource({ data: _data });\n        $schemaList.kendoListView({\n            dataSource: dataSource,\n            selectable: true,\n            template: "<div>#:text#</div>",\n            change: function () {\n                var data = dataSource.view(),\n                    item = data[this.select().index()],\n                    schema = PreferenceManager.getSchema(item.id);\n                renderSchema(item.id, schema);\n            }\n        });\n        var schemaListView = $schemaList.data("kendoListView");\n\n        schemaListView.select(schemaListView.element.children().first());\n\n        // Show Specific Preference for a given preferenceId\n        if (preferenceId) {\n            var i, len, data = dataSource.view();\n            for (i = 0, len = data.length; i < len; i++) {\n                if (data[i].id === preferenceId) {\n                    schemaListView.select(schemaListView.element.children()[i]);\n                    break;\n                }\n            }\n        }\n\n        return dialog;\n    }\n\n    // Public API\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/preference/PreferenceDialog.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, loopfunc: true */\n/*global define, $, _, window, appshell */\n\ndefine('engine/ClipboardManager',['require','exports','module','core/Global','utils/Helper','core/Core','core/Repository'],function (require, exports, module) {\n    \"use strict\";\n\n    var _global    = require(\"core/Global\").global,\n        Helper     = require(\"utils/Helper\"),\n        Core       = require(\"core/Core\"),\n        Repository = require(\"core/Repository\");\n\n    /**\n     * Clipboard Data Kinds\n     * @const\n     */\n    var DK_MODEL = 'model',\n        DK_VIEWS = 'views';\n\n    /**\n     * Clipboard Data\n     */\n    var _clipboard = {\n        kind: null,   // data kind\n        type: null,   // type of element in clipboard\n        data: null\n    };\n\n    /**\n     * Replace all ids of elem and it's descendants\n     *\n     * @private\n     */\n    function _replaceIds(idMap, elem) {\n        elem.traverse(function (e) {\n            var oldId = e._id;\n            var newId = Helper.generateGuid();\n            idMap[oldId] = newId;\n            e._id = newId;\n        });\n    }\n\n    /**\n     * Resolve all refernces of elem\n     *\n     * @private\n     */\n    function _resolveRefs(idMap, elem) {\n        for (var name in elem) {\n            var val = elem[name];\n            if (val && val.$ref) {\n                if (_.has(idMap, val.$ref)) {\n                    elem[name] = idMap[val.$ref];\n                } else {\n                    elem[name] = Repository.get(val.$ref);\n                }\n            }\n        }\n    }\n\n    /**\n     * Set (Copy) Model to Clipboard\n     *\n     * Copy Context\n     * - id : parent's id\n     * - type : parent's type\n     * - field : parent's array or reference field containing the copied model\n     *\n     * @param {Core.Model} model\n     */\n    function setModel(model) {\n        var writer = new Core.Writer();\n        model.save(writer);\n        // Set Clipboard Data\n        _clipboard = {\n            kind: DK_MODEL,\n            type: model.getClassName(),\n            data: writer.current\n        };\n        // Set Copy Context\n        if (model._parent) {\n            _clipboard.context = {\n                id: model._parent._id,\n                type: model._parent.getClassName(),\n                field: model.getParentField()\n            };\n        }\n    }\n\n    /**\n     * Set Views to Clipboard\n     *\n     * @param {Array.<Core.View>} views\n     */\n    function setViews(views) {\n        var writer = new Core.Writer();\n        writer.writeObjArray(\"data\", views);\n        // Set Clipboard Data\n        _clipboard = {\n            kind: DK_VIEWS,\n            type: null,\n            data: writer.current.data\n        };\n        // Set Copy Context\n        if ((views.length > 0) && (views[0].getDiagram())) {\n            var diagram = views[0].getDiagram();\n            _clipboard.context = {\n                id: diagram._id,\n                type: diagram.getClassName()\n            };\n        }\n    }\n\n    /**\n     * Get Model from Clipboard\n     * (All elements are cloned from Clipboard)\n     *\n     * @return {Core.Model}\n     */\n    function getModel() {\n        if (_clipboard !== null && _clipboard.kind === DK_MODEL && _clipboard.data) {\n            var reader = new Core.Reader(_clipboard, _global.type),\n                model  = reader.readObj(\"data\");\n            // resolve all references\n            for (var id in reader.idMap) {\n                var obj = reader.idMap[id];\n                _resolveRefs(reader.idMap, obj);\n            }\n            // replace all ids\n            var replaceMap  = {};\n            _replaceIds(replaceMap, model);\n            return model;\n        }\n        return null;\n    }\n\n    /**\n     * Get Views from Clipboard\n     *\n     * @return {Array.<Core.View>}\n     */\n    function getViews() {\n        if (_clipboard !== null && _clipboard.kind === DK_VIEWS && _clipboard.data) {\n            var reader = new Core.Reader(_clipboard, _global.type),\n                views  = reader.readObjArray(\"data\");\n            // resolve all references\n            for (var id in reader.idMap) {\n                var obj = reader.idMap[id];\n                _resolveRefs(reader.idMap, obj);\n            }\n            // replace all ids\n            var replaceMap  = {};\n            for (var i = 0; i < views.length; i++) {\n                _replaceIds(replaceMap, views[i]);\n            }\n            return views;\n        }\n        return null;\n    }\n\n    /**\n     * Get Kind of Clipboard Data\n     *\n     * @return {Array.<Core.View>}\n     */\n    function getKind() {\n        return _clipboard.kind;\n    }\n\n    /**\n     * Get Type of Element in Clipboard\n     *\n     * @return {string}\n     */\n    function getElementType() {\n        return _clipboard.type;\n    }\n\n    /**\n     * Get Copy Context in Clipboard\n     *\n     * @return {Object}\n     */\n    function getCopyContext() {\n        return _clipboard.context;\n    }\n\n\n    exports.DK_MODEL = DK_MODEL;\n    exports.DK_VIEWS = DK_VIEWS;\n\n    // Public API\n    exports.setModel       = setModel;\n    exports.setViews       = setViews;\n    exports.getModel       = getModel;\n    exports.getViews       = getViews;\n    exports.getKind        = getKind;\n    exports.getElementType = getElementType;\n    exports.getCopyContext = getCopyContext;\n\n});\n\n//# sourceURL=/engine/ClipboardManager.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n// TODO: Close all other ContextMenus when a particular ContextMenu. This should be implemented in ContextMenuManager.\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, document, appshell, Mustache, ContextMenuItem */\n\n/**\n * ContextMenuManager\n */\ndefine(\'menu/ContextMenuManager\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Global\',\'utils/Helper\',\'command/CommandManager\',\'command/KeyBindingManager\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit           = require("utils/AppInit"),\n        global            = require("core/Global").global,\n        Helper            = require("utils/Helper"),\n        CommandManager    = require("command/CommandManager"),\n        KeyBindingManager = require("command/KeyBindingManager");\n\n    /**\n     * Insertion position constants\n     * Used by addMenuItem() to specify the relative position of a newly created menu object\n     * @enum {string}\n     */\n    var BEFORE           = "before",\n        AFTER            = "after",\n        FIRST            = "first",\n        LAST             = "last",\n        FIRST_IN_SECTION = "firstInSection",\n        LAST_IN_SECTION  = "lastInSection";\n\n    /**\n     * Maps ContextMenu id\'s to ContextMenu objects\n     * @type {Object.<string, ContextMenu>}\n     * @private\n     */\n    var contextMenuMap = {};\n\n    /**\n     * Return a new divider id\n     * @return {string}\n     */\n    var _menuDividerIDCount = 1;\n    function _getNextMenuItemDividerID() {\n        return "staruml-contextMenuDivider-" + _menuDividerIDCount++;\n    }\n\n    /**\n     * ContextMenuEntry\n     * @constructor\n     */\n    function ContextMenuEntry(id, contextMenu, dataItem) {\n        this.id = id;\n        this.contextMenu = contextMenu;\n        this.dataItem = dataItem;\n    }\n\n    /**\n     * Add dataItem to a specific position\n     * @private\n     */\n    ContextMenuEntry.prototype._addToPosition = function (dataItem, position, relativeId) {\n        var relativeItem,\n            relativeItemIdx;\n\n        if (!_.isArray(this.dataItem.items)) {\n            this.dataItem.items = [];\n        }\n\n        if (relativeId) {\n            relativeItem = _.find(\n                this.dataItem.items,\n                function (item) {\n                    return item.data === relativeId;\n                }\n            );\n            relativeItemIdx = _.indexOf(this.dataItem.items, relativeItem);\n        }\n        switch (position) {\n        case FIRST:\n            this.dataItem.items.unshift(dataItem);\n            break;\n        case LAST:\n            this.dataItem.items.push(dataItem);\n            break;\n        case BEFORE:\n            this.dataItem.items.splice(relativeItemIdx, 0, dataItem);\n            break;\n        case AFTER:\n            this.dataItem.items.splice(relativeItemIdx + 1, 0, dataItem);\n            break;\n        case FIRST_IN_SECTION:\n            // TODO: Support FIRST_IN_SECTION\n            break;\n        case LAST_IN_SECTION:\n            // TODO: Support LAST_IN_SECTION\n            break;\n        default:\n            this.dataItem.items.push(dataItem);\n            break;\n        }\n\n    };\n\n    /**\n     * Add ContextMenu Item\n     * @param {string | Command} command\n     * @param {?} keyBindings\n     * @param {string} position\n     * @param {string} relativeId\n     * @return {ContextMenuItem}\n     */\n    ContextMenuEntry.prototype.addMenuItem = function (command, iconClass, keyBindings, position, relativeId) {\n        var menuId = this.id,\n            menuItem,\n            id,\n            name,\n            commandID,\n            relativeItem,\n            relativeItemIdx;\n\n        if (!command) {\n            console.error("ContextMenu.addMenuItem(): missing required parameters: command");\n            return null;\n        }\n\n        if (typeof (command) === "string") {\n            commandID = command;\n            command = CommandManager.get(commandID);\n            if (!command) {\n                console.error("ContextMenu.addMenuItem(): commandID not found: " + commandID);\n                return null;\n            }\n            name = command.getName();\n        } else {\n            commandID = command.getID();\n            name = command.getName();\n        }\n\n        id = commandID;\n        if (this.contextMenu.menuItemMap[id]) {\n            console.error("ContextMenuItem added with same id of existing ContextMenuItem: " + id + " of ContextMenu: " + menuId);\n            return null;\n        }\n\n        // Create ContextMenuItem\n        var dataItem = {\n            text: name,\n            data: id,\n            spriteCssClass: iconClass\n        };\n\n        // Set KeyBinding Display\n        var bindings = KeyBindingManager.getKeyBindings(commandID),\n            binding,\n            bindingStr = "",\n            displayStr = "";\n        if (bindings && bindings.length > 0) {\n            binding = bindings[bindings.length - 1];\n            bindingStr = binding.displayKey || binding.key;\n        }\n        if (bindingStr.length > 0) {\n            displayStr = KeyBindingManager.formatKeyDescriptor(bindingStr);\n        }\n        if (displayStr.length > 0) {\n            var s = "<span style=\'margin-right: {{margin}}em;\'></span><span style=\'position: absolute; right: 0; margin-right: 1.2em;\'>{{key}}</span>";\n            dataItem.text += Mustache.render(s, { margin: displayStr.length / 2, key: displayStr});\n            dataItem.encoded = false;\n        }\n\n        menuItem = new ContextMenuItem(id, command, this.contextMenu, dataItem, false);\n        this.contextMenu.menuItemMap[id] = menuItem;\n        this._addToPosition(dataItem, position, relativeId);\n\n        return menuItem;\n    };\n\n\n    /**\n     * Add Divider\n     * @param {string} position\n     * @param {string} relativeId\n     * @return {ContextMenuItem}\n     */\n    ContextMenuEntry.prototype.addMenuDivider = function (position, relativeId) {\n        var id,\n            menuItem;\n\n        var dataItem = {\n            data: id,\n            text: "<hr size=\'0px\'>",\n            encoded: false,\n            enabled: false,\n            cssClass: "k-menu-separator"\n        };\n\n        id = _getNextMenuItemDividerID();\n        menuItem = new ContextMenuItem(id, null, this.contextMenu, dataItem, true);\n        this.contextMenu.menuItemMap[id] = menuItem;\n        this._addToPosition(dataItem, position, relativeId);\n\n        return menuItem;\n    };\n\n    /**\n     * ContextMenu\n     * @constructor\n     * @param {string} id\n     */\n    function ContextMenu(id, anchor, $contextMenu) {\n        this.anchor = anchor;\n        this.$contextMenu = $contextMenu;\n        this._kendoObject = null;\n        this.menuItemMap = {};\n        ContextMenuEntry.call(this, id, this, {});\n    }\n    ContextMenu.prototype = Object.create(ContextMenuEntry.prototype);\n    ContextMenu.prototype.constructor = ContextMenu;\n    ContextMenu.prototype.parentClass = ContextMenuEntry.prototype;\n\n    /**\n     * Initialize ContextMenu items.\n     * For all items, assign this.$element and add data-id attribute, and delete data-data attribute.\n     */\n    ContextMenu.prototype._initializeMenuItems = function () {\n        // Find <LI> element and assign it to ContextMenuItem.$element.\n        var $allItems = $("li.k-item[role=menuitem]", this.$contextMenu);\n        var self = this;\n        $allItems.each(function () {\n            var $item = $(this),\n                data = $item.data("data");\n            if (data) {\n                var menuItemId = JSON.parse(data);\n                var menuItem = self.menuItemMap[menuItemId];\n                menuItem.$element = $item;\n            }\n        });\n\n        // Close all other ContextMenus when a particular ContextMenu popup.\n        $(this.anchor).bind(\'contextmenu\', function () {\n            var id;\n            for (id in contextMenuMap) {\n                if (contextMenuMap.hasOwnProperty(id)) {\n                    if (id !== self.id) {\n                        contextMenuMap[id].close();\n                    }\n                }\n            }\n        });\n    };\n\n    /**\n     * Select handler\n     */\n    ContextMenu.prototype._selected = function (e) {\n        var menuId = JSON.parse(e.item.dataset.data);\n\n        // Execute Command\n        CommandManager.execute(menuId);\n\n        // Hide remained sub-context menu\n        $("div.k-animation-container", this.$contextMenu).hide();\n        $(".k-state-active", this.$contextMenu).removeClass("k-state-active");\n        $(".k-state-border-down", this.$contextMenu).removeClass("k-state-border-down");\n    };\n\n    /**\n     * Close ContextMenu\n     */\n    ContextMenu.prototype.close = function () {\n        if (this._kendoObject) {\n            this._kendoObject.hide();\n        }\n    };\n\n    /**\n     * Get Menu Item\n     * @param {string} id\n     * @return {ContextMenuItem}\n     */\n    ContextMenu.prototype.getMenuItem = function (id) {\n        return this.menuItemMap[id];\n    };\n\n    /**\n     * ContextMenuItem Object\n     * @constructor\n     */\n    function ContextMenuItem(id, command, contextMenu, dataItem, isDivider) {\n        ContextMenuEntry.call(this, id, contextMenu, dataItem);\n        this.isDivider = isDivider;\n        if (!this.isDivider) {\n            // Bind event handlers\n            this._enabledChanged = this._enabledChanged.bind(this);\n            this._checkedChanged = this._checkedChanged.bind(this);\n            this._visibleChanged = this._visibleChanged.bind(this);\n            this._nameChanged = this._nameChanged.bind(this);\n            this._keyBindingAdded = this._keyBindingAdded.bind(this);\n            this._keyBindingRemoved = this._keyBindingRemoved.bind(this);\n            this._command = command;\n            $(this._command)\n                .on("enabledStateChange", this._enabledChanged)\n                .on("checkedStateChange", this._checkedChanged)\n                .on("visibleStateChange", this._visibleChanged)\n                .on("nameChange", this._nameChanged)\n                .on("keyBindingAdded", this._keyBindingAdded)\n                .on("keyBindingRemoved", this._keyBindingRemoved);\n        }\n    }\n    ContextMenuItem.prototype = Object.create(ContextMenuEntry.prototype);\n    ContextMenuItem.prototype.constructor = ContextMenuItem;\n    ContextMenuItem.prototype.parentClass = ContextMenuEntry.prototype;\n\n    /**\n     * Synchronizes MenuItem checked state with underlying Command checked state\n     */\n    ContextMenuItem.prototype._checkedChanged = function () {\n        if (this.$element) {\n            var $inside = $("span.k-link", this.$element),\n                $checkIcon = $inside.children("span.k-i-tick"),\n                checked = !!this._command.getChecked();\n            if (checked) {\n                if ($checkIcon.length === 0) {\n                    $inside.prepend("<span class=\'k-icon k-i-tick\'></span>");\n                }\n            } else {\n                $checkIcon.remove();\n            }\n\n        }\n    };\n\n    /**\n     * Synchronizes MenuItem enabled state with underlying Command enabled state\n     */\n    ContextMenuItem.prototype._enabledChanged = function () {\n        if (this.$element) {\n            var enabled = !!this._command.getEnabled();\n            this.$element.toggleClass("k-state-disabled", !enabled);\n        }\n    };\n\n    /**\n     * Synchronizes MenuItem visible state with underlying Command visible state\n     */\n    ContextMenuItem.prototype._visibleChanged = function () {\n        if (this.$element) {\n            var visible = !!this._command.getVisible();\n            this.$element.toggle(visible);\n        }\n    };\n\n    /**\n     * Synchronizes MenuItem name with underlying Command name\n     */\n    ContextMenuItem.prototype._nameChanged = function () {\n    };\n\n    /**\n     * @private\n     * Updates MenuItem DOM with a keyboard shortcut label\n     */\n    ContextMenuItem.prototype._keyBindingAdded = function (event, keyBinding) {\n    };\n\n    /**\n     * @private\n     * Updates MenuItem DOM to remove keyboard shortcut label\n     */\n    ContextMenuItem.prototype._keyBindingRemoved = function (event, keyBinding) {\n    };\n\n\n    /**\n     * Add ContextMenu\n     * @param {string} id Unique identifier for ContextMenu.\n     * @param {string} anchor JQuery selector string where context menu will be applied.\n     * @return {ContextMenu} ContextMenu Object\n     */\n    function addContextMenu(id, anchor) {\n        var contextMenu,\n            $contextMenu;\n\n        // Insert ContextMenu Holder DIV to Body DOM.\n        $contextMenu = $(Mustache.render("<ul data-id=\'{{id}}\'/>", { id: id }));\n        $(document.body).append($contextMenu);\n\n        // Register to ContextMenuMap\n        contextMenu = new ContextMenu(id, anchor, $contextMenu);\n        contextMenuMap[id] = contextMenu;\n\n        return contextMenu;\n    }\n\n    /**\n     * Remove ContextMenu\n     * @param {string} id Unique identifier for ContextMenu.\n     */\n    function removeContextMenu(id) {\n        delete contextMenuMap[id];\n    }\n\n    /**\n     * Get ContextMenu Object\n     * @param {string} id Unique identifier for ContextMenu.\n     * @return {ContextMenu} ContextMenu Object\n     */\n    function getContextMenu(id) {\n        return contextMenuMap[id];\n    }\n\n    /**\n     * Return All ContextMenu Objects\n     * @return {Array.<ContextMenu>} Array of all ContextMenu Objects.\n     */\n    function getAllContextMenus() {\n        return _.values(contextMenuMap);\n    }\n\n    AppInit.appReady(function () {\n        // Setup all contextMenu with KendoMenuEx\n        var id,\n            contextMenu;\n        for (id in contextMenuMap) {\n            if (contextMenuMap.hasOwnProperty(id)) {\n                contextMenu = contextMenuMap[id];\n\n                if (!contextMenu.dataItem.items) {\n                    console.error("Setup ContextMenu: No MenuItems of ContextMenu: " + contextMenu.id);\n                } else {\n\n                    // Setup KendoMenuEx Object\n                    contextMenu._kendoObject = contextMenu.$contextMenu.kendoMenuEx({\n                        animation: false,\n                        delay: 120,\n                        dataSource: contextMenu.dataItem.items,\n                        anchor: contextMenu.anchor,\n                        select: contextMenu._selected\n                    }).data("kendoMenuEx");\n\n                    // Initialize ContextMenu Object\n                    contextMenu._initializeMenuItems();\n\n                }\n            }\n        }\n    });\n\n    exports.BEFORE           = BEFORE;\n    exports.AFTER            = AFTER;\n    exports.LAST             = LAST;\n    exports.FIRST            = FIRST;\n    exports.FIRST_IN_SECTION = FIRST_IN_SECTION;\n    exports.LAST_IN_SECTION  = LAST_IN_SECTION;\n\n    exports.addContextMenu     = addContextMenu;\n    exports.removeContextMenu  = removeContextMenu;\n    exports.getContextMenu     = getContextMenu;\n    exports.getAllContextMenus = getAllContextMenus;\n\n});\n\n//# sourceURL=/menu/ContextMenuManager.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, app */\n\ndefine(\'preference/DefaultPreferences\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/Initializer\',\'utils/AppInit\',\'core/Core\',\'core/PreferenceManager\'],function (require, exports, module) {\n    "use strict";\n\n    var _global           = require("core/Global").global,\n        Initializer       = require("utils/Initializer"),\n        AppInit           = require("utils/AppInit"),\n        Core              = require("core/Core"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    var generalPreferences = {\n        "view": {\n            text: "Default View Style",\n            type: "Section"\n        },\n        "view.font": {\n            text: "Font Face",\n            description: "Default font face of view elements.",\n            type: "Combo",\n            options: app.config.defaultFonts,\n            default: "Arial"\n        },\n        "view.fontSize": {\n            text: "Font Size",\n            description: "Default font size of view elements.",\n            type: "Combo",\n            options: [\n                { value: 8,  text: "8"  },\n                { value: 9,  text: "9"  },\n                { value: 10, text: "10" },\n                { value: 11, text: "11" },\n                { value: 12, text: "12" },\n                { value: 13, text: "13" },\n                { value: 14, text: "14" },\n                { value: 16, text: "16" },\n                { value: 20, text: "20" },\n                { value: 24, text: "24" },\n                { value: 28, text: "28" },\n                { value: 32, text: "32" }\n            ],\n            default: 13\n        },\n        "view.fillColor": {\n            text: "Fill Color",\n            description: "Default fill color of view elements.",\n            type: "Color",\n            default: "#ffffff"\n        },\n        "view.lineColor": {\n            text: "Line Color",\n            description: "Default line color of view elements.",\n            type: "Color",\n            default: "#000000"\n        },\n        "view.fontColor": {\n            text: "Font Color",\n            description: "Default font color of view elements.",\n            type: "Color",\n            default: "#000000"\n        },\n        "view.lineStyle": {\n            text: "Line Style",\n            description: "Default line style of edge-type view elements.",\n            type: "Dropdown",\n            options: [\n                { value: Core.LS_RECTILINEAR, text: "Rectilinear" },\n                { value: Core.LS_OBLIQUE,     text: "Oblique" },\n                { value: Core.LS_ROUNDRECT,   text: "Rounded Rectilinear" },\n                { value: Core.LS_CURVE,       text: "Curve" }\n            ],\n            default: Core.LS_OBLIQUE\n        },\n        "view.showShadow": {\n            text: "Show Shadow",\n            description: "Default showing shadow of view elements.",\n            type: "Check",\n            default: true\n        },\n        "diagramEditor": {\n            text: "DiagramEditor",\n            type: "Section"\n        },\n        "diagramEditor.showGrid": {\n            text: "Show Grid",\n            description: "Show grid on diagram editor.",\n            type: "Check",\n            default: true\n        },\n        "diagramEditor.gridSize": {\n            text: "Grid Size",\n            description: "Default size (width and height) of grid.",\n            type: "Combo",\n            options: [\n                { value: 2,  text: "2"  },\n                { value: 4,  text: "4"  },\n                { value: 6,  text: "6" },\n                { value: 8,  text: "8" },\n                { value: 10, text: "10" },\n                { value: 12, text: "12" },\n                { value: 14, text: "14" },\n                { value: 16, text: "16" },\n                { value: 18, text: "18" },\n                { value: 20, text: "20" }\n            ],\n            default: 8\n        },\n        "validation": {\n            text: "Validation",\n            type: "Section"\n        },\n        "validation.validateModel": {\n            text: "Validate Model",\n            description: "Validate model asynchronously when opening or saving files.",\n            type: "Check",\n            default: true\n        },\n        "checkUpdate": {\n            text: "Check Updates",\n            type: "Section"\n        },\n        "checkUpdate.showUpdateDialog": {\n            text: "Show Update Dialog",\n            description: "Show update dialog when application start.",\n            type: "Check",\n            default: true\n        }\n    };\n\n    AppInit.htmlReady(function () {\n        PreferenceManager.register("general", "General", generalPreferences);\n    });\n\n});\n\n//# sourceURL=/preference/DefaultPreferences.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, document, appshell */\n\ndefine(\n    /**\n     * Editor\n     * @exports core/Editor\n     */\n    'diagrams/Editor',['require','exports','module','core/Graphics','core/Graphics','core/Graphics','core/Graphics','core/Graphics','core/Graphics','core/Graphics','core/Graphics','core/Graphics','core/Toolkit','core/Core','core/Core','core/Core','core/Core','core/Core','core/Core','core/Core','core/Core'],function (require, exports, module) {\n    \"use strict\";\n\n    var Point             = require(\"core/Graphics\").Point,\n        Rect              = require(\"core/Graphics\").Rect,\n        Color             = require(\"core/Graphics\").Color,\n        Font              = require(\"core/Graphics\").Font,\n        ZoomFactor        = require(\"core/Graphics\").ZoomFactor,\n        GridFactor        = require(\"core/Graphics\").GridFactor,\n        Points            = require(\"core/Graphics\").Points,\n        Coord             = require(\"core/Graphics\").Coord,\n        Canvas            = require(\"core/Graphics\").Canvas,\n        Toolkit           = require(\"core/Toolkit\"),\n        Core              = require(\"core/Core\"),\n        NodeView          = require(\"core/Core\").NodeView,\n        EdgeView          = require(\"core/Core\").EdgeView,\n        LabelView         = require(\"core/Core\").LabelView,\n        ParasiticView     = require(\"core/Core\").ParasiticView,\n        NodeParasiticView = require(\"core/Core\").NodeParasiticView,\n        EdgeParasiticView = require(\"core/Core\").EdgeParasiticView,\n        EdgeLabelView     = require(\"core/Core\").EdgeLabelView;\n\n    var DEFAULT_CANVAS_WIDTH  = 4000,\n        DEFAULT_CANVAS_HEIGHT = 3000;\n\n    // Reduce Max-size Canvas (to avoid Crashes in Windows)\n    if (window.devicePixelRatio && window.devicePixelRatio > 1) {\n        DEFAULT_CANVAS_WIDTH  = 2700;\n        DEFAULT_CANVAS_HEIGHT = 2000;\n    }\n\n    var CURSOR_DEFAULT     = 'default',\n        CURSOR_CROSSHAIR   = 'crosshair',\n        CURSOR_TEXT        = 'text',\n        CURSOR_WAIT        = 'wait',\n        CURSOR_SW_RESIZE   = 'sw-resize',\n        CURSOR_SE_RESIZE   = 'se-resize',\n        CURSOR_NW_RESIZE   = 'nw-resize',\n        CURSOR_NE_RESIZE   = 'ne-resize',\n        CURSOR_N_RESIZE    = 'n-resize',\n        CURSOR_S_RESIZE    = 's-resize',\n        CURSOR_W_RESIZE    = 'w-resize',\n        CURSOR_E_RESIZE    = 'e-resize',\n        CURSOR_NESW_RESIZE = 'nesw-resize',\n        CURSOR_NWSE_RESIZE = 'nwse-resize',\n        CURSOR_NS_RESIZE   = 'ns-resize',\n        CURSOR_EW_RESIZE   = 'ew-resize',\n        CURSOR_HAND        = 'pointer',\n        CURSOR_MOVE        = 'move';\n\n    /**\n     * @constructor\n     *\n     * MouseEvent\n     */\n    function MouseEvent(x, y, button, count, shiftDown, altDown, ctrlDown) {\n        this.x = x;\n        this.y = y;\n        this.button = button;\n        this.count = count;\n        this.shiftDown = shiftDown;\n        this.altDown = altDown;\n        this.ctrlDown = ctrlDown;\n        this.leftButtonDown = false;\n    }\n\n    /* MouseEvent Constants */\n    MouseEvent.BUTTON1 = 0;\n    MouseEvent.BUTTON2 = 1;\n    MouseEvent.BUTTON3 = 2;\n\n    /**\n     * @constructor\n     *\n     * Handler\n     */\n    function Handler() {\n        this.handlerName = null;\n        this.containerView = null;\n    }\n\n    /**\n     * mouseDown\n     * @abstract\n     * @param {Editor} editor\n     * @param {Canvas} canvas\n     * @param {Event} e\n     */\n    Handler.prototype.mouseDown = function (editor, canvas, e) {};\n\n    /**\n     * mouseUp\n     * @abstract\n     * @param {Editor} editor\n     * @param {Canvas} canvas\n     * @param {Event} e\n     */\n    Handler.prototype.mouseUp = function (editor, canvas, e) {};\n\n    /**\n     * mouseUp\n     * @abstract\n     * @param {Editor} editor\n     * @param {Canvas} canvas\n     * @param {Event} e\n     */\n    Handler.prototype.mouseMove = function (editor, canvas, e) {};\n\n    /**\n     * ManipulatorBinder\n     */\n    function ManipulatorBinder(selectHandler) {\n        this.nodeManipulator = new NodeManipulator();\n        this.edgeManipulator = new EdgeManipulator();\n        this.parasiticManipulator = new ParasiticManipulator();\n    }\n\n    /**\n     * bind\n     */\n    ManipulatorBinder.prototype.bind = function (view) {\n        if (view instanceof ParasiticView) {\n            return this.parasiticManipulator;\n        } else if (view instanceof NodeView) {\n            return this.nodeManipulator;\n        } else if (view instanceof EdgeView) {\n            return this.edgeManipulator;\n        }\n        return null;\n    };\n\n    /**\n     * ManipulatorBinder\n     */\n    function ManipulatableNotifier() {\n        // nothing to do.\n    }\n\n    ManipulatableNotifier.prototype.mouseMove = function (editor, canvas, e) {\n        if (editor.diagram.selectedViews.length !== 1) {\n            editor.setCursor(CURSOR_DEFAULT);\n            return;\n        }\n        var z = new Point(e.x, e.y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        var v = editor.diagram.selectedViews[0];\n        var p = new Point(e.x, e.y);\n\n        if (v instanceof NodeView) {\n            var zr = v.getBoundingBox(canvas);\n            Coord.coordTransform2(canvas.zoomFactor, GridFactor.NO_GRID, zr);\n            var mx = (zr.x1 + zr.x2) / 2,\n                my = (zr.y1 + zr.y2) / 2;\n            var b = true;\n            var selectionType = Toolkit.CT_ELSE;\n            if ((v.sizable === Core.SZ_NONE) || (v.autoResize === true)) {\n                selectionType = Toolkit.CT_ELSE;\n                b = false;\n            }\n            if ((b === true) && ((v.sizable === Core.SZ_FREE) || (v.sizable === Core.SZ_RATIO))) {\n                if (Coord.equalPt2(new Point(zr.x1 - Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, zr.y1 - Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE), p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                    selectionType = Toolkit.CT_LT;\n                    b = false;\n                } else if (Coord.equalPt2(new Point(zr.x2 + Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, zr.y1 - Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE), p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                    selectionType = Toolkit.CT_RT;\n                    b = false;\n                } else if (Coord.equalPt2(new Point(zr.x1 - Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, zr.y2 + Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE), p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                    selectionType = Toolkit.CT_LB;\n                    b = false;\n                } else if (Coord.equalPt2(new Point(zr.x2 + Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, zr.y2 + Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE), p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                    selectionType = Toolkit.CT_RB;\n                    b = false;\n                }\n            }\n            if ((b === true) && ((v.sizable === Core.SZ_FREE) || (v.sizable === Core.SZ_VERT))) {\n                if (Coord.equalPt2(new Point(mx, zr.y1 - Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE), p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                    selectionType = Toolkit.CT_MT;\n                    b = false;\n                } else if (Coord.equalPt2(new Point(mx, zr.y2 + Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE), p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                    selectionType = Toolkit.CT_MB;\n                    b = false;\n                }\n            }\n            if ((b === true) && ((v.sizable === Core.SZ_FREE) || (v.sizable === Core.SZ_HORZ))) {\n                if (Coord.equalPt2(new Point(zr.x1 - Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, my), p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                    selectionType = Toolkit.CT_LM;\n                    b = false;\n                } else if (Coord.equalPt2(new Point(zr.x2 + Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, my), p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                    selectionType = Toolkit.CT_RM;\n                    b = false;\n                }\n            }\n            if (b === true) {\n                selectionType = Toolkit.CT_ELSE;\n            }\n            switch (selectionType) {\n            case Toolkit.CT_LT:\n            case Toolkit.CT_RB:\n                editor.setCursor(CURSOR_NWSE_RESIZE);\n                break;\n            case Toolkit.CT_RT:\n            case Toolkit.CT_LB:\n                editor.setCursor(CURSOR_NESW_RESIZE);\n                break;\n            case Toolkit.CT_MT:\n            case Toolkit.CT_MB:\n                editor.setCursor(CURSOR_NS_RESIZE);\n                break;\n            case Toolkit.CT_LM:\n            case Toolkit.CT_RM:\n                editor.setCursor(CURSOR_EW_RESIZE);\n                break;\n            case Toolkit.CT_AREA:\n                editor.setCursor(CURSOR_MOVE);\n                break;\n            case Toolkit.CT_ELSE:\n                editor.setCursor(CURSOR_DEFAULT);\n                break;\n            }\n        } else if (v instanceof EdgeView) {\n            var zp1 = v.points.getPoint(0).copy(),\n                zp2 = v.points.getPoint(v.points.count() - 1).copy();\n            Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, zp1);\n            Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, zp2);\n            if (Coord.equalPt2(zp1, p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                editor.setCursor(CURSOR_HAND);\n            } else if (Coord.equalPt2(zp2, p, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE)) {\n                editor.setCursor(CURSOR_HAND);\n            } else {\n                editor.setCursor(CURSOR_DEFAULT);\n            }\n        }\n    };\n\n    /**\n     * ContainmentHandlingProxy\n     */\n    function ContainmentHandlingProxy() {\n        this.containerView = null;\n        this.activated = true;\n    }\n\n    ContainmentHandlingProxy.prototype.canBeContainedAt = function (view, targetView) {\n        if ((view === null) || (targetView === null) || (view.model === null) || (targetView.model === null)) {\n            return false;\n        }\n        return targetView.canContainView(view) && targetView.model.canContain(view.model);\n    };\n\n    ContainmentHandlingProxy.prototype.canBeContainedAt2 = function (viewSet, targetView) {\n        for (var i = 0, len = viewSet.length; i < len; i++) {\n            if (!this.canBeContainedAt(viewSet[i], targetView)) {\n                return false;\n            }\n        }\n        return true;\n    };\n\n    ContainmentHandlingProxy.prototype.beginHandling = function (editor) {\n        this.containerView = null;\n        this.activated = true;\n    };\n\n    ContainmentHandlingProxy.prototype.mouseMoveHandling = function (editor, canvas, view, targetView) {\n        if ((!this.activated) || (view === null)) {\n            return;\n        }\n        if (this.containerView !== null) {\n            editor.eraseContainingBox(this.containerView);\n        }\n        if (targetView === null) {\n            this.containerView = null;\n        } else if (this.canBeContainedAt(view, targetView)) {\n            this.containerView = targetView;\n        } else {\n            this.containerView = view.containerView;\n        }\n        if (this.containerView !== null) {\n            editor.drawContainingBox(this.containerView);\n        }\n    };\n\n    ContainmentHandlingProxy.prototype.mouseMoveHandling2 = function (editor, canvas, viewSet, targetView) {\n        if ((!this.activated) || (viewSet.length === 0)) {\n            return;\n        }\n        if (this.containerView !== null) {\n            editor.eraseContainingBox(this.containerView);\n        }\n        if (this.targetView === null) {\n            this.containerView = null;\n        } else if (this.canBeContainedAt2(viewSet, targetView)) {\n            this.containerView = targetView;\n        } else {\n            this.containerView = viewSet[0].containerView;\n        }\n        if (this.containerView !== null) {\n            editor.drawContainingBox(this.containerView);\n        }\n    };\n\n    ContainmentHandlingProxy.prototype.mouseUpHandling = function (editor, canvas) {\n        if (!this.activated) {\n            return;\n        }\n        if (this.containerView !== null) {\n            editor.eraseContainingBox(this.containerView);\n        }\n    };\n\n    ContainmentHandlingProxy.prototype.endHandling = function (editor) {\n        this.activated = false;\n        this.containerView = null;\n    };\n\n    /**\n     * SelectHandler Constants\n     * @const\n     */\n    var SM_NONE        = 0,\n        SM_SELECT_AREA = 1,\n        SM_INDIVIDUAL  = 2,\n        SM_ADDITIONAL  = 3,\n        SM_GROUPING    = 4;\n\n    /**\n     * @constructor\n     *\n     * SelectHandler\n     */\n    function SelectHandler() {\n        Handler.apply(this, arguments);\n        this.mode = SM_NONE;\n        this.f1 = new Point(0, 0);\n        this.f2 = new Point(0, 0);\n        this.g1 = new Point(0, 0);\n        this.g2 = new Point(0, 0);\n        this.manipulatorBinder = new ManipulatorBinder(this);\n        this.manipulatableNotifier = new ManipulatableNotifier();\n        this.manipulator = null;\n        this.dragged = false;\n        this.doubleClicked = false;\n        this.boundingBox = new Rect(0, 0, 0, 0);\n        this.view = null;\n    }\n    // SelectHandler inherits Handler\n    SelectHandler.prototype = Object.create(Handler.prototype);\n    SelectHandler.prototype.constructor = SelectHandler;\n\n    /**\n     * hasTheSameContainerView\n     */\n    SelectHandler.prototype.hasTheSameContainerView = function (viewSet) {\n        if (viewSet.length > 1) {\n            var c = viewSet[0].containerView;\n            for (var i = 1, len = viewSet.length; i < len; i++) {\n                if (viewSet[i].containerView != c) {\n                    return false;\n                }\n            }\n        }\n        return true;\n    };\n\n    /**\n     * isPointInSelectionLine\n     */\n    SelectHandler.prototype.isPointInSelectionLine = function (canvas, view, x, y) {\n        if (view === null) {\n            return false;\n        }\n        if (!view.selected) {\n            return false;\n        }\n        var z = new Point(x, y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        if ((view instanceof NodeView) && (view.sizable !== Core.SZ_NONE)) {\n            var zr = view.getBoundingBox(canvas);\n            Coord.coordTransform2(canvas.zoomFactor, GridFactor.NO_GRID, zr);\n            if ((!Coord.ptInRect(x, y, zr)) && view.containsPoint(canvas, z.x, z.y)) {\n                return true;\n            }\n        } else if (view instanceof EdgeView) {\n            return view.containsPoint(canvas, z.x, z.y);\n        }\n        return false;\n    };\n\n    /**\n     * mouseDown\n     *\n     * (Notice) Choice of coordinates in PSelectHandler\n     * -------------------------------------------------\n     * 1. Seleting behavior\n     *    - Zoom only applied coordinate\n     *    - Use FX1, FY1, FX2, FY2 variable\n     * 2. Moving behavior with multiple selected views\n     *    - Zoom and grid applied coordinate\n     *    - Use GX1, GY1, GX2, GY2 variable\n     * 3. Manipulating bevaior with one selected view\n     *    - Propagate coordinate to PManipulator as it is\n     *    - Convert in PManipulator or it's subclasses\n     *\n     * @override\n     */\n    SelectHandler.prototype.mouseDown = function (editor, canvas, e) {\n        // ZoomFactor only applied coordinates.\n        var x = e.x,\n            y = e.y,\n            z = new Point(x, y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        // ZoomFactor and GridFactor applied coordinates\n        var b = new Point(x, y);\n        Coord.coordRevTransform(canvas.zoomFactor, canvas.gridFactor, b);\n        this.f1.setPoint2(z);\n        this.f2.setPoint2(z);\n        this.g1.setPoint2(b);\n        this.g2.setPoint2(b);\n        this.dragged = false;\n        this.doubleClicked = false;\n        if (editor.diagram.selectedViews.length === 1 && this.isPointInSelectionLine(canvas, editor.diagram.selectedViews[0], x, y)) {\n            this.view = editor.diagram.selectedViews[0];\n        } else {\n            this.view = editor.diagram.getViewAt(canvas, z.x, z.y);\n        }\n        if (e.button === MouseEvent.BUTTON1) {\n            // Area selection mode\n            if (this.view === null) {\n                this.mode = SM_SELECT_AREA;\n                editor.drawRubberband(this.f1.x, this.f1.y, this.f2.x, this.f2.y);\n            // Not Area selection mode\n            } else {\n                // View selection/deselection\n                if (e.shiftDown) {\n                    // Shift + Mouse Left button down\n                    if (this.view.selected) {\n                        editor.deselectView(this.view);\n                    } else {\n                        editor.selectAdditionalView(this.view);\n                    }\n                } else {\n                    // Only mouse left button down\n                    if (!this.view.selected) {\n                        editor.selectView(this.view);\n                    }\n                }\n                // Selected view(s) modifying mode\n                if (editor.diagram.selectedViews.length > 0 && e.shiftDown) {\n                    this.mode = SM_ADDITIONAL;\n                } else if (editor.diagram.selectedViews.length === 1) {\n                    // One view selected\n                    this.mode = SM_INDIVIDUAL;\n                    if (e.count === 2) {\n                        this.doubleClicked = true;\n                    }\n                    this.manipulator = this.manipulatorBinder.bind(this.view);\n                    // propagate mouse down event to suitable manipulator\n                    if (this.manipulator !== null) {\n                        this.manipulator.mouseDown(editor, canvas, this.view, e);\n                    }\n                    // Intermin code (Refactoring needed)\n                    if (this.manipulator instanceof NodeManipulator) {\n                        if (this.view.containerChangeable && (this.manipulator.selectedPtType == Toolkit.CT_ELSE)) {\n                            editor.containmentHandlingProxy.beginHandling(editor);\n                        }\n                    }\n                } else if (editor.diagram.selectedViews.length > 1) {\n                    this.mode = SM_GROUPING;\n                    // Multiple views selected\n                    this.boundingBox = editor.diagram.getSelectedBoundingBox(canvas);\n                    var vs = [];\n                    for (var i = 0, len = editor.diagram.selectedViews.length; i < len; i++) {\n                        vs.push(editor.diagram.selectedViews[i]);\n                    }\n                    if (this.hasTheSameContainerView(vs)) {\n                        editor.containmentHandlingProxy.beginHandling(editor);\n                    }\n                } else {\n                    // no selected view\n                    this.mode = SM_NONE;\n                }\n            }\n        } else if (e.button === MouseEvent.BUTTON3) {\n            if (this.view === null) {\n                editor.diagram.deselectAll();\n                editor.selectArea(this.f1.x, this.f1.y, this.f2.x, this.f2.y);\n            } else if (!this.view.selected) {\n                editor.selectView(this.view);\n            }\n            this.mode = SM_NONE;\n        }\n        if (this.containerView !== null) { this.containerView = null; }\n    };\n\n    /**\n     * @override\n     */\n    SelectHandler.prototype.mouseMove = function (editor, canvas, e) {\n        var b, dx, dy, x, y, z, v;\n        // if not mouse left button downed then exit\n\n        if (e.leftButtonDown !== true) {\n            this.manipulatableNotifier.mouseMove(editor, canvas, e);\n            return;\n        }\n\n        // ZoomFactor only applied coordinates\n        x = e.x;\n        y = e.y;\n        z = new Point(x, y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        // ZoomFactor and GridFactor applied coordinates\n        b = new Point(x, y);\n        Coord.coordRevTransform(canvas.zoomFactor, canvas.gridFactor, b);\n\n        // Area selected mode\n        if (this.mode === SM_SELECT_AREA) {\n            // Erase exist bounding box\n            editor.eraseRubberband(this.f1.x, this.f1.y, this.f2.x, this.f2.y);\n            this.f2.setPoint2(z);\n            // Correct selection area not to stray from canvas\n            if (this.f2.x < 0) {\n                this.f2.x = 0.0;\n            }\n            if (this.f2.x > editor.diagramWidth) {\n                this.f2.x = editor.diagramWidth;\n            }\n            if (this.f2.y < 0) {\n                this.f2.y = 0.0;\n            }\n            if (this.f2.y > editor.diagramHeight) {\n                this.f2.x = editor.diagramHeight;\n            }\n            // Draw new bounding box\n            editor.drawRubberband(this.f1.x, this.f1.y, this.f2.x, this.f2.y);\n        } else {\n            // Multiple views selected mode\n            if (this.mode === SM_GROUPING) {\n\n                // View at (z.x, z.y) should be included to make backup for rubberbands\n                // Rubberband를 그리기 전에 Backup이 결정되므로, 반드시 manipulator.moveMove가 호출되기 전에\n                // Backup View들이 결정되어야만 한다.\n                v = editor.diagram.getViewAt(canvas, z.x, z.y);\n                if (v !== null) {\n                    editor.addBackupView(v);\n                }\n\n                dx = b.x - this.g2.x;\n                dy = b.y - this.g2.y;\n                this.g2.setPoint2(b);\n                if (this.dragged) {\n                    // Erase existing bounding box\n                    editor.eraseRubberband(this.boundingBox.x1, this.boundingBox.y1, this.boundingBox.x2, this.boundingBox.y2);\n                } else {\n                    // at starting of drag\n                    this.dragged = true;\n                }\n                // Move bounding box and draw it\n                this.boundingBox.x1 = this.boundingBox.x1 + dx;\n                this.boundingBox.y1 = this.boundingBox.y1 + dy;\n                this.boundingBox.x2 = this.boundingBox.x2 + dx;\n                this.boundingBox.y2 = this.boundingBox.y2 + dy;\n\n                // Correct bounding box not to stray from canvas\n                if (this.boundingBox.x1 < 0) {\n                    this.g2.x = b.x - this.boundingBox.x1;\n                    this.boundingBox.x2 = this.boundingBox.x2 - this.boundingBox.x1;\n                    this.boundingBox.x1 = 0;\n                }\n                if (this.boundingBox.x2 > editor.diagramWidth) {\n                    this.g2.x = this.b.x - (this.boundingBox.x2 - editor.diagramWidth);\n                    this.boundingBox.x1 = editor.diagramWidth - (this.boundingBox.x2 - this.boundingBox.x1);\n                    this.boundingBox.x2 = editor.diagramWidth;\n                }\n                if (this.boundingBox.y1 < 0) {\n                    this.g2.y = b.y - this.boundingBox.y1;\n                    this.boundingBox.y2 = this.boundingBox.y2 - this.boundingBox.y1;\n                    this.boundingBox.y1 = 0;\n                }\n                if (this.boundingBox.y2 > editor.diagramHeight) {\n                    this.g2.y = b.y - (this.boundingBox.y2 - editor.diagramHeight);\n                    this.boundingBox.y1 = editor.diagramHeight - (this.boundingBox.y2 - this.boundingBox.y1);\n                    this.boundingBox.y2 = editor.diagramHeight;\n                }\n                editor.drawRubberband(this.boundingBox.x1, this.boundingBox.y1, this.boundingBox.x2, this.boundingBox.y2);\n\n                // Test containment relation (Refactoring needed)\n                var vs = [];\n                for (var i = 0, len = editor.diagram.selectedViews.length; i < len; i++) {\n                    vs.push(editor.diagram.selectedViews[i]);\n                }\n                editor.containmentHandlingProxy.mouseMoveHandling2(editor, canvas, vs, v);\n\n            // Single view selected mode\n            } else if (this.mode === SM_INDIVIDUAL) {\n                // View at (z.x, z.y) should be included to make backup for rubberbands\n                // Rubberband를 그리기 전에 Backup이 결정되므로, 반드시 manipulator.moveMove가 호출되기 전에\n                // Backup View들이 결정되어야만 한다.\n                v = editor.diagram.getViewAt(canvas, z.x, z.y);\n                if (v !== null) {\n                    editor.addBackupView(v);\n                }\n\n                // propagate mouse move event to suitable manipulator\n                // [Notice] Each manipulator must translate coordinates.\n                if (this.manipulator !== null) {\n                    this.manipulator.mouseMove(editor, canvas, this.view, e);\n                }\n\n                // Test containment relation (Refactoring needed)\n                if (this.manipulator instanceof NodeManipulator) {\n                    if (this.view.containerChangeable && (this.manipulator.selectedPtType === Toolkit.CT_ELSE) && ((this.manipulator.f1.x !== this.manipulator.f2.x) || (this.manipulator.f1.y !== this.manipulator.f2.y))) {\n                        editor.containmentHandlingProxy.mouseMoveHandling(editor, canvas, this.view, v);\n                    }\n                }\n            }\n        }\n    };\n\n    /**\n     * @override\n     */\n    SelectHandler.prototype.mouseUp = function (editor, canvas, e) {\n        var z = new Point(e.x, e.y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        // Area selected mode\n        if (this.mode === SM_SELECT_AREA) {\n            // Erase existing bounding box\n            editor.eraseRubberband(this.f1.x, this.f1.y, this.f2.x, this.f2.y);\n            Coord.normalizeRect2(this.f1, this.f2);\n            if (!e.shiftDown) { editor.diagram.deselectAll(); }\n            editor.selectArea(this.f1.x, this.f1.y, this.f2.x, this.f2.y);\n        // Multiple views selected mode\n        } else if (this.mode === SM_GROUPING) {\n            if (this.dragged) {\n                // Erase existing bounding box\n                editor.eraseRubberband(this.boundingBox.x1, this.boundingBox.y1, this.boundingBox.x2, this.boundingBox.y2);\n            } else if (!e.shiftDown) {\n                editor.selectView(this.view);\n            }\n            editor.containmentHandlingProxy.mouseUpHandling(editor, canvas);\n            // Selected views moved\n            if (this.g2.x !== this.g1.x || this.g2.y !== this.g1.y) {\n                if (editor.containmentHandlingProxy.activated && editor.diagram.selectedViews[0].containerView != editor.containmentHandlingProxy.containerView) {\n                    editor.changeSelectedViewsContainer(this.g2.x - this.g1.x, this.g2.y - this.g1.y, editor.containmentHandlingProxy.containerView);\n                } else {\n                    editor.moveSelectedViews(this.g2.x - this.g1.x, this.g2.y - this.g1.y);\n                }\n            }\n            editor.containmentHandlingProxy.endHandling(editor);\n        // Single view selected mode\n        } else if (this.mode === SM_INDIVIDUAL) {\n            // Test containment relation (Refactoring needed)\n            if (this.manipulator instanceof NodeManipulator) {\n                if (this.manipulator.selectedPtType == Toolkit.CT_ELSE) {\n                    editor.containmentHandlingProxy.mouseUpHandling(editor, canvas);\n                }\n            }\n\n            // Propagate mouse up event to suitable manipulator\n            if (this.manipulator !== null) {\n                this.manipulator.mouseUp(editor, canvas, this.view, e);\n            }\n            if (editor.containmentHandlingProxy.activated) {\n                editor.containmentHandlingProxy.endHandling(editor);\n            }\n            if (this.doubleClicked) {\n                // CAUTION! This event is not occurred. double click event is triggered by canvas.\n                editor.viewDoubleClicked(this.view, z.x, z.y);\n            }\n            this.doubleClicked = false;\n        }\n        editor.clearBackupViews();\n        this.mode = SM_NONE;\n    };\n\n    /**\n     * @constructor\n     */\n    function CreateHandler(id, skeletonKind, callback) {\n        this.id = id;\n        this.skeletonKind = skeletonKind;\n        this.callback = callback;\n        this.leftPressed = false;\n        this.f1 = new Point(0, 0);\n        this.f2 = new Point(0, 0);\n    }\n\n    // CreateHandler inhertis Handler\n    CreateHandler.prototype = Object.create(Handler.prototype);\n    CreateHandler.prototype.constructor = CreateHandler;\n\n    /**\n     * @override\n     */\n    CreateHandler.prototype.mouseDown = function (editor, canvas, e) {\n        if (e.button === MouseEvent.BUTTON1) {\n            this.leftPressed = true;\n            var b = new Point(e.x, e.y);\n            Coord.coordRevTransform(canvas.zoomFactor, canvas.gridFactor, b);\n            this.f1.setPoint2(b);\n            this.f2.setPoint2(b);\n            editor.drawRubberband(this.f1.x, this.f1.y, this.f2.x, this.f2.y, this.skeletonKind, true);\n        }\n    };\n\n    /**\n     * @override\n     */\n    CreateHandler.prototype.mouseMove = function (editor, canvas, e) {\n        var z = new Point(e.x, e.y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        var b = new Point(e.x, e.y);\n        Coord.coordRevTransform(canvas.zoomFactor, canvas.gridFactor, b);\n        if (this.leftPressed) {\n            editor.eraseRubberband(this.f1.x, this.f1.y, this.f2.x, this.f2.y, this.skeletonKind, true);\n            switch (this.skeletonKind) {\n            case 'rect':\n                this.f2.setPoint2(b);\n                break;\n            case 'line':\n                this.f2.setPoint2(z);\n                break;\n            case 'point':\n                this.f2.setPoint2(z);\n            }\n            if (this.f2.x < 0) {\n                this.f2.x = 0;\n            }\n            if (this.f2.x > editor.diagramWidth) {\n                this.f2.x = editor.diagramWidth;\n            }\n            if (this.f2.y < 0) {\n                this.f2.y = 0;\n            }\n            if (this.f2.y > editor.diagramHeight) {\n                this.f2.y = editor.diagramHeight;\n            }\n            editor.drawRubberband(this.f1.x, this.f1.y, this.f2.x, this.f2.y, this.skeletonKind, true);\n        }\n    };\n\n    /**\n     * @override\n     */\n    CreateHandler.prototype.mouseUp = function (editor, canvas, e) {\n        if (e.button === MouseEvent.BUTTON1) {\n            this.leftPressed = false;\n            editor.eraseRubberband(this.f1.x, this.f1.y, this.f2.x, this.f2.y, this.skeletonKind, true);\n            if (this.callback !== null) {\n                this.callback(this.id, editor, this.f1.x, this.f1.y, this.f2.x, this.f2.y);\n            }\n        }\n    };\n\n\n    /**\n     * @constructor\n     *\n     * Manipulator\n     */\n    function Manipulator() {\n        this.dragged = false;\n        this.f1 = null;\n        this.f2 = null;\n    }\n\n    /** @abstract */\n    Manipulator.prototype.beginManipulate = function (editor, canvas, view, x, y) {};\n\n    /** @abstract */\n    Manipulator.prototype.drawSkeleton = function (editor, canvas) {};\n\n    /** @abstract */\n    Manipulator.prototype.eraseSkeleton = function (editor, canvas) {};\n\n    /** @abstract */\n    Manipulator.prototype.moveSkeleton = function (editor, canvas, view, delta) {};\n\n    /** @abstract */\n    Manipulator.prototype.endManipulate = function (editor, canvas, view, dx, dy) {};\n\n    /**\n     *\n     */\n    Manipulator.prototype.mouseDown = function (editor, canvas, view, e) {\n        var x, y, z;\n        x = e.x;\n        y = e.y;\n        this.beginManipulate(editor, canvas, view, x, y);\n        this.dragged = false;\n        z = new Point(x, y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        this.f1 = z.copy();\n        this.f2 = z.copy();\n    };\n\n    /**\n     *\n     */\n    Manipulator.prototype.mouseUp = function (editor, canvas, view, e) {\n        if (this.dragged) {\n            this.eraseSkeleton(editor, canvas);\n        }\n        this.endManipulate(editor, canvas, view, this.f2.x - this.f1.x, this.f2.y - this.f1.y);\n    };\n\n    /**\n     *\n     */\n    Manipulator.prototype.mouseMove = function (editor, canvas, view, e) {\n        var delta, z;\n        z = new Point(e.x, e.y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        delta = new Point(z.x - this.f2.x, z.y - this.f2.y);\n        if (this.dragged) {\n            this.eraseSkeleton(editor, canvas);\n        } else {\n            if ((delta.x !== 0) || (delta.y !== 0)) {\n                this.dragged = true;\n            }\n        }\n        this.moveSkeleton(editor, canvas, view, delta);\n        this.f2.x = this.f2.x + delta.x;\n        this.f2.y = this.f2.y + delta.y;\n        if (this.dragged) {\n            this.drawSkeleton(editor, canvas);\n        }\n    };\n\n\n    /**\n     * @constructor\n     */\n    function NodeManipulator() {\n        Manipulator.apply(this, arguments);\n        this.minX = 0;\n        this.minY = 0;\n        this.movable = Core.MM_FREE;\n        this.sizable = Core.SZ_FREE;\n        this.left = 0;\n        this.top = 0;\n        this.right = 0;\n        this.bottom = 0;\n        this.minRect = null;\n        this.originRect = null;\n        this.ratio = 0;\n        this.selectedPtType = 0;\n    }\n\n    // NodeManipulator inherits Manipulator\n    NodeManipulator.prototype = Object.create(Manipulator.prototype);\n    NodeManipulator.prototype.constructor = NodeManipulator;\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.gridFitX = function (canvas, x) {\n        return x - (x % canvas.gridFactor.width);\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.gridFitY = function (canvas, y) {\n        return y - (y % canvas.gridFactor.height);\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.resizeSkeletonLeft = function (canvas, delta) {\n        var _ref;\n        if ((this.left + delta.x) >= this.minRect.x1) {\n            delta.x = this.minRect.x1 - this.left;\n            this.left = this.minRect.x1;\n        } else if ((this.originRect.x1 <= (_ref = this.left + delta.x) && _ref <= (this.gridFitX(canvas, this.originRect.x1) + canvas.gridFactor.width))) {\n            delta.x = this.originRect.x1 - this.left;\n            this.left = this.originRect.x1;\n        } else {\n            delta.x = delta.x - ((this.left + delta.x) - this.gridFitX(canvas, this.left + delta.x));\n            this.left = this.gridFitX(canvas, this.left + delta.x);\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.resizeSkeletonRight = function (canvas, delta) {\n        var _ref;\n        if ((this.right + delta.x) <= this.minRect.x2 || this.gridFitX(canvas, this.right + delta.x) <= this.minRect.x2) {\n            delta.x = this.minRect.x2 - this.right;\n            this.right = this.minRect.x2;\n        } else if ((this.originRect.x2 <= (_ref = this.right + delta.x) && _ref <= (this.gridFitX(canvas, this.originRect.x2) + canvas.gridFactor.width))) {\n            delta.x = this.originRect.x2 - this.right;\n            this.right = this.originRect.x2;\n        } else {\n            delta.x = delta.x - ((this.right + delta.x) - this.gridFitX(canvas, this.right + delta.x));\n            this.right = this.gridFitX(canvas, this.right + delta.x);\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.resizeSkeletonTop = function (canvas, delta) {\n        var _ref;\n        if ((this.top + delta.y) >= this.minRect.y1) {\n            delta.y = this.minRect.y1 - this.top;\n            this.top = this.minRect.y1;\n        } else if ((this.originRect.y1 <= (_ref = this.top + delta.y) && _ref <= this.gridFitY(canvas, this.originRect.y1) + canvas.gridFactor.height)) {\n            delta.y = this.originRect.y1 - this.top;\n            this.top = this.originRect.y1;\n        } else {\n            delta.y = delta.y - ((this.top + delta.y) - this.gridFitY(canvas, this.top + delta.y));\n            this.top = this.gridFitY(canvas, this.top + delta.y);\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.resizeSkeletonBottom = function (canvas, delta) {\n        var _ref;\n        if ((this.bottom + delta.y) <= this.minRect.y1 || this.gridFitY(canvas, this.bottom + delta.y) <= this.minRect.y2) {\n            delta.y = this.minRect.y2 - this.bottom;\n            this.bottom = this.minRect.y2;\n        } else if ((this.originRect.y2 <= (_ref = this.bottom + delta.y) && _ref <= this.gridFitY(canvas, this.originRect.y2) + canvas.gridFactor.height)) {\n            delta.y = this.originRect.y2 - this.bottom;\n            this.bottom = this.originRect.y2;\n        } else {\n            delta.y = delta.y - ((this.bottom + delta.y) - this.gridFitY(canvas, this.bottom + delta.y));\n            this.bottom = this.gridFitY(canvas, this.bottom + delta.y);\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.moveSkeletonHorz = function (canvas, delta) {\n        var _ref;\n        if ((this.originRect.x1 <= (_ref = this.left + delta.x) && _ref <= this.gridFitX(canvas, this.originRect.x1) + canvas.gridFactor.width)) {\n            delta.x = this.originRect.x1 - this.left;\n            this.left = this.originRect.x1;\n            this.right = this.originRect.x2;\n        } else {\n            delta.x = delta.x - ((this.left + delta.x) - this.gridFitX(canvas, this.left + delta.x));\n            this.left = this.gridFitX(canvas, this.left + delta.x);\n            this.right = this.left + (this.originRect.x2 - this.originRect.x1);\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.moveSkeletonVert = function (canvas, delta) {\n        var _ref;\n        if ((this.originRect.y1 <= (_ref = this.top + delta.y) && _ref <= this.gridFitY(canvas, this.originRect.y1) + canvas.gridFactor.height)) {\n            delta.y = this.originRect.y1 - this.top;\n            this.top = this.originRect.y1;\n            this.bottom = this.originRect.y2;\n        } else {\n            delta.y = delta.y - ((this.top + delta.y) - this.gridFitY(canvas, this.top + delta.y));\n            this.top = this.gridFitY(canvas, this.top + delta.y);\n            this.bottom = this.top + (this.originRect.y2 - this.originRect.y1);\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.restrictNode = function (rect) {\n        if (this.selectedPtType === Toolkit.CT_ELSE) {\n            if (this.left < rect.x1) {\n                this.right = this.right + rect.x1 - this.left;\n                this.left = rect.x1;\n            } else if (this.right > rect.x2) {\n                this.left = this.left + rect.x2 - this.right;\n                this.right = rect.x2;\n            }\n            if (this.top < rect.y1) {\n                this.bottom = this.bottom + rect.y1 - this.top;\n                this.top = rect.y1;\n            } else if (this.bottom > rect.y2) {\n                this.top = this.top + rect.y2 - this.bottom;\n                this.bottom = rect.y2;\n            }\n        } else {\n            if (this.left < rect.x1) {\n                this.left = rect.x1;\n            }\n            if (this.right > rect.x2) {\n                this.right = rect.x2;\n            }\n            if (this.top < rect.y1) {\n                this.top = rect.y1;\n            }\n            if (this.bottom > rect.y2) {\n                this.bottom = rect.y2;\n            }\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.getPermittedRegion = function (editor, canvas, view) {\n        return new Rect(0, 0, editor.diagramWidth, editor.diagramHeight);\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.beginManipulate = function (editor, canvas, view, x, y) {\n        var HHS, SW, middleX, middleY, p, r, z, zr;\n        this.minX = view.minWidth;\n        this.minY = view.minHeight;\n        if (view.autoResize) {\n            this.sizable = Core.SZ_NONE;\n        } else {\n            this.sizable = view.sizable;\n        }\n        this.movable = view.movable;\n\n        // ZoomFactor only applied coordinates.\n        z = new Point(x, y);\n        Coord.coordTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n\n        r = view.getBoundingBox(canvas);\n        this.originRect = r.copy();\n        this.left = r.x1;\n        this.top = r.y1;\n        this.right = r.x2;\n        this.bottom = r.y2;\n\n        if (this.sizable === Core.SZ_RATIO) {\n            this.ratio = (r.x2 - r.x1) / (r.y2 - r.y1);\n        }\n\n        p = new Point(x, y);\n        zr = r.copy();\n        Coord.coordTransform2(canvas.zoomFactor, GridFactor.NO_GRID, zr);\n        middleX = (zr.x1 + zr.x2) / 2;\n        middleY = (zr.y1 + zr.y2) / 2;\n\n        this.selectedPtType = Toolkit.CT_ELSE;\n        /*\n        SW = Toolkit.DEFAULT_SELECTIONLINE_WIDTH;\n        if (this.movable !== Core.MM_NONE && Coord.ptInRect2(p, new Rect(zr.x1 - SW, zr.y1 - SW, zr.x2 + SW, zr.y2 + SW))) {\n            this.selectedPtType = Toolkit.CT_AREA;\n        }\n        */\n        HHS = Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE;\n        switch (this.sizable) {\n        case Core.SZ_NONE:\n            this.selectedPtType = Toolkit.CT_ELSE;\n            break;\n        case Core.SZ_FREE:\n        case Core.SZ_RATIO:\n            if (Coord.equalPt2(new Point(zr.x1 - HHS, zr.y1 - HHS), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_LT;\n            } else if (Coord.equalPt2(new Point(zr.x2 + HHS, zr.y1 - HHS), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_RT;\n            } else if (Coord.equalPt2(new Point(zr.x1 - HHS, zr.y2 + HHS), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_LB;\n            } else if (Coord.equalPt2(new Point(zr.x2 + HHS, zr.y2 + HHS), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_RB;\n            } else if (Coord.equalPt2(new Point(zr.x1 - HHS, middleY), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_LM;\n            } else if (Coord.equalPt2(new Point(zr.x2 + HHS, middleY), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_RM;\n            } else if (Coord.equalPt2(new Point(middleX, zr.y1 - HHS), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_MT;\n            } else if (Coord.equalPt2(new Point(middleX, zr.y2 + HHS), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_MB;\n            }\n            break;\n        case Core.SZ_HORZ:\n            if (Coord.equalPt2(new Point(zr.x1 - HHS, middleY), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_LM;\n            } else if (Coord.equalPt2(new Point(zr.x2 + HHS, middleY), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_RM;\n            }\n            break;\n        case Core.SZ_VERT:\n            if (Coord.equalPt2(new Point(middleX, zr.y1 - HHS), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_MT;\n            } else if (Coord.equalPt2(new Point(middleX, zr.y2 + HHS), p, HHS)) {\n                this.selectedPtType = Toolkit.CT_MB;\n            }\n        }\n        switch (this.selectedPtType) {\n        case Toolkit.CT_AREA:\n            break;\n        case Toolkit.CT_LT:\n        case Toolkit.CT_MT:\n        case Toolkit.CT_LM:\n            this.minRect = new Rect(this.right - this.minX + 1, this.bottom - this.minY + 1, this.right, this.bottom);\n            break;\n        case Toolkit.CT_RB:\n        case Toolkit.CT_MB:\n        case Toolkit.CT_RM:\n            this.minRect = new Rect(this.left, this.top, this.left + this.minX - 1, this.top + this.minY - 1);\n            break;\n        case Toolkit.CT_RT:\n            this.minRect = new Rect(this.left, this.bottom - this.minY + 1, this.left + this.minX - 1, this.bottom);\n            break;\n        case Toolkit.CT_LB:\n            this.minRect = new Rect(this.right - this.minX + 1, this.top, this.right, this.top + this.minY - 1);\n        }\n        switch (this.selectedPtType) {\n        case Toolkit.CT_LT:\n        case Toolkit.CT_RB:\n            editor.setCursor(CURSOR_NWSE_RESIZE);\n            break;\n        case Toolkit.CT_RT:\n        case Toolkit.CT_LB:\n            editor.setCursor(CURSOR_NESW_RESIZE);\n            break;\n        case Toolkit.CT_MT:\n        case Toolkit.CT_MB:\n            editor.setCursor(CURSOR_NS_RESIZE);\n            break;\n        case Toolkit.CT_LM:\n        case Toolkit.CT_RM:\n            editor.setCursor(CURSOR_EW_RESIZE);\n            break;\n        case Toolkit.CT_AREA:\n            editor.setCursor(CURSOR_MOVE);\n            break;\n        case Toolkit.CT_ELSE:\n            editor.setCursor(CURSOR_DEFAULT);\n            break;\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.drawSkeleton = function (editor, canvas) {\n        if (this.movable !== Core.MM_NONE || this.sizable !== Core.SZ_NONE) {\n            return editor.drawRubberband(this.left, this.top, this.right, this.bottom);\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.eraseSkeleton = function (editor, canvas) {\n        if (this.movable !== Core.MM_NONE || this.sizable !== Core.SZ_NONE) {\n            return editor.eraseRubberband(this.left, this.top, this.right, this.bottom);\n        }\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.moveSkeleton = function (editor, canvas, view, delta) {\n        var horzSizable, vertSizable, _ref, _ref1;\n        horzSizable = (_ref = this.sizable) === Core.SZ_HORZ || _ref === Core.SZ_RATIO || _ref === Core.SZ_FREE;\n        vertSizable = (_ref1 = this.sizable) === Core.SZ_VERT || _ref1 === Core.SZ_RATIO || _ref1 === Core.SZ_FREE;\n        switch (this.selectedPtType) {\n        case Toolkit.CT_LT:\n            if (horzSizable) {\n                this.resizeSkeletonLeft(canvas, delta);\n            }\n            if (vertSizable) {\n                this.resizeSkeletonTop(canvas, delta);\n            }\n            break;\n        case Toolkit.CT_RT:\n            if (horzSizable) {\n                this.resizeSkeletonRight(canvas, delta);\n            }\n            if (vertSizable) {\n                this.resizeSkeletonTop(canvas, delta);\n            }\n            break;\n        case Toolkit.CT_LB:\n            if (horzSizable) {\n                this.resizeSkeletonLeft(canvas, delta);\n            }\n            if (vertSizable) {\n                this.resizeSkeletonBottom(canvas, delta);\n            }\n            break;\n        case Toolkit.CT_RB:\n            if (horzSizable) {\n                this.resizeSkeletonRight(canvas, delta);\n            }\n            if (vertSizable) {\n                this.resizeSkeletonBottom(canvas, delta);\n            }\n            break;\n        case Toolkit.CT_LM:\n            if (horzSizable) {\n                this.resizeSkeletonLeft(canvas, delta);\n            }\n            break;\n        case Toolkit.CT_RM:\n            if (horzSizable) {\n                this.resizeSkeletonRight(canvas, delta);\n            }\n            break;\n        case Toolkit.CT_MT:\n            if (vertSizable) {\n                this.resizeSkeletonTop(canvas, delta);\n            }\n            break;\n        case Toolkit.CT_MB:\n            if (vertSizable) {\n                this.resizeSkeletonBottom(canvas, delta);\n            }\n            break;\n        case Toolkit.CT_AREA:\n        case Toolkit.CT_ELSE:\n            switch (this.movable) {\n            case Core.MM_HORZ:\n                this.moveSkeletonHorz(canvas, delta);\n                break;\n            case Core.MM_VERT:\n                this.moveSkeletonVert(canvas, delta);\n                break;\n            case Core.MM_FREE:\n                this.moveSkeletonHorz(canvas, delta);\n                this.moveSkeletonVert(canvas, delta);\n            }\n        }\n        return this.restrictNode(this.getPermittedRegion(editor, canvas, view));\n    };\n\n    /**\n     *\n     */\n    NodeManipulator.prototype.endManipulate = function (editor, canvas, view, dx, dy) {\n        if (this.movable !== Core.MM_NONE || this.sizable !== Core.SZ_NONE) {\n            this.restrictNode(this.getPermittedRegion(editor, canvas, view));\n            if (this.selectedPtType === Toolkit.CT_ELSE || this.selectedPtType === Toolkit.CT_AREA) {\n                var r = view.getBoundingBox(canvas);\n                dx = this.left - r.x1;\n                dy = this.top - r.y1;\n                var c = editor.containmentHandlingProxy.containerView;\n                if (view.containerChangeable && view.containerView !== c) {\n                    switch (this.movable) {\n                    case Core.MM_HORZ:\n                        if (dx !== 0) {\n                            editor.changeViewContainer(view, dx, 0, c);\n                        }\n                        break;\n                    case Core.MM_VERT:\n                        if (dy !== 0) {\n                            editor.changeViewContainer(view, 0, dy, c);\n                        }\n                        break;\n                    case Core.MM_FREE:\n                        if (dx !== 0 || dy !== 0) {\n                            editor.changeViewContainer(view, dx, dy, c);\n                        }\n                    }\n                } else {\n                    switch (this.movable) {\n                    case Core.MM_HORZ:\n                        if (dx !== 0) {\n                            editor.moveView(view, dx, 0, c);\n                        }\n                        break;\n                    case Core.MM_VERT:\n                        if (dy !== 0) {\n                            editor.moveView(view, 0, dy, c);\n                        }\n                        break;\n                    case Core.MM_FREE:\n                        if (dx !== 0 || dy !== 0) {\n                            editor.moveView(view, dx, dy, c);\n                        }\n                    }\n                }\n            } else {\n                editor.resizeNode(view, this.left, this.top, this.right, this.bottom);\n            }\n        }\n        editor.setCursor(CURSOR_DEFAULT);\n    };\n\n\n    /**\n     * @constructor\n     */\n    function EdgeManipulator() {\n        Manipulator.apply(this, arguments);\n        this.points = new Points();\n        this.originPoints = new Points();\n        this.style = 0;\n        this.selectedIndex = 0;\n        this.edgeSelectLocation = 0;\n    }\n\n    // EdgeManipulator inherits Manipulator\n    EdgeManipulator.prototype = Object.create(Manipulator.prototype);\n    EdgeManipulator.prototype.constructor = EdgeManipulator;\n\n    /** EdgeManipulator Constants */\n    EdgeManipulator.SL_TAIL = 0;\n    EdgeManipulator.SL_HEAD = 1;\n    EdgeManipulator.SL_LINE = 2;\n\n    /**\n     * @override\n     */\n    EdgeManipulator.prototype.beginManipulate = function (editor, canvas, view, x, y) {\n        var l, z;\n        z = new Point(x, y);\n        Coord.coordRevTransform(canvas.zoomFactor, GridFactor.NO_GRID, z);\n        l = view;\n        this.points.assign(l.points);\n        this.style = l.lineStyle;\n        if (this.style === Core.LS_RECTILINEAR || this.style === Core.LS_ROUNDRECT) {\n            this.selectedIndex = l.selectedPoint(canvas, z);\n            if (this.selectedIndex === 0) {\n                this.edgeSelectLocation = EdgeManipulator.SL_TAIL;\n            } else if (this.selectedIndex === (this.points.count() - 1)) {\n                this.edgeSelectLocation = EdgeManipulator.SL_HEAD;\n            } else {\n                this.edgeSelectLocation = EdgeManipulator.SL_LINE;\n                if (this.selectedIndex === -1) {\n                    this.selectedIndex = l.containedIndex(canvas, z);\n                }\n            }\n        } else {\n            this.selectedIndex = l.selectedPoint(canvas, z);\n            if (this.selectedIndex !== 0) {\n                this.points.setPoint(0, Coord.getCenter(l.tail.getBoundingBox(canvas)));\n            }\n            if (this.selectedIndex !== (this.points.count() - 1)) {\n                this.points.setPoint(this.points.count() - 1, Coord.getCenter(l.head.getBoundingBox(canvas)));\n            }\n            if (this.selectedIndex === -1) {\n                this.selectedIndex = l.containedIndex(canvas, z);\n                this.selectedIndex++;\n                this.points.insert(this.selectedIndex, z);\n            }\n            if (this.selectedIndex === 0) {\n                this.edgeSelectLocation = EdgeManipulator.SL_TAIL;\n            } else if (this.selectedIndex === (this.points.count() - 1)) {\n                this.edgeSelectLocation = EdgeManipulator.SL_HEAD;\n            } else {\n                this.edgeSelectLocation = EdgeManipulator.SL_LINE;\n            }\n            if (this.edgeSelectLocation === EdgeManipulator.SL_LINE) {\n                editor.setCursor(CURSOR_DEFAULT);\n            } else {\n                editor.setCursor(CURSOR_HAND);\n            }\n        }\n        this.originPoints.assign(this.points);\n    };\n\n    /**\n     * @override\n     */\n    EdgeManipulator.prototype.endManipulate = function (editor, canvas, view, dx, dy) {\n        if ((dx !== 0) || (dy !== 0)) {\n            if (this.style === Core.LS_RECTILINEAR || this.style === Core.LS_ROUNDRECT) {\n                this.points.reduceOrthoLine();\n            } else {\n                this.points.reduceLine();\n            }\n            if (this.edgeSelectLocation === EdgeManipulator.SL_LINE) {\n                editor.modifyEdge(view, this.points);\n            } else {\n                // edgeSelectLocation == SL_HEAD or SL_TAIL\n                var v = editor.diagram.getViewAt(canvas, this.points.getPoint(this.selectedIndex).x, this.points.getPoint(this.selectedIndex).y, true);\n                var oldPart = null;\n                if (this.edgeSelectLocation === EdgeManipulator.SL_TAIL) {\n                    oldPart = view.tail;\n                } else { // SL_HEAD\n                    oldPart = view.head;\n                }\n                if (v !== null) {\n                    if (v === oldPart) {\n                        editor.modifyEdge(view, this.points);\n                    } else {\n                        editor.reconnectEdge(view, this.points, v, this.edgeSelectLocation === EdgeManipulator.SL_TAIL);\n                    }\n                }\n            }\n        }\n        return editor.setCursor(CURSOR_DEFAULT);\n    };\n\n    /**\n     * Fit x coordinate to grid\n     * @param {Canvas} canvas\n     * @param {number} y\n     */\n    EdgeManipulator.prototype.gridFitX = function (canvas, x) {\n        return x - (x % canvas.gridFactor.width);\n    };\n\n    /**\n     * Fit y coordinate to grid\n     * @param {Canvas} canvas\n     * @param {number} y\n     */\n    EdgeManipulator.prototype.gridFitY = function (canvas, y) {\n        return y - (y % canvas.gridFactor.height);\n    };\n\n    /**\n     * Put point bounds on canvas\n     * @param {Editor} editor\n     * @param {Point} p\n     */\n    EdgeManipulator.prototype.putPointBoundsOnCanvas = function (editor, p) {\n        if (p.x < 0) {\n            p.x = 0;\n        } else if (p.x > editor.diagramWidth) {\n            p.x = editor.diagramWidth;\n        }\n        if (p.y < 0) {\n            p.y = 0;\n        } else if (p.y > editor.diagramHeight) {\n            p.y = editor.diagramHeight;\n        }\n    };\n\n    /**\n     * @param {Editor} editor\n     * @param {Canvas} canvas\n     * @param {View} view\n     * @param {number} delta\n     * @override\n     */\n    EdgeManipulator.prototype.moveSkeleton = function (editor, canvas, view, delta) {\n        var op, op1, op2, p, p1, p2, _ref, _ref1, _ref2, _ref3, _ref4, _ref5;\n        if (this.style === Core.LS_RECTILINEAR || this.style === Core.LS_ROUNDRECT) {\n            // Get points at end of the selected line\n            p1 = this.points.getPoint(this.selectedIndex).copy();\n            if (this.edgeSelectLocation === EdgeManipulator.SL_HEAD) {\n                p2 = this.points.getPoint(this.selectedIndex - 1).copy();\n            } else {\n                p2 = this.points.getPoint(this.selectedIndex + 1).copy();\n            }\n            op1 = this.originPoints.getPoint(this.selectedIndex).copy();\n            if (this.edgeSelectLocation === EdgeManipulator.SL_HEAD) {\n                op2 = this.originPoints.getPoint(this.selectedIndex - 1).copy();\n            } else {\n                op2 = this.originPoints.getPoint(this.selectedIndex + 1).copy();\n            }\n\n            // if move front-end or rear-end point\n            if (this.edgeSelectLocation === EdgeManipulator.SL_TAIL || this.edgeSelectLocation === EdgeManipulator.SL_HEAD) {\n                if ((p1.x + delta.x >= op1.x) && (p1.x + delta.x <= this.gridFitX(canvas, op1.x) + canvas.gridFactor.width)) {\n                    delta.x = op1.x - p1.x;\n                    p1.x = op1.x;\n                } else {\n                    delta.x = delta.x - ((p1.x + delta.x) - this.gridFitX(canvas, p1.x + delta.x));\n                    p1.x = this.gridFitX(canvas, p1.x + delta.x);\n                }\n                if ((p1.y + delta.y >= op1.y) && (p1.y + delta.y <= this.gridFitY(canvas, op1.y) + canvas.gridFactor.height)) {\n                    delta.y = op1.y - p1.y;\n                    p1.y = op1.y;\n                } else {\n                    delta.y = delta.y - ((p1.y + delta.y) - this.gridFitY(canvas, p1.y + delta.y));\n                    p1.y = this.gridFitY(canvas, p1.y + delta.y);\n                }\n                if (op1.x === op2.x) {\n                    p2.x = p1.x;\n                }\n                if (op1.y === op2.y) {\n                    p2.y = p1.y;\n                }\n\n            // if move line\n            } else {\n\n                // if vertical line selected, move skeleton\n                if (p1.x === p2.x) {\n                    if ((op1.x <= (_ref2 = p1.x + delta.x) && _ref2 <= this.gridFitX(canvas, op1.x) + canvas.gridFactor.width)) {\n                        delta.x = op1.x - p1.x;\n                        p1.x = op1.x;\n                    } else {\n                        delta.x = delta.x - ((p1.x + delta.x) - this.gridFitX(canvas, p1.x + delta.x));\n                        p1.x = this.gridFitX(canvas, p1.x + delta.x);\n                    }\n                    p2.x = p1.x;\n\n                // if horizontal line selected, move skeleton\n                } else if (p1.y === p2.y) {\n                    if ((op1.y <= (_ref3 = p1.y + delta.y) && _ref3 <= this.gridFitY(canvas, op1.y) + canvas.gridFactor.height)) {\n                        delta.y = op1.y - p1.y;\n                        p1.y = op1.y;\n                    } else {\n                        delta.y = delta.y - ((p1.y + delta.y) - this.gridFitY(canvas, p1.y + delta.y));\n                        p1.y = this.gridFitY(canvas, p1.y + delta.y);\n                    }\n                    p2.y = p1.y;\n                }\n            }\n\n            if ((this.edgeSelectLocation === EdgeManipulator.SL_LINE && this.selectedIndex === 0) || (this.edgeSelectLocation === EdgeManipulator.SL_HEAD && this.points.count() === 2)) {\n                this.points.insert(0, this.points.getPoint(0).copy());\n                this.originPoints.insert(0, this.points.getPoint(0).copy());\n                this.selectedIndex++;\n            } else if ((this.edgeSelectLocation === EdgeManipulator.SL_LINE && this.selectedIndex === (this.points.count() - 2)) || (this.edgeSelectLocation === EdgeManipulator.SL_TAIL && this.points.count() === 2)) {\n                this.points.insert(this.selectedIndex + 1, this.points.getPoint(this.selectedIndex + 1).copy());\n                this.originPoints.insert(this.selectedIndex + 1, this.points.getPoint(this.selectedIndex + 1).copy());\n            }\n\n            // Modify points not to stray from Canvas\n            this.putPointBoundsOnCanvas(editor, p1);\n            this.putPointBoundsOnCanvas(editor, p2);\n\n            // Re-assign selected points to modified.\n            this.points.getPoint(this.selectedIndex).setPoint2(p1);\n            if (this.edgeSelectLocation === EdgeManipulator.SL_HEAD) {\n                this.points.getPoint(this.selectedIndex - 1).setPoint2(p2);\n            } else {\n                this.points.getPoint(this.selectedIndex + 1).setPoint2(p2);\n            }\n\n        } else { // LS_OBLIQUE\n            // Get selected point\n            p = this.points.getPoint(this.selectedIndex).copy();\n            op = this.originPoints.getPoint(this.selectedIndex).copy();\n\n            // Move skeleton\n            if ((op.x <= (_ref4 = p.x + delta.x) && _ref4 <= this.gridFitX(canvas, op.x) + canvas.gridFactor.width)) {\n                delta.x = op.x - p.x;\n                p.x = op.x;\n            } else {\n                delta.x = delta.x - ((p.x + delta.x) - this.gridFitX(canvas, p.x + delta.x));\n                p.x = this.gridFitX(canvas, p.x + delta.x);\n            }\n            if ((op.y <= (_ref5 = p.y + delta.y) && _ref5 <= this.gridFitY(canvas, op.y) + canvas.gridFactor.height)) {\n                delta.y = op.y - p.y;\n                p.y = op.y;\n            } else {\n                delta.y = delta.y - ((p.y + delta.y) - this.gridFitY(canvas, p.y + delta.y));\n                p.y = this.gridFitY(canvas, p.y + delta.y);\n            }\n\n            // Modify point not to stray from Canvas\n            this.putPointBoundsOnCanvas(editor, p);\n\n            // Re-assign selected point to modified.\n            this.points.setPoint(this.selectedIndex, p);\n        }\n    };\n\n    /**\n     * @override\n     */\n    EdgeManipulator.prototype.drawSkeleton = function (editor, canvas) {\n        return editor.drawRubberlines(this.points);\n    };\n\n    /**\n     * @override\n     */\n    EdgeManipulator.prototype.eraseSkeleton = function (editor, canvas) {\n        return editor.eraseRubberlines(this.points);\n    };\n\n\n\n    /**\n     * @constructor\n     */\n    function ParasiticManipulator() {\n        NodeManipulator.apply(this, arguments);\n    }\n\n    // ParasiticManipulator inhertis NodeManipulator\n    ParasiticManipulator.prototype = Object.create(NodeManipulator.prototype);\n    ParasiticManipulator.prototype.constructor = ParasiticManipulator;\n\n    /**\n     * @override\n     */\n    ParasiticManipulator.prototype.endManipulate = function (editor, canvas, view, dx, dy) {\n        var ad, edge, midPointIndex, node, p1, p2;\n        if (dx === 0 && dy === 0) {\n            return;\n        }\n        if (this.selectedPtType === Toolkit.CT_ELSE) {\n            if (view instanceof NodeParasiticView) {\n                node = view.hostNode;\n                if (node === null) {\n                    node = view._parent;\n                }\n                p1 = new Point(Math.floor((node.left + node.getRight()) / 2), Math.floor((node.top + node.getBottom()) / 2));\n                p2 = p1.copy();\n            } else if (view instanceof EdgeParasiticView) {\n                edge = view.hostEdge;\n                if (edge === null) {\n                    edge = view._parent;\n                }\n                switch (view.edgePosition) {\n                case Core.EP_HEAD:\n                    p1 = edge.points.getPoint(edge.points.count() - 1).copy();\n                    p2 = edge.points.getPoint(edge.points.count() - 2).copy();\n                    break;\n                case Core.EP_TAIL:\n                    p1 = edge.points.getPoint(0).copy();\n                    p2 = edge.points.getPoint(1).copy();\n                    break;\n                case Core.EP_MIDDLE:\n                    midPointIndex = Math.floor(edge.points.count() / 2);\n                    if ((edge.points.count() % 2) === 0) {\n                        midPointIndex--;\n                    }\n                    p1 = edge.points.getPoint(midPointIndex).copy();\n                    p2 = edge.points.getPoint(midPointIndex + 1).copy();\n                    if ((edge.points.count() % 2) === 0) {\n                        p1.x = Math.floor((p1.x + p2.x) / 2);\n                        p1.y = Math.floor((p1.y + p2.y) / 2);\n                    }\n                }\n            }\n            var pr = new Rect(p1.x, p1.y, p2.x, p2.y),\n                pp = new Point(Math.floor((this.left + this.right) / 2), Math.floor((this.top + this.bottom) / 2));\n            ad = Coord.getPolar(pr, pp);\n            if (editor !== null) {\n                editor.moveParasiticView(view, ad.alpha, ad.distance);\n            }\n        } else {\n            NodeManipulator.prototype.endManipulate.call(this, editor, canvas, view, dx, dy);\n        }\n    };\n\n    /**\n     * 다이어그램 편집기\n     *\n     * This class dispatches theses events:\n     *     - selectionChanged:      (views)\n     *     - mouseDown:\n     *     - viewDoubleClicked:     (view, x, y)\n     *     - viewMoved:             (views, dx, dy)\n     *     - parasiticViewMoved:    (view, alpha, distance)\n     *     - containerViewChanged:  (views, dx, dy, containerView)\n     *     - nodeResized:           (node, left, top, right, bottom)\n     *     - edgeModified:          (edge, points)\n     *     - edgeReconnected:       (edge, points, newParticipant, isTailSide)\n     *\n     * @constructor\n     */\n    function Editor(config) {\n        var self = this;\n        this.config = config;\n        this.canvasId = config.canvasId;\n        this.canvasElement = document.getElementById(this.canvasId);\n        this.canvas = new Canvas();\n        this.canvas.context = this.canvasElement.getContext(\"2d\");\n        this.canvas.gridFactor = new GridFactor(4.0, 4.0);\n        this.canvas.zoomFactor = new ZoomFactor(1.0, 1.0);\n        this.canvasElement.width = DEFAULT_CANVAS_WIDTH;\n        this.canvasElement.height = DEFAULT_CANVAS_HEIGHT;\n        this.showGrid = true;\n        this.backgroundColor = Color.WHITE;\n        this.containmentHandlingProxy = new ContainmentHandlingProxy();\n        this.diagram = null;\n        this.backupViews = [];\n        this.backup = null;\n        this.backupX = 0;\n        this.backupY = 0;\n        // Setup for High-DPI (Retina) Display\n        if (window.devicePixelRatio) {\n            var w = this.canvasElement.width,\n                h = this.canvasElement.height;\n            this.canvas.ratio = window.devicePixelRatio;\n            this.canvasElement.width = w * this.canvas.ratio;\n            this.canvasElement.height = h * this.canvas.ratio;\n            this.canvasElement.style.width = w + \"px\";\n            this.canvasElement.style.height = h + \"px\";\n        }\n\n        // To check mouse left button down in mouse move event.\n        this.leftButtonDown = false;\n        this.tweakMouseMoveEvent = function (e) {\n            // Check from jQuery UI for IE versions < 9\n            // if ($.browser.msie && !e.button && !(document.documentMode >= 9)) {\n            //     leftButtonDown = false;\n            // }\n            // If left button is not set, set which to 0\n            // This indicates no buttons pressed\n            if(e.which === 1 && !self.leftButtonDown) e.which = 0;\n        };\n\n        this.canvasElement.onmouseup = function (e) {\n            if (e.which === 1) self.leftButtonDown = false;\n            var p = Editor.getMousePos(self.canvasElement, e);\n            var evt = new MouseEvent(p.x, p.y, e.button, 1, e.shiftKey, e.altKey, e.ctrlKey);\n            if (self.activeHandler !== null) {\n                self.activeHandler.mouseUp(self, self.canvas, evt);\n            }\n            self.mouseDown(evt);\n        };\n        this.canvasElement.onmousedown = function (e) {\n            if (e.which === 1) self.leftButtonDown = true;\n            var p = Editor.getMousePos(self.canvasElement, e);\n            var evt = new MouseEvent(p.x, p.y, e.button, 1, e.shiftKey, e.altKey, e.ctrlKey);\n            if (self.activeHandler !== null) {\n                self.activeHandler.mouseDown(self, self.canvas, evt);\n            }\n        };\n        this.canvasElement.onmousemove = function (e) {\n            self.tweakMouseMoveEvent(e);\n            var p = Editor.getMousePos(self.canvasElement, e);\n            var evt = new MouseEvent(p.x, p.y, e.button, 1, e.shiftKey, e.altKey, e.ctrlKey);\n            evt.leftButtonDown = self.leftButtonDown;\n            if (self.activeHandler) {\n                self.activeHandler.mouseMove(self, self.canvas, evt);\n            }\n        };\n        this.canvasElement.ondblclick = function (e) {\n            var p = Editor.getMousePos(self.canvasElement, e);\n            Coord.coordRevTransform(self.canvas.zoomFactor, GridFactor.NO_GRID, p);\n            if (self.diagram) {\n                var view = self.diagram.getViewAt(self.canvas, p.x, p.y);\n                if (view) {\n                    self.viewDoubleClicked(view, p.x, p.y);\n                }\n            }\n        };\n    }\n\n    /**\n     *\n     */\n    Editor.getMousePos = function (ce, e) {\n        var cr = ce.getBoundingClientRect();\n        var p = new Point(e.clientX - cr.left, e.clientY - cr.top);\n        return p;\n    };\n\n    Editor.prototype.setEnabled = function (enabled) {\n        this.canvasElement.style.display = (enabled ? \"\" : \"none\");\n    };\n\n    Editor.prototype.getEnabled = function () {\n        return (this.canvasElement.style.display !== \"none\");\n    };\n\n    /**\n     * 현재 편집중인 다이어그램 리턴\n     *\n     * @return {Diagram}\n     */\n    Editor.prototype.getDiagram = function () {\n        return this.diagram;\n    };\n\n    /**\n     * 다이어그램 편집 영역의 너비를 설정\n     *\n     * @param {number} width\n     */\n    Editor.prototype.setDiagramWidth = function (width) {\n        this.canvasElement.width = width;\n    };\n\n    /**\n     * 다이어그램 편집 영역의 너비를 리턴\n     *\n     * @return {number}\n     */\n    Editor.prototype.getDiagramWidth = function () {\n        return this.canvasElement.width;\n    };\n\n    /**\n     * 다이어그램 편집 영역의 높이를 설정\n     *\n     * @param {number} height\n     */\n    Editor.prototype.setDiagramHeight = function (height) {\n        this.canvasElement.height = height;\n    };\n\n    /**\n     * 다이어그램 편집 영역의 높이를 리턴\n     *\n     * @return {number}\n     */\n    Editor.prototype.getDiagramHeight = function () {\n        return this.canvasElement.height;\n    };\n\n    /**\n     * 줌 팩터를 리턴\n     *\n     * @return {ZoomFactor}\n     */\n    Editor.prototype.getZoomFactor = function () {\n        return this.canvas.zoomFactor;\n    };\n\n    /**\n     * 그리드 팩터를 리턴\n     *\n     * @return {GridFactor}\n     */\n    Editor.prototype.getGridFactor = function () {\n        return this.canvas.gridFactor;\n    };\n\n    Editor.prototype.clearBackupViews = function () {\n        this.backupViews = [];\n    };\n\n    Editor.prototype.addBackupView = function (view) {\n        if (!_.contains(this.backupViews, view)) {\n            this.backupViews.push(view);\n        }\n    };\n\n    Editor.prototype.drawBackground = function (g) {\n        g.clearRect(0, 0, this.canvasElement.width, this.canvasElement.height);\n        if (this.showGrid) {\n            var w  = this.canvas.gridFactor.width * 2 * this.canvas.ratio,\n                h  = this.canvas.gridFactor.height * 2 * this.canvas.ratio,\n                wc = Math.floor(this.getDiagramWidth() / w),\n                hc = Math.floor(this.getDiagramHeight() / h),\n                i;\n            g.beginPath();\n            g.strokeStyle = \"#f5f5f5\";\n            for (i = 0; i < wc; i++) {\n                g.moveTo(i * w + 0.5, 0);\n                g.lineTo(i * w + 0.5, this.canvasElement.height);\n            }\n            for (i = 0; i < hc; i++) {\n                g.moveTo(0, i * h + 0.5);\n                g.lineTo(this.canvasElement.width, i * h + 0.5);\n            }\n            g.stroke();\n        }\n    };\n\n    Editor.prototype.drawRubberband = function (x1, y1, x2, y2, kind, showEnds) {\n        if (kind === null) {\n            kind = \"rect\";\n        }\n        if (showEnds === null) {\n            showEnds = false;\n        }\n        var rect = new Rect(x1, y1, x2, y2);\n        Coord.normalizeRect(rect);\n\n        // Backup area should include backup views.\n        for (var i = 0, len = this.backupViews.length; i < len; i++) {\n            var v = this.backupViews[i];\n            var vr = v.getBoundingBox();\n            Coord.normalizeRect(vr);\n            rect.union(vr);\n        }\n\n        rect.expand(20);\n        Coord.coordTransform2(this.canvas.zoomFactor, GridFactor.NO_GRID, rect);\n\n        // for Retina\n        rect.x1 = rect.x1 * this.canvas.ratio;\n        rect.y1 = rect.y1 * this.canvas.ratio;\n        rect.x2 = rect.x2 * this.canvas.ratio;\n        rect.y2 = rect.y2 * this.canvas.ratio;\n\n        this.backupX = rect.x1;\n        this.backupY = rect.y1;\n        this.backup = this.canvas.context.getImageData(rect.x1, rect.y1, rect.getWidth(), rect.getHeight());\n        Toolkit.drawRange(this.canvas, x1, y1, x2, y2, kind, showEnds);\n    };\n\n    Editor.prototype.eraseRubberband = function (x1, y1, x2, y2, kind, showEnds) {\n        if (kind === null) {\n            kind = \"rect\";\n        }\n        if (showEnds === null) {\n            showEnds = false;\n        }\n        if (this.backup !== null) {\n            this.canvas.context.putImageData(this.backup, this.backupX, this.backupY);\n        }\n    };\n\n    Editor.prototype.drawContainingBox = function (view) {\n        view.drawContainingBox(this.canvas);\n    };\n\n    Editor.prototype.eraseContainingBox = function (view) {\n        view.eraseContainingBox(this.canvas);\n    };\n\n    Editor.prototype.drawRubberlines = function (points) {\n        var rect = points.getBoundingRect();\n        Coord.normalizeRect(rect);\n        rect.expand(20);\n        Coord.coordTransform2(this.canvas.zoomFactor, GridFactor.NO_GRID, rect);\n\n        // for Retina\n        rect.x1 = rect.x1 * this.canvas.ratio;\n        rect.y1 = rect.y1 * this.canvas.ratio;\n        rect.x2 = rect.x2 * this.canvas.ratio;\n        rect.y2 = rect.y2 * this.canvas.ratio;\n\n        this.backupX = rect.x1;\n        this.backupY = rect.y1;\n        this.backup = this.canvas.context.getImageData(rect.x1, rect.y1, rect.getWidth(), rect.getHeight());\n        Toolkit.drawDottedLine(this.canvas, points);\n    };\n\n    Editor.prototype.eraseRubberlines = function (points) {\n        if (this.backup !== null) {\n            this.canvas.context.putImageData(this.backup, this.backupX, this.backupY);\n        }\n    };\n\n    Editor.prototype.browseDiagram = function (diagram) {\n        this.diagram = diagram;\n        this.repaint();\n    };\n\n    Editor.prototype.repaint = function () {\n        if (this.diagram) {\n            this.drawBackground(this.canvas.context);\n            this.diagram.drawDiagram(this.canvas);\n        }\n    };\n\n    /**\n     *\n     * @param {string} cursor\n     */\n    Editor.prototype.setCursor = function (cursor) {\n        document.body.style.cursor = cursor;\n    };\n\n    Editor.prototype.selectView = function (view) {\n        if (this.diagram) {\n            this.diagram.deselectAll();\n            this.diagram.selectView(view);\n            this.repaint();\n            $(this).triggerHandler('selectionChanged', [this.diagram.selectedViews]);\n        }\n    };\n\n    Editor.prototype.deselectView = function (view) {\n        if (this.diagram) {\n            this.diagram.deselectView(view);\n            this.repaint();\n            $(this).triggerHandler('selectionChanged', [this.diagram.selectedViews]);\n        }\n    };\n\n    Editor.prototype.selectAdditionalView = function (view) {\n        if (this.diagram) {\n            this.diagram.selectView(view);\n            this.repaint();\n            $(this).triggerHandler('selectionChanged', [this.diagram.selectedViews]);\n        }\n    };\n\n    Editor.prototype.selectArea = function (x1, y1, x2, y2) {\n        if (this.diagram) {\n            this.diagram.deselectAll();\n            this.diagram.selectArea(this.canvas, x1, y1, x2, y2);\n            this.repaint();\n            $(this).triggerHandler('selectionChanged', [this.diagram.selectedViews]);\n        }\n    };\n\n    Editor.prototype.selectAll = function () {\n        if (this.diagram) {\n            this.diagram.selectAll();\n            this.repaint();\n            $(this).triggerHandler('selectionChanged', [this.diagram.selectedViews]);\n        }\n    };\n\n    Editor.prototype.deselectAll = function () {\n        if (this.diagram) {\n            this.diagram.deselectAll();\n            this.repaint();\n        }\n    };\n\n    /**\n     * @param {Point} point (x, y) of screen.\n     * @return {Point} - point on the canvas\n     */\n    Editor.prototype.convertPosition = function (point) {\n        var $canvas = $(\"#\" + this.canvasId);\n        var offset = $canvas.offset();\n        var p = new Point(point.x - offset.left, point.y - offset.top);\n        Coord.coordRevTransform(this.canvas.zoomFactor, GridFactor.NO_GRID, p);\n        return p;\n    };\n\n    Editor.prototype.mouseDown = function (event) {\n        $(this).triggerHandler('mouseDown', [event]);\n    };\n\n    Editor.prototype.viewDoubleClicked = function (view, x, y) {\n        $(this).triggerHandler('viewDoubleClicked', [view, x, y]);\n    };\n\n    Editor.prototype.moveView = function (view, dx, dy) {\n        $(this).triggerHandler('viewMoved',[[view], dx, dy]);\n    };\n\n    Editor.prototype.moveSelectedViews = function (dx, dy) {\n        $(this).triggerHandler('viewMoved', [this.diagram.selectedViews, dx, dy]);\n    };\n\n    Editor.prototype.moveParasiticView = function (view, alpha, distance) {\n        $(this).triggerHandler('parasiticViewMoved', [view, alpha, distance]);\n    };\n\n    Editor.prototype.changeViewContainer = function (view, dx, dy, containerView) {\n        $(this).triggerHandler('containerViewChanged', [[view], dx, dy, containerView]);\n    };\n\n    Editor.prototype.changeSelectedViewsContainer = function (dx, dy, containerView) {\n        $(this).triggerHandler('containerViewChanged', [this.diagram.selectedViews, dx, dy, containerView]);\n    };\n\n    Editor.prototype.resizeNode = function (node, left, top, right, bottom) {\n        $(this).triggerHandler('nodeResized', [node, left, top, right, bottom]);\n    };\n\n    Editor.prototype.modifyEdge = function (edge, points) {\n        $(this).triggerHandler('edgeModified', [edge, points]);\n    };\n\n    Editor.prototype.reconnectEdge = function (edge, points, newParticipant, isTailSide) {\n        $(this).triggerHandler('edgeReconnected', [edge, points, newParticipant, isTailSide]);\n    };\n\n    // Define Public API\n    exports.Handler                  = Handler;\n    exports.ManipulatorBinder        = ManipulatorBinder;\n    exports.ManipulatableNotifier    = ManipulatableNotifier;\n    exports.ContainmentHandlingProxy = ContainmentHandlingProxy;\n    exports.SelectHandler            = SelectHandler;\n    exports.CreateHandler            = CreateHandler;\n    exports.Manipulator              = Manipulator;\n    exports.NodeManipulator          = NodeManipulator;\n    exports.EdgeManipulator          = EdgeManipulator;\n    exports.ParasiticManipulator     = ParasiticManipulator;\n    exports.Editor                   = Editor;\n\n});\n\n\n//# sourceURL=/diagrams/Editor.js"),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, meta, appshell, kendo, Mustache, type */\n\n/**\n *\n * This module dispatches these events:\n *     - workingDiagramAdd:     (diagram)\n *     - workingDiagramRemove:  (diagram)\n *     - workingDiagramUpdate:  (diagram)\n *     - currentDiagramChanged: (editor)\n *     - repaint:               (editor)\n *     - mouseDown:             (mouseEvent)\n *     - selectionChanged:      (views)\n *     - viewDoubleClicked:     (view, x, y)\n *     - viewMoved:             (views, dx, dy)\n *     - parasiticViewMoved:    (view, alpha, distance)\n *     - containerViewChanged:  (views, dx, dy, containerView)\n *     - nodeResized:           (node, left, top, right, bottom)\n *     - edgeModified:          (edge, points)\n *     - edgeReconnected:       (edge, points, newParticipant, isTailSide)\n */\n\ndefine('diagrams/DiagramManager',['require','exports','module','core/Global','utils/AppInit','core/Repository','engine/ProjectManager','engine/SelectionManager','command/Commands','command/CommandManager','menu/ContextMenuManager','utils/KeyEvent','core/PreferenceManager','preference/DefaultPreferences','core/Graphics','core/Core','core/Core','diagrams/Editor'],function (require, exports, module) {\n    \"use strict\";\n\n    var global             = require(\"core/Global\").global,\n        AppInit            = require(\"utils/AppInit\"),\n        Repository         = require(\"core/Repository\"),\n        ProjectManager     = require(\"engine/ProjectManager\"),\n        SelectionManager   = require(\"engine/SelectionManager\"),\n        Commands           = require(\"command/Commands\"),\n        CommandManager     = require(\"command/CommandManager\"),\n        ContextMenuManager = require(\"menu/ContextMenuManager\"),\n        KeyEvent           = require(\"utils/KeyEvent\"),\n        PreferenceManager  = require(\"core/PreferenceManager\"),\n        DefaultPreferences = require(\"preference/DefaultPreferences\"),\n        Graphics           = require(\"core/Graphics\"),\n        Project            = require(\"core/Core\").Project,\n        Diagram            = require(\"core/Core\").Diagram,\n        Editor             = require(\"diagrams/Editor\").Editor;\n\n    /**\n     * Working diagrams.\n     *\n     * @private\n     * @type {Array.<Diagram>}\n     */\n    var _workingDiagrams = [];\n\n    /**\n     * Diagram editor.\n     *\n     * @private\n     * @type {Editor}\n     */\n    var diagramEditor = null;\n\n    /**\n     * Active handler\n     *\n     * @private\n     * @type {Handler}\n     */\n    var activeHandler = null;\n\n\n    var _doRepaint = true;\n\n\n    var hiddenEditor = null;\n\n    var $diagramArea;\n\n    /**\n     * Open diagram and add to working set\n     *\n     * @param {Diagram} diagram\n     */\n    function openDiagram(diagram) {\n        if (diagram) {\n            diagramEditor.diagram = diagram;\n            if (!_.contains(_workingDiagrams, diagram)) {\n                _workingDiagrams.push(diagram);\n                $(exports).triggerHandler(\"workingDiagramAdd\", [diagram]);\n                _currentDiagramChangedEvent(diagramEditor.diagram);\n            }\n        }\n    }\n\n    /**\n     * Update the diagram.\n     *\n     * @param {Diagram} diagram\n     */\n    function updateDiagram(diagram) {\n        if (diagram) {\n            $(exports).triggerHandler(\"workingDiagramUpdate\", [diagram]);\n        }\n    }\n\n    /**\n     * Close the diagram.\n     *\n     * @param {Diagram} diagram\n     */\n    function closeDiagram(diagram) {\n        if (diagram) {\n            var index = _.indexOf(_workingDiagrams, diagram);\n            if (index >= 0) {\n                // Close the diagram\n                _workingDiagrams.remove(diagram);\n                $(exports).triggerHandler(\"workingDiagramRemove\", [diagram]);\n\n                // if current diagram is closing\n                if (getCurrentDiagram() === diagram) {\n                    if (_workingDiagrams.length === 0) {\n                        setCurrentDiagram(null);\n                    } else if (_workingDiagrams[index]) {\n                        setCurrentDiagram(_workingDiagrams[index]);\n                    } else {\n                        setCurrentDiagram(_workingDiagrams[0]);\n                    }\n                }\n            }\n        }\n    }\n\n    /**\n     * Return all working diagrams.\n     *\n     * @return {Array.<Diagram>}\n     */\n    function getWorkingDiagrams() {\n        return _workingDiagrams;\n    }\n\n    /**\n     * Save a list of working diagrams of current project.\n     */\n    function saveWorkingDiagrams() {\n        var fullPath = ProjectManager.getFilename();\n        if (fullPath) {\n            var current  = getCurrentDiagram() ? getCurrentDiagram()._id : null,\n                ids      = _.map(_workingDiagrams, function (d) { return d._id; }),\n                key      = \"_workingDiagrams.\" + fullPath,\n                value    = { current: current, workingSet: ids };\n            PreferenceManager.set(key, value);\n        }\n    }\n\n    /**\n     * Restore a list of working diagrams and current diagram of current project.\n     */\n    function restoreWorkingDiagrams() {\n        var fullPath = ProjectManager.getFilename();\n        if (fullPath) {\n            var key   = \"_workingDiagrams.\" + fullPath,\n                value = PreferenceManager.get(key);\n            if (value) {\n                if (value.workingSet) {\n                    for (var i = 0, len = value.workingSet.length; i < len; i++) {\n                        var diagram = Repository.get(value.workingSet[i]);\n                        if (diagram) {\n                            openDiagram(diagram);\n                        }\n                    }\n                }\n                if (value.current) {\n                    var current = Repository.get(value.current);\n                    if (current) {\n                        setCurrentDiagram(current);\n                    }\n                }\n            }\n        }\n    }\n\n    /**\n     * Close all diagrams except active diagram.\n     */\n    function closeOthers() {\n        var active = getCurrentDiagram();\n        if (active) {\n            closeAll();\n            setCurrentDiagram(active);\n        }\n    }\n\n    /**\n     * Close all diagrams\n     */\n    function closeAll() {\n        _workingDiagrams = [];\n        $(exports).triggerHandler(\"workingDiagramsClear\");\n        setCurrentDiagram(null);\n    }\n\n    /**\n     * Set current diagram.\n     *\n     * @param {Diagram} diagram\n     * @param {boolean} skipEvent\n     */\n    function setCurrentDiagram(diagram, skipEvent) {\n        if (diagram) {\n            diagramEditor.diagram = diagram;\n            diagramEditor.setEnabled(true);\n            if (!_.contains(_workingDiagrams, diagram)) {\n                openDiagram(diagram);\n            }\n            __fixDiagramProblem(diagram);\n            repaint();\n            if (!skipEvent) {\n                _currentDiagramChangedEvent(diagramEditor.diagram);\n            }\n        } else {\n            diagramEditor.diagram = null;\n            diagramEditor.setEnabled(false);\n            _currentDiagramChangedEvent(diagramEditor.diagram);\n        }\n    }\n\n\n    /**\n     * Return current diagram\n     *\n     * @return {Editor}\n     */\n    function getCurrentDiagram() {\n        if (diagramEditor && diagramEditor.diagram) {\n            return diagramEditor.diagram;\n        }\n        return null;\n    }\n\n    /**\n     * Set next diagram as current diagram\n     */\n    function nextDiagram() {\n        var index = _.indexOf(_workingDiagrams, getCurrentDiagram());\n        if (index >= 0 && index < _workingDiagrams.length - 1) {\n            setCurrentDiagram(_workingDiagrams[index+1]);\n        } else if (index > 0 && index == _workingDiagrams.length - 1) {\n            setCurrentDiagram(_workingDiagrams[0]);\n        }\n    }\n\n    /**\n     * Set previous diagram as current diagram\n     */\n    function previousDiagram() {\n        var index = _.indexOf(_workingDiagrams, getCurrentDiagram());\n        if (index > 0 && index < _workingDiagrams.length) {\n            setCurrentDiagram(_workingDiagrams[index - 1]);\n        } else if (index === 0 && _workingDiagrams.length > 0) {\n            setCurrentDiagram(_workingDiagrams[_workingDiagrams.length - 1]);\n        }\n    }\n\n    /**\n     * Set an active handler.\n     *\n     * @param {Handler} handler\n     */\n    function setActiveHandler(handler) {\n        activeHandler = handler;\n        diagramEditor.activeHandler = activeHandler;\n    }\n\n    /**\n     * Return diagram editor.\n     *\n     * @return {Editor}\n     */\n    function getEditor() {\n        return diagramEditor;\n    }\n\n    /**\n     * Return Hidden Editor which is not shown\n     */\n    function getHiddenEditor() {\n        return hiddenEditor;\n    }\n\n    /**\n     * Select all view elements.\n     */\n    function selectAll() {\n        diagramEditor.selectAll();\n    }\n\n    /**\n     * Deselect all.\n     */\n    function deselectAll() {\n        diagramEditor.deselectAll();\n    }\n\n    /**\n     * Select view in diagram. Open diagram if diagram is not opened and then scroll to the view.\n     *\n     * @param {View} view\n     */\n    function selectInDiagram(view) {\n        var dgm = view.getDiagram();\n        setCurrentDiagram(dgm);\n        var center = view.getBoundingBox().getCenter();\n        scrollTo(center.x, center.y);\n        diagramEditor.deselectAll();\n        diagramEditor.selectView(view);\n        repaint();\n    }\n\n    function needRepaint(changedElements) {\n        var repaintRequired = false;\n        if (diagramEditor.diagram) {\n            if (Array.isArray(changedElements) && changedElements.length > 0) {\n                diagramEditor.diagram.traverse(function (v) {\n                    if (_.contains(changedElements, v)) {\n                        repaintRequired = true;\n                    } else if (v instanceof type.View && _.contains(changedElements, v.model)) {\n                        repaintRequired = true;\n                    }\n                });\n            }\n        }\n        return repaintRequired;\n    }\n\n    function suspendRepaint() {\n        _doRepaint = false;\n    }\n\n    function resumeRepaint() {\n        _doRepaint = true;\n        repaint();\n    }\n\n    /**\n     * Repaint.\n     */\n    function repaint() {\n        if (diagramEditor.diagram && _doRepaint) {\n            diagramEditor.repaint();\n            $(exports).triggerHandler(\"repaint\", [diagramEditor]);\n        }\n    }\n\n    /**\n     * Set zoom level\n     *\n     * @param {number} value Zoom level between 0 and 1.\n     */\n    function setZoomLevel(value) {\n        if (value < 0.1) { value = 0.1; }\n        if (value > 3) { value = 3; }\n        diagramEditor.canvas.zoomFactor.setScale(value);\n        repaint();\n    }\n\n    /**\n     * Return zoom level\n     *\n     * @return {number} Zoom level between 0 and 1.\n     */\n    function getZoomLevel() {\n        return diagramEditor.canvas.zoomFactor.getScale();\n    }\n\n    /**\n     * Toggle grid\n     */\n    function toggleGrid() {\n        if (diagramEditor.showGrid) {\n            hideGrid();\n        } else {\n            showGrid();\n        }\n    }\n\n    /**\n     * Show grid\n     */\n    function showGrid() {\n        diagramEditor.showGrid = true;\n        PreferenceManager.set(\"diagramEditor.showGrid\", isGridVisible());\n        CommandManager.get(Commands.VIEW_SHOW_GRID).setChecked(isGridVisible());\n        repaint();\n    }\n\n    /**\n     * Hide grid\n     */\n    function hideGrid() {\n        diagramEditor.showGrid = false;\n        PreferenceManager.set(\"diagramEditor.showGrid\", isGridVisible());\n        CommandManager.get(Commands.VIEW_SHOW_GRID).setChecked(isGridVisible());\n        repaint();\n    }\n\n    /**\n     * Return whether grid is visible.\n     *\n     * @return {boolean}\n     */\n    function isGridVisible() {\n        return diagramEditor.showGrid;\n    }\n\n    /**\n     * Return view size.\n     *\n     * @param {Point} x is width, y is height.\n     */\n    function getViewportSize() {\n        return new Graphics.Point($diagramArea.width(), $diagramArea.height());\n    }\n\n    /**\n     * Scroll diagram area to the given position\n     *\n     * @param {number} x\n     * @param {number} y\n     * @param {boolean} animation\n     */\n    function scrollTo(x, y, animation) {\n        var sz = getViewportSize(),\n            _x = Math.max(0, Math.floor((x * getZoomLevel()) - (sz.x / 2))),\n            _y = Math.max(0, Math.floor((y * getZoomLevel()) - (sz.y / 2)));\n\n        if (animation === false) {\n            $diagramArea.scrollLeft(_x);\n            $diagramArea.scrollTop(_y);\n        } else {\n            $diagramArea.animate({\n                scrollLeft: _x,\n                scrollTop: _y\n            }, 500);\n        }\n    }\n\n    /**\n     * Get postion of the scroll bar\n     *\n     * @return {Graphics.Point}\n     */\n    function getScrollPosition() {\n        return new Graphics.Point($diagramArea.scrollLeft(), $diagramArea.scrollTop());\n    }\n\n    /**\n     * Return left, top, right, bottom of diagram area with zoom level\n     *\n     * @return {Graphics.Rect}\n     */\n    function getDiagramArea() {\n        var zoom = getZoomLevel(),\n            scroll = getScrollPosition(),\n            size = getViewportSize(),\n            x = Math.round(scroll.x / zoom),\n            y = Math.round(scroll.y / zoom),\n            w = Math.round(size.x / zoom),\n            h = Math.round(size.y / zoom);\n        return new Graphics.Rect(x, y, x + w, y + h);\n    }\n\n    // TODO: DELETE THIS CODE AFTER ENOUGH TIME (BEGIN)\n    // 기존의 잘못된 다이어그램을 수정하기 위한 코드임.\n    function __fixDiagramProblem(diagram) {\n        if (diagram) {\n            // 1) Communication Diagram에서 hostEdge가 null인 UMLCommMessageView를 모두 지움.\n            if (diagram instanceof type.UMLCommunicationDiagram) {\n                _.each(diagram.ownedViews, function (v) {\n                    if (v instanceof type.UMLCommMessageView && v.hostEdge === null) {\n                        diagram.removeOwnedView(v);\n                    }\n                });\n            }\n\n            _.each(diagram.ownedViews, function (v) {\n\n                // 1) model이 없는 UMLGeneralNodeView, UMLGeneralEdgeView를 모두 삭제.\n                if (!v.model && v instanceof type.UMLGeneralNodeView) {\n                    diagram.removeOwnedView(v);\n                }\n                if (!v.model && v instanceof type.UMLGeneralEdgeView) {\n                    diagram.removeOwnedView(v);\n                }\n\n                // 2) _parent가 없는 View들 모두 삭제\n                if (!v._parent) {\n                    diagram.removeOwnedView(v);\n                }\n\n                // 3) head or tail이 없는 EdgeView는 삭제\n                if (v instanceof type.EdgeView) {\n                    if (!v.head || !v.tail) {\n                        diagram.removeOwnedView(v);\n                    }\n                }\n\n                // 4) end1.reference or end2.reference가 없는 UndirectedRelationship을 모두 삭제.\n                if (v.model instanceof type.UndirectedRelationship) {\n                    if (!v.model.end1.reference || !v.model.end2.reference) {\n                        var pf1 = v.model.getParentField();\n                        if (v.model._parent && pf1) {\n                            v.model._parent[pf1].remove(v.model);\n                        }\n                        diagram.removeOwnedView(v);\n                    }\n                }\n\n                // 5) source or target이 없는 DirectedRelationship을 모두 삭제.\n                if (v.model instanceof type.DirectedRelationship) {\n                    if (!v.model.source || !v.model.target) {\n                        var pf2 = v.model.getParentField();\n                        if (v.model._parent && pf2) {\n                            v.model._parent[pf2].remove(v.model);\n                        }\n                        diagram.removeOwnedView(v);\n                    }\n                }\n\n                // 5) nameLabel, stereotypeLabel, propertyLabel, RoleNameLabel, MultiplicityLabel, PropertyLabel, QualifiersCompartment\n                if (v.nameLabel && !Repository.get(v.nameLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.stereotypeLabel && !Repository.get(v.stereotypeLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.propertyLabel && !Repository.get(v.propertyLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.tailRoleNameLabel && !Repository.get(v.tailRoleNameLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.tailPropertyLabel && !Repository.get(v.tailPropertyLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.tailMultiplicityLabel && !Repository.get(v.tailMultiplicityLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.tailQualifiersCompartment && !Repository.get(v.tailQualifiersCompartment._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.headRoleNameLabel && !Repository.get(v.headRoleNameLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.headPropertyLabel && !Repository.get(v.headPropertyLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.headMultiplicityLabel && !Repository.get(v.headMultiplicityLabel._id)) {\n                    diagram.removeOwnedView(v);\n                }\n                if (v.headQualifiersCompartment && !Repository.get(v.headQualifiersCompartment._id)) {\n                    diagram.removeOwnedView(v);\n                }\n\n            });\n\n        }\n    }\n    // TODO: DELETE THIS CODE AFTER ENOUGH TIME (END)\n\n\n    /**\n     * Setup UI.\n     *\n     * @private\n     */\n    function _setupUI() {\n        // 다이어그램 편집기 설정하기.\n        diagramEditor = new Editor({ canvasId: \"diagram-canvas\" });\n        hiddenEditor = new Editor({ canvasId: \"hidden-canvas\"});\n        $diagramArea = $(\"#diagram-area\");\n\n        $(diagramEditor).on('mouseDown', function (event, mouseEvent) {\n            $(exports).triggerHandler('mouseDown', [mouseEvent]);\n        });\n        $(diagramEditor).on('selectionChanged', function (event, views) {\n            $(\"#diagram-canvas\").focus();\n            _triggerSelectionChangedEvent(views);\n        });\n        $(diagramEditor).on('viewDoubleClicked', function (event, view, x, y) {\n            _triggerDoubleClickedEvent(view, x, y);\n        });\n        $(diagramEditor).on('viewMoved', function (event, views, dx, dy) {\n            _triggerViewMovedEvent(views, dx, dy);\n        });\n        $(diagramEditor).on('parasiticViewMoved', function (event, view, alpha, distance) {\n            $(exports).triggerHandler('parasiticViewMoved', [view, alpha, distance]);\n        });\n        $(diagramEditor).on('containerViewChanged', function (event, views, dx, dy, containerView) {\n            $(exports).triggerHandler('containerViewChanged', [views, dx, dy, containerView]);\n        });\n        $(diagramEditor).on('nodeResized', function (event, node, left, top, right, bottom) {\n            $(exports).triggerHandler('nodeResized', [node, left, top, right, bottom]);\n        });\n        $(diagramEditor).on('edgeModified', function (event, edge, points) {\n            $(exports).triggerHandler('edgeModified', [edge, points]);\n        });\n        $(diagramEditor).on('edgeReconnected', function (event, edge, points, newParticipant, isTailSide) {\n            $(exports).triggerHandler('edgeReconnected', [edge, points, newParticipant, isTailSide]);\n        });\n    }\n\n    function _setupKeyBindings() {\n        $(\"#diagram-canvas\").keydown(function (e) {\n            var dx, dy;\n            switch (e.which) {\n            case KeyEvent.DOM_VK_BACK_SPACE:\n            case KeyEvent.DOM_VK_DELETE:\n                CommandManager.execute(Commands.EDIT_DELETE);\n                break;\n            case KeyEvent.DOM_VK_PLUS:\n                if (e.ctrlKey || e.metaKey) {\n                }\n                break;\n            case KeyEvent.DOM_VK_MINUS:\n                if (e.ctrlKey || e.metaKey) {\n                }\n                break;\n            case KeyEvent.DOM_VK_RETURN:\n                if (diagramEditor && diagramEditor.diagram && diagramEditor.diagram.selectedViews.length > 0) {\n                    var v = diagramEditor.diagram.selectedViews[0];\n                    if (v instanceof type.NodeView) {\n                        _triggerDoubleClickedEvent(v, v.left, v.top);\n                    } else if (v instanceof type.EdgeView) {\n                        _triggerDoubleClickedEvent(v);\n                    }\n                }\n                break;\n            case KeyEvent.DOM_VK_TAB:\n                if (diagramEditor && diagramEditor.diagram) {\n                    var idx = _.indexOf(diagramEditor.diagram.ownedViews, diagramEditor.diagram.selectedViews[0]);\n                    if (idx > -1 && idx < diagramEditor.diagram.ownedViews.length-1) {\n                        diagramEditor.diagram.selectedViews = [diagramEditor.diagram.ownedViews[idx+1]];\n                        _triggerSelectionChangedEvent([diagramEditor.diagram.ownedViews[idx+1]]);\n                        repaint();\n                    } else {\n                        diagramEditor.diagram.selectedViews = [diagramEditor.diagram.ownedViews[0]];\n                        _triggerSelectionChangedEvent([diagramEditor.diagram.ownedViews[0]]);\n                        repaint();\n                    }\n                }\n                e.preventDefault();\n                break;\n            case KeyEvent.DOM_VK_UP:\n                if (e.ctrlKey || e.metaKey) {\n                    if (diagramEditor && diagramEditor.diagram && diagramEditor.diagram.selectedViews.length > 0) {\n                        dy = diagramEditor.canvas.gridFactor.height;\n                        _triggerViewMovedEvent(diagramEditor.diagram.selectedViews, 0, -dy);\n                        e.preventDefault();\n                    }\n                }\n                break;\n            case KeyEvent.DOM_VK_DOWN:\n                if (e.ctrlKey || e.metaKey) {\n                    if (diagramEditor && diagramEditor.diagram && diagramEditor.diagram.selectedViews.length > 0) {\n                        dy = diagramEditor.canvas.gridFactor.height;\n                        _triggerViewMovedEvent(diagramEditor.diagram.selectedViews, 0, dy);\n                        e.preventDefault();\n                    }\n                }\n                break;\n            case KeyEvent.DOM_VK_LEFT:\n                if (e.ctrlKey || e.metaKey) {\n                    if (diagramEditor && diagramEditor.diagram && diagramEditor.diagram.selectedViews.length > 0) {\n                        dx = diagramEditor.canvas.gridFactor.width;\n                        _triggerViewMovedEvent(diagramEditor.diagram.selectedViews, -dx, 0);\n                        e.preventDefault();\n                    }\n                }\n                break;\n            case KeyEvent.DOM_VK_RIGHT:\n                if (e.ctrlKey || e.metaKey) {\n                    if (diagramEditor && diagramEditor.diagram && diagramEditor.diagram.selectedViews.length > 0) {\n                        dx = diagramEditor.canvas.gridFactor.width;\n                        _triggerViewMovedEvent(diagramEditor.diagram.selectedViews, dx, 0);\n                        e.preventDefault();\n                    }\n                }\n                break;\n            }\n        });\n    }\n\n    function _setupMouseBindings() {\n        $(\"#diagram-area\").mousewheel(function (e) {\n            if (e.ctrlKey || e.metaKey) {\n                e.preventDefault();\n                if (e.deltaY < 0) {\n                    CommandManager.execute(Commands.VIEW_ZOOM_OUT);\n                } else if (e.deltaY > 0) {\n                    CommandManager.execute(Commands.VIEW_ZOOM_IN);\n                }\n            }\n        });\n    }\n\n    function _setupPreferences() {\n        $(PreferenceManager).on(\"change\", function (event, key, value) {\n            try {\n                if (key === \"diagramEditor.showGrid\") {\n                    diagramEditor.showGrid = value;\n                    CommandManager.get(Commands.VIEW_SHOW_GRID).setChecked(isGridVisible());\n                    repaint();\n                }\n                if (key === \"diagramEditor.gridSize\") {\n                    diagramEditor.canvas.gridFactor.height = value;\n                    diagramEditor.canvas.gridFactor.width = value;\n                    repaint();\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        // read showGird\n        diagramEditor.showGrid = PreferenceManager.get(\"diagramEditor.showGrid\");\n        CommandManager.get(Commands.VIEW_SHOW_GRID).setChecked(isGridVisible());\n\n        // read gridSize\n        var gridSize = PreferenceManager.get(\"diagramEditor.gridSize\");\n        diagramEditor.canvas.gridFactor.height = gridSize;\n        diagramEditor.canvas.gridFactor.width = gridSize;\n    }\n\n    /**\n     * Current Diagram Changed\n     * @event module:diagrams/DiagramManager.currentDiagramChanged\n     * @property {Diagram} diagram\n     */\n    function _currentDiagramChangedEvent(diagram) {\n        $(exports).triggerHandler('currentDiagramChanged', [diagram, diagramEditor]);\n    }\n\n    /**\n     * Selection Changed\n     * @event module:diagrams/DiagramManager.selectionChanged\n     * @property {Array.<View>} views\n     */\n    function _triggerSelectionChangedEvent(views) {\n        $(exports).triggerHandler('selectionChanged', [views]);\n    }\n\n    /**\n     * Element Double Clicked\n     * @event module:diagrams/DiagramManager.viewDoubleClicked\n     * @property {View} view\n     * @property {number} x\n     * @property {number} y\n     */\n    function _triggerDoubleClickedEvent(view, x, y) {\n        $(exports).triggerHandler('viewDoubleClicked', [view, x, y]);\n    }\n\n    /**\n     * Views Moved\n     * @event module:diagrams/DiagramManager.viewMoved\n     * @property {Array.<View>} views\n     * @property {number} dx\n     * @property {number} dy\n     */\n    function _triggerViewMovedEvent(views, dx, dy) {\n        $(exports).triggerHandler('viewMoved', [views, dx, dy]);\n    }\n\n    AppInit.htmlReady(function () {\n        _setupUI();\n        _setupKeyBindings();\n        _setupMouseBindings();\n        _setupPreferences();\n        setCurrentDiagram(null);\n    });\n\n    exports.openDiagram            = openDiagram;\n    exports.updateDiagram          = updateDiagram;\n    exports.closeDiagram           = closeDiagram;\n    exports.getWorkingDiagrams     = getWorkingDiagrams;\n    exports.saveWorkingDiagrams    = saveWorkingDiagrams;\n    exports.restoreWorkingDiagrams = restoreWorkingDiagrams;\n    exports.closeOthers            = closeOthers;\n    exports.closeAll               = closeAll;\n    exports.setCurrentDiagram      = setCurrentDiagram;\n    exports.getCurrentDiagram      = getCurrentDiagram;\n    exports.nextDiagram            = nextDiagram;\n    exports.previousDiagram        = previousDiagram;\n    exports.setActiveHandler       = setActiveHandler;\n    exports.getEditor              = getEditor;\n    exports.getHiddenEditor        = getHiddenEditor;\n    exports.selectAll              = selectAll;\n    exports.deselectAll            = deselectAll;\n    exports.selectInDiagram        = selectInDiagram;\n    exports.needRepaint            = needRepaint;\n    exports.suspendRepaint         = suspendRepaint;\n    exports.resumeRepaint          = resumeRepaint;\n    exports.repaint                = repaint;\n    exports.setZoomLevel           = setZoomLevel;\n    exports.getZoomLevel           = getZoomLevel;\n    exports.toggleGrid             = toggleGrid;\n    exports.showGrid               = showGrid;\n    exports.hideGrid               = hideGrid;\n    exports.isGridVisible          = isGridVisible;\n    exports.getViewportSize        = getViewportSize;\n    exports.scrollTo               = scrollTo;\n    exports.getScrollPosition      = getScrollPosition;\n    exports.getDiagramArea         = getDiagramArea;\n\n});\n\n//# sourceURL=/diagrams/DiagramManager.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, Mustache, type */\n\n/**\n * Factory creates model, view and diagram elements\n *\n * This module dispatches these events:\n * - diagramCreated: (diagram)\n * - elementCreated: (model, view)\n */\ndefine(\'engine/Factory\',[\'require\',\'exports\',\'module\',\'utils/Helper\',\'core/Core\',\'core/OperationBuilder\',\'core/MetaModelManager\',\'core/Repository\',\'engine/Engine\',\'engine/ProjectManager\',\'engine/SelectionManager\',\'dialogs/Dialogs\',\'diagrams/DiagramManager\'],function (require, exports, module) {\n    "use strict";\n\n    var Helper           = require("utils/Helper"),\n        Core             = require("core/Core"),\n        OperationBuilder = require("core/OperationBuilder"),\n        MetaModelManager = require("core/MetaModelManager"),\n        Repository       = require("core/Repository"),\n        Engine           = require("engine/Engine"),\n        ProjectManager   = require("engine/ProjectManager"),\n        SelectionManager = require("engine/SelectionManager"),\n        Dialogs          = require("dialogs/Dialogs"),\n        DiagramManager   = require("diagrams/DiagramManager");\n\n\n    /**\n     * Error Messages\n     * @private\n     */\n    var ERR_INVALID_PARENT = "{{.}} cannot be placed here.",\n        ERR_INVALID_LINK   = "Invalid connection ({{.}})";\n\n    /**\n     * Map for functions to create diagram.\n     *\n     * @private\n     * @type {Object.<string, Function>}\n     */\n    var _diagramFn = {};\n\n    /**\n     * Map for functions to create model element.\n     *\n     * @private\n     * @type {Object.<string, Function>}\n     */\n    var _modelFn = {};\n\n    /**\n     * Map for functions to create both model and view elements.\n     *\n     * @private\n     * @type {Object.<string, Function>}\n     */\n    var _modelAndViewFn = {};\n\n    /**\n     * Map for default options for createModelAndView.\n     *\n     * @private\n     * @type {Object.<string, Object>}\n     */\n    var _modelAndViewOptions = {};\n\n    /**\n     * Map for functions to create a view element of a given model element.\n     *\n     * @private\n     * @type {Object.<string, Function>}\n     */\n    var _viewOfFn = {};\n\n\n    /**\n     * Trigger diagram created event\n     *\n     * @private\n     * @param{Diagram} diagram\n     */\n    function triggerDiagramCreated(diagram) {\n        var d = diagram ? Repository.get(diagram._id) : null;\n        $(exports).triggerHandler(\'diagramCreated\', [d]);\n    }\n\n    /**\n     * Trigger model and view elements created event\n     *\n     * @private\n     * @param{Model} model\n     * @param{View} view\n     */\n    function triggerElementCreated(model, view) {\n        var m = model ? Repository.get(model._id) : null,\n            v = view ? Repository.get(view._id) : null;\n        if (m || v) {\n            $(exports).triggerHandler(\'elementCreated\', [m, v]);\n        }\n    }\n\n    /**\n     * Create a diagram\n     *\n     * @param{string} id\n     * @param{Model} parent\n     * @param{Object} options\n     * @return {Diagram}\n     */\n    function createDiagram(id, parent, options) {\n        var fn = _diagramFn[id];\n        if (!fn) {\n            console.error("Factory.createDiagram(): No such function of id: " + id);\n            return null;\n        }\n        parent = parent || SelectionManager.getSelected() || ProjectManager.getProject();\n        options = options || {};\n        options.id = id;\n        options.diagramType = options.diagramType || id;\n        options.triggerDiagramCreated = triggerDiagramCreated;\n        options.triggerElementCreated = triggerElementCreated;\n        return fn(parent, options);\n    }\n\n    /**\n     * Create a model element\n     *\n     * @param{string} id\n     * @param{Model} parent\n     * @param{string} field\n     * @param{Object} options\n     * @return {Model}\n     */\n    function createModel(id, parent, field, options) {\n        var fn = _modelFn[id];\n        if (!fn) {\n            console.error("Factory.createModel(): No such function of id: " + id);\n            return null;\n        }\n        parent = parent || SelectionManager.getSelected();\n        field = field || "ownedElements";\n        options = options || {};\n        options.id = id;\n        options.modelType = options.modelType || id;\n        options.triggerDiagramCreated = triggerDiagramCreated;\n        options.triggerElementCreated = triggerElementCreated;\n        return fn(parent, field, options);\n    }\n\n    /**\n     * Create model and view elements\n     *\n     * @param{string} id\n     * @param{Model} parent\n     * @param{Diagram} diagram\n     * @param{Object} options\n     * @return {View}\n     */\n    function createModelAndView(id, parent, diagram, options) {\n        var fn = _modelAndViewFn[id];\n        if (!fn) {\n            console.error("Factory.createModelAndView(): No such function of id: " + id);\n            return null;\n        }\n        options = options || {};\n        options.id = id;\n        options.modelType = options.modelType || id;\n        options.viewType = options.viewType || MetaModelManager.getViewTypeOf(options.modelType);\n        options.triggerDiagramCreated = triggerDiagramCreated;\n        options.triggerElementCreated = triggerElementCreated;\n        return fn(parent, diagram, options);\n    }\n\n    /**\n     * Create a view element of a given model based on diagram type\n     *\n     * @param{Model} model\n     * @param{Diagram} diagram\n     * @param{Object} options\n     */\n    function createViewOf(model, diagram, options) {\n        var fn = _viewOfFn[diagram.getClassName()];\n        if (!fn) {\n            console.error("Factory.createViewOf(): No function is registered for diagram type: " + diagram.getClassName());\n            return null;\n        }\n        options = options || {};\n        options.triggerDiagramCreated = triggerDiagramCreated;\n        options.triggerElementCreated = triggerElementCreated;\n        return fn(model, diagram, options);\n    }\n\n    /**\n     * Return all available Ids for \'createDiagram\' function\n     *\n     * @return {Array.<string>}\n     */\n    function getDiagramIds() {\n        return _.keys(_diagramFn);\n    }\n\n    /**\n     * Return all available Ids for \'createModel\' function\n     *\n     * @return {Array.<string>}\n     */\n    function getModelIds() {\n        return _.keys(_modelFn);\n    }\n\n    /**\n     * Return all available Ids for \'createModelAndView\' function\n     *\n     * @return {Array.<string>}\n     */\n    function getModelAndViewIds() {\n        return _.keys(_modelAndViewFn);\n    }\n\n    /**\n     * Register a function to create diagram.\n     *\n     * @param{string} id\n     * @param{Function} fn\n     */\n    function registerDiagramFn(id, fn) {\n        if (_diagramFn[id]) {\n            console.error("Factory.registerDiagramFn(): Already registered id: " + id);\n            return;\n        }\n        _diagramFn[id] = fn;\n    }\n\n    /**\n     * Register a function to create a model element.\n     *\n     * @param{string} id\n     * @param{Function} fn\n     */\n    function registerModelFn(id, fn) {\n        if (_modelFn[id]) {\n            console.error("Factory.registerModelFn(): Already registered id: " + id);\n            return;\n        }\n        _modelFn[id] = fn;\n    }\n\n    /**\n     * Register a function to create both model and view elements.\n     *\n     * @param{string} id\n     * @param{Function} fn\n     * @param{Object} defaultOptions\n     */\n    function registerModelAndViewFn(id, fn, defaultOptions) {\n        if (_modelAndViewFn[id]) {\n            console.error("Factory.registerModelAndViewFn(): Already registered id: " + id);\n            return;\n        }\n        if (defaultOptions) {\n            _modelAndViewOptions[id] = defaultOptions;\n        }\n        _modelAndViewFn[id] = fn;\n    }\n\n    /**\n     * Register a function to create a view element based on diagram type.\n     *\n     * @param{string} diagramType\n     * @param{Function} fn\n     */\n    function registerViewOfFn(diagramType, fn) {\n        if (_viewOfFn[diagramType]) {\n            console.error("Factory.registerViewOfFn(): Already registered diagram type: " + diagramType);\n            return;\n        }\n        _viewOfFn[diagramType] = fn;\n    }\n\n\n    /**\n     * Default precondition for model creation\n     * @param{Object} options\n     */\n    function _defaultModelPrecondition(options) {\n        Helper.assert(\n            options.parent instanceof type.Model,\n            Mustache.render(ERR_INVALID_PARENT, options.modelType)\n        );\n    }\n\n\n\n    /**\n     * Default function to create diagram.\n     *\n     * @param {Model} parent\n     * @param {Object} options Options = { precondition, diagramType, diagramInitializer }\n     */\n    function defaultDiagramFn(parent, options) {\n        var DiagramType        = type[options.diagramType],\n            field              = options.field || "ownedElements",\n            precondition       = options.precondition || _defaultModelPrecondition,\n            diagramInitializer = options.diagramInitializer || null,\n            diagram;\n\n        // Check precondition\n        if (precondition) {\n            options.parent = parent;\n            precondition(options);\n        }\n\n        // Create model element;\n        diagram = new DiagramType();\n        diagram.name = Core.getNewName(parent[field], diagram.getDisplayClassName());\n        if (diagramInitializer) {\n            diagramInitializer(diagram);\n        }\n        Engine.addModel(parent, field, diagram);\n\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        // Trigger event\n        triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    /**\n     * Default function to create model element.\n     *\n     * @param {Model} parent\n     * @param {string} field\n     * @param {Object} options Options = { precondition, modelType, field, modelInitializer }\n     */\n    function defaultModelFn(parent, field, options) {\n        var ModelType        = type[options.modelType],\n            precondition     = options.precondition || _defaultModelPrecondition,\n            modelInitializer = options.modelInitializer || null,\n            model;\n\n        // Check precondition\n        if (precondition) {\n            options.parent = parent;\n            precondition(options);\n        }\n\n        // Create model element;\n        model = new ModelType();\n        model.name = Core.getNewName(parent[field], model.getDisplayClassName());\n        if (modelInitializer) {\n            modelInitializer(model);\n        }\n        Engine.addModel(parent, field, model);\n\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        // Trigger event\n        triggerElementCreated(model, null);\n\n        return model;\n    }\n\n    /**\n     * Default function to create model and view elements.\n     *\n     * @param {Model} parent\n     * @param {Diagram} diagram\n     * @param {Object} options Options = { precondition, modelType, viewType,\n     *      field, modelInitializer, viewInitializer, x1, y1, x2, y2, containerView }\n     */\n    function defaultModelAndViewFn(parent, diagram, options) {\n        // Set default options\n        if (_modelAndViewOptions[options.id]) {\n            _.extend(options, _modelAndViewOptions[options.id]);\n        }\n\n        var ModelType        = type[options.modelType],\n            ViewType         = type[options.viewType],\n            precondition     = options.precondition || _defaultModelPrecondition,\n            field            = options.field || "ownedElements",\n            modelInitializer = options.modelInitializer || null,\n            viewInitializer  = options.viewInitializer || null,\n            x1               = options.x1 || 0,\n            y1               = options.y1 || 0,\n            x2               = options.x2 || 0,\n            y2               = options.y2 || 0,\n            containerView    = options.containerView || null,\n            model,\n            view;\n\n        // Check precondition\n        if (precondition) {\n            options.parent = parent;\n            options.diagram = diagram;\n            precondition(options);\n        }\n\n        // Create model element\n        model = new ModelType();\n        model.name = Core.getNewName(parent[field], model.getDisplayClassName());\n        model._parent = parent;\n        if (modelInitializer) {\n            modelInitializer(model);\n        }\n        OperationBuilder.begin("Create " + model.getDisplayClassName());\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, field, model);\n\n        // Create view element\n        view = new ViewType();\n        view.initialize(null, x1, y1, x2, y2);\n        view.model = model;\n        view._parent = diagram;\n        if (viewInitializer) {\n            viewInitializer(view);\n        }\n        if (containerView) {\n            view.containerView = containerView;\n        }\n\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", view);\n        if (containerView) {\n            OperationBuilder.fieldInsert(containerView, "containedViews", view);\n        }\n\n        // Apply operation\n        OperationBuilder.end();\n        var op = OperationBuilder.getOperation();\n        Repository.doOperation(op);\n\n        // Trigger event\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        triggerElementCreated(model, view);\n        return view || model;\n    }\n\n\n    /**\n     * Default function to create model and view elements of UndirectedRelationship.\n     *\n     * @param {Model} parent\n     * @param {Diagram} diagram\n     * @param {Object} options Options = { precondition, modelType, viewType,\n     *      field, modelInitializer, viewInitializer, x1, y1, x2, y2, containerView }\n     */\n    function defaultUndirectedRelationshipFn(parent, diagram, options) {\n        // Set default options\n        if (_modelAndViewOptions[options.id]) {\n            _.extend(options, _modelAndViewOptions[options.id]);\n        }\n\n        var ModelType        = type[options.modelType],\n            ViewType         = type[options.viewType],\n            precondition     = options.precondition || _defaultModelPrecondition,\n            field            = options.field || "ownedElements",\n            modelInitializer = options.modelInitializer || null,\n            viewInitializer  = options.viewInitializer || null,\n            x1               = options.x1 || 0,\n            y1               = options.y1 || 0,\n            x2               = options.x2 || 0,\n            y2               = options.y2 || 0,\n            tailView         = options.tailView || null,\n            headView         = options.headView || null,\n            tailModel        = options.tailModel || null,\n            headModel        = options.headModel || null,\n            containerView    = options.containerView || null,\n            model,\n            view;\n\n        // Check precondition\n        if (precondition) {\n            options.parent = parent;\n            options.diagram = diagram;\n            precondition(options);\n        }\n\n        // Create model element\n        model = new ModelType();\n        model._parent = parent;\n        model.end1.reference = tailModel;\n        model.end2.reference = headModel;\n        if (modelInitializer) {\n            modelInitializer(model);\n        }\n        OperationBuilder.begin("Create " + model.getDisplayClassName());\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, field, model);\n\n        // Create view element\n        view = new ViewType();\n        view.model = model;\n        view._parent = diagram;\n        view.tail = tailView;\n        view.head = headView;\n        view.initialize(null, x1, y1, x2, y2);\n        if (viewInitializer) {\n            viewInitializer(view);\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", view);\n\n        // Apply operation\n        OperationBuilder.end();\n        var op = OperationBuilder.getOperation();\n        Repository.doOperation(op);\n\n        // Trigger event\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        triggerElementCreated(model, view);\n        return view || model;\n    }\n\n\n    /**\n     * Default function to create model and view elements of DirectedRelationship.\n     *\n     * @param {Model} parent\n     * @param {Diagram} diagram\n     * @param {Object} options Options = { precondition, modelType, viewType,\n     *      field, modelInitializer, viewInitializer, x1, y1, x2, y2, containerView }\n     */\n    function defaultDirectedRelationshipFn(parent, diagram, options) {\n        // Set default options\n        if (_modelAndViewOptions[options.id]) {\n            _.extend(options, _modelAndViewOptions[options.id]);\n        }\n\n        var ModelType        = type[options.modelType],\n            ViewType         = type[options.viewType],\n            precondition     = options.precondition || _defaultModelPrecondition,\n            field            = options.field || "ownedElements",\n            modelInitializer = options.modelInitializer || null,\n            viewInitializer  = options.viewInitializer || null,\n            x1               = options.x1 || 0,\n            y1               = options.y1 || 0,\n            x2               = options.x2 || 0,\n            y2               = options.y2 || 0,\n            tailView         = options.tailView || null,\n            headView         = options.headView || null,\n            tailModel        = options.tailModel || null,\n            headModel        = options.headModel || null,\n            containerView    = options.containerView || null,\n            model,\n            view;\n\n        // Check precondition\n        if (precondition) {\n            options.parent = parent;\n            options.diagram = diagram;\n            precondition(options);\n        }\n\n        // Create model element\n        model = new ModelType();\n        model._parent = parent;\n        model.source = tailModel;\n        model.target = headModel;\n        if (modelInitializer) {\n            modelInitializer(model);\n        }\n        OperationBuilder.begin("Create " + model.getDisplayClassName());\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, field, model);\n\n        // Create view element\n        view = new ViewType();\n        view.model = model;\n        view._parent = diagram;\n        view.tail = tailView;\n        view.head = headView;\n        view.initialize(null, x1, y1, x2, y2);\n        if (viewInitializer) {\n            viewInitializer(view);\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", view);\n\n        // Apply operation\n        OperationBuilder.end();\n        var op = OperationBuilder.getOperation();\n        Repository.doOperation(op);\n\n        // Trigger event\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        triggerElementCreated(model, view);\n        return view || model;\n    }\n\n    /**\n     * Default function to create view element only.\n     *\n     * @param {Model} parent\n     * @param {Diagram} diagram\n     * @param {Object} options Options = { precondition, viewType,\n     *      viewInitializer, x1, y1, x2, y2, containerView }\n     */\n    function defaultViewOnlyFn(parent, diagram, options) {\n        // Set default options\n        if (_modelAndViewOptions[options.id]) {\n            _.extend(options, _modelAndViewOptions[options.id]);\n        }\n\n        var ViewType         = type[options.viewType],\n            precondition     = options.precondition || null,\n            viewInitializer  = options.viewInitializer || null,\n            x1               = options.x1 || 0,\n            y1               = options.y1 || 0,\n            x2               = options.x2 || 0,\n            y2               = options.y2 || 0,\n            containerView    = options.containerView || null,\n            view;\n\n        // Check precondition\n        if (precondition) {\n            options.parent = parent;\n            options.diagram = diagram;\n            precondition(options);\n        }\n\n        view = new ViewType();\n        view.initialize(null, x1, y1, x2, y2);\n        if (viewInitializer) {\n            viewInitializer(view);\n        }\n        Engine.addViews(diagram, [view]);\n\n        // Trigger event\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        triggerElementCreated(null, view);\n        return view;\n    }\n\n    /**\n     * Default function to create edge view element only.\n     *\n     * @param {Model} parent\n     * @param {Diagram} diagram\n     * @param {Object} options Options = { precondition, viewType,\n     *      viewInitializer, x1, y1, x2, y2, containerView }\n     */\n    function defaultEdgeViewOnlyFn(parent, diagram, options) {\n        // Set default options\n        if (_modelAndViewOptions[options.id]) {\n            _.extend(options, _modelAndViewOptions[options.id]);\n        }\n\n        var ViewType         = type[options.viewType],\n            precondition     = options.precondition || null,\n            viewInitializer  = options.viewInitializer || null,\n            x1               = options.x1 || 0,\n            y1               = options.y1 || 0,\n            x2               = options.x2 || 0,\n            y2               = options.y2 || 0,\n            tailView         = options.tailView || null,\n            headView         = options.headView || null,\n            containerView    = options.containerView || null,\n            view;\n\n        // Check precondition\n        if (precondition) {\n            options.parent = parent;\n            options.diagram = diagram;\n            precondition(options);\n        }\n\n        // Create view element\n        view = new ViewType();\n        view.tail = tailView;\n        view.head = headView;\n        view.initialize(null, x1, y1, x2, y2);\n        if (viewInitializer) {\n            viewInitializer(view);\n        }\n        Engine.addViews(diagram, [view]);\n\n        // Trigger event\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        triggerElementCreated(null, view);\n        return view;\n    }\n\n    /**\n     * Create a view of a given model with relationship views\n     *\n     * @param {Editor} editor\n     * @param {number} x\n     * @param {number} y\n     * @param {Model} model\n     * @param {View} containerView\n     */\n    function createViewAndRelationships(editor, x, y, model, containerView) {\n        var ViewType = model.getViewType(),\n            diagram = editor.diagram,\n            view;\n        OperationBuilder.begin(\'add view with relations\');\n        if (ViewType) {\n            view = new ViewType();\n            view.model = model;\n            view.initialize(null, x, y, x, y);\n            view._parent = diagram;\n\n            // TODO: UML Specific Parts (need to be refactored)\n            if (view instanceof type.UMLGeneralNodeView && diagram._parent !== model._parent) {\n                view.showNamespace = true;\n            }\n\n            // add the view\n            OperationBuilder.insert(view);\n            OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n            if (containerView) {\n                OperationBuilder.fieldAssign(view, \'containerView\', containerView);\n                OperationBuilder.fieldInsert(containerView, \'containedViews\', view);\n            }\n\n            // TODO: UML Specific Parts (need to be refactored)\n            // add ConstraintLinkViews (when creating ConstraintView)\n            if (view instanceof type.UMLConstraintView) {\n                _.each(diagram.ownedViews, function (v) {\n                    if (v.model === view.model._parent) {\n                        var linkView = new type.UMLConstraintLinkView();\n                        linkView._parent = diagram;\n                        linkView.tail = view;\n                        linkView.head = v;\n                        linkView.initialize(null, linkView.tail.left, linkView.tail.top, linkView.head.left, linkView.head.top);\n                        OperationBuilder.insert(linkView);\n                        OperationBuilder.fieldInsert(diagram, \'ownedViews\', linkView);\n                    }\n                });\n            }\n\n            // add relation views\n            _.each(Repository.getRelationshipsOf(model), function (rel) {\n                var RelViewType = rel.getViewType(),\n                    relView;\n\n                if (RelViewType) {\n                    // for Directed Relationships\n                    if (rel instanceof type.DirectedRelationship) {\n                        _.each(diagram.ownedViews, function (v) {\n                            if ((v.model === rel.target) && (model === rel.source)) {\n                                relView = new RelViewType();\n                                relView._parent = diagram;\n                                relView.model = rel;\n                                relView.tail = view;\n                                relView.head = v;\n                                relView.initialize(null, relView.tail.left, relView.tail.top, relView.head.left, relView.head.top);\n                                OperationBuilder.insert(relView);\n                                OperationBuilder.fieldInsert(diagram, \'ownedViews\', relView);\n                            } else if ((v.model === rel.source) && (model === rel.target)) {\n                                relView = new RelViewType();\n                                relView._parent = diagram;\n                                relView.model = rel;\n                                relView.tail = v;\n                                relView.head = view;\n                                relView.initialize(null, relView.tail.left, relView.tail.top, relView.head.left, relView.head.top);\n                                OperationBuilder.insert(relView);\n                                OperationBuilder.fieldInsert(diagram, \'ownedViews\', relView);\n                            }\n                        });\n\n                        // create self-links.\n                        if ((rel.source === rel.target) && (rel.source === model)) {\n                            relView = new RelViewType();\n                            relView._parent = diagram;\n                            relView.model = rel;\n                            relView.tail = view;\n                            relView.head = view;\n                            relView.initialize(null, relView.tail.left, relView.tail.top, relView.head.left, relView.head.top);\n                            OperationBuilder.insert(relView);\n                            OperationBuilder.fieldInsert(diagram, \'ownedViews\', relView);\n                        }\n\n                    // for Undirected Relationships\n                    } else if (rel instanceof type.UndirectedRelationship) {\n                        _.each(diagram.ownedViews, function (v) {\n                            if ((v.model === rel.end2.reference) && (model === rel.end1.reference)) {\n                                relView = new RelViewType();\n                                relView._parent = diagram;\n                                relView.model = rel;\n                                relView.tail = view;\n                                relView.head = v;\n                                relView.initialize(null, relView.tail.left, relView.tail.top, relView.head.left, relView.head.top);\n                                OperationBuilder.insert(relView);\n                                OperationBuilder.fieldInsert(diagram, \'ownedViews\', relView);\n                            } else if ((v.model === rel.end1.reference) && (model === rel.end2.reference)) {\n                                relView = new RelViewType();\n                                relView._parent = diagram;\n                                relView.model = rel;\n                                relView.tail = v;\n                                relView.head = view;\n                                relView.initialize(null, relView.tail.left, relView.tail.top, relView.head.left, relView.head.top);\n                                OperationBuilder.insert(relView);\n                                OperationBuilder.fieldInsert(diagram, \'ownedViews\', relView);\n                            }\n                        });\n\n                        // create self-links.\n                        if ((rel.end1.reference === rel.end2.reference) && (rel.end1.reference === model)) {\n                            relView = new RelViewType();\n                            relView._parent = diagram;\n                            relView.model = rel;\n                            relView.tail = view;\n                            relView.head = view;\n                            relView.initialize(null, relView.tail.left, relView.tail.top, relView.head.left, relView.head.top);\n                            OperationBuilder.insert(relView);\n                            OperationBuilder.fieldInsert(diagram, \'ownedViews\', relView);\n                        }\n                    }\n                }\n            });\n            OperationBuilder.end();\n            var cmd = OperationBuilder.getOperation();\n            Repository.doOperation(cmd);\n        }\n\n        // Trigger event\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        triggerElementCreated(null, view);\n        return view;\n    }\n\n    /**\n     * Default function to create view of a particular model on a diagram.\n     *\n     * @param {Model} model\n     * @param {Diagram} diagram\n     * @param {Object} options\n     */\n    function defaultViewOnDiagramFn(model, diagram, options) {\n        var x             = options.x || 0,\n            y             = options.y || 0,\n            editor        = options.editor || DiagramManager.getEditor(),\n            containerView = options.containerView || null;\n\n        // Directed Relationships\n        if (model instanceof type.DirectedRelationship) {\n            var directedView    = diagram.getViewOf(model),\n                sourceView = diagram.getViewOf(model.source),\n                targetView = diagram.getViewOf(model.target);\n            if (directedView) {\n                editor.selectView(directedView);\n                editor.selectAdditionalView(sourceView);\n                editor.selectAdditionalView(targetView);\n                Dialogs.showAlertDialog("Relationship View is already existed in this Diagram.");\n            } else {\n                if (!targetView) {\n                    return createViewAndRelationships(editor, x, y, model.target);\n                }\n                if (!sourceView) {\n                    return createViewAndRelationships(editor, x, y + 100, model.source);\n                }\n                if (targetView && sourceView) {\n                    var DirectedViewType = model.getViewType();\n                    if (DirectedViewType) {\n                        directedView = new DirectedViewType();\n                        directedView._parent = diagram;\n                        directedView.model = model;\n                        directedView.tail = sourceView;\n                        directedView.head = targetView;\n                        directedView.initialize(null, directedView.tail.left, directedView.tail.top, directedView.head.left, directedView.head.top);\n                        Engine.addViews(diagram, [directedView]);\n                        if (directedView) {\n                            directedView = Repository.get(directedView._id);\n                        }\n                        editor.selectView(directedView);\n                        return directedView;\n                    }\n                }\n            }\n\n        // Undirected Relationships\n        } else if (model instanceof type.UndirectedRelationship) {\n            var undirectedView    = diagram.getViewOf(model),\n                end1View = diagram.getViewOf(model.end1.reference),\n                end2View = diagram.getViewOf(model.end2.reference);\n            if (undirectedView) {\n                editor.selectView(undirectedView);\n                editor.selectAdditionalView(end1View);\n                editor.selectAdditionalView(end2View);\n                Dialogs.showAlertDialog("Relationship View is already existed in this Diagram.");\n            } else {\n                if (!end2View) {\n                    return createViewAndRelationships(editor, x, y, model.end2.reference);\n                }\n                if (!end1View) {\n                    return createViewAndRelationships(editor, x, y + 100, model.end1.reference);\n                }\n                if (end1View && end2View) {\n                    var UndirectedViewType = model.getViewType();\n                    if (UndirectedViewType) {\n                        undirectedView = new UndirectedViewType();\n                        undirectedView._parent = diagram;\n                        undirectedView.model = model;\n                        undirectedView.tail = end1View;\n                        undirectedView.head = end2View;\n                        undirectedView.initialize(null, undirectedView.tail.left, undirectedView.tail.top, undirectedView.head.left, undirectedView.head.top);\n                        Engine.addViews(diagram, [undirectedView]);\n                        if (undirectedView) {\n                            undirectedView = Repository.get(undirectedView._id);\n                        }\n                        editor.selectView(undirectedView);\n                        return undirectedView;\n                    }\n                }\n            }\n        } else if (model instanceof type.Diagram) {\n            options.x1 = x;\n            options.y1 = y;\n            options.x2 = x;\n            options.y2 = y;\n            options.modelInitializer = function (elem) {\n                elem.reference = model;\n            };\n            return createModelAndView("Hyperlink", diagram._parent, diagram, options);\n        } else {\n            return createViewAndRelationships(editor, x, y, model);\n        }\n    }\n\n\n    exports.ERR_INVALID_PARENT = ERR_INVALID_PARENT;\n    exports.ERR_INVALID_LINK   = ERR_INVALID_LINK;\n\n    exports.createDiagram          = createDiagram;\n    exports.createModel            = createModel;\n    exports.createModelAndView     = createModelAndView;\n    exports.createViewOf           = createViewOf;\n\n    exports.getDiagramIds          = getDiagramIds;\n    exports.getModelIds            = getModelIds;\n    exports.getModelAndViewIds     = getModelAndViewIds;\n\n    exports.registerDiagramFn      = registerDiagramFn;\n    exports.registerModelFn        = registerModelFn;\n    exports.registerModelAndViewFn = registerModelAndViewFn;\n    exports.registerViewOfFn       = registerViewOfFn;\n\n    exports.defaultDiagramFn                = defaultDiagramFn;\n    exports.defaultModelFn                  = defaultModelFn;\n    exports.defaultModelAndViewFn           = defaultModelAndViewFn;\n    exports.defaultUndirectedRelationshipFn = defaultUndirectedRelationshipFn;\n    exports.defaultDirectedRelationshipFn   = defaultDirectedRelationshipFn;\n    exports.defaultViewOnlyFn               = defaultViewOnlyFn;\n    exports.defaultEdgeViewOnlyFn           = defaultEdgeViewOnlyFn;\n    exports.defaultViewOnDiagramFn          = defaultViewOnDiagramFn;\n    exports.createViewAndRelationships      = createViewAndRelationships;\n\n});\n\n\n//# sourceURL=/engine/Factory.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, browser: true, nomen: true, regexp: true,\nindent: 4, maxerr: 50 */\n/*global define, $, brackets, PathUtils */\n\n/* Functions for working with extension packages */\n\ndefine(\'extensibility/Package\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'filesystem/FileSystem\',\'file/FileUtils\',\'utils/StringUtils\',\'strings\',\'utils/ExtensionLoader\',\'utils/NodeConnection\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit              = require("utils/AppInit"),\n        FileSystem           = require("filesystem/FileSystem"),\n        FileUtils            = require("file/FileUtils"),\n        StringUtils          = require("utils/StringUtils"),\n        Strings              = require("strings"),\n        ExtensionLoader      = require("utils/ExtensionLoader"),\n        NodeConnection       = require("utils/NodeConnection");\n        // PreferencesManager   = require("preferences/PreferencesManager");\n\n    // PreferencesManager.definePreference("proxy", "string");\n\n    var Errors = {\n        ERROR_LOADING: "ERROR_LOADING",\n        MALFORMED_URL: "MALFORMED_URL",\n        UNSUPPORTED_PROTOCOL: "UNSUPPORTED_PROTOCOL"\n    };\n\n    var InstallationStatuses = {\n        FAILED: "FAILED",\n        INSTALLED: "INSTALLED",\n        ALREADY_INSTALLED: "ALREADY_INSTALLED",\n        SAME_VERSION: "SAME_VERSION",\n        OLDER_VERSION: "OLDER_VERSION",\n        NEEDS_UPDATE: "NEEDS_UPDATE",\n        DISABLED: "DISABLED"\n    };\n\n    /**\n     * @private\n     * @type {NodeConnection}\n     * Connects to ExtensionManagerDomain\n     */\n    var _nodeConnection;\n\n    /**\n     * @private\n     * @type{jQuery.Deferred.<NodeConnection>}\n     * A deferred which is resolved with a NodeConnection or rejected if\n     * we are unable to connect to Node.\n     */\n    var _nodeConnectionDeferred = $.Deferred();\n\n    /**\n     * @type {number} Used to generate unique download ids\n     */\n    var _uniqueId = 0;\n\n    function _extensionManagerCall(callback) {\n        if (_nodeConnection.domains.extensionManager) {\n            return callback(_nodeConnection.domains.extensionManager);\n        } else {\n            return new $.Deferred().reject("extensionManager domain is undefined").promise();\n        }\n    }\n\n    /**\n     * TODO: can this go away now that we never call it directly?\n     *\n     * Validates the package at the given path. The actual validation is\n     * handled by the Node server.\n     *\n     * The promise is resolved with an object:\n     * { errors: Array.<{string}>, metadata: { name:string, version:string, ... } }\n     * metadata is pulled straight from package.json and will be undefined\n     * if there are errors or null if the extension did not include package.json.\n     *\n     * @param {string} Absolute path to the package zip file\n     * @param {{requirePackageJSON: ?boolean}} validation options\n     * @return {$.Promise} A promise that is resolved with information about the package\n     */\n    function validate(path, options) {\n        return _extensionManagerCall(function (extensionManager) {\n            var d = new $.Deferred();\n\n            extensionManager.validate(path, options)\n                .done(function (result) {\n                    d.resolve({\n                        errors: result.errors,\n                        metadata: result.metadata\n                    });\n                })\n                .fail(function (error) {\n                    d.reject(error);\n                });\n\n            return d.promise();\n        });\n    }\n\n    /**\n     * Validates and installs the package at the given path. Validation and\n     * installation is handled by the Node process.\n     *\n     * The extension will be installed into the user\'s extensions directory.\n     * If the user already has the extension installed, it will instead go\n     * into their disabled extensions directory.\n     *\n     * The promise is resolved with an object:\n     * { errors: Array.<{string}>, metadata: { name:string, version:string, ... },\n     * disabledReason:string, installedTo:string, commonPrefix:string }\n     * metadata is pulled straight from package.json and is likely to be undefined\n     * if there are errors. It is null if there was no package.json.\n     *\n     * disabledReason is either null or the reason the extension was installed disabled.\n     *\n     * @param {string} path Absolute path to the package zip file\n     * @param {?string} nameHint Hint for the extension folder\'s name (used in favor of\n     *          path\'s filename if present, and if no package metadata present).\n     * @param {?boolean} _doUpdate private argument used to signal an update\n     * @return {$.Promise} A promise that is resolved with information about the package\n     *          (which may include errors, in which case the extension was disabled), or\n     *          rejected with an error object.\n     */\n    function install(path, nameHint, _doUpdate) {\n        return _extensionManagerCall(function (extensionManager) {\n            var d                       = new $.Deferred(),\n                destinationDirectory    = ExtensionLoader.getUserExtensionPath(),\n                disabledDirectory       = destinationDirectory.replace(/\\/user$/, "/disabled"),\n                systemDirectory         = FileUtils.getNativeDirectoryPath() + "/extensions/default/";\n\n            var operation = _doUpdate ? "update" : "install";\n            extensionManager[operation](path, destinationDirectory, {\n                disabledDirectory: disabledDirectory,\n                systemExtensionDirectory: systemDirectory,\n                apiVersion: app.metadata.apiVersion,\n                nameHint: nameHint\n            })\n                .done(function (result) {\n                    if (result.installationStatus !== InstallationStatuses.INSTALLED || _doUpdate) {\n                        d.resolve(result);\n                    } else {\n                        // This was a new extension and everything looked fine.\n                        // We load it into Brackets right away.\n                        ExtensionLoader.loadExtension(result.name, {\n                            // On Windows, it looks like Node converts Unix-y paths to backslashy paths.\n                            // We need to convert them back.\n                            baseUrl: FileUtils.convertWindowsPathToUnixPath(result.installedTo)\n                        }, "main").then(function () {\n                            d.resolve(result);\n                        }, function () {\n                            d.reject(Errors.ERROR_LOADING);\n                        });\n                    }\n                })\n                .fail(function (error) {\n                    d.reject(error);\n                });\n\n            return d.promise();\n        });\n    }\n\n\n\n    /**\n     * Special case handling to make the common case of downloading from GitHub easier; modifies \'urlInfo\' as\n     * needed. Converts a bare GitHub repo URL to the corresponding master ZIP URL; or if given a direct\n     * master ZIP URL already, sets a nicer download filename (both cases use the repo name).\n     *\n     * @param {{url:string, parsed:Array.<string>, filenameHint:string}} urlInfo\n     */\n    function githubURLFilter(urlInfo) {\n        if (urlInfo.parsed.hostname === "github.com" || urlInfo.parsed.hostname === "www.github.com") {\n            // Is it a URL to the root of a repo? (/user/repo)\n            var match = /^\\/[^\\/?]+\\/([^\\/?]+)(\\/?)$/.exec(urlInfo.parsed.pathname);\n            if (match) {\n                if (!match[2]) {\n                    urlInfo.url += "/";\n                }\n                urlInfo.url += "archive/master.zip";\n                urlInfo.filenameHint = match[1] + ".zip";\n\n            } else {\n                // Is it a URL directly to the repo\'s \'master.zip\'? (/user/repo/archive/master.zip)\n                match = /^\\/[^\\/?]+\\/([^\\/?]+)\\/archive\\/master.zip$/.exec(urlInfo.parsed.pathname);\n                if (match) {\n                    urlInfo.filenameHint = match[1] + ".zip";\n                }\n            }\n        }\n    }\n\n    /**\n     * Downloads from the given URL to a temporary location. On success, resolves with the path of the\n     * downloaded file (typically in a temp folder) and a hint for the real filename. On failure, rejects\n     * with an error object.\n     *\n     * @param {string} url URL of the file to be downloaded\n     * @param {number} downloadId Unique number to identify this request\n     * @return {$.Promise}\n     */\n    function download(url, downloadId) {\n        return _extensionManagerCall(function (extensionManager) {\n            var d = new $.Deferred();\n\n            // Validate URL\n            // TODO: PathUtils fails to parse URLs that are missing the protocol part (e.g. starts immediately with "www...")\n            var parsed = PathUtils.parseUrl(url);\n            if (!parsed.hostname) {  // means PathUtils failed to parse at all\n                d.reject(Errors.MALFORMED_URL);\n                return d.promise();\n            }\n            if (parsed.protocol !== "http:" && parsed.protocol !== "https:") {\n                d.reject(Errors.UNSUPPORTED_PROTOCOL);\n                return d.promise();\n            }\n\n            var urlInfo = { url: url, parsed: parsed, filenameHint: parsed.filename };\n            githubURLFilter(urlInfo);\n\n            // Decide download destination\n            var filename = urlInfo.filenameHint;\n            filename = filename.replace(/[^a-zA-Z0-9_\\- \\(\\)\\.]/g, "_"); // make sure it\'s a valid filename\n            if (!filename) {  // in case of URL ending in "/"\n                filename = "extension.zip";\n            }\n\n            // Download the bits (using Node since brackets-shell doesn\'t support binary file IO)\n            var r = extensionManager.downloadFile(downloadId, urlInfo.url, undefined/*, PreferencesManager.get("proxy")*/);\n            r.done(function (result) {\n                d.resolve({ localPath: FileUtils.convertWindowsPathToUnixPath(result), filenameHint: urlInfo.filenameHint });\n            }).fail(function (err) {\n                d.reject(err);\n            });\n\n            return d.promise();\n        });\n    }\n\n    /**\n     * Attempts to synchronously cancel the given pending download. This may not be possible, e.g.\n     * if the download has already finished.\n     *\n     * @param {number} downloadId Identifier previously passed to download()\n     */\n    function cancelDownload(downloadId) {\n        return _extensionManagerCall(function (extensionManager) {\n            return extensionManager.abortDownload(downloadId);\n        });\n    }\n\n\n    /**\n     * On success, resolves with an extension metadata object; at that point, the extension has already\n     * started running in Brackets. On failure (including validation errors), rejects with an error object.\n     *\n     * An error object consists of either a string error code OR an array where the first entry is the error\n     * code and the remaining entries are further info. The error code string is one of either\n     * ExtensionsDomain.Errors or Package.Errors. Use formatError() to convert an error object to a friendly,\n     * localized error message.\n     *\n     * The returned cancel() function will *attempt* to cancel installation, but it is not guaranteed to\n     * succeed. If cancel() succeeds, the Promise is rejected with a CANCELED error code. If we\'re unable\n     * to cancel, the Promise is resolved or rejected normally, as if cancel() had never been called.\n     *\n     * @return {{promise: $.Promise, cancel: function():boolean}}\n     */\n    function installFromURL(url) {\n        var STATE_DOWNLOADING = 1,\n            STATE_INSTALLING = 2,\n            STATE_SUCCEEDED = 3,\n            STATE_FAILED = 4;\n\n        var d = new $.Deferred();\n        var state = STATE_DOWNLOADING;\n\n        var downloadId = (_uniqueId++);\n        download(url, downloadId)\n            .done(function (downloadResult) {\n                state = STATE_INSTALLING;\n\n                install(downloadResult.localPath, downloadResult.filenameHint)\n                    .done(function (result) {\n                        var installationStatus = result.installationStatus;\n                        if (installationStatus === InstallationStatuses.ALREADY_INSTALLED ||\n                                installationStatus === InstallationStatuses.NEEDS_UPDATE ||\n                                installationStatus === InstallationStatuses.SAME_VERSION ||\n                                installationStatus === InstallationStatuses.OLDER_VERSION) {\n                            // We don\'t delete the file in this case, because it will be needed\n                            // if the user is going to install the update.\n                            state = STATE_SUCCEEDED;\n                            result.localPath = downloadResult.localPath;\n                            d.resolve(result);\n                        } else {\n                            FileSystem.getFileForPath(downloadResult.localPath).unlink();\n                            if (result.errors && result.errors.length > 0) {\n                                // Validation errors - for now, only return the first one\n                                state = STATE_FAILED;\n                                d.reject(result.errors[0]);\n                            } else if (result.disabledReason) {\n                                // Extension valid but left disabled (wrong API version, extension name collision, etc.)\n                                state = STATE_FAILED;\n                                d.reject(result.disabledReason);\n                            } else {\n                                // Success! Extension is now running in Brackets\n                                state = STATE_SUCCEEDED;\n                                d.resolve(result);\n                            }\n                        }\n                    })\n                    .fail(function (err) {\n                        // File IO errors, internal error in install()/validate(), or extension startup crashed\n                        state = STATE_FAILED;\n                        FileSystem.getFileForPath(downloadResult.localPath).unlink();\n                        d.reject(err);  // TODO: needs to be err.message ?\n                    });\n            })\n            .fail(function (err) {\n                // Download error (the Node-side download code cleans up any partial ZIP file)\n                state = STATE_FAILED;\n                d.reject(err);\n            });\n\n        return {\n            promise: d.promise(),\n            cancel: function () {\n                if (state === STATE_DOWNLOADING) {\n                    // This will trigger download()\'s fail() handler with CANCELED as the err code\n                    cancelDownload(downloadId);\n                }\n                // Else it\'s too late to cancel; we\'ll continue on through the done() chain and emit\n                // a success result (calling done() handlers) if all else goes well.\n            }\n        };\n    }\n\n    /**\n     * Converts an error object as returned by install() or installFromURL() into a flattened, localized string.\n     * @param {string|Array.<string>} error\n     * @return {string}\n     */\n    function formatError(error) {\n        function localize(key) {\n            if (Strings[key]) {\n                return Strings[key];\n            }\n            console.log("Unknown installation error", key);\n            return Strings.UNKNOWN_ERROR;\n        }\n\n        if (Array.isArray(error)) {\n            error[0] = localize(error[0]);\n            return StringUtils.format.apply(window, error);\n        } else {\n            return localize(error);\n        }\n    }\n\n    /**\n     * Removes the extension at the given path.\n     *\n     * @param {string} path The absolute path to the extension to remove.\n     * @return {$.Promise} A promise that\'s resolved when the extension is removed, or\n     *     rejected if there was an error.\n     */\n    function remove(path) {\n        return _extensionManagerCall(function (extensionManager) {\n            return extensionManager.remove(path);\n        });\n    }\n\n    /**\n     * Install an extension update located at path.\n     * This assumes that the installation was previously attempted\n     * and an installationStatus of "ALREADY_INSTALLED", "NEEDS_UPDATE", "SAME_VERSION",\n     * or "OLDER_VERSION" was the result.\n     *\n     * This workflow ensures that there should not generally be validation errors\n     * because the first pass at installation the extension looked at the metadata\n     * and installed packages.\n     *\n     * @param {string} path to package file\n     * @param {?string} nameHint Hint for the extension folder\'s name (used in favor of\n     *          path\'s filename if present, and if no package metadata present).\n     * @return {$.Promise} A promise that is resolved when the extension is successfully\n     *      installed or rejected if there is a problem.\n     */\n    function installUpdate(path, nameHint) {\n        var d = new $.Deferred();\n        install(path, nameHint, true)\n            .done(function (result) {\n                if (result.installationStatus !== InstallationStatuses.INSTALLED) {\n                    d.reject(result.errors);\n                } else {\n                    d.resolve(result);\n                }\n            })\n            .fail(function (error) {\n                d.reject(error);\n            })\n            .always(function () {\n                FileSystem.getFileForPath(path).unlink();\n            });\n        return d.promise();\n    }\n\n    /**\n     * Allows access to the deferred that manages the node connection. This\n     * is *only* for unit tests. Messing with this not in testing will\n     * potentially break everything.\n     *\n     * @private\n     * @return {jQuery.Deferred} The deferred that manages the node connection\n     */\n    function _getNodeConnectionDeferred() {\n        return _nodeConnectionDeferred;\n    }\n\n    // Initializes node connection\n    // TODO: duplicates code from StaticServer\n    // TODO: can this be done lazily?\n    AppInit.appReady(function () {\n        _nodeConnection = new NodeConnection();\n        _nodeConnection.connect(true).then(function () {\n            var domainPath = FileUtils.getNativeDirectoryPath() + "/" + FileUtils.getNativeModuleDirectoryPath(module) + "/node/ExtensionManagerDomain";\n            _nodeConnection.loadDomains(domainPath, true)\n                .then(\n                    function () {\n                        _nodeConnectionDeferred.resolve();\n                    },\n                    function () { // Failed to connect\n                        console.error("[Extensions] Failed to connect to node", arguments);\n                        _nodeConnectionDeferred.reject();\n                    }\n                );\n        });\n    });\n\n    // For unit tests only\n    exports._getNodeConnectionDeferred = _getNodeConnectionDeferred;\n\n    exports.installFromURL = installFromURL;\n    exports.validate = validate;\n    exports.install = install;\n    exports.remove = remove;\n    exports.installUpdate = installUpdate;\n    exports.formatError = formatError;\n    exports.InstallationStatuses = InstallationStatuses;\n});\n\n//# sourceURL=/extensibility/Package.js'),eval(";(function(exports) {\n\n// export the class if we are in a Node-like system.\nif (typeof module === 'object' && module.exports === exports)\n  exports = module.exports = SemVer;\n\n// The debug function is excluded entirely from the minified version.\n\n// Note: this is the semver.org version of the spec that it implements\n// Not necessarily the package version of this code.\nexports.SEMVER_SPEC_VERSION = '2.0.0';\n\n// The actual regexps go on exports.re\nvar re = exports.re = [];\nvar src = exports.src = [];\nvar R = 0;\n\n// The following Regular Expressions can be used for tokenizing,\n// validating, and parsing SemVer version strings.\n\n// ## Numeric Identifier\n// A single `0`, or a non-zero digit followed by zero or more digits.\n\nvar NUMERICIDENTIFIER = R++;\nsrc[NUMERICIDENTIFIER] = '0|[1-9]\\\\d*';\nvar NUMERICIDENTIFIERLOOSE = R++;\nsrc[NUMERICIDENTIFIERLOOSE] = '[0-9]+';\n\n\n// ## Non-numeric Identifier\n// Zero or more digits, followed by a letter or hyphen, and then zero or\n// more letters, digits, or hyphens.\n\nvar NONNUMERICIDENTIFIER = R++;\nsrc[NONNUMERICIDENTIFIER] = '\\\\d*[a-zA-Z-][a-zA-Z0-9-]*';\n\n\n// ## Main Version\n// Three dot-separated numeric identifiers.\n\nvar MAINVERSION = R++;\nsrc[MAINVERSION] = '(' + src[NUMERICIDENTIFIER] + ')\\\\.' +\n                   '(' + src[NUMERICIDENTIFIER] + ')\\\\.' +\n                   '(' + src[NUMERICIDENTIFIER] + ')';\n\nvar MAINVERSIONLOOSE = R++;\nsrc[MAINVERSIONLOOSE] = '(' + src[NUMERICIDENTIFIERLOOSE] + ')\\\\.' +\n                        '(' + src[NUMERICIDENTIFIERLOOSE] + ')\\\\.' +\n                        '(' + src[NUMERICIDENTIFIERLOOSE] + ')';\n\n// ## Pre-release Version Identifier\n// A numeric identifier, or a non-numeric identifier.\n\nvar PRERELEASEIDENTIFIER = R++;\nsrc[PRERELEASEIDENTIFIER] = '(?:' + src[NUMERICIDENTIFIER] +\n                            '|' + src[NONNUMERICIDENTIFIER] + ')';\n\nvar PRERELEASEIDENTIFIERLOOSE = R++;\nsrc[PRERELEASEIDENTIFIERLOOSE] = '(?:' + src[NUMERICIDENTIFIERLOOSE] +\n                                 '|' + src[NONNUMERICIDENTIFIER] + ')';\n\n\n// ## Pre-release Version\n// Hyphen, followed by one or more dot-separated pre-release version\n// identifiers.\n\nvar PRERELEASE = R++;\nsrc[PRERELEASE] = '(?:-(' + src[PRERELEASEIDENTIFIER] +\n                  '(?:\\\\.' + src[PRERELEASEIDENTIFIER] + ')*))';\n\nvar PRERELEASELOOSE = R++;\nsrc[PRERELEASELOOSE] = '(?:-?(' + src[PRERELEASEIDENTIFIERLOOSE] +\n                       '(?:\\\\.' + src[PRERELEASEIDENTIFIERLOOSE] + ')*))';\n\n// ## Build Metadata Identifier\n// Any combination of digits, letters, or hyphens.\n\nvar BUILDIDENTIFIER = R++;\nsrc[BUILDIDENTIFIER] = '[0-9A-Za-z-]+';\n\n// ## Build Metadata\n// Plus sign, followed by one or more period-separated build metadata\n// identifiers.\n\nvar BUILD = R++;\nsrc[BUILD] = '(?:\\\\+(' + src[BUILDIDENTIFIER] +\n             '(?:\\\\.' + src[BUILDIDENTIFIER] + ')*))';\n\n\n// ## Full Version String\n// A main version, followed optionally by a pre-release version and\n// build metadata.\n\n// Note that the only major, minor, patch, and pre-release sections of\n// the version string are capturing groups.  The build metadata is not a\n// capturing group, because it should not ever be used in version\n// comparison.\n\nvar FULL = R++;\nvar FULLPLAIN = 'v?' + src[MAINVERSION] +\n                src[PRERELEASE] + '?' +\n                src[BUILD] + '?';\n\nsrc[FULL] = '^' + FULLPLAIN + '$';\n\n// like full, but allows v1.2.3 and =1.2.3, which people do sometimes.\n// also, 1.0.0alpha1 (prerelease without the hyphen) which is pretty\n// common in the npm registry.\nvar LOOSEPLAIN = '[v=\\\\s]*' + src[MAINVERSIONLOOSE] +\n                 src[PRERELEASELOOSE] + '?' +\n                 src[BUILD] + '?';\n\nvar LOOSE = R++;\nsrc[LOOSE] = '^' + LOOSEPLAIN + '$';\n\nvar GTLT = R++;\nsrc[GTLT] = '((?:<|>)?=?)';\n\n// Something like \"2.*\" or \"1.2.x\".\n// Note that \"x.x\" is a valid xRange identifer, meaning \"any version\"\n// Only the first item is strictly required.\nvar XRANGEIDENTIFIERLOOSE = R++;\nsrc[XRANGEIDENTIFIERLOOSE] = src[NUMERICIDENTIFIERLOOSE] + '|x|X|\\\\*';\nvar XRANGEIDENTIFIER = R++;\nsrc[XRANGEIDENTIFIER] = src[NUMERICIDENTIFIER] + '|x|X|\\\\*';\n\nvar XRANGEPLAIN = R++;\nsrc[XRANGEPLAIN] = '[v=\\\\s]*(' + src[XRANGEIDENTIFIER] + ')' +\n                   '(?:\\\\.(' + src[XRANGEIDENTIFIER] + ')' +\n                   '(?:\\\\.(' + src[XRANGEIDENTIFIER] + ')' +\n                   '(?:(' + src[PRERELEASE] + ')' +\n                   ')?)?)?';\n\nvar XRANGEPLAINLOOSE = R++;\nsrc[XRANGEPLAINLOOSE] = '[v=\\\\s]*(' + src[XRANGEIDENTIFIERLOOSE] + ')' +\n                        '(?:\\\\.(' + src[XRANGEIDENTIFIERLOOSE] + ')' +\n                        '(?:\\\\.(' + src[XRANGEIDENTIFIERLOOSE] + ')' +\n                        '(?:(' + src[PRERELEASELOOSE] + ')' +\n                        ')?)?)?';\n\n// >=2.x, for example, means >=2.0.0-0\n// <1.x would be the same as \"<1.0.0-0\", though.\nvar XRANGE = R++;\nsrc[XRANGE] = '^' + src[GTLT] + '\\\\s*' + src[XRANGEPLAIN] + '$';\nvar XRANGELOOSE = R++;\nsrc[XRANGELOOSE] = '^' + src[GTLT] + '\\\\s*' + src[XRANGEPLAINLOOSE] + '$';\n\n// Tilde ranges.\n// Meaning is \"reasonably at or greater than\"\nvar LONETILDE = R++;\nsrc[LONETILDE] = '(?:~>?)';\n\nvar TILDETRIM = R++;\nsrc[TILDETRIM] = '(\\\\s*)' + src[LONETILDE] + '\\\\s+';\nre[TILDETRIM] = new RegExp(src[TILDETRIM], 'g');\nvar tildeTrimReplace = '$1~';\n\nvar TILDE = R++;\nsrc[TILDE] = '^' + src[LONETILDE] + src[XRANGEPLAIN] + '$';\nvar TILDELOOSE = R++;\nsrc[TILDELOOSE] = '^' + src[LONETILDE] + src[XRANGEPLAINLOOSE] + '$';\n\n// Caret ranges.\n// Meaning is \"at least and backwards compatible with\"\nvar LONECARET = R++;\nsrc[LONECARET] = '(?:\\\\^)';\n\nvar CARETTRIM = R++;\nsrc[CARETTRIM] = '(\\\\s*)' + src[LONECARET] + '\\\\s+';\nre[CARETTRIM] = new RegExp(src[CARETTRIM], 'g');\nvar caretTrimReplace = '$1^';\n\nvar CARET = R++;\nsrc[CARET] = '^' + src[LONECARET] + src[XRANGEPLAIN] + '$';\nvar CARETLOOSE = R++;\nsrc[CARETLOOSE] = '^' + src[LONECARET] + src[XRANGEPLAINLOOSE] + '$';\n\n// A simple gt/lt/eq thing, or just \"\" to indicate \"any version\"\nvar COMPARATORLOOSE = R++;\nsrc[COMPARATORLOOSE] = '^' + src[GTLT] + '\\\\s*(' + LOOSEPLAIN + ')$|^$';\nvar COMPARATOR = R++;\nsrc[COMPARATOR] = '^' + src[GTLT] + '\\\\s*(' + FULLPLAIN + ')$|^$';\n\n\n// An expression to strip any whitespace between the gtlt and the thing\n// it modifies, so that `> 1.2.3` ==> `>1.2.3`\nvar COMPARATORTRIM = R++;\nsrc[COMPARATORTRIM] = '(\\\\s*)' + src[GTLT] +\n                      '\\\\s*(' + LOOSEPLAIN + '|' + src[XRANGEPLAIN] + ')';\n\n// this one has to use the /g flag\nre[COMPARATORTRIM] = new RegExp(src[COMPARATORTRIM], 'g');\nvar comparatorTrimReplace = '$1$2$3';\n\n\n// Something like `1.2.3 - 1.2.4`\n// Note that these all use the loose form, because they'll be\n// checked against either the strict or loose comparator form\n// later.\nvar HYPHENRANGE = R++;\nsrc[HYPHENRANGE] = '^\\\\s*(' + src[XRANGEPLAIN] + ')' +\n                   '\\\\s+-\\\\s+' +\n                   '(' + src[XRANGEPLAIN] + ')' +\n                   '\\\\s*$';\n\nvar HYPHENRANGELOOSE = R++;\nsrc[HYPHENRANGELOOSE] = '^\\\\s*(' + src[XRANGEPLAINLOOSE] + ')' +\n                        '\\\\s+-\\\\s+' +\n                        '(' + src[XRANGEPLAINLOOSE] + ')' +\n                        '\\\\s*$';\n\n// Star ranges basically just allow anything at all.\nvar STAR = R++;\nsrc[STAR] = '(<|>)?=?\\\\s*\\\\*';\n\n// Compile to actual regexp objects.\n// All are flag-free, unless they were created above with a flag.\nfor (var i = 0; i < R; i++) {\n  ;\n  if (!re[i])\n    re[i] = new RegExp(src[i]);\n}\n\nexports.parse = parse;\nfunction parse(version, loose) {\n  var r = loose ? re[LOOSE] : re[FULL];\n  return (r.test(version)) ? new SemVer(version, loose) : null;\n}\n\nexports.valid = valid;\nfunction valid(version, loose) {\n  var v = parse(version, loose);\n  return v ? v.version : null;\n}\n\n\nexports.clean = clean;\nfunction clean(version, loose) {\n  var s = parse(version, loose);\n  return s ? s.version : null;\n}\n\nexports.SemVer = SemVer;\n\nfunction SemVer(version, loose) {\n  if (version instanceof SemVer) {\n    if (version.loose === loose)\n      return version;\n    else\n      version = version.version;\n  }\n\n  if (!(this instanceof SemVer))\n    return new SemVer(version, loose);\n\n  ;\n  this.loose = loose;\n  var m = version.trim().match(loose ? re[LOOSE] : re[FULL]);\n\n  if (!m)\n    throw new TypeError('Invalid Version: ' + version);\n\n  this.raw = version;\n\n  // these are actually numbers\n  this.major = +m[1];\n  this.minor = +m[2];\n  this.patch = +m[3];\n\n  // numberify any prerelease numeric ids\n  if (!m[4])\n    this.prerelease = [];\n  else\n    this.prerelease = m[4].split('.').map(function(id) {\n      return (/^[0-9]+$/.test(id)) ? +id : id;\n    });\n\n  this.build = m[5] ? m[5].split('.') : [];\n  this.format();\n}\n\nSemVer.prototype.format = function() {\n  this.version = this.major + '.' + this.minor + '.' + this.patch;\n  if (this.prerelease.length)\n    this.version += '-' + this.prerelease.join('.');\n  return this.version;\n};\n\nSemVer.prototype.inspect = function() {\n  return '<SemVer \"' + this + '\">';\n};\n\nSemVer.prototype.toString = function() {\n  return this.version;\n};\n\nSemVer.prototype.compare = function(other) {\n  ;\n  if (!(other instanceof SemVer))\n    other = new SemVer(other, this.loose);\n\n  return this.compareMain(other) || this.comparePre(other);\n};\n\nSemVer.prototype.compareMain = function(other) {\n  if (!(other instanceof SemVer))\n    other = new SemVer(other, this.loose);\n\n  return compareIdentifiers(this.major, other.major) ||\n         compareIdentifiers(this.minor, other.minor) ||\n         compareIdentifiers(this.patch, other.patch);\n};\n\nSemVer.prototype.comparePre = function(other) {\n  if (!(other instanceof SemVer))\n    other = new SemVer(other, this.loose);\n\n  // NOT having a prerelease is > having one\n  if (this.prerelease.length && !other.prerelease.length)\n    return -1;\n  else if (!this.prerelease.length && other.prerelease.length)\n    return 1;\n  else if (!this.prerelease.lenth && !other.prerelease.length)\n    return 0;\n\n  var i = 0;\n  do {\n    var a = this.prerelease[i];\n    var b = other.prerelease[i];\n    ;\n    if (a === undefined && b === undefined)\n      return 0;\n    else if (b === undefined)\n      return 1;\n    else if (a === undefined)\n      return -1;\n    else if (a === b)\n      continue;\n    else\n      return compareIdentifiers(a, b);\n  } while (++i);\n};\n\nSemVer.prototype.inc = function(release) {\n  switch (release) {\n    case 'major':\n      this.major++;\n      this.minor = -1;\n    case 'minor':\n      this.minor++;\n      this.patch = -1;\n    case 'patch':\n      this.patch++;\n      this.prerelease = [];\n      break;\n    case 'prerelease':\n      if (this.prerelease.length === 0)\n        this.prerelease = [0];\n      else {\n        var i = this.prerelease.length;\n        while (--i >= 0) {\n          if (typeof this.prerelease[i] === 'number') {\n            this.prerelease[i]++;\n            i = -2;\n          }\n        }\n        if (i === -1) // didn't increment anything\n          this.prerelease.push(0);\n      }\n      break;\n\n    default:\n      throw new Error('invalid increment argument: ' + release);\n  }\n  this.format();\n  return this;\n};\n\nexports.inc = inc;\nfunction inc(version, release, loose) {\n  try {\n    return new SemVer(version, loose).inc(release).version;\n  } catch (er) {\n    return null;\n  }\n}\n\nexports.compareIdentifiers = compareIdentifiers;\n\nvar numeric = /^[0-9]+$/;\nfunction compareIdentifiers(a, b) {\n  var anum = numeric.test(a);\n  var bnum = numeric.test(b);\n\n  if (anum && bnum) {\n    a = +a;\n    b = +b;\n  }\n\n  return (anum && !bnum) ? -1 :\n         (bnum && !anum) ? 1 :\n         a < b ? -1 :\n         a > b ? 1 :\n         0;\n}\n\nexports.rcompareIdentifiers = rcompareIdentifiers;\nfunction rcompareIdentifiers(a, b) {\n  return compareIdentifiers(b, a);\n}\n\nexports.compare = compare;\nfunction compare(a, b, loose) {\n  return new SemVer(a, loose).compare(b);\n}\n\nexports.compareLoose = compareLoose;\nfunction compareLoose(a, b) {\n  return compare(a, b, true);\n}\n\nexports.rcompare = rcompare;\nfunction rcompare(a, b, loose) {\n  return compare(b, a, loose);\n}\n\nexports.sort = sort;\nfunction sort(list, loose) {\n  return list.sort(function(a, b) {\n    return exports.compare(a, b, loose);\n  });\n}\n\nexports.rsort = rsort;\nfunction rsort(list, loose) {\n  return list.sort(function(a, b) {\n    return exports.rcompare(a, b, loose);\n  });\n}\n\nexports.gt = gt;\nfunction gt(a, b, loose) {\n  return compare(a, b, loose) > 0;\n}\n\nexports.lt = lt;\nfunction lt(a, b, loose) {\n  return compare(a, b, loose) < 0;\n}\n\nexports.eq = eq;\nfunction eq(a, b, loose) {\n  return compare(a, b, loose) === 0;\n}\n\nexports.neq = neq;\nfunction neq(a, b, loose) {\n  return compare(a, b, loose) !== 0;\n}\n\nexports.gte = gte;\nfunction gte(a, b, loose) {\n  return compare(a, b, loose) >= 0;\n}\n\nexports.lte = lte;\nfunction lte(a, b, loose) {\n  return compare(a, b, loose) <= 0;\n}\n\nexports.cmp = cmp;\nfunction cmp(a, op, b, loose) {\n  var ret;\n  switch (op) {\n    case '===': ret = a === b; break;\n    case '!==': ret = a !== b; break;\n    case '': case '=': case '==': ret = eq(a, b, loose); break;\n    case '!=': ret = neq(a, b, loose); break;\n    case '>': ret = gt(a, b, loose); break;\n    case '>=': ret = gte(a, b, loose); break;\n    case '<': ret = lt(a, b, loose); break;\n    case '<=': ret = lte(a, b, loose); break;\n    default: throw new TypeError('Invalid operator: ' + op);\n  }\n  return ret;\n}\n\nexports.Comparator = Comparator;\nfunction Comparator(comp, loose) {\n  if (comp instanceof Comparator) {\n    if (comp.loose === loose)\n      return comp;\n    else\n      comp = comp.value;\n  }\n\n  if (!(this instanceof Comparator))\n    return new Comparator(comp, loose);\n\n  ;\n  this.loose = loose;\n  this.parse(comp);\n\n  if (this.semver === ANY)\n    this.value = '';\n  else\n    this.value = this.operator + this.semver.version;\n}\n\nvar ANY = {};\nComparator.prototype.parse = function(comp) {\n  var r = this.loose ? re[COMPARATORLOOSE] : re[COMPARATOR];\n  var m = comp.match(r);\n\n  if (!m)\n    throw new TypeError('Invalid comparator: ' + comp);\n\n  this.operator = m[1];\n  // if it literally is just '>' or '' then allow anything.\n  if (!m[2])\n    this.semver = ANY;\n  else {\n    this.semver = new SemVer(m[2], this.loose);\n\n    // <1.2.3-rc DOES allow 1.2.3-beta (has prerelease)\n    // >=1.2.3 DOES NOT allow 1.2.3-beta\n    // <=1.2.3 DOES allow 1.2.3-beta\n    // However, <1.2.3 does NOT allow 1.2.3-beta,\n    // even though `1.2.3-beta < 1.2.3`\n    // The assumption is that the 1.2.3 version has something you\n    // *don't* want, so we push the prerelease down to the minimum.\n    if (this.operator === '<' && !this.semver.prerelease.length) {\n      this.semver.prerelease = ['0'];\n      this.semver.format();\n    }\n  }\n};\n\nComparator.prototype.inspect = function() {\n  return '<SemVer Comparator \"' + this + '\">';\n};\n\nComparator.prototype.toString = function() {\n  return this.value;\n};\n\nComparator.prototype.test = function(version) {\n  ;\n  return (this.semver === ANY) ? true :\n         cmp(version, this.operator, this.semver, this.loose);\n};\n\n\nexports.Range = Range;\nfunction Range(range, loose) {\n  if ((range instanceof Range) && range.loose === loose)\n    return range;\n\n  if (!(this instanceof Range))\n    return new Range(range, loose);\n\n  this.loose = loose;\n\n  // First, split based on boolean or ||\n  this.raw = range;\n  this.set = range.split(/\\s*\\|\\|\\s*/).map(function(range) {\n    return this.parseRange(range.trim());\n  }, this).filter(function(c) {\n    // throw out any that are not relevant for whatever reason\n    return c.length;\n  });\n\n  if (!this.set.length) {\n    throw new TypeError('Invalid SemVer Range: ' + range);\n  }\n\n  this.format();\n}\n\nRange.prototype.inspect = function() {\n  return '<SemVer Range \"' + this.range + '\">';\n};\n\nRange.prototype.format = function() {\n  this.range = this.set.map(function(comps) {\n    return comps.join(' ').trim();\n  }).join('||').trim();\n  return this.range;\n};\n\nRange.prototype.toString = function() {\n  return this.range;\n};\n\nRange.prototype.parseRange = function(range) {\n  var loose = this.loose;\n  range = range.trim();\n  ;\n  // `1.2.3 - 1.2.4` => `>=1.2.3 <=1.2.4`\n  var hr = loose ? re[HYPHENRANGELOOSE] : re[HYPHENRANGE];\n  range = range.replace(hr, hyphenReplace);\n  ;\n  // `> 1.2.3 < 1.2.5` => `>1.2.3 <1.2.5`\n  range = range.replace(re[COMPARATORTRIM], comparatorTrimReplace);\n  ;\n\n  // `~ 1.2.3` => `~1.2.3`\n  range = range.replace(re[TILDETRIM], tildeTrimReplace);\n\n  // `^ 1.2.3` => `^1.2.3`\n  range = range.replace(re[CARETTRIM], caretTrimReplace);\n\n  // normalize spaces\n  range = range.split(/\\s+/).join(' ');\n\n  // At this point, the range is completely trimmed and\n  // ready to be split into comparators.\n\n  var compRe = loose ? re[COMPARATORLOOSE] : re[COMPARATOR];\n  var set = range.split(' ').map(function(comp) {\n    return parseComparator(comp, loose);\n  }).join(' ').split(/\\s+/);\n  if (this.loose) {\n    // in loose mode, throw out any that are not valid comparators\n    set = set.filter(function(comp) {\n      return !!comp.match(compRe);\n    });\n  }\n  set = set.map(function(comp) {\n    return new Comparator(comp, loose);\n  });\n\n  return set;\n};\n\n// Mostly just for testing and legacy API reasons\nexports.toComparators = toComparators;\nfunction toComparators(range, loose) {\n  return new Range(range, loose).set.map(function(comp) {\n    return comp.map(function(c) {\n      return c.value;\n    }).join(' ').trim().split(' ');\n  });\n}\n\n// comprised of xranges, tildes, stars, and gtlt's at this point.\n// already replaced the hyphen ranges\n// turn into a set of JUST comparators.\nfunction parseComparator(comp, loose) {\n  ;\n  comp = replaceCarets(comp, loose);\n  ;\n  comp = replaceTildes(comp, loose);\n  ;\n  comp = replaceXRanges(comp, loose);\n  ;\n  comp = replaceStars(comp, loose);\n  ;\n  return comp;\n}\n\nfunction isX(id) {\n  return !id || id.toLowerCase() === 'x' || id === '*';\n}\n\n// ~, ~> --> * (any, kinda silly)\n// ~2, ~2.x, ~2.x.x, ~>2, ~>2.x ~>2.x.x --> >=2.0.0 <3.0.0\n// ~2.0, ~2.0.x, ~>2.0, ~>2.0.x --> >=2.0.0 <2.1.0\n// ~1.2, ~1.2.x, ~>1.2, ~>1.2.x --> >=1.2.0 <1.3.0\n// ~1.2.3, ~>1.2.3 --> >=1.2.3 <1.3.0\n// ~1.2.0, ~>1.2.0 --> >=1.2.0 <1.3.0\nfunction replaceTildes(comp, loose) {\n  return comp.trim().split(/\\s+/).map(function(comp) {\n    return replaceTilde(comp, loose);\n  }).join(' ');\n}\n\nfunction replaceTilde(comp, loose) {\n  var r = loose ? re[TILDELOOSE] : re[TILDE];\n  return comp.replace(r, function(_, M, m, p, pr) {\n    ;\n    var ret;\n\n    if (isX(M))\n      ret = '';\n    else if (isX(m))\n      ret = '>=' + M + '.0.0-0 <' + (+M + 1) + '.0.0-0';\n    else if (isX(p))\n      // ~1.2 == >=1.2.0- <1.3.0-\n      ret = '>=' + M + '.' + m + '.0-0 <' + M + '.' + (+m + 1) + '.0-0';\n    else if (pr) {\n      ;\n      if (pr.charAt(0) !== '-')\n        pr = '-' + pr;\n      ret = '>=' + M + '.' + m + '.' + p + pr +\n            ' <' + M + '.' + (+m + 1) + '.0-0';\n    } else\n      // ~1.2.3 == >=1.2.3-0 <1.3.0-0\n      ret = '>=' + M + '.' + m + '.' + p + '-0' +\n            ' <' + M + '.' + (+m + 1) + '.0-0';\n\n    ;\n    return ret;\n  });\n}\n\n// ^ --> * (any, kinda silly)\n// ^2, ^2.x, ^2.x.x --> >=2.0.0 <3.0.0\n// ^2.0, ^2.0.x --> >=2.0.0 <3.0.0\n// ^1.2, ^1.2.x --> >=1.2.0 <2.0.0\n// ^1.2.3 --> >=1.2.3 <2.0.0\n// ^1.2.0 --> >=1.2.0 <2.0.0\nfunction replaceCarets(comp, loose) {\n  return comp.trim().split(/\\s+/).map(function(comp) {\n    return replaceCaret(comp, loose);\n  }).join(' ');\n}\n\nfunction replaceCaret(comp, loose) {\n  var r = loose ? re[CARETLOOSE] : re[CARET];\n  return comp.replace(r, function(_, M, m, p, pr) {\n    ;\n    var ret;\n\n    if (isX(M))\n      ret = '';\n    else if (isX(m))\n      ret = '>=' + M + '.0.0-0 <' + (+M + 1) + '.0.0-0';\n    else if (isX(p))\n      if (M === '0') ret = '>=' + M + '.' + m + '.0-0 <' + M + '.' + (+m + 1) + '.0-0';\n      else ret = '>=' + M + '.' + m + '.0-0 <' + (+M + 1) + '.0.0-0';\n    else if (pr) {\n      ;\n      if (pr.charAt(0) !== '-')\n        pr = '-' + pr;\n      if (M === '0')\n        if (m === '0') ret = '=' + M + '.' + m + '.' + p + pr;\n        else ret = '>=' + M + '.' + m + '.' + p + pr +\n                   ' <' + M + '.' + (+m + 1) + '.0-0';\n      else ret = '>=' + M + '.' + m + '.' + p + pr +\n                 ' <' + (+M + 1) + '.0.0-0';\n    } else\n      if (M === '0')\n        if (m === '0') ret = '=' + M + '.' + m + '.' + p;\n        else ret = '>=' + M + '.' + m + '.' + p + '-0' +\n                   ' <' + M + '.' + (+m + 1) + '.0-0';\n      else ret = '>=' + M + '.' + m + '.' + p + '-0' +\n                 ' <' + (+M + 1) + '.0.0-0';\n\n    ;\n    return ret;\n  });\n}\n\nfunction replaceXRanges(comp, loose) {\n  ;\n  return comp.split(/\\s+/).map(function(comp) {\n    return replaceXRange(comp, loose);\n  }).join(' ');\n}\n\nfunction replaceXRange(comp, loose) {\n  comp = comp.trim();\n  var r = loose ? re[XRANGELOOSE] : re[XRANGE];\n  return comp.replace(r, function(ret, gtlt, M, m, p, pr) {\n    ;\n    var xM = isX(M);\n    var xm = xM || isX(m);\n    var xp = xm || isX(p);\n    var anyX = xp;\n\n    if (gtlt === '=' && anyX)\n      gtlt = '';\n\n    if (gtlt && anyX) {\n      // replace X with 0, and then append the -0 min-prerelease\n      if (xM)\n        M = 0;\n      if (xm)\n        m = 0;\n      if (xp)\n        p = 0;\n\n      if (gtlt === '>') {\n        // >1 => >=2.0.0-0\n        // >1.2 => >=1.3.0-0\n        // >1.2.3 => >= 1.2.4-0\n        gtlt = '>=';\n        if (xM) {\n          // no change\n        } else if (xm) {\n          M = +M + 1;\n          m = 0;\n          p = 0;\n        } else if (xp) {\n          m = +m + 1;\n          p = 0;\n        }\n      }\n\n\n      ret = gtlt + M + '.' + m + '.' + p + '-0';\n    } else if (xM) {\n      // allow any\n      ret = '*';\n    } else if (xm) {\n      // append '-0' onto the version, otherwise\n      // '1.x.x' matches '2.0.0-beta', since the tag\n      // *lowers* the version value\n      ret = '>=' + M + '.0.0-0 <' + (+M + 1) + '.0.0-0';\n    } else if (xp) {\n      ret = '>=' + M + '.' + m + '.0-0 <' + M + '.' + (+m + 1) + '.0-0';\n    }\n\n    ;\n\n    return ret;\n  });\n}\n\n// Because * is AND-ed with everything else in the comparator,\n// and '' means \"any version\", just remove the *s entirely.\nfunction replaceStars(comp, loose) {\n  ;\n  // Looseness is ignored here.  star is always as loose as it gets!\n  return comp.trim().replace(re[STAR], '');\n}\n\n// This function is passed to string.replace(re[HYPHENRANGE])\n// M, m, patch, prerelease, build\n// 1.2 - 3.4.5 => >=1.2.0-0 <=3.4.5\n// 1.2.3 - 3.4 => >=1.2.0-0 <3.5.0-0 Any 3.4.x will do\n// 1.2 - 3.4 => >=1.2.0-0 <3.5.0-0\nfunction hyphenReplace($0,\n                       from, fM, fm, fp, fpr, fb,\n                       to, tM, tm, tp, tpr, tb) {\n\n  if (isX(fM))\n    from = '';\n  else if (isX(fm))\n    from = '>=' + fM + '.0.0-0';\n  else if (isX(fp))\n    from = '>=' + fM + '.' + fm + '.0-0';\n  else\n    from = '>=' + from;\n\n  if (isX(tM))\n    to = '';\n  else if (isX(tm))\n    to = '<' + (+tM + 1) + '.0.0-0';\n  else if (isX(tp))\n    to = '<' + tM + '.' + (+tm + 1) + '.0-0';\n  else if (tpr)\n    to = '<=' + tM + '.' + tm + '.' + tp + '-' + tpr;\n  else\n    to = '<=' + to;\n\n  return (from + ' ' + to).trim();\n}\n\n\n// if ANY of the sets match ALL of its comparators, then pass\nRange.prototype.test = function(version) {\n  if (!version)\n    return false;\n  for (var i = 0; i < this.set.length; i++) {\n    if (testSet(this.set[i], version))\n      return true;\n  }\n  return false;\n};\n\nfunction testSet(set, version) {\n  for (var i = 0; i < set.length; i++) {\n    if (!set[i].test(version))\n      return false;\n  }\n  return true;\n}\n\nexports.satisfies = satisfies;\nfunction satisfies(version, range, loose) {\n  try {\n    range = new Range(range, loose);\n  } catch (er) {\n    return false;\n  }\n  return range.test(version);\n}\n\nexports.maxSatisfying = maxSatisfying;\nfunction maxSatisfying(versions, range, loose) {\n  return versions.filter(function(version) {\n    return satisfies(version, range, loose);\n  }).sort(function(a, b) {\n    return rcompare(a, b, loose);\n  })[0] || null;\n}\n\nexports.validRange = validRange;\nfunction validRange(range, loose) {\n  try {\n    // Return '*' instead of '' so that truthiness works.\n    // This will throw if it's invalid anyway\n    return new Range(range, loose).range || '*';\n  } catch (er) {\n    return null;\n  }\n}\n\n// Use the define() function if we're in AMD land\nif (typeof define === 'function' && define.amd)\n  define('extensibility/node/node_modules/semver/semver.browser',exports);\n\n})(\n  typeof exports === 'object' ? exports :\n  typeof define === 'function' && define.amd ? {} :\n  semver = {}\n);\n\n//# sourceURL=/extensibility/node/node_modules/semver/semver.browser.js"),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */\n/*global define, window, $, brackets, semver */\n/*unittests: ExtensionManager*/\n\n/**\n * The ExtensionManager fetches/caches the extension registry and provides\n * information about the status of installed extensions. ExtensionManager raises the\n * following events:\n *     statusChange - indicates that an extension has been installed/uninstalled or\n *         its status has otherwise changed. Second parameter is the id of the\n *         extension.\n *     registryUpdate - indicates that an existing extension was synchronized\n *         with new data from the registry.\n */\n\ndefine(\'extensibility/ExtensionManager\',[\'require\',\'exports\',\'module\',\'file/FileUtils\',\'extensibility/Package\',\'utils/Async\',\'utils/ExtensionLoader\',\'filesystem/FileSystem\',\'strings\',\'utils/StringUtils\',\'extensibility/node/node_modules/semver/semver.browser\'],function (require, exports, module) {\n    "use strict";\n\n    var FileUtils        = require("file/FileUtils"),\n        Package          = require("extensibility/Package"),\n        Async            = require("utils/Async"),\n        ExtensionLoader  = require("utils/ExtensionLoader"),\n        FileSystem       = require("filesystem/FileSystem"),\n        Strings          = require("strings"),\n        StringUtils      = require("utils/StringUtils");\n\n    // semver.browser is an AMD-compatible module\n    var semver = require("extensibility/node/node_modules/semver/semver.browser");\n\n    /**\n     * Extension status constants.\n     */\n    var ENABLED      = "enabled",\n        START_FAILED = "startFailed";\n\n    /**\n     * Extension location constants.\n     */\n    var LOCATION_DEFAULT = "default",\n        LOCATION_DEV     = "dev",\n        LOCATION_USER    = "user",\n        LOCATION_UNKNOWN = "unknown";\n\n    /**\n     * @private\n     * @type {Object.<string, {metadata: Object, path: string, status: string}>}\n     * The set of all known extensions, both from the registry and locally installed.\n     * The keys are either "name" from package.json (for extensions that have package metadata)\n     * or the last segment of local file paths (for installed legacy extensions\n     * with no package metadata). The fields of each record are:\n     *     registryInfo: object containing the info for this id from the main registry (containing metadata, owner,\n     *         and versions). This will be null for legacy extensions.\n     *     installInfo: object containing the info for a locally-installed extension:\n     *         metadata: the package metadata loaded from the local package.json, or null if it\'s a legacy extension.\n     *             This will be different from registryInfo.metadata if there\'s a newer version in the registry.\n     *         path: the local path to the extension folder on disk\n     *         locationType: general type of installation; one of the LOCATION_* constants above\n     *         status: the current status, one of the status constants above\n     */\n    var extensions = {};\n\n    /**\n     * Requested changes to the installed extensions.\n     */\n    var _idsToRemove = [],\n        _idsToUpdate = [];\n\n    /**\n     * @private\n     * Synchronizes the information between the public registry and the installed\n     * extensions. Specifically, this makes the `owner` available in each and sets\n     * an `updateAvailable` flag.\n     *\n     * @param {string} id of the extension to synchronize\n     */\n    function synchronizeEntry(id) {\n        var entry = extensions[id];\n\n        // Do nothing if we only have one set of data\n        if (!entry || !entry.installInfo || !entry.registryInfo) {\n            return;\n        }\n\n        entry.installInfo.owner = entry.registryInfo.owner;\n        if (entry.installInfo.metadata && entry.installInfo.metadata.version && semver.lt(entry.installInfo.metadata.version, entry.registryInfo.metadata.version)) {\n            // Note: available update may still be incompatible; we check for this when rendering the Update button in ExtensionManagerView._renderItem()\n            entry.registryInfo.updateAvailable = true;\n            entry.installInfo.updateAvailable = true;\n        } else {\n            entry.installInfo.updateAvailable = false;\n            entry.registryInfo.updateAvailable = false;\n        }\n\n        $(exports).triggerHandler("registryUpdate", [id]);\n    }\n\n    /**\n     * @private\n     * Sets our data. For unit testing only.\n     */\n    function _setExtensions(newExtensions) {\n        exports.extensions = extensions = newExtensions;\n        Object.keys(extensions).forEach(function (id) {\n            synchronizeEntry(id);\n        });\n    }\n\n    /**\n     * @private\n     * Clears out our existing data. For unit testing only.\n     */\n    function _reset() {\n        exports.extensions = extensions = {};\n        _idsToRemove = [];\n        _idsToUpdate = [];\n    }\n\n    /**\n     * Downloads the registry of Brackets extensions and stores the information in our\n     * extension info.\n     *\n     * @return {$.Promise} a promise that\'s resolved with the registry JSON data\n     * or rejected if the server can\'t be reached.\n     */\n    function downloadRegistry() {\n        var result = new $.Deferred();\n        $.ajax({\n            url: app.config.extension_registry,\n            dataType: "json",\n            cache: false\n        })\n            .done(function (data) {\n                Object.keys(data).forEach(function (id) {\n                    if (!extensions[id]) {\n                        extensions[id] = {};\n                    }\n                    extensions[id].registryInfo = data[id];\n                    synchronizeEntry(id);\n                });\n                result.resolve();\n            })\n            .fail(function () {\n                result.reject();\n            });\n        return result.promise();\n    }\n\n    /**\n     * @private\n     * Loads the package.json file in the given extension folder.\n     * @param {string} folder The extension folder.\n     * @return {$.Promise} A promise object that is resolved with the parsed contents of the package.json file,\n     *     or rejected if there is no package.json or the contents are not valid JSON.\n     */\n    function _loadPackageJson(folder) {\n        var file = FileSystem.getFileForPath(folder + "/package.json"),\n            result = new $.Deferred();\n        FileUtils.readAsText(file)\n            .done(function (text) {\n                try {\n                    var json = JSON.parse(text);\n                    result.resolve(json);\n                } catch (e) {\n                    result.reject();\n                }\n            })\n            .fail(function () {\n                result.reject();\n            });\n        return result.promise();\n    }\n\n    /**\n     * @private\n     * When an extension is loaded, fetches the package.json and stores the extension in our map.\n     * @param {$.Event} e The event object\n     * @param {string} path The local path of the loaded extension\'s folder.\n     */\n    function _handleExtensionLoad(e, path) {\n        function setData(id, metadata) {\n            var locationType,\n                userExtensionPath = ExtensionLoader.getUserExtensionPath();\n            if (path.indexOf(userExtensionPath) === 0) {\n                locationType = LOCATION_USER;\n            } else {\n                var segments = path.split("/"), parent;\n                if (segments.length > 2) {\n                    parent = segments[segments.length - 2];\n                }\n                if (parent === "dev") {\n                    locationType = LOCATION_DEV;\n                } else if (parent === "default") {\n                    locationType = LOCATION_DEFAULT;\n                } else {\n                    locationType = LOCATION_UNKNOWN;\n                }\n            }\n            if (!extensions[id]) {\n                extensions[id] = {};\n            }\n            extensions[id].installInfo = {\n                metadata: metadata,\n                path: path,\n                locationType: locationType,\n                status: (e.type === "loadFailed" ? START_FAILED : ENABLED)\n            };\n            synchronizeEntry(id);\n            $(exports).triggerHandler("statusChange", [id]);\n        }\n\n        _loadPackageJson(path)\n            .done(function (metadata) {\n                setData(metadata.name, metadata);\n            })\n            .fail(function () {\n                // If there\'s no package.json, this is a legacy extension. It was successfully loaded,\n                // but we don\'t have an official ID or metadata for it, so we just create an id and\n                // "title" for it (which is the last segment of its pathname)\n                // and record that it\'s enabled.\n                var match = path.match(/\\/([^\\/]+)$/),\n                    name = (match && match[1]) || path,\n                    metadata = { name: name, title: name };\n                setData(name, metadata);\n            });\n    }\n\n    /**\n     * Determines if the given versions[] entry is compatible with the given Brackets API version, and if not\n     * specifies why.\n     * @param {Object} extVersion\n     * @param {string} apiVersion\n     * @return {{isCompatible: boolean, requiresNewer: ?boolean, compatibleVersion: ?string}}\n     */\n    function getCompatibilityInfoForVersion(extVersion, apiVersion) {\n        var requiredVersion = (extVersion.staruml || (extVersion.engines && extVersion.engines.staruml)),\n            result = {};\n        result.isCompatible = !requiredVersion || semver.satisfies(apiVersion, requiredVersion);\n        if (result.isCompatible) {\n            result.compatibleVersion = extVersion.version;\n        } else {\n            // Find out reason for incompatibility\n            if (requiredVersion.charAt(0) === \'<\') {\n                result.requiresNewer = false;\n            } else if (requiredVersion.charAt(0) === \'>\') {\n                result.requiresNewer = true;\n            } else if (requiredVersion.charAt(0) === "~") {\n                var compareVersion = requiredVersion.slice(1);\n                // Need to add .0s to this style of range in order to compare (since valid version\n                // numbers must have major/minor/patch).\n                if (compareVersion.match(/^[0-9]+$/)) {\n                    compareVersion += ".0.0";\n                } else if (compareVersion.match(/^[0-9]+\\.[0-9]+$/)) {\n                    compareVersion += ".0";\n                }\n                result.requiresNewer = semver.lt(apiVersion, compareVersion);\n            }\n        }\n        return result;\n    }\n\n    /**\n     * Finds the newest version of the entry that is compatible with the given Brackets API version, if any.\n     * @param {Object} entry The registry entry to check.\n     * @param {string} apiVersion The Brackets API version to check against.\n     * @return {{isCompatible: boolean, requiresNewer: ?boolean, compatibleVersion: ?string, isLatestVersion: boolean}}\n     *      Result contains an "isCompatible" member saying whether it\'s compatible. If compatible, "compatibleVersion"\n     *      specifies the newest version that is compatible and "isLatestVersion" indicates if this is the absolute\n     *      latest version of the extension or not. If !isCompatible or !isLatestVersion, "requiresNewer" says whether\n     *      the latest version is incompatible due to requiring a newer (vs. older) version of Brackets.\n     */\n    function getCompatibilityInfo(entry, apiVersion) {\n        if (!entry.versions) {\n            var fallback = getCompatibilityInfoForVersion(entry.metadata, apiVersion);\n            if (fallback.isCompatible) {\n                fallback.isLatestVersion = true;\n            }\n            return fallback;\n        }\n\n        var i = entry.versions.length - 1,\n            latestInfo = getCompatibilityInfoForVersion(entry.versions[i], apiVersion);\n\n        if (latestInfo.isCompatible) {\n            latestInfo.isLatestVersion = true;\n            return latestInfo;\n        } else {\n            // Look at earlier versions (skipping very latest version since we already checked it)\n            for (i--; i >= 0; i--) {\n                var compatInfo = getCompatibilityInfoForVersion(entry.versions[i], apiVersion);\n                if (compatInfo.isCompatible) {\n                    compatInfo.isLatestVersion = false;\n                    compatInfo.requiresNewer = latestInfo.requiresNewer;\n                    return compatInfo;\n                }\n            }\n\n            // No version is compatible, so just return info for the latest version\n            return latestInfo;\n        }\n    }\n\n    /**\n     * Given an extension id and version number, returns the URL for downloading that extension from\n     * the repository. Does not guarantee that the extension exists at that URL.\n     * @param {string} id The extension\'s name from the metadata.\n     * @param {string} version The version to download.\n     * @return {string} The URL to download the extension from.\n     */\n    function getExtensionURL(id, version) {\n        return StringUtils.format(app.config.extension_url, id, version);\n    }\n\n    /**\n     * Removes the installed extension with the given id.\n     * @param {string} id The id of the extension to remove.\n     * @return {$.Promise} A promise that\'s resolved when the extension is removed or\n     *     rejected with an error if there\'s a problem with the removal.\n     */\n    function remove(id) {\n        var result = new $.Deferred();\n        if (extensions[id] && extensions[id].installInfo) {\n            Package.remove(extensions[id].installInfo.path)\n                .done(function () {\n                    extensions[id].installInfo = null;\n                    result.resolve();\n                    $(exports).triggerHandler("statusChange", [id]);\n                })\n                .fail(function (err) {\n                    result.reject(err);\n                });\n        } else {\n            result.reject(StringUtils.format(Strings.EXTENSION_NOT_INSTALLED, id));\n        }\n        return result.promise();\n    }\n\n    /**\n     * Updates an installed extension with the given package file.\n     * @param {string} id of the extension\n     * @param {string} packagePath path to the package file\n     * @return {$.Promise} A promise that\'s resolved when the extension is updated or\n     *     rejected with an error if there\'s a problem with the update.\n     */\n    function update(id, packagePath) {\n        return Package.installUpdate(packagePath, id);\n    }\n\n    /**\n     * Deletes any temporary files left behind by extensions that\n     * were marked for update.\n     */\n    function cleanupUpdates() {\n        Object.keys(_idsToUpdate).forEach(function (id) {\n            var filename = _idsToUpdate[id].localPath;\n            if (filename) {\n                FileSystem.getFileForPath(filename).unlink();\n            }\n        });\n        _idsToUpdate = {};\n    }\n\n    /**\n     * Unmarks all extensions marked for removal.\n     */\n    function unmarkAllForRemoval() {\n        _idsToRemove = {};\n    }\n\n    /**\n     * Marks an extension for later removal, or unmarks an extension previously marked.\n     * @param {string} id The id of the extension to mark for removal.\n     * @param {boolean} mark Whether to mark or unmark it.\n     */\n    function markForRemoval(id, mark) {\n        if (mark) {\n            _idsToRemove[id] = true;\n        } else {\n            delete _idsToRemove[id];\n        }\n        $(exports).triggerHandler("statusChange", [id]);\n    }\n\n    /**\n     * Returns true if an extension is marked for removal.\n     * @param {string} id The id of the extension to check.\n     * @return {boolean} true if it\'s been marked for removal, false otherwise.\n     */\n    function isMarkedForRemoval(id) {\n        return !!(_idsToRemove[id]);\n    }\n\n    /**\n     * Returns true if there are any extensions marked for removal.\n     * @return {boolean} true if there are extensions to remove\n     */\n    function hasExtensionsToRemove() {\n        return Object.keys(_idsToRemove).length > 0;\n    }\n\n    /**\n     * If a downloaded package appears to be an update, mark the extension for update.\n     * If an extension was previously marked for removal, marking for update will\n     * turn off the removal mark.\n     * @param {Object} installationResult info about the install provided by the Package.download function\n     */\n    function updateFromDownload(installationResult) {\n        var installationStatus = installationResult.installationStatus;\n        if (installationStatus === Package.InstallationStatuses.ALREADY_INSTALLED ||\n                installationStatus === Package.InstallationStatuses.NEEDS_UPDATE ||\n                installationStatus === Package.InstallationStatuses.SAME_VERSION ||\n                installationStatus === Package.InstallationStatuses.OLDER_VERSION) {\n            var id = installationResult.name;\n            delete _idsToRemove[id];\n            _idsToUpdate[id] = installationResult;\n            $(exports).triggerHandler("statusChange", [id]);\n        }\n    }\n\n    /**\n     * Removes the mark for an extension to be updated on restart. Also deletes the\n     * downloaded package file.\n     * @param {string} id The id of the extension for which the update is being removed\n     */\n    function removeUpdate(id) {\n        var installationResult = _idsToUpdate[id];\n        if (!installationResult) {\n            return;\n        }\n        if (installationResult.localPath) {\n            FileSystem.getFileForPath(installationResult.localPath).unlink();\n        }\n        delete _idsToUpdate[id];\n        $(exports).triggerHandler("statusChange", [id]);\n    }\n\n    /**\n     * Returns true if an extension is marked for update.\n     * @param {string} id The id of the extension to check.\n     * @return {boolean} true if it\'s been marked for update, false otherwise.\n     */\n    function isMarkedForUpdate(id) {\n        return !!(_idsToUpdate[id]);\n    }\n\n    /**\n     * Returns true if there are any extensions marked for update.\n     * @return {boolean} true if there are extensions to update\n     */\n    function hasExtensionsToUpdate() {\n        return Object.keys(_idsToUpdate).length > 0;\n    }\n\n    /**\n     * Removes extensions previously marked for removal.\n     * @return {$.Promise} A promise that\'s resolved when all extensions are removed, or rejected\n     *     if one or more extensions can\'t be removed. When rejected, the argument will be an\n     *     array of error objects, each of which contains an "item" property with the id of the\n     *     failed extension and an "error" property with the actual error.\n     */\n    function removeMarkedExtensions() {\n        return Async.doInParallel_aggregateErrors(\n            Object.keys(_idsToRemove),\n            function (id) {\n                return remove(id);\n            }\n        );\n    }\n\n    /**\n     * Updates extensions previously marked for update.\n     * @return {$.Promise} A promise that\'s resolved when all extensions are updated, or rejected\n     *     if one or more extensions can\'t be updated. When rejected, the argument will be an\n     *     array of error objects, each of which contains an "item" property with the id of the\n     *     failed extension and an "error" property with the actual error.\n     */\n    function updateExtensions() {\n        return Async.doInParallel_aggregateErrors(\n            Object.keys(_idsToUpdate),\n            function (id) {\n                var installationResult = _idsToUpdate[id];\n                return update(installationResult.name, installationResult.localPath);\n            }\n        );\n    }\n\n    // Listen to extension load and loadFailed events\n    $(ExtensionLoader)\n        .on("load", _handleExtensionLoad)\n        .on("loadFailed", _handleExtensionLoad);\n\n    // Public exports\n    exports.downloadRegistry = downloadRegistry;\n    exports.getCompatibilityInfo = getCompatibilityInfo;\n    exports.getExtensionURL = getExtensionURL;\n    exports.remove = remove;\n    exports.update = update;\n    exports.extensions = extensions;\n    exports.cleanupUpdates = cleanupUpdates;\n    exports.markForRemoval = markForRemoval;\n    exports.isMarkedForRemoval = isMarkedForRemoval;\n    exports.unmarkAllForRemoval = unmarkAllForRemoval;\n    exports.hasExtensionsToRemove = hasExtensionsToRemove;\n    exports.updateFromDownload = updateFromDownload;\n    exports.removeUpdate = removeUpdate;\n    exports.isMarkedForUpdate = isMarkedForUpdate;\n    exports.hasExtensionsToUpdate = hasExtensionsToUpdate;\n    exports.removeMarkedExtensions = removeMarkedExtensions;\n    exports.updateExtensions = updateExtensions;\n\n    exports.ENABLED = ENABLED;\n    exports.START_FAILED = START_FAILED;\n\n    exports.LOCATION_DEFAULT = LOCATION_DEFAULT;\n    exports.LOCATION_DEV = LOCATION_DEV;\n    exports.LOCATION_USER = LOCATION_USER;\n    exports.LOCATION_UNKNOWN = LOCATION_UNKNOWN;\n\n    // For unit testing only\n    exports._reset = _reset;\n    exports._setExtensions = _setExtensions;\n});\n\n//# sourceURL=/extensibility/ExtensionManager.js'),eval('define(\'text!htmlContent/install-extension-dialog.html\',[],function () { return \'<div class="install-extension-dialog dialog modal" data-title="{{title}}">\\n    <div class="dialog-body">\\n        <div class="field-container input-field">\\n            <input type="text" placeholder="{{INSTALL_EXTENSION_HINT}}" class="k-input k-textbox url" />\\n        </div>\\n        <div class="field-container message-field">\\n            <div class="message"></div>\\n        </div>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n        <button class="k-button dialog-button primary" data-button-id="ok" disabled>{{Strings.INSTALL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/install-extension-dialog.html'),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, window, $, brackets, Mustache, document */\n/*unittests: Install Extension Dialog*/\n\ndefine(\'extensibility/InstallExtensionDialog\',[\'require\',\'exports\',\'module\',\'dialogs/Dialogs\',\'utils/StringUtils\',\'strings\',\'filesystem/FileSystem\',\'utils/KeyEvent\',\'extensibility/Package\',\'text!htmlContent/install-extension-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var Dialogs                = require("dialogs/Dialogs"),\n        StringUtils            = require("utils/StringUtils"),\n        Strings                = require("strings"),\n        FileSystem             = require("filesystem/FileSystem"),\n        KeyEvent               = require("utils/KeyEvent"),\n        Package                = require("extensibility/Package"),\n        InstallDialogTemplate  = require("text!htmlContent/install-extension-dialog.html");\n\n    var STATE_CLOSED              = 0,\n        STATE_START               = 1,\n        STATE_VALID_URL           = 2,\n        STATE_INSTALLING          = 3,\n        STATE_INSTALLED           = 4,\n        STATE_INSTALL_FAILED      = 5,\n        STATE_CANCELING_INSTALL   = 6,\n        STATE_CANCELING_HUNG      = 7,\n        STATE_INSTALL_CANCELED    = 8,\n        STATE_ALREADY_INSTALLED   = 9,\n        STATE_OVERWRITE_CONFIRMED = 10,\n        STATE_NEEDS_UPDATE        = 11;\n\n    /**\n     * @constructor\n     * Creates a new extension installer dialog.\n     * @param {{install: function(url), cancel: function()}} installer The installer backend to use.\n     */\n    function InstallExtensionDialog(installer, _isUpdate) {\n        this._installer = installer;\n        this._state = STATE_CLOSED;\n        this._installResult = null;\n        this._isUpdate = _isUpdate;\n\n        // Timeout before we allow user to leave STATE_INSTALL_CANCELING without waiting for a resolution\n        // (per-instance so we can poke it for unit testing)\n        this._cancelTimeout = 10 * 1000;\n    }\n\n    /** @type {jQuery} The dialog root. */\n    InstallExtensionDialog.prototype.$dlg = null;\n\n    /** @type {jQuery} The url input field. */\n    InstallExtensionDialog.prototype.$url = null;\n\n    /** @type {jQuery} The ok button. */\n    InstallExtensionDialog.prototype.$okButton = null;\n\n    /** @type {jQuery} The cancel button. */\n    InstallExtensionDialog.prototype.$cancelButton = null;\n\n    /** @type {jQuery} The area containing the url input label and field. */\n    InstallExtensionDialog.prototype.$inputArea = null;\n\n    /** @type {jQuery} The area containing the installation message and spinner. */\n    InstallExtensionDialog.prototype.$msgArea = null;\n\n    /** @type {jQuery} The span containing the installation message. */\n    InstallExtensionDialog.prototype.$msg = null;\n\n    /** @type {$.Deferred} A deferred that\'s resolved/rejected when the dialog is closed and\n        something has/hasn\'t been installed successfully. */\n    InstallExtensionDialog.prototype._dialogDeferred = null;\n\n\n    /** @type {{install: function(url), cancel: function()}} installer The installer backend for this dialog. */\n    InstallExtensionDialog.prototype._installer = null;\n\n    /** @type {number} The current state of the dialog; one of the STATE_* constants above. */\n    InstallExtensionDialog.prototype._state = null;\n\n    /**\n     * @private\n     * Transitions the dialog into a new state as the installation proceeds.\n     * @param {number} newState The state to transition into; one of the STATE_* variables.\n     */\n    InstallExtensionDialog.prototype._enterState = function (newState) {\n        var url,\n            msg,\n            self = this,\n            prevState = this._state;\n\n        // Store the new state up front in case some of the processing below ends up changing\n        // the state again immediately.\n        this._state = newState;\n\n        switch (newState) {\n        case STATE_START:\n            // This should match the default appearance of the dialog when it first opens.\n            this.$msg.find(".spinner").remove();\n            this.$msgArea.hide();\n            this.$inputArea.show();\n            this.$okButton\n                .prop("disabled", true)\n                .text(Strings.INSTALL);\n            break;\n\n        case STATE_VALID_URL:\n            this.$okButton.prop("disabled", false);\n            break;\n\n        case STATE_INSTALLING:\n            url = this.$url.val().trim();\n            this.$inputArea.hide();\n            this.$msg.text(StringUtils.format(Strings.INSTALLING_FROM, url))\n                .append("<div class=\'spinner\'/>");\n            this.$msgArea.show();\n            this.$okButton.prop("disabled", true);\n            this._installer.install(url)\n                .done(function (result) {\n                    self._installResult = result;\n                    if (result.installationStatus === Package.InstallationStatuses.ALREADY_INSTALLED ||\n                            result.installationStatus === Package.InstallationStatuses.OLDER_VERSION ||\n                            result.installationStatus === Package.InstallationStatuses.SAME_VERSION) {\n                        self._enterState(STATE_ALREADY_INSTALLED);\n                    } else if (result.installationStatus === Package.InstallationStatuses.NEEDS_UPDATE) {\n                        self._enterState(STATE_NEEDS_UPDATE);\n                    } else {\n                        self._enterState(STATE_INSTALLED);\n                    }\n                })\n                .fail(function (err) {\n                    // If the "failure" is actually a user-requested cancel, don\'t show an error UI\n                    if (err === "CANCELED") {\n                        console.assert(self._state === STATE_CANCELING_INSTALL || self._state === STATE_CANCELING_HUNG);\n                        self._enterState(STATE_INSTALL_CANCELED);\n                    } else {\n                        self._errorMessage = Package.formatError(err);\n                        self._enterState(STATE_INSTALL_FAILED);\n                    }\n                });\n            break;\n\n        case STATE_CANCELING_INSTALL:\n            // This should call back the STATE_INSTALLING fail() handler above, unless it\'s too late to cancel\n            // in which case we\'ll still jump to STATE_INSTALLED after this\n            this.$cancelButton.prop("disabled", true);\n            this.$msg.text(Strings.CANCELING_INSTALL);\n            this._installer.cancel();\n            window.setTimeout(function () {\n                if (self._state === STATE_CANCELING_INSTALL) {\n                    self._enterState(STATE_CANCELING_HUNG);\n                }\n            }, this._cancelTimeout);\n            break;\n\n        case STATE_CANCELING_HUNG:\n            this.$msg.text(Strings.CANCELING_HUNG);\n            this.$okButton\n                .removeAttr("disabled")\n                .text(Strings.CLOSE);\n            break;\n\n        case STATE_INSTALLED:\n        case STATE_INSTALL_FAILED:\n        case STATE_INSTALL_CANCELED:\n        case STATE_NEEDS_UPDATE:\n            if (newState === STATE_INSTALLED) {\n                msg = Strings.INSTALL_SUCCEEDED;\n            } else if (newState === STATE_INSTALL_FAILED) {\n                msg = Strings.INSTALL_FAILED;\n            } else if (newState === STATE_NEEDS_UPDATE) {\n                msg = Strings.EXTENSION_UPDATE_INSTALLED;\n            } else {\n                msg = Strings.INSTALL_CANCELED;\n            }\n            this.$msg.html($("<strong/>").text(msg));\n            if (this._errorMessage) {\n                this.$msg.append($("<p/>").text(this._errorMessage));\n            }\n            this.$okButton\n                .removeAttr("disabled")\n                .text(Strings.CLOSE);\n            this.$cancelButton.hide();\n            break;\n\n        case STATE_ALREADY_INSTALLED:\n            var installResult = this._installResult;\n            var status = installResult.installationStatus;\n            var msgText = Strings["EXTENSION_" + status];\n            if (status === Package.InstallationStatuses.OLDER_VERSION) {\n                msgText = StringUtils.format(msgText, installResult.metadata.version, installResult.installedVersion);\n            }\n            this.$msg.text(msgText);\n            this.$okButton\n                .prop("disabled", false)\n                .text(Strings.OVERWRITE);\n            break;\n\n        case STATE_OVERWRITE_CONFIRMED:\n            this._enterState(STATE_CLOSED);\n            break;\n\n        case STATE_CLOSED:\n            $(document.body).off(".installDialog");\n\n           // Only resolve as successful if we actually installed something.\n            Dialogs.cancelModalDialogIfOpen("install-extension-dialog");\n            if (prevState === STATE_INSTALLED || prevState === STATE_NEEDS_UPDATE ||\n                    prevState === STATE_OVERWRITE_CONFIRMED) {\n                this._dialogDeferred.resolve(this._installResult);\n            } else {\n                this._dialogDeferred.reject();\n            }\n            break;\n        }\n    };\n\n    /**\n     * @private\n     * Handle a click on the Cancel button, which either cancels an ongoing installation (leaving\n     * the dialog open), or closes the dialog if no installation is in progress.\n     */\n    InstallExtensionDialog.prototype._handleCancel = function () {\n        if (this._state === STATE_INSTALLING) {\n            this._enterState(STATE_CANCELING_INSTALL);\n        } else if (this._state === STATE_ALREADY_INSTALLED) {\n            // If we were prompting the user about overwriting a previous installation,\n            // and the user cancels, we can delete the downloaded file.\n            if (this._installResult && this._installResult.localPath) {\n                var filename = this._installResult.localPath;\n                FileSystem.getFileForPath(filename).unlink();\n            }\n            this._enterState(STATE_CLOSED);\n        } else if (this._state !== STATE_CANCELING_INSTALL) {\n            this._enterState(STATE_CLOSED);\n        }\n    };\n\n    /**\n     * @private\n     * Handle a click on the default button, which is "Install" while we\'re waiting for the\n     * user to enter a URL, and "Close" once we\'ve successfully finished installation.\n     */\n    InstallExtensionDialog.prototype._handleOk = function () {\n        if (this._state === STATE_INSTALLED ||\n                this._state === STATE_INSTALL_FAILED ||\n                this._state === STATE_INSTALL_CANCELED ||\n                this._state === STATE_CANCELING_HUNG ||\n                this._state === STATE_NEEDS_UPDATE) {\n            // In these end states, this is a "Close" button: just close the dialog and indicate\n            // success.\n            this._enterState(STATE_CLOSED);\n        } else if (this._state === STATE_VALID_URL) {\n            this._enterState(STATE_INSTALLING);\n        } else if (this._state === STATE_ALREADY_INSTALLED) {\n            this._enterState(STATE_OVERWRITE_CONFIRMED);\n        }\n    };\n\n    /**\n     * @private\n     * Handle key up events on the document. We use this to detect the Esc key.\n     */\n    InstallExtensionDialog.prototype._handleKeyUp = function (e) {\n        if (e.keyCode === KeyEvent.DOM_VK_ESCAPE) {\n            this._handleCancel();\n        }\n    };\n\n    /**\n     * @private\n     * Handle typing in the URL field.\n     */\n    InstallExtensionDialog.prototype._handleUrlInput = function (e) {\n        var url     = this.$url.val().trim(),\n            valid   = (url !== "");\n        if (!valid && this._state === STATE_VALID_URL) {\n            this._enterState(STATE_START);\n        } else if (valid && this._state === STATE_START) {\n            this._enterState(STATE_VALID_URL);\n        }\n    };\n\n    /**\n     * @private\n     * Closes the dialog if it\'s not already closed. For unit testing only.\n     */\n    InstallExtensionDialog.prototype._close = function () {\n        if (this._state !== STATE_CLOSED) {\n            this._enterState(STATE_CLOSED);\n        }\n    };\n\n    /**\n     * Initialize and show the dialog.\n     * @param {string=} urlToInstall If specified, immediately starts installing the given file as if the user had\n     *     specified it.\n     * @return {$.Promise} A promise object that will be resolved when the selected extension\n     *     has finished installing, or rejected if the dialog is cancelled.\n     */\n    InstallExtensionDialog.prototype.show = function (urlToInstall) {\n        if (this._state !== STATE_CLOSED) {\n            // Somehow the dialog got invoked twice. Just ignore this.\n            return this._dialogDeferred.promise();\n        }\n\n        var context = {\n            Strings: Strings,\n            title: this._isUpdate ? Strings.UPDATE_EXTENSION_TITLE : Strings.INSTALL_EXTENSION_TITLE,\n            includeBrowseExtensions: !!app.config.extension_listing_url\n        };\n\n        // We ignore the promise returned by showModalDialogUsingTemplate, since we\'re managing the\n        // lifecycle of the dialog ourselves.\n        Dialogs.showModalDialogUsingTemplate(Mustache.render(InstallDialogTemplate, context), false);\n\n        this.$dlg          = $(".install-extension-dialog.instance");\n        this.$url          = this.$dlg.find(".url").focus();\n        this.$okButton     = this.$dlg.find(".dialog-button[data-button-id=\'ok\']");\n        this.$cancelButton = this.$dlg.find(".dialog-button[data-button-id=\'cancel\']");\n        this.$inputArea    = this.$dlg.find(".input-field");\n        this.$msgArea      = this.$dlg.find(".message-field");\n        this.$msg          = this.$msgArea.find(".message");\n\n        this.$okButton.on("click", this._handleOk.bind(this));\n        this.$cancelButton.on("click", this._handleCancel.bind(this));\n        this.$url.on("input", this._handleUrlInput.bind(this));\n        $(document.body).on("keyup.installDialog", this._handleKeyUp.bind(this));\n\n        this._enterState(STATE_START);\n        if (urlToInstall) {\n            // Act as if the user had manually entered the URL.\n            this.$url.val(urlToInstall);\n            this._enterState(STATE_VALID_URL);\n            this._enterState(STATE_INSTALLING);\n        }\n\n        this._dialogDeferred = new $.Deferred();\n        return this._dialogDeferred.promise();\n    };\n\n\n    /** Mediates between this module and the Package extension-installation utils. Mockable for unit-testing. */\n    function InstallerFacade() { }\n    InstallerFacade.prototype.install = function (url) {\n        if (this.pendingInstall) {\n            console.error("Extension installation already pending");\n            return new $.Deferred().reject("DOWNLOAD_ID_IN_USE").promise();\n        }\n        this.pendingInstall = Package.installFromURL(url);\n\n        // Store now since we\'ll null pendingInstall immediately if the promise was resolved synchronously\n        var promise = this.pendingInstall.promise;\n\n        var self = this;\n        this.pendingInstall.promise.always(function () {\n            self.pendingInstall = null;\n        });\n\n        return promise;\n    };\n    InstallerFacade.prototype.cancel = function () {\n        this.pendingInstall.cancel();\n    };\n\n    /**\n     * @private\n     * Show a dialog that allows the user to enter the URL of an extension ZIP file to install.\n     * @return {$.Promise} A promise object that will be resolved when the selected extension\n     *     has finished installing, or rejected if the dialog is cancelled.\n     */\n    function showDialog() {\n        var dlg = new InstallExtensionDialog(new InstallerFacade());\n        return dlg.show();\n    }\n\n    /**\n     * @private\n     * Show the installation dialog and automatically begin installing the given URL.\n     * @param {string=} urlToInstall If specified, immediately starts installing the given file as if the user had\n     *     specified it.\n     * @return {$.Promise} A promise object that will be resolved when the selected extension\n     *     has finished installing, or rejected if the dialog is cancelled.\n     */\n    function installUsingDialog(urlToInstall, _isUpdate) {\n        var dlg = new InstallExtensionDialog(new InstallerFacade(), _isUpdate);\n        return dlg.show(urlToInstall);\n    }\n\n    /**\n     * @private\n     * Show the update dialog and automatically begin downloading the update from the given URL.\n     * @param {string} urlToUpdate URL to download\n     * @return {$.Promise} A promise object that will be resolved when the selected extension\n     *     has finished downloading, or rejected if the dialog is cancelled.\n     */\n    function updateUsingDialog(urlToUpdate) {\n        return installUsingDialog(urlToUpdate, true);\n    }\n\n    exports.showDialog          = showDialog;\n    exports.installUsingDialog  = installUsingDialog;\n    exports.updateUsingDialog   = updateUsingDialog;\n\n    // Exposed for unit testing only\n    exports._Dialog = InstallExtensionDialog;\n});\n\n//# sourceURL=/extensibility/InstallExtensionDialog.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n/*\n * N.B.: This file was copied from `lib/registry_utils.js` in `https://github.com/adobe/brackets-registry`.\n * We can\'t use the exact same file currently because Brackets uses AMD-style modules, so this version has\n * the AMD wrapper added (and is reindented to avoid JSLint complaints).. If changes are made here, the \n * version in the registry app should be kept in sync.\n * In the future, we should have a better mechanism for sharing code between the two.\n */\n\n/*jslint vars: true, plusplus: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define*/\n\ndefine(\'extensibility/registry_utils\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n    \n    // From Brackets StringUtils\n    function htmlEscape(str) {\n        return String(str)\n            .replace(/&/g, "&amp;")\n            .replace(/"/g, "&quot;")\n            .replace(/\'/g, "&#39;")\n            .replace(/</g, "&lt;")\n            .replace(/>/g, "&gt;");\n    }\n    \n    /**\n     * Gets the last version from the given object and returns the short form of its date.\n     * Assumes "this" is the current template context.\n     * @return {string} The formatted date.\n     */\n    exports.lastVersionDate = function () {\n        var result;\n        if (this.versions && this.versions.length) {\n            result = this.versions[this.versions.length - 1].published;\n            if (result) {\n                // Just return the ISO-formatted date, which is the portion up to the "T".\n                var dateEnd = result.indexOf("T");\n                if (dateEnd !== -1) {\n                    result = result.substr(0, dateEnd);\n                }\n            }\n        }\n        return result || "";\n    };\n    \n    /**\n     * Returns a more friendly display form of the owner\'s internal user id.\n     * Assumes "this" is the current template context.\n     * @return {string} A display version in the form "id (service)".\n     */\n    exports.formatUserId = function () {\n        var friendlyName;\n        if (this.owner) {\n            var nameComponents = this.owner.split(":");\n            friendlyName = nameComponents[1];\n        }\n        return friendlyName;\n    };\n    \n    /**\n     * Given a registry item, returns a URL that represents its owner\'s page on the auth service.\n     * Currently only handles GitHub.\n     * Assumes "this" is the current template context.\n     * @return {string} A link to that user\'s page on the service.\n     */\n    exports.ownerLink = function () {\n        var url;\n        if (this.owner) {\n            var nameComponents = this.owner.split(":");\n            if (nameComponents[0] === "github") {\n                url = "https://github.com/" + nameComponents[1];\n            }\n        }\n        return url;\n    };\n\n    /**\n     * Given a registry item, formats the author information, including a link to the owner\'s\n     * github page (if available) and the author\'s name from the metadata.\n     */\n    exports.authorInfo = function () {\n        var result = "",\n            ownerLink = exports.ownerLink.call(this),\n            userId = exports.formatUserId.call(this);\n        if (this.metadata && this.metadata.author) {\n            result = htmlEscape(this.metadata.author.name || this.metadata.author);\n        } else if (userId) {\n            result = htmlEscape(userId);\n        }\n        if (ownerLink) {\n            result = "<a href=\'" + htmlEscape(ownerLink) + "\' title=\'" + htmlEscape(ownerLink) + "\'>" + result + "</a>";\n        }\n        return result;\n    };\n\n    /**\n     * Returns an array of current registry entries, sorted by the publish date of the latest version of each entry.\n     * @param {object} registry The unsorted registry.\n     * @param {string} subkey The subkey to look for the registry metadata in. If unspecified, assumes\n     *     we should look at the top level of the object.\n     * @return {Array} Sorted array of registry entries.\n     */\n    exports.sortRegistry = function (registry, subkey) {\n        function getPublishTime(entry) {\n            if (entry.versions) {\n                return new Date(entry.versions[entry.versions.length - 1].published).getTime();\n            }\n            \n            return Number.NEGATIVE_INFINITY;\n        }\n        \n        var sortedEntries = [];\n    \n        // Sort the registry by last published date (newest first).\n        Object.keys(registry).forEach(function (key) {\n            sortedEntries.push(registry[key]);\n        });\n        sortedEntries.sort(function (entry1, entry2) {\n            return getPublishTime((subkey && entry2[subkey]) || entry2) -\n                getPublishTime((subkey && entry1[subkey]) || entry1);\n        });\n        \n        return sortedEntries;\n    };\n});\n//# sourceURL=/extensibility/registry_utils.js'),eval('define(\'text!htmlContent/extension-manager-view-item.html\',[],function () { return \'<tr>\\n    <td class="ext-info">\\n        <span class="ext-name">{{#metadata.title}}{{metadata.title}}{{/metadata.title}}{{^metadata.title}}{{metadata.name}}{{/metadata.title}}</span>\\n        <span class="muted ext-version">{{metadata.version}}</span>\\n        {{#authorInfo}}\\n        <span class="muted ext-author-name">{{{authorInfo}}}</span>\\n        {{/authorInfo}}\\n        {{#hasVersionInfo}}\\n            <span class="muted ext-date"> &mdash; {{lastVersionDate}}</span>\\n        {{/hasVersionInfo}}\\n    </td>\\n    <td class="ext-desc">\\n        {{#showInstallButton}}\\n            {{^isCompatible}}\\n                <div class="alert warning">\\n                    {{#requiresNewer}}{{Strings.EXTENSION_INCOMPATIBLE_NEWER}}{{/requiresNewer}}\\n                    {{^requiresNewer}}{{Strings.EXTENSION_INCOMPATIBLE_OLDER}}{{/requiresNewer}}\\n                </div>\\n            {{/isCompatible}}\\n            {{#isCompatible}}\\n                {{^isCompatibleLatest}}\\n                    <div class="alert warning">\\n                        {{installWarning}}\\n                    </div>\\n                {{/isCompatibleLatest}}\\n            {{/isCompatible}}\\n        {{/showInstallButton}}\\n        {{metadata.description}}\\n        {{^metadata.description}}\\n            <p class="muted"><em>{{Strings.EXTENSION_NO_DESCRIPTION}}</em></p>\\n        {{/metadata.description}}\\n        <div>\\n            {{#metadata.issues}}\\n            <a title="{{metadata.issues}}" href="{{metadata.issues}}">{{Strings.EXTENSION_ISSUES}}</a>&nbsp;&nbsp;\\n            {{/metadata.issues}}\\n            {{#metadata.homepage}}\\n            <a title="{{metadata.homepage}}" href="{{metadata.homepage}}">{{Strings.EXTENSION_MORE_INFO}}</a>\\n            {{/metadata.homepage}}\\n        </div>\\n        {{#metadata.keywords.length}}\\n            <br/>\\n            <span class="ext-keywords">{{Strings.EXTENSION_KEYWORDS}}:\\n            {{#metadata.keywords}}\\n                {{.}}\\n            {{/metadata.keywords}}\\n            </span>\\n        {{/metadata.keywords.length}}\\n    </td>\\n    <!--\\n    <td class="ext-author">\\n        {{#authorImageUrl}}\\n        <img class="ext-author-image" src="{{authorImageUrl}}">\\n        {{/authorImageUrl}}\\n        {{^authorImageUrl}}\\n        <div>Author:</div>\\n        {{/authorImageUrl}}\\n    </td>\\n    -->\\n    <td class="ext-action">\\n        {{#showInstallButton}}\\n            <button class="k-button install" data-extension-id="{{metadata.name}}" {{^allowInstall}}disabled{{/allowInstall}}>\\n                {{^isInstalled}}{{Strings.INSTALL}}{{/isInstalled}}\\n                {{#isInstalled}}{{Strings.EXTENSION_INSTALLED}}{{/isInstalled}}\\n            </button>\\n        {{/showInstallButton}}\\n        {{#isInstalled}}\\n            {{#showUpdateButton}}\\n                <button class="k-button update" data-extension-id="{{metadata.name}}" {{^allowUpdate}}disabled title="{{updateNotAllowedReason}}"{{/allowUpdate}}>\\n                    {{Strings.UPDATE}}\\n                </button>\\n            {{/showUpdateButton}}\\n            {{#removalAllowed}}\\n                <button class="k-button remove" data-extension-id="{{metadata.name}}" {{^allowRemove}}disabled title="{{Strings.CANT_REMOVE_DEV}}"{{/allowRemove}}>\\n                    {{Strings.REMOVE}}\\n                </button>\\n            {{/removalAllowed}}\\n            {{#failedToStart}}\\n                {{^isMarkedForRemoval}}\\n                    {{Strings.EXTENSION_ERROR}} {{#allowRemove}}(<a class="remove" data-extension-id="{{metadata.name}}">{{Strings.REMOVE}}</a>){{/allowRemove}}\\n                {{/isMarkedForRemoval}}\\n            {{/failedToStart}}\\n            {{#isMarkedForRemoval}}\\n                {{Strings.MARKED_FOR_REMOVAL}} (<a class="undo-remove" data-extension-id="{{metadata.name}}" href="#">{{Strings.UNDO_REMOVE}}</a>)\\n            {{/isMarkedForRemoval}}\\n            {{#isMarkedForUpdate}}\\n                {{Strings.MARKED_FOR_UPDATE}} (<a class="undo-update" data-extension-id="{{metadata.name}}" href="#">{{Strings.UNDO_UPDATE}}</a>)\\n            {{/isMarkedForUpdate}}\\n        {{/isInstalled}}\\n    </td>\\n</tr>\\n\';});\n\n//# sourceURL=/text!htmlContent/extension-manager-view-item.html'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, window, $, brackets, Mustache, app, md5 */\n/*unittests: ExtensionManager*/\n\ndefine(\'extensibility/ExtensionManagerView\',[\'require\',\'exports\',\'module\',\'strings\',\'utils/StringUtils\',\'utils/Helper\',\'extensibility/ExtensionManager\',\'extensibility/registry_utils\',\'extensibility/InstallExtensionDialog\',\'text!htmlContent/extension-manager-view-item.html\'],function (require, exports, module) {\n    "use strict";\n\n    var Strings                   = require("strings"),\n        StringUtils               = require("utils/StringUtils"),\n        Helper                    = require("utils/Helper"),\n        ExtensionManager          = require("extensibility/ExtensionManager"),\n        registry_utils            = require("extensibility/registry_utils"),\n        InstallExtensionDialog    = require("extensibility/InstallExtensionDialog"),\n        itemTemplate              = require("text!htmlContent/extension-manager-view-item.html");\n\n    /**\n     * @constructor\n     * Creates a view enabling the user to install and manage extensions. Must be initialized\n     * with initialize(). When the view is closed, dispose() must be called.\n     */\n    function ExtensionManagerView() {\n    }\n\n    /**\n     * Initializes the view to show a set of extensions.\n     * @param {ExtensionManagerViewModel} model Model object containing extension data to view\n     * @return {$.Promise} a promise that\'s resolved once the view has been initialized. Never\n     *     rejected.\n     */\n    ExtensionManagerView.prototype.initialize = function (model) {\n        var self = this,\n            result = new $.Deferred();\n        this.model = model;\n        this._itemTemplate = itemTemplate; /*Mustache.compile(itemTemplate);*/\n        this._itemViews = {};\n        this.$el = $("<div class=\'extension-list\' id=\'" + this.model.source + "\'/>");\n        this._$emptyMessage = $("<div class=\'empty-message\'/>")\n            .appendTo(this.$el);\n        this._$infoMessage = $("<div class=\'info-message\'/>")\n            .appendTo(this.$el).html(this.model.infoMessage);\n        this._$table = $("<table class=\'table\'/>").appendTo(this.$el);\n\n        this.model.initialize().done(function () {\n            self._setupEventHandlers();\n        }).always(function () {\n            self._render();\n            result.resolve();\n        });\n\n        return result.promise();\n    };\n\n    /**\n     * @type {jQueryObject}\n     * The root of the view\'s DOM tree.\n     */\n    ExtensionManagerView.prototype.$el = null;\n\n    /**\n     * @type {Model}\n     * The view\'s model. Handles sorting and filtering of items in the view.\n     */\n    ExtensionManagerView.prototype.model = null;\n\n    /**\n     * @type {jQueryObject}\n     * Element showing a message when there are no extensions.\n     */\n    ExtensionManagerView.prototype._$emptyMessage = null;\n\n    /**\n     * @private\n     * @type {jQueryObject}\n     * The root of the table inside the view.\n     */\n    ExtensionManagerView.prototype._$table = null;\n\n    /**\n     * @private\n     * @type {function} The compiled template we use for rendering items in the extension list.\n     */\n    ExtensionManagerView.prototype._itemTemplate = null;\n\n    /**\n     * @private\n     * @type {Object.<string, jQueryObject>}\n     * The individual views for each item, keyed by the extension ID.\n     */\n    ExtensionManagerView.prototype._itemViews = null;\n\n    /**\n     * @private\n     * Attaches our event handlers. We wait to do this until we\'ve fully fetched the extension list.\n     */\n    ExtensionManagerView.prototype._setupEventHandlers = function () {\n        var self = this;\n\n        // Listen for model data and filter changes.\n        $(this.model)\n            .on("filter", function () {\n                self._render();\n            })\n            .on("change", function (e, id) {\n                var extensions = self.model.extensions,\n                    $oldItem = self._itemViews[id];\n                self._updateMessage();\n                if (self.model.filterSet.indexOf(id) === -1) {\n                    // This extension is not in the filter set. Remove it from the view if we\n                    // were rendering it previously.\n                    if ($oldItem) {\n                        $oldItem.remove();\n                        delete self._itemViews[id];\n                    }\n                } else {\n                    // Render the item, replacing the old item if we had previously rendered it.\n                    var $newItem = self._renderItem(extensions[id], self.model._getEntry(id));\n                    if ($oldItem) {\n                        $oldItem.replaceWith($newItem);\n                        self._itemViews[id] = $newItem;\n                    }\n                }\n            });\n\n        // UI event handlers\n        this.$el\n            .on("click", "a", function (e) {\n                var $target = $(e.target);\n                if ($target.hasClass("undo-remove")) {\n                    ExtensionManager.markForRemoval($target.attr("data-extension-id"), false);\n                } else if ($target.hasClass("remove")) {\n                    ExtensionManager.markForRemoval($target.attr("data-extension-id"), true);\n                } else if ($target.hasClass("undo-update")) {\n                    ExtensionManager.removeUpdate($target.attr("data-extension-id"));\n                }\n            })\n            .on("click", "button.install", function (e) {\n                self._installUsingDialog($(e.target).attr("data-extension-id"));\n            })\n            .on("click", "button.update", function (e) {\n                self._installUsingDialog($(e.target).attr("data-extension-id"), true);\n            })\n            .on("click", "button.remove", function (e) {\n                ExtensionManager.markForRemoval($(e.target).attr("data-extension-id"), true);\n            });\n    };\n\n    /**\n     * @private\n     * Renders the view for a single extension entry.\n     * @param {Object} entry The extension entry to render.\n     * @param {Object} info The extension\'s metadata.\n     * @return {jQueryObject} The rendered node as a jQuery object.\n     */\n    ExtensionManagerView.prototype._renderItem = function (entry, info) {\n        // Create a Mustache context object containing the entry data and our helper functions.\n\n        // Start with the basic info from the given entry, either the installation info or the\n        // registry info depending on what we\'re listing.\n        var context = $.extend({}, info);\n\n        // Normally we would merge the strings into the context we\'re passing into the template,\n        // but since we\'re instantiating the template for every item, it seems wrong to take the hit\n        // of copying all the strings into the context, so we just make it a subfield.\n        context.Strings = Strings;\n\n        // Calculate various bools, since Mustache doesn\'t let you use expressions and interprets\n        // arrays as iteration contexts.\n        context.isInstalled = !!entry.installInfo;\n        context.failedToStart = (entry.installInfo && entry.installInfo.status === ExtensionManager.START_FAILED);\n        context.hasVersionInfo = !!info.versions;\n\n        if (entry.registryInfo) {\n            var latestVerCompatInfo = ExtensionManager.getCompatibilityInfo(entry.registryInfo, app.metadata.apiVersion);\n            context.isCompatible = latestVerCompatInfo.isCompatible;\n            context.requiresNewer = latestVerCompatInfo.requiresNewer;\n            context.isCompatibleLatest = latestVerCompatInfo.isLatestVersion;\n            if (!context.isCompatibleLatest) {\n                var installWarningBase = context.requiresNewer ? Strings.EXTENSION_LATEST_INCOMPATIBLE_NEWER : Strings.EXTENSION_LATEST_INCOMPATIBLE_OLDER;\n                context.installWarning = StringUtils.format(installWarningBase, entry.registryInfo.versions[entry.registryInfo.versions.length - 1].version, latestVerCompatInfo.compatibleVersion);\n            }\n        } else {\n            // We should only get here when viewing the Installed tab and some extensions don\'t exist in the registry\n            // (or registry is offline). These flags *should* always be ignored in that scenario, but just in case...\n            context.isCompatible = context.isCompatibleLatest = true;\n        }\n\n        context.isMarkedForRemoval = ExtensionManager.isMarkedForRemoval(info.metadata.name);\n        context.isMarkedForUpdate = ExtensionManager.isMarkedForUpdate(info.metadata.name);\n\n        context.showInstallButton = (this.model.source === this.model.SOURCE_REGISTRY) && !context.updateAvailable;\n        context.showUpdateButton = context.updateAvailable && !context.isMarkedForUpdate && !context.isMarkedForRemoval;\n\n        context.allowInstall = context.isCompatible && !context.isInstalled;\n\n        var isInstalledInUserFolder = (entry.installInfo && entry.installInfo.locationType === ExtensionManager.LOCATION_USER);\n        context.allowRemove = isInstalledInUserFolder;\n        context.allowUpdate = context.showUpdateButton && context.isCompatible && context.isCompatibleLatest && isInstalledInUserFolder;\n        if (!context.allowUpdate) {\n            context.updateNotAllowedReason = isInstalledInUserFolder ? Strings.CANT_UPDATE : Strings.CANT_UPDATE_DEV;\n        }\n\n        context.removalAllowed = this.model.source === "installed" &&\n            !context.failedToStart && !context.isMarkedForUpdate && !context.isMarkedForRemoval;\n\n        // Copy over helper functions that we share with the registry app.\n        ["lastVersionDate", "authorInfo"].forEach(function (helper) {\n            context[helper] = registry_utils[helper].call(context);\n        });\n\n        // Author image\n        if (context.metadata.author && context.metadata.author.email) {\n            context.authorImageUrl = "http://www.gravatar.com/avatar/" + md5(context.metadata.author.email) + ".png";\n        }\n\n        return $(Mustache.render(this._itemTemplate, context));\n    };\n\n    /**\n     * @private\n     * Display an optional message (hiding the extension list if displayed)\n     * @return {boolean} Returns true if a message is displayed\n     */\n    ExtensionManagerView.prototype._updateMessage = function () {\n        if (this.model.message) {\n            this._$emptyMessage.css("display", "block");\n            this._$emptyMessage.html(this.model.message);\n            this._$infoMessage.css("display", "none");\n            this._$table.css("display", "none");\n\n            return true;\n        } else {\n            this._$emptyMessage.css("display", "none");\n            this._$infoMessage.css("display", this.model.infoMessage ? "block" : "none");\n            this._$table.css("display", "");\n\n            return false;\n        }\n    };\n\n    /**\n     * @private\n     * Renders the extension entry table based on the model\'s current filter set. Will create\n     * new items for entries that haven\'t yet been rendered, but will not re-render existing items.\n     */\n    ExtensionManagerView.prototype._render = function () {\n        var self = this,\n            $item;\n\n        this._$table.empty();\n        this._updateMessage();\n\n        this.model.filterSet.forEach(function (id) {\n            var $item = self._itemViews[id];\n            if (!$item) {\n                $item = self._renderItem(self.model.extensions[id], self.model._getEntry(id));\n                self._itemViews[id] = $item;\n            }\n            $item.appendTo(self._$table);\n        });\n\n        $(this).triggerHandler("render");\n    };\n\n    /**\n     * @private\n     * Install the extension with the given ID using the install dialog.\n     * @param {string} id ID of the extension to install.\n     */\n    ExtensionManagerView.prototype._installUsingDialog = function (id, _isUpdate) {\n        var entry = this.model.extensions[id];\n        if (entry && entry.registryInfo) {\n            var compatInfo = ExtensionManager.getCompatibilityInfo(entry.registryInfo, app.metadata.apiVersion),\n                url = ExtensionManager.getExtensionURL(id, compatInfo.compatibleVersion);\n\n            // TODO: this should set .done on the returned promise\n            if (_isUpdate) {\n                InstallExtensionDialog.updateUsingDialog(url).done(ExtensionManager.updateFromDownload);\n            } else {\n                InstallExtensionDialog.installUsingDialog(url);\n            }\n        }\n    };\n\n    /**\n     * Filters the contents of the view.\n     * @param {string} query The query to filter by.\n     */\n    ExtensionManagerView.prototype.filter = function (query) {\n        this.model.filter(query);\n    };\n\n    exports.ExtensionManagerView = ExtensionManagerView;\n});\n\n//# sourceURL=/extensibility/ExtensionManagerView.js'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *  \n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"), \n * to deal in the Software without restriction, including without limitation \n * the rights to use, copy, modify, merge, publish, distribute, sublicense, \n * and/or sell copies of the Software, and to permit persons to whom the \n * Software is furnished to do so, subject to the following conditions:\n *  \n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *  \n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING \n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER \n * DEALINGS IN THE SOFTWARE.\n * \n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, window, $, brackets, Mustache */\n/*unittests: ExtensionManager*/\n\ndefine(\'extensibility/ExtensionManagerViewModel\',[\'require\',\'exports\',\'module\',\'extensibility/ExtensionManager\',\'extensibility/Package\',\'extensibility/registry_utils\',\'strings\'],function (require, exports, module) {\n    "use strict";\n    \n    var ExtensionManager = require("extensibility/ExtensionManager"),\n        Package          = require("extensibility/Package"),\n        registry_utils   = require("extensibility/registry_utils"),\n        Strings          = require("strings");\n\n    /**\n     * @private\n     * @type {Array}\n     * A list of fields to search when trying to search for a query string in an object. Each field is \n     * represented as an array of keys to recurse downward through the object. We store this here to avoid \n     * doing it for each search call.\n     */\n    var _searchFields = [["metadata", "name"], ["metadata", "title"], ["metadata", "description"],\n                         ["metadata", "author", "name"], ["metadata", "keywords"], ["owner"]];\n    /**\n     * @constructor\n     * The base model for the ExtensionManagerView. Keeps track of the extensions that are currently visible\n     * and manages sorting/filtering them. Must be disposed with dispose() when done.\n     * Events:\n     *     change - triggered when the data for a given extension changes. Second parameter is the extension id.\n     *     filter - triggered whenever the filtered set changes (including on initialize).\n     *\n     */\n    function ExtensionManagerViewModel() {\n        this._handleStatusChange = this._handleStatusChange.bind(this);\n        \n        // Listen for extension status changes.\n        $(ExtensionManager)\n            .on("statusChange." + this.source, this._handleStatusChange)\n            .on("registryUpdate." + this.source, this._handleStatusChange);\n    }\n    \n    /**\n     * @type {string}\n     * Constant indicating that this model/view should initialize from the main extension registry.\n     */\n    ExtensionManagerViewModel.prototype.SOURCE_REGISTRY = "registry";\n    \n    /**\n     * @type {string}\n     * Constant indicating that this model/view should initialize from the list of locally installed extensions.\n     */\n    ExtensionManagerViewModel.prototype.SOURCE_INSTALLED = "installed";\n    \n    /**\n     * @type {Object}\n     * The current set of extensions managed by this model. Same as ExtensionManager.extensions.\n     */\n    ExtensionManagerViewModel.prototype.extensions = null;\n    \n    /**\n     * @type {string}\n     * The current source for the model; one of the SOURCE_* keys above.\n     */\n    ExtensionManagerViewModel.prototype.source = null;\n    \n    /**\n     * @type {Array.<Object>}\n     * The list of IDs of items matching the current query and sorted with the current sort.\n     */\n    ExtensionManagerViewModel.prototype.filterSet = null;\n    \n    /**\n     * @type {Object}\n     * The list of all ids from the extension list, sorted with the current sort.\n     */\n    ExtensionManagerViewModel.prototype.sortedFullSet = null;\n    \n    /**\n     * @private\n     * @type {string}\n     * The last query we filtered by. Used to optimize future searches.\n     */\n    ExtensionManagerViewModel.prototype._lastQuery = null;\n    \n    /**\n     * @type {string}\n     * Info message to display to the user when listing extensions\n     */\n    ExtensionManagerViewModel.prototype.infoMessage = null;\n    \n    /**\n     * @type {string}\n     * An optional message to display to the user\n     */\n    ExtensionManagerViewModel.prototype.message = null;\n    \n    /**\n     * @type {number}\n     * Number to show in tab\'s notification icon. No icon shown if 0.\n     */\n    ExtensionManagerViewModel.prototype.notifyCount = 0;\n    \n    /**\n     * @private {$.Promise}\n     * Internal use only to track when initialization fails, see usage in _updateMessage.\n     */\n    ExtensionManagerViewModel.prototype._initializeFromSourcePromise = null;\n    \n    /**\n     * Unregisters listeners when we\'re done.\n     */\n    ExtensionManagerViewModel.prototype.dispose = function () {\n        $(ExtensionManager).off("." + this.source);\n    };\n    \n    /**\n     * @private\n     * Sets up the initial filtered set based on the sorted full set.\n     */\n    ExtensionManagerViewModel.prototype._setInitialFilter = function () {\n        // Initial filtered list is the same as the sorted list.\n        this.filterSet = _.clone(this.sortedFullSet);\n        $(this).triggerHandler("filter");\n    };\n    \n    /**\n     * @private\n     * Re-sorts the current full set based on the source we\'re viewing.\n     * The base implementation does nothing.\n     */\n    ExtensionManagerViewModel.prototype._sortFullSet = function () { };\n\n    /**\n     * Initializes the model from the source.\n     */\n    ExtensionManagerViewModel.prototype.initialize = function () {\n        var self = this;\n\n        this._initializeFromSourcePromise = this._initializeFromSource().always(function () {\n            self._updateMessage();\n        });\n        \n        return this._initializeFromSourcePromise;\n    };\n    \n    /**\n     * @private\n     * Updates the initial set and filter as necessary when the status of an extension changes,\n     * and notifies listeners of the change.\n     * @param {$.Event} e The jQuery event object.\n     * @param {string} id The id of the extension whose status changed.\n     */\n    ExtensionManagerViewModel.prototype._handleStatusChange = function (e, id) {\n        $(this).triggerHandler("change", id);\n    };\n    \n    /**\n     * @private\n     * Searches for the given query in the current extension list and updates the filter set,\n     * dispatching a filter event.\n     * @param {string} query The string to search for.\n     * @param {boolean} force If true, always filter starting with the full set, not the last\n     *     query\'s filter.\n     */\n    ExtensionManagerViewModel.prototype.filter = function (query, force) {\n        var self = this, initialList;\n        if (!force && this._lastQuery && query.indexOf(this._lastQuery) === 0) {\n            // This is the old query with some new letters added, so we know we can just\n            // search in the current filter set. (This is true even if query has spaces).\n            initialList = this.filterSet;\n        } else {\n            // This is a new query, so start with the full list.\n            initialList = this.sortedFullSet;\n        }\n        \n        var keywords = query.toLowerCase().split(/\\s+/);\n        \n        // Takes \'extensionList\' and returns a version filtered to only those that match \'keyword\'\n        function filterForKeyword(extensionList, word) {\n            var filteredList = [];\n            extensionList.forEach(function (id) {\n                var entry = self._getEntry(id);\n                if (entry && self._entryMatchesQuery(entry, word)) {\n                    filteredList.push(id);\n                }\n            });\n            return filteredList;\n        }\n        \n        // "AND" the keywords together: successively filter down the result set by each keyword in turn\n        var i, currentList = initialList;\n        for (i = 0; i < keywords.length; i++) {\n            currentList = filterForKeyword(currentList, keywords[i]);\n        }\n        \n        this._lastQuery = query;\n        this.filterSet = currentList;\n\n        this._updateMessage();\n\n        $(this).triggerHandler("filter");\n    };\n\n    /**\n     * @private\n     * Updates an optional message displayed to the user along with the extension list.\n     */\n    ExtensionManagerViewModel.prototype._updateMessage = function () {\n        if (this._initializeFromSourcePromise && this._initializeFromSourcePromise.state() === "rejected") {\n            this.message = Strings.EXTENSION_MANAGER_ERROR_LOAD;\n        } else if (this.filterSet && this.filterSet.length === 0) {\n            this.message = this.sortedFullSet && this.sortedFullSet.length ? Strings.NO_EXTENSION_MATCHES : Strings.NO_EXTENSIONS;\n        } else {\n            this.message = null;\n        }\n    };\n    \n    /**\n     * @private\n     * This is to be overridden by subclasses to provide the metadata for the extension\n     * with the provided `id`.\n     *\n     * @param {string} id of the extension\n     * @return {Object?} extension metadata or null if there\'s no matching extension\n     */\n    ExtensionManagerViewModel.prototype._getEntry = function (id) {\n        return null;\n    };\n    \n    /**\n     * @private\n     * Tests if the given entry matches the query.\n     * @param {Object} entry The extension entry to test.\n     * @param {string} query The query to match against.\n     * @return {boolean} Whether the query matches.\n     */\n    ExtensionManagerViewModel.prototype._entryMatchesQuery = function (entry, query) {\n        return query === "" ||\n            _searchFields.some(function (fieldSpec) {\n                var i, cur = entry;\n                for (i = 0; i < fieldSpec.length; i++) {\n                    // Recurse downward through the specified fields to the leaf value.\n                    cur = cur[fieldSpec[i]];\n                    if (!cur) {\n                        return false;\n                    }\n                }\n                // If the leaf value is an array (like keywords), search each item, otherwise\n                // just search in the string.\n                if (Array.isArray(cur)) {\n                    return cur.some(function (keyword) {\n                        return keyword.toLowerCase().indexOf(query) !== -1;\n                    });\n                } else if (fieldSpec[fieldSpec.length - 1] === "owner") {\n                    // Special handling: ignore the authentication source when querying,\n                    // since it\'s not useful to search on\n                    var components = cur.split(":");\n                    if (components[1].toLowerCase().indexOf(query) !== -1) {\n                        return true;\n                    }\n                } else if (cur.toLowerCase().indexOf(query) !== -1) {\n                    return true;\n                }\n            });\n    };\n    \n    /**\n     * @constructor\n     * The model for the ExtensionManagerView that is responsible for handling registry-based extensions. \n     * This extends ExtensionManagerViewModel.\n     * Must be disposed with dispose() when done.\n     *\n     * Events:\n     *     change - triggered when the data for a given extension changes. Second parameter is the extension id.\n     *     filter - triggered whenever the filtered set changes (including on initialize).\n     */\n    function RegistryViewModel() {\n        ExtensionManagerViewModel.call(this);\n        this.infoMessage = Strings.REGISTRY_SANITY_CHECK_WARNING;\n    }\n    \n    RegistryViewModel.prototype = Object.create(ExtensionManagerViewModel.prototype);\n    RegistryViewModel.prototype.constructor = RegistryViewModel;\n    \n    /**\n     * @type {string}\n     * RegistryViewModels always have a source of SOURCE_REGISTRY.\n     */\n    RegistryViewModel.prototype.source = ExtensionManagerViewModel.prototype.SOURCE_REGISTRY;\n    \n    /**\n     * Initializes the model from the remote extension registry.\n     * @return {$.Promise} a promise that\'s resolved with the registry JSON data\n     * or rejected if the server can\'t be reached.\n     */\n    RegistryViewModel.prototype._initializeFromSource = function () {\n        var self = this;\n        return ExtensionManager.downloadRegistry()\n            .done(function () {\n                self.extensions = ExtensionManager.extensions;\n                \n                // Sort the registry by last published date and store the sorted list of IDs.\n                self.sortedFullSet = registry_utils.sortRegistry(self.extensions, "registryInfo")\n                    .filter(function (entry) {\n                        return entry.registryInfo !== undefined;\n                    })\n                    .map(function (entry) {\n                        return entry.registryInfo.metadata.name;\n                    });\n                self._setInitialFilter();\n            })\n            .fail(function () {\n                self.extensions = [];\n                self.sortedFullSet = [];\n                self.filterSet = [];\n            });\n    };\n    \n    /**\n     * @private\n     * Finds the extension metadata by id. If there is no extension matching the given id,\n     * this returns `null`.\n     * @param {string} id of the extension\n     * @return {Object?} extension metadata or null if there\'s no matching extension\n     */\n    RegistryViewModel.prototype._getEntry = function (id) {\n        var entry = this.extensions[id];\n        if (entry) {\n            return entry.registryInfo;\n        }\n        return entry;\n    };\n    \n    /**\n     * @constructor\n     * The model for the ExtensionManagerView that is responsible for handling previously-installed extensions. \n     * This extends ExtensionManagerViewModel.\n     * Must be disposed with dispose() when done.\n     *\n     * Events:\n     *     change - triggered when the data for a given extension changes. Second parameter is the extension id.\n     *     filter - triggered whenever the filtered set changes (including on initialize).\n     */\n    function InstalledViewModel() {\n        ExtensionManagerViewModel.call(this);\n    }\n    \n    InstalledViewModel.prototype = Object.create(ExtensionManagerViewModel.prototype);\n    InstalledViewModel.prototype.constructor = InstalledViewModel;\n    \n    /**\n     * @type {string}\n     * InstalledViewModels always have a source of SOURCE_INSTALLED.\n     */\n    InstalledViewModel.prototype.source = ExtensionManagerViewModel.prototype.SOURCE_INSTALLED;\n    \n    /**\n     * Initializes the model from the set of locally installed extensions, sorted\n     * alphabetically by id (or name of the extension folder for legacy extensions).\n     * @return {$.Promise} a promise that\'s resolved when we\'re done initializing.\n     */\n    InstalledViewModel.prototype._initializeFromSource = function () {\n        var self = this;\n        this.extensions = ExtensionManager.extensions;\n        this.sortedFullSet = Object.keys(this.extensions)\n            .filter(function (key) {\n                return self.extensions[key].installInfo &&\n                    self.extensions[key].installInfo.locationType !== ExtensionManager.LOCATION_DEFAULT;\n            });\n        this._sortFullSet();\n        this._setInitialFilter();\n        this._countUpdates();\n        \n        return new $.Deferred().resolve().promise();\n    };\n    \n    /**\n     * @private\n     * Re-sorts the current full set\n     */\n    InstalledViewModel.prototype._sortFullSet = function () {\n        var self = this;\n        \n        this.sortedFullSet = this.sortedFullSet.sort(function (key1, key2) {\n            var metadata1 = self.extensions[key1].installInfo.metadata,\n                metadata2 = self.extensions[key2].installInfo.metadata,\n                id1 = (metadata1.title || metadata1.name).toLowerCase(),\n                id2 = (metadata2.title || metadata2.name).toLowerCase();\n            if (id1 < id2) {\n                return -1;\n            } else if (id1 === id2) {\n                return 0;\n            } else {\n                return 1;\n            }\n        });\n    };\n    \n    /**\n     * @private\n     * Updates notifyCount based on number of extensions with an update available\n     */\n    InstalledViewModel.prototype._countUpdates = function () {\n        var self = this;\n        this.notifyCount = 0;\n        this.sortedFullSet.forEach(function (key) {\n            if (self.extensions[key].installInfo.updateAvailable) {\n                self.notifyCount++;\n            }\n        });\n    };\n    \n    /**\n     * @private\n     * Updates the initial set and filter as necessary when the status of an extension changes,\n     * and notifies listeners of the change.\n     * @param {$.Event} e The jQuery event object.\n     * @param {string} id The id of the extension whose status changed.\n     */\n    InstalledViewModel.prototype._handleStatusChange = function (e, id) {\n        var index = this.sortedFullSet.indexOf(id),\n            refilter = false;\n        if (index !== -1 && !this.extensions[id].installInfo) {\n            // This was in our set, but was uninstalled. Remove it.\n            this.sortedFullSet.splice(index, 1);\n            this._countUpdates();  // may also affect update count\n            refilter = true;\n        } else if (index === -1 && this.extensions[id].installInfo) {\n            // This was not in our set, but is now installed. Add it and resort.\n            this.sortedFullSet.push(id);\n            this._sortFullSet();\n            refilter = true;\n        }\n        if (refilter) {\n            this.filter(this._lastQuery || "", true);\n        }\n        \n        if (this.extensions[id].installInfo) {\n            // If our count of available updates may have been affected, re-count\n            this._countUpdates();\n        }\n        \n        ExtensionManagerViewModel.prototype._handleStatusChange.call(this, e, id);\n    };\n    \n    /**\n     * @private\n     * Finds the extension metadata by id. If there is no extension matching the given id,\n     * this returns `null`.\n     * @param {string} id of the extension\n     * @return {Object?} extension metadata or null if there\'s no matching extension\n     */\n    InstalledViewModel.prototype._getEntry = function (id) {\n        var entry = this.extensions[id];\n        if (entry) {\n            return entry.installInfo;\n        }\n        return entry;\n    };\n\n    exports.RegistryViewModel = RegistryViewModel;\n    exports.InstalledViewModel = InstalledViewModel;\n});\n//# sourceURL=/extensibility/ExtensionManagerViewModel.js'),eval('define(\'text!htmlContent/extension-manager-dialog.html\',[],function () { return \'<div class="extension-manager-dialog dialog modal" data-title="Extension Manager">\\n    <div class="dialog-toolbar">\\n        <div class="radio-button-bar left">\\n            <label class="radio-button-bar__item">\\n                <input id="extension-manager-radio-button-registry" type="radio" name="extension-tab" value="registry">\\n                <label for="extension-manager-radio-button-registry" class="k-button button-registry">Registry</label>\\n            </label>\\n            <label class="radio-button-bar__item">\\n                <input id="extension-manager-radio-button-installed" type="radio" name="extension-tab" value="installed">\\n                <label for="extension-manager-radio-button-installed" class="k-button button-installed">Installed<span class="update-notifications"></span></label>\\n            </label>\\n        </div>\\n\\n        <input type="search" value="" placeholder="search" class="search k-search-input">\\n    </div>\\n    <div class="dialog-body no-padding">\\n        <div class="spinner"></div>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button left install-from-url">Install From Url...</button>\\n        <button class="k-button dialog-button primary" data-button-id="close">{{Strings.CLOSE}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/extension-manager-dialog.html'),eval('/*\n * Copyright (c) 2013 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global brackets, define, $, Mustache, app, window */\n\ndefine(\'extensibility/ExtensionManagerDialog\',[\'require\',\'exports\',\'module\',\'dialogs/Dialogs\',\'extensibility/Package\',\'strings\',\'utils/StringUtils\',\'command/Commands\',\'command/CommandManager\',\'extensibility/InstallExtensionDialog\',\'utils/AppInit\',\'utils/Async\',\'extensibility/ExtensionManager\',\'extensibility/ExtensionManagerView\',\'extensibility/ExtensionManagerViewModel\',\'text!htmlContent/extension-manager-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var Dialogs                     = require("dialogs/Dialogs"),\n        Package                     = require("extensibility/Package"),\n        Strings                     = require("strings"),\n        StringUtils                 = require("utils/StringUtils"),\n        Commands                    = require("command/Commands"),\n        CommandManager              = require("command/CommandManager"),\n        InstallExtensionDialog      = require("extensibility/InstallExtensionDialog"),\n        AppInit                     = require("utils/AppInit"),\n        Async                       = require("utils/Async"),\n        ExtensionManager            = require("extensibility/ExtensionManager"),\n        ExtensionManagerView        = require("extensibility/ExtensionManagerView").ExtensionManagerView,\n        ExtensionManagerViewModel   = require("extensibility/ExtensionManagerViewModel"),\n        dialogTemplate              = require("text!htmlContent/extension-manager-dialog.html");\n\n    var _activeTabIndex;\n\n    /**\n     * @private\n     * Triggers changes requested by the dialog UI.\n     */\n    function _performChanges() {\n        // If an extension was removed or updated, prompt the user to quit Brackets.\n        var hasRemovedExtensions = ExtensionManager.hasExtensionsToRemove(),\n            hasUpdatedExtensions = ExtensionManager.hasExtensionsToUpdate();\n        if (!hasRemovedExtensions && !hasUpdatedExtensions) {\n            return;\n        }\n\n        var buttonLabel = Strings.CHANGE_AND_RELOAD;\n        if (hasRemovedExtensions && !hasUpdatedExtensions) {\n            buttonLabel = Strings.REMOVE_AND_RELOAD;\n        } else if (hasUpdatedExtensions && !hasRemovedExtensions) {\n            buttonLabel = Strings.UPDATE_AND_RELOAD;\n        }\n\n        var dlg = Dialogs.showModalDialog(\n            "change-marked-extensions",\n            Strings.CHANGE_AND_RELOAD_TITLE,\n            Strings.CHANGE_AND_RELOAD_MESSAGE,\n            [\n                {\n                    className : Dialogs.DIALOG_BTN_CLASS_NORMAL,\n                    id        : Dialogs.DIALOG_BTN_CANCEL,\n                    text      : Strings.CANCEL\n                },\n                {\n                    className : Dialogs.DIALOG_BTN_CLASS_PRIMARY,\n                    id        : Dialogs.DIALOG_BTN_OK,\n                    text      : buttonLabel\n                }\n            ],\n            false\n        ),\n            $dlg = dlg.getElement();\n\n        $dlg.one("buttonClick", function (e, buttonId) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                // Disable the dialog buttons so the user can\'t dismiss it,\n                // and show a message indicating that we\'re doing the updates,\n                // in case it takes a long time.\n                $dlg.find(".dialog-button").prop("disabled", true);\n                $dlg.find(".close").hide();\n                $dlg.find(".dialog-message")\n                    .text(Strings.PROCESSING_EXTENSIONS)\n                    .append("<span class=\'spinner spin\'/>");\n\n                ExtensionManager.removeMarkedExtensions()\n                    .done(function () {\n                        ExtensionManager.updateExtensions()\n                            .done(function () {\n                                dlg.close();\n                                CommandManager.execute(Commands.APP_RELOAD);\n                            })\n                            .fail(function (errorArray) {\n                                dlg.close();\n\n                                // This error case should be very uncommon.\n                                // Just let the user know that we couldn\'t update\n                                // this extension and log the errors to the console.\n                                var ids = [];\n                                errorArray.forEach(function (errorObj) {\n                                    ids.push(errorObj.item);\n                                    if (errorObj.error && errorObj.error.forEach) {\n                                        console.error("Errors for ", errorObj.item);\n                                        errorObj.error.forEach(function (error) {\n                                            console.error(Package.formatError(error));\n                                        });\n                                    }\n                                });\n                                Dialogs.showErrorDialog(\n                                    StringUtils.format(Strings.EXTENSION_MANAGER_UPDATE_ERROR, ids.join(", "))\n                                ).done(function () {\n                                    // We still have to reload even if some of the removals failed.\n                                    CommandManager.execute(Commands.APP_RELOAD);\n                                });\n                            });\n                    })\n                    .fail(function (errorArray) {\n                        dlg.close();\n                        ExtensionManager.cleanupUpdates();\n\n                        var ids = [];\n                        errorArray.forEach(function (errorObj) {\n                            ids.push(errorObj.item);\n                        });\n\n                        Dialogs.showErrorDialog(\n                            StringUtils.format(Strings.EXTENSION_MANAGER_REMOVE_ERROR, ids.join(", "))\n                        ).done(function () {\n                            // We still have to reload even if some of the removals failed.\n                            CommandManager.execute(Commands.APP_RELOAD);\n                        });\n                    });\n            } else {\n                dlg.close();\n                ExtensionManager.cleanupUpdates();\n                ExtensionManager.unmarkAllForRemoval();\n            }\n        });\n    }\n\n    /**\n     * @private\n     * Show a dialog that allows the user to browse and manage extensions.\n     */\n    function showDialog() {\n        var models = [],\n            views  = [],\n            context = { Strings: Strings, showRegistry: !!app.config.extension_registry };\n\n        // Load registry only if the registry URL exists\n        if (context.showRegistry) {\n            models.push(new ExtensionManagerViewModel.RegistryViewModel());\n        }\n        models.push(new ExtensionManagerViewModel.InstalledViewModel());\n\n        // Open the Dialog\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(dialogTemplate, context), false);\n        var $dlg = dialog.getElement(),\n            $search = $dlg.find(".search"),\n            $extensionTab = $("input[name=\'extension-tab\']", $dlg);\n\n        // When dialog closes, dismiss models and commit changes\n        dialog.done(function () {\n            models.forEach(function (model) {\n                model.dispose();\n            });\n\n            _performChanges();\n        });\n\n\n        function updateTab(active) {\n            switch (active) {\n            case "registry":\n                _activeTabIndex = 0;\n                $dlg.find(".extension-list#registry").show();\n                $dlg.find(".extension-list#installed").hide();\n                break;\n            case "installed":\n                _activeTabIndex = 1;\n                $dlg.find(".extension-list#registry").hide();\n                $dlg.find(".extension-list#installed").show();\n                break;\n            }\n\n            // Clear search\n            $search.val("");\n            views.forEach(function (view, index) {\n                view.filter("");\n            });\n\n            models[_activeTabIndex].scrollPos = $(".modal-body", $dlg).scrollTop();\n            $(".modal-body", $dlg).scrollTop(models[_activeTabIndex].scrollPos || 0);\n        }\n\n        $extensionTab.change(function () {\n            updateTab(this.value);\n        });\n\n        // Update & hide/show the notification overlay on a tab\'s icon, based on its model\'s notifyCount\n        function updateNotificationIcon(index) {\n            // Only in Installed tab\n            if (index === 1) {\n                var model = models[index],\n                    $notificationIcon = $dlg.find(".update-notifications");\n                if (model.notifyCount) {\n                    $notificationIcon.text(model.notifyCount);\n                    $notificationIcon.show();\n                } else {\n                    $notificationIcon.hide();\n                }\n            }\n        }\n\n        // Initialize models and create a view for each model\n        var modelInitPromise = Async.doInParallel(models, function (model, index) {\n            var view    = new ExtensionManagerView(),\n                promise = view.initialize(model),\n                lastNotifyCount;\n\n            promise.always(function () {\n                views[index] = view;\n\n                lastNotifyCount = model.notifyCount;\n                updateNotificationIcon(index);\n            });\n\n            $(model).on("change", function () {\n                if (lastNotifyCount !== model.notifyCount) {\n                    lastNotifyCount = model.notifyCount;\n                    updateNotificationIcon(index);\n                }\n            });\n\n            return promise;\n        }, true);\n\n\n        modelInitPromise.always(function () {\n            $(".spinner", $dlg).remove();\n\n            views.forEach(function (view) {\n                view.$el.appendTo($(".dialog-body", $dlg));\n            });\n\n            // Filter the views when the user types in the search field.\n            $dlg.on("input", ".search", function (e) {\n                var query = $(this).val();\n                views.forEach(function (view) {\n                    view.filter(query);\n                });\n            });\n\n            // Show the first tab\n            $extensionTab.val(["registry"]);\n            updateTab("registry");\n        });\n\n        // Handle the close button.\n        $(".extension-manager-dialog button[data-button-id=close]")\n            .click(function () {\n                dialog.close();\n            });\n\n        // Handle the install button.\n        $(".extension-manager-dialog .install-from-url")\n            .click(function () {\n                InstallExtensionDialog.showDialog().done(ExtensionManager.updateFromDownload);\n            });\n\n    }\n\n    AppInit.htmlReady(function () {\n        $("#toolbar-extension-manager").click(function () {\n            showDialog();\n        });\n    });\n\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/extensibility/ExtensionManagerDialog.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo */\n\ndefine(\'explorer/NavigatorView\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'command/CommandManager\',\'command/Commands\',\'strings\',\'utils/Resizer\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit             = require("utils/AppInit"),\n        CommandManager      = require("command/CommandManager"),\n        Commands            = require("command/Commands"),\n        Strings             = require("strings"),\n        Resizer             = require("utils/Resizer");\n\n    var $navigator,\n        $modelExplorer,\n        $editors;\n\n    var EXPLORER_MIN_HEIGHT = 40;\n\n    /**\n     * Toggle NavigatorView\n     */\n    function toggle() {\n        Resizer.toggle($navigator);\n    }\n\n    /**\n     * Show NavigatorView\n     */\n    function show() {\n        Resizer.show($navigator);\n    }\n\n    /**\n     * Hide NavigatorView\n     */\n    function hide() {\n        Resizer.hide($navigator);\n    }\n\n    /**\n     * Return whether NavigatorView is visible or not\n     *\n     * @return {booean}\n     */\n    function isVisible() {\n        return Resizer.isVisible($navigator);\n    }\n\n\n    function _limitEditorsHeight() {\n        if (($navigator.height() - EXPLORER_MIN_HEIGHT) < $editors.height()) {\n            $editors.height($navigator.height() - EXPLORER_MIN_HEIGHT);\n            $modelExplorer.find(".view-content").triggerHandler("scroll");\n            $editors.find(".view-content").triggerHandler("scroll");\n        }\n    }\n\n    AppInit.htmlReady(function () {\n        $navigator = $("#navigator");\n        $modelExplorer = $("#model-explorer");\n        $editors = $("#editors");\n\n        // Navigator Resizing\n\n        $navigator.on("panelResizeStart", function (evt, width) {\n            $navigator.find(".scroller-shadow").css("display", "none");\n        });\n\n        $navigator.on("panelResizeUpdate", function (evt, width) {\n            _limitEditorsHeight();\n        });\n\n        $navigator.on("panelResizeEnd", function (evt, width) {\n            $navigator.find(".scroller-shadow").css("display", "block");\n            $modelExplorer.find(".view-content").triggerHandler("scroll");\n            $editors.find(".view-content").triggerHandler("scroll");\n        });\n\n        $navigator.on("panelCollapsed", function (evt, width) {\n            CommandManager.get(Commands.VIEW_HIDE_NAVIGATOR).setName(Strings.CMD_VIEW_SHOW_NAVIGATOR);\n        });\n\n        $navigator.on("panelExpanded", function (evt, width) {\n            $navigator.find(".scroller-shadow").css("display", "block");\n            $modelExplorer.find(".view-content").triggerHandler("scroll");\n            $editors.find(".view-content").triggerHandler("scroll");\n            CommandManager.get(Commands.VIEW_HIDE_NAVIGATOR).setName(Strings.CMD_VIEW_HIDE_NAVIGATOR);\n        });\n\n        // AppInit.htmlReady in utils/Resizer executes before, so it\'s possible that the sidebar\n        // is collapsed before we add the event. Check here initially\n        if (!$navigator.is(":visible")) {\n            $navigator.trigger("panelCollapsed");\n        }\n\n        // Toolbox Resizing\n\n        $editors.on("panelResizeStart", function (evt, width) {\n            $editors.find(".scroller-shadow").css("display", "none");\n        });\n\n        $editors.on("panelResizeUpdate", function (evt, width) {\n        });\n\n        $editors.on("panelResizeEnd", function (evt, width) {\n            $editors.find(".scroller-shadow").css("display", "block");\n            $modelExplorer.find(".view-content").triggerHandler("scroll");\n            $editors.find(".view-content").triggerHandler("scroll");\n        });\n\n        $editors.on("panelCollapsed", function (evt, width) {\n        });\n\n        $editors.on("panelExpanded", function (evt, width) {\n            $editors.find(".scroller-shadow").css("display", "block");\n            $modelExplorer.find(".view-content").triggerHandler("scroll");\n            $editors.find(".view-content").triggerHandler("scroll");\n        });\n\n        // Window Resizing\n\n        $(window).resize(function () {\n            $modelExplorer.find(".view-content").triggerHandler("scroll");\n            $editors.find(".view-content").triggerHandler("scroll");\n            if ($navigator.height() > EXPLORER_MIN_HEIGHT) {\n                _limitEditorsHeight();\n            }\n        });\n\n\n    });\n\n    CommandManager.register(Strings.CMD_VIEW_HIDE_NAVIGATOR, Commands.VIEW_HIDE_NAVIGATOR, toggle);\n\n    exports.toggle    = toggle;\n    exports.show      = show;\n    exports.hide      = hide;\n    exports.isVisible = isVisible;\n\n});\n\n//# sourceURL=/explorer/NavigatorView.js'),eval('define(\'text!htmlContent/model-explorer-view.html\',[],function () { return \'<div id="model-explorer-holder" class="view-holder">\\n    <div class="view-header">\\n        <span class="view-header-title">Model Explorer</span>\\n\\n        <div id="model-explorer-quick-search-wrapper" style="position: absolute; margin: 0; left: 120px; right: 35px; top: 5px;">\\n            <input id="model-explorer-quick-search" type="search" title="Quick Search">\\n        </div>\\n\\n        <button id="model-explorer-settings" class="k-button icon-button quiet view-header-button" style="right: 5px;" title="Configure Model Explorer">\\n            <span class="view-header-button-icon" style="background: url(\\\'styles/icons/topcoat-settings-13.svg\\\') 0 0 no-repeat;"></span>\\n        </button>\\n\\n    </div>\\n    <div class="view-content">\\n        <div class="treeview-wrapper">\\n            <div class="treeview"></div>\\n        </div>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/model-explorer-view.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo */\n\n/**\n *\n * This module dispatches these events:\n *     - selected:        (elem)\n *     - doubleClicked:   (elem)\n *     - dragOver:        ({source, target, allow});\n *     - dragOverDiagram: ({source, diagram, allow});\n *     - drop:            (source, target)\n *     - dropOnDiagram    (source, diagram)\n */\ndefine(\'explorer/ModelExplorerView\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'utils/ViewUtils\',\'core/Core\',\'core/Repository\',\'core/PreferenceManager\',\'command/Commands\',\'command/CommandManager\',\'strings\',\'menu/ContextMenuManager\',\'engine/ProjectManager\',\'diagrams/DiagramManager\',\'text!htmlContent/model-explorer-view.html\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit            = require("utils/AppInit"),\n        ViewUtils          = require("utils/ViewUtils"),\n        Core               = require("core/Core"),\n        Repository         = require("core/Repository"),\n        PreferenceManager  = require("core/PreferenceManager"),\n        Commands           = require("command/Commands"),\n        CommandManager     = require("command/CommandManager"),\n        Strings            = require("strings"),\n        ContextMenuManager = require("menu/ContextMenuManager"),\n        ProjectManager     = require("engine/ProjectManager"),\n        DiagramManager     = require("diagrams/DiagramManager"),\n        viewTemplate       = require("text!htmlContent/model-explorer-view.html");\n\n    /**\n     * JQuery for Explorer Pane\n     * @private\n     * @type {JQuery}\n     */\n    var $viewContent = null;\n\n    /**\n     * JQuery for Tree\n     * @private\n     * @type {JQuery}\n     */\n    var $tree = null;\n\n    /**\n     * KendoTreeView object\n     * @private\n     * @type {kendo.ui.TreeView}\n     */\n    var _tree = null;\n\n    /**\n     * DataSource for TreeView\n     * @private\n     * @type {kendo.data.HierarchicalDataSource}\n     */\n    var _dataSource = null;\n\n    /**\n     * Temporal variable for storing mouse position when draging.\n     * @private\n     * @type {{x:number, y:number}}\n     */\n    var _dragPosition = {x: 0, y: 0};\n\n    /**\n     * Sort type\n     * @private\n     */\n    var _sortBy;\n\n    /**\n     * Show stereotype\n     * @private\n     */\n    var _showStereotype = true;\n\n\n    /**\n     * Sort type constants\n     * @const\n     */\n    var SORT_BY_ADDED = "added",\n        SORT_BY_NAME  = "name";\n\n    /**\n     * Context Menu for Settings\n     * @private\n     * @type {ContextMenu}\n     */\n    var settingsMenu;\n\n    /**\n     * Commands for Settings\n     */\n    var CMD_EXPLORER_SETTINGS_SORT_BY_ADDED        = "explorer.settings.sort-by-added",\n        CMD_EXPLORER_SETTINGS_SORT_BY_NAME         = "explorer.settings.sort-by-name",\n        CMD_EXPLORER_SETTINGS_SHOW_STEREOTYPE_TEXT = "explorer.settings.show-stereotype-text";\n\n    /**\n     * Setup Tree\n     * @private\n     */\n    function _setupTree() {\n        // Setup Custom DataSource for TreeView\n        _dataSource = new kendo.data.HierarchicalDataSource({\n            transport: {\n                read: function (options) {\n                    var id = options.data._id,\n                        elem;\n                    if (id) {\n                        elem = Repository.get(id);\n                        var children = _sort(_getChildren(elem));\n                        var nodes = [];\n                        _.each(children, function (child) {\n                            var node = _makeNode(child);\n                            nodes.push(node);\n                        });\n                        options.success(nodes);\n                    } else {\n                        elem = ProjectManager.getProject();\n                        if (elem) {\n                            var node = _makeNode(elem);\n                            options.success([node]);\n                        } else {\n                            options.success([]);\n                        }\n                    }\n                },\n                create: function (options) {\n                },\n                destroy: function (options) {\n                },\n            },\n            schema: {\n                model: {\n                    id: "_id",\n                    hasChildren: "hasChildren"\n                }\n            }\n        });\n\n        // Setup TreeView\n        $tree.kendoTreeView({\n            loadOnDemand: true,\n            animation: false,\n            dataSource: _dataSource,\n            dataSpriteCssClassField: "sprite",\n            dataTextField: "text",\n            dragAndDrop: true,\n            select: function (e) {\n                $tree.focus();\n                var item = this.dataItem(e.node);\n                var elem = Repository.get(item._id);\n                console.log(elem);\n                $(exports).triggerHandler(\'selected\', [elem]);\n            },\n            drag: function (e) {\n                _dragPosition.x = e.pageX;\n                _dragPosition.y = e.pageY;\n                var sourceItem, targetItem, dragEvent;\n                if (e.dropTarget.id && e.dropTarget.id == "diagram-canvas") {\n                    // When dragging to Diagram Area\n                    sourceItem = this.dataItem(e.sourceNode);\n                    var source     = Repository.get(sourceItem._id);\n                    if (DiagramManager.getEditor()) {\n                        var diagram = DiagramManager.getEditor().diagram;\n                        if (diagram && (diagram != source)) {\n                            dragEvent = {\n                                source: source,\n                                diagram: diagram,\n                                accept: false\n                            };\n                            $(exports).triggerHandler(\'dragOverDiagram\', [dragEvent]);\n                            if (dragEvent.accept === false) {\n                                e.setStatusClass("k-denied");\n                            } else {\n                                e.setStatusClass("k-add");\n                                e.preventDefault();\n                            }\n                        }\n                    }\n                } else {\n                    // When dragging inside ModelExplorer\n                    if (e.statusClass !== "add") {\n                        e.setStatusClass("k-denied");\n                    }\n                    sourceItem = this.dataItem(e.sourceNode);\n                    targetItem = this.dataItem(e.dropTarget);\n                    if (sourceItem && targetItem && sourceItem._id && targetItem._id) {\n                        dragEvent = {\n                            source: Repository.get(sourceItem._id),\n                            target: Repository.get(targetItem._id),\n                            accept: false\n                        };\n                        if (dragEvent.source != dragEvent.target) {\n                            $(exports).triggerHandler(\'dragOver\', [dragEvent]);\n                        }\n                        if (dragEvent.accept === false) {\n                            e.setStatusClass("k-denied");\n                        }\n                    }\n                }\n            },\n            drop: function (e) {\n                var sourceItem,\n                    targetItem,\n                    dropEvent;\n                if (e.dropTarget.id && e.dropTarget.id == "diagram-canvas") {\n                    // When drop on Diagram Area\n                    sourceItem = this.dataItem(e.sourceNode);\n                    var source     = Repository.get(sourceItem._id);\n                    if (DiagramManager.getEditor()) {\n                        var diagram = DiagramManager.getEditor().diagram;\n                        if (diagram && (diagram != source)) {\n                            dropEvent = {\n                                source: source,\n                                diagram: diagram,\n                                x: _dragPosition.x,\n                                y: _dragPosition.y\n                            };\n                            if (e.valid) {\n                                $(exports).triggerHandler(\'dropOnDiagram\', [dropEvent]);\n                            }\n                        }\n                    e.setValid(false);\n                    }\n                } else {\n                    // When drop on ModelExplorer\n                    sourceItem = this.dataItem(e.sourceNode);\n                    targetItem = this.dataItem(e.destinationNode);\n                    if (sourceItem && targetItem && sourceItem._id && targetItem._id) {\n                        dropEvent = {\n                            source: Repository.get(sourceItem._id),\n                            target: Repository.get(targetItem._id)\n                        };\n                        if (e.valid) {\n                            if (e.dropPosition == \'over\') {\n                                $(exports).triggerHandler(\'drop\', [dropEvent]);\n                            }\n                        }\n                    }\n                    e.setValid(false);\n                }\n            }\n        });\n        _tree = $tree.data("kendoTreeView");\n\n        // Setup Double Clicked event.\n        $tree.dblclick(function (e) {\n            var node = _tree.select();\n            var item = _tree.dataItem(node);\n            if (item && item._id) {\n                var elem = Repository.get(item._id);\n                $(exports).triggerHandler("doubleClicked", [elem]);\n                e.stopImmediatePropagation();\n            }\n        });\n\n        // Support Right-click Selection\n        $tree.on("mousedown", ".k-item", function (e) {\n            if (e.which === 3) { // Right-click\n                e.stopImmediatePropagation();\n                _tree.select(this);\n                var item = _tree.dataItem(this);\n                var elem = Repository.get(item._id);\n                $(exports).triggerHandler(\'selected\', [elem]);\n            }\n        });\n    }\n\n\n    /**\n     * Return JSON object for tree node for a given element.\n     * @private\n     * @param {Core.Element} elem\n     * @return {{_id: string, text: string, sprite: string, hasChildren: boolean}}\n     */\n    function _makeNode(elem) {\n        var options = {\n            showStereotype: _showStereotype\n        };\n        return {\n            _id: elem._id,\n            text: elem.getNodeText(options),\n            sprite: elem.getNodeIcon(elem),\n            hasChildren: (elem.getChildNodes().length > 0),\n            _name: elem.name,\n            _namespace: elem._parent ? elem._parent.name || "" : ""\n        };\n    }\n\n    /**\n     * Return child elements.\n     * @private\n     * @param {Element} elem\n     * @return {Array.<Element>}\n     */\n    function _getChildren(elem) {\n        return elem.getChildNodes();\n    }\n\n    /**\n     * Sort elements.\n     * @private\n     * @param {Array.<Element>} elements\n     * @return {Array.<Element>} - 정렬된 요소들의 배열\n     */\n    function _sort(elements) {\n        if (_sortBy === SORT_BY_ADDED) {\n            return _.sortBy(elements, function (child, idx) {\n                return child.getOrdering(idx);\n            });\n        } else if (_sortBy === SORT_BY_NAME) {\n            var alphaSorted = _.sortBy(elements, function (child, idx) {\n                return child.name;\n            });\n            return _.sortBy(elements, function (child, idx) {\n                return child.getOrdering(_.indexOf(alphaSorted, child));\n            });\n        } else {\n            return elements;\n        }\n    }\n\n    /**\n     * Get a tree node of an element.\n     * @private\n     * @param {Element} elem\n     * @return {}\n     */\n    function _getNode(elem) {\n        var item = _dataSource.get(elem._id);\n        if (item && item.uid) {\n            var node = _tree.findByUid(item.uid);\n            return node;\n        }\n        return null;\n    }\n\n    /**\n     * Append a tree node.\n     * @private\n     * @param {Core.Element} elem\n     * @param {boolean} expandAllParents\n     */\n    function add(elem, expandAllParents) {\n        // Extend all parents\n        if (elem && elem._parent) {\n            if (expandAllParents) {\n                expand(elem._parent, true);\n            }\n        }\n        // If parent node is not loaded, the element is added when expanding it\'s parent.\n        // Then, the node should not be added to avoid duplicated addition of the node.\n        var node = _getNode(elem);\n        if (node === null) {\n            if (elem._parent) {\n                var parentNode = _getNode(elem._parent),\n                    children = _getChildren(elem._parent);\n                if (!_.contains(children, elem)) {\n                    children.push(elem);\n                }\n                // Find the position to be added\n                var sorted = _sort(children),\n                    position = _.indexOf(sorted, elem);\n                if ((position > -1) && (position < (sorted.length - 1))) {\n                    var after = sorted[position + 1];\n                    var afterNode = _getNode(after);\n                    if (afterNode) {\n                        _tree.insertBefore(_makeNode(elem), afterNode);\n                    } else {\n                        _tree.append(_makeNode(elem), parentNode);\n                    }\n                } else {\n                    _tree.append(_makeNode(elem), parentNode);\n                }\n            } else {\n                // Add as a root element\n                _tree.append(_makeNode(elem));\n            }\n        }\n    }\n\n    /**\n     * Update tree node correspond to a given Element\n     * @private\n     * @param {Core.Element} elem\n     */\n    function update(elem) {\n        var node = _getNode(elem);\n        if (node) {\n            var update = _makeNode(elem);\n            // update text\n            _tree.text(node, update.text);\n            // update icon\n            var iconNode = node.children().children(".k-in").children(".k-sprite");\n            if (iconNode.length > 0 && !iconNode.hasClass(update.sprite)) {\n                iconNode.attr("class", "k-sprite " + update.sprite);\n            }\n            // update position (sort)\n            var item = _tree.dataItem(node);\n            if (_sortBy === SORT_BY_NAME && item._name !== elem.name) {\n                remove(elem);\n                add(elem);\n            }\n        }\n    }\n\n    /**\n     * Remove a tree node.\n     * @private\n     * @param {Core.Element} elem\n     */\n    function remove(elem) {\n        var node = _getNode(elem);\n        if (node) {\n            _tree.remove(node);\n        }\n    }\n\n    /**\n     * Rebuild all tree nodes.\n     * @private\n     */\n    function rebuild() {\n        if (_tree && _tree.destroy) {\n            _tree.destroy();\n        }\n        if ($tree) {\n            $tree.children().remove();\n        }\n        _setupTree();\n    }\n\n    /**\n     * Expand a tree node\n     * @param {Core.Model} elem\n     * @param {?boolean} expandAllParents - true 인 경우, 모든 조상들을 expand 함.\n     */\n    function expand(elem, expandAllParents) {\n        if (elem) {\n            // Expand all parents\n            if ((expandAllParents === true) && elem._parent && elem._parent._id) {\n                expand(elem._parent, expandAllParents);\n            }\n            // Expand current node\n            var node = _getNode(elem);\n            if (node) {\n                _tree.expand(node);\n            }\n        }\n    }\n\n    /**\n     * Collapse a tree node\n     * @param {Core.Model} elem\n     */\n    function collapse(elem) {\n        if (elem) {\n            var node = _getNode(elem);\n            if (node) {\n                _tree.collapse(node);\n            }\n        }\n    }\n\n    /**\n     * Select an element\n     * @param {Core.Model} elem\n     * @param {boolean} scrollTo\n     */\n    function select(elem, scrollTo) {\n        if (elem) {\n            expand(elem._parent, true);\n            var node = _getNode(elem);\n            if (node) {\n                _tree.select(node);\n                // Scroll to node\n                if (scrollTo === true) {\n                    $viewContent.animate({\n                        scrollTop: node.offset().top - $viewContent.offset().top + $viewContent.scrollTop()\n                    }, 500);\n                }\n                $(exports).triggerHandler(\'selected\', [elem]);\n            }\n        } else {\n            _tree.select($());\n        }\n    }\n\n    /**\n     * Deselect\n     */\n    function deselect() {\n        _tree.select($());\n    }\n\n    /**\n     * Return a selected element in treeview\n     * @private\n     */\n    function _getSelected() {\n        var node = _tree.select(),\n            elem = null;\n        if (node.length > 0) {\n            var item = _tree.dataItem(node[0]);\n            elem = Repository.get(item._id);\n        }\n        return elem;\n    }\n\n    function _updateSettingsMenu() {\n        CommandManager.get(CMD_EXPLORER_SETTINGS_SORT_BY_ADDED).setChecked(_sortBy === SORT_BY_ADDED);\n        CommandManager.get(CMD_EXPLORER_SETTINGS_SORT_BY_NAME).setChecked(_sortBy === SORT_BY_NAME);\n        CommandManager.get(CMD_EXPLORER_SETTINGS_SHOW_STEREOTYPE_TEXT).setChecked(_showStereotype);\n    }\n\n    /**\n     * Sort elements by added\n     * @private\n     */\n    function _handleSortByAdded() {\n        var elem = _getSelected();\n        _sortBy = SORT_BY_ADDED;\n        PreferenceManager.set("explorer.sortBy", _sortBy);\n        _updateSettingsMenu();\n        rebuild();\n        if (elem) {\n            select(elem, true);\n        } else {\n            expand(ProjectManager.getProject());\n        }\n    }\n\n    /**\n     * Sort elements by name\n     * @private\n     */\n    function _handleSortByName() {\n        var elem = _getSelected();\n        _sortBy = SORT_BY_NAME;\n        PreferenceManager.set("explorer.sortBy", _sortBy);\n        _updateSettingsMenu();\n        rebuild();\n        if (elem) {\n            select(elem, true);\n        } else {\n            expand(ProjectManager.getProject());\n        }\n    }\n\n    /**\n     * Show/hide stereotype text\n     * @private\n     */\n    function _toggleStereotypeText() {\n        var elem = _getSelected();\n        _showStereotype = !_showStereotype;\n        PreferenceManager.set("explorer.showStereotype", _showStereotype);\n        _updateSettingsMenu();\n        rebuild();\n        if (elem) {\n            select(elem, true);\n        } else {\n            expand(ProjectManager.getProject());\n        }\n    }\n\n    /**\n     * Setup Settings Menu\n     * @private\n     */\n    function _setupSettingsMenu() {\n        // Register Commands\n        CommandManager.register(Strings.EXPLORER_SETTINGS_SORT_BY_ADDED,        CMD_EXPLORER_SETTINGS_SORT_BY_ADDED,        _handleSortByAdded);\n        CommandManager.register(Strings.EXPLORER_SETTINGS_SORT_BY_NAME,         CMD_EXPLORER_SETTINGS_SORT_BY_NAME,         _handleSortByName);\n        CommandManager.register(Strings.EXPLORER_SETTINGS_SHOW_STEREOTYPE_TEXT, CMD_EXPLORER_SETTINGS_SHOW_STEREOTYPE_TEXT, _toggleStereotypeText);\n\n        // Define Settings Menu\n        var CONTEXT_MENU = "model-explorer-settings";\n        settingsMenu = ContextMenuManager.addContextMenu(CONTEXT_MENU, "#model-explorer-settings");\n        settingsMenu.addMenuItem(CMD_EXPLORER_SETTINGS_SORT_BY_ADDED);\n        settingsMenu.addMenuItem(CMD_EXPLORER_SETTINGS_SORT_BY_NAME);\n        settingsMenu.addMenuDivider();\n        settingsMenu.addMenuItem(CMD_EXPLORER_SETTINGS_SHOW_STEREOTYPE_TEXT);\n\n        var $settingsButton = $("#model-explorer-settings");\n        $settingsButton.on(\'contextmenu\', function (e) {\n            e.preventDefault();\n            e.stopImmediatePropagation();\n        });\n        $settingsButton.click(function (e) {\n            e.preventDefault();\n            var $btn = $("#model-explorer-settings");\n            var pos = $btn.offset();\n            e.pageY = pos.top + $btn.outerHeight() + 2;\n            settingsMenu._kendoObject.show(settingsMenu._kendoObject.options.anchor, e);\n            return false;\n        });\n    }\n\n\n    function _setupQuickSearch() {\n        var searchDataSource =  new kendo.data.DataSource({\n            transport: {\n                read: function (options) {\n                    if (options.data.filter && options.data.filter.filters.length > 0) {\n                        var keyword = options.data.filter.filters[0].value;\n                        var results = Repository.search(keyword);\n                        options.success(_.map(results, _makeNode));\n                    } else {\n                        options.success([]);\n                    }\n                }\n            },\n            serverFiltering: true\n        });\n\n        $("#model-explorer-quick-search").kendoAutoComplete({\n            dataTextField: "text",\n            minLength: 1,\n            filter: "contains",\n            select: function(e) {\n                var item = this.dataItem(e.item.index());\n                var elem = Repository.get(item._id);\n                if (elem) {\n                    select(elem, true);\n                }\n            },\n            template: "<div style=\'white-space: nowrap\'>" +\n                "<span class=\'k-sprite #:data.sprite#\'></span>" +\n                "<span style=\'margin-left: 5px\'>#:data.text#</span>" +\n                "#if (data._namespace.length > 0) {# <span style=\'margin-left: 5px; font-size: 11px; color: rgb(139,139,139);\'> — #:data._namespace#</span> #}#<div>",\n            dataSource: searchDataSource\n        });\n    }\n\n    AppInit.htmlReady(function () {\n        _sortBy = PreferenceManager.get("explorer.sortBy", SORT_BY_ADDED);\n        _showStereotype = PreferenceManager.get("explorer.showStereotype", true);\n\n        // Setup UI\n        var $view = $(viewTemplate);\n        $viewContent = $view.find(".view-content");\n        $tree = $view.find(".treeview");\n        $("#model-explorer").append($view);\n        ViewUtils.addScrollerShadow($viewContent.get(0), null, true);\n\n        // Rebuild TreeView\n        rebuild();\n\n        // Setup Settings\n        _setupSettingsMenu();\n\n\n        _setupQuickSearch();\n    });\n\n    AppInit.appReady(function () {\n        _updateSettingsMenu();\n    });\n\n\n    // Define public API\n    exports.add                = add;\n    exports.remove             = remove;\n    exports.update             = update;\n    exports.rebuild            = rebuild;\n    exports.expand             = expand;\n    exports.collapse           = collapse;\n    exports.select             = select;\n    exports.deselect           = deselect;\n\n});\n\n//# sourceURL=/explorer/ModelExplorerView.js'),eval('define(\'text!htmlContent/editors-holder.html\',[],function () { return \'<div class="view-holder">\\n    <div class="view-header">\\n        <span class="view-header-title">Editors</span>\\n    </div>\\n    <div id="editors-holder" class="view-content">\\n\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/editors-holder.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, meta, appshell, kendo, Mustache */\n\n/**\n * EditorsHolder\n *\n * This contains StyleEditorView, PropertyEditorView, and DocumentationEditorView\n */\ndefine(\'editors/EditorsHolder\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'command/CommandManager\',\'command/Commands\',\'strings\',\'utils/Resizer\',\'utils/ViewUtils\',\'text!htmlContent/editors-holder.html\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit        = require("utils/AppInit"),\n        CommandManager = require("command/CommandManager"),\n        Commands       = require("command/Commands"),\n        Strings        = require("strings"),\n        Resizer        = require("utils/Resizer"),\n        ViewUtils      = require("utils/ViewUtils"),\n        viewTemplate   = require("text!htmlContent/editors-holder.html");\n\n    var $editors;\n\n    /**\n     * Toggle EditorsHolder\n     */\n    function toggle() {\n        Resizer.toggle($editors);\n    }\n\n    /**\n     * Show EditorsHolder\n     */\n    function show() {\n        Resizer.show($editors);\n    }\n\n    /**\n     * Hide EditorsHolder\n     */\n    function hide() {\n        Resizer.hide($editors);\n    }\n\n    /**\n     * Return whether EditorsHolder is visible or not\n     *\n     * @return {booean}\n     */\n    function isVisible() {\n        return Resizer.isVisible($editors);\n    }\n\n    AppInit.htmlReady(function () {\n        var $view = $(viewTemplate),\n            $content = $view.find(".view-content");\n\n        $editors = $("#editors");\n        $editors.append($view);\n        ViewUtils.addScrollerShadow($content.get(0), null, true);\n\n        $editors.on("panelCollapsed", function (evt, width) {\n            CommandManager.get(Commands.VIEW_HIDE_EDITORS).setName(Strings.CMD_VIEW_SHOW_EDITORS);\n        });\n\n        $editors.on("panelExpanded", function (evt, width) {\n            CommandManager.get(Commands.VIEW_HIDE_EDITORS).setName(Strings.CMD_VIEW_HIDE_EDITORS);\n        });\n\n        // AppInit.htmlReady in utils/Resizer executes before, so it\'s possible that the sidebar\n        // is collapsed before we add the event. Check here initially\n        if (!$editors.is(":visible")) {\n            $editors.trigger("panelCollapsed");\n        }\n    });\n\n    CommandManager.register(Strings.CMD_VIEW_HIDE_EDITORS, Commands.VIEW_HIDE_EDITORS, toggle);\n\n    exports.toggle    = toggle;\n    exports.show      = show;\n    exports.hide      = hide;\n    exports.isVisible = isVisible;\n\n});\n\n\n//# sourceURL=/editors/EditorsHolder.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, meta, app, appshell, kendo, Mustache */\n\ndefine(\'editors/EditorView\',[\'require\',\'exports\',\'module\',\'core/PreferenceManager\'],function (require, exports, module) {\n    "use strict";\n\n    var PreferenceManager = require("core/PreferenceManager");\n\n    function setupEditorView($view) {\n        var viewId    = $view.data("id"),\n            $suppress = $view.find(".suppress-button"),\n            $body     = $view.find(".editor-view-body");\n\n        // Read EditorView\'s Preferences\n        var elementPrefs = PreferenceManager.getViewState(viewId);\n        if (!elementPrefs) {\n            elementPrefs = { visible: true };\n        }\n        $body.toggle(elementPrefs.visible);\n        $suppress.toggleClass("collapsed", !elementPrefs.visible);\n\n        // Toggle Button\n        $suppress.click(function () {\n            $body.slideToggle(function () {\n                var elementPrefs = {\n                    visible: $body.is(":visible")\n                };\n                PreferenceManager.setViewState(viewId, elementPrefs);\n                $suppress.toggleClass("collapsed", !elementPrefs.visible);\n            });\n        });\n\n        // Append to Editors-Holder\n        $("#editors-holder").append($view);\n    }\n\n    exports.setupEditorView = setupEditorView;\n\n});\n\n//# sourceURL=/editors/EditorView.js'),eval('define(\'text!htmlContent/style-editor-view.html\',[],function () { return \'<div class="editor-view" data-id="styleEditor">\\n    <hr>\\n    <div class="editor-view-header">\\n        <span class="suppress-button"></span>\\n        <label class="header">Styles</label>\\n    </div>\\n    <div class="editor-view-body">\\n        <table class=\\\'property-grid small\\\'>\\n            <!-- Font -->\\n            <tr>\\n                <td>\\n                    Font\\n                </td>\\n                <td>\\n                    <div style="width: 100%; display: block; position: relative; padding-right: 60px; box-sizing: border-box;">\\n                        <label class="font-face k-combo small" style="width: 100%; margin: 0; padding: 0">\\n                            <input type="text" style="width: 100%;">\\n                            <select tabindex="-1">\\n                            </select>\\n                        </label>\\n                        <label class="font-size k-combo small" style="position: absolute; right: 0; top: 0; width: 55px;">\\n                            <input type="text" style="width: 100%;">\\n                            <select tabindex="-1">\\n                                <option value="8">8</option>\\n                                <option value="9">9</option>\\n                                <option value="10">10</option>\\n                                <option value="11">11</option>\\n                                <option value="12">12</option>\\n                                <option value="13">13</option>\\n                                <option value="15">15</option>\\n                                <option value="16">16</option>\\n                                <option value="18">18</option>\\n                                <option value="20">20</option>\\n                                <option value="22">22</option>\\n                                <option value="24">24</option>\\n                                <option value="26">26</option>\\n                                <option value="28">28</option>\\n                                <option value="32">32</option>\\n                                <option value="36">36</option>\\n                                <option value="42">42</option>\\n                            </select>\\n                        </label>\\n                    </div>\\n                </td>\\n            </tr>\\n\\n            <!-- Color -->\\n            <tr>\\n                <td>\\n                    Color\\n                </td>\\n                <td>\\n                    <span class="k-sprite tool-icon-fill-color" style="margin-right: 3px;"></span>\\n                    <input type="color" class="fill-color k-color-input small" title="Fill Color" value="#ffffff">\\n                    <span style="display: inline-block; width: 3px;"></span>\\n\\n                    <span class="k-sprite tool-icon-line-color" style="margin-right: 3px;"></span>\\n                    <input type="color" class="line-color k-color-input small" title="Line Color" value="#000000">\\n                    <span style="display: inline-block; width: 3px;"></span>\\n\\n                    <span class="k-sprite tool-icon-font-color" style="margin-right: 3px;"></span>\\n                    <input type="color" class="font-color k-color-input small" title="Text Color" value="#000000">\\n                </td>\\n            </tr>\\n\\n            <!-- Line Style -->\\n            <tr>\\n                <td>\\n                    Line Style\\n                </td>\\n                <td>\\n                    <div class="radio-button-bar icon-group">\\n                        <label class="radio-button-bar__item">\\n                            <input id="style-editor-line-style-rectilinear" type="radio" name="line-style" value="0">\\n                            <label for="style-editor-line-style-rectilinear" class="line-style k-button small" title="Rectilinear"><span class="k-sprite tool-icon-rectilinear"/></label>\\n                        </label>\\n                        <label class="radio-button-bar__item">\\n                            <input id="style-editor-line-style-oblique" type="radio" name="line-style" value="1">\\n                            <label for="style-editor-line-style-oblique" class="line-style k-button small" title="Oblique"><span class="k-sprite tool-icon-oblique"/></label>\\n                        </label>\\n                        <label class="radio-button-bar__item">\\n                            <input id="style-editor-line-style-roundrect" type="radio" name="line-style" value="2">\\n                            <label for="style-editor-line-style-roundrect" class="line-style k-button small" title="Rounded Rectilinear"><span class="k-sprite tool-icon-rounded-rectilinear"/></label>\\n                        </label>\\n                        <label class="radio-button-bar__item">\\n                            <input id="style-editor-line-style-curve" type="radio" name="line-style" value="3">\\n                            <label for="style-editor-line-style-curve" class="line-style k-button small" title="Curve"><span class="k-sprite tool-icon-curve"/></label>\\n                        </label>\\n                    </div>\\n                </td>\\n            </tr>\\n\\n            <!-- Format -->\\n            <tr>\\n                <td>\\n                    Format\\n                </td>\\n                <td>\\n                    <div class="k-button-bar icon-group">\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="auto-resize">\\n                            <label class="k-button auto-resize" title="Auto Resize"><span class="k-sprite tool-icon-auto-resize"/></label>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="show-shadow">\\n                            <label class="k-button show-shadow" title="Show Shadow"><span class="k-sprite tool-icon-show-shadow"/></label>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="word-wrap">\\n                            <label class="k-button word-wrap" title="Word Wrap"><span class="k-sprite tool-icon-word-wrap"/></label>\\n                        </div>\\n                    </div>\\n                    <div style="display: inline-block; vertical-align: bottom">\\n                        <label class="k-select small stereotype-display">\\n                            <select title="Stereotype Display">\\n                                <option value="none">None</option>\\n                                <option value="label">Label</option>\\n                                <option value="decoration">Decoration</option>\\n                                <option value="decoration-label">Decoration with Label</option>\\n                                <option value="icon">Icon</option>\\n                                <option value="icon-label">Icon with Label</option>\\n                            </select>\\n                        </label>\\n                    </div>\\n                </td>\\n            </tr>\\n            <tr>\\n                <td></td>\\n                <td>\\n                    <div class="k-button-bar icon-group">\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="show-visibility">\\n                            <label class="k-button show-visibility" title="Show Visibility"><span class="k-sprite tool-icon-show-visibility"/></label>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="show-namespace">\\n                            <label class="k-button show-namespace" title="Show Namespace"><span class="k-sprite tool-icon-show-namespace"/></label>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="show-property">\\n                            <label class="k-button show-property" title="Show Property"><span class="k-sprite tool-icon-show-property"/></label>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="show-type">\\n                            <label class="k-button show-type" title="Show Type"><span class="k-sprite tool-icon-show-type"/></label>\\n                        </div>\\n                    </div>\\n                    <div class="k-button-bar icon-group">\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="suppress-attributes">\\n                            <label class="k-button suppress-attributes" title="Suppress Attributes"><span class="k-sprite tool-icon-suppress-attributes"/></label>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="suppress-operations">\\n                            <label class="k-button suppress-operations" title="Suppress Operations"><span class="k-sprite tool-icon-suppress-operations"/></label>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="suppress-receptions">\\n                            <label class="k-button suppress-receptions" title="Suppress Receptions"><span class="k-sprite tool-icon-suppress-receptions"/></label>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <input type="checkbox" class="suppress-literals">\\n                            <label class="k-button suppress-literals" title="Suppress Literals"><span class="k-sprite tool-icon-suppress-literals"/></label>\\n                        </div>\\n                    </div>\\n                </td>\\n            </tr>\\n\\n            <!-- Alignment -->\\n            <tr>\\n                <td>\\n                    Alignment\\n                </td>\\n                <td>\\n                    <div class="k-button-bar icon-group">\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button send-to-back" title="Send to Back"><span class="k-sprite tool-icon-sendtoback"/></button>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button bring-to-front" title="Bring to Front"><span class="k-sprite tool-icon-bringtofront"/></button>\\n                        </div>\\n                    </div>\\n                    <div class="k-button-bar icon-group">\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button align-left" title="Align Left"><span class="k-sprite tool-icon-align-left"/></button>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button align-center" title="Align Center"><span class="k-sprite tool-icon-align-center"/></button>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button align-right" title="Align Right"><span class="k-sprite tool-icon-align-right"/></button>\\n                        </div>\\n                    </div>\\n                    <div class="k-button-bar icon-group">\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button align-top" title="Align Top"><span class="k-sprite tool-icon-align-top"/></button>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button align-middle" title="Align Middle"><span class="k-sprite tool-icon-align-middle"/></button>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button align-bottom" title="Align Bottom"><span class="k-sprite tool-icon-align-bottom"/></button>\\n                        </div>\\n                    </div>\\n                    <div class="k-button-bar icon-group">\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button space-equal-horz" title="Space Equally, Horizontally"><span class="k-sprite tool-icon-align-space-even-horz"/></button>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button space-equal-vert" title="Space Equally, Vertically"><span class="k-sprite tool-icon-align-space-even-vert"/></button>\\n                        </div>\\n                    </div>\\n                    <div class="k-button-bar icon-group">\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button set-width-equal" title="Set Width Equally"><span class="k-sprite tool-icon-set-width-equal"/></button>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button set-height-equal" title="Set Height Equally"><span class="k-sprite tool-icon-set-height-equal"/></button>\\n                        </div>\\n                        <div class="k-button-bar__item">\\n                            <button class="k-button set-size-equal" title="Set Size Equally"><span class="k-sprite tool-icon-set-size-equal"/></button>\\n                        </div>\\n                    </div>\\n                </td>\\n            </tr>\\n\\n        </table>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/style-editor-view.html'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, meta, app, appshell, kendo, Mustache */\n\n/**\n *\n * This module dispatches these events:\n *     - styleChanged: (views, field, value)\n */\ndefine('editors/StyleEditorView',['require','exports','module','utils/AppInit','utils/Helper','core/PreferenceManager','dialogs/Dialogs','command/CommandManager','editors/EditorView','text!htmlContent/style-editor-view.html','editors/EditorsHolder'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit           = require(\"utils/AppInit\"),\n        Helper            = require(\"utils/Helper\"),\n        PreferenceManager = require(\"core/PreferenceManager\"),\n        Dialogs           = require(\"dialogs/Dialogs\"),\n        CommandManager    = require(\"command/CommandManager\"),\n        EditorView        = require(\"editors/EditorView\"),\n        viewTemplate      = require(\"text!htmlContent/style-editor-view.html\");\n\n    // for Loading Order\n    require(\"editors/EditorsHolder\");\n\n    var _views = null;\n\n    var $view,\n        $propertyGrid,\n        $fontFaceEdit, $fontFaceSelect,\n        $fontSizeEdit, $fontSizeSelect,\n        $fillColor, $lineColor, $fontColor,\n        $lineStyleRadio, $lineStyleButtons,\n\n        $autoResizeButton, $autoResizeCheckbox,\n        $showShadowButton, $showShadowCheckbox,\n        $wordWrapButton, $wordWrapCheckbox,\n        $stereotypeDisplaySelect,\n\n        $showVisibilityButton, $showVisibilityCheckbox,\n        $showNamespaceButton, $showNamespaceCheckbox,\n        $showPropertyButton, $showPropertyCheckbox,\n        $showTypeButton, $showTypeCheckbox,\n\n        $suppressAttributesButton, $suppressAttributesCheckbox,\n        $suppressOperationsButton, $suppressOperationsCheckbox,\n        $suppressReceptionsButton, $suppressReceptionsCheckbox,\n        $suppressLiteralsButton, $suppressLiteralsCheckbox,\n\n        $sendToBackButton, $bringToFrontButton,\n        $alignLeftButton, $alignRightButton, $alignCenterButton,\n        $alignTopButton, $alignBottomButton, $alignMiddleButton,\n        $spaceEqualHorz, $spaceEqualVert,\n        $setWidthEqual, $setSizeEqual, $setHeightEqual;\n\n    function _setupStyleWidgets() {\n        $propertyGrid       = $view.find(\".property-grid\");\n        $fontFaceEdit       = $view.find(\".font-face input\");\n        $fontFaceSelect     = $view.find(\".font-face select\");\n        $fontSizeEdit       = $view.find(\".font-size input\");\n        $fontSizeSelect     = $view.find(\".font-size select\");\n        $fillColor          = $view.find(\".fill-color\");\n        $lineColor          = $view.find(\".line-color\");\n        $fontColor          = $view.find(\".font-color\");\n        $lineStyleRadio     = $(\"input[name='line-style']\", $view);\n        $lineStyleButtons   = $(\"button.line-style\", $view);\n\n        $autoResizeButton   = $(\"label.auto-resize\", $view);\n        $autoResizeCheckbox = $(\"input.auto-resize\", $view);\n        $showShadowButton   = $(\"label.show-shadow\", $view);\n        $showShadowCheckbox = $(\"input.show-shadow\", $view);\n        $wordWrapButton     = $(\"label.word-wrap\", $view);\n        $wordWrapCheckbox   = $(\"input.word-wrap\", $view);\n        $stereotypeDisplaySelect = $view.find(\".stereotype-display select\");\n\n        $showVisibilityButton   = $(\"label.show-visibility\", $view);\n        $showVisibilityCheckbox = $(\"input.show-visibility\", $view);\n        $showNamespaceButton    = $(\"label.show-namespace\", $view);\n        $showNamespaceCheckbox  = $(\"input.show-namespace\", $view);\n        $showPropertyButton     = $(\"label.show-property\", $view);\n        $showPropertyCheckbox   = $(\"input.show-property\", $view);\n        $showTypeButton         = $(\"label.show-type\", $view);\n        $showTypeCheckbox       = $(\"input.show-type\", $view);\n\n        $suppressAttributesButton   = $(\"label.suppress-attributes\", $view);\n        $suppressAttributesCheckbox = $(\"input.suppress-attributes\", $view);\n        $suppressOperationsButton   = $(\"label.suppress-operations\", $view);\n        $suppressOperationsCheckbox = $(\"input.suppress-operations\", $view);\n        $suppressReceptionsButton   = $(\"label.suppress-receptions\", $view);\n        $suppressReceptionsCheckbox = $(\"input.suppress-receptions\", $view);\n        $suppressLiteralsButton     = $(\"label.suppress-literals\", $view);\n        $suppressLiteralsCheckbox   = $(\"input.suppress-literals\", $view);\n\n        $sendToBackButton   = $(\".send-to-back\", $view);\n        $bringToFrontButton = $(\".bring-to-front\", $view);\n        $alignLeftButton    = $(\".align-left\", $view);\n        $alignRightButton   = $(\".align-right\", $view);\n        $alignCenterButton  = $(\".align-center\", $view);\n        $alignTopButton     = $(\".align-top\", $view);\n        $alignBottomButton  = $(\".align-bottom\", $view);\n        $alignMiddleButton  = $(\".align-middle\", $view);\n        $spaceEqualHorz     = $(\".space-equal-horz\", $view);\n        $spaceEqualVert     = $(\".space-equal-vert\", $view);\n        $setWidthEqual      = $(\".set-width-equal\", $view);\n        $setHeightEqual     = $(\".set-height-equal\", $view);\n        $setSizeEqual       = $(\".set-size-equal\", $view);\n\n        // Font Face\n        $fontFaceEdit.change(function () {\n            $(exports).triggerHandler('styleChanged', [_views, 'font.face', $fontFaceEdit.val()]);\n        });\n\n        // Font Face <select>\n        $fontFaceSelect.change(function () {\n            $fontFaceEdit.val($fontFaceSelect.val());\n            $fontFaceEdit.change();\n        });\n        // (for Windows) To prevent immediate collapse of dropdown in comboBox.\n        $fontFaceSelect.click(function (e) {\n            $fontFaceSelect.focus();\n            e.stopPropagation();\n            return false;\n        });\n\n        // Font Size\n        $fontSizeEdit.change(function () {\n            if (!_.isNaN(parseInt($fontSizeEdit.val()))) {\n                $(exports).triggerHandler('styleChanged', [_views, 'font.size', $fontSizeEdit.val()]);\n            } else {\n                // TODO: Show message? (Only number is allowed)\n            }\n        });\n\n        // Font Size <select>\n        $fontSizeSelect.change(function () {\n            $fontSizeEdit.val($fontSizeSelect.val());\n            $fontSizeEdit.change();\n        });\n        // (for Windows) To prevent immediate collapse of dropdown in comboBox.\n        $fontSizeSelect.click(function (e) {\n            $fontSizeSelect.focus();\n            e.stopPropagation();\n            return false;\n        });\n\n        // Fill Color\n        $fillColor.click(function () {\n            Dialogs.showColorDialog($fillColor.val())\n                .done(function (buttonId, color) {\n                    if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                        $fillColor.val(color);\n                        $(exports).triggerHandler('styleChanged', [_views, 'fillColor', $fillColor.val()]);\n                    }\n                });\n        });\n\n        // Line Color\n        $lineColor.click(function () {\n            Dialogs.showColorDialog($lineColor.val())\n                .done(function (buttonId, color) {\n                    if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                        $lineColor.val(color);\n                        $(exports).triggerHandler('styleChanged', [_views, 'lineColor', $lineColor.val()]);\n                    }\n                });\n        });\n\n        // Fill Color\n        $fontColor.click(function () {\n            Dialogs.showColorDialog($fontColor.val())\n                .done(function (buttonId, color) {\n                    if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                        $fontColor.val(color);\n                        $(exports).triggerHandler('styleChanged', [_views, 'fontColor', $fontColor.val()]);\n                    }\n                });\n        });\n\n        // Line Style\n        $lineStyleRadio.change(function () {\n            $(exports).triggerHandler('styleChanged', [_views, 'lineStyle', parseInt(this.value)]);\n        });\n\n        // Format\n        $autoResizeButton.click(function () {\n            $autoResizeCheckbox.prop('checked', !$autoResizeCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'autoResize', $autoResizeCheckbox.is(':checked')]);\n        });\n        $showShadowButton.click(function () {\n            $showShadowCheckbox.prop('checked', !$showShadowCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'showShadow', $showShadowCheckbox.is(':checked')]);\n        });\n        $wordWrapButton.click(function () {\n            $wordWrapCheckbox.prop('checked', !$wordWrapCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'wordWrap', $wordWrapCheckbox.is(':checked')]);\n        });\n        $stereotypeDisplaySelect.change(function () {\n            $(exports).triggerHandler('styleChanged', [_views, 'stereotypeDisplay', $stereotypeDisplaySelect.val()]);\n        });\n\n        $showVisibilityButton.click(function () {\n            $showVisibilityCheckbox.prop('checked', !$showVisibilityCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'showVisibility', $showVisibilityCheckbox.is(':checked')]);\n        });\n        $showNamespaceButton.click(function () {\n            $showNamespaceCheckbox.prop('checked', !$showNamespaceCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'showNamespace', $showNamespaceCheckbox.is(':checked')]);\n        });\n        $showPropertyButton.click(function () {\n            $showPropertyCheckbox.prop('checked', !$showPropertyCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'showProperty', $showPropertyCheckbox.is(':checked')]);\n        });\n        $showTypeButton.click(function () {\n            $showTypeCheckbox.prop('checked', !$showTypeCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'showType', $showTypeCheckbox.is(':checked')]);\n        });\n\n        $suppressAttributesButton.click(function () {\n            $suppressAttributesCheckbox.prop('checked', !$suppressAttributesCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'suppressAttributes', $suppressAttributesCheckbox.is(':checked')]);\n        });\n        $suppressOperationsButton.click(function () {\n            $suppressOperationsCheckbox.prop('checked', !$suppressOperationsCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'suppressOperations', $suppressOperationsCheckbox.is(':checked')]);\n        });\n        $suppressReceptionsButton.click(function () {\n            $suppressReceptionsCheckbox.prop('checked', !$suppressReceptionsCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'suppressReceptions', $suppressReceptionsCheckbox.is(':checked')]);\n        });\n        $suppressLiteralsButton.click(function () {\n            $suppressLiteralsCheckbox.prop('checked', !$suppressLiteralsCheckbox.is(':checked'));\n            $(exports).triggerHandler('styleChanged', [_views, 'suppressLiterals', $suppressLiteralsCheckbox.is(':checked')]);\n        });\n\n        // Alignment\n        $sendToBackButton.click(function () {\n            CommandManager.execute(\"format.alignment.sendToBack\");\n        });\n        $bringToFrontButton.click(function () {\n            CommandManager.execute(\"format.alignment.bringToFront\");\n        });\n        $alignLeftButton.click(function () {\n            CommandManager.execute(\"format.alignment.alignLeft\");\n        });\n        $alignCenterButton.click(function () {\n            CommandManager.execute(\"format.alignment.alignCenter\");\n        });\n        $alignRightButton.click(function () {\n            CommandManager.execute(\"format.alignment.alignRight\");\n        });\n        $alignTopButton.click(function () {\n            CommandManager.execute(\"format.alignment.alignTop\");\n        });\n        $alignMiddleButton.click(function () {\n            CommandManager.execute(\"format.alignment.alignMiddle\");\n        });\n        $alignBottomButton.click(function () {\n            CommandManager.execute(\"format.alignment.alignBottom\");\n        });\n        $spaceEqualHorz.click(function () {\n            CommandManager.execute(\"format.alignment.spaceEquallyHorizontally\");\n        });\n        $spaceEqualVert.click(function () {\n            CommandManager.execute(\"format.alignment.spaceEquallyVertically\");\n        });\n        $setWidthEqual.click(function () {\n            CommandManager.execute(\"format.alignment.setWidthEqually\");\n        });\n        $setHeightEqual.click(function () {\n            CommandManager.execute(\"format.alignment.setHeightEqually\");\n        });\n        $setSizeEqual.click(function () {\n            CommandManager.execute(\"format.alignment.setSizeEqually\");\n        });\n\n    }\n\n    function _setFontFace(face) {\n        if (face) {\n            $fontFaceEdit.val(face);\n            $fontFaceSelect.val(face);\n        } else {\n            $fontFaceEdit.prop(\"placeholder\", \"—\");\n            $fontFaceEdit.val(\"\");\n            $fontFaceSelect.val(null);\n        }\n    }\n\n    function _setFontSize(size) {\n        if (_.isNumber(size)) {\n            $fontSizeEdit.val(size);\n            $fontSizeSelect.val(size);\n        } else {\n            $fontSizeEdit.prop(\"placeholder\", \"—\");\n            $fontSizeEdit.val(\"\");\n            $fontSizeSelect.val(null);\n        }\n    }\n\n    function _setFillColor(color) {\n        if (color) {\n            $fillColor.val(color);\n        } else {\n            $fillColor.val(\"\");\n        }\n    }\n\n    function _setLineColor(color) {\n        if (color) {\n            $lineColor.val(color);\n        } else {\n            $lineColor.val(\"\");\n        }\n    }\n\n    function _setFontColor(color) {\n        if (color) {\n            $fontColor.val(color);\n        } else {\n            $fontColor.val(\"\");\n        }\n    }\n\n    function _setLineStyle(lineStyle) {\n        if (_.isNumber(lineStyle)) {\n            $lineStyleRadio.val([lineStyle]);\n        } else {\n            $lineStyleRadio.attr(\"checked\", false);\n        }\n    }\n\n    function _setStereotypeDisplay(value) {\n        if (value) {\n            $stereotypeDisplaySelect.val(value);\n        } else {\n            $stereotypeDisplaySelect.val(null);\n        }\n    }\n\n    function _setCheckbox($widget, value) {\n        if (_.isBoolean(value)) {\n            $widget.prop('checked', value);\n        } else {\n            $widget.prop('checked', false);\n        }\n    }\n\n    function _updateStyles(views) {\n        if (views && views.length > 0) {\n            $view.show();\n            var fonts      = _.map(views, function (v) { return v.font; }),\n                fontFace   = Helper.reduceValue(fonts, 'face'),\n                fontSize   = Helper.reduceValue(fonts, 'size'),\n                fillColor  = Helper.reduceValue(views, 'fillColor'),\n                lineColor  = Helper.reduceValue(views, 'lineColor'),\n                fontColor  = Helper.reduceValue(views, 'fontColor'),\n                lineStyle  = Helper.reduceValue(views, 'lineStyle'),\n                autoResize = Helper.reduceValue(views, 'autoResize'),\n                showShadow = Helper.reduceValue(views, 'showShadow'),\n                wordWrap   = Helper.reduceValue(views, 'wordWrap'),\n                stereotypeDisplay = Helper.reduceValue(views, 'stereotypeDisplay'),\n                showVisibility = Helper.reduceValue(views, 'showVisibility'),\n                showNamespace  = Helper.reduceValue(views, 'showNamespace'),\n                showProperty   = Helper.reduceValue(views, 'showProperty'),\n                showType       = Helper.reduceValue(views, 'showType'),\n                suppressAttributes = Helper.reduceValue(views, 'suppressAttributes'),\n                suppressOperations = Helper.reduceValue(views, 'suppressOperations'),\n                suppressReceptions = Helper.reduceValue(views, 'suppressReceptions'),\n                suppressLiterals   = Helper.reduceValue(views, 'suppressLiterals');\n\n            _setFontFace(fontFace);\n            _setFontSize(fontSize);\n            _setFillColor(fillColor);\n            _setLineColor(lineColor);\n            _setFontColor(fontColor);\n            _setLineStyle(lineStyle);\n\n            _setCheckbox($autoResizeCheckbox, autoResize);\n            _setCheckbox($showShadowCheckbox, showShadow);\n            _setCheckbox($wordWrapCheckbox, wordWrap);\n            _setStereotypeDisplay(stereotypeDisplay);\n\n            _setCheckbox($showVisibilityCheckbox, showVisibility);\n            _setCheckbox($showNamespaceCheckbox, showNamespace);\n            _setCheckbox($showPropertyCheckbox, showProperty);\n            _setCheckbox($showTypeCheckbox, showType);\n\n            _setCheckbox($suppressAttributesCheckbox, suppressAttributes);\n            _setCheckbox($suppressOperationsCheckbox, suppressOperations);\n            _setCheckbox($suppressReceptionsCheckbox, suppressReceptions);\n            _setCheckbox($suppressLiteralsCheckbox, suppressLiterals);\n\n        } else {\n            $view.hide();\n        }\n    }\n\n    /**\n     * Show styles of a given a list of views\n     * @param {Array.<View>} views\n     */\n    function show(views) {\n        _views = views;\n        _updateStyles(views);\n    }\n\n    AppInit.htmlReady(function () {\n        var strings = {\n            defaultFonts: app.config.defaultFonts\n        };\n        $view = $(Mustache.render(viewTemplate, strings));\n        _setupStyleWidgets();\n        EditorView.setupEditorView($view);\n    });\n\n    AppInit.appReady(function () {\n        // Add font list to StyleEditor after loading all fonts\n        var $fontSelect = $view.find(\"label.font-face > select\");\n        _.each(app.config.defaultFonts, function (font) {\n            var s = \"<option value='{{value}}'>{{text}}</option>\";\n            $fontSelect.append(Mustache.render(s, font));\n        });\n    });\n\n    exports.show = show;\n\n});\n\n//# sourceURL=/editors/StyleEditorView.js"),eval("define('text!htmlContent/element-list-editor-dialog.html',[],function () { return '<div class=\"element-list-editor dialog modal\" data-title=\"{{name}}\">\\n    <div class=\"dialog-body no-padding\">\\n        <div class=\"listview-wrapper\">\\n            <div class=\"listview\"></div>\\n        </div>\\n    </div>\\n    <div class=\"dialog-footer\">\\n        <div class=\"left\" style=\"text-align: left;\">\\n            <button class=\\'k-button add-element\\'>Add<!--<span class=\\'k-sprite quick-edit-icon-add\\'></span>--></button>\\n            <button class=\\'k-button delete-element\\'>Remove<!--<span class=\\'k-sprite quick-edit-icon-delete\\'></span>--></button>\\n            <button class=\\'k-button move-up\\'>Up<!--<span class=\\'k-sprite quick-edit-icon-moveup\\'></span>--></button>\\n            <button class=\\'k-button move-down\\'>Down<!--<span class=\\'k-sprite quick-edit-icon-movedown\\'></span>--></button>\\n        </div>\\n        <button class=\"k-button dialog-button primary\" data-button-id=\"close\">{{Strings.CLOSE}}</button>\\n    </div>\\n</div>\\n';});\n\n//# sourceURL=/text!htmlContent/element-list-editor-dialog.html"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache */\n\n/**\n * Element List Editor Dialog\n */\ndefine(\'dialogs/ElementListEditorDialog\',[\'require\',\'exports\',\'module\',\'utils/Helper\',\'utils/ViewUtils\',\'core/Repository\',\'engine/Engine\',\'dialogs/ElementPickerDialog\',\'dialogs/Dialogs\',\'strings\',\'text!htmlContent/element-list-editor-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var Helper              = require("utils/Helper"),\n        ViewUtils           = require("utils/ViewUtils"),\n        Repository          = require("core/Repository"),\n        Engine              = require("engine/Engine"),\n        ElementPickerDialog = require("dialogs/ElementPickerDialog"),\n        Dialogs             = require("dialogs/Dialogs"),\n        Strings             = require("strings"),\n        dialogTemplate      = require("text!htmlContent/element-list-editor-dialog.html");\n\n    var UI_ELEMENTLIST_EDITOR = "#ui-element-list-editor",\n        UI_CLOSE_BUTTON       = "#ui-element-list-editor-button-close";\n\n\n    /**\n     * DataSource for ListView\n     * @private\n     * @type {kendo.data.DataSource}\n     */\n    var dataSource = new kendo.data.DataSource();\n\n    /**\n     * Convert Core.Model to DataSource Item\n     * @private\n     * @type {kendo.data.DataSource}\n     */\n    function _toDataItem(elem) {\n        return {\n            id: elem._id,\n            icon: elem.getNodeIcon(),\n            text: elem.getNodeText(),\n            path: elem.getPathname()\n        };\n    }\n\n    function updateDataSource(elem, field) {\n        if (_.isArray(elem[field])) {\n            dataSource.data([]);\n            var ref = elem[field];\n            for (var i = 0, len = ref.length; i < len; i++) {\n                var item = ref[i];\n                dataSource.add(_toDataItem(item));\n            }\n        }\n    }\n\n    /**\n     * Show Element List Editor Dialog.\n     *\n     * @param {Core.Model} elem\n     * @param {string} field Array field name to be listed\n     */\n    function showDialog(elem, field) {\n        var context = {\n            Strings: Strings,\n            name: elem.name + "." + field\n        };\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(dialogTemplate, context), true, function ($dlg) {\n\n        });\n\n        var $dlg = dialog.getElement(),\n            $listview = $dlg.find(".listview"),\n            $addElement = $dlg.find(".add-element"),\n            $deleteElement = $dlg.find(".delete-element"),\n            $moveUp = $dlg.find(".move-up"),\n            $moveDown = $dlg.find(".move-down");\n\n        var $wrapper = $dlg.find(".listview-wrapper");\n        ViewUtils.addScrollerShadow($wrapper, null, true);\n\n        // Setup ListView\n        updateDataSource(elem, field);\n        $listview.kendoListView({\n            dataSource: dataSource,\n            template: "<div style=\'padding: 0.2em;\'><span class=\'k-sprite #=icon#\'></span>#:text# (#=path#)</div>",\n            selectable: true\n        });\n        var listview = $listview.data("kendoListView");\n\n        // Add Button\n        $addElement.click(function () {\n            ElementPickerDialog.showDialog("Select Element to Add", null, null).done(function (buttonId, selected) {\n                if (buttonId === Dialogs.DIALOG_BTN_OK && selected !== null) {\n                    Engine.addItem(elem, field, selected);\n                    dataSource.add(_toDataItem(selected));\n                }\n            });\n        });\n\n        // Delete Button\n        $deleteElement.click(function () {\n            var selected = listview.select();\n            if (selected && selected.length > 0) {\n                var dataItem = dataSource.getByUid(selected[0].dataset.uid);\n                var item = Repository.get(dataItem.id);\n                if (item) {\n                    Engine.removeItem(elem, field, item);\n                    dataSource.remove(dataItem);\n                }\n            }\n        });\n\n        // Move Up Button\n        $moveUp.click(function () {\n            var selected = listview.select();\n            if (selected && selected.length > 0) {\n                var dataItem = dataSource.getByUid(selected[0].dataset.uid);\n                var item = Repository.get(dataItem.id);\n                if (item) {\n                    Engine.moveUp(elem, field, item);\n                    dataSource.remove(dataItem);\n                    dataSource.insert(_.indexOf(elem[field], item), dataItem);\n                }\n            }\n        });\n\n        // Move Down Button\n        $moveDown.click(function () {\n            var selected = listview.select();\n            if (selected && selected.length > 0) {\n                var dataItem = dataSource.getByUid(selected[0].dataset.uid);\n                var item = Repository.get(dataItem.id);\n                if (item) {\n                    Engine.moveDown(elem, field, item);\n                    dataSource.remove(dataItem);\n                    dataSource.insert(_.indexOf(elem[field], item), dataItem);\n                }\n            }\n        });\n\n        return dialog;\n    }\n\n    // Public API\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/dialogs/ElementListEditorDialog.js'),eval('define(\'text!htmlContent/property-editor-view.html\',[],function () { return \'<div class="editor-view" data-id="propertyEditor">\\n    <hr>\\n    <div class="editor-view-header">\\n        <span class="suppress-button"></span>\\n        <label class="header">Properties</label>\\n    </div>\\n    <div class="editor-view-body">\\n        <table class=\\\'property-grid small\\\'>\\n        </table>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view.html'),eval('define(\'text!htmlContent/property-editor-view-item-text.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <input type="text" class="k-input k-textbox small" value="{{val}}">\\n    </td>\\n</tr>\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-text.html'),eval('define(\'text!htmlContent/property-editor-view-item-multiline.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <!--\\n        <span style="display: inline-block; padding-right: 25px; position: relative; width: 100%; box-sizing: border-box;">\\n            <input type="text" class="k-input k-textbox small" value="{{val}}" disabled>\\n            <button class="k-button icon-button small quiet search-button" style="position: absolute; right: 0; top: 0;"></button>\\n        </span>\\n        -->\\n        <textarea class="k-textbox small" style="height: 70px;"></textarea>\\n    </td>\\n</tr>\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-multiline.html'),eval('define(\'text!htmlContent/property-editor-view-item-check.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <label class="k-checkbox">\\n            <input type="checkbox">\\n            <div class="k-checkbox__checkmark"></div>\\n        </label>\\n    </td>\\n</tr>\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-check.html'),eval('define(\'text!htmlContent/property-editor-view-item-integer.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <input type="text" class="k-input k-textbox small" value="{{val}}">\\n    </td>\\n</tr>\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-integer.html'),eval('define(\'text!htmlContent/property-editor-view-item-enum.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <label style="width: 100%; padding: 0; margin: 0;" class="k-select small">\\n            <select style="width: 100%;">\\n                {{#items}}\\n                <option value="{{.}}">{{.}}</option>\\n                {{/items}}\\n            </select>\\n        </label>\\n    </td>\\n</tr>\\n\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-enum.html'),eval('define(\'text!htmlContent/property-editor-view-item-combo.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <label class="k-combo small" style="width: 100%; margin: 0; padding: 0">\\n            <input type="text" style="width: 100%;">\\n            <select>\\n                {{#options}}\\n                <option value="{{.}}">{{.}}</option>\\n                {{/options}}\\n            </select>\\n        </label>\\n    </td>\\n</tr>\\n\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-combo.html'),eval('define(\'text!htmlContent/property-editor-view-item-ref.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <span style="display: inline-block; padding-right: 25px; position: relative; width: 100%; box-sizing: border-box;">\\n            <div style="display: inline-block">\\n                <span class="icon"></span>\\n                <span class="name">{{val.name}}</span>\\n            </div>\\n            <button tabindex="-1" id="{{buttonId}}" class="k-button icon-button small quiet search-button" style="position: absolute; right: 0; top: 0;"></button>\\n        </span>\\n    </td>\\n</tr>\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-ref.html'),eval('define(\'text!htmlContent/property-editor-view-item-reflist.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <span style="display: inline-block; padding-right: 25px; position: relative; width: 100%; box-sizing: border-box;">\\n            <div style="display: inline-block">\\n            {{#elements}}\\n                <div style="display: block">\\n                    <span class="k-sprite icon-{{getClassName}}"></span>\\n                    <span>{{name}}</span>\\n                </div>\\n            {{/elements}}\\n            {{^elements}}\\n                <span>—</span>\\n            {{/elements}}\\n            </div>\\n            <button class="k-button icon-button small quiet search-button" style="position: absolute; right: 0; top: 0;"></button>\\n        </span>\\n    </td>\\n</tr>\\n\\n\\n\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-reflist.html'),eval('define(\'text!htmlContent/property-editor-view-item-var.html\',[],function () { return \'<tr>\\n    <td>\\n        <span class="property-name">{{prefix}}</span>{{name}}\\n    </td>\\n    <td>\\n        <span style="display: inline-block; padding-right: 25px; position: relative; width: 100%; box-sizing: border-box;">\\n            {{#element}}\\n            <span class="k-sprite icon-{{element.getClassName}}" style="position: absolute; left: 4px; top: 3px; z-index: 1;"></span>\\n            {{/element}}\\n            <input type="text" class="small" value="{{val}}" style="width: 100%;">\\n            <button tabindex="-1" id="{{buttonId}}" class="k-button icon-button small quiet search-button" style="position: absolute; right: 0; top: 0;"></button>\\n        </span>\\n    </td>\\n</tr>\\n\';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-var.html'),eval("define('text!htmlContent/property-editor-view-item-separator.html',[],function () { return '<tr>\\n    <td colspan=\\'2\\' class=\\'ui-property-separator\\'>\\n        <div></div>\\n    </td>\\n</tr>';});\n\n//# sourceURL=/text!htmlContent/property-editor-view-item-separator.html"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, loopfunc: true */\n/*global define, $, _, window, type, meta, appshell, kendo, Mustache */\n\n/**\n *\n * This module dispatches these events:\n *     - propertyChanged: (element, field, value)\n */\ndefine(\'editors/PropertyEditorView\',[\'require\',\'exports\',\'module\',\'core/Core\',\'core/MetaModelManager\',\'core/Global\',\'utils/AppInit\',\'utils/Helper\',\'core/Repository\',\'dialogs/Dialogs\',\'dialogs/ElementPickerDialog\',\'dialogs/ElementListEditorDialog\',\'editors/EditorView\',\'text!htmlContent/property-editor-view.html\',\'text!htmlContent/property-editor-view-item-text.html\',\'text!htmlContent/property-editor-view-item-multiline.html\',\'text!htmlContent/property-editor-view-item-check.html\',\'text!htmlContent/property-editor-view-item-integer.html\',\'text!htmlContent/property-editor-view-item-enum.html\',\'text!htmlContent/property-editor-view-item-combo.html\',\'text!htmlContent/property-editor-view-item-ref.html\',\'text!htmlContent/property-editor-view-item-reflist.html\',\'text!htmlContent/property-editor-view-item-var.html\',\'text!htmlContent/property-editor-view-item-separator.html\',\'editors/EditorsHolder\',\'editors/StyleEditorView\'],function (require, exports, module) {\n    "use strict";\n\n    var Core                    = require("core/Core"),\n        MetaModelManager        = require("core/MetaModelManager"),\n        global                  = require("core/Global").global,\n        AppInit                 = require("utils/AppInit"),\n        Helper                  = require("utils/Helper"),\n        Repository              = require("core/Repository"),\n        Dialogs                 = require("dialogs/Dialogs"),\n        ElementPickerDialog     = require("dialogs/ElementPickerDialog"),\n        ElementListEditorDialog = require("dialogs/ElementListEditorDialog"),\n        EditorView              = require("editors/EditorView"),\n        viewTemplate            = require("text!htmlContent/property-editor-view.html"),\n        textItemTemplate        = require("text!htmlContent/property-editor-view-item-text.html"),\n        multilineItemTemplate   = require("text!htmlContent/property-editor-view-item-multiline.html"),\n        checkItemTemplate       = require("text!htmlContent/property-editor-view-item-check.html"),\n        integerItemTemplate     = require("text!htmlContent/property-editor-view-item-integer.html"),\n        enumItemTemplate        = require("text!htmlContent/property-editor-view-item-enum.html"),\n        comboItemTemplate       = require("text!htmlContent/property-editor-view-item-combo.html"),\n        refItemTemplate         = require("text!htmlContent/property-editor-view-item-ref.html"),\n        reflistItemTemplate     = require("text!htmlContent/property-editor-view-item-reflist.html"),\n        varItemTemplate         = require("text!htmlContent/property-editor-view-item-var.html"),\n        separatorItemTemplate   = require("text!htmlContent/property-editor-view-item-separator.html");\n\n    // for Loading Order\n    require("editors/EditorsHolder");\n    require("editors/StyleEditorView");\n\n    var $view = null;\n\n    /**\n     * JQuery Object for Property Grid\n     * @private\n     */\n    var $propertyGrid = null;\n\n\n    /**\n     * Kendo Widgets must be destroyed before showing new properties\n     */\n    var _destroyedBeforeShowing = [];\n\n\n    function _getStrings(prefix, name, elems, field) {\n        var strings = {},\n            name_   = name.replace(\'.\', \'-\'),\n            prefix_ = (prefix ? prefix.replace(\'.\', \'-\') : \'\');\n        strings.prefix    = (prefix ? prefix_ + \'.\' : \'\');\n        strings.name      = name;\n        strings.val       = Helper.reduceValue(elems, field);\n        return strings;\n    }\n\n    function _makeNode(elem) {\n        return {\n            _id: elem._id,\n            text: elem.getNodeText(),\n            sprite: elem.getNodeIcon(elem),\n            hasChildren: (elem.getChildNodes().length > 0),\n            _name: elem.name,\n            _namespace: elem._parent ? elem._parent.name || "" : ""\n        };\n    }\n\n    function _getMetaAttr(elem, field) {\n        var attrs = elem.getMetaAttributes();\n        var attr = _.find(attrs, function (e) {\n            return (e.name === field);\n        });\n        return attr;\n    }\n\n    /**\n     * 텍스트 프로퍼티를 정의.\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineTextProperty(prefix, name, elems, field) {\n        var strings = _getStrings(prefix, name, elems, field),\n            $item   = $(Mustache.render(textItemTemplate, strings)),\n            $edit   = $item.find("input");\n\n        $propertyGrid.append($item);\n\n        if (strings.val !== null) {\n            $edit.val(strings.val);\n            $edit.prop("placeholder", "");\n        } else {\n            $edit.val("");\n            $edit.prop("placeholder", "—");\n        }\n\n        $edit.change(function () {\n            $(exports).triggerHandler(\'propertyChanged\', [elems, field, $edit.val()]);\n        });\n    }\n\n    /**\n     * Define Multiline Property\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineMultilineTextProperty(prefix, name, elems, field) {\n        var strings = _getStrings(prefix, name, elems, field),\n            $item   = $(Mustache.render(multilineItemTemplate, strings)),\n            $edit   = $item.find("textarea"),\n            $button = $item.find("button");\n\n        $propertyGrid.append($item);\n\n        if (strings.val !== null) {\n            $edit.val(strings.val);\n            $edit.prop("placeholder", "");\n        } else {\n            $edit.val("");\n            $edit.prop("placeholder", "—");\n        }\n\n        $edit.change(function () {\n            $(exports).triggerHandler(\'propertyChanged\', [elems, field, $edit.val()]);\n        });\n    }\n\n    /**\n     * 체크 프로퍼티를 정의.\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineCheckProperty(prefix, name, elems, field) {\n        var strings = _getStrings(prefix, name, elems, field),\n            $item   = $(Mustache.render(checkItemTemplate, strings)),\n            $check  = $item.find("input");\n\n        $propertyGrid.append($item);\n\n        if (strings.val !== null) {\n            $check.attr(\'checked\', strings.val);\n        } else {\n            $check.attr(\'checked\', false);\n        }\n\n        $check.change(function () {\n            $(exports).triggerHandler(\'propertyChanged\', [elems, field, $check.is(":checked")]);\n        });\n    }\n\n    /**\n     * 정수형 프로퍼티를 정의.\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {Array.<string>} items\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineIntegerProperty(prefix, name, elems, field) {\n        var strings = _getStrings(prefix, name, elems, field),\n            $item   = $(Mustache.render(integerItemTemplate, strings)),\n            $edit   = $item.find("input");\n\n        $propertyGrid.append($item);\n\n        if (strings.val !== null) {\n            $edit.val(strings.val);\n            $edit.prop("placeholder", "");\n        } else {\n            $edit.val("");\n            $edit.prop("placeholder", "—");\n        }\n\n        $edit.change(function () {\n            var value = parseInt($edit.val());\n            if (_.isNumber(value)) {\n                $(exports).triggerHandler(\'propertyChanged\', [elems, field, value]);\n            }\n        });\n    }\n\n    /**\n     * 열거형 프로퍼티를 정의.\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {{literals: Array.<{string}>}} enumType\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineEnumProperty(prefix, name, enumType, elems, field) {\n        var strings = _getStrings(prefix, name, elems, field);\n        strings.items = enumType.literals;\n\n        var $item   = $(Mustache.render(enumItemTemplate, strings)),\n            $select = $item.find("select");\n\n        $propertyGrid.append($item);\n\n        if (strings.val !== null) {\n            $select.val(strings.val);\n        } else {\n            $select.val(null);\n        }\n\n        $select.change(function () {\n            $(exports).triggerHandler(\'propertyChanged\', [elems, field, $select.val()]);\n        });\n    }\n\n    /**\n     * 콤보형 프로퍼티를 정의.\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {Array.<string>} options\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineComboProperty(prefix, name, options, elems, field) {\n        var strings = _getStrings(prefix, name, elems, field);\n        strings.options = options;\n\n        var $item   = $(Mustache.render(comboItemTemplate, strings)),\n            $edit   = $item.find("input"),\n            $select = $item.find("select");\n\n        $propertyGrid.append($item);\n\n        if (strings.val !== null) {\n            $edit.val(strings.val);\n            $edit.prop("placeholder", "");\n        } else {\n            $edit.val("");\n            $edit.prop("placeholder", "—");\n        }\n\n        $edit.change(function () {\n            $(exports).triggerHandler(\'propertyChanged\', [elems, field, $edit.val()]);\n        });\n\n        $select.val(strings.val);\n        $select.change(function () {\n            $edit.val($select.val());\n            $edit.change();\n        });\n        // (for Windows) To prevent immediate collapse of dropdown in comboBox.\n        $select.click(function (e) {\n            $select.focus();\n            e.stopPropagation();\n            return false;\n        });\n\n    }\n\n    /**\n     * 레퍼런스 프로퍼티를 정의.\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineRefProperty(prefix, name, elems, field, readOnly) {\n        var strings = _getStrings(prefix, name, elems, field);\n        if (!(strings.val instanceof type.Model)) {\n            strings.val = null;\n        }\n\n        var $item   = $(Mustache.render(refItemTemplate, strings)),\n            $icon   = $item.find(".icon"),\n            $name   = $item.find(".name"),\n            $button = $item.find("button");\n\n        $propertyGrid.append($item);\n\n        function _setElement(elem) {\n            if (elem instanceof type.Model) {\n                $icon.show();\n                $icon.removeClass();\n                $icon.addClass("k-sprite " + elem.getNodeIcon());\n                $name.html(elem.name);\n            } else {\n                $icon.hide();\n                $name.html("—");\n            }\n        }\n\n        _setElement(strings.val);\n\n        if (readOnly) {\n            $button.hide();\n        }\n\n        var metaAttr = _getMetaAttr(elems[0], field);\n        var selectableType = null;\n        if (metaAttr) {\n            selectableType = type[metaAttr.type];\n        }\n\n        $button.click(function () {\n            var initialSelection = Helper.reduceValue(elems, field);\n            ElementPickerDialog.showDialog("Select Element", initialSelection, selectableType).done(function (buttonId, selected) {\n                if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                    _setElement(selected);\n                    $(exports).triggerHandler(\'propertyChanged\', [elems, field, selected]);\n                }\n            });\n        });\n    }\n\n    /**\n     * 레퍼런스 리스트 프로퍼티를 정의.\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineRefListProperty(prefix, name, elems, field) {\n        var strings = _getStrings(prefix, name, elems, field);\n        if (elems.length === 1) {\n            strings.elements = elems[0][field];\n        }\n\n        var $item   = $(Mustache.render(reflistItemTemplate, strings)),\n            $button = $item.find("button");\n\n        $propertyGrid.append($item);\n\n        $button.click(function () {\n            ElementListEditorDialog.showDialog(elems[0], field);\n        });\n    }\n\n    /**\n     * Variant 타입 프로퍼티를 정의.\n     *\n     * @private\n     * @param {string} prefix?\n     * @param {string} name\n     * @param {Element} elems\n     * @param {string} field\n     */\n    function defineVarProperty(prefix, name, attrType, elems, field) {\n        var strings = _getStrings(prefix, name, elems, field);\n        if (strings.val instanceof type.Model) {\n            strings.element = strings.val;\n            strings.val = strings.val.name;\n        }\n\n        var $item   = $(Mustache.render(varItemTemplate, strings)),\n            $edit   = $item.find("input"),\n            $button = $item.find("button");\n\n        $propertyGrid.append($item);\n\n        var _autoCompleteSelected = null;\n\n        var autoComplete  = $edit.kendoAutoComplete({\n            dataTextField: "text",\n            minLength: 1,\n            filter: "contains",\n            select: function(e) {\n                var item = this.dataItem(e.item.index());\n                var elem = Repository.get(item._id);\n                if (elem) {\n                    _autoCompleteSelected = elem;\n                    $(exports).triggerHandler(\'propertyChanged\', [elems, field, _autoCompleteSelected]);\n                }\n            },\n            change: function (e) {\n                var value = this.value();\n                if (!_autoCompleteSelected || _autoCompleteSelected.name !== value) {\n                    var ref = null;\n                    var typeFilter = type[attrType];\n                    // Lookup element by name and type\n                    if (value && value.length > 0) {\n                        if (elems.length > 0 && elems[0]._parent !== null) {\n                            ref = Repository.lookupAndFind(elems[0]._parent, value, typeFilter);\n                        }\n                    }\n                    $(exports).triggerHandler(\'propertyChanged\', [elems, field, (ref ? ref : value)]);\n                    _autoCompleteSelected = null;\n                }\n            },\n            template: "<div style=\'white-space: nowrap\'>" +\n                "<span class=\'k-sprite #:data.sprite#\'></span>" +\n                "<span style=\'margin-left: 5px\'>#:data.text#</span>" +\n                "#if (data._namespace.length > 0) {# <span style=\'margin-left: 5px; font-size: 11px; color: rgb(139,139,139);\'> — #:data._namespace#</span> #}#<div>",\n            dataSource: {\n                transport: {\n                    read: function (options) {\n                        if (options.data.filter && options.data.filter.filters.length > 0) {\n                            var keyword = options.data.filter.filters[0].value;\n                            var results = Repository.search(keyword, type[attrType]);\n                            options.success(_.map(results, _makeNode));\n                        } else {\n                            options.success([]);\n                        }\n                    }\n                },\n                serverFiltering: true\n            }\n        }).data("kendoAutoComplete");\n\n        // autoComplete must be destroyed\n        _destroyedBeforeShowing.push(autoComplete);\n\n\n        if (strings.val !== null) {\n            $edit.val(strings.val);\n            $edit.prop("placeholder", "");\n        } else {\n            $edit.val("");\n            $edit.prop("placeholder", "—");\n        }\n\n        // Make a space to show icon on left side.\n        if (strings.element) {\n            var $autocomplete = $item.find("span.k-autocomplete");\n            // $edit.css("padding-left", "1.8em");\n            $autocomplete.css("padding-left", "1.8em");\n            $autocomplete.css("background-color", $edit.css("background-color"));\n        }\n\n        /*\n        $edit.change(function () {\n            var val = $edit.val();\n            var ref = null;\n            var typeFilter = type[attrType];\n            // Lookup element by name and type\n            if (val && val.length > 0) {\n                if (elems.length > 0 && elems[0]._parent !== null) {\n                    ref = Repository.lookupAndFind(elems[0]._parent, val, typeFilter);\n                }\n            }\n            $(exports).triggerHandler(\'propertyChanged\', [elems, field, (ref ? ref : val)]);\n        });\n        */\n\n        $button.click(function () {\n            var initialSelection = Helper.reduceValue(elems, field);\n            ElementPickerDialog.showDialog("Select Type", initialSelection, type.UMLClassifier).done(function (buttonId, selected) {\n                if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                    $edit.val((selected ? selected.name : \'\'));\n                    $(exports).triggerHandler(\'propertyChanged\', [elems, field, selected]);\n                }\n            });\n        });\n    }\n\n    function defineAttributes(prefix, elems, attrs) {\n        _.forEach(attrs, function (attr) {\n            if (attr.visible === true) {\n                switch (attr.kind) {\n                case Core.ATTR_KIND_PRIM:\n                    switch (attr.type) {\n                    case \'String\':\n                        if (attr.options) {\n                            defineComboProperty(prefix, attr.name, attr.options, elems, attr.name);\n                        } else if (attr.multiline === true) {\n                            defineMultilineTextProperty(prefix, attr.name, elems, attr.name);\n                        } else {\n                            defineTextProperty(prefix, attr.name, elems, attr.name);\n                        }\n                        break;\n                    case \'Boolean\':\n                        defineCheckProperty(prefix, attr.name, elems, attr.name);\n                        break;\n                    case \'Integer\':\n                        defineIntegerProperty(prefix, attr.name, elems, attr.name);\n                        break;\n                    }\n                    break;\n                case Core.ATTR_KIND_ENUM:\n                    defineEnumProperty(prefix, attr.name, meta[attr.type], elems, attr.name);\n                    break;\n                case Core.ATTR_KIND_REF:\n                    defineRefProperty(prefix, attr.name, elems, attr.name, attr.readOnly);\n                    break;\n                case Core.ATTR_KIND_REFS:\n                    defineRefListProperty(prefix, attr.name, elems, attr.name);\n                    break;\n                case Core.ATTR_KIND_VAR:\n                    defineVarProperty(prefix, attr.name, attr.type, elems, attr.name);\n                    break;\n                }\n            }\n        });\n    }\n\n    /**\n     * Show properties of a given element.\n     *\n     * @param {Core.Element} elem\n     */\n    function show(elems) {\n        if (elems && elems.length > 0) {\n            var i, len;\n            // Destroy previous kendo widgets (e.g. AutoComplete)\n            if (_destroyedBeforeShowing.length > 0) {\n                for (i = 0, len = _destroyedBeforeShowing.length; i < len; i++) {\n                    _destroyedBeforeShowing[i].destroy();\n                }\n            }\n\n            $view.show();\n            $propertyGrid.empty();\n            var commonType = Core.getCommonType(elems);\n            var attrs = MetaModelManager.getMetaAttributes(commonType.name);\n            defineAttributes(null, elems, attrs);\n            for (i = 0, len = attrs.length; i < len; i++) {\n                var attr = attrs[i];\n                if (attr.expand === true) {\n                    var name = attr.name;\n                    var values = _.map(elems, function (e) { return e[name]; });\n                    var commonType2 = Core.getCommonType(values);\n                    var attrs2 = MetaModelManager.getMetaAttributes(commonType2.name);\n                    $propertyGrid.append(Mustache.render(separatorItemTemplate, { name: name, type: commonType2.name }));\n                    defineAttributes(name, values, attrs2);\n                }\n            }\n        } else {\n            $view.hide();\n        }\n    }\n\n    AppInit.htmlReady(function () {\n        $view = $(viewTemplate);\n        $propertyGrid = $view.find(".property-grid");\n        EditorView.setupEditorView($view);\n    });\n\n    // Define public API\n    exports.show = show;\n\n});\n\n\n//# sourceURL=/editors/PropertyEditorView.js'),eval('define(\'text!htmlContent/documentation-editor-view.html\',[],function () { return \'<div class="editor-view" data-id="documentationEditor">\\n    <hr>\\n    <div class="editor-view-header">\\n        <span class="suppress-button"></span>\\n        <label class="header">Documentation</label>\\n    </div>\\n    <div class="editor-view-body" style="padding-left: 1em; padding-right: 1em; padding-bottom: 1em; width: 100%; padding-top: 0.5em; box-sizing: border-box;">\\n        <textarea id=\\\'documentation\\\' class=\\\'k-textbox small\\\' style="width: 100%; height: 180px;"></textarea>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/documentation-editor-view.html'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo */\n\n/**\n *\n * This module dispatches these events:\n *     - documentationChanged: (element, documentation)\n */\ndefine('editors/DocumentationEditorView',['require','exports','module','utils/Helper','utils/AppInit','core/Repository','editors/EditorView','text!htmlContent/documentation-editor-view.html','editors/EditorsHolder','editors/PropertyEditorView'],function (require, exports, module) {\n    \"use strict\";\n\n    var Helper       = require(\"utils/Helper\"),\n        AppInit      = require(\"utils/AppInit\"),\n        Repository   = require(\"core/Repository\"),\n        EditorView   = require(\"editors/EditorView\"),\n        viewTemplate = require(\"text!htmlContent/documentation-editor-view.html\");\n\n    // for Loading Order\n    require(\"editors/EditorsHolder\");\n    require(\"editors/PropertyEditorView\");\n\n    var $view = null;\n\n    var _element = null;\n\n    /**\n     * Show documentation of a given element\n     *\n     * @param {Model} elem\n     */\n    function show(elem) {\n        if (elem instanceof type.ExtensibleModel) {\n            $view.show();\n            _element = elem;\n            $(\"#documentation\").val(_element.documentation);\n        } else {\n            $view.hide();\n            _element = null;\n        }\n    }\n\n    AppInit.htmlReady(function () {\n        $view = $(viewTemplate);\n        var $documentation = $view.find(\"#documentation\");\n        $documentation.change(function () {\n            $(exports).triggerHandler('documentationChanged', [_element, $documentation.val()]);\n        });\n        EditorView.setupEditorView($view);\n    });\n\n    // Define public API\n    exports.show = show;\n\n});\n\n//# sourceURL=/editors/DocumentationEditorView.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo */\n\n/**\n * SidebarView\n *\n * This contains ToolboxView and WorkingDiagramView\n */\ndefine(\'diagrams/SidebarView\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'command/CommandManager\',\'command/Commands\',\'strings\',\'utils/Resizer\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit             = require("utils/AppInit"),\n        CommandManager      = require("command/CommandManager"),\n        Commands            = require("command/Commands"),\n        Strings             = require("strings"),\n        Resizer             = require("utils/Resizer");\n\n    var $sidebar,\n        $workingDiagrams,\n        $toolbox;\n\n    var WORKING_DIAGRAM_MIN_HEIGHT = 40;\n\n    /**\n     * Toggle SidebarView\n     */\n    function toggle() {\n        Resizer.toggle($sidebar);\n    }\n\n    /**\n     * Show SidebarView\n     */\n    function show() {\n        Resizer.show($sidebar);\n    }\n\n    /**\n     * Hide SidebarView\n     */\n    function hide() {\n        Resizer.hide($sidebar);\n    }\n\n    /**\n     * Return whether SidebarView is visible or not\n     *\n     * @return {booean}\n     */\n    function isVisible() {\n        return Resizer.isVisible($sidebar);\n    }\n\n\n    function _limitToolboxHeight() {\n        if (($sidebar.height() - WORKING_DIAGRAM_MIN_HEIGHT) < $toolbox.height()) {\n            $toolbox.height($sidebar.height() - WORKING_DIAGRAM_MIN_HEIGHT);\n            $workingDiagrams.find(".view-content").triggerHandler("scroll");\n            $toolbox.find(".view-content").triggerHandler("scroll");\n        }\n    }\n\n    AppInit.htmlReady(function () {\n        $sidebar = $("#sidebar");\n        $workingDiagrams = $("#working-diagrams");\n        $toolbox = $("#toolbox");\n\n        var $viewContents = $sidebar.find(".view-content");\n\n        // Sidebar Resizing\n\n        $sidebar.on("panelResizeStart", function (evt, width) {\n            $sidebar.find(".scroller-shadow").css("display", "none");\n        });\n\n        $sidebar.on("panelResizeUpdate", function (evt, width) {\n        });\n\n        $sidebar.on("panelResizeEnd", function (evt, width) {\n            $sidebar.find(".scroller-shadow").css("display", "block");\n            $workingDiagrams.find(".view-content").triggerHandler("scroll");\n            $toolbox.find(".view-content").triggerHandler("scroll");\n        });\n\n        $sidebar.on("panelCollapsed", function (evt, width) {\n            CommandManager.get(Commands.VIEW_HIDE_SIDEBAR).setName(Strings.CMD_VIEW_SHOW_SIDEBAR);\n        });\n\n        $sidebar.on("panelExpanded", function (evt, width) {\n            $sidebar.find(".scroller-shadow").css("display", "block");\n            $workingDiagrams.find(".view-content").triggerHandler("scroll");\n            $toolbox.find(".view-content").triggerHandler("scroll");\n            CommandManager.get(Commands.VIEW_HIDE_SIDEBAR).setName(Strings.CMD_VIEW_HIDE_SIDEBAR);\n        });\n\n        // AppInit.htmlReady in utils/Resizer executes before, so it\'s possible that the sidebar\n        // is collapsed before we add the event. Check here initially\n        if (!$sidebar.is(":visible")) {\n            $sidebar.trigger("panelCollapsed");\n        }\n\n\n        // Toolbox Resizing\n\n        $toolbox.on("panelResizeStart", function (evt, height) {\n            $toolbox.find(".scroller-shadow").css("display", "none");\n        });\n\n        $toolbox.on("panelResizeUpdate", function (evt, height) {\n            _limitToolboxHeight();\n        });\n\n        $toolbox.on("panelResizeEnd", function (evt, height) {\n            $toolbox.find(".scroller-shadow").css("display", "block");\n            $workingDiagrams.find(".view-content").triggerHandler("scroll");\n            $toolbox.find(".view-content").triggerHandler("scroll");\n        });\n\n        $toolbox.on("panelCollapsed", function (evt, height) {\n        });\n\n        $toolbox.on("panelExpanded", function (evt, height) {\n            $toolbox.find(".scroller-shadow").css("display", "block");\n            $workingDiagrams.find(".view-content").triggerHandler("scroll");\n            $toolbox.find(".view-content").triggerHandler("scroll");\n        });\n\n        // Window Resizing\n\n        $(window).resize(function () {\n            $workingDiagrams.find(".view-content").triggerHandler("scroll");\n            $toolbox.find(".view-content").triggerHandler("scroll");\n            if ($sidebar.height() > WORKING_DIAGRAM_MIN_HEIGHT) {\n                _limitToolboxHeight();\n            }\n        });\n\n    });\n\n    CommandManager.register(Strings.CMD_VIEW_HIDE_SIDEBAR, Commands.VIEW_HIDE_SIDEBAR, toggle);\n\n    exports.toggle    = toggle;\n    exports.show      = show;\n    exports.hide      = hide;\n    exports.isVisible = isVisible;\n\n});\n\n//# sourceURL=/diagrams/SidebarView.js'),eval("/**!\n * Sortable\n * @author	RubaXa   <trash@rubaxa.org>\n * @license MIT\n */\n\n\n(function (factory){\n	\"use strict\";\n\n	if( typeof define === \"function\" && define.amd ){\n		define('thirdparty/sortable/Sortable',factory);\n	}\n	else if( typeof module != \"undefined\" && typeof module.exports != \"undefined\" ){\n		module.exports = factory();\n	}\n	else {\n		window[\"Sortable\"] = factory();\n	}\n})(function (){\n	\"use strict\";\n\n	var\n		  dragEl\n		, ghostEl\n		, rootEl\n		, nextEl\n\n		, lastEl\n		, lastCSS\n		, lastRect\n\n		, activeGroup\n\n		, tapEvt\n		, touchEvt\n\n		, expando = 'Sortable' + (new Date).getTime()\n\n		, win = window\n		, document = win.document\n		, parseInt = win.parseInt\n		, supportIEdnd = !!document.createElement('div').dragDrop\n\n		, _silent = false\n\n		, _createEvent = function (event/**String*/, item/**HTMLElement*/){\n			var evt = document.createEvent('Event');\n			evt.initEvent(event, true, true);\n			evt.item = item;\n			return evt;\n		}\n\n		, noop = function (){}\n		, slice = [].slice\n\n		, touchDragOverListeners = []\n\n		, pointerdown\n		, pointerup\n		, pointermove\n		, pointercancel\n	;\n\n\n	/**\n	 * @class  Sortable\n	 * @param  {HTMLElement}  el\n	 * @param  {Object}  [options]\n	 * @constructor\n	 */\n	function Sortable(el, options){\n		this.el = el; // root element\n		this.options = options = (options || {});\n\n\n		// Defaults\n		options.group = options.group || Math.random();\n		options.handle = options.handle || null;\n		options.draggable = options.draggable || el.children[0] && el.children[0].nodeName || (/[uo]l/i.test(el.nodeName) ? 'li' : '*');\n		options.ghostClass = options.ghostClass || 'sortable-ghost';\n		options.ignore = options.ignore || 'a, img';\n\n		options.onAdd = _bind(this, options.onAdd || noop);\n		options.onUpdate = _bind(this, options.onUpdate || noop);\n		options.onRemove = _bind(this, options.onRemove || noop);\n		options.onStart = _bind(this, options.onStart || noop);\n		options.onEnd = _bind(this, options.onEnd || noop);\n\n\n		// Export group name\n		el[expando] = options.group;\n\n\n		// Bind all private methods\n		for( var fn in this ){\n			if( fn.charAt(0) === '_' ){\n				this[fn] = _bind(this, this[fn]);\n			}\n		}\n\n		// Detect IE10/IE11+\n		if (window.onpointerdown !== undefined) {\n			pointerdown = 'pointerdown';\n			pointerup = 'pointerup';\n			pointermove = 'pointermove';\n			pointercancel = 'pointercancel';\n		} else {\n			pointerdown = 'MSPointerDown';\n			pointerup = 'MSPointerUp';\n			pointermove = 'MSPointerMove';\n			pointercancel = 'MSPointerCancel';\n		}\n\n		// Bind events\n		_on(el, 'add', options.onAdd);\n		_on(el, 'update', options.onUpdate);\n		_on(el, 'remove', options.onRemove);\n		_on(el, 'start', options.onStart);\n		_on(el, 'stop', options.onEnd);\n\n		_on(el, 'mousedown', this._onTapStart);\n		_on(el, 'touchstart', this._onTapStart);\n		supportIEdnd && _on(el, 'selectstart', this._onTapStart);\n\n		_on(el, 'dragover', this._onDragOver);\n		_on(el, 'dragenter', this._onDragOver);\n		_on(el, pointerdown, this._onTapStart);\n\n		_css(el, 'touch-action', 'none');\n		_css(el, '-ms-touch-action', 'none');\n\n		touchDragOverListeners.push(this._onDragOver);\n	}\n\n\n	Sortable.prototype = {\n		constructor: Sortable,\n\n\n		_applyEffects: function (){\n			_toggleClass(dragEl, this.options.ghostClass, true);\n		},\n\n\n		_onTapStart: function (evt/**Event|TouchEvent|PointerEvent*/){\n			var\n				  touch = evt.touches && evt.touches[0]\n				, target = (touch || evt).target\n				, options =  this.options\n				, el = this.el\n			;\n\n			if( options.handle ){\n				target = _closest(target, options.handle, el);\n			}\n\n			target = _closest(target, options.draggable, el);\n\n			// IE 9 Support\n			if( target && evt.type == 'selectstart' ){\n				if( target.tagName != 'A' && target.tagName != 'IMG'){\n					target.dragDrop();\n				}\n			}\n\n			if( target && !dragEl && (target.parentNode === el) ){\n				tapEvt = evt;\n				target.draggable = true;\n\n				// Disable \"draggable\"\n				Array.prototype.forEach.call(options.ignore.split(','), function (criteria) {\n					_find(target, criteria.trim(), _disableDraggable);\n				});\n\n				if( touch ){\n					// Touch device support\n					tapEvt = {\n						  target:  target\n						, clientX: touch.clientX\n						, clientY: touch.clientY\n					};\n					this._onDragStart(tapEvt, true);\n					evt.preventDefault();\n				}\n				\n				if (evt.type == 'pointerdown' || evt.type == 'MSPointerDown') {\n					this._onDragStart(tapEvt, true);\n					evt.preventDefault();\n				}\n\n				_on(this.el, 'dragstart', this._onDragStart);\n				_on(this.el, 'dragend', this._onDrop);\n				_on(document, 'dragover', _globalDragOver);\n\n\n				try {\n					if( document.selection ){\n						document.selection.empty();\n					} else {\n						window.getSelection().removeAllRanges()\n					}\n				} catch (err){ }\n			}\n		},\n\n\n		_emulateDragOver: function (){\n			if( touchEvt ){\n				_css(ghostEl, 'display', 'none');\n\n				var\n					  target = document.elementFromPoint(touchEvt.clientX, touchEvt.clientY)\n					, parent = target\n					, group = this.options.group\n					, i = touchDragOverListeners.length\n				;\n\n				if( parent ){\n					do {\n						if( parent[expando] === group ){\n							while( i-- ){\n								touchDragOverListeners[i]({\n									clientX: touchEvt.clientX,\n									clientY: touchEvt.clientY,\n									target: target,\n									rootEl: parent\n								});\n							}\n							break;\n						}\n\n						target = parent; // store last element\n					}\n					while( parent = parent.parentNode );\n				}\n\n				_css(ghostEl, 'display', '');\n			}\n		},\n\n\n		_onTouchMove: function (evt/**TouchEvent|PointerEvent*/){\n			if( tapEvt ){\n				var\n					  touch = evt.touches[0]\n					, dx = touch.clientX - tapEvt.clientX\n					, dy = touch.clientY - tapEvt.clientY\n				;\n\n				touchEvt = touch;\n				_css(ghostEl, 'webkitTransform', 'translate3d('+dx+'px,'+dy+'px,0)');\n				_css(ghostEl, 'mozTransform', 'translate3d('+dx+'px,'+dy+'px,0)');\n				_css(ghostEl, 'msTransform', 'translate3d('+dx+'px,'+dy+'px,0)');\n				_css(ghostEl, 'transform', 'translate3d('+dx+'px,'+dy+'px,0)');\n				evt.preventDefault();\n			}\n		},\n\n\n		_onDragStart: function (evt/**Event*/, isTouch/**Boolean*/){\n			var\n				  target = evt.target\n				, dataTransfer = evt.dataTransfer\n			;\n\n			rootEl = this.el;\n			dragEl = target;\n			nextEl = target.nextSibling;\n			activeGroup = this.options.group;\n\n			if( isTouch ){\n				var\n					  rect = target.getBoundingClientRect()\n					, css = _css(target)\n					, ghostRect\n				;\n\n				ghostEl = target.cloneNode(true);\n\n				_css(ghostEl, 'top', rect.top - parseInt(css.marginTop, 10));\n				_css(ghostEl, 'left', rect.left - parseInt(css.marginLeft, 10));\n				_css(ghostEl, 'width', rect.width);\n				_css(ghostEl, 'height', rect.height);\n				_css(ghostEl, 'opacity', '0.8');\n				_css(ghostEl, 'position', 'fixed');\n				_css(ghostEl, 'zIndex', '100000');\n\n				rootEl.appendChild(ghostEl);\n\n				// Fixing dimensions.\n				ghostRect = ghostEl.getBoundingClientRect();\n				_css(ghostEl, 'width', rect.width*2 - ghostRect.width);\n				_css(ghostEl, 'height', rect.height*2 - ghostRect.height);\n\n				// Bind touch events\n				_on(document, 'touchmove', this._onTouchMove);\n				_on(document, 'touchend', this._onDrop);\n				_on(document, 'touchcancel', this._onDrop);\n				_on(document, pointermove, this._onTouchMove);\n				_on(document, pointerup, this._onDrop);\n				_on(document, pointercancel, this._onDrop);\n\n				this._loopId = setInterval(this._emulateDragOver, 150);\n			}\n			else {\n				dataTransfer.effectAllowed = 'move';\n				dataTransfer.setData('Text', target.textContent);\n\n				_on(document, 'drop', this._onDrop);\n			}\n\n			dragEl.dispatchEvent(_createEvent('start', dragEl));\n			setTimeout(this._applyEffects);\n		},\n\n\n		_onDragOver: function (evt/**Event*/){\n			if( !_silent && (activeGroup === this.options.group) && (evt.rootEl === void 0 || evt.rootEl === this.el) ){\n				var\n					  el = this.el\n					, target = _closest(evt.target, this.options.draggable, el)\n				;\n\n				if( el.children.length === 0 || el.children[0] === ghostEl || (el === evt.target) && _ghostInBottom(el, evt) ){\n					el.appendChild(dragEl);\n				}\n				else if( target && target !== dragEl && (target.parentNode[expando] !== void 0) ){\n					if( lastEl !== target ){\n						lastEl = target;\n						lastCSS = _css(target);\n						lastRect = target.getBoundingClientRect();\n					}\n\n\n					var\n						  rect = lastRect\n						, width = rect.right - rect.left\n						, height = rect.bottom - rect.top\n						, floating = /left|right|inline/.test(lastCSS.cssFloat + lastCSS.display)\n						, skew = (floating ? (evt.clientX - rect.left)/width : (evt.clientY - rect.top)/height) > .5\n						, isWide = (target.offsetWidth > dragEl.offsetWidth)\n						, isLong = (target.offsetHeight > dragEl.offsetHeight)\n						, nextSibling = target.nextSibling\n						, after\n					;\n\n					_silent = true;\n					setTimeout(_unsilent, 30);\n\n					if( floating ){\n						after = (target.previousElementSibling === dragEl) && !isWide || (skew > .5) && isWide\n					} else {\n						after = (target.nextElementSibling !== dragEl) && !isLong || (skew > .5) && isLong;\n					}\n\n					if( after && !nextSibling ){\n						el.appendChild(dragEl);\n					} else {\n						target.parentNode.insertBefore(dragEl, after ? nextSibling : target);\n					}\n				}\n			}\n		},\n\n\n		_onDrop: function (evt/**Event*/){\n			clearInterval(this._loopId);\n\n			// Unbind events\n			_off(document, 'drop', this._onDrop);\n			_off(document, 'dragover', _globalDragOver);\n\n			_off(this.el, 'dragend', this._onDrop);\n			_off(this.el, 'dragstart', this._onDragStart);\n			_off(this.el, 'selectstart', this._onTapStart);\n\n			_off(document, 'touchmove', this._onTouchMove);\n			_off(document, 'touchend', this._onDrop);\n			_off(document, 'touchcancel', this._onDrop);\n			_off(document, pointermove, this._onTouchMove);\n			_off(document, pointerup, this._onDrop);\n			_off(document, pointercancel, this._onDrop);\n\n\n			if( evt ){\n				evt.preventDefault();\n				evt.stopPropagation();\n\n				if( ghostEl ){\n					ghostEl.parentNode.removeChild(ghostEl);\n				}\n\n				if( dragEl ){\n					_disableDraggable(dragEl);\n					_toggleClass(dragEl, this.options.ghostClass, false);\n\n					if( !rootEl.contains(dragEl) ){\n						// Remove event\n						rootEl.dispatchEvent(_createEvent('remove', dragEl));\n\n						// Add event\n						dragEl.dispatchEvent(_createEvent('add', dragEl));\n					}\n					else if( dragEl.nextSibling !== nextEl ){\n						// Update event\n						dragEl.dispatchEvent(_createEvent('update', dragEl));\n					}\n					dragEl.dispatchEvent(_createEvent('stop', dragEl));\n				}\n\n				// Set NULL\n				rootEl =\n				dragEl =\n				ghostEl =\n				nextEl =\n\n				tapEvt =\n				touchEvt =\n\n				lastEl =\n				lastCSS =\n\n				activeGroup = null;\n			}\n		},\n\n\n		destroy: function (){\n			var el = this.el, options = this.options;\n\n			_off(el, 'add', options.onAdd);\n			_off(el, 'update', options.onUpdate);\n			_off(el, 'remove', options.onRemove);\n			_off(el, 'start', options.onStart);\n			_off(el, 'stop', options.onEnd);\n			_off(el, 'mousedown', this._onTapStart);\n			_off(el, 'touchstart', this._onTapStart);\n			_off(el, 'selectstart', this._onTapStart);\n			_off(el, pointerdown, this._onTapStart);\n\n			_off(el, 'dragover', this._onDragOver);\n			_off(el, 'dragenter', this._onDragOver);\n\n			//remove draggable attributes\n			Array.prototype.forEach.call(el.querySelectorAll('[draggable]'), function(el) {\n				el.removeAttribute('draggable');\n			});\n\n			touchDragOverListeners.splice(touchDragOverListeners.indexOf(this._onDragOver), 1);\n\n			this._onDrop();\n\n			this.el = null;\n		}\n	};\n\n	function _bind(ctx, fn){\n		var args = slice.call(arguments, 2);\n		return	fn.bind ? fn.bind.apply(fn, [ctx].concat(args)) : function (){\n			return fn.apply(ctx, args.concat(slice.call(arguments)));\n		};\n	}\n\n\n	function _closest(el, selector, ctx){\n		if( selector === '*' ){\n			return el;\n		}\n		else if( el ){\n			ctx = ctx || document;\n			selector = selector.split('.');\n\n			var\n				  tag = selector.shift().toUpperCase()\n				, re = new RegExp('\\\\s('+selector.join('|')+')\\\\s', 'g')\n			;\n\n			do {\n				if(\n					   (tag === '' || el.nodeName == tag)\n					&& (!selector.length || ((' '+el.className+' ').match(re) || []).length == selector.length)\n				){\n					return	el;\n				}\n			}\n			while( el !== ctx && (el = el.parentNode) );\n		}\n\n		return	null;\n	}\n\n\n	function _globalDragOver(evt){\n		evt.dataTransfer.dropEffect = 'move';\n		evt.preventDefault();\n	}\n\n\n	function _on(el, event, fn){\n		el.addEventListener(event, fn, false);\n	}\n\n\n	function _off(el, event, fn){\n		el.removeEventListener(event, fn, false);\n	}\n\n\n	function _toggleClass(el, name, state){\n		if( el ){\n			if( el.classList ){\n				el.classList[state ? 'add' : 'remove'](name);\n			}\n			else {\n				var className = (' '+el.className+' ').replace(/\\s+/g, ' ').replace(' '+name+' ', '');\n				el.className = className + (state ? ' '+name : '')\n			}\n		}\n	}\n\n\n	function _css(el, prop, val){\n		if( el && el.style ){\n			if( val === void 0 ){\n				if( document.defaultView && document.defaultView.getComputedStyle ){\n					val = document.defaultView.getComputedStyle(el, '');\n				}\n				else if( el.currentStyle ){\n					val	= el.currentStyle;\n				}\n				return	prop === void 0 ? val : val[prop];\n			} else {\n				el.style[prop] = val + (typeof val === 'string' ? '' : 'px');\n			}\n		}\n	}\n\n\n	function _find(ctx, tagName, iterator){\n		if( ctx ){\n			var list = ctx.getElementsByTagName(tagName), i = 0, n = list.length;\n			if( iterator ){\n				for( ; i < n; i++ ){\n					iterator(list[i], i);\n				}\n			}\n			return	list;\n		}\n		return	[];\n	}\n\n\n	function _disableDraggable(el){\n		return el.draggable = false;\n	}\n\n\n	function _unsilent(){\n		_silent = false;\n	}\n\n\n	function _ghostInBottom(el, evt){\n		var last = el.lastElementChild.getBoundingClientRect();\n		return evt.clientY - (last.top + last.height) > 5; // min delta\n	}\n\n\n\n	// Export utils\n	Sortable.utils = {\n		on: _on,\n		off: _off,\n		css: _css,\n		find: _find,\n		bind: _bind,\n		closest: _closest,\n		toggleClass: _toggleClass\n	};\n\n\n	Sortable.version = '0.3.0';\n\n	// Export\n	return	Sortable;\n});\n\n//# sourceURL=/thirdparty/sortable/Sortable.js"),eval('define(\'text!htmlContent/working-diagrams-view.html\',[],function () { return \'<div class="view-holder">\\n    <div class="view-header">\\n        <span class="view-header-title">Working Diagrams</span>\\n    </div>\\n    <div class="view-content">\\n        <ul class="listview"></ul>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/working-diagrams-view.html'),eval('define(\'text!htmlContent/working-diagrams-view-item.html\',[],function () { return \'<li data-id="{{_id}}" class="working-diagram">\\n    <span data-id="{{_id}}" class="close-button"></span>\\n    <span class="diagram-icon k-sprite icon-{{getClassName}}"></span>\\n    <span class="name">{{name}}</span>\\n    {{#_parent}}\\n    <span class="namespace"> — {{name}}</span>\\n    {{/_parent}}\\n</li>\';});\n\n//# sourceURL=/text!htmlContent/working-diagrams-view-item.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache */\n\n/**\n * WorkingDiagramView\n */\ndefine(\'diagrams/WorkingDiagramsView\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'utils/ViewUtils\',\'core/Repository\',\'engine/Engine\',\'command/Commands\',\'command/CommandManager\',\'menu/ContextMenuManager\',\'diagrams/DiagramManager\',\'dialogs/Dialogs\',\'thirdparty/sortable/Sortable\',\'text!htmlContent/working-diagrams-view.html\',\'text!htmlContent/working-diagrams-view-item.html\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit            = require("utils/AppInit"),\n        ViewUtils          = require("utils/ViewUtils"),\n        Repository         = require("core/Repository"),\n        Engine             = require("engine/Engine"),\n        Commands           = require("command/Commands"),\n        CommandManager     = require("command/CommandManager"),\n        ContextMenuManager = require("menu/ContextMenuManager"),\n        DiagramManager     = require("diagrams/DiagramManager"),\n        Dialogs            = require("dialogs/Dialogs"),\n        Sortable           = require("thirdparty/sortable/Sortable"),\n        viewTemplate       = require("text!htmlContent/working-diagrams-view.html"),\n        itemTemplate       = require("text!htmlContent/working-diagrams-view-item.html");\n\n    var CMD_DIAGRAM_RENAME = "working-diagrams.rename";\n\n    /**\n     * JQuery Object for ListView\n     * @private\n     * @type{JQuery}\n     */\n    var $listview = null;\n\n\n    /**\n     * Add Working Diagram\n     * @private\n     */\n    function _addWorkingDiagram(diagram) {\n        if (diagram) {\n            var $item = $(Mustache.render(itemTemplate, diagram));\n            $(".close-button", $item).click(function (e) {\n                DiagramManager.closeDiagram(diagram);\n                e.stopPropagation();\n            });\n            $item.click(function () {\n                DiagramManager.setCurrentDiagram(diagram);\n            });\n            $listview.append($item);\n        }\n    }\n\n    /**\n     * Remove Working Diagram\n     * @private\n     */\n    function _removeWorkingDiagram(diagram) {\n        if (diagram) {\n            var $item = $(".working-diagram[data-id=\'" + diagram._id + "\']", $listview);\n            $item.remove();\n        }\n    }\n\n\n    /**\n     * Update Working Diagram\n     * @private\n     */\n    function _updateWorkingDiagram(diagram) {\n        if (diagram) {\n            var $item      = $(".working-diagram[data-id=\'" + diagram._id + "\']", $listview),\n                $name      = $(".name", $item);\n            $name.html(diagram.name);\n        }\n    }\n\n    /**\n     * Clear all working diagrams\n     * @private\n     */\n    function _clearWorkingDiagrams() {\n        $listview.empty();\n    }\n\n    /**\n     * Update Working Diagrams\n     * @private\n     */\n    function _updateAllWorkingDiagrams(diagrams) {\n        $listview.empty();\n        for (var i = 0, len = diagrams.length; i < len; i++) {\n            var item = diagrams[i];\n            _addWorkingDiagram(item);\n        }\n    }\n\n\n    /**\n     * Set Current Diagram\n     * @private\n     */\n    function _setCurrentDiagram(diagram) {\n        if (diagram) {\n            $(".working-diagram", $listview).removeClass("selected");\n            var $item = $(".working-diagram[data-id=\'" + diagram._id + "\']", $listview);\n            $item.addClass("selected");\n        }\n    }\n\n\n    /**\n     * ContextMenu for Diagram Navigation\n     * @private\n     */\n    function _setupContextMenu() {\n        var CONTEXT_MENU = "context-menu-working-diagrams";\n        var contextMenu;\n        contextMenu = ContextMenuManager.addContextMenu(CONTEXT_MENU, "#working-diagrams ul.listview");\n        contextMenu.addMenuItem(Commands.VIEW_CLOSE_DIAGRAM);\n        contextMenu.addMenuItem(Commands.VIEW_CLOSE_OTHER_DIAGRAMS);\n        contextMenu.addMenuItem(Commands.VIEW_CLOSE_ALL_DIAGRAMS);\n        contextMenu.addMenuDivider();\n        contextMenu.addMenuItem(CMD_DIAGRAM_RENAME);\n    }\n\n    function _handleRename() {\n        var dgm = DiagramManager.getCurrentDiagram();\n        Dialogs.showInputDialog("Enter diagram name", dgm.name)\n            .done(function (modal, result) {\n                if (modal === Dialogs.DIALOG_BTN_OK) {\n                    console.log(modal, result);\n                    Engine.setProperty(dgm, "name", result);\n                }\n            });\n    }\n\n    AppInit.htmlReady(function () {\n        var $workingDiagrams = $("#working-diagrams"),\n            $view = $(viewTemplate),\n            $content = $view.find(".view-content");\n        $workingDiagrams.append($view);\n        $listview = $workingDiagrams.find(".listview");\n\n        ViewUtils.addScrollerShadow($content.get(0), null, true);\n\n        $(DiagramManager).on("workingDiagramsClear", function (evt) {\n            try {\n                _clearWorkingDiagrams();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(DiagramManager).on("workingDiagramAdd", function (evt, diagram) {\n            try {\n                _addWorkingDiagram(diagram);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(DiagramManager).on("workingDiagramRemove", function (evt, diagram) {\n            try {\n                _removeWorkingDiagram(diagram);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(DiagramManager).on("workingDiagramUpdate", function (evt, diagram) {\n            try {\n                _updateWorkingDiagram(diagram);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(DiagramManager).on("currentDiagramChanged", function (evt, diagram, editor) {\n            try {\n                if (diagram) {\n                    _setCurrentDiagram(diagram);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        // Make listview sortable\n        var sort = new Sortable($listview.get(0));\n\n        // Setup Context Menu for Working Diagrams\n        _setupContextMenu();\n\n    });\n\n    CommandManager.register("Rename", CMD_DIAGRAM_RENAME, _handleRename);\n\n});\n\n//# sourceURL=/diagrams/WorkingDiagramsView.js'),eval('define(\'text!htmlContent/toolbox-view.html\',[],function () { return \'<div class="view-holder">\\n    <div class="view-header">\\n        <span class="view-header-title">Toolbox</span>\\n        <div class="buttons">\\n            <button class="select-button k-button icon-button quiet"><span class="k-sprite tool-icon-select"></span></button>\\n        </div>\\n    </div>\\n    <div class="view-content">\\n        <div id=\\\'ui-toolbox\\\' style=\\\'width: 100%; height: 100%;\\\'></div>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/toolbox-view.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache */\n\n/**\n *\n * This module dispatches these events:\n *     - elementCreated:       (id, editor, x1, y1, x2, y2)\n *     - activeHandlerChanged: (handler)\n */\ndefine(\'diagrams/ToolboxView\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/AppInit\',\'command/CommandManager\',\'command/Commands\',\'strings\',\'utils/Resizer\',\'utils/ViewUtils\',\'utils/KeyEvent\',\'diagrams/Editor\',\'diagrams/Editor\',\'diagrams/DiagramManager\',\'text!htmlContent/toolbox-view.html\'],function (require, exports, module) {\n    "use strict";\n\n    var global         = require("core/Global").global,\n        AppInit        = require("utils/AppInit"),\n        CommandManager = require("command/CommandManager"),\n        Commands       = require("command/Commands"),\n        Strings        = require("strings"),\n        Resizer        = require("utils/Resizer"),\n        ViewUtils      = require("utils/ViewUtils"),\n        KeyEvent       = require("utils/KeyEvent"),\n        SelectHandler  = require("diagrams/Editor").SelectHandler,\n        CreateHandler  = require("diagrams/Editor").CreateHandler,\n        DiagramManager = require("diagrams/DiagramManager"),\n        viewTemplate   = require("text!htmlContent/toolbox-view.html");\n\n    var SELECTOR_ID = \'toolbox-select\';\n\n\n    var $toolbox,\n        $toolboxPanelBar = null,\n        $buttons = null,\n        $selectButton = null;\n\n    /**\n     * Array for Groups\n     * @private\n     * @type {Array.<{id:string, title:string, allowedDiagrams:Array.<Core.Diagram>}>}\n     */\n    var _groups = [];\n\n    /**\n     * Active Item\n     * @private\n     * @type {string}\n     */\n    var _activeItem = null;\n\n    /**\n     * Active Item Locked\n     * @private\n     * @type {boolean}\n     */\n    var _activeItemLocked = false;\n\n    /**\n     * Map for Handlers\n     * @private\n     * @type {Object.<string,Handlers.Handler>}\n     */\n    var _handlers = {};\n\n    /**\n     * Panel Bar\n     * @private\n     * @type {kendo.ui.PanelBar}\n     */\n    var _panelBar = null;\n\n\n    /**\n     * Toggle ToolboxView\n     */\n    function toggle() {\n        Resizer.toggle($toolbox);\n    }\n\n    /**\n     * Show ToolboxView\n     */\n    function show() {\n        Resizer.show($toolbox);\n    }\n\n    /**\n     * Hide ToolboxView\n     */\n    function hide() {\n        Resizer.hide($toolbox);\n    }\n\n    /**\n     * Return whether ToolboxView is visible or not\n     *\n     * @return {booean}\n     */\n    function isVisible() {\n        return Resizer.isVisible($toolbox);\n    }\n\n    function _createEventHandler(id, editor, x1, y1, x2, y2) {\n        $(exports).triggerHandler(\'elementCreated\', [id, editor, x1, y1, x2, y2]);\n        // 요소 생성 후, Select로 돌아감.\n        if (!_activeItemLocked) {\n            selectItem();\n        }\n    }\n\n    function _triggerHandlerChangeEvent(handler) {\n        $(exports).triggerHandler(\'activeHandlerChanged\', handler);\n    }\n\n    /**\n     * Activate a given item\n     * @private\n     * @param {string} itemId\n     */\n    function _setActiveItem(itemId) {\n        if (itemId === SELECTOR_ID) {\n            $("#" + _activeItem + " > span").removeClass("k-state-selected k-state-focused"); // deselect\n            $selectButton.addClass("k-state-selected");\n        } else {\n            $selectButton.removeClass("k-state-selected");\n        }\n        _setActiveItemLock(false);\n        _activeItem = itemId;\n        var handler = _handlers[itemId];\n        DiagramManager.setActiveHandler(handler);\n        _triggerHandlerChangeEvent(handler);\n    }\n\n    /**\n     * Active 아이템을 고정시킨다.\n     * @param {string} itemId\n     * @param {boolean} locked\n     */\n    function _setActiveItemLock(locked) {\n        _activeItemLocked = locked;\n        if (_activeItemLocked) {\n            $("#" + _activeItem + " > span").append("<span class=\'k-sprite icon-Lock toolbox-lock\' style=\'float: none; margin-left: 2px; margin-bottom: 7px;\'></span>");\n        } else {\n            $(".toolbox-lock").remove();\n        }\n    }\n\n    /**\n     * Add a group.\n     *\n     * @param {string} groupId\n     * @param {string} title\n     * @param {!Array.<Core.Diagram>} allowedDiagrams\n     *      This group only available for allowed diagrams.\n     *      If empty array is passed, then this group will be available for any diagrams\n     */\n    function addGroup(groupId, title, allowedDiagrams) {\n        var group = {\n            id: groupId,\n            title: title,\n            allowedDiagrams: allowedDiagrams\n        };\n        // var html = "<li id=\'{{id}}\'>{{title}}<ul><li id=\'toolbox-select\'><span class=\'k-sprite tool-icon-select\'></span>Select</li></ul></li>";\n        var html = "<li id=\'{{id}}\'>{{title}}<ul></li>";\n        $("#ui-toolbox").append(Mustache.render(html, group));\n        _groups.add(group);\n    }\n\n    /**\n     * Add an item to a group.\n     *\n     * @param {string} id\n     * @param {string} groupId\n     * @param {string} title\n     * @param {string} iconClass\n     * @param {string} skeleton "line" | "rect" | "point"\n     */\n    function addItem(id, groupId, title, iconClass, skeleton) {\n        var html = "<li id=\'" + id + "\'><span class=\'k-sprite " + iconClass + "\'></span><span>" + title + "</span></li>";\n        $("#" + groupId + " > ul").append(html);\n        if (!_handlers[id]) {\n            _handlers[id] = new CreateHandler(id, skeleton, _createEventHandler);\n        }\n    }\n\n    /**\n     * Show groups only available for a given diagram.\n     *\n     * @param {Core.Diagram} diagram\n     */\n    function setDiagram(diagram) {\n        if (diagram) {\n            $toolboxPanelBar.show();\n            $buttons.show();\n            _.forEach(_groups, function (group) {\n                var visible = false;\n                if (_.isArray(group.allowedDiagrams) && (group.allowedDiagrams.length > 0)) {\n                    if (_.indexOf(group.allowedDiagrams, diagram.constructor) > -1) {\n                        visible = true;\n                    }\n                } else {\n                    visible = true;\n                }\n                $("#" + group.id).toggle(visible);\n            });\n        } else {\n            $toolboxPanelBar.hide();\n            $buttons.hide();\n        }\n    }\n\n    /**\n     * Select an item.\n     *\n     * @param {string} itemId\n     */\n    function selectItem(itemId) {\n        if (itemId && (itemId !== SELECTOR_ID)) {\n            _setActiveItem(itemId);\n        } else {\n            _setActiveItem(SELECTOR_ID);\n            _setActiveItemLock(false);\n        }\n    }\n\n    /**\n     * Expand a group.\n     *\n     * @param {string} groupId\n     */\n    function expandGroup(groupId) {\n        if (groupId) {\n            _panelBar.expand($("#ui-toolbox #" + groupId));\n        }\n    }\n\n    /**\n     * Collapse a group.\n     *\n     * @param {string} groupId\n     */\n    function collapseGroup(groupId) {\n        if (groupId) {\n            _panelBar.collapse($("#ui-toolbox #" + groupId));\n        }\n    }\n\n    /**\n     * Get a handler.\n     *\n     * @param {string} id\n     * @return {Handler}\n     */\n    function getHandler(id) {\n        return _handlers[id];\n    }\n\n    /**\n     * Setup for UI\n     */\n    function _setupUI() {\n        // Configure Handlers\n        _handlers[SELECTOR_ID] = new SelectHandler();\n\n        // Setup PanelBar\n        $("#ui-toolbox").kendoPanelBar({\n            expandMode: "multiple",\n            activate: function (e) {\n                selectItem();\n            },\n            select: function (e) {\n                if (e.item.id === _activeItem) {\n                    if (_activeItemLocked === true) {\n                        _setActiveItemLock(false);\n                        _setActiveItem(SELECTOR_ID);\n                    } else {\n                        _setActiveItemLock(true);\n                    }\n                } else if (_handlers[e.item.id]) {\n                    // Handler에 등록된 e.item.id 가 존재하는 경우에만 동작.\n                    // Group을 Collapse될 때, 그 내부에 ActiveItem이 존재하면 e.item.id 는 Group의 id가 됨.\n                    _setActiveItemLock(false);\n                    _setActiveItem(e.item.id);\n                } else {\n                    // 적합한 Handler가 없는 경우에는 SELECTOR가 선택됨.\n                    _setActiveItemLock(false);\n                    _setActiveItem(SELECTOR_ID);\n                }\n            }\n        });\n        _panelBar = $("#ui-toolbox").data("kendoPanelBar");\n\n        // Process Key Events (ESC == Go to Selection Mode)\n        $("body").keydown(function (event) {\n            switch (event.which) {\n            case KeyEvent.DOM_VK_ESCAPE:\n                _setActiveItem(SELECTOR_ID);\n                break;\n            }\n        });\n\n    }\n\n\n\n    /* Initialization when DOM Ready */\n    AppInit.htmlReady(function () {\n        var $view    = $(viewTemplate),\n            $content = $view.find(".view-content");\n\n        $toolbox = $("#toolbox");\n        $toolboxPanelBar = $view.find("#ui-toolbox");\n        $buttons = $view.find(".buttons");\n        $toolbox.append($view);\n        ViewUtils.addScrollerShadow($content.get(0), null, true);\n\n        $selectButton = $(".select-button", $view);\n        $selectButton.click(function () {\n            _setActiveItem(SELECTOR_ID);\n        });\n\n        $toolbox.on("panelCollapsed", function (evt, width) {\n            CommandManager.get(Commands.VIEW_HIDE_TOOLBOX).setName(Strings.CMD_VIEW_SHOW_TOOLBOX);\n        });\n\n        $toolbox.on("panelExpanded", function (evt, width) {\n            CommandManager.get(Commands.VIEW_HIDE_TOOLBOX).setName(Strings.CMD_VIEW_HIDE_TOOLBOX);\n        });\n\n        // AppInit.htmlReady in utils/Resizer executes before, so it\'s possible that the sidebar\n        // is collapsed before we add the event. Check here initially\n        if (!$toolbox.is(":visible")) {\n            $toolbox.trigger("panelCollapsed");\n        }\n\n\n    });\n\n    /* Initialization when DOM Ready */\n    AppInit.appReady(function () {\n        _setupUI();\n        // Expand the first group\n        if (_groups.length > 0) {\n            _panelBar.expand($("#" + _groups[0].id));\n        }\n    });\n\n    CommandManager.register(Strings.CMD_VIEW_HIDE_TOOLBOX, Commands.VIEW_HIDE_TOOLBOX, toggle);\n\n    // Define public API\n    exports.toggle        = toggle;\n    exports.show          = show;\n    exports.hide          = hide;\n    exports.isVisible     = isVisible;\n    exports.addGroup      = addGroup;\n    exports.addItem       = addItem;\n    exports.setDiagram    = setDiagram;\n    exports.expandGroup   = expandGroup;\n    exports.collapseGroup = collapseGroup;\n    exports.selectItem    = selectItem;\n    exports.getHandler    = getHandler;\n\n});\n\n//# sourceURL=/diagrams/ToolboxView.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo */\n\n/**\n * Toolbar\n */\ndefine(\'ui/Toolbar\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/PreferenceManager\',\'command/CommandManager\',\'command/Commands\',\'strings\',\'utils/Resizer\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit             = require("utils/AppInit"),\n        PreferenceManager   = require("core/PreferenceManager"),\n        CommandManager      = require("command/CommandManager"),\n        Commands            = require("command/Commands"),\n        Strings             = require("strings"),\n        Resizer             = require("utils/Resizer");\n\n    var $toolbar,\n        $contentHolder;\n\n    /**\n     * Toggle Toolbar\n     */\n    function toggle() {\n        if (isVisible()) {\n            hide();\n        } else {\n            show();\n        }\n    }\n\n    /**\n     * Show Toolbar\n     */\n    function show() {\n        var elementPrefs = {\n            size: $toolbar.outerWidth(),\n            visible: true\n        };\n        $toolbar.show();\n        $contentHolder.css("right", elementPrefs.size);\n        $toolbar.trigger("panelExpanded", [elementPrefs.size]);\n        PreferenceManager.setViewState("toolbar", elementPrefs);\n\n        CommandManager.get(Commands.VIEW_HIDE_TOOLBAR).setName(Strings.CMD_VIEW_HIDE_TOOLBAR);\n    }\n\n    /**\n     * Hide Toolbar\n     */\n    function hide() {\n        var elementPrefs = {\n            size: $toolbar.outerWidth(),\n            visible: false\n        };\n        $toolbar.hide();\n        $contentHolder.css("right", 0);\n        $toolbar.trigger("panelCollapsed", [0]);\n        PreferenceManager.setViewState("toolbar", elementPrefs);\n\n        CommandManager.get(Commands.VIEW_HIDE_TOOLBAR).setName(Strings.CMD_VIEW_SHOW_TOOLBAR);\n    }\n\n    /**\n     * Return whether Toolbar is visible or not\n     *\n     * @return {booean}\n     */\n    function isVisible() {\n        return $toolbar.is(":visible");\n    }\n\n    AppInit.htmlReady(function () {\n        $toolbar = $("#toolbar");\n        $contentHolder = $(".content-holder");\n\n        var elementPrefs = PreferenceManager.getViewState("toolbar");\n        if (elementPrefs.visible === false) {\n            hide();\n        } else {\n            show();\n        }\n    });\n\n    CommandManager.register(Strings.CMD_VIEW_HIDE_TOOLBAR, Commands.VIEW_HIDE_TOOLBAR, toggle);\n\n    exports.toggle = toggle;\n    exports.show   = show;\n    exports.hide   = hide;\n\n});\n\n//# sourceURL=/ui/Toolbar.js'),eval('define(\'text!htmlContent/statusbar.html\',[],function () { return \'<div class="left-section">\\n    <div class="selected-element" style="height: 100%;">\\n    </div>\\n</div>\\n<div class="right-section">\\n    <div class="validation item" title="Validation Results" ><span class="validation-icon k-sprite validation-ok"></span></div>\\n    <div class="zoom item" title="Zoom Level">100%</div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/statusbar.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell */\n\n/**\n * StatusBar\n */\ndefine(\'ui/StatusBar\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/AppInit\',\'core/PreferenceManager\',\'command/CommandManager\',\'command/Commands\',\'strings\',\'text!htmlContent/statusbar.html\'],function (require, exports, module) {\n    "use strict";\n\n    var global            = require("core/Global").global,\n        AppInit           = require("utils/AppInit"),\n        PreferenceManager = require("core/PreferenceManager"),\n        CommandManager    = require("command/CommandManager"),\n        Commands          = require("command/Commands"),\n        Strings           = require("strings"),\n        template          = require("text!htmlContent/statusbar.html");\n\n    var $statusbar,\n        $mainView,\n        $view,\n        $selectedElement,\n        $zoom,\n        $validation;\n\n    /**\n     * Zoom의 비율을 나타내기\n     *\n     * @private\n     * @param {number} value\n     */\n    function setZoomLevel(value) {\n        if (value) {\n            $zoom.html(Math.round(value * 100) + "%");\n        } else {\n            $zoom.html("100%");\n        }\n    }\n\n    /**\n     * Set Validity\n     *\n     * @private\n     * @param {boolean} value\n     */\n    function setValidity(value) {\n        if (value) {\n            $validation.addClass("validation-ok");\n            $validation.removeClass("validation-error");\n        } else {\n            $validation.addClass("validation-error");\n            $validation.removeClass("validation-ok");\n        }\n    }\n\n    /**\n     * 선택된 요소의 정보를 나타내기\n     *\n     * @private\n     * @param {Core.Model} elem\n     */\n    function setElement(elem) {\n\n        function _addItem(_elem) {\n            if (_elem) {\n                var text;\n                if (_elem === elem) {\n                    text = "<span class=\'element\'>"\n                         + "<span class=\'k-sprite " + _elem.getNodeIcon() + "\'></span>"\n                         + "<span style=\'color: #8fddff\'>" + _elem.name + "</span>"\n                         + "<span style=\'color: #8b8b8b; font-size: 11px;\'> [" + _elem.getClassName() + "]</span>"\n                         + "</span>";\n                } else {\n                    if (_elem instanceof type.Project) {\n                        text = "<span class=\'element\'>"\n                             + "<span class=\'k-sprite " + _elem.getNodeIcon() + "\'></span>"\n                             + "</span>";\n                    } else {\n                        text = "<span class=\'element\'>"\n                             + "<span class=\'k-sprite " + _elem.getNodeIcon() + "\'></span>"\n                             + "<span>" + _elem.name + "</span>"\n                             + "</span>";\n                    }\n                }\n                $selectedElement.prepend(text);\n                if (_elem._parent) {\n                    _addItem(_elem._parent);\n                }\n            }\n        }\n\n        if (elem) {\n            // var text = "<span class=\'k-sprite " + elem.getNodeIcon() + "\'></span> [" + elem.getClassName() + "] " + elem.getPathname();\n            // $selectedElement.html(text);\n            $selectedElement.children().remove();\n            _addItem(elem);\n        } else {\n            $selectedElement.html("&nbsp;");\n        }\n    }\n\n    /**\n     * Toggle StatusBar\n     */\n    function toggle() {\n        if (isVisible()) {\n            hide();\n        } else {\n            show();\n        }\n    }\n\n    /**\n     * Show StatusBar\n     */\n    function show() {\n        var elementPrefs = {\n            size: $statusbar.outerHeight(),\n            visible: true\n        };\n        $statusbar.show();\n        $mainView.css("bottom", elementPrefs.size);\n        $statusbar.trigger("panelExpanded", [elementPrefs.size]);\n        PreferenceManager.setViewState("statusbar", elementPrefs);\n\n        CommandManager.get(Commands.VIEW_HIDE_STATUSBAR).setName(Strings.CMD_VIEW_HIDE_STATUSBAR);\n    }\n\n    /**\n     * Hide StatusBar\n     */\n    function hide() {\n        var elementPrefs = {\n            size: $statusbar.outerHeight(),\n            visible: false\n        };\n        $statusbar.hide();\n        $mainView.css("bottom", 0);\n        $statusbar.trigger("panelCollapsed", [0]);\n        PreferenceManager.setViewState("statusbar", elementPrefs);\n\n        CommandManager.get(Commands.VIEW_HIDE_STATUSBAR).setName(Strings.CMD_VIEW_SHOW_STATUSBAR);\n    }\n\n    /**\n     * Return whether StatusBar is visible or not\n     *\n     * @return {booean}\n     */\n    function isVisible() {\n        return $statusbar.is(":visible");\n    }\n\n    AppInit.htmlReady(function () {\n        $statusbar = $("#statusbar");\n        $mainView = $(".main-view");\n\n        $view = $(template);\n        $selectedElement = $view.find(".selected-element");\n        $zoom = $view.find(".zoom");\n        $validation = $view.find(".validation-icon");\n\n        $statusbar.append($view);\n\n        if (!$statusbar.is(":visible")) {\n            hide();\n        }\n\n        var elementPrefs = PreferenceManager.getViewState("statusbar");\n        if (elementPrefs.visible === false) {\n            hide();\n        } else {\n            show();\n        }\n    });\n\n    CommandManager.register(Strings.CMD_VIEW_HIDE_STATUSBAR, Commands.VIEW_HIDE_STATUSBAR, toggle);\n\n    // Public API\n    exports.toggle       = toggle;\n    exports.show         = show;\n    exports.hide         = hide;\n    exports.isVisible    = isVisible;\n    exports.setZoomLevel = setZoomLevel;\n    exports.setValidity  = setValidity;\n    exports.setElement   = setElement;\n\n});\n\n//# sourceURL=/ui/StatusBar.js'),eval('define(\'text!htmlContent/unregistered-dialog.html\',[],function () { return \'<div class="unregistered-dialog dialog modal" data-title="Unregistered Version">\\n    <div class="dialog-body">\\n        <div class="ad"></div>\\n        <div>Thank you for evaluating StarUML. This is unregistered evaluation version. Although no time limit for evaluation, a license should be purchased for continued use. If you want to purchase a license, please click "Buy Now" button.</div>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button left buy-now primary">Buy Now</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">Evaluate</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/unregistered-dialog.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache */\n\ndefine(\'license/UnregisteredDialog\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/Initializer\',\'strings\',\'dialogs/Dialogs\',\'license/LicenseManager\',\'text!htmlContent/unregistered-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var global         = require("core/Global").global,\n        Initializer    = require("utils/Initializer"),\n        Strings        = require("strings"),\n        Dialogs        = require("dialogs/Dialogs"),\n        LicenseManager = require("license/LicenseManager");\n\n    var unregisteredDialogTemplate = require("text!htmlContent/unregistered-dialog.html");\n\n    /**\n     * Show License Manager Dialog\n     * @return {$.Promise}\n     */\n    function showDialog() {\n        var context = {\n            Strings: Strings,\n            metadata: global.app.metadata\n        };\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(unregisteredDialogTemplate, context));\n\n        var $dlg    = dialog.getElement(),\n            $buyNow = $dlg.find(".buy-now"),\n            $ad     = $dlg.find(".ad");\n\n        $buyNow.click(function () {\n            appshell.app.openURLInDefaultBrowser(global.app.config.purchase_url);\n        });\n\n        $.ajax({url: "http://starumlad-7a0.kxcdn.com/ad.json", cache: false})\n            .done(function (data) {\n                console.log(data);\n                if (data.image && data.image.length > 0) {\n                    $ad.append("<a href=\'" + data.url + "\'><img src=\'" + data.image + "\'></a>");\n                }\n            })\n\n        dialog.done(function (buttonId) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n            }\n        });\n\n        return dialog;\n    }\n\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/license/UnregisteredDialog.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, JSEncrypt*/\n\ndefine(\'license/LicenseManager\',[\'require\',\'exports\',\'module\',\'core/Global\',\'engine/ProjectManager\',\'utils/Initializer\',\'utils/AppInit\',\'utils/NodeDomain\',\'file/FileUtils\',\'filesystem/FileSystem\',\'license/UnregisteredDialog\'],function (require, exports, module) {\n    "use strict";\n\n    var global         = require("core/Global").global,\n        ProjectManager = require("engine/ProjectManager"),\n        Initializer    = require("utils/Initializer"),\n        AppInit        = require("utils/AppInit"),\n        NodeDomain     = require("utils/NodeDomain"),\n        FileUtils      = require("file/FileUtils"),\n        FileSystem     = require("filesystem/FileSystem");\n\n    var UnregisteredDialog = require("license/UnregisteredDialog");\n\n    var PK =\n    "-----BEGIN RSA PRIVATE KEY-----\\n" +\n    "MIIBOgIBAAJBAKMtaq6K5rkfHskR2sYRvUU0APpGlpCBz8bhbqBaMmjFot9XZy/9\\n" +\n    "B+BuqF4yIytW7404U4D6tkGTsL237i/vhRsCAwEAAQJAUmYuxvDxTW3/l71iYj62\\n" +\n    "+uklzTwz78j9aQqRWHvzlJ/ZlKHPit8sA7GZ16ArN8DoHx+Slh6pm4tDkNgz+Evc\\n" +\n    "MQIhANr1M6M2GfzXX0PQxIkKypn7F6C1t2r9ivUjt9uRFP8nAiEAvshu1HKSPwrf\\n" +\n    "i1gtNHOoak1Er7e2lizWEmiyG48XAu0CIAOf/ZvdIG8ELcqUoTxXlmUYOBrsBz5Z\\n" +\n    "Njy6u6eB7mQPAiEAq7Qqg0vmMZTwojGm5akeLyYBpoAadUg/FmxhuAntNPUCIAZQ\\n" +\n    "iopKN9x4H2zAC7zhkBNeane6xGNLaR5QOG510Vk6\\n" +\n    "-----END RSA PRIVATE KEY-----";\n\n    /**\n     * @enum\n     */\n    var STAT_UNREGISTERED = "UNREGISTERED",\n        STAT_REGISTERED   = "REGISTERED";\n\n    // Status\n    var status = null;\n\n    // License Info\n    var licenseInfo = null;\n\n    // LicenseManager Domain\n    var LicenseManagerDomain;\n\n    /**\n     * Get Registration Status\n     * @return {string}\n     */\n    function getStatus() {\n        return status;\n    }\n\n    /**\n     * Set Registration Status\n     * @return {string}\n     */\n    function setStatus(newStat) {\n        if (status !== newStat) {\n            status = newStat;\n            $(exports).triggerHandler("statusChanged", [status]);\n        }\n    }\n\n    /**\n     * Get License Infomation\n     * @return {Object}\n     */\n    function getLicenseInfo() {\n        return licenseInfo;\n    }\n\n    /**\n     * Register LicenseKey\n     * @param {string} licenseKey\n     * @return {$.Promise}\n     */\n    function register(name, licenseKey) {\n        var path    = appshell.app.getApplicationSupportDirectory() + "/LICENSE",\n            file    = FileSystem.getFileForPath(path);\n        var license = {\n                name: name,\n                licenseKey: licenseKey\n            };\n        return FileUtils.writeText(file, JSON.stringify(license), true);\n    }\n\n    function doValidate(data) {\n        var result = new $.Deferred();\n        try {\n            licenseInfo = JSON.parse(data);\n            LicenseManagerDomain.exec("validate", PK, licenseInfo.name, global.app.metadata.config.product_id, licenseInfo.licenseKey)\n                .done(function (validated) {\n                    if (validated === false) { // failed\n                        licenseInfo = null;\n                        setStatus(STAT_UNREGISTERED);\n                        result.reject("Invalid License Key");\n                    } else { // success\n                        licenseInfo = validated;\n                        setStatus(STAT_REGISTERED);\n                        result.resolve(getStatus());\n                    }\n                })\n                .fail(function (err) {\n                    licenseInfo = null;\n                    setStatus(STAT_UNREGISTERED);\n                    result.reject(err);\n                });\n        } catch (ex) {\n            console.error(ex);\n            licenseInfo = null;\n            setStatus(STAT_UNREGISTERED);\n            result.reject(ex);\n        }\n        return result.promise();\n    }\n\n    /**\n     * Validate LicenseKey\n     * @return {$.Promise}\n     */\n    function validate() {\n        var result = new $.Deferred(),\n            path   = appshell.app.getApplicationSupportDirectory() + "/LICENSE",\n            file   = FileSystem.getFileForPath(path);\n        FileUtils.readAsText(file)\n            .done(function (data) {\n                doValidate(data).then(result.resolve, result.reject);\n            })\n            .fail(function (err) {\n                // Find LICENSE file on a secondary path\n                var path2 = FileUtils.getApplicationDirectoryPath() + "LICENSE",\n                    file2 = FileSystem.getFileForPath(path2);\n                FileUtils.readAsText(file2)\n                    .done(function (data2) {\n                        doValidate(data2).then(result.resolve, result.reject);\n                    })\n                    .fail(function (err2) {\n                        licenseInfo = null;\n                        setStatus(STAT_UNREGISTERED);\n                        result.reject(err);\n                    });\n            });\n        return result.promise();\n    }\n\n    function checkValidity() {\n        var domainPath = FileUtils.getNativeDirectoryPath() + "/" + FileUtils.getNativeModuleDirectoryPath(module) + "/node/LicenseManagerDomain";\n        LicenseManagerDomain = new NodeDomain("LicenseManager", domainPath);\n        // setStatus(STAT_UNREGISTERED);\n        validate().fail(function (err) {\n            UnregisteredDialog.showDialog();\n        });\n    }\n\n    // Initializes Node Connection\n    AppInit.appReady(function () {\n        checkValidity();\n    });\n\n\n    // Check License When File Save\n    var LICENSE_CHECK_PROBABILITY = 0.2;\n    $(ProjectManager).on(\'projectSaved\', function (event, filename, project) {\n        var val = Math.floor(Math.random() * (1.0 / LICENSE_CHECK_PROBABILITY));\n        if (val === 0) {\n            checkValidity();\n        }\n    });\n\n    exports.STAT_UNREGISTERED = STAT_UNREGISTERED;\n    exports.STAT_REGISTERED   = STAT_REGISTERED;\n\n    exports.getStatus      = getStatus;\n    exports.setStatus      = setStatus;\n    exports.getLicenseInfo = getLicenseInfo;\n    exports.register       = register;\n    exports.validate       = validate;\n\n});\n\n//# sourceURL=/license/LicenseManager.js'),eval('define(\'text!htmlContent/about-dialog.html\',[],function () { return \'<div class="about-dialog dialog modal" data-title="About">\\n    <div class="dialog-body">\\n        <table>\\n            <tr>\\n                <td valign="top">\\n                    <img style="margin-right: 1em;" src="{{metadata.config.app_icon}}">\\n                </td>\\n                <td valign="top">\\n                    <div style="font-size: 20px;">{{metadata.name}}</div>\\n                    <div style="font-size: 14px;" class="license"></div>\\n                    <div style="font-size: 14px;" class="licenseType"></div>\\n                    <div style="font-size: 14px;" class="quantity"></div>\\n                    <br>\\n                    <div>{{metadata.copyright}}</div>\\n                    <div><b>Version {{metadata.version}}</b></div>\\n                    <div><a class="thirdparty" href="#">Third party softwares</a></div>\\n                </td>\\n            </tr>\\n        </table>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button primary" data-button-id="close">{{Strings.CLOSE}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/about-dialog.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache*/\n\ndefine(\'dialogs/AboutDialog\',[\'require\',\'exports\',\'module\',\'core/Global\',\'strings\',\'file/FileUtils\',\'dialogs/Dialogs\',\'license/LicenseManager\',\'text!htmlContent/about-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var global         = require("core/Global").global,\n        Strings        = require("strings"),\n        FileUtils      = require("file/FileUtils"),\n        Dialogs        = require("dialogs/Dialogs"),\n        LicenseManager = require("license/LicenseManager");\n\n    var aboutDialogTemplate = require("text!htmlContent/about-dialog.html");\n\n    /**\n     * Show About Dialog\n     * @return {$.Promise}\n     */\n    function showDialog() {\n        var context = {\n            Strings: Strings,\n            metadata: global.app.metadata\n        };\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(aboutDialogTemplate, context));\n        var $dlg         = dialog.getElement(),\n            $license     = $dlg.find(".license"),\n            $licenseType = $dlg.find(".licenseType"),\n            $quantity    = $dlg.find(".quantity"),\n            $thirdparty  = $dlg.find(".thirdparty");\n\n        $thirdparty.click(function () {\n            var path = "file://" + FileUtils.getNativeDirectoryPath() + "/thirdparty/thirdparty-licenses.html";\n            appshell.app.openURLInDefaultBrowser(path);\n        });\n\n        // set license info\n        var licenseStatus = LicenseManager.getStatus();\n        if (licenseStatus === LicenseManager.STAT_REGISTERED) {\n            var info = LicenseManager.getLicenseInfo(),\n                licenseTypeName;\n            switch (info.licenseType) {\n            case "PS":\n                licenseTypeName = "Personal";\n                break;\n            case "CO":\n                licenseTypeName = "Commercial";\n                break;\n            case "ED":\n                licenseTypeName = "Educational";\n                break;\n            case "CR":\n                licenseTypeName = "Classroom";\n                break;\n            }\n            $license.html("Licensed to " + info.name);\n            $licenseType.html(licenseTypeName + " License");\n            $quantity.html(info.quantity + " User(s)");\n        } else {\n            $license.html(licenseStatus);\n        }\n        return dialog;\n    }\n\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/dialogs/AboutDialog.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, kendo, Mustache */\n\ndefine(\'ui/QuickEdit\',[\'require\',\'exports\',\'module\',\'core/Core\',\'core/Graphics\',\'core/Graphics\',\'core/Repository\',\'engine/Engine\',\'diagrams/DiagramManager\',\'command/KeyBindingManager\',\'utils/Helper\',\'utils/KeyEvent\'],function (require, exports, module) {\n    "use strict";\n\n    var Core              = require("core/Core"),\n        Graphics          = require("core/Graphics"),\n        Coord             = require("core/Graphics").Coord,\n        Repository        = require("core/Repository"),\n        Engine            = require("engine/Engine"),\n        DiagramManager    = require("diagrams/DiagramManager"),\n        KeyBindingManager = require("command/KeyBindingManager"),\n        Helper            = require("utils/Helper"),\n        KeyEvent          = require("utils/KeyEvent");\n\n    var TEMPLATE_BEGIN = "<div id=\'quick-edit\' class=\'quick-edit modal instance\' style=\'left: {{left}}px; top: {{top}}px; width: {{width}}px;\'>",\n        TEMPLATE_END   = "<button id=\'quick-edit-outfocus\' style=\'width: 0; height: 0\'></button></div>";\n\n    var TEMPLATE_EDIT =\n        "<input type=\'text\' id=\'quick-edit-{{id}}\' class=\'quick-edit-input quick-edit-hint\' style=\'width: {{width}}px; height: {{height}}px; font-family: {{fontFamily}}; font-size: {{fontSize}}px\' value=\'{{text}}\' title=\'{{hint}}\'>";\n\n    var TEMPLATE_TEXTAREA =\n        "<textarea id=\'quick-edit-{{id}}\' class=\'quick-edit-textarea quick-edit-hint\' style=\'width: {{width}}px; height: {{height}}px; font-family: {{fontFamily}}; font-size: {{fontSize}}px\' title=\'{{hint}}\'>{{text}}</textarea>";\n\n    var TEMPLATE_DROPDOWN =\n        "<button id=\'quick-edit-{{id}}\' title=\'{{hint}}\' class=\'k-button icon-button small quick-edit-hint\' style=\'position: absolute; top: 0px; left: {{left}}px\'><span class=\'k-sprite\'></span></button>" +\n        "<div style=\'position: absolute; margin: 0; left: {{left}}px; top: 25px\'>" +\n            "<ul id=\'quick-edit-{{id}}-dropdown\' class=\'k-context-menu quick-edit-dropdown\' style=\'width: {{dropdownWidth}}px\'>" +\n                "{{#items}}" +\n                "<li data-data=\'{{value}}\'><span class=\'k-sprite {{icon}}\'></span>{{text}}</li>" +\n                "{{/items}}" +\n            "</ul>" +\n        "</div>";\n\n    var TEMPLATE_BUTTON =\n        "<button id=\'quick-edit-{{id}}\' class=\'k-button icon-button small quick-edit-hint\' style=\'position: absolute; top: {{top}}px; left: {{left}}px\' title=\'{{fullHint}}\'><span class=\'k-sprite {{icon}}\'></span></button>";\n\n    var TEMPLATE_COMBO =\n        "<div id=\'quick-edit-{{id}}-container\' class=\'quick-edit-hint quick-edit-combo\' style=\'position: absolute; left: {{left}}px; top: 0px\' title=\'{{hint}}\'>" +\n            "<label id=\'quick-edit-{{id}}\' class=\'widget k-combo small\' style=\'width: {{width}}px\'>" +\n                "<input type=\'text\' value=\'{{text}}\'>" +\n                "<select>" +\n                    "{{#items}}" +\n                    "<option value=\'{{value}}\'>{{text}}</option>" +\n                    "{{/items}}" +\n                "</select>" +\n            "</label>" +\n        "</div>";\n\n\n    var INPUT_WIDTH    = 60,\n        TEXTAREA_WIDTH = 60,\n        DROPDOWN_WIDTH = 25,\n        BUTTON_WIDTH   = 25,\n        BUTTON_HEIGHT  = 25,\n        COMBO_WIDTH    = 60;\n\n    /**\n     * QuickEdit List\n     * @type {}\n     */\n    var quickEditList = [];\n\n    /**\n     *\n     */\n    var currentQuickEdit = null;\n\n    /**\n     *\n     */\n    var currentQuickEditItems = {};\n\n    /**\n     *\n     */\n    var tooltip = null;\n\n\n    var _preventClose = false;\n\n    function renderItem(editor, view, item) {\n        currentQuickEditItems[item.id] = item;\n        switch (item.type) {\n        case \'input\':\n            item.width = item.width - 9; // padding\n            item.height = Math.round(item.height + 6);\n            item.fontFamily = view.font.face;\n            item.fontSize = Coord.valueTransform(editor.canvas.zoomFactor, view.font.size);\n            return Mustache.render(TEMPLATE_EDIT, item);\n        case \'textarea\':\n            item.width = item.width - 9; // padding\n            item.fontFamily = view.font.face;\n            item.fontSize = Coord.valueTransform(editor.canvas.zoomFactor, view.font.size);\n            return Mustache.render(TEMPLATE_TEXTAREA, item);\n        case \'dropdown\':\n            return Mustache.render(TEMPLATE_DROPDOWN, item);\n        case \'button\':\n            if (item.key) {\n                // TODO: This is temporal implementation of showing key-shortcuts.\n                var displayStr = item.key;\n                if (app.platform === "mac") {\n                    displayStr = displayStr.replace("Ctrl-", "Cmd-");\n                    displayStr = displayStr.replace("Delete", "Backspace");\n                }\n                item.fullHint = item.hint + " (" + KeyBindingManager.formatKeyDescriptor(displayStr) + ")";\n            } else {\n                item.fullHint = item.hint;\n            }\n            return Mustache.render(TEMPLATE_BUTTON, item);\n        case \'combo\':\n            return Mustache.render(TEMPLATE_COMBO, item);\n        }\n    }\n\n    /**\n     * Hide Dropdowns\n     * @private\n     * @param {string} exceptId Exception Dropdown Id\n     */\n    function _hideDropdowns(exceptId) {\n        _.forEach(currentQuickEditItems, function (item) {\n            if ((item.type == \'dropdown\') && (item.id != exceptId)) {\n                $("#quick-edit-" + item.id).removeClass("k-state-selected");\n                $("#quick-edit-" + item.id + "-dropdown").hide();\n            }\n        });\n    }\n\n    function buildItem(config, item, view) {\n        var $input,\n            $textarea,\n            $button,\n            $dropdown,\n            $edit,\n            $select;\n        switch (item.type) {\n        case \'input\':\n            $input = $("#quick-edit-" + item.id);\n            $input.change(function () {\n                var newValue = $(this).val();\n                var feedback = { success: true };\n                _triggerChangeEvent(config.id, item.id, view, newValue, feedback);\n                if (!feedback.success) {\n                    _preventClose = true;\n                    _.defer(function () {\n                        $("#quick-edit").effect("shake", { distance: 10 }, function () {\n                            // when shake animation completed.\n                            _preventClose = false;\n                            $input.select();\n                        });\n                    });\n                } else {\n                    $input.data(\'oldValue\', newValue);\n                }\n\n            });\n            $input.focus(function () {\n                $input.select();\n                _hideDropdowns(null);\n            });\n            $input.focus();\n            break;\n        case \'textarea\':\n            $textarea = $("#quick-edit-" + item.id);\n            $textarea.change(function () {\n                _triggerChangeEvent(config.id, item.id, view, $(this).val());\n            });\n            $textarea.focus(function () {\n                $textarea.select();\n                _hideDropdowns(null);\n            });\n            $textarea.focus();\n            break;\n        case \'dropdown\':\n            $button   = $("#quick-edit-" + item.id);\n            $dropdown = $("#quick-edit-" + item.id + "-dropdown");\n            $dropdown.kendoMenu({\n                orientation: \'vertical\',\n                select: function (e) {\n                    var value = e.item.dataset.data;\n                    if (value == "true") { value = true; }\n                    if (value == "false") { value = false; }\n                    setDropdownValue(item.id, value);\n                    $button.removeClass("k-state-selected");\n                    $dropdown.hide();\n                    _triggerChangeEvent(config.id, item.id, view, value);\n                }\n            });\n            $button.click(function () {\n                _hideDropdowns(item.id);\n                $button.toggleClass("k-state-selected");\n                $dropdown.toggle($button.hasClass("k-state-selected"));\n            });\n            break;\n        case \'button\':\n            $button   = $("#quick-edit-" + item.id);\n            $button.click(function () {\n                _hideDropdowns(null);\n                _triggerClickEvent(config.id, item.id, view);\n            });\n            break;\n        case \'combo\':\n            $edit  = $("#quick-edit-" + item.id + " > input");\n            $select = $("#quick-edit-" + item.id + " > select");\n\n            $edit.change(function () {\n                _triggerChangeEvent(config.id, item.id, view, $edit.val());\n            });\n\n            $select.val($edit.val());\n            $select.change(function () {\n                $edit.val($select.val());\n                $edit.change();\n            });\n            // (for Windows) To prevent immediate collapse of dropdown in comboBox.\n            $select.click(function (e) {\n                $select.focus();\n                e.stopPropagation();\n                return false;\n            });\n\n            $("#quick-edit-" + item.id + "-container input").focus(function () {\n                _hideDropdowns(null);\n            });\n            break;\n        }\n    }\n\n    function getWidth(item) {\n        var w = 0;\n        switch (item.type) {\n        case \'input\':\n            w = INPUT_WIDTH;\n            break;\n        case \'textarea\':\n            w = TEXTAREA_WIDTH;\n            break;\n        case \'dropdown\':\n            if (item.width) {\n                w = item.width;\n            } else {\n                w = DROPDOWN_WIDTH;\n            }\n            break;\n        case \'button\':\n            w = BUTTON_WIDTH;\n            break;\n        case \'combo\':\n            if (item.width) {\n                w = item.width;\n            } else {\n                w = COMBO_WIDTH;\n            }\n            break;\n        }\n        return w;\n    }\n\n    function getHeight(item) {\n        var h = 0;\n        switch (item.type) {\n        case \'button\':\n            h = BUTTON_HEIGHT;\n            break;\n        }\n        return h;\n    }\n\n    /**\n     * Get Double Clicked Position of Edge\n     * @param {GraphicClasses.Points} points\n     * @param {number} x\n     * @param {number} y\n     * @return {string} - \'center\' | \'tail\' | \'head\'\n     */\n    function getPositionOfEdge(points, x, y) {\n        var midPointIndex = 0,\n            midPoint = new Graphics.Point();\n        if (points.count() > 2) {\n            midPointIndex = Math.trunc(points.count() / 2);\n        } else {\n            midPointIndex = 0;\n        }\n        midPoint.x = Math.trunc((points.points[midPointIndex].x + points.points[midPointIndex+1].x) / 2);\n        midPoint.y = Math.trunc((points.points[midPointIndex].y + points.points[midPointIndex+1].y) / 2);\n        var L1 = Math.trunc(Math.sqrt(Math.pow(points.points[0].x - x,2) + Math.pow(points.points[0].y - y,2))),\n            L2 = Math.trunc(Math.sqrt(Math.pow(points.points[points.count()-1].x - x,2) + Math.pow(points.points[points.count()-1].y - y,2))),\n            L3 = Math.trunc(Math.sqrt(Math.pow(midPoint.x - x,2) + Math.pow(midPoint.y - y,2))),\n            minValue = Math.min(Math.min(L1, L2), L3);\n        if (minValue == L3) {\n            return \'center\';\n        } else {\n            if (minValue == L1) {\n                return \'tail\';\n            } else if (minValue == L2) {\n                return \'head\';\n            }\n        }\n    }\n\n    /**\n     * Render QuickEdit\n     *\n     * @param {Editor} editor\n     */\n    function render(editor, rect, view, config) {\n        Coord.coordTransform2(editor.canvas.zoomFactor, Graphics.GridFactor.NO_GRID, rect);\n\n        // insert html tags\n        var html = Mustache.render(TEMPLATE_BEGIN, {left: rect.x1, top: rect.y1, width: rect.getWidth()}),\n            item, x, y, w, h;\n        if (config.leftSection) {\n            x = -2;\n            y = 0;\n            w = rect.getWidth();\n            h = rect.getHeight();\n            _.forEach(config.leftSection, function (item) {\n                w = getWidth(item);\n                x = x - w;\n                item.left = x;\n                item.top = y;\n                item.width = w;\n                item.height = h;\n                html += renderItem(editor, view, item);\n            });\n        }\n\n        if (config.leftSection2) {\n            x = -2;\n            y = BUTTON_HEIGHT;\n            w = rect.getWidth();\n            h = rect.getHeight();\n            _.forEach(config.leftSection2, function (item) {\n                w = getWidth(item);\n                x = x - w;\n                item.left = x;\n                item.top = y;\n                item.width = w;\n                item.height = h;\n                html += renderItem(editor, view, item);\n            });\n        }\n\n        if (config.main) {\n            item = config.main;\n            item.left = rect.x1;\n            item.top = rect.y1;\n            item.width = rect.getWidth();\n            item.height = rect.getHeight();\n            html += renderItem(editor, view, item);\n        }\n\n        if (config.rightSection) {\n            x = rect.getWidth() + 3;\n            y = 0;\n            w = rect.getWidth();\n            h = rect.getHeight();\n            _.forEach(config.rightSection, function (item) {\n                w = getWidth(item);\n                item.left = x;\n                item.top = y;\n                item.width = w;\n                item.height = h;\n                html += renderItem(editor, view, item);\n                w = getWidth(item);\n                x = x + w;\n            });\n        }\n\n        if (config.rightSection2) {\n            x = rect.getWidth() + 3;\n            y = BUTTON_HEIGHT;\n            w = rect.getWidth();\n            h = rect.getHeight();\n            _.forEach(config.rightSection2, function (item) {\n                w = getWidth(item);\n                item.left = x;\n                item.top = y;\n                item.width = w;\n                item.height = h;\n                html += renderItem(editor, view, item);\n                w = getWidth(item);\n                x = x + w;\n            });\n        }\n\n        if (config.rightSection3) {\n            x = rect.getWidth() + 3;\n            y = BUTTON_HEIGHT * 2;\n            w = rect.getWidth();\n            h = rect.getHeight();\n            _.forEach(config.rightSection3, function (item) {\n                w = getWidth(item);\n                item.left = x;\n                item.top = y;\n                item.width = w;\n                item.height = h;\n                html += renderItem(editor, view, item);\n                w = getWidth(item);\n                x = x + w;\n            });\n        }\n\n        if (config.rightSection4) {\n            x = rect.getWidth() + 3;\n            y = BUTTON_HEIGHT * 3;\n            w = rect.getWidth();\n            h = rect.getHeight();\n            _.forEach(config.rightSection4, function (item) {\n                w = getWidth(item);\n                item.left = x;\n                item.top = y;\n                item.width = w;\n                item.height = h;\n                html += renderItem(editor, view, item);\n                w = getWidth(item);\n                x = x + w;\n            });\n        }\n\n        html += TEMPLATE_END;\n        $("body").append(html);\n        // bind events\n        if (config.leftSection) {\n            _.forEach(config.leftSection, function (item) {\n                buildItem(config, item, view);\n            });\n        }\n        if (config.leftSection2) {\n            _.forEach(config.leftSection2, function (item) {\n                buildItem(config, item, view);\n            });\n        }\n        if (config.main) {\n            buildItem(config, config.main, view);\n        }\n        if (config.rightSection) {\n            _.forEach(config.rightSection, function (item) {\n                buildItem(config, item, view);\n            });\n        }\n        if (config.rightSection2) {\n            _.forEach(config.rightSection2, function (item) {\n                buildItem(config, item, view);\n            });\n        }\n        if (config.rightSection3) {\n            _.forEach(config.rightSection3, function (item) {\n                buildItem(config, item, view);\n            });\n        }\n        if (config.rightSection4) {\n            _.forEach(config.rightSection4, function (item) {\n                buildItem(config, item, view);\n            });\n        }\n        // keydown events\n        $("#quick-edit").keydown(function (e) {\n            // default key actions\n            switch (e.which) {\n            case KeyEvent.DOM_VK_RETURN:\n                if (e.shiftKey) {\n                    // If shift key pressed, do not close.\n                } else {\n                    close(true);\n                }\n                break;\n            case KeyEvent.DOM_VK_ESCAPE:\n                close();\n                break;\n            case KeyEvent.DOM_VK_BACK_SPACE:\n                e.stopPropagation();\n                break;\n            case KeyEvent.DOM_VK_DELETE:\n                e.stopPropagation();\n                break;\n            case KeyEvent.DOM_VK_UP:\n            case KeyEvent.DOM_VK_DOWN:\n                if (e.ctrlKey || e.metaKey) {\n                    e.preventDefault();\n                    e.stopPropagation();\n                }\n                break;\n            default:\n                // event.stopPropagation();\n                break;\n            }\n            _triggerKeydownEvent(config.id, item.id, view, e);\n        });\n        // Tooltips\n        /*\n        tooltip = $("#quick-edit").kendoTooltip({\n            filter: ".quick-edit-hint",\n            width: 100,\n            position: "top"\n        }).data("kendoTooltip");\n        */\n    }\n\n    /**\n     * Set Input Value\n     */\n    function setInputValue(id, value) {\n        var $input = $("#quick-edit-" + id);\n        $input.val(value);\n    }\n\n    /**\n     *\n     */\n    function setTextAreaValue(id, value) {\n        $("#quick-edit-" + id).val(value);\n    }\n\n    /**\n     *\n     */\n    function setDropdownValue(id, value) {\n        var item = currentQuickEditItems[id];\n        if (item) {\n            _.forEach(item.items, function (option) {\n                if (option.value == value) {\n                    $("#quick-edit-" + id).empty();\n                    $("#quick-edit-" + id).append("<span class=\'k-sprite " + option.icon + "\'></span>");\n                }\n            });\n        }\n    }\n\n    /**\n     *\n     */\n    function setComboBoxValue(id, value) {\n        var $edit   = $("#quick-edit-" + id + " > input"),\n            $select = $("#quick-edit-" + id + " > select");\n        $edit.val(value);\n        $select.val(value);\n    }\n\n    /**\n     * register QuickEdit\n     * @param {Object} data\n     */\n    function register(data) {\n        quickEditList.push(data);\n    }\n\n    /**\n     * Open QuickEdit on a specific view.\n     * @param {Editor} editor\n     * @param {View} view\n     * @param {number} x\n     * @param {number} y\n     */\n    function open(editor, view, x, y) {\n        var canvas   = $("#" + editor.canvasId),\n            offset   = canvas.offset(),\n            position = new Graphics.Point(offset.left, offset.top);\n        Coord.coordRevTransform(editor.canvas.zoomFactor, Graphics.GridFactor.NO_GRID, position);\n        close();\n        for (var i = 0; i < quickEditList.length; i++) {\n            var quickEdit = quickEditList[i];\n            var pass = {};\n            if (quickEdit.when(view, x, y, pass)) {\n                if (quickEdit.popup) {\n                    currentQuickEdit = quickEdit;\n                    currentQuickEditItems = {};\n                    quickEdit.popup(editor, position, view, pass);\n                }\n                if (quickEdit.init) {\n                    quickEdit.init(view, pass);\n                }\n                // select text in input field\n                $("#quick-edit > input[type=text]").select();\n                // Disable Scrolling\n                $("#diagram-area").css("overflow", "hidden");\n                // Trigger Event\n                _triggerOpenEvent(editor, view, x, y);\n                break;\n            }\n        }\n    }\n\n    /**\n     * Close QuickEdit\n     * @param {boolean} applyChanges\n     */\n    function close(applyChanges) {\n        // Apply Changes before Closing\n        if (applyChanges === true) {\n            // To trigger change events of input and textarea elements,\n            // Focus on a zero-sized button element.\n            $("#quick-edit-outfocus").focus();\n        }\n        // Prevent Closing\n        if (_preventClose) {\n            return;\n        }\n        // Hide Tooltips\n        if (tooltip) {\n            tooltip.hide();\n        }\n        // Remove QuickEdit\n        var $quickEdit = $("#quick-edit");\n        if ($quickEdit.length > 0) {\n            $("#quick-edit").remove();\n            _triggerCloseEvent();\n        }\n        // Enable Scrolling\n        $("#diagram-area").css("overflow", "scroll");\n        // Set Focus to Diagram Area (to allow Enter key to popup QuickEdit)\n        $("#diagram-canvas").focus();\n    }\n\n    /**\n     * Change Event\n     * @event module:ui/QuickEdit.change\n     * @property {string} id QuickEdit Id.\n     * @property {string} itemId Item Id.\n     * @property {View} view View\n     * @property {*} value Value\n     */\n    function _triggerChangeEvent(id, itemId, view, value, feedback) {\n        DiagramManager.suspendRepaint();\n        $(exports).triggerHandler(\'change\', [id, itemId, view, value, feedback]);\n        DiagramManager.resumeRepaint();\n    }\n\n    /**\n     * Click Event\n     * @event module:ui/QuickEdit.click\n     * @property {string} id QuickEdit Id.\n     * @property {string} itemId Item Id.\n     * @property {View} view View\n     * @property {*} value Value\n     */\n    function _triggerClickEvent(id, itemId, view) {\n        DiagramManager.suspendRepaint();\n        $(exports).triggerHandler(\'click\', [id, itemId, view]);\n        DiagramManager.resumeRepaint();\n    }\n\n    /**\n     * Keydown Event\n     * @event module:ui/QuickEdit.keydown\n     * @property {string} id QuickEdit Id.\n     * @property {string} itemId Item Id.\n     * @property {View} view View\n     * @property {JQuery.Event} event\n     */\n    function _triggerKeydownEvent(id, itemId, view, event) {\n        $(exports).triggerHandler(\'keydown\', [id, itemId, view, event]);\n    }\n\n    /**\n     * Open Event\n     * @event module:ui/QuickEdit.open\n     * @property {Editor} editor Diagram Editor\n     * @property {View} view View\n     * @property {number} x\n     * @property {number} y\n     */\n    function _triggerOpenEvent(editor, view, x, y) {\n        $(exports).triggerHandler(\'open\', [editor, view, x, y]);\n    }\n\n    /**\n     * Close Event\n     * @event module:ui/QuickEdit.close\n     */\n    function _triggerCloseEvent() {\n        $(exports).triggerHandler(\'close\', []);\n    }\n\n    function addInQuickEdit(parentModel, field, parentView, compartmentField, Type, namePrefix, initializer) {\n        var item = new Type();\n        item.name = Core.getNewName(parentModel[field], namePrefix);\n        if (initializer) {\n            initializer(item);\n        }\n        Engine.addModel(parentModel, field, item);\n        close();\n        _.defer(function () {\n            _.each(parentView[compartmentField].subViews, function (subView) {\n                if (subView.model._id == item._id) {\n                    open(DiagramManager.getEditor(), subView, subView.left, subView.top);\n                }\n            });\n        });\n    }\n\n    function deleteInQuickEdit(compartmentItemView) {\n        var idx = _.indexOf(compartmentItemView._parent.subViews, compartmentItemView);\n        Engine.deleteElements([compartmentItemView.model], []);\n        close();\n        var subView = compartmentItemView._parent.subViews[idx];\n        if (subView) {\n            _.defer(function () {\n                open(DiagramManager.getEditor(), subView, subView.left, subView.top);\n            });\n        }\n    }\n\n    function openUpInQuickEdit(compartmentItemView) {\n        var idx = _.indexOf(compartmentItemView._parent.subViews, compartmentItemView);\n        if (idx > 0) {\n            var target = compartmentItemView._parent.subViews[idx - 1];\n            _.defer(function () {\n                open(DiagramManager.getEditor(), target, target.left, target.top);\n            });\n        }\n    }\n\n    function openDownInQuickEdit(compartmentItemView) {\n        var idx = _.indexOf(compartmentItemView._parent.subViews, compartmentItemView);\n        if (idx < (compartmentItemView._parent.subViews.length - 1)) {\n            var target = compartmentItemView._parent.subViews[idx + 1];\n            _.defer(function () {\n                open(DiagramManager.getEditor(), target, target.left, target.top);\n            });\n        }\n    }\n\n    function moveUpInQuickEdit(parentModel, field, compartmentItemView) {\n        Engine.moveUp(parentModel, field, compartmentItemView.model);\n        close();\n        _.defer(function () {\n            _.each(compartmentItemView._parent.subViews, function (subView) {\n                if (subView.model._id == compartmentItemView.model._id) {\n                    open(DiagramManager.getEditor(), subView, subView.left, subView.top);\n                }\n            });\n        });\n    }\n\n    function moveDownInQuickEdit(parentModel, field, compartmentItemView) {\n        Engine.moveDown(parentModel, field, compartmentItemView.model);\n        close();\n        _.defer(function () {\n            _.each(compartmentItemView._parent.subViews, function (subView) {\n                if (subView.model._id == compartmentItemView.model._id) {\n                    open(DiagramManager.getEditor(), subView, subView.left, subView.top);\n                }\n            });\n        });\n    }\n\n    function getHeadNodes(view, edgeType) {\n        var edges = _.filter(Repository.getEdgeViewsOf(view), function (e) {\n            return (e instanceof edgeType && e.tail === view);\n        });\n        var nodes = _.map(edges, function (e) { return e.head; });\n        return nodes;\n    }\n\n    function getTailNodes(view, edgeType) {\n        var edges = _.filter(Repository.getEdgeViewsOf(view), function (e) {\n            return (e instanceof edgeType && e.head === view);\n        });\n        var nodes = _.map(edges, function (e) { return e.tail; });\n        return nodes;\n    }\n\n    function getBothNodes(view, edgeType, predicate) {\n        var edges = _.filter(Repository.getEdgeViewsOf(view), function (e) {\n            return (e instanceof edgeType && (predicate ? predicate(e) : true));\n        });\n        var nodes = _.map(edges, function (e) {\n            if (e.head === view) {\n                return e.tail;\n            } else {\n                return e.head;\n            }\n        });\n        return nodes;\n    }\n\n    function setInside(rect) {\n        if (rect.x1 < 0) { rect.x1 = 0; }\n        if (rect.x2 < 0) { rect.x2 = 0; }\n        if (rect.y1 < 0) { rect.y1 = 0; }\n        if (rect.y2 < 0) { rect.y2 = 0; }\n    }\n\n    function getEdgeViewOption(fromView, toView) {\n        return {\n            tailView  : fromView,\n            headView  : toView,\n            tailModel : fromView.model || null,\n            headModel : toView.model || null\n        };\n    }\n\n    function getTopPosition(view, refNodes, gap) {\n        gap = gap || 100;\n        var rect = {\n            x1: view.left,\n            y1: view.top - gap,\n            x2: view.left,\n            y2: view.top - gap\n        };\n        if (refNodes.length > 0) {\n            var rightMost = _.max(refNodes, function (e) { return e.getRight(); });\n            if (Graphics.Coord.rectInRect(rightMost.getBoundingBox(), DiagramManager.getDiagramArea())) {\n                rect = {\n                    x1 : rightMost.getRight() + 10,\n                    y1 : rightMost.top,\n                    x2 : rightMost.getRight() + 20,\n                    y2 : rightMost.top + 10\n                };\n            }\n        }\n        setInside(rect);\n        return rect;\n    }\n\n    function getBottomPosition(view, refNodes, gap) {\n        gap = gap || 70;\n        var rect = {\n            x1: view.left,\n            y1: view.getBottom() + gap,\n            x2: view.left,\n            y2: view.getBottom() + gap\n        };\n        if (refNodes.length > 0) {\n            var rightMost = _.max(refNodes, function (e) { return e.getRight(); });\n            if (Graphics.Coord.rectInRect(rightMost.getBoundingBox(), DiagramManager.getDiagramArea())) {\n                rect = {\n                    x1 : rightMost.getRight() + 10,\n                    y1 : rightMost.top,\n                    x2 : rightMost.getRight() + 20,\n                    y2 : rightMost.top + 10\n                };\n            }\n        }\n        setInside(rect);\n        return rect;\n    }\n\n    function getLeftPosition(view, refNodes) {\n        var rect = {\n            x1: view.left - 100,\n            y1: view.top,\n            x2: view.left - 100,\n            y2: view.top\n        };\n        if (refNodes.length > 0) {\n            var bottomMost = _.max(refNodes, function (e) { return e.getBottom(); });\n            if (Graphics.Coord.rectInRect(bottomMost.getBoundingBox(), DiagramManager.getDiagramArea())) {\n                rect = {\n                    x1 : bottomMost.left,\n                    y1 : bottomMost.getBottom() + 10,\n                    x2 : bottomMost.getRight(),\n                    y2 : bottomMost.getBottom() + 20\n                };\n            }\n        }\n        setInside(rect);\n        return rect;\n    }\n\n    function getRightPosition(view, refNodes) {\n        var rect = {\n            x1: view.getRight() + 70,\n            y1: view.top,\n            x2: view.getRight() + 70,\n            y2: view.top\n        };\n        if (refNodes.length > 0) {\n            var bottomMost = _.max(refNodes, function (e) { return e.getBottom(); });\n            if (Graphics.Coord.rectInRect(bottomMost.getBoundingBox(), DiagramManager.getDiagramArea())) {\n                rect = {\n                    x1 : bottomMost.left,\n                    y1 : bottomMost.getBottom() + 10,\n                    x2 : bottomMost.getRight(),\n                    y2 : bottomMost.getBottom() + 20\n                };\n            }\n        }\n        setInside(rect);\n        return rect;\n    }\n\n\n    // Define public API\n    exports.render            = render;\n    exports.setInputValue     = setInputValue;\n    exports.setTextAreaValue  = setTextAreaValue;\n    exports.setDropdownValue  = setDropdownValue;\n    exports.setComboBoxValue  = setComboBoxValue;\n    exports.getPositionOfEdge = getPositionOfEdge;\n    exports.register          = register;\n    exports.open              = open;\n    exports.close             = close;\n\n    exports.addInQuickEdit      = addInQuickEdit;\n    exports.deleteInQuickEdit   = deleteInQuickEdit;\n    exports.openUpInQuickEdit   = openUpInQuickEdit;\n    exports.openDownInQuickEdit = openDownInQuickEdit;\n    exports.moveUpInQuickEdit   = moveUpInQuickEdit;\n    exports.moveDownInQuickEdit = moveDownInQuickEdit;\n\n    exports.getHeadNodes = getHeadNodes;\n    exports.getTailNodes = getTailNodes;\n    exports.getBothNodes = getBothNodes;\n\n    exports.setInside         = setInside;\n    exports.getEdgeViewOption = getEdgeViewOption;\n    exports.getTopPosition    = getTopPosition;\n    exports.getBottomPosition = getBottomPosition;\n    exports.getLeftPosition   = getLeftPosition;\n    exports.getRightPosition  = getRightPosition;\n\n});\n\n//# sourceURL=/ui/QuickEdit.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, Mustache, app, document */\n\n/**\n * MenuManager\n */\ndefine(\'menu/MenuManager\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Global\',\'utils/Helper\',\'command/Commands\',\'command/CommandManager\',\'command/KeyBindingManager\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit           = require("utils/AppInit"),\n        global            = require("core/Global").global,\n        Helper            = require("utils/Helper"),\n        Commands          = require("command/Commands"),\n        CommandManager    = require("command/CommandManager"),\n        KeyBindingManager = require("command/KeyBindingManager");\n\n    /**\n     * Error Codes from AppShell\n     * @enum {number}\n     */\n    var NO_ERROR           = 0,\n        ERR_UNKNOWN        = 1,\n        ERR_INVALID_PARAMS = 2,\n        ERR_NOT_FOUND      = 3;\n\n    /**\n     * Insertion position constants\n     * Used by addMenu(), addMenuItem(), and addSubMenu() to\n     * specify the relative position of a newly created menu object\n     * @enum {string}\n     */\n    var BEFORE           = "before",\n        AFTER            = "after",\n        FIRST            = "first",\n        LAST             = "last",\n        FIRST_IN_SECTION = "firstInSection",\n        LAST_IN_SECTION  = "lastInSection";\n\n    /**\n     * Other constants\n     */\n    var DIVIDER = "---";\n\n    /**\n     * Menu Data\n     */\n    var menuData = [];\n\n    /**\n     * Maps menuID\'s to Menu objects\n     * @type {Object.<string, Menu>}\n     * @private\n     */\n    var menuMap = {};\n\n    /**\n     * Maps menuItemID\'s to MenuItem objects\n     * @type {Object.<string, MenuItem>}\n     * @private\n     */\n    var menuItemMap = {};\n\n\n    var $menubar,\n        $mainView;\n\n    /**\n     * Kendo Menu Object\n     * @private\n     */\n    var kendoMenu;\n\n    /**\n     * Add menuItem to a given position\n     * @private\n     */\n    function _addToPosition(array, dataItem, position, relativeId) {\n        var relativeItem,\n            relativeItemIdx;\n\n        if (relativeId) {\n            relativeItem = _.find(\n                array,\n                function (item) {\n                    return item.id === relativeId;\n                }\n            );\n            relativeItemIdx = _.indexOf(array, relativeItem);\n        }\n        switch (position) {\n        case FIRST:\n            array.unshift(dataItem);\n            break;\n        case LAST:\n            array.push(dataItem);\n            break;\n        case BEFORE:\n            array.splice(relativeItemIdx, 0, dataItem);\n            break;\n        case AFTER:\n            array.splice(relativeItemIdx + 1, 0, dataItem);\n            break;\n        case FIRST_IN_SECTION:\n            // TODO: Support FIRST_IN_SECTION\n            break;\n        case LAST_IN_SECTION:\n            // TODO: Support LAST_IN_SECTION\n            break;\n        default:\n            array.push(dataItem);\n            break;\n        }\n\n    }\n\n\n    /**\n     * Return shortcut key string for commandID\n     * @private\n     * @param {string} commandID\n     * @return {string}\n     */\n    function _getDisplayKey(commandID) {\n        var bindings = KeyBindingManager.getKeyBindings(commandID),\n            binding,\n            bindingStr = "",\n            displayStr = "";\n        if (bindings && bindings.length > 0) {\n            binding = bindings[bindings.length - 1];\n            bindingStr = binding.displayKey || binding.key;\n        }\n        if (bindingStr.length > 0) {\n            displayStr = KeyBindingManager.formatKeyDescriptor(bindingStr);\n        }\n        return displayStr;\n    }\n\n    /**\n     * Return menu name with display key\n     */\n    function _getMenuString(name, displayKey) {\n        var str = name;\n        if (displayKey && displayKey.length > 0) {\n\n        }\n        var _key = "<span style=\'margin-right: {{margin}}em;\'></span><span class=\'key-binding\' style=\'position: absolute; right: 0; margin-right: 1.2em;\'>{{key}}</span>";\n        str += Mustache.render(_key, { margin: displayKey.length / 2, key: displayKey});\n        return str;\n    }\n\n    /**\n     * Retrieves the Menu object for the corresponding id.\n     * @param {string} id Unique identifier or command id for Menu.\n     * @return {Menu} Menu Object\n     */\n    function getMenu(id) {\n        return menuMap[id];\n    }\n\n    /**\n     * Retrieves the map of all Menu objects.\n     * @return {Object.<string, Menu>} Map of All Menus.\n     */\n    function getAllMenus() {\n        return menuMap;\n    }\n\n    /**\n     * Retrieves the MenuItem object for the corresponding id.\n     * @param {string} id Unique identifier or command id for MenuItem.\n     * @return {MenuItem} MenuItem Object\n     */\n    function getMenuItem(id) {\n        return menuItemMap[id];\n    }\n\n    var _menuDividerIDCount = 1;\n    function _getNextMenuItemDividerID() {\n        return "staruml-menuDivider-" + _menuDividerIDCount++;\n    }\n\n\n    function isHTMLMenu() {\n        return (!app.nativeMenus);\n    }\n\n    /**\n     * MenuEntry.\n     * Abstraction of Menu and MenuItem.\n     * @constructor\n     * @param {string} id\n     * @param {string} text\n     */\n    function MenuEntry(id, text) {\n\n        /** @member {string} */\n        this.id = id;\n\n        /** @member {string} */\n        this.text = text;\n\n        /** @member {string} */\n        this.data = id; // for "data-data" attribute for HTML menu item\n\n        /** @member {Array.<MenuEntry>} */\n        this.items = [];\n    }\n\n    /**\n     * Add MenuItem\n     * @param {!string | Command} commandId\n     * @param {} keyBindings\n     * @param {?string} position\n     * @param {?string} relativeId\n     * @param {?string} iconClass\n     */\n    MenuEntry.prototype.addMenuItem = function (command, keyBindings, position, relativeId, iconClass) {\n        var menuId = this.id,\n            menuItem,\n            id,\n            name,\n            commandID;\n\n        if (!command) {\n            console.error("addMenuItem(): missing required parameters: command");\n            return null;\n        }\n\n        if (typeof (command) === "string") {\n            if (command === DIVIDER) {\n                name = DIVIDER;\n                commandID = _getNextMenuItemDividerID();\n            } else {\n                commandID = command;\n                command = CommandManager.get(commandID);\n                if (!command) {\n                    console.error("addMenuItem(): commandID not found: " + commandID);\n                    return null;\n                }\n                name = command.getName();\n            }\n        } else {\n            commandID = command.getID();\n            name = command.getName();\n        }\n\n        id = commandID;\n        if (menuItemMap[id]) {\n            console.log("MenuItem added with same id of existing MenuItem: " + id);\n            return null;\n        }\n\n        menuItem = new MenuItem(id, command, name, iconClass);\n        _addToPosition(this.items, menuItem, position, relativeId); // add to sub menu items\n        menuItemMap[id] = menuItem; // register to menuItemMap\n\n        // Set KeyBinding Display\n        var bindings = KeyBindingManager.getKeyBindings(commandID),\n            binding,\n            bindingStr = "",\n            displayStr = "";\n        if (bindings && bindings.length > 0) {\n            binding = bindings[bindings.length - 1];\n            bindingStr = binding.displayKey || binding.key;\n        }\n        if (bindingStr.length > 0) {\n            displayStr = KeyBindingManager.formatKeyDescriptor(bindingStr);\n        }\n\n        // Add Menu Item as Native or HTML\n        if (isHTMLMenu()) {\n            if (command === DIVIDER) {\n                if (this.$element) {\n                    var dividerJson = {\n                        text: "<hr size=\'0px\'>",\n                        data: id,\n                        encoded: false,\n                        enabled: false,\n                        cssClass: "k-menu-separator"\n                    };\n                    kendoMenu.append([dividerJson], this.$element);\n                    menuItem.$element = $(\'li[data-data=""\' + id + \'""]\', this.$element);\n                } else {\n                    menuItem.text = "<hr size=\'0px\'>";\n                    menuItem.encoded = false;\n                    menuItem.enabled = false;\n                    menuItem.cssClass = "k-menu-separator";\n                }\n            } else {\n                if (this.$element) { // HTML Menu is already built.\n                    var jsonItem = {\n                        text: _getMenuString(name, displayStr),\n                        data: id,\n                        encoded: false,\n                        items: [],\n                        spriteCssClass: iconClass\n                    };\n                    kendoMenu.append([jsonItem], this.$element);\n                    menuItem.$element = $(\'li[data-data=""\' + id + \'""]\', this.$element);\n                } else { // HTML Menu is not yey build\n                    if (displayStr.length > 0) {\n                        menuItem.text = _getMenuString(name, displayStr);\n                        menuItem.encoded = false;\n                    }\n                }\n            }\n            menuItem.isNative = false;\n        } else {\n            appshell.app.addMenuItem(menuId, name, commandID, bindingStr, displayStr, position, relativeId, function (err) {\n                switch (err) {\n                case NO_ERROR:\n                    break;\n                case ERR_INVALID_PARAMS:\n                    console.error("addMenuItem(): Invalid Parameters when adding the command " + commandID);\n                    break;\n                case ERR_NOT_FOUND:\n                    console.error("_getRelativeMenuItem(): MenuItem with Command id " + relativeId + " not found in the Menu " + menuId);\n                    break;\n                default:\n                    console.error("addMenuItem(); Unknown Error (" + err + ") when adding the command " + commandID);\n                }\n            });\n            menuItem.isNative = true;\n        }\n\n        // Initialize MenuItem state\n        if (menuItem.isDivider) {\n            menuItem.dividerId = commandID;\n        } else {\n            if (keyBindings) {\n                // Add key bindings. The MenuItem listens to the Command object to update MenuItem DOM with shortcuts.\n                if (!Array.isArray(keyBindings)) {\n                    keyBindings = [keyBindings];\n                }\n            }\n\n            // Note that keyBindings passed during MenuItem creation take precedent over any existing key bindings\n            KeyBindingManager.addBinding(commandID, keyBindings);\n\n            // Look for existing key bindings\n            // _addExistingKeyBinding(menuItem, commandID);\n\n            menuItem._checkedChanged();\n            menuItem._enabledChanged();\n            menuItem._nameChanged();\n        }\n\n        return menuItem;\n    };\n\n    /**\n     * Add Menu Divier\n     */\n    MenuEntry.prototype.addMenuDivider = function (position, relativeId) {\n        this.addMenuItem(DIVIDER, "", position, relativeId);\n    };\n\n    /**\n     * Removes the specified menu item from this Menu. Key bindings are unaffected; use KeyBindingManager\n     * directly to remove key bindings if desired.\n     *\n     * @param {!string | Command} command - command the menu would execute if we weren\'t deleting it.\n     */\n    MenuEntry.prototype.removeMenuItem = function (command) {\n        var menuItemID,\n            commandID,\n            menuItem;\n\n        if (!command) {\n            console.error("removeMenuItem(): missing required parameters: command");\n            return;\n        }\n\n        if (typeof (command) === "string") {\n            var commandObj = CommandManager.get(command);\n            if (!commandObj) {\n                console.error("removeMenuItem(): command not found: " + command);\n                return;\n            }\n\n            commandID = command;\n        } else {\n            commandID = command.getID();\n        }\n        menuItemID = commandID;\n        menuItem = getMenuItem(menuItemID);\n\n        // Remove all sub menu items\n        if (menuItem) {\n            _.forEach(menuItem.items, function (item, idx) {\n                if (item.isDivider) {\n                    menuItem.removeMenuDivider(item.id);\n                } else {\n                    commandID = item.getCommand();\n                    menuItem.removeMenuItem(commandID);\n                }\n            });\n            menuItem.items = [];\n\n        }\n\n        // Remove this menu item.\n        if (isHTMLMenu()) {\n            if (menuItem && menuItem.$element) {\n                menuItem.$element.remove();\n            }\n        } else {\n            appshell.app.removeMenuItem(menuItemID, function (err) {\n                if (err) {\n                    console.error("removeMenuItem() -- command not found: " + commandID + " (error: " + err + ")");\n                }\n            });\n        }\n\n        delete menuItemMap[menuItemID];\n    };\n\n    /**\n     * Removes the specified menu divider from this Menu.\n     *\n     * @param {!string} menuItemID - the menu item id of the divider to remove.\n     */\n    MenuEntry.prototype.removeMenuDivider = function (menuItemID) {\n        var menuItem;\n\n        if (!menuItemID) {\n            console.error("removeMenuDivider(): missing required parameters: menuItemID");\n            return;\n        }\n\n        menuItem = getMenuItem(menuItemID);\n\n        if (!menuItem) {\n            console.error("removeMenuDivider(): parameter menuItemID: %s is not a valid menu item id", menuItemID);\n            return;\n        }\n\n        if (!menuItem.isDivider) {\n            console.error("removeMenuDivider(): parameter menuItemID: %s is not a menu divider", menuItemID);\n            return;\n        }\n\n        appshell.app.removeMenuItem(menuItem.dividerId, function (err) {\n            if (err) {\n                console.error("removeMenuDivider() -- divider not found: %s (error: %s)", menuItemID, err);\n            }\n        });\n\n        if (!menuItemMap[menuItemID]) {\n            console.error("removeMenuDivider(): menu divider not found in menuItemMap: %s", menuItemID);\n            return;\n        }\n\n        delete menuItemMap[menuItemID];\n\n    };\n\n\n    /**\n     * Menu\n     * @constructor\n     */\n    function Menu(id, text) {\n        MenuEntry.call(this, id, text);\n    }\n    Menu.prototype = Object.create(MenuEntry.prototype);\n    Menu.prototype.constructor = Menu;\n    Menu.prototype.parentClass = MenuEntry.prototype;\n\n    /**\n     * Menu Item\n     * @constructor\n     */\n    function MenuItem(id, command, text, iconClass) {\n        MenuEntry.call(this, id, text);\n        this.isDivider = (command === DIVIDER);\n        this.isNative = !isHTMLMenu();\n        this.spriteCssClass = iconClass;\n        if (!this.isDivider) {\n            // Bind event handlers\n            this._enabledChanged = this._enabledChanged.bind(this);\n            this._checkedChanged = this._checkedChanged.bind(this);\n            this._visibleChanged = this._visibleChanged.bind(this);\n            this._nameChanged = this._nameChanged.bind(this);\n            this._keyBindingAdded = this._keyBindingAdded.bind(this);\n            this._keyBindingRemoved = this._keyBindingRemoved.bind(this);\n            this._command = command;\n            $(this._command)\n                .on("enabledStateChange", this._enabledChanged)\n                .on("checkedStateChange", this._checkedChanged)\n                .on("visibleStateChange", this._visibleChanged)\n                .on("nameChange", this._nameChanged)\n                .on("keyBindingAdded", this._keyBindingAdded)\n                .on("keyBindingRemoved", this._keyBindingRemoved);\n        }\n    }\n    MenuItem.prototype = Object.create(MenuEntry.prototype);\n    MenuItem.prototype.constructor = MenuItem;\n    MenuItem.prototype.parentClass = MenuEntry.prototype;\n\n    /**\n     * Synchronizes MenuItem checked state with underlying Command checked state\n     */\n    MenuItem.prototype._checkedChanged = function () {\n        var checked = !!this._command.getChecked();\n        if (this.isNative) {\n            var enabled = !!this._command.getEnabled();\n            appshell.app.setMenuItemState(this._command.getID(), enabled, checked, function (err) {\n                if (err) {\n                    console.log("Error setting menu item state: " + err);\n                }\n            });\n        } else {\n            // HTML Menu\n            if (this.$element) {\n                var $inside = $("span.k-link", this.$element),\n                    $checkIcon = $inside.children("span.k-i-tick");\n                if (checked) {\n                    if ($checkIcon.length === 0) {\n                        $inside.prepend("<span class=\'k-icon k-i-tick\'></span>");\n                    }\n                } else {\n                    $checkIcon.remove();\n                }\n\n            }\n        }\n    };\n\n    /**\n     * Synchronizes MenuItem enabled state with underlying Command enabled state\n     */\n    MenuItem.prototype._enabledChanged = function () {\n        var enabled = !!this._command.getEnabled();\n        if (this.isNative) {\n            var checked = !!this._command.getChecked();\n            appshell.app.setMenuItemState(this._command.getID(), enabled, checked, function (err) {\n                if (err) {\n                    console.log("Error setting menu item state: " + err);\n                }\n            });\n        } else {\n            // HTML Menu\n            if (this.$element) {\n                this.$element.toggleClass("k-state-disabled", !enabled);\n            }\n        }\n    };\n\n    /**\n     * Synchronizes MenuItem visible state with underlying Command visible state\n     * (Temporally visible is same with enabled)\n     */\n    MenuItem.prototype._visibleChanged = function () {\n        var visible = !!this._command.getVisible();\n        if (this.isNative) {\n            var checked = !!this._command.getChecked();\n            appshell.app.setMenuItemState(this._command.getID(), visible, checked, function (err) {\n                if (err) {\n                    console.log("Error setting menu item state: " + err);\n                }\n            });\n        } else {\n            // HTML Menu\n            if (this.$element) {\n                this.$element.toggle(visible);\n            }\n        }\n    };\n\n    /**\n     * Synchronizes MenuItem name with underlying Command name\n     */\n    MenuItem.prototype._nameChanged = function () {\n        var self = this;\n        if (this.isNative) {\n            appshell.app.setMenuTitle(this._command.getID(), this._command.getName(), function (err) {\n                if (err) {\n                    console.log(self._command);\n                    console.log("Error setting menu title: " + err);\n                }\n            });\n        } else {\n            // HTML Menu\n            if (!this.$element) {\n                this.text = _getMenuString(this._command.getName(), _getDisplayKey(this._command.getID()));\n                this.encoded = false;\n            } else {\n                var $inside = $("span.k-link", this.$element);\n                $inside.html(_getMenuString(this._command.getName(), _getDisplayKey(this._command.getID())));\n            }\n        }\n    };\n\n    /**\n     * @private\n     * Updates MenuItem DOM with a keyboard shortcut label\n     */\n    MenuItem.prototype._keyBindingAdded = function (event, keyBinding) {\n        var shortcutKey = keyBinding.displayKey || keyBinding.key;\n        if (this.isNative) {\n            appshell.app.setMenuItemShortcut(this._command.getID(), shortcutKey, KeyBindingManager.formatKeyDescriptor(shortcutKey), function (err) {\n                if (err) {\n                    console.error("Error setting menu item shortcut: " + err);\n                }\n            });\n        } else {\n            // HTML Menu\n            if (!this.$element) {\n                this.text = _getMenuString(this._command.getName(), _getDisplayKey(this._command.getID()));\n                this.encoded = false;\n            } else {\n                var $inside = $("span.k-link", this.$element);\n                $inside.html(_getMenuString(this._command.getName(), _getDisplayKey(this._command.getID())));\n            }\n        }\n    };\n\n    /**\n     * @private\n     * Updates MenuItem DOM to remove keyboard shortcut label\n     */\n    MenuItem.prototype._keyBindingRemoved = function (event, keyBinding) {\n        if (this.isNative) {\n            appshell.app.setMenuItemShortcut(this._command.getID(), "", "", function (err) {\n                if (err) {\n                    console.error("Error setting menu item shortcut: " + err);\n                }\n            });\n        } else {\n            // HTML Menu\n        }\n    };\n\n    /**\n     * Gets the Command associated with a MenuItem\n     * @return {Command}\n     */\n    MenuItem.prototype.getCommand = function () {\n        return this._command;\n    };\n\n    // ----\n\n    /**\n     * Add Menu\n     * @param {string | Command} id\n     *     Unique identifier for Menu. Typically use the id of command corresponding the menu.\n     * @param {?string} position - constant defining the position of new MenuItem relative to\n     *      other MenuItems. Values:\n     *          - With no relativeID, use Menus.FIRST or LAST (default is LAST)\n     *          - Relative to a command id, use BEFORE or AFTER (required)\n     *          - Relative to a MenuSection, use FIRST_IN_SECTION or LAST_IN_SECTION (required)\n     * @param {?string} relativeId - command id. Required for all position constants except FIRST and LAST.\n     */\n    function addMenu(id, position, relativeId) {\n        var menu,\n            commandID,\n            _id,\n            name;\n\n        if (!id) {\n            console.error("addMenu(): missing required parameters: id");\n            return null;\n        }\n\n        if (typeof (id) === "string") {\n            commandID = id;\n            id = CommandManager.get(commandID);\n            if (!id) {\n                console.error("addMenu(): commandID not found: " + commandID);\n                return null;\n            }\n            name = id.getName();\n        } else {\n            commandID = id.getID();\n            name = id.getName();\n        }\n\n        // Guard against duplicate menu ids\n        _id = commandID;\n        if (menuMap[_id]) {\n            console.error("Menu added with same name and id of existing Menu: " + _id);\n            return null;\n        }\n\n        menu = new Menu(_id, name);\n        menuMap[_id] = menu; // register to menuMap\n        _addToPosition(menuData, menu, position, relativeId); // add to menuData\n\n        if (isHTMLMenu()) {\n            // ...\n        } else {\n            appshell.app.addMenu(name, _id, position, relativeId, function (err) {\n                switch (err) {\n                case NO_ERROR:\n                    // Make sure name is up to date\n                    appshell.app.setMenuTitle(_id, name, function (err) {\n                        if (err) {\n                            console.error("setMenuTitle() -- error: " + err);\n                        }\n                    });\n                    break;\n                case ERR_UNKNOWN:\n                    console.error("addMenu(): Unknown Error when adding the menu " + _id);\n                    break;\n                case ERR_INVALID_PARAMS:\n                    console.error("addMenu(): Invalid Parameters when adding the menu " + _id);\n                    break;\n                case ERR_NOT_FOUND:\n                    console.error("addMenu(): Menu with command " + relativeId + " could not be found when adding the menu " + _id);\n                    break;\n                default:\n                    console.error("addMenu(): Unknown Error (" + err + ") when adding the menu " + _id);\n                }\n            });\n        }\n        return menu;\n    }\n\n    /**\n     * Removes a top-level menu from the application menu bar which may be native or HTML-based.\n     *\n     * @param {!string} id\n     *     Unique identifier for Menu. Typically use the id of command corresponding the menu.\n     */\n    function removeMenu(id) {\n        var menu,\n            commandID = "";\n\n        if (!id) {\n            console.error("removeMenu(): missing required parameter: id");\n            return;\n        }\n\n        if (!menuMap[id]) {\n            console.error("removeMenu(): menu id not found: %s", id);\n            return;\n        }\n\n        // Remove all of the menu items in the menu\n        menu = getMenu(id);\n        _.forEach(menu.items, function (item, key) {\n            if (item.isDivider) {\n                menu.removeMenuDivider(item.id);\n            } else {\n                commandID = item.getCommand();\n                menu.removeMenuItem(commandID);\n            }\n\n        });\n        menu.items = [];\n\n        // Remove this menu.\n        if (isHTMLMenu()) {\n            // ...\n        } else {\n            appshell.app.removeMenu(id, function (err) {\n                if (err) {\n                    console.error("removeMenu() -- id not found: " + id + " (error: " + err + ")");\n                }\n            });\n        }\n        delete menuMap[id];\n    }\n\n\n\n    /**\n     * 메인 메뉴를 셋팅한다.\n     */\n    function _setupMainMenu() {\n        // TODO: Rewrite shellAPI binding (refer to Brackets)\n        if (window.appshell) {\n            appshell.shellAPI = {\n                executeCommand: function (id) {\n                    if ($(".modal.instance").length > 0) {\n                        // When modal dialog is showing, application exit is not allowed.\n                        // Return true results preventing application exit.\n                        return (id === Commands.FILE_CLOSE_WINDOW);\n                    }\n\n                    // For allow Ctrl+C,V,X,Z,Y for focused <input> and <textarea>\n                    if (id === Commands.EDIT_COPY ||\n                        id === Commands.EDIT_CUT ||\n                        id === Commands.EDIT_DELETE ||\n                        id === Commands.EDIT_PASTE ||\n                        id === Commands.EDIT_UNDO ||\n                        id === Commands.EDIT_REDO) {\n                        if (document.activeElement.nodeName === "TEXTAREA" ||\n                            document.activeElement.nodeName === "INPUT") {\n                            return false;\n                        }\n                    }\n\n                    // BELOW IS FROM BRACKETS (utils.ShellAPI.js)\n                    // Use E for Error so that uglify doesn\'t change this to simply Error()\n                    var promise, E = Error, e = new E(), stackDepth = e.stack.split("\\n").length;\n\n                    // This function should *only* be called as a top-level function. If the current\n                    // stack depth is > 2, it is most likely because we are at a breakpoint.\n                    if (stackDepth < 3) {\n                        promise = CommandManager.execute(id);\n                    } else {\n                        console.error("Skipping command " + id + " because it looks like you are " +\n                                      "at a breakpoint. If you are NOT at a breakpoint, please " +\n                                      "file a bug and mention this comment. Stack depth = " + stackDepth + ".");\n                    }\n                    return (promise && _.isFunction(promise.state) && promise.state() === "rejected") ? false : true;\n                }\n            };\n        }\n    }\n\n    function _setupHTMLMenu() {\n        if (isHTMLMenu()) {\n\n            // Remove empty items\n            _.forEach(menuItemMap, function (item, key) {\n                if (item.items.length < 1) {\n                    delete item.items;\n                }\n            });\n\n            // Create main menu\n            $menubar = $("#menu-bar");\n            $mainView = $(".main-view");\n            $menubar.kendoMenu({\n                dataSource: menuData,\n                animation: false,\n                select: function (e) {\n                    var menuId = JSON.parse(e.item.dataset.data);\n                    CommandManager.execute(menuId);\n                }\n            });\n            kendoMenu = $menubar.data("kendoMenu");\n            $mainView.css("top", $menubar.outerHeight());\n\n            // Initialize menu items.\n            // For all <LI> element and assign it to menuItem.$element.\n            var $allItems = $("li.k-item[role=menuitem]", $menubar);\n            $allItems.each(function () {\n                var $item = $(this),\n                    data = $item.data("data");\n                if (data) {\n                    var menuItemId = JSON.parse(data);\n                    var menuItem = menuItemMap[menuItemId] || menuMap[menuItemId];\n                    menuItem.$element = $item;\n                }\n            });\n\n        }\n    }\n\n    AppInit.htmlReady(function () {\n        _setupMainMenu();\n    });\n\n    AppInit.appReady(function () {\n        _setupHTMLMenu();\n    });\n\n    exports.BEFORE           = BEFORE;\n    exports.AFTER            = AFTER;\n    exports.LAST             = LAST;\n    exports.FIRST            = FIRST;\n    exports.FIRST_IN_SECTION = FIRST_IN_SECTION;\n    exports.LAST_IN_SECTION  = LAST_IN_SECTION;\n    exports.DIVIDER          = DIVIDER;\n\n    exports.getMenu     = getMenu;\n    exports.getAllMenus = getAllMenus;\n    exports.getMenuItem = getMenuItem;\n    exports.addMenu     = addMenu;\n    exports.removeMenu  = removeMenu;\n\n    // for Test\n    exports.menuMap     = menuMap;\n    exports.menuItemMap = menuItemMap;\n\n\n});\n\n//# sourceURL=/menu/MenuManager.js'),eval('define(\'text!htmlContent/update-dialog.html\',[],function () { return \'<div class="update-dialog dialog modal" data-title="Update Available">\\n    <div class="dialog-body no-padding">\\n        <div style="position: absolute; left: 1.5em; top: 1em;">\\n            <img src="styles/icons/package.svg" width="64px" height="64px">\\n        </div>\\n        <div style="margin-left: 80px;">\\n            <div class="release-version">Release {{release.version}} ({{release.date}})</div>\\n            <ul>\\n                {{#release.features}}\\n                <li>\\n                    {{name}}\\n                    <span class="desc">{{description}}</span>\\n                </li>\\n                {{/release.features}}\\n            </ul>\\n        </div>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button left" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.DOWNLOAD}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/update-dialog.html'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell */\ndefine(\'utils/CheckUpdates\',[\'require\',\'exports\',\'module\',\'core/Global\',\'core/Repository\',\'command/Commands\',\'command/CommandManager\',\'menu/MenuManager\',\'explorer/ModelExplorerView\',\'dialogs/Dialogs\',\'strings\',\'utils/PanelManager\',\'utils/ExtensionUtils\',\'extensibility/ExtensionManager\',\'text!htmlContent/update-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var global               = require("core/Global").global,\n        Repository           = require("core/Repository"),\n        Commands             = require("command/Commands"),\n        CommandManager       = require("command/CommandManager"),\n        MenuManager          = require("menu/MenuManager"),\n        ModelExplorerView    = require("explorer/ModelExplorerView"),\n        Dialogs              = require("dialogs/Dialogs"),\n        Strings              = require("strings"),\n        PanelManager         = require("utils/PanelManager"),\n        ExtensionUtils       = require("utils/ExtensionUtils"),\n        ExtensionManager     = require("extensibility/ExtensionManager"),\n        updateDialogTemplate = require("text!htmlContent/update-dialog.html");\n\n    \n    function _setUpdateAvailableIcon(newRelease) {\n        $("#toolbar-updates-available").remove();\n        var $button = $("<a id=\'toolbar-updates-available\' href=\'#\' title=\'Updates Available\'></a>");\n        $("#toolbar .buttons").append($button);\n        $button.click(function () {\n            showUpdateDialog(newRelease);\n        });\n    }\n        \n    /**\n     * Show Update Dialog\n     * @return {Dialog}\n     */\n    function showUpdateDialog(release) {\n        var context = { Strings: Strings, release: release };\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(updateDialogTemplate, context), true, function ($dlg) {});\n        var $dlg = dialog.getElement();\n        var $download = $dlg.find("button[data-button-id=\'ok\']");\n        $download.click(function () {\n            appshell.app.openURLInDefaultBrowser(global.app.config.download_url);\n        });\n        return dialog;\n    }\n\n    /**\n     * Check for updates\n     * @return {$.Promise}\n     */\n    function checkForUpdates() {\n        var result = new $.Deferred(),\n            params = {\n                p: navigator.appVersion,\n                v: app.metadata.version\n            };\n        $.ajax(app.config.update_info_url, { data: params })\n            .done(function (release) {\n                if (app.metadata.version !== release.version) {\n                    _setUpdateAvailableIcon(release);\n                    result.resolve(release);\n                } else {\n                    result.reject("You\'re up to date!");\n                }\n            })\n            .fail(function (err) {\n                result.reject("Failed to check for updates - Error " + err.status);\n            });\n        return result.promise();\n    }\n    \n\n    /**\n     * Check extensions updates\n     */\n    function checkExtensionUpdates() {\n        ExtensionManager.downloadRegistry().done(function () {\n            var updateAvailable = false;\n            _.each(ExtensionManager.extensions, function (ext) {\n                if (ext.installInfo && ext.installInfo.updateAvailable) {\n                    updateAvailable = true;\n                }\n            });\n            if (updateAvailable) {\n                $("#toolbar-extension-manager").addClass("selected");\n            } else {\n                $("#toolbar-extension-manager").removeClass("selected");\n            }\n        });\n    }\n\n    exports.checkForUpdates       = checkForUpdates;\n    exports.showUpdateDialog      = showUpdateDialog;\n    exports.checkExtensionUpdates = checkExtensionUpdates;\n\n});\n//# sourceURL=/utils/CheckUpdates.js'),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define */\n\ndefine(\'utils/Analytics\',[\'require\',\'exports\',\'module\'],function (require, exports, module) {\n    "use strict";\n\n    var CID_KEY = "__staruml_client_uuid__";\n\n    /**\n     * Generate UUID (version 4)\n     */\n    function _generateUUID(){\n        var d = new Date().getTime();\n        var uuid = \'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx\'.replace(/[xy]/g, function(c) {\n            var r = (d + Math.random()*16)%16 | 0;\n            d = Math.floor(d/16);\n            return (c==\'x\' ? r : (r&0x7|0x8)).toString(16);\n        });\n        return uuid;\n    };\n\n    /**\n     * Return Client ID (Random UUID version 4)\n     */\n    function getCID() {\n        var uuid = localStorage.getItem(CID_KEY);\n        if (!uuid) {\n            uuid = _generateUUID();\n            localStorage.setItem(CID_KEY, uuid);\n        }\n        return uuid;\n    }\n\n    function _event(category, action, label, value) {\n        var params = {\n            v   : 1,\n            tid : "UA-50972178-1",\n            cid : getCID(),\n            t   : "event",\n            ec  : category,\n            ea  : action\n        };\n\n        if (label) {\n            params.el = label;\n        }\n        if (value) {\n            params.ev = value;\n        }\n\n        var r = $.ajax({\n            type : "POST",\n            url  : "http://www.google-analytics.com/collect",\n            data : params\n        });\n    }\n\n    function sendExecution() {\n        _event("execution", app.platform, app.metadata.version, 1);\n    }\n\n    function sendCommand(commandId) {\n        _event("command", commandId);\n    }\n\n    function sendError(op, cmd, stack, message, url, lineNumber) {\n        var params = {\n            cid: getCID(),\n            platform: app.platform,\n            version: app.metadata.version,\n            op: op,\n            cmd: cmd,\n            stack: stack,\n            message: message,\n            url: url,\n            lineNumber: lineNumber\n        };\n        $.ajax({\n            type: "POST",\n            url: "http://staruml.io/_dev/error_log",\n            data: params\n        });\n    }\n\n    exports.getCID        = getCID;\n    exports.sendExecution = sendExecution;\n    exports.sendCommand   = sendCommand;\n    exports.sendError     = sendError;\n\n});\n\n//# sourceURL=/utils/Analytics.js'),eval('define(\'text!htmlContent/validation-result-panel.html\',[],function () { return \'<div id="validation-results" class="validation-result-panel bottom-panel vert-resizable top-resizer no-focus">\\n    <div class="toolbar">\\n        <div class="title">Validation Errors</div>\\n        <a href="#" class="close"></a>\\n    </div>\\n    <div class="panel-content resizable-content">\\n        <div class="listview"></div>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/validation-result-panel.html'),eval("define('text!htmlContent/validation-error-item.html',[],function () { return '<div class=\\'validation-error-item\\'>\\n    <span style=\"margin-bottom: 2px; margin-right: 3px;\" class=\"k-sprite validation-error\"></span>\\n    <span>#:message#</span>\\n    <span>(#:ruleId#)</span>\\n    —\\n    <span class=\\'k-sprite #=icon#\\'></span>\\n    <span class=\\'name\\'>#:name#</span>\\n    <span class=\\'type\\'>[#:type#]</span>\\n</div>\\n';});\n\n//# sourceURL=/text!htmlContent/validation-error-item.html"),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, browser: true */\n/*global $, _, define, kendo, app, C2S */\n\ndefine(\'validation/Validator\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Repository\',\'engine/ProjectManager\',\'core/PreferenceManager\',\'utils/PanelManager\',\'file/FileUtils\',\'filesystem/FileSystem\',\'metadata-json/MetadataJson\',\'explorer/ModelExplorerView\',\'ui/StatusBar\',\'text!htmlContent/validation-result-panel.html\',\'text!htmlContent/validation-error-item.html\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit           = require("utils/AppInit"),\n        Repository        = require("core/Repository"),\n        ProjectManager    = require("engine/ProjectManager"),\n        PreferenceManager = require("core/PreferenceManager"),\n        PanelManager      = require("utils/PanelManager"),\n        FileUtils         = require("file/FileUtils"),\n        FileSystem        = require("filesystem/FileSystem"),\n        MetadataJson      = require("metadata-json/MetadataJson"),\n        ModelExplorerView = require("explorer/ModelExplorerView"),\n        StatusBar         = require("ui/StatusBar"),\n        validationResultPanelTemplate = require("text!htmlContent/validation-result-panel.html"),\n        validationErrorItemTemplate   = require("text!htmlContent/validation-error-item.html");\n\n    var $validationResultPanel,\n        $title,\n        $close,\n        $listView,\n        $validationStatus;\n\n    var validationResultPanel;\n\n    /**\n     * DataSource for ListView\n     * @type {kendo.data.DataSource}\n     */\n    var dataSource = new kendo.data.DataSource();\n\n    function validate(filename) {\n        if (PreferenceManager.get("validation.validateModel")) {\n            return MetadataJson.validate(filename)\n                .done(function (result) {\n                    _setValidationErrors(result);\n                })\n                .fail(function (err) {\n                    console.error(err);\n                });\n        }\n    }\n\n    function _clearValidationErrors() {\n        dataSource.data([]);\n    }\n\n    function _addValidationError(item) {\n        var element = Repository.get(item.id);\n        if (element) {\n            dataSource.add({\n                id: item.id,\n                ruleId: item.ruleId,\n                message: item.message,\n                icon: element.getNodeIcon(),\n                name: element.name,\n                type: element.getClassName()\n            });\n        }\n    }\n\n    function _setValidationErrors(errors) {\n        if (Array.isArray(errors) && errors.length > 0) {\n            StatusBar.setValidity(false);\n            _clearValidationErrors();\n            _.each(errors, function (item) {\n                _addValidationError(item);\n            });\n            $title.html("Validation Results — " + errors.length + " errors.");\n        } else {\n            StatusBar.setValidity(true);\n            _clearValidationErrors();\n            $title.html("Validation Results — No errors.");\n        }\n    }\n\n    // Setup Validation Result Panel\n    AppInit.htmlReady(function () {\n        $validationResultPanel = $(validationResultPanelTemplate);\n        $title = $validationResultPanel.find(".title");\n        $close = $validationResultPanel.find(".close");\n        $close.click(function () {\n            validationResultPanel.hide();\n        });\n        $listView = $validationResultPanel.find(".listview");\n        validationResultPanel = PanelManager.createBottomPanel("?", $validationResultPanel, 29);\n        $listView.kendoListView({\n            dataSource: dataSource,\n            template: validationErrorItemTemplate,\n            selectable: true,\n            change: function () {\n                var data = dataSource.view(),\n                    item = data[this.select().index()],\n                    element = Repository.get(item.id);\n                if (element) {\n                    ModelExplorerView.select(element, true);\n                }\n            }\n        });\n\n        // Validation in StatusBar\n        $validationStatus = $(".statusbar .validation");\n        $validationStatus.click(function () {\n            if (validationResultPanel.isVisible()) {\n                validationResultPanel.hide();\n            } else {\n                validationResultPanel.show();\n            }\n        });\n\n        $(ProjectManager).on(\'projectLoaded\', function (event, filename, project) {\n            try {\n                if (filename) {\n                    validate(filename);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(ProjectManager).on(\'projectSaved\', function (event, filename, project) {\n            try {\n                if (filename) {\n                    validate(filename);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(ProjectManager).on(\'projectClosed\', function (event) {\n            try {\n                _setValidationErrors();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n    });\n\n    exports.validate = validate;\n\n});\n\n//# sourceURL=/validation/Validator.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, browser: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define*/\n\ndefine(\'core/Core-meta\',[\'require\',\'exports\',\'module\',\'core/MetaModelManager\'],function (require, exports, module) {\n    "use strict";\n\n    var MetaModelManager  = require("core/MetaModelManager");\n\n    var metaModel = {\n        "TagKind": {\n            "kind": "enum",\n            "literals": [ "string", "reference", "boolean", "number", "hidden" ]\n        },\n        "Element": {\n            "kind": "class",\n            "attributes": [\n                { "name": "_id",     "kind": "prim", "type": "String" },\n                { "name": "_parent", "kind": "ref",  "type": "Element" }\n            ]\n        },\n        "Model": {\n            "kind": "class",\n            "super": "Element",\n            "attributes": [\n                { "name": "name",          "kind": "prim", "type": "String", "visible": true },\n                { "name": "ownedElements", "kind": "objs", "type": "Element" }\n            ]\n        },\n        "Tag": {\n            "kind": "class",\n            "super": "Model",\n            "attributes": [\n                { "name": "kind",      "kind": "enum", "type": "TagKind", "visible": true },\n                { "name": "value",     "kind": "prim", "type": "String",  "visible": true, "multiline": true },\n                { "name": "reference", "kind": "ref",  "type": "Model",   "visible": true },\n                { "name": "checked",   "kind": "prim", "type": "Boolean", "visible": true },\n                { "name": "number",    "kind": "prim", "type": "Integer", "visible": true }\n            ]\n        },\n        "Hyperlink": {\n            "kind": "class",\n            "super": "Model",\n            "attributes": [\n                { "name": "reference", "kind": "ref",  "type": "Model",   "visible": true },\n                { "name": "url",       "kind": "prim", "type": "String",  "visible": true }\n            ],\n            "view": "HyperlinkView"\n        },\n        "ExtensibleModel": {\n            "kind": "class",\n            "super": "Model",\n            "attributes": [\n                { "name": "documentation", "kind": "prim", "type": "String" },\n                { "name": "tags",          "kind": "objs", "type": "Tag" }\n            ]\n        },\n        "Relationship": {\n            "kind": "class",\n            "super": "ExtensibleModel"\n        },\n        "DirectedRelationship": {\n            "kind": "class",\n            "super": "Relationship",\n            "attributes": [\n                { "name": "source", "kind": "ref", "type": "Model", "visible": true, "readOnly": true },\n                { "name": "target", "kind": "ref", "type": "Model", "visible": true, "readOnly": true }\n            ]\n        },\n        "RelationshipEnd": {\n            "kind": "class",\n            "super": "ExtensibleModel",\n            "attributes": [\n                { "name": "reference", "kind": "ref", "type": "Model", "visible": true, "readOnly": true }\n            ]\n        },\n        "UndirectedRelationship": {\n            "kind": "class",\n            "super": "Relationship",\n            "attributes": [\n                { "name": "end1", "kind": "obj", "type": "RelationshipEnd", "expand": true },\n                { "name": "end2", "kind": "obj", "type": "RelationshipEnd", "expand": true }\n            ]\n        },\n        "View": {\n            "kind": "class",\n            "super": "Element",\n            "attributes": [\n                { "name": "model",               "kind": "ref",    "type": "Element" },\n                { "name": "subViews",            "kind": "objs",   "type": "View" },\n                { "name": "containerView",       "kind": "ref",    "type": "View" },\n                { "name": "containedViews",      "kind": "refs",   "type": "View" },\n                { "name": "visible",             "kind": "prim",   "type": "Boolean", "default": true },\n                { "name": "enabled",             "kind": "prim",   "type": "Boolean", "default": true },\n                { "name": "selected",            "kind": "prim",   "type": "Boolean", "transient": true },\n                { "name": "lineColor",           "kind": "prim",   "type": "String" },\n                { "name": "fillColor",           "kind": "prim",   "type": "String" },\n                { "name": "fontColor",           "kind": "prim",   "type": "String" },\n                { "name": "font",                "kind": "custom", "type": "Font" },\n                { "name": "parentStyle",         "kind": "prim",   "type": "Boolean", "default": false, "transient": true },\n                { "name": "showShadow",          "kind": "prim",   "type": "Boolean", "default": true },\n                { "name": "containerChangeable", "kind": "prim",   "type": "Boolean", "default": false, "transient": false },\n                { "name": "containerExtending",  "kind": "prim",   "type": "Boolean", "default": false, "transient": false }\n            ]\n        },\n        "NodeView": {\n            "kind": "class",\n            "super": "View",\n            "attributes": [\n                { "name": "left",       "kind": "prim", "type": "Integer", "default": 0 },\n                { "name": "top",        "kind": "prim", "type": "Integer", "default": 0 },\n                { "name": "width",      "kind": "prim", "type": "Integer", "default": 0 },\n                { "name": "height",     "kind": "prim", "type": "Integer", "default": 0 },\n                { "name": "autoResize", "kind": "prim", "type": "Boolean", "default": false }\n            ]\n        },\n        "EdgeView": {\n            "kind": "class",\n            "super": "View",\n            "attributes": [\n                { "name": "head",      "kind": "ref",    "type": "View" },\n                { "name": "tail",      "kind": "ref",    "type": "View" },\n                { "name": "lineStyle", "kind": "prim",   "type": "Integer" },\n                { "name": "points",    "kind": "custom", "type": "Points" }\n            ]\n        },\n        "LabelView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "underline",           "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "text",                "kind": "prim", "type": "String" },\n                { "name": "horizontalAlignment", "kind": "prim", "type": "Integer", "default": 2 },\n                { "name": "verticalAlignment",   "kind": "prim", "type": "Integer", "default": 5 },\n                { "name": "direction",           "kind": "prim", "type": "Integer", "default": 0, "transient": true },\n                { "name": "wordWrap",            "kind": "prim", "type": "Boolean", "default": false }\n            ]\n        },\n        "ParasiticView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "alpha",    "kind": "prim", "type": "Real" },\n                { "name": "distance", "kind": "prim", "type": "Real" }\n            ]\n        },\n        "NodeParasiticView": {\n            "kind": "class",\n            "super": "ParasiticView",\n            "attributes": [\n                { "name": "hostNode", "kind": "ref", "type": "NodeView" }\n            ]\n        },\n        "EdgeParasiticView": {\n            "kind": "class",\n            "super": "ParasiticView",\n            "attributes": [\n                { "name": "hostEdge",     "kind": "ref",  "type": "EdgeView" },\n                { "name": "edgePosition", "kind": "prim", "type": "Integer", "default": 0 }\n            ]\n        },\n        "NodeLabelView": {\n            "kind": "class",\n            "super": "NodeParasiticView",\n            "attributes": [\n                { "name": "underline",           "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "text",                "kind": "prim", "type": "String" },\n                { "name": "horizontalAlignment", "kind": "prim", "type": "Integer", "default": 2 },\n                { "name": "verticalAlignment",   "kind": "prim", "type": "Integer", "default": 5 },\n                { "name": "wordWrap",            "kind": "prim", "type": "Boolean", "default": false }\n            ]\n        },\n        "EdgeLabelView": {\n            "kind": "class",\n            "super": "EdgeParasiticView",\n            "attributes": [\n                { "name": "underline",           "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "text",                "kind": "prim", "type": "String" },\n                { "name": "horizontalAlignment", "kind": "prim", "type": "Integer", "default": 2 },\n                { "name": "verticalAlignment",   "kind": "prim", "type": "Integer", "default": 5 },\n                { "name": "wordWrap",            "kind": "prim", "type": "Boolean", "default": false }\n            ]\n        },\n        "NodeNodeView": {\n            "kind": "class",\n            "super": "NodeParasiticView"\n        },\n        "EdgeNodeView": {\n            "kind": "class",\n            "super": "EdgeParasiticView"\n        },\n        "Diagram": {\n            "kind": "class",\n            "super": "ExtensibleModel",\n            "attributes": [\n                { "name": "visible",        "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "defaultDiagram", "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "ownedViews",     "kind": "objs", "type": "View" }\n            ]\n        },\n        "Project": {\n            "kind": "class",\n            "super": "ExtensibleModel",\n            "attributes": [\n                { "name": "author",         "kind": "prim", "type": "String",  "visible": true },\n                { "name": "company",        "kind": "prim", "type": "String",  "visible": true },\n                { "name": "copyright",      "kind": "prim", "type": "String",  "visible": true },\n                { "name": "version",        "kind": "prim", "type": "String",  "visible": true }\n            ]\n        }\n    };\n\n    // Register Meta-model\n    MetaModelManager.register(metaModel);\n\n});\n\n//# sourceURL=/core/Core-meta.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, browser: true */\n/*global define, $, _, */\n\ndefine(\'core/Validator\',[\'require\',\'exports\',\'module\',\'core/Global\',\'core/Repository\'],function (require, exports, module) {\n    "use strict";\n\n    var _global    = require("core/Global").global,\n        Repository = require("core/Repository");\n\n    /**\n     * Add validation rules\n     *\n     * @param {Array.<Object>} rules\n     */\n    function addRules(rules) {\n        _.each(rules, function (rule) {\n            _global.rules[rule.id] = rule;\n        });\n    }\n\n    /**\n     * Validate and return validation errors\n     *\n     * @return {Array.<Object>}\n     */\n    function validate() {\n        var failed = [];\n\n        _.each(_global.rules, function (rule) {\n\n            // Get instances of types in rule.appliesTo\n            var targets = Repository.getInstancesOf(rule.appliesTo);\n\n            // Rejects instances of types in rule.exceptions\n            if (rule.exceptions && rule.exceptions.length > 0) {\n                targets = _.reject(targets, function (target) {\n                    return _.some(rule.exceptions, function (exception) {\n                        return target instanceof _global.type[exception];\n                    });\n                });\n            }\n\n            // Check constraints\n            _.each(targets, function (target) {\n                try {\n                    if (!rule.constraint(target)) {\n                        var item = {\n                            id: target._id,\n                            ruleId: rule.id,\n                            message: rule.message\n                        };\n                        failed.push(item);\n                    }\n                } catch (err) {\n                    console.log("[Validator] Failed to apply the rule (" + rule.id + ") on ", target);\n                    console.error(err);\n                }\n            });\n        });\n\n        return failed;\n    }\n\n    exports.addRules = addRules;\n    exports.validate = validate;\n\n});\n\n//# sourceURL=/core/Validator.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, browser: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, type*/\n\ndefine(\'core/Core-rules\',[\'require\',\'exports\',\'module\',\'core/Validator\'],function (require, exports, module) {\n    "use strict";\n\n    var Validator = require("core/Validator");\n\n    var rules = [\n        {\n            id: "CORE001",\n            message: "Name expected.",\n            appliesTo: [ "Project", "Tag" ],\n            constraint: function (elem) {\n                return (elem.name && elem.name.length > 0);\n            }\n        },\n        {\n            id: "CORE002",\n            message: "[Critical] DirectedRelationship should have source and target.",\n            appliesTo: [ "DirectedRelationship" ],\n            constraint: function (elem) {\n                return (elem.source instanceof type.Model &&\n                        elem.target instanceof type.Model);\n            }\n        },\n        {\n            id: "CORE003",\n            message: "[Critical] UndirectedRelationship should have two ends.",\n            appliesTo: [ "UndirectedRelationship" ],\n            constraint: function (elem) {\n                return (elem.end1 instanceof type.RelationshipEnd &&\n                        elem.end1.reference instanceof type.Model &&\n                        elem.end2 instanceof type.RelationshipEnd &&\n                        elem.end2.reference instanceof type.Model);\n            }\n        }\n    ];\n\n    Validator.addRules(rules);\n\n});\n\n//# sourceURL=/core/Core-rules.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell */\ndefine('uml/UMLCommands',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n\n    // File\n    exports.FILE_NEW_FROM_TEMPLATE_UML_MINIMAL       = 'file.newFromTemplate.UMLMinimal';\n    exports.FILE_NEW_FROM_TEMPLATE_UML_CONVENTIONAL  = 'file.newFromTemplate.UMLConventional';\n    exports.FILE_NEW_FROM_TEMPLATE_4P1VIEWMODEL      = 'file.newFromTemplate.4p1Viewmodel';\n    exports.FILE_NEW_FROM_TEMPLATE_RATIONAL          = 'file.newFromTemplate.Rational';\n\n    // Format\n    exports.FORMAT_STEREOTYPE                        = 'format.stereotype';\n    exports.FORMAT_STEREOTYPE_NONE                   = 'format.stereotype.none';\n    exports.FORMAT_STEREOTYPE_LABEL                  = 'format.stereotype.label';\n    exports.FORMAT_STEREOTYPE_DECORATION             = 'format.stereotype.decoration';\n    exports.FORMAT_STEREOTYPE_DECORATION_LABEL       = 'format.stereotype.decorationLabel';\n    exports.FORMAT_STEREOTYPE_ICON                   = 'format.stereotype.icon';\n    exports.FORMAT_STEREOTYPE_ICON_LABEL             = 'format.stereotype.iconLabel';\n    exports.FORMAT_WORD_WRAP                         = 'format.wordWrap';\n    exports.FORMAT_SHOW_VISIBILITY                   = 'format.showVisibility';\n    exports.FORMAT_SHOW_NAMESPACE                    = 'format.showNamespace';\n    exports.FORMAT_SHOW_PROPERTY                     = 'format.showProperty';\n    exports.FORMAT_SHOW_TYPE                         = 'format.showType';\n    exports.FORMAT_SHOW_MULTIPLICITY                 = 'format.showMultiplicity';\n    exports.FORMAT_SHOW_OPERATION_SIGNATURE          = 'format.showOperationSignature';\n    exports.FORMAT_SUPPRESS_ATTRIBUTES               = 'format.suppressAttributes';\n    exports.FORMAT_SUPPRESS_OPERATIONS               = 'format.suppressOperations';\n    exports.FORMAT_SUPPRESS_RECEPTIONS               = 'format.suppressReceptions';\n    exports.FORMAT_SUPPRESS_LITERALS                 = 'format.suppressLiterals';\n\n    // Packages\n    exports.MODEL_ADD_MODEL                          = 'model.add.model';\n    exports.MODEL_ADD_SUBSYSTEM                      = 'model.add.subsystem';\n    exports.MODEL_ADD_PACKAGE                        = 'model.add.package';\n    exports.MODEL_ADD_PROFILE                        = 'model.add.profile';\n    // Classes\n    exports.MODEL_ADD_CLASS                          = 'model.add.class';\n    exports.MODEL_ADD_INTERFACE                      = 'model.add.interface';\n    exports.MODEL_ADD_SIGNAL                         = 'model.add.signal';\n    exports.MODEL_ADD_DATATYPE                       = 'model.add.datatype';\n    exports.MODEL_ADD_PRIMITIVETYPE                  = 'model.add.primitiveType';\n    exports.MODEL_ADD_ENUMERATION                    = 'model.add.enumeration';\n    exports.MODEL_ADD_ARTIFACT                       = 'model.add.artifact';\n    exports.MODEL_ADD_COMPONENT                      = 'model.add.component';\n    exports.MODEL_ADD_NODE                           = 'model.add.node';\n    exports.MODEL_ADD_USECASE                        = 'model.add.usecase';\n    exports.MODEL_ADD_ACTOR                          = 'model.add.actor';\n    exports.MODEL_ADD_STEREOTYPE                     = 'model.add.stereotype';\n    // Instances\n    exports.MODEL_ADD_OBJECT                         = 'model.add.object';\n    exports.MODEL_ADD_ARTIFACTINSTANCE               = 'model.add.artifactInstance';\n    exports.MODEL_ADD_COMPONENTINSTANCE              = 'model.add.componentInstance';\n    exports.MODEL_ADD_NODEINSTANCE                   = 'model.add.nodeInstance';\n    // Behaviors\n    exports.MODEL_ADD_COLLABORATION                  = 'model.add.collaboration';\n    exports.MODEL_ADD_STATEMACHINE                   = 'model.add.stateMachine';\n    exports.MODEL_ADD_ACTIVITY                       = 'model.add.activity';\n    exports.MODEL_ADD_INTERACTION                    = 'model.add.interaction';\n    exports.MODEL_ADD_OPAQUEBEHAVIOR                 = 'model.add.opaqueBehavior';\n    // Features\n    exports.MODEL_ADD_ENUMERATIONLITERAL             = 'model.add.enumerationLiteral';\n    exports.MODEL_ADD_ATTRIBUTE                      = 'model.add.attribute';\n    exports.MODEL_ADD_OPERATION                      = 'model.add.operation';\n    exports.MODEL_ADD_RECEPTION                      = 'model.add.reception';\n    exports.MODEL_ADD_PORT                           = 'model.add.port';\n    exports.MODEL_ADD_PARAMETER                      = 'model.add.parameter';\n    exports.MODEL_ADD_TEMPLATEPARAMETER              = 'model.add.templateParameter';\n    exports.MODEL_ADD_EXTENSIONPOINT                 = 'model.add.extensionPoint';\n    exports.MODEL_ADD_SLOT                           = 'model.add.slot';\n    // States\n    exports.MODEL_ADD_STATE                          = 'model.add.state';\n    exports.MODEL_ADD_REGION                         = 'model.add.region';\n    exports.MODEL_ADD_ENTRY_ACTIVITY                 = 'model.add.entryActivity';\n    exports.MODEL_ADD_DO_ACTIVITY                    = 'model.add.doActivity';\n    exports.MODEL_ADD_EXIT_ACTIVITY                  = 'model.add.exitActivity';\n    exports.MODEL_ADD_TRIGGER                        = 'model.add.trigger';\n    exports.MODEL_ADD_EFFECT                         = 'model.add.effect';\n    // Actions\n    exports.MODEL_ADD_ACTION                         = 'model.add.action';\n    // Common\n    exports.MODEL_ADD_CONSTRAINT                     = 'model.add.constraint';\n    // Diagrams\n    exports.MODEL_ADD_DIAGRAM_CLASS                  = 'model.addDiagram.class';\n    exports.MODEL_ADD_DIAGRAM_PACKAGE                = 'model.addDiagram.package';\n    exports.MODEL_ADD_DIAGRAM_OBJECT                 = 'model.addDiagram.object';\n    exports.MODEL_ADD_DIAGRAM_COMPOSITESTRUCTURE     = 'model.addDiagram.compositeStructure';\n    exports.MODEL_ADD_DIAGRAM_COMPONENT              = 'model.addDiagram.component';\n    exports.MODEL_ADD_DIAGRAM_DEPLOYMENT             = 'model.addDiagram.deployment';\n    exports.MODEL_ADD_DIAGRAM_USECASE                = 'model.addDiagram.usecase';\n    exports.MODEL_ADD_DIAGRAM_SEQUENCE               = 'model.addDiagram.sequence';\n    exports.MODEL_ADD_DIAGRAM_COMMUNICATION          = 'model.addDiagram.communication';\n    exports.MODEL_ADD_DIAGRAM_STATECHART             = 'model.addDiagram.statechart';\n    exports.MODEL_ADD_DIAGRAM_ACTIVITY               = 'model.addDiagram.activity';\n    exports.MODEL_ADD_DIAGRAM_PROFILE                = 'model.addDiagram.profile';\n    // Profiles\n    exports.MODEL_APPLY_PROFILE                      = 'model.applyProfile';\n    exports.MODEL_APPLY_PROFILE_UML_STANDARD         = 'model.applyProfile.UMLStandard';\n\n\n});\n\n//# sourceURL=/uml/UMLCommands.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell */\ndefine(\'menu/DefaultMenus\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'utils/Initializer\',\'core/Global\',\'utils/Helper\',\'command/Commands\',\'command/CommandManager\',\'menu/MenuManager\',\'menu/ContextMenuManager\',\'uml/UMLCommands\',\'strings\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit            = require("utils/AppInit"),\n        Initializer        = require("utils/Initializer"),\n        global             = require("core/Global").global,\n        Helper             = require("utils/Helper"),\n        Commands           = require("command/Commands"),\n        CommandManager     = require("command/CommandManager"),\n        MenuManager        = require("menu/MenuManager"),\n        ContextMenuManager = require("menu/ContextMenuManager"),\n        UMLCommands        = require("uml/UMLCommands"),\n        Strings            = require("strings");\n\n    /**\n     * Default Context Menus\n     */\n    var ContextMenus = {\n        DIAGRAM  : "diagram-context-menu",\n        EXPLORER : "explorer-context-menu"\n    };\n\n\n    function setupMenus() {\n        var menu,\n            menuItem;\n\n        // File Menu\n        menu = MenuManager.addMenu(Commands.FILE);\n        menu.addMenuItem(Commands.FILE_NEW);\n        menu.addMenuItem(Commands.FILE_NEW_FROM_TEMPLATE);\n        menu.addMenuItem(Commands.FILE_OPEN);\n        // menu.addMenuItem(Commands.FILE_OPEN_RECENT);\n        menu.addMenuItem(Commands.FILE_SAVE);\n        menu.addMenuItem(Commands.FILE_SAVE_AS);\n        menu.addMenuItem(Commands.FILE_CLOSE);\n        menu.addMenuDivider();\n        menuItem = menu.addMenuItem(Commands.FILE_IMPORT);\n        menuItem.addMenuItem(Commands.FILE_IMPORT_FRAGMENT);\n        menuItem = menu.addMenuItem(Commands.FILE_EXPORT);\n        menuItem.addMenuItem(Commands.FILE_EXPORT_FRAGMENT);\n        if (global.app.platform !== "mac") {\n            menu.addMenuDivider();\n            menu.addMenuItem(Commands.FILE_PREFERENCES);\n        }\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.FILE_PRINT_TO_PDF);\n        if (global.app.platform !== "mac") {\n            menu.addMenuDivider();\n            menu.addMenuItem(Commands.FILE_QUIT);\n        }\n\n        // Edit Menu\n        menu = MenuManager.addMenu(Commands.EDIT);\n        menu.addMenuItem(Commands.EDIT_UNDO);\n        menu.addMenuItem(Commands.EDIT_REDO);\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.EDIT_CUT);\n        menu.addMenuItem(Commands.EDIT_COPY);\n        menu.addMenuItem(Commands.EDIT_PASTE);\n        menu.addMenuItem(Commands.EDIT_DELETE);\n        menu.addMenuItem(Commands.EDIT_DELETE_FROM_MODEL);\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.EDIT_MOVE_UP);\n        menu.addMenuItem(Commands.EDIT_MOVE_DOWN);\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.EDIT_SELECT_ALL);\n        menu.addMenuItem(Commands.EDIT_SELECT_IN_EXPLORER);\n        menu.addMenuItem(Commands.EDIT_SELECT_IN_DIAGRAM);\n\n        // Format Menu\n        menu = MenuManager.addMenu(Commands.FORMAT);\n        menu.addMenuItem(Commands.FORMAT_FONT);\n        menu.addMenuItem(Commands.FORMAT_FILL_COLOR);\n        menu.addMenuItem(Commands.FORMAT_LINE_COLOR);\n        menu.addMenuDivider();\n        menuItem = menu.addMenuItem(Commands.FORMAT_LINE_STYLE);\n        menuItem.addMenuItem(Commands.FORMAT_LINE_STYLE_RECTILINEAR);\n        menuItem.addMenuItem(Commands.FORMAT_LINE_STYLE_OBLIQUE);\n        menuItem.addMenuItem(Commands.FORMAT_LINE_STYLE_ROUNDRECT);\n        menuItem.addMenuItem(Commands.FORMAT_LINE_STYLE_CURVE);\n        menu.addMenuItem(Commands.FORMAT_AUTO_RESIZE);\n        menu.addMenuItem(Commands.FORMAT_SHOW_SHADOW);\n\n        // Model Menu\n        menu = MenuManager.addMenu(Commands.MODEL);\n        menu.addMenuItem(Commands.MODEL_ADD_DIAGRAM);\n        menu.addMenuItem(Commands.MODEL_ADD);\n\n        // Tools Menu\n        menu = MenuManager.addMenu(Commands.TOOLS);\n        menu.addMenuItem(Commands.TOOLS_EXTENSION_MANAGER);\n\n        // View Menu\n        menu = MenuManager.addMenu(Commands.VIEW);\n        menu.addMenuItem(Commands.VIEW_CLOSE_DIAGRAM);\n        menu.addMenuItem(Commands.VIEW_CLOSE_OTHER_DIAGRAMS);\n        menu.addMenuItem(Commands.VIEW_CLOSE_ALL_DIAGRAMS);\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.VIEW_NEXT_DIAGRAM);\n        menu.addMenuItem(Commands.VIEW_PREVIOUS_DIAGRAM);\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.VIEW_ZOOM_IN);\n        menu.addMenuItem(Commands.VIEW_ZOOM_OUT);\n        menu.addMenuItem(Commands.VIEW_ACTUAL_SIZE);\n        menu.addMenuItem(Commands.VIEW_FIT_TO_WINDOW);\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.VIEW_SHOW_GRID);\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.VIEW_HIDE_SIDEBAR);\n        menu.addMenuItem(Commands.VIEW_HIDE_NAVIGATOR);\n        menu.addMenuItem(Commands.VIEW_HIDE_TOOLBAR);\n        menu.addMenuItem(Commands.VIEW_HIDE_STATUSBAR);\n        menu.addMenuItem(Commands.VIEW_HIDE_TOOLBOX);\n        menu.addMenuItem(Commands.VIEW_HIDE_EDITORS);\n\n        // Help Menu\n        menu = MenuManager.addMenu(Commands.HELP, MenuManager.LAST);\n        if (global.app.platform !== "mac") {\n            menu.addMenuItem(Commands.HELP_ABOUT);\n        }\n        menu.addMenuItem(Commands.HELP_CHECK_FOR_UPDATES);\n        menu.addMenuItem(Commands.HELP_ENTER_LICENSE);\n        menu.addMenuDivider();\n        menu.addMenuItem(Commands.HELP_DOCUMENTATION);\n        menu.addMenuItem(Commands.HELP_FORUM);\n        // menu.addMenuItem(Commands.HELP_RELEASE_NOTE);\n        menu.addMenuItem(Commands.HELP_REQUEST_FEATURE);\n    }\n\n    /**\n     * ContextMenu for Diagram\n     */\n    function setupContextMenuForDiagram() {\n        var contextMenu,\n            menu,\n            menuItem;\n\n        contextMenu = ContextMenuManager.addContextMenu(ContextMenus.DIAGRAM, "#diagram-canvas");\n\n        // Add Menu\n        contextMenu.addMenuItem(Commands.MODEL_ADD, null, null, ContextMenuManager.FIRST);\n        contextMenu.addMenuDivider(ContextMenuManager.AFTER, Commands.MODEL_ADD);\n\n        // Format Menu\n        menu = contextMenu.addMenuItem(Commands.FORMAT);\n        menu.addMenuItem(Commands.FORMAT_FONT);\n        menu.addMenuItem(Commands.FORMAT_FILL_COLOR);\n        menu.addMenuItem(Commands.FORMAT_LINE_COLOR);\n        menu.addMenuDivider();\n        menuItem = menu.addMenuItem(Commands.FORMAT_LINE_STYLE);\n        menuItem.addMenuItem(Commands.FORMAT_LINE_STYLE_RECTILINEAR);\n        menuItem.addMenuItem(Commands.FORMAT_LINE_STYLE_OBLIQUE);\n        menuItem.addMenuItem(Commands.FORMAT_LINE_STYLE_ROUNDRECT);\n        menuItem.addMenuItem(Commands.FORMAT_LINE_STYLE_CURVE);\n        menu.addMenuItem(Commands.FORMAT_AUTO_RESIZE);\n        menu.addMenuItem(Commands.FORMAT_SHOW_SHADOW);\n\n        // Edit Menu\n        contextMenu.addMenuDivider();\n        contextMenu.addMenuItem(Commands.EDIT_CUT);\n        contextMenu.addMenuItem(Commands.EDIT_COPY);\n        contextMenu.addMenuItem(Commands.EDIT_PASTE);\n        contextMenu.addMenuItem(Commands.EDIT_DELETE);\n        contextMenu.addMenuItem(Commands.EDIT_DELETE_FROM_MODEL);\n        contextMenu.addMenuDivider();\n        contextMenu.addMenuItem(Commands.EDIT_SELECT_ALL);\n        contextMenu.addMenuItem(Commands.EDIT_SELECT_IN_EXPLORER);\n\n    }\n\n    /**\n     * ContextMenu for Explorer\n     */\n    function setupContextMenuForExplorer() {\n        var contextMenu,\n            menu;\n\n        contextMenu = ContextMenuManager.addContextMenu(ContextMenus.EXPLORER, "#model-explorer-holder .treeview");\n\n        // Add Diagram and Add Menu\n        contextMenu.addMenuItem(Commands.MODEL_ADD_DIAGRAM, null, null, ContextMenuManager.FIRST);\n        contextMenu.addMenuItem(Commands.MODEL_ADD, null, null, ContextMenuManager.AFTER, Commands.MODEL_ADD_DIAGRAM);\n        contextMenu.addMenuDivider(ContextMenuManager.AFTER, Commands.MODEL_ADD);\n\n        // Edit Menu\n        contextMenu.addMenuItem(Commands.EDIT_CUT);\n        contextMenu.addMenuItem(Commands.EDIT_COPY);\n        contextMenu.addMenuItem(Commands.EDIT_PASTE);\n        contextMenu.addMenuItem(Commands.EDIT_DELETE_FROM_MODEL);\n        contextMenu.addMenuDivider();\n        contextMenu.addMenuItem(Commands.EDIT_MOVE_UP);\n        contextMenu.addMenuItem(Commands.EDIT_MOVE_DOWN);\n        contextMenu.addMenuDivider();\n        contextMenu.addMenuItem(Commands.EDIT_SELECT_IN_DIAGRAM);\n    }\n\n    AppInit.htmlReady(function () {\n        setupMenus();\n        setupContextMenuForDiagram();\n        setupContextMenuForExplorer();\n    });\n\n    exports.contextMenus = ContextMenus;\n\n});\n\n//# sourceURL=/menu/DefaultMenus.js'),eval('define(\'text!htmlContent/element-list-picker-dialog.html\',[],function () { return \'<div class="element-list-picker dialog modal" data-title="{{title}}">\\n    <div class="dialog-body no-padding">\\n        <div class="listview-wrapper">\\n            <ul class="listview"></ul>\\n        </div>\\n    </div>\\n    <div class="dialog-footer">\\n        <div class="left" style="text-align: left;">\\n            <label class="k-checkbox left">\\n                <input class="unspecified" type="checkbox">\\n                <div class="k-checkbox__checkmark"></div>\\n                <span style="padding-left: 7px;">Do not specify</span>\\n            </label>\\n        </div>\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.OK}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/element-list-picker-dialog.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache */\n\n/**\n * Element List Picker Dialog\n */\ndefine(\'dialogs/ElementListPickerDialog\',[\'require\',\'exports\',\'module\',\'utils/Helper\',\'utils/ViewUtils\',\'core/Repository\',\'engine/Engine\',\'strings\',\'dialogs/Dialogs\',\'text!htmlContent/element-list-picker-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var Helper        = require("utils/Helper"),\n        ViewUtils     = require("utils/ViewUtils"),\n        Repository    = require("core/Repository"),\n        Engine        = require("engine/Engine"),\n        Strings       = require("strings"),\n        Dialogs       = require("dialogs/Dialogs");\n\n    var UI_ELEMENT_LIST_PICKER = "#ui-element-list-picker";\n\n    var dialogTemplate = require("text!htmlContent/element-list-picker-dialog.html");\n\n    /**\n     * DataSource for ListView\n     * @private\n     * @type {kendo.data.DataSource}\n     */\n    var dataSource = new kendo.data.DataSource();\n\n    var selectedElement = null;\n\n    /**\n     * Convert Core.Model to DataSource Item\n     * @private\n     * @type {kendo.data.DataSource}\n     */\n    function _toDataItem(elem) {\n        return {\n            id: elem._id,\n            icon: elem.getNodeIcon(),\n            text: elem.getNodeText(),\n            path: elem.getPathname()\n        };\n    }\n\n    function updateDataSource(elems) {\n        dataSource.data([]);\n        for (var i = 0, len = elems.length; i < len; i++) {\n            var item = elems[i];\n            dataSource.add(_toDataItem(item));\n        }\n    }\n\n    /**\n     * Show Element List Picker Dialog.\n     *\n     * @param {string} title\n     * @param {Array.<Core.Model>} elems\n     */\n    function showDialog(title, elems) {\n        var context = {\n            Strings: Strings,\n            title: title\n        };\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(dialogTemplate, context), true, function ($dlg) {\n            $dlg.data("returnValue", selectedElement);\n        });\n\n        var $dlg = dialog.getElement(),\n            $listview = $dlg.find(".listview"),\n            $unspecified = $dlg.find(".unspecified");\n\n        var $wrapper = $dlg.find(".listview-wrapper");\n        ViewUtils.addScrollerShadow($wrapper, null, true);\n\n        // Setup ListView\n        selectedElement = null;\n        updateDataSource(elems);\n        $listview.kendoListView({\n            dataSource: dataSource,\n            template: "<div style=\'padding: 0.2em;\'><span class=\'k-sprite #=icon#\' style=\'margin-right: 3px\'></span>#:text# (#=path#)</div>",\n            selectable: true,\n            change: function (e) {\n                var selected = this.select();\n                if (selected && selected.length > 0) {\n                    var dataItem = dataSource.getByUid(selected[0].dataset.uid);\n                    selectedElement = Repository.get(dataItem.id);\n                    $unspecified.attr("checked", false);\n                }\n            }\n        });\n        var listview = $listview.data("kendoListView");\n\n        // Setup Unspecified\n        $unspecified.attr("checked", true);\n        $unspecified.change(function () {\n            var checked = $unspecified.is(":checked");\n            if (checked) {\n                listview.select($()); // clear selection\n                selectedElement = null;\n            }\n        });\n\n        return dialog;\n    }\n\n\n    // Public API\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/dialogs/ElementListPickerDialog.js'),eval('define(\'text!htmlContent/print-dialog.html\',[],function () { return \'<div class="print-dialog dialog modal" data-title="Print to PDF">\\n    <div class="dialog-body">\\n        <table>\\n            <tr>\\n                <td>\\n                    <div>Print Range</div>\\n                    <div style="margin: 5px;">\\n                        <label class="k-radio-button">\\n                            <input type="radio" name="print-range" value="all">\\n                            <div class="k-radio-button__checkmark"></div>\\n                            All Diagrams\\n                        </label>\\n                        <br>\\n                        <label class="k-radio-button">\\n                            <input type="radio" name="print-range" value="current">\\n                            <div class="k-radio-button__checkmark"></div>\\n                            Current Diagram\\n                        </label>\\n                    </div>\\n                </td>\\n                <td>\\n                    <div>Page Layout</div>\\n                    <div style="margin: 5px;">\\n                        <div class="radio-button-bar">\\n                            <label class="radio-button-bar__item">\\n                                <input id="print-dialog-layout-landscape" type="radio" name="page-layout" value="landscape">\\n                                <label for="print-dialog-layout-landscape" class="k-button">Landscape</label>\\n                            </label>\\n                            <label class="radio-button-bar__item">\\n                                <input id="print-dialog-layout-portrait" type="radio" name="page-layout" value="portrait">\\n                                <label for="print-dialog-layout-portrait" class="k-button">Portrait</label>\\n                            </label>\\n                        </div>\\n                    </div>\\n                </td>\\n            </tr>\\n            <tr>\\n                <td>\\n                    <div style="margin-top: 10px;">\\n                        <label class="k-checkbox">\\n                            <input type="checkbox" class="show-diagram-name">\\n                            <div class="k-checkbox__checkmark"></div>\\n                            Show Diagram Name\\n                        </label>\\n                    </div>\\n                </td>\\n                <td>\\n                    <div>Page Size</div>\\n                    <div style="margin: 5px;">\\n                        <label class="k-select" style="width: 100px;">\\n                            <select style="width: 100%;" class="page-size">\\n                                <option value="4A0">4A0</option>\\n                                <option value="2A0">2A0</option>\\n                                <option value="A0">A0</option>\\n                                <option value="A1">A1</option>\\n                                <option value="A2">A2</option>\\n                                <option value="A3">A3</option>\\n                                <option value="A4">A4</option>\\n                                <option value="A5">A5</option>\\n                                <option value="A6">A6</option>\\n                                <option value="A7">A7</option>\\n                                <option value="A8">A8</option>\\n                                <option value="A9">A9</option>\\n                                <option value="A10">A10</option>\\n                                <option value="B0">B0</option>\\n                                <option value="B1">B1</option>\\n                                <option value="B2">B2</option>\\n                                <option value="B3">B3</option>\\n                                <option value="B4">B4</option>\\n                                <option value="B5">B5</option>\\n                                <option value="B6">B6</option>\\n                                <option value="B7">B7</option>\\n                                <option value="B8">B8</option>\\n                                <option value="B9">B9</option>\\n                                <option value="B10">B10</option>\\n                                <option value="C0">C0</option>\\n                                <option value="C1">C1</option>\\n                                <option value="C2">C2</option>\\n                                <option value="C3">C3</option>\\n                                <option value="C4">C4</option>\\n                                <option value="C5">C5</option>\\n                                <option value="C6">C6</option>\\n                                <option value="C7">C7</option>\\n                                <option value="C8">C8</option>\\n                                <option value="C9">C9</option>\\n                                <option value="C10">C10</option>\\n                                <option value="RA0">RA0</option>\\n                                <option value="RA1">RA1</option>\\n                                <option value="RA2">RA2</option>\\n                                <option value="RA3">RA3</option>\\n                                <option value="RA4">RA4</option>\\n                                <option value="SRA0">SRA0</option>\\n                                <option value="SRA1">SRA1</option>\\n                                <option value="SRA2">SRA2</option>\\n                                <option value="SRA3">SRA3</option>\\n                                <option value="SRA4">SRA4</option>\\n                                <option value="EXECUTIVE">Executive</option>\\n                                <option value="FOLIO">Folio</option>\\n                                <option value="LEGAL">Legal</option>\\n                                <option value="LETTER">Letter</option>\\n                                <option value="TABLOID">Tabloid</option>\\n                            </select>\\n                        </label>\\n                    </div>\\n                </td>\\n            </tr>\\n        </table>\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button primary" data-button-id="save">{{Strings.SAVE}}</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/print-dialog.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache*/\n\ndefine(\'dialogs/PrintDialog\',[\'require\',\'exports\',\'module\',\'core/Global\',\'strings\',\'file/FileUtils\',\'dialogs/Dialogs\',\'core/PreferenceManager\',\'text!htmlContent/print-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var global            = require("core/Global").global,\n        Strings           = require("strings"),\n        FileUtils         = require("file/FileUtils"),\n        Dialogs           = require("dialogs/Dialogs"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    var printDialogTemplate = require("text!htmlContent/print-dialog.html");\n\n    /**\n     * Show Dialog\n     * @return {$.Promise}\n     */\n    function showDialog() {\n        var context = {\n            Strings: Strings\n        };\n\n        // Print Options (Default)\n        var printOptions = PreferenceManager.get("print.options");\n        if (!printOptions) {\n            printOptions = {\n                range: "all",\n                layout: "landscape",\n                size: "A4",\n                showName: true\n            };\n        }\n\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(printDialogTemplate, context), true, function ($dlg) {\n            $dlg.data("returnValue", printOptions);\n        });\n\n        var $dlg           = dialog.getElement(),\n            $rangeRadio    = $dlg.find("input[name=\'print-range\']"),\n            $layoutRadio   = $dlg.find("input[name=\'page-layout\']"),\n            $sizeSelect    = $dlg.find("select.page-size"),\n            $showNameCheck = $dlg.find("input.show-diagram-name");\n\n        // Set Default Values\n        $rangeRadio.val([printOptions.range]);\n        $layoutRadio.val([printOptions.layout]);\n        $sizeSelect.val(printOptions.size);\n        $showNameCheck.attr(\'checked\', printOptions.showName);\n\n        // On Changes\n        $rangeRadio.change(function () {\n            printOptions.range = this.value;\n            PreferenceManager.set("print.options", printOptions);\n        });\n        $layoutRadio.change(function () {\n            printOptions.layout = this.value;\n            PreferenceManager.set("print.options", printOptions);\n        });\n        $sizeSelect.change(function () {\n            printOptions.size = $sizeSelect.val();\n            PreferenceManager.set("print.options", printOptions);\n        });\n        $showNameCheck.change(function () {\n            printOptions.showName = $showNameCheck.is(":checked");\n            PreferenceManager.set("print.options", printOptions);\n        });\n\n        return dialog;\n    }\n\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/dialogs/PrintDialog.js'),eval('define(\'text!htmlContent/enter-license-dialog.html\',[],function () { return \'<div class="enter-license-dialog dialog modal" data-title="Enter License">\\n    <div class="dialog-body">\\n        <div>Enter Name and License Key.</div>\\n        <table>\\n            <tr>\\n                <td>Name</td>\\n                <td><input type=\\\'text\\\' class=\\\'k-input k-textbox license-owner\\\'></td>\\n            </tr>\\n            <tr>\\n                <td>License Key</td>\\n                <td><textarea class=\\\'k-input k-textbox license-key\\\' rows="7"></textarea></td>\\n            </tr>\\n        </table>\\n\\n    </div>\\n    <div class="dialog-footer">\\n        <button class="k-button dialog-button left buy-now">Buy Now</button>\\n        <button class="k-button dialog-button" data-button-id="cancel">{{Strings.CANCEL}}</button>\\n        <button class="k-button dialog-button primary" data-button-id="ok">{{Strings.OK}}</button>\\n    </div>\\n</div>\\n\';});\n\n//# sourceURL=/text!htmlContent/enter-license-dialog.html'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, kendo, Mustache */\n\ndefine(\'license/EnterLicenseDialog\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/Initializer\',\'strings\',\'dialogs/Dialogs\',\'license/LicenseManager\',\'text!htmlContent/enter-license-dialog.html\'],function (require, exports, module) {\n    "use strict";\n\n    var global         = require("core/Global").global,\n        Initializer    = require("utils/Initializer"),\n        Strings        = require("strings"),\n        Dialogs        = require("dialogs/Dialogs"),\n        LicenseManager = require("license/LicenseManager");\n\n    var enterLicenseDialogTemplate = require("text!htmlContent/enter-license-dialog.html");\n\n    /**\n     * Show License Manager Dialog\n     * @return {$.Promise}\n     */\n    function showDialog() {\n        var context = {\n            Strings: Strings,\n            metadata: global.app.metadata\n        };\n        var dialog = Dialogs.showModalDialogUsingTemplate(Mustache.render(enterLicenseDialogTemplate, context));\n\n        var $dlg = dialog.getElement(),\n            $buyNow = $dlg.find(".buy-now"),\n            $licenseOwner = $dlg.find(".license-owner"),\n            $licenseKey = $dlg.find(".license-key");\n\n        $buyNow.click(function () {\n            appshell.app.openURLInDefaultBrowser(global.app.config.purchase_url);\n        });\n\n        dialog.done(function (buttonId) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                LicenseManager.register($licenseOwner.val().trim(), $licenseKey.val().trim())\n                    .done(function () {\n                        LicenseManager.validate()\n                            .done(function (status) {\n                                Dialogs.showInfoDialog("Congratulation! Your license is successfully registered.");\n                            })\n                            .fail(function (err) {\n                                Dialogs.showAlertDialog("Failed: " + err);\n                            });\n                    })\n                    .fail(function (err) {\n                        Dialogs.showAlertDialog("Failed to register License Key");\n                    });\n            }\n        });\n\n        return dialog;\n    }\n\n    exports.showDialog = showDialog;\n\n});\n\n//# sourceURL=/license/EnterLicenseDialog.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, localStorage, type, setTimeout */\n\ndefine(\'command/DefaultCommandHandlers\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/Initializer\',\'utils/AppInit\',\'utils/Helper\',\'utils/Constants\',\'core/Graphics\',\'core/Core\',\'core/Repository\',\'command/CommandManager\',\'command/Commands\',\'menu/MenuManager\',\'engine/ProjectManager\',\'engine/Engine\',\'engine/Factory\',\'engine/SelectionManager\',\'engine/ClipboardManager\',\'dialogs/Dialogs\',\'dialogs/ElementPickerDialog\',\'dialogs/ElementListPickerDialog\',\'filesystem/FileSystem\',\'filesystem/FileSystemError\',\'file/FileUtils\',\'strings\',\'diagrams/DiagramManager\',\'explorer/ModelExplorerView\',\'ui/StatusBar\',\'ui/Toast\',\'extensibility/ExtensionManagerDialog\',\'dialogs/AboutDialog\',\'preference/PreferenceDialog\',\'dialogs/PrintDialog\',\'utils/CheckUpdates\',\'metadata-json/MetadataJson\',\'license/LicenseManager\',\'license/EnterLicenseDialog\'],function (require, exports, module) {\n    "use strict";\n\n    var global                  = require("core/Global").global,\n        Initializer             = require("utils/Initializer"),\n        AppInit                 = require("utils/AppInit"),\n        Helper                  = require("utils/Helper"),\n        Constants               = require("utils/Constants"),\n        Graphics                = require("core/Graphics"),\n        Core                    = require("core/Core"),\n        Repository              = require("core/Repository"),\n        CommandManager          = require("command/CommandManager"),\n        Commands                = require("command/Commands"),\n        MenuManager             = require("menu/MenuManager"),\n        ProjectManager          = require("engine/ProjectManager"),\n        Engine                  = require("engine/Engine"),\n        Factory                 = require("engine/Factory"),\n        SelectionManager        = require("engine/SelectionManager"),\n        ClipboardManager        = require("engine/ClipboardManager"),\n        Dialogs                 = require("dialogs/Dialogs"),\n        ElementPickerDialog     = require("dialogs/ElementPickerDialog"),\n        ElementListPickerDialog = require("dialogs/ElementListPickerDialog"),\n        FileSystem              = require("filesystem/FileSystem"),\n        FileSystemError         = require("filesystem/FileSystemError"),\n        FileUtils               = require("file/FileUtils"),\n        Strings                 = require("strings"),\n        DiagramManager          = require("diagrams/DiagramManager"),\n        ModelExplorerView       = require("explorer/ModelExplorerView"),\n        StatusBar               = require("ui/StatusBar"),\n        Toast                   = require("ui/Toast"),\n        ExtensionManagerDialog  = require("extensibility/ExtensionManagerDialog"),\n        AboutDialog             = require("dialogs/AboutDialog"),\n        PreferenceDialog        = require("preference/PreferenceDialog"),\n        PrintDialog             = require("dialogs/PrintDialog"),\n        CheckUpdates            = require("utils/CheckUpdates"),\n        MetadataJson            = require("metadata-json/MetadataJson"),\n        LicenseManager          = require("license/LicenseManager"),\n        EnterLicenseDialog      = require("license/EnterLicenseDialog");\n\n    /** Unique token used to indicate user-driven cancellation of Save As (as opposed to file IO error) */\n    var USER_CANCELED = { userCanceled: true };\n\n    function _showFileError(err) {\n        if (err === FileSystemError.NOT_READABLE) {\n            Dialogs.showErrorDialog("Cannot open the file. (Only UTF-8 encoded files are supported)");\n        } else if (err === FileSystemError.NOT_WRITABLE) {\n            Dialogs.showErrorDialog("Cannot save to file.");\n        } else if (err === FileSystemError.NOT_FOUND) {\n            Dialogs.showErrorDialog("File not found.");\n        } else {\n            Dialogs.showErrorDialog("File System Error (Error=" + err + ")");\n        }\n    }\n\n    /*\n     * File Command Handlers\n     */\n\n    /**\n     * New Project (from Template)\n     * @param {?string} template - Fullpath for a template to be loaded.\n     *      If not provided "<APP_BASE>/templates/Default.mdj" will be loaded.\n     * @return {$.Promise}\n     */\n    function _doFileNew(template) {\n        if (!template) {\n            template = FileUtils.getApplicationDirectoryPath() + "templates/" + app.config.defaultTemplate;\n        }\n        localStorage.removeItem("__working_filename");\n        return ProjectManager.loadAsTemplate(template);\n    }\n\n    /**\n     * Handler for File New\n     * @param {string} template\n     * @return {$.Promise}\n     */\n    function _handleFileNew(template) {\n        var result = new $.Deferred();\n        if (Repository.isModified()) {\n            Dialogs.showSaveConfirmDialog(ProjectManager.getFilename())\n                .done(function (buttonId) {\n                    switch (buttonId) {\n                    case Dialogs.DIALOG_BTN_DONTSAVE:\n                        _doFileNew(template).then(result.resolve, result.reject);\n                        break;\n                    case Dialogs.DIALOG_BTN_SAVE:\n                        _handleFileSave()\n                            .done(function () {\n                                _doFileNew(template).then(result.resolve, result.reject);\n                            })\n                            .fail(function (err) {\n                                result.reject(err);\n                            });\n                        break;\n                    case Dialogs.DIALOG_BTN_CANCEL:\n                        result.reject(USER_CANCELED);\n                        break;\n                    }\n                });\n        } else {\n            _doFileNew(template).then(result.resolve, result.reject);\n        }\n        return result.promise();\n    }\n\n    function _doFileSave(fullPath) {\n        var result = new $.Deferred();\n        ProjectManager.save(fullPath)\n            .done(function () {\n                result.resolve();\n            })\n            .fail(function (err) {\n                _showFileError(err);\n                console.error(err);\n                result.reject(err);\n            });\n        return result.promise();\n    }\n\n    /**\n     * Handler for File Save\n     * @param {string} fullPath\n     * @param {boolean} saveAs\n     * @return {$.Promise}\n     */\n    function _handleFileSave(fullPath, saveAs) {\n        var result = new $.Deferred();\n\n        // Set focus to body in order to apply changes of documentation editor\n        $("#diagram-canvas").focus();\n\n        if (fullPath) {\n            ProjectManager.save(fullPath).then(result.resolve, result.reject);\n        } else {\n            if (ProjectManager.getFilename() && !saveAs) {\n                return _doFileSave(ProjectManager.getFilename()).then(result.resolve, result.reject);\n            } else {\n                FileSystem.showSaveDialog("Save Project As", null, "Untitled." + Constants.APP_EXT, function (err, selectedPath) {\n                    if (!err) {\n                        if (selectedPath) {\n                            if (!FileUtils.getFileExtension(selectedPath)) {\n                                selectedPath = selectedPath + "." + Constants.APP_EXT;\n                            }\n                            _doFileSave(selectedPath).then(result.resolve, result.reject);\n                        } else {\n                            result.reject(USER_CANCELED);\n                        }\n                    } else {\n                        result.reject(err);\n                    }\n                });\n            }\n        }\n        return result.promise();\n    }\n\n\n    function _doFileOpen(fullPath) {\n        var result = new $.Deferred();\n        if (Repository.isModified()) {\n            Dialogs.showSaveConfirmDialog(ProjectManager.getFilename())\n                .done(function (buttonId) {\n                    switch (buttonId) {\n                    case Dialogs.DIALOG_BTN_DONTSAVE:\n                        ProjectManager.load(fullPath).then(result.resolve, result.reject);\n                        break;\n                    case Dialogs.DIALOG_BTN_SAVE:\n                        _handleFileSave()\n                            .done(function () {\n                                ProjectManager.load(fullPath)\n                                    .done(function () {\n                                        result.resolve();\n                                    })\n                                    .fail(function (err) {\n                                        _showFileError(err);\n                                        console.error(err);\n                                        result.reject(err);\n                                    });\n                            })\n                            .fail(function (err) {\n                                result.reject(err);\n                            });\n                        break;\n                    case Dialogs.DIALOG_BTN_CANCEL:\n                        result.reject(USER_CANCELED);\n                        break;\n                    }\n                });\n        } else {\n            ProjectManager.load(fullPath).then(result.resolve, result.reject);\n        }\n        return result.promise();\n    }\n\n    /**\n     * Handler for File Open\n     * param {string} fullPath\n     * @return {$.Promise}\n     */\n    function _handleFileOpen(fullPath) {\n        var result = new $.Deferred();\n        if (fullPath) {\n            _doFileOpen(fullPath).then(result.resolve, result.reject);\n        } else {\n            FileSystem.showOpenDialog(false, false, "Select a model file...", null, ["umlj", Constants.APP_EXT], function (err, files) {\n                if (!err) {\n                    if (files.length > 0) {\n                        _doFileOpen(files[0]).then(result.resolve, result.reject);\n                    } else {\n                        result.reject(USER_CANCELED);\n                    }\n                } else {\n                    result.reject(err);\n                }\n            });\n        }\n        return result.promise();\n    }\n\n    /**\n     * Handler for Import Fragment\n     * param {string} fullPath\n     * @return {$.Promise}\n     */\n    function _handleImportFragment(fullPath) {\n        var result = new $.Deferred();\n        if (fullPath) {\n            ProjectManager.importFromFile(ProjectManager.getProject(), fullPath).then(result.resolve, result.reject);\n        } else {\n            FileSystem.showOpenDialog(false, false, "Select a model fragment to import...", null, ["umlf", Constants.FRAG_EXT], function (err, files) {\n                if (!err) {\n                    if (files.length > 0) {\n                        ProjectManager.importFromFile(ProjectManager.getProject(), files[0])\n                            .done(function () {\n                                result.resolve();\n                            })\n                            .fail(function (err) {\n                                _showFileError(err);\n                                console.error(err);\n                                result.reject(err);\n                            });\n                    } else {\n                        result.reject(USER_CANCELED);\n                    }\n                } else {\n                    result.reject(err);\n                }\n            });\n        }\n        return result.promise();\n    }\n\n\n    function _doFileExport(elem, filename) {\n        var result = new $.Deferred();\n        ProjectManager.exportToFile(elem, filename)\n            .done(function () {\n                result.resolve();\n            })\n            .fail(function (err) {\n                _showFileError(err);\n                console.error(err);\n                result.reject(err);\n            });\n        return result.promise();\n    }\n\n    /**\n     * Handler for Export Fragment\n     * param {Element} element\n     * param {string} fullPath\n     * @return {$.Promise}\n     */\n    function _handleExportFragment(element, fullPath) {\n        var result = new $.Deferred();\n        if (element) {\n            if (fullPath) {\n                ProjectManager.exportToFile(element, fullPath).then(result.resolve, result.reject);\n            } else {\n                FileSystem.showSaveDialog("Export Model Fragment", null, "Fragment." + Constants.FRAG_EXT, function (err, filename) {\n                    if (!err) {\n                        if (filename) {\n                            if (!FileUtils.getFileExtension(filename)) {\n                                filename = filename + "." + Constants.FRAG_EXT;\n                            }\n                            _doFileExport(element, filename).then(result.resolve, result.reject);\n                        } else {\n                            result.reject(USER_CANCELED);\n                        }\n                    } else {\n                        result.reject(err);\n                    }\n                });\n            }\n        } else {\n            if (fullPath) {\n                ElementPickerDialog.showDialog("Select Element to Export", null, null)\n                    .done(function (buttonId, selected) {\n                        if (buttonId === Dialogs.DIALOG_BTN_OK && selected !== null) {\n                            _doFileExport(selected, fullPath).then(result.resolve, result.reject);\n                        } else {\n                            result.reject(USER_CANCELED);\n                        }\n                    });\n            } else {\n                ElementPickerDialog.showDialog("Select Element to Export", null, null)\n                    .done(function (buttonId, selected) {\n                        if (buttonId === Dialogs.DIALOG_BTN_OK && selected !== null) {\n                            FileSystem.showSaveDialog("Export Model Fragment", null, "Fragment." + Constants.FRAG_EXT, function (err, filename) {\n                                if (!err) {\n                                    if (filename) {\n                                        if (!FileUtils.getFileExtension(filename)) {\n                                            filename = filename + "." + Constants.FRAG_EXT;\n                                        }\n                                        _doFileExport(selected, filename).then(result.resolve, result.reject);\n                                    } else {\n                                        result.reject(USER_CANCELED);\n                                    }\n                                } else {\n                                    result.reject(err);\n                                }\n                            });\n                        } else {\n                            result.reject(USER_CANCELED);\n                        }\n                    });\n            }\n        }\n        return result.promise();\n    }\n\n    /**\n     * Handler for File Close\n     * @return {$.Promise}\n     */\n    function _handleFileClose() {\n        var result = new $.Deferred();\n        if (Repository.isModified()) {\n            Dialogs.showSaveConfirmDialog(ProjectManager.getFilename())\n                .done(function (buttonId) {\n                    switch (buttonId) {\n                    case Dialogs.DIALOG_BTN_DONTSAVE:\n                        ProjectManager.closeProject();\n                        result.resolve();\n                        break;\n                    case Dialogs.DIALOG_BTN_SAVE:\n                        _handleFileSave()\n                            .then(function () {\n                                ProjectManager.closeProject();\n                                result.resolve();\n                            })\n                            .fail(function (err) {\n                                result.reject(err);\n                            });\n                        break;\n                    case Dialogs.DIALOG_BTN_CANCEL:\n                        result.reject();\n                        break;\n                    }\n                });\n        } else {\n            ProjectManager.closeProject();\n            result.resolve();\n        }\n        localStorage.removeItem("__working_filename");\n        return result.promise();\n    }\n\n    /**\n     * Handler for Preference\n     */\n    function _handlePreferences(preferenceId) {\n        PreferenceDialog.showDialog(preferenceId);\n    }\n\n    /**\n     * Handler for Print to PDF\n     */\n    function _handlePrintToPDF() {\n        var result = new $.Deferred();\n        if (Repository.isModified() || !ProjectManager.getFilename()) {\n            Dialogs.showInfoDialog("Save changes before printing").done(function () {\n                result.reject(USER_CANCELED);\n            });\n        } else {\n            PrintDialog.showDialog().done(function (buttonId, printOptions) {\n                if (buttonId === Dialogs.DIALOG_BTN_SAVE) {\n                    var diagrams = [];\n                    if (printOptions.range === "current") {\n                        var current = DiagramManager.getCurrentDiagram();\n                        if (current) {\n                            diagrams.push(current);\n                        }\n                    } else {\n                       diagrams = Repository.getInstancesOf("Diagram");\n                    }\n                    if (diagrams.length > 0) {\n                        var fn = FileUtils.convertToWindowsFilename(ProjectManager.getProject().name);\n                        FileSystem.showSaveDialog("Print to PDF", null, fn + ".pdf", function (err, filename) {\n                            if (!err) {\n                                if (filename) {\n                                    if (!FileUtils.getFileExtension(filename)) {\n                                        filename = filename + ".pdf";\n                                    }\n                                    MetadataJson.exportToPDF(ProjectManager.getFilename(), diagrams, filename, printOptions)\n                                        .done(function () {\n                                            result.resolve();\n                                        })\n                                        .fail(function (err) {\n                                            Dialogs.showErrorDialog("Failed to generate PDF. (Error=" + err + ")");\n                                            console.error(err);\n                                            result.reject();\n                                        });\n                                } else {\n                                    result.reject(USER_CANCELED);\n                                }\n                            } else {\n                                result.reject(err);\n                            }\n                        });\n                    } else {\n                        Dialogs.showAlertDialog("No current diagram.").done(function () {\n                            result.reject(USER_CANCELED);\n                        });\n                    }\n                } else {\n                    result.reject(USER_CANCELED);\n                }\n            });\n\n        }\n        return result.promise();\n    }\n\n    function _beforeQuit() {\n        DiagramManager.saveWorkingDiagrams();\n        localStorage.removeItem("__backup_filename");\n    }\n\n    /**\n     * Handler for Close Window.\n     * Returning rejected promise will close application, otherwise application will not be closed.\n     */\n    function _handleFileCloseWindow() {\n        var result = new $.Deferred();\n        if (Repository.isModified()) {\n            _handleFileClose()\n                .done(function () {\n                    CommandManager.execute(Commands.FILE_QUIT);\n                });\n            result.resolve();\n        } else {\n            _beforeQuit();\n            result.reject();\n        }\n        return result.promise();\n    }\n\n    function _handleFileQuit() {\n        _beforeQuit();\n        appshell.app.quit();\n    }\n\n    /*\n     * Edit Command Handlers\n     */\n\n    function _handleUndo() {\n        Repository.undo();\n    }\n\n    function _handleRedo() {\n        Repository.redo();\n    }\n\n    function _handleCut() {\n        var models = SelectionManager.getSelectedModels(),\n            views  = SelectionManager.getSelectedViews();\n        // Cut Model\n        if (models.length === 1 && views.length === 0 && models[0].canCopy()) {\n            ClipboardManager.setModel(models[0]);\n            _handleDeleteFromModel();\n        }\n        // Cut Views\n        var diagram = DiagramManager.getEditor().diagram;\n        if (views.length > 0 && diagram && diagram.canCopyViews() && diagram.canDeleteViews()) {\n            ClipboardManager.setViews(views);\n            _handleDelete();\n        }\n    }\n\n    function _handleCopy() {\n        var models = SelectionManager.getSelectedModels(),\n            views  = SelectionManager.getSelectedViews();\n        // Copy Model\n        if (models.length === 1 && views.length === 0 && models[0].canCopy()) {\n            ClipboardManager.setModel(models[0]);\n        }\n        // Copy Views: remove views where canCopy() === false\n        if (views.length > 0) {\n            var diagram = DiagramManager.getEditor().diagram;\n            for (var i = views.length - 1; i >= 0; i--) {\n                if (!views[i].canCopy()) {\n                    diagram.deselectView(views[i]);\n                }\n            }\n            views = diagram.selectedViews;\n            if (views.length > 0 && diagram && diagram.canCopyViews()) {\n                ClipboardManager.setViews(views);\n            } else {\n                if (diagram instanceof type.UMLSequenceDiagram || diagram instanceof type.UMLCommunicationDiagram) {\n                    Dialogs.showInfoDialog("Copying view elements in Sequence or Communication Diagram is not supported.\\n To copy the entire Sequence or Communication Diagram, copy \'Interaction\' or \'Collaboration (or Classifier)\' containing the Diagram in Explorer.");\n                } else {\n                    Dialogs.showInfoDialog("No views to copy or some fo selected views cannot be copied.");\n                }\n            }\n        }\n    }\n\n    function _handlePaste() {\n        var kind = ClipboardManager.getKind();\n        // Paste Model\n        if (kind === ClipboardManager.DK_MODEL) {\n            var parent  = SelectionManager.getSelected(),\n                context = ClipboardManager.getCopyContext();\n            if (parent && parent.canPaste(ClipboardManager.getElementType(), context)) {\n                var model = ClipboardManager.getModel();\n                Engine.addModel(parent, context.field, model);\n            }\n        // Paste Views\n        } else if (kind === ClipboardManager.DK_VIEWS) {\n            var views = ClipboardManager.getViews();\n            var diagram = DiagramManager.getEditor().diagram;\n\n            if (views.length > 0 && diagram) {\n\n                // Views in clipboard can be pasted in diagram\n                if (diagram.canPasteViews(views)) {\n                    // Deselect all views.\n                    DiagramManager.deselectAll();\n\n                    // Compute bounding box of views\n                    var boundingBox = views[0].getBoundingBox(),\n                        i, len;\n                    for (i = 0, len = views.length; i < len; i++) {\n                        boundingBox.union(views[i].getBoundingBox());\n                    }\n\n                    // Compute dx, dy to place pasted views on the center of screen\n                    var areaCenter = DiagramManager.getDiagramArea().getCenter(),\n                        boundCenter = boundingBox.getCenter(),\n                        dx = Math.round(areaCenter.x - boundCenter.x),\n                        dy = Math.round(areaCenter.y - boundCenter.y);\n\n                    // Move views to be paste as (dx, dy).\n                    for (i = 0, len = views.length; i < len; i++) {\n                        views[i].move(DiagramManager.getEditor().canvas, dx, dy);\n                    }\n\n                    Engine.addViews(diagram, views);\n                    // Select the pasted views.\n                    var newViews = [];\n                    for (i = 0, len = views.length; i < len; i++) {\n                        var v = Repository.get(views[i]._id);\n                        diagram.selectView(v);\n                        newViews.push(v);\n                    }\n                    SelectionManager.selectViews(newViews);\n                } else {\n                    Dialogs.showInfoDialog("Views in clipboard cannot be pasted in this diagram.");\n                }\n            }\n        }\n    }\n\n    function _handleDelete() {\n        var diagram = DiagramManager.getEditor().diagram;\n        if (diagram && diagram.canDeleteViews()) {\n            var views = SelectionManager.getSelectedViews();\n            Engine.deleteElements([], views);\n            SelectionManager.deselectAll();\n        } else {\n            // Try to delete views only for the elements recommended to be deleted with models (e.g. Lifelines)\n            Dialogs.showModalDialog(\n                "",\n                "Delete Views Only",\n                "Do you want to delete views only?",\n                [\n                    { id: "delete-from-model", text: "Delete from Model", className: Dialogs.DIALOG_BTN_CLASS_LEFT },\n                    { id: "delete-views-only", text: "Delete Views Only", className: Dialogs.DIALOG_BTN_CLASS_PRIMARY },\n                    { id: Dialogs.DIALOG_BTN_CANCEL, text: Strings.CANCEL }\n                ]\n            ).done(function (buttonId) {\n                switch (buttonId) {\n                case "delete-from-model":\n                    CommandManager.execute(Commands.EDIT_DELETE_FROM_MODEL);\n                    break;\n                case "delete-views-only":\n                    var _views = SelectionManager.getSelectedViews();\n                    Engine.deleteElements([], _views);\n                    SelectionManager.deselectAll();\n                    break;\n                }\n                console.log(buttonId);\n            });\n            // Dialogs.showAlertDialog("Selected views should be deleted with models by \'Delete from Model\'");\n        }\n    }\n\n    function _handleDeleteFromModel() {\n        var models = _.clone(SelectionManager.getSelectedModels());\n        _.each(SelectionManager.getSelectedViews(), function (view) {\n            if (view.model && !_.contains(models, view.model)) {\n                models.push(view.model);\n            }\n        });\n        if (models.length > 0) {\n            _.each(models, function (e) {\n                if (e instanceof type.Diagram) {\n                    DiagramManager.closeDiagram(e);\n                }\n            });\n            Engine.deleteElements(models, []);\n            SelectionManager.deselectAll();\n        }\n    }\n\n    function _handleMoveUp() {\n        var elem = SelectionManager.getSelected();\n        if (elem) {\n            Engine.moveUp(elem._parent, elem.getParentField(), elem);\n        }\n    }\n\n    function _handleMoveDown() {\n        var elem = SelectionManager.getSelected();\n        if (elem) {\n            Engine.moveDown(elem._parent, elem.getParentField(), elem);\n        }\n    }\n\n    function _handleSelectAll() {\n        // Select text in focused <input>, <textarea> when press Ctrl+A.\n        var $focused = $(\'input:focus, textarea:focus\');\n        if ($focused.length > 0) {\n            $focused.select();\n        // Otherwise, select views in diagram.\n        } else {\n            DiagramManager.selectAll();\n        }\n    }\n\n    function _handleSelectInExplorer() {\n        var models = SelectionManager.getSelectedModels();\n        if (models.length > 0) {\n            ModelExplorerView.select(models[0], true);\n        }\n    }\n\n    function _handleSelectInDiagram() {\n        var models = SelectionManager.getSelectedModels();\n        if (models.length > 0) {\n            var views = _.reject(Repository.getViewsOf(models[0]), function (v) {\n                return !(v._parent instanceof type.Diagram);\n            });\n            if (views.length === 1) {\n                DiagramManager.selectInDiagram(views[0]);\n            } else if (views.length > 1) {\n                var diagrams   = [],\n                    diagramMap = {};\n                _.each(views, function (v) {\n                    var d = v.getDiagram();\n                    diagrams.push(d);\n                    diagramMap[d._id] = v;\n                });\n                ElementListPickerDialog.showDialog("Select Diagram to show", diagrams)\n                    .done(function (buttonId, selected) {\n                        if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                            if (selected) {\n                                var selectedDiagram = diagramMap[selected._id];\n                                DiagramManager.selectInDiagram(selectedDiagram);\n                            }\n                        }\n                    });\n            } else {\n                Toast.info("No diagrams to show");\n            }\n        }\n    }\n\n    /*\n     * Format Command Handlers\n     */\n\n    function _handleFont() {\n        var views = SelectionManager.getSelectedViews();\n        var fonts = _.map(views, function (view) { return view.font; });\n        var font = {\n            face: Helper.reduceValue(fonts, \'face\'),\n            size: Helper.reduceValue(fonts, \'size\'),\n            color: Helper.reduceValue(views, \'fontColor\')\n        };\n        Dialogs.showFontDialog(font).done(function (buttonId, result) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                if (result.size && !_.isNumber(result.size)) {\n                    result.size = parseInt(result.size);\n                }\n                Engine.setFont(DiagramManager.getEditor(), views, result.face, result.size, result.color);\n            }\n        });\n    }\n\n    function _handleFillColor() {\n        var views = SelectionManager.getSelectedViews();\n        var color = Helper.reduceValue(views, \'fillColor\');\n        Dialogs.showColorDialog(color).done(function (buttonId, result) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                Engine.setFillColor(DiagramManager.getEditor(), views, result);\n            }\n        });\n    }\n\n    function _handleLineColor() {\n        var views = SelectionManager.getSelectedViews();\n        var color = Helper.reduceValue(views, \'lineColor\');\n        Dialogs.showColorDialog(color).done(function (buttonId, result) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                Engine.setLineColor(DiagramManager.getEditor(), views, result);\n            }\n        });\n    }\n\n    function _handleLineStyleRectilinear() {\n        var views = SelectionManager.getSelectedViews();\n        Engine.setElemsProperty(views, \'lineStyle\', Core.LS_RECTILINEAR);\n    }\n\n    function _handleLineStyleOblique() {\n        var views = SelectionManager.getSelectedViews();\n        Engine.setElemsProperty(views, \'lineStyle\', Core.LS_OBLIQUE);\n    }\n\n    function _handleLineStyleRoundRect() {\n        var views = SelectionManager.getSelectedViews();\n        Engine.setElemsProperty(views, \'lineStyle\', Core.LS_ROUNDRECT);\n    }\n\n    function _handleLineStyleCurve() {\n        var views = SelectionManager.getSelectedViews();\n        Engine.setElemsProperty(views, \'lineStyle\', Core.LS_CURVE);\n    }\n\n    function _handleAutoResize() {\n        var views = SelectionManager.getSelectedViews();\n        var autoResize = Helper.reduceValue(views, \'autoResize\');\n        Engine.setElemsProperty(views, \'autoResize\', !autoResize);\n    }\n\n    function _handleShowShadow() {\n        var views = SelectionManager.getSelectedViews();\n        var showShadow = Helper.reduceValue(views, \'showShadow\');\n        Engine.setElemsProperty(views, \'showShadow\', !showShadow);\n    }\n\n    /*\n     * View Command Handlers\n     */\n\n    function _handleCloseDiagram() {\n        DiagramManager.closeDiagram(DiagramManager.getCurrentDiagram());\n    }\n\n    function _handleCloseOtherDiagrams() {\n        DiagramManager.closeOthers();\n    }\n\n    function _handleCloseAllDiagrams() {\n        DiagramManager.closeAll();\n    }\n\n    function _handleNextDiagram() {\n        DiagramManager.nextDiagram();\n    }\n\n    function _handlePreviousDiagram() {\n        DiagramManager.previousDiagram();\n    }\n\n    function _handleZoomIn() {\n        DiagramManager.setZoomLevel(DiagramManager.getZoomLevel() + 0.1);\n        StatusBar.setZoomLevel(DiagramManager.getZoomLevel());\n    }\n\n    function _handleZoomOut() {\n        DiagramManager.setZoomLevel(DiagramManager.getZoomLevel() - 0.1);\n        StatusBar.setZoomLevel(DiagramManager.getZoomLevel());\n    }\n\n    function _handleActualSize() {\n        DiagramManager.setZoomLevel(1);\n        StatusBar.setZoomLevel(DiagramManager.getZoomLevel());\n    }\n\n    function _handleFitToWindow() {\n        var diagram = DiagramManager.getCurrentDiagram();\n        if (diagram) {\n            var size = DiagramManager.getViewportSize(),\n                box  = diagram.getBoundingBox(),\n                hr   = size.x / box.x2,\n                wr   = size.y / box.y2,\n                zoom = Math.min(hr, wr);\n            if (zoom > 1) { zoom = 1; }\n            DiagramManager.setZoomLevel(zoom);\n            DiagramManager.scrollTo(0, 0, true);\n            StatusBar.setZoomLevel(DiagramManager.getZoomLevel());\n        }\n    }\n\n    function _handleShowGrid() {\n        DiagramManager.toggleGrid();\n    }\n\n    /*\n     * Tools Command Handlers\n     */\n\n    function _handleExtensionManager() {\n        ExtensionManagerDialog.showDialog();\n    }\n\n    /*\n     * Help Command Handlers\n     */\n\n    function _handleAbout() {\n        AboutDialog.showDialog();\n    }\n\n    function _handleCheckForUpdates() {\n        CheckUpdates.checkForUpdates()\n            .done(function (newRelease) {\n                CheckUpdates.showUpdateDialog(newRelease);\n            })\n            .fail(function (noUpdates) {\n                Dialogs.showInfoDialog(noUpdates);\n            });\n    }\n\n    function _handleEnterLicense() {\n        if (LicenseManager.getStatus() === LicenseManager.STAT_REGISTERED) {\n            Dialogs.showInfoDialog("You already have a valid license.");\n        } else {\n            EnterLicenseDialog.showDialog();\n        }\n    }\n\n    function _handleDocumentation() {\n        appshell.app.openURLInDefaultBrowser(global.app.config.documentation_url);\n    }\n\n    function _handleForum() {\n        appshell.app.openURLInDefaultBrowser(global.app.config.forum_url);\n    }\n\n    function _handleReleaseNotes() {\n        appshell.app.openURLInDefaultBrowser(global.app.config.release_notes_url);\n    }\n\n    function _handleRequestFeature() {\n        appshell.app.openURLInDefaultBrowser(global.app.config.feature_request_url);\n    }\n\n    /*\n     * Internal Command Handlers\n     */\n\n    function _handleReload() {\n        setTimeout(function () {\n            // Remove all menus before reload (One reason is to remove menus registered by extensions)\n            _.forEach(MenuManager.getAllMenus(), function (value, key) {\n                MenuManager.removeMenu(key);\n            });\n            window.location.reload(true);\n        }, 100);\n    }\n\n    function _updateCommands() {\n        try {\n            var models = SelectionManager.getSelectedModels(),\n                views = SelectionManager.getSelectedViews();\n\n            // The reason why we don\'t disable Copy, Cut, Paste, Delete Menus is to allow\n            // the browser\'s original actions. (e.g. editing in Input or TextArea)\n            // Update Delete From Model Menu\n            var _deleteFromModelCmd = CommandManager.get(Commands.EDIT_DELETE_FROM_MODEL);\n            if (models.length > 0 && models[0].canDelete()) {\n                _deleteFromModelCmd.setEnabled(true);\n            } else {\n                _deleteFromModelCmd.setEnabled(false);\n            }\n            // Update Font, Line Color, Fill Color Menus\n            CommandManager.get(Commands.FORMAT_FONT).setEnabled(views.length > 0);\n            CommandManager.get(Commands.FORMAT_FILL_COLOR).setEnabled(views.length > 0);\n            CommandManager.get(Commands.FORMAT_LINE_COLOR).setEnabled(views.length > 0);\n\n            // Update LineStyle Menu\n            CommandManager.get(Commands.FORMAT_LINE_STYLE).setEnabled(views.length > 0);\n            var lineStyle = Helper.reduceValue(views, "lineStyle");\n            CommandManager.get(Commands.FORMAT_LINE_STYLE_RECTILINEAR).setChecked(lineStyle === Core.LS_RECTILINEAR);\n            CommandManager.get(Commands.FORMAT_LINE_STYLE_OBLIQUE).setChecked(lineStyle === Core.LS_OBLIQUE);\n            CommandManager.get(Commands.FORMAT_LINE_STYLE_ROUNDRECT).setChecked(lineStyle === Core.LS_ROUNDRECT);\n            CommandManager.get(Commands.FORMAT_LINE_STYLE_CURVE).setChecked(lineStyle === Core.LS_CURVE);\n\n            // Update AutoResize Menu\n            CommandManager.get(Commands.FORMAT_AUTO_RESIZE).setEnabled(views.length > 0);\n            var autoResize = Helper.reduceValue(views, \'autoResize\');\n            CommandManager.get(Commands.FORMAT_AUTO_RESIZE).setChecked(autoResize === true);\n\n            // Update ShowShadow Menu\n            CommandManager.get(Commands.FORMAT_SHOW_SHADOW).setEnabled(views.length > 0);\n            var showShadow = Helper.reduceValue(views, \'showShadow\');\n            CommandManager.get(Commands.FORMAT_SHOW_SHADOW).setChecked(showShadow === true);\n\n            // TODO: Update Model > Add\n            /*\n            var i, len, _addEnable = false;\n            for (i = 0, len = addCommands.length; i < len; i++) {\n                if (CommandManager.get(addCommands[i]).getVisible()) {\n                    _addEnable = true;\n                }\n            }\n            CommandManager.get(UMLCommands.MODEL_ADD).setEnabled(_addEnable);\n            */\n\n            // TODO: Update Model > Add Diagram\n            /*\n            var _addDiagramEnable = false;\n            for (i = 0, len = addDiagramCommands.length; i < len; i++) {\n                if (CommandManager.get(addDiagramCommands[i]).getVisible()) {\n                    _addDiagramEnable = true;\n                }\n            }\n            CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM).setEnabled(_addDiagramEnable);\n            */\n\n            /*\n            var modelSize = models.length,\n                viewSize = views.length;\n            MenuManager.setCommonMenuItemEnable(Commands.EDIT_DELETE_FROM_MODEL, false);\n            MenuManager.setCommonMenuItemEnable(Commands.FORMAT, false);\n            if (viewSize > 0) {\n                MenuManager.setCommonMenuItemEnable(Commands.FORMAT, true);\n            }\n            var lineStyle = Helper.reduceValue(views, \'lineStyle\');\n            MenuManager.setCommonMenuItemChecked(Commands.FORMAT_LINE_STYLE_OBLIQUE, (lineStyle === Core.LS_OBLIQUE));\n            MenuManager.setCommonMenuItemChecked(Commands.FORMAT_LINE_STYLE_RECTILINEAR, (lineStyle === Core.LS_RECTILINEAR));\n            var autoResize = Helper.reduceValue(views, \'autoResize\');\n            MenuManager.setCommonMenuItemChecked(Commands.FORMAT_AUTO_RESIZE, (autoResize === true));\n            */\n        } catch (err) {\n            console.error(err);\n        }\n    }\n\n    // Set some command strings\n    var quitString  = Strings.CMD_QUIT;\n    if (app.platform === "win") {\n        quitString  = Strings.CMD_EXIT;\n    }\n\n    // Register Commands\n    // File\n    CommandManager.register(Strings.CMD_FILE,                    Commands.FILE,                    CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_FILE_NEW,                Commands.FILE_NEW,                _handleFileNew);\n    CommandManager.register(Strings.CMD_FILE_NEW_FROM_TEMPLATE,  Commands.FILE_NEW_FROM_TEMPLATE,  _handleFileNew);\n    CommandManager.register(Strings.CMD_FILE_OPEN,               Commands.FILE_OPEN,               _handleFileOpen);\n    CommandManager.register(Strings.CMD_FILE_SAVE,               Commands.FILE_SAVE,               _handleFileSave);\n    CommandManager.register(Strings.CMD_FILE_SAVE_AS,            Commands.FILE_SAVE_AS,            _.partial(_handleFileSave, undefined, true));\n    CommandManager.register(Strings.CMD_FILE_IMPORT,             Commands.FILE_IMPORT,             CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_FILE_IMPORT_FRAGMENT,    Commands.FILE_IMPORT_FRAGMENT,    _handleImportFragment);\n    CommandManager.register(Strings.CMD_FILE_EXPORT,             Commands.FILE_EXPORT,             CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_FILE_EXPORT_FRAGMENT,    Commands.FILE_EXPORT_FRAGMENT,    _handleExportFragment);\n    CommandManager.register(Strings.CMD_FILE_CLOSE,              Commands.FILE_CLOSE,              _handleFileClose);\n    CommandManager.register(Strings.CMD_FILE_PREFERENCES,        Commands.FILE_PREFERENCES,        _handlePreferences);\n    CommandManager.register(Strings.CMD_FILE_PREFERENCES,        "debug.openPrefsInSplitView",     _handlePreferences); // Mac OS X\'s Default Preferences Command ID\n    CommandManager.register(Strings.CMD_FILE_PRINT_TO_PDF,       Commands.FILE_PRINT_TO_PDF,       _handlePrintToPDF);\n    CommandManager.register(quitString,                          Commands.FILE_QUIT,               _handleFileQuit);\n\n    // Edit\n    CommandManager.register(Strings.CMD_EDIT,                    Commands.EDIT,                    CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_EDIT_UNDO,               Commands.EDIT_UNDO,               _handleUndo);\n    CommandManager.register(Strings.CMD_EDIT_REDO,               Commands.EDIT_REDO,               _handleRedo);\n    CommandManager.register(Strings.CMD_EDIT_CUT,                Commands.EDIT_CUT,                _handleCut);\n    CommandManager.register(Strings.CMD_EDIT_COPY,               Commands.EDIT_COPY,               _handleCopy);\n    CommandManager.register(Strings.CMD_EDIT_PASTE,              Commands.EDIT_PASTE,              _handlePaste);\n    CommandManager.register(Strings.CMD_EDIT_DELETE,             Commands.EDIT_DELETE,             _handleDelete);\n    CommandManager.register(Strings.CMD_EDIT_DELETE_FROM_MODEL,  Commands.EDIT_DELETE_FROM_MODEL,  _handleDeleteFromModel);\n    CommandManager.register(Strings.CMD_EDIT_MOVE_UP,            Commands.EDIT_MOVE_UP,            _handleMoveUp);\n    CommandManager.register(Strings.CMD_EDIT_MOVE_DOWN,          Commands.EDIT_MOVE_DOWN,          _handleMoveDown);\n    CommandManager.register(Strings.CMD_EDIT_SELECT_ALL,         Commands.EDIT_SELECT_ALL,         _handleSelectAll);\n    CommandManager.register(Strings.CMD_EDIT_SELECT_IN_EXPLORER, Commands.EDIT_SELECT_IN_EXPLORER, _handleSelectInExplorer);\n    CommandManager.register(Strings.CMD_EDIT_SELECT_IN_DIAGRAM,  Commands.EDIT_SELECT_IN_DIAGRAM,  _handleSelectInDiagram);\n\n    // Format\n    CommandManager.register(Strings.CMD_FORMAT,                        Commands.FORMAT,            CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_FORMAT_FONT,                   Commands.FORMAT_FONT,       _handleFont);\n    CommandManager.register(Strings.CMD_FORMAT_FILL_COLOR,             Commands.FORMAT_FILL_COLOR, _handleFillColor);\n    CommandManager.register(Strings.CMD_FORMAT_LINE_COLOR,             Commands.FORMAT_LINE_COLOR, _handleLineColor);\n    CommandManager.register(Strings.CMD_FORMAT_LINE_STYLE,             Commands.FORMAT_LINE_STYLE, CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_FORMAT_LINE_STYLE_RECTILINEAR, Commands.FORMAT_LINE_STYLE_RECTILINEAR, _handleLineStyleRectilinear);\n    CommandManager.register(Strings.CMD_FORMAT_LINE_STYLE_OBLIQUE,     Commands.FORMAT_LINE_STYLE_OBLIQUE,     _handleLineStyleOblique);\n    CommandManager.register(Strings.CMD_FORMAT_LINE_STYLE_ROUNDRECT,   Commands.FORMAT_LINE_STYLE_ROUNDRECT,   _handleLineStyleRoundRect);\n    CommandManager.register(Strings.CMD_FORMAT_LINE_STYLE_CURVE,       Commands.FORMAT_LINE_STYLE_CURVE,       _handleLineStyleCurve);\n    CommandManager.register(Strings.CMD_FORMAT_AUTO_RESIZE,            Commands.FORMAT_AUTO_RESIZE, _handleAutoResize);\n    CommandManager.register(Strings.CMD_FORMAT_SHOW_SHADOW,            Commands.FORMAT_SHOW_SHADOW, _handleShowShadow);\n\n    // Model\n    CommandManager.register(Strings.CMD_MODEL,                         Commands.MODEL,             CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_MODEL_ADD,                     Commands.MODEL_ADD,         Factory.createModel);\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM,             Commands.MODEL_ADD_DIAGRAM, Factory.createDiagram);\n\n    // Tools\n    CommandManager.register(Strings.CMD_TOOLS,                         Commands.TOOLS,                     CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_TOOLS_EXTENSION_MANAGER,       Commands.TOOLS_EXTENSION_MANAGER,   _handleExtensionManager);\n\n    // View\n    CommandManager.register(Strings.CMD_VIEW,                          Commands.VIEW,                      CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_VIEW_CLOSE_DIAGRAM,            Commands.VIEW_CLOSE_DIAGRAM,        _handleCloseDiagram);\n    CommandManager.register(Strings.CMD_VIEW_CLOSE_OTHER_DIAGRAMS,     Commands.VIEW_CLOSE_OTHER_DIAGRAMS, _handleCloseOtherDiagrams);\n    CommandManager.register(Strings.CMD_VIEW_CLOSE_ALL_DIAGRAMS,       Commands.VIEW_CLOSE_ALL_DIAGRAMS,   _handleCloseAllDiagrams);\n    CommandManager.register(Strings.CMD_VIEW_NEXT_DIAGRAM,             Commands.VIEW_NEXT_DIAGRAM,         _handleNextDiagram);\n    CommandManager.register(Strings.CMD_VIEW_PREVIOUS_DIAGRAM,         Commands.VIEW_PREVIOUS_DIAGRAM,     _handlePreviousDiagram);\n    CommandManager.register(Strings.CMD_VIEW_ZOOM_IN,                  Commands.VIEW_ZOOM_IN,              _handleZoomIn);\n    CommandManager.register(Strings.CMD_VIEW_ZOOM_OUT,                 Commands.VIEW_ZOOM_OUT,             _handleZoomOut);\n    CommandManager.register(Strings.CMD_VIEW_ACTUAL_SIZE,              Commands.VIEW_ACTUAL_SIZE,          _handleActualSize);\n    CommandManager.register(Strings.CMD_VIEW_FIT_TO_WINDOW,            Commands.VIEW_FIT_TO_WINDOW,        _handleFitToWindow);\n    CommandManager.register(Strings.CMD_VIEW_SHOW_GRID,                Commands.VIEW_SHOW_GRID,            _handleShowGrid);\n\n    // Help\n    CommandManager.register(Strings.CMD_HELP,                          Commands.HELP,                      CommandManager.doNothing);\n    CommandManager.register(Strings.CMD_HELP_ABOUT,                    Commands.HELP_ABOUT,                _handleAbout);\n    CommandManager.register(Strings.CMD_HELP_CHECK_FOR_UPDATES,        Commands.HELP_CHECK_FOR_UPDATES,    _handleCheckForUpdates);\n    CommandManager.register(Strings.CMD_HELP_ENTER_LICENSE,            Commands.HELP_ENTER_LICENSE,        _handleEnterLicense);\n    CommandManager.register(Strings.CMD_HELP_DOCUMENTATION,            Commands.HELP_DOCUMENTATION,        _handleDocumentation);\n    CommandManager.register(Strings.CMD_HELP_FORUM,                    Commands.HELP_FORUM,                _handleForum);\n    CommandManager.register(Strings.CMD_HELP_RELEASE_NOTE,             Commands.HELP_RELEASE_NOTE,         _handleReleaseNotes);\n    CommandManager.register(Strings.CMD_HELP_REQUEST_FEATURE,          Commands.HELP_REQUEST_FEATURE,      _handleRequestFeature);\n\n    // Register Internal Commands\n    CommandManager.registerInternal(Commands.FILE_CLOSE_WINDOW, _handleFileCloseWindow);\n    CommandManager.registerInternal(Commands.APP_RELOAD,        _handleReload);\n\n    // Update Commands\n    $(SelectionManager).on("selectionChanged", _updateCommands);\n    $(Repository).on("operationExecuted", _updateCommands);\n\n});\n\n//# sourceURL=/command/DefaultCommandHandlers.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, localStorage*/\ndefine(\'menu/RecentFiles\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'engine/ProjectManager\',\'command/Commands\',\'command/CommandManager\',\'menu/MenuManager\',\'strings\',\'menu/DefaultMenus\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit        = require("utils/AppInit"),\n        ProjectManager = require("engine/ProjectManager"),\n        Commands       = require("command/Commands"),\n        CommandManager = require("command/CommandManager"),\n        MenuManager    = require("menu/MenuManager"),\n        Strings        = require("strings");\n\n    require("menu/DefaultMenus");\n\n    /**\n     * Maxmimum number of recent files\n     * @const\n     */\n    var MAX_SIZE = 10;\n\n    /**\n     * Item Id of localStorage\n     * @const\n     */\n    var STORAGE_ITEM_ID = "staruml-recent-file-list";\n\n    /**\n     * List of Recent Files\n     * @type{Array.<string>}\n     */\n    var _recentFiles = [];\n\n    /**\n     * Return command id for index-th recent files.\n     */\n    function _getCommandId(index) {\n        return Commands.FILE_OPEN_RECENT + "_files_" + index;\n    }\n\n    /**\n     * Add a file to Recent file list.\n     *\n     */\n    function push(filename) {\n        if (filename) {\n\n            // Remove already existing filename to prevent duplicated filenames.\n            var idx = _.indexOf(_recentFiles, filename);\n            if (idx > -1) {\n                _recentFiles.splice(idx, 1);\n            }\n\n            // Add on top.\n            _recentFiles.push(filename);\n        }\n\n        // Keep total number of files less than MAX_SIZE\n        while (_recentFiles.length > MAX_SIZE) {\n            _recentFiles.splice(0, 1);\n        }\n\n        // Save the recent files to localStorage\n        localStorage.setItem(STORAGE_ITEM_ID, JSON.stringify(_recentFiles));\n    }\n\n    /**\n     * Remove a file from Recent file list.\n     */\n    function remove(filename) {\n        _recentFiles.remove(filename);\n\n        // Save the recent files to localStorage\n        localStorage.setItem(STORAGE_ITEM_ID, JSON.stringify(_recentFiles));\n    }\n\n\n    function _updateMenus() {\n        var menu = MenuManager.getMenuItem(Commands.FILE_OPEN_RECENT),\n            i,\n            id,\n            menuItem;\n\n        // Remove Recent File Menus\n        for (i = 0; i < MAX_SIZE; i++) {\n            id = _getCommandId(i);\n            menuItem = MenuManager.getMenuItem(id);\n            if (menuItem) {\n                menu.removeMenuItem(id);\n            }\n        }\n\n        // Add Recent File Menus (Most recent to be in top)\n        for (i = _recentFiles.length - 1; i >= 0; i--) {\n            id = _getCommandId(i);\n            var cmd = CommandManager.get(id);\n            if (cmd) {\n                menu.addMenuItem(id);\n                cmd.setName(_recentFiles[i]);\n            }\n        }\n    }\n\n    AppInit.htmlReady(function () {\n        // Load recent files from localStorage\n        var data = localStorage.getItem(STORAGE_ITEM_ID);\n        if (data) {\n            _recentFiles = JSON.parse(data);\n        }\n\n        // Add Open Recent Menu\n        var menu = MenuManager.getMenu(Commands.FILE);\n        menu.addMenuItem(Commands.FILE_OPEN_RECENT, null, MenuManager.AFTER, Commands.FILE_OPEN);\n\n        // Update Menus\n        _updateMenus();\n    });\n\n    /**\n     * Handle Open Recent Files Command Execution\n     */\n    function _handleOpenRecent(index) {\n        var fullPath = _recentFiles[index];\n        if (fullPath) {\n            CommandManager.execute(Commands.FILE_OPEN, fullPath);\n        }\n    }\n\n    /**\n     * Handles event "projectLoaded" of ProjectManager.\n     * Add to the loaded file to recent file list.\n     */\n    function _handleProjectLoaded(event, filename, project) {\n        try {\n            if (filename && filename.length > 0) {\n                push(filename);\n                _updateMenus();\n            }\n        } catch (err) {\n            console.error(err);\n        }\n    }\n\n    // Register Commands\n    CommandManager.register(Strings.CMD_FILE_OPEN_RECENT, Commands.FILE_OPEN_RECENT, _handleOpenRecent);\n    var i, len, commandID;\n    for (i = 0, len = MAX_SIZE; i < len; i++) {\n        commandID = _getCommandId(i);\n        CommandManager.register("-", commandID, _.partial(_handleOpenRecent, i));\n    }\n\n    // Update Recent Files\n    $(ProjectManager).on("projectLoaded", _handleProjectLoaded);\n\n});\n\n//# sourceURL=/menu/RecentFiles.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, loopfunc: true */\n/*global define, $, _, window, appshell, meta, type */\n\ndefine(\'uml/UML\',[\'require\',\'exports\',\'module\',\'core/Core\',\'core/MetaModelManager\',\'core/Repository\'],function (require, exports, module) {\n    "use strict";\n\n    var Core             = require("core/Core"),\n        MetaModelManager = require("core/MetaModelManager"),\n        Repository       = require("core/Repository");\n\n    /**************************************************************************\n     *                                                                        *\n     *                       CONSTANTS AND ENUMERATIONS                       *\n     *                                                                        *\n     *************************************************************************/\n\n    /**\n     * UMLVisibilityKind\n     * @enum\n     */\n    var VK_PUBLIC    = \'public\',\n        VK_PROTECTED = \'protected\',\n        VK_PRIVATE   = \'private\',\n        VK_PACKAGE   = \'package\';\n\n    /**\n     * UMLAggregationKind\n     * @enum\n     */\n    var AK_NONE      = \'none\',\n        AK_SHARED    = \'shared\',\n        AK_COMPOSITE = \'composite\';\n\n    /**\n     * UMLDirectionKind\n     * @enum\n     */\n    var DK_IN     = \'in\',\n        DK_INOUT  = \'inout\',\n        DK_OUT    = \'out\',\n        DK_RETURN = \'return\';\n\n    /**\n     * UMLCallConcurrencyKind\n     * @enum\n     */\n    var CCK_SEQUENTIAL = \'sequential\',\n        CCK_GUARDED    = \'guarded\',\n        CCK_CONCURRENT = \'concurrent\';\n\n    /**\n     * UMLConnectorKind\n     * @enum\n     */\n    var CK_ASSEMBLY   = \'assembly\',\n        CK_DELEGATION = \'delegation\';\n\n    /**\n     * UMLMessageSort\n     * @enum\n     */\n    var MS_SYNCHCALL     = \'synchCall\',\n        MS_ASYNCHCALL    = \'asynchCall\',\n        MS_ASYNCHSIGNAL  = \'asynchSignal\',\n        MS_CREATEMESSAGE = \'createMessage\',\n        MS_DELETEMESSAGE = \'deleteMessage\',\n        MS_REPLY         = \'reply\';\n\n    /**\n     * UMLPseudostateKind\n     * @enum\n     */\n    var PSK_INITIAL         = \'initial\',\n        PSK_DEEPHISTORY     = \'deepHistory\',\n        PSK_SHALLOWHISTORY  = \'shallowHistory\',\n        PSK_JOIN            = \'join\',\n        PSK_FORK            = \'fork\',\n        PSK_JUNCTION        = \'junction\',\n        PSK_CHOICE          = \'choice\',\n        PSK_ENTRYPOINT      = \'entryPoint\',\n        PSK_EXITPOINT       = \'exitPoint\',\n        PSK_TERMINATE       = \'terminate\';\n\n    /**\n     * UMLTransitionKind\n     * @enum\n     */\n    var TK_EXTERNAL = \'external\',\n        TK_INTERNAL = \'internal\',\n        TK_LOCAL    = \'local\';\n\n    /**\n     * UMLEventKind\n     * @enum\n     */\n    var EK_SIGNAL     = \'signal\',\n        EK_CALL       = \'call\',\n        EK_CHANGE     = \'change\',\n        EK_TIME       = \'time\',\n        EK_ANYRECEIVE = \'anyReceive\';\n\n    /**\n     * UMLInteractionOperatorKind\n     * @enum\n     */\n    var IOK_ALT      = \'alt\',\n        IOK_OPT      = \'opt\',\n        IOK_PAR      = \'par\',\n        IOK_LOOP     = \'loop\',\n        IOK_CRITICAL = \'critical\',\n        IOK_NEG      = \'neg\',\n        IOK_ASSERT   = \'assert\',\n        IOK_STRICT   = \'strict\',\n        IOK_SEQ      = \'seq\',\n        IOK_IGNORE   = \'ignore\',\n        IOK_CONSIDER = \'consider\',\n        IOK_BREAK    = \'break\';\n\n    /**\n     * UMLActionKind\n     * @enum\n     */\n    var ACK_OPAQUE       = "opaque",\n        ACK_CREATE       = "create",\n        ACK_DESTROY      = "destroy",\n        ACK_READ         = "read",\n        ACK_WRITE        = "write",\n        ACK_INSERT       = "insert",\n        ACK_DELETE       = "delete",\n        ACK_SENDSIGNAL   = "sendSignal",\n        ACK_ACCEPTSIGNAL = "acceptSignal",\n        ACK_TRIGGEREVENT = "triggerEvent",\n        ACK_ACCEPTEVENT  = "acceptEvent",\n        ACK_STRUCTURED   = "structured",\n        ACK_TIMEEVENT    = "timeEvent";\n\n    /**\n     * UMLObjectNodeOrderingKind\n     * @enum\n     */\n    var ONOK_UNORDERED = \'unordered\',\n        ONOK_ORDERED   = \'ordered\',\n        ONOK_LIFO      = \'LIFO\',\n        ONOK_FIFO      = \'FIFO\';\n\n    /**\n     * UMLExpansionKind\n     * @enum\n     */\n    var EK_PARALLEL  = \'parallel\',\n        EK_ITERATIVE = \'iterative\',\n        EK_STREAM    = \'stream\';\n\n    /**\n     * Stereotype Display\n     * @enum\n     */\n    var SD_NONE             = \'none\',\n        SD_LABEL            = \'label\',\n        SD_DECORATION       = \'decoration\',\n        SD_DECORATION_LABEL = \'decoration-label\',\n        SD_ICON             = \'icon\',\n        SD_ICON_LABEL       = \'icon-label\';\n\n    /**************************************************************************\n     *                                                                        *\n     *                                 BACKBONE                               *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLElementMixin\n     *\n     * @mixin\n     */\n    var UMLElementMixin = {\n\n        /**\n         * Get class name for display\n         *\n         * @memberof UMLElementMixin\n         * @return {string}\n         */\n        getDisplayClassName: function () {\n            var name = this.getClassName();\n            return name.substring(3, name.length);\n        },\n\n        /**\n         * Get visibility string\n         *\n         * @memberof UMLElementMixin\n         * @return {string}\n         */\n        getVisibilityString: function () {\n            switch (this.visibility) {\n            case VK_PUBLIC:\n                return "+";\n            case VK_PROTECTED:\n                return "#";\n            case VK_PRIVATE:\n                return "-";\n            case VK_PACKAGE:\n                return "~";\n            }\n        },\n\n        /**\n         * Get string representation of this element\n         *\n         * @memberof UMLElementMixin\n         * @param {boolean} includeVisibility\n         * @return {string}\n         */\n        getString: function (includeVisibility) {\n            var _string = this.name;\n            if (includeVisibility) {\n                _string = this.getVisibilityString() + _string;\n            }\n            return _string;\n        },\n\n\n        /**\n         * Get stereotype string\n         *\n         * @memberof UMLElementMixin\n         * @return {string}\n         */\n        getStereotypeString: function () {\n            if (_.isString(this.stereotype) && (this.stereotype.length > 0)) {\n                return "«" + this.stereotype + "»";\n            } else if (this.stereotype instanceof type.Model) {\n                return "«" + this.stereotype.name + "»";\n            }\n            return "";\n        },\n\n        /**\n         * Get namespace string\n         *\n         * @memberof UMLElementMixin\n         * @return {string}\n         */\n        getNamespaceString: function () {\n            if (this._parent) {\n                return "(from " + this._parent.name + ")";\n            } else {\n                return "(from Root)";\n            }\n        },\n\n        /**\n         * Return an array of tag strings\n         *\n         * @memberof UMLElementMixin\n         * @return {Array.<string>}\n         */\n        getTagStringArray: function () {\n            var tagArray = [];\n            if (this.tags && this.tags.length > 0) {\n                var i, len, tag;\n                for (i = 0, len = this.tags.length; i < len; i++) {\n                    tag = this.tags[i];\n                    switch (tag.kind) {\n                    case Core.TK_STRING:\n                        tagArray.push(tag.name + \'="\' + tag.value + \'"\');\n                        break;\n                    case Core.TK_REFERENCE:\n                        if (tag.reference instanceof Core.Model) {\n                            tagArray.push(tag.name + \'=\' + tag.reference.name);\n                        } else {\n                            tagArray.push(tag.name + \'=null\');\n                        }\n                        break;\n                    case Core.TK_BOOLEAN:\n                        tagArray.push(tag.name + \'=\' + tag.checked);\n                        break;\n                    case Core.TK_NUMBER:\n                        tagArray.push(tag.name + \'=\' + tag.number);\n                        break;\n                    // TK_HIDDEN is not shown in Diagram.\n                    }\n                }\n            }\n            return tagArray;\n        },\n\n        /**\n         * Return property string\n         *\n         * @return {string}\n         */\n        getPropertyString: function () {\n            var props = this.getTagStringArray();\n            if (props.length > 0) {\n                return "{" + props.join(", ") + "}";\n            }\n            return "";\n        }\n\n    };\n\n    /**\n     * UMLModelElement\n     *\n     * @constructor\n     * @extends ExtensibleModel\n     * @mixes UMLElementMixin\n     */\n    function UMLModelElement() {\n        type.ExtensibleModel.apply(this, arguments);\n\n        /** @member {UMLStereotype} */\n        this.stereotype = null;\n\n        /** @member {string} */\n        this.visibility = VK_PUBLIC;\n\n        /** @member {Array.<UMLTemplateParameter>} */\n        this.templateParameters = [];\n\n        // mixin UMLElementMixin\n        _.extend(UMLModelElement.prototype, UMLElementMixin);\n    }\n    // inherits from Model\n    UMLModelElement.prototype = Object.create(type.ExtensibleModel.prototype);\n    UMLModelElement.prototype.constructor = UMLModelElement;\n\n    UMLModelElement.prototype.getNodeText = function (options) {\n        var text = "";\n        options = options || {};\n        // stereotype\n        if (options.showStereotype !== false) {\n            text += this.getStereotypeString();\n        }\n        // name and type\n        if (this.name && this.name.length > 0) {\n            text += this.name;\n        } else {\n            text += "(" + this.getDisplayClassName() + ")";\n        }\n        return text;\n    };\n\n    UMLModelElement.prototype.getNodeIcon = function () {\n        if (this.stereotype && this.stereotype.icon && this.stereotype.icon.smallIcon && this.stereotype.icon.smallIcon.trim().length > 0) {\n            return this.stereotype.getIconClass();\n        }\n        return type.ExtensibleModel.prototype.getNodeIcon.call(this);\n    };\n\n    /**\n     * Get dependencies\n     *\n     * @return {Array.<Element>}\n     */\n    UMLModelElement.prototype.getDependencies = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLDependency) && (r.source === self);\n            });\n        return _.map(rels, function (g) { return g.target; });\n    };\n\n    /**\n     * Get dependants\n     *\n     * @return {Array.<Element>}\n     */\n    UMLModelElement.prototype.getDependants = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLDependency) && (r.target === self);\n            });\n        return _.map(rels, function (g) { return g.source; });\n    };\n\n    /**\n     * Get constraints of this element\n     *\n     * @return {Array.<UMLConstraint>}\n     */\n    UMLModelElement.prototype.getConstraints = function () {\n        return _.filter(this.ownedElements, function (e) { return (e instanceof type.UMLConstraint); });\n    };\n\n    /**\n     * UMLConstraint\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLConstraint() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {string} */\n        this.specification = \'\';\n\n        /** @member {Array.<UMLModelElement>} */\n        this.constrainedElements = [];\n    }\n    // inherits from UMLModelElement\n    UMLConstraint.prototype = Object.create(UMLModelElement.prototype);\n    UMLConstraint.prototype.constructor = UMLConstraint;\n\n    UMLConstraint.prototype.getNodeText = function (options) {\n        var text = "";\n        options = options || {};\n        // stereotype\n        if (options.showStereotype !== false) {\n            if (_.isString(this.stereotype) && (this.stereotype.length > 0)) {\n                text += "«" + this.stereotype + "» ";\n            } else if (this.stereotype !== null) {\n                text += "«" + this.stereotype.name + "» ";\n            }\n        }\n        // name and type\n        if (this.name && this.name.length > 0) {\n            text += this.name;\n        } else {\n            text += "(" + this.getDisplayClassName() + ")";\n        }\n        // collection name\n        var collection = this.getParentField();\n        if (collection !== "ownedElements") {\n            text += " <@" + collection + ">";\n        }\n        return text;\n    };\n\n\n    /**\n     * UMLTemplateParameter\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLTemplateParameter() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {string} */\n        this.parameterType = \'\';\n\n        /** @member {string} */\n        this.defaultValue = \'\';\n    }\n    // inherits from UMLModelElement\n    UMLTemplateParameter.prototype = Object.create(UMLModelElement.prototype);\n    UMLTemplateParameter.prototype.constructor = UMLTemplateParameter;\n\n    UMLTemplateParameter.prototype.getString = function (options) {\n        var text = "";\n        text += this.getStereotypeString();\n        text += this.name;\n        if (options && options.showType) {\n            text += (this.parameterType.length > 0 ? ": " + this.parameterType : "");\n        }\n        text += (this.defaultValue.length > 0 ? " = " + this.defaultValue : "");\n        return text;\n    };\n\n\n    /**\n     * UMLFeature\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLFeature() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.isStatic = false;\n\n        /** @member {boolean} */\n        this.isLeaf = false;\n    }\n    // inherits from UMLModelElement\n    UMLFeature.prototype = Object.create(UMLModelElement.prototype);\n    UMLFeature.prototype.constructor = UMLFeature;\n\n\n    /**\n     * UMLStructuralFeature\n     * @constructor\n     * @extends UMLFeature\n     */\n    function UMLStructuralFeature() {\n        UMLFeature.apply(this, arguments);\n\n        /** @member {string} */\n        this.type = \'\';\n\n        /** @member {string} */\n        this.multiplicity = \'\';\n\n        /** @member {boolean} */\n        this.isReadOnly = false;\n\n        /** @member {boolean} */\n        this.isOrdered = false;\n\n        /** @member {boolean} */\n        this.isUnique = false;\n\n        /** @member {string} */\n        this.defaultValue = \'\';\n    }\n    // inherits from UMLFeature\n    UMLStructuralFeature.prototype = Object.create(UMLFeature.prototype);\n    UMLStructuralFeature.prototype.constructor = UMLStructuralFeature;\n\n    /**\n     * Get type string\n     *\n     * @return {string}\n     */\n    UMLStructuralFeature.prototype.getTypeString = function () {\n        if (this.type) {\n            if (_.isString(this.type) && (this.type.length > 0)) {\n                return this.type;\n            } else if ((this.type !== null) && (this.type.name)) {\n                return this.type.name;\n            }\n        }\n        return null;\n    };\n\n    UMLStructuralFeature.prototype.getPropertyString = function () {\n        var props = [];\n        if (this.isReadOnly === true) { props.push("readOnly"); }\n        if (this.isOrdered === true) { props.push("ordered"); }\n        if (this.isUnique === true) { props.push("unique"); }\n        props = _.union(props, this.getTagStringArray());\n        if (props.length > 0) {\n            return "{" + props.join(", ") + "}";\n        }\n        return "";\n    };\n\n    UMLStructuralFeature.prototype.getString = function (options) {\n        var text = "";\n        text += this.name;\n        if (options && options.showType) {\n            text += (this.getTypeString() !== null ? ": " + this.getTypeString() : "");\n        }\n        if (options && options.showMultiplicity) {\n            text += (this.multiplicity.length > 0 ? "[" + this.multiplicity + "]" : "");\n        }\n        text += (this.defaultValue.length > 0 ? " = " + this.defaultValue : "");\n        if (options && options.showProperty) {\n            var prop = this.getPropertyString();\n            text += (prop.length > 0 ? " " + prop : "");\n        }\n        return text;\n    };\n\n    /**\n     * UMLParameter\n     * @constructor\n     * @extends UMLStructuralFeature\n     */\n    function UMLParameter() {\n        UMLStructuralFeature.apply(this, arguments);\n\n        /** @member {string} */\n        this.direction = DK_IN;\n    }\n    // inherits from UMLStructuralFeature\n    UMLParameter.prototype = Object.create(UMLStructuralFeature.prototype);\n    UMLParameter.prototype.constructor = UMLParameter;\n\n    /**\n     * Get parameter direction string\n     *\n     * @return {string}\n     */\n    UMLParameter.prototype.getDirectionString = function (options) {\n        switch (this.direction) {\n        case DK_IN:\n            return "";\n        case DK_INOUT:\n            return "inout ";\n        case DK_OUT:\n            return "out ";\n        case DK_RETURN:\n            return "";\n        }\n    };\n\n    UMLParameter.prototype.getString = function (options) {\n        var text = "";\n        text += this.getDirectionString();\n        text += this.name;\n        if (options && options.showType) {\n            text += (this.getTypeString() !== null ? ": " + this.getTypeString() : "");\n        }\n        if (options && options.showMultiplicity) {\n            text += (this.multiplicity.length > 0 ? "[" + this.multiplicity + "]" : "");\n        }\n        text += (this.defaultValue.length > 0 ? " = " + this.defaultValue : "");\n        return text;\n    };\n\n\n    /**\n     * UMLBehavioralFeature\n     * @constructor\n     * @extends UMLFeature\n     */\n    function UMLBehavioralFeature() {\n        UMLFeature.apply(this, arguments);\n\n        /** @member {Array.<UMLParameter>} */\n        this.parameters = [];\n\n        /** @member {Array.<UMLSignal>} */\n        this.raisedExceptions = [];\n\n        /** @member {string} */\n        this.concurrency = CCK_SEQUENTIAL;\n    }\n    // inherits from UMLFeature\n    UMLBehavioralFeature.prototype = Object.create(UMLFeature.prototype);\n    UMLBehavioralFeature.prototype.constructor = UMLBehavioralFeature;\n\n    /**\n     * Get return parameter(s)\n     *\n     * @return {UMLParameter}\n     */\n    UMLBehavioralFeature.prototype.getReturnParameter = function () {\n        var i, len;\n        for (i = 0, len = this.parameters.length; i < len; i++) {\n            var param = this.parameters[i];\n            if (param.direction === DK_RETURN) {\n                return param;\n            }\n        }\n        return null;\n    };\n\n    /**\n     * Get non-return parameters\n     *\n     * @return {Array.<UMLParameter>}\n     */\n    UMLBehavioralFeature.prototype.getNonReturnParameters = function () {\n        var i, len, params = [];\n        for (i = 0, len = this.parameters.length; i < len; i++) {\n            var param = this.parameters[i];\n            if (param.direction !== DK_RETURN) {\n                params.push(param);\n            }\n        }\n        return params;\n    };\n\n    /**\n     * Get parameters string\n     *\n     * @return {string}\n     */\n    UMLBehavioralFeature.prototype.getParametersString = function (options) {\n        var i, len, terms = [],\n            params = this.getNonReturnParameters();\n        for (i = 0, len = params.length; i < len; i++) {\n            var param = params[i];\n            terms.push(param.getString(options));\n        }\n        return "(" + terms.join(", ") + ")";\n    };\n\n    /**\n     * Get return parameter string\n     *\n     * @return {string}\n     */\n    UMLBehavioralFeature.prototype.getReturnString = function (options) {\n        var returnParam = this.getReturnParameter();\n        var text = "";\n        if (returnParam) {\n            if (options && options.showType) {\n                text += returnParam.getTypeString();\n            }\n            if (options && options.showMultiplicity) {\n                text += (returnParam.multiplicity.length > 0 ? "[" + returnParam.multiplicity + "]" : "");\n            }\n        }\n        return text;\n    };\n\n    UMLBehavioralFeature.prototype.getString = function (options) {\n        var text = "";\n        if (options && (options.stereotypeDisplay === SD_LABEL || options.stereotypeDisplay === SD_DECORATION_LABEL || options.stereotypeDisplay === SD_ICON_LABEL)) {\n            text += this.getStereotypeString();\n        }\n        if (options && options.showVisibility) {\n            text += this.getVisibilityString();\n        }\n        text += this.name;\n        if (options && options.showOperationSignature) {\n            text += this.getParametersString(options);\n            text += (this.getReturnString(options).length > 0 ? ": " + this.getReturnString(options) : "");\n        } else {\n            text += "()";\n        }\n        if (options && options.showProperty) {\n            var prop = this.getPropertyString();\n            text += (prop.length > 0 ? " " + prop : "");\n        }\n        return text;\n    };\n\n\n    /**\n     * UMLAttribute\n     * @constructor\n     * @extends UMLStructuralFeature\n     */\n    function UMLAttribute() {\n        UMLStructuralFeature.apply(this, arguments);\n\n        /** @member {string} */\n        this.isDerived = false;\n\n        /** @member {string} */\n        this.aggregation = AK_NONE;\n\n        /** @member {boolean} */\n        this.isID = false;\n    }\n    // inherits from UMLStructuralFeature\n    UMLAttribute.prototype = Object.create(UMLStructuralFeature.prototype);\n    UMLAttribute.prototype.constructor = UMLAttribute;\n\n    UMLAttribute.prototype.getPropertyString = function () {\n        var props = [];\n        if (this.isID === true) { props.push("id"); }\n        if (this.isReadOnly === true) { props.push("readOnly"); }\n        if (this.isOrdered === true) { props.push("ordered"); }\n        if (this.isUnique === true) { props.push("unique"); }\n        props = _.union(props, this.getTagStringArray());\n        if (props.length > 0) {\n            return "{" + props.join(", ") + "}";\n        }\n        return "";\n    };\n\n    UMLAttribute.prototype.getString = function (options) {\n        var text = "";\n        if (options && (options.stereotypeDisplay === SD_LABEL || options.stereotypeDisplay === SD_DECORATION_LABEL || options.stereotypeDisplay === SD_ICON_LABEL)) {\n            text += this.getStereotypeString();\n        }\n        if (options && options.showVisibility) {\n            text += this.getVisibilityString();\n        }\n        text += (this.isDerived === true ? "/" : "");\n        text += this.name;\n        if (options && options.showType) {\n            text += (this.getTypeString() !== null ? ": " + this.getTypeString() : "");\n        }\n        if (options && options.showMultiplicity) {\n            text += (this.multiplicity.length > 0 ? "[" + this.multiplicity + "]" : "");\n        }\n        text += (this.defaultValue.length > 0 ? " = " + this.defaultValue : "");\n        if (options && options.showProperty) {\n            var prop = this.getPropertyString();\n            text += (prop.length > 0 ? " " + prop : "");\n        }\n        return text;\n    };\n\n\n    /**\n     * UMLOperation\n     * @constructor\n     * @extends UMLBehavioralFeature\n     */\n    function UMLOperation() {\n        UMLBehavioralFeature.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.isQuery = false;\n\n        /** @member {boolean} */\n        this.isAbstract = false;\n\n        /** @member {string} */\n        this.specification = \'\';\n\n        /** @member {Array.<UMLConstraint>} */\n        this.preconditions = [];\n\n        /** @member {Array.<UMLConstraint>} */\n        this.bodyConditions = [];\n\n        /** @member {Array.<UMLConstraint>} */\n        this.postconditions = [];\n    }\n    // inherits from UMLBehavioralFeature\n    UMLOperation.prototype = Object.create(UMLBehavioralFeature.prototype);\n    UMLOperation.prototype.constructor = UMLOperation;\n\n    UMLOperation.prototype.getPropertyString = function () {\n        var props = [];\n        if (this.isQuery === true) { props.push("query"); }\n        var returnParam = this.getReturnParameter();\n        if (returnParam) {\n            if (returnParam.isOrdered === true) { props.push("ordered"); }\n            if (returnParam.isUnique === true) { props.push("unique"); }\n        }\n        props = _.union(props, this.getTagStringArray());\n        if (props.length > 0) {\n            return "{" + props.join(", ") + "}";\n        }\n        return "";\n    };\n\n    /**\n     * UMLReception\n     * @constructor\n     * @extends UMLBehavioralFeature\n     */\n    function UMLReception() {\n        UMLBehavioralFeature.apply(this, arguments);\n\n        /** @member {UMLSignal} */\n        this.signal = null;\n    }\n    // inherits from UMLBehavioralFeature\n    UMLReception.prototype = Object.create(UMLBehavioralFeature.prototype);\n    UMLReception.prototype.constructor = UMLReception;\n\n    UMLReception.prototype.getNodeText = function (options) {\n        var text = "";\n        if (this.signal instanceof type.UMLSignal) {\n            return "(«signal» " + this.signal.name + ")";\n        }\n        return UMLModelElement.prototype.getNodeText.call(this);\n    };\n\n    UMLReception.prototype.getString = function (options) {\n        if (this.signal instanceof UMLSignal) {\n            var text = "";\n            if (options && (options.stereotypeDisplay === SD_LABEL || options.stereotypeDisplay === SD_DECORATION_LABEL || options.stereotypeDisplay === SD_ICON_LABEL)) {\n                text += "«signal»";\n            }\n            if (options && options.showVisibility) {\n                text += this.getVisibilityString();\n            }\n            text += this.signal.name;\n            if (options && options.showOperationSignature) {\n                var i, len, attr, term, _type, terms = [];\n                for (i = 0, len = this.signal.attributes.length; i < len; i++) {\n                    attr = this.signal.attributes[i];\n                    term = attr.name;\n                    if (options.showType) {\n                        _type = attr.getTypeString();\n                        if (_type) {\n                            term += ": " + attr.getTypeString();\n                        }\n                    }\n                    terms.push(term);\n                }\n                text += "(" + terms.join(", ") + ")";\n            } else {\n                text += "()";\n            }\n            if (options && options.showProperty) {\n                var prop = this.getPropertyString();\n                text += (prop.length > 0 ? " " + prop : "");\n            }\n            return text;\n        } else {\n            return UMLBehavioralFeature.prototype.getString.call(this, options);\n        }\n    };\n\n    /**\n     * UMLClassifier\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLClassifier() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {Array.<UMLAttribute>} */\n        this.attributes = [];\n\n        /** @member {Array.<UMLOperation>} */\n        this.operations = [];\n\n        /** @member {Array.<UMLReception>} */\n        this.receptions = [];\n\n        /** @member {Array.<UMLBehavior>} */\n        this.behaviors = [];\n\n        /** @member {boolean} */\n        this.isAbstract = false;\n\n        /** @member {boolean} */\n        this.isFinalSpecialization = false;\n\n        /** @member {boolean} */\n        this.isLeaf = false;\n    }\n    // inherits from UMLModelElement\n    UMLClassifier.prototype = Object.create(UMLModelElement.prototype);\n    UMLClassifier.prototype.constructor = UMLClassifier;\n\n    UMLClassifier.prototype.getPropertyString = function () {\n        var props = [];\n        if (this.isLeaf === true) { props.push("leaf"); }\n        props = _.union(props, this.getTagStringArray());\n        if (props.length > 0) {\n            return "{" + props.join(", ") + "}";\n        }\n        return "";\n    };\n\n    /**\n     * Get general elements\n     *\n     * @param {boolean} includeInterfaces\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.getGeneralElements = function (includeInterfaces) {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return ((r instanceof type.UMLGeneralization) && (r.source === self)) ||\n                       (includeInterfaces && (r instanceof type.UMLInterfaceRealization) && (r.source === self));\n            });\n        return _.map(rels, function (g) { return g.target; });\n    };\n\n    /**\n     * Get special elements\n     *\n     * @param {boolean} includeInterfaces\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.getSpecialElements = function (includeInterfaces) {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return ((r instanceof type.UMLGeneralization) && (r.target === self)) ||\n                       (includeInterfaces && (r instanceof type.UMLInterfaceRealization) && (r.target === self));\n            });\n        return _.map(rels, function (g) { return g.source; });\n    };\n\n    /**\n     * Get ancestors\n     *\n     * @param {boolean} includeInterfaces\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.getAncestors = function (includeInterfaces) {\n        var ancestors = this.getGeneralElements(includeInterfaces),\n            size = 0;\n        do {\n            size = ancestors.length;\n            _.each(ancestors, function (e) {\n                ancestors = _.union(ancestors, e.getGeneralElements(includeInterfaces));\n            });\n        } while (size < ancestors.length);\n        return ancestors;\n    };\n\n    /**\n     * Get descendants\n     *\n     * @param {boolean} includeInterfaces\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.getDescendants = function (includeInterfaces) {\n        var descendants = this.getSpecialElements(includeInterfaces),\n            size = 0;\n        do {\n            size = descendants.length;\n            _.each(descendants, function (e) {\n                descendants = _.union(descendants, e.getSpecialElements(includeInterfaces));\n            });\n        } while (size < descendants.length);\n        return descendants;\n    };\n\n    /**\n     * Check a given element is a general element of this element\n     *\n     * @param {Element} elem\n     * @param {boolean} includeInterfaces\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.isGeneralElement = function (elem, includeInterfaces) {\n        return _.contains(this.getGeneralElements(includeInterfaces), elem);\n    };\n\n    /**\n     * Check a given element is a special element of this element\n     *\n     * @param {Element} elem\n     * @param {boolean} includeInterfaces\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.isSpecialElement = function (elem, includeInterfaces) {\n        return _.contains(this.getSpecialElements(includeInterfaces), elem);\n    };\n\n    /**\n     * Check a given element is an ancestor of this element\n     *\n     * @param {Element} elem\n     * @param {boolean} includeInterfaces\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.isAncestor = function (elem, includeInterfaces) {\n        return _.contains(this.getAncestors(includeInterfaces), elem);\n    };\n\n    /**\n     * Check a given element is a descendant of this element\n     *\n     * @param {Element} elem\n     * @param {boolean} includeInterfaces\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.isDescendant = function (elem, includeInterfaces) {\n        return _.contains(this.getDescendants(includeInterfaces), elem);\n    };\n\n    /**\n     * Get all inherited attributes\n     *\n     * @param {boolean} includeInterfaces\n     * @return {Array.<UMLAttribute>}\n     */\n    UMLClassifier.prototype.getInheritedAttributes = function (includeInterfaces) {\n        var ancestors = this.getAncestors(includeInterfaces),\n            inherited = [];\n        _.each(ancestors, function (e) {\n            if (Array.isArray(e.attributes)) {\n                Array.prototype.push.apply(inherited, e.attributes);\n            }\n        });\n        return inherited;\n    };\n\n    /**\n     * Get all inherited operations\n     *\n     * @param {boolean} includeInterfaces\n     * @return {Array.<UMLOperation>}\n     */\n    UMLClassifier.prototype.getInheritedOperations = function (includeInterfaces) {\n        var ancestors  = this.getAncestors(includeInterfaces),\n            interfaces = this.getInterfaces(),\n            inherited  = [];\n        _.each(ancestors, function (e) {\n            if (Array.isArray(e.operations)) {\n                Array.prototype.push.apply(inherited, e.operations);\n            }\n        });\n        if (includeInterfaces) {\n            _.each(interfaces, function (e) {\n                if (Array.isArray(e.operations)) {\n                    Array.prototype.push.apply(inherited, e.getInheritedOperations(includeInterfaces));\n                    Array.prototype.push.apply(inherited, e.operations);\n                }\n            });\n        }\n        return inherited;\n    };\n\n    /**\n     * Get all interfaces of this element is realizing\n     *\n     * @return {Array.<UMLInterface>}\n     */\n    UMLClassifier.prototype.getInterfaces = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLInterfaceRealization) && (r.source === self);\n            });\n        return _.map(rels, function (g) { return g.target; });\n    };\n\n    /**\n     * Get all components of this element is realizing\n     *\n     * @return {Array.<UMLComponent>}\n     */\n    UMLClassifier.prototype.getComponents = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLComponentRealization) && (r.source === self);\n            });\n        return _.map(rels, function (g) { return g.target; });\n    };\n\n    /**\n     * Get all interfaces of this element is realizing\n     *\n     * @return {Array.<Element>}\n     */\n    UMLClassifier.prototype.getDeploymentTargets = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLDeployment) && (r.source === self);\n            });\n        return _.map(rels, function (g) { return g.target; });\n    };\n\n    /**\n     * Get all association ends linked to this element\n     *\n     * @param {boolean} counterpart Returns whether counterpart (opposite-side) association ends or not.\n     * @return {Array.<UMLAssociationEnd>}\n     */\n    UMLClassifier.prototype.getAssociationEnds = function (counterpart) {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) { return (r instanceof type.UMLAssociation); }),\n            ends = _.map(rels, function (r) {\n                if (counterpart === true) {\n                    return (r.end1.reference === self ? r.end2 : r.end1);\n                } else {\n                    return (r.end1.reference === self ? r.end1 : r.end2);\n                }\n            });\n        return ends;\n    };\n\n    /**\n     * UMLDirectedRelationship\n     * @constructor\n     * @extends DirectedRelationship\n     * @mixes UMLElementMixin\n     */\n    function UMLDirectedRelationship() {\n        type.DirectedRelationship.apply(this, arguments);\n\n        /** @member {UMLStereotype} */\n        this.stereotype = null;\n\n        /** @member {string} */\n        this.visibility = VK_PUBLIC;\n        // mixin UMLElementMixin\n        _.extend(UMLDirectedRelationship.prototype, UMLElementMixin);\n    }\n    // inherits from DirectedRelationship\n    UMLDirectedRelationship.prototype = Object.create(type.DirectedRelationship.prototype);\n    UMLDirectedRelationship.prototype.constructor = UMLDirectedRelationship;\n\n\n    /**\n     * UMLRelationshipEnd\n     * @constructor\n     * @extends RelationshipEnd\n     * @mixes UMLElementMixin\n     */\n    function UMLRelationshipEnd() {\n        type.RelationshipEnd.apply(this, arguments);\n\n        /** @member {UMLStereotype} */\n        this.stereotype = null;\n\n        /** @member {string} */\n        this.visibility = VK_PUBLIC;\n\n        /** @member {boolean} */\n        this.navigable = true;\n\n        /** @member {string} */\n        this.aggregation = AK_NONE;\n\n        /** @member {string} */\n        this.multiplicity = \'\';\n\n        /** @member {string} */\n        this.defaultValue = \'\';\n\n        /** @member {boolean} */\n        this.isReadOnly = false;\n\n        /** @member {boolean} */\n        this.isOrdered = false;\n\n        /** @member {boolean} */\n        this.isUnique = false;\n\n        /** @member {boolean} */\n        this.isDerived = false;\n\n        /** @member {boolean} */\n        this.isID = false;\n\n        // mixin UMLElementMixin\n        _.extend(UMLRelationshipEnd.prototype, UMLElementMixin);\n    }\n    // inherits from RelationshipEnd\n    UMLRelationshipEnd.prototype = Object.create(type.RelationshipEnd.prototype);\n    UMLRelationshipEnd.prototype.constructor = UMLRelationshipEnd;\n\n    UMLRelationshipEnd.prototype.getPropertyString = function () {\n        var props = [];\n        if (this.isID === true) { props.push("id"); }\n        if (this.isReadOnly === true) { props.push("readOnly"); }\n        if (this.isOrdered === true) { props.push("ordered"); }\n        if (this.isUnique === true) { props.push("unique"); }\n        props = _.union(props, this.getTagStringArray());\n        if (props.length > 0) {\n            return "{" + props.join(", ") + "}";\n        }\n        return "";\n    };\n\n\n    /**\n     * UMLUndirectedRelationship\n     * @constructor\n     * @extends UndirectedRelationship\n     * @mixes UMLElementMixin\n     */\n    function UMLUndirectedRelationship() {\n        type.UndirectedRelationship.apply(this, arguments);\n\n        /** @member {UMLStereotype} */\n        this.stereotype = null;\n\n        /** @member {string} */\n        this.visibility = VK_PUBLIC;\n\n        // mixin UMLElementMixin\n        _.extend(UMLUndirectedRelationship.prototype, UMLElementMixin);\n    }\n    // inherits from UndirectedRelationship\n    UMLUndirectedRelationship.prototype = Object.create(type.UndirectedRelationship.prototype);\n    UMLUndirectedRelationship.prototype.constructor = UMLUndirectedRelationship;\n\n    UMLUndirectedRelationship.prototype.getNodeText = function (options) {\n        var text = "";\n        options = options || {};\n        // stereotype\n        if (options.showStereotype !== false) {\n            text += this.getStereotypeString();\n        }\n        // name and type\n        if (this.name && this.name.length > 0) {\n            text += this.name + " ";\n        }\n        // end1 and end2\n        text += "(" + this.end1.reference.name + "—" + this.end2.reference.name + ")";\n        return text.trim();\n    };\n\n\n    /**************************************************************************\n     *                                                                        *\n     *                           COMMON BEHAVIORS                             *\n     *                                                                        *\n     **************************************************************************/\n\n\n    /**\n     * UMLBehavior\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLBehavior() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.isReentrant = true;\n\n        /** @member {string} */\n        this.parameters = [];\n\n        /** @member {string} */\n        this.preconditions = [];\n\n        /** @member {string} */\n        this.postconditions = [];\n    }\n    // inherits from UMLModelElement\n    UMLBehavior.prototype = Object.create(UMLModelElement.prototype);\n    UMLBehavior.prototype.constructor = UMLBehavior;\n\n\n    /**\n     * UMLOpaqueBehavior\n     * @constructor\n     * @extends UMLBehavior\n     */\n    function UMLOpaqueBehavior() {\n        UMLBehavior.apply(this, arguments);\n    }\n    // inherits from UMLBehavior\n    UMLOpaqueBehavior.prototype = Object.create(UMLBehavior.prototype);\n    UMLOpaqueBehavior.prototype.constructor = UMLOpaqueBehavior;\n\n\n    /**\n     * UMLEvent\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLEvent() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {string} */\n        this.kind = EK_ANYRECEIVE;\n\n        /** @member {string} */\n        this.value = \'\';\n\n        /** @member {string} */\n        this.expression = \'\';\n\n        /** @member {UMLOperation} */\n        this.targetOperation = null;\n\n        /** @member {UMLSignal} */\n        this.targetSignal = null;\n    }\n    // inherits from UMLModelElement\n    UMLEvent.prototype = Object.create(UMLModelElement.prototype);\n    UMLEvent.prototype.constructor = UMLEvent;\n\n    /**************************************************************************\n     *                                                                        *\n     *                                  CLASSES                               *\n     *                                                                        *\n     **************************************************************************/\n\n\n    /**\n     * UMLPackage\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLPackage() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {Array.<UMLModelElement>} */\n        this.importedElements = [];\n    }\n    // inherits from UMLModelElement\n    UMLPackage.prototype = Object.create(UMLModelElement.prototype);\n    UMLPackage.prototype.constructor = UMLPackage;\n\n    UMLPackage.prototype.canContainDiagramKind = function (kind) {\n        return (kind === "UMLClassDiagram") ||\n               (kind === "UMLPackageDiagram") ||\n               (kind === "UMLObjectDiagram") ||\n               (kind === "UMLCompositeStructureDiagram") ||\n               (kind === "UMLComponentDiagram") ||\n               (kind === "UMLDeploymentDiagram") ||\n               (kind === "UMLUseCaseDiagram");\n    };\n\n    UMLPackage.prototype.canContainKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLClassifier") ||\n               MetaModelManager.isKindOf(kind, "UMLPackage") ||\n               MetaModelManager.isKindOf(kind, "UMLInstance");\n    };\n\n    /**\n     * UMLModel\n     * @constructor\n     * @extends UMLPackage\n     */\n    function UMLModel() {\n        UMLPackage.apply(this, arguments);\n\n        /** @member {string} */\n        this.viewpoint = \'\';\n    }\n    // inherits from UMLPackage\n    UMLModel.prototype = Object.create(UMLPackage.prototype);\n    UMLModel.prototype.constructor = UMLModel;\n\n\n    /**\n     * UMLClass\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLClass() {\n        UMLClassifier.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.isActive = false;\n    }\n    // inherits from UMLClassifier\n    UMLClass.prototype = Object.create(UMLClassifier.prototype);\n    UMLClass.prototype.constructor = UMLClass;\n\n\n    /**\n     * UMLInterface\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLInterface() {\n        UMLClassifier.apply(this, arguments);\n    }\n    // inherits from UMLClassifier\n    UMLInterface.prototype = Object.create(UMLClassifier.prototype);\n    UMLInterface.prototype.constructor = UMLInterface;\n\n    /**\n     * Get all implementing classifiers of this interfaces\n     *\n     * @return {Array.<UMLClassifier>}\n     */\n    UMLInterface.prototype.getImplementingClassifiers = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLInterfaceRealization) && (r.target === self);\n            });\n        return _.map(rels, function (g) { return g.source; });\n    };\n\n    /**\n     * UMLSignal\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLSignal() {\n        UMLClassifier.apply(this, arguments);\n    }\n    // inherits from UMLClassifier\n    UMLSignal.prototype = Object.create(UMLClassifier.prototype);\n    UMLSignal.prototype.constructor = UMLSignal;\n\n    /**\n     * UMLDataType\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLDataType() {\n        UMLClassifier.apply(this, arguments);\n    }\n    // inherits from UMLClassifier\n    UMLDataType.prototype = Object.create(UMLClassifier.prototype);\n    UMLDataType.prototype.constructor = UMLDataType;\n\n    /**\n     * UMLPrimitiveType\n     * @constructor\n     * @extends UMLDataType\n     */\n    function UMLPrimitiveType() {\n        UMLDataType.apply(this, arguments);\n    }\n    // inherits from UMLDataType\n    UMLPrimitiveType.prototype = Object.create(UMLDataType.prototype);\n    UMLPrimitiveType.prototype.constructor = UMLPrimitiveType;\n\n    /**\n     * UMLEnumerationLiteral\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLEnumerationLiteral() {\n        UMLModelElement.apply(this, arguments);\n    }\n    // inherits from UMLModelElement\n    UMLEnumerationLiteral.prototype = Object.create(UMLModelElement.prototype);\n    UMLEnumerationLiteral.prototype.constructor = UMLEnumerationLiteral;\n\n    UMLEnumerationLiteral.prototype.getString = function (options) {\n        var text = "";\n        if (options && (options.stereotypeDisplay === SD_LABEL || options.stereotypeDisplay === SD_DECORATION_LABEL || options.stereotypeDisplay === SD_ICON_LABEL)) {\n            text += this.getStereotypeString();\n        }\n        text += this.name;\n        if (options && options.showProperty) {\n            var prop = this.getPropertyString();\n            text += (prop.length > 0 ? " " + prop : "");\n        }\n        return text;\n    };\n\n\n    /**\n     * UMLEnumeration\n     * @constructor\n     * @extends UMLDataType\n     */\n    function UMLEnumeration() {\n        UMLDataType.apply(this, arguments);\n\n        /** @member {Array.<UMLEnumerationLiteral>} */\n        this.literals = [];\n    }\n    // inherits from UMLDataType\n    UMLEnumeration.prototype = Object.create(UMLDataType.prototype);\n    UMLEnumeration.prototype.constructor = UMLEnumeration;\n\n    /**\n     * UMLDependency\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLDependency() {\n        UMLDirectedRelationship.apply(this, arguments);\n\n        /** @member {string} */\n        this.mapping = \'\';\n    }\n    // inherits from UMLDirectedRelationship\n    UMLDependency.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLDependency.prototype.constructor = UMLDependency;\n\n\n    /**\n     * UMLAbstraction\n     * @constructor\n     * @extends UMLDependency\n     */\n    function UMLAbstraction() {\n        UMLDependency.apply(this, arguments);\n    }\n    // inherits from UMLDependency\n    UMLAbstraction.prototype = Object.create(UMLDependency.prototype);\n    UMLAbstraction.prototype.constructor = UMLAbstraction;\n\n\n    /**\n     * UMLRealization\n     * @constructor\n     * @extends UMLAbstraction\n     */\n    function UMLRealization() {\n        UMLAbstraction.apply(this, arguments);\n    }\n    // inherits from UMLAbstraction\n    UMLRealization.prototype = Object.create(UMLAbstraction.prototype);\n    UMLRealization.prototype.constructor = UMLRealization;\n\n\n    /**\n     * UMLGeneralization\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLGeneralization() {\n        UMLDirectedRelationship.apply(this, arguments);\n\n        /** @member {string} */\n        this.discriminator = \'\';\n    }\n    // inherits from UMLDirectedRelationship\n    UMLGeneralization.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLGeneralization.prototype.constructor = UMLGeneralization;\n\n\n    /**\n     * UMLInterfaceRealization\n     * @constructor\n     * @extends UMLRealization\n     */\n    function UMLInterfaceRealization() {\n        UMLRealization.apply(this, arguments);\n    }\n    // inherits from UMLRealization\n    UMLInterfaceRealization.prototype = Object.create(UMLRealization.prototype);\n    UMLInterfaceRealization.prototype.constructor = UMLInterfaceRealization;\n\n\n    /**\n     * UMLComponentRealization\n     * @constructor\n     * @extends UMLRealization\n     */\n    function UMLComponentRealization() {\n        UMLRealization.apply(this, arguments);\n    }\n    // inherits from UMLRealization\n    UMLComponentRealization.prototype = Object.create(UMLRealization.prototype);\n    UMLComponentRealization.prototype.constructor = UMLComponentRealization;\n\n\n    /**\n     * UMLAssociationEnd\n     * @constructor\n     * @extends UMLRelationshipEnd\n     */\n    function UMLAssociationEnd() {\n        UMLRelationshipEnd.apply(this, arguments);\n\n        /** @member {Array.<UMLAttribute>} */\n        this.qualifiers = [];\n    }\n    // inherits from UMLRelationshipEnd\n    UMLAssociationEnd.prototype = Object.create(UMLRelationshipEnd.prototype);\n    UMLAssociationEnd.prototype.constructor = UMLAssociationEnd;\n\n    UMLAssociationEnd.prototype.getPropertyString = function () {\n        var props = [];\n        if (this.isID === true) { props.push("id"); }\n        if (this.isReadOnly === true) { props.push("readOnly"); }\n        if (this.isOrdered === true) { props.push("ordered"); }\n        if (this.isUnique === true) { props.push("unique"); }\n        props = _.union(props, this.getTagStringArray());\n        if (props.length > 0) {\n            return "{" + props.join(", ") + "}";\n        }\n        return "";\n    };\n\n    /**\n     * UMLAssociation\n     * @constructor\n     * @extends UMLUndirectedRelationship\n     */\n    function UMLAssociation() {\n        UMLUndirectedRelationship.apply(this, arguments);\n\n        /** @member {UMLAssociationEnd} */\n        this.end1 = new UMLAssociationEnd();\n        this.end1._parent = this;\n\n        /** @member {UMLAssociationEnd} */\n        this.end2 = new UMLAssociationEnd();\n        this.end2._parent = this;\n\n        /** @member {boolean} */\n        this.isDerived = false;\n    }\n    // inherits from UMLUndirectedRelationship\n    UMLAssociation.prototype = Object.create(UMLUndirectedRelationship.prototype);\n    UMLAssociation.prototype.constructor = UMLAssociation;\n\n\n    /**\n     * UMLAssociationClassLink\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLAssociationClassLink() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {UMLClass} */\n        this.classSide = null;\n\n        /** @member {UMLAssociation} */\n        this.associationSide = null;\n    }\n    // inherits from UMLModelElement\n    UMLAssociationClassLink.prototype = Object.create(UMLModelElement.prototype);\n    UMLAssociationClassLink.prototype.constructor = UMLAssociationClassLink;\n\n    UMLAssociationClassLink.prototype.getNodeText = function () {\n        return "(" + this.classSide.name + ")";\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                               INSTANCES                                *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLSlot\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLSlot() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {UMLStructuralFeature} */\n        this.definingFeature = null;\n\n        /** @member {string|UMLClassifier} */\n        this.type = \'\';\n\n        /** @member {string} */\n        this.value = \'\';\n    }\n    // inherits from UMLModelElement\n    UMLSlot.prototype = Object.create(UMLModelElement.prototype);\n    UMLSlot.prototype.constructor = UMLSlot;\n\n    /**\n     * Get type string\n     * @return {string}\n     */\n    UMLSlot.prototype.getTypeString = function () {\n        if (this.type) {\n            if (_.isString(this.type) && (this.type.length > 0)) {\n                return this.type;\n            } else if ((this.type !== null) && (this.type.name)) {\n                return this.type.name;\n            }\n        }\n        return null;\n    };\n\n    UMLSlot.prototype.getString = function (options) {\n        var text = "";\n        if (options && (options.stereotypeDisplay === SD_LABEL || options.stereotypeDisplay === SD_DECORATION_LABEL || options.stereotypeDisplay === SD_ICON_LABEL)) {\n            text += this.getStereotypeString();\n        }\n        if (options && options.showVisibility) {\n            text += this.getVisibilityString();\n        }\n        var hasFeature = (this.definingFeature instanceof type.UMLStructuralFeature);\n        text += hasFeature ? this.definingFeature.name : this.name;\n        if (options && options.showType) {\n            var _type = (hasFeature ? this.definingFeature.getTypeString() : this.getTypeString()) || "";\n            text += (_type.length > 0 ? ": " + _type : "");\n            if (hasFeature) {\n                text += (this.definingFeature.multiplicity.length > 0 ? "[" + this.definingFeature.multiplicity + "]" : "");\n            }\n        }\n        text += (this.value.length > 0 ? " = " + this.value : "");\n        if (options && options.showProperty) {\n            var prop = (hasFeature ? this.definingFeature.getPropertyString() : this.getPropertyString());\n            text += (prop.length > 0 ? " " + prop : "");\n        }\n        return text;\n    };\n\n    /**\n     * UMLInstance\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLInstance() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {UMLClassifier} */\n        this.classifier = null;\n\n        /** @member {Array.<UMLSlot>} */\n        this.slots = [];\n\n        /** @member {string} */\n        this.value = null;\n    }\n    // inherits from UMLModelElement\n    UMLInstance.prototype = Object.create(UMLModelElement.prototype);\n    UMLInstance.prototype.constructor = UMLInstance;\n\n    /**\n     * Get type string\n     *\n     * @return {string}\n     */\n    UMLInstance.prototype.getTypeString = function () {\n        if (this.classifier && this.classifier.name) {\n            return this.classifier.name;\n        } else if (_.isString(this.classifier) && (this.classifier.length > 0)) {\n            return this.classifier;\n        }\n        return null;\n    };\n\n    UMLInstance.prototype.getString = function (options) {\n        var text = "";\n        text += this.name;\n        if (options && options.showType) {\n            text += (this.getTypeString() !== null ? ": " + this.getTypeString() : "");\n        }\n        return text;\n    };\n\n\n    /**\n     * UMLObject\n     * @constructor\n     * @extends UMLInstance\n     */\n    function UMLObject() {\n        UMLInstance.apply(this, arguments);\n    }\n    // inherits from UMLInstance\n    UMLObject.prototype = Object.create(UMLInstance.prototype);\n    UMLObject.prototype.constructor = UMLObject;\n\n\n    /**\n     * UMLArtifactInstance\n     * @constructor\n     * @extends UMLInstance\n     */\n    function UMLArtifactInstance() {\n        UMLInstance.apply(this, arguments);\n    }\n    // inherits from UMLInstance\n    UMLArtifactInstance.prototype = Object.create(UMLInstance.prototype);\n    UMLArtifactInstance.prototype.constructor = UMLArtifactInstance;\n\n    /**\n     * UMLComponentInstance\n     * @constructor\n     * @extends UMLInstance\n     */\n    function UMLComponentInstance() {\n        UMLInstance.apply(this, arguments);\n    }\n    // inherits from UMLInstance\n    UMLComponentInstance.prototype = Object.create(UMLInstance.prototype);\n    UMLComponentInstance.prototype.constructor = UMLComponentInstance;\n\n\n    /**\n     * UMLNodeInstance\n     * @constructor\n     * @extends UMLInstance\n     */\n    function UMLNodeInstance() {\n        UMLInstance.apply(this, arguments);\n    }\n    // inherits from UMLInstance\n    UMLNodeInstance.prototype = Object.create(UMLInstance.prototype);\n    UMLNodeInstance.prototype.constructor = UMLNodeInstance;\n\n\n    /**\n     * UMLLinkEnd\n     * @constructor\n     * @extends UMLRelationshipEnd\n     */\n    function UMLLinkEnd() {\n        UMLRelationshipEnd.apply(this, arguments);\n    }\n    // inherits from UMLRelationshipEnd\n    UMLLinkEnd.prototype = Object.create(UMLRelationshipEnd.prototype);\n    UMLLinkEnd.prototype.constructor = UMLLinkEnd;\n\n    UMLLinkEnd.prototype.getPropertyString = function () {\n        var props = [];\n        if (this.isID === true) { props.push("id"); }\n        if (this.isReadOnly === true) { props.push("readOnly"); }\n        if (this.isOrdered === true) { props.push("ordered"); }\n        if (this.isUnique === true) { props.push("unique"); }\n        props = _.union(props, this.getTagStringArray());\n        if (props.length > 0) {\n            return "{" + props.join(", ") + "}";\n        }\n        return "";\n    };\n\n\n    /**\n     * UMLLink\n     * @constructor\n     * @extends UMLUndirectedRelationship\n     */\n    function UMLLink() {\n        UMLUndirectedRelationship.apply(this, arguments);\n\n        /** @member {UMLLinkEnd} */\n        this.end1 = new UMLLinkEnd();\n        this.end1._parent = this;\n\n        /** @member {UMLLinkEnd} */\n        this.end2 = new UMLLinkEnd();\n        this.end2._parent = this;\n\n        /** @member {UMLAssociation} */\n        this.association = null;\n    }\n    // inherits from UMLUndirectedRelationship\n    UMLLink.prototype = Object.create(UMLUndirectedRelationship.prototype);\n    UMLLink.prototype.constructor = UMLLink;\n\n    /**************************************************************************\n     *                                                                        *\n     *                          COMPOSITE STRUCTURES                          *\n     *                                                                        *\n     **************************************************************************/\n\n\n    /**\n     * UMLPort\n     * @constructor\n     * @extends UMLAttribute\n     */\n    function UMLPort() {\n        UMLAttribute.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.isBehavior = false;\n\n        /** @member {boolean} */\n        this.isService = false;\n\n        /** @member {boolean} */\n        this.isConjugated = false;\n    }\n    // inherits from UMLAttribute\n    UMLPort.prototype = Object.create(UMLAttribute.prototype);\n    UMLPort.prototype.constructor = UMLPort;\n\n\n    /**\n     * UMLConnectorEnd\n     * @constructor\n     * @extends UMLRelationshipEnd\n     */\n    function UMLConnectorEnd() {\n        UMLRelationshipEnd.apply(this, arguments);\n    }\n    // inherits from UMLRelationshipEnd\n    UMLConnectorEnd.prototype = Object.create(UMLRelationshipEnd.prototype);\n    UMLConnectorEnd.prototype.constructor = UMLConnectorEnd;\n\n\n    /**\n     * UMLConnector\n     * @constructor\n     * @extends UMLUndirectedRelationship\n     */\n    function UMLConnector() {\n        UMLUndirectedRelationship.apply(this, arguments);\n\n        /** @member {UMLConnectorEnd} */\n        this.end1 = new UMLConnectorEnd();\n        this.end1._parent = this;\n\n        /** @member {UMLConnectorEnd} */\n        this.end2 = new UMLConnectorEnd();\n        this.end2._parent = this;\n\n        /** @member {UMLAssociation} */\n        this.type = null;\n\n        /** @member {UMLConnectorKind} */\n        this.kind = CK_ASSEMBLY;\n    }\n    // inherits from UMLUndirectedRelationship\n    UMLConnector.prototype = Object.create(UMLUndirectedRelationship.prototype);\n    UMLConnector.prototype.constructor = UMLConnector;\n\n\n    /**\n     * UMLCollaboration\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLCollaboration() {\n        UMLClassifier.apply(this, arguments);\n    }\n    // inherits from UMLClassifier\n    UMLCollaboration.prototype = Object.create(UMLClassifier.prototype);\n    UMLCollaboration.prototype.constructor = UMLCollaboration;\n\n\n    /**\n     * UMLCollaborationUse\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLCollaborationUse() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {UMLCollaboration} */\n        this.type = null;\n    }\n    // inherits from UMLModelElement\n    UMLCollaborationUse.prototype = Object.create(UMLModelElement.prototype);\n    UMLCollaborationUse.prototype.constructor = UMLCollaborationUse;\n\n\n    /**\n     * UMLRoleBinding\n     * @constructor\n     * @extends UMLDependency\n     */\n    function UMLRoleBinding() {\n        UMLDependency.apply(this, arguments);\n\n        /** @member {string} */\n        this.roleName = \'\';\n    }\n    // inherits from UMLDependency\n    UMLRoleBinding.prototype = Object.create(UMLDependency.prototype);\n    UMLRoleBinding.prototype.constructor = UMLRoleBinding;\n\n    /**************************************************************************\n     *                                                                        *\n     *                               COMPONENTS                               *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLArtifact\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLArtifact() {\n        UMLClassifier.apply(this, arguments);\n\n        /** @member {string} */\n        this.fileName = \'\';\n    }\n    // inherits from UMLClassifier\n    UMLArtifact.prototype = Object.create(UMLClassifier.prototype);\n    UMLArtifact.prototype.constructor = UMLArtifact;\n\n    /**\n     * UMLComponent\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLComponent() {\n        UMLClassifier.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.isIndirectlyInstantiated = true;\n    }\n    // inherits from UMLClassifier\n    UMLComponent.prototype = Object.create(UMLClassifier.prototype);\n    UMLComponent.prototype.constructor = UMLComponent;\n\n    /**\n     * Get all classifier realizing this component\n     *\n     * @return {Array.<UMLClassifier>}\n     */\n    UMLComponent.prototype.getRealizingClassifiers = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLComponentRealization) && (r.target === self);\n            });\n        return _.map(rels, function (g) { return g.source; });\n    };\n\n\n    /**\n     * UMLSubsystem\n     * @constructor\n     * @extends UMLComponent\n     */\n    function UMLSubsystem() {\n        UMLComponent.apply(this, arguments);\n    }\n    // inherits from UMLComponent\n    UMLSubsystem.prototype = Object.create(UMLComponent.prototype);\n    UMLSubsystem.prototype.constructor = UMLSubsystem;\n\n    /**************************************************************************\n     *                                                                        *\n     *                                DEPLOYMENTS                             *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLNode\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLNode() {\n        UMLClassifier.apply(this, arguments);\n    }\n    // inherits from UMLClassifier\n    UMLNode.prototype = Object.create(UMLClassifier.prototype);\n    UMLNode.prototype.constructor = UMLNode;\n\n    /**\n     * Get all element deployed in this node\n     *\n     * @return {Array.<Element>}\n     */\n    UMLNode.prototype.getDeployedElements = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLDeployment) && (r.target === self);\n            });\n        return _.map(rels, function (g) { return g.source; });\n    };\n\n    /**\n     * UMLDeployment\n     * @constructor\n     * @extends UMLDependency\n     */\n    function UMLDeployment() {\n        UMLDependency.apply(this, arguments);\n    }\n    // inherits from UMLDependency\n    UMLDeployment.prototype = Object.create(UMLDependency.prototype);\n    UMLDeployment.prototype.constructor = UMLDeployment;\n\n\n    /**\n     * UMLCommunicationPath\n     * @constructor\n     * @extends UMLAssociation\n     */\n    function UMLCommunicationPath() {\n        UMLAssociation.apply(this, arguments);\n    }\n    // inherits from UMLAssociation\n    UMLCommunicationPath.prototype = Object.create(UMLAssociation.prototype);\n    UMLCommunicationPath.prototype.constructor = UMLCommunicationPath;\n\n    /**************************************************************************\n     *                                                                        *\n     *                                USE CASES                               *\n     *                                                                        *\n     *************************************************************************/\n\n    /**\n     * UMLExtensionPoint\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLExtensionPoint() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {string} */\n        this.location = "";\n    }\n    // inherits from UMLModelElement\n    UMLExtensionPoint.prototype = Object.create(UMLModelElement.prototype);\n    UMLExtensionPoint.prototype.constructor = UMLExtensionPoint;\n\n    UMLExtensionPoint.prototype.getString = function (options) {\n        var text = "";\n        if (options && (options.stereotypeDisplay === SD_LABEL || options.stereotypeDisplay === SD_DECORATION_LABEL || options.stereotypeDisplay === SD_ICON_LABEL)) {\n            text += this.getStereotypeString();\n        }\n        text += this.name;\n        text += (this.location ? ": " + this.location : "");\n        if (options && options.showProperty) {\n            var prop = this.getPropertyString();\n            text += (prop.length > 0 ? " " + prop : "");\n        }\n        return text;\n    };\n\n\n    /**\n     * UMLUseCase\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLUseCase() {\n        UMLClassifier.apply(this, arguments);\n\n        /** @member {Array.<UMLExtensionPoint>} */\n        this.extensionPoints = [];\n    }\n    // inherits from UMLClassifier\n    UMLUseCase.prototype = Object.create(UMLClassifier.prototype);\n    UMLUseCase.prototype.constructor = UMLUseCase;\n\n    /**\n     * Get all actors associated with this use case\n     *\n     * @return {Array.<UMLActor>}\n     */\n    UMLUseCase.prototype.getActors = function () {\n        var associated = _.map(this.getAssociationEnds(true), function (e) { return e.reference; });\n        return _.filter(associated, function (asso) { return (asso instanceof type.UMLActor); });\n    };\n\n    /**\n     * Get use cases directly included in this use case\n     *\n     * @return {Array.<UMLUseCase>}\n     */\n    UMLUseCase.prototype.getIncludedUseCases = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLInclude) && (r.source === self);\n            });\n        return _.map(rels, function (g) { return g.target; });\n    };\n\n    /**\n     * Get use cases extending this use case\n     *\n     * @return {Array.<UMLUseCase>}\n     */\n    UMLUseCase.prototype.getExtendingUseCases = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLExtend) && (r.target === self);\n            });\n        return _.map(rels, function (g) { return g.source; });\n    };\n\n    /**\n     * Get all included use cases\n     *\n     * @return {Array.<UMLUseCase>}\n     */\n    UMLUseCase.prototype.getAllIncludedUseCases = function () {\n        var includings = this.getIncludedUseCases(),\n            size = 0;\n        do {\n            size = includings.length;\n            _.each(includings, function (e) {\n                includings = _.union(includings, e.getIncludedUseCases());\n            });\n        } while (size < includings.length);\n        return includings;\n    };\n\n    /**\n     * UMLActor\n     * @constructor\n     * @extends UMLClassifier\n     */\n    function UMLActor() {\n        UMLClassifier.apply(this, arguments);\n    }\n    // inherits from UMLClassifier\n    UMLActor.prototype = Object.create(UMLClassifier.prototype);\n    UMLActor.prototype.constructor = UMLActor;\n\n    /**\n     * Get use cases associated with this actor\n     *\n     * @return {Array.<UMLUseCase>}\n     */\n    UMLActor.prototype.getUseCases = function () {\n        var associated = _.map(this.getAssociationEnds(true), function (e) { return e.reference; });\n        return _.filter(associated, function (asso) { return (asso instanceof type.UMLUseCase); });\n    };\n\n\n    /**\n     * UMLInclude\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLInclude() {\n        UMLDirectedRelationship.apply(this, arguments);\n    }\n    // inherits from UMLDirectedRelationship\n    UMLInclude.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLInclude.prototype.constructor = UMLInclude;\n\n\n    /**\n     * UMLExtend\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLExtend() {\n        UMLDirectedRelationship.apply(this, arguments);\n\n        /** @member {string} */\n        this.condition = "";\n\n        /** @member {Array.<UMLExtensionPoint>} */\n        this.extensionLocations = [];\n    }\n    // inherits from UMLDirectedRelationship\n    UMLExtend.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLExtend.prototype.constructor = UMLExtend;\n\n    /**\n     * UMLUseCaseSubject\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLUseCaseSubject() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {UMLClassifier} */\n        this.represent = null;\n    }\n    // inherits from UMLModelElement\n    UMLUseCaseSubject.prototype = Object.create(UMLModelElement.prototype);\n    UMLUseCaseSubject.prototype.constructor = UMLUseCaseSubject;\n\n\n    /**************************************************************************\n     *                                                                        *\n     *                             STATE MACHINES                             *\n     *                                                                        *\n     *************************************************************************/\n\n\n    /**\n     * UMLStateMachine\n     * @constructor\n     * @extends UMLBehavior\n     */\n    function UMLStateMachine() {\n        UMLBehavior.apply(this, arguments);\n\n        /** @member {Array.<UMLRegion>} */\n        this.regions = [];\n    }\n    // inherits from UMLBehavior\n    UMLStateMachine.prototype = Object.create(UMLBehavior.prototype);\n    UMLStateMachine.prototype.constructor = UMLStateMachine;\n\n\n    /**\n     * UMLRegion\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLRegion() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {Array.<UMLVertex>} */\n        this.vertices = [];\n\n        /** @member {Array.<UMLTransition>} */\n        this.transitions = [];\n    }\n    // inherits from UMLModelElement\n    UMLRegion.prototype = Object.create(UMLModelElement.prototype);\n    UMLRegion.prototype.constructor = UMLRegion;\n\n    UMLRegion.prototype.canContainKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLVertex");\n    };\n\n\n    /**\n     * UMLVertex\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLVertex() {\n        UMLModelElement.apply(this, arguments);\n    }\n    // inherits from UMLModelElement\n    UMLVertex.prototype = Object.create(UMLModelElement.prototype);\n    UMLVertex.prototype.constructor = UMLVertex;\n\n\n    /**\n     * Get incoming transitions\n     *\n     * @return {Array.<UMLTransition>}\n     */\n    UMLVertex.prototype.getIncomingTransitions = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLTransition) && (r.target === self);\n            });\n        return rels;\n    };\n\n    /**\n     * Get outgoing transitions\n     *\n     * @return {Array.<UMLTransition>}\n     */\n    UMLVertex.prototype.getOutgoingTransitions = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLTransition) && (r.source === self);\n            });\n        return rels;\n    };\n\n    /**\n     * UMLPseudostate\n     * @constructor\n     * @extends UMLVertex\n     */\n    function UMLPseudostate() {\n        UMLVertex.apply(this, arguments);\n\n        /** @member {string} */\n        this.kind = PSK_INITIAL;\n    }\n    // inherits from UMLVertex\n    UMLPseudostate.prototype = Object.create(UMLVertex.prototype);\n    UMLPseudostate.prototype.constructor = UMLPseudostate;\n\n    UMLPseudostate.prototype.getNodeIcon = function () {\n        switch (this.kind) {\n        case PSK_INITIAL:\n            return "icon-UMLInitialState";\n        case PSK_DEEPHISTORY:\n            return "icon-UMLDeepHistory";\n        case PSK_SHALLOWHISTORY:\n            return "icon-UMLShallowHistory";\n        case PSK_JOIN:\n            return "icon-UMLJoin";\n        case PSK_FORK:\n            return "icon-UMLFork";\n        case PSK_JUNCTION:\n            return "icon-UMLJunction";\n        case PSK_CHOICE:\n            return "icon-UMLChoice";\n        case PSK_ENTRYPOINT:\n            return "icon-UMLEntryPoint";\n        case PSK_EXITPOINT:\n            return "icon-UMLExitPoint";\n        case PSK_TERMINATE:\n            return "icon-UMLTerminate";\n        }\n    };\n\n\n    /**\n     * UMLConnectionPointReference\n     * @constructor\n     * @extends UMLVertex\n     */\n    function UMLConnectionPointReference() {\n        UMLVertex.apply(this, arguments);\n\n        /** @member {UMLPseudostate} */\n        this.entry = [];\n\n        /** @member {UMLPseudostate} */\n        this.exit = [];\n    }\n    // inherits from UMLVertex\n    UMLConnectionPointReference.prototype = Object.create(UMLVertex.prototype);\n    UMLConnectionPointReference.prototype.constructor = UMLConnectionPointReference;\n\n\n    /**\n     * UMLState\n     * @constructor\n     * @extends UMLVertex\n     */\n    function UMLState() {\n        UMLVertex.apply(this, arguments);\n\n        /** @member {Array.<UMLRegion>} */\n        this.regions = [];\n\n        /** @member {Array.<UMLBehavior>} */\n        this.entryActivities = [];\n\n        /** @member {Array.<UMLBehavior>} */\n        this.doActivities = [];\n\n        /** @member {Array.<UMLBehavior>} */\n        this.exitActivities = [];\n\n        /** @member {UMLStateMachine} */\n        this.submachine = null;\n\n        /** @member {UMLConnectionPointReference} */\n        this.connections = [];\n    }\n    // inherits from UMLVertex\n    UMLState.prototype = Object.create(UMLVertex.prototype);\n    UMLState.prototype.constructor = UMLState;\n\n    UMLState.prototype.getInternalTransitions = function () {\n        var self = this, internalTransitions = [];\n        _.each(this.regions, function (region) {\n            _.each(region.transitions, function (t) {\n                if (t.source === self && t.target === self && t.kind === TK_INTERNAL) {\n                    internalTransitions.push(t);\n                }\n            });\n        });\n        return internalTransitions;\n    };\n\n    /**\n     * UMLFinalState\n     * @constructor\n     * @extends UMLState\n     */\n    function UMLFinalState() {\n        UMLState.apply(this, arguments);\n    }\n    // inherits from UMLState\n    UMLFinalState.prototype = Object.create(UMLState.prototype);\n    UMLFinalState.prototype.constructor = UMLFinalState;\n\n\n    /**\n     * UMLTransition\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLTransition() {\n        UMLDirectedRelationship.apply(this, arguments);\n\n        /** @member {string} */\n        this.kind = TK_EXTERNAL;\n\n        /** @member {string} */\n        this.guard = \'\';\n\n        /** @member {Array.<UMLEvent>} */\n        this.triggers = [];\n\n        /** @member {Array.<UMLBehavior>} */\n        this.effects = [];\n    }\n    // inherits from UMLDirectedRelationship\n    UMLTransition.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLTransition.prototype.constructor = UMLTransition;\n\n    UMLTransition.prototype.getString = function () {\n        var i, len, text = "";\n        // triggers\n        if (this.triggers.length > 0) {\n            var triggers = [];\n            for (i = 0, len = this.triggers.length; i < len; i++) {\n                triggers.push(this.triggers[i].name);\n            }\n            text += triggers.join(", ");\n        }\n        // guard\n        if (this.guard.length > 0) {\n            text += " [" + this.guard + "]";\n        }\n        // effects\n        if (this.effects.length > 0) {\n            var effects = [];\n            for (i = 0, len = this.effects.length; i < len; i++) {\n                effects.push(this.effects[i].name);\n            }\n            text += " / " + effects.join(", ");\n        }\n        if (text.length > 0) {\n            text = (this.name.length > 0 ? this.name + " : " + text : text);\n        } else {\n            text = this.name;\n        }\n        return text;\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                                ACTIVITIES                              *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLActivity\n     * @constructor\n     * @extends UMLActivity\n     */\n    function UMLActivity() {\n        UMLBehavior.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.isReadOnly = false;\n\n        /** @member {boolean} */\n        this.isSingleExecution = false;\n\n        /** @member {Array.<UMLActivityNode>} */\n        this.nodes = [];\n\n        /** @member {Array.<UMLActivityEdge>} */\n        this.edges = [];\n\n        /** @member {Array.<UMLActivityGroup>} */\n        this.groups = [];\n    }\n    // inherits from UMLBehavior\n    UMLActivity.prototype = Object.create(UMLBehavior.prototype);\n    UMLActivity.prototype.constructor = UMLActivity;\n\n\n    /**\n     * UMLPin\n     * @constructor\n     * @extends UMLStructuralFeature\n     */\n    function UMLPin() {\n        UMLStructuralFeature.apply(this, arguments);\n    }\n    // inherits from UMLStructuralFeature\n    UMLPin.prototype = Object.create(UMLStructuralFeature.prototype);\n    UMLPin.prototype.constructor = UMLPin;\n\n\n    /**\n     * UMLInputPin\n     * @constructor\n     * @extends UMLPin\n     */\n    function UMLInputPin() {\n        UMLPin.apply(this, arguments);\n    }\n    // inherits from UMLPin\n    UMLInputPin.prototype = Object.create(UMLPin.prototype);\n    UMLInputPin.prototype.constructor = UMLInputPin;\n\n\n    /**\n     * UMLOutputPin\n     * @constructor\n     * @extends UMLPin\n     */\n    function UMLOutputPin() {\n        UMLPin.apply(this, arguments);\n    }\n    // inherits from UMLPin\n    UMLOutputPin.prototype = Object.create(UMLPin.prototype);\n    UMLOutputPin.prototype.constructor = UMLOutputPin;\n\n\n    /**\n     * UMLExpansionNode\n     * @constructor\n     * @extends UMLPin\n     */\n    function UMLExpansionNode() {\n        UMLPin.apply(this, arguments);\n    }\n    // inherits from UMLPin\n    UMLExpansionNode.prototype = Object.create(UMLPin.prototype);\n    UMLExpansionNode.prototype.constructor = UMLExpansionNode;\n\n\n    /**\n     * UMLActivityNode\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLActivityNode() {\n        UMLModelElement.apply(this, arguments);\n    }\n    // inherits from UMLModelElement\n    UMLActivityNode.prototype = Object.create(UMLModelElement.prototype);\n    UMLActivityNode.prototype.constructor = UMLActivityNode;\n\n\n    /**\n     * Get incoming edges\n     *\n     * @return {Array.<UMLActivityEdge>}\n     */\n    UMLActivityNode.prototype.getIncomingEdges = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLActivityEdge) && (r.target === self);\n            });\n        return rels;\n    };\n\n    /**\n     * Get outgoing edges\n     *\n     * @return {Array.<UMLActivityEdge>}\n     */\n    UMLActivityNode.prototype.getOutgoingEdges = function () {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) {\n                return (r instanceof type.UMLActivityEdge) && (r.source === self);\n            });\n        return rels;\n    };\n\n\n    /**\n     * UMLAction\n     * @constructor\n     * @extends UMLActivityNode\n     */\n    function UMLAction() {\n        UMLActivityNode.apply(this, arguments);\n\n        /** @member {string} */\n        this.kind = ACK_OPAQUE;\n\n        /** @member {Array.<UMLInputPin>} */\n        this.inputs = [];\n\n        /** @member {Array.<UMLOutputPin>} */\n        this.outputs = [];\n\n        /** @member {Array.<UMLEvent>} */\n        this.triggers = [];\n\n        /** @member {UMLModelElement} */\n        this.target = null;\n\n        /** @member {UMLActivity} */\n        this.subactivity = null;\n\n        /** @member {boolean} */\n        this.isLocallyReentrant = false;\n\n        /** @member {boolean} */\n        this.isSynchronous = true;\n\n        /** @member {string} */\n        this.language = \'\';\n\n        /** @member {string} */\n        this.body = \'\';\n\n        /** @member {Array.<UMLConstraint>} */\n        this.localPreconditions = [];\n\n        /** @member {Array.<UMLConstraint>} */\n        this.localPostconditions = [];\n    }\n    // inherits from UMLActivityNode\n    UMLAction.prototype = Object.create(UMLActivityNode.prototype);\n    UMLAction.prototype.constructor = UMLAction;\n\n    UMLAction.prototype.getNodeIcon = function () {\n        switch (this.kind) {\n        case ACK_OPAQUE:\n            return "icon-UMLAction";\n        case ACK_CREATE:\n            return "icon-UMLAction";\n        case ACK_DESTROY:\n            return "icon-UMLAction";\n        case ACK_READ:\n            return "icon-UMLAction";\n        case ACK_WRITE:\n            return "icon-UMLAction";\n        case ACK_INSERT:\n            return "icon-UMLAction";\n        case ACK_DELETE:\n            return "icon-UMLAction";\n        case ACK_STRUCTURED:\n            return "icon-UMLAction";\n        case ACK_SENDSIGNAL:\n            return "icon-UMLSendSignal";\n        case ACK_ACCEPTSIGNAL:\n            return "icon-UMLAcceptSignal";\n        case ACK_TRIGGEREVENT:\n            return "icon-UMLSendSignal";\n        case ACK_ACCEPTEVENT:\n            return "icon-UMLAcceptSignal";\n        case ACK_TIMEEVENT:\n            return "icon-UMLAcceptTimeEvent";\n        }\n    };\n\n\n    /**\n     * UMLObjectNode\n     * @constructor\n     * @extends UMLActivityNode\n     */\n    function UMLObjectNode() {\n        UMLActivityNode.apply(this, arguments);\n\n        /** @member {UMLClassifier} */\n        this.type = null;\n\n        /** @member {boolean} */\n        this.isControlType = false;\n\n        /** @member {string} */\n        this.ordering = ONOK_FIFO;\n    }\n    // inherits from UMLActivityNode\n    UMLObjectNode.prototype = Object.create(UMLActivityNode.prototype);\n    UMLObjectNode.prototype.constructor = UMLObjectNode;\n\n    UMLObjectNode.prototype.getTypeString = function () {\n        if (_.isString(this.type) && (this.type.length > 0)) {\n            return this.type;\n        } else if ((this.type !== null) && (this.type.name)) {\n            return this.type.name;\n        }\n        return null;\n    };\n\n    UMLObjectNode.prototype.getString = function () {\n        var text = "";\n        text += this.name;\n        text += (this.getTypeString() !== null ? ": " + this.getTypeString() : "");\n        return text;\n    };\n\n\n    /**\n     * UMLCentralBufferNode\n     * @constructor\n     * @extends UMLObjectNode\n     */\n    function UMLCentralBufferNode() {\n        UMLObjectNode.apply(this, arguments);\n    }\n    // inherits from UMLObjectNode\n    UMLCentralBufferNode.prototype = Object.create(UMLObjectNode.prototype);\n    UMLCentralBufferNode.prototype.constructor = UMLCentralBufferNode;\n\n\n    /**\n     * UMLDataStoreNode\n     * @constructor\n     * @extends UMLCentralBufferNode\n     */\n    function UMLDataStoreNode() {\n        UMLCentralBufferNode.apply(this, arguments);\n    }\n    // inherits from UMLCentralBufferNode\n    UMLDataStoreNode.prototype = Object.create(UMLCentralBufferNode.prototype);\n    UMLDataStoreNode.prototype.constructor = UMLDataStoreNode;\n\n\n    /**\n     * UMLControlNode\n     * @constructor\n     * @extends UMLActivityNode\n     */\n    function UMLControlNode() {\n        UMLActivityNode.apply(this, arguments);\n    }\n    // inherits from UMLActivityNode\n    UMLControlNode.prototype = Object.create(UMLActivityNode.prototype);\n    UMLControlNode.prototype.constructor = UMLControlNode;\n\n\n    /**\n     * UMLInitialNode\n     * @constructor\n     * @extends UMLControlNode\n     */\n    function UMLInitialNode() {\n        UMLControlNode.apply(this, arguments);\n    }\n    // inherits from UMLControlNode\n    UMLInitialNode.prototype = Object.create(UMLControlNode.prototype);\n    UMLInitialNode.prototype.constructor = UMLInitialNode;\n\n\n    /**\n     * UMLFinalNode\n     * @constructor\n     * @extends UMLControlNode\n     */\n    function UMLFinalNode() {\n        UMLControlNode.apply(this, arguments);\n    }\n    // inherits from UMLControlNode\n    UMLFinalNode.prototype = Object.create(UMLControlNode.prototype);\n    UMLFinalNode.prototype.constructor = UMLFinalNode;\n\n\n    /**\n     * UMLActivityFinalNode\n     * @constructor\n     * @extends UMLFinalNode\n     */\n    function UMLActivityFinalNode() {\n        UMLFinalNode.apply(this, arguments);\n    }\n    // inherits from UMLFinalNode\n    UMLActivityFinalNode.prototype = Object.create(UMLFinalNode.prototype);\n    UMLActivityFinalNode.prototype.constructor = UMLActivityFinalNode;\n\n\n    /**\n     * UMLFlowFinalNode\n     * @constructor\n     * @extends UMLFinalNode\n     */\n    function UMLFlowFinalNode() {\n        UMLFinalNode.apply(this, arguments);\n    }\n    // inherits from UMLFinalNode\n    UMLFlowFinalNode.prototype = Object.create(UMLFinalNode.prototype);\n    UMLFlowFinalNode.prototype.constructor = UMLFlowFinalNode;\n\n\n    /**\n     * UMLForkNode\n     * @constructor\n     * @extends UMLControlNode\n     */\n    function UMLForkNode() {\n        UMLControlNode.apply(this, arguments);\n    }\n    // inherits from UMLControlNode\n    UMLForkNode.prototype = Object.create(UMLControlNode.prototype);\n    UMLForkNode.prototype.constructor = UMLForkNode;\n\n\n    /**\n     * UMLJoinNode\n     * @constructor\n     * @extends UMLControlNode\n     */\n    function UMLJoinNode() {\n        UMLControlNode.apply(this, arguments);\n    }\n    // inherits from UMLControlNode\n    UMLJoinNode.prototype = Object.create(UMLControlNode.prototype);\n    UMLJoinNode.prototype.constructor = UMLJoinNode;\n\n\n    /**\n     * UMLMergeNode\n     * @constructor\n     * @extends UMLControlNode\n     */\n    function UMLMergeNode() {\n        UMLControlNode.apply(this, arguments);\n    }\n    // inherits from UMLControlNode\n    UMLMergeNode.prototype = Object.create(UMLControlNode.prototype);\n    UMLMergeNode.prototype.constructor = UMLMergeNode;\n\n\n    /**\n     * UMLDecisionNode\n     * @constructor\n     * @extends UMLControlNode\n     */\n    function UMLDecisionNode() {\n        UMLControlNode.apply(this, arguments);\n    }\n    // inherits from UMLControlNode\n    UMLDecisionNode.prototype = Object.create(UMLControlNode.prototype);\n    UMLDecisionNode.prototype.constructor = UMLDecisionNode;\n\n\n    /**\n     * UMLActivityGroup\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLActivityGroup() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {Array.<UMLActivityNode>} */\n        this.nodes = [];\n\n        /** @member {Array.<UMLActivityEdge>} */\n        this.edges = [];\n\n        /** @member {Array.<UMLActivityGroup>} */\n        this.subgroups = [];\n    }\n    // inherits from UMLModelElement\n    UMLActivityGroup.prototype = Object.create(UMLModelElement.prototype);\n    UMLActivityGroup.prototype.constructor = UMLActivityGroup;\n\n\n    /**\n     * UMLActivityPartition\n     * @constructor\n     * @extends UMLActivityGroup\n     */\n    function UMLActivityPartition() {\n        UMLActivityGroup.apply(this, arguments);\n    }\n    // inherits from UMLActivityGroup\n    UMLActivityPartition.prototype = Object.create(UMLActivityGroup.prototype);\n    UMLActivityPartition.prototype.constructor = UMLActivityPartition;\n\n    UMLActivityPartition.prototype.getNodeIcon = function () {\n        return "icon-UMLSwimlaneVert";\n    };\n\n    UMLActivityPartition.prototype.canContainKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLActivityNode");\n    };\n\n\n    /**\n     * UMLInterruptibleActivityRegion\n     * @constructor\n     * @extends UMLActivityGroup\n     */\n    function UMLInterruptibleActivityRegion() {\n        UMLActivityGroup.apply(this, arguments);\n    }\n    // inherits from UMLActivityGroup\n    UMLInterruptibleActivityRegion.prototype = Object.create(UMLActivityGroup.prototype);\n    UMLInterruptibleActivityRegion.prototype.constructor = UMLInterruptibleActivityRegion;\n\n    UMLInterruptibleActivityRegion.prototype.canContainKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLActivityNode");\n    };\n\n\n    /**\n     * UMLStructuredActivityNode\n     * @constructor\n     * @extends UMLAction\n     */\n    function UMLStructuredActivityNode() {\n        UMLAction.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.mustIsolate = false;\n\n        /** @member {Array.<UMLActivityNode>} */\n        this.nodes = [];\n\n        /** @member {Array.<UMLActivityEdge>} */\n        this.edges = [];\n    }\n    // inherits from UMLAction\n    UMLStructuredActivityNode.prototype = Object.create(UMLAction.prototype);\n    UMLStructuredActivityNode.prototype.constructor = UMLStructuredActivityNode;\n\n    UMLStructuredActivityNode.prototype.getNodeIcon = function () {\n        return "icon-UMLStructuredActivityNode";\n    };\n\n    UMLStructuredActivityNode.prototype.canContainKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLActivityNode");\n    };\n\n\n    /**\n     * UMLExpansionRegion\n     * @constructor\n     * @extends UMLStructuredActivityNode\n     */\n    function UMLExpansionRegion() {\n        UMLStructuredActivityNode.apply(this, arguments);\n\n        /** @member {UMLExpansionKind} */\n        this.mode = EK_ITERATIVE;\n    }\n    // inherits from UMLStructuredActivityNode\n    UMLExpansionRegion.prototype = Object.create(UMLStructuredActivityNode.prototype);\n    UMLExpansionRegion.prototype.constructor = UMLExpansionRegion;\n\n    UMLExpansionRegion.prototype.getNodeIcon = function () {\n        return "icon-UMLExpansionRegion";\n    };\n\n\n    /**\n     * UMLExceptionHandler\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLExceptionHandler() {\n        UMLDirectedRelationship.apply(this, arguments);\n\n        /** @member {Array.<UMLClassifier>} */\n        this.exceptionTypes = [];\n\n        /** @member {UMLActivityNode} */\n        this.handlerBody = null;\n    }\n    // inherits from UMLDirectedRelationship\n    UMLExceptionHandler.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLExceptionHandler.prototype.constructor = UMLExceptionHandler;\n\n\n    /**\n     * UMLActivityEdge\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLActivityEdge() {\n        UMLDirectedRelationship.apply(this, arguments);\n\n        /** @member {string} */\n        this.guard = \'\';\n\n        /** @member {string} */\n        this.weight = \'\';\n    }\n    // inherits from UMLDirectedRelationship\n    UMLActivityEdge.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLActivityEdge.prototype.constructor = UMLActivityEdge;\n\n    UMLActivityEdge.prototype.getString = function () {\n        var text = this.name;\n        // guard\n        if (this.guard.length > 0) {\n            text += " [" + this.guard + "]";\n        }\n        // weight\n        if (this.weight.length > 0) {\n            text += " {weight=" + this.weight + "}";\n        }\n        return text;\n    };\n\n\n    /**\n     * UMLControlFlow\n     * @constructor\n     * @extends UMLActivityEdge\n     */\n    function UMLControlFlow() {\n        UMLActivityEdge.apply(this, arguments);\n    }\n    // inherits from UMLActivityEdge\n    UMLControlFlow.prototype = Object.create(UMLActivityEdge.prototype);\n    UMLControlFlow.prototype.constructor = UMLControlFlow;\n\n\n    /**\n     * UMLObjectFlow\n     * @constructor\n     * @extends UMLActivityEdge\n     */\n    function UMLObjectFlow() {\n        UMLActivityEdge.apply(this, arguments);\n    }\n    // inherits from UMLActivityEdge\n    UMLObjectFlow.prototype = Object.create(UMLActivityEdge.prototype);\n    UMLObjectFlow.prototype.constructor = UMLObjectFlow;\n\n\n    /**\n     * UMLActivityInterrupt\n     * @constructor\n     * @extends UMLActivityEdge\n     */\n    function UMLActivityInterrupt() {\n        UMLActivityEdge.apply(this, arguments);\n    }\n    // inherits from UMLActivityEdge\n    UMLActivityInterrupt.prototype = Object.create(UMLActivityEdge.prototype);\n    UMLActivityInterrupt.prototype.constructor = UMLActivityInterrupt;\n\n\n    /**************************************************************************\n     *                                                                        *\n     *                               INTERACTIONS                             *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLInteractionFragment\n     * @constructor\n     * @extends UMLBehavior\n     */\n    function UMLInteractionFragment() {\n        UMLBehavior.apply(this, arguments);\n    }\n    // inherits from UMLBehavior\n    UMLInteractionFragment.prototype = Object.create(UMLBehavior.prototype);\n    UMLInteractionFragment.prototype.constructor = UMLInteractionFragment;\n\n\n    /**\n     * UMLInteraction\n     * @constructor\n     * @extends UMLInteractionFragment\n     */\n    function UMLInteraction() {\n        UMLInteractionFragment.apply(this, arguments);\n\n        /** @member {UMLMessage} */\n        this.messages = [];\n\n        /** @member {UMLMessageEndpoint} */\n        this.participants = [];\n\n        /** @member {UMLInteractionFragment} */\n        this.fragments = [];\n    }\n    // inherits from UMLInteractionFragment\n    UMLInteraction.prototype = Object.create(UMLInteractionFragment.prototype);\n    UMLInteraction.prototype.constructor = UMLInteraction;\n\n\n    /**\n     * UMLStateInvariant\n     * @constructor\n     * @extends UMLInteractionFragment\n     */\n    function UMLStateInvariant() {\n        UMLInteractionFragment.apply(this, arguments);\n\n        /** @member {UMLLifeline} */\n        this.covered = null;\n\n        /** @member {string} */\n        this.invariant = \'\';\n    }\n    // inherits from UMLInteractionFragment\n    UMLStateInvariant.prototype = Object.create(UMLInteractionFragment.prototype);\n    UMLStateInvariant.prototype.constructor = UMLStateInvariant;\n\n\n    /**\n     * UMLContinuation\n     * @constructor\n     * @extends UMLInteractionFragment\n     */\n    function UMLContinuation() {\n        UMLInteractionFragment.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.setting = false;\n    }\n    // inherits from UMLInteractionFragment\n    UMLContinuation.prototype = Object.create(UMLInteractionFragment.prototype);\n    UMLContinuation.prototype.constructor = UMLContinuation;\n\n\n    /**\n     * UMLInteractionOperand\n     * @constructor\n     * @extends UMLInteractionFragment\n     */\n    function UMLInteractionOperand() {\n        UMLInteractionFragment.apply(this, arguments);\n\n        /** @member {string} */\n        this.guard = \'\';\n    }\n    // inherits from UMLInteractionFragment\n    UMLInteractionOperand.prototype = Object.create(UMLInteractionFragment.prototype);\n    UMLInteractionOperand.prototype.constructor = UMLInteractionOperand;\n\n\n    /**\n     * UMLCombinedFragment\n     * @constructor\n     * @extends UMLInteractionFragment\n     */\n    function UMLCombinedFragment() {\n        UMLInteractionFragment.apply(this, arguments);\n\n        /** @member {string} */\n        this.interactionOperator = IOK_SEQ;\n\n        /** @member {UMLInteractionOperand} */\n        this.operands = [];\n    }\n    // inherits from UMLInteractionFragment\n    UMLCombinedFragment.prototype = Object.create(UMLInteractionFragment.prototype);\n    UMLCombinedFragment.prototype.constructor = UMLCombinedFragment;\n\n\n    /**\n     * UMLInteractionUse\n     * @constructor\n     * @extends UMLInteractionFragment\n     */\n    function UMLInteractionUse() {\n        UMLInteractionFragment.apply(this, arguments);\n\n        /** @member {UMLInteraction} */\n        this.refersTo = null;\n\n        /** @member {string} */\n        this["arguments"] = \'\';\n\n        /** @member {string} */\n        this.returnValue = \'\';\n\n        /** @member {UMLStructuralFeature} */\n        this.returnValueRecipient = null;\n    }\n    // inherits from UMLInteractionFragment\n    UMLInteractionUse.prototype = Object.create(UMLInteractionFragment.prototype);\n    UMLInteractionUse.prototype.constructor = UMLInteractionUse;\n\n\n    /**\n     * UMLMessageEndpoint\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLMessageEndpoint() {\n        UMLModelElement.apply(this, arguments);\n    }\n    // inherits from UMLModelElement\n    UMLMessageEndpoint.prototype = Object.create(UMLModelElement.prototype);\n    UMLMessageEndpoint.prototype.constructor = UMLMessageEndpoint;\n\n\n    /**\n     * UMLLifeline\n     * @constructor\n     * @extends UMLMessageEndpoint\n     */\n    function UMLLifeline() {\n        UMLMessageEndpoint.apply(this, arguments);\n\n        /** @member {string} */\n        this.selector = \'\';\n\n        /** @member {UMLStructuralFeature} */\n        this.represent = null;\n\n        /** @member {boolean} */\n        this.isMultiInstance = false;\n    }\n    // inherits from UMLMessageEndpoint\n    UMLLifeline.prototype = Object.create(UMLMessageEndpoint.prototype);\n    UMLLifeline.prototype.constructor = UMLLifeline;\n\n    UMLLifeline.prototype.getTypeString = function () {\n        if (this.represent) {\n            if (_.isString(this.represent.type) && (this.represent.type.length > 0)) {\n                return this.represent.type;\n            } else if ((this.represent.type !== null) && (this.represent.type.name)) {\n                return this.represent.type.name;\n            }\n        }\n        return null;\n    };\n\n    UMLLifeline.prototype.getString = function (options) {\n        var text = "";\n        text += this.name;\n        text += (this.selector.length > 0 ? "[" + this.selector + "]" : "");\n        if (options && options.showType) {\n            text += (this.getTypeString() !== null ? ": " + this.getTypeString() : "");\n        }\n        return text;\n    };\n\n    UMLLifeline.prototype.getNodeText = function (options) {\n        var text = "",\n            typeStr = this.getTypeString();\n        options = options || {};\n        if (options.showStereotype !== false) {\n            text += this.getStereotypeString();\n        }\n        text += this.name;\n        if (typeStr) {\n            text += ": " + typeStr;\n        }\n        if (!text) {\n            text = "(Lifeline)";\n        }\n        return text;\n    };\n\n    /**\n     * UMLGate\n     * @constructor\n     * @extends UMLMessageEndpoint\n     */\n    function UMLGate() {\n        UMLMessageEndpoint.apply(this, arguments);\n    }\n    // inherits from UMLMessageEndpoint\n    UMLGate.prototype = Object.create(UMLMessageEndpoint.prototype);\n    UMLGate.prototype.constructor = UMLGate;\n\n\n    /**\n     * UMLEndpoint\n     * @constructor\n     * @extends UMLMessageEndpoint\n     */\n    function UMLEndpoint() {\n        UMLMessageEndpoint.apply(this, arguments);\n    }\n    // inherits from UMLMessageEndpoint\n    UMLEndpoint.prototype = Object.create(UMLMessageEndpoint.prototype);\n    UMLEndpoint.prototype.constructor = UMLEndpoint;\n\n\n    /**\n     * UMLMessage\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLMessage() {\n        UMLDirectedRelationship.apply(this, arguments);\n\n        /** @member {string} */\n        this.messageSort = MS_SYNCHCALL;\n\n        /** @member {UMLOperation} */\n        this.signature = null;\n\n        /** @member {UMLConnector} */\n        this.connector = null;\n\n        /** @member {string} */\n        this["arguments"] = \'\';\n\n        /** @member {string} */\n        this.assignmentTarget = \'\';\n\n        /** @member {string} */\n        this.guard = \'\';\n\n        /** @member {string} */\n        this.iteration = \'\';\n\n        /** @member {boolean} */\n        this.isConcurrentIteration = false;\n    }\n    // inherits from UMLDirectedRelationship\n    UMLMessage.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLMessage.prototype.constructor = UMLMessage;\n\n    UMLMessage.prototype.getString = function (options) {\n        var s = \'\';\n        // Sequence Number\n        if (options && options.showSequenceNumber && this._parent && this._parent.messages) {\n            s += _.indexOf(this._parent.messages, this) + 1;\n            s += \' \';\n        }\n        // Guard\n        if (this.guard.length > 0) {\n            s += \'[\' + this.guard + \'] \';\n        }\n        // Iteration\n        if (this.iteration.length > 0) {\n            s += (this.isConcurrentIteration ? \'*||[\' : \'*[\') + this.iteration + \'] \';\n        }\n        if (s.length > 0) {\n            s += ": ";\n        }\n        // Assignament Target\n        if (this.assignmentTarget.length > 0) {\n            s += this.assignmentTarget + " = ";\n        }\n        // Message Signature Part\n        if (this.signature instanceof type.UMLOperation) {\n            s += this.signature.name;\n            if (options && options.showSignature) {\n                if (this["arguments"].length > 0) {\n                    s += "(" + this["arguments"] + ")";\n                } else {\n                    s += this.signature.getParametersString(options);\n                    var r = this.signature.getReturnString();\n                    if (options.showType && r.length > 0) {\n                        s += ":" + r;\n                    }\n                }\n            } else {\n                s += "()";\n            }\n        } else if (this.signature instanceof type.UMLSignal) {\n            s += this.signature.getString();\n            if (options && options.showSignature && this["arguments"].length > 0) {\n                s += "(" + this["arguments"] + ")";\n            }\n        } else {\n            s += this.name;\n            if (options && options.showSignature && this["arguments"].length > 0) {\n                s += "(" + this["arguments"] + ")";\n            }\n        }\n        return s;\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                                 PROFILES                               *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLProfile\n     * @constructor\n     * @extends UMLPackage\n     */\n    function UMLProfile() {\n        UMLPackage.apply(this, arguments);\n    }\n    // inherits from UMLPackage\n    UMLProfile.prototype = Object.create(UMLPackage.prototype);\n    UMLProfile.prototype.constructor = UMLProfile;\n\n    UMLProfile.prototype.canContainDiagramKind = function (kind) {\n        return (kind === "UMLProfileDiagram");\n    };\n\n\n    /**\n     * UMLImage\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLImage() {\n        UMLModelElement.apply(this, arguments);\n\n        /** @member {number} */\n        this.width = 40;\n\n        /** @member {number} */\n        this.height = 40;\n\n        /** @member {string} */\n        this.content = \'\';\n\n        /** @member {string} */\n        this.smallIcon = \'\';\n    }\n    // inherits from UMLModelElement\n    UMLImage.prototype = Object.create(UMLModelElement.prototype);\n    UMLImage.prototype.constructor = UMLImage;\n\n\n    /**\n     * UMLStereotype\n     * @constructor\n     * @extends UMLClass\n     */\n    function UMLStereotype() {\n        UMLClass.apply(this, arguments);\n\n        /** @member {UMLImage} */\n        this.icon = new UMLImage();\n        this.icon._parent = this;\n    }\n    // inherits from UMLClass\n    UMLStereotype.prototype = Object.create(UMLClass.prototype);\n    UMLStereotype.prototype.constructor = UMLStereotype;\n\n    UMLStereotype.prototype.getIconClass = function () {\n        var name = this.getPathname().replace(/::/g, "-");\n        return "icon-" + name.replace(/[^a-zA-Z0-9_\\-]/g, "-");\n    };\n\n    /**\n     * UMLMetaClass\n     * @constructor\n     * @extends UMLModelElement\n     */\n    function UMLMetaClass() {\n        UMLModelElement.apply(this, arguments);\n    }\n    // inherits from UMLModelElement\n    UMLMetaClass.prototype = Object.create(UMLModelElement.prototype);\n    UMLMetaClass.prototype.constructor = UMLMetaClass;\n\n    /**\n     * UMLExtension\n     * @constructor\n     * @extends UMLDirectedRelationship\n     */\n    function UMLExtension() {\n        UMLDirectedRelationship.apply(this, arguments);\n    }\n    // inherits from UMLDirectedRelationship\n    UMLExtension.prototype = Object.create(UMLDirectedRelationship.prototype);\n    UMLExtension.prototype.constructor = UMLExtension;\n\n    /* ************************** Type definitions ***************************/\n\n    // Backbone\n    type.UMLModelElement             = UMLModelElement;\n    type.UMLConstraint               = UMLConstraint;\n    type.UMLTemplateParameter        = UMLTemplateParameter;\n    type.UMLFeature                  = UMLFeature;\n    type.UMLStructuralFeature        = UMLStructuralFeature;\n    type.UMLParameter                = UMLParameter;\n    type.UMLBehavioralFeature        = UMLBehavioralFeature;\n    type.UMLAttribute                = UMLAttribute;\n    type.UMLOperation                = UMLOperation;\n    type.UMLReception                = UMLReception;\n    type.UMLClassifier               = UMLClassifier;\n    type.UMLDirectedRelationship     = UMLDirectedRelationship;\n    type.UMLRelationshipEnd          = UMLRelationshipEnd;\n    type.UMLUndirectedRelationship   = UMLUndirectedRelationship;\n    // Common Behaviors\n    type.UMLBehavior                 = UMLBehavior;\n    type.UMLOpaqueBehavior           = UMLOpaqueBehavior;\n    type.UMLEvent                    = UMLEvent;\n    // Classes\n    type.UMLPackage                  = UMLPackage;\n    type.UMLModel                    = UMLModel;\n    type.UMLClass                    = UMLClass;\n    type.UMLInterface                = UMLInterface;\n    type.UMLSignal                   = UMLSignal;\n    type.UMLDataType                 = UMLDataType;\n    type.UMLPrimitiveType            = UMLPrimitiveType;\n    type.UMLEnumerationLiteral       = UMLEnumerationLiteral;\n    type.UMLEnumeration              = UMLEnumeration;\n    type.UMLDependency               = UMLDependency;\n    type.UMLAbstraction              = UMLAbstraction;\n    type.UMLRealization              = UMLRealization;\n    type.UMLGeneralization           = UMLGeneralization;\n    type.UMLInterfaceRealization     = UMLInterfaceRealization;\n    type.UMLAssociationEnd           = UMLAssociationEnd;\n    type.UMLAssociation              = UMLAssociation;\n    type.UMLAssociationClassLink     = UMLAssociationClassLink;\n    // Instances\n    type.UMLSlot                     = UMLSlot;\n    type.UMLInstance                 = UMLInstance;\n    type.UMLObject                   = UMLObject;\n    type.UMLArtifactInstance         = UMLArtifactInstance;\n    type.UMLComponentInstance        = UMLComponentInstance;\n    type.UMLNodeInstance             = UMLNodeInstance;\n    type.UMLLinkEnd                  = UMLLinkEnd;\n    type.UMLLink                     = UMLLink;\n    // Composite Structures\n    type.UMLPort                     = UMLPort;\n    type.UMLConnectorEnd             = UMLConnectorEnd;\n    type.UMLConnector                = UMLConnector;\n    type.UMLCollaboration            = UMLCollaboration;\n    type.UMLCollaborationUse         = UMLCollaborationUse;\n    type.UMLRoleBinding              = UMLRoleBinding;\n    // Components\n    type.UMLArtifact                 = UMLArtifact;\n    type.UMLComponent                = UMLComponent;\n    type.UMLSubsystem                = UMLSubsystem;\n    type.UMLComponentRealization     = UMLComponentRealization;\n    // Deployments\n    type.UMLNode                     = UMLNode;\n    type.UMLDeployment               = UMLDeployment;\n    type.UMLCommunicationPath        = UMLCommunicationPath;\n    // Use Cases\n    type.UMLExtensionPoint           = UMLExtensionPoint;\n    type.UMLUseCase                  = UMLUseCase;\n    type.UMLActor                    = UMLActor;\n    type.UMLInclude                  = UMLInclude;\n    type.UMLExtend                   = UMLExtend;\n    type.UMLUseCaseSubject           = UMLUseCaseSubject;\n    // State Machines\n    type.UMLStateMachine             = UMLStateMachine;\n    type.UMLRegion                   = UMLRegion;\n    type.UMLVertex                   = UMLVertex;\n    type.UMLConnectionPointReference = UMLConnectionPointReference;\n    type.UMLPseudostate              = UMLPseudostate;\n    type.UMLState                    = UMLState;\n    type.UMLFinalState               = UMLFinalState;\n    type.UMLTransition               = UMLTransition;\n    // Activity Graphs\n    type.UMLActivity                 = UMLActivity;\n    type.UMLPin                      = UMLPin;\n    type.UMLInputPin                 = UMLInputPin;\n    type.UMLOutputPin                = UMLOutputPin;\n    type.UMLExpansionNode            = UMLExpansionNode;\n    type.UMLActivityNode             = UMLActivityNode;\n    type.UMLAction                   = UMLAction;\n    type.UMLObjectNode               = UMLObjectNode;\n    type.UMLCentralBufferNode        = UMLCentralBufferNode;\n    type.UMLDataStoreNode            = UMLDataStoreNode;\n    type.UMLControlNode              = UMLControlNode;\n    type.UMLInitialNode              = UMLInitialNode;\n    type.UMLFinalNode                = UMLFinalNode;\n    type.UMLActivityFinalNode        = UMLActivityFinalNode;\n    type.UMLFlowFinalNode            = UMLFlowFinalNode;\n    type.UMLForkNode                 = UMLForkNode;\n    type.UMLJoinNode                 = UMLJoinNode;\n    type.UMLMergeNode                = UMLMergeNode;\n    type.UMLDecisionNode             = UMLDecisionNode;\n    type.UMLActivityGroup            = UMLActivityGroup;\n    type.UMLActivityPartition        = UMLActivityPartition;\n    type.UMLInterruptibleActivityRegion = UMLInterruptibleActivityRegion;\n    type.UMLStructuredActivityNode   = UMLStructuredActivityNode;\n    type.UMLExpansionRegion          = UMLExpansionRegion;\n    type.UMLExceptionHandler         = UMLExceptionHandler;\n    type.UMLActivityEdge             = UMLActivityEdge;\n    type.UMLControlFlow              = UMLControlFlow;\n    type.UMLObjectFlow               = UMLObjectFlow;\n    type.UMLActivityInterrupt        = UMLActivityInterrupt;\n    // Interactions\n    type.UMLInteractionFragment      = UMLInteractionFragment;\n    type.UMLInteraction              = UMLInteraction;\n    type.UMLStateInvariant           = UMLStateInvariant;\n    type.UMLContinuation             = UMLContinuation;\n    type.UMLInteractionOperand       = UMLInteractionOperand;\n    type.UMLCombinedFragment         = UMLCombinedFragment;\n    type.UMLInteractionUse           = UMLInteractionUse;\n    type.UMLMessageEndpoint          = UMLMessageEndpoint;\n    type.UMLLifeline                 = UMLLifeline;\n    type.UMLGate                     = UMLGate;\n    type.UMLEndpoint                 = UMLEndpoint;\n    type.UMLMessage                  = UMLMessage;\n    // Profiles\n    type.UMLProfile                  = UMLProfile;\n    type.UMLImage                    = UMLImage;\n    type.UMLStereotype               = UMLStereotype;\n    type.UMLMetaClass                = UMLMetaClass;\n    type.UMLExtension                = UMLExtension;\n\n    // Public API\n    exports.VK_PUBLIC          = VK_PUBLIC;\n    exports.VK_PROTECTED       = VK_PROTECTED;\n    exports.VK_PRIVATE         = VK_PRIVATE;\n    exports.VK_PACKAGE         = VK_PACKAGE;\n\n    exports.AK_NONE            = AK_NONE;\n    exports.AK_SHARED          = AK_SHARED;\n    exports.AK_COMPOSITE       = AK_COMPOSITE;\n\n    exports.DK_IN              = DK_IN;\n    exports.DK_INOUT           = DK_INOUT;\n    exports.DK_OUT             = DK_OUT;\n    exports.DK_RETURN          = DK_RETURN;\n\n    exports.CCK_SEQUENTIAL     = CCK_SEQUENTIAL;\n    exports.CCK_GUARDED        = CCK_GUARDED;\n    exports.CCK_CONCURRENT     = CCK_CONCURRENT;\n\n    exports.CK_ASSEMBLY        = CK_ASSEMBLY;\n    exports.CK_DELEGATION      = CK_DELEGATION;\n\n    exports.MS_SYNCHCALL       = MS_SYNCHCALL;\n    exports.MS_ASYNCHCALL      = MS_ASYNCHCALL;\n    exports.MS_ASYNCHSIGNAL    = MS_ASYNCHSIGNAL;\n    exports.MS_CREATEMESSAGE   = MS_CREATEMESSAGE;\n    exports.MS_DELETEMESSAGE   = MS_DELETEMESSAGE;\n    exports.MS_REPLY           = MS_REPLY;\n\n    exports.PSK_INITIAL         = PSK_INITIAL;\n    exports.PSK_DEEPHISTORY     = PSK_DEEPHISTORY;\n    exports.PSK_SHALLOWHISTORY  = PSK_SHALLOWHISTORY;\n    exports.PSK_JOIN            = PSK_JOIN;\n    exports.PSK_FORK            = PSK_FORK;\n    exports.PSK_JUNCTION        = PSK_JUNCTION;\n    exports.PSK_CHOICE          = PSK_CHOICE;\n    exports.PSK_ENTRYPOINT      = PSK_ENTRYPOINT;\n    exports.PSK_EXITPOINT       = PSK_EXITPOINT;\n    exports.PSK_TERMINATE       = PSK_TERMINATE;\n\n    exports.TK_EXTERNAL         = TK_EXTERNAL;\n    exports.TK_INTERNAL         = TK_INTERNAL;\n    exports.TK_LOCAL            = TK_LOCAL;\n\n    exports.EK_SIGNAL           = EK_SIGNAL;\n    exports.EK_CALL             = EK_CALL;\n    exports.EK_CHANGE           = EK_CHANGE;\n    exports.EK_TIME             = EK_TIME;\n    exports.EK_ANYRECEIVE       = EK_ANYRECEIVE;\n\n    exports.IOK_ALT            = IOK_ALT;\n    exports.IOK_OPT            = IOK_OPT;\n    exports.IOK_PAR            = IOK_PAR;\n    exports.IOK_LOOP           = IOK_LOOP;\n    exports.IOK_CRITICAL       = IOK_CRITICAL;\n    exports.IOK_NEG            = IOK_NEG;\n    exports.IOK_ASSERT         = IOK_ASSERT;\n    exports.IOK_STRICT         = IOK_STRICT;\n    exports.IOK_SEQ            = IOK_SEQ;\n    exports.IOK_IGNORE         = IOK_IGNORE;\n    exports.IOK_CONSIDER       = IOK_CONSIDER;\n\n    exports.ACK_OPAQUE       = ACK_OPAQUE;\n    exports.ACK_CREATE       = ACK_CREATE;\n    exports.ACK_DESTROY      = ACK_DESTROY;\n    exports.ACK_READ         = ACK_READ;\n    exports.ACK_WRITE        = ACK_WRITE;\n    exports.ACK_INSERT       = ACK_INSERT;\n    exports.ACK_DELETE       = ACK_DELETE;\n    exports.ACK_SENDSIGNAL   = ACK_SENDSIGNAL;\n    exports.ACK_ACCEPTSIGNAL = ACK_ACCEPTSIGNAL;\n    exports.ACK_TRIGGEREVENT = ACK_TRIGGEREVENT;\n    exports.ACK_ACCEPTEVENT  = ACK_ACCEPTEVENT;\n    exports.ACK_STRUCTURED   = ACK_STRUCTURED;\n    exports.ACK_TIMEEVENT    = ACK_TIMEEVENT;\n\n    exports.ONOK_UNORDERED = ONOK_UNORDERED;\n    exports.ONOK_ORDERED   = ONOK_ORDERED;\n    exports.ONOK_LIFO      = ONOK_LIFO;\n    exports.ONOK_FIFO      = ONOK_FIFO;\n\n    exports.EK_PARALLEL  = EK_PARALLEL;\n    exports.EK_ITERATIVE = EK_ITERATIVE;\n    exports.EK_STREAM    = EK_STREAM;\n\n    exports.SD_NONE             = SD_NONE;\n    exports.SD_LABEL            = SD_LABEL;\n    exports.SD_DECORATION       = SD_DECORATION;\n    exports.SD_DECORATION_LABEL = SD_DECORATION_LABEL;\n    exports.SD_ICON             = SD_ICON;\n    exports.SD_ICON_LABEL       = SD_ICON_LABEL;\n\n});\n\n//# sourceURL=/uml/UML.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, evil: true, loopfunc: true */\n/*global define, $, _, window, meta, type, appshell */\n\ndefine(\'uml/UMLDiagram\',[\'require\',\'exports\',\'module\',\'core/Global\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Toolkit\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/Core\',\'core/MetaModelManager\',\'core/PreferenceManager\',\'core/Repository\',\'uml/UML\'],function (require, exports, module) {\n    "use strict";\n\n    var global            = require("core/Global").global,\n        Graphics          = require("core/Graphics"),\n        Point             = require("core/Graphics").Point,\n        Rect              = require("core/Graphics").Rect,\n        Color             = require("core/Graphics").Color,\n        Font              = require("core/Graphics").Font,\n        ZoomFactor        = require("core/Graphics").ZoomFactor,\n        GridFactor        = require("core/Graphics").GridFactor,\n        Points            = require("core/Graphics").Points,\n        Coord             = require("core/Graphics").Coord,\n        Canvas            = require("core/Graphics").Canvas,\n        Toolkit           = require("core/Toolkit"),\n        Core              = require("core/Core"),\n        Diagram           = require("core/Core").Diagram,\n        Cursor            = require("core/Core").Cursor,\n        MouseEvent        = require("core/Core").MouseEvent,\n        NodeView          = require("core/Core").NodeView,\n        EdgeView          = require("core/Core").EdgeView,\n        LabelView         = require("core/Core").LabelView,\n        ParasiticView     = require("core/Core").ParasiticView,\n        NodeParasiticView = require("core/Core").NodeParasiticView,\n        EdgeParasiticView = require("core/Core").EdgeParasiticView,\n        NodeLabelView     = require("core/Core").NodeLabelView,\n        EdgeLabelView     = require("core/Core").EdgeLabelView,\n        NodeNodeView      = require("core/Core").NodeNodeView,\n        EdgeNodeView      = require("core/Core").EdgeNodeView,\n        MetaModelManager  = require("core/MetaModelManager"),\n        PreferenceManager = require("core/PreferenceManager"),\n        Repository        = require("core/Repository"),\n        UML               = require("uml/UML");\n\n    /**\n     * View Constants\n     * @private\n     * @const {number}\n     */\n    var SHADOW_OFFSET = 7,\n        SHADOW_ALPHA = 0.2,\n        SHADOW_COLOR = Color.LIGHT_GRAY,\n\n        // When use a sequence of labels, leave space after LabelView\n        LABEL_INTERVAL = 3,\n\n        COMPARTMENT_ITEM_INTERVAL = 2,\n        COMPARTMENT_LEFT_PADDING = 5,\n        COMPARTMENT_RIGHT_PADDING = 5,\n        COMPARTMENT_TOP_PADDING = 5,\n        COMPARTMENT_BOTTOM_PADDING = 5,\n\n        DECORATION_ICON_WIDTH = 24,\n        DECORATION_ICON_HEIGHT = 24,\n\n        ICONICVIEW_ICONMINWIDTH = 24,\n        ICONICVIEW_ICONMINHEIGHT = 24,\n\n        TEMPLATEPARAMETERCOMPARTMENT_OVERLAP = 5,\n        TEMPLATEPARAMETERCOMPARTMENT_LEFT_MARGIN = 20,\n        TEMPLATEPARAMETERCOMPARTMENT_RIGHT_OCCUPY = 10,\n\n        CLASS_ACTIVE_VERTLINE_WIDTH = 3,\n\n        PACKAGE_MINWIDTH = 50,\n        PACKAGE_MINHEIGHT = 40,\n        PACKAGE_TAB_HEIGHT = 15,\n\n        INTERFACE_ICONMINWIDTH = 24,\n        INTERFACE_ICONMINHEIGHT = 24,\n\n        PORT_MINWIDTH = 14,\n        PORT_MINHEIGHT = 14,\n\n        ACTOR_ICON_MINHEIGHT = 55,\n        ACTOR_ICON_MINWIDTH = 26,\n        ACTOR_RATIO_PERCENT = (ACTOR_ICON_MINWIDTH / ACTOR_ICON_MINHEIGHT) * 100,\n\n        USECASE_MINWIDTH = 70,\n        USECASE_MINHEIGHT = 32,\n        USECASE_ICON_MINWIDTH = 70,\n        USECASE_ICON_MINHEIGHT = 32,\n        USECASE_RATIO_PERCENT = (USECASE_ICON_MINWIDTH / USECASE_ICON_MINHEIGHT) * 100,\n\n        ARTIFACT_ICON_MINWIDTH = 40,\n        ARTIFACT_ICON_MINHEIGHT = 50,\n        ARTIFACT_RATIO_PERCENT = (ARTIFACT_ICON_MINWIDTH / ARTIFACT_ICON_MINHEIGHT) * 100,\n\n        COMPONENT_MINWIDTH = 50,\n        COMPONENT_MINHEIGHT = 45,\n        COMPONENT_ICON_MINWIDTH = 50,\n        COMPONENT_ICON_MINHEIGHT = 45,\n        COMPONENT_RATIO_PERCENT = (COMPONENT_ICON_MINWIDTH / COMPONENT_ICON_MINHEIGHT) * 100,\n        COMPONENT_STATIC_MARGIN = 20,\n        COMPONENT_BALANCED_HEIGHT = 45,\n        COMPONENT_RECT_INDENT = 10,\n\n        NODE_MINWIDTH = 45,\n        NODE_MINHEIGHT = 45,\n        NODE_RATIO_PERCENT = (NODE_MINWIDTH / NODE_MINHEIGHT) * 100,\n        NODE_STATIC_MARGIN = 10,\n\n        // State Machines\n        STATE_MINWIDTH = 60,\n        STATE_MINHEIGHT = 40,\n        STATE_ROUND = 10,\n\n        REGION_MINWIDTH = 50,\n        REGION_MINHEIGHT = 50,\n\n        INITIALSTATE_MINWIDTHH = 20,\n        INITIALSTATE_MINHEIGHT = 20,\n        FINALSTATE_MINWIDTHH = 26,\n        FINALSTATE_MINHEIGHT = 26,\n        HISTORYSTATE_MINWIDTHH = 26,\n        HISTORYSTATE_MINHEIGHT = 26,\n        JOIN_MINLENGTH = 70,\n        JOIN_MINTHICK = 6,\n        FORK_MINLENGTH = 70,\n        FORK_MINTHICK = 6,\n        CHOICE_MINWIDTH = 23,\n        CHOICE_MINHEIGHT = 19,\n        JUNCTION_MINWIDTH = 15,\n        JUNCTION_MINHEIGHT = 15,\n        ENTRYPOINT_MINWIDTH = 15,\n        ENTRYPOINT_MINHEIGHT = 15,\n        EXITPOINT_MINWIDTH = 15,\n        EXITPOINT_MINHEIGHT = 15,\n        TERMINATE_MINWIDTH = 26,\n        TERMINATE_MINHEIGHT = 26,\n        SELFTRANSITION_DISTANCE = 30,\n\n        CONNECTIONPOINT_MINWIDTH = 14,\n        CONNECTIONPOINT_MINHEIGHT = 14,\n\n        // Activities\n        ACTION_MINWIDTH = 60,\n        ACTION_MINHEIGHT = 40,\n        ACTION_ROUND = 10,\n\n        PIN_MINWIDTH = 18,\n        PIN_MINHEIGHT = 18,\n\n        INITIALNODE_MINWIDTH = 20,\n        INITIALNODE_MINHEIGHT = 20,\n        ACTIVITYFINALNODE_MINWIDTH = 26,\n        ACTIVITYFINALNODE_MINHEIGHT = 26,\n        FLOWFINALNODE_MINWIDTH = 26,\n        FLOWFINALNODE_MINHEIGHT = 26,\n        FORKNODE_MINLENGTH = 70,\n        FORKNODE_MINTHICK = 6,\n        JOINNODE_MINLENGTH = 70,\n        JOINNODE_MINTHICK = 6,\n        MERGENODE_MINWIDTH = 23,\n        MERGENODE_MINHEIGHT = 19,\n        DECISIONNODE_MINWIDTH = 23,\n        DECISIONNODE_MINHEIGHT = 19,\n\n        SWIMLANE_HORIZ_LEFT = 50,\n        SWIMLANE_VERT_TOP = 50,\n        SWIMLANE_VERT_MINWIDTH = 50,\n        SWIMLANE_VERT_MINHEIGHT = 300,\n        SWIMLANE_HORIZ_MINWIDTH = SWIMLANE_VERT_MINHEIGHT,\n        SWIMLANE_HORIZ_MINHEIGHT = SWIMLANE_VERT_MINWIDTH,\n        SWIMLANE_HEADER_TOP_MARGIN = 4,\n        SWIMLANE_HEADER_BOTTOM_MARGIN = 4,\n        SWIMLANE_HEADER_LEFT_MARGIN = 10,\n        SWIMLANE_HEADER_RIGHT_MARGIN = 10,\n        SWIMLANE_PEN_WIDTH = 2,\n\n        // Interactions\n        COLLABORATION_MINHEIGHT = USECASE_MINHEIGHT,\n        COLLABORATION_MINWIDTH = USECASE_MINWIDTH,\n\n        ACTIVATION_MINWIDTH = 14,\n        ACTIVATION_MINHEIGHT = 25,\n        LIFELINE_MINWIDTH = 10, // should be even number\n        LIFELINE_MINHEIGHT = 50,\n        LIFELINE_TOP_POSITION = 40,\n        SEQ_OBJECT_MINHEIGHT = 40,\n        SEQ_OBJECT_MINWIDTH = 30,\n        MULTI_INSTANCE_MARGIN = 5,\n        SELF_MESSAGE_WIDTH = 30,\n        SELF_MESSAGE_HEIGHT = 20,\n        SEQ_OBJECT_MESSAGE_MIN_INTERVAL = 10,\n\n        STATEINVARIANT_MINWIDTH = 50,\n        STATEINVARIANT_MINHEIGHT = 20,\n\n        CONTINUATION_MINWIDTH = 50,\n        CONTINUATION_MINHEIGHT = 20,\n\n        FRAME_MINWIDTH = 80,\n        FRAME_MINHEIGHT = 50,\n        FRAME_CONTENT_MINWIDTH = 10,\n        FRAME_CONTENT_MINHEIGHT = 30,\n\n        COMBINEDFRAGMENT_MINWIDTH = FRAME_MINWIDTH,\n        COMBINEDFRAGMENT_MINHEIGHT = FRAME_MINHEIGHT,\n        COMBINEDFRAGMENT_CONTENT_MINWIDTH = FRAME_CONTENT_MINWIDTH,\n        COMBINEDFRAGMENT_CONTENT_MINHEIGHT = FRAME_CONTENT_MINHEIGHT,\n\n        INTERACTIONOPERAND_MINWIDTH = COMBINEDFRAGMENT_MINWIDTH,\n        INTERACTIONOPERAND_MINHEIGHT = COMBINEDFRAGMENT_CONTENT_MINHEIGHT,\n        INTERACTIONOPERAND_GUARD_HORZ_MARGIN = 20,\n        INTERACTIONOPERAND_GUARD_VERT_MARGIN = 15,\n\n        HYPERLINK_MINWIDTH = 100,\n        HYPERLINK_MINHEIGHT = 20,\n\n        CUSTOM_TEXT_MINWIDTH = 10,\n        CUSTOM_TEXT_MINHEIGHT = 10,\n\n        NOTE_MINWIDTH = 50,\n        NOTE_MINHEIGHT = 20,\n        NOTE_FOLDING_SIZE = 10;\n\n    function hasValue(value) {\n        return (value && value.length > 0);\n    }\n\n    function getValue(prefix, value, postfix) {\n        if (hasValue(value)) {\n            return prefix + value + postfix;\n        } else {\n            return "";\n        }\n    }\n\n    function getTypeName(prefix, type) {\n        if (_.isString(type) && (type.length > 0)) {\n            return prefix + type;\n        } else if (type instanceof global.type.Model) {\n            return prefix + type.name;\n        } else {\n            return "";\n        }\n    }\n\n    function getTheta(x1, y1, x2, y2) {\n        var x = x1 - x2;\n        var y = y1 - y2;\n        var th = Math.atan(Math.abs(y) / (Math.abs(x) + 0.0000000001));\n        if (x > 0) {\n            if (y > 0) {\n                th = Math.PI - th;\n            } else {\n                th = th + Math.PI;\n            }\n        } else if (y < 0) {\n            th = 2 * Math.PI - th;\n        }\n        return th;\n    }\n\n    /**\n     * Draw image\n     * @private\n     * @param {Canvas} canvas\n     * @param {Rect} rect\n     * @param {type.UMLImage} image\n     */\n    function drawImage(canvas, rect, image) {\n        if (image && image.content) {\n            try {\n                eval(image.content);\n            } catch (err) {\n                console.log(err);\n            }\n        }\n    }\n\n    /**************************************************************************\n     *                                                                        *\n     *                               COMMON VIEWS                             *\n     *                                                                        *\n     *************************************************************************/\n\n    /**\n     * UMLDiagram\n     * @constructor\n     * @extends Diagram\n     */\n    function UMLDiagram() {\n        Diagram.apply(this, arguments);\n        this.name = null;\n    }\n    // inherits from Diagram\n    UMLDiagram.prototype = Object.create(Diagram.prototype);\n    UMLDiagram.prototype.constructor = UMLDiagram;\n\n    UMLDiagram.prototype.getDisplayClassName = function () {\n        var name = this.getClassName();\n        return name.substring(3, name.length);\n    };\n\n    /**\n     * UMLCompartmentView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLCompartmentView() {\n        NodeView.apply(this, arguments);\n        this.selectable = Core.SK_PROPAGATE;\n        this.parentStyle = true;\n\n        /* temporal */\n        this._leftPadding   = COMPARTMENT_LEFT_PADDING;\n        this._rightPadding  = COMPARTMENT_RIGHT_PADDING;\n        this._topPadding    = COMPARTMENT_TOP_PADDING;\n        this._bottomPadding = COMPARTMENT_BOTTOM_PADDING;\n        this._itemInterval  = COMPARTMENT_ITEM_INTERVAL;\n    }\n    // inherits from NodeView\n    UMLCompartmentView.prototype = Object.create(NodeView.prototype);\n    UMLCompartmentView.prototype.constructor = UMLCompartmentView;\n\n    /**\n     * size\n     */\n    UMLCompartmentView.prototype.size = function (canvas) {\n        var i, len, w = 0, h = this._topPadding;\n        for (i = 0, len = this.subViews.length; i < len; i++) {\n            var item = this.subViews[i];\n            if (item.parentStyle) {\n                item.font.size = item._parent.font.size;\n            }\n            item.size(canvas);\n            if (item.visible) {\n                if (w < item.minWidth) {\n                    w = item.minWidth;\n                }\n                if (i > 0) {\n                    h += this._itemInterval;\n                }\n                h += item.minHeight;\n            }\n        }\n        this.minWidth = w + this._leftPadding + this._rightPadding;\n        this.minHeight = h + this._bottomPadding;\n        this.sizeConstraints();\n    };\n\n    /**\n     * arrange\n     */\n    UMLCompartmentView.prototype.arrange = function (canvas) {\n        var i, len, h = this._topPadding;\n        for (i = 0, len = this.subViews.length; i < len; i++) {\n            var item = this.subViews[i];\n            if (item.visible) {\n                if (i > 0) { h += this._itemInterval; }\n                item.left = this.left + this._leftPadding;\n                item.top = this.top + h;\n                item.width = this.width - this._leftPadding - this._rightPadding;\n                h += item.height;\n            }\n            item.arrange(canvas);\n        }\n        h += this._bottomPadding;\n        this.height = h;\n        this.sizeConstraints();\n    };\n\n\n    /**\n     * UMLNameCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLNameCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n        this.selectable = Core.SK_NO;\n\n        /** @member {LabelView} */\n        this.stereotypeLabel = new LabelView();\n        this.stereotypeLabel.horizontalAlignment = Graphics.AL_CENTER;\n        this.stereotypeLabel.parentStyle = true;\n        this.addSubView(this.stereotypeLabel);\n\n        /** @member {LabelView} */\n        this.nameLabel = new LabelView();\n        this.nameLabel.horizontalAlignment = Graphics.AL_CENTER;\n        this.nameLabel.parentStyle = true;\n        this.addSubView(this.nameLabel);\n\n        /** @member {LabelView} */\n        this.namespaceLabel = new LabelView();\n        this.namespaceLabel.horizontalAlignment = Graphics.AL_CENTER;\n        this.namespaceLabel.parentStyle = true;\n        this.addSubView(this.namespaceLabel);\n\n        /** @member {LabelView} */\n        this.propertyLabel = new LabelView();\n        this.propertyLabel.horizontalAlignment = Graphics.AL_RIGHT;\n        this.propertyLabel.parentStyle = true;\n        this.addSubView(this.propertyLabel);\n\n        /** @member {boolean} */\n        this.wordWrap = false;\n    }\n    // inherits from UMLCompartmentView\n    UMLNameCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLNameCompartmentView.prototype.constructor = UMLNameCompartmentView;\n\n    UMLNameCompartmentView.prototype.update = function (canvas) {\n        this.namespaceLabel.font.size = this.font.size * 0.9;\n        if (this.model) {\n            if (this.model.isAbstract === true) {\n                this.nameLabel.font.style = Graphics.FS_BOLD_ITALIC;\n            } else {\n                this.nameLabel.font.style = Graphics.FS_BOLD;\n            }\n            if (this.model instanceof type.UMLInstance) {\n                this.nameLabel.underline = true;\n            }\n            this.nameLabel.wordWrap = this.wordWrap;\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n    UMLNameCompartmentView.prototype.size = function (canvas) {\n        UMLCompartmentView.prototype.size.call(this, canvas);\n        this.stereotypeLabel.height = this.stereotypeLabel.minHeight;\n        this.nameLabel.height = this.nameLabel.minHeight;\n        this.namespaceLabel.height = this.namespaceLabel.minHeight;\n        this.propertyLabel.height = this.propertyLabel.minHeight;\n        if (this._parent instanceof type.UMLGeneralNodeView && this._parent.stereotypeDisplay === UML.SD_DECORATION) {\n            this.minHeight = DECORATION_ICON_HEIGHT + COMPARTMENT_TOP_PADDING + COMPARTMENT_BOTTOM_PADDING;\n            this.sizeConstraints();\n        }\n    };\n\n\n    /**\n     * UMLAttributeView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLAttributeView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLAttributeView.prototype = Object.create(LabelView.prototype);\n    UMLAttributeView.prototype.constructor = UMLAttributeView;\n\n    UMLAttributeView.prototype.update = function (canvas) {\n        var options = {\n            showVisibility    : true,\n            stereotypeDisplay : UML.SD_LABEL,\n            showProperty      : true,\n            showType          : true,\n            showMultiplicity  : true\n        };\n        if (this._parent && this._parent._parent) {\n            options.showVisibility    = this._parent._parent.showVisibility;\n            options.stereotypeDisplay = this._parent._parent.stereotypeDisplay;\n            options.showProperty      = this._parent._parent.showProperty;\n            options.showType          = this._parent._parent.showType;\n            options.showMultiplicity  = this._parent._parent.showMultiplicity;\n        }\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            this.text = this.model.getString(options);\n            this.underline = (this.model.isStatic === true);\n        }\n        LabelView.prototype.update.call(this, canvas);\n    };\n\n    UMLAttributeView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLAttributeCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLAttributeCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLAttributeCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLAttributeCompartmentView.prototype.constructor = UMLAttributeCompartmentView;\n\n    UMLAttributeCompartmentView.prototype.update = function (canvas) {\n        if (this.model.attributes) {\n            var i, len, tempViews = this.subViews;\n            this.subViews = [];\n            for (i = 0, len = this.model.attributes.length; i < len; i++) {\n                var attr = this.model.attributes[i];\n                var attrView = _.find(tempViews, function (v) { return v.model === attr; });\n                if (!attrView) {\n                    attrView = new UMLAttributeView();\n                    attrView.model = attr;\n                    attrView._parent = this;\n                    // attrView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', attrView);\n                } else {\n                    this.addSubView(attrView);\n                }\n                attrView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLOperationView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLOperationView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLOperationView.prototype = Object.create(LabelView.prototype);\n    UMLOperationView.prototype.constructor = UMLOperationView;\n\n    UMLOperationView.prototype.update = function (canvas) {\n        var options = {\n            showVisibility         : true,\n            stereotypeDisplay      : UML.SD_LABEL,\n            showProperty           : true,\n            showType               : true,\n            showMultiplicity       : true,\n            showOperationSignature : true\n        };\n        if (this._parent && (this._parent._parent instanceof type.UMLClassifierView)) {\n            options.showVisibility         = this._parent._parent.showVisibility;\n            options.stereotypeDisplay      = this._parent._parent.stereotypeDisplay;\n            options.showProperty           = this._parent._parent.showProperty;\n            options.showType               = this._parent._parent.showType;\n            options.showMultiplicity       = this._parent._parent.showMultiplicity;\n            options.showOperationSignature = this._parent._parent.showOperationSignature;\n        }\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            this.text = this.model.getString(options);\n            this.underline = (this.model.isStatic === true);\n            if (this.model.isAbstract) {\n                this.font.style = Graphics.FS_ITALIC;\n            } else {\n                this.font.style = Graphics.FS_NORMAL;\n            }\n        }\n        LabelView.prototype.update.call(this, canvas);\n    };\n\n    UMLOperationView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLOperationCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLOperationCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLOperationCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLOperationCompartmentView.prototype.constructor = UMLOperationCompartmentView;\n\n    UMLOperationCompartmentView.prototype.update = function (canvas) {\n        if (this.model.operations) {\n            var i, len, tempViews = this.subViews;\n            this.subViews = [];\n            for (i = 0, len = this.model.operations.length; i < len; i++) {\n                var op = this.model.operations[i];\n                var opView = _.find(tempViews, function (v) { return v.model === op; });\n                if (!opView) {\n                    opView = new UMLOperationView();\n                    opView.model = op;\n                    opView._parent = this;\n                    // opView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', opView);\n                } else {\n                    this.addSubView(opView);\n                }\n                opView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLReceptionView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLReceptionView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLReceptionView.prototype = Object.create(LabelView.prototype);\n    UMLReceptionView.prototype.constructor = UMLReceptionView;\n\n    UMLReceptionView.prototype.update = function (canvas) {\n        var options = {\n            showVisibility         : true,\n            stereotypeDisplay      : UML.SD_LABEL,\n            showProperty           : true,\n            showType               : true,\n            showOperationSignature : true\n        };\n        if (this._parent && (this._parent._parent instanceof type.UMLClassifierView)) {\n            options.showVisibility         = this._parent._parent.showVisibility;\n            options.stereotypeDisplay      = this._parent._parent.stereotypeDisplay;\n            options.showProperty           = this._parent._parent.showProperty;\n            options.showType               = this._parent._parent.showType;\n            options.showOperationSignature = this._parent._parent.showOperationSignature;\n        }\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            this.text = this.model.getString(options);\n            this.underline = (this.model.isStatic === true);\n        }\n        LabelView.prototype.update.call(this, canvas);\n    };\n\n    UMLReceptionView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLReceptionCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLReceptionCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLReceptionCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLReceptionCompartmentView.prototype.constructor = UMLReceptionCompartmentView;\n\n    UMLReceptionCompartmentView.prototype.update = function (canvas) {\n        if (this.model.receptions) {\n            var i, len, tempViews = this.subViews;\n            this.subViews = [];\n            for (i = 0, len = this.model.receptions.length; i < len; i++) {\n                var rcp = this.model.receptions[i];\n                var rcpView = _.find(tempViews, function (v) { return v.model === rcp; });\n                if (!rcpView) {\n                    rcpView = new UMLReceptionView();\n                    rcpView.model = rcp;\n                    rcpView._parent = this;\n                    // opView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', rcpView);\n                } else {\n                    this.addSubView(rcpView);\n                }\n                rcpView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLTemplateParameterView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLTemplateParameterView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLTemplateParameterView.prototype = Object.create(LabelView.prototype);\n    UMLTemplateParameterView.prototype.constructor = UMLTemplateParameterView;\n\n    UMLTemplateParameterView.prototype.update = function (canvas) {\n        var options = {\n            showType : true\n        };\n        if (this._parent && (this._parent._parent instanceof type.UMLClassifierView)) {\n            options.showType = this._parent._parent.showType;\n        }\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            this.text = this.model.getString(options);\n        }\n        LabelView.prototype.update.call(this, canvas);\n    };\n\n    UMLTemplateParameterView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLTemplateParameterCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLTemplateParameterCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLTemplateParameterCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLTemplateParameterCompartmentView.prototype.constructor = UMLTemplateParameterCompartmentView;\n\n    UMLTemplateParameterCompartmentView.prototype.update = function (canvas) {\n        if (this.model.templateParameters) {\n            var i, len, tempViews = this.subViews;\n            this.subViews = [];\n            for (i = 0, len = this.model.templateParameters.length; i < len; i++) {\n                var tp = this.model.templateParameters[i];\n                var tpView = _.find(tempViews, function (v) { return v.model === tp; });\n                if (!tpView) {\n                    tpView = new UMLTemplateParameterView();\n                    tpView.model = tp;\n                    tpView._parent = this;\n                    // tpView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', tpView);\n                } else {\n                    this.addSubView(tpView);\n                }\n                tpView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n    UMLTemplateParameterCompartmentView.prototype.drawShadow = function (canvas) {\n        canvas.storeState();\n        canvas.alpha = SHADOW_ALPHA;\n        canvas.fillColor = SHADOW_COLOR;\n        canvas.fillRect(\n            this.left + SHADOW_OFFSET,\n            this.top + SHADOW_OFFSET,\n            this.getRight() + SHADOW_OFFSET,\n            this.getBottom() + SHADOW_OFFSET\n        );\n        canvas.restoreState();\n    };\n\n    UMLTemplateParameterCompartmentView.prototype.drawObject = function (canvas) {\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom(), [3]);\n    };\n\n\n    /**\n     * UMLGeneralNodeView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLGeneralNodeView() {\n        NodeView.apply(this, arguments);\n\n        /** @member {string} */\n        this.stereotypeDisplay = UML.SD_LABEL;\n\n        /** @member {boolean} */\n        this.showVisibility = true;\n\n        /** @member {boolean} */\n        this.showNamespace = false;\n\n        /** @member {boolean} */\n        this.showProperty = true;\n\n        /** @member {boolean} */\n        this.showType = true;\n\n        /** @member {boolean} */\n        this.wordWrap = false;\n\n        /** @member {UMLNameCompartmentView} */\n        this.nameCompartment = new UMLNameCompartmentView();\n        this.nameCompartment.parentStyle = true;\n        this.addSubView(this.nameCompartment);\n\n        /* temporal */\n        this.mainRect = new Rect(0, 0, 0, 0);\n        this.iconRect = new Rect(0, 0, 0, 0);\n        this.iconRatio = 100;\n    }\n    // inherits from NodeView\n    UMLGeneralNodeView.prototype = Object.create(NodeView.prototype);\n    UMLGeneralNodeView.prototype.constructor = UMLGeneralNodeView;\n\n    UMLGeneralNodeView.prototype.hasStereotypeIcon = function () {\n        return (this.model && this.model.stereotype && this.model.stereotype.icon && this.model.stereotype.icon.content.trim().length > 0);\n    };\n\n    UMLGeneralNodeView.prototype.computeIconRect = function (rect, ratioPercent) {\n        var rr = rect.getRatioPercent(), ir = ratioPercent,\n            x = 0, y = 0, h = 0, w = 0;\n        if (rr >= ir) {\n            h = rect.getHeight();\n            w = h * ir / 100;\n            x = rect.x1 + (rect.x2 - rect.x1 - w) / 2;\n            y = rect.y1;\n        } else {\n            w = rect.getWidth();\n            h = w * 100 / ir;\n            y = rect.y1 + (rect.y2 - rect.y1 - h) / 2;\n            x = rect.x1;\n        }\n        return new Rect(x, y, x + w, y + h);\n    };\n\n    UMLGeneralNodeView.prototype.drawIcon = function (canvas, rect) {\n        if (this.hasStereotypeIcon()) {\n            var ratioRect = this.computeIconRect(rect, (this.model.stereotype.icon.width / this.model.stereotype.icon.height) * 100);\n            drawImage(canvas, ratioRect, this.model.stereotype.icon);\n        } else {\n            canvas.rect(rect.x1, rect.y1, rect.x2, rect.y2);\n            canvas.line(rect.x1, rect.y1, rect.x2, rect.y2);\n            canvas.line(rect.x2, rect.y1, rect.x1, rect.y2);\n        }\n    };\n\n    UMLGeneralNodeView.prototype.drawDecorationIcon = function (canvas, rect) {\n        this.drawIcon(canvas, rect);\n    };\n\n    UMLGeneralNodeView.prototype.getAllCompartments = function () {\n        var i, len, comps = [];\n        for (i = 0, len = this.subViews.length; i < len; i++) {\n            var v = this.subViews[i];\n            if (v instanceof UMLCompartmentView) {\n                comps.push(v);\n            }\n        }\n        return comps;\n    };\n\n    UMLGeneralNodeView.prototype.getSizeOfAllCompartments = function (canvas) {\n        var comps = this.getAllCompartments();\n        var i, len, w = 0, h = 0;\n        for (i = 0, len = comps.length; i < len; i++) {\n            var comp = comps[i];\n            comp.size(canvas);\n            if (comp.visible) {\n                if (w < comp.minWidth) {\n                    w = comp.minWidth;\n                }\n                h += comp.minHeight;\n            }\n        }\n        return new Point(w, h);\n    };\n\n    UMLGeneralNodeView.prototype.arrangeAllCompartments = function (rect, canvas) {\n        var comps = this.getAllCompartments();\n        var i, len, _y = rect.y1;\n        for (i = 0, len = comps.length; i < len; i++) {\n            var comp = comps[i];\n            if (comp.visible) {\n                comp.left = rect.x1;\n                comp.top = _y;\n                comp.width = (rect.x2 - rect.x1) + 1;\n                comp.height = comp.minHeight;\n                comp.arrange(canvas);\n                _y += comp.height;\n            }\n        }\n    };\n\n    UMLGeneralNodeView.prototype.delimitContainingBoundary = function (canvas) {\n        var i, len, r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        for (i = 0, len = this.containedViews.length; i < len; i++) {\n            if (this.containedViews[i].containerExtending) {\n                var vr = this.containedViews[i].getBoundingBox(canvas);\n                vr = new Rect(vr.x1 - COMPARTMENT_LEFT_PADDING, vr.y1 - COMPARTMENT_TOP_PADDING,\n                              vr.x2 + COMPARTMENT_RIGHT_PADDING, vr.y2 + COMPARTMENT_BOTTOM_PADDING);\n                r = Coord.unionRect(r, vr);\n            }\n        }\n        this.left = r.x1;\n        this.top = r.y1;\n        this.setRight(r.x2);\n        this.setBottom(r.y2);\n    };\n\n    UMLGeneralNodeView.prototype.getStereotypeLabelText = function () {\n        return this.model.getStereotypeString();\n    };\n\n    UMLGeneralNodeView.prototype.update = function (canvas) {\n        if (this.model) {\n            // nameCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.nameCompartment.model !== this.model) {\n                Repository.bypassFieldAssign(this.nameCompartment, \'model\', this.model);\n            }\n            this.nameCompartment.stereotypeLabel.text = this.getStereotypeLabelText();\n            this.nameCompartment.nameLabel.text = this.model.name;\n            this.nameCompartment.namespaceLabel.text = this.model.getNamespaceString();\n            this.nameCompartment.propertyLabel.text = this.model.getPropertyString();\n            this.nameCompartment.namespaceLabel.visible = this.showNamespace;\n            this.nameCompartment.propertyLabel.visible = this.showProperty;\n            this.nameCompartment.wordWrap = this.wordWrap;\n            switch (this.stereotypeDisplay) {\n            case UML.SD_NONE:\n                this.nameCompartment.stereotypeLabel.visible = false;\n                break;\n            case UML.SD_LABEL:\n                this.nameCompartment.stereotypeLabel.visible = (this.nameCompartment.stereotypeLabel.text.length > 0);\n                break;\n            case UML.SD_DECORATION:\n                this.nameCompartment.stereotypeLabel.visible = false;\n                break;\n            case UML.SD_DECORATION_LABEL:\n                this.nameCompartment.stereotypeLabel.visible = true;\n                break;\n            case UML.SD_ICON:\n                this.nameCompartment.stereotypeLabel.visible = false;\n                break;\n            case UML.SD_ICON_LABEL:\n                this.nameCompartment.stereotypeLabel.visible = true;\n                break;\n            }\n            if (this.nameCompartment.propertyLabel.text.length === 0) {\n                this.nameCompartment.propertyLabel.visible = false;\n            }\n        }\n        NodeView.prototype.update.call(this, canvas);\n    };\n\n    UMLGeneralNodeView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.sizeCommon(canvas);\n        switch (this.stereotypeDisplay) {\n        case UML.SD_NONE:\n            this.sizeAsCanonicalForm(canvas, false);\n            break;\n        case UML.SD_LABEL:\n            this.sizeAsCanonicalForm(canvas, true);\n            break;\n        case UML.SD_DECORATION:\n            this.sizeAsDecorationForm(canvas, false);\n            break;\n        case UML.SD_DECORATION_LABEL:\n            this.sizeAsDecorationForm(canvas, true);\n            break;\n        case UML.SD_ICON:\n            this.sizeAsIconicForm(canvas, false);\n            break;\n        case UML.SD_ICON_LABEL:\n            this.sizeAsIconicForm(canvas, true);\n            break;\n        }\n        this.delimitContainingBoundary(canvas);\n    };\n\n    UMLGeneralNodeView.prototype.sizeCommon = function (canvas) {\n    };\n\n    UMLGeneralNodeView.prototype.sizeAsCanonicalForm = function (canvas, showLabel) {\n        var sz = this.getSizeOfAllCompartments(canvas);\n        this.minWidth = sz.x;\n        this.minHeight = sz.y;\n    };\n\n    UMLGeneralNodeView.prototype.sizeAsDecorationForm = function (canvas, showLabel) {\n        var sz = this.getSizeOfAllCompartments(canvas);\n        this.minWidth = Math.max(this.nameCompartment.minWidth + DECORATION_ICON_WIDTH + COMPARTMENT_RIGHT_PADDING + COMPARTMENT_LEFT_PADDING, sz.x);\n        this.minHeight = sz.y;\n    };\n\n    UMLGeneralNodeView.prototype.sizeAsIconicForm = function (canvas, showLabel) {\n        var sz = this.getSizeOfAllCompartments(canvas);\n        if (this.hasStereotypeIcon()) {\n            this.minWidth = Math.max(sz.x, this.model.stereotype.icon.width);\n            this.minHeight = this.model.stereotype.icon.height + sz.y;\n        } else {\n            this.minWidth = Math.max(sz.x, ICONICVIEW_ICONMINWIDTH);\n            this.minHeight = ICONICVIEW_ICONMINHEIGHT + sz.y;\n        }\n    };\n\n    UMLGeneralNodeView.prototype.arrangeObject = function (canvas) {\n        this.arrangeCommon(canvas);\n        switch (this.stereotypeDisplay) {\n        case UML.SD_NONE:\n            this.arrangeAsCanonicalForm(canvas, false);\n            break;\n        case UML.SD_LABEL:\n            this.arrangeAsCanonicalForm(canvas, true);\n            break;\n        case UML.SD_DECORATION:\n            this.arrangeAsDecorationForm(canvas, false);\n            break;\n        case UML.SD_DECORATION_LABEL:\n            this.arrangeAsDecorationForm(canvas, true);\n            break;\n        case UML.SD_ICON:\n            this.arrangeAsIconicForm(canvas, false);\n            break;\n        case UML.SD_ICON_LABEL:\n            this.arrangeAsIconicForm(canvas, true);\n            break;\n        }\n        NodeView.prototype.arrangeObject.call(this, canvas);\n    };\n\n    UMLGeneralNodeView.prototype.arrangeCommon = function (canvas) {\n         this.mainRect.setRect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n    UMLGeneralNodeView.prototype.arrangeAsCanonicalForm = function (canvas, showLabel) {\n        this.arrangeAllCompartments(this.mainRect, canvas);\n    };\n\n    UMLGeneralNodeView.prototype.arrangeAsDecorationForm = function (canvas, showLabel) {\n        this.arrangeAllCompartments(this.mainRect, canvas);\n        this.nameCompartment.width = this.width - DECORATION_ICON_WIDTH - COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING;\n        this.nameCompartment.arrange(canvas);\n    };\n\n    UMLGeneralNodeView.prototype.arrangeAsIconicForm = function (canvas, showLabel) {\n        var sz = this.getSizeOfAllCompartments(canvas);\n        var r  = new Rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2 - sz.y);\n        this.iconRect = this.computeIconRect(r, this.iconRatio);\n        var r2 = new Rect(this.mainRect.x1, this.mainRect.y1 + this.iconRect.getHeight(), this.mainRect.x2, this.mainRect.y2);\n        this.arrangeAllCompartments(r2, canvas);\n    };\n\n    UMLGeneralNodeView.prototype.drawShadow = function (canvas) {\n        canvas.storeState();\n        canvas.alpha = SHADOW_ALPHA;\n        canvas.lineColor = "#ffffff00";\n        canvas.fillColor = SHADOW_COLOR;\n        this.drawShadowCommon(canvas);\n        switch (this.stereotypeDisplay) {\n        case UML.SD_NONE:\n            this.drawShadowAsCanonicalForm(canvas, false);\n            break;\n        case UML.SD_LABEL:\n            this.drawShadowAsCanonicalForm(canvas, true);\n            break;\n        case UML.SD_DECORATION:\n            this.drawShadowAsDecorationForm(canvas, false);\n            break;\n        case UML.SD_DECORATION_LABEL:\n            this.drawShadowAsDecorationForm(canvas, true);\n            break;\n        case UML.SD_ICON:\n            this.drawShadowAsIconicForm(canvas, false);\n            break;\n        case UML.SD_ICON_LABEL:\n            this.drawShadowAsIconicForm(canvas, true);\n            break;\n        }\n        canvas.restoreState();\n        NodeView.prototype.drawShadow.call(this, canvas);\n    };\n\n    UMLGeneralNodeView.prototype.drawShadowCommon = function (canvas) {\n    };\n\n    UMLGeneralNodeView.prototype.drawShadowAsCanonicalForm = function (canvas, showLabel) {\n        canvas.fillRect(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET\n        );\n    };\n\n    UMLGeneralNodeView.prototype.drawShadowAsDecorationForm = function (canvas, showLabel) {\n        this.drawShadowAsCanonicalForm(canvas);\n    };\n\n    UMLGeneralNodeView.prototype.drawShadowAsIconicForm = function (canvas, showLabel) {\n        /*\n        canvas.fillRect(\n            this.iconRect.x1 + SHADOW_OFFSET,\n            this.iconRect.y1 + SHADOW_OFFSET,\n            this.iconRect.x2 + SHADOW_OFFSET,\n            this.iconRect.y2 + SHADOW_OFFSET\n        );\n        */\n    };\n\n    UMLGeneralNodeView.prototype.drawObject = function (canvas) {\n        this.drawCommon(canvas);\n        switch (this.stereotypeDisplay) {\n        case UML.SD_NONE:\n            this.drawAsCanonicalForm(canvas, false);\n            break;\n        case UML.SD_LABEL:\n            this.drawAsCanonicalForm(canvas, true);\n            break;\n        case UML.SD_DECORATION:\n            this.drawAsDecorationForm(canvas, false);\n            break;\n        case UML.SD_DECORATION_LABEL:\n            this.drawAsDecorationForm(canvas, true);\n            break;\n        case UML.SD_ICON:\n            this.drawAsIconicForm(canvas, false);\n            break;\n        case UML.SD_ICON_LABEL:\n            this.drawAsIconicForm(canvas, true);\n            break;\n        }\n        NodeView.prototype.drawObject.call(this, canvas);\n    };\n\n    UMLGeneralNodeView.prototype.drawCommon = function (canvas) {\n        if ((this.stereotypeDisplay !== UML.SD_ICON) && (this.stereotypeDisplay !== UML.SD_ICON_LABEL)) {\n            canvas.fillRect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n            canvas.rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n        }\n    };\n\n    UMLGeneralNodeView.prototype.drawAsCanonicalForm = function (canvas, showLabel) {\n    };\n\n    UMLGeneralNodeView.prototype.drawAsDecorationForm = function (canvas, showLabel) {\n        var r   = new Rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2),\n            icon_x = r.x2 - COMPARTMENT_RIGHT_PADDING - DECORATION_ICON_WIDTH,\n            icon_y  = r.y1 + COMPARTMENT_TOP_PADDING;\n        this.iconRect = this.computeIconRect(new Rect(icon_x, icon_y, icon_x + DECORATION_ICON_WIDTH, icon_y + DECORATION_ICON_HEIGHT), this.iconRatio);\n        this.drawDecorationIcon(canvas, this.iconRect);\n    };\n\n    UMLGeneralNodeView.prototype.drawAsIconicForm = function (canvas, showLabel) {\n        var _iconWidth  = this.iconRect.getWidth(),\n            _iconHeight = this.iconRect.getHeight(),\n            _x          = (this.left + this.getRight()) / 2;\n        this.drawIcon(canvas, new Rect(_x - (_iconWidth / 2), this.top, _x + (_iconWidth / 2), this.top + _iconHeight));\n    };\n\n\n    /**\n     * UMLFloatingNodeView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLFloatingNodeView() {\n        NodeView.apply(this, arguments);\n        this.containerChangeable = false;\n        this.containerExtending = false;\n\n        /** @member {NodeLabelView} */\n        this.nameLabel = new NodeLabelView();\n        this.nameLabel.distance = 20;\n        this.nameLabel.alpha = 3 * Math.PI / 4;\n        this.addSubView(this.nameLabel);\n\n        /** @member {NodeLabelView} */\n        this.stereotypeLabel = new NodeLabelView();\n        this.stereotypeLabel.distance = 35;\n        this.stereotypeLabel.alpha = 3 * Math.PI / 4;\n        this.addSubView(this.stereotypeLabel);\n\n        /** @member {NodeLabelView} */\n        this.propertyLabel = new NodeLabelView();\n        this.propertyLabel.distance = 20;\n        this.propertyLabel.alpha = -3 * Math.PI / 4;\n        this.addSubView(this.propertyLabel);\n\n        /** @member {boolean} */\n        this.showProperty = true;\n\n    }\n    // inherits from LabelView\n    UMLFloatingNodeView.prototype = Object.create(NodeView.prototype);\n    UMLFloatingNodeView.prototype.constructor = UMLFloatingNodeView;\n\n    UMLFloatingNodeView.prototype.update = function (canvas) {\n        NodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameLabel.text = this.model.name;\n            if (this.model.stereotype !== null) {\n                this.stereotypeLabel.text = this.model.getStereotypeString();\n            }\n            // propertyLabel\n            this.propertyLabel.text = this.model.getPropertyString();\n            this.propertyLabel.visible = (this.showProperty ? this.propertyLabel.text.length > 0 : false);\n            // nameLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.nameLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.nameLabel, \'model\', this.model);\n            }\n            // stereotypeLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.stereotypeLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.stereotypeLabel, \'model\', this.model);\n            }\n            // propertyLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.propertyLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.propertyLabel, \'model\', this.model);\n            }\n        }\n    };\n\n    /**\n     * Compute Junction Point\n     *\n     * @param {Rect} r\n     * @param {Point} p\n     * @return {Point}\n     */\n    UMLFloatingNodeView.prototype._junction2 = function (r, p) {\n        var c = new Point();\n        c.x = (r.x1 + r.x2) / 2;\n        c.y = (r.y1 + r.y2) / 2;\n        if ((c.x === p.x) || (c.y === p.y)) {\n            return Coord.orthoJunction(r, p);\n        }\n        var lean = (p.y - c.y) / (p.x - c.x);\n        // contact points\n        var cp = [];\n        cp[0] = new Point(r.x1, Math.round(lean * (r.x1 - c.x) + c.y)); // left\n        cp[1] = new Point(r.x2, Math.round(lean * (r.x2 - c.x) + c.y)); // right\n        cp[2] = new Point(Math.round((r.y1 - c.y) / lean + c.x), r.y1); // top\n        cp[3] = new Point(Math.round((r.y2 - c.y) / lean + c.x), r.y2); // bottom\n\n        var i;\n        if (Coord.ptInRect2(p, r)) {\n            var idx = 0;\n            var md = Math.sqrt(Math.square(cp[0].x - p.x) + Math.square(cp[0].y - p.y));\n            for (i = 1; i <= 3; i++) {\n                var d = Math.sqrt(Math.square(cp[i].x - p.x) + Math.square(cp[i].y - p.y));\n                if (d < md) {\n                    md = d;\n                    idx = i;\n                }\n            }\n            return cp[idx];\n        } else {\n            var cpRect = new Rect(c.x, c.y, p.x, p.y);\n            Coord.normalizeRect(cpRect);\n            c.x = cpRect.x1; c.y = cpRect.y1;\n            p.x = cpRect.x2; p.y = cpRect.y2;\n            i = -1;\n            do {\n                i++;\n            } while (!(((r.x1 <= cp[i].x) && (cp[i].x <= r.x2) &&\n                        (r.y1 <= cp[i].y) && (cp[i].y <= r.y2) &&\n                        (c.x <= cp[i].x)  && (cp[i].x <= p.x)  &&\n                        (c.y <= cp[i].y)  && (cp[i].y <= p.y)) || (i > 4)));\n            if (i > 3) {\n                return new Point((r.x1 + r.x2) / 2, (r.y1 + r.y2) / 2);\n            } else {\n                return cp[i];\n            }\n        }\n    };\n\n    UMLFloatingNodeView.prototype.arrange = function (canvas) {\n        this.nameLabel.visible = (this.nameLabel.text.length > 0);\n        this.stereotypeLabel.visible = (this.stereotypeLabel.text.length > 0);\n        NodeView.prototype.arrange.call(this, canvas);\n    };\n\n\n    /**\n     * UMLGeneralEdgeView\n     * @constructor\n     * @extends EdgeView\n     */\n    function UMLGeneralEdgeView() {\n        EdgeView.apply(this, arguments);\n\n        /** @member {string} */\n        this.stereotypeDisplay = UML.SD_LABEL;\n\n        /** @member {boolean} */\n        this.showVisibility = true;\n\n        /** @member {boolean} */\n        this.showProperty = true;\n\n        /** @member {EdgeLabelView} */\n        this.nameLabel = new EdgeLabelView();\n        this.nameLabel.hostEdge = this;\n        this.nameLabel.edgePosition = Core.EP_MIDDLE;\n        this.nameLabel.distance = 15;\n        this.nameLabel.alpha = Math.PI / 2;\n        this.addSubView(this.nameLabel);\n\n        /** @member {EdgeLabelView} */\n        this.stereotypeLabel = new EdgeLabelView();\n        this.stereotypeLabel.hostEdge = this;\n        this.stereotypeLabel.edgePosition = Core.EP_MIDDLE;\n        this.stereotypeLabel.distance = 30;\n        this.stereotypeLabel.alpha = Math.PI / 2;\n        this.addSubView(this.stereotypeLabel);\n\n        /** @member {EdgeLabelView} */\n        this.propertyLabel = new EdgeLabelView();\n        this.propertyLabel.hostEdge = this;\n        this.propertyLabel.edgePosition = Core.EP_MIDDLE;\n        this.propertyLabel.distance = 15;\n        this.propertyLabel.alpha = -Math.PI / 2;\n        this.addSubView(this.propertyLabel);\n    }\n    // inherits from EdgeView\n    UMLGeneralEdgeView.prototype = Object.create(EdgeView.prototype);\n    UMLGeneralEdgeView.prototype.constructor = UMLGeneralEdgeView;\n\n    UMLGeneralEdgeView.prototype.update = function (canvas) {\n        if (this.model) {\n            // nameLabel\n            this.nameLabel.visible = (this.model.name.length > 0);\n            if (this.model.name) {\n                this.nameLabel.text = this.model.getString(this.showVisibility);\n            }\n            // stereotypeLabel\n            this.stereotypeLabel.visible =\n                this.model.stereotype && (this.stereotypeDisplay === UML.SD_LABEL ||\n                                          this.stereotypeDisplay === UML.SD_DECORATION_LABEL ||\n                                          this.stereotypeDisplay === UML.SD_ICON_LABEL);\n            if (this.model.stereotype) {\n                this.stereotypeLabel.text = this.model.getStereotypeString();\n            }\n            // propertyLabel\n            this.propertyLabel.text = this.model.getPropertyString();\n            this.propertyLabel.visible = (this.showProperty ? this.propertyLabel.text.length > 0 : false);\n            // nameLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.nameLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.nameLabel, \'model\', this.model);\n            }\n            // stereotypeLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.stereotypeLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.stereotypeLabel, \'model\', this.model);\n            }\n            // propertyLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.propertyLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.propertyLabel, \'model\', this.model);\n            }\n        }\n        EdgeView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLClassifierView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLClassifierView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = true;\n\n        /** @member {boolean} */\n        this.suppressAttributes = false;\n\n        /** @member {boolean} */\n        this.suppressOperations = false;\n\n        /** @member {boolean} */\n        this.suppressReceptions = true;\n\n        /** @member {boolean} */\n        this.showMultiplicity = true;\n\n        /** @member {boolean} */\n        this.showOperationSignature = true;\n\n        /** @member {UMLAttributeCompartmentView} */\n        this.attributeCompartment = new UMLAttributeCompartmentView();\n        this.attributeCompartment.parentStyle = true;\n        this.addSubView(this.attributeCompartment);\n\n        /** @member {UMLOperationCompartmentView} */\n        this.operationCompartment = new UMLOperationCompartmentView();\n        this.operationCompartment.parentStyle = true;\n        this.addSubView(this.operationCompartment);\n\n        /** @member {UMLReceptionCompartmentView} */\n        this.receptionCompartment = new UMLReceptionCompartmentView();\n        this.receptionCompartment.parentStyle = true;\n        this.addSubView(this.receptionCompartment);\n\n        /** @member {UMLTemplateParameterCompartmentView} */\n        this.templateParameterCompartment = new UMLTemplateParameterCompartmentView();\n        this.templateParameterCompartment.parentStyle = true;\n        this.addSubView(this.templateParameterCompartment);\n    }\n    // inherits from UMLGeneralNodeView\n    UMLClassifierView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLClassifierView.prototype.constructor = UMLClassifierView;\n\n    UMLClassifierView.prototype.getAllCompartments = function () {\n        return [\n            this.nameCompartment,\n            this.attributeCompartment,\n            this.operationCompartment,\n            this.receptionCompartment\n        ];\n    };\n\n    UMLClassifierView.prototype.update = function (canvas) {\n        // attributeCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.attributeCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.attributeCompartment, \'model\', this.model);\n        }\n        // operationCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.operationCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.operationCompartment, \'model\', this.model);\n        }\n        // receptionCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.receptionCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.receptionCompartment, \'model\', this.model);\n        }\n        // templateParameterCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.templateParameterCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.templateParameterCompartment, \'model\', this.model);\n        }\n        if (this.model) {\n            if (this.model.templateParameters && this.model.templateParameters.length > 0) {\n                this.templateParameterCompartment.visible = true;\n            } else {\n                this.templateParameterCompartment.visible = false;\n            }\n        }\n        if ((this.stereotypeDisplay === UML.SD_ICON) || (this.stereotypeDisplay === UML.SD_ICON_LABEL)) {\n            this.templateParameterCompartment.visible = false;\n        }\n        this.attributeCompartment.visible = !this.suppressAttributes;\n        this.operationCompartment.visible = !this.suppressOperations;\n        this.receptionCompartment.visible = !this.suppressReceptions;\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n    };\n\n    UMLClassifierView.prototype.sizeObject = function (canvas) {\n        UMLGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        if (this.templateParameterCompartment.visible) {\n            this.templateParameterCompartment.size(canvas);\n            this.templateParameterCompartment.width = this.templateParameterCompartment.minWidth;\n            this.templateParameterCompartment.height = this.templateParameterCompartment.minHeight;\n            this.minWidth = Math.max(this.minWidth, this.templateParameterCompartment.width) + TEMPLATEPARAMETERCOMPARTMENT_LEFT_MARGIN + TEMPLATEPARAMETERCOMPARTMENT_RIGHT_OCCUPY;\n            this.minHeight = this.minHeight + this.templateParameterCompartment.height - TEMPLATEPARAMETERCOMPARTMENT_OVERLAP;\n        }\n    };\n\n\n    UMLClassifierView.prototype.arrangeCommon = function (canvas) {\n        UMLGeneralNodeView.prototype.arrangeCommon.call(this, canvas);\n        if (this.templateParameterCompartment.visible) {\n            this.templateParameterCompartment.left = this.getRight() - this.templateParameterCompartment.width;\n            this.templateParameterCompartment.top = this.top;\n            this.templateParameterCompartment.arrange(canvas);\n            var x1 = this.left,\n                y1 = this.top + this.templateParameterCompartment.height - TEMPLATEPARAMETERCOMPARTMENT_OVERLAP,\n                x2 = this.getRight() - TEMPLATEPARAMETERCOMPARTMENT_RIGHT_OCCUPY,\n                y2 = this.getBottom();\n            this.mainRect.setRect(x1, y1, x2, y2);\n        }\n    };\n\n    UMLClassifierView.prototype.drawObject = function (canvas) {\n        UMLGeneralNodeView.prototype.drawObject.call(this, canvas);\n        if (this.attributeCompartment.visible) {\n            canvas.line(\n                this.attributeCompartment.left,\n                this.attributeCompartment.top,\n                this.attributeCompartment.getRight(),\n                this.attributeCompartment.top);\n        }\n        if (this.operationCompartment.visible) {\n            canvas.line(\n                this.operationCompartment.left,\n                this.operationCompartment.top,\n                this.operationCompartment.getRight(),\n                this.operationCompartment.top);\n        }\n        if (this.receptionCompartment.visible) {\n            canvas.line(\n                this.receptionCompartment.left,\n                this.receptionCompartment.top,\n                this.receptionCompartment.getRight(),\n                this.receptionCompartment.top);\n        }\n    };\n\n\n    /**\n     * UMLUndirectedRelationshipView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLUndirectedRelationshipView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_FLAT;\n        this.lineMode = Core.LM_SOLID;\n\n        /** @member {boolean} */\n        this.showMultiplicity = true;\n\n        /** @member {boolean} */\n        this.showType = true;\n\n        /** @member {EdgeLabelView} */\n        this.tailRoleNameLabel = new EdgeLabelView();\n        this.tailRoleNameLabel.hostEdge = this;\n        this.tailRoleNameLabel.edgePosition = Core.EP_TAIL;\n        this.tailRoleNameLabel.alpha = Math.PI / 6;\n        this.tailRoleNameLabel.distance = 30;\n        this.addSubView(this.tailRoleNameLabel);\n\n        /** @member {EdgeLabelView} */\n        this.tailPropertyLabel = new EdgeLabelView();\n        this.tailPropertyLabel.hostEdge = this;\n        this.tailPropertyLabel.edgePosition = Core.EP_TAIL;\n        this.tailPropertyLabel.alpha = Math.PI / 4;\n        this.tailPropertyLabel.distance = 40;\n        this.addSubView(this.tailPropertyLabel);\n\n        /** @member {EdgeLabelView} */\n        this.tailMultiplicityLabel = new EdgeLabelView();\n        this.tailMultiplicityLabel.hostEdge = this;\n        this.tailMultiplicityLabel.edgePosition = Core.EP_TAIL;\n        this.tailMultiplicityLabel.alpha = -Math.PI / 6;\n        this.tailMultiplicityLabel.distance = 25;\n        this.addSubView(this.tailMultiplicityLabel);\n\n        /** @member {EdgeLabelView} */\n        this.headRoleNameLabel = new EdgeLabelView();\n        this.headRoleNameLabel.hostEdge = this;\n        this.headRoleNameLabel.edgePosition = Core.EP_HEAD;\n        this.headRoleNameLabel.alpha = -Math.PI / 6;\n        this.headRoleNameLabel.distance = 30;\n        this.addSubView(this.headRoleNameLabel);\n\n        /** @member {EdgeLabelView} */\n        this.headPropertyLabel = new EdgeLabelView();\n        this.headPropertyLabel.hostEdge = this;\n        this.headPropertyLabel.edgePosition = Core.EP_HEAD;\n        this.headPropertyLabel.alpha = -Math.PI / 4;\n        this.headPropertyLabel.distance = 40;\n        this.addSubView(this.headPropertyLabel);\n\n        /** @member {EdgeLabelView} */\n        this.headMultiplicityLabel = new EdgeLabelView();\n        this.headMultiplicityLabel.hostEdge = this;\n        this.headMultiplicityLabel.edgePosition = Core.EP_HEAD;\n        this.headMultiplicityLabel.alpha = Math.PI / 6;\n        this.headMultiplicityLabel.distance = 25;\n        this.addSubView(this.headMultiplicityLabel);\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLUndirectedRelationshipView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLUndirectedRelationshipView.prototype.constructor = UMLUndirectedRelationshipView;\n\n    UMLUndirectedRelationshipView.prototype.update = function (canvas) {\n        if (this.model) {\n            // RoleName Labels\n            this.tailRoleNameLabel.visible = (this.model.end1.name.length > 0);\n            if (this.model.end1.name) {\n                this.tailRoleNameLabel.text = this.model.end1.getString(this.showVisibility);\n            }\n            this.headRoleNameLabel.visible = (this.model.end2.name.length > 0);\n            if (this.model.end2.name) {\n                this.headRoleNameLabel.text = this.model.end2.getString(this.showVisibility);\n            }\n            // isDerived\n            if (this.model.end1.isDerived === true) {\n                this.tailRoleNameLabel.text = "/ " + this.tailRoleNameLabel.text;\n            }\n            if (this.model.end2.isDerived === true) {\n                this.headRoleNameLabel.text = "/ " + this.headRoleNameLabel.text;\n            }\n\n            // Property Labels\n            this.tailPropertyLabel.text = this.model.end1.getPropertyString();\n            this.headPropertyLabel.text = this.model.end2.getPropertyString();\n            this.tailPropertyLabel.visible = (this.showProperty ? this.tailPropertyLabel.text.length > 0 : false);\n            this.headPropertyLabel.visible = (this.showProperty ? this.headPropertyLabel.text.length > 0 : false);\n\n            // Multiplicity Labels\n            this.tailMultiplicityLabel.visible = (this.showMultiplicity && this.model.end1.multiplicity.length > 0);\n            if (this.model.end1.multiplicity) {\n                this.tailMultiplicityLabel.text = this.model.end1.multiplicity;\n            }\n            this.headMultiplicityLabel.visible = (this.showMultiplicity && this.model.end2.multiplicity.length > 0);\n            if (this.model.end2.multiplicity) {\n                this.headMultiplicityLabel.text = this.model.end2.multiplicity;\n            }\n            // Set End Styles\n            var tailNavigable = this.model.end1.navigable;\n            var headNavigable = this.model.end2.navigable;\n            if (tailNavigable && !headNavigable) {\n                this.tailEndStyle = Core.ES_STICK_ARROW;\n                this.headEndStyle = Core.ES_FLAT;\n                switch (this.model.end1.aggregation) {\n                case UML.AK_SHARED:\n                    this.tailEndStyle = Core.ES_ARROW_DIAMOND;\n                    break;\n                case UML.AK_COMPOSITE:\n                    this.tailEndStyle = Core.ES_ARROW_FILLED_DIAMOND;\n                    break;\n                }\n                switch (this.model.end2.aggregation) {\n                case UML.AK_SHARED:\n                    this.headEndStyle = Core.ES_DIAMOND;\n                    break;\n                case UML.AK_COMPOSITE:\n                    this.headEndStyle = Core.ES_FILLED_DIAMOND;\n                    break;\n                }\n            } else if (!tailNavigable && headNavigable) {\n                this.tailEndStyle = Core.ES_FLAT;\n                this.headEndStyle = Core.ES_STICK_ARROW;\n                switch (this.model.end1.aggregation) {\n                case UML.AK_SHARED:\n                    this.tailEndStyle = Core.ES_DIAMOND;\n                    break;\n                case UML.AK_COMPOSITE:\n                    this.tailEndStyle = Core.ES_FILLED_DIAMOND;\n                    break;\n                }\n                switch (this.model.end2.aggregation) {\n                case UML.AK_SHARED:\n                    this.headEndStyle = Core.ES_ARROW_DIAMOND;\n                    break;\n                case UML.AK_COMPOSITE:\n                    this.headEndStyle = Core.ES_ARROW_FILLED_DIAMOND;\n                    break;\n                }\n            } else {\n                this.tailEndStyle = Core.ES_FLAT;\n                this.headEndStyle = Core.ES_FLAT;\n                switch (this.model.end1.aggregation) {\n                case UML.AK_SHARED:\n                    this.tailEndStyle = Core.ES_DIAMOND;\n                    break;\n                case UML.AK_COMPOSITE:\n                    this.tailEndStyle = Core.ES_FILLED_DIAMOND;\n                    break;\n                }\n                switch (this.model.end2.aggregation) {\n                case UML.AK_SHARED:\n                    this.headEndStyle = Core.ES_DIAMOND;\n                    break;\n                case UML.AK_COMPOSITE:\n                    this.headEndStyle = Core.ES_FILLED_DIAMOND;\n                    break;\n                }\n            }\n            // tailRoleNameLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.tailRoleNameLabel.model !== this.model.end1) {\n                Repository.bypassFieldAssign(this.tailRoleNameLabel, \'model\', this.model.end1);\n            }\n            // tailPropertyLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.tailPropertyLabel.model !== this.model.end1) {\n                Repository.bypassFieldAssign(this.tailPropertyLabel, \'model\', this.model.end1);\n            }\n            // tailMultiplicityLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.tailMultiplicityLabel.model !== this.model.end1) {\n                Repository.bypassFieldAssign(this.tailMultiplicityLabel, \'model\', this.model.end1);\n            }\n            // headRoleNameLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.headRoleNameLabel.model !== this.model.end1) {\n                Repository.bypassFieldAssign(this.headRoleNameLabel, \'model\', this.model.end2);\n            }\n            // headPropertyLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.headPropertyLabel.model !== this.model.end1) {\n                Repository.bypassFieldAssign(this.headPropertyLabel, \'model\', this.model.end2);\n            }\n            // headMultiplicityLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.headMultiplicityLabel.model !== this.model.end1) {\n                Repository.bypassFieldAssign(this.headMultiplicityLabel, \'model\', this.model.end2);\n            }\n        }\n        UMLGeneralEdgeView.prototype.update.call(this, canvas);\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                            CLASS DIAGRAM VIEWS                         *\n     *                                                                        *\n     *************************************************************************/\n\n    /**\n     * UMLClassDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLClassDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLClassDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLClassDiagram.prototype.constructor = UMLClassDiagram;\n\n    UMLClassDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLPackage) ||\n               (model instanceof type.UMLClassifier) ||\n               (model instanceof type.UMLInstance) ||\n               (model instanceof type.UMLPort) ||\n               (model instanceof type.UMLAttribute) ||\n               (model instanceof type.UMLGeneralization) ||\n               (model instanceof type.UMLDependency) ||\n               (model instanceof type.UMLInterfaceRealization) ||\n               (model instanceof type.UMLComponentRealization) ||\n               (model instanceof type.UMLAssociation) ||\n               (model instanceof type.UMLLink) ||\n               (model instanceof type.UMLConnector);\n    };\n\n\n    /**\n     * UMLClassView\n     * @constructor\n     * @extends UMLClassifierView\n     */\n    function UMLClassView() {\n        UMLClassifierView.apply(this, arguments);\n        this.fillColor  = PreferenceManager.get("uml.class.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.class.stereotypeDisplay", UML.SD_LABEL);\n        this.suppressAttributes = PreferenceManager.get("uml.class.suppressAttributes", false);\n        this.suppressOperations = PreferenceManager.get("uml.class.suppressOperations", false);\n    }\n    // inherits from UMLClassifierView\n    UMLClassView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLClassView.prototype.constructor = UMLClassView;\n\n    UMLClassView.prototype.drawCommon = function (canvas) {\n        UMLClassifierView.prototype.drawCommon.call(this, canvas);\n        if (this.model && this.model.isActive) {\n            canvas.line(this.mainRect.x1 + CLASS_ACTIVE_VERTLINE_WIDTH, this.mainRect.y1, this.mainRect.x1 + CLASS_ACTIVE_VERTLINE_WIDTH, this.mainRect.y2);\n            canvas.line(this.mainRect.x2 - CLASS_ACTIVE_VERTLINE_WIDTH, this.mainRect.y1, this.mainRect.x2 - CLASS_ACTIVE_VERTLINE_WIDTH, this.mainRect.y2);\n        }\n    };\n\n\n    /**\n     * UMLInterfaceView\n     * @constructor\n     * @extends UMLClassifierView\n     */\n    function UMLInterfaceView() {\n        UMLClassifierView.apply(this, arguments);\n\n        /** temporal */\n        this.depViews = null;\n        this.relViews = null;\n\n        this.fillColor  = PreferenceManager.get("uml.interface.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.interface.stereotypeDisplay", UML.SD_ICON);\n        this.suppressAttributes = PreferenceManager.get("uml.interface.suppressAttributes", true);\n        this.suppressOperations = PreferenceManager.get("uml.interface.suppressOperations", true);\n    }\n    // inherits from UMLClassifierView\n    UMLInterfaceView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLInterfaceView.prototype.constructor = UMLInterfaceView;\n\n    UMLInterfaceView.prototype.collectSupplierDependencyViews = function () {\n        var views = [];\n        for (var i = 0, len = this.getDiagram().ownedViews.length; i < len; i++) {\n            var v = this.getDiagram().ownedViews[i];\n            if ((v instanceof UMLDependencyView) && (v.head === this)) {\n                views.push(v);\n            }\n        }\n        return views;\n    };\n\n    UMLInterfaceView.prototype.collectSupplierRealizationViews = function () {\n        var views = [];\n        for (var i = 0, len = this.getDiagram().ownedViews.length; i < len; i++) {\n            var v = this.getDiagram().ownedViews[i];\n            if ((v instanceof UMLInterfaceRealizationView) && (v.head === this)) {\n                views.push(v);\n            }\n        }\n        return views;\n    };\n\n    UMLInterfaceView.prototype.getStereotypeLabelText = function () {\n        return "«interface»";\n    };\n\n    UMLInterfaceView.prototype.update = function (canvas) {\n        UMLClassifierView.prototype.update.call(this, canvas);\n    };\n\n    UMLInterfaceView.prototype.arrangeCommon = function (canvas) {\n        this.depViews = this.collectSupplierDependencyViews();\n        this.relViews = this.collectSupplierRealizationViews();\n        UMLClassifierView.prototype.arrangeCommon.call(this, canvas);\n    };\n\n    UMLInterfaceView.prototype.drawShadowAsIconicForm = function (canvas) {\n        if ((this.relViews.length > 0) || (this.depViews.length === 0)) {\n            canvas.fillEllipse(\n                this.iconRect.x1 + SHADOW_OFFSET,\n                this.iconRect.y1 + SHADOW_OFFSET,\n                this.iconRect.x2 + SHADOW_OFFSET,\n                this.iconRect.y2 + SHADOW_OFFSET\n            );\n        }\n    };\n\n    UMLInterfaceView.prototype.drawIcon = function (canvas, rect) {\n        if ((this.depViews.length > 0) && (this.relViews.length > 0)) {\n            this.drawBallAndSocketNotation(canvas, rect, this.depViews);\n        } else if (this.depViews.length > 0) {\n            this.drawSocketNotation(canvas, rect, this.depViews);\n        } else {\n            this.drawBallNotation(canvas, rect);\n        }\n    };\n\n    UMLInterfaceView.prototype.drawDecorationIcon = function (canvas, rect) {\n        this.drawBallNotation(canvas, new Rect(rect.x1 + 3, rect.y1 + 3, rect.x2 - 3, rect.y2 - 3));\n    };\n\n    UMLInterfaceView.prototype.drawBallNotation = function (canvas, rect) {\n        canvas.fillEllipse(rect.x1, rect.y1, rect.x2, rect.y2);\n        canvas.ellipse(rect.x1, rect.y1, rect.x2, rect.y2);\n    };\n\n    UMLInterfaceView.prototype.drawSocketNotation = function (canvas, rect, supplierDependencyViews) {\n        var i, len, v, c = Coord.getCenter(rect);\n        for (i = 0, len = supplierDependencyViews.length; i < len; i++) {\n            v = supplierDependencyViews[i];\n            v.arrange(canvas);\n            var b      = Coord.junction(rect, v.points.getPoint(v.points.count() - 1)),\n                theta  = Coord.getAngle(c.x, c.y, b.x, b.y),\n                radius = Math.min(rect.getWidth() / 2, rect.getHeight() / 2);\n            canvas.arc(c.x, c.y, radius, theta - Math.PI / 2, theta + Math.PI / 2, false);\n        }\n    };\n\n    UMLInterfaceView.prototype.drawBallAndSocketNotation = function (canvas, rect, supplierDependencyViews) {\n        this.drawSocketNotation(canvas, rect, supplierDependencyViews);\n        var r = new Rect(rect.x1 + 3, rect.y1 + 3, rect.x2 - 3, rect.y2 - 3);\n        this.drawBallNotation(canvas, r);\n    };\n\n\n    /**\n     * UMLSignalView\n     * @constructor\n     * @extends UMLClassifierView\n     */\n    function UMLSignalView() {\n        UMLClassifierView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.signal.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLClassifierView\n    UMLSignalView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLSignalView.prototype.constructor = UMLSignalView;\n\n    UMLSignalView.prototype.getStereotypeLabelText = function () {\n        return "«signal»";\n    };\n\n\n    /**\n     * UMLDataTypeView\n     * @constructor\n     * @extends UMLClassifierView\n     */\n    function UMLDataTypeView() {\n        UMLClassifierView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.datatype.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.suppressAttributes = true;\n        this.suppressOperations = true;\n    }\n    // inherits from UMLClassifierView\n    UMLDataTypeView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLDataTypeView.prototype.constructor = UMLDataTypeView;\n\n    UMLDataTypeView.prototype.getStereotypeLabelText = function () {\n        return "«dataType»";\n    };\n\n\n    /**\n     * UMLPrimitiveTypeView\n     * @constructor\n     * @extends UMLClassifierView\n     */\n    function UMLPrimitiveTypeView() {\n        UMLClassifierView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.datatype.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.suppressAttributes = true;\n        this.suppressOperations = true;\n    }\n    // inherits from UMLClassifierView\n    UMLPrimitiveTypeView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLPrimitiveTypeView.prototype.constructor = UMLPrimitiveTypeView;\n\n    UMLPrimitiveTypeView.prototype.getStereotypeLabelText = function () {\n        return "«primitiveType»";\n    };\n\n\n    /**\n     * UMLEnumerationLiteralView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLEnumerationLiteralView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLEnumerationLiteralView.prototype = Object.create(LabelView.prototype);\n    UMLEnumerationLiteralView.prototype.constructor = UMLEnumerationLiteralView;\n\n    UMLEnumerationLiteralView.prototype.update = function (canvas) {\n        var options = {\n            showVisibility    : true,\n            stereotypeDisplay : UML.SD_LABEL,\n            showProperty      : true\n        };\n        if (this._parent && (this._parent._parent instanceof UMLClassifierView)) {\n            options.showVisibility    = this._parent._parent.showVisibility;\n            options.stereotypeDisplay = this._parent._parent.stereotypeDisplay;\n            options.showProperty      = this._parent._parent.showProperty;\n        }\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            this.text = this.model.getString(options);\n        }\n        LabelView.prototype.update.call(this, canvas);\n    };\n\n    UMLEnumerationLiteralView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLEnumerationLiteralCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLEnumerationLiteralCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLEnumerationLiteralCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLEnumerationLiteralCompartmentView.prototype.constructor = UMLEnumerationLiteralCompartmentView;\n\n    UMLEnumerationLiteralCompartmentView.prototype.update = function (canvas) {\n        if (this.model.literals) {\n            var tempViews = this.subViews;\n            this.subViews = [];\n            for (var i = 0, len = this.model.literals.length; i < len; i++) {\n                var literal = this.model.literals[i];\n                var literalView = _.find(tempViews, function (v) { return v.model === literal; });\n                if (!literalView) {\n                    literalView = new UMLEnumerationLiteralView();\n                    literalView.model = literal;\n                    literalView._parent = this;\n                    literalView._parent = this;\n                    // literalView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', literalView);\n                } else {\n                    this.addSubView(literalView);\n                }\n                literalView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLEnumerationView\n     * @constructor\n     * @extends UMLClassifierView\n     */\n    function UMLEnumerationView() {\n        UMLClassifierView.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.suppressLiterals = false;\n\n        /** @member {UMLEnumerationLiteralCompartmentView} */\n        this.enumerationLiteralCompartment = new UMLEnumerationLiteralCompartmentView();\n        this.enumerationLiteralCompartment.parentStyle = true;\n        this.addSubView(this.enumerationLiteralCompartment);\n\n        this.fillColor  = PreferenceManager.get("uml.enumeration.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.enumeration.stereotypeDisplay", UML.SD_LABEL);\n        this.suppressAttributes = PreferenceManager.get("uml.enumeration.suppressAttributes", true);\n        this.suppressOperations = PreferenceManager.get("uml.enumeration.suppressOperations", true);\n        this.suppressLiterals   = PreferenceManager.get("uml.enumeration.suppressLiterals", false);\n    }\n    // inherits from UMLClassifierView\n    UMLEnumerationView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLEnumerationView.prototype.constructor = UMLEnumerationView;\n\n    UMLEnumerationView.prototype.getAllCompartments = function () {\n        return [\n            this.nameCompartment,\n            this.enumerationLiteralCompartment,\n            this.attributeCompartment,\n            this.operationCompartment\n        ];\n    };\n\n    UMLEnumerationView.prototype.getStereotypeLabelText = function () {\n        return "«enumeration»";\n    };\n\n    UMLEnumerationView.prototype.update = function (canvas) {\n        // enumerationLiteralCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.enumerationLiteralCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.enumerationLiteralCompartment, \'model\', this.model);\n        }\n        this.enumerationLiteralCompartment.visible = !this.suppressLiterals;\n        UMLClassifierView.prototype.update.call(this, canvas);\n    };\n\n    UMLEnumerationView.prototype.drawObject = function (canvas) {\n        UMLClassifierView.prototype.drawObject.call(this, canvas);\n        if (this.enumerationLiteralCompartment.visible) {\n            canvas.line(\n                this.enumerationLiteralCompartment.left,\n                this.enumerationLiteralCompartment.top,\n                this.enumerationLiteralCompartment.getRight(),\n                this.enumerationLiteralCompartment.top);\n        }\n    };\n\n\n    /**\n     * UMLGeneralizationView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLGeneralizationView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_TRIANGLE;\n        this.lineMode = Core.LM_SOLID;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLGeneralizationView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLGeneralizationView.prototype.constructor = UMLGeneralizationView;\n\n    UMLGeneralizationView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLClassifier);\n    };\n\n    /**\n     * UMLDependencyView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLDependencyView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLDependencyView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLDependencyView.prototype.constructor = UMLDependencyView;\n\n    UMLDependencyView.prototype.arrangeObject = function (canvas) {\n        UMLGeneralEdgeView.prototype.arrangeObject.call(this, canvas);\n        this.lineMode = Core.LM_DOT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        if (this.head instanceof UMLInterfaceView) {\n            var v = this.head;\n            if ((v.stereotypeDisplay === UML.SD_ICON) || (v.stereotypeDisplay === UML.SD_ICON_LABEL)) {\n                this.lineMode = Core.LM_SOLID;\n                this.headEndStyle = Core.ES_FLAT;\n                var c = this.points.count();\n                var p = Coord.junction(v.iconRect, this.points.getPoint(c - 1));\n                this.points.setPoint(c - 1, p);\n                if ((this.lineStyle === Core.LS_RECTILINEAR || this.lineStyle === Core.LS_ROUNDRECT) && (c >= 2)) {\n                    var p2 = this.points.getPoint(c - 2);\n                    if (Math.abs(p2.x - p.x) > Math.abs(p2.y - p.y)) {\n                        p2.y = p.y;\n                    } else {\n                        p2.x = p.x;\n                    }\n                    this.points.setPoint(c - 2, p2);\n                }\n            }\n        }\n    };\n\n    UMLDependencyView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLModelElement);\n    };\n\n    /**\n     * UMLRealizationView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLRealizationView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_TRIANGLE;\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLRealizationView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLRealizationView.prototype.constructor = UMLRealizationView;\n\n    /**\n     * UMLInterfaceRealizationView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLInterfaceRealizationView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_TRIANGLE;\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLInterfaceRealizationView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLInterfaceRealizationView.prototype.constructor = UMLInterfaceRealizationView;\n\n    UMLInterfaceRealizationView.prototype.arrangeObject = function (canvas) {\n        UMLGeneralEdgeView.prototype.arrangeObject.call(this, canvas);\n        this.lineMode = Core.LM_DOT;\n        this.headEndStyle = Core.ES_TRIANGLE;\n        if (this.head instanceof UMLInterfaceView) {\n            var v = this.head;\n            if ((v.stereotypeDisplay === UML.SD_ICON) || (v.stereotypeDisplay === UML.SD_ICON_LABEL)) {\n                this.lineMode = Core.LM_SOLID;\n                this.headEndStyle = Core.ES_FLAT;\n                var c = this.points.count();\n                var p = Coord.junction(v.iconRect, this.points.getPoint(c - 1));\n                this.points.setPoint(c - 1, p);\n                if ((this.lineStyle === Core.LS_RECTILINEAR || this.lineStyle === Core.LS_ROUNDRECT) && (c >= 2)) {\n                    var p2 = this.points.getPoint(c - 2);\n                    if (Math.abs(p2.x - p.x) > Math.abs(p2.y - p.y)) {\n                        p2.y = p.y;\n                    } else {\n                        p2.x = p.x;\n                    }\n                    this.points.setPoint(c - 2, p2);\n                }\n            }\n        }\n    };\n\n    UMLInterfaceRealizationView.prototype.canConnectTo = function (view, isTail) {\n        return (isTail && view.model instanceof type.UMLClassifier) ||\n               (!isTail && view.model instanceof type.UMLInterface);\n    };\n\n    /**\n     * UMLQualifierCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLQualifierCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLQualifierCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLQualifierCompartmentView.prototype.constructor = UMLQualifierCompartmentView;\n\n    UMLQualifierCompartmentView.prototype.update = function (canvas) {\n        if (this.model.qualifiers) {\n            var tempViews = this.subViews;\n            this.subViews = [];\n            for (var i = 0, len = this.model.qualifiers.length; i < len; i++) {\n                var attr = this.model.qualifiers[i];\n                var attrView = _.find(tempViews, function (v) { return v.model == attr; });\n                if (!attrView) {\n                    attrView = new UMLAttributeView();\n                    attrView.model = attr;\n                    attrView._parent = this;\n                    // attrView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', attrView);\n                } else {\n                    this.addSubView(attrView);\n                }\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n    UMLQualifierCompartmentView.prototype.drawShadow = function (canvas) {\n        canvas.storeState();\n        canvas.alpha = SHADOW_ALPHA;\n        canvas.fillColor = SHADOW_COLOR;\n        canvas.fillRect(\n            this.left + SHADOW_OFFSET,\n            this.top + SHADOW_OFFSET,\n            this.getRight() + SHADOW_OFFSET,\n            this.getBottom() + SHADOW_OFFSET\n        );\n        canvas.restoreState();\n    };\n\n    UMLQualifierCompartmentView.prototype.drawObject = function (canvas) {\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n\n    /**\n     * UMLAssociationView\n     * @constructor\n     * @extends UMLUndirectedRelationshipView\n     */\n    function UMLAssociationView() {\n        UMLUndirectedRelationshipView.apply(this, arguments);\n\n        /** @member {} */\n        this.tailQualifiersCompartment = new UMLQualifierCompartmentView();\n        this.addSubView(this.tailQualifiersCompartment);\n\n        /** @member {} */\n        this.headQualifiersCompartment = new UMLQualifierCompartmentView();\n        this.addSubView(this.headQualifiersCompartment);\n    }\n    // inherits from UMLUndirectedRelationshipView\n    UMLAssociationView.prototype = Object.create(UMLUndirectedRelationshipView.prototype);\n    UMLAssociationView.prototype.constructor = UMLAssociationView;\n\n    UMLAssociationView.prototype.update = function (canvas) {\n        if (this.model) {\n            this.tailQualifiersCompartment.visible = this.model.end1.qualifiers.length > 0;\n            this.headQualifiersCompartment.visible = this.model.end2.qualifiers.length > 0;\n            // tailQualifiersCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.tailQualifiersCompartment.model !== this.model.end1) {\n                Repository.bypassFieldAssign(this.tailQualifiersCompartment, \'model\', this.model.end1);\n            }\n            // headQualifiersCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.headQualifiersCompartment.model !== this.model.end2) {\n                Repository.bypassFieldAssign(this.headQualifiersCompartment, \'model\', this.model.end2);\n            }\n        }\n        UMLUndirectedRelationshipView.prototype.update.call(this, canvas);\n    };\n\n    UMLAssociationView.prototype.sizeObject = function (canvas) {\n        UMLUndirectedRelationshipView.prototype.sizeObject.call(this, canvas);\n        this.tailQualifiersCompartment.sizeObject(canvas);\n        this.headQualifiersCompartment.sizeObject(canvas);\n    };\n\n    UMLAssociationView.prototype.arrangeQualifierCompartment = function (canvas, qv, isTail) {\n        var jp, nv, p1, pn;\n        if (isTail) {\n            nv = this.tail;\n            p1 = this.points.getPoint(1);\n            jp = 0;\n        } else {\n            nv = this.head;\n            p1 = this.points.getPoint(this.points.count() - 2);\n            jp = this.points.count() - 1;\n        }\n        if (qv.visible) {\n            qv.width = qv.minWidth;\n            qv.height = qv.minHeight;\n            pn = Coord.junction(nv.getBoundingBox(canvas), p1);\n            var dx = Math.abs(p1.x - pn.x),\n                dy = Math.abs(p1.y - pn.y);\n            if ((pn.x <= p1.x) && (dx >= dy)) {\n                qv.left = nv.getRight();\n                qv.top = pn.y - qv.height / 2;\n            } else if ((pn.y >= p1.y) && (dy > dx)) {\n                qv.left = pn.x - qv.width / 2;\n                qv.top = nv.top - qv.height + 1;\n            } else if ((pn.x > p1.x) && (dx >= dy)) {\n                qv.left = nv.left - qv.width + 1;\n                qv.top = pn.y - qv.height / 2;\n            } else if ((pn.y < p1.y) && (dy > dx)) {\n                qv.left = pn.x - qv.width / 2;\n                qv.top = nv.getBottom();\n            }\n            var p = Coord.junction(qv.getBoundingBox(canvas), p1);\n            this.points.setPoint(jp, p);\n        }\n    };\n\n    UMLAssociationView.prototype.arrange = function (canvas) {\n        this.recalcPoints(canvas);\n        this.arrangeQualifierCompartment(canvas, this.tailQualifiersCompartment, true);\n        this.arrangeQualifierCompartment(canvas, this.headQualifiersCompartment, false);\n        UMLUndirectedRelationshipView.prototype.arrange.call(this, canvas);\n        var t = null, h = null;\n        if (this.tailQualifiersCompartment.visible) {\n            t = this.tail;\n            this.tail = this.tailQualifiersCompartment;\n        }\n        if (this.headQualifiersCompartment.visible) {\n            h = this.head;\n            this.head = this.headQualifiersCompartment;\n        }\n        this.recalcPoints(canvas);\n        if (this.tailQualifiersCompartment.visible) {\n            this.tail = t;\n        }\n        if (this.headQualifiersCompartment.visible) {\n            this.head = h;\n        }\n    };\n\n    UMLAssociationView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLClassifier);\n    };\n\n    /**\n     * UMLAssociationClassLinkView\n     * @constructor\n     * @extends EdgeView\n     */\n    function UMLAssociationClassLinkView() {\n        EdgeView.apply(this, arguments);\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from EdgeView\n    UMLAssociationClassLinkView.prototype = Object.create(EdgeView.prototype);\n    UMLAssociationClassLinkView.prototype.constructor = UMLAssociationClassLinkView;\n\n    UMLAssociationClassLinkView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLClass || view.model instanceof type.UMLAssociation);\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                        PACKAGE DIAGRAM VIEWS                           *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLPackageDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLPackageDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLPackageDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLPackageDiagram.prototype.constructor = UMLPackageDiagram;\n\n    UMLPackageDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLPackage) ||\n               (model instanceof type.UMLSubsystem) ||\n               (model instanceof type.UMLDependency);\n    };\n\n\n    /**\n     * UMLPackageView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLPackageView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = true;\n        this.fillColor = PreferenceManager.get("uml.package.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLPackageView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLPackageView.prototype.constructor = UMLPackageView;\n\n    UMLPackageView.prototype.canContainViewKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLClassifierView") ||\n               MetaModelManager.isKindOf(kind, "UMLPackageView") ||\n               MetaModelManager.isKindOf(kind, "UMLObjectView") ||\n               MetaModelManager.isKindOf(kind, "UMLArtifactInstanceView") ||\n               MetaModelManager.isKindOf(kind, "UMLComponentInstanceView") ||\n               MetaModelManager.isKindOf(kind, "UMLNodeInstanceView");\n    };\n\n    UMLPackageView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n    };\n\n    UMLPackageView.prototype.sizeObject = function (canvas) {\n        UMLGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = Math.max(this.minWidth, PACKAGE_MINWIDTH);\n        this.minHeight = Math.max(PACKAGE_TAB_HEIGHT + this.minHeight, PACKAGE_MINHEIGHT);\n    };\n\n    UMLPackageView.prototype.arrangeCommon = function (canvas) {\n        UMLGeneralNodeView.prototype.arrangeCommon.call(this, canvas);\n        if ((this.stereotypeDisplay !== UML.SD_ICON) && (this.stereotypeDisplay !== UML.SD_ICON_LABEL)) {\n            this.mainRect.setRect(this.left, this.top + PACKAGE_TAB_HEIGHT, this.getRight(), this.getBottom());\n        }\n    };\n\n    UMLPackageView.prototype.drawShadowAsCanonicalForm = function (canvas, showLabel) {\n        var tabRightX = this.left + (this.getRight() - this.left) * 2 / 5,\n            tabBottomY = this.top + PACKAGE_TAB_HEIGHT;\n        canvas.fillRect(this.left + SHADOW_OFFSET, this.top + SHADOW_OFFSET, tabRightX + SHADOW_OFFSET, tabBottomY + SHADOW_OFFSET);\n        canvas.fillRect(this.left + SHADOW_OFFSET, tabBottomY - 1 + SHADOW_OFFSET, this.getRight() + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET);\n    };\n\n    UMLPackageView.prototype.drawAsCanonicalForm = function (canvas, showLabel) {\n        var tabRightX = this.left + (this.getRight() - this.left) * 2 / 5,\n            tabBottomY = this.top + PACKAGE_TAB_HEIGHT;\n        canvas.fillRect(this.left, this.top, tabRightX, tabBottomY);\n        canvas.rect(this.left, this.top, tabRightX, tabBottomY);\n        canvas.fillRect(this.left, tabBottomY - 1, this.getRight(), this.getBottom());\n        canvas.rect(this.left, tabBottomY - 1, this.getRight(), this.getBottom());\n        UMLGeneralNodeView.prototype.drawAsCanonicalForm.call(this, canvas, showLabel);\n    };\n\n    UMLPackageView.prototype.drawAsDecorationForm = function (canvas, showLabel) {\n        this.drawAsCanonicalForm(canvas, showLabel);\n        UMLGeneralNodeView.prototype.drawAsDecorationForm.call(this, canvas, showLabel);\n    };\n\n\n    /**\n     * UMLModelView\n     * @constructor\n     * @extends UMLPackageView\n     */\n    function UMLModelView() {\n        UMLPackageView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.model.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLPackageView\n    UMLModelView.prototype = Object.create(UMLPackageView.prototype);\n    UMLModelView.prototype.constructor = UMLModelView;\n\n    UMLModelView.prototype.drawAsCanonicalForm = function (canvas, showLabel) {\n        UMLPackageView.prototype.drawAsCanonicalForm.call(this, canvas, showLabel);\n        var tabRightX = this.left + (this.getRight() - this.left) * 2 / 5;\n        canvas.polyline([new Point(tabRightX - 9, this.top + 3), new Point(tabRightX - 15, this.top + 10), new Point(tabRightX - 3, this.top + 10), new Point(tabRightX - 9, this.top + 3)]);\n    };\n\n\n    /**\n     * UMLSubsystemView\n     * @constructor\n     * @extends UMLPackageView\n     */\n    function UMLSubsystemView() {\n        UMLPackageView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.subsystem.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLPackageView\n    UMLSubsystemView.prototype = Object.create(UMLPackageView.prototype);\n    UMLSubsystemView.prototype.constructor = UMLSubsystemView;\n\n    UMLSubsystemView.prototype.drawAsCanonicalForm = function (canvas, showLabel) {\n        UMLPackageView.prototype.drawAsCanonicalForm.call(this, canvas, showLabel);\n        var tabRightX = this.left + (this.getRight() - this.left) * 2 / 5;\n        canvas.polyline([new Point(tabRightX - 8, this.top + 3), new Point(tabRightX - 8, this.top + 7)]);\n        canvas.polyline([new Point(tabRightX - 11, this.top + 11), new Point(tabRightX - 11, this.top + 7), new Point(tabRightX - 5, this.top + 7)]);\n        canvas.polyline([new Point(tabRightX - 5, this.top + 11), new Point(tabRightX - 5, this.top + 7), new Point(tabRightX - 11, this.top + 7)]);\n    };\n\n\n    /**\n     * UMLContainmentView\n     * @constructor\n     * @extends EdgeView\n     */\n    function UMLContainmentView() {\n        EdgeView.apply(this, arguments);\n        this.headEndStyle = Core.ES_CIRCLE_PLUS;\n    }\n    // inherits from EdgeView\n    UMLContainmentView.prototype = Object.create(EdgeView.prototype);\n    UMLContainmentView.prototype.constructor = UMLContainmentView;\n\n    UMLContainmentView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLModelElement);\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                  COMPOSITE STRUCTURE DIAGRAM VIEWS                     *\n     *                                                                        *\n     **************************************************************************/\n\n\n    /**\n     * UMLCompositeStructureDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLCompositeStructureDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLCompositeStructureDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLCompositeStructureDiagram.prototype.constructor = UMLCompositeStructureDiagram;\n\n    UMLCompositeStructureDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLPackage) ||\n               (model instanceof type.UMLClassifier) ||\n               (model instanceof type.UMLPort) ||\n               (model instanceof type.UMLAttribute) ||\n               (model instanceof type.UMLCollaborationUse) ||\n               (model instanceof type.UMLGeneralization) ||\n               (model instanceof type.UMLDependency) ||\n               (model instanceof type.UMLInterfaceRealization) ||\n               (model instanceof type.UMLComponentRealization) ||\n               (model instanceof type.UMLAssociation) ||\n               (model instanceof type.UMLLink) ||\n               (model instanceof type.UMLConnector);\n    };\n\n\n    /**\n     * UMLPortView\n     * @constructor\n     * @extends UMLFloatingNodeView\n     */\n    function UMLPortView() {\n        UMLFloatingNodeView.apply(this, arguments);\n        this.sizable = Core.SZ_NONE;\n        this.fillColor = PreferenceManager.get("uml.port.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n\n        /** @member {boolean} */\n        this.showVisibility = false;\n\n        /** @member {boolean} */\n        this.showType = true;\n\n        /** @member {boolean} */\n        this.showMultiplicity = true;\n    }\n    // inherits from LabelView\n    UMLPortView.prototype = Object.create(UMLFloatingNodeView.prototype);\n    UMLPortView.prototype.constructor = UMLPortView;\n\n    UMLPortView.prototype.update = function (canvas) {\n        UMLFloatingNodeView.prototype.update.call(this, canvas);\n        var options = {\n            showVisibility   : this.showVisibility,\n            showType         : this.showType,\n            showMultiplicity : this.showMultiplicity,\n            showProperty     : false\n        };\n        this.nameLabel.text = this.model.getString(options);\n        this.nameLabel.underline = (this.model.isStatic === true);\n    };\n\n    UMLPortView.prototype.sizeObject = function (canvas) {\n        UMLFloatingNodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = PORT_MINWIDTH;\n        this.minHeight = PORT_MINHEIGHT;\n    };\n\n    UMLPortView.prototype.arrange = function (canvas) {\n        if (this.containerView) {\n            var r = this.containerView.getBoundingBox(canvas);\n            var c = Coord.getCenter(new Rect(this.left, this.top, this.getRight(), this.getBottom()));\n            var p = this._junction2(r, c);\n            this.left = p.x - PORT_MINWIDTH / 2;\n            this.top = p.y - PORT_MINHEIGHT / 2;\n            this.setRight(p.x + PORT_MINWIDTH / 2);\n            this.setBottom(p.y + PORT_MINHEIGHT / 2);\n        }\n        UMLFloatingNodeView.prototype.arrange.call(this, canvas);\n    };\n\n    UMLPortView.prototype.drawObject = function (canvas) {\n        UMLFloatingNodeView.prototype.drawObject.call(this, canvas);\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n\n    /**\n     * UMLPartView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLPartView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = false;\n        this.showVisibility = false;\n\n        /** @member {boolean} */\n        this.showMultiplicity = true;\n\n        this.fillColor = PreferenceManager.get("uml.part.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLPartView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLPartView.prototype.constructor = UMLPartView;\n\n    UMLPartView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model && (this.model instanceof type.UMLAttribute)) {\n            this.nameCompartment.nameLabel.text = this.model.getString(this);\n            this.nameCompartment.nameLabel.underline = (this.model.isStatic === true);\n        }\n    };\n\n    UMLPartView.prototype.drawObject = function (canvas) {\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        UMLGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * UMLConnectorView\n     * @constructor\n     * @extends UMLUndirectedRelationshipView\n     */\n    function UMLConnectorView() {\n        UMLUndirectedRelationshipView.apply(this, arguments);\n    }\n    // inherits from UMLUndirectedRelationshipView\n    UMLConnectorView.prototype = Object.create(UMLUndirectedRelationshipView.prototype);\n    UMLConnectorView.prototype.constructor = UMLConnectorView;\n\n    UMLConnectorView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLModelElement);\n    };\n\n    UMLConnectorView.prototype.update = function (canvas) {\n        UMLUndirectedRelationshipView.prototype.update.call(this, canvas);\n        if (this.model) {\n            // nameLabel\n            var text = "";\n            if (this.model.name) {\n                text += this.model.getString(this.showVisibility);\n            }\n            if (this.model.type && this.showType) {\n                text += ": " + this.model.type.name;\n            }\n            this.nameLabel.text = text;\n            this.nameLabel.visible = (text.length > 0);\n        }\n    };\n\n    /**\n     * UMLCollaborationView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLCollaborationView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.iconRatio = USECASE_RATIO_PERCENT;\n\n        /** @member {UMLTemplateParameterCompartmentView} */\n        this.templateParameterCompartment = new UMLTemplateParameterCompartmentView();\n        this.templateParameterCompartment.selectable = Core.SK_PROPAGATE;\n        this.templateParameterCompartment.parentStyle = true;\n        this.addSubView(this.templateParameterCompartment);\n\n        this.fillColor = PreferenceManager.get("uml.collaboration.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLCollaborationView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLCollaborationView.prototype.constructor = UMLCollaborationView;\n\n    UMLCollaborationView.prototype.update = function (canvas) {\n        // templateParameterCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.templateParameterCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.templateParameterCompartment, \'model\', this.model);\n        }\n        if (this.model) {\n            if (this.model.templateParameters && this.model.templateParameters.length > 0) {\n                this.templateParameterCompartment.visible = true;\n            } else {\n                this.templateParameterCompartment.visible = false;\n            }\n        }\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n    };\n\n    UMLCollaborationView.prototype.drawShadowAsCanonicalForm = function (canvas, showLabel) {\n        canvas.fillEllipse(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET);\n    };\n\n    UMLCollaborationView.prototype.drawShadowAsDecorationForm = function (canvas) {\n        canvas.fillRect(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET\n        );\n    };\n\n    UMLCollaborationView.prototype.sizeObjectAsCanonicalForm = function (canvas) {\n        UMLGeneralNodeView.prototype.sizeObjectAsCanonicalForm.call(this, canvas);\n        // Calculating minimum size <= minimum Nampcompartment\'s circumscription size\n        // rectangle\'s circumscription\'s height and width are Sqrt(2) times of rectangle\'s height and width\n        var w = Math.max(Math.trunc(Math.sqrt(2) * this.nameCompartment.minWidth), COLLABORATION_MINWIDTH);\n        var h = Math.max(Math.trunc(Math.sqrt(2) * this.nameCompartment.minHeight), COLLABORATION_MINHEIGHT);\n        if (this.templateParameterCompartment.visible) {\n            this.templateParameterCompartment.size(canvas);\n            w = Math.max(w, this.templateParameterCompartment.minWidth);\n            w = w + TEMPLATEPARAMETERCOMPARTMENT_LEFT_MARGIN + TEMPLATEPARAMETERCOMPARTMENT_RIGHT_OCCUPY;\n            h = h + this.templateParameterCompartment.minHeight - TEMPLATEPARAMETERCOMPARTMENT_OVERLAP;\n        }\n        this.minWidth = w;\n        this.minHeight = h;\n    };\n\n    UMLCollaborationView.prototype.sizeAsIconicForm = function (canvas, showLabel) {\n        if (this.hasStereotypeIcon()) {\n            UMLGeneralNodeView.prototype.sizeAsIconicForm.call(this, canvas, showLabel);\n        } else {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            this.minWidth = Math.max(sz.x, USECASE_ICON_MINWIDTH);\n            this.minHeight = USECASE_ICON_MINHEIGHT + sz.y;\n        }\n    };\n\n    UMLCollaborationView.prototype.arrangeObjectAsCanonicalForm = function (canvas) {\n        UMLGeneralNodeView.prototype.arrangeObjectAsCanonicalForm.call(this, canvas);\n        // Arranging view objects.\n        if (this.templateParameterCompartment.visible) {\n            this.templateParameterCompartment.left = this.getRight() - TEMPLATEPARAMETERCOMPARTMENT_RIGHT_OCCUPY - this.templateParameterCompartment.minWidth;\n            this.templateParameterCompartment.top = this.top;\n            this.templateParameterCompartment.setRight(this.getRight());\n            this.templateParameterCompartment.arrange(canvas);\n            this.nameCompartment.left = this.left;\n            this.nameCompartment.top = ((this.getBottom() + this.top + this.templateParameterCompartment.height - TEMPLATEPARAMETERCOMPARTMENT_OVERLAP) / 2) - (this.nameCompartment.height / 2);\n            this.nameCompartment.setRight(this.getRight() - TEMPLATEPARAMETERCOMPARTMENT_RIGHT_OCCUPY);\n            this.nameCompartment.arrange(canvas);\n            var x1 = this.left,\n                y1 = this.top + this.templateParameterCompartment.height - TEMPLATEPARAMETERCOMPARTMENT_OVERLAP,\n                x2 = this.getRight() - TEMPLATEPARAMETERCOMPARTMENT_RIGHT_OCCUPY,\n                y2 = this.getBottom();\n            this.mainRect.setRect(x1, y1, x2, y2);\n        } else {\n            this.nameCompartment.left = this.left;\n            this.nameCompartment.top = ((this.getBottom() + this.top) / 2) - (this.nameCompartment.height / 2);\n            this.nameCompartment.setRight(this.getRight());\n            this.nameCompartment.arrange(canvas);\n        }\n    };\n\n    UMLCollaborationView.prototype.drawIcon = function (canvas, rect) {\n        if (this.hasStereotypeIcon()) {\n            var ratioRect = this.computeIconRect(rect, (this.model.stereotype.icon.width / this.model.stereotype.icon.height) * 100);\n            drawImage(canvas, ratioRect, this.model.stereotype.icon);\n        } else {\n            var r = rect;\n            var x, y;\n            var w = r.x2 - r.x1;\n            var h = r.y2 - r.y1;\n            var rr = w * 100 / h;\n            var ir = USECASE_ICON_MINWIDTH * 100 / USECASE_ICON_MINHEIGHT;\n            if (rr >= ir) {\n                h = (r.y2 - r.y1);\n                w = h * ir / 100;\n                x = r.x1 + (r.x2 - r.x1 - w) / 2;\n                y = r.y1;\n            } else {\n                w = (r.x2 - r.x1);\n                h = w * 100 / ir;\n                y = r.y1 + (r.y2 - r.y1 - h) / 2;\n                x = r.x1;\n            }\n            canvas.fillEllipse(x, y, x + w, y + h);\n            canvas.ellipse(x, y, x + w, y + h, [5]);\n        }\n    };\n\n    UMLCollaborationView.prototype.drawCommon = function (canvas) {\n        if (this.stereotypeDisplay === UML.SD_DECORATION || this.stereotypeDisplay === UML.SD_DECORATION_LABEL) {\n            canvas.fillRect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n            canvas.rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n        }\n    };\n\n    UMLCollaborationView.prototype.drawAsCanonicalForm = function (canvas) {\n        var r = new Rect();\n        if (this.templateParameterCompartment.visible) {\n            r.x1 = this.left;\n            r.y1 = this.top + this.templateParameterCompartment.height - TEMPLATEPARAMETERCOMPARTMENT_OVERLAP;\n            r.x2 = this.getRight() - TEMPLATEPARAMETERCOMPARTMENT_RIGHT_OCCUPY;\n            r.y2 = this.getBottom();\n        } else {\n            r.setRect(this.left, this.top, this.getRight(), this.getBottom());\n        }\n        canvas.fillEllipse(r.x1, r.y1, r.x2, r.y2);\n        canvas.ellipse(r.x1, r.y1, r.x2, r.y2, [5]);\n    };\n\n\n    /**\n     * UMLCollaborationUseView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLCollaborationUseView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.collaborationuse.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLCollaborationUseView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLCollaborationUseView.prototype.constructor = UMLCollaborationUseView;\n\n    UMLCollaborationUseView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model.type && this.model.type.name) {\n            this.nameCompartment.nameLabel.text = this.model.name + ": " + this.model.type.name;\n        }\n    };\n\n    UMLCollaborationUseView.prototype.drawShadowAsCanonicalForm = function (canvas) {\n        canvas.fillEllipse(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET);\n    };\n\n    UMLCollaborationUseView.prototype.sizeObject = function (canvas) {\n        UMLGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        // Calculating minimum size <= minimum Nampcompartment\'s circumscription size\n        // rectangle\'s circumscription\'s height and width are Sqrt(2) times of rectangle\'s height and width\n        var w = Math.max(Math.trunc(Math.sqrt(2) * this.nameCompartment.minWidth), COLLABORATION_MINWIDTH);\n        var h = Math.max(Math.trunc(Math.sqrt(2) * this.nameCompartment.minHeight), COLLABORATION_MINHEIGHT);\n        this.minWidth = w;\n        this.minHeight = h;\n    };\n\n    UMLCollaborationUseView.prototype.arrangeObject = function (canvas) {\n        UMLGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameCompartment.left = this.left;\n        this.nameCompartment.top = ((this.getBottom() + this.top) / 2) - (this.nameCompartment.height / 2);\n        this.nameCompartment.setRight(this.getRight());\n        this.nameCompartment.arrange(canvas);\n    };\n\n    UMLCollaborationUseView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.fillEllipse(r.x1, r.y1, r.x2, r.y2);\n        canvas.ellipse(r.x1, r.y1, r.x2, r.y2, [5]);\n    };\n\n\n    /**\n     * UMLRoleBindingView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLRoleBindingView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_DOT;\n\n        /** @member {EdgeLabelView} */\n        this.roleNameLabel = new EdgeLabelView();\n        this.roleNameLabel.hostEdge = this;\n        this.roleNameLabel.edgePosition = Core.EP_HEAD;\n        this.roleNameLabel.alpha = -Math.PI / 6;\n        this.roleNameLabel.distance = 30;\n        this.addSubView(this.roleNameLabel);\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLRoleBindingView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLRoleBindingView.prototype.constructor = UMLRoleBindingView;\n\n    UMLRoleBindingView.prototype.update = function (canvas) {\n        if (this.model) {\n            this.roleNameLabel.visible = (this.model.roleName && this.model.roleName.length > 0);\n            if (this.model.roleName) {\n                this.roleNameLabel.text = this.model.roleName;\n            }\n            // roleNameLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.roleNameLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.roleNameLabel, \'model\', this.model);\n            }\n        }\n        UMLGeneralEdgeView.prototype.update.call(this, canvas);\n    };\n\n    UMLRoleBindingView.prototype.canConnectTo = function (view, isTail) {\n        return (isTail && view.model instanceof type.UMLCollaborationUse) ||\n               (!isTail && view.model instanceof type.UMLAttribute);\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                           OBJECT DIAGRAM VIEWS                         *\n     *                                                                        *\n     *************************************************************************/\n\n    /**\n     * UMLObjectDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLObjectDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLObjectDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLObjectDiagram.prototype.constructor = UMLObjectDiagram;\n\n    UMLObjectDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLClassifier) ||\n               (model instanceof type.UMLInstance) ||\n               (model instanceof type.UMLDependency) ||\n               (model instanceof type.UMLLink);\n    };\n\n\n    /**\n     * UMLSlotView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLSlotView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLSlotView.prototype = Object.create(LabelView.prototype);\n    UMLSlotView.prototype.constructor = UMLSlotView;\n\n    UMLSlotView.prototype.update = function (canvas) {\n        var options = {\n            showVisibility    : true,\n            stereotypeDisplay : UML.SD_LABEL,\n            showProperty      : true,\n            showType          : true\n        };\n        if (this._parent && this._parent._parent) {\n            options.showVisibility    = this._parent._parent.showVisibility;\n            options.stereotypeDisplay = this._parent._parent.stereotypeDisplay;\n            options.showProperty      = this._parent._parent.showProperty;\n            options.showType          = this._parent._parent.showType;\n        }\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            this.text = this.model.getString(options);\n        }\n        LabelView.prototype.update.call(this, canvas);\n    };\n\n\n    UMLSlotView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLSlotCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLSlotCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLSlotCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLSlotCompartmentView.prototype.constructor = UMLSlotCompartmentView;\n\n    UMLSlotCompartmentView.prototype.update = function (canvas) {\n        if (this.model.slots) {\n            var tempViews = this.subViews;\n            this.subViews = [];\n            for (var i = 0, len = this.model.slots.length; i < len; i++) {\n                var slot = this.model.slots[i];\n                var slotView = _.find(tempViews, function (v) { return v.model == slot; });\n                if (!slotView) {\n                    slotView = new UMLSlotView();\n                    slotView.model = slot;\n                    slotView._parent = this;\n                    // slotView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', slotView);\n                } else {\n                    this.addSubView(slotView);\n                }\n                slotView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLObjectView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLObjectView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = true;\n\n        /** @member {UMLSlotCompartmentView} */\n        this.slotCompartment = new UMLSlotCompartmentView();\n        this.slotCompartment.selectable = Core.SK_PROPAGATE;\n        this.slotCompartment.parentStyle = true;\n        this.addSubView(this.slotCompartment);\n\n        this.fillColor = PreferenceManager.get("uml.object.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLObjectView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLObjectView.prototype.constructor = UMLObjectView;\n\n    UMLObjectView.prototype.update = function (canvas) {\n        // slotCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.slotCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.slotCompartment, \'model\', this.model);\n        }\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameCompartment.nameLabel.text = this.model.getString(this);\n            if (this.model.slots && this.model.slots.length > 0) {\n                this.slotCompartment.visible = true;\n            } else {\n                this.slotCompartment.visible = false;\n            }\n            if (this.model.value !== null && this.model.value.length > 0) {\n                this.nameCompartment.namespaceLabel.text = this.model.value;\n                this.nameCompartment.namespaceLabel.visible = true;\n            }\n        }\n    };\n\n    UMLObjectView.prototype.drawObject = function (canvas) {\n        UMLGeneralNodeView.prototype.drawObject.call(this, canvas);\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        if (this.model.isMultiInstance) {\n            canvas.rect(r.x1 + MULTI_INSTANCE_MARGIN, r.y1 + MULTI_INSTANCE_MARGIN, r.x2 + MULTI_INSTANCE_MARGIN, r.y2 + MULTI_INSTANCE_MARGIN);\n        }\n        canvas.fillRect(r.x1, r.y1, r.x2, r.y2);\n        canvas.rect(r.x1, r.y1, r.x2, r.y2);\n        if (this.model.classifier && this.model.classifier.isActive) {\n            canvas.line(this.left + CLASS_ACTIVE_VERTLINE_WIDTH, this.top, this.left + CLASS_ACTIVE_VERTLINE_WIDTH, this.getBottom());\n            canvas.line(this.getRight() - CLASS_ACTIVE_VERTLINE_WIDTH, this.top, this.getRight() - CLASS_ACTIVE_VERTLINE_WIDTH, this.getBottom());\n        }\n        if (this.slotCompartment.visible) {\n            canvas.line(\n                this.slotCompartment.left,\n                this.slotCompartment.top,\n                this.slotCompartment.getRight(),\n                this.slotCompartment.top);\n        }\n    };\n\n\n    /**\n     * UMLLinkView\n     * @constructor\n     * @extends UMLUndirectedRelationshipView\n     */\n    function UMLLinkView() {\n        UMLUndirectedRelationshipView.apply(this, arguments);\n    }\n    // inherits from UMLUndirectedRelationshipView\n    UMLLinkView.prototype = Object.create(UMLUndirectedRelationshipView.prototype);\n    UMLLinkView.prototype.constructor = UMLLinkView;\n\n    UMLLinkView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLInstance);\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                         COMPONENT DIAGRAM VIEWS                        *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLComponentDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLComponentDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLComponentDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLComponentDiagram.prototype.constructor = UMLComponentDiagram;\n\n    UMLComponentDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLPackage) ||\n               (model instanceof type.UMLClassifier) ||\n               (model instanceof type.UMLInstance) ||\n               (model instanceof type.UMLPort) ||\n               (model instanceof type.UMLAttribute) ||\n               (model instanceof type.UMLGeneralization) ||\n               (model instanceof type.UMLDependency) ||\n               (model instanceof type.UMLInterfaceRealization) ||\n               (model instanceof type.UMLComponentRealization) ||\n               (model instanceof type.UMLAssociation) ||\n               (model instanceof type.UMLLink) ||\n               (model instanceof type.UMLConnector);\n    };\n\n    /**\n     * UMLArtifactViewMixin\n     * @mixin\n     */\n    var UMLArtifactViewMixin = {\n\n        sizeAsIconicForm: function (canvas, showLabel) {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            this.minWidth = Math.max(sz.x, ARTIFACT_ICON_MINWIDTH);\n            this.minHeight = ARTIFACT_ICON_MINHEIGHT + sz.y;\n        },\n\n        drawShadowAsIconicForm: function (canvas) {\n            var w = this.iconRect.x2 - this.iconRect.x1,\n                h = this.iconRect.y2 - this.iconRect.y1,\n                x = this.iconRect.x2 - w * 30 / 100,\n                y = this.iconRect.y1 + w * 30 / 100;\n            canvas.fillPolygon([\n                new Point(this.iconRect.x1 + SHADOW_OFFSET, this.iconRect.y1 + SHADOW_OFFSET),\n                new Point(x + SHADOW_OFFSET, this.iconRect.y1 + SHADOW_OFFSET),\n                new Point(this.iconRect.x2 + SHADOW_OFFSET, y + SHADOW_OFFSET),\n                new Point(this.iconRect.x2 + SHADOW_OFFSET, this.iconRect.y2 + SHADOW_OFFSET),\n                new Point(this.iconRect.x1 + SHADOW_OFFSET, this.iconRect.y2 + SHADOW_OFFSET)]);\n        },\n\n        drawIcon: function (canvas, rect) {\n            var w = rect.x2 - rect.x1,\n                h = rect.y2 - rect.y1,\n                x = rect.x2 - w * 30 / 100,\n                y = rect.y1 + w * 30 / 100;\n            canvas.fillPolygon([new Point(rect.x1, rect.y1), new Point(x, rect.y1), new Point(rect.x2, y), new Point(rect.x2, rect.y2), new Point(rect.x1, rect.y2)]);\n            canvas.polygon([new Point(rect.x1, rect.y1), new Point(x, rect.y1), new Point(rect.x2, y), new Point(rect.x2, rect.y2), new Point(rect.x1, rect.y2)]);\n            canvas.polygon([new Point(x, rect.y1), new Point(x, y), new Point(rect.x2, y)]);\n        }\n\n    };\n\n\n    /**\n     * UMLArtifactView\n     * @constructor\n     * @mixes UMLArtifactViewMixin\n     * @extends UMLClassifierView\n     */\n    function UMLArtifactView() {\n        UMLClassifierView.apply(this, arguments);\n        this.iconRatio = ARTIFACT_RATIO_PERCENT;\n        // mixin UMLArtifactViewMixin\n        _.extend(UMLArtifactView.prototype, UMLArtifactViewMixin);\n\n        this.fillColor  = PreferenceManager.get("uml.artifact.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.artifact.stereotypeDisplay", UML.SD_ICON);\n        this.suppressAttributes = PreferenceManager.get("uml.artifact.suppressAttributes", true);\n        this.suppressOperations = PreferenceManager.get("uml.artifact.suppressOperations", true);\n    }\n    // inherits from UMLClassifierView\n    UMLArtifactView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLArtifactView.prototype.constructor = UMLArtifactView;\n\n    UMLArtifactView.prototype.getStereotypeLabelText = function () {\n        return "«artifact»";\n    };\n\n\n    /**\n     * UMLArtifactInstanceView\n     * @constructor\n     * @mixes UMLArtifactViewMixin\n     * @extends UMLGeneralNodeView\n     */\n    function UMLArtifactInstanceView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.iconRatio = ARTIFACT_RATIO_PERCENT;\n        this.containerChangeable = true;\n        // mixin UMLArtifactViewMixin\n        _.extend(UMLArtifactInstanceView.prototype, UMLArtifactViewMixin);\n\n        this.fillColor = PreferenceManager.get("uml.artifactinstance.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay = PreferenceManager.get("uml.artifact.stereotypeDisplay", UML.SD_ICON);\n    }\n    // inherits from UMLGeneralNodeView\n    UMLArtifactInstanceView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLArtifactInstanceView.prototype.constructor = UMLArtifactInstanceView;\n\n    UMLArtifactInstanceView.prototype.getStereotypeLabelText = function () {\n        return "«artifact»";\n    };\n\n    UMLArtifactInstanceView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameCompartment.nameLabel.text = this.model.getString(this);\n        }\n    };\n\n    /**\n     * UMLComponentViewMixin\n     * @mixin\n     */\n    var UMLComponentViewMixin = {\n\n        sizeAsCanonicalForm: function (canvas, showLabel) {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            this.minWidth = Math.max(sz.x + COMPONENT_STATIC_MARGIN, COMPONENT_MINWIDTH);\n            this.minHeight = Math.max(sz.y, COMPONENT_MINHEIGHT);\n        },\n\n        sizeAsIconicForm: function (canvas, showLabel) {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            this.minWidth = Math.max(sz.x, COMPONENT_ICON_MINWIDTH);\n            this.minHeight = COMPONENT_ICON_MINHEIGHT + sz.y;\n        },\n\n        arrangeCommon: function (canvas) {\n            UMLGeneralNodeView.prototype.arrangeCommon.call(this, canvas);\n            if (this.stereotypeDisplay == UML.SD_NONE || this.stereotypeDisplay == UML.SD_LABEL) {\n                this.mainRect.setRect(this.left + COMPONENT_STATIC_MARGIN, this.top, this.getRight(), this.getBottom());\n            }\n        },\n\n        drawShadowAsCanonicalForm: function (canvas) {\n            canvas.fillRect(\n                this.left + COMPONENT_RECT_INDENT + SHADOW_OFFSET,\n                this.top + SHADOW_OFFSET,\n                this.getRight() + SHADOW_OFFSET,\n                this.getBottom() + SHADOW_OFFSET);\n        },\n\n        drawCommon: function (canvas) {\n            if (this.stereotypeDisplay == UML.SD_DECORATION) {\n                canvas.fillRect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n                canvas.rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n            }\n        },\n\n        drawIcon: function (canvas, rect) {\n            canvas.fillRect(rect.x1 + COMPONENT_RECT_INDENT, rect.y1, rect.x2, rect.y2);\n            canvas.rect(rect.x1 + COMPONENT_RECT_INDENT, rect.y1, rect.x2, rect.y2);\n            canvas.fillRect(rect.x1, rect.y1 + 7, rect.x1 + 20, rect.y1 + 17);\n            canvas.rect(rect.x1, rect.y1 + 7, rect.x1 + 20, rect.y1 + 17);\n            canvas.fillRect(rect.x1, rect.y1 + 27, rect.x1 + 20, rect.y1 + 37);\n            canvas.rect(rect.x1, rect.y1 + 27, rect.x1 + 20, rect.y1 + 37);\n        },\n\n        drawDecorationIcon: function (canvas, rect) {\n            canvas.fillRect(rect.x1 + 9, rect.y1 + 5,  rect.x1 + 22, rect.y1 + 21);\n            canvas.rect(rect.x1 + 9, rect.y1 + 5,  rect.x1 + 22, rect.y1 + 21);\n            canvas.fillRect(rect.x1 + 6, rect.y1 + 8,  rect.x1 + 13, rect.y1 + 12);\n            canvas.rect(rect.x1 + 6, rect.y1 + 8,  rect.x1 + 13, rect.y1 + 12);\n            canvas.fillRect(rect.x1 + 6, rect.y1 + 14, rect.x1 + 13, rect.y1 + 18);\n            canvas.rect(rect.x1 + 6, rect.y1 + 14, rect.x1 + 13, rect.y1 + 18);\n        },\n\n        drawAsCanonicalForm: function (canvas, showLabel) {\n            UMLGeneralNodeView.prototype.drawAsCanonicalForm.call(this, canvas, showLabel);\n            canvas.fillRect(this.left + COMPONENT_RECT_INDENT, this.top, this.getRight(), this.getBottom());\n            canvas.rect(this.left + COMPONENT_RECT_INDENT, this.top, this.getRight(), this.getBottom());\n            canvas.fillRect(this.left, this.top + 7, this.left + 20, this.top + 17);\n            canvas.rect(this.left, this.top + 7, this.left + 20, this.top + 17);\n            canvas.fillRect(this.left, this.top + 27, this.left + 20, this.top + 37);\n            canvas.rect(this.left, this.top + 27, this.left + 20, this.top + 37);\n        },\n\n        drawAsDecorationForm: function (canvas, showLabel) {\n            canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n            UMLGeneralNodeView.prototype.drawAsDecorationForm.call(this, canvas, showLabel);\n        }\n\n    };\n\n\n    /**\n     * UMLComponentView\n     * @constructor\n     * @mixes UMLComponentViewMixin\n     * @extends UMLClassifierView\n     */\n    function UMLComponentView() {\n        UMLClassifierView.apply(this, arguments);\n\n        // mixin UMLComponentViewMixin\n        _.extend(UMLComponentView.prototype, UMLComponentViewMixin);\n\n        this.fillColor = PreferenceManager.get("uml.component.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.component.stereotypeDisplay", UML.SD_LABEL);\n        this.suppressAttributes = PreferenceManager.get("uml.component.suppressAttributes", true);\n        this.suppressOperations = PreferenceManager.get("uml.component.suppressOperations", true);\n    }\n    // inherits from UMLClassifierView\n    UMLComponentView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLComponentView.prototype.constructor = UMLComponentView;\n\n\n    /**\n     * UMLComponentInstanceView\n     * @constructor\n     * @mixes UMLComponentViewMixin\n     * @extends UMLGeneralNodeView\n     */\n    function UMLComponentInstanceView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = true;\n        // mixin UMLComponentViewMixin\n        _.extend(UMLComponentInstanceView.prototype, UMLComponentViewMixin);\n\n        this.fillColor = PreferenceManager.get("uml.componentinstance.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.component.stereotypeDisplay", UML.SD_LABEL);\n    }\n    // inherits from UMLGeneralNodeView\n    UMLComponentInstanceView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLComponentInstanceView.prototype.constructor = UMLComponentInstanceView;\n\n    UMLComponentInstanceView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameCompartment.nameLabel.text = this.model.getString(this);\n        }\n    };\n\n\n    /**\n     * UMLComponentRealizationView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLComponentRealizationView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_TRIANGLE;\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLComponentRealizationView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLComponentRealizationView.prototype.constructor = UMLComponentRealizationView;\n\n    UMLComponentRealizationView.prototype.canConnectTo = function (view, isTail) {\n        return (isTail && view.model instanceof type.UMLClassifier) ||\n               (!isTail && view.model instanceof type.UMLComponent);\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                        DEPLOYMENT DIAGRAM VIEWS                        *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLDeploymentDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLDeploymentDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLDeploymentDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLDeploymentDiagram.prototype.constructor = UMLDeploymentDiagram;\n\n    UMLDeploymentDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLPackage) ||\n               (model instanceof type.UMLClassifier) ||\n               (model instanceof type.UMLInstance) ||\n               (model instanceof type.UMLPort) ||\n               (model instanceof type.UMLAttribute) ||\n               (model instanceof type.UMLGeneralization) ||\n               (model instanceof type.UMLDependency) ||\n               (model instanceof type.UMLInterfaceRealization) ||\n               (model instanceof type.UMLComponentRealization) ||\n               (model instanceof type.UMLAssociation) ||\n               (model instanceof type.UMLLink) ||\n               (model instanceof type.UMLConnector);\n    };\n\n\n    /**\n     * UMLNodeViewMixin\n     * @mixin\n     */\n    var UMLNodeViewMixin = {\n\n        sizeAsCanonicalForm: function (canvas, showLabel) {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            this.minWidth = Math.max(sz.x + NODE_STATIC_MARGIN, NODE_MINWIDTH);\n            this.minHeight = Math.max(sz.y + NODE_STATIC_MARGIN, NODE_MINHEIGHT);\n        },\n\n        sizeAsIconicForm: function (canvas, showLabel) {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            this.minWidth = Math.max(sz.x, NODE_MINWIDTH);\n            this.minHeight = NODE_MINHEIGHT + sz.y;\n        },\n\n        arrangeCommon: function (canvas) {\n            UMLGeneralNodeView.prototype.arrangeCommon.call(this, canvas);\n            if (this.stereotypeDisplay == UML.SD_NONE || this.stereotypeDisplay == UML.SD_LABEL) {\n                this.mainRect.setRect(this.left, this.top + NODE_STATIC_MARGIN, this.getRight() - NODE_STATIC_MARGIN, this.getBottom());\n            }\n        },\n\n        drawShadowAsCanonicalForm: function (canvas) {\n            var r = this.getRight() - 1, b = this.getBottom()- 1;\n            canvas.fillPolygon([\n                new Point(this.left + SHADOW_OFFSET, this.top + NODE_STATIC_MARGIN + SHADOW_OFFSET),\n                new Point(r - NODE_STATIC_MARGIN + SHADOW_OFFSET, this.top + NODE_STATIC_MARGIN + SHADOW_OFFSET),\n                new Point(r - NODE_STATIC_MARGIN + SHADOW_OFFSET, b + SHADOW_OFFSET),\n                new Point(this.left + SHADOW_OFFSET, b + SHADOW_OFFSET)]);\n            canvas.fillPolygon([\n                new Point(this.left + SHADOW_OFFSET, this.top + NODE_STATIC_MARGIN + SHADOW_OFFSET),\n                new Point(this.left + NODE_STATIC_MARGIN + SHADOW_OFFSET, this.top + SHADOW_OFFSET),\n                new Point(r + SHADOW_OFFSET, this.top + SHADOW_OFFSET),\n                new Point(r - NODE_STATIC_MARGIN + SHADOW_OFFSET, this.top + NODE_STATIC_MARGIN + SHADOW_OFFSET)]);\n            canvas.fillPolygon([\n                new Point(r + SHADOW_OFFSET, this.top + SHADOW_OFFSET),\n                new Point(r - NODE_STATIC_MARGIN + SHADOW_OFFSET, this.top + NODE_STATIC_MARGIN + SHADOW_OFFSET),\n                new Point(r - NODE_STATIC_MARGIN + SHADOW_OFFSET, b + SHADOW_OFFSET),\n                new Point(r + SHADOW_OFFSET, b - NODE_STATIC_MARGIN + SHADOW_OFFSET)]);\n        },\n\n        drawShadowAsIconicForm: function (canvas) {\n            var r = this.iconRect.x2 - 1, b = this.iconRect.y2 - 1;\n            canvas.fillPolygon([\n                new Point(this.iconRect.x1 + SHADOW_OFFSET, this.iconRect.y1+NODE_STATIC_MARGIN + SHADOW_OFFSET),\n                new Point(r-NODE_STATIC_MARGIN + SHADOW_OFFSET, this.iconRect.y1+NODE_STATIC_MARGIN + SHADOW_OFFSET),\n                new Point(r-NODE_STATIC_MARGIN + SHADOW_OFFSET, b + SHADOW_OFFSET),\n                new Point(this.iconRect.x1 + SHADOW_OFFSET, b + SHADOW_OFFSET)]);\n            canvas.fillPolygon([\n                new Point(this.iconRect.x1 + SHADOW_OFFSET, this.iconRect.y1+NODE_STATIC_MARGIN + SHADOW_OFFSET),\n                new Point(this.iconRect.x1+NODE_STATIC_MARGIN + SHADOW_OFFSET, this.iconRect.y1 + SHADOW_OFFSET),\n                new Point(r + SHADOW_OFFSET, this.iconRect.y1 + SHADOW_OFFSET),\n                new Point(r-NODE_STATIC_MARGIN + SHADOW_OFFSET, this.iconRect.y1+NODE_STATIC_MARGIN + SHADOW_OFFSET)]);\n            canvas.fillPolygon([\n                new Point(r + SHADOW_OFFSET, this.iconRect.y1 + SHADOW_OFFSET),\n                new Point(r-NODE_STATIC_MARGIN + SHADOW_OFFSET, this.iconRect.y1+NODE_STATIC_MARGIN + SHADOW_OFFSET),\n                new Point(r-NODE_STATIC_MARGIN + SHADOW_OFFSET, b + SHADOW_OFFSET),\n                new Point(r + SHADOW_OFFSET, b-NODE_STATIC_MARGIN + SHADOW_OFFSET)]);\n        },\n\n        drawCommon: function (canvas) {\n            if (this.stereotypeDisplay == UML.SD_DECORATION) {\n                canvas.fillRect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n                canvas.rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n            }\n        },\n\n        drawIcon: function (canvas, rect) {\n            var r = rect.x2 - 1, b = rect.y2 - 1;\n            canvas.fillPolygon([new Point(rect.x1, rect.y1+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, rect.y1+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, b), new Point(rect.x1, b)]);\n            canvas.fillPolygon([new Point(rect.x1, rect.y1+NODE_STATIC_MARGIN), new Point(rect.x1+NODE_STATIC_MARGIN, rect.y1), new Point(r, rect.y1), new Point(r-NODE_STATIC_MARGIN, rect.y1+NODE_STATIC_MARGIN)]);\n            canvas.fillPolygon([new Point(r, rect.y1), new Point(r-NODE_STATIC_MARGIN, rect.y1+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, b), new Point(r, b-NODE_STATIC_MARGIN)]);\n            canvas.polygon([new Point(rect.x1, rect.y1+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, rect.y1+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, b), new Point(rect.x1, b)]);\n            canvas.polygon([new Point(rect.x1, rect.y1+NODE_STATIC_MARGIN), new Point(rect.x1+NODE_STATIC_MARGIN, rect.y1), new Point(r, rect.y1), new Point(r-NODE_STATIC_MARGIN, rect.y1+NODE_STATIC_MARGIN)]);\n            canvas.polygon([new Point(r, rect.y1), new Point(r-NODE_STATIC_MARGIN, rect.y1+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, b), new Point(r, b-NODE_STATIC_MARGIN)]);\n        },\n\n        drawDecorationIcon: function (canvas, rect) {\n            canvas.rect(rect.x1 + 3, rect.y1 + 10, rect.x1 + 19, rect.y1 + 20);\n            canvas.polygon([new Point(rect.x1 + 3, rect.y1 + 10), new Point(rect.x1 + 8, rect.y1 + 5), new Point(rect.x1 + 24, rect.y1 + 5), new Point(rect.x1 + 19, rect.y1 + 10)]);\n            canvas.polygon([new Point(rect.x1 + 19, rect.y1 + 10), new Point(rect.x1 + 24, rect.y1 + 5), new Point(rect.x1 + 24, rect.y1 + 15), new Point(rect.x1 + 19, rect.y1 + 20)]);\n        },\n\n        drawAsCanonicalForm: function (canvas) {\n            UMLGeneralNodeView.prototype.drawAsCanonicalForm.call(this, canvas);\n            var r = this.getRight() - 1, b = this.getBottom()- 1;\n            canvas.fillPolygon([new Point(this.left, this.top+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, this.top+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, b), new Point(this.left, b)]);\n            canvas.fillPolygon([new Point(this.left, this.top+NODE_STATIC_MARGIN), new Point(this.left+NODE_STATIC_MARGIN, this.top), new Point(r, this.top), new Point(r-NODE_STATIC_MARGIN, this.top+NODE_STATIC_MARGIN)]);\n            canvas.fillPolygon([new Point(r, this.top), new Point(r-NODE_STATIC_MARGIN, this.top+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, b), new Point(r, b-NODE_STATIC_MARGIN)]);\n            canvas.polygon([new Point(this.left, this.top+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, this.top+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, b), new Point(this.left, b)]);\n            canvas.polygon([new Point(this.left, this.top+NODE_STATIC_MARGIN), new Point(this.left+NODE_STATIC_MARGIN, this.top), new Point(r, this.top), new Point(r-NODE_STATIC_MARGIN, this.top+NODE_STATIC_MARGIN)]);\n            canvas.polygon([new Point(r, this.top), new Point(r-NODE_STATIC_MARGIN, this.top+NODE_STATIC_MARGIN), new Point(r-NODE_STATIC_MARGIN, b), new Point(r, b-NODE_STATIC_MARGIN)]);\n        },\n\n        drawAsDecorationForm: function (canvas, showLabel) {\n            canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n            UMLGeneralNodeView.prototype.drawAsDecorationForm.call(this, canvas, showLabel);\n        }\n    };\n\n\n    /**\n     * UMLNodeView\n     * @constructor\n     * @mixes UMLNodeViewMixin\n     * @extends UMLClassifierView\n     */\n    function UMLNodeView() {\n        UMLClassifierView.apply(this, arguments);\n\n        this.iconRatio = NODE_RATIO_PERCENT;\n        // mixin UMLNodeViewMixin\n        _.extend(UMLNodeView.prototype, UMLNodeViewMixin);\n\n        this.fillColor = PreferenceManager.get("uml.node.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.node.stereotypeDisplay", UML.SD_LABEL);\n        this.suppressAttributes = PreferenceManager.get("uml.node.suppressAttributes", true);\n        this.suppressOperations = PreferenceManager.get("uml.node.suppressOperations", true);\n    }\n    // inherits from UMLClassifierView\n    UMLNodeView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLNodeView.prototype.constructor = UMLNodeView;\n\n\n    /**\n     * UMLNodeInstanceView\n     * @constructor\n     * @mixes UMLNodeViewMixin\n     * @extends UMLGeneralNodeView\n     */\n    function UMLNodeInstanceView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.iconRatio = NODE_RATIO_PERCENT;\n        this.containerChangeable = true;\n        // mixin UMLNodeViewMixin\n        _.extend(UMLNodeInstanceView.prototype, UMLNodeViewMixin);\n\n        this.fillColor = PreferenceManager.get("uml.nodeinstance.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.node.stereotypeDisplay", UML.SD_LABEL);\n    }\n    // inherits from UMLGeneralNodeView\n    UMLNodeInstanceView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLNodeInstanceView.prototype.constructor = UMLNodeInstanceView;\n\n    UMLNodeInstanceView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameCompartment.nameLabel.text = this.model.getString(this);\n        }\n    };\n\n\n    /**\n     * UMLDeploymentView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLDeploymentView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLDeploymentView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLDeploymentView.prototype.constructor = UMLDeploymentView;\n\n    UMLDeploymentView.prototype.update = function (canvas) {\n        UMLGeneralEdgeView.prototype.update.call(this, canvas);\n        this.stereotypeLabel.visible = true;\n        this.stereotypeLabel.text = "«deploy»";\n    };\n\n    UMLDeploymentView.prototype.canConnectTo = function (view, isTail) {\n        return (isTail && view.model instanceof type.UMLClassifier) ||\n               (!isTail && view.model instanceof type.UMLNode);\n    };\n\n    /**\n     * UMLCommunicationPathView\n     * @constructor\n     * @extends UMLAssociationView\n     */\n    function UMLCommunicationPathView() {\n        UMLAssociationView.apply(this, arguments);\n    }\n    // inherits from UMLAssociationView\n    UMLCommunicationPathView.prototype = Object.create(UMLAssociationView.prototype);\n    UMLCommunicationPathView.prototype.constructor = UMLCommunicationPathView;\n\n    UMLCommunicationPathView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLNode);\n    };\n\n\n    /**************************************************************************\n     *                                                                        *\n     *                         USE-CASE DIAGRAM VIEWS                         *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLUseCaseDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLUseCaseDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLUseCaseDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLUseCaseDiagram.prototype.constructor = UMLUseCaseDiagram;\n\n    UMLUseCaseDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLPackage) ||\n               (model instanceof type.UMLClassifier) ||\n               (model instanceof type.UMLUseCaseSubject) ||\n               (model instanceof type.UMLInstance) ||\n               (model instanceof type.UMLGeneralization) ||\n               (model instanceof type.UMLDependency) ||\n               (model instanceof type.UMLInterfaceRealization) ||\n               (model instanceof type.UMLComponentRealization) ||\n               (model instanceof type.UMLAssociation) ||\n               (model instanceof type.UMLLink) ||\n               (model instanceof type.UMLConnector);\n    };\n\n    UMLUseCaseDiagram.prototype.layout = function (direction, separations) {\n        if (!direction) {\n            direction = Core.DIRECTION_RL;\n        }\n        UMLDiagram.prototype.layout.call(this, direction, separations);\n    };\n\n\n    /**\n     * UMLExtensionPointView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLExtensionPointView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLExtensionPointView.prototype = Object.create(LabelView.prototype);\n    UMLExtensionPointView.prototype.constructor = UMLExtensionPointView;\n\n    UMLExtensionPointView.prototype.update = function (canvas) {\n        var options = {\n            stereotypeDisplay : UML.SD_LABEL,\n            showProperty      : true\n        };\n        if (this._parent && this._parent._parent) {\n            options.stereotypeDisplay = this._parent._parent.stereotypeDisplay;\n            options.showProperty      = this._parent._parent.showProperty;\n        }\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            this.text = this.model.getString(options);\n        }\n        LabelView.prototype.update.call(this, canvas);\n    };\n\n    UMLExtensionPointView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLExtensionPointCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLExtensionPointCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLExtensionPointCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLExtensionPointCompartmentView.prototype.constructor = UMLExtensionPointCompartmentView;\n\n    UMLExtensionPointCompartmentView.prototype.update = function (canvas) {\n        if (this.model.extensionPoints) {\n            var tempViews = this.subViews;\n            this.subViews = [];\n            for (var i = 0, len = this.model.extensionPoints.length; i < len; i++) {\n                var ep = this.model.extensionPoints[i];\n                var epView = _.find(tempViews, function (v) { return v.model == ep; });\n                if (!epView) {\n                    epView = new UMLExtensionPointView();\n                    epView.model = ep;\n                    epView._parent = this;\n                    // epView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', epView);\n                } else {\n                    this.addSubView(epView);\n                }\n                epView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLUseCaseView\n     * @constructor\n     * @extends UMLClassifierView\n     */\n    function UMLUseCaseView() {\n        UMLClassifierView.apply(this, arguments);\n        this.iconRatio = USECASE_RATIO_PERCENT;\n\n        /** @member {UMLExtensionPointCompartmentView} */\n        this.extensionPointCompartment = new UMLExtensionPointCompartmentView();\n        this.extensionPointCompartment.parentStyle = true;\n        this.addSubView(this.extensionPointCompartment);\n\n        this.fillColor = PreferenceManager.get("uml.usecase.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.usecase.stereotypeDisplay", UML.SD_LABEL);\n        this.suppressAttributes = PreferenceManager.get("uml.usecase.suppressAttributes", true);\n        this.suppressOperations = PreferenceManager.get("uml.usecase.suppressOperations", true);\n    }\n    // inherits from UMLClassifierView\n    UMLUseCaseView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLUseCaseView.prototype.constructor = UMLUseCaseView;\n\n    UMLUseCaseView.prototype.getAllCompartments = function () {\n        return [\n            this.nameCompartment,\n            this.attributeCompartment,\n            this.operationCompartment,\n            this.extensionPointCompartment\n        ];\n    };\n\n    UMLUseCaseView.prototype.update = function (canvas) {\n        // extensionPointCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.extensionPointCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.extensionPointCompartment, \'model\', this.model);\n        }\n        if (this.model) {\n            if (this.model.extensionPoints && this.model.extensionPoints.length > 0) {\n                this.extensionPointCompartment.visible = true;\n            } else {\n                this.extensionPointCompartment.visible = false;\n            }\n        }\n        UMLClassifierView.prototype.update.call(this, canvas);\n    };\n\n    UMLUseCaseView.prototype.sizeAsCanonicalForm = function (canvas, showLabel) {\n        UMLClassifierView.prototype.sizeAsCanonicalForm.call(this, canvas, showLabel);\n        // Calculating minimum size <= minimum Namecompartment circumscription size\n        // rectangle\'s circumscription ellipse height and width are rectangle\'s height and width * Sqrt(2)\n        var w = Math.max(Math.trunc(Math.sqrt(2) * this.nameCompartment.minWidth), USECASE_ICON_MINWIDTH);\n        if (this.attributeCompartment.visible) {\n            w = Math.max(w, this.attributeCompartment.minWidth);\n        }\n        if (this.operationCompartment.visible) {\n            w = Math.max(w, this.operationCompartment.minWidth);\n        }\n        if (this.extensionPointCompartment.visible) {\n            w = Math.max(w, this.extensionPointCompartment.minWidth);\n        }\n        this.minWidth = w;\n        var h = Math.max(Math.trunc(Math.sqrt(2) * this.nameCompartment.minHeight), USECASE_ICON_MINHEIGHT);\n        if (this.attributeCompartment.visible) {\n            h = h + this.attributeCompartment.minHeight;\n        }\n        if (this.operationCompartment.visible) {\n            h = h + this.operationCompartment.minHeight;\n        }\n        if (this.extensionPointCompartment.visible) {\n            h = h + this.extensionPointCompartment.minHeight;\n        }\n        this.minHeight = h;\n    };\n\n    UMLUseCaseView.prototype.sizeAsIconicForm = function (canvas, showLabel) {\n        if (this.hasStereotypeIcon()) {\n            UMLClassifierView.prototype.sizeAsIconicForm.call(this, canvas, showLabel);\n        } else {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            this.minWidth = Math.max(sz.x, USECASE_ICON_MINWIDTH);\n            this.minHeight = USECASE_ICON_MINHEIGHT + sz.y;\n        }\n    };\n\n    UMLUseCaseView.prototype.arrangeAsCanonicalForm = function (canvas, showLabel) {\n        UMLClassifierView.prototype.arrangeAsCanonicalForm.call(this, canvas, showLabel);\n        var y = this.getBottom();\n        if (this.extensionPointCompartment.visible) {\n            y = y - this.extensionPointCompartment.height;\n            this.extensionPointCompartment.left = this.left;\n            this.extensionPointCompartment.setRight(this.getRight());\n            this.extensionPointCompartment.top = y;\n            this.extensionPointCompartment.arrange(canvas);\n        }\n        if (this.operationCompartment.visible) {\n            y = y - this.operationCompartment.height;\n            this.operationCompartment.left = this.left;\n            this.operationCompartment.setRight(this.getRight());\n            this.operationCompartment.top = y;\n            this.operationCompartment.arrange(canvas);\n        }\n        if (this.attributeCompartment.visible) {\n            y = y - this.attributeCompartment.height;\n            this.attributeCompartment.left = this.left;\n            this.attributeCompartment.setRight(this.getRight());\n            this.attributeCompartment.top = y;\n            this.attributeCompartment.arrange(canvas);\n        }\n        this.nameCompartment.width = Math.trunc(1 / Math.sqrt(2) * this.width);\n        this.nameCompartment.left = this.left + (this.width - this.nameCompartment.width) / 2;\n        this.nameCompartment.top = ((y + this.top) / 2) - (this.nameCompartment.height / 2);\n        this.nameCompartment.arrange(canvas);\n    };\n\n    UMLUseCaseView.prototype.drawShadowAsCanonicalForm = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        var EXTENSIONPOINTS_MARGIN_TOP = 5;\n        if (this.extensionPointCompartment.visible) {\n            r.y2 = r.y2 - this.extensionPointCompartment.height - EXTENSIONPOINTS_MARGIN_TOP;\n        }\n        if (this.operationCompartment.visible) {\n            r.y2 = r.y2 - this.operationCompartment.height;\n        }\n        if (this.attributeCompartment.visible) {\n            r.y2 = r.y2 - this.attributeCompartment.height;\n        }\n        canvas.fillEllipse(\n            r.x1 + SHADOW_OFFSET,\n            r.y1 + SHADOW_OFFSET,\n            r.x2 + SHADOW_OFFSET,\n            r.y2 + SHADOW_OFFSET);\n    };\n\n    UMLUseCaseView.prototype.drawShadowAsDecorationForm = function (canvas) {\n        canvas.fillRect(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET\n        );\n    };\n\n    UMLUseCaseView.prototype.drawShadowAsIconicForm = function (canvas) {\n        if (this.hasStereotypeIcon()) {\n            UMLClassifierView.prototype.drawShadowAsIconicForm.call(this, canvas);\n        } else {\n            canvas.fillEllipse(\n                this.iconRect.x1 + SHADOW_OFFSET,\n                this.iconRect.y1 + SHADOW_OFFSET,\n                this.iconRect.x2 + SHADOW_OFFSET,\n                this.iconRect.y2 + SHADOW_OFFSET\n            );\n        }\n    };\n\n    UMLUseCaseView.prototype.drawIcon = function (canvas, rect) {\n        if (this.hasStereotypeIcon()) {\n            var ratioRect = this.computeIconRect(rect, (this.model.stereotype.icon.width / this.model.stereotype.icon.height) * 100);\n            drawImage(canvas, ratioRect, this.model.stereotype.icon);\n        } else {\n            var r = rect;\n            var x, y;\n            var w = r.x2 - r.x1;\n            var h = r.y2 - r.y1;\n            var rr = w * 100 / h;\n            var ir = USECASE_ICON_MINWIDTH * 100 / USECASE_ICON_MINHEIGHT;\n            if (rr >= ir) {\n                h = (r.y2 - r.y1);\n                w = h * ir / 100;\n                x = r.x1 + (r.x2 - r.x1 - w) / 2;\n                y = r.y1;\n            } else {\n                w = (r.x2 - r.x1);\n                h = w * 100 / ir;\n                y = r.y1 + (r.y2 - r.y1 - h) / 2;\n                x = r.x1;\n            }\n            canvas.fillEllipse(x, y, x + w, y + h);\n            canvas.ellipse(x, y, x + w, y + h);\n        }\n    };\n\n    UMLUseCaseView.prototype.drawObject = function (canvas) {\n        UMLClassifierView.prototype.drawObject.call(this, canvas);\n        if (this.extensionPointCompartment.visible) {\n            canvas.line(\n                this.extensionPointCompartment.left,\n                this.extensionPointCompartment.top,\n                this.extensionPointCompartment.getRight(),\n                this.extensionPointCompartment.top);\n        }\n    };\n\n    UMLUseCaseView.prototype.drawCommon = function (canvas) {\n        if (this.stereotypeDisplay === UML.SD_DECORATION || this.stereotypeDisplay === UML.SD_DECORATION_LABEL) {\n            canvas.fillRect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n            canvas.rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n        }\n    };\n\n    UMLUseCaseView.prototype.drawAsCanonicalForm = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        var EXTENSIONPOINTS_MARGIN_TOP = 5;\n        if (this.extensionPointCompartment.visible) {\n            r.y2 = r.y2 - this.extensionPointCompartment.height - EXTENSIONPOINTS_MARGIN_TOP;\n        }\n        if (this.operationCompartment.visible) {\n            r.y2 = r.y2 - this.operationCompartment.height;\n        }\n        if (this.attributeCompartment.visible) {\n            r.y2 = r.y2 - this.attributeCompartment.height;\n        }\n        canvas.fillEllipse(r.x1, r.y1, r.x2, r.y2);\n        canvas.ellipse(r.x1, r.y1, r.x2, r.y2);\n    };\n\n\n    /**\n     * UMLActorView\n     * @constructor\n     * @extends UMLClassifierView\n     */\n    function UMLActorView() {\n        UMLClassifierView.apply(this, arguments);\n        this.iconRatio = ACTOR_RATIO_PERCENT;\n\n        this.fillColor = PreferenceManager.get("uml.actor.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this.stereotypeDisplay  = PreferenceManager.get("uml.actor.stereotypeDisplay", UML.SD_LABEL);\n        this.suppressAttributes = PreferenceManager.get("uml.actor.suppressAttributes", true);\n        this.suppressOperations = PreferenceManager.get("uml.actor.suppressOperations", true);\n    }\n    // inherits from UMLClassifierView\n    UMLActorView.prototype = Object.create(UMLClassifierView.prototype);\n    UMLActorView.prototype.constructor = UMLActorView;\n\n    UMLActorView.prototype.sizeAsCanonicalForm = function (canvas, showLabel) {\n        this.sizeAsIconicForm(canvas, showLabel);\n    };\n\n    UMLActorView.prototype.sizeAsIconicForm = function (canvas, showLabel) {\n        if (this.hasStereotypeIcon()) {\n            UMLClassifierView.prototype.sizeAsIconicForm.call(this, canvas, showLabel);\n        } else {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            this.minWidth = Math.max(sz.x, ACTOR_ICON_MINWIDTH);\n            this.minHeight = ACTOR_ICON_MINHEIGHT + sz.y;\n        }\n    };\n\n    UMLActorView.prototype.arrangeAsCanonicalForm = function (canvas, showLabel) {\n        this.arrangeAsIconicForm(canvas, showLabel);\n    };\n\n    UMLActorView.prototype.drawShadowAsCanonicalForm = function (canvas) {};\n\n    UMLActorView.prototype.drawShadowAsDecorationForm = function (canvas) {\n        canvas.fillRect(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET\n        );\n    };\n\n    UMLActorView.prototype.drawShadowAsIconicForm = function (canvas) {};\n\n    UMLActorView.prototype.drawIcon = function (canvas, rect) {\n        if (this.hasStereotypeIcon()) {\n            var ratioRect = this.computeIconRect(rect, (this.model.stereotype.icon.width / this.model.stereotype.icon.height) * 100);\n            drawImage(canvas, ratioRect, this.model.stereotype.icon);\n        } else {\n            var cx, h, w, xa, xh, ya, yh, yl;\n            w = rect.x2 - rect.x1;\n            h = rect.y2 - rect.y1;\n            xh = w * 16 / 100;\n            xa = w * 14 / 100;\n            yh = rect.y1 + h * 34 / 100;\n            ya = rect.y1 + h * 46 / 100;\n            yl = rect.y1 + h * 66 / 100;\n            cx = rect.x1 + w / 2;\n            canvas.fillEllipse(rect.x1 + xh, rect.y1 + 1, rect.x2 - xh, yh);\n            canvas.ellipse(rect.x1 + xh, rect.y1 + 1, rect.x2 - xh, yh);\n            canvas.polyline([new Point(cx, yh), new Point(cx, yl)]);\n            canvas.polyline([new Point(rect.x1 + xa, ya), new Point(rect.x2 - xa, ya)]);\n            canvas.polyline([new Point(cx, yl), new Point(rect.x1, rect.y2 - 1)]);\n            canvas.polyline([new Point(cx, yl), new Point(rect.x2, rect.y2 - 1)]);\n        }\n    };\n\n    UMLActorView.prototype.drawCommon = function (canvas) {\n        if (this.stereotypeDisplay === UML.SD_DECORATION || this.stereotypeDisplay === UML.SD_DECORATION_LABEL) {\n            canvas.fillRect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n            canvas.rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2);\n        }\n    };\n\n    UMLActorView.prototype.drawAsCanonicalForm = function (canvas) {\n        this.drawAsIconicForm(canvas);\n    };\n\n\n    /**\n     * UMLIncludeView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLIncludeView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLIncludeView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLIncludeView.prototype.constructor = UMLIncludeView;\n\n    UMLIncludeView.prototype.update = function (canvas) {\n        UMLGeneralEdgeView.prototype.update.call(this, canvas);\n        this.stereotypeLabel.visible = true;\n        this.stereotypeLabel.text = "«include»";\n    };\n\n    UMLIncludeView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLUseCase);\n    };\n\n    /**\n     * UMLExtendView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLExtendView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLExtendView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLExtendView.prototype.constructor = UMLExtendView;\n\n    UMLExtendView.prototype.update = function (canvas) {\n        UMLGeneralEdgeView.prototype.update.call(this, canvas);\n        this.stereotypeLabel.visible = true;\n        this.stereotypeLabel.text = "«extend»";\n    };\n\n    UMLExtendView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLUseCase);\n    };\n\n    /**\n     * UMLUseCaseSubjectView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLUseCaseSubjectView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.zIndex = -1;\n    }\n    // inherits from UMLGeneralNodeView\n    UMLUseCaseSubjectView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLUseCaseSubjectView.prototype.constructor = UMLUseCaseSubjectView;\n\n    UMLUseCaseSubjectView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model.represent && this.model.represent.name) {\n            this.nameCompartment.nameLabel.text = this.model.represent.name;\n        }\n    };\n\n    UMLUseCaseSubjectView.prototype.drawShadowAsCanonicalForm = function (canvas) {\n        canvas.fillRect(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET);\n    };\n\n    UMLUseCaseSubjectView.prototype.arrangeObject = function (canvas) {\n        UMLGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n    };\n\n    UMLUseCaseSubjectView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.fillRect(r.x1, r.y1, r.x2, r.y2);\n        canvas.rect(r.x1, r.y1, r.x2, r.y2);\n    };\n\n\n    /**************************************************************************\n     *                                                                        *\n     *                        STATECHART DIAGRAM VIEWS                        *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLStatechartDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLStatechartDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLStatechartDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLStatechartDiagram.prototype.constructor = UMLStatechartDiagram;\n\n    UMLStatechartDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLState) ||\n               (model instanceof type.UMLPseudostate) ||\n               (model instanceof type.UMLConnectionPointReference) ||\n               (model instanceof type.UMLStateMachine);\n    };\n\n    UMLStatechartDiagram.prototype.layout = function (direction, separations) {\n        if (!direction) {\n            direction = Core.DIRECTION_RL;\n        }\n        UMLDiagram.prototype.layout.call(this, direction, separations);\n    };\n\n\n    /**\n     * UMLPseudostateView\n     * @constructor\n     * @extends UMLFloatingNodeView\n     */\n    function UMLPseudostateView() {\n        UMLFloatingNodeView.apply(this, arguments);\n        this.sizable = Core.SZ_FREE;\n        this.containerChangeable = true;\n    }\n    // inherits from UMLFloatingNodeView\n    UMLPseudostateView.prototype = Object.create(UMLFloatingNodeView.prototype);\n    UMLPseudostateView.prototype.constructor = UMLPseudostateView;\n\n    UMLPseudostateView.prototype.sizeObject = function (canvas) {\n        UMLFloatingNodeView.prototype.sizeObject.call(this, canvas);\n        switch (this.model.kind) {\n        case UML.PSK_INITIAL:\n            this.minWidth = INITIALSTATE_MINWIDTHH;\n            this.minHeight = INITIALSTATE_MINHEIGHT;\n            break;\n        case UML.PSK_DEEPHISTORY:\n            this.minWidth = HISTORYSTATE_MINWIDTHH;\n            this.minHeight = HISTORYSTATE_MINHEIGHT;\n            break;\n        case UML.PSK_SHALLOWHISTORY:\n            this.minWidth = HISTORYSTATE_MINWIDTHH;\n            this.minHeight = HISTORYSTATE_MINHEIGHT;\n            break;\n        case UML.PSK_JOIN:\n            if (this.height > this.width) {\n                // Vertical Synchronization\n                this.minWidth = JOIN_MINTHICK;\n                this.minHeight = JOIN_MINLENGTH;\n                this.width = this.minWidth;\n            } else {\n                // Horizontal Synchronization\n                this.minWidth = JOIN_MINLENGTH;\n                this.minHeight = JOIN_MINTHICK;\n                this.height = this.minHeight;\n            }\n            break;\n        case UML.PSK_FORK:\n            if (this.height > this.width) {\n                // Vertical Synchronization\n                this.minWidth = FORK_MINTHICK;\n                this.minHeight = FORK_MINLENGTH;\n                this.width = this.minWidth;\n            } else {\n                // Horizontal Synchronization\n                this.minWidth = FORK_MINLENGTH;\n                this.minHeight = FORK_MINTHICK;\n                this.height = this.minHeight;\n            }\n            break;\n        case UML.PSK_JUNCTION:\n            this.minWidth = JUNCTION_MINWIDTH;\n            this.minHeight = JUNCTION_MINHEIGHT;\n            break;\n        case UML.PSK_CHOICE:\n            this.minWidth = CHOICE_MINWIDTH;\n            this.minHeight = CHOICE_MINHEIGHT;\n            break;\n        case UML.PSK_ENTRYPOINT:\n            this.minWidth = ENTRYPOINT_MINWIDTH;\n            this.minHeight = ENTRYPOINT_MINHEIGHT;\n            break;\n        case UML.PSK_EXITPOINT:\n            this.minWidth = EXITPOINT_MINWIDTH;\n            this.minHeight = EXITPOINT_MINHEIGHT;\n            break;\n        case UML.PSK_TERMINATE:\n            this.minWidth = TERMINATE_MINWIDTH;\n            this.minHeight = TERMINATE_MINHEIGHT;\n            break;\n        }\n        this.sizeConstraints();\n    };\n\n    UMLPseudostateView.prototype.arrange = function (canvas) {\n        UMLFloatingNodeView.prototype.arrange.call(this, canvas);\n        this.stereotypeLabel.visible = false;\n        this.nameLabel.visible = false;\n    };\n\n    UMLPseudostateView.prototype.drawShadow = function (canvas) {\n        canvas.storeState();\n        canvas.alpha = SHADOW_ALPHA;\n        canvas.fillColor = SHADOW_COLOR;\n        switch (this.model.kind) {\n        case UML.PSK_INITIAL:\n        case UML.PSK_DEEPHISTORY:\n        case UML.PSK_SHALLOWHISTORY:\n        case UML.PSK_JUNCTION:\n        case UML.PSK_ENTRYPOINT:\n        case UML.PSK_EXITPOINT:\n            canvas.fillEllipse(\n                this.left        + SHADOW_OFFSET,\n                this.top         + SHADOW_OFFSET,\n                this.getRight()  + SHADOW_OFFSET,\n                this.getBottom() + SHADOW_OFFSET\n            );\n            break;\n        case UML.PSK_JOIN:\n        case UML.PSK_FORK:\n            canvas.fillRoundRect(this.left + SHADOW_OFFSET, this.top + SHADOW_OFFSET, this.getRight() + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET, 3);\n            break;\n        case UML.PSK_CHOICE:\n            var x = this.left + (this.width / 2);\n            var y = this.top + (this.height / 2);\n            canvas.fillPolygon([\n                new Point(this.left + SHADOW_OFFSET, y + SHADOW_OFFSET),\n                new Point(x + SHADOW_OFFSET, this.top + SHADOW_OFFSET),\n                new Point(this.getRight() + SHADOW_OFFSET, y + SHADOW_OFFSET),\n                new Point(x + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET),\n                new Point(this.left + SHADOW_OFFSET, y + SHADOW_OFFSET)]);\n            break;\n        case UML.PSK_TERMINATE:\n            // No shadow\n            break;\n        }\n        canvas.restoreState();\n        UMLFloatingNodeView.prototype.drawShadow.call(this, canvas);\n    };\n\n    UMLPseudostateView.prototype.drawObject = function (canvas) {\n        var x, y, sz, p, d;\n        switch (this.model.kind) {\n        case UML.PSK_INITIAL:\n            canvas.fillColor = this.lineColor;\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            break;\n        case UML.PSK_DEEPHISTORY:\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n            sz = canvas.textExtent(\'H*\');\n            x = this.left + (this.width - sz.x) / 2;\n            y = this.top + (this.height - sz.y) / 2;\n            canvas.textOut(x, y, \'H*\');\n            break;\n        case UML.PSK_SHALLOWHISTORY:\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n            sz = canvas.textExtent(\'H\');\n            x = this.left + (this.width - sz.x) / 2;\n            y = this.top + (this.height - sz.y) / 2;\n            canvas.textOut(x, y, \'H\');\n            break;\n        case UML.PSK_JOIN:\n            canvas.fillColor = this.lineColor;\n            if (Math.abs(this.getRight() - this.left) >= Math.abs(this.getBottom() - this.top)) {\n                canvas.fillRoundRect(this.left, this.top, this.getRight(), this.top + JOIN_MINTHICK, 3);\n            } else {\n                canvas.fillRoundRect(this.left, this.top, this.left + JOIN_MINTHICK, this.getBottom(), 3);\n            }\n            break;\n        case UML.PSK_FORK:\n            canvas.fillColor = this.lineColor;\n            if (Math.abs(this.getRight() - this.left) >= Math.abs(this.getBottom() - this.top)) {\n                canvas.fillRoundRect(this.left, this.top, this.getRight(), this.top + FORK_MINTHICK, 3);\n            } else {\n                canvas.fillRoundRect(this.left, this.top, this.left + FORK_MINTHICK, this.getBottom(), 3);\n            }\n            break;\n        case UML.PSK_JUNCTION:\n            canvas.fillColor = this.lineColor;\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n            break;\n        case UML.PSK_CHOICE:\n            x = (this.left + this.getRight()) / 2;\n            y = (this.top + this.getBottom()) / 2;\n            canvas.fillPolygon([new Point(this.left, y), new Point(x, this.top), new Point(this.getRight(), y), new Point(x, this.getBottom()), new Point(this.left, y)]);\n            canvas.polygon([new Point(this.left, y), new Point(x, this.top), new Point(this.getRight(), y), new Point(x, this.getBottom()), new Point(this.left, y)]);\n            break;\n        case UML.PSK_ENTRYPOINT:\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n            break;\n        case UML.PSK_EXITPOINT:\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n            p = Coord.getCenter(new Rect(this.left, this.top, this.getRight(), this.getBottom()));\n            d = Math.round(Math.sqrt(2) * this.width / 4);\n            canvas.line(p.x - d, p.y - d, p.x + d, p.y + d);\n            canvas.line(p.x + d, p.y - d, p.x - d, p.y + d);\n            break;\n        case UML.PSK_TERMINATE:\n            p = Coord.getCenter(new Rect(this.left, this.top, this.getRight(), this.getBottom()));\n            d = Math.round(Math.sqrt(2) * this.width / 4);\n            canvas.line(p.x - d, p.y - d, p.x + d, p.y + d);\n            canvas.line(p.x + d, p.y - d, p.x - d, p.y + d);\n            break;\n        }\n    };\n\n\n    /**\n     * UMLFinalStateView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLFinalStateView() {\n        NodeView.apply(this, arguments);\n        this.sizable = Core.SZ_FREE;\n        this.containerChangeable = true;\n    }\n    // inherits from NodeView\n    UMLFinalStateView.prototype = Object.create(NodeView.prototype);\n    UMLFinalStateView.prototype.constructor = UMLFinalStateView;\n\n    UMLFinalStateView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = FINALSTATE_MINWIDTHH;\n        this.minHeight = FINALSTATE_MINHEIGHT;\n        this.sizeConstraints();\n    };\n\n    UMLFinalStateView.prototype.drawShadow = function (canvas) {\n        canvas.storeState();\n        canvas.alpha = SHADOW_ALPHA;\n        canvas.fillColor = SHADOW_COLOR;\n        canvas.fillEllipse(\n            this.left        + SHADOW_OFFSET,\n            this.top         + SHADOW_OFFSET,\n            this.getRight()  + SHADOW_OFFSET,\n            this.getBottom() + SHADOW_OFFSET\n        );\n        canvas.restoreState();\n        NodeView.prototype.drawShadow.call(this, canvas);\n    };\n\n    UMLFinalStateView.prototype.drawObject = function (canvas) {\n        canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.fillColor = this.lineColor;\n        canvas.fillEllipse(this.left+5, this.top+5, this.getRight()-5, this.getBottom()-5);\n    };\n\n\n    /**\n     * UMLConnectionPointReferenceView\n     * @constructor\n     * @extends UMLFloatingNodeView\n     */\n    function UMLConnectionPointReferenceView() {\n        UMLFloatingNodeView.apply(this, arguments);\n        this.sizable = Core.SZ_NONE;\n    }\n    // inherits from LabelView\n    UMLConnectionPointReferenceView.prototype = Object.create(UMLFloatingNodeView.prototype);\n    UMLConnectionPointReferenceView.prototype.constructor = UMLConnectionPointReferenceView;\n\n    UMLConnectionPointReferenceView.prototype.update = function (canvas) {\n        UMLFloatingNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            var connectedPoints = [];\n            _.each(this.model.entry, function (point) {\n                if (point.name && point.name.length > 0) {\n                    connectedPoints.push(point.name);\n                }\n            });\n            _.each(this.model.exit, function (point) {\n                if (point.name && point.name.length > 0) {\n                    connectedPoints.push(point.name);\n                }\n            });\n            if (connectedPoints.length > 0) {\n                this.nameLabel.text = connectedPoints.join(", ");\n            }\n        }\n    };\n\n    UMLConnectionPointReferenceView.prototype.sizeObject = function (canvas) {\n        UMLFloatingNodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = CONNECTIONPOINT_MINWIDTH;\n        this.minHeight = CONNECTIONPOINT_MINHEIGHT;\n    };\n\n    UMLConnectionPointReferenceView.prototype.arrange = function (canvas) {\n        if (this.containerView) {\n            var r = this.containerView.getBoundingBox(canvas);\n            var c = Coord.getCenter(new Rect(this.left, this.top, this.getRight(), this.getBottom()));\n            var p = this._junction2(r, c);\n            this.left = p.x - CONNECTIONPOINT_MINWIDTH / 2;\n            this.top = p.y - CONNECTIONPOINT_MINHEIGHT / 2;\n            this.setRight(p.x + CONNECTIONPOINT_MINWIDTH / 2);\n            this.setBottom(p.y + CONNECTIONPOINT_MINHEIGHT / 2);\n        }\n        UMLFloatingNodeView.prototype.arrange.call(this, canvas);\n    };\n\n    UMLConnectionPointReferenceView.prototype.drawObject = function (canvas) {\n        UMLFloatingNodeView.prototype.drawObject.call(this, canvas);\n        if (this.model.exit.length > 0) {\n            // draw exitPoint\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n            var p = Coord.getCenter(new Rect(this.left, this.top, this.getRight(), this.getBottom()));\n            var d = Math.round(Math.sqrt(2) * this.width / 4);\n            canvas.line(p.x - d, p.y - d, p.x + d, p.y + d);\n            canvas.line(p.x + d, p.y - d, p.x - d, p.y + d);\n        } else {\n            // draw entryPoint\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n        }\n    };\n\n\n    /**\n     * UMLInternalActivityView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLInternalActivityView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLInternalActivityView.prototype = Object.create(LabelView.prototype);\n    UMLInternalActivityView.prototype.constructor = UMLInternalActivityView;\n\n    UMLInternalActivityView.prototype.update = function (canvas) {\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            var text = "";\n            if (_.contains(this.model._parent.entryActivities, this.model)) {\n                text += "entry/";\n            } else if (_.contains(this.model._parent.doActivities, this.model)) {\n                text += "do/";\n            } else if (_.contains(this.model._parent.exitActivities, this.model)) {\n                text += "exit/";\n            }\n            text += this.model.name;\n            this.text = text;\n        }\n        LabelView.prototype.update.call(this, canvas);\n    };\n\n    UMLInternalActivityView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLInternalActivityCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLInternalActivityCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLInternalActivityCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLInternalActivityCompartmentView.prototype.constructor = UMLInternalActivityCompartmentView;\n\n    UMLInternalActivityCompartmentView.prototype.update = function (canvas) {\n        if (this.model.entryActivities && this.model.doActivities && this.model.exitActivities) {\n            var tempViews = this.subViews;\n            this.subViews = [];\n            var actions = _.union(this.model.entryActivities, this.model.doActivities, this.model.exitActivities);\n            for (var i = 0, len = actions.length; i < len; i++) {\n                var action = actions[i];\n                var actionView = _.find(tempViews, function (v) { return v.model == action; });\n                if (!actionView) {\n                    actionView = new UMLInternalActivityView();\n                    actionView.model = action;\n                    actionView._parent = this;\n                    // actionView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', actionView);\n                } else {\n                    this.addSubView(actionView);\n                }\n                actionView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLInternalTransitionView\n     * @constructor\n     * @extends LabelView\n     */\n    function UMLInternalTransitionView() {\n        LabelView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n        this.horizontalAlignment = Graphics.AL_LEFT;\n    }\n    // inherits from LabelView\n    UMLInternalTransitionView.prototype = Object.create(LabelView.prototype);\n    UMLInternalTransitionView.prototype.constructor = UMLInternalTransitionView;\n\n    UMLInternalTransitionView.prototype.update = function (canvas) {\n        LabelView.prototype.update.call(this, canvas);\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            this.text = this.model.getString();\n        }\n    };\n\n    UMLInternalTransitionView.prototype.size = function (canvas) {\n        LabelView.prototype.size.call(this, canvas);\n        this.height = this.minHeight;\n    };\n\n\n    /**\n     * UMLInternalTransitionCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLInternalTransitionCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n    }\n    // inherits from UMLCompartmentView\n    UMLInternalTransitionCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLInternalTransitionCompartmentView.prototype.constructor = UMLInternalTransitionCompartmentView;\n\n    UMLInternalTransitionCompartmentView.prototype.update = function (canvas) {\n        var tempViews = this.subViews;\n        this.subViews = [];\n        var internals = this.model.getInternalTransitions();\n        for (var i = 0, len = internals.length; i < len; i++) {\n            var trans = internals[i];\n            var transView = _.find(tempViews, function (v) { return v.model === trans; });\n            if (!transView) {\n                transView = new UMLInternalTransitionView();\n                transView.model = trans;\n                transView._parent = this;\n                // actionView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                Repository.bypassInsert(this, \'subViews\', transView);\n            } else {\n                this.addSubView(transView);\n            }\n            transView.setup(canvas);\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLRegionView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLRegionView() {\n        NodeView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.movable = Core.MM_NONE;\n        this.sizable = Core.SZ_VERT;\n        this.parentStyle = true;\n    }\n    // inherits from LabelView\n    UMLRegionView.prototype = Object.create(NodeView.prototype);\n    UMLRegionView.prototype.constructor = UMLRegionView;\n\n    UMLRegionView.prototype._isTopRegionView = function () {\n        var result = true;\n        if (this._parent !== null) {\n            for (var i = 0, len = this._parent.subViews.length; i < len; i++) {\n                var v = this._parent.subViews[i];\n                if ((v instanceof UMLRegionView) && (v !== this)) {\n                    if (v.top < this.top) {\n                        result = false;\n                        return result;\n                    }\n                }\n            }\n        }\n        return result;\n    };\n\n    UMLRegionView.prototype.update = function (canvas) {\n        NodeView.prototype.update.call(this, canvas);\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n    };\n\n    UMLRegionView.prototype.sizeObject = function (canvas) {\n        this.minWidth = REGION_MINWIDTH;\n        this.minHeight = REGION_MINHEIGHT;\n        NodeView.prototype.sizeObject.call(this, canvas);\n    };\n\n    UMLRegionView.prototype.drawObject = function (canvas) {\n        NodeView.prototype.drawObject.call(this, canvas);\n        if (!this._isTopRegionView()) {\n            canvas.line(this.left, this.top, this.getRight(), this.top, [10,3]);\n        }\n    };\n\n    UMLRegionView.prototype.canContainViewKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLStateView") ||\n               MetaModelManager.isKindOf(kind, "UMLPseudostateView") ||\n               MetaModelManager.isKindOf(kind, "UMLFinalStateView");\n    };\n\n\n    /**\n     * UMLDecompositionCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLDecompositionCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n        this.minHeight = 15;\n\n        /* temporal */\n        this._leftPadding   = 0;\n        this._rightPadding  = 0;\n        this._topPadding    = 0;\n        this._bottomPadding = 0;\n        this._itemInterval  = 0;\n    }\n    // inherits from UMLCompartmentView\n    UMLDecompositionCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLDecompositionCompartmentView.prototype.constructor = UMLDecompositionCompartmentView;\n\n    UMLDecompositionCompartmentView.prototype.update = function (canvas) {\n        if (this.model.regions) {\n            var tempViews = this.subViews;\n            this.subViews = [];\n            for (var i = 0, len = this.model.regions.length; i < len; i++) {\n                var region = this.model.regions[i];\n                var regionView = _.find(tempViews, function (v) { return v.model == region; });\n                if (!regionView) {\n                    regionView = new UMLRegionView();\n                    regionView.model = region;\n                    regionView._parent = this;\n                    regionView._parent = this;\n                    // UMLRegionView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', regionView);\n                } else {\n                    this.addSubView(regionView);\n                }\n                regionView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n    UMLDecompositionCompartmentView.prototype.sizeObject = function (canvas) {\n        UMLCompartmentView.prototype.sizeObject.call(this, canvas);\n        var h = 0;\n        for (var i = 0, len = this.subViews.length; i < len; i++) {\n            var regionView = this.subViews[i];\n            h += regionView.height;\n        }\n        this.minHeight = h;\n    };\n\n\n    /**\n     * UMLStateView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLStateView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = true;\n\n        /** @member {UMLInternalActivityCompartmentView} */\n        this.internalActivityCompartment = new UMLInternalActivityCompartmentView();\n        this.internalActivityCompartment.parentStyle = true;\n        this.addSubView(this.internalActivityCompartment);\n\n        /** @member {UMLInternalTransitionCompartmentView} */\n        this.internalTransitionCompartment = new UMLInternalTransitionCompartmentView();\n        this.internalTransitionCompartment.parentStyle = true;\n        this.addSubView(this.internalTransitionCompartment);\n\n        /** @member {UMLDecompositionCompartmentView} */\n        this.decompositionCompartment = new UMLDecompositionCompartmentView();\n        this.decompositionCompartment.parentStyle = true;\n        this.addSubView(this.decompositionCompartment);\n\n        this.fillColor = PreferenceManager.get("uml.state.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLStateView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLStateView.prototype.constructor = UMLStateView;\n\n    UMLStateView.prototype.getAllCompartments = function () {\n        return [\n            this.nameCompartment,\n            this.internalActivityCompartment,\n            this.internalTransitionCompartment,\n            this.decompositionCompartment\n        ];\n    };\n\n    UMLStateView.prototype.update = function (canvas) {\n        // internalActivityCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.internalActivityCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.internalActivityCompartment, \'model\', this.model);\n        }\n        // internalTransitionCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.internalTransitionCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.internalTransitionCompartment, \'model\', this.model);\n        }\n        // decompositionCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.decompositionCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.decompositionCompartment, \'model\', this.model);\n        }\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.sizable = Core.SZ_FREE;\n            if (this.model.entryActivities.length + this.model.doActivities.length + this.model.exitActivities.length > 0) {\n                this.internalActivityCompartment.visible = true;\n            } else {\n                this.internalActivityCompartment.visible = false;\n            }\n            if (this.model.getInternalTransitions().length > 0) {\n                this.internalTransitionCompartment.visible = true;\n            } else {\n                this.internalTransitionCompartment.visible = false;\n            }\n\n            if (this.model.submachine !== null && this.showType) {\n                this.nameCompartment.nameLabel.text = this.model.name + ": " + this.model.submachine.name;\n            }\n        }\n    };\n\n    UMLStateView.prototype.sizeObject = function (canvas) {\n        UMLGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        var sz = this.getSizeOfAllCompartments(canvas);\n        this.minWidth = Math.max(sz.x, STATE_MINWIDTH);\n        if (this.model.submachine !== null) {\n            this.minHeight = Math.max(sz.y + 16, STATE_MINHEIGHT);\n        } else {\n            this.minHeight = Math.max(sz.y, STATE_MINHEIGHT);\n        }\n        this.sizeConstraints();\n    };\n\n    UMLStateView.prototype.drawShadowAsCanonicalForm = function (canvas, showLabel) {\n        canvas.fillRoundRect(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET,\n            STATE_ROUND\n        );\n    };\n\n    UMLStateView.prototype.drawShadowAsDecorationForm = function (canvas) {\n        this.drawShadowAsCanonicalForm(canvas);\n    };\n\n    UMLStateView.prototype.drawShadowAsIconicForm = function (canvas) {\n        this.drawShadowAsCanonicalForm(canvas);\n    };\n\n    UMLStateView.prototype.drawObject = function (canvas) {\n        canvas.fillRoundRect(this.left, this.top, this.getRight(), this.getBottom(), STATE_ROUND);\n        canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), STATE_ROUND);\n        if (this.internalActivityCompartment.visible) {\n            canvas.line(\n                this.internalActivityCompartment.left,\n                this.internalActivityCompartment.top,\n                this.internalActivityCompartment.getRight(),\n                this.internalActivityCompartment.top);\n        }\n        if (this.internalTransitionCompartment.visible) {\n            canvas.line(\n                this.internalTransitionCompartment.left,\n                this.internalTransitionCompartment.top,\n                this.internalTransitionCompartment.getRight(),\n                this.internalTransitionCompartment.top);\n        }\n        if (this.decompositionCompartment.visible && this.decompositionCompartment.subViews.length > 0) {\n            canvas.line(\n                this.decompositionCompartment.left,\n                this.decompositionCompartment.top,\n                this.decompositionCompartment.getRight(),\n                this.decompositionCompartment.top);\n        }\n        if (this.model.submachine !== null) {\n            canvas.ellipse(this.getRight()-26, this.getBottom()-16, this.getRight()-20, this.getBottom()-10);\n            canvas.line(this.getRight()-20, this.getBottom()-13, this.getRight()-14, this.getBottom()-13);\n            canvas.ellipse(this.getRight()-14, this.getBottom()-16, this.getRight()-8, this.getBottom()-10);\n        }\n    };\n\n\n    /**\n     * UMLTransitionView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLTransitionView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_SOLID;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLTransitionView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLTransitionView.prototype.constructor = UMLTransitionView;\n\n    UMLTransitionView.prototype.update = function (canvas) {\n        UMLGeneralEdgeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameLabel.text = this.model.getString();\n            this.nameLabel.visible = (this.nameLabel.text.length > 0);\n        }\n    };\n\n    UMLTransitionView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLVertex);\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                         ACTIVITY DIAGRAM VIEWS                         *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLActivityDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLActivityDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLActivityDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLActivityDiagram.prototype.constructor = UMLActivityDiagram;\n\n    UMLActivityDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLActivity) ||\n               (model instanceof type.UMLAction) ||\n               (model instanceof type.UMLActivityNode) ||\n               (model instanceof type.UMLActivityPartition) ||\n               (model instanceof type.UMLPin);\n    };\n\n    UMLActivityDiagram.prototype.layout = function (direction, separations) {\n        if (!direction) {\n            direction = Core.DIRECTION_BT;\n        }\n        UMLDiagram.prototype.layout.call(this, direction, separations);\n    };\n\n\n    /**\n     * UMLPinView\n     * @constructor\n     * @extends UMLFloatingNodeView\n     */\n    function UMLPinView() {\n        UMLFloatingNodeView.apply(this, arguments);\n        this.sizable = Core.SZ_NONE;\n    }\n    // inherits from UMLFloatingNodeView\n    UMLPinView.prototype = Object.create(UMLFloatingNodeView.prototype);\n    UMLPinView.prototype.constructor = UMLPinView;\n\n    UMLPinView.prototype.update = function (canvas) {\n        UMLFloatingNodeView.prototype.update.call(this, canvas);\n        var options = {\n            showProperty      : true,\n            showType          : true,\n            showMultiplicity  : true\n        };\n        if (this.model) {\n            this.nameLabel.text = this.model.getString(options);\n            this.nameLabel.underline = (this.model.isStatic === true);\n        }\n    };\n\n    UMLPinView.prototype.sizeObject = function (canvas) {\n        UMLFloatingNodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = PIN_MINWIDTH;\n        this.minHeight = PIN_MINHEIGHT;\n    };\n\n    UMLPinView.prototype.getPosition = function (canvas) {\n        var RANGE = Math.round(PIN_MINWIDTH / 2);\n        if (this.containerView) {\n            var r = this.containerView.getBoundingBox(canvas);\n            var b = this.getBoundingBox(canvas);\n            if (r.y1-RANGE <= b.y2 && b.y2 <= r.y1+RANGE) {\n                return "top";\n            } else if (r.y2-RANGE <= b.y1 && b.y1 <= r.y2+RANGE) {\n                return "bottom";\n            } else if (r.x1-RANGE <= b.x2 && b.x2 <= r.x1+RANGE) {\n                return "left";\n            } else if (r.x2-RANGE <= b.x1 && b.x1 <= r.x2+RANGE) {\n                return "right";\n            }\n        }\n        return "else"; // default\n    };\n\n    UMLPinView.prototype.drawArrow = function (canvas, direction) {\n        var MARGIN = 2;\n        var rect = this.getBoundingBox(canvas),\n            c = Coord.getCenter(rect),\n            t = new Point(c.x, rect.y1 + MARGIN),\n            b = new Point(c.x, rect.y2 - MARGIN),\n            l = new Point(rect.x1 + MARGIN, c.y),\n            r = new Point(rect.x2 - MARGIN, c.y);\n        switch (direction) {\n        case "up":\n            canvas.line(b.x, b.y, t.x, t.y);\n            canvas.line(t.x, t.y, t.x - 3, t.y + 3);\n            canvas.line(t.x, t.y, t.x + 3, t.y + 3);\n            break;\n        case "down":\n            canvas.line(b.x, b.y, t.x, t.y);\n            canvas.line(b.x, b.y, b.x - 3, b.y - 3);\n            canvas.line(b.x, b.y, b.x + 3, b.y - 3);\n            break;\n        case "left":\n            canvas.line(l.x, l.y, r.x, r.y);\n            canvas.line(l.x, l.y, l.x + 3, l.y - 3);\n            canvas.line(l.x, l.y, l.x + 3, l.y + 3);\n            break;\n        case "right":\n            canvas.line(l.x, l.y, r.x, r.y);\n            canvas.line(r.x, r.y, r.x - 3, r.y - 3);\n            canvas.line(r.x, r.y, r.x - 3, r.y + 3);\n            break;\n        }\n    };\n\n    UMLPinView.prototype.arrange = function (canvas) {\n        if (this.containerView) {\n            var r = this.containerView.getBoundingBox(canvas);\n            var box = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n            var c = Coord.getCenter(box);\n            var p = this._junction2(r, c);\n            if (!Coord.ptInRect2(p, box)) {\n                if (r.x1 < p.x) {\n                    this.left = p.x;\n                } else {\n                    this.left = p.x - PIN_MINWIDTH + 1;\n                }\n                if (r.y1 < p.y) {\n                    this.top = p.y;\n                } else {\n                    this.top = p.y - PIN_MINHEIGHT + 1;\n                }\n            }\n            this.width = this.minWidth;\n            this.height = this.minHeight;\n        }\n        UMLFloatingNodeView.prototype.arrange.call(this, canvas);\n    };\n\n    UMLPinView.prototype.drawObject = function (canvas) {\n        UMLFloatingNodeView.prototype.drawObject.call(this, canvas);\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n\n    /**\n     * UMLInputPinView\n     * @constructor\n     * @extends UMLPinView\n     */\n    function UMLInputPinView() {\n        UMLPinView.apply(this, arguments);\n        this.sizable = Core.SZ_NONE;\n    }\n    // inherits from LabelView\n    UMLInputPinView.prototype = Object.create(UMLPinView.prototype);\n    UMLInputPinView.prototype.constructor = UMLInputPinView;\n\n    UMLInputPinView.prototype.drawObject = function (canvas) {\n        UMLPinView.prototype.drawObject.call(this, canvas);\n        switch (this.getPosition(canvas)) {\n        case "top":    this.drawArrow(canvas, "down");  break;\n        case "bottom": this.drawArrow(canvas, "up");    break;\n        case "left":   this.drawArrow(canvas, "right"); break;\n        case "right":  this.drawArrow(canvas, "left");  break;\n        }\n    };\n\n\n    /**\n     * UMLOutputPinView\n     * @constructor\n     * @extends UMLPinView\n     */\n    function UMLOutputPinView() {\n        UMLPinView.apply(this, arguments);\n        this.sizable = Core.SZ_NONE;\n    }\n    // inherits from LabelView\n    UMLOutputPinView.prototype = Object.create(UMLPinView.prototype);\n    UMLOutputPinView.prototype.constructor = UMLOutputPinView;\n\n    UMLOutputPinView.prototype.drawObject = function (canvas) {\n        UMLPinView.prototype.drawObject.call(this, canvas);\n        switch (this.getPosition(canvas)) {\n        case "top":    this.drawArrow(canvas, "up");  break;\n        case "bottom": this.drawArrow(canvas, "down");    break;\n        case "left":   this.drawArrow(canvas, "left"); break;\n        case "right":  this.drawArrow(canvas, "right");  break;\n        }\n    };\n\n\n    /**\n     * UMLExpansionNodeView\n     * @constructor\n     * @extends UMLPinView\n     */\n    function UMLExpansionNodeView() {\n        UMLPinView.apply(this, arguments);\n    }\n    // inherits from UMLPinView\n    UMLExpansionNodeView.prototype = Object.create(UMLPinView.prototype);\n    UMLExpansionNodeView.prototype.constructor = UMLExpansionNodeView;\n\n    UMLExpansionNodeView.prototype.sizeObject = function (canvas) {\n        UMLPinView.prototype.sizeObject.call(this, canvas);\n        switch (this.getPosition(canvas)) {\n        case "top":\n        case "bottom":\n            this.minWidth = PIN_MINHEIGHT * 4;\n            this.minHeight = PIN_MINHEIGHT;\n            break;\n        case "left":\n        case "right":\n            this.minWidth = PIN_MINWIDTH;\n            this.minHeight = PIN_MINWIDTH * 4;\n            break;\n        }\n    };\n\n    UMLExpansionNodeView.prototype.drawObject = function (canvas) {\n        UMLPinView.prototype.drawObject.call(this, canvas);\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        switch (this.getPosition(canvas)) {\n        case "top":\n        case "bottom":\n            canvas.line(this.left + PIN_MINHEIGHT, this.top, this.left + PIN_MINHEIGHT, this.getBottom());\n            canvas.line(this.left + PIN_MINHEIGHT * 2, this.top, this.left + PIN_MINHEIGHT * 2, this.getBottom());\n            canvas.line(this.left + PIN_MINHEIGHT * 3, this.top, this.left + PIN_MINHEIGHT * 3, this.getBottom());\n            break;\n        case "left":\n        case "right":\n            canvas.line(this.left, this.top + PIN_MINWIDTH, this.getRight(), this.top + PIN_MINWIDTH);\n            canvas.line(this.left, this.top + PIN_MINWIDTH * 2, this.getRight(), this.top + PIN_MINWIDTH * 2);\n            canvas.line(this.left, this.top + PIN_MINWIDTH * 3, this.getRight(), this.top + PIN_MINWIDTH * 3);\n            break;\n        }\n    };\n\n\n    /**\n     * UMLActionView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLActionView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = true;\n        this.fillColor = PreferenceManager.get("uml.action.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLActionView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLActionView.prototype.constructor = UMLActionView;\n\n    UMLActionView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            if (this.model.subactivity instanceof type.UMLActivity) {\n                this.nameCompartment.nameLabel.text = this.model.name + ":" + this.model.subactivity.name;\n            }\n        }\n    };\n\n    UMLActionView.prototype.sizeObject = function (canvas) {\n        UMLGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        var sz = this.getSizeOfAllCompartments(canvas);\n        if (this.model.kind === UML.ACK_TIMEEVENT) {\n            this.minWidth = Math.max(sz.x, ICONICVIEW_ICONMINWIDTH);\n            this.minHeight = ICONICVIEW_ICONMINHEIGHT + sz.y;\n        } else {\n            this.minWidth = Math.max(sz.x, ACTION_MINWIDTH);\n            if (this.model.submachine !== null) {\n                this.minHeight = Math.max(sz.y + 16, ACTION_MINHEIGHT);\n            } else {\n                this.minHeight = Math.max(sz.y, ACTION_MINHEIGHT);\n            }\n        }\n        this.sizeConstraints();\n    };\n\n    UMLActionView.prototype.arrangeObject = function (canvas) {\n        UMLGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        if (this.model.kind === UML.ACK_TIMEEVENT) {\n            var sz = this.getSizeOfAllCompartments(canvas);\n            var r  = new Rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y2 - sz.y);\n            this.iconRect = this.computeIconRect(r, this.iconRatio);\n            var r2 = new Rect(this.mainRect.x1, this.mainRect.y1 + this.iconRect.getHeight(), this.mainRect.x2, this.mainRect.y2);\n            this.arrangeAllCompartments(r2, canvas);\n        }\n    };\n\n    UMLActionView.prototype.drawShadowAsCanonicalForm = function (canvas, showLabel) {\n        var p;\n        switch (this.model.kind) {\n        case UML.ACK_OPAQUE:\n        case UML.ACK_CREATE:\n        case UML.ACK_DESTROY:\n        case UML.ACK_READ:\n        case UML.ACK_WRITE:\n        case UML.ACK_INSERT:\n        case UML.ACK_DELETE:\n        case UML.ACK_TRIGGEREVENT:\n        case UML.ACK_STRUCTURED:\n            canvas.fillRoundRect(\n                this.mainRect.x1 + SHADOW_OFFSET,\n                this.mainRect.y1 + SHADOW_OFFSET,\n                this.mainRect.x2 + SHADOW_OFFSET,\n                this.mainRect.y2 + SHADOW_OFFSET,\n                STATE_ROUND\n            );\n            break;\n        case UML.ACK_SENDSIGNAL:\n            p = [\n                new Point(this.left + SHADOW_OFFSET, this.top + SHADOW_OFFSET),\n                new Point(this.left + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET),\n                new Point(this.getRight() - this.height / 4 + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET),\n                new Point(this.getRight() + SHADOW_OFFSET, this.top + this.height / 2 + SHADOW_OFFSET),\n                new Point(this.getRight() - this.height / 4 + SHADOW_OFFSET, this.top + SHADOW_OFFSET)\n            ];\n            canvas.fillPolygon(p);\n            break;\n        case UML.ACK_ACCEPTSIGNAL:\n        case UML.ACK_ACCEPTEVENT:\n            p = [\n                new Point(this.left + SHADOW_OFFSET, this.top + SHADOW_OFFSET),\n                new Point(this.getRight() + SHADOW_OFFSET, this.top + SHADOW_OFFSET),\n                new Point(this.getRight() + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET),\n                new Point(this.left + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET),\n                new Point(this.left + this.height / 4 + SHADOW_OFFSET, this.top + this.height / 2 + SHADOW_OFFSET)\n                ];\n            canvas.fillPolygon(p);\n            break;\n        case UML.ACK_TIMEEVENT:\n            break;\n        }\n\n    };\n\n    UMLActionView.prototype.drawShadowAsDecorationForm = function (canvas) {\n        this.drawShadowAsCanonicalForm(canvas);\n    };\n\n    UMLActionView.prototype.drawShadowAsIconicForm = function (canvas) {\n        this.drawShadowAsCanonicalForm(canvas);\n    };\n\n    UMLActionView.prototype.drawObject = function (canvas) {\n        switch (this.model.kind) {\n        case UML.ACK_OPAQUE:\n        case UML.ACK_CREATE:\n        case UML.ACK_DESTROY:\n        case UML.ACK_READ:\n        case UML.ACK_WRITE:\n        case UML.ACK_INSERT:\n        case UML.ACK_DELETE:\n        case UML.ACK_TRIGGEREVENT:\n        case UML.ACK_STRUCTURED:\n            canvas.fillRoundRect(this.left, this.top, this.getRight(), this.getBottom(), ACTION_ROUND);\n            canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), ACTION_ROUND);\n            if (this.model.subactivity !== null) {\n                canvas.ellipse(this.getRight()-26, this.getBottom()-16, this.getRight()-20, this.getBottom()-10);\n                canvas.line(this.getRight()-20, this.getBottom()-13, this.getRight()-14, this.getBottom()-13);\n                canvas.ellipse(this.getRight()-14, this.getBottom()-16, this.getRight()-8, this.getBottom()-10);\n            }\n            break;\n        case UML.ACK_SENDSIGNAL:\n            var polygon = [\n                new Point(this.left, this.top),\n                new Point(this.left, this.getBottom()),\n                new Point(this.getRight() - this.height / 4, this.getBottom()),\n                new Point(this.getRight(), this.top + this.height / 2),\n                new Point(this.getRight() - this.height / 4, this.top)\n            ];\n            canvas.fillPolygon(polygon);\n            canvas.polygon(polygon);\n            break;\n        case UML.ACK_ACCEPTSIGNAL:\n        case UML.ACK_ACCEPTEVENT:\n            var p = [\n                new Point(this.left, this.top),\n                new Point(this.getRight(), this.top),\n                new Point(this.getRight(), this.getBottom()),\n                new Point(this.left, this.getBottom()),\n                new Point(this.left + this.height / 4, this.top + this.height / 2)\n                ];\n            canvas.fillPolygon(p);\n            canvas.polygon(p);\n            break;\n        case UML.ACK_TIMEEVENT:\n            var _iconWidth  = this.iconRect.getWidth(),\n                _iconHeight = this.iconRect.getHeight(),\n                _x          = (this.left + this.getRight()) / 2,\n                _r          = new Rect(_x - (_iconWidth / 2), this.top, _x + (_iconWidth / 2), this.top + _iconHeight);\n            var p2 = [\n                new Point(_r.x1, _r.y1),\n                new Point(_r.x2, _r.y1),\n                new Point(_r.x1, _r.y2),\n                new Point(_r.x2, _r.y2),\n                new Point(_r.x1, _r.y1)\n            ];\n            canvas.fillPolygon(p2);\n            canvas.polygon(p2);\n            break;\n        }\n    };\n\n\n    /**\n     * UMLObjectNodeView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLObjectNodeView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = true;\n        this.fillColor = PreferenceManager.get("uml.objectnode.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLObjectNodeView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLObjectNodeView.prototype.constructor = UMLObjectNodeView;\n\n    UMLObjectNodeView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameCompartment.nameLabel.text = this.model.getString();\n        }\n    };\n\n    UMLObjectNodeView.prototype.drawObject = function (canvas) {\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        UMLGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * UMLCentralBufferNodeView\n     * @constructor\n     * @extends UMLObjectNodeView\n     */\n    function UMLCentralBufferNodeView() {\n        UMLObjectNodeView.apply(this, arguments);\n    }\n    // inherits from UMLObjectNodeView\n    UMLCentralBufferNodeView.prototype = Object.create(UMLObjectNodeView.prototype);\n    UMLCentralBufferNodeView.prototype.constructor = UMLCentralBufferNodeView;\n\n    UMLCentralBufferNodeView.prototype.getStereotypeLabelText = function () {\n        return "«centralBuffer»";\n    };\n\n\n    /**\n     * UMLDataStoreNodeView\n     * @constructor\n     * @extends UMLObjectNodeView\n     */\n    function UMLDataStoreNodeView() {\n        UMLObjectNodeView.apply(this, arguments);\n    }\n    // inherits from UMLObjectNodeView\n    UMLDataStoreNodeView.prototype = Object.create(UMLObjectNodeView.prototype);\n    UMLDataStoreNodeView.prototype.constructor = UMLDataStoreNodeView;\n\n    UMLDataStoreNodeView.prototype.getStereotypeLabelText = function () {\n        return "«datastore»";\n    };\n\n\n    /**\n     * UMLControlNodeView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLControlNodeView() {\n        NodeView.apply(this, arguments);\n        this.containerChangeable = true;\n        this.sizable = Core.SZ_FREE;\n    }\n    // inherits from NodeView\n    UMLControlNodeView.prototype = Object.create(NodeView.prototype);\n    UMLControlNodeView.prototype.constructor = UMLControlNodeView;\n\n    UMLControlNodeView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        if (this.model instanceof type.UMLInitialNode) {\n            this.minWidth = INITIALNODE_MINWIDTH;\n            this.minHeight = INITIALNODE_MINHEIGHT;\n        } else if (this.model instanceof type.UMLActivityFinalNode) {\n            this.minWidth = ACTIVITYFINALNODE_MINWIDTH;\n            this.minHeight = ACTIVITYFINALNODE_MINHEIGHT;\n        } else if (this.model instanceof type.UMLFlowFinalNode) {\n            this.minWidth = FLOWFINALNODE_MINWIDTH;\n            this.minHeight = FLOWFINALNODE_MINHEIGHT;\n        } else if (this.model instanceof type.UMLForkNode) {\n            if (this.height > this.width) {\n                // Vertical Synchronization\n                this.minWidth = FORKNODE_MINTHICK;\n                this.minHeight = FORKNODE_MINLENGTH;\n                this.width = this.minWidth;\n            } else {\n                // Horizontal Synchronization\n                this.minWidth = FORKNODE_MINLENGTH;\n                this.minHeight = FORKNODE_MINTHICK;\n                this.height = this.minHeight;\n            }\n        } else if (this.model instanceof type.UMLJoinNode) {\n            if (this.height > this.width) {\n                // Vertical Synchronization\n                this.minWidth = JOINNODE_MINTHICK;\n                this.minHeight = JOINNODE_MINLENGTH;\n                this.width = this.minWidth;\n            } else {\n                // Horizontal Synchronization\n                this.minWidth = JOINNODE_MINLENGTH;\n                this.minHeight = JOINNODE_MINTHICK;\n                this.height = this.minHeight;\n            }\n        } else if (this.model instanceof type.UMLMergeNode) {\n            this.minWidth = MERGENODE_MINWIDTH;\n            this.minHeight = MERGENODE_MINHEIGHT;\n        } else if (this.model instanceof type.UMLDecisionNode) {\n            this.minWidth = DECISIONNODE_MINWIDTH;\n            this.minHeight = DECISIONNODE_MINHEIGHT;\n        }\n        this.sizeConstraints();\n    };\n\n    UMLControlNodeView.prototype.drawShadow = function (canvas) {\n        canvas.storeState();\n        canvas.alpha = SHADOW_ALPHA;\n        canvas.fillColor = SHADOW_COLOR;\n        if ((this.model instanceof type.UMLInitialNode) ||\n            (this.model instanceof type.UMLActivityFinalNode) ||\n            (this.model instanceof type.UMLFlowFinalNode)) {\n            canvas.fillEllipse(\n                this.left        + SHADOW_OFFSET,\n                this.top         + SHADOW_OFFSET,\n                this.getRight()  + SHADOW_OFFSET,\n                this.getBottom() + SHADOW_OFFSET\n            );\n        } else if ((this.model instanceof type.UMLForkNode) || (this.model instanceof type.UMLJoinNode)) {\n            canvas.fillRoundRect(this.left + SHADOW_OFFSET, this.top + SHADOW_OFFSET, this.getRight() + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET, 3);\n        } else if ((this.model instanceof type.UMLMergeNode) || (this.model instanceof type.UMLDecisionNode)) {\n            var x = this.left + (this.width / 2);\n            var y = this.top + (this.height / 2);\n            canvas.fillPolygon([\n                new Point(this.left + SHADOW_OFFSET, y + SHADOW_OFFSET),\n                new Point(x + SHADOW_OFFSET, this.top + SHADOW_OFFSET),\n                new Point(this.getRight() + SHADOW_OFFSET, y + SHADOW_OFFSET),\n                new Point(x + SHADOW_OFFSET, this.getBottom() + SHADOW_OFFSET),\n                new Point(this.left + SHADOW_OFFSET, y + SHADOW_OFFSET)]);\n        }\n        canvas.restoreState();\n        NodeView.prototype.drawShadow.call(this, canvas);\n    };\n\n    UMLControlNodeView.prototype.drawObject = function (canvas) {\n        var x, y, sz;\n        if (this.model instanceof type.UMLInitialNode) {\n            canvas.fillColor = this.lineColor;\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n        } else if (this.model instanceof type.UMLActivityFinalNode) {\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.fillColor = this.lineColor;\n            canvas.fillEllipse(this.left+5, this.top+5, this.getRight()-5, this.getBottom()-5);\n        } else if (this.model instanceof type.UMLFlowFinalNode) {\n            canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n            canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n            var p = Coord.getCenter(new Rect(this.left, this.top, this.getRight(), this.getBottom()));\n            var d = Math.round(Math.sqrt(2) * this.width / 4);\n            canvas.line(p.x - d, p.y - d, p.x + d, p.y + d);\n            canvas.line(p.x + d, p.y - d, p.x - d, p.y + d);\n        } else if (this.model instanceof type.UMLForkNode) {\n            canvas.fillColor = this.lineColor;\n            if (Math.abs(this.getRight() - this.left) >= Math.abs(this.getBottom() - this.top)) {\n                canvas.fillRoundRect(this.left, this.top, this.getRight(), this.top + FORKNODE_MINTHICK, 3);\n            } else {\n                canvas.fillRoundRect(this.left, this.top, this.left + FORKNODE_MINTHICK, this.getBottom(), 3);\n            }\n        } else if (this.model instanceof type.UMLJoinNode) {\n            canvas.fillColor = this.lineColor;\n            if (Math.abs(this.getRight() - this.left) >= Math.abs(this.getBottom() - this.top)) {\n                canvas.fillRoundRect(this.left, this.top, this.getRight(), this.top + JOINNODE_MINTHICK, 3);\n            } else {\n                canvas.fillRoundRect(this.left, this.top, this.left + JOINNODE_MINTHICK, this.getBottom(), 3);\n            }\n        } else if ((this.model instanceof type.UMLMergeNode) || (this.model instanceof type.UMLDecisionNode)) {\n            x = (this.left + this.getRight()) / 2;\n            y = (this.top + this.getBottom()) / 2;\n            canvas.fillPolygon([new Point(this.left, y), new Point(x, this.top), new Point(this.getRight(), y), new Point(x, this.getBottom()), new Point(this.left, y)]);\n            canvas.polygon([new Point(this.left, y), new Point(x, this.top), new Point(this.getRight(), y), new Point(x, this.getBottom()), new Point(this.left, y)]);\n        }\n    };\n\n\n    /**\n     * UMLControlFlowView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLControlFlowView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_SOLID;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLControlFlowView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLControlFlowView.prototype.constructor = UMLControlFlowView;\n\n    UMLControlFlowView.prototype.update = function (canvas) {\n        UMLGeneralEdgeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameLabel.text = this.model.getString();\n            this.nameLabel.visible = (this.nameLabel.text.length > 0);\n        }\n    };\n\n    UMLControlFlowView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLActivityNode);\n    };\n\n    /**\n     * UMLObjectFlowView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLObjectFlowView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_SOLID;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLObjectFlowView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLObjectFlowView.prototype.constructor = UMLObjectFlowView;\n\n    UMLObjectFlowView.prototype.update = function (canvas) {\n        UMLGeneralEdgeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameLabel.text = this.model.getString();\n            this.nameLabel.visible = (this.nameLabel.text.length > 0);\n        }\n    };\n\n    UMLObjectFlowView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLActivityNode || view.model instanceof type.UMLPin);\n    };\n\n\n    /**\n     * UMLZigZagAdornmentView\n     * @constructor\n     * @extends EdgeNodeView\n     */\n    function UMLZigZagAdornmentView() {\n        EdgeNodeView.apply(this, arguments);\n        this.edgePosition = Core.EP_MIDDLE;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_FREE;\n        this.alpha = Math.PI / 2;\n        this.distance = 20;\n    }\n    // inherits from EdgeNodeView\n    UMLZigZagAdornmentView.prototype = Object.create(EdgeNodeView.prototype);\n    UMLZigZagAdornmentView.prototype.constructor = UMLZigZagAdornmentView;\n\n\n    UMLZigZagAdornmentView.prototype.update = function () {\n        EdgeNodeView.prototype.update.call(this);\n    };\n\n    UMLZigZagAdornmentView.prototype.sizeObject = function (canvas) {\n        EdgeNodeView.prototype.sizeObject.call(this, canvas);\n        this.width = 25;\n        this.height = 20;\n    };\n\n    UMLZigZagAdornmentView.prototype.arrangeObject = function (canvas) {\n        EdgeNodeView.prototype.arrangeObject.call(this, canvas);\n    };\n\n    UMLZigZagAdornmentView.prototype.drawObject = function (canvas) {\n        EdgeNodeView.prototype.drawObject.call(this, canvas);\n        canvas.line(this.left, this.top, this.getRight(), this.top);\n        canvas.line(this.getRight(), this.top, this.left, this.getBottom() - 5);\n        canvas.line(this.left, this.getBottom() - 5, this.getRight(), this.getBottom() - 5);\n        canvas.line(this.getRight(), this.getBottom() - 5, this.getRight() - 5, this.getBottom());\n        canvas.line(this.getRight(), this.getBottom() - 5, this.getRight() - 5, this.getBottom() - 10);\n    };\n\n    /** Cannot be copied to clipboard. */\n    UMLZigZagAdornmentView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /** Cannnot be deleted view only. */\n    UMLZigZagAdornmentView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * UMLExceptionHandlerView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLExceptionHandlerView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_SOLID;\n\n        /** @member {UMLZigZagAdornmentView} */\n        this.adornment = new UMLZigZagAdornmentView();\n        this.adornment.parentStyle = true;\n        this.addSubView(this.adornment);\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLExceptionHandlerView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLExceptionHandlerView.prototype.constructor = UMLExceptionHandlerView;\n\n    UMLExceptionHandlerView.prototype.update = function () {\n        UMLGeneralEdgeView.prototype.update.call(this);\n        // adornment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.adornment.model !== this.model) {\n            Repository.bypassFieldAssign(this.adornment, \'model\', this.model);\n        }\n    };\n\n    UMLExceptionHandlerView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLActivityNode);\n    };\n\n\n    /**\n     * UMLActivityInterruptView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLActivityInterruptView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_STICK_ARROW;\n        this.lineMode = Core.LM_SOLID;\n\n        /** @member {UMLZigZagAdornmentView} */\n        this.adornment = new UMLZigZagAdornmentView();\n        this.adornment.parentStyle = true;\n        this.addSubView(this.adornment);\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLActivityInterruptView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLActivityInterruptView.prototype.constructor = UMLActivityInterruptView;\n\n    UMLActivityInterruptView.prototype.update = function () {\n        UMLGeneralEdgeView.prototype.update.call(this);\n        // adornment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.adornment.model !== this.model) {\n            Repository.bypassFieldAssign(this.adornment, \'model\', this.model);\n        }\n    };\n\n    UMLActivityInterruptView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof type.UMLActivityNode);\n    };\n\n\n    /**\n     * UMLSwimlaneView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLSwimlaneView() {\n        NodeView.apply(this, arguments);\n\n        /** @member {Boolean} */\n        this.isVertical = true;\n\n        /** @member {LabelView} */\n        this.nameLabel = new LabelView();\n        this.nameLabel.horizontalAlignment = Graphics.AL_CENTER;\n        this.nameLabel.verticalAlignment = Graphics.AL_TOP;\n        this.nameLabel.selectable = Core.SK_NO;\n        this.nameLabel.parentStyle = true;\n        this.addSubView(this.nameLabel);\n    }\n    // inherits from NodeView\n    UMLSwimlaneView.prototype = Object.create(NodeView.prototype);\n    UMLSwimlaneView.prototype.constructor = UMLSwimlaneView;\n\n    UMLSwimlaneView.prototype.canContainViewKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLActionView") ||\n               MetaModelManager.isKindOf(kind, "UMLControlNodeView") ||\n               MetaModelManager.isKindOf(kind, "UMLObjectNodeView") ||\n               MetaModelManager.isKindOf(kind, "UMLFinalStateView");\n    };\n\n    UMLSwimlaneView.prototype.update = function (canvas) {\n        if (this.model) {\n            this.nameLabel.text = this.model.name;\n            this.nameLabel.visible = (this.model.name.length > 0);\n            if (this.isVertical) {\n                this.nameLabel.direction = Core.DK_HORZ;\n            } else {\n                this.nameLabel.direction = Core.DK_VERT;\n            }\n        }\n    };\n\n    UMLSwimlaneView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        var w, h;\n        if (this.isVertical) {\n            w = this.nameLabel.minWidth + SWIMLANE_HEADER_LEFT_MARGIN + SWIMLANE_HEADER_RIGHT_MARGIN;\n            this.minWidth = Math.max(w, SWIMLANE_VERT_MINWIDTH);\n            h = this.nameLabel.minHeight + SWIMLANE_HEADER_TOP_MARGIN + SWIMLANE_HEADER_BOTTOM_MARGIN;\n            this.minHeight = Math.max(h, SWIMLANE_VERT_MINHEIGHT);\n        } else {\n            w = this.nameLabel.minWidth + SWIMLANE_HEADER_TOP_MARGIN + SWIMLANE_HEADER_BOTTOM_MARGIN;\n            this.minWidth = Math.max(w, SWIMLANE_HORIZ_MINWIDTH);\n            h = this.nameLabel.minHeight + SWIMLANE_HEADER_LEFT_MARGIN + SWIMLANE_HEADER_RIGHT_MARGIN;\n            this.minHeight = Math.max(h, SWIMLANE_HORIZ_MINHEIGHT);\n        }\n    };\n\n    UMLSwimlaneView.prototype.arrangeObject = function (canvas) {\n        // not inherited (SwimlaneView must not be affected by \'AutoResize\' attribute.\n        if (this.isVertical) {\n            this.nameLabel.direction = Core.DK_HORZ;\n            this.nameLabel.width = this.nameLabel.minWidth;\n            this.nameLabel.height = this.nameLabel.minHeight;\n            this.nameLabel.left = this.left;\n            this.nameLabel.setRight(this.getRight());\n            this.nameLabel.top = this.top + SWIMLANE_HEADER_TOP_MARGIN;\n        } else {\n            this.nameLabel.direction = Core.DK_VERT;\n            this.nameLabel.width = this.nameLabel.minWidth;\n            this.nameLabel.height = this.nameLabel.minHeight;\n            this.nameLabel.left = this.left + SWIMLANE_HEADER_TOP_MARGIN;\n            this.nameLabel.top = this.top;\n            this.nameLabel.setBottom(this.getBottom());\n        }\n    };\n\n    UMLSwimlaneView.prototype.drawObject = function (canvas) {\n        NodeView.prototype.drawObject.call(this, canvas);\n        canvas.lineWidth = SWIMLANE_PEN_WIDTH;\n        if (this.isVertical) {\n            var h = this.top + this.nameLabel.height + SWIMLANE_HEADER_TOP_MARGIN + SWIMLANE_HEADER_BOTTOM_MARGIN;\n            canvas.fillRect(this.left, this.top, this.getRight(), h);\n            canvas.rect(this.left, this.top, this.getRight(), h);\n            canvas.polyline([new Point(this.left, this.getBottom()), new Point(this.left, this.top), new Point(this.getRight(), this.top), new Point(this.getRight(), this.getBottom())]);\n        } else {\n            var w = this.left + this.nameLabel.width + SWIMLANE_HEADER_TOP_MARGIN + SWIMLANE_HEADER_BOTTOM_MARGIN;\n            canvas.fillRect(this.left, this.top, w, this.getBottom());\n            canvas.rect(this.left, this.top, w, this.getBottom());\n            canvas.polyline([new Point(this.getRight(), this.top), new Point(this.left, this.top), new Point(this.left, this.getBottom()), new Point(this.getRight(), this.getBottom())]);\n        }\n        canvas.lineWidth = 1;\n    };\n\n\n    /**\n     * UMLInterruptibleActivityRegionView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLInterruptibleActivityRegionView() {\n        NodeView.apply(this, arguments);\n    }\n    // inherits from NodeView\n    UMLInterruptibleActivityRegionView.prototype = Object.create(NodeView.prototype);\n    UMLInterruptibleActivityRegionView.prototype.constructor = UMLInterruptibleActivityRegionView;\n\n    UMLInterruptibleActivityRegionView.prototype.canContainViewKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLActionView") ||\n               MetaModelManager.isKindOf(kind, "UMLControlNodeView") ||\n               MetaModelManager.isKindOf(kind, "UMLObjectNodeView") ||\n               MetaModelManager.isKindOf(kind, "UMLFinalStateView");\n    };\n\n    UMLInterruptibleActivityRegionView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = 30;\n        this.minHeight = 30;\n    };\n\n    UMLInterruptibleActivityRegionView.prototype.drawObject = function (canvas) {\n        canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), ACTION_ROUND, [3]);\n    };\n\n\n    /**\n     * UMLStructuredActivityNodeView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLStructuredActivityNodeView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.containerChangeable = true;\n        this.fillColor = PreferenceManager.get("uml.action.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLStructuredActivityNodeView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLStructuredActivityNodeView.prototype.constructor = UMLStructuredActivityNodeView;\n\n    UMLStructuredActivityNodeView.prototype.getStereotypeLabelText = function () {\n        return "«structured»";\n    };\n\n    UMLStructuredActivityNodeView.prototype.canContainViewKind = function (kind) {\n        return MetaModelManager.isKindOf(kind, "UMLActionView") ||\n               MetaModelManager.isKindOf(kind, "UMLControlNodeView") ||\n               MetaModelManager.isKindOf(kind, "UMLObjectNodeView") ||\n               MetaModelManager.isKindOf(kind, "UMLFinalStateView") ||\n               MetaModelManager.isKindOf(kind, "UMLStructuredActivityNodeView");\n    };\n\n    UMLStructuredActivityNodeView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        this.nameCompartment.stereotypeLabel.horizontalAlignment = Graphics.AL_LEFT;\n        if (this.model) {\n            if (this.model.subactivity instanceof type.UMLActivity) {\n                this.nameCompartment.nameLabel.text = this.model.name + ":" + this.model.subactivity.name;\n            }\n        }\n    };\n\n    UMLStructuredActivityNodeView.prototype.drawShadowAsCanonicalForm = function (canvas, showLabel) {\n        canvas.fillRoundRect(\n            this.mainRect.x1 + SHADOW_OFFSET,\n            this.mainRect.y1 + SHADOW_OFFSET,\n            this.mainRect.x2 + SHADOW_OFFSET,\n            this.mainRect.y2 + SHADOW_OFFSET,\n            STATE_ROUND\n        );\n    };\n\n    UMLStructuredActivityNodeView.prototype.drawShadowAsDecorationForm = function (canvas) {\n        this.drawShadowAsCanonicalForm(canvas);\n    };\n\n    UMLStructuredActivityNodeView.prototype.drawShadowAsIconicForm = function (canvas) {\n        this.drawShadowAsCanonicalForm(canvas);\n    };\n\n    UMLStructuredActivityNodeView.prototype.drawObject = function (canvas) {\n        canvas.fillRoundRect(this.left, this.top, this.getRight(), this.getBottom(), ACTION_ROUND);\n        canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), ACTION_ROUND, [3]);\n        if (this.model.subactivity !== null) {\n            canvas.ellipse(this.getRight()-26, this.getBottom()-16, this.getRight()-20, this.getBottom()-10);\n            canvas.line(this.getRight()-20, this.getBottom()-13, this.getRight()-14, this.getBottom()-13);\n            canvas.ellipse(this.getRight()-14, this.getBottom()-16, this.getRight()-8, this.getBottom()-10);\n        }\n    };\n\n\n    /**\n     * UMLExpansionRegionView\n     * @constructor\n     * @extends UMLStructuredActivityNodeView\n     */\n    function UMLExpansionRegionView() {\n        UMLStructuredActivityNodeView.apply(this, arguments);\n    }\n    // inherits from UMLStructuredActivityNodeView\n    UMLExpansionRegionView.prototype = Object.create(UMLStructuredActivityNodeView.prototype);\n    UMLExpansionRegionView.prototype.constructor = UMLExpansionRegionView;\n\n    UMLExpansionRegionView.prototype.getStereotypeLabelText = function () {\n        switch (this.model.mode) {\n        case UML.EK_PARALLEL:\n            return "«parallel»";\n        case UML.EK_ITERATIVE:\n            return "«iterative»";\n        case UML.EK_STREAM:\n            return "«stream»";\n        }\n        return this.model.getStereotypeString();\n    };\n\n\n    /**************************************************************************\n     *                                                                        *\n     *                         SEQUENCE DIAGRAM VIEWS                         *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLSequenceDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLSequenceDiagram() {\n        UMLDiagram.apply(this, arguments);\n        this.showSequenceNumber = true;\n        this.showSignature = true;\n        this.showActivation = true;\n    }\n    // inherits from UMLDiagram\n    UMLSequenceDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLSequenceDiagram.prototype.constructor = UMLSequenceDiagram;\n\n    UMLSequenceDiagram.prototype.canAcceptModel = function (model) {\n        if (model instanceof type.Hyperlink || model instanceof type.Diagram) {\n            return true;\n        } else if (model instanceof type.UMLMessageEndpoint ||\n            model instanceof type.UMLCombinedFragment ||\n            model instanceof type.UMLStateInvariant ||\n            model instanceof type.UMLInteraction ||\n            model instanceof type.UMLInteractionUse ||\n            model instanceof type.UMLContinuation ||\n            model instanceof type.UMLMessage) {\n            return _.every(this.ownedViews, function (v) { return v.model !== model; });\n        } else {\n            return (model instanceof type.UMLConstraint) ||\n                   (model instanceof type.UMLClassifier);\n        }\n    };\n\n    UMLSequenceDiagram.prototype.drawDiagram = function (canvas, drawSelection) {\n        var i,\n            len,\n            view;\n\n        // Regulate sequence number of message views\n        for (i = 0, len = this.ownedViews.length; i < len; i++) {\n            view = this.ownedViews[i];\n            if (view instanceof UMLSeqMessageView) {\n                view.regulateSequenceNumber();\n            }\n        }\n        UMLDiagram.prototype.drawDiagram.call(this, canvas, drawSelection);\n    };\n\n    UMLSequenceDiagram.prototype.layout = function (direction, separations) {\n        // TODO: Layout for Sequence Diagram\n    };\n\n\n    /**\n     * UMLLinePartView (Line Part of LifelineView)\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLLinePartView() {\n        NodeView.apply(this, arguments);\n        this.movable = Core.MM_NONE;\n        this.sizable = Core.SZ_VERT;\n        this.selectable = Core.SK_NO;\n    }\n    // inherits from NodeView\n    UMLLinePartView.prototype = Object.create(NodeView.prototype);\n    UMLLinePartView.prototype.constructor = UMLLinePartView;\n\n    /**\n     * @param {number} yPosition\n     * @return {UMLActivationView}\n     */\n    UMLLinePartView.prototype.getActivationAt = function (yPosition) {\n        var frontMostActivation = null;\n        // find the most extruded Activation(FrontMostActivation)\n        // regard Activation whose Top position is the lowest as the most extruded Activation\n        for (var i = 0, len = this.getDiagram().ownedViews.length; i < len; i++) {\n            if (this.getDiagram().ownedViews[i] instanceof UMLSeqMessageView) {\n                var msg = this.getDiagram().ownedViews[i];\n                if (msg.head === this) {\n                    if (msg.activation.visible && (msg.activation.top <= yPosition) && (msg.activation.getBottom() > yPosition)) {\n                        if (frontMostActivation !== null) {\n                            if (msg.activation.top > frontMostActivation.top) {\n                                frontMostActivation = msg.activation;\n                            }\n                        } else {\n                            frontMostActivation = msg.activation;\n                        }\n                    }\n                }\n            }\n        }\n        return frontMostActivation;\n    };\n\n    UMLLinePartView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        // Auto-expanding Lifeline\n        var bottomMost = 0;\n        for (var i = 0, len = this.getDiagram().ownedViews.length; i < len; i++) {\n            if (this.getDiagram().ownedViews[i] instanceof UMLSeqMessageView) {\n                var msg = this.getDiagram().ownedViews[i];\n                if (msg.model && (msg.model.source === this.model || msg.model.target === this.model)) {\n                    var box = msg.getBoundingBox(canvas);\n                    if (bottomMost < box.y2) {\n                        bottomMost = box.y2;\n                    }\n                }\n            }\n        }\n        this.minWidth = 1;\n        if (bottomMost > (this.minHeight + this.top)) {\n            this.minHeight = (bottomMost - this.top) + 15;\n        } else {\n            this.minHeight = LIFELINE_MINHEIGHT;\n        }\n        this.sizeConstraints();\n    };\n\n    UMLLinePartView.prototype.arrangeObject = function (canvas) {\n        NodeView.prototype.arrangeObject.call(this, canvas);\n        this.width = this.minWidth;\n    };\n\n    UMLLinePartView.prototype.drawObject = function (canvas) {\n        NodeView.prototype.drawObject.call(this, canvas);\n        var c = this.left + (this.width / 2);\n        canvas.line(c, this.top, c, this.getBottom(), [3]);\n    };\n\n\n    /**\n     * UMLSeqLifelineView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLSeqLifelineView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.movable = Core.MM_HORZ;\n\n        /** @member {UMLLinePartView} */\n        this.linePart = new UMLLinePartView();\n        this.addSubView(this.linePart);\n\n        this.fillColor = PreferenceManager.get("uml.lifeline.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLSeqLifelineView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLSeqLifelineView.prototype.constructor = UMLSeqLifelineView;\n\n    UMLSeqLifelineView.prototype.drawIcon = function (canvas, rect) {\n        if (this.model) {\n            if (this.model.stereotype && this.model.stereotype.icon) {\n                UMLGeneralNodeView.prototype.drawIcon.call(this, canvas, rect);\n            } else if (this.model.represent && (this.model.represent.type instanceof type.Model)) {\n                var iconRatioBackup = this.iconRatio;\n                if (this.model.represent.type instanceof type.UMLActor) {\n                    this.iconRatio = ACTOR_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLActorView.prototype.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLUseCase) {\n                    this.iconRatio = USECASE_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLUseCaseView.prototype.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLInterface) {\n                    this.iconRatio = 100;\n                    this.arrangeObject(canvas);\n                    UMLInterfaceView.prototype.drawBallNotation.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLArtifact) {\n                    this.iconRatio = ARTIFACT_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLArtifactViewMixin.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLComponent) {\n                    this.iconRatio = COMPONENT_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLComponentViewMixin.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLNode) {\n                    this.iconRatio = NODE_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLNodeViewMixin.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type.stereotype && this.model.represent.type.stereotype.icon) {\n                    var _icon = this.model.represent.type.stereotype.icon;\n                    var _rect = this.computeIconRect(rect, (_icon.width / _icon.height) * 100);\n                    drawImage(canvas, _rect, _icon);\n                } else {\n                    UMLGeneralNodeView.prototype.drawIcon.call(this, canvas, rect);\n                }\n                this.iconRatio = iconRatioBackup;\n                this.arrangeObject(canvas);\n            } else {\n                UMLGeneralNodeView.prototype.drawIcon.call(this, canvas, rect);\n            }\n        }\n    };\n\n    UMLSeqLifelineView.prototype.update = function () {\n        UMLGeneralNodeView.prototype.update.call(this);\n        // linePart가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n        if (this.linePart.model !== this.model) {\n            Repository.bypassFieldAssign(this.linePart, \'model\', this.model);\n        }\n        if (this.model) {\n            this.nameCompartment.nameLabel.text = this.model.getString(this);\n            this.nameCompartment.nameLabel.underline = false;\n        }\n    };\n\n    UMLSeqLifelineView.prototype.sizeAsCanonicalForm = function (canvas, showLabel) {\n        var sz = this.getSizeOfAllCompartments(canvas);\n        this.minWidth = Math.max(SEQ_OBJECT_MINWIDTH, sz.x + COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING);\n        this.minHeight = Math.max(SEQ_OBJECT_MINHEIGHT, sz.y) + this.linePart.minHeight;\n    };\n\n    UMLSeqLifelineView.prototype.sizeAsDecorationForm = function (canvas, showLabel) {\n        var sz = this.getSizeOfAllCompartments(canvas);\n        this.minWidth = Math.max(SEQ_OBJECT_MINWIDTH, this.nameCompartment.minWidth + DECORATION_ICON_WIDTH + COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING);\n        this.minHeight = Math.max(SEQ_OBJECT_MINHEIGHT, sz.y) + this.linePart.minHeight;\n        this.sizeConstraints();\n    };\n\n    UMLSeqLifelineView.prototype.sizeAsIconicForm = function (canvas, showLabel) {\n        var sz = this.getSizeOfAllCompartments(canvas);\n        this.minWidth = Math.max(sz.x, ICONICVIEW_ICONMINWIDTH);\n        this.minHeight = ICONICVIEW_ICONMINHEIGHT + sz.y + this.linePart.minHeight;\n    };\n\n    UMLSeqLifelineView.prototype.arrangeCommon = function (canvas) {\n        // If a create message is connected, Y position should be determined by the Message\'s Y position.\n        var _createMessage = null;\n        for (var i = 0, len = this.getDiagram().ownedViews.length; i < len; i++) {\n            var v = this.getDiagram().ownedViews[i];\n            if (v instanceof UMLSeqMessageView &&\n                v.model.messageSort === UML.MS_CREATEMESSAGE &&\n                v.head === this.linePart) {\n                _createMessage = v;\n                break;\n            }\n        }\n        if (_createMessage) {\n            this.top = Math.round(_createMessage.points.getPoint(1).y - (SEQ_OBJECT_MINHEIGHT / 2));\n        } else {\n            this.top = LIFELINE_TOP_POSITION;\n        }\n        this.mainRect.setRect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n    UMLSeqLifelineView.prototype.arrangeAsCanonicalForm = function (canvas, showLabel) {\n        // not inherited: must not be affected by \'AutoResize\' attribute.\n        UMLGeneralNodeView.prototype.arrangeAsCanonicalForm.call(this, canvas, showLabel);\n        this.nameCompartment.height = SEQ_OBJECT_MINHEIGHT;\n        this.linePart.top = this.nameCompartment.getBottom() + 1;\n        this.linePart.left = Math.round(this.left + (this.width / 2));\n        this.linePart.setBottom(this.getBottom());\n    };\n\n    UMLSeqLifelineView.prototype.arrangeAsDecorationForm = function (canvas, showLabel) {\n        // not inherited: must not be affected by \'AutoResize\' attribute.\n        UMLGeneralNodeView.prototype.arrangeAsDecorationForm.call(this, canvas, showLabel);\n        this.linePart.top = this.nameCompartment.getBottom() + 1;\n        this.linePart.left = Math.round(this.left + (this.width / 2));\n        this.linePart.setBottom(this.getBottom());\n    };\n\n    UMLSeqLifelineView.prototype.arrangeAsIconicForm = function (canvas, showLabel) {\n        // UMLGeneralNodeView.prototype.arrangeAsIconicForm.call(this, canvas);\n        var sz = this.getSizeOfAllCompartments(canvas);\n        var r  = new Rect(this.mainRect.x1, this.mainRect.y1, this.mainRect.x2, this.mainRect.y1 + SEQ_OBJECT_MINHEIGHT);\n        this.iconRect = this.computeIconRect(r, this.iconRatio);\n        var r2 = new Rect(this.mainRect.x1, this.mainRect.y1 + this.iconRect.getHeight(), this.mainRect.x2, this.mainRect.y2);\n        this.arrangeAllCompartments(r2, canvas);\n        this.linePart.top = this.nameCompartment.getBottom() + 1;\n        this.linePart.left = Math.round(this.left + (this.width / 2));\n        this.linePart.setBottom(this.getBottom());\n    };\n\n    UMLSeqLifelineView.prototype.drawCommon = function (canvas) {\n        // draw nothing\n    };\n\n    UMLSeqLifelineView.prototype.drawAsCanonicalForm = function (canvas, showLabel) {\n        UMLGeneralNodeView.prototype.drawAsCanonicalForm.call(this, canvas);\n        var r = new Rect(this.left, this.top, this.getRight(), this.nameCompartment.getBottom());\n        if (this.model.isMultiInstance) {\n            canvas.rect(r.x1 + MULTI_INSTANCE_MARGIN, r.y1 + MULTI_INSTANCE_MARGIN, r.x2 + MULTI_INSTANCE_MARGIN, r.y2 + MULTI_INSTANCE_MARGIN);\n        }\n        canvas.fillRect(r.x1, r.y1, r.x2, r.y2);\n        canvas.rect(r.x1, r.y1, r.x2, r.y2);\n        if (this.model.represent && this.model.represent.type && this.model.represent.type.isActive === true) {\n            canvas.line(this.left + CLASS_ACTIVE_VERTLINE_WIDTH, this.nameCompartment.top, this.left + CLASS_ACTIVE_VERTLINE_WIDTH, this.nameCompartment.getBottom());\n            canvas.line(this.getRight() - CLASS_ACTIVE_VERTLINE_WIDTH, this.nameCompartment.top, this.getRight() - CLASS_ACTIVE_VERTLINE_WIDTH, this.nameCompartment.getBottom());\n        }\n    };\n\n    UMLSeqLifelineView.prototype.drawAsDecorationForm = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.nameCompartment.getBottom());\n        if (this.model.isMultiInstance) {\n            canvas.rect(r.x1 + MULTI_INSTANCE_MARGIN, r.y1 + MULTI_INSTANCE_MARGIN, r.x2 + MULTI_INSTANCE_MARGIN, r.y2 + MULTI_INSTANCE_MARGIN);\n        }\n        canvas.fillRect(r.x1, r.y1, r.x2, r.y2);\n        canvas.rect(r.x1, r.y1, r.x2, r.y2);\n        if (this.model.represent && this.model.represent.type && this.model.represent.type.isActive === true) {\n            canvas.line(this.left + CLASS_ACTIVE_VERTLINE_WIDTH, this.nameCompartment.top, this.left + CLASS_ACTIVE_VERTLINE_WIDTH, this.nameCompartment.getBottom());\n            canvas.line(this.getRight() - CLASS_ACTIVE_VERTLINE_WIDTH, this.nameCompartment.top, this.getRight() - CLASS_ACTIVE_VERTLINE_WIDTH, this.nameCompartment.getBottom());\n        }\n        UMLGeneralNodeView.prototype.drawAsDecorationForm.call(this, canvas);\n    };\n\n    UMLSeqLifelineView.prototype.drawAsIconicForm = function (canvas) {\n        UMLGeneralNodeView.prototype.drawAsIconicForm.call(this, canvas);\n    };\n\n    UMLSeqLifelineView.prototype.drawShadowAsCanonicalForm = function (canvas) {\n        canvas.fillRect(this.left + SHADOW_OFFSET, this.top + SHADOW_OFFSET, this.getRight() + SHADOW_OFFSET, this.nameCompartment.getBottom() + SHADOW_OFFSET);\n    };\n\n    UMLSeqLifelineView.prototype.drawShadowAsDecorationForm = function (canvas) {\n        canvas.fillRect(this.left + SHADOW_OFFSET, this.top + SHADOW_OFFSET, this.getRight() + SHADOW_OFFSET, this.nameCompartment.getBottom() + SHADOW_OFFSET);\n    };\n\n    UMLSeqLifelineView.prototype.drawShadowAsIconicForm = function (canvas) {\n        /*\n        canvas.fillRect(\n            this.iconRect.x1 + SHADOW_OFFSET,\n            this.iconRect.y1 + SHADOW_OFFSET,\n            this.iconRect.x2 + SHADOW_OFFSET,\n            this.iconRect.y2 + SHADOW_OFFSET\n        );\n        */\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLSeqLifelineView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLSeqLifelineView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    var MESSAGEENDPOINT_MINWIDTH = 15,\n        MESSAGEENDPOINT_MINHEIGHT = 15;\n\n    /**\n     * UMLMessageEndpointView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLMessageEndpointView() {\n        NodeView.apply(this, arguments);\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_HORZ;\n    }\n    // inherits from NodeView\n    UMLMessageEndpointView.prototype = Object.create(NodeView.prototype);\n    UMLMessageEndpointView.prototype.constructor = UMLMessageEndpointView;\n\n    UMLMessageEndpointView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = MESSAGEENDPOINT_MINWIDTH;\n        this.minHeight = MESSAGEENDPOINT_MINHEIGHT;\n    };\n\n    /**\n     * Movable freely when nothing connected,\n     * but movable horizontally only when a message is connected.\n     */\n    UMLMessageEndpointView.prototype.arrangeObject = function (canvas) {\n        this.width = this.minWidth;\n        this.height = this.minHeight;\n        var edges = Repository.getEdgeViewsOf(this);\n        if (edges.length > 0) {\n            this.movable = Core.MM_HORZ;\n        } else {\n            this.movable = Core.MM_FREE;\n        }\n        NodeView.prototype.arrangeObject.call(this, canvas);\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLMessageEndpointView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLMessageEndpointView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * UMLEndpointView\n     * @constructor\n     * @extends UMLMessageEndpointView\n     */\n    function UMLEndpointView() {\n        UMLMessageEndpointView.apply(this, arguments);\n    }\n    // inherits from UMLMessageEndpointView\n    UMLEndpointView.prototype = Object.create(UMLMessageEndpointView.prototype);\n    UMLEndpointView.prototype.constructor = UMLEndpointView;\n\n    UMLEndpointView.prototype.drawObject = function (canvas) {\n        canvas.fillColor = this.lineColor;\n        canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n\n    /**\n     * UMLGateView\n     * @constructor\n     * @extends UMLMessageEndpointView\n     */\n    function UMLGateView() {\n        UMLMessageEndpointView.apply(this, arguments);\n    }\n    // inherits from UMLMessageEndpointView\n    UMLGateView.prototype = Object.create(UMLMessageEndpointView.prototype);\n    UMLGateView.prototype.constructor = UMLGateView;\n\n    UMLGateView.prototype.drawObject = function (canvas) {\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n\n    /**\n     * UMLActivationView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLActivationView() {\n        NodeView.apply(this, arguments);\n        this.sizable = Core.SZ_VERT;\n        this.movable = Core.MM_NONE;\n    }\n    // inherits from NodeView\n    UMLActivationView.prototype = Object.create(NodeView.prototype);\n    UMLActivationView.prototype.constructor = UMLActivationView;\n\n    UMLActivationView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = ACTIVATION_MINWIDTH;\n        this.minHeight = ACTIVATION_MINHEIGHT;\n    };\n\n    UMLActivationView.prototype.arrangeObject = function (canvas) {\n        var messageView      = this._parent,\n            linePart         = messageView.head,\n            parentActivation = (linePart instanceof UMLLinePartView ? linePart.getActivationAt(this.top - 1) : null);\n\n        this.top = messageView.points.getPoint(messageView.points.count() - 1).y;\n        this.width = ACTIVATION_MINWIDTH;\n        // Left position extrudes a little right than Parent Activation\n        if (parentActivation !== null) {\n            this.left = parentActivation.left + (ACTIVATION_MINWIDTH / 2);\n        } else {\n            this.left = linePart.left - (ACTIVATION_MINWIDTH / 2);\n        }\n        // if Activation is not shown\n        if (!this.visible) {\n            if (parentActivation !== null) {\n                this.left = parentActivation.left;\n            } else {\n                this.left = linePart.left;\n            }\n        // if Activation is shown\n        } else {\n            // Height must wrap all Child Activations at least.\n            //   - Child Activation satisfies below conditions\n            //     (1) in all current LifeLine\'s In-coming Messages,\n            //     (2) if its head side\'s y position is between Top and Bottom,\n            //     (3) the message\'s Activation is really Child Activation.\n            var minimumBottom = this.top + ACTIVATION_MINHEIGHT;\n            for (var i = 0, len = this.getDiagram().ownedViews.length; i < len; i++) {\n                if (this.getDiagram().ownedViews[i] instanceof UMLSeqMessageView) {\n                    var msg = this.getDiagram().ownedViews[i];\n                    if ((msg.head == linePart) && (msg != messageView)) {\n                        var y = msg.points.getPoint(msg.points.count() - 1).y;\n                        if ((this.top <= y) && (this.getBottom() > y)) {\n                            if (msg.activation.visible && (msg.activation.getBottom() > minimumBottom)) {\n                                minimumBottom = msg.activation.getBottom();\n                            }\n                        }\n                    }\n                }\n            }\n            if (this.getBottom() < minimumBottom + 3) {\n                this.setBottom(minimumBottom + 3);\n            }\n        }\n        NodeView.prototype.arrangeObject.call(this, canvas);\n    };\n\n    UMLActivationView.prototype.drawObject = function (canvas) {\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLActivationView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLActivationView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * UMLSeqMessageView\n     * @constructor\n     * @extends EdgeView\n     */\n    function UMLSeqMessageView() {\n        EdgeView.apply(this, arguments);\n        this.lineStyle = Core.LS_RECTILINEAR;\n        this.zIndex = 1;\n\n        /** @member {EdgeLabelView} */\n        this.nameLabel = new EdgeLabelView();\n        this.nameLabel.hostEdge = this;\n        this.nameLabel.edgePosition = Core.EP_MIDDLE;\n        this.nameLabel.distance = 10;\n        this.nameLabel.alpha = Math.PI / 2;\n        this.addSubView(this.nameLabel);\n\n        /** @member {EdgeLabelView} */\n        this.stereotypeLabel = new EdgeLabelView();\n        this.stereotypeLabel.hostEdge = this;\n        this.stereotypeLabel.edgePosition = Core.EP_MIDDLE;\n        this.stereotypeLabel.distance = 25;\n        this.stereotypeLabel.alpha = Math.PI / 2;\n        this.addSubView(this.stereotypeLabel);\n\n        /** @member {EdgeLabelView} */\n        this.propertyLabel = new EdgeLabelView();\n        this.propertyLabel.hostEdge = this;\n        this.propertyLabel.edgePosition = Core.EP_MIDDLE;\n        this.propertyLabel.distance = 10;\n        this.propertyLabel.alpha = -Math.PI / 2;\n        this.addSubView(this.propertyLabel);\n\n        /** @member {UMLActivationView} */\n        this.activation = new UMLActivationView();\n        this.addSubView(this.activation);\n\n        /** @member {boolean} */\n        this.showProperty = true;\n\n        /** @member {boolean} */\n        this.showType = true;\n    }\n    // inherits from EdgeView\n    UMLSeqMessageView.prototype = Object.create(EdgeView.prototype);\n    UMLSeqMessageView.prototype.constructor = UMLSeqMessageView;\n\n    UMLSeqMessageView.prototype._fixPointCount = function (cnt, xpos, ypos) {\n        if (this.points.count() != cnt) {\n            this.points.clear();\n            for (var pi = 1; pi <= cnt; pi++) {\n                this.points.add(new Point(xpos, ypos));\n            }\n        }\n    };\n\n    /**\n     * @private\n     */\n    UMLSeqMessageView.prototype._getSequenceNumberByPos = function (xpos, ypos) {\n        var seqNum = 0,\n            views  = this.getDiagram().ownedViews;\n        for (var i = 0, len = views.length; i < len; i++) {\n            var v = views[i];\n            if (v instanceof UMLSeqMessageView) {\n                if (v.points.getPoint(0).y < ypos) {\n                    seqNum = seqNum + 1;\n                } else if (v.points.getPoint(0).y == ypos) {\n                    if (v.points.getPoint(0).x < xpos) {\n                        seqNum = seqNum + 1;\n                    }\n                }\n            }\n        }\n        return seqNum;\n    };\n\n    /**\n     * @private\n     */\n    UMLSeqMessageView.prototype._indexOfMessageView = function (model) {\n        var dgm = this.getDiagram();\n        for (var i = 0, len = dgm.ownedViews.length; i < len; i++) {\n            var v = dgm.ownedViews[i];\n            if (v.model == model) {\n                return i;\n            }\n        }\n        return -1;\n    };\n\n    UMLSeqMessageView.prototype.regulateSequenceNumber = function () {\n        // var fromLine       = this.tail,\n        //     fromActivation = fromLine.getActivationAt(this.points.getPoint(0).y),\n        var dgm            = this.getDiagram(),\n            msg            = this.model,\n            interaction    = this.model._parent,\n            sn             = _.indexOf(interaction.messages, msg),\n            sn2            = this._getSequenceNumberByPos(this.points.getPoint(0).x, this.points.getPoint(0).y);\n        // determine sequence number\n        if (sn != sn2) {\n            // change by ownedViews\' index to correspond to SequenceNumber.\n            var c = this._indexOfMessageView(interaction.messages[sn2]);\n            dgm.ownedViews.remove(this);\n            dgm.ownedViews.insert(c, this);\n            // move at interactions\' index to correspond to SequenceNumber.\n            interaction.messages.remove(msg);\n            interaction.messages.insert(sn2, msg);\n        }\n    };\n\n    UMLSeqMessageView.prototype.update = function () {\n        EdgeView.prototype.update.call(this);\n        if (this.model) {\n            var options = {\n                showSequenceNumber : this.getDiagram().showSequenceNumber,\n                showSignature      : this.getDiagram().showSignature,\n                showActivation     : this.getDiagram().showActivation,\n                showType           : this.showType\n            };\n            this.nameLabel.text = this.model.getString(options);\n            this.nameLabel.visible = (this.nameLabel.text.length > 0);\n            this.stereotypeLabel.visible = (this.stereotypeLabel.text.length > 0);\n            this.activation.visible = (this.head instanceof UMLLinePartView &&\n                                       options.showActivation &&\n                                       ((this.model.messageSort === UML.MS_SYNCHCALL) ||\n                                        (this.model.messageSort === UML.MS_ASYNCHCALL) ||\n                                        (this.model.messageSort === UML.MS_DELETEMESSAGE)));\n            // line style\n            if ((this.model.messageSort === UML.MS_REPLY) || (this.model.messageSort === UML.MS_CREATEMESSAGE)) {\n                this.lineMode = Core.LM_DOT;\n            } else {\n                this.lineMode = Core.LM_SOLID;\n            }\n            // head end style\n            if ((this.model.messageSort === UML.MS_ASYNCHCALL) ||\n                (this.model.messageSort === UML.MS_ASYNCHSIGNAL) ||\n                (this.model.messageSort === UML.MS_CREATEMESSAGE) ||\n                (this.model.messageSort === UML.MS_REPLY)) {\n                this.headEndStyle = Core.ES_STICK_ARROW;\n            } else {\n                this.headEndStyle = Core.ES_SOLID_ARROW;\n            }\n            // stereotype\n            if (this.model.messageSort === UML.MS_CREATEMESSAGE) {\n                this.stereotypeLabel.text = "«create»";\n            } else if (this.model.messageSort === UML.MS_DELETEMESSAGE) {\n                this.stereotypeLabel.text = "«destroy»";\n            } else if (hasValue(this.model.stereotype)) {\n                this.stereotypeLabel.text = this.model.getStereotypeString();\n            } else {\n                this.stereotypeLabel.text = "";\n            }\n            // propertyLabel\n            this.propertyLabel.text = this.model.getPropertyString();\n            this.propertyLabel.visible = (this.showProperty ? this.propertyLabel.text.length > 0 : false);\n            // activation가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.activation.model !== this.model) {\n                Repository.bypassFieldAssign(this.activation, \'model\', this.model);\n            }\n            // nameLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.nameLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.nameLabel, \'model\', this.model);\n            }\n            // stereotypeLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.stereotypeLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.stereotypeLabel, \'model\', this.model);\n            }\n            // propertyLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.propertyLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.propertyLabel, \'model\', this.model);\n            }\n        }\n    };\n\n    UMLSeqMessageView.prototype.arrange = function (canvas) {\n        // arrange activation\n        this.activation.arrangeObject(canvas);\n        // if head or tail is MessageEndpointView\n        if (this.head instanceof UMLMessageEndpointView) {\n            this.head.top = this.points.getPoint(1).y - (this.head.height / 2);\n        }\n        if (this.tail instanceof UMLMessageEndpointView) {\n            this.tail.top = this.points.getPoint(1).y - (this.tail.height / 2);\n        }\n        EdgeView.prototype.arrange.call(this, canvas);\n        // if create message, head\'s X position should be the left of Lifeline.\n        if (this.model.messageSort === UML.MS_CREATEMESSAGE) {\n             if (this.points.getPoint(1).x > this.points.getPoint(0).x) {\n                 if (this.head._parent instanceof UMLSeqLifelineView) {\n                     var lifeline = this.head._parent;\n                     this.points.getPoint(1).x = lifeline.left;\n                 }\n             }\n        }\n    };\n\n    UMLSeqMessageView.prototype.arrangeObject = function (canvas) {\n        // default variable values\n        this.lineStyle = Core.LS_RECTILINEAR;\n        EdgeView.prototype.arrangeObject.call(this, canvas);\n\n        var fromLifeline   = this.tail,\n            fromActivation = (fromLifeline instanceof UMLLinePartView ? fromLifeline.getActivationAt(this.points.getPoint(0).y) : null),\n            toLifeline     = this.head,\n            toActivation   = (toLifeline instanceof UMLLinePartView ? toLifeline.getActivationAt(this.points.getPoint(this.points.count() - 1).y) : null);\n\n        // (1) in case of Self message\n        if (this.head === this.tail) {\n            this._fixPointCount(4, this.points.getPoint(0).x, this.points.getPoint(0).y);\n            if ((fromActivation !== null) && fromActivation.visible) {\n                this.points.setPoint(0, new Point(fromActivation.getRight(), this.points.getPoint(0).y));\n            } else {\n                this.points.setPoint(0, new Point(fromLifeline.getRight(), this.points.getPoint(0).y));\n            }\n            this.points.setPoint(1, new Point(this.points.getPoint(0).x + SELF_MESSAGE_WIDTH, this.points.getPoint(0).y));\n            this.points.setPoint(2, new Point(this.points.getPoint(0).x + SELF_MESSAGE_WIDTH, this.points.getPoint(0).y + SELF_MESSAGE_HEIGHT));\n            if ((toActivation !== null) && toActivation.visible) {\n                this.points.setPoint(3, new Point(this.activation.getRight(), this.points.getPoint(0).y + SELF_MESSAGE_HEIGHT));\n            } else {\n                this.points.setPoint(3, new Point(this.tail.getRight(), this.points.getPoint(0).y + SELF_MESSAGE_HEIGHT));\n            }\n        // (2) in case of left-to-right directed message\n        } else if (this.points.getPoint(1).x > this.points.getPoint(0).x) {\n            this._fixPointCount(2, this.points.getPoint(0).x, this.points.getPoint(0).y);\n            if ((fromActivation !== null) && fromActivation.visible) {\n                this.points.setPoint(0, new Point(fromActivation.getRight(), this.points.getPoint(0).y));\n            } else {\n                this.points.setPoint(0, new Point(fromLifeline.getRight(), this.points.getPoint(0).y));\n            }\n            if ((toActivation !== null) && toActivation.visible) {\n                this.points.setPoint(1, new Point(this.activation.left, this.points.getPoint(0).y));\n            } else {\n                this.points.setPoint(1, new Point(this.head.left, this.points.getPoint(0).y));\n            }\n        // (3) in case of right-to-left direced message\n        } else {\n            this._fixPointCount(2, this.points.getPoint(0).x, this.points.getPoint(0).y);\n            if ((fromActivation !== null) && fromActivation.visible) {\n                this.points.setPoint(0, new Point(fromActivation.left, this.points.getPoint(0).y));\n            } else {\n                this.points.setPoint(0, new Point(fromLifeline.left, this.points.getPoint(0).y));\n            }\n            if ((toActivation !== null) && toActivation.visible) {\n                this.points.setPoint(1, new Point(this.activation.getRight(), this.points.getPoint(0).y));\n            } else {\n                this.points.setPoint(1, new Point(this.head.getRight(), this.points.getPoint(0).y));\n            }\n        }\n        this.update();\n    };\n\n    UMLSeqMessageView.prototype.drawObject = function (canvas) {\n        EdgeView.prototype.drawObject.call(this, canvas);\n        if (this.model.messageSort === UML.MS_DELETEMESSAGE) {\n            var lifeline = this.head,\n                bottom   = lifeline.getBottom(),\n                center   = (lifeline.left + lifeline.getRight()) / 2;\n            canvas.line(center - 10, bottom - 10, center + 10, bottom + 10);\n            canvas.line(center + 10, bottom - 10, center - 10, bottom + 10);\n        }\n    };\n\n    /**\n     *\n     */\n    UMLSeqMessageView.prototype.initialize = function (canvas, x1, y1, x2, y2) {\n        this.lineStyle = Core.LS_RECTILINEAR;\n        if (this.head != this.tail) {\n            this.points.clear();\n            this.points.add(new Point(this.tail.getRight(), y1));\n            this.points.add(new Point(this.head.left, y2));\n        } else { // Self Message\n            this.points.clear();\n            this.points.add(new Point(x1, y1));\n            this.points.add(new Point(x1, y1));\n            this.points.add(new Point(x1, y1));\n            this.points.add(new Point(x1, y1));\n        }\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLSeqMessageView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLSeqMessageView.prototype.canDelete = function () {\n        return false;\n    };\n\n    /**\n     * Determine where it can be connected to\n     */\n    UMLSeqMessageView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model && view.model instanceof type.UMLMessageEndpoint);\n    };\n\n    /**\n     * UMLStateInvariantView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLStateInvariantView() {\n        NodeView.apply(this, arguments);\n        this.movable = Core.MM_VERT;\n        this.zIndex = 1;\n\n        /** @member {LabelView} */\n        this.invariantLabel = new LabelView();\n        this.invariantLabel.horizontalAlignment = Graphics.AL_CENTER;\n        this.invariantLabel.verticalAlignment = Graphics.AL_MIDDLE;\n        this.addSubView(this.invariantLabel);\n    }\n    // inherits from NodeView\n    UMLStateInvariantView.prototype = Object.create(NodeView.prototype);\n    UMLStateInvariantView.prototype.constructor = UMLStateInvariantView;\n\n    UMLStateInvariantView.prototype.update = function (canvas) {\n        NodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.invariantLabel.text = "{" + this.model.invariant + "}";\n        }\n    };\n\n    UMLStateInvariantView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = Math.max(this.invariantLabel.minWidth, STATEINVARIANT_MINWIDTH);\n        this.minHeight = Math.max(this.invariantLabel.minHeight, STATEINVARIANT_MINHEIGHT);\n        this.sizeConstraints();\n    };\n\n    UMLStateInvariantView.prototype.arrangeObject = function (canvas) {\n        NodeView.prototype.arrangeObject.call(this, canvas);\n        if (this._parent instanceof UMLSeqLifelineView) {\n            this.left = this._parent.linePart.left - (this.width / 2);\n        }\n        this.invariantLabel.top  = this.top + (this.height - this.invariantLabel.height) / 2;\n        this.invariantLabel.left  = this.left + (this.width - this.invariantLabel.width) / 2;\n    };\n\n    UMLStateInvariantView.prototype.drawObject = function (canvas) {\n        canvas.fillRoundRect(this.left, this.top, this.getRight(), this.getBottom(), STATE_ROUND);\n        canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), STATE_ROUND);\n        NodeView.prototype.drawObject.call(this, canvas);\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLStateInvariantView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLStateInvariantView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * UMLContinuationView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLContinuationView() {\n        NodeView.apply(this, arguments);\n        this.zIndex = 1;\n\n        /** @member {LabelView} */\n        this.nameLabel = new LabelView();\n        this.nameLabel.horizontalAlignment = Graphics.AL_CENTER;\n        this.nameLabel.verticalAlignment = Graphics.AL_MIDDLE;\n        this.addSubView(this.nameLabel);\n    }\n    // inherits from NodeView\n    UMLContinuationView.prototype = Object.create(NodeView.prototype);\n    UMLContinuationView.prototype.constructor = UMLContinuationView;\n\n    UMLContinuationView.prototype.update = function (canvas) {\n        NodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameLabel.text = this.model.name;\n        }\n    };\n\n    UMLContinuationView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = Math.max(this.nameLabel.minWidth, CONTINUATION_MINWIDTH);\n        this.minHeight = Math.max(this.nameLabel.minHeight, CONTINUATION_MINHEIGHT);\n        this.sizeConstraints();\n    };\n\n    UMLContinuationView.prototype.arrangeObject = function (canvas) {\n        NodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameLabel.top  = this.top + (this.height - this.nameLabel.height) / 2;\n        this.nameLabel.left  = this.left + (this.width - this.nameLabel.width) / 2;\n    };\n\n    UMLContinuationView.prototype.drawObject = function (canvas) {\n        canvas.fillRoundRect(this.left, this.top, this.getRight(), this.getBottom(), STATE_ROUND);\n        canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), STATE_ROUND);\n        NodeView.prototype.drawObject.call(this, canvas);\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLContinuationView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLContinuationView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * UMLCustomFrameView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLCustomFrameView() {\n        NodeView.apply(this, arguments);\n\n        /** @member {LabelView} */\n        this.nameLabel = new LabelView();\n        this.nameLabel.parentStyle = true;\n        this.addSubView(this.nameLabel);\n\n        /** @member {LabelView} */\n        this.frameTypeLabel = new LabelView();\n        this.frameTypeLabel.parentStyle = true;\n        this.addSubView(this.frameTypeLabel);\n    }\n    // inherits from NodeView\n    UMLCustomFrameView.prototype = Object.create(NodeView.prototype);\n    UMLCustomFrameView.prototype.constructor = UMLCustomFrameView;\n\n    UMLCustomFrameView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        var h = Math.max(this.frameTypeLabel.minHeight, this.nameLabel.minHeight) + COMPARTMENT_TOP_PADDING + COMPARTMENT_BOTTOM_PADDING;\n        this.minHeight = Math.max(h + FRAME_CONTENT_MINHEIGHT, FRAME_MINHEIGHT);\n        var w = this.frameTypeLabel.minWidth + this.nameLabel.minWidth;\n        if (this.frameTypeLabel.visible && this.frameTypeLabel.visible) {\n            w = w + LABEL_INTERVAL;\n        }\n        w = w + COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING + h / 2;\n        this.minWidth = Math.max(w + FRAME_CONTENT_MINWIDTH, FRAME_MINWIDTH);\n        this.sizeConstraints();\n    };\n\n    UMLCustomFrameView.prototype.arrangeObject = function (canvas) {\n        NodeView.prototype.arrangeObject.call(this, canvas);\n        this.frameTypeLabel.font.style = Graphics.FS_BOLD;\n        this.frameTypeLabel.left = this.left + COMPARTMENT_LEFT_PADDING;\n        this.frameTypeLabel.top = this.top + COMPARTMENT_TOP_PADDING;\n        this.frameTypeLabel.setRight(this.frameTypeLabel.left + this.frameTypeLabel.minWidth);\n        if (this.frameTypeLabel.visible) {\n            this.nameLabel.left = this.frameTypeLabel.getRight() + LABEL_INTERVAL * 2;\n        } else {\n            this.nameLabel.left = this.left + COMPARTMENT_LEFT_PADDING;\n        }\n        this.nameLabel.top = this.top + COMPARTMENT_TOP_PADDING;\n        this.nameLabel.setRight(this.nameLabel.left + this.nameLabel.minWidth);\n    };\n\n    UMLCustomFrameView.prototype.drawObject = function (canvas) {\n        NodeView.prototype.drawObject.call(this, canvas);\n        var y = Math.max(this.frameTypeLabel.getBottom(), this.nameLabel.getBottom()) + COMPARTMENT_BOTTOM_PADDING,\n            h = y - this.top,\n            x = this.nameLabel.getRight() + COMPARTMENT_RIGHT_PADDING + h / 2;\n        canvas.polyline([new Point(this.left, this.top), new Point(this.getRight(), this.top), new Point(this.getRight(), this.getBottom()), new Point(this.left, this.getBottom()), new Point(this.left, this.top)]);\n        canvas.fillPolygon([new Point(this.left, this.top), new Point(this.left, y), new Point(x - h / 2, y), new Point(x, this.top + h / 2), new Point(x, this.top), new Point(this.left, this.top)]);\n        canvas.polygon([new Point(this.left, this.top), new Point(this.left, y), new Point(x - h / 2, y), new Point(x, this.top + h / 2), new Point(x, this.top), new Point(this.left, this.top)]);\n    };\n\n\n    /**\n     * UMLFrameView\n     * @constructor\n     * @extends UMLCustomFrameView\n     */\n    function UMLFrameView() {\n        UMLCustomFrameView.apply(this, arguments);\n        this.zIndex = -1;\n    }\n    // inherits from UMLCustomFrameView\n    UMLFrameView.prototype = Object.create(UMLCustomFrameView.prototype);\n    UMLFrameView.prototype.constructor = UMLFrameView;\n\n    UMLFrameView.prototype.update = function (canvas) {\n        UMLCustomFrameView.prototype.update.call(this, canvas);\n        if (this.model) {\n            // frame kind\n            if (this.model instanceof type.UMLClass || this.model instanceof type.UMLClassDiagram) {\n                this.frameTypeLabel.text = "class";\n            } else if (this.model instanceof type.UMLComponent || this.model instanceof type.UMLComponentDiagram) {\n                this.frameTypeLabel.text = "component";\n            } else if (this.model instanceof type.UMLDeploymentDiagram) {\n                this.frameTypeLabel.text = "deployment";\n            } else if (this.model instanceof type.UMLInteraction || this.model instanceof type.UMLSequenceDiagram || this.model instanceof type.UMLCommunicationDiagram) {\n                this.frameTypeLabel.text = "interaction";\n            } else if (this.model instanceof type.UMLStateMachine || this.model instanceof type.UMLStatechartDiagram) {\n                this.frameTypeLabel.text = "state machine";\n            } else if (this.model instanceof type.UMLActivity || this.model instanceof type.UMLActivityDiagram) {\n                this.frameTypeLabel.text = "activity";\n            } else if (this.model instanceof type.UMLUseCase || this.model instanceof type.UMLUseCaseDiagram) {\n                this.frameTypeLabel.text = "use case";\n            } else if (this.model instanceof type.UMLPackage || this.model instanceof type.UMLPackageDiagram) {\n                this.frameTypeLabel.text = "package";\n            }\n            // name\n            if (this.model.name.length > 0) {\n                this.nameLabel.text = this.model.name;\n            }\n        }\n    };\n\n\n    /**\n     * UMLInteractionOperandView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLInteractionOperandView() {\n        NodeView.apply(this, arguments);\n        this.selectable = Core.SK_YES;\n        this.movable = Core.MM_NONE;\n        this.sizable = Core.SZ_VERT;\n        this.parentStyle = true;\n        this.minHeight = 15;\n        this.height = 30;\n\n        /** @member {LabelView} */\n        this.guardLabel = new LabelView();\n        this.addSubView(this.guardLabel);\n    }\n    // inherits from LabelView\n    UMLInteractionOperandView.prototype = Object.create(NodeView.prototype);\n    UMLInteractionOperandView.prototype.constructor = UMLInteractionOperandView;\n\n    UMLInteractionOperandView.prototype._isTopOperandView = function () {\n        var result = true;\n        if (this._parent !== null) {\n            for (var i = 0, len = this._parent.subViews.length; i < len; i++) {\n                var v = this._parent.subViews[i];\n                if ((v instanceof UMLInteractionOperandView) && (v !== this)) {\n                    if (v.top < this.top) {\n                        result = false;\n                        return result;\n                    }\n                }\n            }\n        }\n        return result;\n    };\n\n    UMLInteractionOperandView.prototype.update = function (canvas) {\n        NodeView.prototype.update.call(this, canvas);\n        if (this._parent) {\n            this.visible = this._parent.visible;\n        }\n        if (this.model) {\n            if (hasValue(this.model.guard)) {\n                this.guardLabel.text = \'[\' + this.model.guard + \']\';\n            } else {\n                this.guardLabel.text = \'\';\n            }\n            this.guardLabel.visible = (this.guardLabel.text.length > 0);\n        }\n    };\n\n    UMLInteractionOperandView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = Math.max(this.guardLabel.minWidth + INTERACTIONOPERAND_GUARD_HORZ_MARGIN * 2, INTERACTIONOPERAND_MINWIDTH);\n        this.minHeight = Math.max(this.guardLabel.minHeight + INTERACTIONOPERAND_GUARD_VERT_MARGIN * 2, INTERACTIONOPERAND_MINHEIGHT);\n    };\n\n    UMLInteractionOperandView.prototype.arrangeObject = function (canvas) {\n        NodeView.prototype.arrangeObject.call(this, canvas);\n        this.guardLabel.left = this.left + INTERACTIONOPERAND_GUARD_HORZ_MARGIN;\n        this.guardLabel.top = this.top + INTERACTIONOPERAND_GUARD_VERT_MARGIN;\n        this.guardLabel.width = this.guardLabel.minWidth;\n        this.guardLabel.height = this.guardLabel.minHeight;\n    };\n\n    UMLInteractionOperandView.prototype.drawObject = function (canvas) {\n        NodeView.prototype.drawObject.call(this, canvas);\n        if (!this._isTopOperandView()) {\n            canvas.line(this.left, this.top, this.getRight(), this.top, [10,3]);\n        }\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLInteractionOperandView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLInteractionOperandView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * UMLInteractionOperandCompartmentView\n     * @constructor\n     * @extends UMLCompartmentView\n     */\n    function UMLInteractionOperandCompartmentView() {\n        UMLCompartmentView.apply(this, arguments);\n        this.minHeight = 15;\n\n        /* temporal */\n        this._leftPadding   = 0;\n        this._rightPadding  = 0;\n        this._topPadding    = 0;\n        this._bottomPadding = 0;\n        this._itemInterval  = 0;\n    }\n    // inherits from UMLCompartmentView\n    UMLInteractionOperandCompartmentView.prototype = Object.create(UMLCompartmentView.prototype);\n    UMLInteractionOperandCompartmentView.prototype.constructor = UMLInteractionOperandCompartmentView;\n\n    UMLInteractionOperandCompartmentView.prototype.update = function (canvas) {\n        if (this.model.operands) {\n            var tempViews = this.subViews;\n            this.subViews = [];\n            for (var i = 0, len = this.model.operands.length; i < len; i++) {\n                var operand = this.model.operands[i];\n                var operandView = _.find(tempViews, function (v) { return v.model == operand; });\n                if (!operandView) {\n                    operandView = new UMLInteractionOperandView();\n                    operandView.model = operand;\n                    operandView._parent = this;\n                    operandView._parent = this;\n                    // UMLInteractionOperandView가 Repository에 정상적으로 등록될 수 있도록 Bypass Command에 의해서 생성한다.\n                    Repository.bypassInsert(this, \'subViews\', operandView);\n                } else {\n                    this.addSubView(operandView);\n                }\n                operandView.setup(canvas);\n            }\n        }\n        UMLCompartmentView.prototype.update.call(this, canvas);\n    };\n\n\n    /**\n     * UMLCombinedFragmentView\n     * @constructor\n     * @extends UMLCustomFrameView\n     */\n    function UMLCombinedFragmentView() {\n        UMLCustomFrameView.apply(this, arguments);\n\n        /** @member {UMLInteractionOperandCompartmentView} */\n        this.operandCompartment = new UMLInteractionOperandCompartmentView();\n        this.operandCompartment.parentStyle = true;\n        this.addSubView(this.operandCompartment);\n    }\n    // inherits from UMLCustomFrameView\n    UMLCombinedFragmentView.prototype = Object.create(UMLCustomFrameView.prototype);\n    UMLCombinedFragmentView.prototype.constructor = UMLCombinedFragmentView;\n\n    UMLCombinedFragmentView.prototype._carryOnOperandViews = function () {\n        if (this.subViews.length > 0) {\n            var i, len, interOpViews = [];\n            for(i = 0, len = this.subViews.length; i < len; i++) {\n                if (this.subViews[i] instanceof UMLInteractionOperandView) {\n                    interOpViews.push(this.subViews[i]);\n                }\n            }\n            if (interOpViews.length > 0) {\n                _.sortBy(interOpViews, function (view) { return view.top; });\n                var firstIOV = interOpViews[0];\n                firstIOV.top = Math.max(this.frameTypeLabel.getBottom(), this.nameLabel.getBottom()) + COMPARTMENT_BOTTOM_PADDING;\n                for (i = 1; i <= (interOpViews.length - 1); i++) {\n                    interOpViews[i].top = interOpViews[i - 1].getBottom();\n                }\n                var lastIOV = interOpViews[interOpViews.length - 1];\n                lastIOV.setBottom(this.getBottom());\n                this.minHeight = lastIOV.top + lastIOV.minHeight - this.top;\n            }\n        }\n    };\n\n    UMLCombinedFragmentView.prototype.update = function (canvas) {\n        if (this.operandCompartment.model !== this.model) {\n            // operandCompartment가 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            Repository.bypassFieldAssign(this.operandCompartment, \'model\', this.model);\n        }\n        if (this.model) {\n            this.nameLabel.text = this.model.name;\n            this.frameTypeLabel.text = this.model.interactionOperator;\n            this.nameLabel.visible = (this.nameLabel.text.length > 0);\n            this.frameTypeLabel.visible = (this.frameTypeLabel.text.length > 0);\n            if (this.model.operands && this.model.operands.length > 0) {\n                this.operandCompartment.visible = true;\n            } else {\n                this.operandCompartment.visible = false;\n            }\n        }\n        UMLCustomFrameView.prototype.update.call(this, canvas);\n    };\n\n    UMLCombinedFragmentView.prototype.sizeObject = function (canvas) {\n        UMLCustomFrameView.prototype.sizeObject.call(this, canvas);\n        if (this.operandCompartment.visible) {\n            var h = Math.max(this.frameTypeLabel.height, this.nameLabel.height) + COMPARTMENT_TOP_PADDING + COMPARTMENT_BOTTOM_PADDING;\n            this.minHeight = h + this.operandCompartment.height;\n        }\n    };\n\n\n    UMLCombinedFragmentView.prototype.arrangeObject = function (canvas) {\n        UMLCustomFrameView.prototype.arrangeObject.call(this, canvas);\n        if (this.operandCompartment.visible) {\n            var h = Math.max(this.frameTypeLabel.height, this.nameLabel.height) + COMPARTMENT_TOP_PADDING + COMPARTMENT_BOTTOM_PADDING;\n            this.operandCompartment.left = this.left;\n            this.operandCompartment.top = this.top + h;\n            this.operandCompartment.width = this.width;\n            this.operandCompartment.height = this.height - h;\n            this.operandCompartment.arrange(canvas);\n        }\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLCombinedFragmentView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLCombinedFragmentView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * UMLInteractionUseView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLInteractionUseView() {\n        NodeView.apply(this, arguments);\n        this.zIndex = 1;\n\n        /** @member {LabelView} */\n        this.nameLabel = new LabelView();\n        this.addSubView(this.nameLabel);\n\n        /** @member {LabelView} */\n        this.frameTypeLabel = new LabelView();\n        this.frameTypeLabel.font.style = Graphics.FS_BOLD;\n        this.addSubView(this.frameTypeLabel);\n    }\n    // inherits from NodeView\n    UMLInteractionUseView.prototype = Object.create(NodeView.prototype);\n    UMLInteractionUseView.prototype.constructor = UMLInteractionUseView;\n\n    UMLInteractionUseView.prototype.update = function (canvas) {\n        NodeView.prototype.update.call(this, canvas);\n        this.frameTypeLabel.text = \'ref\';\n        if (this.model) {\n            var s = "";\n            if (this.model.returnValueRecipient) {\n                s += this.model.returnValueRecipient.name + " = ";\n            }\n            if (this.model.refersTo) {\n                s += this.model.refersTo.name;\n            }\n            if (this.model.arguments.length > 0) {\n                s += "(" + this.model.arguments + ")";\n            }\n            if (this.model.returnValue.length > 0) {\n                s += ": " + this.model.returnValue;\n            }\n            this.nameLabel.text = s;\n        }\n    };\n\n    UMLInteractionUseView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        var h = this.frameTypeLabel.minHeight + this.nameLabel.minHeight + COMPARTMENT_TOP_PADDING + COMPARTMENT_BOTTOM_PADDING;\n        this.minHeight = Math.max(h, FRAME_MINHEIGHT);\n        var w = Math.max(this.frameTypeLabel.width, this.nameLabel.width);\n        w = w + COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING;\n        this.minWidth = Math.max(w, FRAME_MINWIDTH);\n        this.sizeConstraints();\n    };\n\n    UMLInteractionUseView.prototype.arrangeObject = function (canvas) {\n        NodeView.prototype.arrangeObject.call(this, canvas);\n        this.frameTypeLabel.left = this.left + COMPARTMENT_LEFT_PADDING;\n        this.frameTypeLabel.top = this.top + COMPARTMENT_TOP_PADDING;\n        this.frameTypeLabel.setRight(this.frameTypeLabel.left + this.frameTypeLabel.minWidth);\n        this.nameLabel.top = this.top + (this.height - this.nameLabel.height) / 2;\n        this.nameLabel.left = this.left + (this.width - this.nameLabel.width) / 2;\n    };\n\n    UMLInteractionUseView.prototype.drawObject = function (canvas) {\n        NodeView.prototype.drawObject.call(this, canvas);\n        var y = this.frameTypeLabel.getBottom() + COMPARTMENT_BOTTOM_PADDING,\n            h = y - this.top,\n            x = this.frameTypeLabel.getRight() + COMPARTMENT_RIGHT_PADDING + h / 2;\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.polygon([new Point(this.left, this.top), new Point(this.left, y), new Point(x - h / 2, y), new Point(x, this.top + h / 2), new Point(x, this.top), new Point(this.left, this.top)]);\n    };\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLInteractionUseView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLInteractionUseView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**************************************************************************\n     *                                                                        *\n     *                       COMMUNICATION DIAGRAM VIEWS                      *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLCommunicationDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLCommunicationDiagram() {\n        UMLDiagram.apply(this, arguments);\n        this.showSequenceNumber = true;\n        this.showSignature = true;\n    }\n    // inherits from UMLDiagram\n    UMLCommunicationDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLCommunicationDiagram.prototype.constructor = UMLCommunicationDiagram;\n\n    UMLCommunicationDiagram.prototype.canAcceptModel = function (model) {\n        if (model instanceof type.Hyperlink || model instanceof type.Diagram) {\n            return true;\n        } else if (model instanceof type.UMLLifeline) {\n            return _.every(this.ownedViews, function (v) { return v.model !== model; });\n        } else if (model instanceof type.UMLMessage) {\n            return _.some(this.ownedViews, function (v) { return v.model === model.source; }) &&\n                   _.some(this.ownedViews, function (v) { return v.model === model.target; }) &&\n                   _.every(this.ownedViews, function (v) { return v.model !== model; });\n        } else {\n            return (model instanceof type.UMLConstraint) ||\n                   (model instanceof type.UMLClassifier);\n        }\n    };\n\n    /**\n     * UMLCommLifelineView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLCommLifelineView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.lifeline.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLCommLifelineView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLCommLifelineView.prototype.constructor = UMLCommLifelineView;\n\n    UMLCommLifelineView.prototype.update = function (canvas) {\n        UMLGeneralNodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameCompartment.nameLabel.text = this.model.getString(this);\n            this.nameCompartment.nameLabel.underline = false;\n        }\n    };\n\n    UMLCommLifelineView.prototype.drawIcon = function (canvas, rect) {\n        if (this.model) {\n            if (this.model.stereotype && this.model.stereotype.icon) {\n                UMLGeneralNodeView.prototype.drawIcon.call(this, canvas, rect);\n            } else if (this.model.represent && (this.model.represent.type instanceof type.Model)) {\n                var iconRatioBackup = this.iconRatio;\n                if (this.model.represent.type instanceof type.UMLActor) {\n                    this.iconRatio = ACTOR_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLActorView.prototype.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLUseCase) {\n                    this.iconRatio = USECASE_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLUseCaseView.prototype.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLInterface) {\n                    this.iconRatio = 100;\n                    this.arrangeObject(canvas);\n                    UMLInterfaceView.prototype.drawBallNotation.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLArtifact) {\n                    this.iconRatio = ARTIFACT_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLArtifactViewMixin.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLComponent) {\n                    this.iconRatio = COMPONENT_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLComponentViewMixin.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type instanceof type.UMLNode) {\n                    this.iconRatio = NODE_RATIO_PERCENT;\n                    this.arrangeObject(canvas);\n                    UMLNodeViewMixin.drawIcon.call(this, canvas, this.iconRect);\n                } else if (this.model.represent.type.stereotype && this.model.represent.type.stereotype.icon) {\n                    drawImage(canvas, rect, this.model.represent.type.stereotype.icon);\n                } else {\n                    UMLGeneralNodeView.prototype.drawIcon.call(this, canvas, rect);\n                }\n                this.iconRatio = iconRatioBackup;\n                this.arrangeObject(canvas);\n            } else {\n                UMLGeneralNodeView.prototype.drawIcon.call(this, canvas, rect);\n            }\n        }\n    };\n\n    UMLCommLifelineView.prototype.drawAsCanonicalForm = function (canvas, showLabel) {\n        UMLGeneralNodeView.prototype.drawAsCanonicalForm.call(this, canvas);\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        if (this.model.isMultiInstance) {\n            canvas.rect(r.x1 + MULTI_INSTANCE_MARGIN, r.y1 + MULTI_INSTANCE_MARGIN, r.x2 + MULTI_INSTANCE_MARGIN, r.y2 + MULTI_INSTANCE_MARGIN);\n        }\n        canvas.fillRect(r.x1, r.y1, r.x2, r.y2);\n        canvas.rect(r.x1, r.y1, r.x2, r.y2);\n    };\n\n    UMLCommLifelineView.prototype.drawAsDecorationForm = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        if (this.model.isMultiInstance) {\n            canvas.rect(r.x1 + MULTI_INSTANCE_MARGIN, r.y1 + MULTI_INSTANCE_MARGIN, r.x2 + MULTI_INSTANCE_MARGIN, r.y2 + MULTI_INSTANCE_MARGIN);\n        }\n        canvas.fillRect(r.x1, r.y1, r.x2, r.y2);\n        canvas.rect(r.x1, r.y1, r.x2, r.y2);\n        UMLGeneralNodeView.prototype.drawAsDecorationForm.call(this, canvas);\n    };\n\n    UMLCommLifelineView.prototype.drawAsIconicForm = function (canvas) {\n        UMLGeneralNodeView.prototype.drawAsIconicForm.call(this, canvas);\n    };\n\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLCommLifelineView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLCommLifelineView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * UMLCommMessageView\n     * @constructor\n     * @extends EdgeNodeView\n     */\n    function UMLCommMessageView() {\n        EdgeNodeView.apply(this, arguments);\n        this.edgePosition = Core.EP_MIDDLE;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_FREE;\n        this.alpha = Math.PI / 2;\n        this.distance = 10;\n\n        this.headPoint = new Point();\n        this.tailPoint = new Point();\n        this.arrowPoint1 = new Point();\n        this.arrowPoint2 = new Point();\n\n        /** @member {NodeLabelView} */\n        this.nameLabel = new NodeLabelView();\n        this.nameLabel.distance = 10;\n        this.nameLabel.alpha = Math.PI / 2;\n        this.addSubView(this.nameLabel);\n\n        /** @member {NodeLabelView} */\n        this.stereotypeLabel = new NodeLabelView();\n        this.stereotypeLabel.distance = 25;\n        this.stereotypeLabel.alpha = Math.PI / 2;\n        this.addSubView(this.stereotypeLabel);\n\n        /** @member {NodeLabelView} */\n        this.propertyLabel = new NodeLabelView();\n        this.propertyLabel.distance = 10;\n        this.propertyLabel.alpha = Math.PI / 2;\n        this.addSubView(this.propertyLabel);\n\n        /** @member {boolean} */\n        this.showProperty = true;\n\n        /** @member {boolean} */\n        this.showType = true;\n    }\n    // inherits from EdgeNodeView\n    UMLCommMessageView.prototype = Object.create(EdgeNodeView.prototype);\n    UMLCommMessageView.prototype.constructor = UMLCommMessageView;\n\n    UMLCommMessageView.prototype._calcPosition = function (canvas) {\n        var midPointIndex = Math.floor(this.hostEdge.points.count() / 2);\n        if (this.hostEdge.points.count() % 2 === 0) {\n            midPointIndex--;\n        }\n\n        var p1 = this.hostEdge.points.getPoint(midPointIndex);\n        var p2 = this.hostEdge.points.getPoint(midPointIndex+1);\n        var tempP1 = p1.copy();\n        var tempP2 = p2.copy();\n        if ((this.hostEdge.points.count() % 2) === 0) {\n            tempP1.x = Math.floor((tempP1.x + tempP2.x) / 2);\n            tempP1.y = Math.floor((tempP1.y + tempP2.y) / 2);\n        }\n\n        // Calc Theta of Link\n        var tempTh = getTheta(tempP1.x, tempP1.y, tempP2.x, tempP2.y);\n\n        // Calc Head and Tail Points of Message\n        var tempMiddleX = (tempP1.x + tempP2.x) / 2;\n        var tempMiddleY = (tempP1.y + tempP2.y) / 2;\n        var tempX = this.distance * Math.cos(tempTh + this.alpha);\n        var tempY = this.distance * Math.sin(tempTh + this.alpha);\n\n        tempMiddleX = tempMiddleX + tempX;\n        tempMiddleY = tempMiddleY - tempY;\n\n        tempX = 20 * Math.cos(tempTh);\n        tempY = 20 * Math.sin(tempTh);\n\n        tempMiddleX = this.left + Math.abs(tempX);\n        tempMiddleY = this.top + Math.abs(tempY);\n\n        var tempHPointX, tempHPointY, tempTPointX, tempTPointY;\n\n        if (this.model.source === this.hostEdge.tail.model) {\n            // Forward Stimulus\n            tempHPointX = tempMiddleX + tempX;\n            tempHPointY = tempMiddleY - tempY;\n            tempTPointX = tempMiddleX - tempX;\n            tempTPointY = tempMiddleY + tempY;\n        } else {\n            // Reverse Stimulus\n            tempTPointX = tempMiddleX + tempX;\n            tempTPointY = tempMiddleY - tempY;\n            tempHPointX = tempMiddleX - tempX;\n            tempHPointY = tempMiddleY + tempY;\n        }\n\n        this.headPoint.x = Math.floor(tempHPointX);\n        this.headPoint.y = Math.floor(tempHPointY);\n        this.tailPoint.x = Math.floor(tempTPointX);\n        this.tailPoint.y = Math.floor(tempTPointY);\n\n        // Calc Arrow Points of Message;\n        var rt = new Rect(this.headPoint.x, this.headPoint.y, this.tailPoint.x, this.tailPoint.y);\n        var a = rt.y2 - rt.y1;\n        var b = (rt.x2 - rt.x1 + 0.00001);\n        var th = Math.atan(a / b);\n        if (((a < 0) && (b < 0)) || ((a > 0) && (b < 0)) || ((a === 0) && (b < 0))) {\n            th = th + Math.PI;\n        }\n        var th1 = th - Math.PI / 8;\n        var th2 = th + Math.PI / 8;\n        this.arrowPoint1.x = Math.floor(12 * Math.cos(th1)) + rt.x1;\n        this.arrowPoint1.y = Math.floor(12 * Math.sin(th1)) + rt.y1;\n        this.arrowPoint2.x = Math.floor(12 * Math.cos(th2)) + rt.x1;\n        this.arrowPoint2.y = Math.floor(12 * Math.sin(th2)) + rt.y1;\n\n        // this.left = Math.min(this.headPoint.x, this.tailPoint.x);\n        // this.top = Math.min(this.headPoint.y, this.tailPoint.y);\n        // this.setRight(Math.max(this.headPoint.x, this.tailPoint.x));\n        // this.setBottom(Math.max(this.headPoint.y, this.tailPoint.y));\n        this.width = Math.max(this.headPoint.x, this.tailPoint.x) - Math.min(this.headPoint.x, this.tailPoint.x);\n        this.height = Math.max(this.headPoint.y, this.tailPoint.y) - Math.min(this.headPoint.y, this.tailPoint.y);\n    };\n\n    UMLCommMessageView.prototype.update = function () {\n        EdgeNodeView.prototype.update.call(this);\n        if (this.model) {\n            var options = {\n                showSequenceNumber : this.getDiagram().showSequenceNumber,\n                showSignature      : this.getDiagram().showSignature,\n                showType           : this.showType\n            };\n            this.nameLabel.text = this.model.getString(options);\n            // stereotype\n            if (this.model.messageSort === UML.MS_CREATEMESSAGE) {\n                this.stereotypeLabel.text = "«create»";\n            } else if (this.model.messageSort === UML.MS_DELETEMESSAGE) {\n                this.stereotypeLabel.text = "«destroy»";\n            } else if (hasValue(this.model.stereotype)) {\n                this.stereotypeLabel.text = this.model.getStereotypeString();\n            } else {\n                this.stereotypeLabel.text = "";\n            }\n            // propertyLabel\n            this.propertyLabel.text = this.model.getPropertyString();\n            this.propertyLabel.visible = (this.showProperty ? this.propertyLabel.text.length > 0 : false);\n            // nameLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.nameLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.nameLabel, \'model\', this.model);\n            }\n            // stereotypeLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.stereotypeLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.stereotypeLabel, \'model\', this.model);\n            }\n            // propertyLabel이 model을 정상적으로 reference 할 수 있도록 Bypass Command에 의해서 설정한다.\n            if (this.propertyLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.propertyLabel, \'model\', this.model);\n            }\n        }\n    };\n\n    UMLCommMessageView.prototype.containsPoint = function (canvas, x, y) {\n        var r = this.getBoundingBox(canvas);\n        // Expand selectable area because it is difficult to select UMLCommMessageView when it is not diagonal.\n        r.expand(10);\n        return Coord.ptInRect(x, y, r);\n    };\n\n    UMLCommMessageView.prototype.arrangeObject = function (canvas) {\n        EdgeNodeView.prototype.arrangeObject.call(this, canvas);\n        var v = this.propertyLabel.visible;\n        this.nameLabel.visible = (this.nameLabel.text.length > 0);\n        this.stereotypeLabel.visible = (this.stereotypeLabel.text.length > 0);\n        this.propertyLabel.visible = (this.propertyLabel.text.length > 0);\n        this._calcPosition(canvas);\n        // this.width = Math.max(this.headPoint.x, this.tailPoint.x) - Math.min(this.headPoint.x, this.tailPoint.x);\n        // this.height = Math.max(this.headPoint.y, this.tailPoint.y) - Math.min(this.headPoint.y, this.tailPoint.y);\n\n        // Reassign distance of this.nameLabel, this.stereotypeLabel\n        if (!v && this.propertyLabel.visible) {\n            if (this.nameLabel.distance < 25) {\n                if (this.stereotypeLabel.distance <= this.nameLabel.distance + 15) {\n                    this.stereotypeLabel.distance = this.stereotypeLabel.distance + 15;\n                }\n                this.nameLabel.distance = this.nameLabel.distance + 15;\n            }\n        }\n        /*\n        // Arrange this.nameLabel\n        var p1 = this.tailPoint;\n        var p2 = this.headPoint;\n        var p = Coord.getPointAwayLine(p1, p2, this.nameLabel.alpha, this.nameLabel.distance);\n        this.nameLabel.left = (p.x + p1.x) - (this.nameLabel.width / 2);\n        this.nameLabel.top = (p.y + p1.y) - (this.nameLabel.height / 2);\n        // Arrange this.stereotypeLabel\n        p = Coord.getPointAwayLine(p1, p2, this.stereotypeLabel.alpha, this.stereotypeLabel.distance);\n        this.stereotypeLabel.left = (p.x + p1.x) - (this.stereotypeLabel.width / 2);\n        this.stereotypeLabel.top = (p.y + p1.y) - (this.stereotypeLabel.height / 2);\n        // Arrange this.propertyLabel\n        p = Coord.getPointAwayLine(p1, p2, this.propertyLabel.alpha, this.propertyLabel.distance);\n        this.propertyLabel.left = (p.x + p1.x) - (this.propertyLabel.width / 2);\n        this.propertyLabel.top = (p.y + p1.y) - (this.propertyLabel.height / 2);\n        */\n        // call Update here because Action\'s changed are not reflected\n        this.update();\n    };\n\n    UMLCommMessageView.prototype.drawObject = function (canvas) {\n        EdgeNodeView.prototype.drawObject.call(this, canvas);\n        // message body\n        if (this.model.messageSort == UML.MS_REPLY) {\n            canvas.line(this.headPoint.x, this.headPoint.y, this.tailPoint.x, this.tailPoint.y, [3]);\n        } else {\n            canvas.line(this.headPoint.x, this.headPoint.y, this.tailPoint.x, this.tailPoint.y);\n        }\n        // message head\n        if ((this.model.messageSort === UML.MS_ASYNCHCALL) ||\n            (this.model.messageSort === UML.MS_ASYNCHSIGNAL) ||\n            (this.model.messageSort === UML.MS_CREATEMESSAGE) ||\n            (this.model.messageSort === UML.MS_REPLY)) {\n            canvas.polyline([this.arrowPoint1, this.headPoint, this.arrowPoint2]);\n        } else {\n            canvas.fillColor = this.lineColor;\n            canvas.fillPolygon([this.arrowPoint1, this.headPoint, this.arrowPoint2]);\n            canvas.fillColor = this.fillColor;\n        }\n    };\n\n    UMLCommMessageView.prototype.drawSelection = function (canvas) {\n        Toolkit.drawHighlighter(canvas, this.tailPoint.x, this.tailPoint.y, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, true, Toolkit.HIGHLIGHTER_COLOR);\n        Toolkit.drawHighlighter(canvas, this.headPoint.x, this.headPoint.y, Toolkit.DEFAULT_HALF_HIGHLIGHTER_SIZE, true, Toolkit.HIGHLIGHTER_COLOR);\n    };\n\n\n    /**\n     * Cannot be copied to clipboard.\n     */\n    UMLCommMessageView.prototype.canCopy = function () {\n        return false;\n    };\n\n    /**\n     * Cannnot be deleted view only.\n     */\n    UMLCommMessageView.prototype.canDelete = function () {\n        return false;\n    };\n\n    /**************************************************************************\n     *                                                                        *\n     *                          PROFILE DIAGRAM VIEWS                         *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * UMLProfileDiagram\n     * @constructor\n     * @extends UMLDiagram\n     */\n    function UMLProfileDiagram() {\n        UMLDiagram.apply(this, arguments);\n    }\n    // inherits from UMLDiagram\n    UMLProfileDiagram.prototype = Object.create(UMLDiagram.prototype);\n    UMLProfileDiagram.prototype.constructor = UMLProfileDiagram;\n\n    UMLProfileDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.UMLConstraint) ||\n               (model instanceof type.UMLMetaClass) ||\n               (model instanceof type.UMLStereotype) ||\n               (model instanceof type.UMLEnumeration) ||\n               (model instanceof type.UMLGeneralization) ||\n               (model instanceof type.UMLDependency) ||\n               (model instanceof type.UMLInterfaceRealization) ||\n               (model instanceof type.UMLComponentRealization) ||\n               (model instanceof type.UMLAssociation);\n    };\n\n\n    /**\n     * UMLProfileView\n     * @constructor\n     * @extends UMLPackageView\n     */\n    function UMLProfileView() {\n        UMLPackageView.apply(this, arguments);\n    }\n    // inherits from UMLPackageView\n    UMLProfileView.prototype = Object.create(UMLPackageView.prototype);\n    UMLProfileView.prototype.constructor = UMLProfileView;\n\n    UMLProfileView.prototype.getStereotypeLabelText = function () {\n        return "«profile»";\n    };\n\n\n    /**\n     * UMLMetaClassView\n     * @constructor\n     * @extends UMLGeneralNodeView\n     */\n    function UMLMetaClassView() {\n        UMLGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.metaclass.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLGeneralNodeView\n    UMLMetaClassView.prototype = Object.create(UMLGeneralNodeView.prototype);\n    UMLMetaClassView.prototype.constructor = UMLMetaClassView;\n\n    UMLMetaClassView.prototype.getStereotypeLabelText = function () {\n        return "«metaClass»";\n    };\n\n    /**\n     * UMLStereotypeView\n     * @constructor\n     * @extends UMLClassView\n     */\n    function UMLStereotypeView() {\n        UMLClassView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.stereotype.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from UMLClassView\n    UMLStereotypeView.prototype = Object.create(UMLClassView.prototype);\n    UMLStereotypeView.prototype.constructor = UMLStereotypeView;\n\n    UMLStereotypeView.prototype.getStereotypeLabelText = function () {\n        return "«stereotype»";\n    };\n\n\n    /**\n     * UMLExtensionView\n     * @constructor\n     * @extends UMLGeneralEdgeView\n     */\n    function UMLExtensionView() {\n        UMLGeneralEdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_SOLID_ARROW;\n        this.lineMode = Core.LM_SOLID;\n    }\n    // inherits from UMLGeneralEdgeView\n    UMLExtensionView.prototype = Object.create(UMLGeneralEdgeView.prototype);\n    UMLExtensionView.prototype.constructor = UMLExtensionView;\n\n    UMLExtensionView.prototype.canConnectTo = function (view, isTail) {\n        return (isTail && view.model instanceof type.UMLStereotype) ||\n               (!isTail && view.model instanceof type.UMLMetaClass);\n    };\n\n\n    /**************************************************************************\n     *                                                                        *\n     *                            ANNOTATION VIEWS                            *\n     *                                                                        *\n     **************************************************************************/\n\n    /**\n     * HyperlinkView\n     * @constructor\n     * @extends NodeView\n     */\n    function HyperlinkView() {\n        NodeView.apply(this, arguments);\n\n        /** @member {LabelView} */\n        this.nameLabel = new LabelView();\n        this.nameLabel.parentStyle = true;\n        this.addSubView(this.nameLabel);\n\n        /** @member {LabelView} */\n        this.typeLabel = new LabelView();\n        this.typeLabel.parentStyle = true;\n        this.addSubView(this.typeLabel);\n    }\n    // inherits from NodeView\n    HyperlinkView.prototype = Object.create(NodeView.prototype);\n    HyperlinkView.prototype.constructor = HyperlinkView;\n\n    HyperlinkView.prototype.sizeObject = function (canvas) {\n        NodeView.prototype.sizeObject.call(this, canvas);\n        var h = this.typeLabel.minHeight + COMPARTMENT_TOP_PADDING + COMPARTMENT_BOTTOM_PADDING;\n        this.minHeight = Math.max(h, HYPERLINK_MINHEIGHT);\n        var w = this.typeLabel.width + this.nameLabel.width + (COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING) * 2;\n        this.minWidth = Math.max(w, HYPERLINK_MINWIDTH);\n        this.sizeConstraints();\n    };\n\n    HyperlinkView.prototype.arrangeObject = function (canvas) {\n        NodeView.prototype.arrangeObject.call(this, canvas);\n        this.typeLabel.left = this.left + COMPARTMENT_LEFT_PADDING;\n        this.typeLabel.top = this.top + COMPARTMENT_TOP_PADDING;\n        this.typeLabel.setRight(this.typeLabel.left + this.typeLabel.minWidth);\n        this.nameLabel.top = this.top + COMPARTMENT_TOP_PADDING;\n        this.nameLabel.left = this.typeLabel.getRight() + COMPARTMENT_RIGHT_PADDING + COMPARTMENT_LEFT_PADDING;\n    };\n\n    HyperlinkView.prototype.drawObject = function (canvas) {\n        NodeView.prototype.drawObject.call(this, canvas);\n        var x = this.typeLabel.getRight() + COMPARTMENT_RIGHT_PADDING;\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.line(x, this.top, x, this.getBottom());\n    };\n\n    HyperlinkView.prototype.update = function (canvas) {\n        NodeView.prototype.update.call(this, canvas);\n        this.typeLabel.font.style = Graphics.FS_BOLD;\n        this.typeLabel.text = "link";\n        if (this.model && this.model.reference instanceof type.Model) {\n            this.nameLabel.text = this.model.reference.name;\n        } else {\n            this.nameLabel.text = this.model.url;\n        }\n    };\n\n    /**\n     * UMLCustomTextView\n     * @constructor\n     * @extends NodeView\n     */\n    function UMLCustomTextView() {\n        NodeView.apply(this, arguments);\n\n        /** @member {string} */\n        this.text = \'\';\n\n        /** @member {boolean} */\n        this.wordWrap = true;\n\n        /* transient */\n        this._rightPadding = 0;\n    }\n    // inherits from NodeView\n    UMLCustomTextView.prototype = Object.create(NodeView.prototype);\n    UMLCustomTextView.prototype.constructor = UMLCustomTextView;\n\n    UMLCustomTextView.prototype.sizeObject = function (canvas) {\n        var size, marg, minW, minH, w, h;\n        var lines = null;\n        if (this.text && this.text.length > 0) {\n            lines = this.text.split("\\n");\n        }\n        w = 0;\n        h = 0;\n        if (lines !== null && lines.length > 0) {\n            for (var i = 0, len = lines.length; i < len; i++) {\n                if (this.wordWrap) {\n                    marg = COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING + this._rightPadding;\n                    minW = canvas.textExtent(lines[i], 1).x;\n                    minH = canvas.textExtent(lines[i], this.width - marg).y;\n                    w = Math.max(w, minW);\n                    h = h + minH + 2;\n                } else {\n                    var sz = canvas.textExtent(lines[i]);\n                    w = Math.max(w, sz.x);\n                    h = h + canvas.textExtent(\'^_\').y + 2;\n                }\n            }\n        }\n        w += COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING + this._rightPadding;\n        h += COMPARTMENT_TOP_PADDING + COMPARTMENT_BOTTOM_PADDING;\n        this.minWidth = Math.max(CUSTOM_TEXT_MINWIDTH, w);\n        this.minHeight = Math.max(CUSTOM_TEXT_MINHEIGHT, h);\n        NodeView.prototype.sizeObject.call(this, canvas);\n    };\n\n    UMLCustomTextView.prototype.drawObject = function (canvas) {\n        NodeView.prototype.drawObject.call(this, canvas);\n        if (this.text && this.text.length > 0) {\n            var lines = this.text.split("\\n");\n            var x1 = this.left + COMPARTMENT_LEFT_PADDING,\n                x2 = this.getRight() - COMPARTMENT_RIGHT_PADDING,\n                y = this.top + COMPARTMENT_TOP_PADDING;\n            for (var i = 0, len = lines.length; i < len; i++) {\n                var sz = canvas.textExtent(lines[i], this.width - 1);\n                var r = new Graphics.Rect(x1, y, x2, y + sz.y + 2);\n                canvas.textOut2(r, lines[i], Graphics.AL_LEFT, Graphics.AL_TOP, false, this.wordWrap);\n                y = y + sz.y + 2;\n            }\n        }\n    };\n\n\n    /**\n     * UMLTextView\n     * @constructor\n     * @extends UMLCustomTextView\n     */\n    function UMLTextView() {\n        UMLCustomTextView.apply(this, arguments);\n    }\n    // inherits from UMLCustomTextView\n    UMLTextView.prototype = Object.create(UMLCustomTextView.prototype);\n    UMLTextView.prototype.constructor = UMLTextView;\n\n\n    /**\n     * UMLCustomNoteView\n     * @constructor\n     * @extends UMLCustomTextView\n     */\n    function UMLCustomNoteView() {\n        UMLCustomTextView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("uml.note.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n        this._rightPadding = NOTE_FOLDING_SIZE;\n    }\n    // inherits from UMLCustomTextView\n    UMLCustomNoteView.prototype = Object.create(UMLCustomTextView.prototype);\n    UMLCustomNoteView.prototype.constructor = UMLCustomNoteView;\n\n    UMLCustomNoteView.prototype.drawObject = function (canvas) {\n        var r = this.getRight() - 1, b = this.getBottom() - 1;\n        var pts = [new Point(this.left, this.top), new Point(r - NOTE_FOLDING_SIZE, this.top),\n                   new Point(r, this.top + NOTE_FOLDING_SIZE),\n                   new Point(r, b),\n                   new Point(this.left, b),\n                   new Point(this.left, this.top)];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        canvas.polygon([new Point(r - NOTE_FOLDING_SIZE, this.top),\n                        new Point(r - NOTE_FOLDING_SIZE, this.top + NOTE_FOLDING_SIZE),\n                        new Point(r, this.top + NOTE_FOLDING_SIZE)]);\n        UMLCustomTextView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * UMLNoteView\n     * @constructor\n     * @extends UMLCustomNoteView\n     */\n    function UMLNoteView() {\n        UMLCustomNoteView.apply(this, arguments);\n    }\n    // inherits from UMLCustomNoteView\n    UMLNoteView.prototype = Object.create(UMLCustomNoteView.prototype);\n    UMLNoteView.prototype.constructor = UMLNoteView;\n\n\n    /**\n     * UMLConstraintView\n     * @constructor\n     * @extends UMLCustomNoteView\n     */\n    function UMLConstraintView() {\n        UMLCustomNoteView.apply(this, arguments);\n    }\n    // inherits from UMLCustomNoteView\n    UMLConstraintView.prototype = Object.create(UMLCustomNoteView.prototype);\n    UMLConstraintView.prototype.constructor = UMLConstraintView;\n\n    UMLConstraintView.prototype.update = function (canvas) {\n        if (typeof this.model.specification === "string") {\n            this.text = "{" + this.model.specification + "}";\n            var parentField = this.model.getParentField();\n            switch (parentField) {\n            case "preconditions":       this.text = "«precondition»\\n" + this.text;       break;\n            case "postconditions":      this.text = "«postcondition»\\n" + this.text;      break;\n            case "bodyConditions":      this.text = "«bodyCondition»\\n" + this.text;      break;\n            case "localPreconditions":  this.text = "«localPrecondition»\\n" + this.text;  break;\n            case "localPostconditions": this.text = "«localPostcondition»\\n" + this.text; break;\n            }\n        }\n    };\n\n\n    /**\n     * UMLNoteLinkView\n     * @constructor\n     * @extends EdgeView\n     */\n    function UMLNoteLinkView() {\n        EdgeView.apply(this, arguments);\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from EdgeView\n    UMLNoteLinkView.prototype = Object.create(EdgeView.prototype);\n    UMLNoteLinkView.prototype.constructor = UMLNoteLinkView;\n\n\n    /**\n     * UMLConstraintLinkView\n     * @constructor\n     * @extends EdgeView\n     */\n    function UMLConstraintLinkView() {\n        EdgeView.apply(this, arguments);\n        this.lineMode = Core.LM_DOT;\n    }\n    // inherits from EdgeView\n    UMLConstraintLinkView.prototype = Object.create(EdgeView.prototype);\n    UMLConstraintLinkView.prototype.constructor = UMLConstraintLinkView;\n\n\n    /**\n     * ShapeView\n     * @constructor\n     * @extends NodeView\n     */\n    function ShapeView() {\n        NodeView.apply(this, arguments);\n    }\n    // inherits from NodeView\n    ShapeView.prototype = Object.create(NodeView.prototype);\n    ShapeView.prototype.constructor = ShapeView;\n\n\n    /**\n     * RectangleView\n     * @constructor\n     * @extends ShapeView\n     */\n    function RectangleView() {\n        ShapeView.apply(this, arguments);\n    }\n    // inherits from ShapeView\n    RectangleView.prototype = Object.create(ShapeView.prototype);\n    RectangleView.prototype.constructor = RectangleView;\n\n    RectangleView.prototype.drawObject = function (canvas) {\n        ShapeView.prototype.drawObject.call(this, canvas);\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n\n    /**\n     * RoundRectView\n     * @constructor\n     * @extends ShapeView\n     */\n    function RoundRectView() {\n        ShapeView.apply(this, arguments);\n    }\n    // inherits from ShapeView\n    RoundRectView.prototype = Object.create(ShapeView.prototype);\n    RoundRectView.prototype.constructor = RoundRectView;\n\n    RoundRectView.prototype.drawObject = function (canvas) {\n        ShapeView.prototype.drawObject.call(this, canvas);\n        var r = Math.max(this.width, this.height);\n        canvas.fillRoundRect(this.left, this.top, this.getRight(), this.getBottom(), r / 6);\n        canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), r / 6);\n    };\n\n\n    /**\n     * EllipseView\n     * @constructor\n     * @extends ShapeView\n     */\n    function EllipseView() {\n        ShapeView.apply(this, arguments);\n    }\n    // inherits from ShapeView\n    EllipseView.prototype = Object.create(ShapeView.prototype);\n    EllipseView.prototype.constructor = EllipseView;\n\n    EllipseView.prototype.drawObject = function (canvas) {\n        ShapeView.prototype.drawObject.call(this, canvas);\n        canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n    };\n\n    /* -------------------------------------------------------------------------------------------- */\n\n    // Type definitions\n\n    // Common Views\n    type.UMLDiagram                           = UMLDiagram;\n    type.UMLCompartmentView                   = UMLCompartmentView;\n    type.UMLNameCompartmentView               = UMLNameCompartmentView;\n    type.UMLAttributeView                     = UMLAttributeView;\n    type.UMLAttributeCompartmentView          = UMLAttributeCompartmentView;\n    type.UMLOperationView                     = UMLOperationView;\n    type.UMLOperationCompartmentView          = UMLOperationCompartmentView;\n    type.UMLReceptionView                     = UMLReceptionView;\n    type.UMLReceptionCompartmentView          = UMLReceptionCompartmentView;\n    type.UMLTemplateParameterView             = UMLTemplateParameterView;\n    type.UMLTemplateParameterCompartmentView  = UMLTemplateParameterCompartmentView;\n    type.UMLGeneralNodeView                   = UMLGeneralNodeView;\n    type.UMLFloatingNodeView                  = UMLFloatingNodeView;\n    type.UMLGeneralEdgeView                   = UMLGeneralEdgeView;\n    type.UMLClassifierView                    = UMLClassifierView;\n    type.UMLUndirectedRelationshipView        = UMLUndirectedRelationshipView;\n    // Class Diagram Views\n    type.UMLClassDiagram                      = UMLClassDiagram;\n    type.UMLClassView                         = UMLClassView;\n    type.UMLInterfaceView                     = UMLInterfaceView;\n    type.UMLSignalView                        = UMLSignalView;\n    type.UMLDataTypeView                      = UMLDataTypeView;\n    type.UMLPrimitiveTypeView                 = UMLPrimitiveTypeView;\n    type.UMLEnumerationLiteralView            = UMLEnumerationLiteralView;\n    type.UMLEnumerationLiteralCompartmentView = UMLEnumerationLiteralCompartmentView;\n    type.UMLEnumerationView                   = UMLEnumerationView;\n    type.UMLGeneralizationView                = UMLGeneralizationView;\n    type.UMLDependencyView                    = UMLDependencyView;\n    type.UMLRealizationView                   = UMLRealizationView;\n    type.UMLInterfaceRealizationView          = UMLInterfaceRealizationView;\n    type.UMLQualifierCompartmentView          = UMLQualifierCompartmentView;\n    type.UMLAssociationView                   = UMLAssociationView;\n    type.UMLAssociationClassLinkView          = UMLAssociationClassLinkView;\n    // Package Diagram Views\n    type.UMLPackageDiagram                    = UMLPackageDiagram;\n    type.UMLPackageView                       = UMLPackageView;\n    type.UMLModelView                         = UMLModelView;\n    type.UMLSubsystemView                     = UMLSubsystemView;\n    type.UMLContainmentView                   = UMLContainmentView;\n    // Composite Structure Diagram Views\n    type.UMLCompositeStructureDiagram         = UMLCompositeStructureDiagram;\n    type.UMLPortView                          = UMLPortView;\n    type.UMLPartView                          = UMLPartView;\n    type.UMLConnectorView                     = UMLConnectorView;\n    type.UMLCollaborationView                 = UMLCollaborationView;\n    type.UMLCollaborationUseView              = UMLCollaborationUseView;\n    type.UMLRoleBindingView                   = UMLRoleBindingView;\n    // Object Diagram Views\n    type.UMLObjectDiagram                     = UMLObjectDiagram;\n    type.UMLSlotView                          = UMLSlotView;\n    type.UMLSlotCompartmentView               = UMLSlotCompartmentView;\n    type.UMLObjectView                        = UMLObjectView;\n    type.UMLLinkView                          = UMLLinkView;\n    // Component Diagram Views\n    type.UMLComponentDiagram                  = UMLComponentDiagram;\n    type.UMLArtifactView                      = UMLArtifactView;\n    type.UMLArtifactInstanceView              = UMLArtifactInstanceView;\n    type.UMLComponentView                     = UMLComponentView;\n    type.UMLComponentInstanceView             = UMLComponentInstanceView;\n    type.UMLComponentRealizationView          = UMLComponentRealizationView;\n    // Deployment Diagram Views\n    type.UMLDeploymentDiagram                 = UMLDeploymentDiagram;\n    type.UMLNodeView                          = UMLNodeView;\n    type.UMLNodeInstanceView                  = UMLNodeInstanceView;\n    type.UMLDeploymentView                    = UMLDeploymentView;\n    type.UMLCommunicationPathView             = UMLCommunicationPathView;\n    // Use Case Diagram Views\n    type.UMLUseCaseDiagram                    = UMLUseCaseDiagram;\n    type.UMLExtensionPointView                = UMLExtensionPointView;\n    type.UMLExtensionPointCompartmentView     = UMLExtensionPointCompartmentView;\n    type.UMLUseCaseView                       = UMLUseCaseView;\n    type.UMLActorView                         = UMLActorView;\n    type.UMLIncludeView                       = UMLIncludeView;\n    type.UMLExtendView                        = UMLExtendView;\n    type.UMLUseCaseSubjectView                = UMLUseCaseSubjectView;\n    // Statechart Diagram Views\n    type.UMLStatechartDiagram                 = UMLStatechartDiagram;\n    type.UMLPseudostateView                   = UMLPseudostateView;\n    type.UMLFinalStateView                    = UMLFinalStateView;\n    type.UMLConnectionPointReferenceView      = UMLConnectionPointReferenceView;\n    type.UMLInternalActivityView              = UMLInternalActivityView;\n    type.UMLInternalActivityCompartmentView   = UMLInternalActivityCompartmentView;\n    type.UMLInternalTransitionView            = UMLInternalTransitionView;\n    type.UMLInternalTransitionCompartmentView = UMLInternalTransitionCompartmentView;\n    type.UMLRegionView                        = UMLRegionView;\n    type.UMLDecompositionCompartmentView      = UMLDecompositionCompartmentView;\n    type.UMLStateView                         = UMLStateView;\n    type.UMLTransitionView                    = UMLTransitionView;\n    // Activity Diagram Views\n    type.UMLActivityDiagram                   = UMLActivityDiagram;\n    type.UMLPinView                           = UMLPinView;\n    type.UMLInputPinView                      = UMLInputPinView;\n    type.UMLOutputPinView                     = UMLOutputPinView;\n    type.UMLExpansionNodeView                 = UMLExpansionNodeView;\n    type.UMLActionView                        = UMLActionView;\n    type.UMLObjectNodeView                    = UMLObjectNodeView;\n    type.UMLCentralBufferNodeView             = UMLCentralBufferNodeView;\n    type.UMLDataStoreNodeView                 = UMLDataStoreNodeView;\n    type.UMLControlNodeView                   = UMLControlNodeView;\n    type.UMLControlFlowView                   = UMLControlFlowView;\n    type.UMLObjectFlowView                    = UMLObjectFlowView;\n    type.UMLZigZagAdornmentView               = UMLZigZagAdornmentView;\n    type.UMLExceptionHandlerView              = UMLExceptionHandlerView;\n    type.UMLActivityInterruptView             = UMLActivityInterruptView;\n    type.UMLSwimlaneView                      = UMLSwimlaneView;\n    type.UMLInterruptibleActivityRegionView   = UMLInterruptibleActivityRegionView;\n    type.UMLStructuredActivityNodeView        = UMLStructuredActivityNodeView;\n    type.UMLExpansionRegionView               = UMLExpansionRegionView;\n    // Sequence Diagram Views\n    type.UMLSequenceDiagram                   = UMLSequenceDiagram;\n    type.UMLLinePartView                      = UMLLinePartView;\n    type.UMLSeqLifelineView                   = UMLSeqLifelineView;\n    type.UMLMessageEndpointView               = UMLMessageEndpointView;\n    type.UMLEndpointView                      = UMLEndpointView;\n    type.UMLGateView                          = UMLGateView;\n    type.UMLActivationView                    = UMLActivationView;\n    type.UMLSeqMessageView                    = UMLSeqMessageView;\n    type.UMLStateInvariantView                = UMLStateInvariantView;\n    type.UMLContinuationView                  = UMLContinuationView;\n    type.UMLCustomFrameView                   = UMLCustomFrameView;\n    type.UMLFrameView                         = UMLFrameView;\n    type.UMLInteractionOperandView            = UMLInteractionOperandView;\n    type.UMLInteractionOperandCompartmentView = UMLInteractionOperandCompartmentView;\n    type.UMLCombinedFragmentView              = UMLCombinedFragmentView;\n    type.UMLInteractionUseView                = UMLInteractionUseView;\n    // Communication Diagram Views\n    type.UMLCommunicationDiagram              = UMLCommunicationDiagram;\n    type.UMLCommLifelineView                  = UMLCommLifelineView;\n    type.UMLCommMessageView                   = UMLCommMessageView;\n    // Profile Diagram Views\n    type.UMLProfileDiagram                    = UMLProfileDiagram;\n    type.UMLProfileView                       = UMLProfileView;\n    type.UMLMetaClassView                     = UMLMetaClassView;\n    type.UMLStereotypeView                    = UMLStereotypeView;\n    type.UMLExtensionView                     = UMLExtensionView;\n    // Annotation Views\n    type.HyperlinkView                        = HyperlinkView;\n    type.UMLCustomTextView                    = UMLCustomTextView;\n    type.UMLTextView                          = UMLTextView;\n    type.UMLCustomNoteView                    = UMLCustomNoteView;\n    type.UMLNoteView                          = UMLNoteView;\n    type.UMLConstraintView                    = UMLConstraintView;\n    type.UMLNoteLinkView                      = UMLNoteLinkView;\n    type.UMLConstraintLinkView                = UMLConstraintLinkView;\n    type.ShapeView                            = ShapeView;\n    type.RectangleView                        = RectangleView;\n    type.RoundRectView                        = RoundRectView;\n    type.EllipseView                          = EllipseView;\n\n});\n\n//# sourceURL=/uml/UMLDiagram.js'),eval('define(\'text!grammars/uml-name.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * General Expression Grammar\\n * e.g.) << stereotype >> name\\n */\\n\\nstart\\n    = stereotype:stereotype? _ visibility:visibility? _ name:identifier _\\n    {\\n        var ast = { name: name };\\n        if (stereotype) { ast.stereotype = stereotype; }\\n        if (visibility) { ast.visibility = visibility; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\nvisibility\\n    = "+" { return \\\'public\\\'; }\\n    / "#" { return \\\'protected\\\'; }\\n    / "-" { return \\\'private\\\'; }\\n    / "~" { return \\\'package\\\'; }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?\\\\[\\\\]\\\'"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/uml-name.pegjs'),eval('define(\'text!grammars/uml-attribute.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * Attribute Expression Grammar\\n * e.g.) << stereotype >> name : type [ multiplicity ] = defaultValue\\n */\\n\\nstart\\n    = stereotype:stereotype? _ visibility:visibility? _ name:identifier _ type:type? _ multi:multiplicity? _ value:defaultValue?\\n    {\\n        var ast = { name: name };\\n        if (stereotype) { ast.stereotype = stereotype; }\\n        if (visibility) { ast.visibility = visibility; }\\n        if (type) { ast.type = type; }\\n        if (multi) { ast.multiplicity = multi; }\\n        if (value) { ast.defaultValue = value; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\nvisibility\\n    = "+" { return \\\'public\\\'; }\\n    / "#" { return \\\'protected\\\'; }\\n    / "-" { return \\\'private\\\'; }\\n    / "~" { return \\\'package\\\'; }\\n\\ntype\\n    = ":" _ type:typeExpression { return type; }\\n\\ndefaultValue\\n    = "=" _ value:identifier { return value; }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?\\\\[\\\\]"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\ntypeExpression\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{}<>;.?"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\nmultiplicity\\n    = "[" _ multi:multiplicityExpression? _ "]"\\n    {\\n        if (multi) { return multi; }\\n    }\\n\\nmultiplicityExpression\\n    = id:[0-9.\\\\*]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/uml-attribute.pegjs'),eval('define(\'text!grammars/uml-operation.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * Operation Expression Grammar\\n * e.g.) << stereotype >> +name (in param:type, out param:type): returnType\\n */\\n\\nstart\\n    = stereotype:stereotype? _ visibility:visibility? _ name:identifier _ parameters:parameters? _ returnType:type? _ multi:multiplicity?\\n    {\\n        var ast = { name: name };\\n        if (stereotype) { ast.stereotype = stereotype; }\\n        if (visibility) { ast.visibility = visibility; }\\n        if (parameters) { ast.parameters = parameters; }\\n        if (returnType) { ast.returnType = returnType; }\\n        if (multi)      { ast.returnMultiplicity = multi; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\nvisibility\\n    = "+" { return \\\'public\\\'; }\\n    / "#" { return \\\'protected\\\'; }\\n    / "-" { return \\\'private\\\'; }\\n    / "~" { return \\\'package\\\'; }\\n\\nparameters\\n    = "(" _ params:paramList? _ ")" { return params; }\\n\\nparamList\\n    = param:param _ "," _ params:paramList\\n    {\\n        var list = [ param ];\\n        if (params && params.length > 0) {\\n            for(var i = 0; i < params.length; i++) {\\n                list.push(params[i]);\\n            }\\n        }\\n        return list;\\n    }\\n    / param:param\\n    {\\n        return [ param ];\\n    }\\n\\nparam\\n    = direction:direction? _ name:identifier _ type:type? _ multi:multiplicity? _ defaultValue:defaultValue?\\n    {\\n        var ast = { name: name };\\n        if (direction) { ast.direction = direction; }\\n        if (type) { ast.type = type; }\\n        if (multi) { ast.multiplicity = multi; }\\n        if (defaultValue) { ast.defaultValue = defaultValue; }\\n        return ast;\\n    }\\n\\ndirection\\n    = "in" __    { return "in"; }\\n    / "out" __   { return "out"; }\\n    / "inout" __ { return "inout"; }\\n\\ntype\\n    = ":" _ type:typeExpression { return type; }\\n\\ndefaultValue\\n    = "=" _ value:identifier { return value; }\\n\\ntypeExpression\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{}<>;.?"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?\\\\[\\\\]"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\nmultiplicity\\n    = "[" _ multi:multiplicityExpression? _ "]"\\n    {\\n        if (multi) { return multi; }\\n    }\\n\\nmultiplicityExpression\\n    = id:[0-9.\\\\*]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\\n__\\n    = [ \\\\t\\\\n\\\\r]+\\n\';});\n\n//# sourceURL=/text!grammars/uml-operation.pegjs'),eval('define(\'text!grammars/uml-templateparameter.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * TemplateParameter Expression Grammar\\n * e.g.) << stereotype >> name : type = defaultValue\\n */\\n\\nstart\\n    = stereotype:stereotype? _ name:identifier _ type:type? _ value:defaultValue?\\n    {\\n        var ast = { name: name };\\n        if (stereotype) { ast.stereotype = stereotype; }\\n        if (type) { ast.parameterType = type; }\\n        if (value) { ast.defaultValue = value; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\ntype\\n    = ":" _ type:typeExpression { return type; }\\n\\ndefaultValue\\n    = "=" _ value:identifier { return value; }\\n\\ntypeExpression\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{}<>;.?\\\\[\\\\]"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?\\\\[\\\\]"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/uml-templateparameter.pegjs'),eval('define(\'text!grammars/uml-object.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * Object Expression Grammar\\n * e.g.) << stereotype >> name : type\\n */\\n\\nstart\\n    = stereotype:stereotype? _ visibility:visibility? _ name:identifier _ type:type?\\n    {\\n        var ast = { name: name };\\n        if (stereotype) { ast.stereotype = stereotype; }\\n        if (visibility) { ast.visibility = visibility; }\\n        if (type) { ast.type = type; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\nvisibility\\n    = "+" { return \\\'public\\\'; }\\n    / "#" { return \\\'protected\\\'; }\\n    / "-" { return \\\'private\\\'; }\\n    / "~" { return \\\'package\\\'; }\\n\\ntype\\n    = ":" _ type:typeExpression { return type; }\\n\\ntypeExpression\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{}<>;.?\\\\[\\\\]\\\'"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?\\\\[\\\\]\\\'"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/uml-object.pegjs'),eval('define(\'text!grammars/uml-slot.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * Slot Expression Grammar\\n * e.g.) << stereotype >> name : type = value\\n */\\n\\nstart\\n    = stereotype:stereotype? _ visibility:visibility? _ name:identifier _ type:type? _ value:valueExpression?\\n    {\\n        var ast = { name: name };\\n        if (stereotype) { ast.stereotype = stereotype; }\\n        if (visibility) { ast.visibility = visibility; }\\n        if (type) { ast.type = type; }\\n        if (value) { ast.value = value; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\nvisibility\\n    = "+" { return \\\'public\\\'; }\\n    / "#" { return \\\'protected\\\'; }\\n    / "-" { return \\\'private\\\'; }\\n    / "~" { return \\\'package\\\'; }\\n\\ntype\\n    = ":" _ type:typeExpression { return type; }\\n\\nvalueExpression\\n    = "=" _ value:identifier { return value; }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?\\\\[\\\\]\\\'"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\ntypeExpression\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{}<>;.?\\\\[\\\\]\\\'"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/uml-slot.pegjs'),eval('define(\'text!grammars/uml-lifeline.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * Lifeline Expression Grammar\\n * e.g.) << stereotype >> name [selector] : type\\n */\\n\\nstart\\n    = stereotype:stereotype? _ visibility:visibility? _ name:identifier _ selector:selector? _ type:type?\\n    {\\n        var ast = { name: name };\\n        if (stereotype) { ast.stereotype = stereotype; }\\n        if (visibility) { ast.visibility = visibility; }\\n        if (selector) { ast.selector = selector; }\\n        if (type) { ast.type = type; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\nvisibility\\n    = "+" { return \\\'public\\\'; }\\n    / "#" { return \\\'protected\\\'; }\\n    / "-" { return \\\'private\\\'; }\\n    / "~" { return \\\'package\\\'; }\\n\\nselector\\n    = "[" _ sel:identifier _ "]" { return sel; }\\n\\ntype\\n    = ":" _ type:typeExpression { return type; }\\n\\ntypeExpression\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{}<>;.?\\\\[\\\\]\\\'"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?\\\'"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/uml-lifeline.pegjs'),eval('define(\'text!grammars/uml-message.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * Message Expression Grammar\\n * e.g.) << stereotype >> assignmentTarget = name (arguments)\\n */\\n\\nstart\\n    = stereotype:stereotype? _ assignmentTarget:assignmentTarget? _ name:identifier _ arguments:arguments?\\n    {\\n        var ast = { name: name };\\n        if (stereotype) { ast.stereotype = stereotype; }\\n        if (assignmentTarget) { ast.assignmentTarget = assignmentTarget; }\\n        if (arguments) { ast.arguments = arguments; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\nassignmentTarget\\n    = target:identifier _ "=" { return target; }\\n\\narguments\\n    = "(" _ arg:identifier _ ")" { return arg; }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};,.?\\\\[\\\\]"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/uml-message.pegjs'),eval('define(\'text!grammars/uml-transition.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * Transition Expression Grammar\\n * e.g.) triggers guard effect\\n */\\n\\nstart\\n    = triggers:triggers? _ guard:guard? _ effect:effect?\\n    {\\n        var ast = {};\\n        if (triggers) { ast.triggers = triggers; }\\n        if (guard) { ast.guard = guard; }\\n        if (effect) { ast.effect = effect; }\\n        return ast;\\n    }\\n\\nstereotype\\n    = "<<" _ id:identifier _ ">>" { return id; }\\n\\ntriggers\\n    = event:identifier _ "," _ events:triggers\\n    {\\n        var list = [ event ];\\n        if (events && events.length > 0) {\\n            for(var i = 0; i < events.length; i++) {\\n                list.push(events[i]);\\n            }\\n        }\\n        return list;\\n    }\\n    / event:identifier\\n    {\\n        return [ event ];\\n    }\\n\\nguard\\n    = "[" _ guard:identifier _ "]" { return guard; }\\n\\neffect\\n    = "/" _ effect:identifier { return effect; }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/uml-transition.pegjs'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, type, PEG */\n\ndefine('uml/UMLUtils',['require','exports','module','core/Core','core/Repository','core/OperationBuilder','engine/Engine','uml/UML','text!grammars/uml-name.pegjs','text!grammars/uml-attribute.pegjs','text!grammars/uml-operation.pegjs','text!grammars/uml-templateparameter.pegjs','text!grammars/uml-object.pegjs','text!grammars/uml-slot.pegjs','text!grammars/uml-lifeline.pegjs','text!grammars/uml-message.pegjs','text!grammars/uml-transition.pegjs'],function (require, exports, module) {\n    \"use strict\";\n\n    var Core                     = require(\"core/Core\"),\n        Repository               = require(\"core/Repository\"),\n        OperationBuilder         = require(\"core/OperationBuilder\"),\n        Engine                   = require(\"engine/Engine\"),\n        UML                      = require(\"uml/UML\");\n\n    var GrammarName              = require(\"text!grammars/uml-name.pegjs\"),\n        GrammarAttribute         = require(\"text!grammars/uml-attribute.pegjs\"),\n        GrammarOperation         = require(\"text!grammars/uml-operation.pegjs\"),\n        GrammarTemplateParameter = require(\"text!grammars/uml-templateparameter.pegjs\"),\n        GrammarObject            = require(\"text!grammars/uml-object.pegjs\"),\n        GrammarSlot              = require(\"text!grammars/uml-slot.pegjs\"),\n        GrammarLifeline          = require(\"text!grammars/uml-lifeline.pegjs\"),\n        GrammarMessage           = require(\"text!grammars/uml-message.pegjs\"),\n        GrammarTransition        = require(\"text!grammars/uml-transition.pegjs\");\n\n    /**\n     * Parse Name Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseNameExpression(expr) {\n        var parser = PEG.buildParser(GrammarName);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    /**\n     * Parse Attribute Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseAttributeExpression(expr) {\n        var parser = PEG.buildParser(GrammarAttribute);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    /**\n     * Parse Operation Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseOperationExpression(expr) {\n        var parser = PEG.buildParser(GrammarOperation);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    /**\n     * Parse TemplateParameter Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseTemplateParameterExpression(expr) {\n        var parser = PEG.buildParser(GrammarTemplateParameter);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    /**\n     * Parse Object Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseObjectExpression(expr) {\n        var parser = PEG.buildParser(GrammarObject);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    /**\n     * Parse Slot Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseSlotExpression(expr) {\n        var parser = PEG.buildParser(GrammarSlot);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    /**\n     * Parse Lifeline Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseLifelineExpression(expr) {\n        var parser = PEG.buildParser(GrammarLifeline);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    /**\n     * Parse Message Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseMessageExpression(expr) {\n        var parser = PEG.buildParser(GrammarMessage);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    /**\n     * Parse Transition Expression\n     * @param {string} expr\n     * @return {Object} - Abstract Syntax Tree\n     */\n    function parseTransitionExpression(expr) {\n        var parser = PEG.buildParser(GrammarTransition);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    function getVisibility(elem) {\n        switch (elem.visibility) {\n        case 'public':    return '+';\n        case 'protected': return '#';\n        case 'private':   return '-';\n        case 'package':   return '~';\n        }\n    }\n\n    function getTypeName(type) {\n        if (_.isString(type) && type.length > 0) {\n            return type;\n        } else if (type && type.name) {\n            return type.name;\n        }\n        return null;\n    }\n\n    function getNameExpression(elem) {\n        var str = elem.name;\n        var stereotype = getTypeName(elem.stereotype);\n        if (stereotype) {\n            str = \"<<\" + stereotype + \">>\" + str;\n        }\n        return str;\n    }\n\n    function getAttributeExpression(elem) {\n        var str = getNameExpression(elem);\n        var type = getTypeName(elem.type);\n        if (type) {\n            str = str + \": \" + type;\n        }\n        if (elem.multiplicity && elem.multiplicity.length > 0) {\n            str = str + \"[\" + elem.multiplicity + \"]\";\n        }\n        if (elem.defaultValue) {\n            str = str + \" = \" + elem.defaultValue;\n        }\n        return str;\n    }\n\n    function getOperationExpression(elem) {\n        var str = getNameExpression(elem);\n        var paramStr = [];\n        var params = elem.getNonReturnParameters();\n        if (params && params.length > 0) {\n            _.each(params, function (param) {\n                var s = param.name;\n                if (param.direction) { s = param.direction + \" \" + s; }\n                var paramType = getTypeName(param.type);\n                if (paramType) { s = s + \":\" + paramType; }\n                if (param.multiplicity && param.multiplicity.length > 0) { s = s + \"[\" + param.multiplicity + \"]\"; }\n                if (param.defaultValue) { s = s + \" = \" + param.defaultValue; }\n                paramStr.push(s);\n            });\n        }\n        str = str + \"(\" + paramStr.join(\", \") + \")\";\n        var returnParam = elem.getReturnParameter();\n        if (returnParam) {\n            var returnType = getTypeName(returnParam.type);\n            if (returnType) {\n                str = str + \": \" + returnType;\n            }\n            if (returnParam.multiplicity && returnParam.multiplicity.length > 0) {\n                str = str + \"[\" + returnParam.multiplicity + \"]\";\n            }\n        }\n        return str;\n    }\n\n    function getTemplateParameterExpression(elem) {\n        var str = elem.name;\n        if (elem.parameterType) { str = str + \": \" + elem.parameterType; }\n        if (elem.defaultValue) { str = str + \" = \" + elem.defaultValue; }\n        return str;\n    }\n\n    function getObjectExpression(elem) {\n        var str = getNameExpression(elem);\n        var type = getTypeName(elem.classifier);\n        if (type) {\n            str = str + \": \" + type;\n        }\n        return str;\n    }\n\n    function getSlotExpression(elem) {\n        var str = getNameExpression(elem);\n        var type = getTypeName(elem.type);\n        if (type) {\n            str = str + \": \" + type;\n        }\n        if (elem.value) {\n            str = str + \" = \" + elem.value;\n        }\n        return str;\n    }\n\n    function getLifelineExpression(elem) {\n        var str = getNameExpression(elem);\n        if (elem.selector.length > 0) {\n            str += \"[\" + elem.selector + \"]\";\n        }\n        if (elem.represent) {\n            var type = getTypeName(elem.represent.type);\n            if (type) {\n                str = str + \": \" + type;\n            }\n        }\n        return str;\n    }\n\n    function getMessageExpression(elem) {\n        var str = \"\";\n        // stereotype\n        var stereotype = getTypeName(elem.stereotype);\n        if (stereotype) {\n            str += \"<<\" + stereotype + \">>\";\n        }\n        // assignment target\n        if (elem.assignmentTarget.length > 0) {\n            str += elem.assignmentTarget + \" = \";\n        }\n        // name\n        str += elem.name;\n        // arguments\n        if (elem.arguments.length > 0) {\n            str += \"(\" + elem.arguments + \")\";\n        }\n        return str;\n    }\n\n    function getTransitionExpression(elem) {\n        var str = \"\";\n        var triggers = [];\n        _.each(elem.triggers, function (t) {\n            triggers.push(t.name);\n        });\n        str = str + triggers.join(\", \");\n        if (elem.guard && elem.guard.length > 0) {\n            str = str + \" [\" + elem.guard + \"]\";\n        }\n        if (elem.effects.length > 0) {\n            str = str + \" / \" + elem.effects[0].name;\n        }\n        return str;\n    }\n\n    /**\n     * 요소의 Visibility 값을 변경\n     */\n    function setVisibility(elem, value) {\n        Engine.setProperty(elem, 'visibility', value);\n    }\n\n    /**\n     * 요소의 name, visibility, stereotype 값을 변경\n     */\n    function setNameExpression(elem, ast) {\n        var fields = {};\n        // name\n        if (ast.name) {\n            fields.name = ast.name.trim();\n        }\n        // visibility\n        if (ast.visibility) {\n            fields.visibility = ast.visibility;\n        }\n        // stereotype\n        if (ast.stereotype && (ast.stereotype.length > 0)) {\n            var match = Repository.lookupAndFind(elem, ast.stereotype, type.UMLStereotype);\n            if (match) {\n                fields.stereotype = match;\n            } else {\n                fields.stereotype = ast.stereotype.trim();\n            }\n        } else {\n            fields.stereotype = null;\n        }\n        Engine.setProperties(elem, fields);\n    }\n\n    /**\n     * Attribute Expression을 변경\n     */\n    function setAttributeExpression(elem, ast) {\n        var match, fields = {};\n        // name\n        if (ast.name) {\n            fields.name = ast.name.trim();\n        }\n        // visibility\n        if (ast.visibility) {\n            fields.visibility = ast.visibility;\n        }\n        // stereotype\n        if (ast.stereotype && (ast.stereotype.length > 0)) {\n            match = Repository.lookupAndFind(elem, ast.stereotype, type.UMLStereotype);\n            if (match) {\n                fields.stereotype = match;\n            } else {\n                fields.stereotype = ast.stereotype.trim();\n            }\n        } else {\n            fields.stereotype = null;\n        }\n        // type\n        if (ast.type && (ast.type.length > 0)) {\n            match = Repository.lookupAndFind(elem, ast.type, type.UMLClassifier);\n            if (match) {\n                fields.type = match;\n            } else {\n                fields.type = ast.type.trim();\n            }\n        } else {\n            fields.type = null;\n        }\n        // multiplicity\n        if (ast.multiplicity && ast.multiplicity.length > 0) {\n            fields.multiplicity = ast.multiplicity;\n        } else {\n            fields.multiplicity = \"\";\n        }\n        // defaultValue\n        if (ast.defaultValue) {\n            fields.defaultValue = ast.defaultValue.trim();\n        } else {\n            fields.defaultValue = '';\n        }\n        Engine.setProperties(elem, fields);\n    }\n\n    /**\n     * Operation Expression을 변경\n     */\n    function setOperationExpression(elem, ast) {\n        var match;\n        OperationBuilder.begin('change operation');\n        // name\n        if (ast.name) {\n            OperationBuilder.fieldAssign(elem, 'name', ast.name.trim());\n        }\n        // visibility\n        if (ast.visibility) {\n            OperationBuilder.fieldAssign(elem, 'visibility', ast.visibility);\n        }\n        // stereotype\n        if (ast.stereotype && (ast.stereotype.length > 0)) {\n            match = Repository.lookupAndFind(elem, ast.stereotype, type.UMLStereotype);\n            if (match) {\n                OperationBuilder.fieldAssign(elem, 'stereotype', match);\n            } else {\n                OperationBuilder.fieldAssign(elem, 'stereotype', ast.stereotype.trim());\n            }\n        } else {\n            OperationBuilder.fieldAssign(elem, 'stereotype', null);\n        }\n        // parameters (파라미터들이 변경되었을때)\n        // 1) 기존의 파라미터들을 유지하고 name, type, direction 값들만 업데이트 한다.\n        // 2) 기존의 파라미터의 수가 모자라면 새로 생성해서 추가한다.\n        // 3) 기존의 파라미터의 수가 넘어가면 삭제한다.\n\n        if (!ast.parameters) {\n            console.log(ast);\n            ast.parameters = [];\n        }\n        if (ast.parameters) {\n            var nonReturnParams = elem.getNonReturnParameters();\n            for (var i = 0; i < ast.parameters.length; i++) {\n                var param = ast.parameters[i];\n                if (nonReturnParams[i]) {\n                    // parameter name\n                    if (param.name) {\n                        OperationBuilder.fieldAssign(nonReturnParams[i], 'name', param.name);\n                    }\n                    // parameter type\n                    if (param.type && (param.type.length > 0)) {\n                        match = Repository.lookupAndFind(elem, param.type, type.UMLClassifier);\n                        if (match) {\n                            OperationBuilder.fieldAssign(nonReturnParams[i], 'type', match);\n                        } else {\n                            OperationBuilder.fieldAssign(nonReturnParams[i], 'type', param.type.trim());\n                        }\n                    } else {\n                        OperationBuilder.fieldAssign(nonReturnParams[i], 'type', null);\n                    }\n                    // direction\n                    if (param.direction) {\n                        OperationBuilder.fieldAssign(nonReturnParams[i], 'direction', param.direction);\n                    } else {\n                        OperationBuilder.fieldAssign(nonReturnParams[i], 'direction', UML.DK_IN);\n                    }\n                    // multiplicity\n                    if (param.multiplicity) {\n                        OperationBuilder.fieldAssign(nonReturnParams[i], 'multiplicity', param.multiplicity);\n                    } else {\n                        OperationBuilder.fieldAssign(nonReturnParams[i], 'multiplicity', '');\n                    }\n                    // defaultValue\n                    if (param.defaultValue) {\n                        OperationBuilder.fieldAssign(nonReturnParams[i], 'defaultValue', param.defaultValue.trim());\n                    } else {\n                        OperationBuilder.fieldAssign(nonReturnParams[i], 'defaultValue', '');\n                    }\n                } else {\n                    var newParam = new type.UMLParameter();\n                    newParam._parent = elem;\n                    // new name\n                    newParam.name = param.name.trim();\n                    // new parameter type\n                    if (param.type && (param.type.length > 0)) {\n                        match = Repository.lookupAndFind(elem, param.type, type.UMLClassifier);\n                        if (match) {\n                            newParam.type = match;\n                        } else {\n                            newParam.type = param.type.trim();\n                        }\n                    } else {\n                        newParam.type = null;\n                    }\n                    // direction\n                    if (param.direction) {\n                        newParam.direction = param.direction;\n                    } else {\n                        newParam.direction = UML.DK_IN;\n                    }\n                    // multiplicity\n                    if (param.multiplicity) {\n                        newParam.multiplicity = param.multiplicity.trim();\n                    } else {\n                        newParam.multiplicity = '';\n                    }\n                    // defaultValue\n                    if (param.defaultValue) {\n                        newParam.defaultValue = param.defaultValue.trim();\n                    } else {\n                        newParam.defaultValue = '';\n                    }\n                    OperationBuilder.insert(newParam);\n                    OperationBuilder.fieldInsert(elem, 'parameters', newParam);\n                }\n            }\n            if (nonReturnParams.length > ast.parameters.length) {\n                for (var j = ast.parameters.length; j < nonReturnParams.length; j++) {\n                    OperationBuilder.fieldRemove(elem, 'parameters', nonReturnParams[j]);\n                    OperationBuilder.remove(nonReturnParams[j]);\n                }\n            }\n        }\n        // returnType\n        var returnParam = elem.getReturnParameter();\n        if (returnParam) {\n            if (ast.returnType) {\n                if (ast.returnType.length > 0) {\n                    match = Repository.lookupAndFind(elem, ast.returnType, type.UMLClassifier);\n                    if (match) {\n                        OperationBuilder.fieldAssign(returnParam, 'type', match);\n                    } else {\n                        OperationBuilder.fieldAssign(returnParam, 'type', ast.returnType.trim());\n                    }\n                    if (ast.returnMultiplicity) {\n                        OperationBuilder.fieldAssign(returnParam, 'multiplicity', ast.returnMultiplicity);\n                    } else {\n                        OperationBuilder.fieldAssign(returnParam, 'multiplicity', '');\n                    }\n                } else {\n                    OperationBuilder.fieldAssign(returnParam, 'type', null);\n                }\n            } else {\n                if (returnParam) {\n                    OperationBuilder.fieldRemove(elem, 'parameters', returnParam);\n                    OperationBuilder.remove(returnParam);\n                }\n            }\n        } else {\n            if (ast.returnType) {\n                returnParam = new type.UMLParameter();\n                returnParam._parent = elem;\n                returnParam.direction = UML.DK_RETURN;\n                if (ast.returnType.length > 0) {\n                    match = Repository.lookupAndFind(elem, ast.returnType, type.UMLClassifier);\n                    if (match) {\n                        returnParam.type = match;\n                    } else {\n                        returnParam.type = ast.returnType.trim();\n                    }\n                    if (ast.returnMultiplicity) {\n                        returnParam.multiplicity = ast.returnMultiplicity.trim();\n                    } else {\n                        returnParam.multiplicity = '';\n                    }\n                } else {\n                    returnParam.type = null;\n                }\n                OperationBuilder.insert(returnParam);\n                OperationBuilder.fieldInsert(elem, 'parameters', returnParam);\n            }\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n    /**\n     * TemplateParameter Expression을 변경\n     */\n    function setTemplateParameterExpression(elem, ast) {\n        var fields = {};\n        // name\n        if (ast.name) {\n            fields.name = ast.name.trim();\n        }\n        // stereotype\n        if (ast.stereotype && (ast.stereotype.length > 0)) {\n            var match = Repository.lookupAndFind(elem, ast.stereotype, type.UMLStereotype);\n            if (match) {\n                fields.stereotype = match;\n            } else {\n                fields.stereotype = ast.stereotype.trim();\n            }\n        } else {\n            fields.stereotype = null;\n        }\n        // parameterType\n        if (ast.parameterType) {\n            fields.parameterType = ast.parameterType.trim();\n        } else {\n            fields.parameterType = '';\n        }\n        // defaultValue\n        if (ast.defaultValue) {\n            fields.defaultValue = ast.defaultValue.trim();\n        } else {\n            fields.defaultValue = '';\n        }\n        Engine.setProperties(elem, fields);\n    }\n\n    /**\n     * Object Expression을 변경\n     */\n    function setObjectExpression(elem, ast) {\n        var match, fields = {};\n        // name\n        if (ast.name) {\n            fields.name = ast.name.trim();\n        }\n        // visibility\n        if (ast.visibility) {\n            fields.visibility = ast.visibility;\n        }\n        // stereotype\n        if (ast.stereotype && (ast.stereotype.length > 0)) {\n            match = Repository.lookupAndFind(elem, ast.stereotype, type.UMLStereotype);\n            if (match) {\n                fields.stereotype = match;\n            } else {\n                fields.stereotype = ast.stereotype.trim();\n            }\n        } else {\n            fields.stereotype = null;\n        }\n        // type\n        if (ast.type && (ast.type.length > 0)) {\n            match = Repository.lookupAndFind(elem, ast.type, type.UMLClassifier);\n            if (match) {\n                fields.classifier = match;\n            } else {\n                fields.classifier = ast.type.trim();\n            }\n        } else {\n            fields.classifier = null;\n        }\n        Engine.setProperties(elem, fields);\n    }\n\n    /**\n     * Slot Expression을 변경\n     */\n    function setSlotExpression(elem, ast) {\n        var match, fields = {};\n        // name\n        if (ast.name) {\n            fields.name = ast.name.trim();\n        }\n        // visibility\n        if (ast.visibility) {\n            fields.visibility = ast.visibility;\n        }\n        // stereotype\n        if (ast.stereotype && (ast.stereotype.length > 0)) {\n            match = Repository.lookupAndFind(elem, ast.stereotype, type.UMLStereotype);\n            if (match) {\n                fields.stereotype = match;\n            } else {\n                fields.stereotype = ast.stereotype.trim();\n            }\n        } else {\n            fields.stereotype = null;\n        }\n        // type\n        if (ast.type && (ast.type.length > 0)) {\n            match = Repository.lookupAndFind(elem, ast.type, type.UMLClassifier);\n            if (match) {\n                fields.type = match;\n            } else {\n                fields.type = ast.type.trim();\n            }\n        } else {\n            fields.type = null;\n        }\n        // value\n        if (ast.value) {\n            fields.value = ast.value.trim();\n        } else {\n            fields.value = '';\n        }\n        Engine.setProperties(elem, fields);\n    }\n\n\n    /**\n     * Lifeline Expression을 변경\n     */\n    function setLifelineExpression(elem, ast) {\n        var match, fields = {};\n        // name\n        if (ast.name) {\n            fields.name = ast.name.trim();\n        }\n        // visibility\n        if (ast.visibility) {\n            fields.visibility = ast.visibility;\n        }\n        // stereotype\n        if (ast.stereotype && (ast.stereotype.length > 0)) {\n            match = Repository.lookupAndFind(elem, ast.stereotype, type.UMLStereotype);\n            if (match) {\n                fields.stereotype = match;\n            } else {\n                fields.stereotype = ast.stereotype.trim();\n            }\n        } else {\n            fields.stereotype = null;\n        }\n        // selector\n        if (ast.selector) {\n            fields.selector = ast.selector.trim();\n        } else {\n            fields.selector = '';\n        }\n        OperationBuilder.begin(\"set lifeline expression\");\n        for (var key in fields) {\n            OperationBuilder.fieldAssign(elem, key, fields[key]);\n        }\n        // type\n        if (elem.represent) {\n            if (ast.type && (ast.type.length > 0)) {\n                match = Repository.lookupAndFind(elem, ast.type, type.UMLClassifier);\n                if (match) {\n                    OperationBuilder.fieldAssign(elem.represent, 'type', match);\n                } else {\n                    OperationBuilder.fieldAssign(elem.represent, 'type', ast.type.trim());\n                }\n            } else {\n                OperationBuilder.fieldAssign(elem.represent, 'type', null);\n            }\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n    /**\n     * Message Expression을 변경\n     */\n    function setMessageExpression(elem, ast) {\n        var fields = {};\n        // name\n        if (ast.name) {\n            fields.name = ast.name.trim();\n        }\n        // stereotype\n        if (ast.stereotype && (ast.stereotype.length > 0)) {\n            var match = Repository.lookupAndFind(elem, ast.stereotype, type.UMLStereotype);\n            if (match) {\n                fields.stereotype = match;\n            } else {\n                fields.stereotype = ast.stereotype.trim();\n            }\n        } else {\n            fields.stereotype = null;\n        }\n        // assignmentTarget\n        if (ast.assignmentTarget) {\n            fields.assignmentTarget = ast.assignmentTarget.trim();\n        } else {\n            fields.assignmentTarget = '';\n        }\n        // arguments\n        if (ast.arguments) {\n            fields.arguments = ast.arguments.trim();\n        } else {\n            fields.arguments = '';\n        }\n        Engine.setProperties(elem, fields);\n    }\n\n    /**\n     * Transition Expression을 변경\n     */\n    function setTransitionExpression(elem, ast) {\n        OperationBuilder.begin('change transition');\n        // triggers\n        if (!ast.triggers) { ast.triggers = []; }\n        for (var i = 0, len = ast.triggers.length; i < len; i++) {\n            if (elem.triggers[i]) {\n                OperationBuilder.fieldAssign(elem.triggers[i], 'name', ast.triggers[i].trim());\n            } else {\n                var newTrigger = new type.UMLEvent();\n                newTrigger._parent = elem;\n                newTrigger.name = ast.triggers[i].trim();\n                OperationBuilder.insert(newTrigger);\n                OperationBuilder.fieldInsert(elem, 'triggers', newTrigger);\n            }\n        }\n        if (elem.triggers.length > ast.triggers.length) {\n            for (var j = ast.triggers.length; j < elem.triggers.length; j++) {\n                OperationBuilder.fieldRemove(elem, 'triggers', elem.triggers[j]);\n                OperationBuilder.remove(elem.triggers[j]);\n            }\n        }\n        // guard\n        if (ast.guard) {\n            OperationBuilder.fieldAssign(elem, 'guard', ast.guard.trim());\n        }\n        // effect\n        var effectCount = (ast.effect && ast.effect.length > 0 ? 1 : 0);\n        if (effectCount > 0) {\n            if (elem.effects.length > 0) {\n                OperationBuilder.fieldAssign(elem.effects[0], 'name', ast.effect.trim());\n            } else {\n                var newEffect = new type.UMLOpaqueBehavior();\n                newEffect._parent = elem;\n                newEffect.name = ast.effect.trim();\n                OperationBuilder.insert(newEffect);\n                OperationBuilder.fieldInsert(elem, 'effects', newEffect);\n            }\n        }\n        if (elem.effects.length > effectCount) {\n            for (var k = effectCount; k < elem.effects.length; k++) {\n                OperationBuilder.fieldRemove(elem, 'effects', elem.effects[k]);\n                OperationBuilder.remove(elem.effects[k]);\n            }\n        }\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n    }\n\n\n    /**\n     * 모델 요소에 대응하는 ViewType을 리턴\n     */\n    function getViewTypeOf(model) {\n        switch (model.getClassName()) {\n        // Common\n        case 'UMLConstraint':               return type.UMLConstraintView;\n        // Classes\n        case 'UMLClass':                    return type.UMLClassView;\n        case 'UMLInterface':                return type.UMLInterfaceView;\n        case 'UMLSignal':                   return type.UMLSignalView;\n        case 'UMLDataType':                 return type.UMLDataTypeView;\n        case 'UMLPrimitiveType':            return type.UMLPrimitiveTypeView;\n        case 'UMLEnumeration':              return type.UMLEnumerationView;\n        case 'UMLGeneralization':           return type.UMLGeneralizationView;\n        case 'UMLAssociation':              return type.UMLAssociationView;\n        case 'UMLDependency':               return type.UMLDependencyView;\n        case 'UMLInterfaceRealization':     return type.UMLInterfaceRealizationView;\n        // Packages\n        case 'UMLModel':                    return type.UMLModelView;\n        case 'UMLSubsystem':                return type.UMLSubsystemView;\n        case 'UMLPackage':                  return type.UMLPackageView;\n        // Instances\n        case 'UMLObject':                   return type.UMLObjectView;\n        case 'UMLArtifactInstance':         return type.UMLArtifactInstanceView;\n        case 'UMLComponentInstance':        return type.UMLComponentInstanceView;\n        case 'UMLNodeInstance':             return type.UMLNodeInstanceView;\n        case 'UMLLink':                     return type.UMLLinkView;\n        // Use Cases\n        case 'UMLUseCase':                  return type.UMLUseCaseView;\n        case 'UMLActor':                    return type.UMLActorView;\n        case 'UMLInclude':                  return type.UMLIncludeView;\n        case 'UMLExtend':                   return type.UMLExtendView;\n        // Components and Deployments\n        case 'UMLArtifact':                 return type.UMLArtifactView;\n        case 'UMLComponent':                return type.UMLComponentView;\n        case 'UMLNode':                     return type.UMLNodeView;\n        case 'UMLComponentRealization':     return type.UMLComponentRealizationView;\n        case 'UMLDeployment':               return type.UMLDeploymentView;\n        case 'UMLCommunicationPath':        return type.UMLCommunicationPathView;\n        // Composite Structure\n        case 'UMLPort':                     return type.UMLPortView;\n        case 'UMLAttribute':                return type.UMLPartView;\n        case 'UMLCollaboration':            return type.UMLCollaborationView;\n        case 'UMLCollaborationUse':         return type.UMLCollaborationUseView;\n        case 'UMLConnector':                return type.UMLConnectorView;\n        case 'UMLRoleBinding':              return type.UMLRoleBindingView;\n        // Sequence\n        case 'UMLCombinedFragment':         return type.UMLCombinedFragmentView;\n        case 'UMLInteractionUse':           return type.UMLInteractionUseView;\n        // Communication\n        // Statechart\n        case 'UMLState':                    return type.UMLStateView;\n        case 'UMLPseudostate':              return type.UMLPseudostateView;\n        case 'UMLFinalState':               return type.UMLFinalStateView;\n        case 'UMLConnectionPointReference': return type.UMLConnectionPointReferenceView;\n        case 'UMLTransition':               return type.UMLTransitionView;\n        // Activity\n        case 'UMLAction':                   return type.UMLActionView;\n        case 'UMLInitialNode':              return type.UMLControlNodeView;\n        case 'UMLFinalNode':                return type.UMLControlNodeView;\n        case 'UMLActivityFinalNode':        return type.UMLControlNodeView;\n        case 'UMLFlowFinalNode':            return type.UMLControlNodeView;\n        case 'UMLForkNode':                 return type.UMLControlNodeView;\n        case 'UMLJoinNode':                 return type.UMLControlNodeView;\n        case 'UMLMergeNode':                return type.UMLControlNodeView;\n        case 'UMLDecisionNode':             return type.UMLControlNodeView;\n        case 'UMLObjectNode':               return type.UMLObjectNodeView;\n        case 'UMLControlFlow':              return type.UMLControlFlowView;\n        case 'UMLObjectFlow':               return type.UMLObjectFlowView;\n        case 'UMLActivityPartition':        return type.UMLSwimlaneView;\n        // Profile\n        case 'UMLProfile':                  return type.UMLProfileView;\n        case 'UMLMetaClass':                return type.UMLMetaClassView;\n        case 'UMLStereotype':               return type.UMLStereotypeView;\n        case 'UMLExtension':                return type.UMLExtensionView;\n        default:                            return null;\n        }\n    }\n\n    exports.parseNameExpression              = parseNameExpression;\n    exports.parseAttributeExpression         = parseAttributeExpression;\n    exports.parseOperationExpression         = parseOperationExpression;\n    exports.parseTemplateParameterExpression = parseTemplateParameterExpression;\n    exports.parseObjectExpression            = parseObjectExpression;\n    exports.parseSlotExpression              = parseSlotExpression;\n    exports.parseLifelineExpression          = parseLifelineExpression;\n    exports.parseMessageExpression           = parseMessageExpression;\n    exports.parseTransitionExpression        = parseTransitionExpression;\n\n    exports.getVisibility                    = getVisibility;\n    exports.getNameExpression                = getNameExpression;\n    exports.getAttributeExpression           = getAttributeExpression;\n    exports.getOperationExpression           = getOperationExpression;\n    exports.getTemplateParameterExpression   = getTemplateParameterExpression;\n    exports.getObjectExpression              = getObjectExpression;\n    exports.getSlotExpression                = getSlotExpression;\n    exports.getLifelineExpression            = getLifelineExpression;\n    exports.getMessageExpression             = getMessageExpression;\n    exports.getTransitionExpression          = getTransitionExpression;\n\n    exports.setVisibility                    = setVisibility;\n    exports.setNameExpression                = setNameExpression;\n    exports.setAttributeExpression           = setAttributeExpression;\n    exports.setOperationExpression           = setOperationExpression;\n    exports.setTemplateParameterExpression   = setTemplateParameterExpression;\n    exports.setObjectExpression              = setObjectExpression;\n    exports.setSlotExpression                = setSlotExpression;\n    exports.setLifelineExpression            = setLifelineExpression;\n    exports.setMessageExpression             = setMessageExpression;\n    exports.setTransitionExpression          = setTransitionExpression;\n\n    exports.getViewTypeOf                    = getViewTypeOf;\n\n});\n\n//# sourceURL=/uml/UMLUtils.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, vars: true */\n/*global define, $, _, window, appshell, type, Mustache */\n\ndefine(\'uml/UMLFactory\',[\'require\',\'exports\',\'module\',\'utils/Helper\',\'core/Core\',\'core/OperationBuilder\',\'core/Repository\',\'core/MetaModelManager\',\'engine/ProjectManager\',\'engine/Engine\',\'engine/Factory\',\'engine/SelectionManager\',\'diagrams/DiagramManager\',\'dialogs/Dialogs\',\'uml/UML\',\'uml/UMLDiagram\',\'uml/UMLUtils\'],function (require, exports, module) {\n    "use strict";\n\n    var Helper           = require("utils/Helper"),\n        Core             = require("core/Core"),\n        OperationBuilder = require("core/OperationBuilder"),\n        Repository       = require("core/Repository"),\n        MetaModelManager = require("core/MetaModelManager"),\n        ProjectManager   = require("engine/ProjectManager"),\n        Engine           = require("engine/Engine"),\n        Factory          = require("engine/Factory"),\n        SelectionManager = require("engine/SelectionManager"),\n        DiagramManager   = require("diagrams/DiagramManager"),\n        Dialogs          = require("dialogs/Dialogs"),\n        UML              = require("uml/UML"),\n        UMLDiagram       = require("uml/UMLDiagram"),\n        UMLUtils         = require("uml/UMLUtils");\n\n    // Precondition functions .................................................\n\n    function featurePrecondition(options) {\n        Helper.assert(\n            options.parent instanceof type.UMLClassifier,\n            Mustache.render(Factory.ERR_INVALID_PARENT, options.modelType)\n        );\n    }\n\n    function modelElementLinkPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLModelElement) && (options.headModel instanceof type.UMLModelElement),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function featureLinkPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLFeature) && (options.headModel instanceof type.UMLFeature),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function classifierLinkPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLClassifier) && (options.headModel instanceof type.UMLClassifier),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function roleBindingPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLCollaborationUse) && (options.headModel instanceof type.UMLAttribute),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function componentRealizationPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLClassifier) && (options.headModel instanceof type.UMLComponent),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function deploymentPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLClassifier) && (options.headModel instanceof type.UMLNode),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function nodeLinkPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLNode) && (options.headModel instanceof type.UMLNode),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function instanceLinkPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLInstance) && (options.headModel instanceof type.UMLInstance),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function useCaseLinkPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLUseCase) && (options.headModel instanceof type.UMLUseCase),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function interactionPrecondition(options) {\n        Helper.assert(\n            options.parent instanceof type.UMLInteraction,\n            Mustache.render(Factory.ERR_INVALID_PARENT, options.modelType)\n        );\n    }\n\n    function activityPrecondition(options) {\n        Helper.assert(\n            options.parent instanceof type.UMLActivity,\n            Mustache.render(Factory.ERR_INVALID_PARENT, options.modelType)\n        );\n    }\n\n    function pinPrecondition(options) {\n        Helper.assert(\n            options.parent instanceof type.UMLAction,\n            Mustache.render(Factory.ERR_INVALID_PARENT, options.modelType)\n        );\n    }\n\n    function flowPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLPin || options.tailModel instanceof type.UMLActivityNode) &&\n            (options.headModel instanceof type.UMLPin || options.headModel instanceof type.UMLActivityNode),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function actionLinkPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLAction) &&\n            (options.headModel instanceof type.UMLAction),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    function extensionPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.UMLStereotype) && (options.headModel instanceof type.UMLMetaClass),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n\n    // Create diagram functions ...............................................\n\n    function structuralDiagramFn(parent, options) {\n        var DiagramType = type[options.diagramType],\n            model,\n            diagram;\n        parent = parent || ProjectManager.getProject();\n        if (parent instanceof type.Project) {\n            model = new type.UMLModel();\n            model.name = Core.getNewName(parent.ownedElements, "Model");\n            model._parent = parent;\n            diagram = new DiagramType();\n            diagram.name = Core.getNewName(parent.ownedElements, diagram.getDisplayClassName());\n            model.ownedElements.push(diagram);\n            diagram._parent = model;\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', model);\n        } else {\n            diagram = new DiagramType();\n            diagram.name = Core.getNewName(parent.ownedElements, diagram.getDisplayClassName());\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    function profileDiagramFn(parent, options) {\n        var profile, diagram;\n        if (parent instanceof type.UMLProfile) {\n            diagram = new type.UMLProfileDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'ProfileDiagram\');\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        } else {\n            profile = new type.UMLProfile();\n            profile.name = Core.getNewName(parent.ownedElements, \'Profile\');\n            profile._parent = parent;\n            diagram = new type.UMLProfileDiagram();\n            diagram.name = Core.getNewName(profile.ownedElements, \'ProfileDiagram\');\n            profile.ownedElements.push(diagram);\n            diagram._parent = profile;\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', profile);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    function compositeStructureDiagramFn(parent, options) {\n        var collaboration, diagram;\n        if (parent instanceof type.UMLClassifier) {\n            diagram = new type.UMLCompositeStructureDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'CompositeStructureDiagram\');\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        } else {\n            collaboration = new type.UMLCollaboration();\n            collaboration.name = Core.getNewName(parent.collaborations, \'Collaboration\');\n            collaboration._parent = parent;\n            diagram = new type.UMLCompositeStructureDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'Composite Structures\');\n            collaboration.ownedElements.push(diagram);\n            diagram._parent = collaboration;\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', collaboration);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    function _addFrame(diagram) {\n        var frame = new type.UMLFrameView();\n        diagram.ownedViews.push(frame);\n        frame._parent = diagram;\n        frame.model = diagram;\n        frame.initialize(null, 5, 5, 700, 600);\n    }\n\n    function sequenceDiagramFn(parent, options) {\n        var collaboration, interaction, frame, diagram;\n        if (parent instanceof type.UMLInteraction) {\n            diagram = new type.UMLSequenceDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'SequenceDiagram\');\n            _addFrame(diagram);\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        } else if (parent instanceof type.UMLClassifier) {\n            interaction = new type.UMLInteraction();\n            interaction.name = Core.getNewName(parent.ownedElements, \'Interaction\');\n            diagram = new type.UMLSequenceDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'SequenceDiagram\');\n            interaction.ownedElements.push(diagram);\n            diagram._parent = interaction;\n            _addFrame(diagram);\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', interaction);\n        } else {\n            collaboration = new type.UMLCollaboration();\n            collaboration.name = Core.getNewName(parent.ownedElements, \'Collaboration\');\n            collaboration._parent = parent;\n            interaction = new type.UMLInteraction();\n            interaction.name = Core.getNewName(collaboration.ownedElements, \'Interaction\');\n            collaboration.ownedElements.push(interaction);\n            interaction._parent = collaboration;\n            diagram = new type.UMLSequenceDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'SequenceDiagram\');\n            interaction.ownedElements.push(diagram);\n            diagram._parent = interaction;\n            _addFrame(diagram);\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', collaboration);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    function communicationDiagramFn(parent, options) {\n        var collaboration, interaction, diagram, frame;\n        if (parent instanceof type.UMLInteraction) {\n            diagram = new type.UMLCommunicationDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'CommunicationDiagram\');\n            _addFrame(diagram);\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        } else if (parent instanceof type.UMLCollaboration) {\n            interaction = new type.UMLInteraction();\n            interaction.name = Core.getNewName(parent.ownedElements, \'Interaction\');\n            diagram = new type.UMLCommunicationDiagram();\n            diagram.name = Core.getNewName(interaction.ownedElements, \'CommunicationDiagram\');\n            interaction.ownedElements.push(diagram);\n            diagram._parent = interaction;\n            _addFrame(diagram);\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', interaction);\n        } else {\n            collaboration = new type.UMLCollaboration();\n            collaboration.name = Core.getNewName(parent.ownedElements, \'Collaboration\');\n            collaboration._parent = parent;\n            interaction = new type.UMLInteraction();\n            interaction.name = Core.getNewName(collaboration.ownedElements, \'Interaction\');\n            collaboration.ownedElements.push(interaction);\n            interaction._parent = collaboration;\n            diagram = new type.UMLCommunicationDiagram();\n            diagram.name = Core.getNewName(interaction.ownedElements, \'CommunicationDiagram\');\n            interaction.ownedElements.push(diagram);\n            diagram._parent = interaction;\n            _addFrame(diagram);\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', collaboration);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    function statechartDiagramFn(parent, options) {\n        var stateMachine, region, diagram;\n        if (parent instanceof type.UMLStateMachine) {\n            diagram = new type.UMLStatechartDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'StatechartDiagram\');\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        } else {\n            // StateMachine\n            stateMachine = new type.UMLStateMachine();\n            stateMachine.name = Core.getNewName(parent.ownedElements, \'StateMachine\');\n            stateMachine._parent = parent;\n            region = new type.UMLRegion();\n            stateMachine.regions.push(region);\n            region._parent = stateMachine;\n            // Diagram\n            diagram = new type.UMLStatechartDiagram();\n            diagram.name = Core.getNewName(stateMachine.ownedElements, \'StatechartDiagram\');\n            stateMachine.ownedElements.push(diagram);\n            diagram._parent = stateMachine;\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', stateMachine);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    function activityDiagramFn(parent, options) {\n        var activity, diagram;\n        if (parent instanceof type.UMLActivity) {\n            diagram = new type.UMLActivityDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, \'ActivityDiagram\');\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        } else {\n            activity = new type.UMLActivity();\n            activity.name = Core.getNewName(parent.stateMachines, \'Activity\');\n            activity._parent = parent;\n            diagram = new type.UMLActivityDiagram();\n            diagram.name = Core.getNewName(activity.ownedElements, \'ActivityDiagram\');\n            activity.ownedElements.push(diagram);\n            diagram._parent = activity;\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', activity);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    // Create model functions ..................................................\n\n    function stateMachineFn(parent, field, options) {\n        var model, region;\n        Helper.assert(\n            parent instanceof type.Model,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "StateMachine")\n        );\n        model = new type.UMLStateMachine();\n        model.name = Core.getNewName(parent.ownedElements, \'StateMachine\');\n        model._parent = parent;\n        region = new type.UMLRegion();\n        model.regions.push(region);\n        region._parent = model;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        Engine.addModel(parent, field, model);\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        options.triggerElementCreated(model, null);\n        return model;\n    }\n\n    // Create model and view functions .........................................\n\n    function associationClassFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            ((options.tailModel instanceof type.UMLClass) && (options.headModel instanceof type.UMLAssociation)) ||\n            ((options.headModel instanceof type.UMLClass) && (options.tailModel instanceof type.UMLAssociation)) ||\n            ((options.tailModel instanceof type.UMLClassifier) && (options.headModel instanceof type.UMLClassifier)),\n            Mustache.render(Factory.ERR_INVALID_LINK, "AssociationClass")\n        );\n        if ((options.tailModel instanceof type.UMLClassifier) && (options.headModel instanceof type.UMLClassifier)) {\n            // 두 개의 Classifier를 연결한 경우 : Class, Association, AssociationClassLink를 생성.\n            // create models (Class, Association, AssociationClassLink)\n            model = new type.UMLClass();\n            model.name = Core.getNewName(parent.ownedElements, "AssociationClass");\n            model._parent = parent;\n            var association = new type.UMLAssociation();\n            association.end1.reference = options.tailModel;\n            association.end2.reference = options.headModel;\n            model.ownedElements.push(association);\n            association._parent = model;\n            var associationClassLink = new type.UMLAssociationClassLink();\n            associationClassLink.classSide = model;\n            associationClassLink.associationSide = association;\n            model.ownedElements.push(associationClassLink);\n            associationClassLink._parent = model;\n            if (options.modelInitializer) {\n                options.modelInitializer(associationClassLink);\n            }\n            // create views (ClassView, AssociationView, AssociationClassLinkView)\n            var midX = (options.x1 + options.x2) / 2,\n                midY = (options.y1 + options.y2) / 2,\n                CLASS_DISTANCE = 30;\n            var classView = new type.UMLClassView();\n            classView.model = model;\n            classView._parent = diagram;\n            classView.initialize(null, midX, midY + CLASS_DISTANCE, midX, midY + CLASS_DISTANCE);\n            var associationView = new type.UMLAssociationView();\n            associationView.model = association;\n            associationView.tail = options.tailView;\n            associationView.head = options.headView;\n            associationView.initialize(null, options.x1, options.y1, options.x2, options.y2);\n            associationView._parent = diagram;\n            var associationClassLinkView = new type.UMLAssociationClassLinkView();\n            associationClassLinkView.model = associationClassLink;\n            associationClassLinkView.tail = classView;\n            associationClassLinkView.head = associationView;\n            associationClassLinkView.initialize(null, midX, midY, midX + CLASS_DISTANCE, midY + CLASS_DISTANCE);\n            associationClassLinkView._parent = diagram;\n            if (options.viewInitializer) {\n                options.viewInitializer(associationClassLinkView);\n            }\n            // make command\n            OperationBuilder.begin(\'add element\');\n            OperationBuilder.insert(model);\n            OperationBuilder.fieldInsert(parent, \'ownedElements\', model);\n            OperationBuilder.insert(classView);\n            OperationBuilder.insert(associationView);\n            OperationBuilder.insert(associationClassLinkView);\n            OperationBuilder.fieldInsert(diagram, \'ownedViews\', classView);\n            OperationBuilder.fieldInsert(diagram, \'ownedViews\', associationView);\n            OperationBuilder.fieldInsert(diagram, \'ownedViews\', associationClassLinkView);\n            OperationBuilder.end();\n            Repository.doOperation(OperationBuilder.getOperation());\n        } else {\n            model = new type.UMLAssociationClassLink();\n            view = new type.UMLAssociationClassLinkView();\n            view.lineStyle = Core.LS_OBLIQUE;\n            if ((options.tailModel instanceof type.UMLClass) && (options.headModel instanceof type.UMLAssociation)) {\n                // Class에서 Association로 연결한 경우 : AssociationClassLink를 생성.\n                model.classSide = options.tailModel;\n                model.associationSide = options.headModel;\n                view.tail = options.tailView;\n                view.head = options.headView;\n            } else if ((options.headModel instanceof type.UMLClass) && (options.tailModel instanceof type.UMLAssociation)) {\n                // Association에서 Class로 연결한 경우 : AssociationClassLink를 생성.\n                model.classSide = options.headModel;\n                model.associationSide = options.tailModel;\n                view.tail = options.headView;\n                view.head = options.tailView;\n            }\n            view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n            Engine.addModelAndView(diagram, model, view, model.classSide, \'ownedElements\');\n        }\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function interfaceRealizationFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            ((options.tailModel instanceof type.UMLClassifier) && (options.headModel instanceof type.UMLInterface)) ||\n            ((options.tailModel instanceof type.UMLPort) && (options.headModel instanceof type.UMLInterface)),\n            Mustache.render(Factory.ERR_INVALID_LINK, "Interface Realization")\n        );\n        if (options.tailModel instanceof type.UMLPort) {\n            Helper.assert(options.tailModel.type instanceof type.UMLClassifier, "Port should have type to create Interface Realization");\n            if (options.tailModel.type instanceof type.UMLClassifier) {\n                var rs = Repository.getRelationshipsOf(options.tailModel.type, function (r) {\n                    return (r instanceof type.UMLInterfaceRealization &&\n                            r.source === options.tailModel.type &&\n                            r.target === options.headModel);\n                });\n                options.tailModel = options.tailModel.type;\n                // there already interface realization is.\n                if (rs.length > 0) {\n                    view = new type.UMLInterfaceRealizationView();\n                    view.tail = options.tailView;\n                    view.head = options.headView;\n                    view.model = rs[0];\n                    view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n                    Engine.addViews(diagram, [view]);\n                }\n            }\n        }\n        model = new type.UMLInterfaceRealization();\n        model.source = options.tailModel;\n        model.target = options.headModel;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        view = new type.UMLInterfaceRealizationView();\n        view.tail = options.tailView;\n        view.head = options.headView;\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        Engine.addModelAndView(diagram, model, view, options.tailModel, \'ownedElements\');\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function containmentFn(parent, diagram, options) {\n        var view;\n        Helper.assert(\n            (options.tailModel instanceof type.UMLModelElement) && (options.headModel instanceof type.UMLModelElement),\n            Mustache.render(Factory.ERR_INVALID_LINK, "Containment")\n        );\n        if (!_.contains(options.headModel.ownedElements, options.tailModel)) {\n            Engine.relocate(options.tailModel, options.headModel, \'ownedElements\');\n        }\n        view = new type.UMLContainmentView();\n        view.tail = options.tailView;\n        view.head = options.headView;\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        Engine.addViews(diagram, [view]);\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(null, view);\n        return view;\n    }\n\n    function partFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            parent instanceof type.UMLClassifier || parent instanceof type.UMLAttribute,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "UMLAttribute")\n        );\n        if (parent instanceof type.UMLAttribute) {\n            Helper.assert(\n                parent.type instanceof type.UMLClassifier,\n                "Type should be assigned to have internal Parts"\n            );\n            parent = parent.type;\n        }\n        // Create Part (Attribute)\n        model = new type.UMLAttribute();\n        model.name = Core.getNewName(parent.attributes, "Part");\n        model._parent = parent;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        OperationBuilder.begin("Create Part");\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, "attributes", model);\n        // Create PartView\n        view = new type.UMLPartView();\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        view.model = model;\n        view._parent = diagram;\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        if (options.containerView) {\n            view.containerView = options.containerView;\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", view);\n        if (options.containerView) {\n            OperationBuilder.fieldInsert(options.containerView, "containedViews", view);\n        }\n        // Apply operation\n        OperationBuilder.end();\n        var op = OperationBuilder.getOperation();\n        Repository.doOperation(op);\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function portFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            parent instanceof type.UMLClassifier || parent instanceof type.UMLAttribute,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "UMLPort")\n        );\n        if (parent instanceof type.UMLAttribute) {\n            Helper.assert(\n                parent.type instanceof type.UMLClassifier,\n                "Type should be assigned to have Ports"\n            );\n            parent = parent.type;\n        }\n        // Create Port\n        model = new type.UMLPort();\n        model.name = Core.getNewName(parent.attributes, "Port");\n        model._parent = parent;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        OperationBuilder.begin("Create Port");\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, "attributes", model);\n        // Create PartView\n        view = new type.UMLPortView();\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        view.model = model;\n        view._parent = diagram;\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        if (options.containerView) {\n            view.containerView = options.containerView;\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", view);\n        if (options.containerView) {\n            OperationBuilder.fieldInsert(options.containerView, "containedViews", view);\n        }\n        // Apply operation\n        OperationBuilder.end();\n        var op = OperationBuilder.getOperation();\n        Repository.doOperation(op);\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function lifelineFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            (parent instanceof type.UMLInteraction) && (parent._parent instanceof type.UMLClassifier),\n            Mustache.render(Factory.ERR_INVALID_PARENT, "UMLLifeline")\n        );\n        // Create Role (Attribute)\n        var classifier = parent._parent;\n        var role = new type.UMLAttribute();\n        role.name = Core.getNewName(classifier.attributes, "Role");\n        role._parent = classifier;\n        // Create Lifeline\n        model = new type.UMLLifeline();\n        model.name = Core.getNewName(parent.participants, "Lifeline");\n        model.represent = role;\n        model._parent = parent;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        // Create LifelineView\n        if (diagram instanceof type.UMLSequenceDiagram) {\n            view = new type.UMLSeqLifelineView();\n            if (Math.abs(options.y2 - options.y1) < 200) {\n                options.y2 = options.y1 + 200;\n            }\n        } else if (diagram instanceof type.UMLCommunicationDiagram) {\n            view = new type.UMLCommLifelineView();\n            if (Math.abs(options.x2 - options.x1) < 100) {\n                options.x2 = options.x1 + 100;\n            }\n            if (Math.abs(options.y2 - options.y1) < 30) {\n                options.y2 = options.y1 + 30;\n            }\n        }\n        view._parent = diagram;\n        view.model = model;\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        // Make Command\n        OperationBuilder.begin(\'add element\');\n        OperationBuilder.insert(role);\n        OperationBuilder.fieldInsert(classifier, \'attributes\', role);\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, \'participants\', model);\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n        OperationBuilder.end();\n        Repository.doOperation(OperationBuilder.getOperation());\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function messageFn(parent, diagram, options) {\n        var model, view, connector, connectorView;\n        options.connector = options.connector || null;\n        if (diagram instanceof type.UMLSequenceDiagram) {\n            // if linked to UMLSeqLifelineView\n            if (options.tailView instanceof type.UMLSeqLifelineView) {\n                options.tailView = options.tailView.linePart;\n            }\n            if (options.headView instanceof type.UMLSeqLifelineView) {\n                options.headView = options.headView.linePart;\n            }\n            // If linked to UMLActivationView.\n            if (options.tailView instanceof type.UMLActivationView) {\n                options.tailModel = options.tailView._parent.head.model;\n                options.tailView = options.tailView._parent.head;\n            }\n            if (options.headView instanceof type.UMLActivationView) {\n                options.headModel = options.headView._parent.head.model;\n                options.headView = options.headView._parent.head;\n            }\n            Helper.assert(\n                (options.tailView instanceof type.UMLLinePartView || options.tailView instanceof type.UMLMessageEndpointView) &&\n                (options.headView instanceof type.UMLLinePartView || options.headView instanceof type.UMLMessageEndpointView),\n                Mustache.render(Factory.ERR_INVALID_LINK, "UMLMessage")\n            );\n            model = new type.UMLMessage();\n            model.name = Core.getNewName(parent.messages, "Message");\n            model.connector = options.connector;\n            model.source = options.tailModel;\n            model.target = options.headModel;\n            if (options.modelInitializer) {\n                options.modelInitializer(model);\n            }\n            view = new type.UMLSeqMessageView();\n            view.tail = options.tailView;\n            view.head = options.headView;\n            view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n            if (options.viewInitializer) {\n                options.viewInitializer(view);\n            }\n            Engine.addModelAndView(diagram, model, view, parent, \'messages\');\n        } else if (diagram instanceof type.UMLCommunicationDiagram) {\n            // Connect between two UMLCommLifelineViews\n            if (options.tailView instanceof type.UMLCommLifelineView && options.headView instanceof type.UMLCommLifelineView) {\n                options.direction = options.direction || "forward";\n                connector = new type.UMLConnector();\n                connector._parent = options.tailModel.represent;\n                connector.end1.reference = options.tailModel.represent;\n                connector.end2.reference = options.headModel.represent;\n                connectorView = new type.UMLConnectorView();\n                connectorView.tail = options.tailView;\n                connectorView.head = options.headView;\n                connectorView.model = connector;\n                connectorView._parent = diagram;\n                connectorView.initialize(null, options.x1, options.y1, options.x2, options.y2);\n                // Make Command\n                OperationBuilder.begin(\'add element\');\n                // add connector\n                OperationBuilder.insert(connector);\n                OperationBuilder.fieldInsert(options.tailModel.represent, \'ownedElements\', connector);\n                OperationBuilder.insert(connectorView);\n                OperationBuilder.fieldInsert(diagram, \'ownedViews\', connectorView);\n                // add message\n                model = new type.UMLMessage();\n                model.name = Core.getNewName(parent.messages, "Message");\n                model._parent = parent;\n                model.connector = connector;\n                if (options.direction === "forward") {\n                    model.source = options.tailModel;\n                    model.target = options.headModel;\n                } else { // reverse\n                    model.source = options.headModel;\n                    model.target = options.tailModel;\n                }\n                if (options.modelInitializer) {\n                    options.modelInitializer(model);\n                }\n                view = new type.UMLCommMessageView();\n                view.hostEdge = connectorView;\n                view.model = model;\n                view._parent = diagram;\n                view.initialize(null, connectorView.points.points[0].x, connectorView.points.points[0].y, connectorView.points.points[0].x, connectorView.points.points[0].y);\n                if (options.viewInitializer) {\n                    options.viewInitializer(view);\n                }\n                OperationBuilder.insert(model);\n                OperationBuilder.fieldInsert(parent, \'messages\', model);\n                OperationBuilder.insert(view);\n                OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n                OperationBuilder.end();\n                Repository.doOperation(OperationBuilder.getOperation());\n            } else { // Click on UMLConnectorView\n                options.direction = options.direction || "forward";\n                Helper.assert(\n                    (options.headView instanceof type.UMLConnectorView),\n                    Mustache.render(Factory.ERR_INVALID_LINK, "Message")\n                );\n                model = new type.UMLMessage();\n                model.name = Core.getNewName(parent.messages, "Message");\n                model.connector = options.connector;\n                if (options.direction === "forward") {\n                    model.source = options.headView.tail.model;\n                    model.target = options.headView.head.model;\n                } else { // reverse\n                    model.source = options.headView.head.model;\n                    model.target = options.headView.tail.model;\n                }\n                if (options.modelInitializer) {\n                    options.modelInitializer(model);\n                }\n                view = new type.UMLCommMessageView();\n                view.hostEdge = options.headView;\n                view.initialize(null, options.x2, options.y2, options.x2, options.y2);\n                if (options.viewInitializer) {\n                    options.viewInitializer(view);\n                }\n                Engine.addModelAndView(diagram, model, view, parent, \'messages\');\n            }\n        }\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function foundMessageFn(parent, diagram, options) {\n        var model, view, endpoint, endpointView;\n        // if linked to UMLSeqLifelineView\n        if (options.headView instanceof type.UMLSeqLifelineView) {\n            options.headView = options.headView.linePart;\n        }\n        // If linked to UMLActivationView.\n        if (options.headView instanceof type.UMLActivationView) {\n            options.headModel = options.headView._parent.head.model;\n            options.headView = options.headView._parent.head;\n        }\n        Helper.assert(\n            parent instanceof type.UMLInteraction,\n            Mustache.render(Factory.ERR_INVALID_PARENT, options.modelType)\n        );\n        Helper.assert(\n            (options.headView instanceof type.UMLLinePartView),\n            Mustache.render(Factory.ERR_INVALID_LINK, "UMLFoundMessage")\n        );\n        OperationBuilder.begin(\'add element\');\n        // endpoint\n        endpoint = new type.UMLEndpoint();\n        endpoint.name = Core.getNewName(parent.participants, endpoint.getDisplayClassName());\n        endpoint._parent = parent;\n        OperationBuilder.insert(endpoint);\n        OperationBuilder.fieldInsert(parent, \'participants\', endpoint);\n        endpointView = new type.UMLEndpointView();\n        endpointView.initialize(null, options.x1, options.y1, options.x1, options.y1);\n        endpointView.model = endpoint;\n        endpointView._parent = diagram;\n        OperationBuilder.insert(endpointView);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", endpointView);\n        // message\n        model = new type.UMLMessage();\n        model._parent = parent;\n        model.name = Core.getNewName(parent.messages, "Message");\n        model.source = endpoint;\n        model.target = options.headModel;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, \'messages\', model);\n        view = new type.UMLSeqMessageView();\n        view.model = model;\n        view._parent = diagram;\n        view.tail = endpointView;\n        view.head = options.headView;\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n        OperationBuilder.end();\n        Repository.doOperation(OperationBuilder.getOperation());\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function lostMessageFn(parent, diagram, options) {\n        var model, view, endpoint, endpointView;\n        // if linked to UMLSeqLifelineView\n        if (options.tailView instanceof type.UMLSeqLifelineView) {\n            options.tailView = options.tailView.linePart;\n        }\n        // If linked to UMLActivationView.\n        if (options.tailView instanceof type.UMLActivationView) {\n            options.tailModel = options.tailView._parent.head.model;\n            options.tailView = options.tailView._parent.head;\n        }\n        Helper.assert(\n            parent instanceof type.UMLInteraction,\n            Mustache.render(Factory.ERR_INVALID_PARENT, options.modelType)\n        );\n        Helper.assert(\n            (options.tailView instanceof type.UMLLinePartView),\n            Mustache.render(Factory.ERR_INVALID_LINK, "UMLFoundMessage")\n        );\n        OperationBuilder.begin(\'add element\');\n        // endpoint\n        endpoint = new type.UMLEndpoint();\n        endpoint.name = Core.getNewName(parent.participants, endpoint.getDisplayClassName());\n        endpoint._parent = parent;\n        OperationBuilder.insert(endpoint);\n        OperationBuilder.fieldInsert(parent, \'participants\', endpoint);\n        endpointView = new type.UMLEndpointView();\n        endpointView.initialize(null, options.x2, options.y2, options.x2, options.y2);\n        endpointView.model = endpoint;\n        endpointView._parent = diagram;\n        OperationBuilder.insert(endpointView);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", endpointView);\n        // message\n        model = new type.UMLMessage();\n        model._parent = parent;\n        model.name = Core.getNewName(parent.messages, "Message");\n        model.source = options.tailModel;\n        model.target = endpoint;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, \'messages\', model);\n        view = new type.UMLSeqMessageView();\n        view.model = model;\n        view._parent = diagram;\n        view.tail = options.tailView;\n        view.head = endpointView;\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n        OperationBuilder.end();\n        Repository.doOperation(OperationBuilder.getOperation());\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function stateInvariantFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            options.headModel instanceof type.UMLLifeline,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "State Invariant")\n        );\n        // make command\n        OperationBuilder.begin(\'add element\');\n        // model\n        model = new type.UMLStateInvariant();\n        model.name = Core.getNewName(parent.fragments, "StateInvariant");\n        model.covered = options.headModel;\n        model._parent = parent;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, \'fragments\', model);\n        // view\n        view = new type.UMLStateInvariantView();\n        view.initialize(null, options.x2, options.y2, options.x2, options.y2);\n        view.model = model;\n        view._parent = options.headView;\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(options.headView, \'subViews\', view);\n        OperationBuilder.end();\n        var cmd = OperationBuilder.getOperation();\n        Repository.doOperation(cmd);\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function combinedFragmentFn(parent, diagram, options) {\n        var model, view, operand;\n        Helper.assert(\n            (parent instanceof type.UMLInteraction),\n            Mustache.render(Factory.ERR_INVALID_PARENT, "CombinedFragment")\n        );\n        model = new type.UMLCombinedFragment();\n        model.name = Core.getNewName(parent.fragments, "CombinedFragment");\n        operand = new type.UMLInteractionOperand();\n        operand.name = Core.getNewName(model.operands, "Operand");\n        model.operands.push(operand);\n        operand._parent = model;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        view = new type.UMLCombinedFragmentView();\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        Engine.addModelAndView(diagram, model, view, parent, \'fragments\');\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function connectorFn(parent, diagram, options) {\n        var model, view;\n        if (diagram instanceof type.UMLCommunicationDiagram) {\n            Helper.assert(\n                parent._parent instanceof type.UMLCollaboration,\n                Mustache.render(Factory.ERR_INVALID_PARENT, "Connector")\n            );\n            Helper.assert(\n                (options.tailModel instanceof type.UMLLifeline) && (options.headModel instanceof type.UMLLifeline),\n                Mustache.render(Factory.ERR_INVALID_LINK, "Connector")\n            );\n            Helper.assert(\n                (options.tailModel.represent instanceof type.UMLFeature) && (options.headModel.represent instanceof type.UMLFeature),\n                Mustache.render(Factory.ERR_INVALID_LINK, "Connector")\n            );\n            model = new type.UMLConnector();\n            model.end1.reference = options.tailModel.represent;\n            model.end2.reference = options.headModel.represent;\n            if (options.modelInitializer) {\n                options.modelInitializer(model);\n            }\n            view = new type.UMLConnectorView();\n            view.tail = options.tailView;\n            view.head = options.headView;\n            view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n            if (options.viewInitializer) {\n                options.viewInitializer(view);\n            }\n            Engine.addModelAndView(diagram, model, view, options.tailModel.represent, \'ownedElements\');\n            if (model) {\n                model = Repository.get(model._id);\n            }\n            if (view) {\n                view = Repository.get(view._id);\n            }\n            options.triggerElementCreated(model, view);\n            return view;\n        } else {\n            options.precondition = featureLinkPrecondition;\n            return Factory.defaultUndirectedRelationshipFn(parent, diagram, options);\n        }\n    }\n\n    function stateFn(parent, diagram, options) {\n        var model, view, region;\n        options.regionCount = options.regionCount || 0;\n        options.submachine = options.submachine || null;\n        Helper.assert(\n            parent instanceof type.UMLStateMachine,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "State")\n        );\n        model = new type.UMLState();\n        model.name = Core.getNewName(parent.regions[0].vertices, "State");\n        model.submachine = options.submachine;\n        for (var i = 0; i < options.regionCount; i++) {\n            region = new type.UMLRegion();\n            region._parent = model;\n            model.regions.push(region);\n        }\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        view = new type.UMLStateView();\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        Engine.addModelAndView(diagram, model, view, parent.regions[0], \'vertices\');\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function submachineStateFn(parent, diagram, options) {\n        var model, view, stateMachine = null;\n        Helper.assert(\n            parent instanceof type.UMLStateMachine,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "State")\n        );\n        // Create a StateMachine\n        stateMachine = new type.UMLStateMachine();\n        stateMachine.name = Core.getNewName(parent._parent.ownedElements, stateMachine.getDisplayClassName());\n        stateMachine._parent = parent._parent;\n        OperationBuilder.begin("Create SubmachineState");\n        OperationBuilder.insert(stateMachine);\n        OperationBuilder.fieldInsert(parent._parent, \'ownedElements\', stateMachine);\n\n        // Create model element\n        model = new type.UMLState();\n        model.name = Core.getNewName(parent.regions[0].vertices, model.getDisplayClassName());\n        model.submachine = stateMachine;\n        model._parent = parent.regions[0];\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent.regions[0], \'vertices\', model);\n\n        // Create view element\n        view = new type.UMLStateView();\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        view.model = model;\n        view._parent = diagram;\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        if (options.containerView) {\n            view.containerView = options.containerView;\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", view);\n        if (options.containerView) {\n            OperationBuilder.fieldInsert(options.containerView, "containedViews", view);\n        }\n\n        // Apply operation\n        OperationBuilder.end();\n        var op = OperationBuilder.getOperation();\n        Repository.doOperation(op);\n\n        // Trigger event\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view || model;\n    }\n\n    function pseudostateFn(parent, diagram, options) {\n        var model, view;\n        options.pseudostateKind = options.pseudostateKind || UML.PSK_INITIAL;\n        Helper.assert(\n            parent instanceof type.UMLStateMachine,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "Pseudostate")\n        );\n        model = new type.UMLPseudostate();\n        model.kind = options.pseudostateKind;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        view = new type.UMLPseudostateView();\n        view.initialize(null, options.x2, options.y2, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        Engine.addModelAndView(diagram, model, view, parent.regions[0], \'vertices\');\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function finalStateFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            parent instanceof type.UMLStateMachine,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "FinalState")\n        );\n        model = new type.UMLFinalState();\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        view = new type.UMLFinalStateView();\n        view.initialize(null, options.x2, options.y2, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        Engine.addModelAndView(diagram, model, view, parent.regions[0], \'vertices\');\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function connectionPointReferenceFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            parent instanceof type.UMLState,\n            Mustache.render(Factory.ERR_INVALID_PARENT, "ConnectionPointReference")\n        );\n        model = new type.UMLConnectionPointReference();\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        view = new type.UMLConnectionPointReferenceView();\n        view.initialize(null, options.x2, options.y2, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        Engine.addModelAndView(diagram, model, view, parent, \'connections\', options.containerView);\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function transitionFn(parent, diagram, options) {\n        var model, view;\n        Helper.assert(\n            (options.tailModel instanceof type.UMLVertex) && (options.headModel instanceof type.UMLVertex),\n            Mustache.render(Factory.ERR_INVALID_LINK, "Transition")\n        );\n        model = new type.UMLTransition();\n        model.source = options.tailModel;\n        model.target = options.headModel;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        view = new type.UMLTransitionView();\n        view.tail = options.tailView;\n        view.head = options.headView;\n        view.initialize(null, options.x1, options.y1, options.x1, options.y1);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        Engine.addModelAndView(diagram, model, view, parent.regions[0], \'transitions\');\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view;\n    }\n\n    function objectFlowFn(parent, diagram, options) {\n        var model, pin1, pin2, view, pinView1, pinView2;\n        options.parent = parent;\n        options.diagram = diagram;\n        flowPrecondition(options);\n\n        // Create model element\n        model = new type.UMLObjectFlow();\n        model._parent = parent;\n        model.source = options.tailModel;\n        model.target = options.headModel;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        OperationBuilder.begin("Create " + model.getDisplayClassName());\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, "edges", model);\n\n        // Create view element\n        view = new type.UMLObjectFlowView();\n        view.model = model;\n        view._parent = diagram;\n        view.tail = options.tailView;\n        view.head = options.headView;\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", view);\n\n        // Create Input/OutputPins when connect between two Actions\n        if (options.tailModel instanceof type.UMLAction && options.headModel instanceof type.UMLAction) {\n            // OutputPin\n            pin1 = new type.UMLOutputPin();\n            pin1._parent = options.tailModel;\n            OperationBuilder.insert(pin1);\n            OperationBuilder.fieldInsert(options.tailModel, "outputs", pin1);\n            OperationBuilder.fieldAssign(model, "source", pin1);\n            pinView1 = new type.UMLOutputPinView();\n            pinView1.model = pin1;\n            pinView1._parent = diagram;\n            pinView1.containerView = options.tailView;\n            pinView1.initialize(null, options.x1, options.y1, options.x1, options.y1);\n            OperationBuilder.insert(pinView1);\n            OperationBuilder.fieldInsert(diagram, "ownedViews", pinView1);\n            OperationBuilder.fieldInsert(options.tailView, "containedViews", pinView1);\n            OperationBuilder.fieldAssign(view, "tail", pinView1);\n            // InputPin\n            pin2 = new type.UMLInputPin();\n            pin2._parent = options.headModel;\n            OperationBuilder.insert(pin2);\n            OperationBuilder.fieldInsert(options.headModel, "inputs", pin2);\n            OperationBuilder.fieldAssign(model, "target", pin2);\n            pinView2 = new type.UMLInputPinView();\n            pinView2.model = pin2;\n            pinView2._parent = diagram;\n            pinView2.containerView = options.headView;\n            pinView2.initialize(null, options.x2, options.y2, options.x2, options.y2);\n            OperationBuilder.insert(pinView2);\n            OperationBuilder.fieldInsert(diagram, "ownedViews", pinView2);\n            OperationBuilder.fieldInsert(options.headView, "containedViews", pinView2);\n            OperationBuilder.fieldAssign(view, "head", pinView2);\n        }\n\n        // Apply operation\n        OperationBuilder.end();\n        var op = OperationBuilder.getOperation();\n        Repository.doOperation(op);\n\n        // Trigger event\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view || model;\n    }\n\n\n    function exceptionHandlerFn(parent, diagram, options) {\n        var model, view, pin, pinView;\n        options.parent = parent;\n        options.diagram = diagram;\n        actionLinkPrecondition(options);\n\n        // Create InputPin for exceptionInput\n        OperationBuilder.begin("Create ExceptionHandler");\n        pin = new type.UMLInputPin();\n        pin._parent = options.headModel;\n        OperationBuilder.insert(pin);\n        OperationBuilder.fieldInsert(options.headModel, "inputs", pin);\n        pinView = new type.UMLInputPinView();\n        pinView.model = pin;\n        pinView._parent = diagram;\n        pinView.containerView = options.headView;\n        pinView.initialize(null, options.x2, options.y2, options.x2, options.y2);\n        OperationBuilder.insert(pinView);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", pinView);\n        OperationBuilder.fieldInsert(options.headView, "containedViews", pinView);\n\n        // Create model element\n        model = new type.UMLExceptionHandler();\n        model._parent = parent;\n        model.source = options.tailModel;\n        model.target = pin;\n        model.handlerBody = options.headModel;\n        if (options.modelInitializer) {\n            options.modelInitializer(model);\n        }\n        OperationBuilder.insert(model);\n        OperationBuilder.fieldInsert(parent, "edges", model);\n\n        // Create view element\n        view = new type.UMLExceptionHandlerView();\n        view.model = model;\n        view._parent = diagram;\n        view.tail = options.tailView;\n        view.head = pinView;\n        view.initialize(null, options.x1, options.y1, options.x2, options.y2);\n        if (options.viewInitializer) {\n            options.viewInitializer(view);\n        }\n        OperationBuilder.insert(view);\n        OperationBuilder.fieldInsert(diagram, "ownedViews", view);\n\n        // Apply operation\n        OperationBuilder.end();\n        var op = OperationBuilder.getOperation();\n        Repository.doOperation(op);\n\n        // Trigger event\n        if (model) {\n            model = Repository.get(model._id);\n        }\n        if (view) {\n            view = Repository.get(view._id);\n        }\n        options.triggerElementCreated(model, view);\n        return view || model;\n    }\n\n\n    // Create view of functions ................................................\n\n    function viewForGeneralDiagramFn(model, diagram, options) {\n        var x             = options.x || 0,\n            y             = options.y || 0,\n            editor        = options.editor || DiagramManager.getEditor(),\n            containerView = options.containerView || null;\n\n        // Port\n        if (model instanceof type.UMLPort) {\n            if (containerView instanceof type.UMLClassifierView && containerView.model === model._parent) {\n                return Factory.createViewAndRelationships(editor, x, y, model, containerView);\n            } else {\n                Dialogs.showAlertDialog("Port should be dropped on it\'s parent Classifier");\n            }\n\n        // Part\n        } else if (model instanceof type.UMLAttribute) {\n            if (containerView instanceof type.UMLClassifierView && containerView.model === model._parent) {\n                return Factory.createViewAndRelationships(editor, x, y, model, containerView);\n            } else {\n                Dialogs.showAlertDialog("Part (Attribute) should be dropped on it\'s parent Classifier");\n            }\n\n        // Connector\n        } else if (model instanceof type.UMLConnector) {\n            var connectorView      = diagram.getViewOf(model),\n                port1View          = diagram.getViewOf(model.end1.reference, type.UMLPortView) || diagram.getViewOf(model.end1.reference, type.UMLPartView),\n                port2View          = diagram.getViewOf(model.end2.reference, type.UMLPortView) || diagram.getViewOf(model.end2.reference, type.UMLPartView);\n            if (connectorView) {\n                editor.selectView(connectorView);\n                editor.selectAdditionalView(port1View);\n                editor.selectAdditionalView(port2View);\n                Dialogs.showAlertDialog("Connector View is already existed in this Diagram.");\n            } else {\n                if (!port2View) {\n                    var port2ContainerView = diagram.getViewOf(model.end2.reference._parent);\n                    if (!port2ContainerView) {\n                        Factory.createViewAndRelationships(editor, x, y, model.end2.reference._parent);\n                        port2ContainerView = diagram.getViewOf(model.end2.reference._parent);\n                    }\n                    Factory.createViewAndRelationships(editor, x, y, model.end2.reference, port2ContainerView);\n                }\n                if (!port1View) {\n                    var port1ContainerView = diagram.getViewOf(model.end1.reference._parent);\n                    if (!port1ContainerView) {\n                        Factory.createViewAndRelationships(editor, x, y + 100, model.end1.reference._parent);\n                        port1ContainerView = diagram.getViewOf(model.end1.reference._parent);\n                    }\n                    Factory.createViewAndRelationships(editor, x, y + 100, model.end1.reference, port1ContainerView);\n                }\n                if (port1View && port2View) {\n                    connectorView = new type.UMLConnectorView();\n                    connectorView.model = model;\n                    connectorView.tail = port1View;\n                    connectorView.head = port2View;\n                    connectorView.initialize(null, connectorView.tail.left, connectorView.tail.top, connectorView.head.left, connectorView.head.top);\n                    if (options.viewInitializer) {\n                        options.viewInitializer(connectorView);\n                    }\n                    Engine.addViews(diagram, [connectorView]);\n                    if (connectorView) {\n                        connectorView = Repository.get(connectorView._id);\n                    }\n                    options.triggerElementCreated(null, connectorView);\n                    editor.selectView(connectorView);\n                }\n                return connectorView;\n            }\n\n        // AssociationClass\n        } else if (model instanceof type.UMLClass) {\n            var assoClassLink = null;\n            _.each(model.ownedElements, function (e) {\n               if (e instanceof type.UMLAssociationClassLink) {\n                   assoClassLink = e;\n               }\n            });\n            if (assoClassLink && assoClassLink.classSide === model) {\n                var linkView  = diagram.getViewOf(assoClassLink),\n                    classView = diagram.getViewOf(model),\n                    assoView  = diagram.getViewOf(assoClassLink.associationSide);\n                if (linkView) {\n                    editor.selectView(linkView);\n                    editor.selectView(classView);\n                    editor.selectView(assoView);\n                    Dialogs.showAlertDialog("AssociationClass View is already existed in this Diagram.");\n                } else {\n                    if (!classView) {\n                        Factory.createViewAndRelationships(editor, x, y, model);\n                        classView = diagram.getViewOf(model);\n                    }\n                    if (assoView) {\n                        linkView = new type.UMLAssociationClassLinkView();\n                        linkView.model = assoClassLink;\n                        linkView.tail = classView;\n                        linkView.head = assoView;\n                        linkView.initialize(null, linkView.tail.left, linkView.tail.top, linkView.head.left, linkView.head.top);\n                        if (options.viewInitializer) {\n                            options.viewInitializer(linkView);\n                        }\n                        Engine.addViews(diagram, [linkView]);\n                        if (linkView) {\n                            linkView = Repository.get(linkView._id);\n                        }\n                        options.triggerElementCreated(null, linkView);\n                        editor.selectView(linkView);\n                    }\n                    return linkView;\n                }\n            } else {\n                return Factory.createViewAndRelationships(editor, x, y, model);\n            }\n\n        // Directed Relationships\n        } else if (model instanceof type.DirectedRelationship) {\n            var directedView    = diagram.getViewOf(model),\n                sourceView = diagram.getViewOf(model.source),\n                targetView = diagram.getViewOf(model.target);\n            if (directedView) {\n                editor.selectView(directedView);\n                editor.selectAdditionalView(sourceView);\n                editor.selectAdditionalView(targetView);\n                Dialogs.showAlertDialog("Relationship View is already existed in this Diagram.");\n            } else {\n                if (!targetView) {\n                    Factory.createViewAndRelationships(editor, x, y, model.target);\n                }\n                if (!sourceView) {\n                    Factory.createViewAndRelationships(editor, x, y + 100, model.source);\n                }\n                if (targetView && sourceView) {\n                    var DirectedViewType = UMLUtils.getViewTypeOf(model);\n                    if (DirectedViewType) {\n                        directedView = new DirectedViewType();\n                        directedView.model = model;\n                        directedView.tail = sourceView;\n                        directedView.head = targetView;\n                        directedView.initialize(null, directedView.tail.left, directedView.tail.top, directedView.head.left, directedView.head.top);\n                        if (options.viewInitializer) {\n                            options.viewInitializer(directedView);\n                        }\n                        Engine.addViews(diagram, [directedView]);\n                        if (directedView) {\n                            directedView = Repository.get(directedView._id);\n                        }\n                        options.triggerElementCreated(null, directedView);\n                        editor.selectView(directedView);\n                    }\n                }\n                return directedView;\n            }\n\n        // Undirected Relationships\n        } else if (model instanceof type.UndirectedRelationship) {\n            var undirectedView    = diagram.getViewOf(model),\n                end1View = diagram.getViewOf(model.end1.reference),\n                end2View = diagram.getViewOf(model.end2.reference);\n            if (undirectedView) {\n                editor.selectView(undirectedView);\n                editor.selectAdditionalView(end1View);\n                editor.selectAdditionalView(end2View);\n                Dialogs.showAlertDialog("Relationship View is already existed in this Diagram.");\n            } else {\n                if (!end2View) {\n                    Factory.createViewAndRelationships(editor, x, y, model.end2.reference);\n                }\n                if (!end1View) {\n                    Factory.createViewAndRelationships(editor, x, y + 100, model.end1.reference);\n                }\n                if (end1View && end2View) {\n                    var UndirectedViewType = UMLUtils.getViewTypeOf(model);\n                    if (UndirectedViewType) {\n                        undirectedView = new UndirectedViewType();\n                        undirectedView.model = model;\n                        undirectedView.tail = end1View;\n                        undirectedView.head = end2View;\n                        undirectedView.initialize(null, undirectedView.tail.left, undirectedView.tail.top, undirectedView.head.left, undirectedView.head.top);\n                        if (options.viewInitializer) {\n                            options.viewInitializer(undirectedView);\n                        }\n                        Engine.addViews(diagram, [undirectedView]);\n                        if (undirectedView) {\n                            undirectedView = Repository.get(undirectedView._id);\n                        }\n                        editor.selectView(undirectedView);\n                    }\n                }\n                return undirectedView;\n            }\n        } else if (model instanceof type.Diagram) {\n            return Factory.defaultViewOnDiagramFn(model, diagram, options);\n        } else {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        }\n    }\n\n\n    function viewForSequenceDiagramFn(model, diagram, options) {\n        var x             = options.x || 0,\n            y             = options.y || 0,\n            editor        = options.editor || null,\n            containerView = options.containerView || null,\n            view;\n\n        if (model instanceof type.UMLClassifier) {\n            // Create Role (Attribute)\n            var parent = diagram._parent;\n            var classifier = parent._parent;\n            var role = new type.UMLAttribute();\n            role.name = Core.getNewName(classifier.attributes, "Role");\n            role.type = model;\n            role._parent = classifier;\n            // Create Lifeline\n            var lifeline = new type.UMLLifeline();\n            lifeline.name = Core.getNewName(parent.participants, "Lifeline");\n            lifeline.represent = role;\n            lifeline._parent = parent;\n            // Create LifelineView\n            view = new type.UMLSeqLifelineView();\n            view._parent = diagram;\n            view.model = lifeline;\n            if ((model instanceof type.UMLActor) ||\n                (model instanceof type.UMLUseCase) ||\n                (model instanceof type.UMLInterface) ||\n                (model instanceof type.UMLArtifact) ||\n                (model instanceof type.UMLComponent) ||\n                (model instanceof type.UMLNode) ||\n                (model.stereotype && model.stereotype.icon)) {\n                view.stereotypeDisplay = UML.SD_ICON;\n            }\n            view.initialize(null, x, y, x, y);\n            if (options.viewInitializer) {\n                options.viewInitializer(view);\n            }\n            // Make Command\n            OperationBuilder.begin(\'add element\');\n            OperationBuilder.insert(role);\n            OperationBuilder.fieldInsert(classifier, \'attributes\', role);\n            OperationBuilder.insert(lifeline);\n            OperationBuilder.fieldInsert(parent, \'participants\', lifeline);\n            OperationBuilder.insert(view);\n            OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n            OperationBuilder.end();\n            Repository.doOperation(OperationBuilder.getOperation());\n            if (view) {\n                view = Repository.get(view._id);\n            }\n            options.triggerElementCreated(null, view);\n            return view;\n        } else if (model instanceof type.UMLCombinedFragment) {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        } else if (model instanceof type.UMLInteraction) {\n            options.x1 = x;\n            options.y1 = y;\n            options.x2 = x;\n            options.y2 = y;\n            options.modelInitializer = function (elem) {\n                elem.refersTo = model;\n            };\n            return Factory.createModelAndView("UMLInteractionUse", diagram._parent, diagram, options);\n        } else if (model instanceof type.UMLInteractionUse) {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        } else if (model instanceof type.UMLContinuation) {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        } else if (model instanceof type.UMLLifeline) {\n            view = new type.UMLSeqLifelineView();\n            view.model = model;\n            view.initialize(null, x, y, x, y);\n            if (options.viewInitializer) {\n                options.viewInitializer(view);\n            }\n            Engine.addViews(diagram, [view]);\n            if (view) {\n                view = Repository.get(view._id);\n            }\n            options.triggerElementCreated(null, view);\n            return view;\n        } else if (model instanceof type.UMLGate) {\n            view = new type.UMLGateView();\n            view.model = model;\n            view.initialize(null, x, y, x, y);\n            if (options.viewInitializer) {\n                options.viewInitializer(view);\n            }\n            Engine.addViews(diagram, [view]);\n            if (view) {\n                view = Repository.get(view._id);\n            }\n            options.triggerElementCreated(null, view);\n            return view;\n        } else if (model instanceof type.UMLEndpoint) {\n            view = new type.UMLEndpointView();\n            view.model = model;\n            view.initialize(null, x, y, x, y);\n            if (options.viewInitializer) {\n                options.viewInitializer(view);\n            }\n            Engine.addViews(diagram, [view]);\n            if (view) {\n                view = Repository.get(view._id);\n            }\n            options.triggerElementCreated(null, view);\n            return view;\n        } else if (model instanceof type.UMLStateInvariant) {\n            var lifelineView = null;\n            _.each(diagram.ownedViews, function (v) {\n                if (v instanceof type.UMLSeqLifelineView && v.model === model.covered) {\n                    lifelineView = v;\n                }\n            });\n            if (lifelineView) {\n                view = new type.UMLStateInvariantView();\n                view.initialize(null, x, y, x, y);\n                view.model = model;\n                view._parent = lifelineView;\n                if (options.viewInitializer) {\n                    options.viewInitializer(view);\n                }\n                OperationBuilder.begin(\'add element\');\n                OperationBuilder.insert(view);\n                OperationBuilder.fieldInsert(lifelineView, \'subViews\', view);\n                OperationBuilder.end();\n                Repository.doOperation(OperationBuilder.getOperation());\n                if (view) {\n                    view = Repository.get(view._id);\n                }\n                options.triggerElementCreated(null, view);\n            }\n            return view;\n        } else if (model instanceof type.UMLMessage) {\n            var sourceView, targetView;\n            _.each(diagram.ownedViews, function (v) {\n                if (v.model === model.source) {\n                    sourceView = v;\n                } else if (v.model === model.target) {\n                    targetView = v;\n                }\n            });\n            sourceView = sourceView instanceof type.UMLSeqLifelineView ? sourceView.linePart : sourceView;\n            targetView = targetView instanceof type.UMLSeqLifelineView ? targetView.linePart : targetView;\n            if (sourceView && targetView) {\n                view = new type.UMLSeqMessageView();\n                view.tail = sourceView;\n                view.head = targetView;\n                view.model = model;\n                view.initialize(null, sourceView.left, y, targetView.left, y);\n                if (options.viewInitializer) {\n                    options.viewInitializer(view);\n                }\n                Engine.addViews(diagram, [view]);\n                if (view) {\n                    view = Repository.get(view._id);\n                }\n                options.triggerElementCreated(null, view);\n            }\n            return view;\n        } else if (model instanceof type.Diagram) {\n            return Factory.defaultViewOnDiagramFn(model, diagram, options);\n        } else {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        }\n    }\n\n    function viewForCommunicationDiagramFn(model, diagram, options) {\n        var x             = options.x || 0,\n            y             = options.y || 0,\n            editor        = options.editor || null,\n            containerView = options.containerView || null,\n            view;\n\n        if (model instanceof type.UMLClassifier) {\n            // Create Role (Attribute)\n            var parent = diagram._parent;\n            var classifier = parent._parent;\n            var role = new type.UMLAttribute();\n            role.name = Core.getNewName(classifier.attributes, "Role");\n            role.type = model;\n            role._parent = classifier;\n            // Create Lifeline\n            var lifeline = new type.UMLLifeline();\n            lifeline.name = Core.getNewName(parent.participants, "Lifeline");\n            lifeline.represent = role;\n            lifeline._parent = parent;\n            // Create LifelineView\n            view = new type.UMLCommLifelineView();\n            view._parent = diagram;\n            view.model = lifeline;\n            if ((model instanceof type.UMLActor) ||\n                (model instanceof type.UMLUseCase) ||\n                (model instanceof type.UMLInterface) ||\n                (model instanceof type.UMLArtifact) ||\n                (model instanceof type.UMLComponent) ||\n                (model instanceof type.UMLNode) ||\n                (model.stereotype && model.stereotype.icon)) {\n                view.stereotypeDisplay = UML.SD_ICON;\n            }\n            view.initialize(null, x, y, x, y);\n            if (options.viewInitializer) {\n                options.viewInitializer(view);\n            }\n            // Make Command\n            OperationBuilder.begin(\'add element\');\n            OperationBuilder.insert(role);\n            OperationBuilder.fieldInsert(classifier, \'attributes\', role);\n            OperationBuilder.insert(lifeline);\n            OperationBuilder.fieldInsert(parent, \'participants\', lifeline);\n            OperationBuilder.insert(view);\n            OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n            OperationBuilder.end();\n            Repository.doOperation(OperationBuilder.getOperation());\n            if (view) {\n                view = Repository.get(view._id);\n            }\n            options.triggerElementCreated(null, view);\n            return view;\n        } else if (model instanceof type.UMLLifeline) {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        } else if (model instanceof type.UMLConnector) {\n            var _ev1 = _.find(diagram.ownedViews, function (v) {\n                    return (v instanceof type.UMLCommLifelineView && v.model.represent === model.end1.reference);\n                }),\n                _ev2 = _.find(diagram.ownedViews, function (v) {\n                    return (v instanceof type.UMLCommLifelineView && v.model.represent === model.end2.reference);\n                });\n            if (_ev1 && _ev2) {\n                view = new type.UMLConnectorView();\n                view._parent = diagram;\n                view.model = model;\n                view.tail = _ev1;\n                view.head = _ev2;\n                view.initialize(null, _ev1.left, _ev1.top, _ev2.left, _ev2.top);\n                if (options.viewInitializer) {\n                    options.viewInitializer(view);\n                }\n                Engine.addViews(diagram, [view]);\n                if (view) {\n                    view = Repository.get(view._id);\n                }\n                options.triggerElementCreated(null, view);\n            }\n            return view;\n        } else if (model instanceof type.UMLMessage) {\n            var connector, connectorView = null;\n            if (model.connector) {\n                _.each(diagram.ownedViews, function (v) {\n                    if (v instanceof type.UMLConnectorView && v.model === model.connector) {\n                        connectorView = v;\n                    }\n                });\n            } else {\n                var _tailView = _.find(diagram.ownedViews, function (v) { return v.model === model.source; }),\n                    _headView = _.find(diagram.ownedViews, function (v) { return v.model === model.target; });\n                if (_tailView && _headView) {\n                    connector = new type.UMLConnector();\n                    connector.end1.reference = _tailView.model;\n                    connector.end2.reference = _headView.model;\n                    connectorView = new type.UMLConnectorView();\n                    connectorView._parent = diagram;\n                    connectorView.model = connector;\n                    connectorView.tail = _tailView;\n                    connectorView.head = _headView;\n                    connectorView.initialize(null, _tailView.left, _tailView.top, _headView.left, _headView.top);\n                    Engine.addModelAndView(diagram, connector, connectorView, _headView.model.represent, \'ownedElements\');\n                    Engine.setProperty(model, \'connector\', connector);\n                }\n            }\n            if (connectorView) {\n                view = new type.UMLCommMessageView();\n                view.model = model;\n                view.hostEdge = connectorView;\n                view.initialize(null, x, y, x, y);\n                if (options.viewInitializer) {\n                    options.viewInitializer(view);\n                }\n                Engine.addViews(diagram, [view]);\n                if (view) {\n                    view = Repository.get(view._id);\n                }\n                options.triggerElementCreated(null, view);\n            }\n            return view;\n        } else if (model instanceof type.Diagram) {\n            return Factory.defaultViewOnDiagramFn(model, diagram, options);\n        } else {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        }\n    }\n\n\n    function viewForStatechartDiagramFn(model, diagram, options) {\n        var x             = options.x || 0,\n            y             = options.y || 0,\n            editor        = options.editor || null,\n            containerView = options.containerView || null;\n\n        if (model instanceof type.UMLConnectionPointReference) {\n            if (containerView instanceof type.UMLStateView && containerView.model === model._parent) {\n                return Factory.createViewAndRelationships(editor, x, y, model, containerView);\n            } else {\n                Dialogs.showAlertDialog("ConnectionPointReference should be dropped on it\'s parent State");\n            }\n        } else if (model instanceof type.UMLStateMachine) {\n            options.x1 = x;\n            options.y1 = y;\n            options.x2 = x;\n            options.y2 = y;\n            options.submachine = model;\n            return Factory.createModelAndView("UMLState", diagram._parent, diagram, options);\n        } else if (model instanceof type.Diagram) {\n            return Factory.defaultViewOnDiagramFn(model, diagram, options);\n        } else {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        }\n    }\n\n    function viewForActivityDiagramFn(model, diagram, options) {\n        var x             = options.x || 0,\n            y             = options.y || 0,\n            editor        = options.editor || null,\n            containerView = options.containerView || null;\n\n        // InputPin/OutputPin\n        if (model instanceof type.UMLInputPin || model instanceof type.UMLOutputPin) {\n            if (containerView instanceof type.UMLActionView && containerView.model === model._parent) {\n                return Factory.createViewAndRelationships(editor, x, y, model, containerView);\n            } else {\n                Dialogs.showAlertDialog("Pin should be dropped on it\'s parent Action");\n            }\n        } else if (model instanceof type.UMLActivity) {\n            // Create Action\n            var parent = diagram._parent;\n            var action = new type.UMLAction();\n            action.name = "";\n            action.subactivity = model;\n            action._parent = model;\n            // Create ActionView\n            var view = new type.UMLActionView();\n            view._parent = diagram;\n            view.model = action;\n            view.initialize(null, x, y, x, y);\n            if (options.viewInitializer) {\n                options.viewInitializer(view);\n            }\n            // Make Command\n            OperationBuilder.begin(\'add element\');\n            OperationBuilder.insert(action);\n            OperationBuilder.fieldInsert(parent, \'nodes\', action);\n            OperationBuilder.insert(view);\n            OperationBuilder.fieldInsert(diagram, \'ownedViews\', view);\n            OperationBuilder.end();\n            Repository.doOperation(OperationBuilder.getOperation());\n            if (view) {\n                view = Repository.get(view._id);\n            }\n            options.triggerElementCreated(null, view);\n            return view;\n        } else if (model instanceof type.Diagram) {\n            return Factory.defaultViewOnDiagramFn(model, diagram, options);\n        } else {\n            return Factory.createViewAndRelationships(editor, x, y, model);\n        }\n    }\n\n\n    // ========================================================================\n    // Create Diagram\n    // ========================================================================\n\n    Factory.registerDiagramFn("UMLClassDiagram",              structuralDiagramFn);\n    Factory.registerDiagramFn("UMLCompositeStructureDiagram", compositeStructureDiagramFn);\n    Factory.registerDiagramFn("UMLPackageDiagram",            structuralDiagramFn);\n    Factory.registerDiagramFn("UMLObjectDiagram",             structuralDiagramFn);\n    Factory.registerDiagramFn("UMLComponentDiagram",          structuralDiagramFn);\n    Factory.registerDiagramFn("UMLDeploymentDiagram",         structuralDiagramFn);\n    Factory.registerDiagramFn("UMLProfileDiagram",            profileDiagramFn);\n    Factory.registerDiagramFn("UMLUseCaseDiagram",            structuralDiagramFn);\n    Factory.registerDiagramFn("UMLSequenceDiagram",           sequenceDiagramFn);\n    Factory.registerDiagramFn("UMLCommunicationDiagram",      communicationDiagramFn);\n    Factory.registerDiagramFn("UMLStatechartDiagram",         statechartDiagramFn);\n    Factory.registerDiagramFn("UMLActivityDiagram",           activityDiagramFn);\n\n    // ========================================================================\n    // Create Model\n    // ========================================================================\n\n    // Packages\n    Factory.registerModelFn("UMLModel",     Factory.defaultModelFn);\n    Factory.registerModelFn("UMLPackage",   Factory.defaultModelFn);\n    Factory.registerModelFn("UMLSubsystem", Factory.defaultModelFn);\n    Factory.registerModelFn("UMLProfile",   Factory.defaultModelFn);\n    // Classifiers\n    Factory.registerModelFn("UMLClass",         Factory.defaultModelFn);\n    Factory.registerModelFn("UMLInterface",     Factory.defaultModelFn);\n    Factory.registerModelFn("UMLSignal",        Factory.defaultModelFn);\n    Factory.registerModelFn("UMLDataType",      Factory.defaultModelFn);\n    Factory.registerModelFn("UMLPrimitiveType", Factory.defaultModelFn);\n    Factory.registerModelFn("UMLEnumeration",   Factory.defaultModelFn);\n    Factory.registerModelFn("UMLArtifact",      Factory.defaultModelFn);\n    Factory.registerModelFn("UMLComponent",     Factory.defaultModelFn);\n    Factory.registerModelFn("UMLNode",          Factory.defaultModelFn);\n    Factory.registerModelFn("UMLUseCase",       Factory.defaultModelFn);\n    Factory.registerModelFn("UMLActor",         Factory.defaultModelFn);\n    Factory.registerModelFn("UMLStereotype",    Factory.defaultModelFn);\n    // Instances\n    Factory.registerModelFn("UMLObject",            Factory.defaultModelFn);\n    Factory.registerModelFn("UMLArtifactInstance",  Factory.defaultModelFn);\n    Factory.registerModelFn("UMLComponentInstance", Factory.defaultModelFn);\n    Factory.registerModelFn("UMLNodeInstance",      Factory.defaultModelFn);\n    // Behaviors\n    Factory.registerModelFn("UMLCollaboration",      Factory.defaultModelFn);\n    Factory.registerModelFn("UMLInteraction",        Factory.defaultModelFn);\n    Factory.registerModelFn("UMLStateMachine",       stateMachineFn);\n    Factory.registerModelFn("UMLActivity",           Factory.defaultModelFn);\n    Factory.registerModelFn("UMLOpaqueBehavior",     Factory.defaultModelFn);\n    Factory.registerModelFn("UMLInteractionOperand", Factory.defaultModelFn);\n    // Features\n    Factory.registerModelFn("UMLTemplateParameter",  Factory.defaultModelFn);\n    Factory.registerModelFn("UMLParameter",          Factory.defaultModelFn);\n    Factory.registerModelFn("UMLEnumerationLiteral", Factory.defaultModelFn);\n    Factory.registerModelFn("UMLAttribute",          Factory.defaultModelFn);\n    Factory.registerModelFn("UMLPort",               Factory.defaultModelFn);\n    Factory.registerModelFn("UMLOperation",          Factory.defaultModelFn);\n    Factory.registerModelFn("UMLReception",          Factory.defaultModelFn);\n    Factory.registerModelFn("UMLExtensionPoint",     Factory.defaultModelFn);\n    Factory.registerModelFn("UMLSlot",               Factory.defaultModelFn);\n    // States\n    Factory.registerModelFn("UMLState",  Factory.defaultModelFn);\n    Factory.registerModelFn("UMLRegion", Factory.defaultModelFn);\n    // Actions\n    Factory.registerModelFn("UMLAction", Factory.defaultModelFn);\n    Factory.registerModelFn("UMLEvent",  Factory.defaultModelFn);\n    // Common\n    Factory.registerModelFn("UMLConstraint", Factory.defaultModelFn);\n\n    // ========================================================================\n    // Create Model And View\n    // ========================================================================\n\n    // Classes\n    Factory.registerModelAndViewFn("UMLClass",                Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLInterface",            Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLSignal",               Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLDataType",             Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLPrimitiveType",        Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLEnumeration",          Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLAssociation",          Factory.defaultUndirectedRelationshipFn, { precondition: classifierLinkPrecondition });\n    Factory.registerModelAndViewFn("UMLAssociationClass",     associationClassFn);\n    Factory.registerModelAndViewFn("UMLDependency",           Factory.defaultDirectedRelationshipFn, { precondition: modelElementLinkPrecondition } );\n    Factory.registerModelAndViewFn("UMLGeneralization",       Factory.defaultDirectedRelationshipFn, { precondition: classifierLinkPrecondition } );\n    Factory.registerModelAndViewFn("UMLInterfaceRealization", interfaceRealizationFn);\n\n    // Packages\n    Factory.registerModelAndViewFn("UMLModel",       Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLPackage",     Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLSubsystem",   Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLContainment", containmentFn);\n\n    // Composite Structures\n    Factory.registerModelAndViewFn("UMLPort",             portFn);\n    Factory.registerModelAndViewFn("UMLPart",             partFn);\n    Factory.registerModelAndViewFn("UMLCollaboration",    Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLCollaborationUse", Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLConnector",        connectorFn);\n    Factory.registerModelAndViewFn("UMLRoleBinding",      Factory.defaultDirectedRelationshipFn, { precondition: roleBindingPrecondition });\n    Factory.registerModelAndViewFn("UMLRealization",      Factory.defaultDirectedRelationshipFn, { precondition: modelElementLinkPrecondition });\n\n    // Components\n    Factory.registerModelAndViewFn("UMLArtifact",             Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLComponent",            Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLComponentRealization", Factory.defaultDirectedRelationshipFn, { precondition: componentRealizationPrecondition });\n\n    // Deployments\n    Factory.registerModelAndViewFn("UMLNode",              Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLDeployment",        Factory.defaultDirectedRelationshipFn, { precondition: deploymentPrecondition });\n    Factory.registerModelAndViewFn("UMLCommunicationPath", Factory.defaultUndirectedRelationshipFn, { precondition: nodeLinkPrecondition });\n\n    // Instances\n    Factory.registerModelAndViewFn("UMLObject",            Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLArtifactInstance",  Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLComponentInstance", Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLNodeInstance",      Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLLink",              Factory.defaultUndirectedRelationshipFn, { precondition: instanceLinkPrecondition });\n\n    // Use Cases\n    Factory.registerModelAndViewFn("UMLUseCaseSubject", Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLUseCase",        Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLActor",          Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLInclude",        Factory.defaultDirectedRelationshipFn, { precondition: useCaseLinkPrecondition });\n    Factory.registerModelAndViewFn("UMLExtend",         Factory.defaultDirectedRelationshipFn, { precondition: useCaseLinkPrecondition });\n\n    // Interactions\n    Factory.registerModelAndViewFn("UMLLifeline",         lifelineFn);\n    Factory.registerModelAndViewFn("UMLEndpoint",         Factory.defaultModelAndViewFn, { precondition: interactionPrecondition, field: \'participants\' });\n    Factory.registerModelAndViewFn("UMLGate",             Factory.defaultModelAndViewFn, { precondition: interactionPrecondition, field: \'participants\' });\n    Factory.registerModelAndViewFn("UMLMessage",          messageFn);\n    Factory.registerModelAndViewFn("UMLFoundMessage",     foundMessageFn);\n    Factory.registerModelAndViewFn("UMLLostMessage",      lostMessageFn);\n    Factory.registerModelAndViewFn("UMLStateInvariant",   stateInvariantFn);\n    Factory.registerModelAndViewFn("UMLContinuation",     Factory.defaultModelAndViewFn, { precondition: interactionPrecondition, field: \'fragments\' });\n    Factory.registerModelAndViewFn("UMLCombinedFragment", combinedFragmentFn);\n    Factory.registerModelAndViewFn("UMLInteractionUse",   Factory.defaultModelAndViewFn, { precondition: interactionPrecondition, field: \'fragments\' });\n    Factory.registerModelAndViewFn("UMLFrame",            Factory.defaultViewOnlyFn, { viewType: "UMLFrameView" });\n\n    // State Machines\n    Factory.registerModelAndViewFn("UMLState",                    stateFn);\n    Factory.registerModelAndViewFn("UMLSubmachineState",          submachineStateFn);\n    Factory.registerModelAndViewFn("UMLPseudostate",              pseudostateFn);\n    Factory.registerModelAndViewFn("UMLFinalState",               finalStateFn);\n    Factory.registerModelAndViewFn("UMLConnectionPointReference", connectionPointReferenceFn);\n    Factory.registerModelAndViewFn("UMLTransition",               transitionFn);\n\n    // Activities\n    Factory.registerModelAndViewFn("UMLAction",            Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLObjectNode",        Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLCentralBufferNode", Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLDataStoreNode",     Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLInitialNode",       Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLActivityFinalNode", Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLFlowFinalNode",     Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLForkNode",          Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLJoinNode",          Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLMergeNode",         Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLDecisionNode",      Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLInputPin",          Factory.defaultModelAndViewFn, { precondition: pinPrecondition, field: \'inputs\' });\n    Factory.registerModelAndViewFn("UMLOutputPin",         Factory.defaultModelAndViewFn, { precondition: pinPrecondition, field: \'outputs\' });\n    Factory.registerModelAndViewFn("UMLActivityPartition", Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'groups\' });\n    Factory.registerModelAndViewFn("UMLInterruptibleActivityRegion", Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'groups\' });\n    Factory.registerModelAndViewFn("UMLStructuredActivityNode", Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLExpansionRegion",   Factory.defaultModelAndViewFn, { precondition: activityPrecondition, field: \'nodes\' });\n    Factory.registerModelAndViewFn("UMLInputExpansionNode", Factory.defaultModelAndViewFn, { modelType: "UMLExpansionNode", viewType: "UMLExpansionNodeView", precondition: pinPrecondition, field: \'inputs\',  modelInitializer: function (m) { m.name = ""; } });\n    Factory.registerModelAndViewFn("UMLOutputExpansionNode",Factory.defaultModelAndViewFn, { modelType: "UMLExpansionNode", viewType: "UMLExpansionNodeView", precondition: pinPrecondition, field: \'outputs\', modelInitializer: function (m) { m.name = ""; } });\n    Factory.registerModelAndViewFn("UMLControlFlow",       Factory.defaultDirectedRelationshipFn, { precondition: flowPrecondition, field: \'edges\' });\n    Factory.registerModelAndViewFn("UMLObjectFlow",        objectFlowFn);\n    Factory.registerModelAndViewFn("UMLExceptionHandler",  exceptionHandlerFn);\n    Factory.registerModelAndViewFn("UMLActivityInterrupt", Factory.defaultDirectedRelationshipFn, { precondition: flowPrecondition, field: \'edges\' });\n\n    // Profiles\n    Factory.registerModelAndViewFn("UMLProfile",    Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLStereotype", Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("UMLMetaClass",  Factory.defaultModelAndViewFn, { modelInitializer: function (m) { m.name = "UMLClass"; } });\n    Factory.registerModelAndViewFn("UMLExtension",  Factory.defaultDirectedRelationshipFn, { precondition: extensionPrecondition });\n\n    // ========================================================================\n    // Create View\n    // ========================================================================\n\n    Factory.registerViewOfFn("UMLClassDiagram",              viewForGeneralDiagramFn);\n    Factory.registerViewOfFn("UMLCompositeStructureDiagram", viewForGeneralDiagramFn);\n    Factory.registerViewOfFn("UMLPackageDiagram",            viewForGeneralDiagramFn);\n    Factory.registerViewOfFn("UMLObjectDiagram",             viewForGeneralDiagramFn);\n    Factory.registerViewOfFn("UMLComponentDiagram",          viewForGeneralDiagramFn);\n    Factory.registerViewOfFn("UMLDeploymentDiagram",         viewForGeneralDiagramFn);\n    Factory.registerViewOfFn("UMLProfileDiagram",            viewForGeneralDiagramFn);\n    Factory.registerViewOfFn("UMLUseCaseDiagram",            viewForGeneralDiagramFn);\n    Factory.registerViewOfFn("UMLSequenceDiagram",           viewForSequenceDiagramFn);\n    Factory.registerViewOfFn("UMLCommunicationDiagram",      viewForCommunicationDiagramFn);\n    Factory.registerViewOfFn("UMLStatechartDiagram",         viewForStatechartDiagramFn);\n    Factory.registerViewOfFn("UMLActivityDiagram",           viewForActivityDiagramFn);\n\n});\n\n//# sourceURL=/uml/UMLFactory.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, type */\n\ndefine('uml/UMLQuickEdit',['require','exports','module','ui/QuickEdit','core/Graphics','core/Core','uml/UMLUtils'],function (require, exports, module) {\n    \"use strict\";\n\n    var QuickEdit  = require(\"ui/QuickEdit\"),\n        Graphics   = require(\"core/Graphics\"),\n        Core       = require(\"core/Core\"),\n        UMLUtils   = require(\"uml/UMLUtils\");\n\n    /**\n     * @const\n     * QuickEdit IDs\n     */\n    var QUICKEDIT_GENERAL              = 'general',\n        QUICKEDIT_FLOATINGNODE         = 'floating-node',\n        QUICKEDIT_PACKAGE              = 'package',\n        QUICKEDIT_SUBSYSTEM            = 'subsystem',\n        QUICKEDIT_CLASSIFIER           = 'classifier',\n        QUICKEDIT_CLASS                = 'class',\n        QUICKEDIT_INTERFACE            = 'interface',\n        QUICKEDIT_COMPONENT            = 'component',\n        QUICKEDIT_NODE                 = 'node',\n        QUICKEDIT_ACTOR                = 'actor',\n        QUICKEDIT_USECASE              = 'usecase',\n        QUICKEDIT_ENUMERATION          = 'enumeration',\n        QUICKEDIT_METACLASS            = 'metaclass',\n        QUICKEDIT_STEREOTYPE           = 'stereotype',\n        QUICKEDIT_ATTRIBUTE            = 'attribute',\n        QUICKEDIT_OPERATION            = 'operation',\n        QUICKEDIT_RECEPTION            = 'reception',\n        QUICKEDIT_PORT                 = 'port',\n        QUICKEDIT_PART                 = 'part',\n        QUICKEDIT_TEMPLATE_PARAMETER   = 'templateParameter',\n        QUICKEDIT_QUALIFIER            = 'qualifier',\n        QUICKEDIT_LITERAL              = 'literal',\n        QUICKEDIT_EXTENSION_POINT      = 'extensionPoint',\n        QUICKEDIT_OBJECT               = 'object',\n        QUICKEDIT_SLOT                 = 'slot',\n        QUICKEDIT_SEQ_LIFELINE         = 'seq-lifeline',\n        QUICKEDIT_SEQ_MESSAGE          = 'seq-message',\n        QUICKEDIT_COMM_LIFELINE        = 'comm-lifeline',\n        QUICKEDIT_COMM_MESSAGE         = 'comm-message',\n        QUICKEDIT_CONNECTOR            = 'connector',\n        QUICKEDIT_STATEINVARIANT       = 'stateinvariant',\n        QUICKEDIT_CONTINUATION         = 'continuation',\n        QUICKEDIT_COMBINEDFRAGMENT     = 'combined-fragment',\n        QUICKEDIT_OPERAND              = 'operand',\n        QUICKEDIT_DIRECTED             = 'directed',\n        QUICKEDIT_UNDIRECTED           = 'undirected',\n        QUICKEDIT_UNDIRECTED_HEAD      = 'undirected-head',\n        QUICKEDIT_UNDIRECTED_TAIL      = 'undirected-tail',\n        QUICKEDIT_ASSOCIATION          = 'association',\n        QUICKEDIT_ASSOCIATION_HEAD     = 'association-head',\n        QUICKEDIT_ASSOCIATION_TAIL     = 'association-tail',\n        QUICKEDIT_STATE                = 'state',\n        QUICKEDIT_INTERNAL_ACTIVITY    = 'internal-activity',\n        QUICKEDIT_INTERNAL_TRANSITION  = 'internal-transition',\n        QUICKEDIT_TRANSITION           = 'transition',\n        QUICKEDIT_ACTION               = 'action',\n        QUICKEDIT_OBJECT_NODE          = 'object-node',\n        QUICKEDIT_ACTIVITYEDGE         = 'activityedge',\n        QUICKEDIT_PARTITION            = 'partition',\n        QUICKEDIT_CONSTRAINT           = 'constraint',\n        QUICKEDIT_TEXTAREA             = 'textarea';\n\n    /**\n     * @const\n     * QuickEdit Item IDs\n     */\n    var ITEM_EDIT                = 'edit',\n        ITEM_VISIBILITY          = 'visibility',\n        ITEM_AGGREGATION         = 'aggregation',\n        ITEM_NAVIGABILITY        = 'navigability',\n        ITEM_MESSAGE_SORT        = 'message-sort',\n        ITEM_ADD                 = 'add',\n        ITEM_DELETE              = 'delete',\n        ITEM_MOVEUP              = 'moveup',\n        ITEM_MOVEDOWN            = 'movedown',\n        ITEM_ADD_ATTRIBUTE       = 'add-attribute',\n        ITEM_ADD_OPERATION       = 'add-operation',\n        ITEM_ADD_RECEPTION       = 'add-reception',\n        ITEM_ADD_QUALIFIER       = 'add-qualifier',\n        ITEM_ADD_LITERAL         = 'add-literal',\n        ITEM_ADD_SLOT            = 'add-slot',\n        ITEM_ADD_EXTENSIONPOINT  = 'add-extension-point',\n        ITEM_MULTIPLICITY        = 'multiplicity',\n        ITEM_SELECT_TYPE         = 'select-type',\n        ITEM_CREATE_TYPE         = 'create-type',\n        ITEM_SELECT_OPERATION    = 'select-operation',\n        ITEM_CREATE_OPERATION    = 'create-operation',\n        ITEM_SELECT_SIGNAL       = 'select-signal',\n        ITEM_CREATE_SIGNAL       = 'create-signal',\n        ITEM_ADD_OPERAND         = 'add-operand',\n        ITEM_ADD_REGION          = 'add-region',\n        ITEM_ADD_ENTRY_ACTIVITY  = 'add-entry-activity',\n        ITEM_ADD_DO_ACTIVITY     = 'add-do-activity',\n        ITEM_ADD_EXIT_ACTIVITY   = 'add-exit-activity',\n        ITEM_ADD_INTERNAL_TRANSITION = 'add-internal-transition',\n        ITEM_ADD_TRIGGER         = 'add-trigger',\n        ITEM_ADD_EFFECT          = 'add-effect',\n        ITEM_ADD_NOTE            = 'add-note',\n        ITEM_ADD_SUBPACKAGE      = 'add-subpackage',\n        ITEM_ADD_DEPENDING_PACKAGE = 'add-depending-package',\n        ITEM_ADD_DEPENDANT_PACKAGE = 'add-dependant-package',\n        ITEM_ADD_SUPERCLASS      = 'add-superclass',\n        ITEM_ADD_SUBCLASS        = 'add-subclass',\n        ITEM_ADD_ASSOCIATION     = 'add-association',\n        ITEM_ADD_AGGREGATION     = 'add-aggregation',\n        ITEM_ADD_COMPOSITION     = 'add-composition',\n        ITEM_ADD_PORT            = 'add-port',\n        ITEM_ADD_PART            = 'add-part',\n        ITEM_ADD_CONNECTED_PART  = 'add-connected-part',\n        ITEM_ADD_LINKED_OBJECT   = 'add-linked-object',\n        ITEM_ADD_PROVIDED_INTERFACE = 'add-provided-interface',\n        ITEM_ADD_REQUIRED_INTERFACE = 'add-required-interface',\n        ITEM_ADD_SUPERINTERFACE  = 'add-superinterface',\n        ITEM_ADD_SUBINTERFACE    = 'add-subinterface',\n        ITEM_ADD_REALIZING_CLASS = 'add-realizing-class',\n        ITEM_ADD_COMMUNICATING_NODE = 'add-communicating-node',\n        ITEM_ADD_DEPLOYED_ARTIFACT  = 'add-deployed-artifact',\n        ITEM_ADD_DEPLOYED_COMPONENT = 'add-deployed-component',\n        ITEM_ADD_SUPERACTOR         = 'add-superactor',\n        ITEM_ADD_SUBACTOR           = 'add-subactor',\n        ITEM_ADD_ASSOCIATED_USECASE = 'add-associated-usecase',\n        ITEM_ADD_ASSOCIATED_ACTOR   = 'add-associated-actor',\n        ITEM_ADD_INCLUDED_USECASE   = 'add-included-usecase',\n        ITEM_ADD_EXTENDED_USECASE   = 'add-extended-usecase',\n        ITEM_ADD_MESSAGE_LIFELINE   = 'add-message-lifeline',\n        ITEM_ADD_CREATE_MESSAGE_LIFELINE = 'add-create-message-lifeline',\n        ITEM_ADD_SELF_MESSAGE       = 'add-self-message',\n        ITEM_ADD_MESSAGE_FROM_GATE  = 'add-message-from-gate',\n        ITEM_ADD_MESSAGE_TO_GATE    = 'add-message-to-gate',\n        ITEM_ADD_FOUND_MESSAGE      = 'add-found-message',\n        ITEM_ADD_LOST_MESSAGE       = 'add-lost-message',\n        ITEM_ADD_REPLY_MESSAGE      = 'add-reply-message',\n        ITEM_ADD_CONNECTED_LIFELINE = 'add-connected-lifeline',\n        ITEM_ADD_SELF_CONNECTOR     = 'add-self-connector',\n        ITEM_ADD_FORWARD_MESSAGE    = 'add-forward-message',\n        ITEM_ADD_REVERSE_MESSAGE    = 'add-reverse-message',\n        ITEM_ADD_OUTGOING_TRANSITION = 'add-outgoing-transition',\n        ITEM_ADD_INCOMING_TRANSITION = 'add-incoming-transition',\n        ITEM_ADD_CONNECTION_POINT_REFERENCE = 'add-connection-point-reference',\n        ITEM_ADD_INITIAL_STATE      = 'add-initial-state',\n        ITEM_ADD_FINAL_STATE        = 'add-final-state',\n        ITEM_ADD_CHOICE             = 'add-choice',\n        ITEM_ADD_FORK               = 'add-fork',\n        ITEM_ADD_JOIN               = 'add-join',\n        ITEM_ADD_OUTGOING_CONTROL_FLOW = 'add-outgoing-control-flow',\n        ITEM_ADD_INCOMING_CONTROL_FLOW = 'add-incoming-control-flow',\n        ITEM_ADD_OUTGOING_OBJECT_FLOW  = 'add-outgoing-object-flow',\n        ITEM_ADD_INCOMING_OBJECT_FLOW  = 'add-incoming-object-flow',\n        ITEM_ADD_DECISION              = 'add-decision',\n        ITEM_ADD_MERGE                 = 'add-merge',\n        ITEM_ADD_ACTIVITY_FORK         = 'add-activity-fork',\n        ITEM_ADD_ACTIVITY_JOIN         = 'add-activity-join',\n        ITEM_ADD_INPUT_PIN             = 'add-input-pin',\n        ITEM_ADD_OUTPUT_PIN            = 'add-output-pin',\n        ITEM_ADD_INITIAL_NODE          = 'add-initial-node',\n        ITEM_ADD_FINAL_NODE            = 'add-final-note',\n        ITEM_ADD_STEREOTYPE            = 'add-stereotype',\n        ITEM_ADD_SUPERSTEREOTYPE       = 'add-superstereotype',\n        ITEM_ADD_SUBSTEREOTYPE         = 'add-substereotype';\n\n\n    /* Quick Edit Dropdowns */\n    var QE_VISIBILITY = {\n            type: 'dropdown',\n            id: ITEM_VISIBILITY,\n            hint: 'Visibility',\n            dropdownWidth: 140,\n            items: [\n                { icon: 'quick-edit-icon-public',    text: 'public',    value: 'public' },\n                { icon: 'quick-edit-icon-protected', text: 'protected', value: 'protected' },\n                { icon: 'quick-edit-icon-private',   text: 'private',   value: 'private' },\n                { icon: 'quick-edit-icon-package',   text: 'package',   value: 'package' }\n            ]\n        },\n        QE_AGGREGATION = {\n            type: 'dropdown',\n            id: ITEM_AGGREGATION,\n            hint: 'AggregationKind',\n            dropdownWidth: 150,\n            items: [\n                { icon: 'quick-edit-icon-associate', text: 'none',      value: 'none' },\n                { icon: 'quick-edit-icon-shared',    text: 'shared',    value: 'shared' },\n                { icon: 'quick-edit-icon-composite', text: 'composite', value: 'composite' }\n            ]\n        },\n        QE_NAVIGABILITY = {\n            type: 'dropdown',\n            id: ITEM_NAVIGABILITY,\n            hint: 'Navigability',\n            dropdownWidth: 170,\n            items: [\n                { icon: 'quick-edit-icon-navigable',    text: 'Navigable',      value: true },\n                { icon: 'quick-edit-icon-notnavigable', text: 'Not Navigable',  value: false }\n            ]\n        },\n        QE_MESSAGE_SORT = {\n            type: 'dropdown',\n            id: ITEM_MESSAGE_SORT,\n            hint: 'MessageSort',\n            dropdownWidth: 170,\n            items: [\n                { icon: 'quick-edit-icon-synch-call',     text: 'synchCall',     value: 'synchCall' },\n                { icon: 'quick-edit-icon-asynch-call',    text: 'asynchCall',    value: 'asynchCall' },\n                { icon: 'quick-edit-icon-asynch-signal',  text: 'asynchSignal',  value: 'asynchSignal' },\n                { icon: 'quick-edit-icon-create-message', text: 'createMessage', value: 'createMessage' },\n                { icon: 'quick-edit-icon-delete-message', text: 'deleteMessage', value: 'deleteMessage' },\n                { icon: 'quick-edit-icon-reply',          text: 'reply',         value: 'reply' }\n            ]\n        };\n\n    /* Quick Edit Buttons */\n    var QE_ADD                 = { type: 'button', id: ITEM_ADD,                 icon: 'quick-edit-icon-add',              hint: 'Add',        key: 'Ctrl-Enter' },\n        QE_DELETE              = { type: 'button', id: ITEM_DELETE,              icon: 'quick-edit-icon-delete',           hint: 'Delete',     key: 'Ctrl-Delete' },\n        QE_MOVEUP              = { type: 'button', id: ITEM_MOVEUP,              icon: 'quick-edit-icon-moveup',           hint: 'Move Up',    key: 'Ctrl-Up' },\n        QE_MOVEDOWN            = { type: 'button', id: ITEM_MOVEDOWN,            icon: 'quick-edit-icon-movedown',         hint: 'Move Down',  key: 'Ctrl-Down' },\n        QE_ADD_ATTRIBUTE       = { type: 'button', id: ITEM_ADD_ATTRIBUTE,       icon: 'quick-edit-icon-attribute',        hint: 'Add Attribute', key: 'Ctrl-Enter' },\n        QE_ADD_OPERATION       = { type: 'button', id: ITEM_ADD_OPERATION,       icon: 'quick-edit-icon-operation',        hint: 'Add Operation', key: 'Ctrl-Shift-Enter' },\n        QE_ADD_RECEPTION       = { type: 'button', id: ITEM_ADD_RECEPTION,       icon: 'quick-edit-icon-reception',        hint: 'Add Reception' },\n        QE_ADD_QUALIFIER       = { type: 'button', id: ITEM_ADD_QUALIFIER,       icon: 'quick-edit-icon-attribute',        hint: 'Add Qualifier' },\n        QE_ADD_LITERAL         = { type: 'button', id: ITEM_ADD_LITERAL,         icon: 'quick-edit-icon-literal',          hint: 'Add Literal', key: 'Ctrl-Enter' },\n        QE_ADD_SLOT            = { type: 'button', id: ITEM_ADD_SLOT,            icon: 'quick-edit-icon-slot',             hint: 'Add Slot', key: 'Ctrl-Enter' },\n        QE_ADD_EXTENSIONPOINT  = { type: 'button', id: ITEM_ADD_EXTENSIONPOINT,  icon: 'quick-edit-icon-extension-point',  hint: 'Add Extension Point' },\n        QE_SELECT_TYPE         = { type: 'button', id: ITEM_SELECT_TYPE,         icon: 'quick-edit-icon-select-type',      hint: 'Select Type' },\n        QE_CREATE_TYPE         = { type: 'button', id: ITEM_CREATE_TYPE,         icon: 'quick-edit-icon-create-type',      hint: 'Create Type' },\n        QE_SELECT_OPERATION    = { type: 'button', id: ITEM_SELECT_OPERATION,    icon: 'quick-edit-icon-select-operation', hint: 'Select Operation' },\n        QE_CREATE_OPERATION    = { type: 'button', id: ITEM_CREATE_OPERATION,    icon: 'quick-edit-icon-create-operation', hint: 'Create Operation' },\n        QE_SELECT_SIGNAL       = { type: 'button', id: ITEM_SELECT_SIGNAL,       icon: 'quick-edit-icon-select-signal',    hint: 'Select Signal' },\n        QE_CREATE_SIGNAL       = { type: 'button', id: ITEM_CREATE_SIGNAL,       icon: 'quick-edit-icon-create-signal',    hint: 'Create Signal' },\n        QE_ADD_OPERAND         = { type: 'button', id: ITEM_ADD_OPERAND,         icon: 'quick-edit-icon-operand',          hint: 'Add Operand' },\n        QE_ADD_REGION          = { type: 'button', id: ITEM_ADD_REGION,          icon: 'quick-edit-icon-region',           hint: 'Add Region' },\n        QE_ADD_ENTRY_ACTIVITY  = { type: 'button', id: ITEM_ADD_ENTRY_ACTIVITY,  icon: 'quick-edit-icon-entry-activity',   hint: 'Add Entry Activity' },\n        QE_ADD_DO_ACTIVITY     = { type: 'button', id: ITEM_ADD_DO_ACTIVITY,     icon: 'quick-edit-icon-do-activity',      hint: 'Add Do Activity' },\n        QE_ADD_EXIT_ACTIVITY   = { type: 'button', id: ITEM_ADD_EXIT_ACTIVITY,   icon: 'quick-edit-icon-exit-activity',    hint: 'Add Exit Activity' },\n        QE_ADD_INTERNAL_TRANSITION = { type: 'button', id: ITEM_ADD_INTERNAL_TRANSITION, icon: 'quick-edit-icon-add-internal-transition', hint: 'Add Internal Transition' },\n        QE_ADD_TRIGGER         = { type: 'button', id: ITEM_ADD_TRIGGER,         icon: 'quick-edit-icon-event',            hint: 'Add Trigger Event' },\n        QE_ADD_EFFECT          = { type: 'button', id: ITEM_ADD_EFFECT,          icon: 'quick-edit-icon-effect',           hint: 'Add Effect Behavior' },\n        QE_ADD_NOTE            = { type: 'button', id: ITEM_ADD_NOTE,            icon: 'quick-edit-icon-add-note',         hint: 'Add Note' },\n        QE_ADD_SUBPACKAGE      = { type: 'button', id: ITEM_ADD_SUBPACKAGE,      icon: 'quick-edit-icon-add-subpackage',   hint: 'Add Sub-Package' },\n        QE_ADD_DEPENDING_PACKAGE = { type: 'button', id: ITEM_ADD_DEPENDING_PACKAGE, icon: 'quick-edit-icon-add-depending-package', hint: 'Add Depending Package' },\n        QE_ADD_DEPENDANT_PACKAGE = { type: 'button', id: ITEM_ADD_DEPENDANT_PACKAGE, icon: 'quick-edit-icon-add-dependant-package', hint: 'Add Dependant Package' },\n        QE_ADD_SUPERCLASS      = { type: 'button', id: ITEM_ADD_SUPERCLASS,      icon: 'quick-edit-icon-add-superclass',   hint: 'Add Super-Class' },\n        QE_ADD_SUBCLASS        = { type: 'button', id: ITEM_ADD_SUBCLASS,        icon: 'quick-edit-icon-add-subclass',     hint: 'Add Sub-Class' },\n        QE_ADD_ASSOCIATION     = { type: 'button', id: ITEM_ADD_ASSOCIATION,     icon: 'quick-edit-icon-add-association',  hint: 'Add Association' },\n        QE_ADD_AGGREGATION     = { type: 'button', id: ITEM_ADD_AGGREGATION,     icon: 'quick-edit-icon-add-aggregation',  hint: 'Add Aggregation' },\n        QE_ADD_COMPOSITION     = { type: 'button', id: ITEM_ADD_COMPOSITION,     icon: 'quick-edit-icon-add-composition',  hint: 'Add Composition' },\n        QE_ADD_PORT            = { type: 'button', id: ITEM_ADD_PORT,            icon: 'quick-edit-icon-add-port',         hint: 'Add Port' },\n        QE_ADD_PART            = { type: 'button', id: ITEM_ADD_PART,            icon: 'quick-edit-icon-add-part',         hint: 'Add Part' },\n        QE_ADD_CONNECTED_PART  = { type: 'button', id: ITEM_ADD_CONNECTED_PART,  icon: 'quick-edit-icon-add-connected-part', hint: 'Add Connected Part' },\n        QE_ADD_LINKED_OBJECT   = { type: 'button', id: ITEM_ADD_LINKED_OBJECT,   icon: 'quick-edit-icon-add-linked-object',  hint: 'Add Linked Object' },\n        QE_ADD_PROVIDED_INTERFACE = { type: 'button', id: ITEM_ADD_PROVIDED_INTERFACE, icon: 'quick-edit-icon-add-interface', hint: 'Add Provided Interface' },\n        QE_ADD_REQUIRED_INTERFACE = { type: 'button', id: ITEM_ADD_REQUIRED_INTERFACE, icon: 'quick-edit-icon-add-required-interface', hint: 'Add Required Interface' },\n        QE_ADD_SUPERINTERFACE  = { type: 'button', id: ITEM_ADD_SUPERINTERFACE,  icon: 'quick-edit-icon-add-superinterface', hint: 'Add Super-Interface' },\n        QE_ADD_SUBINTERFACE    = { type: 'button', id: ITEM_ADD_SUBINTERFACE,    icon: 'quick-edit-icon-add-subinterface', hint: 'Add Sub-Interface' },\n        QE_ADD_REALIZING_CLASS = { type: 'button', id: ITEM_ADD_REALIZING_CLASS, icon: 'quick-edit-icon-add-realizing-class', hint: 'Add Realizing Class' },\n        QE_ADD_COMMUNICATING_NODE = { type: 'button', id: ITEM_ADD_COMMUNICATING_NODE, icon: 'quick-edit-icon-add-communicating-node', hint: 'Add Communicating Node' },\n        QE_ADD_DEPLOYED_ARTIFACT  = { type: 'button', id: ITEM_ADD_DEPLOYED_ARTIFACT,  icon: 'quick-edit-icon-add-deployed-artifact',  hint: 'Add Deployed Artifact' },\n        QE_ADD_DEPLOYED_COMPONENT = { type: 'button', id: ITEM_ADD_DEPLOYED_COMPONENT, icon: 'quick-edit-icon-add-deployed-component', hint: 'Add Deployed Component' },\n        QE_ADD_SUPERACTOR         = { type: 'button', id: ITEM_ADD_SUPERACTOR,         icon: 'quick-edit-icon-add-superactor',         hint: 'Add Super-Actor' },\n        QE_ADD_SUBACTOR           = { type: 'button', id: ITEM_ADD_SUBACTOR,           icon: 'quick-edit-icon-add-subactor',           hint: 'Add Sub-Actor' },\n        QE_ADD_ASSOCIATED_USECASE = { type: 'button', id: ITEM_ADD_ASSOCIATED_USECASE, icon: 'quick-edit-icon-add-associated-usecase', hint: 'Add Associated Use Case' },\n        QE_ADD_ASSOCIATED_ACTOR   = { type: 'button', id: ITEM_ADD_ASSOCIATED_ACTOR, icon: 'quick-edit-icon-add-associated-actor', hint: 'Add Associated Actor' },\n        QE_ADD_INCLUDED_USECASE   = { type: 'button', id: ITEM_ADD_INCLUDED_USECASE, icon: 'quick-edit-icon-add-included-usecase', hint: 'Add Included Use Case' },\n        QE_ADD_EXTENDED_USECASE   = { type: 'button', id: ITEM_ADD_EXTENDED_USECASE, icon: 'quick-edit-icon-add-extended-usecase', hint: 'Add Extended Use Case' },\n        QE_ADD_MESSAGE_LIFELINE   = { type: 'button', id: ITEM_ADD_MESSAGE_LIFELINE,  icon: 'quick-edit-icon-add-message-lifeline',  hint: 'Add Message with Lifeline' },\n        QE_ADD_CREATE_MESSAGE_LIFELINE = { type: 'button', id: ITEM_ADD_CREATE_MESSAGE_LIFELINE, icon: 'quick-edit-icon-add-create-message-lifeline', hint: 'Add Create Message with Lifeline' },\n        QE_ADD_SELF_MESSAGE       = { type: 'button', id: ITEM_ADD_SELF_MESSAGE,      icon: 'quick-edit-icon-add-self-message',      hint: 'Add Self Message' },\n        QE_ADD_MESSAGE_FROM_GATE  = { type: 'button', id: ITEM_ADD_MESSAGE_FROM_GATE, icon: 'quick-edit-icon-add-message-from-gate', hint: 'Add Message from Gate' },\n        QE_ADD_MESSAGE_TO_GATE    = { type: 'button', id: ITEM_ADD_MESSAGE_TO_GATE,   icon: 'quick-edit-icon-add-message-to-gate',   hint: 'Add Message to Gate' },\n        QE_ADD_FOUND_MESSAGE      = { type: 'button', id: ITEM_ADD_FOUND_MESSAGE,     icon: 'quick-edit-icon-add-found-message',     hint: 'Add Found Message' },\n        QE_ADD_LOST_MESSAGE       = { type: 'button', id: ITEM_ADD_LOST_MESSAGE,      icon: 'quick-edit-icon-add-lost-message',      hint: 'Add Lost Message' },\n        QE_ADD_REPLY_MESSAGE      = { type: 'button', id: ITEM_ADD_REPLY_MESSAGE,     icon: 'quick-edit-icon-add-reply-message',     hint: 'Add Reply Message' },\n        QE_ADD_CONNECTED_LIFELINE = { type: 'button', id: ITEM_ADD_CONNECTED_LIFELINE, icon: 'quick-edit-icon-add-connected-lifeline', hint: 'Add Connected Lifeline' },\n        QE_ADD_SELF_CONNECTOR     = { type: 'button', id: ITEM_ADD_SELF_CONNECTOR,     icon: 'quick-edit-icon-add-self-connector',     hint: 'Add Self Connector' },\n        QE_ADD_FORWARD_MESSAGE    = { type: 'button', id: ITEM_ADD_FORWARD_MESSAGE,    icon: 'quick-edit-icon-add-forward-message',    hint: 'Add Forward Message' },\n        QE_ADD_REVERSE_MESSAGE    = { type: 'button', id: ITEM_ADD_REVERSE_MESSAGE,    icon: 'quick-edit-icon-add-reverse-message',    hint: 'Add Reverse Message' },\n        QE_ADD_OUTGOING_TRANSITION = { type: 'button', id: ITEM_ADD_OUTGOING_TRANSITION, icon: 'quick-edit-icon-add-outgoing-transition', hint: 'Add Outgoing Transition' },\n        QE_ADD_INCOMING_TRANSITION = { type: 'button', id: ITEM_ADD_INCOMING_TRANSITION, icon: 'quick-edit-icon-add-incoming-transition', hint: 'Add Incoming Transition' },\n        QE_ADD_CONNECTION_POINT_REFERENCE = { type: 'button', id: ITEM_ADD_CONNECTION_POINT_REFERENCE, icon: 'quick-edit-icon-add-connection-point-reference', hint: 'Add ConnectionPointReference' },\n        QE_ADD_INITIAL_STATE   = { type: 'button', id: ITEM_ADD_INITIAL_STATE, icon: 'quick-edit-icon-add-initial-state', hint: 'Add Initial State' },\n        QE_ADD_FINAL_STATE     = { type: 'button', id: ITEM_ADD_FINAL_STATE,   icon: 'quick-edit-icon-add-final-state',   hint: 'Add Final State' },\n        QE_ADD_CHOICE          = { type: 'button', id: ITEM_ADD_CHOICE,        icon: 'quick-edit-icon-add-choice',        hint: 'Add Choice' },\n        QE_ADD_FORK            = { type: 'button', id: ITEM_ADD_FORK,          icon: 'quick-edit-icon-add-fork',          hint: 'Add Fork' },\n        QE_ADD_JOIN            = { type: 'button', id: ITEM_ADD_JOIN,          icon: 'quick-edit-icon-add-join',          hint: 'Add Join' },\n        QE_ADD_OUTGOING_CONTROL_FLOW = { type: 'button', id: ITEM_ADD_OUTGOING_CONTROL_FLOW, icon: 'quick-edit-icon-add-outgoing-control-flow', hint: 'Add Outgoing Control Flow' },\n        QE_ADD_INCOMING_CONTROL_FLOW = { type: 'button', id: ITEM_ADD_INCOMING_CONTROL_FLOW, icon: 'quick-edit-icon-add-incoming-control-flow', hint: 'Add Incoming Control Flow' },\n        QE_ADD_OUTGOING_OBJECT_FLOW  = { type: 'button', id: ITEM_ADD_OUTGOING_OBJECT_FLOW,  icon: 'quick-edit-icon-add-outgoing-object-flow',  hint: 'Add Outgoing Object Flow' },\n        QE_ADD_INCOMING_OBJECT_FLOW  = { type: 'button', id: ITEM_ADD_INCOMING_OBJECT_FLOW,  icon: 'quick-edit-icon-add-incoming-object-flow',  hint: 'Add Incoming Object Flow' },\n        QE_ADD_DECISION              = { type: 'button', id: ITEM_ADD_DECISION,      icon: 'quick-edit-icon-add-decision', hint: 'Add Decision' },\n        QE_ADD_MERGE                 = { type: 'button', id: ITEM_ADD_MERGE,         icon: 'quick-edit-icon-add-merge',    hint: 'Add Merge' },\n        QE_ADD_ACTIVITY_FORK         = { type: 'button', id: ITEM_ADD_ACTIVITY_FORK, icon: 'quick-edit-icon-add-fork',     hint: 'Add Fork' },\n        QE_ADD_ACTIVITY_JOIN         = { type: 'button', id: ITEM_ADD_ACTIVITY_JOIN, icon: 'quick-edit-icon-add-join',     hint: 'Add Join' },\n        QE_ADD_INPUT_PIN             = { type: 'button', id: ITEM_ADD_INPUT_PIN,    icon: 'quick-edit-icon-add-input-pin',    hint: 'Add Input Pin' },\n        QE_ADD_OUTPUT_PIN            = { type: 'button', id: ITEM_ADD_OUTPUT_PIN,   icon: 'quick-edit-icon-add-output-pin',   hint: 'Add Output Pin' },\n        QE_ADD_INITIAL_NODE          = { type: 'button', id: ITEM_ADD_INITIAL_NODE, icon: 'quick-edit-icon-add-initial-node', hint: 'Add Initial Node' },\n        QE_ADD_FINAL_NODE            = { type: 'button', id: ITEM_ADD_FINAL_NODE,   icon: 'quick-edit-icon-add-final-node',   hint: 'Add Final Node' },\n        QE_ADD_STEREOTYPE            = { type: 'button', id: ITEM_ADD_STEREOTYPE,   icon: 'quick-edit-icon-add-stereotype',   hint: 'Add Stereotype' },\n        QE_ADD_SUPERSTEREOTYPE       = { type: 'button', id: ITEM_ADD_SUPERSTEREOTYPE, icon: 'quick-edit-icon-add-superstereotype', hint: 'Add Super-Stereotype' },\n        QE_ADD_SUBSTEREOTYPE         = { type: 'button', id: ITEM_ADD_SUBSTEREOTYPE,   icon: 'quick-edit-icon-add-substereotype',   hint: 'Add Sub-Stereotype' };\n\n    /* Quick Edit Inputs */\n    var QE_EDIT_GENERAL    = { type: 'input', id: ITEM_EDIT, hint: \"<< stereotype >> name\" },\n        QE_EDIT_ATTRIBUTE  = { type: 'input', id: ITEM_EDIT, hint: \"<< stereotype >> name : type [ multiplicity ] = value\" },\n        QE_EDIT_OPERATION  = { type: 'input', id: ITEM_EDIT, hint: \"<< stereotype >> name (param1 : type1 [multiplicity] = default1, param2, ...) : returnType\" },\n        QE_EDIT_TEMPARAM   = { type: 'input', id: ITEM_EDIT, hint: \"<< stereotype >> name : type = value\" },\n        QE_EDIT_ROLENAME   = { type: 'input', id: ITEM_EDIT, hint: \"Role name\", width: 100 },\n        QE_EDIT_INSTANCE   = { type: 'input', id: ITEM_EDIT, hint: \"<< stereotype >> name : type\" },\n        QE_EDIT_SLOT       = { type: 'input', id: ITEM_EDIT, hint: \"<< stereotype >> name : type = value\" },\n        QE_EDIT_LIFELINE   = { type: 'input', id: ITEM_EDIT, hint: \"<< stereotype >> name [selector] : type\" },\n        QE_EDIT_MESSAGE    = { type: 'input', id: ITEM_EDIT, hint: \"<< stereotype >> target = name (arguments)\" },\n        QE_EDIT_INVARIANT  = { type: 'input', id: ITEM_EDIT, hint: \"invariant\" },\n        QE_EDIT_GUARD      = { type: 'input', id: ITEM_EDIT, hint: \"guard\" },\n        QE_EDIT_TRANSITION = { type: 'input', id: ITEM_EDIT, hint: \"trigger1, trigger2, ... [ guard ] / effect\", width: 120 },\n        QE_EDIT_TEXTAREA   = { type: 'textarea', id: ITEM_EDIT, hint: \"Edit text (Shift+Enter = New Line)\" };\n\n    /* Quick Edit ComboBoxes */\n    var QE_MULTIPLICITY = {\n            type: 'combo',\n            id: ITEM_MULTIPLICITY,\n            hint: 'Multiplicity',\n            width: 70,\n            items: [\n                { text: '0..1', value: '0..1' },\n                { text: '1',    value: '1' },\n                { text: '0..*', value: '0..*' },\n                { text: '1..*', value: '1..*' },\n                { text: '*',    value: '*' }\n            ]\n        };\n\n    // QuickEdit for Common\n    // - Model, Collaboration, CollaborationUse\n    // - ComponentInstance, NodeInstance, ArtifactInstance, UseCaseSubjectView\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLModelView) ||\n                   (view instanceof type.UMLCollaborationView) ||\n                   (view instanceof type.UMLCollaborationUseView) ||\n                   (view instanceof type.UMLComponentInstanceView) ||\n                   (view instanceof type.UMLNodeInstanceView) ||\n                   (view instanceof type.UMLArtifactInstanceView) ||\n                   (view instanceof type.UMLUseCaseSubjectView) ||\n                   (view instanceof type.UMLStructuredActivityNodeView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_GENERAL,\n                main: QE_EDIT_GENERAL,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Floating Nodes\n    // - Pseudostate, ConnectionPointReference\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLPseudostateView) ||\n                   (view instanceof type.UMLConnectionPointReferenceView) ||\n                   ((view instanceof Core.NodeLabelView) && (view._parent instanceof type.UMLPseudostateView)) ||\n                   ((view instanceof Core.NodeLabelView) && (view._parent instanceof type.UMLConnectionPointReferenceView));\n        },\n        popup: function (editor, position, view) {\n            var endView = view.nameLabel;\n            if (view instanceof Core.NodeLabelView) { endView = view._parent.nameLabel; }\n            var rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, endView, {\n                id: QUICKEDIT_FLOATINGNODE,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Subsystem\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLSubsystemView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_SUBSYSTEM,\n                main: QE_EDIT_GENERAL,\n                leftSection:   [ QE_VISIBILITY ],\n                leftSection2:  [ QE_ADD_NOTE ],\n                rightSection:  [ QE_ADD_PROVIDED_INTERFACE, QE_ADD_REQUIRED_INTERFACE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Package\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLPackageView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_PACKAGE,\n                main: QE_EDIT_GENERAL,\n                leftSection:   [ QE_VISIBILITY ],\n                leftSection2:  [ QE_ADD_NOTE ],\n                rightSection:  [ QE_ADD_SUBPACKAGE ],\n                rightSection2: [ QE_ADD_DEPENDANT_PACKAGE, QE_ADD_DEPENDING_PACKAGE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Classifiers\n    // - Signal, DataType, PrimitiveType, Artifact\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLSignalView) ||\n                   (view instanceof type.UMLDataTypeView) ||\n                   (view instanceof type.UMLPrimitiveTypeView) ||\n                   (view instanceof type.UMLArtifactView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_CLASSIFIER,\n                main: QE_EDIT_GENERAL,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_ATTRIBUTE, QE_ADD_OPERATION, QE_ADD_RECEPTION ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Class\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLClassView && !(view instanceof type.UMLStereotypeView));\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_CLASS,\n                main: QE_EDIT_GENERAL,\n                leftSection:   [ QE_VISIBILITY ],\n                leftSection2:  [ QE_ADD_NOTE ],\n                rightSection:  [ QE_ADD_ATTRIBUTE, QE_ADD_OPERATION, QE_ADD_RECEPTION ],\n                rightSection2: [ QE_ADD_SUBCLASS, QE_ADD_SUPERCLASS, QE_ADD_PROVIDED_INTERFACE, QE_ADD_REQUIRED_INTERFACE ],\n                rightSection3: [ QE_ADD_ASSOCIATION, QE_ADD_AGGREGATION, QE_ADD_COMPOSITION ],\n                rightSection4: [ QE_ADD_PORT, QE_ADD_PART ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Interface\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLInterfaceView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_INTERFACE,\n                main: QE_EDIT_GENERAL,\n                leftSection:   [ QE_VISIBILITY ],\n                leftSection2:  [ QE_ADD_NOTE ],\n                rightSection:  [ QE_ADD_ATTRIBUTE, QE_ADD_OPERATION, QE_ADD_RECEPTION ],\n                rightSection2: [ QE_ADD_SUBINTERFACE, QE_ADD_SUPERINTERFACE, QE_ADD_REALIZING_CLASS ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Component\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLComponentView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_COMPONENT,\n                main: QE_EDIT_GENERAL,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_ATTRIBUTE, QE_ADD_OPERATION, QE_ADD_RECEPTION ],\n                rightSection2: [ QE_ADD_PROVIDED_INTERFACE, QE_ADD_REQUIRED_INTERFACE ],\n                rightSection3: [ QE_ADD_PORT, QE_ADD_PART ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Node\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLNodeView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_NODE,\n                main: QE_EDIT_GENERAL,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_ATTRIBUTE, QE_ADD_OPERATION, QE_ADD_RECEPTION ],\n                rightSection2: [ QE_ADD_COMMUNICATING_NODE ],\n                rightSection3: [ QE_ADD_DEPLOYED_ARTIFACT, QE_ADD_DEPLOYED_COMPONENT ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Actor\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLActorView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_ACTOR,\n                main: QE_EDIT_GENERAL,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_ATTRIBUTE, QE_ADD_OPERATION ],\n                rightSection2: [ QE_ADD_SUBACTOR, QE_ADD_SUPERACTOR ],\n                rightSection3: [ QE_ADD_ASSOCIATED_USECASE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for UseCase\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLUseCaseView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_USECASE,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_EXTENSIONPOINT ],\n                rightSection2: [ QE_ADD_ASSOCIATED_ACTOR, QE_ADD_INCLUDED_USECASE, QE_ADD_EXTENDED_USECASE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Enumeration\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLEnumerationView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_ENUMERATION,\n                main: QE_EDIT_GENERAL,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_LITERAL, QE_ADD_OPERATION ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for MetaClass\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLMetaClassView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_METACLASS,\n                main: QE_EDIT_GENERAL,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_STEREOTYPE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Stereotype\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLStereotypeView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_STEREOTYPE,\n                main: QE_EDIT_GENERAL,\n                leftSection:   [ QE_VISIBILITY ],\n                leftSection2:  [ QE_ADD_NOTE ],\n                rightSection:  [ QE_ADD_ATTRIBUTE, QE_ADD_OPERATION ],\n                rightSection2: [ QE_ADD_SUBSTEREOTYPE, QE_ADD_SUPERSTEREOTYPE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Port\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLPortView) ||\n                   ((view instanceof Core.NodeLabelView) && (view._parent instanceof type.UMLPortView));\n        },\n        popup: function (editor, position, view) {\n            var endView = view.nameLabel;\n            if (view instanceof Core.NodeLabelView) { endView = view._parent.nameLabel; }\n            var rect  = new Graphics.Rect(endView.left, endView.top, endView.left+100, endView.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, endView, {\n                id: QUICKEDIT_PORT,\n                main: QE_EDIT_ATTRIBUTE,\n                leftSection: [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_SELECT_TYPE, QE_CREATE_TYPE ],\n                rightSection2: [ QE_ADD_PROVIDED_INTERFACE, QE_ADD_REQUIRED_INTERFACE, QE_ADD_CONNECTED_PART ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getAttributeExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Part\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLPartView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_PART,\n                main: QE_EDIT_ATTRIBUTE,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_SELECT_TYPE, QE_CREATE_TYPE ],\n                rightSection2: [ QE_ADD_PORT, QE_ADD_PART, QE_ADD_CONNECTED_PART ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getAttributeExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Attribute\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLAttributeView) && (view._parent instanceof type.UMLAttributeCompartmentView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_ATTRIBUTE,\n                main: QE_EDIT_ATTRIBUTE,\n                leftSection: [ QE_VISIBILITY ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getAttributeExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Operation\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLOperationView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_OPERATION,\n                main: QE_EDIT_OPERATION,\n                leftSection: [ QE_VISIBILITY ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getOperationExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Reception\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLReceptionView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_RECEPTION,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_VISIBILITY, QE_SELECT_SIGNAL ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for TemplateParameter\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLTemplateParameterView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_TEMPLATE_PARAMETER,\n                main: QE_EDIT_TEMPARAM,\n                leftSection: [ QE_VISIBILITY ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getTemplateParameterExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Qualifier\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLAttributeView) && (view._parent instanceof type.UMLQualifierCompartmentView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_QUALIFIER,\n                main: QE_EDIT_ATTRIBUTE,\n                leftSection: [ QE_VISIBILITY ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getAttributeExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Literal\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLEnumerationLiteralView) && (view._parent instanceof type.UMLEnumerationLiteralCompartmentView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_LITERAL,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_VISIBILITY ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for ExtensionPoint\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLExtensionPointView) && (view._parent instanceof type.UMLExtensionPointCompartmentView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_EXTENSION_POINT,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_VISIBILITY ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Object\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLObjectView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_OBJECT,\n                main: QE_EDIT_INSTANCE,\n                leftSection: [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_SLOT ],\n                rightSection2: [ QE_ADD_LINKED_OBJECT ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getObjectExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Slot\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLSlotView) && (view._parent instanceof type.UMLSlotCompartmentView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_SLOT,\n                main: QE_EDIT_SLOT,\n                leftSection: [ QE_VISIBILITY ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getSlotExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Lifeline (Sequence)\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLSeqLifelineView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_SEQ_LIFELINE,\n                main: QE_EDIT_LIFELINE,\n                leftSection: [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_SELECT_TYPE, QE_CREATE_TYPE ],\n                rightSection2: [ QE_ADD_MESSAGE_LIFELINE, QE_ADD_CREATE_MESSAGE_LIFELINE, QE_ADD_SELF_MESSAGE ],\n                rightSection3: [ QE_ADD_FOUND_MESSAGE, QE_ADD_LOST_MESSAGE ],\n                rightSection4: [ QE_ADD_MESSAGE_FROM_GATE, QE_ADD_MESSAGE_TO_GATE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getLifelineExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for Message (Sequence)\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLSeqMessageView) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLSeqMessageView));\n        },\n        popup: function (editor, position, view) {\n            var endView = view.nameLabel;\n            if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n            if (view instanceof Core.NodeLabelView) { endView = view._parent.nameLabel; }\n            var rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, endView, {\n                id: QUICKEDIT_SEQ_MESSAGE,\n                main: QE_EDIT_MESSAGE,\n                leftSection: [ QE_MESSAGE_SORT ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_SELECT_OPERATION, QE_CREATE_OPERATION, QE_SELECT_SIGNAL, QE_CREATE_SIGNAL ],\n                rightSection2: [ QE_ADD_REPLY_MESSAGE ]\n            });\n        },\n        init: function (view) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getMessageExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_MESSAGE_SORT, view.model.messageSort);\n        }\n    });\n\n    // QuickEdit for Message (Communication)\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLCommMessageView) ||\n                   ((view instanceof Core.NodeLabelView) && (view._parent instanceof type.UMLCommMessageView));\n        },\n        popup: function (editor, position, view) {\n            var endView = view.nameLabel;\n            if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n            if (view instanceof Core.NodeLabelView) { endView = view._parent.nameLabel; }\n            var rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, endView, {\n                id: QUICKEDIT_COMM_MESSAGE,\n                main: QE_EDIT_MESSAGE,\n                leftSection: [ QE_MESSAGE_SORT ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_SELECT_OPERATION, QE_CREATE_OPERATION, QE_SELECT_SIGNAL, QE_CREATE_SIGNAL ],\n                rightSection2: [ QE_ADD_REPLY_MESSAGE ]\n            });\n        },\n        init: function (view) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getMessageExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_MESSAGE_SORT, view.model.messageSort);\n        }\n    });\n\n    // QuickEdit for Lifeline of (Communication)\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLCommLifelineView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_COMM_LIFELINE,\n                main: QE_EDIT_LIFELINE,\n                leftSection: [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_SELECT_TYPE, QE_CREATE_TYPE ],\n                rightSection2: [ QE_ADD_CONNECTED_LIFELINE, QE_ADD_SELF_CONNECTOR ],\n                rightSection3: [ QE_ADD_FORWARD_MESSAGE, QE_ADD_REVERSE_MESSAGE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getLifelineExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for StateInvariant\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLStateInvariantView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.invariantLabel,\n                rect  = new Graphics.Rect(label.left, label.top, label.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_STATEINVARIANT,\n                main: QE_EDIT_INVARIANT,\n                leftSection: [],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, view.model.invariant);\n        }\n    });\n\n    // QuickEdit for Continuation\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLContinuationView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameLabel,\n                rect  = new Graphics.Rect(label.left, label.top, label.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_CONTINUATION,\n                main: QE_EDIT_GENERAL,\n                leftSection: [],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n        }\n    });\n\n    // QuickEdit for CombinedFragment\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLCombinedFragmentView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameLabel,\n                rect  = new Graphics.Rect(label.left, label.top, label.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_COMBINEDFRAGMENT,\n                main: QE_EDIT_GENERAL,\n                leftSection: [],\n                rightSection: [ QE_ADD_OPERAND ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n        }\n    });\n\n    // QuickEdit for InteractionOperand\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLInteractionOperandView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.guardLabel,\n                rect  = new Graphics.Rect(label.left, label.top, label.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_OPERAND,\n                main: QE_EDIT_GUARD,\n                leftSection: [],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, view.model.guard);\n        }\n    });\n\n    // QuickEdit for Directed Relation\n    // - Generalization, Dependency, Realization, Include, Extend, Extension,\n    //   ExceptionHandler\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLGeneralizationView) ||\n                   (view instanceof type.UMLDependencyView) ||\n                   (view instanceof type.UMLInterfaceRealizationView) ||\n                   (view instanceof type.UMLComponentRealizationView) ||\n                   (view instanceof type.UMLIncludeView) ||\n                   (view instanceof type.UMLExtendView) ||\n                   (view instanceof type.UMLExtensionView) ||\n                   (view instanceof type.UMLExceptionHandlerView) ||\n                   (view instanceof type.UMLActivityInterruptView) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLGeneralizationView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLDependencyView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLInterfaceRealizationView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLComponentRealizationView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLIncludeView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLExtendView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLExtensionView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLExceptionHandlerView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLActivityInterruptView));\n        },\n        popup: function (editor, position, view) {\n            var endView = view.nameLabel;\n            if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n            var rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, endView, {\n                id: QUICKEDIT_DIRECTED,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            QuickEdit.setInputValue(ITEM_EDIT, model.name);\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.visibility);\n        }\n    });\n\n    // QuickEdit for Undirected Relation (Link)\n    QuickEdit.register({\n        when: function (view, x, y, pass) {\n            pass.position = 'center';\n            // when UndirectedRelationship was clicked\n            if (view instanceof type.UMLLinkView) {\n                pass.position = QuickEdit.getPositionOfEdge(view.points, x, y);\n                return true;\n            }\n            // when Labels belongs to UndirectedRelationship was clicked\n            if ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLLinkView)) {\n                if ((view === view._parent.headRoleNameLabel) ||\n                    (view === view._parent.headPropertyLabel) ||\n                    (view === view._parent.headMultiplicityLabel)) {\n                    pass.position = 'head';\n                } else if ((view === view._parent.tailRoleNameLabel) ||\n                    (view === view._parent.tailPropertyLabel) ||\n                    (view === view._parent.tailMultiplicityLabel)) {\n                    pass.position = 'tail';\n                }\n                return true;\n            }\n            return false;\n        },\n        popup: function (editor, position, view, pass) {\n            var endView, rect;\n            switch (pass.position) {\n            case 'center':\n                endView = view.nameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_UNDIRECTED,\n                    main: QE_EDIT_GENERAL,\n                    leftSection: [ QE_VISIBILITY ],\n                    leftSection2: [ QE_ADD_NOTE ],\n                    rightSection: []\n                });\n                break;\n            case 'tail':\n                endView = view.tailRoleNameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.tailRoleNameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_UNDIRECTED_TAIL,\n                    main: QE_EDIT_ROLENAME,\n                    leftSection: [ QE_AGGREGATION, QE_NAVIGABILITY, QE_VISIBILITY ],\n                    rightSection: [ QE_MULTIPLICITY ]\n                });\n                break;\n            case 'head':\n                endView = view.headRoleNameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.headRoleNameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_UNDIRECTED_HEAD,\n                    main: QE_EDIT_ROLENAME,\n                    leftSection: [ QE_AGGREGATION, QE_NAVIGABILITY, QE_VISIBILITY ],\n                    rightSection: [ QE_MULTIPLICITY ]\n                });\n                break;\n            }\n        },\n        init: function (view, pass) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            switch (pass.position) {\n            case 'center':\n                QuickEdit.setInputValue(ITEM_EDIT, model.name);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.visibility);\n                break;\n            case 'tail':\n                QuickEdit.setInputValue(ITEM_EDIT, model.end1.name);\n                QuickEdit.setDropdownValue(ITEM_AGGREGATION, model.end1.aggregation);\n                QuickEdit.setDropdownValue(ITEM_NAVIGABILITY, model.end1.navigable);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.end1.visibility);\n                QuickEdit.setComboBoxValue(ITEM_MULTIPLICITY, model.end1.multiplicity);\n                break;\n            case 'head':\n                QuickEdit.setInputValue(ITEM_EDIT, model.end2.name);\n                QuickEdit.setDropdownValue(ITEM_AGGREGATION, model.end2.aggregation);\n                QuickEdit.setDropdownValue(ITEM_NAVIGABILITY, model.end2.navigable);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.end2.visibility);\n                QuickEdit.setComboBoxValue(ITEM_MULTIPLICITY, model.end2.multiplicity);\n                break;\n            }\n        }\n    });\n\n    // QuickEdit for Connector\n    QuickEdit.register({\n        when: function (view, x, y, pass) {\n            pass.position = 'center';\n            // when ConnectorView was clicked\n            if (view instanceof type.UMLConnectorView) {\n                pass.position = QuickEdit.getPositionOfEdge(view.points, x, y);\n                return true;\n            }\n            // when Labels belongs to ConnectorView was clicked\n            if ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLConnectorView)) {\n                if ((view === view._parent.headRoleNameLabel) ||\n                    (view === view._parent.headPropertyLabel) ||\n                    (view === view._parent.headMultiplicityLabel)) {\n                    pass.position = 'head';\n                } else if ((view === view._parent.tailRoleNameLabel) ||\n                    (view === view._parent.tailPropertyLabel) ||\n                    (view === view._parent.tailMultiplicityLabel)) {\n                    pass.position = 'tail';\n                }\n                return true;\n            }\n            return false;\n        },\n        popup: function (editor, position, view, pass) {\n            var endView, rect;\n            switch (pass.position) {\n            case 'center':\n                endView = view.nameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                if (view.getDiagram() instanceof type.UMLCommunicationDiagram) {\n                    QuickEdit.render(editor, rect, endView, {\n                        id: QUICKEDIT_CONNECTOR,\n                        main: QE_EDIT_GENERAL,\n                        leftSection: [ QE_VISIBILITY ],\n                        leftSection2: [ QE_ADD_NOTE ],\n                        rightSection: [ QE_ADD_FORWARD_MESSAGE, QE_ADD_REVERSE_MESSAGE ]\n                    });\n                } else {\n                    QuickEdit.render(editor, rect, endView, {\n                        id: QUICKEDIT_CONNECTOR,\n                        main: QE_EDIT_GENERAL,\n                        leftSection: [ QE_VISIBILITY ],\n                        leftSection2: [ QE_ADD_NOTE ]\n                    });\n                }\n                break;\n            case 'tail':\n                endView = view.tailRoleNameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.tailRoleNameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_UNDIRECTED_TAIL,\n                    main: QE_EDIT_ROLENAME,\n                    leftSection: [ QE_AGGREGATION, QE_NAVIGABILITY, QE_VISIBILITY ],\n                    rightSection: [ QE_MULTIPLICITY ]\n                });\n                break;\n            case 'head':\n                endView = view.headRoleNameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.headRoleNameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_UNDIRECTED_HEAD,\n                    main: QE_EDIT_ROLENAME,\n                    leftSection: [ QE_AGGREGATION, QE_NAVIGABILITY, QE_VISIBILITY ],\n                    rightSection: [ QE_MULTIPLICITY ]\n                });\n                break;\n            }\n        },\n        init: function (view, pass) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            switch (pass.position) {\n            case 'center':\n                QuickEdit.setInputValue(ITEM_EDIT, model.name);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.visibility);\n                break;\n            case 'tail':\n                QuickEdit.setInputValue(ITEM_EDIT, model.end1.name);\n                QuickEdit.setDropdownValue(ITEM_AGGREGATION, model.end1.aggregation);\n                QuickEdit.setDropdownValue(ITEM_NAVIGABILITY, model.end1.navigable);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.end1.visibility);\n                QuickEdit.setComboBoxValue(ITEM_MULTIPLICITY, model.end1.multiplicity);\n                break;\n            case 'head':\n                QuickEdit.setInputValue(ITEM_EDIT, model.end2.name);\n                QuickEdit.setDropdownValue(ITEM_AGGREGATION, model.end2.aggregation);\n                QuickEdit.setDropdownValue(ITEM_NAVIGABILITY, model.end2.navigable);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.end2.visibility);\n                QuickEdit.setComboBoxValue(ITEM_MULTIPLICITY, model.end2.multiplicity);\n                break;\n            }\n        }\n    });\n\n    // QuickEdit for Association\n    QuickEdit.register({\n        when: function (view, x, y, pass) {\n            pass.position = 'center';\n            // when AssocationView was clicked\n            if (view instanceof type.UMLAssociationView) {\n                pass.position = QuickEdit.getPositionOfEdge(view.points, x, y);\n                return true;\n            }\n            // when Labels belongs to AssociationView was clicked\n            if ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLAssociationView)) {\n                if ((view === view._parent.headRoleNameLabel) ||\n                    (view === view._parent.headPropertyLabel) ||\n                    (view === view._parent.headMultiplicityLabel)) {\n                    pass.position = 'head';\n                } else if ((view === view._parent.tailRoleNameLabel) ||\n                    (view === view._parent.tailPropertyLabel) ||\n                    (view === view._parent.tailMultiplicityLabel)) {\n                    pass.position = 'tail';\n                }\n                return true;\n            }\n            return false;\n        },\n        popup: function (editor, position, view, pass) {\n            var endView, rect;\n            switch (pass.position) {\n            case 'center':\n                endView = view.nameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_ASSOCIATION,\n                    main: QE_EDIT_GENERAL,\n                    leftSection: [ QE_VISIBILITY ],\n                    leftSection2: [ QE_ADD_NOTE ],\n                    rightSection: []\n                });\n                break;\n            case 'tail':\n                endView = view.tailRoleNameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.tailRoleNameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_ASSOCIATION_TAIL,\n                    main: QE_EDIT_ROLENAME,\n                    leftSection: [ QE_ADD_QUALIFIER, QE_AGGREGATION, QE_NAVIGABILITY, QE_VISIBILITY ],\n                    rightSection: [ QE_MULTIPLICITY ]\n                });\n                break;\n            case 'head':\n                endView = view.headRoleNameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.headRoleNameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_ASSOCIATION_HEAD,\n                    main: QE_EDIT_ROLENAME,\n                    leftSection: [ QE_ADD_QUALIFIER, QE_AGGREGATION, QE_NAVIGABILITY, QE_VISIBILITY ],\n                    rightSection: [ QE_MULTIPLICITY ]\n                });\n                break;\n            }\n        },\n        init: function (view, pass) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            switch (pass.position) {\n            case 'center':\n                QuickEdit.setInputValue(ITEM_EDIT, model.name);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.visibility);\n                break;\n            case 'tail':\n                QuickEdit.setInputValue(ITEM_EDIT, model.end1.name);\n                QuickEdit.setDropdownValue(ITEM_AGGREGATION, model.end1.aggregation);\n                QuickEdit.setDropdownValue(ITEM_NAVIGABILITY, model.end1.navigable);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.end1.visibility);\n                QuickEdit.setComboBoxValue(ITEM_MULTIPLICITY, model.end1.multiplicity);\n                break;\n            case 'head':\n                QuickEdit.setInputValue(ITEM_EDIT, model.end2.name);\n                QuickEdit.setDropdownValue(ITEM_AGGREGATION, model.end2.aggregation);\n                QuickEdit.setDropdownValue(ITEM_NAVIGABILITY, model.end2.navigable);\n                QuickEdit.setDropdownValue(ITEM_VISIBILITY, model.end2.visibility);\n                QuickEdit.setComboBoxValue(ITEM_MULTIPLICITY, model.end2.multiplicity);\n                break;\n            }\n        }\n    });\n\n    // QuickEdit for State\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLStateView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_STATE,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_ADD_REGION, QE_ADD_CONNECTION_POINT_REFERENCE ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_ENTRY_ACTIVITY, QE_ADD_DO_ACTIVITY, QE_ADD_EXIT_ACTIVITY, QE_ADD_INTERNAL_TRANSITION ],\n                rightSection2: [ QE_ADD_OUTGOING_TRANSITION, QE_ADD_INCOMING_TRANSITION ],\n                rightSection3: [ QE_ADD_INITIAL_STATE, QE_ADD_FINAL_STATE, QE_ADD_CHOICE ],\n                rightSection4: [ QE_ADD_FORK, QE_ADD_JOIN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n        }\n    });\n\n    // QuickEdit for Internal Activity\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLInternalActivityView) && (view._parent instanceof type.UMLInternalActivityCompartmentView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_INTERNAL_ACTIVITY,\n                main: QE_EDIT_GENERAL,\n                leftSection: [],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n        }\n    });\n\n    // QuickEdit for Internal Transition\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLInternalTransitionView) && (view._parent instanceof type.UMLInternalTransitionCompartmentView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_INTERNAL_TRANSITION,\n                main: QE_EDIT_GENERAL,\n                leftSection: [],\n                rightSection: [ QE_ADD_TRIGGER, QE_ADD_EFFECT ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n        }\n    });\n\n    // QuickEdit for Transition\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLTransitionView) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLTransitionView));\n        },\n        popup: function (editor, position, view) {\n            var endView = view.nameLabel;\n            if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n            var rect  = new Graphics.Rect(endView.left, endView.top, endView.left+120, endView.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, endView, {\n                id: QUICKEDIT_TRANSITION,\n                main: QE_EDIT_TRANSITION,\n                leftSection: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_TRIGGER, QE_ADD_EFFECT ]\n            });\n        },\n        init: function (view) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getTransitionExpression(view.model));\n        }\n    });\n\n    // QuickEdit for Action\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLActionView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_ACTION,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_ADD_OUTPUT_PIN, QE_ADD_INPUT_PIN ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_TRIGGER ],\n                rightSection2: [ QE_ADD_OUTGOING_CONTROL_FLOW, QE_ADD_INCOMING_CONTROL_FLOW, QE_ADD_OUTGOING_OBJECT_FLOW, QE_ADD_INCOMING_OBJECT_FLOW ],\n                rightSection3: [ QE_ADD_DECISION, QE_ADD_MERGE, QE_ADD_ACTIVITY_FORK, QE_ADD_ACTIVITY_JOIN ],\n                rightSection4: [ QE_ADD_INITIAL_NODE, QE_ADD_FINAL_NODE ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n        }\n    });\n\n    // QuickEdit for ObjectNode\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLObjectNodeView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameCompartment.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_OBJECT_NODE,\n                main: QE_EDIT_GENERAL,\n                leftSection:  [ QE_VISIBILITY ],\n                leftSection2: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_OUTGOING_OBJECT_FLOW, QE_ADD_INCOMING_OBJECT_FLOW ],\n                rightSection2: [ QE_ADD_OUTGOING_CONTROL_FLOW, QE_ADD_INCOMING_CONTROL_FLOW ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_VISIBILITY, view.model.visibility);\n        }\n    });\n\n    // QuickEdit for ActivityEdge\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLControlFlowView) ||\n                   (view instanceof type.UMLObjectFlowView) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLControlFlowView)) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.UMLObjectFlowView));\n        },\n        popup: function (editor, position, view) {\n            var endView = view.nameLabel;\n            if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n            var rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, endView, {\n                id: QUICKEDIT_ACTIVITYEDGE,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_ADD_NOTE ],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            QuickEdit.setInputValue(ITEM_EDIT, model.name);\n        }\n    });\n\n    // QuickEdit for Partition\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLSwimlaneView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameLabel,\n                rect  = new Graphics.Rect(label.left, label.top, label.getRight(), label.getBottom());\n            if (view.isVertical !== true) {\n                rect.setRect(label.left, label.top, label.left + label.height, label.top + label.width);\n            }\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_PARTITION,\n                main: QE_EDIT_GENERAL,\n                leftSection: [],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, UMLUtils.getNameExpression(view.model));\n        }\n    });\n\n    // QuickEdit for Constraint\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLConstraintView);\n        },\n        popup: function (editor, position, view) {\n            var rect = view.getBoundingBox(editor.canvas);\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_CONSTRAINT,\n                main: QE_EDIT_TEXTAREA,\n                leftSection: [],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, view.model.specification);\n        }\n    });\n\n    // QuickEdit for TextArea\n    // - Text, Note\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.UMLTextView) ||\n                   (view instanceof type.UMLNoteView);\n        },\n        popup: function (editor, position, view) {\n            var rect = view.getBoundingBox(editor.canvas);\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_TEXTAREA,\n                main: QE_EDIT_TEXTAREA,\n                leftSection: [],\n                rightSection: []\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, view.text);\n        }\n    });\n\n    // Define public API\n    exports.QUICKEDIT_GENERAL             = QUICKEDIT_GENERAL;\n    exports.QUICKEDIT_FLOATINGNODE        = QUICKEDIT_FLOATINGNODE;\n    exports.QUICKEDIT_SUBSYSTEM           = QUICKEDIT_SUBSYSTEM;\n    exports.QUICKEDIT_PACKAGE             = QUICKEDIT_PACKAGE;\n    exports.QUICKEDIT_CLASSIFIER          = QUICKEDIT_CLASSIFIER;\n    exports.QUICKEDIT_CLASS               = QUICKEDIT_CLASS;\n    exports.QUICKEDIT_INTERFACE           = QUICKEDIT_INTERFACE;\n    exports.QUICKEDIT_COMPONENT           = QUICKEDIT_COMPONENT;\n    exports.QUICKEDIT_NODE                = QUICKEDIT_NODE;\n    exports.QUICKEDIT_ACTOR               = QUICKEDIT_ACTOR;\n    exports.QUICKEDIT_USECASE             = QUICKEDIT_USECASE;\n    exports.QUICKEDIT_ENUMERATION         = QUICKEDIT_ENUMERATION;\n    exports.QUICKEDIT_METACLASS           = QUICKEDIT_METACLASS;\n    exports.QUICKEDIT_STEREOTYPE          = QUICKEDIT_STEREOTYPE;\n    exports.QUICKEDIT_ATTRIBUTE           = QUICKEDIT_ATTRIBUTE;\n    exports.QUICKEDIT_OPERATION           = QUICKEDIT_OPERATION;\n    exports.QUICKEDIT_RECEPTION           = QUICKEDIT_RECEPTION;\n    exports.QUICKEDIT_PORT                = QUICKEDIT_PORT;\n    exports.QUICKEDIT_PART                = QUICKEDIT_PART;\n    exports.QUICKEDIT_TEMPLATE_PARAMETER  = QUICKEDIT_TEMPLATE_PARAMETER;\n    exports.QUICKEDIT_QUALIFIER           = QUICKEDIT_QUALIFIER;\n    exports.QUICKEDIT_LITERAL             = QUICKEDIT_LITERAL;\n    exports.QUICKEDIT_EXTENSION_POINT     = QUICKEDIT_EXTENSION_POINT;\n    exports.QUICKEDIT_OBJECT              = QUICKEDIT_OBJECT;\n    exports.QUICKEDIT_SLOT                = QUICKEDIT_SLOT;\n    exports.QUICKEDIT_SEQ_LIFELINE        = QUICKEDIT_SEQ_LIFELINE;\n    exports.QUICKEDIT_SEQ_MESSAGE         = QUICKEDIT_SEQ_MESSAGE;\n    exports.QUICKEDIT_COMM_LIFELINE       = QUICKEDIT_COMM_LIFELINE;\n    exports.QUICKEDIT_COMM_MESSAGE        = QUICKEDIT_COMM_MESSAGE;\n    exports.QUICKEDIT_CONNECTOR           = QUICKEDIT_CONNECTOR;\n    exports.QUICKEDIT_STATEINVARIANT      = QUICKEDIT_STATEINVARIANT;\n    exports.QUICKEDIT_CONTINUATION        = QUICKEDIT_CONTINUATION;\n    exports.QUICKEDIT_COMBINEDFRAGMENT    = QUICKEDIT_COMBINEDFRAGMENT;\n    exports.QUICKEDIT_OPERAND             = QUICKEDIT_OPERAND;\n    exports.QUICKEDIT_DIRECTED            = QUICKEDIT_DIRECTED;\n    exports.QUICKEDIT_UNDIRECTED          = QUICKEDIT_UNDIRECTED;\n    exports.QUICKEDIT_UNDIRECTED_HEAD     = QUICKEDIT_UNDIRECTED_HEAD;\n    exports.QUICKEDIT_UNDIRECTED_TAIL     = QUICKEDIT_UNDIRECTED_TAIL;\n    exports.QUICKEDIT_ASSOCIATION         = QUICKEDIT_ASSOCIATION;\n    exports.QUICKEDIT_ASSOCIATION_HEAD    = QUICKEDIT_ASSOCIATION_HEAD;\n    exports.QUICKEDIT_ASSOCIATION_TAIL    = QUICKEDIT_ASSOCIATION_TAIL;\n    exports.QUICKEDIT_STATE               = QUICKEDIT_STATE;\n    exports.QUICKEDIT_INTERNAL_ACTIVITY   = QUICKEDIT_INTERNAL_ACTIVITY;\n    exports.QUICKEDIT_INTERNAL_TRANSITION = QUICKEDIT_INTERNAL_TRANSITION;\n    exports.QUICKEDIT_TRANSITION          = QUICKEDIT_TRANSITION;\n    exports.QUICKEDIT_ACTION              = QUICKEDIT_ACTION;\n    exports.QUICKEDIT_OBJECT_NODE         = QUICKEDIT_OBJECT_NODE;\n    exports.QUICKEDIT_ACTIVITYEDGE        = QUICKEDIT_ACTIVITYEDGE;\n    exports.QUICKEDIT_PARTITION           = QUICKEDIT_PARTITION;\n    exports.QUICKEDIT_CONSTRAINT          = QUICKEDIT_CONSTRAINT;\n    exports.QUICKEDIT_TEXTAREA            = QUICKEDIT_TEXTAREA;\n\n    exports.ITEM_EDIT                = ITEM_EDIT;\n    exports.ITEM_VISIBILITY          = ITEM_VISIBILITY;\n    exports.ITEM_AGGREGATION         = ITEM_AGGREGATION;\n    exports.ITEM_NAVIGABILITY        = ITEM_NAVIGABILITY;\n    exports.ITEM_MESSAGE_SORT        = ITEM_MESSAGE_SORT;\n    exports.ITEM_ADD                 = ITEM_ADD;\n    exports.ITEM_DELETE              = ITEM_DELETE;\n    exports.ITEM_MOVEUP              = ITEM_MOVEUP;\n    exports.ITEM_MOVEDOWN            = ITEM_MOVEDOWN;\n    exports.ITEM_ADD_ATTRIBUTE       = ITEM_ADD_ATTRIBUTE;\n    exports.ITEM_ADD_OPERATION       = ITEM_ADD_OPERATION;\n    exports.ITEM_ADD_RECEPTION       = ITEM_ADD_RECEPTION;\n    exports.ITEM_ADD_QUALIFIER       = ITEM_ADD_QUALIFIER;\n    exports.ITEM_ADD_LITERAL         = ITEM_ADD_LITERAL;\n    exports.ITEM_ADD_SLOT            = ITEM_ADD_SLOT;\n    exports.ITEM_ADD_EXTENSIONPOINT  = ITEM_ADD_EXTENSIONPOINT;\n    exports.ITEM_MULTIPLICITY        = ITEM_MULTIPLICITY;\n    exports.ITEM_SELECT_TYPE         = ITEM_SELECT_TYPE;\n    exports.ITEM_CREATE_TYPE         = ITEM_CREATE_TYPE;\n    exports.ITEM_SELECT_OPERATION    = ITEM_SELECT_OPERATION;\n    exports.ITEM_CREATE_OPERATION    = ITEM_CREATE_OPERATION;\n    exports.ITEM_SELECT_SIGNAL       = ITEM_SELECT_SIGNAL;\n    exports.ITEM_CREATE_SIGNAL       = ITEM_CREATE_SIGNAL;\n    exports.ITEM_ADD_OPERAND         = ITEM_ADD_OPERAND;\n    exports.ITEM_ADD_REGION          = ITEM_ADD_REGION;\n    exports.ITEM_ADD_ENTRY_ACTIVITY  = ITEM_ADD_ENTRY_ACTIVITY;\n    exports.ITEM_ADD_DO_ACTIVITY     = ITEM_ADD_DO_ACTIVITY;\n    exports.ITEM_ADD_EXIT_ACTIVITY   = ITEM_ADD_EXIT_ACTIVITY;\n    exports.ITEM_ADD_INTERNAL_TRANSITION = ITEM_ADD_INTERNAL_TRANSITION;\n    exports.ITEM_ADD_TRIGGER         = ITEM_ADD_TRIGGER;\n    exports.ITEM_ADD_EFFECT          = ITEM_ADD_EFFECT;\n    exports.ITEM_ADD_NOTE            = ITEM_ADD_NOTE;\n    exports.ITEM_ADD_SUBPACKAGE      = ITEM_ADD_SUBPACKAGE;\n    exports.ITEM_ADD_DEPENDING_PACKAGE = ITEM_ADD_DEPENDING_PACKAGE;\n    exports.ITEM_ADD_DEPENDANT_PACKAGE = ITEM_ADD_DEPENDANT_PACKAGE;\n    exports.ITEM_ADD_SUPERCLASS      = ITEM_ADD_SUPERCLASS;\n    exports.ITEM_ADD_SUBCLASS        = ITEM_ADD_SUBCLASS;\n    exports.ITEM_ADD_ASSOCIATION     = ITEM_ADD_ASSOCIATION;\n    exports.ITEM_ADD_AGGREGATION     = ITEM_ADD_AGGREGATION;\n    exports.ITEM_ADD_COMPOSITION     = ITEM_ADD_COMPOSITION;\n    exports.ITEM_ADD_PORT            = ITEM_ADD_PORT;\n    exports.ITEM_ADD_PART            = ITEM_ADD_PART;\n    exports.ITEM_ADD_CONNECTED_PART  = ITEM_ADD_CONNECTED_PART;\n    exports.ITEM_ADD_LINKED_OBJECT   = ITEM_ADD_LINKED_OBJECT;\n    exports.ITEM_ADD_PROVIDED_INTERFACE = ITEM_ADD_PROVIDED_INTERFACE;\n    exports.ITEM_ADD_REQUIRED_INTERFACE = ITEM_ADD_REQUIRED_INTERFACE;\n    exports.ITEM_ADD_SUPERINTERFACE  = ITEM_ADD_SUPERINTERFACE;\n    exports.ITEM_ADD_SUBINTERFACE    = ITEM_ADD_SUBINTERFACE;\n    exports.ITEM_ADD_REALIZING_CLASS = ITEM_ADD_REALIZING_CLASS;\n    exports.ITEM_ADD_COMMUNICATING_NODE = ITEM_ADD_COMMUNICATING_NODE;\n    exports.ITEM_ADD_DEPLOYED_ARTIFACT  = ITEM_ADD_DEPLOYED_ARTIFACT;\n    exports.ITEM_ADD_DEPLOYED_COMPONENT = ITEM_ADD_DEPLOYED_COMPONENT;\n    exports.ITEM_ADD_SUPERACTOR         = ITEM_ADD_SUPERACTOR;\n    exports.ITEM_ADD_SUBACTOR           = ITEM_ADD_SUBACTOR;\n    exports.ITEM_ADD_ASSOCIATED_USECASE = ITEM_ADD_ASSOCIATED_USECASE;\n    exports.ITEM_ADD_ASSOCIATED_ACTOR   = ITEM_ADD_ASSOCIATED_ACTOR;\n    exports.ITEM_ADD_INCLUDED_USECASE   = ITEM_ADD_INCLUDED_USECASE;\n    exports.ITEM_ADD_EXTENDED_USECASE   = ITEM_ADD_EXTENDED_USECASE;\n    exports.ITEM_ADD_MESSAGE_LIFELINE   = ITEM_ADD_MESSAGE_LIFELINE;\n    exports.ITEM_ADD_CREATE_MESSAGE_LIFELINE = ITEM_ADD_CREATE_MESSAGE_LIFELINE;\n    exports.ITEM_ADD_SELF_MESSAGE       = ITEM_ADD_SELF_MESSAGE;\n    exports.ITEM_ADD_MESSAGE_FROM_GATE  = ITEM_ADD_MESSAGE_FROM_GATE;\n    exports.ITEM_ADD_MESSAGE_TO_GATE    = ITEM_ADD_MESSAGE_TO_GATE;\n    exports.ITEM_ADD_FOUND_MESSAGE      = ITEM_ADD_FOUND_MESSAGE;\n    exports.ITEM_ADD_LOST_MESSAGE       = ITEM_ADD_LOST_MESSAGE;\n    exports.ITEM_ADD_REPLY_MESSAGE      = ITEM_ADD_REPLY_MESSAGE;\n    exports.ITEM_ADD_CONNECTED_LIFELINE = ITEM_ADD_CONNECTED_LIFELINE;\n    exports.ITEM_ADD_SELF_CONNECTOR     = ITEM_ADD_SELF_CONNECTOR;\n    exports.ITEM_ADD_FORWARD_MESSAGE    = ITEM_ADD_FORWARD_MESSAGE;\n    exports.ITEM_ADD_REVERSE_MESSAGE    = ITEM_ADD_REVERSE_MESSAGE;\n    exports.ITEM_ADD_OUTGOING_TRANSITION = ITEM_ADD_OUTGOING_TRANSITION;\n    exports.ITEM_ADD_INCOMING_TRANSITION = ITEM_ADD_INCOMING_TRANSITION;\n    exports.ITEM_ADD_CONNECTION_POINT_REFERENCE = ITEM_ADD_CONNECTION_POINT_REFERENCE;\n    exports.ITEM_ADD_INITIAL_STATE      = ITEM_ADD_INITIAL_STATE;\n    exports.ITEM_ADD_FINAL_STATE        = ITEM_ADD_FINAL_STATE;\n    exports.ITEM_ADD_CHOICE             = ITEM_ADD_CHOICE;\n    exports.ITEM_ADD_FORK               = ITEM_ADD_FORK;\n    exports.ITEM_ADD_JOIN               = ITEM_ADD_JOIN;\n    exports.ITEM_ADD_OUTGOING_CONTROL_FLOW = ITEM_ADD_OUTGOING_CONTROL_FLOW;\n    exports.ITEM_ADD_INCOMING_CONTROL_FLOW = ITEM_ADD_INCOMING_CONTROL_FLOW;\n    exports.ITEM_ADD_OUTGOING_OBJECT_FLOW  = ITEM_ADD_OUTGOING_OBJECT_FLOW;\n    exports.ITEM_ADD_INCOMING_OBJECT_FLOW  = ITEM_ADD_INCOMING_OBJECT_FLOW;\n    exports.ITEM_ADD_DECISION              = ITEM_ADD_DECISION;\n    exports.ITEM_ADD_MERGE                 = ITEM_ADD_MERGE;\n    exports.ITEM_ADD_ACTIVITY_FORK         = ITEM_ADD_ACTIVITY_FORK;\n    exports.ITEM_ADD_ACTIVITY_JOIN         = ITEM_ADD_ACTIVITY_JOIN;\n    exports.ITEM_ADD_INPUT_PIN             = ITEM_ADD_INPUT_PIN;\n    exports.ITEM_ADD_OUTPUT_PIN            = ITEM_ADD_OUTPUT_PIN;\n    exports.ITEM_ADD_INITIAL_NODE          = ITEM_ADD_INITIAL_NODE;\n    exports.ITEM_ADD_FINAL_NODE            = ITEM_ADD_FINAL_NODE;\n    exports.ITEM_ADD_STEREOTYPE            = ITEM_ADD_STEREOTYPE;\n    exports.ITEM_ADD_SUPERSTEREOTYPE       = ITEM_ADD_SUPERSTEREOTYPE;\n    exports.ITEM_ADD_SUBSTEREOTYPE         = ITEM_ADD_SUBSTEREOTYPE;\n\n});\n\n//# sourceURL=/uml/UMLQuickEdit.js"),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, browser: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define*/\n\ndefine(\'uml/UML-meta\',[\'require\',\'exports\',\'module\',\'core/MetaModelManager\',\'core/Core-meta\'],function (require, exports, module) {\n    "use strict";\n\n    var MetaModelManager  = require("core/MetaModelManager");\n    require("core/Core-meta");\n\n    var metaModel = {\n        "UMLVisibilityKind": {\n            "kind": "enum",\n            "literals": [ "public", "protected", "private", "package" ]\n        },\n        "UMLAggregationKind": {\n            "kind": "enum",\n            "literals": [ "none", "shared", "composite" ]\n        },\n        "UMLDirectionKind": {\n            "kind": "enum",\n            "literals": [ "in", "inout", "out", "return" ]\n        },\n        "UMLCallConcurrencyKind": {\n            "kind": "enum",\n            "literals": [ "sequential", "guarded", "concurrent" ]\n        },\n        "UMLConnectorKind": {\n            "kind": "enum",\n            "literals": [ "assembly", "delegation" ]\n        },\n        "UMLMessageSort": {\n            "kind": "enum",\n            "literals": [ "synchCall", "asynchCall", "asynchSignal", "createMessage", "deleteMessage", "reply" ]\n        },\n        "UMLPseudostateKind": {\n            "kind": "enum",\n            "literals": [ "initial", "deepHistory", "shallowHistory", "join", "fork", "junction", "choice", "entryPoint", "exitPoint", "terminate" ]\n        },\n        "UMLTransitionKind": {\n            "kind": "enum",\n            "literals": [ "external", "internal", "local" ]\n        },\n        "UMLEventKind": {\n            "kind": "enum",\n            "literals": [ "signal", "call", "change", "time", "anyReceive" ]\n        },\n        "UMLInteractionOperatorKind": {\n            "kind": "enum",\n            "literals": [ "alt", "opt", "par", "loop", "critical", "neg", "assert", "strict", "seq", "ignore", "consider", "break" ]\n        },\n        "UMLActionKind": {\n            "kind": "enum",\n            "literals": [ "opaque", "create", "destroy", "read", "write", "insert", "delete", "sendSignal", "acceptSignal", "triggerEvent", "acceptEvent", "structured", "timeEvent" ]\n        },\n        "UMLObjectNodeOrderingKind": {\n            "kind": "enum",\n            "literals": [ "unordered", "ordered", "LIFO", "FIFO" ]\n        },\n        "UMLExpansionKind": {\n            "kind": "enum",\n            "literals": [ "parallel", "iterative", "stream" ]\n        },\n        "UMLStereotypeDisplayKind": {\n            "kind": "enum",\n            "literals": [ "none", "label", "decoration", "decoration-label", "icon", "icon-label" ]\n        },\n        "UMLModelElement": {\n            "kind": "class",\n            "super": "ExtensibleModel",\n            "attributes": [\n                { "name": "stereotype",         "kind": "var",  "type": "UMLStereotype", "visible": true },\n                { "name": "visibility",         "kind": "enum", "type": "UMLVisibilityKind", "default": "public", "visible": true },\n                { "name": "templateParameters", "kind": "objs", "type": "UMLTemplateParameter" }\n            ]\n        },\n        "UMLConstraint": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "specification",       "kind": "prim", "type": "String", "visible": true, "multiline": true },\n                { "name": "constrainedElements", "kind": "refs", "type": "UMLModelElement", "visible": true }\n            ],\n            "view": "UMLConstraintView"\n        },\n        "UMLTemplateParameter": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "parameterType", "kind": "prim", "type": "String", "visible": true },\n                { "name": "defaultValue",  "kind": "prim", "type": "String", "visible": true }\n            ],\n            "ordering": 1101\n        },\n        "UMLFeature": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "isStatic", "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isLeaf",   "kind": "prim", "type": "Boolean", "visible": true, "default": false }\n            ]\n        },\n        "UMLStructuralFeature": {\n            "kind": "class",\n            "super": "UMLFeature",\n            "attributes": [\n                { "name": "type",          "kind": "var",  "type": "UMLClassifier", "visible": true },\n                { "name": "multiplicity",  "kind": "prim", "type": "String", "visible": true, "options": [ "0..1", "1", "0..*", "1..*", "*" ] },\n                { "name": "isReadOnly",    "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isOrdered",     "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isUnique",      "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "defaultValue",  "kind": "prim", "type": "String", "visible": true }\n            ]\n        },\n        "UMLParameter": {\n            "kind": "class",\n            "super": "UMLStructuralFeature",\n            "attributes": [\n                { "name": "direction",    "kind": "enum", "type": "UMLDirectionKind", "visible": true, "default": "in" }\n            ],\n            "ordering": 1102\n        },\n        "UMLBehavioralFeature": {\n            "kind": "class",\n            "super": "UMLFeature",\n            "attributes": [\n                { "name": "parameters",       "kind": "objs", "type": "UMLParameter", "visible": true },\n                { "name": "raisedExceptions", "kind": "refs", "type": "UMLClassifier", "visible": true },\n                { "name": "concurrency",      "kind": "enum", "type": "UMLCallConcurrencyKind", "visible": true, "default": "sequential" }\n            ]\n        },\n        "UMLAttribute": {\n            "kind": "class",\n            "super": "UMLStructuralFeature",\n            "attributes": [\n                { "name": "isDerived",    "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "aggregation",  "kind": "enum", "type": "UMLAggregationKind", "visible": true, "default": "none" },\n                { "name": "isID",         "kind": "prim", "type": "Boolean", "visible": true, "default": false }\n            ],\n            "view": "UMLPartView",\n            "ordering": 1105\n        },\n        "UMLOperation": {\n            "kind": "class",\n            "super": "UMLBehavioralFeature",\n            "attributes": [\n                { "name": "isQuery",        "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isAbstract",     "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "specification",  "kind": "prim", "type": "String", "visible": true, "multiline": true },\n                { "name": "preconditions",  "kind": "objs", "type": "UMLConstraint" },\n                { "name": "bodyConditions", "kind": "objs", "type": "UMLConstraint" },\n                { "name": "postconditions", "kind": "objs", "type": "UMLConstraint" }\n            ],\n            "ordering": 1106\n        },\n        "UMLReception": {\n            "kind": "class",\n            "super": "UMLBehavioralFeature",\n            "attributes": [\n                { "name": "signal", "kind": "ref", "type": "UMLSignal", "visible": true }\n            ],\n            "ordering": 1107\n        },\n        "UMLClassifier": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "attributes",            "kind": "objs", "type": "UMLAttribute" },\n                { "name": "operations",            "kind": "objs", "type": "UMLOperation" },\n                { "name": "receptions",            "kind": "objs", "type": "UMLReception" },\n                { "name": "behaviors",             "kind": "objs", "type": "UMLBehavior" },\n                { "name": "isAbstract",            "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isFinalSpecialization", "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isLeaf",                "kind": "prim", "type": "Boolean", "visible": true, "default": false }\n            ]\n        },\n        "UMLDirectedRelationship": {\n            "kind": "class",\n            "super": "DirectedRelationship",\n            "attributes": [\n                { "name": "stereotype",         "kind": "var",  "type": "UMLStereotype", "visible": true },\n                { "name": "visibility",         "kind": "enum", "type": "UMLVisibilityKind", "default": "public", "visible": true }\n            ]\n        },\n        "UMLRelationshipEnd": {\n            "kind": "class",\n            "super": "RelationshipEnd",\n            "attributes": [\n                { "name": "stereotype",   "kind": "prim", "type": "String", "visible": true },\n                { "name": "visibility",   "kind": "enum", "type": "UMLVisibilityKind", "default": "public", "visible": true },\n                { "name": "navigable",    "kind": "prim", "type": "Boolean", "visible": true, "default": true },\n                { "name": "aggregation",  "kind": "enum", "type": "UMLAggregationKind", "visible": true, "default": "none" },\n                { "name": "multiplicity", "kind": "prim", "type": "String", "visible": true, "options": [ "0..1", "1", "0..*", "1..*", "*" ] },\n                { "name": "defaultValue", "kind": "prim", "type": "String", "visible": true },\n                { "name": "isReadOnly",   "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isOrdered",    "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isUnique",     "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isDerived",    "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isID",         "kind": "prim", "type": "Boolean", "visible": true, "default": false }\n            ]\n        },\n        "UMLUndirectedRelationship": {\n            "kind": "class",\n            "super": "UndirectedRelationship",\n            "attributes": [\n                { "name": "stereotype", "kind": "var",  "type": "UMLStereotype", "visible": true },\n                { "name": "visibility", "kind": "enum", "type": "UMLVisibilityKind", "default": "public", "visible": true }\n            ]\n        },\n        "UMLBehavior": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "isReentrant",    "kind": "prim", "type": "Boolean", "visible": true, "default": true },\n                { "name": "parameters",     "kind": "objs", "type": "UMLParameter" },\n                { "name": "preconditions",  "kind": "objs", "type": "UMLConstraint" },\n                { "name": "postconditions", "kind": "objs", "type": "UMLConstraint" }\n            ]\n        },\n        "UMLOpaqueBehavior": {\n            "kind": "class",\n            "super": "UMLBehavior"\n        },\n        "UMLEvent": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "kind",            "kind": "enum", "type": "UMLEventKind", "visible": true, "default": "anyReceive" },\n                { "name": "value",           "kind": "prim", "type": "String",       "visible": true },\n                { "name": "expression",      "kind": "prim", "type": "String",       "visible": true },\n                { "name": "targetOperation", "kind": "ref",  "type": "UMLOperation", "visible": true },\n                { "name": "targetSignal",    "kind": "ref",  "type": "UMLSignal",    "visible": true }\n            ]\n        },\n        "UMLPackage": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "importedElements", "kind": "refs", "type": "UMLModelElement", "visible": true }\n            ],\n            "view": "UMLPackageView",\n            "ordering": 1001\n        },\n        "UMLModel": {\n            "kind": "class",\n            "super": "UMLPackage",\n            "attributes": [\n                { "name": "viewpoint", "kind": "prim", "type": "String", "visible": true }\n            ],\n            "view": "UMLModelView",\n            "ordering": 1000\n        },\n        "UMLClass": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "attributes": [\n                { "name": "isActive",   "kind": "prim", "type": "Boolean", "visible": true, "default": false }\n            ],\n            "view": "UMLClassView",\n            "ordering": 1201\n        },\n        "UMLInterface": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "view": "UMLInterfaceView",\n            "ordering": 1202\n        },\n        "UMLSignal": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "view": "UMLSignalView",\n            "ordering": 1203\n        },\n        "UMLDataType": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "view": "UMLDataTypeView",\n            "ordering": 1204\n        },\n        "UMLPrimitiveType": {\n            "kind": "class",\n            "super": "UMLDataType",\n            "view": "UMLPrimitiveTypeView",\n            "ordering": 1205\n        },\n        "UMLEnumerationLiteral": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "ordering": 1103\n        },\n        "UMLEnumeration": {\n            "kind": "class",\n            "super": "UMLDataType",\n            "attributes": [\n                { "name": "literals", "kind": "objs", "type": "UMLEnumerationLiteral" }\n            ],\n            "view": "UMLEnumerationView",\n            "ordering": 1206\n        },\n        "UMLDependency": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "attributes": [\n                { "name": "mapping", "kind": "prim", "type": "String", "visible": true }\n            ],\n            "view": "UMLDependencyView",\n            "ordering": 1806\n        },\n        "UMLAbstraction": {\n            "kind": "class",\n            "super": "UMLDependency",\n            "ordering": 1811\n        },\n        "UMLRealization": {\n            "kind": "class",\n            "super": "UMLAbstraction",\n            "ordering": 1808,\n            "view": "UMLRealizationView"\n        },\n        "UMLGeneralization": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "attributes": [\n                { "name": "discriminator", "kind": "prim", "type": "String", "visible": true }\n            ],\n            "view": "UMLGeneralizationView",\n            "ordering": 1807\n        },\n        "UMLInterfaceRealization": {\n            "kind": "class",\n            "super": "UMLRealization",\n            "view": "UMLInterfaceRealizationView",\n            "ordering": 1809\n        },\n        "UMLComponentRealization": {\n            "kind": "class",\n            "super": "UMLRealization",\n            "view": "UMLComponentRealizationView",\n            "ordering": 1810\n        },\n        "UMLAssociationEnd": {\n            "kind": "class",\n            "super": "UMLRelationshipEnd",\n            "attributes": [\n                { "name": "qualifiers",    "kind": "objs", "type": "UMLAttribute", "visible": true }\n            ],\n            "ordering": 1801\n        },\n        "UMLAssociation": {\n            "kind": "class",\n            "super": "UMLUndirectedRelationship",\n            "attributes": [\n                { "name": "isDerived", "kind": "prim", "type": "Boolean", "visible": true, "default": false }\n            ],\n            "view": "UMLAssociationView",\n            "ordering": 1802\n        },\n        "UMLAssociationClassLink": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "classSide",       "kind": "ref", "type": "UMLClass", "visible": true },\n                { "name": "associationSide", "kind": "ref", "type": "UMLAssociation", "visible": true }\n            ],\n            "view": "UMLAssociationClassLinkView",\n            "ordering": 1803\n        },\n        "UMLSlot": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "definingFeature", "kind": "ref",  "type": "UMLStructuralFeature", "visible": true },\n                { "name": "type",            "kind": "var",  "type": "UMLClassifier", "visible": true },\n                { "name": "value",           "kind": "prim", "type": "String", "visible": true }\n            ],\n            "ordering": 1109\n        },\n        "UMLInstance": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "classifier", "kind": "var",  "type": "UMLClassifier", "visible": true },\n                { "name": "slots",      "kind": "objs", "type": "UMLSlot" },\n                { "name": "value",      "kind": "prim", "type": "String", "visible": true }\n            ],\n            "ordering": 1301\n        },\n        "UMLObject": {\n            "kind": "class",\n            "super": "UMLInstance",\n            "view": "UMLObjectView",\n            "ordering": 1302\n        },\n        "UMLArtifactInstance": {\n            "kind": "class",\n            "super": "UMLInstance",\n            "view": "UMLArtifactInstanceView",\n            "ordering": 1303\n        },\n        "UMLComponentInstance": {\n            "kind": "class",\n            "super": "UMLInstance",\n            "view": "UMLComponentInstanceView",\n            "ordering": 1304\n        },\n        "UMLNodeInstance": {\n            "kind": "class",\n            "super": "UMLInstance",\n            "view": "UMLNodeInstanceView",\n            "ordering": 1305\n        },\n        "UMLLinkEnd": {\n            "kind": "class",\n            "super": "UMLRelationshipEnd",\n            "ordering": 1814\n        },\n        "UMLLink": {\n            "kind": "class",\n            "super": "UMLUndirectedRelationship",\n            "attributes": [\n                { "name": "association",  "kind": "ref",  "type": "UMLAssociation", "visible": true }\n            ],\n            "view": "UMLLinkView",\n            "ordering": 1815\n        },\n        "UMLPort": {\n            "kind": "class",\n            "super": "UMLAttribute",\n            "attributes": [\n                { "name": "isBehavior",   "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isService",    "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isConjugated", "kind": "prim", "type": "Boolean", "visible": true, "default": false }\n            ],\n            "view": "UMLPortView",\n            "ordering": 1104\n        },\n        "UMLConnectorEnd": {\n            "kind": "class",\n            "super": "UMLRelationshipEnd",\n            "ordering": 1812\n        },\n        "UMLConnector": {\n            "kind": "class",\n            "super": "UMLUndirectedRelationship",\n            "attributes": [\n                { "name": "type", "kind": "ref",  "type": "UMLAssociation",   "visible": true },\n                { "name": "kind", "kind": "enum", "type": "UMLConnectorKind", "visible": true, "default": "assembly" },\n            ],\n            "view": "UMLConnectorView",\n            "ordering": 1813\n        },\n        "UMLCollaboration": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "view": "UMLCollaborationView",\n            "ordering": 1401\n        },\n        "UMLCollaborationUse": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "type", "kind": "ref", "type": "UMLCollaboration", "visible": true }\n            ],\n            "view": "UMLCollaborationUseView",\n            "ordering": 1405\n        },\n        "UMLRoleBinding": {\n            "kind": "class",\n            "super": "UMLDependency",\n            "attributes": [\n                { "name": "roleName", "kind": "prim", "type": "String", "visible": true }\n            ],\n            "view": "UMLRoleBindingView",\n            "ordering": 1818\n        },\n        "UMLArtifact": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "attributes": [\n                { "name": "fileName", "kind": "prim", "type": "String", "visible": true }\n            ],\n            "view": "UMLArtifactView",\n            "ordering": 1207\n        },\n        "UMLComponent": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "attributes": [\n                { "name": "isIndirectlyInstantiated", "kind": "prim", "type": "Boolean", "visible": true, "default": true }\n            ],\n            "view": "UMLComponentView",\n            "ordering": 1208\n        },\n        "UMLSubsystem": {\n            "kind": "class",\n            "super": "UMLComponent",\n            "view": "UMLSubsystemView",\n            "ordering": 1002\n        },\n        "UMLNode": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "view": "UMLNodeView",\n            "ordering": 1209\n        },\n        "UMLDeployment": {\n            "kind": "class",\n            "super": "UMLDependency",\n            "view": "UMLDeploymentView",\n            "ordering": 1804\n        },\n        "UMLCommunicationPath": {\n            "kind": "class",\n            "super": "UMLAssociation",\n            "view": "UMLCommunicationPathView",\n            "ordering": 1805\n        },\n        "UMLExtensionPoint": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "location", "kind": "prim", "type": "String", "visible": true }\n            ],\n            "ordering": 1108\n        },\n        "UMLUseCase": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "attributes": [\n                { "name": "extensionPoints", "kind": "objs", "type": "UMLExtensionPoint", "visible": true }\n            ],\n            "view": "UMLUseCaseView",\n            "ordering": 1210\n        },\n        "UMLActor": {\n            "kind": "class",\n            "super": "UMLClassifier",\n            "view": "UMLActorView",\n            "ordering": 1211\n        },\n        "UMLInclude": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "view": "UMLIncludeView",\n            "ordering": 1816\n        },\n        "UMLExtend": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "attributes": [\n                { "name": "condition", "kind": "prim", "type": "String", "visible": true },\n                { "name": "extensionLocations", "kind": "refs", "type": "UMLExtensionPoint", "visible": true }\n            ],\n            "view": "UMLExtendView",\n            "ordering": 1817\n        },\n        "UMLUseCaseSubject": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "represent", "kind": "ref", "type": "UMLClassifier", "visible": true }\n            ],\n            "view": "UMLUseCaseSubjectView",\n            "ordering": 1003\n        },\n        "UMLStateMachine": {\n            "kind": "class",\n            "super": "UMLBehavior",\n            "attributes": [\n                { "name": "regions", "kind": "objs", "type": "UMLRegion"}\n            ],\n            "ordering": 1402\n        },\n        "UMLRegion": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "vertices",    "kind": "objs", "type": "UMLVertex" },\n                { "name": "transitions", "kind": "objs", "type": "UMLTransition" }\n            ],\n            "ordering": 1407\n        },\n        "UMLVertex": {\n            "kind": "class",\n            "super": "UMLModelElement"\n        },\n        "UMLPseudostate": {\n            "kind": "class",\n            "super": "UMLVertex",\n            "attributes": [\n                { "name": "kind", "kind": "enum", "type": "UMLPseudostateKind", "visible": true }\n            ],\n            "view": "UMLPseudostateView",\n            "ordering": 1408\n        },\n        "UMLConnectionPointReference": {\n            "kind": "class",\n            "super": "UMLVertex",\n            "attributes": [\n                { "name": "entry", "kind": "refs", "type": "UMLPseudostate", "visible": true },\n                { "name": "exit",  "kind": "refs", "type": "UMLPseudostate", "visible": true }\n            ],\n            "view": "UMLConnectionPointReferenceView",\n            "ordering": 1408\n        },\n        "UMLState": {\n            "kind": "class",\n            "super": "UMLVertex",\n            "attributes": [\n                { "name": "regions",         "kind": "objs", "type": "UMLRegion" },\n                { "name": "entryActivities", "kind": "objs", "type": "UMLBehavior" },\n                { "name": "doActivities",    "kind": "objs", "type": "UMLBehavior" },\n                { "name": "exitActivities",  "kind": "objs", "type": "UMLBehavior" },\n                { "name": "submachine",      "kind": "ref",  "type": "UMLStateMachine", "visible": true },\n                { "name": "connections",     "kind": "objs", "type": "UMLConnectionPointReference" }\n            ],\n            "view": "UMLStateView",\n            "ordering": 1409\n        },\n        "UMLFinalState": {\n            "kind": "class",\n            "super": "UMLState",\n            "view": "UMLFinalStateView",\n            "ordering": 1410\n        },\n        "UMLTransition": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "attributes": [\n                { "name": "kind",       "kind": "enum", "type": "UMLTransitionKind", "visible": true, "default": "external" },\n                { "name": "guard",      "kind": "prim", "type": "String",            "visible": true },\n                { "name": "triggers",   "kind": "objs", "type": "UMLEvent" },\n                { "name": "effects",    "kind": "objs", "type": "UMLBehavior" }\n            ],\n            "view": "UMLTransitionView",\n            "ordering": 1819\n        },\n        "UMLActivity": {\n            "kind": "class",\n            "super": "UMLBehavior",\n            "attributes": [\n                { "name": "isReadOnly",        "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isSingleExecution", "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "nodes",             "kind": "objs", "type": "UMLActivityNode" },\n                { "name": "edges",             "kind": "objs", "type": "UMLActivityEdge" },\n                { "name": "groups",            "kind": "objs", "type": "UMLActivityGroup" }\n            ],\n            "ordering": 1404\n        },\n        "UMLPin": {\n            "kind": "class",\n            "super": "UMLStructuralFeature"\n        },\n        "UMLInputPin": {\n            "kind": "class",\n            "super": "UMLPin",\n            "view": "UMLInputPinView"\n        },\n        "UMLOutputPin": {\n            "kind": "class",\n            "super": "UMLPin",\n            "view": "UMLOutputPinView"\n        },\n        "UMLExpansionNode": {\n            "kind": "class",\n            "super": "UMLPin",\n            "view": "UMLExpansionNodeView",\n            "ordering": 1412\n        },\n        "UMLActivityNode": {\n            "kind": "class",\n            "super": "UMLModelElement"\n        },\n        "UMLAction": {\n            "kind": "class",\n            "super": "UMLActivityNode",\n            "attributes": [\n                { "name": "kind",                "kind": "enum", "type": "UMLActionKind", "visible": true, "default": "opaque" },\n                { "name": "inputs",              "kind": "objs", "type": "UMLInputPin" },\n                { "name": "outputs",             "kind": "objs", "type": "UMLOutputPin" },\n                { "name": "triggers",            "kind": "objs", "type": "UMLEvent" },\n                { "name": "target",              "kind": "ref",  "type": "UMLModelElement", "visible": true },\n                { "name": "subactivity",         "kind": "ref",  "type": "UMLActivity", "visible": true },\n                { "name": "isLocallyReentrant",  "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "isSynchronous",       "kind": "prim", "type": "Boolean", "visible": true, "default": true },\n                { "name": "language",            "kind": "prim", "type": "String",  "visible": true },\n                { "name": "body",                "kind": "prim", "type": "String",  "visible": true, "multiline": true },\n                { "name": "localPreconditions",  "kind": "objs", "type": "UMLConstraint" },\n                { "name": "localPostconditions", "kind": "objs", "type": "UMLConstraint" }\n            ],\n            "view": "UMLActionView",\n            "ordering": 1411\n        },\n        "UMLStructuredActivityNode": {\n            "kind": "class",\n            "super": "UMLAction",\n            "attributes": [\n                { "name": "mustIsolate",  "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "nodes", "kind": "objs", "type": "UMLActivityNode" },\n                { "name": "edges", "kind": "objs", "type": "UMLActivityEdge" }\n            ],\n            "view": "UMLStructuredActivityNodeView",\n            "ordering": 1412\n        },\n        "UMLExpansionRegion": {\n            "kind": "class",\n            "super": "UMLStructuredActivityNode",\n            "attributes": [\n                { "name": "mode",  "kind": "enum", "type": "UMLExpansionKind", "visible": true, "default": "iterative" }\n            ],\n            "view": "UMLExpansionRegionView",\n            "ordering": 1412\n        },\n        "UMLObjectNode": {\n            "kind": "class",\n            "super": "UMLActivityNode",\n            "attributes": [\n                { "name": "type",          "kind": "var",  "type": "UMLClassifier", "visible": true },\n                { "name": "isControlType", "kind": "prim", "type": "Boolean", "visible": true, "default": false },\n                { "name": "ordering",      "kind": "enum", "type": "UMLObjectNodeOrderingKind", "visible": true }\n            ],\n            "view": "UMLObjectNodeView",\n            "ordering": 1412\n        },\n        "UMLCentralBufferNode": {\n            "kind": "class",\n            "super": "UMLObjectNode",\n            "view": "UMLCentralBufferNodeView",\n            "ordering": 1412\n        },\n        "UMLDataStoreNode": {\n            "kind": "class",\n            "super": "UMLCentralBufferNode",\n            "view": "UMLDataStoreNodeView",\n            "ordering": 1412\n        },\n        "UMLControlNode": {\n            "kind": "class",\n            "super": "UMLActivityNode",\n            "ordering": 1413\n        },\n        "UMLInitialNode": {\n            "kind": "class",\n            "super": "UMLControlNode",\n            "view": "UMLControlNodeView",\n            "ordering": 1414\n        },\n        "UMLFinalNode": {\n            "kind": "class",\n            "super": "UMLControlNode",\n            "view": "UMLControlNodeView",\n            "ordering": 1415\n        },\n        "UMLActivityFinalNode": {\n            "kind": "class",\n            "super": "UMLFinalNode",\n            "view": "UMLControlNodeView",\n            "ordering": 1416\n        },\n        "UMLFlowFinalNode": {\n            "kind": "class",\n            "super": "UMLFinalNode",\n            "view": "UMLControlNodeView",\n            "ordering": 1417\n        },\n        "UMLForkNode": {\n            "kind": "class",\n            "super": "UMLControlNode",\n            "view": "UMLControlNodeView",\n            "ordering": 1418\n        },\n        "UMLJoinNode": {\n            "kind": "class",\n            "super": "UMLControlNode",\n            "view": "UMLControlNodeView",\n            "ordering": 1419\n        },\n        "UMLMergeNode": {\n            "kind": "class",\n            "super": "UMLControlNode",\n            "view": "UMLControlNodeView",\n            "ordering": 1420\n        },\n        "UMLDecisionNode": {\n            "kind": "class",\n            "super": "UMLControlNode",\n            "view": "UMLControlNodeView",\n            "ordering": 1421\n        },\n        "UMLActivityGroup": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "subgroups", "kind": "objs", "type": "UMLActivityGroup" },\n                { "name": "nodes",     "kind": "objs", "type": "UMLActivityNode" },\n                { "name": "edges",     "kind": "objs", "type": "UMLActivityEdge" }\n            ],\n            "ordering": 1422\n        },\n        "UMLActivityPartition": {\n            "kind": "class",\n            "super": "UMLActivityGroup",\n            "view": "UMLSwimlaneView",\n            "ordering": 1423\n        },\n        "UMLInterruptibleActivityRegion": {\n            "kind": "class",\n            "super": "UMLActivityGroup",\n            "view": "UMLInterruptibleActivityRegionView",\n            "ordering": 1423\n        },\n        "UMLExceptionHandler": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "view": "UMLExceptionHandlerView",\n            "attributes": [\n                { "name": "exceptionTypes", "kind": "refs", "type": "UMLClassifier",   "visible": true },\n                { "name": "handlerBody",    "kind": "ref",  "type": "UMLActivityNode", "visible": true }\n            ],\n            "ordering": 1820\n        },\n        "UMLActivityEdge": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "attributes": [\n                { "name": "guard",  "kind": "prim", "type": "String", "visible": true },\n                { "name": "weight", "kind": "prim", "type": "String", "visible": true }\n            ]\n        },\n        "UMLControlFlow": {\n            "kind": "class",\n            "super": "UMLActivityEdge",\n            "view": "UMLControlFlowView",\n            "ordering": 1820\n        },\n        "UMLObjectFlow": {\n            "kind": "class",\n            "super": "UMLActivityEdge",\n            "view": "UMLObjectFlowView",\n            "ordering": 1821\n        },\n        "UMLActivityInterrupt": {\n            "kind": "class",\n            "super": "UMLActivityEdge",\n            "view": "UMLActivityInterruptView",\n            "ordering": 1821\n        },\n        "UMLInteractionFragment": {\n            "kind": "class",\n            "super": "UMLBehavior"\n        },\n        "UMLInteraction": {\n            "kind": "class",\n            "super": "UMLInteractionFragment",\n            "attributes": [\n                { "name": "messages",     "kind": "objs", "type": "UMLMessage" },\n                { "name": "participants", "kind": "objs", "type": "UMLMessageEndpoint" },\n                { "name": "fragments",    "kind": "objs", "type": "UMLInteractionFragment" }\n            ],\n            "ordering": 1403\n        },\n        "UMLStateInvariant": {\n            "kind": "class",\n            "super": "UMLInteractionFragment",\n            "attributes": [\n                { "name": "covered",   "kind": "ref",  "type": "UMLLifeline", "visible": true },\n                { "name": "invariant", "kind": "prim", "type": "String", "visible": true }\n            ],\n            "view": "UMLStateInvariantView",\n            "ordering": 1424\n        },\n        "UMLContinuation": {\n            "kind": "class",\n            "super": "UMLInteractionFragment",\n            "attributes": [\n                { "name": "setting", "kind": "prim", "type": "Boolean", "visible": true, "default": false }\n            ],\n            "view": "UMLContinuationView",\n            "ordering": 1425\n        },\n        "UMLInteractionOperand": {\n            "kind": "class",\n            "super": "UMLInteractionFragment",\n            "attributes": [\n                { "name": "guard", "kind": "prim", "type": "String", "visible": true }\n            ],\n            "ordering": 1427\n        },\n        "UMLCombinedFragment": {\n            "kind": "class",\n            "super": "UMLInteractionFragment",\n            "attributes": [\n                { "name": "interactionOperator",  "kind": "enum", "type": "UMLInteractionOperatorKind", "visible": true },\n                { "name": "operands",             "kind": "objs", "type": "UMLInteractionOperand" }\n            ],\n            "view": "UMLCombinedFragmentView",\n            "ordering": 1426\n        },\n        "UMLInteractionUse": {\n            "kind": "class",\n            "super": "UMLInteractionFragment",\n            "attributes": [\n                { "name": "refersTo",             "kind": "ref",  "type": "UMLInteraction", "visible": true },\n                { "name": "arguments",            "kind": "prim", "type": "String", "visible": true },\n                { "name": "returnValue",          "kind": "prim", "type": "String", "visible": true },\n                { "name": "returnValueRecipient", "kind": "ref",  "type": "UMLStructuralFeature", "visible": true }\n            ],\n            "view": "UMLInteractionUseView",\n            "ordering": 1406\n        },\n        "UMLMessageEndpoint": {\n            "kind": "class",\n            "super": "UMLModelElement"\n        },\n        "UMLLifeline": {\n            "kind": "class",\n            "super": "UMLMessageEndpoint",\n            "attributes": [\n                { "name": "selector",        "kind": "prim", "type": "String", "visible": true },\n                { "name": "represent",       "kind": "ref",  "type": "UMLStructuralFeature", "visible": true },\n                { "name": "isMultiInstance", "kind": "prim", "type": "Boolean", "visible": true }\n            ],\n            "view": "UMLCommLifelineView",\n            "ordering": 1428\n        },\n        "UMLGate": {\n            "kind": "class",\n            "super": "UMLMessageEndpoint",\n            "view": "UMLGateView",\n            "ordering": 1429\n        },\n        "UMLEndpoint": {\n            "kind": "class",\n            "super": "UMLMessageEndpoint",\n            "view": "UMLEndpointView",\n            "ordering": 1430\n        },\n        "UMLMessage": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "attributes": [\n                { "name": "messageSort",           "kind": "enum", "type": "UMLMessageSort",  "visible": true },\n                { "name": "signature",             "kind": "ref",  "type": "UMLModelElement", "visible": true },\n                { "name": "connector",             "kind": "ref",  "type": "UMLConnector",    "visible": true },\n                { "name": "arguments",             "kind": "prim", "type": "String",          "visible": true },\n                { "name": "assignmentTarget",      "kind": "prim", "type": "String",          "visible": true },\n                { "name": "guard",                 "kind": "prim", "type": "String",          "visible": true },\n                { "name": "iteration",             "kind": "prim", "type": "String",          "visible": true },\n                { "name": "isConcurrentIteration", "kind": "prim", "type": "Boolean",         "visible": true }\n            ],\n            "ordering": 1822\n        },\n        "UMLProfile": {\n            "kind": "class",\n            "super": "UMLPackage",\n            "view": "UMLProfileView",\n            "ordering": 1901\n        },\n        "UMLImage": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "attributes": [\n                { "name": "width",     "kind": "prim", "type": "Integer",  "visible": true },\n                { "name": "height",    "kind": "prim", "type": "Integer",  "visible": true },\n                { "name": "content",   "kind": "prim", "type": "String",   "visible": true, "multiline": true },\n                { "name": "smallIcon", "kind": "prim", "type": "String",   "visible": true, "multiline": true }\n            ],\n            "ordering": 1902\n        },\n        "UMLStereotype": {\n            "kind": "class",\n            "super": "UMLClass",\n            "attributes": [\n                { "name": "icon",    "kind": "obj",  "type": "UMLImage", "visible": true, "expand": true }\n            ],\n            "view": "UMLStereotypeView",\n            "ordering": 1903\n        },\n        "UMLMetaClass": {\n            "kind": "class",\n            "super": "UMLModelElement",\n            "view": "UMLMetaClassView",\n            "ordering": 1904\n        },\n        "UMLExtension": {\n            "kind": "class",\n            "super": "UMLDirectedRelationship",\n            "view": "UMLExtensionView",\n            "ordering": 1905\n        },\n        "UMLDiagram": {\n            "kind": "class",\n            "super": "Diagram",\n            "views": [\n                "UMLCustomTextView",\n                "ShapeView",\n                "UMLNoteLinkView",\n                "UMLConstraintLinkView",\n                "UMLFrameView"\n            ]\n        },\n        "UMLCompartmentView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "UMLNameCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView",\n            "attributes": [\n                { "name": "stereotypeLabel", "kind": "ref", "type": "LabelView", "embedded": "subViews" },\n                { "name": "nameLabel",       "kind": "ref", "type": "LabelView", "embedded": "subViews" },\n                { "name": "namespaceLabel",  "kind": "ref", "type": "LabelView", "embedded": "subViews" },\n                { "name": "propertyLabel",   "kind": "ref", "type": "LabelView", "embedded": "subViews" },\n                { "name": "wordWrap",        "kind": "prim", "type": "Boolean", "default": false, "transient": true }\n            ]\n        },\n        "UMLAttributeView": {\n            "kind": "class",\n            "super": "LabelView"\n        },\n        "UMLAttributeCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLOperationView": {\n            "kind": "class",\n            "super": "LabelView"\n        },\n        "UMLOperationCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLReceptionView": {\n            "kind": "class",\n            "super": "LabelView"\n        },\n        "UMLReceptionCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLTemplateParameterView": {\n            "kind": "class",\n            "super": "LabelView"\n        },\n        "UMLTemplateParameterCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLGeneralNodeView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "stereotypeDisplay", "kind": "enum", "type": "UMLStereotypeDisplayKind", "default": "label" },\n                { "name": "showVisibility",    "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "showNamespace",     "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "showProperty",      "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "showType",          "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "nameCompartment",   "kind": "ref",  "type": "UMLNameCompartmentView", "embedded": "subViews" },\n                { "name": "wordWrap",          "kind": "prim", "type": "Boolean", "default": false }\n            ]\n        },\n        "UMLFloatingNodeView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "nameLabel",       "kind": "ref", "type": "NodeLabelView", "embedded": "subViews" },\n                { "name": "stereotypeLabel", "kind": "ref", "type": "NodeLabelView", "embedded": "subViews" },\n                { "name": "propertyLabel",   "kind": "ref", "type": "NodeLabelView", "embedded": "subViews" },\n                { "name": "showProperty",    "kind": "prim", "type": "Boolean", "default": true }\n            ]\n        },\n        "UMLGeneralEdgeView": {\n            "kind": "class",\n            "super": "EdgeView",\n            "attributes": [\n                { "name": "stereotypeDisplay", "kind": "enum", "type": "UMLStereotypeDisplayKind", "default": "label" },\n                { "name": "showVisibility",    "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "showProperty",      "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "nameLabel",         "kind": "ref",  "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "stereotypeLabel",   "kind": "ref",  "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "propertyLabel",     "kind": "ref",  "type": "EdgeLabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLClassifierView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView",\n            "attributes": [\n                { "name": "suppressAttributes",           "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "suppressOperations",           "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "suppressReceptions",           "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "showMultiplicity",             "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "showOperationSignature",       "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "attributeCompartment",         "kind": "ref",  "type": "UMLAttributeCompartmentView", "embedded": "subViews" },\n                { "name": "operationCompartment",         "kind": "ref",  "type": "UMLOperationCompartmentView", "embedded": "subViews" },\n                { "name": "receptionCompartment",         "kind": "ref",  "type": "UMLReceptionCompartmentView", "embedded": "subViews" },\n                { "name": "templateParameterCompartment", "kind": "ref",  "type": "UMLTemplateParameterCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLUndirectedRelationshipView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView",\n            "attributes": [\n                { "name": "showMultiplicity",          "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "showType",                  "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "tailRoleNameLabel",         "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "tailPropertyLabel",         "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "tailMultiplicityLabel",     "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "headRoleNameLabel",         "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "headPropertyLabel",         "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "headMultiplicityLabel",     "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLClassDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "views": [\n                "UMLPackageView",\n                "UMLModelView",\n                "UMLSubsystemView",\n                "UMLContainmentView",\n                "UMLClassView",\n                "UMLInterfaceView",\n                "UMLSignalView",\n                "UMLDataTypeView",\n                "UMLPrimitiveTypeView",\n                "UMLEnumerationView",\n                "UMLDependencyView",\n                "UMLGeneralizationView",\n                "UMLRealizationView",\n                "UMLInterfaceRealizationView",\n                "UMLAssociationView",\n                "UMLAssociationClassLinkView",\n                "UMLPortView",\n                "UMLPartView",\n                "UMLConnectorView",\n                "UMLCollaborationView",\n                "UMLCollaborationUseView",\n                "UMLRoleBindingView",\n                "UMLObjectView",\n                "UMLLinkView",\n                "UMLArtifactView",\n                "UMLComponentView",\n                "UMLArtifactInstanceView",\n                "UMLComponentInstanceView",\n                "UMLComponentRealizationView",\n                "UMLNodeView",\n                "UMLNodeInstanceView",\n                "UMLDeploymentView",\n                "UMLCommunicationPathView",\n                "UMLUseCaseView",\n                "UMLActorView",\n                "UMLUseCaseSubjectView",\n                "UMLExtendView",\n                "UMLIncludeView",\n                "UMLProfileView",\n                "UMLExtensionView",\n                "UMLMetaClassView",\n                "UMLStereotypeView"\n            ]\n        },\n        "UMLClassView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLInterfaceView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLSignalView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLDataTypeView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLPrimitiveTypeView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLEnumerationLiteralView": {\n            "kind": "class",\n            "super": "LabelView",\n            "attributes": []\n        },\n        "UMLEnumerationLiteralCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLEnumerationView": {\n            "kind": "class",\n            "super": "UMLClassifierView",\n            "attributes": [\n                { "name": "suppressLiterals",              "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "enumerationLiteralCompartment", "kind": "ref",  "type": "UMLEnumerationLiteralCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLGeneralizationView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLDependencyView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLRealizationView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLInterfaceRealizationView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLQualifierCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView",\n            "attributes": []\n        },\n        "UMLAssociationView": {\n            "kind": "class",\n            "super": "UMLUndirectedRelationshipView",\n            "attributes": [\n                { "name": "tailQualifiersCompartment", "kind": "ref", "type": "UMLQualifierCompartmentView", "embedded": "subViews" },\n                { "name": "headQualifiersCompartment", "kind": "ref", "type": "UMLQualifierCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLAssociationClassLinkView": {\n            "kind": "class",\n            "super": "EdgeView"\n        },\n        "UMLPackageDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "views": [\n                "UMLPackageView",\n                "UMLModelView",\n                "UMLSubsystemView",\n                "UMLContainmentView",\n                "UMLClassView",\n                "UMLInterfaceView",\n                "UMLSignalView",\n                "UMLDataTypeView",\n                "UMLPrimitiveTypeView",\n                "UMLEnumerationView",\n                "UMLDependencyView",\n                "UMLGeneralizationView",\n                "UMLRealizationView",\n                "UMLInterfaceRealizationView",\n                "UMLAssociationView",\n                "UMLAssociationClassLinkView",\n                "UMLPortView",\n                "UMLPartView",\n                "UMLConnectorView",\n                "UMLCollaborationView",\n                "UMLCollaborationUseView",\n                "UMLRoleBindingView",\n                "UMLObjectView",\n                "UMLLinkView",\n                "UMLArtifactView",\n                "UMLComponentView",\n                "UMLArtifactInstanceView",\n                "UMLComponentInstanceView",\n                "UMLComponentRealizationView",\n                "UMLNodeView",\n                "UMLNodeInstanceView",\n                "UMLDeploymentView",\n                "UMLCommunicationPathView",\n                "UMLUseCaseView",\n                "UMLActorView",\n                "UMLUseCaseSubjectView",\n                "UMLExtendView",\n                "UMLIncludeView",\n                "UMLProfileView",\n                "UMLExtensionView",\n                "UMLMetaClassView",\n                "UMLStereotypeView"\n            ]\n        },\n        "UMLPackageView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLModelView": {\n            "kind": "class",\n            "super": "UMLPackageView"\n        },\n        "UMLSubsystemView": {\n            "kind": "class",\n            "super": "UMLPackageView"\n        },\n        "UMLContainmentView": {\n            "kind": "class",\n            "super": "EdgeView"\n        },\n        "UMLCompositeStructureDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "views": [\n                "UMLPackageView",\n                "UMLModelView",\n                "UMLSubsystemView",\n                "UMLContainmentView",\n                "UMLClassView",\n                "UMLInterfaceView",\n                "UMLSignalView",\n                "UMLDataTypeView",\n                "UMLPrimitiveTypeView",\n                "UMLEnumerationView",\n                "UMLDependencyView",\n                "UMLGeneralizationView",\n                "UMLRealizationView",\n                "UMLInterfaceRealizationView",\n                "UMLAssociationView",\n                "UMLAssociationClassLinkView",\n                "UMLPortView",\n                "UMLPartView",\n                "UMLConnectorView",\n                "UMLCollaborationView",\n                "UMLCollaborationUseView",\n                "UMLRoleBindingView",\n                "UMLObjectView",\n                "UMLLinkView",\n                "UMLArtifactView",\n                "UMLComponentView",\n                "UMLArtifactInstanceView",\n                "UMLComponentInstanceView",\n                "UMLComponentRealizationView",\n                "UMLNodeView",\n                "UMLNodeInstanceView",\n                "UMLDeploymentView",\n                "UMLCommunicationPathView",\n                "UMLUseCaseView",\n                "UMLActorView",\n                "UMLUseCaseSubjectView",\n                "UMLExtendView",\n                "UMLIncludeView",\n                "UMLProfileView",\n                "UMLExtensionView",\n                "UMLMetaClassView",\n                "UMLStereotypeView"\n            ]\n        },\n        "UMLPortView": {\n            "kind": "class",\n            "super": "UMLFloatingNodeView",\n            "attributes": [\n                { "name": "showVisibility",   "kind": "prim", "type": "Boolean", "default": false },\n                { "name": "showType",         "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "showMultiplicity", "kind": "prim", "type": "Boolean", "default": true }\n            ]\n        },\n        "UMLPartView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView",\n            "attributes": [\n                { "name": "showMultiplicity", "kind": "prim", "type": "Boolean", "default": true }\n            ]\n        },\n        "UMLConnectorView": {\n            "kind": "class",\n            "super": "UMLUndirectedRelationshipView"\n        },\n        "UMLCollaborationView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView",\n            "attributes": [\n                { "name": "templateParameterCompartment", "kind": "ref", "type": "UMLTemplateParameterCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLCollaborationUseView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLRoleBindingView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView",\n            "attributes": [\n                { "name": "roleNameLabel", "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLObjectDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "views": [\n                "UMLPackageView",\n                "UMLModelView",\n                "UMLSubsystemView",\n                "UMLContainmentView",\n                "UMLClassView",\n                "UMLInterfaceView",\n                "UMLSignalView",\n                "UMLDataTypeView",\n                "UMLPrimitiveTypeView",\n                "UMLEnumerationView",\n                "UMLDependencyView",\n                "UMLGeneralizationView",\n                "UMLRealizationView",\n                "UMLInterfaceRealizationView",\n                "UMLAssociationView",\n                "UMLAssociationClassLinkView",\n                "UMLPortView",\n                "UMLPartView",\n                "UMLConnectorView",\n                "UMLCollaborationView",\n                "UMLCollaborationUseView",\n                "UMLRoleBindingView",\n                "UMLObjectView",\n                "UMLLinkView",\n                "UMLArtifactView",\n                "UMLComponentView",\n                "UMLArtifactInstanceView",\n                "UMLComponentInstanceView",\n                "UMLComponentRealizationView",\n                "UMLNodeView",\n                "UMLNodeInstanceView",\n                "UMLDeploymentView",\n                "UMLCommunicationPathView",\n                "UMLUseCaseView",\n                "UMLActorView",\n                "UMLUseCaseSubjectView",\n                "UMLExtendView",\n                "UMLIncludeView",\n                "UMLProfileView",\n                "UMLExtensionView",\n                "UMLMetaClassView",\n                "UMLStereotypeView"\n            ]\n        },\n        "UMLSlotView": {\n            "kind": "class",\n            "super": "LabelView"\n        },\n        "UMLSlotCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLObjectView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView",\n            "attributes": [\n                { "name": "slotCompartment", "kind": "ref", "type": "UMLSlotCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLLinkView": {\n            "kind": "class",\n            "super": "UMLUndirectedRelationshipView"\n        },\n        "UMLComponentDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "views": [\n                "UMLPackageView",\n                "UMLModelView",\n                "UMLSubsystemView",\n                "UMLContainmentView",\n                "UMLClassView",\n                "UMLInterfaceView",\n                "UMLSignalView",\n                "UMLDataTypeView",\n                "UMLPrimitiveTypeView",\n                "UMLEnumerationView",\n                "UMLDependencyView",\n                "UMLGeneralizationView",\n                "UMLRealizationView",\n                "UMLInterfaceRealizationView",\n                "UMLAssociationView",\n                "UMLAssociationClassLinkView",\n                "UMLPortView",\n                "UMLPartView",\n                "UMLConnectorView",\n                "UMLCollaborationView",\n                "UMLCollaborationUseView",\n                "UMLRoleBindingView",\n                "UMLObjectView",\n                "UMLLinkView",\n                "UMLArtifactView",\n                "UMLComponentView",\n                "UMLArtifactInstanceView",\n                "UMLComponentInstanceView",\n                "UMLComponentRealizationView",\n                "UMLNodeView",\n                "UMLNodeInstanceView",\n                "UMLDeploymentView",\n                "UMLCommunicationPathView",\n                "UMLUseCaseView",\n                "UMLActorView",\n                "UMLUseCaseSubjectView",\n                "UMLExtendView",\n                "UMLIncludeView",\n                "UMLProfileView",\n                "UMLExtensionView",\n                "UMLMetaClassView",\n                "UMLStereotypeView"\n            ]\n        },\n        "UMLArtifactView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLArtifactInstanceView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLComponentView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLComponentInstanceView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLComponentRealizationView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLDeploymentDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "views": [\n                "UMLPackageView",\n                "UMLModelView",\n                "UMLSubsystemView",\n                "UMLContainmentView",\n                "UMLClassView",\n                "UMLInterfaceView",\n                "UMLSignalView",\n                "UMLDataTypeView",\n                "UMLPrimitiveTypeView",\n                "UMLEnumerationView",\n                "UMLDependencyView",\n                "UMLGeneralizationView",\n                "UMLRealizationView",\n                "UMLInterfaceRealizationView",\n                "UMLAssociationView",\n                "UMLAssociationClassLinkView",\n                "UMLPortView",\n                "UMLPartView",\n                "UMLConnectorView",\n                "UMLCollaborationView",\n                "UMLCollaborationUseView",\n                "UMLRoleBindingView",\n                "UMLObjectView",\n                "UMLLinkView",\n                "UMLArtifactView",\n                "UMLComponentView",\n                "UMLArtifactInstanceView",\n                "UMLComponentInstanceView",\n                "UMLComponentRealizationView",\n                "UMLNodeView",\n                "UMLNodeInstanceView",\n                "UMLDeploymentView",\n                "UMLCommunicationPathView",\n                "UMLUseCaseView",\n                "UMLActorView",\n                "UMLUseCaseSubjectView",\n                "UMLExtendView",\n                "UMLIncludeView",\n                "UMLProfileView",\n                "UMLExtensionView",\n                "UMLMetaClassView",\n                "UMLStereotypeView"\n            ]\n        },\n        "UMLNodeView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLNodeInstanceView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLDeploymentView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLCommunicationPathView": {\n            "kind": "class",\n            "super": "UMLAssociationView"\n        },\n        "UMLUseCaseDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "views": [\n                "UMLPackageView",\n                "UMLModelView",\n                "UMLSubsystemView",\n                "UMLContainmentView",\n                "UMLClassView",\n                "UMLInterfaceView",\n                "UMLSignalView",\n                "UMLDataTypeView",\n                "UMLPrimitiveTypeView",\n                "UMLEnumerationView",\n                "UMLDependencyView",\n                "UMLGeneralizationView",\n                "UMLRealizationView",\n                "UMLInterfaceRealizationView",\n                "UMLAssociationView",\n                "UMLAssociationClassLinkView",\n                "UMLPortView",\n                "UMLPartView",\n                "UMLConnectorView",\n                "UMLCollaborationView",\n                "UMLCollaborationUseView",\n                "UMLRoleBindingView",\n                "UMLObjectView",\n                "UMLLinkView",\n                "UMLArtifactView",\n                "UMLComponentView",\n                "UMLArtifactInstanceView",\n                "UMLComponentInstanceView",\n                "UMLComponentRealizationView",\n                "UMLNodeView",\n                "UMLNodeInstanceView",\n                "UMLDeploymentView",\n                "UMLCommunicationPathView",\n                "UMLUseCaseView",\n                "UMLActorView",\n                "UMLUseCaseSubjectView",\n                "UMLExtendView",\n                "UMLIncludeView",\n                "UMLProfileView",\n                "UMLExtensionView",\n                "UMLMetaClassView",\n                "UMLStereotypeView"\n            ]\n        },\n        "UMLExtensionPointView": {\n            "kind": "class",\n            "super": "LabelView",\n            "attributes": []\n        },\n        "UMLExtensionPointCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLUseCaseView": {\n            "kind": "class",\n            "super": "UMLClassifierView",\n            "attributes": [\n                { "name": "extensionPointCompartment", "kind": "ref",  "type": "UMLExtensionPointCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLActorView": {\n            "kind": "class",\n            "super": "UMLClassifierView"\n        },\n        "UMLIncludeView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLExtendView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLUseCaseSubjectView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLStatechartDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram"\n        },\n        "UMLPseudostateView": {\n            "kind": "class",\n            "super": "UMLFloatingNodeView"\n        },\n        "UMLFinalStateView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "UMLConnectionPointReferenceView": {\n            "kind": "class",\n            "super": "UMLFloatingNodeView"\n        },\n        "UMLInternalActivityView": {\n            "kind": "class",\n            "super": "LabelView"\n        },\n        "UMLInternalActivityCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLInternalTransitionView": {\n            "kind": "class",\n            "super": "LabelView"\n        },\n        "UMLInternalTransitionCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLRegionView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "UMLDecompositionCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLStateView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView",\n            "attributes": [\n                { "name": "internalActivityCompartment",   "kind": "ref",  "type": "UMLInternalActivityCompartmentView", "embedded": "subViews" },\n                { "name": "internalTransitionCompartment", "kind": "ref",  "type": "UMLInternalTransitionCompartmentView", "embedded": "subViews" },\n                { "name": "decompositionCompartment",      "kind": "ref",  "type": "UMLDecompositionCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLTransitionView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLActivityDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram"\n        },\n        "UMLPinView": {\n            "kind": "class",\n            "super": "UMLFloatingNodeView"\n        },\n        "UMLInputPinView": {\n            "kind": "class",\n            "super": "UMLPinView"\n        },\n        "UMLOutputPinView": {\n            "kind": "class",\n            "super": "UMLPinView"\n        },\n        "UMLExpansionNodeView": {\n            "kind": "class",\n            "super": "UMLPinView"\n        },\n        "UMLActionView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLStructuredActivityNodeView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLExpansionRegionView": {\n            "kind": "class",\n            "super": "UMLStructuredActivityNodeView"\n        },\n        "UMLObjectNodeView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLCentralBufferNodeView": {\n            "kind": "class",\n            "super": "UMLObjectNodeView"\n        },\n        "UMLDataStoreNodeView": {\n            "kind": "class",\n            "super": "UMLObjectNodeView"\n        },\n        "UMLControlNodeView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "UMLControlFlowView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLObjectFlowView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "UMLZigZagAdornmentView": {\n            "kind": "class",\n            "super": "EdgeNodeView"\n        },\n        "UMLExceptionHandlerView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView",\n            "attributes": [\n                { "name": "adornment",  "kind": "ref",  "type": "UMLZigZagAdornmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLActivityInterruptView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView",\n            "attributes": [\n                { "name": "adornment",  "kind": "ref",  "type": "UMLZigZagAdornmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLSwimlaneView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "nameLabel",  "kind": "ref",  "type": "LabelView", "embedded": "subViews" },\n                { "name": "isVertical", "kind": "prim", "type": "Boolean", "default": true }\n            ]\n        },\n        "UMLInterruptibleActivityRegionView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "UMLSequenceDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "attributes": [\n                { "name": "showSequenceNumber", "kind": "prim", "type": "Boolean", "visible": true, "default": true },\n                { "name": "showSignature",      "kind": "prim", "type": "Boolean", "visible": true, "default": true },\n                { "name": "showActivation",     "kind": "prim", "type": "Boolean", "visible": true, "default": true }\n            ]\n        },\n        "UMLLinePartView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "UMLSeqLifelineView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView",\n            "attributes": [\n                { "name": "linePart", "kind": "ref", "type": "UMLLinePartView", "embedded": "subViews" }\n            ]\n        },\n        "UMLMessageEndpointView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "UMLEndpointView": {\n            "kind": "class",\n            "super": "UMLMessageEndpointView"\n        },\n        "UMLGateView": {\n            "kind": "class",\n            "super": "UMLMessageEndpointView"\n        },\n        "UMLActivationView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "UMLSeqMessageView": {\n            "kind": "class",\n            "super": "EdgeView",\n            "attributes": [\n                { "name": "nameLabel",       "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "stereotypeLabel", "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "propertyLabel",   "kind": "ref", "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "activation",      "kind": "ref", "type": "UMLActivationView", "embedded": "subViews" },\n                { "name": "showProperty",    "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "showType",        "kind": "prim", "type": "Boolean", "default": true }\n            ]\n        },\n        "UMLStateInvariantView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "invariantLabel", "kind": "ref", "type": "LabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLContinuationView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "nameLabel", "kind": "ref", "type": "LabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLCustomFrameView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "nameLabel",      "kind": "ref", "type": "LabelView", "embedded": "subViews" },\n                { "name": "frameTypeLabel", "kind": "ref", "type": "LabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLFrameView": {\n            "kind": "class",\n            "super": "UMLCustomFrameView"\n        },\n        "UMLInteractionOperandView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "guardLabel", "kind": "ref", "type": "LabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLInteractionOperandCompartmentView": {\n            "kind": "class",\n            "super": "UMLCompartmentView"\n        },\n        "UMLCombinedFragmentView": {\n            "kind": "class",\n            "super": "UMLCustomFrameView",\n            "attributes": [\n                { "name": "operandCompartment", "kind": "ref", "type": "UMLInteractionOperandCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "UMLInteractionUseView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "nameLabel",      "kind": "ref", "type": "LabelView", "embedded": "subViews" },\n                { "name": "frameTypeLabel", "kind": "ref", "type": "LabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLCommunicationDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "attributes": [\n                { "name": "showSequenceNumber", "kind": "prim", "type": "Boolean", "visible": true, "default": true },\n                { "name": "showSignature",      "kind": "prim", "type": "Boolean", "visible": true, "default": true }\n            ]\n        },\n        "UMLCommLifelineView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLCommMessageView": {\n            "kind": "class",\n            "super": "EdgeNodeView",\n            "attributes": [\n                { "name": "nameLabel",       "kind": "ref", "type": "NodeLabelView", "embedded": "subViews" },\n                { "name": "stereotypeLabel", "kind": "ref", "type": "NodeLabelView", "embedded": "subViews" },\n                { "name": "propertyLabel",   "kind": "ref", "type": "NodeLabelView", "embedded": "subViews" },\n                { "name": "showProperty",    "kind": "prim", "type": "Boolean", "default": true },\n                { "name": "showType",        "kind": "prim", "type": "Boolean", "default": true }\n            ]\n        },\n        "UMLProfileDiagram": {\n            "kind": "class",\n            "super": "UMLDiagram",\n            "views": [\n                "UMLPackageView",\n                "UMLModelView",\n                "UMLSubsystemView",\n                "UMLContainmentView",\n                "UMLClassView",\n                "UMLInterfaceView",\n                "UMLSignalView",\n                "UMLDataTypeView",\n                "UMLPrimitiveTypeView",\n                "UMLEnumerationView",\n                "UMLDependencyView",\n                "UMLGeneralizationView",\n                "UMLRealizationView",\n                "UMLInterfaceRealizationView",\n                "UMLAssociationView",\n                "UMLAssociationClassLinkView",\n                "UMLPortView",\n                "UMLPartView",\n                "UMLConnectorView",\n                "UMLCollaborationView",\n                "UMLCollaborationUseView",\n                "UMLRoleBindingView",\n                "UMLObjectView",\n                "UMLLinkView",\n                "UMLArtifactView",\n                "UMLComponentView",\n                "UMLArtifactInstanceView",\n                "UMLComponentInstanceView",\n                "UMLComponentRealizationView",\n                "UMLNodeView",\n                "UMLNodeInstanceView",\n                "UMLDeploymentView",\n                "UMLCommunicationPathView",\n                "UMLUseCaseView",\n                "UMLActorView",\n                "UMLUseCaseSubjectView",\n                "UMLExtendView",\n                "UMLIncludeView",\n                "UMLProfileView",\n                "UMLExtensionView",\n                "UMLMetaClassView",\n                "UMLStereotypeView"\n            ]\n        },\n        "UMLProfileView": {\n            "kind": "class",\n            "super": "UMLPackageView"\n        },\n        "UMLMetaClassView": {\n            "kind": "class",\n            "super": "UMLGeneralNodeView"\n        },\n        "UMLStereotypeView": {\n            "kind": "class",\n            "super": "UMLClassView"\n        },\n        "UMLExtensionView": {\n            "kind": "class",\n            "super": "UMLGeneralEdgeView"\n        },\n        "HyperlinkView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "nameLabel", "kind": "ref", "type": "LabelView", "embedded": "subViews" },\n                { "name": "typeLabel", "kind": "ref", "type": "LabelView", "embedded": "subViews" }\n            ]\n        },\n        "UMLCustomTextView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "text",     "kind": "prim", "type": "String" },\n                { "name": "wordWrap", "kind": "prim", "type": "Boolean", "default": true }\n            ]\n        },\n        "UMLTextView": {\n            "kind": "class",\n            "super": "UMLCustomTextView"\n        },\n        "UMLCustomNoteView": {\n            "kind": "class",\n            "super": "UMLCustomTextView"\n        },\n        "UMLNoteView": {\n            "kind": "class",\n            "super": "UMLCustomNoteView"\n        },\n        "UMLConstraintView": {\n            "kind": "class",\n            "super": "UMLCustomNoteView"\n        },\n        "UMLNoteLinkView": {\n            "kind": "class",\n            "super": "EdgeView"\n        },\n        "UMLConstraintLinkView": {\n            "kind": "class",\n            "super": "EdgeView"\n        },\n        "ShapeView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "RectangleView": {\n            "kind": "class",\n            "super": "ShapeView"\n        },\n        "RoundRectView": {\n            "kind": "class",\n            "super": "ShapeView"\n        },\n        "EllipseView": {\n            "kind": "class",\n            "super": "ShapeView"\n        }\n    };\n\n    // Register Meta-model\n    MetaModelManager.register(metaModel);\n\n});\n\n//# sourceURL=/uml/UML-meta.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, browser: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, _, type, meta*/\n\ndefine(\'uml/UML-rules\',[\'require\',\'exports\',\'module\',\'core/Validator\'],function (require, exports, module) {\n    "use strict";\n\n    var Validator = require("core/Validator");\n\n    var rules = [\n        {\n            id: "UML001",\n            message: "Name expected.",\n            appliesTo: [ "UMLModelElement" ],\n            exceptions: [ "UMLDirectedRelationship", "UMLRelationshipEnd", "UMLUndirectedRelationship", "UMLAssociationClassLink", "UMLRegion", "UMLPseudostate", "UMLFinalState", "UMLControlNode", "UMLEndpoint", "UMLGate", "UMLImage", "UMLLifeline", "UMLInstance" ],\n            constraint: function (elem) {\n                if (elem instanceof type.UMLParameter) {\n                    return (elem.direction === "return" ? true : elem.name.length > 0);\n                } else {\n                    return (elem.name.length > 0);\n                }\n            }\n        },\n        {\n            id: "UML002",\n            message: "Name is already defined.",\n            appliesTo: [ "UMLModelElement" ],\n            exceptions: [ "UMLOperation" ],\n            constraint: function (elem) {\n                var parent   = elem._parent,\n                    field    = elem.getParentField(),\n                    siblings = parent[field],\n                    i,\n                    len;\n                for (i = 0, len = siblings.length; i < len; i++) {\n                    var sibling = siblings[i];\n                    if (elem !== sibling && elem.name.length > 0 && elem.name === sibling.name) {\n                        return false;\n                    }\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML003",\n            message: "Conflict with inherited attributes.",\n            appliesTo: [ "UMLAttribute" ],\n            constraint: function (elem) {\n                if (elem._parent instanceof type.UMLClassifier) {\n                    var inherited = elem._parent.getInheritedAttributes(),\n                        names     = _.map(inherited, function (e) { return e.name; });\n                    return !_.contains(names, elem.name);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML004",\n            message: "Signature conflict.",\n            appliesTo: [ "UMLOperation" ],\n            constraint: function (elem) {\n                var options = {\n                        stereotypeDisplay: "none",\n                        showVisibility: false,\n                        showProperty: false,\n                        showType: true,\n                        showOperationSignature: true\n                    },\n                    signature = elem.getString(options);\n                if (elem._parent.operations) {\n                    var i, len;\n                    for (i = 0, len = elem._parent.operations.length; i < len; i++) {\n                        var op  = elem._parent.operations[i],\n                            sig = op.getString(options);\n                        if (op !== elem && sig === signature) {\n                            return false;\n                        }\n                    }\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML006",\n            message: "Final and leaf element must not have child.",\n            appliesTo: [ "UMLClassifier" ],\n            constraint: function (elem) {\n                if (elem.isFinalSpecialization === true || elem.isLeaf === true) {\n                    var subs = elem.getSpecialElements();\n                    return (subs.length === 0);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML007",\n            message: "Duplicated generalizations.",\n            appliesTo: [ "UMLClassifier" ],\n            constraint: function (elem) {\n                var gens = elem.getGeneralElements();\n                return (gens.length === _.uniq(gens).length);\n            }\n        },\n        {\n            id: "UML008",\n            message: "Circular generalizations.",\n            appliesTo: [ "UMLClassifier" ],\n            constraint: function (elem) {\n                var ancestors = elem.getAncestors();\n                return (!_.contains(ancestors, elem));\n            }\n        },\n        {\n            id: "UML009",\n            message: "Duplicated realizations.",\n            appliesTo: [ "UMLClassifier" ],\n            constraint: function (elem) {\n                var reals = elem.getInterfaces();\n                return (reals.length === _.uniq(reals).length);\n            }\n        },\n        {\n            id: "UML010",\n            message: "Duplicated role names of associated classifiers.",\n            appliesTo: [ "UMLClassifier" ],\n            constraint: function (elem) {\n                var ends  = elem.getAssociationEnds(true), // get counterpart ends.\n                    names = [];\n                _.each(ends, function (e) {\n                    if (e.navigable && e.name) {\n                        names.push(e.name);\n                    }\n                });\n                _.map(ends, function (e) { return e.name; });\n                return (names.length === _.uniq(names).length);\n            }\n        },\n        {\n            id: "UML011",\n            message: "All attributes and operations of interface must be public.",\n            appliesTo: [ "UMLInterface" ],\n            constraint: function (elem) {\n                function bePublic(e) {\n                    return e.visibility === "public";\n                }\n                return _.every(elem.attributes, bePublic) && _.every(elem.operations, bePublic);\n            }\n        },\n        {\n            id: "UML012",\n            message: "Aggregation must be one in an association.",\n            appliesTo: [ "UMLAssociation" ],\n            constraint: function (elem) {\n                function isAggregation(e) {\n                    return e.aggregation === "shared" || e.aggregation === "composite";\n                }\n                return !(isAggregation(elem.end1) && isAggregation(elem.end2));\n            }\n        },\n        {\n            id: "UML013",\n            message: "Type of an artifact instance must be an artifact.",\n            appliesTo: [ "UMLArtifactInstance" ],\n            constraint: function (elem) {\n                return (!elem.classifier || elem.classifier instanceof type.UMLArtifact);\n            }\n        },\n        {\n            id: "UML014",\n            message: "Type of a component instance must be a component.",\n            appliesTo: [ "UMLComponentInstance" ],\n            constraint: function (elem) {\n                return (!elem.classifier || elem.classifier instanceof type.UMLComponent);\n            }\n        },\n        {\n            id: "UML015",\n            message: "Type of a node instance must be a node.",\n            appliesTo: [ "UMLNodeInstance" ],\n            constraint: function (elem) {\n                return (!elem.classifier || elem.classifier instanceof type.UMLNode);\n            }\n        },\n        {\n            id: "UML016",\n            message: "A metaclass must have name defined in metamodel.",\n            appliesTo: [ "UMLMetaClass" ],\n            constraint: function (elem) {\n                return (meta[elem.name]);\n            }\n        },\n        {\n            id: "UML017",\n            message: "A stereotype must inherits from stereotype.",\n            appliesTo: [ "UMLStereotype" ],\n            constraint: function (elem) {\n                var gens = elem.getGeneralElements();\n                return _.every(gens, function (g) { return (g instanceof type.UMLStereotype); });\n            }\n        },\n        {\n            id: "UML018",\n            message: "A stereotype must contained by a profile.",\n            appliesTo: [ "UMLStereotype" ],\n            constraint: function (elem) {\n                var found = false;\n                elem.traverseUp(function (e) {\n                    if (e instanceof type.UMLProfile) { found = true; }\n                });\n                return found;\n            }\n        },\n        {\n            id: "UML019",\n            message: "An actor only can associate with use cases, components, and classes.",\n            appliesTo: [ "UMLActor" ],\n            constraint: function (elem) {\n                var ends = elem.getAssociationEnds(true);\n                return _.every(ends, function (e) {\n                    return (e.reference instanceof type.UMLUseCase) ||\n                           (e.reference instanceof type.UMLComponent) ||\n                           (e.reference instanceof type.UMLClass);\n                });\n            }\n        },\n        {\n            id: "UML020",\n            message: "A use case cannot include use cases that directly or indirectly include it.",\n            appliesTo: [ "UMLUseCase" ],\n            constraint: function (elem) {\n                var includings = elem.getAllIncludedUseCases();\n                return (!_.contains(includings, elem));\n            }\n        },\n        {\n            id: "UML021",\n            message: "An initial vertex can have at most one outgoing transition.",\n            appliesTo: [ "UMLPseudostate" ],\n            constraint: function (elem) {\n                if (elem.kind === \'initial\') {\n                    return (elem.getOutgoingTransitions().length <= 1);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML022",\n            message: "The outgoing transition from an initial vertex must not have a trigger or guard.",\n            appliesTo: [ "UMLPseudostate" ],\n            constraint: function (elem) {\n                if (elem.kind === \'initial\') {\n                    var outs = elem.getOutgoingTransitions();\n                    return _.every(outs, function (t) {\n                        return (t.triggers.length === 0) && (!t.guard);\n                    });\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML023",\n            message: "History vertices can have at most one outgoing transition.",\n            appliesTo: [ "UMLPseudostate" ],\n            constraint: function (elem) {\n                if (elem.kind === \'deepHistory\' || elem.kind === \'shallowHistory\') {\n                    return (elem.getOutgoingTransitions().length <= 1);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML024",\n            message: "A join vertex must have at least two incoming transitions and exactly one outgoing transition.",\n            appliesTo: [ "UMLPseudostate" ],\n            constraint: function (elem) {\n                if (elem.kind === \'join\') {\n                    return (elem.getIncomingTransitions().length >= 2) &&\n                           (elem.getOutgoingTransitions().length === 1);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML025",\n            message: "A fork vertex must have at least two outgoing transitions and exactly one incoming transition.",\n            appliesTo: [ "UMLPseudostate" ],\n            constraint: function (elem) {\n                if (elem.kind === \'fork\') {\n                    return (elem.getOutgoingTransitions().length >= 2) &&\n                           (elem.getIncomingTransitions().length === 1);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML026",\n            message: "A junction vertex must have at least one incoming and one outgoing transition.",\n            appliesTo: [ "UMLPseudostate" ],\n            constraint: function (elem) {\n                if (elem.kind === \'junction\') {\n                    return (elem.getOutgoingTransitions().length >= 1) &&\n                           (elem.getIncomingTransitions().length >= 1);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML027",\n            message: "A choice vertex must have at least one incoming and one outgoing transition.",\n            appliesTo: [ "UMLPseudostate" ],\n            constraint: function (elem) {\n                if (elem.kind === \'choice\') {\n                    return (elem.getOutgoingTransitions().length >= 1) &&\n                           (elem.getIncomingTransitions().length === 1);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML028",\n            message: "Only submachine states can have connection point references.",\n            appliesTo: [ "UMLState" ],\n            constraint: function (elem) {\n                if (elem.connections.length > 0) {\n                    return (elem.submachine instanceof type.UMLStateMachine);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML029",\n            message: "A state is not allowed to have both a submachine and regions.",\n            appliesTo: [ "UMLState" ],\n            constraint: function (elem) {\n                return !(elem.regions.length > 0 && elem.submachine instanceof type.UMLStateMachine);\n            }\n        },\n        {\n            id: "UML030",\n            message: "A region can have at most one initial vertex.",\n            appliesTo: [ "UMLRegion" ],\n            constraint: function (elem) {\n                var vs = _.filter(elem.vertices, function (v) {\n                    return (v instanceof type.UMLPseudostate) && (v.kind === \'initial\');\n                });\n                return vs.length <= 1;\n            }\n        },\n        {\n            id: "UML031",\n            message: "A region can have at most one deep history vertex.",\n            appliesTo: [ "UMLRegion" ],\n            constraint: function (elem) {\n                var vs = _.filter(elem.vertices, function (v) {\n                    return (v instanceof type.UMLPseudostate) && (v.kind === \'deepHistory\');\n                });\n                return vs.length <= 1;\n            }\n        },\n        {\n            id: "UML032",\n            message: "A region can have at most one shallow history vertex.",\n            appliesTo: [ "UMLRegion" ],\n            constraint: function (elem) {\n                var vs = _.filter(elem.vertices, function (v) {\n                    return (v instanceof type.UMLPseudostate) && (v.kind === \'shallowHistory\');\n                });\n                return vs.length <= 1;\n            }\n        },\n        {\n            id: "UML033",\n            message: "A final state cannot have any outgoing transitions.",\n            appliesTo: [ "UMLFinalState" ],\n            constraint: function (elem) {\n                return (elem.getOutgoingTransitions().length === 0);\n            }\n        },\n        {\n            id: "UML034",\n            message: "A final state cannot have regions.",\n            appliesTo: [ "UMLFinalState" ],\n            constraint: function (elem) {\n                return (elem.regions.length === 0);\n            }\n        },\n        {\n            id: "UML035",\n            message: "A final state cannot reference a submachine.",\n            appliesTo: [ "UMLFinalState" ],\n            constraint: function (elem) {\n                return (!elem.submachine);\n            }\n        },\n        {\n            id: "UML036",\n            message: "A final state has no entry behavior.",\n            appliesTo: [ "UMLFinalState" ],\n            constraint: function (elem) {\n                return (elem.entryActivities.length === 0);\n            }\n        },\n        {\n            id: "UML037",\n            message: "A final state has no exit behavior.",\n            appliesTo: [ "UMLFinalState" ],\n            constraint: function (elem) {\n                return (elem.exitActivities.length === 0);\n            }\n        },\n        {\n            id: "UML038",\n            message: "A final state has no state (doActivity) behavior.",\n            appliesTo: [ "UMLFinalState" ],\n            constraint: function (elem) {\n                return (elem.doActivities.length === 0);\n            }\n        },\n        {\n            id: "UML039",\n            message: "A fork segment must not have guards or triggers.",\n            appliesTo: [ "UMLTransition" ],\n            constraint: function (elem) {\n                if (elem.source instanceof type.UMLPseudostate && elem.source.kind === \'fork\') {\n                    return (elem.triggers.length === 0) && (!elem.guard);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML040",\n            message: "A join segment must not have guards or triggers.",\n            appliesTo: [ "UMLTransition" ],\n            constraint: function (elem) {\n                if (elem.target instanceof type.UMLPseudostate && elem.target.kind === \'join\') {\n                    return (elem.triggers.length === 0) && (!elem.guard);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML041",\n            message: "A fork segment must always target a state.",\n            appliesTo: [ "UMLTransition" ],\n            constraint: function (elem) {\n                if (elem.source instanceof type.UMLPseudostate && elem.source.kind === \'fork\') {\n                    return (elem.target instanceof type.UMLState);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML042",\n            message: "A join segment must always originate from a state.",\n            appliesTo: [ "UMLTransition" ],\n            constraint: function (elem) {\n                if (elem.target instanceof type.UMLPseudostate && elem.target.kind === \'join\') {\n                    return (elem.source instanceof type.UMLState);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML043",\n            message: "Transitions outgoing pseudostates may not have a trigger (except for those coming out of the initial pseudostate).",\n            appliesTo: [ "UMLTransition" ],\n            constraint: function (elem) {\n                if (elem.source instanceof type.UMLPseudostate && elem.source.kind !== \'initial\') {\n                    return (elem.triggers.length === 0);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML044",\n            message: "The classifier context of a state machine cannot be an interface.",\n            appliesTo: [ "UMLStateMachine" ],\n            constraint: function (elem) {\n                return !(elem._parent instanceof type.UMLInterface);\n            }\n        },\n        {\n            id: "UML045",\n            message: "A decision node has one or two incoming edges and at least one outgoing edge.",\n            appliesTo: [ "UMLDecisionNode" ],\n            constraint: function (elem) {\n                return (elem.getIncomingEdges().length === 1 || elem.getIncomingEdges().length === 2) &&\n                       (elem.getOutgoingEdges().length >= 1);\n            }\n        },\n        {\n            id: "UML046",\n            message: "A merge node has one outgoing edge.",\n            appliesTo: [ "UMLMergeNode" ],\n            constraint: function (elem) {\n                return (elem.getOutgoingEdges().length === 1);\n            }\n        },\n        {\n            id: "UML047",\n            message: "The edges coming into and out of a merge node must be either all object flows or all control flows.",\n            appliesTo: [ "UMLMergeNode" ],\n            constraint: function (elem) {\n                var edges = _.union(elem.getIncomingEdges(), elem.getOutgoingEdges());\n                return _.every(edges, function (e) { return (e instanceof type.UMLControlFlow); }) ||\n                       _.every(edges, function (e) { return (e instanceof type.UMLObjectFlow); });\n            }\n        },\n        {\n            id: "UML048",\n            message: "An initial node has no incoming edges.",\n            appliesTo: [ "UMLInitialNode" ],\n            constraint: function (elem) {\n                return (elem.getIncomingEdges().length === 0);\n            }\n        },\n        {\n            id: "UML049",\n            message: "Only control edges can have initial nodes as source.",\n            appliesTo: [ "UMLInitialNode" ],\n            constraint: function (elem) {\n                return _.every(elem.getOutgoingEdges(), function (e) { return (e instanceof type.UMLControlFlow); });\n            }\n        },\n        {\n            id: "UML050",\n            message: "A final node has no outgoing edges.",\n            appliesTo: [ "UMLFinalNode" ],\n            constraint: function (elem) {\n                return (elem.getOutgoingEdges().length === 0);\n            }\n        },\n        {\n            id: "UML051",\n            message: "A fork node has one incoming edge.",\n            appliesTo: [ "UMLForkNode" ],\n            constraint: function (elem) {\n                return (elem.getIncomingEdges().length === 1);\n            }\n        },\n        {\n            id: "UML052",\n            message: "The edges coming into and out of a fork node must be either all object flows or all control flows.",\n            appliesTo: [ "UMLForkNode" ],\n            constraint: function (elem) {\n                var edges = _.union(elem.getIncomingEdges(), elem.getOutgoingEdges());\n                return _.every(edges, function (e) { return (e instanceof type.UMLControlFlow); }) ||\n                       _.every(edges, function (e) { return (e instanceof type.UMLObjectFlow); });\n            }\n        },\n        {\n            id: "UML053",\n            message: "A join node has one outgoing edge.",\n            appliesTo: [ "UMLJoinNode" ],\n            constraint: function (elem) {\n                return (elem.getOutgoingEdges().length === 1);\n            }\n        },\n        {\n            id: "UML054",\n            message: "The edges coming into and out of a join node must be either all object flows or all control flows.",\n            appliesTo: [ "UMLJoinNode" ],\n            constraint: function (elem) {\n                var edges = _.union(elem.getIncomingEdges(), elem.getOutgoingEdges());\n                return _.every(edges, function (e) { return (e instanceof type.UMLControlFlow); }) ||\n                       _.every(edges, function (e) { return (e instanceof type.UMLObjectFlow); });\n            }\n        },\n        {\n            id: "UML055",\n            message: "All edges coming into or going out of object nodes must be object flow edges.",\n            appliesTo: [ "UMLObjectNode" ],\n            constraint: function (elem) {\n                var edges = _.union(elem.getIncomingEdges(), elem.getOutgoingEdges());\n                return _.every(edges, function (e) { return (e instanceof type.UMLObjectFlow); });\n            }\n        },\n        {\n            id: "UML056",\n            message: "Control flows may not have object nodes at either end, except for object nodes with control type.",\n            appliesTo: [ "UMLControlFlow" ],\n            constraint: function (elem) {\n                if (elem.source instanceof type.UMLObjectNode && elem.target instanceof type.UMLObjectNode) {\n                    return (elem.source.isControlType || elem.target.isControlType);\n                }\n                return true;\n            }\n        },\n        {\n            id: "UML057",\n            message: "Object flows may not have actions at either end.",\n            appliesTo: [ "UMLObjectFlow" ],\n            constraint: function (elem) {\n                return !(elem.source instanceof type.UMLAction && elem.target instanceof type.UMLAction);\n            }\n        }\n    ];\n\n    Validator.addRules(rules);\n\n});\n\n//# sourceURL=/uml/UML-rules.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell */\ndefine(\'uml/UMLCommandHandlers\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/AppInit\',\'utils/Helper\',\'core/Core\',\'core/Repository\',\'engine/ProjectManager\',\'engine/Engine\',\'engine/Factory\',\'engine/SelectionManager\',\'command/CommandManager\',\'command/Commands\',\'uml/UMLCommands\',\'uml/UML\',\'uml/UMLDiagram\',\'uml/UMLFactory\',\'filesystem/FileSystem\',\'filesystem/FileSystemError\',\'file/FileUtils\',\'dialogs/Dialogs\',\'ui/Toast\',\'diagrams/DiagramManager\',\'explorer/ModelExplorerView\',\'strings\'],function (require, exports, module) {\n    "use strict";\n\n    var global            = require("core/Global").global,\n        AppInit           = require("utils/AppInit"),\n        Helper            = require("utils/Helper"),\n        Core              = require("core/Core"),\n        Repository        = require("core/Repository"),\n        ProjectManager    = require("engine/ProjectManager"),\n        Engine            = require("engine/Engine"),\n        Factory           = require("engine/Factory"),\n        SelectionManager  = require("engine/SelectionManager"),\n        CommandManager    = require("command/CommandManager"),\n        Commands          = require("command/Commands"),\n        UMLCommands       = require("uml/UMLCommands"),\n        UML               = require("uml/UML"),\n        UMLDiagram        = require("uml/UMLDiagram"),\n        UMLFactory        = require("uml/UMLFactory"),\n        FileSystem        = require("filesystem/FileSystem"),\n        FileSystemError   = require("filesystem/FileSystemError"),\n        FileUtils         = require("file/FileUtils"),\n        Dialogs           = require("dialogs/Dialogs"),\n        Toast             = require("ui/Toast"),\n        DiagramManager    = require("diagrams/DiagramManager"),\n        ModelExplorerView = require("explorer/ModelExplorerView"),\n        Strings           = require("strings");\n\n    function _openDiagram(id) {\n        var diagram = Repository.get(id);\n        if (diagram instanceof type.Diagram) {\n            DiagramManager.setCurrentDiagram(diagram);\n            ModelExplorerView.select(diagram);\n        }\n    }\n\n    /*\n     * New From Template Command Handlers\n     */\n\n    function _handleUMLMinimalTemplate() {\n        var path = FileUtils.getApplicationDirectoryPath() + "templates/UMLMinimal.mdj";\n        return CommandManager.execute(Commands.FILE_NEW, path);\n    }\n\n    function _handleUMLConventionalTemplate() {\n        var path = FileUtils.getApplicationDirectoryPath() + "templates/UMLConventional.mdj";\n        return CommandManager.execute(Commands.FILE_NEW, path);\n    }\n\n    function _handle4p1ViewModelTemplate() {\n        var path = FileUtils.getApplicationDirectoryPath() + "templates/4p1ViewModel.mdj";\n        return CommandManager.execute(Commands.FILE_NEW, path);\n    }\n\n    function _handleRationalTemplate() {\n        var path = FileUtils.getApplicationDirectoryPath() + "templates/Rational.mdj";\n        return CommandManager.execute(Commands.FILE_NEW, path);\n    }\n\n    /*\n     * Model Apply Command Handlers\n     */\n\n    /**\n     * @param {string} fullPath - Fullpath for profile to be loaded.\n     */\n    function _handleApplyProfile(fullPath) {\n        return ProjectManager.importFromFile(ProjectManager.getProject(), fullPath);\n    }\n\n    function _handleApplyUMLStandardProfile() {\n        var filename = FileUtils.getApplicationDirectoryPath() + "profiles/UMLStandardProfile.mfj";\n        return _handleApplyProfile(filename).done(function () {\n            Toast.info("UMLStandardProfile is successfully applied. Look at under Project.");\n        });\n    }\n\n    function _handleEntryActivity() {\n        Dialogs.showSelectRadioDialog(\n            "Select one to create as an entry Activity",\n            [\n                { text: "OpaqueBehavior", value: "UMLOpaqueBehavior", checked: true},\n                { text: "Activity",       value: "UMLActivity"},\n                { text: "StateMachine",   value: "UMLStateMachine"},\n                { text: "Interaction",    value: "UMLInteraction"}\n            ]\n        ).done(function (buttonId, selected) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                Factory.createModel(selected, null, "entryActivities");\n            }\n        });\n    }\n\n    function _handleDoActivity() {\n        Dialogs.showSelectRadioDialog(\n            "Select one to create as an do Activity",\n            [\n                { text: "OpaqueBehavior", value: "UMLOpaqueBehavior", checked: true},\n                { text: "Activity",       value: "UMLActivity"},\n                { text: "StateMachine",   value: "UMLStateMachine"},\n                { text: "Interaction",    value: "UMLInteraction"}\n            ]\n        ).done(function (buttonId, selected) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                Factory.createModel(selected, null, "doActivities");\n            }\n        });\n    }\n\n    function _handleExitActivity() {\n        Dialogs.showSelectRadioDialog(\n            "Select one to create as an exit Activity",\n            [\n                { text: "OpaqueBehavior", value: "UMLOpaqueBehavior", checked: true},\n                { text: "Activity",       value: "UMLActivity"},\n                { text: "StateMachine",   value: "UMLStateMachine"},\n                { text: "Interaction",    value: "UMLInteraction"}\n            ]\n        ).done(function (buttonId, selected) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                Factory.createModel(selected, null, "exitActivities");\n            }\n        });\n    }\n\n    function _handleEffect() {\n        Dialogs.showSelectRadioDialog(\n            "Select one to create as an effect Behavior",\n            [\n                { text: "OpaqueBehavior", value: "UMLOpaqueBehavior", checked: true},\n                { text: "Activity",       value: "UMLActivity"},\n                { text: "StateMachine",   value: "UMLStateMachine"},\n                { text: "Interaction",    value: "UMLInteraction"}\n            ]\n        ).done(function (buttonId, selected) {\n            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                Factory.createModel(selected, null, "effects");\n            }\n        });\n    }\n\n    function _handleConstraint() {\n        var parent = SelectionManager.getSelected();\n        if (parent instanceof type.UMLOperation) {\n            Dialogs.showSelectRadioDialog(\n                "Select kind of Constraint of Operation",\n                [\n                    { text: "Typical Constraint", value: "ownedElements", checked: true},\n                    { text: "Precondition",       value: "preconditions"},\n                    { text: "BodyCondition",      value: "bodyConditions"},\n                    { text: "Postcondition",      value: "postconditions"}\n                ]\n            ).done(function (buttonId, selected) {\n                if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                    Factory.createModel("UMLConstraint", parent, selected);\n                }\n            });\n        } else if (parent instanceof type.UMLBehavior) {\n            Dialogs.showSelectRadioDialog(\n                "Select kind of Constraint of Behavior",\n                [\n                    { text: "Typical Constraint", value: "ownedElements", checked: true},\n                    { text: "Precondition",       value: "preconditions"},\n                    { text: "Postcondition",      value: "postconditions"}\n                ]\n            ).done(function (buttonId, selected) {\n                if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                    Factory.createModel("UMLConstraint", parent, selected);\n                }\n            });\n        } else if (parent instanceof type.UMLAction) {\n            Dialogs.showSelectRadioDialog(\n                "Select kind of Constraint of Action",\n                [\n                    { text: "Typical Constraint", value: "ownedElements", checked: true},\n                    { text: "LocalPrecondition",  value: "localPreconditions"},\n                    { text: "LocalPostcondition", value: "localPostconditions"}\n                ]\n            ).done(function (buttonId, selected) {\n                if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                    Factory.createModel("UMLConstraint", parent, selected);\n                }\n            });\n        } else {\n            Factory.createModel("UMLConstraint", parent, "ownedElements");\n        }\n    }\n\n\n    function _handleStereotypeDisplay(value) {\n        var views = SelectionManager.getSelectedViews();\n        Engine.setElemsProperty(views, \'stereotypeDisplay\', value);\n    }\n\n    function _handleToggleProperty(field) {\n        var views = SelectionManager.getSelectedViews(),\n            boolVal = Helper.reduceValue(views, field);\n        if (_.isBoolean(boolVal)) {\n            Engine.setElemsProperty(views, field, !boolVal);\n        } else {\n            Engine.setElemsProperty(views, field, true);\n        }\n    }\n\n    function _updateCommands() {\n        var models = SelectionManager.getSelectedModels(),\n            views = SelectionManager.getSelectedViews(),\n            selected = SelectionManager.getSelected();\n\n        var isNone = (!selected),\n            isProject = selected instanceof Core.Project,\n            isExtensibleModel = selected instanceof type.ExtensibleModel,\n            isModelElement = selected instanceof type.UMLModelElement,\n            isPackage = selected instanceof type.UMLPackage || selected instanceof type.UMLSubsystem,\n            isProfile = selected instanceof type.UMLProfile,\n            isClassifier = selected instanceof type.UMLClassifier,\n            isEnumeration = selected instanceof type.UMLEnumeration,\n            isUseCase = selected instanceof type.UMLUseCase,\n            isAssociation = selected instanceof type.UMLAssociation,\n            isOperation = selected instanceof type.UMLOperation,\n            isInstance = selected instanceof type.UMLInstance,\n            isInteraction = selected instanceof type.UMLInteraction,\n            isStateMachine = selected instanceof type.UMLStateMachine,\n            isRegion = selected instanceof type.UMLRegion,\n            isState = selected instanceof type.UMLState,\n            isTransition = selected instanceof type.UMLTransition,\n            isActivity = selected instanceof type.UMLActivity,\n            isAction = selected instanceof type.UMLAction;\n\n        // Model: Packages\n        CommandManager.get(UMLCommands.MODEL_ADD_MODEL).setVisible(isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_SUBSYSTEM).setVisible(isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_PACKAGE).setVisible(isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_PROFILE).setVisible(isProject || isPackage);\n\n        // Model: Classes\n        CommandManager.get(UMLCommands.MODEL_ADD_CLASS).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_INTERFACE).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_SIGNAL).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_DATATYPE).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_PRIMITIVETYPE).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_ENUMERATION).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_ARTIFACT).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_COMPONENT).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_NODE).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_USECASE).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_ACTOR).setVisible(isProject || isPackage || isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_STEREOTYPE).setVisible(isProfile);\n\n        // Model: Instances\n        // TODO: Menus for instace types never be enabled. (Delete them or enable them when an element is selected)\n        CommandManager.get(UMLCommands.MODEL_ADD_OBJECT).setVisible(false);\n        CommandManager.get(UMLCommands.MODEL_ADD_ARTIFACTINSTANCE).setVisible(false);\n        CommandManager.get(UMLCommands.MODEL_ADD_COMPONENTINSTANCE).setVisible(false);\n        CommandManager.get(UMLCommands.MODEL_ADD_NODEINSTANCE).setVisible(false);\n\n        // Model: Behaviors\n        CommandManager.get(UMLCommands.MODEL_ADD_COLLABORATION).setVisible(isClassifier || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_INTERACTION).setVisible(isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_STATEMACHINE).setVisible(isClassifier || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_ACTIVITY).setVisible(isClassifier || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_OPAQUEBEHAVIOR).setVisible(isClassifier);\n\n        // Model: Features\n        CommandManager.get(UMLCommands.MODEL_ADD_TEMPLATEPARAMETER).setVisible(isClassifier || isOperation);\n        CommandManager.get(UMLCommands.MODEL_ADD_PARAMETER).setVisible(isOperation);\n        CommandManager.get(UMLCommands.MODEL_ADD_ENUMERATIONLITERAL).setVisible(isEnumeration);\n        CommandManager.get(UMLCommands.MODEL_ADD_ATTRIBUTE).setVisible(isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_PORT).setVisible(isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_OPERATION).setVisible(isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_RECEPTION).setVisible(isClassifier);\n        CommandManager.get(UMLCommands.MODEL_ADD_EXTENSIONPOINT).setVisible(isUseCase);\n        CommandManager.get(UMLCommands.MODEL_ADD_SLOT).setVisible(isInstance);\n\n        // Model: States\n        CommandManager.get(UMLCommands.MODEL_ADD_STATE).setVisible(isRegion);\n        CommandManager.get(UMLCommands.MODEL_ADD_REGION).setVisible(isState);\n        CommandManager.get(UMLCommands.MODEL_ADD_ENTRY_ACTIVITY).setVisible(isState);\n        CommandManager.get(UMLCommands.MODEL_ADD_DO_ACTIVITY).setVisible(isState);\n        CommandManager.get(UMLCommands.MODEL_ADD_EXIT_ACTIVITY).setVisible(isState);\n        CommandManager.get(UMLCommands.MODEL_ADD_TRIGGER).setVisible(isAction || isTransition);\n        CommandManager.get(UMLCommands.MODEL_ADD_EFFECT).setVisible(isTransition);\n\n        // Model: Actions\n        CommandManager.get(UMLCommands.MODEL_ADD_ACTION).setVisible(isActivity);\n\n        // Model: Common\n        CommandManager.get(UMLCommands.MODEL_ADD_CONSTRAINT).setVisible(isModelElement);\n\n        // Model: Diagrams\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_USECASE).setVisible(isNone || isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_CLASS).setVisible(isNone || isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_PACKAGE).setVisible(isNone || isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_OBJECT).setVisible(isNone || isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_COMPONENT).setVisible(isNone || isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_DEPLOYMENT).setVisible(isNone || isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_PROFILE).setVisible(isNone || isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_COMPOSITESTRUCTURE).setVisible(isNone || isProject || isPackage);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_SEQUENCE).setVisible(isNone || isProject || isPackage || isClassifier || isOperation || isInteraction);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_COMMUNICATION).setVisible(isNone || isProject || isPackage || isClassifier || isOperation || isInteraction);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_STATECHART).setVisible(isNone || isProject || isPackage || isClassifier || isOperation || isStateMachine);\n        CommandManager.get(UMLCommands.MODEL_ADD_DIAGRAM_ACTIVITY).setVisible(isNone || isProject || isPackage || isClassifier || isOperation || isActivity);\n\n        // Update Format > StereotypeDisplay\n        CommandManager.get(UMLCommands.FORMAT_STEREOTYPE).setEnabled(views.length > 0);\n        if (views.length > 0) {\n            var stereotypeDisplay = Helper.reduceValue(views, \'stereotypeDisplay\');\n            CommandManager.get(UMLCommands.FORMAT_STEREOTYPE_NONE).setChecked(stereotypeDisplay === UML.SD_NONE);\n            CommandManager.get(UMLCommands.FORMAT_STEREOTYPE_LABEL).setChecked(stereotypeDisplay === UML.SD_LABEL);\n            CommandManager.get(UMLCommands.FORMAT_STEREOTYPE_DECORATION).setChecked(stereotypeDisplay === UML.SD_DECORATION);\n            CommandManager.get(UMLCommands.FORMAT_STEREOTYPE_DECORATION_LABEL).setChecked(stereotypeDisplay === UML.SD_DECORATION_LABEL);\n            CommandManager.get(UMLCommands.FORMAT_STEREOTYPE_ICON).setChecked(stereotypeDisplay === UML.SD_ICON);\n            CommandManager.get(UMLCommands.FORMAT_STEREOTYPE_ICON_LABEL).setChecked(stereotypeDisplay === UML.SD_ICON_LABEL);\n        }\n\n        // Update Format > WordWrap\n        CommandManager.get(UMLCommands.FORMAT_WORD_WRAP).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_WORD_WRAP).setChecked(Helper.reduceValue(views, "wordWrap"));\n\n        // Update Format > ShowVisibility\n        CommandManager.get(UMLCommands.FORMAT_SHOW_VISIBILITY).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SHOW_VISIBILITY).setChecked(Helper.reduceValue(views, "showVisibility"));\n\n        // Update Format > ShowNamespace\n        CommandManager.get(UMLCommands.FORMAT_SHOW_NAMESPACE).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SHOW_NAMESPACE).setChecked(Helper.reduceValue(views, "showNamespace"));\n\n        // Update Format > ShowProperty\n        CommandManager.get(UMLCommands.FORMAT_SHOW_PROPERTY).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SHOW_PROPERTY).setChecked(Helper.reduceValue(views, "showProperty"));\n\n        // Update Format > ShowType\n        CommandManager.get(UMLCommands.FORMAT_SHOW_TYPE).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SHOW_TYPE).setChecked(Helper.reduceValue(views, "showType"));\n\n        // Update Format > ShowMultiplicity\n        CommandManager.get(UMLCommands.FORMAT_SHOW_MULTIPLICITY).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SHOW_MULTIPLICITY).setChecked(Helper.reduceValue(views, "showMultiplicity"));\n\n        // Update Format > ShowOperationSignature\n        CommandManager.get(UMLCommands.FORMAT_SHOW_OPERATION_SIGNATURE).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SHOW_OPERATION_SIGNATURE).setChecked(Helper.reduceValue(views, "showOperationSignature"));\n\n        // Update Format > SuppressAttributes\n        CommandManager.get(UMLCommands.FORMAT_SUPPRESS_ATTRIBUTES).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SUPPRESS_ATTRIBUTES).setChecked(Helper.reduceValue(views, "suppressAttributes"));\n\n        // Update Format > SuppressOperations\n        CommandManager.get(UMLCommands.FORMAT_SUPPRESS_OPERATIONS).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SUPPRESS_OPERATIONS).setChecked(Helper.reduceValue(views, "suppressOperations"));\n\n        // Update Format > SuppressReceptions\n        CommandManager.get(UMLCommands.FORMAT_SUPPRESS_RECEPTIONS).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SUPPRESS_RECEPTIONS).setChecked(Helper.reduceValue(views, "suppressReceptions"));\n\n        // Update Format > SuppressLiterals\n        CommandManager.get(UMLCommands.FORMAT_SUPPRESS_LITERALS).setEnabled(views.length > 0);\n        CommandManager.get(UMLCommands.FORMAT_SUPPRESS_LITERALS).setChecked(Helper.reduceValue(views, "suppressLiterals"));\n    }\n\n    // Register Commands\n    CommandManager.register(Strings.CMD_FILE_NEW_FROM_TEMPLATE_UML_MINIMAL,      UMLCommands.FILE_NEW_FROM_TEMPLATE_UML_MINIMAL,      _handleUMLMinimalTemplate);\n    CommandManager.register(Strings.CMD_FILE_NEW_FROM_TEMPLATE_UML_CONVENTIONAL, UMLCommands.FILE_NEW_FROM_TEMPLATE_UML_CONVENTIONAL, _handleUMLConventionalTemplate);\n    CommandManager.register(Strings.CMD_FILE_NEW_FROM_TEMPLATE_4P1VIEWMODEL,     UMLCommands.FILE_NEW_FROM_TEMPLATE_4P1VIEWMODEL,     _handle4p1ViewModelTemplate);\n    CommandManager.register(Strings.CMD_FILE_NEW_FROM_TEMPLATE_RATIONAL,         UMLCommands.FILE_NEW_FROM_TEMPLATE_RATIONAL,         _handleRationalTemplate);\n    // Model: Profiles\n    CommandManager.register(Strings.CMD_MODEL_APPLY_PROFILE,              UMLCommands.MODEL_APPLY_PROFILE,              _handleApplyProfile);\n    CommandManager.register(Strings.CMD_MODEL_APPLY_PROFILE_UML_STANDARD, UMLCommands.MODEL_APPLY_PROFILE_UML_STANDARD, _handleApplyUMLStandardProfile);\n    // Model: Packages\n    CommandManager.register(Strings.CMD_MODEL_ADD_MODEL,     UMLCommands.MODEL_ADD_MODEL,     _.partial(Factory.createModel, "UMLModel"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_SUBSYSTEM, UMLCommands.MODEL_ADD_SUBSYSTEM, _.partial(Factory.createModel, "UMLSubsystem"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_PACKAGE,   UMLCommands.MODEL_ADD_PACKAGE,   _.partial(Factory.createModel, "UMLPackage"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_PROFILE,   UMLCommands.MODEL_ADD_PROFILE,   _.partial(Factory.createModel, "UMLProfile"));\n    // Model: Classes\n    CommandManager.register(Strings.CMD_MODEL_ADD_CLASS,         UMLCommands.MODEL_ADD_CLASS,         _.partial(Factory.createModel, "UMLClass"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_INTERFACE,     UMLCommands.MODEL_ADD_INTERFACE,     _.partial(Factory.createModel, "UMLInterface"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_SIGNAL,        UMLCommands.MODEL_ADD_SIGNAL,        _.partial(Factory.createModel, "UMLSignal"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DATATYPE,      UMLCommands.MODEL_ADD_DATATYPE,      _.partial(Factory.createModel, "UMLDataType"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_PRIMITIVETYPE, UMLCommands.MODEL_ADD_PRIMITIVETYPE, _.partial(Factory.createModel, "UMLPrimitiveType"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_ENUMERATION,   UMLCommands.MODEL_ADD_ENUMERATION,   _.partial(Factory.createModel, "UMLEnumeration"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_ARTIFACT,      UMLCommands.MODEL_ADD_ARTIFACT,      _.partial(Factory.createModel, "UMLArtifact"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_COMPONENT,     UMLCommands.MODEL_ADD_COMPONENT,     _.partial(Factory.createModel, "UMLComponent"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_NODE,          UMLCommands.MODEL_ADD_NODE,          _.partial(Factory.createModel, "UMLNode"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_USECASE,       UMLCommands.MODEL_ADD_USECASE,       _.partial(Factory.createModel, "UMLUseCase"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_ACTOR,         UMLCommands.MODEL_ADD_ACTOR,         _.partial(Factory.createModel, "UMLActor"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_STEREOTYPE,    UMLCommands.MODEL_ADD_STEREOTYPE,    _.partial(Factory.createModel, "UMLStereotype"));\n    // Model: Instances\n    // TODO: Menus for instace types never be enabled. (Delete them or enable them when an element is selected)\n    CommandManager.register(Strings.CMD_MODEL_ADD_OBJECT,            UMLCommands.MODEL_ADD_OBJECT,            _.partial(Factory.createModel, "UMLObject"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_ARTIFACTINSTANCE,  UMLCommands.MODEL_ADD_ARTIFACTINSTANCE,  _.partial(Factory.createModel, "UMLArtifactInstance"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_COMPONENTINSTANCE, UMLCommands.MODEL_ADD_COMPONENTINSTANCE, _.partial(Factory.createModel, "UMLComponentInstance"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_NODEINSTANCE,      UMLCommands.MODEL_ADD_NODEINSTANCE,      _.partial(Factory.createModel, "UMLNodeInstance"));\n    // Model: Behaviors\n    CommandManager.register(Strings.CMD_MODEL_ADD_COLLABORATION,  UMLCommands.MODEL_ADD_COLLABORATION,  _.partial(Factory.createModel, "UMLCollaboration"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_INTERACTION,    UMLCommands.MODEL_ADD_INTERACTION,    _.partial(Factory.createModel, "UMLInteraction"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_STATEMACHINE,   UMLCommands.MODEL_ADD_STATEMACHINE,   _.partial(Factory.createModel, "UMLStateMachine"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_ACTIVITY,       UMLCommands.MODEL_ADD_ACTIVITY,       _.partial(Factory.createModel, "UMLActivity"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_OPAQUEBEHAVIOR, UMLCommands.MODEL_ADD_OPAQUEBEHAVIOR, _.partial(Factory.createModel, "UMLOpaqueBehavior"));\n    // Model: Features\n    CommandManager.register(Strings.CMD_MODEL_ADD_TEMPLATEPARAMETER,  UMLCommands.MODEL_ADD_TEMPLATEPARAMETER,  _.partial(Factory.createModel, "UMLTemplateParameter",  null, "templateParameters"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_PARAMETER,          UMLCommands.MODEL_ADD_PARAMETER,          _.partial(Factory.createModel, "UMLParameter",          null, "parameters"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_ENUMERATIONLITERAL, UMLCommands.MODEL_ADD_ENUMERATIONLITERAL, _.partial(Factory.createModel, "UMLEnumerationLiteral", null, "literals"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_ATTRIBUTE,          UMLCommands.MODEL_ADD_ATTRIBUTE,          _.partial(Factory.createModel, "UMLAttribute",          null, "attributes"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_PORT,               UMLCommands.MODEL_ADD_PORT,               _.partial(Factory.createModel, "UMLPort",               null, "attributes"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_OPERATION,          UMLCommands.MODEL_ADD_OPERATION,          _.partial(Factory.createModel, "UMLOperation",          null, "operations"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_RECEPTION,          UMLCommands.MODEL_ADD_RECEPTION,          _.partial(Factory.createModel, "UMLReception",          null, "receptions"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_EXTENSIONPOINT,     UMLCommands.MODEL_ADD_EXTENSIONPOINT,     _.partial(Factory.createModel, "UMLExtensionPoint",     null, "extensionPoints"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_SLOT,               UMLCommands.MODEL_ADD_SLOT,               _.partial(Factory.createModel, "UMLSlot",               null, "slots"));\n    // Model: States\n    CommandManager.register(Strings.CMD_MODEL_ADD_STATE,          UMLCommands.MODEL_ADD_STATE,          _.partial(Factory.createModel, "UMLState",  null, "vertices"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_REGION,         UMLCommands.MODEL_ADD_REGION,         _.partial(Factory.createModel, "UMLRegion", null, "regions"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_ENTRY_ACTIVITY, UMLCommands.MODEL_ADD_ENTRY_ACTIVITY, _handleEntryActivity);\n    CommandManager.register(Strings.CMD_MODEL_ADD_DO_ACTIVITY,    UMLCommands.MODEL_ADD_DO_ACTIVITY,    _handleDoActivity);\n    CommandManager.register(Strings.CMD_MODEL_ADD_EXIT_ACTIVITY,  UMLCommands.MODEL_ADD_EXIT_ACTIVITY,  _handleExitActivity);\n    CommandManager.register(Strings.CMD_MODEL_ADD_TRIGGER,        UMLCommands.MODEL_ADD_TRIGGER,        _.partial(Factory.createModel, "UMLEvent",  null, "triggers"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_EFFECT,         UMLCommands.MODEL_ADD_EFFECT,         _handleEffect);\n    // Model: Actions\n    CommandManager.register(Strings.CMD_MODEL_ADD_ACTION,  UMLCommands.MODEL_ADD_ACTION,  _.partial(Factory.createModel, "UMLAction", null, "nodes"));\n    // Model: Common\n    CommandManager.register(Strings.CMD_MODEL_ADD_CONSTRAINT, UMLCommands.MODEL_ADD_CONSTRAINT, _handleConstraint);\n    // Model: Diagrams\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_USECASE,            UMLCommands.MODEL_ADD_DIAGRAM_USECASE,            _.partial(Factory.createDiagram, "UMLUseCaseDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_CLASS,              UMLCommands.MODEL_ADD_DIAGRAM_CLASS,              _.partial(Factory.createDiagram, "UMLClassDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_PACKAGE,            UMLCommands.MODEL_ADD_DIAGRAM_PACKAGE,            _.partial(Factory.createDiagram, "UMLPackageDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_OBJECT,             UMLCommands.MODEL_ADD_DIAGRAM_OBJECT,             _.partial(Factory.createDiagram, "UMLObjectDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_COMPONENT,          UMLCommands.MODEL_ADD_DIAGRAM_COMPONENT,          _.partial(Factory.createDiagram, "UMLComponentDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_DEPLOYMENT,         UMLCommands.MODEL_ADD_DIAGRAM_DEPLOYMENT,         _.partial(Factory.createDiagram, "UMLDeploymentDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_PROFILE,            UMLCommands.MODEL_ADD_DIAGRAM_PROFILE,            _.partial(Factory.createDiagram, "UMLProfileDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_COMPOSITESTRUCTURE, UMLCommands.MODEL_ADD_DIAGRAM_COMPOSITESTRUCTURE, _.partial(Factory.createDiagram, "UMLCompositeStructureDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_SEQUENCE,           UMLCommands.MODEL_ADD_DIAGRAM_SEQUENCE,           _.partial(Factory.createDiagram, "UMLSequenceDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_COMMUNICATION,      UMLCommands.MODEL_ADD_DIAGRAM_COMMUNICATION,      _.partial(Factory.createDiagram, "UMLCommunicationDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_STATECHART,         UMLCommands.MODEL_ADD_DIAGRAM_STATECHART,         _.partial(Factory.createDiagram, "UMLStatechartDiagram"));\n    CommandManager.register(Strings.CMD_MODEL_ADD_DIAGRAM_ACTIVITY,           UMLCommands.MODEL_ADD_DIAGRAM_ACTIVITY,           _.partial(Factory.createDiagram, "UMLActivityDiagram"));\n    // Format\n    CommandManager.register(Strings.CMD_FORMAT_STEREOTYPE,                    UMLCommands.FORMAT_STEREOTYPE,                  _handleStereotypeDisplay);\n    CommandManager.register(Strings.CMD_FORMAT_STEREOTYPE_NONE,               UMLCommands.FORMAT_STEREOTYPE_NONE,             _.partial(_handleStereotypeDisplay, UML.SD_NONE));\n    CommandManager.register(Strings.CMD_FORMAT_STEREOTYPE_LABEL,              UMLCommands.FORMAT_STEREOTYPE_LABEL,            _.partial(_handleStereotypeDisplay, UML.SD_LABEL));\n    CommandManager.register(Strings.CMD_FORMAT_STEREOTYPE_DECORATION,         UMLCommands.FORMAT_STEREOTYPE_DECORATION,       _.partial(_handleStereotypeDisplay, UML.SD_DECORATION));\n    CommandManager.register(Strings.CMD_FORMAT_STEREOTYPE_DECORATION_LABEL,   UMLCommands.FORMAT_STEREOTYPE_DECORATION_LABEL, _.partial(_handleStereotypeDisplay, UML.SD_DECORATION_LABEL));\n    CommandManager.register(Strings.CMD_FORMAT_STEREOTYPE_ICON,               UMLCommands.FORMAT_STEREOTYPE_ICON,             _.partial(_handleStereotypeDisplay, UML.SD_ICON));\n    CommandManager.register(Strings.CMD_FORMAT_STEREOTYPE_ICON_LABEL,         UMLCommands.FORMAT_STEREOTYPE_ICON_LABEL,       _.partial(_handleStereotypeDisplay, UML.SD_ICON_LABEL));\n    CommandManager.register(Strings.CMD_FORMAT_WORD_WRAP,                     UMLCommands.FORMAT_WORD_WRAP,                   _.partial(_handleToggleProperty, "wordWrap"));\n    CommandManager.register(Strings.CMD_FORMAT_SHOW_VISIBILITY,               UMLCommands.FORMAT_SHOW_VISIBILITY,             _.partial(_handleToggleProperty, "showVisibility"));\n    CommandManager.register(Strings.CMD_FORMAT_SHOW_NAMESPACE,                UMLCommands.FORMAT_SHOW_NAMESPACE,              _.partial(_handleToggleProperty, "showNamespace"));\n    CommandManager.register(Strings.CMD_FORMAT_SHOW_PROPERTY,                 UMLCommands.FORMAT_SHOW_PROPERTY,               _.partial(_handleToggleProperty, "showProperty"));\n    CommandManager.register(Strings.CMD_FORMAT_SHOW_TYPE,                     UMLCommands.FORMAT_SHOW_TYPE,                   _.partial(_handleToggleProperty, "showType"));\n    CommandManager.register(Strings.CMD_FORMAT_SHOW_MULTIPLICITY,             UMLCommands.FORMAT_SHOW_MULTIPLICITY,           _.partial(_handleToggleProperty, "showMultiplicity"));\n    CommandManager.register(Strings.CMD_FORMAT_SHOW_OPERATION_SIGNATURE,      UMLCommands.FORMAT_SHOW_OPERATION_SIGNATURE,    _.partial(_handleToggleProperty, "showOperationSignature"));\n    CommandManager.register(Strings.CMD_FORMAT_SUPPRESS_ATTRIBUTES,           UMLCommands.FORMAT_SUPPRESS_ATTRIBUTES,         _.partial(_handleToggleProperty, "suppressAttributes"));\n    CommandManager.register(Strings.CMD_FORMAT_SUPPRESS_OPERATIONS,           UMLCommands.FORMAT_SUPPRESS_OPERATIONS,         _.partial(_handleToggleProperty, "suppressOperations"));\n    CommandManager.register(Strings.CMD_FORMAT_SUPPRESS_RECEPTIONS,           UMLCommands.FORMAT_SUPPRESS_RECEPTIONS,         _.partial(_handleToggleProperty, "suppressReceptions"));\n    CommandManager.register(Strings.CMD_FORMAT_SUPPRESS_LITERALS,             UMLCommands.FORMAT_SUPPRESS_LITERALS,           _.partial(_handleToggleProperty, "suppressLiterals"));\n\n    // Update Commands\n    $(SelectionManager).on("selectionChanged", _updateCommands);\n    $(Repository).on("operationExecuted", _updateCommands);\n\n});\n\n//# sourceURL=/uml/UMLCommandHandlers.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n// TODO: Separate UMLQuickEdit.js\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, type */\n\ndefine('uml/UMLToolbox',['require','exports','module','utils/AppInit','core/Repository','engine/Factory','diagrams/ToolboxView','diagrams/DiagramManager','dialogs/Dialogs','ui/Toast','ui/QuickEdit','dialogs/ElementPickerDialog','uml/UML','uml/UMLDiagram','uml/UMLFactory'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit                 = require(\"utils/AppInit\"),\n        Repository              = require(\"core/Repository\"),\n        Factory                 = require(\"engine/Factory\"),\n        Toolbox                 = require(\"diagrams/ToolboxView\"),\n        DiagramManager          = require(\"diagrams/DiagramManager\"),\n        Dialogs                 = require(\"dialogs/Dialogs\"),\n        Toast                   = require(\"ui/Toast\"),\n        QuickEdit               = require(\"ui/QuickEdit\"),\n        ElementPickerDialog     = require(\"dialogs/ElementPickerDialog\"),\n        UML                     = require(\"uml/UML\"),\n        UMLDiagram              = require(\"uml/UMLDiagram\"),\n        UMLFactory              = require(\"uml/UMLFactory\");\n\n\n    /**\n     * Toolbox Groups\n     * @const\n     */\n    var TXG_USECASE            = 'txg-usecase',\n        TXG_CLASS              = 'txg-class',\n        TXG_CLASS_ADV          = 'txg-class-adv',\n        TXG_PACKAGE            = 'txg-package',\n        TXG_INSTANCE           = 'txg-instance',\n        TXG_COMPOSITESTRUCTURE = 'txg-compositestructure',\n        TXG_SEQUENCE           = 'txg-sequence',\n        TXG_SEQUENCE_ADV       = 'txg-sequence-adv',\n        TXG_COMMUNICATION      = 'txg-communication',\n        TXG_STATECHART         = 'txg-statechart',\n        TXG_STATECHART_ADV     = 'txg-statechart-adv',\n        TXG_ACTIVITY           = 'txg-activity',\n        TXG_ACTIVITY_ADV       = 'txg-activity-adv',\n        TXG_COMPONENT          = 'txg-component',\n        TXG_DEPLOYMENT         = 'txg-deployment',\n        TXG_PROFILE            = 'txg-profile';\n\n    /**\n     * Toolbox Items\n     * @const\n     */\n    var // Classes (Basic)\n        TX_CLASS                = 'tx-class',\n        TX_INTERFACE            = 'tx-interface',\n        TX_ASSOCIATION          = 'tx-association',\n        TX_DIRECTEDASSOCIATION  = 'tx-directedassociation',\n        TX_AGGREGATION          = 'tx-aggregation',\n        TX_COMPOSITION          = 'tx-composition',\n        TX_GENERALIZATION       = 'tx-generalization',\n        TX_DEPENDENCY           = 'tx-dependency',\n        TX_INTERFACEREALIZATION = 'tx-interface-realization',\n        // Classes (Advanced)\n        TX_SIGNAL               = 'tx-signal',\n        TX_DATATYPE             = 'tx-datatype',\n        TX_PRIMITIVETYPE        = 'tx-primitivetype',\n        TX_ENUMERATION          = 'tx-enumeration',\n        TX_ASSOCIATIONCLASS     = 'tx-associationclass',\n        // Packages\n        TX_PACKAGE              = 'tx-package',\n        TX_MODEL                = 'tx-model',\n        TX_SUBSYSTEM            = 'tx-subsystem',\n        TX_CONTAINMENT          = 'tx-containment',\n        // Components\n        TX_ARTIFACT             = 'tx-artifact',\n        TX_COMPONENT            = 'tx-component',\n        TX_COMPONENTREALIZATION = 'tx-component-realization',\n        // Deployments\n        TX_NODE                 = 'tx-node',\n        TX_DEPLOYMENT           = 'tx-deployment',\n        TX_COMMUNICATIONPATH    = 'tx-communicationpath',\n        // Use Cases\n        TX_USECASE              = 'tx-usecase',\n        TX_ACTOR                = 'tx-actor',\n        TX_INCLUDE              = 'tx-include',\n        TX_EXTEND               = 'tx-extend',\n        TX_USECASESUBJECT       = 'tx-usecase-subject',\n        // Instances\n        TX_OBJECT               = 'tx-object',\n        TX_ARTIFACTINSTANCE     = 'tx-artifactinstance',\n        TX_COMPONENTINSTANCE    = 'tx-componentinstance',\n        TX_NODEINSTANCE         = 'tx-nodeinstance',\n        TX_LINK                 = 'tx-link',\n        TX_DIRECTEDLINK         = 'tx-directedlink',\n        // Composite Structures\n        TX_PORT                 = 'tx-port',\n        TX_PART                 = 'tx-part',\n        TX_CONNECTOR            = 'tx-connector',\n        TX_COLLABORATION        = 'tx-collaboration',\n        TX_COLLABORATIONUSE     = 'tx-collaborationuse',\n        TX_ROLEBINDING          = 'tx-rolebinding',\n        TX_REALIZATION          = 'tx-realization',\n        // Interactions (Basic) (Sequence Diagram)\n        TX_LIFELINE             = 'tx-lifeline',\n        TX_MESSAGE              = 'tx-message',\n        TX_SELFMESSAGE          = 'tx-selfmessage',\n        TX_ASYNC_MESSAGE        = 'tx-async-message',\n        TX_REPLY_MESSAGE        = 'tx-reply-message',\n        TX_CREATE_MESSAGE       = 'tx-create-message',\n        TX_DELETE_MESSAGE       = 'tx-delete-message',\n        TX_ASYNC_SIGNAL_MESSAGE = 'tx-async-signal-message',\n        // Interactions (Advanced) (Sequence Diagram)\n        TX_FOUND_MESSAGE        = 'tx-found-message',\n        TX_LOST_MESSAGE         = 'tx-lost-message',\n        TX_ENDPOINT             = 'tx-endpoint',\n        TX_GATE                 = 'tx-gate',\n        TX_STATEINVARIANT       = 'tx-stateinvariant',\n        TX_CONTINUATION         = 'tx-continuation',\n        TX_COMBINEDFRAGMENT     = 'tx-combinedfragment',\n        TX_INTERACTIONUSE       = 'tx-interactionuse',\n        // Interactions (Communication Diagram)\n        TX_COMMLIFELINE         = 'tx-commlifeline',\n        TX_COMMCONNECTOR        = 'tx-commconnector',\n        TX_COMMSELFCONNECTOR    = 'tx-commselfconnector',\n        TX_FORWARDMESSAGE       = 'tx-forwardmessage',\n        TX_REVERSEMESSAGE       = 'tx-reversemessage',\n        TX_FRAME                = 'tx-frame',\n        // State Machines (Basic)\n        TX_STATE                = 'tx-state',\n        TX_COMPOSITESTATE       = 'tx-compositestate',\n        TX_INITIALSTATE         = 'tx-initialstate',\n        TX_CHOICE               = 'tx-choice',\n        TX_JOIN                 = 'tx-join',\n        TX_FORK                 = 'tx-fork',\n        TX_FINALSTATE           = 'tx-finalstate',\n        TX_CONNECTIONPOINT      = 'tx-connection-point',\n        TX_TRANSITION           = 'tx-transition',\n        TX_SELFTRANSITION       = 'tx-selftransition',\n        // State Machines (Advanced)\n        TX_SUBMACHINESTATE      = 'tx-submachinestate',\n        TX_ORTHOGONALSTATE      = 'tx-orthogonalstate',\n        TX_JUNCTION             = 'tx-junction',\n        TX_SHALLOWHISTORY       = 'tx-shallowhistory',\n        TX_DEEPHISTORY          = 'tx-deephistory',\n        TX_ENTRYPOINT           = 'tx-entrypoint',\n        TX_EXITPOINT            = 'tx-exitpoint',\n        TX_TERMINATE            = 'tx-terminate',\n        // Activities (Basic)\n        TX_ACTION               = 'tx-action',\n        TX_INITIALNODE          = 'tx-initialnode',\n        TX_ACTIVITYFINALNODE    = 'tx-activityfinalnode',\n        TX_FORKNODE             = 'tx-forknode',\n        TX_JOINNODE             = 'tx-joinnode',\n        TX_MERGENODE            = 'tx-mergenode',\n        TX_DECISIONNODE         = 'tx-decisionnode',\n        TX_CONTROLFLOW          = 'tx-controlflow',\n        // Activities (Advanced)\n        TX_INPUTPIN             = 'tx-inputpin',\n        TX_OUTPUTPIN            = 'tx-outputpin',\n        TX_SWIMLANEVERT         = 'tx-swimlanevert',\n        TX_SWIMLANEHORZ         = 'tx-swimlanehorz',\n        TX_SENDSIGNAL           = 'tx-sendsignal',\n        TX_ACCEPTSIGNAL         = 'tx-acceptsignal',\n        TX_ACCEPTTIMEEVENT      = 'tx-acceptTimeEvent',\n        TX_FLOWFINALNODE        = 'tx-flowfinalnode',\n        TX_OBJECTNODE           = 'tx-objectnode',\n        TX_CENTRALBUFFERNODE    = 'tx-centralbuffernode',\n        TX_DATASTORENODE        = 'tx-datastorenode',\n        TX_INTERRUPTIBLEACTIVITYREGION = 'tx-interruptibleactivityregion',\n        TX_STRUCTUREDACTIVITYNODE = 'tx-structuredactivitynode',\n        TX_EXPANSIONREGION      = 'tx-expansionregion',\n        TX_INPUTEXPANSIONNODE   = 'tx-inputexpansionnode',\n        TX_OUTPUTEXPANSIONNODE  = 'tx-outputexpansionnode',\n        TX_OBJECTFLOW           = 'tx-objectflow',\n        TX_EXCEPTIONHANDLER     = 'tx-exceptionhandler',\n        TX_ACTIVITYINTERRUPT    = 'tx-activityinterrupt',\n        // Profile\n        TX_METACLASS            = 'tx-metaclass',\n        TX_STEREOTYPE           = 'tx-stereotype',\n        TX_EXTENSION            = 'tx-extension';\n\n\n    /**\n     * Error Messages\n     */\n    var ERR_INVALID_PARENT = \"{{.}} cannot be placed here.\",\n        ERR_INVALID_LINK   = \"Invalid connection ({{.}})\";\n\n    function _getViewByTypes(diagram, canvas, x, y, viewTypes) {\n        var i, len, _type, _view;\n        for (i = 0, len = viewTypes.length; i < len; i++) {\n            _type = viewTypes[i];\n            _view  = diagram.getViewAt(canvas, x, y, true, _type);\n            if (_view) {\n                return _view;\n            }\n        }\n        return null;\n    }\n\n    function setupToolbox() {\n        // Use Case\n        Toolbox.addGroup(TXG_USECASE, 'UseCase', [type.UMLUseCaseDiagram]);\n        Toolbox.addItem(TX_PACKAGE,             TXG_USECASE, 'Package',              'icon-UMLPackage',             'rect');\n        Toolbox.addItem(TX_USECASESUBJECT,      TXG_USECASE, 'Use Case Subject',     'icon-UMLUseCaseSubject',      'rect');\n        Toolbox.addItem(TX_USECASE,             TXG_USECASE, 'Use Case',             'icon-UMLUseCase',             'rect');\n        Toolbox.addItem(TX_ACTOR,               TXG_USECASE, 'Actor',                'icon-UMLActor',               'rect');\n        Toolbox.addItem(TX_FRAME,               TXG_USECASE, 'Frame',                'icon-UMLFrame',               'rect');\n        Toolbox.addItem(TX_ASSOCIATION,         TXG_USECASE, 'Association',          'icon-UMLAssociation',         'line');\n        Toolbox.addItem(TX_DIRECTEDASSOCIATION, TXG_USECASE, 'Directed Association', 'icon-UMLDirectedAssociation', 'line');\n        Toolbox.addItem(TX_GENERALIZATION,      TXG_USECASE, 'Generalization',       'icon-UMLGeneralization',      'line');\n        Toolbox.addItem(TX_DEPENDENCY,          TXG_USECASE, 'Dependency',           'icon-UMLDependency',          'line');\n        Toolbox.addItem(TX_INCLUDE,             TXG_USECASE, 'Include',              'icon-UMLInclude',             'line');\n        Toolbox.addItem(TX_EXTEND,              TXG_USECASE, 'Extend',               'icon-UMLExtend',              'line');\n        // Classes (Basic)\n        Toolbox.addGroup(TXG_CLASS, 'Classes (Basic)', [type.UMLClassDiagram, type.UMLCompositeStructureDiagram]);\n        Toolbox.addItem(TX_CLASS,                TXG_CLASS, 'Class',                 'icon-UMLClass',                'rect');\n        Toolbox.addItem(TX_INTERFACE,            TXG_CLASS, 'Interface',             'icon-UMLInterface',            'rect');\n        Toolbox.addItem(TX_ASSOCIATION,          TXG_CLASS, 'Association',           'icon-UMLAssociation',          'line');\n        Toolbox.addItem(TX_DIRECTEDASSOCIATION,  TXG_CLASS, 'Directed Association',  'icon-UMLDirectedAssociation',  'line');\n        Toolbox.addItem(TX_AGGREGATION,          TXG_CLASS, 'Aggregation',           'icon-UMLAggregation',          'line');\n        Toolbox.addItem(TX_COMPOSITION,          TXG_CLASS, 'Composition',           'icon-UMLComposition',          'line');\n        Toolbox.addItem(TX_DEPENDENCY,           TXG_CLASS, 'Dependency',            'icon-UMLDependency',           'line');\n        Toolbox.addItem(TX_GENERALIZATION,       TXG_CLASS, 'Generalization',        'icon-UMLGeneralization',       'line');\n        Toolbox.addItem(TX_INTERFACEREALIZATION, TXG_CLASS, 'Interface Realization', 'icon-UMLInterfaceRealization', 'line');\n        // Classes (Advanced)\n        Toolbox.addGroup(TXG_CLASS_ADV, 'Classes (Advanced)', [type.UMLClassDiagram, type.UMLCompositeStructureDiagram]);\n        Toolbox.addItem(TX_SIGNAL,              TXG_CLASS_ADV, 'Signal',               'icon-UMLSignal',               'rect');\n        Toolbox.addItem(TX_DATATYPE,            TXG_CLASS_ADV, 'DataType',             'icon-UMLDataType',             'rect');\n        Toolbox.addItem(TX_PRIMITIVETYPE,       TXG_CLASS_ADV, 'PrimitiveType',        'icon-UMLPrimitiveType',        'rect');\n        Toolbox.addItem(TX_ENUMERATION,         TXG_CLASS_ADV, 'Enumeration',          'icon-UMLEnumeration',          'rect');\n        Toolbox.addItem(TX_FRAME,               TXG_CLASS_ADV, 'Frame',                'icon-UMLFrame',                'rect');\n        Toolbox.addItem(TX_ASSOCIATIONCLASS,    TXG_CLASS_ADV, 'Association Class',    'icon-UMLAssociationClassLink', 'line');\n        // Packages\n        Toolbox.addGroup(TXG_PACKAGE, 'Packages', [type.UMLClassDiagram, type.UMLPackageDiagram]);\n        Toolbox.addItem(TX_PACKAGE,             TXG_PACKAGE, 'Package',               'icon-UMLPackage',              'rect');\n        Toolbox.addItem(TX_MODEL,               TXG_PACKAGE, 'Model',                 'icon-UMLModel',                'rect');\n        Toolbox.addItem(TX_SUBSYSTEM,           TXG_PACKAGE, 'Subsystem',             'icon-UMLSubsystem',            'rect');\n        Toolbox.addItem(TX_CONTAINMENT,         TXG_PACKAGE, 'Containment',           'icon-UMLContainment',          'line');\n        Toolbox.addItem(TX_DEPENDENCY,          TXG_PACKAGE, 'Dependency',            'icon-UMLDependency',           'line');\n        // Component\n        Toolbox.addGroup(TXG_COMPONENT, 'Component', [type.UMLComponentDiagram, type.UMLDeploymentDiagram]);\n        Toolbox.addItem(TX_COMPONENT,            TXG_COMPONENT, 'Component',             'icon-UMLComponent',            'rect');\n        Toolbox.addItem(TX_ARTIFACT,             TXG_COMPONENT, 'Artifact',              'icon-UMLArtifact',             'rect');\n        Toolbox.addItem(TX_INTERFACE,            TXG_COMPONENT, 'Interface',             'icon-UMLInterface',            'rect');\n        Toolbox.addItem(TX_FRAME,                TXG_COMPONENT, 'Frame',                 'icon-UMLFrame',                'rect');\n        Toolbox.addItem(TX_DEPENDENCY,           TXG_COMPONENT, 'Dependency',            'icon-UMLDependency',           'line');\n        Toolbox.addItem(TX_INTERFACEREALIZATION, TXG_COMPONENT, 'Interface Realization', 'icon-UMLInterfaceRealization', 'line');\n        Toolbox.addItem(TX_COMPONENTREALIZATION, TXG_COMPONENT, 'Component Realization', 'icon-UMLComponentRealization', 'line');\n        // Deployment\n        Toolbox.addGroup(TXG_DEPLOYMENT, 'Deployment', [type.UMLComponentDiagram, type.UMLDeploymentDiagram]);\n        Toolbox.addItem(TX_NODE,                TXG_DEPLOYMENT, 'Node',                 'icon-UMLNode',                'rect');\n        Toolbox.addItem(TX_DEPLOYMENT,          TXG_DEPLOYMENT, 'Deployment',           'icon-UMLDependency',          'line');\n        Toolbox.addItem(TX_COMMUNICATIONPATH,   TXG_DEPLOYMENT, 'Communication Path',   'icon-UMLCommunicationPath',   'line');\n        // Composite Structure\n        Toolbox.addGroup(TXG_COMPOSITESTRUCTURE, 'Composite Structure', [type.UMLClassDiagram, type.UMLCompositeStructureDiagram, type.UMLComponentDiagram]);\n        Toolbox.addItem(TX_COLLABORATION,    TXG_COMPOSITESTRUCTURE, 'Collaboration',     'icon-UMLCollaboration',    'rect');\n        Toolbox.addItem(TX_PORT,             TXG_COMPOSITESTRUCTURE, 'Port',              'icon-UMLPort',             'point');\n        Toolbox.addItem(TX_PART,             TXG_COMPOSITESTRUCTURE, 'Part',              'icon-UMLPart',             'rect');\n        Toolbox.addItem(TX_CONNECTOR,        TXG_COMPOSITESTRUCTURE, 'Connector',         'icon-UMLConnector',        'line');\n        Toolbox.addItem(TX_COLLABORATIONUSE, TXG_COMPOSITESTRUCTURE, 'Collaboration Use', 'icon-UMLCollaborationUse', 'rect');\n        Toolbox.addItem(TX_ROLEBINDING,      TXG_COMPOSITESTRUCTURE, 'Role Binding',      'icon-UMLDependency',       'line');\n        Toolbox.addItem(TX_REALIZATION,      TXG_COMPOSITESTRUCTURE, 'Realization',       'icon-UMLRealization',      'line');\n        // Instances\n        Toolbox.addGroup(TXG_INSTANCE, 'Instances', [type.UMLClassDiagram, type.UMLObjectDiagram, type.UMLComponentDiagram, type.UMLDeploymentDiagram]);\n        Toolbox.addItem(TX_OBJECT,            TXG_INSTANCE, 'Object',               'icon-UMLObject',            'rect');\n        Toolbox.addItem(TX_ARTIFACTINSTANCE,  TXG_INSTANCE, 'Artifact Instance',    'icon-UMLArtifactInstance',  'rect');\n        Toolbox.addItem(TX_COMPONENTINSTANCE, TXG_INSTANCE, 'Component Instance',   'icon-UMLComponentInstance', 'rect');\n        Toolbox.addItem(TX_NODEINSTANCE,      TXG_INSTANCE, 'Node Instance',        'icon-UMLNodeInstance',      'rect');\n        Toolbox.addItem(TX_LINK,              TXG_INSTANCE, 'Link',                 'icon-UMLLink',              'line');\n        Toolbox.addItem(TX_DIRECTEDLINK,      TXG_INSTANCE, 'Directed Link',        'icon-UMLDirectedLink',      'line');\n        // Sequence (Basic)\n        Toolbox.addGroup(TXG_SEQUENCE, 'Interactions (Basic)', [type.UMLSequenceDiagram]);\n        Toolbox.addItem(TX_LIFELINE,             TXG_SEQUENCE, 'Lifeline',             'icon-UMLLifeline',           'rect');\n        Toolbox.addItem(TX_MESSAGE,              TXG_SEQUENCE, 'Message',              'icon-UMLMessage',            'line');\n        Toolbox.addItem(TX_SELFMESSAGE,          TXG_SEQUENCE, 'Self Message',         'icon-UMLSelfMessage',        'point');\n        Toolbox.addItem(TX_ASYNC_MESSAGE,        TXG_SEQUENCE, 'Async Message',        'icon-UMLAsyncMessage',       'line');\n        Toolbox.addItem(TX_REPLY_MESSAGE,        TXG_SEQUENCE, 'Reply Message',        'icon-UMLReplyMessage',       'line');\n        Toolbox.addItem(TX_CREATE_MESSAGE,       TXG_SEQUENCE, 'Create Message',       'icon-UMLCreateMessage',      'line');\n        Toolbox.addItem(TX_DELETE_MESSAGE,       TXG_SEQUENCE, 'Delete Message',       'icon-UMLDeleteMessage',      'line');\n        Toolbox.addItem(TX_ASYNC_SIGNAL_MESSAGE, TXG_SEQUENCE, 'Async Signal Message', 'icon-UMLAsyncSignalMessage', 'line');\n        // Sequence (Advanced)\n        Toolbox.addGroup(TXG_SEQUENCE_ADV, 'Interactions (Advanced)', [type.UMLSequenceDiagram]);\n        Toolbox.addItem(TX_FOUND_MESSAGE,      TXG_SEQUENCE_ADV, 'Found Message',       'icon-UMLFoundMessage',       'line');\n        Toolbox.addItem(TX_LOST_MESSAGE,       TXG_SEQUENCE_ADV, 'Lost Message',        'icon-UMLLostMessage',        'line');\n        Toolbox.addItem(TX_ENDPOINT,           TXG_SEQUENCE_ADV, 'Endpoint',            'icon-UMLEndpoint',           'point');\n        Toolbox.addItem(TX_GATE,               TXG_SEQUENCE_ADV, 'Gate',                'icon-UMLGate',               'point');\n        Toolbox.addItem(TX_STATEINVARIANT,     TXG_SEQUENCE_ADV, 'State Invariant',     'icon-UMLStateInvariant',     'point');\n        Toolbox.addItem(TX_CONTINUATION,       TXG_SEQUENCE_ADV, 'Continuation',        'icon-UMLContinuation',       'rect');\n        Toolbox.addItem(TX_COMBINEDFRAGMENT,   TXG_SEQUENCE_ADV, 'Combined Fragment',   'icon-UMLCombinedFragment',   'rect');\n        Toolbox.addItem(TX_INTERACTIONUSE,     TXG_SEQUENCE_ADV, 'Interaction Use',     'icon-UMLInteractionUse',     'rect');\n        Toolbox.addItem(TX_FRAME,              TXG_SEQUENCE_ADV, 'Frame',               'icon-UMLFrame',              'rect');\n        // Communication\n        Toolbox.addGroup(TXG_COMMUNICATION, 'Communication', [type.UMLCommunicationDiagram]);\n        Toolbox.addItem(TX_COMMLIFELINE,      TXG_COMMUNICATION, 'Lifeline',        'icon-UMLObject',         'rect');\n        Toolbox.addItem(TX_COMMCONNECTOR,     TXG_COMMUNICATION, 'Connector',       'icon-UMLConnector',      'line');\n        Toolbox.addItem(TX_COMMSELFCONNECTOR, TXG_COMMUNICATION, 'Self Connector',  'icon-UMLSelfConnector',  'point');\n        Toolbox.addItem(TX_FORWARDMESSAGE,    TXG_COMMUNICATION, 'Forward Message', 'icon-UMLForwardMessage', 'line');\n        Toolbox.addItem(TX_REVERSEMESSAGE,    TXG_COMMUNICATION, 'Reverse Message', 'icon-UMLReverseMessage', 'line');\n        Toolbox.addItem(TX_FRAME,             TXG_COMMUNICATION, 'Frame',           'icon-UMLFrame',          'rect');\n        // Statechart (Basic)\n        Toolbox.addGroup(TXG_STATECHART, 'Statechart (Basic)', [type.UMLStatechartDiagram]);\n        Toolbox.addItem(TX_STATE,           TXG_STATECHART, 'Simple State',     'icon-UMLState',           'rect');\n        Toolbox.addItem(TX_INITIALSTATE,    TXG_STATECHART, 'Initial State',    'icon-UMLInitialState',    'point');\n        Toolbox.addItem(TX_CHOICE,          TXG_STATECHART, 'Choice',           'icon-UMLChoice',          'point');\n        Toolbox.addItem(TX_JOIN,            TXG_STATECHART, 'Join',             'icon-UMLJoin',            'rect');\n        Toolbox.addItem(TX_FORK,            TXG_STATECHART, 'Fork',             'icon-UMLFork',            'rect');\n        Toolbox.addItem(TX_FINALSTATE,      TXG_STATECHART, 'Final State',      'icon-UMLFinalState',      'point');\n        Toolbox.addItem(TX_TRANSITION,      TXG_STATECHART, 'Transition',       'icon-UMLTransition',      'line');\n        Toolbox.addItem(TX_SELFTRANSITION,  TXG_STATECHART, 'Self Transition',  'icon-UMLSelfTransition',  'point');\n        // Statechart (Advanced)\n        Toolbox.addGroup(TXG_STATECHART_ADV, 'Statechart (Advanced)', [type.UMLStatechartDiagram]);\n        Toolbox.addItem(TX_COMPOSITESTATE,  TXG_STATECHART_ADV, 'Composite State',            'icon-UMLCompositeState',           'rect');\n        Toolbox.addItem(TX_SUBMACHINESTATE, TXG_STATECHART_ADV, 'Submachine State',           'icon-UMLSubmachineState',          'rect');\n        Toolbox.addItem(TX_ORTHOGONALSTATE, TXG_STATECHART_ADV, 'Orthogonal State',           'icon-UMLOrthogonalState',          'rect');\n        Toolbox.addItem(TX_JUNCTION,        TXG_STATECHART_ADV, 'Junction',                   'icon-UMLJunction',                 'point');\n        Toolbox.addItem(TX_SHALLOWHISTORY,  TXG_STATECHART_ADV, 'Shallow History',            'icon-UMLShallowHistory',           'point');\n        Toolbox.addItem(TX_DEEPHISTORY,     TXG_STATECHART_ADV, 'Deep History',               'icon-UMLDeepHistory',              'point');\n        Toolbox.addItem(TX_ENTRYPOINT,      TXG_STATECHART_ADV, 'Entry Point',                'icon-UMLEntryPoint',               'point');\n        Toolbox.addItem(TX_EXITPOINT,       TXG_STATECHART_ADV, 'Exit Point',                 'icon-UMLExitPoint',                'point');\n        Toolbox.addItem(TX_TERMINATE,       TXG_STATECHART_ADV, 'Terminate',                  'icon-UMLTerminate',                'point');\n        Toolbox.addItem(TX_CONNECTIONPOINT, TXG_STATECHART_ADV, 'Connection Point Reference', 'icon-UMLConnectionPointReference', 'point');\n        Toolbox.addItem(TX_FRAME,           TXG_STATECHART_ADV, 'Frame',                      'icon-UMLFrame',                    'rect');\n        // Activity (Basic)\n        Toolbox.addGroup(TXG_ACTIVITY, 'Activities (Basic)', [type.UMLActivityDiagram]);\n        Toolbox.addItem(TX_ACTION,            TXG_ACTIVITY, 'Action',                'icon-UMLAction',            'rect');\n        Toolbox.addItem(TX_INITIALNODE,       TXG_ACTIVITY, 'Initial',               'icon-UMLInitialNode',       'point');\n        Toolbox.addItem(TX_ACTIVITYFINALNODE, TXG_ACTIVITY, 'Final',                 'icon-UMLActivityFinalNode', 'point');\n        Toolbox.addItem(TX_FORKNODE,          TXG_ACTIVITY, 'Fork',                  'icon-UMLForkNode',          'rect');\n        Toolbox.addItem(TX_JOINNODE,          TXG_ACTIVITY, 'Join',                  'icon-UMLJoinNode',          'rect');\n        Toolbox.addItem(TX_MERGENODE,         TXG_ACTIVITY, 'Merge',                 'icon-UMLMergeNode',         'point');\n        Toolbox.addItem(TX_DECISIONNODE,      TXG_ACTIVITY, 'Decision',              'icon-UMLDecisionNode',      'point');\n        Toolbox.addItem(TX_CONTROLFLOW,       TXG_ACTIVITY, 'Control Flow',          'icon-UMLControlFlow',       'line');\n        // Activity (Advanced)\n        Toolbox.addGroup(TXG_ACTIVITY_ADV, 'Activities (Advanced)', [type.UMLActivityDiagram]);\n        Toolbox.addItem(TX_SWIMLANEVERT,      TXG_ACTIVITY_ADV, 'Swimlane (Vertical)',   'icon-UMLSwimlaneVert',      'rect');\n        Toolbox.addItem(TX_SWIMLANEHORZ,      TXG_ACTIVITY_ADV, 'Swimlane (Horizontal)', 'icon-UMLSwimlaneHorz',      'rect');\n        Toolbox.addItem(TX_INTERRUPTIBLEACTIVITYREGION, TXG_ACTIVITY_ADV, 'Interruptible Activity Region', 'icon-UMLInterruptibleActivityRegion', 'rect');\n        Toolbox.addItem(TX_STRUCTUREDACTIVITYNODE, TXG_ACTIVITY_ADV, 'Structured Activity', 'icon-UMLStructuredActivityNode', 'rect');\n        Toolbox.addItem(TX_INPUTPIN,          TXG_ACTIVITY_ADV, 'Input Pin',             'icon-UMLInputPin',          'point');\n        Toolbox.addItem(TX_OUTPUTPIN,         TXG_ACTIVITY_ADV, 'Output Pin',            'icon-UMLOutputPin',         'point');\n        Toolbox.addItem(TX_SENDSIGNAL,        TXG_ACTIVITY_ADV, 'Send Signal',           'icon-UMLSendSignal',        'rect');\n        Toolbox.addItem(TX_ACCEPTSIGNAL,      TXG_ACTIVITY_ADV, 'Accept Signal',         'icon-UMLAcceptSignal',      'rect');\n        Toolbox.addItem(TX_ACCEPTTIMEEVENT,   TXG_ACTIVITY_ADV, 'Accept Time Event',     'icon-UMLAcceptTimeEvent',   'rect');\n        Toolbox.addItem(TX_FLOWFINALNODE,     TXG_ACTIVITY_ADV, 'Flow Final',            'icon-UMLFlowFinalNode',     'point');\n        Toolbox.addItem(TX_FRAME,             TXG_ACTIVITY_ADV, 'Frame',                 'icon-UMLFrame',             'rect');\n        Toolbox.addItem(TX_OBJECTNODE,        TXG_ACTIVITY_ADV, 'Object Node',           'icon-UMLObjectNode',        'rect');\n        Toolbox.addItem(TX_CENTRALBUFFERNODE, TXG_ACTIVITY_ADV, 'Central Buffer',        'icon-UMLCentralBufferNode', 'rect');\n        Toolbox.addItem(TX_DATASTORENODE,     TXG_ACTIVITY_ADV, 'Datastore',             'icon-UMLDataStoreNode',     'rect');\n        Toolbox.addItem(TX_EXPANSIONREGION,   TXG_ACTIVITY_ADV, 'Expansion Region',      'icon-UMLExpansionRegion',   'rect');\n        Toolbox.addItem(TX_INPUTEXPANSIONNODE, TXG_ACTIVITY_ADV,'Input Expansion Node',  'icon-UMLInputExpansionNode','point');\n        Toolbox.addItem(TX_OUTPUTEXPANSIONNODE,TXG_ACTIVITY_ADV,'Output Expansion Node', 'icon-UMLOutputExpansionNode','point');\n        Toolbox.addItem(TX_OBJECTFLOW,        TXG_ACTIVITY_ADV, 'Object Flow',           'icon-UMLObjectFlow',        'line');\n        Toolbox.addItem(TX_EXCEPTIONHANDLER,  TXG_ACTIVITY_ADV, 'Exception Handler',     'icon-UMLExceptionHandler',  'line');\n        Toolbox.addItem(TX_ACTIVITYINTERRUPT, TXG_ACTIVITY_ADV, 'Activity Interrupt',    'icon-UMLActivityInterrupt', 'line');\n        // Profile\n        Toolbox.addGroup(TXG_PROFILE, 'Profiles', [type.UMLProfileDiagram]);\n        Toolbox.addItem(TX_METACLASS,      TXG_PROFILE, 'MetaClass',      'icon-UMLMetaClass',      'rect');\n        Toolbox.addItem(TX_STEREOTYPE,     TXG_PROFILE, 'Stereotype',     'icon-UMLStereotype',     'rect');\n        Toolbox.addItem(TX_ENUMERATION,    TXG_PROFILE, 'Enumeration',    'icon-UMLEnumeration',    'rect');\n        Toolbox.addItem(TX_EXTENSION,      TXG_PROFILE, 'Extension',      'icon-UMLExtension',      'line');\n        Toolbox.addItem(TX_GENERALIZATION, TXG_PROFILE, 'Generalization', 'icon-UMLGeneralization', 'line');\n\n        // Event Handling\n        $(Toolbox).on('elementCreated', function (event, id, editor, x1, y1, x2, y2) {\n            try {\n                var diagram   = editor.diagram,\n                    parent    = diagram._parent,\n                    model     = null,\n                    view      = null,\n                    tailView  = diagram.getViewAt(editor.canvas, x1, y1, true),\n                    headView  = diagram.getViewAt(editor.canvas, x2, y2, true),\n                    tailModel = tailView ? tailView.model : null,\n                    headModel = headView ? headView.model : null,\n                    viewTypes;\n\n                var options = {\n                        x1        : x1,\n                        y1        : y1,\n                        x2        : x2,\n                        y2        : y2,\n                        tailView  : diagram.getViewAt(editor.canvas, x1, y1, true),\n                        headView  : diagram.getViewAt(editor.canvas, x2, y2, true),\n                        tailModel : tailView ? tailView.model : null,\n                        headModel : headView ? headView.model : null\n                    };\n\n                switch (id) {\n                // Classes\n                case TX_CLASS:\n                    view = Factory.createModelAndView(\"UMLClass\", parent, diagram, options);\n                    break;\n                case TX_INTERFACE:\n                    view = Factory.createModelAndView(\"UMLInterface\", parent, diagram, options);\n                    break;\n                case TX_SIGNAL:\n                    view = Factory.createModelAndView(\"UMLSignal\", parent, diagram, options);\n                    break;\n                case TX_DATATYPE:\n                    view = Factory.createModelAndView(\"UMLDataType\", parent, diagram, options);\n                    break;\n                case TX_PRIMITIVETYPE:\n                    view = Factory.createModelAndView(\"UMLPrimitiveType\", parent, diagram, options);\n                    break;\n                case TX_ENUMERATION:\n                    view = Factory.createModelAndView(\"UMLEnumeration\", parent, diagram, options);\n                    break;\n                case TX_ASSOCIATION:\n                    view = Factory.createModelAndView(\"UMLAssociation\", tailModel, diagram, options);\n                    break;\n                case TX_DIRECTEDASSOCIATION:\n                    options.modelInitializer = function (model) {\n                        model.end1.navigable = false;\n                    };\n                    view = Factory.createModelAndView(\"UMLAssociation\", tailModel, diagram, options);\n                    break;\n                case TX_AGGREGATION:\n                    options.modelInitializer = function (model) {\n                        model.end2.aggregation = UML.AK_SHARED;\n                    };\n                    view = Factory.createModelAndView(\"UMLAssociation\", tailModel, diagram, options);\n                    break;\n                case TX_COMPOSITION:\n                    options.modelInitializer = function (model) {\n                        model.end2.aggregation = UML.AK_COMPOSITE;\n                    };\n                    view = Factory.createModelAndView(\"UMLAssociation\", tailModel, diagram, options);\n                    break;\n                case TX_ASSOCIATIONCLASS:\n                    view = Factory.createModelAndView(\"UMLAssociationClass\", tailModel, diagram, options);\n                    break;\n                case TX_DEPENDENCY:\n                    view = Factory.createModelAndView(\"UMLDependency\", tailModel, diagram, options);\n                    break;\n                case TX_GENERALIZATION:\n                    view = Factory.createModelAndView(\"UMLGeneralization\", tailModel, diagram, options);\n                    break;\n                case TX_INTERFACEREALIZATION:\n                    view = Factory.createModelAndView(\"UMLInterfaceRealization\", tailModel, diagram, options);\n                    break;\n\n                // Packages\n                case TX_MODEL:\n                    view = Factory.createModelAndView(\"UMLModel\", parent, diagram, options);\n                    break;\n                case TX_PACKAGE:\n                    view = Factory.createModelAndView(\"UMLPackage\", parent, diagram, options);\n                    break;\n                case TX_SUBSYSTEM:\n                    view = Factory.createModelAndView(\"UMLSubsystem\", parent, diagram, options);\n                    break;\n                case TX_CONTAINMENT:\n                    view = Factory.createModelAndView(\"UMLContainment\", tailModel, diagram, options);\n                    break;\n\n                // Composite Structures\n                case TX_PORT:\n                    options.containerView = headView;\n                    view = Factory.createModelAndView(\"UMLPort\", headModel, diagram, options);\n                    break;\n                case TX_PART:\n                    options.containerView = headView;\n                    view = Factory.createModelAndView(\"UMLPart\", headModel, diagram, options);\n                    break;\n                case TX_COLLABORATION:\n                    view = Factory.createModelAndView(\"UMLCollaboration\", parent, diagram, options);\n                    break;\n                case TX_COLLABORATIONUSE:\n                    view = Factory.createModelAndView(\"UMLCollaborationUse\", parent, diagram, options);\n                    break;\n                case TX_CONNECTOR:\n                    view = Factory.createModelAndView(\"UMLConnector\", tailModel, diagram, options);\n                    break;\n                case TX_ROLEBINDING:\n                    view = Factory.createModelAndView(\"UMLRoleBinding\", tailModel, diagram, options);\n                    break;\n                case TX_REALIZATION:\n                    view = Factory.createModelAndView(\"UMLRealization\", tailModel, diagram, options);\n                    break;\n\n                // Components\n                case TX_COMPONENT:\n                    view = Factory.createModelAndView(\"UMLComponent\", parent, diagram, options);\n                    break;\n                case TX_ARTIFACT:\n                    view = Factory.createModelAndView(\"UMLArtifact\", parent, diagram, options);\n                    break;\n                case TX_COMPONENTREALIZATION:\n                    view = Factory.createModelAndView(\"UMLComponentRealization\", tailModel, diagram, options);\n                    break;\n\n                // Deployments\n                case TX_NODE:\n                    view = Factory.createModelAndView(\"UMLNode\", parent, diagram, options);\n                    break;\n                case TX_DEPLOYMENT:\n                    view = Factory.createModelAndView(\"UMLDeployment\", tailModel, diagram, options);\n                    break;\n                case TX_COMMUNICATIONPATH:\n                    view = Factory.createModelAndView(\"UMLCommunicationPath\", tailModel, diagram, options);\n                    break;\n\n                // Instances\n                case TX_OBJECT:\n                    view = Factory.createModelAndView(\"UMLObject\", parent, diagram, options);\n                    break;\n                case TX_ARTIFACTINSTANCE:\n                    view = Factory.createModelAndView(\"UMLArtifactInstance\", parent, diagram, options);\n                    break;\n                case TX_COMPONENTINSTANCE:\n                    view = Factory.createModelAndView(\"UMLComponentInstance\", parent, diagram, options);\n                    break;\n                case TX_NODEINSTANCE:\n                    view = Factory.createModelAndView(\"UMLNodeInstance\", parent, diagram, options);\n                    break;\n                case TX_LINK:\n                    view = Factory.createModelAndView(\"UMLLink\", tailModel, diagram, options);\n                    break;\n                case TX_DIRECTEDLINK:\n                    options.modelInitializer = function (m) { m.end1.navigable = false; };\n                    view = Factory.createModelAndView(\"UMLLink\", tailModel, diagram, options);\n                    break;\n\n                // Use Cases\n                case TX_USECASESUBJECT:\n                    view = Factory.createModelAndView(\"UMLUseCaseSubject\", parent, diagram, options);\n                    break;\n                case TX_USECASE:\n                    view = Factory.createModelAndView(\"UMLUseCase\", parent, diagram, options);\n                    break;\n                case TX_ACTOR:\n                    view = Factory.createModelAndView(\"UMLActor\", parent, diagram, options);\n                    break;\n                case TX_INCLUDE:\n                    view = Factory.createModelAndView(\"UMLInclude\", tailModel, diagram, options);\n                    break;\n                case TX_EXTEND:\n                    view = Factory.createModelAndView(\"UMLExtend\", tailModel, diagram, options);\n                    break;\n\n                // Interactions (Sequence Diagram)\n                case TX_LIFELINE:\n                    view = Factory.createModelAndView(\"UMLLifeline\", parent, diagram, options);\n                    break;\n                case TX_ENDPOINT:\n                    view = Factory.createModelAndView(\"UMLEndpoint\", parent, diagram, options);\n                    break;\n                case TX_GATE:\n                    view = Factory.createModelAndView(\"UMLGate\", parent, diagram, options);\n                    break;\n                case TX_MESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView, type.UMLMessageEndpointView ];\n                    options.tailView  = _getViewByTypes(diagram, editor.canvas, x1, y1, viewTypes);\n                    options.headView  = _getViewByTypes(diagram, editor.canvas, x2, y2, viewTypes);\n                    options.tailModel = options.tailView ? options.tailView.model : null;\n                    options.headModel = options.headView ? options.headView.model : null;\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n                case TX_SELFMESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView ];\n                    options.headView  = _getViewByTypes(diagram, editor.canvas, x2, y2, viewTypes);\n                    options.tailView = options.headView;\n                    options.headModel = options.headView ? options.headView.model : null;\n                    options.tailModel = options.headModel;\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n                case TX_ASYNC_MESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView, type.UMLMessageEndpointView ];\n                    options.tailView  = _getViewByTypes(diagram, editor.canvas, x1, y1, viewTypes);\n                    options.headView  = _getViewByTypes(diagram, editor.canvas, x2, y2, viewTypes);\n                    options.tailModel = options.tailView ? options.tailView.model : null;\n                    options.headModel = options.headView ? options.headView.model : null;\n                    options.modelInitializer = function (model) {\n                        model.messageSort = UML.MS_ASYNCHCALL;\n                    };\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n                case TX_REPLY_MESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView, type.UMLMessageEndpointView ];\n                    options.tailView  = _getViewByTypes(diagram, editor.canvas, x1, y1, viewTypes);\n                    options.headView  = _getViewByTypes(diagram, editor.canvas, x2, y2, viewTypes);\n                    options.tailModel = options.tailView ? options.tailView.model : null;\n                    options.headModel = options.headView ? options.headView.model : null;\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                        model.messageSort = UML.MS_REPLY;\n                    };\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n                case TX_CREATE_MESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView, type.UMLMessageEndpointView ];\n                    options.tailView  = _getViewByTypes(diagram, editor.canvas, x1, y1, viewTypes);\n                    options.headView  = _getViewByTypes(diagram, editor.canvas, x2, y2, viewTypes);\n                    options.tailModel = options.tailView ? options.tailView.model : null;\n                    options.headModel = options.headView ? options.headView.model : null;\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                        model.messageSort = UML.MS_CREATEMESSAGE;\n                    };\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n                case TX_DELETE_MESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView, type.UMLMessageEndpointView ];\n                    options.tailView  = _getViewByTypes(diagram, editor.canvas, x1, y1, viewTypes);\n                    options.headView  = _getViewByTypes(diagram, editor.canvas, x2, y2, viewTypes);\n                    options.tailModel = options.tailView ? options.tailView.model : null;\n                    options.headModel = options.headView ? options.headView.model : null;\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                        model.messageSort = UML.MS_DELETEMESSAGE;\n                    };\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n                case TX_ASYNC_SIGNAL_MESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView, type.UMLMessageEndpointView ];\n                    options.tailView  = _getViewByTypes(diagram, editor.canvas, x1, y1, viewTypes);\n                    options.headView  = _getViewByTypes(diagram, editor.canvas, x2, y2, viewTypes);\n                    options.tailModel = options.tailView ? options.tailView.model : null;\n                    options.headModel = options.headView ? options.headView.model : null;\n                    options.modelInitializer = function (model) {\n                        model.messageSort = UML.MS_ASYNCHSIGNAL;\n                    };\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n                case TX_FOUND_MESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView, type.UMLMessageEndpointView ];\n                    options.headView  = _getViewByTypes(diagram, editor.canvas, x2, y2, viewTypes);\n                    options.headModel = options.headView ? options.headView.model : null;\n                    view = Factory.createModelAndView(\"UMLFoundMessage\", parent, diagram, options);\n                    break;\n                case TX_LOST_MESSAGE:\n                    viewTypes = [ type.UMLSeqLifelineView, type.UMLMessageEndpointView ];\n                    options.tailView  = _getViewByTypes(diagram, editor.canvas, x1, y1, viewTypes);\n                    options.tailModel = options.tailView ? options.tailView.model : null;\n                    view = Factory.createModelAndView(\"UMLLostMessage\", parent, diagram, options);\n                    break;\n                case TX_STATEINVARIANT:\n                    view = Factory.createModelAndView(\"UMLStateInvariant\", parent, diagram, options);\n                    break;\n                case TX_CONTINUATION:\n                    view = Factory.createModelAndView(\"UMLContinuation\", parent, diagram, options);\n                    break;\n                case TX_COMBINEDFRAGMENT:\n                    view = Factory.createModelAndView(\"UMLCombinedFragment\", parent, diagram, options);\n                    break;\n                case TX_INTERACTIONUSE:\n                    view = Factory.createModelAndView(\"UMLInteractionUse\", parent, diagram, options);\n                    break;\n                case TX_FRAME:\n                    ElementPickerDialog.showDialog(\"Select an element to be represented as Frame\", null, type.Model).done(function (buttonId, selected) {\n                        if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                            options.viewInitializer = function (v) {\n                                v.model = selected;\n                            };\n                            view = Factory.createModelAndView(\"UMLFrame\", parent, diagram, options);\n                        }\n                    });\n                    break;\n\n                // Interactions (Communication Diagram)\n                case TX_COMMLIFELINE:\n                    view = Factory.createModelAndView(\"UMLLifeline\", parent, diagram, options);\n                    break;\n                case TX_COMMCONNECTOR:\n                    view = Factory.createModelAndView(\"UMLConnector\", parent, diagram, options);\n                    break;\n                case TX_COMMSELFCONNECTOR:\n                    options.tailModel = options.headModel;\n                    options.tailView = options.headView;\n                    view = Factory.createModelAndView(\"UMLConnector\", parent, diagram, options);\n                    break;\n                case TX_FORWARDMESSAGE:\n                    options.direction = \"forward\";\n                    options.connector = options.headModel;\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n                case TX_REVERSEMESSAGE:\n                    options.direction = \"reverse\";\n                    options.connector = options.headModel;\n                    view = Factory.createModelAndView(\"UMLMessage\", parent, diagram, options);\n                    break;\n\n                // State Machines\n                case TX_STATE:\n                    view = Factory.createModelAndView(\"UMLState\", parent, diagram, options);\n                    break;\n                case TX_COMPOSITESTATE:\n                    options.regionCount = 1;\n                    view = Factory.createModelAndView(\"UMLState\", parent, diagram, options);\n                    break;\n                case TX_SUBMACHINESTATE:\n                    view = Factory.createModelAndView(\"UMLSubmachineState\", parent, diagram, options);\n                    break;\n                case TX_ORTHOGONALSTATE:\n                    options.regionCount = 2;\n                    view = Factory.createModelAndView(\"UMLState\", parent, diagram, options);\n                    break;\n                case TX_INITIALSTATE:\n                    options.pseudostateKind = UML.PSK_INITIAL;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_JUNCTION:\n                    options.pseudostateKind = UML.PSK_JUNCTION;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_CHOICE:\n                    options.pseudostateKind = UML.PSK_CHOICE;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_SHALLOWHISTORY:\n                    options.pseudostateKind = UML.PSK_SHALLOWHISTORY;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_DEEPHISTORY:\n                    options.pseudostateKind = UML.PSK_DEEPHISTORY;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_JOIN:\n                    options.pseudostateKind = UML.PSK_JOIN;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_FORK:\n                    options.pseudostateKind = UML.PSK_FORK;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_ENTRYPOINT:\n                    options.pseudostateKind = UML.PSK_ENTRYPOINT;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_EXITPOINT:\n                    options.pseudostateKind = UML.PSK_EXITPOINT;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_TERMINATE:\n                    options.pseudostateKind = UML.PSK_TERMINATE;\n                    view = Factory.createModelAndView(\"UMLPseudostate\", parent, diagram, options);\n                    break;\n                case TX_FINALSTATE:\n                    view = Factory.createModelAndView(\"UMLFinalState\", parent, diagram, options);\n                    break;\n                case TX_CONNECTIONPOINT:\n                    options.containerView = headView;\n                    view = Factory.createModelAndView(\"UMLConnectionPointReference\", headModel, diagram, options);\n                    break;\n                case TX_TRANSITION:\n                    view = Factory.createModelAndView(\"UMLTransition\", parent, diagram, options);\n                    break;\n                case TX_SELFTRANSITION:\n                    options.tailModel = options.headModel;\n                    options.tailView = options.headView;\n                    view = Factory.createModelAndView(\"UMLTransition\", parent, diagram, options);\n                    break;\n\n                // Activities\n                case TX_ACTION:\n                    view = Factory.createModelAndView(\"UMLAction\", parent, diagram, options);\n                    break;\n                case TX_OBJECTNODE:\n                    view = Factory.createModelAndView(\"UMLObjectNode\", parent, diagram, options);\n                    break;\n                case TX_CENTRALBUFFERNODE:\n                    view = Factory.createModelAndView(\"UMLCentralBufferNode\", parent, diagram, options);\n                    break;\n                case TX_DATASTORENODE:\n                    view = Factory.createModelAndView(\"UMLDataStoreNode\", parent, diagram, options);\n                    break;\n                case TX_INITIALNODE:\n                    view = Factory.createModelAndView(\"UMLInitialNode\", parent, diagram, options);\n                    break;\n                case TX_ACTIVITYFINALNODE:\n                    view = Factory.createModelAndView(\"UMLActivityFinalNode\", parent, diagram, options);\n                    break;\n                case TX_FLOWFINALNODE:\n                    view = Factory.createModelAndView(\"UMLFlowFinalNode\", parent, diagram, options);\n                    break;\n                case TX_FORKNODE:\n                    view = Factory.createModelAndView(\"UMLForkNode\", parent, diagram, options);\n                    break;\n                case TX_JOINNODE:\n                    view = Factory.createModelAndView(\"UMLJoinNode\", parent, diagram, options);\n                    break;\n                case TX_MERGENODE:\n                    view = Factory.createModelAndView(\"UMLMergeNode\", parent, diagram, options);\n                    break;\n                case TX_DECISIONNODE:\n                    view = Factory.createModelAndView(\"UMLDecisionNode\", parent, diagram, options);\n                    break;\n                case TX_INPUTPIN:\n                    options.containerView = headView;\n                    view = Factory.createModelAndView(\"UMLInputPin\", headModel, diagram, options);\n                    break;\n                case TX_OUTPUTPIN:\n                    options.containerView = headView;\n                    view = Factory.createModelAndView(\"UMLOutputPin\", headModel, diagram, options);\n                    break;\n                case TX_INPUTEXPANSIONNODE:\n                    options.containerView = headView;\n                    view = Factory.createModelAndView(\"UMLInputExpansionNode\", headModel, diagram, options);\n                    break;\n                case TX_OUTPUTEXPANSIONNODE:\n                    options.containerView = headView;\n                    view = Factory.createModelAndView(\"UMLOutputExpansionNode\", headModel, diagram, options);\n                    break;\n                case TX_SWIMLANEVERT:\n                    options.viewInitializer = function (v) { v.isVertical = true; };\n                    view = Factory.createModelAndView(\"UMLActivityPartition\", parent, diagram, options);\n                    break;\n                case TX_INTERRUPTIBLEACTIVITYREGION:\n                    view = Factory.createModelAndView(\"UMLInterruptibleActivityRegion\", parent, diagram, options);\n                    break;\n                case TX_STRUCTUREDACTIVITYNODE:\n                    view = Factory.createModelAndView(\"UMLStructuredActivityNode\", parent, diagram, options);\n                    break;\n                case TX_EXPANSIONREGION:\n                    view = Factory.createModelAndView(\"UMLExpansionRegion\", parent, diagram, options);\n                    break;\n                case TX_SWIMLANEHORZ:\n                    options.viewInitializer = function (v) { v.isVertical = false; };\n                    view = Factory.createModelAndView(\"UMLActivityPartition\", parent, diagram, options);\n                    break;\n                case TX_SENDSIGNAL:\n                    options.modelInitializer = function (m) { m.kind = UML.ACK_SENDSIGNAL; };\n                    view = Factory.createModelAndView(\"UMLAction\", parent, diagram, options);\n                    break;\n                case TX_ACCEPTSIGNAL:\n                    options.modelInitializer = function (m) { m.kind = UML.ACK_ACCEPTSIGNAL; };\n                    view = Factory.createModelAndView(\"UMLAction\", parent, diagram, options);\n                    break;\n                case TX_ACCEPTTIMEEVENT:\n                    options.modelInitializer = function (m) { m.kind = UML.ACK_TIMEEVENT; };\n                    view = Factory.createModelAndView(\"UMLAction\", parent, diagram, options);\n                    break;\n                case TX_CONTROLFLOW:\n                    view = Factory.createModelAndView(\"UMLControlFlow\", parent, diagram, options);\n                    break;\n                case TX_OBJECTFLOW:\n                    view = Factory.createModelAndView(\"UMLObjectFlow\", parent, diagram, options);\n                    break;\n                case TX_EXCEPTIONHANDLER:\n                    view = Factory.createModelAndView(\"UMLExceptionHandler\", parent, diagram, options);\n                    break;\n                case TX_ACTIVITYINTERRUPT:\n                    view = Factory.createModelAndView(\"UMLActivityInterrupt\", parent, diagram, options);\n                    break;\n\n                // Profiles\n                case TX_METACLASS:\n                    view = Factory.createModelAndView(\"UMLMetaClass\", parent, diagram, options);\n                    break;\n                case TX_STEREOTYPE:\n                    view = Factory.createModelAndView(\"UMLStereotype\", parent, diagram, options);\n                    break;\n                case TX_EXTENSION:\n                    view = Factory.createModelAndView(\"UMLExtension\", tailModel, diagram, options);\n                    break;\n                }\n\n                // Open QuickEdit for the created view.\n                if (view) {\n                    view = Repository.get(view._id);\n                    if (view instanceof type.NodeView) {\n                        _.defer(function () {\n                            QuickEdit.open(editor, view, view.left, view.top);\n                        });\n                    } else if (view instanceof type.EdgeView) {\n                        _.defer(function () {\n                            QuickEdit.open(editor, view, Math.round((x1 + x2) / 2), Math.round((y1 + y2) / 2));\n                        });\n                    }\n                }\n\n            } catch (err) {\n                if (_.isString(err)) {\n                    Toast.error(err);\n                } else {\n                    console.error(err.stack);\n                }\n            }\n        });\n    }\n\n    function setupDiagramManager() {\n        $(DiagramManager).on('currentDiagramChanged', function (event, diagram, editor) {\n            try {\n                if (diagram) {\n                    if (diagram instanceof type.UMLUseCaseDiagram) {\n                        Toolbox.expandGroup(TXG_USECASE);\n                    } else if (diagram instanceof type.UMLClassDiagram) {\n                        Toolbox.expandGroup(TXG_CLASS);\n                    } else if (diagram instanceof type.UMLCompositeStructureDiagram) {\n                        Toolbox.expandGroup(TXG_CLASS);\n                        Toolbox.expandGroup(TXG_COMPOSITESTRUCTURE);\n                    } else if (diagram instanceof type.UMLPackageDiagram) {\n                        Toolbox.expandGroup(TXG_PACKAGE);\n                    } else if (diagram instanceof type.UMLObjectDiagram) {\n                        Toolbox.expandGroup(TXG_INSTANCE);\n                    } else if (diagram instanceof type.UMLSequenceDiagram) {\n                        Toolbox.expandGroup(TXG_SEQUENCE);\n                    } else if (diagram instanceof type.UMLCommunicationDiagram) {\n                        Toolbox.expandGroup(TXG_COMMUNICATION);\n                    } else if (diagram instanceof type.UMLStatechartDiagram) {\n                        Toolbox.expandGroup(TXG_STATECHART);\n                    } else if (diagram instanceof type.UMLActivityDiagram) {\n                        Toolbox.expandGroup(TXG_ACTIVITY);\n                    } else if (diagram instanceof type.UMLComponentDiagram) {\n                        Toolbox.expandGroup(TXG_COMPONENT);\n                        Toolbox.expandGroup(TXG_DEPLOYMENT);\n                    } else if (diagram instanceof type.UMLDeploymentDiagram) {\n                        Toolbox.expandGroup(TXG_COMPONENT);\n                        Toolbox.expandGroup(TXG_DEPLOYMENT);\n                    } else if (diagram instanceof type.UMLProfileDiagram) {\n                        Toolbox.expandGroup(TXG_PROFILE);\n                    }\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n\n    AppInit.htmlReady(function () {\n        setupToolbox();\n        setupDiagramManager();\n    });\n\n});\n\n//# sourceURL=/uml/UMLToolbox.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell */\ndefine(\'uml/UMLMenus\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Global\',\'utils/Helper\',\'command/Commands\',\'command/CommandManager\',\'menu/MenuManager\',\'menu/ContextMenuManager\',\'menu/DefaultMenus\',\'strings\',\'uml/UMLCommands\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit            = require("utils/AppInit"),\n        global             = require("core/Global").global,\n        Helper             = require("utils/Helper"),\n        Commands           = require("command/Commands"),\n        CommandManager     = require("command/CommandManager"),\n        MenuManager        = require("menu/MenuManager"),\n        ContextMenuManager = require("menu/ContextMenuManager"),\n        DefaultMenus       = require("menu/DefaultMenus"),\n        Strings            = require("strings"),\n        UMLCommands        = require("uml/UMLCommands");\n\n    /**\n     * Add context menu items to add an UML element.\n     */\n    function _addElementMenus(menu) {\n        // Model: Features\n        menu.addMenuItem(UMLCommands.MODEL_ADD_TEMPLATEPARAMETER, "icon-UMLTemplateParameter");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_PARAMETER, "icon-UMLParameter");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ENUMERATIONLITERAL, "icon-UMLEnumerationLiteral");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ATTRIBUTE, "icon-UMLAttribute");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_PORT, "icon-UMLPort");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_OPERATION, "icon-UMLOperation");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_RECEPTION, "icon-UMLReception");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_EXTENSIONPOINT, "icon-UMLExtensionPoint");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_SLOT, "icon-UMLSlot");\n        // Model: Packages\n        menu.addMenuItem(UMLCommands.MODEL_ADD_MODEL, "icon-UMLModel");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_SUBSYSTEM, "icon-UMLSubsystem");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_PACKAGE, "icon-UMLPackage");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_PROFILE, "icon-UMLProfile");\n        // Model: Classes\n        menu.addMenuItem(UMLCommands.MODEL_ADD_CLASS, "icon-UMLClass");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_INTERFACE, "icon-UMLInterface");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_SIGNAL, "icon-UMLSignal");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DATATYPE, "icon-UMLDataType");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_PRIMITIVETYPE, "icon-UMLPrimitiveType");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ENUMERATION, "icon-UMLEnumeration");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ARTIFACT, "icon-UMLArtifact");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_COMPONENT, "icon-UMLComponent");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_NODE, "icon-UMLNode");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_USECASE, "icon-UMLUseCase");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ACTOR, "icon-UMLActor");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_STEREOTYPE, "icon-UMLStereotype");\n        // Model: Instances\n        menu.addMenuItem(UMLCommands.MODEL_ADD_OBJECT, "icon-UMLObject");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ARTIFACTINSTANCE, "icon-UMLArtifactInstance");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_COMPONENTINSTANCE, "icon-UMLComponentInstance");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_NODEINSTANCE, "icon-UMLNodeInstance");\n        // Model: Behaviors\n        menu.addMenuItem(UMLCommands.MODEL_ADD_COLLABORATION, "icon-UMLCollaboration");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_INTERACTION, "icon-UMLInteraction");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_STATEMACHINE, "icon-UMLStateMachine");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ACTIVITY, "icon-UMLActivity");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_OPAQUEBEHAVIOR, "icon-UMLOpaqueBehavior");\n        // Model: States\n        menu.addMenuItem(UMLCommands.MODEL_ADD_STATE, "icon-UMLState");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_REGION, "icon-UMLRegion");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ENTRY_ACTIVITY, "icon-UMLEntryActivity");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DO_ACTIVITY, "icon-UMLDoActivity");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_EXIT_ACTIVITY, "icon-UMLExitActivity");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_TRIGGER, "icon-UMLEvent");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_EFFECT,  "icon-UMLEffect");\n        // Model: Actions\n        menu.addMenuItem(UMLCommands.MODEL_ADD_ACTION, "icon-UMLAction");\n        // Model: Common\n        menu.addMenuItem(UMLCommands.MODEL_ADD_CONSTRAINT, "icon-UMLConstraint");\n    }\n\n    /**\n     * Add context menu items to add an UML diagram.\n     */\n    function _addDiagramMenus(menu) {\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_CLASS, "icon-UMLClassDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_PACKAGE, "icon-UMLPackageDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_OBJECT, "icon-UMLObjectDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_COMPOSITESTRUCTURE, "icon-UMLCompositeStructureDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_COMPONENT, "icon-UMLComponentDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_DEPLOYMENT, "icon-UMLDeploymentDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_USECASE, "icon-UMLUseCaseDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_SEQUENCE, "icon-UMLSequenceDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_COMMUNICATION, "icon-UMLCommunicationDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_STATECHART, "icon-UMLStatechartDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_ACTIVITY, "icon-UMLActivityDiagram");\n        menu.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_PROFILE, "icon-UMLProfileDiagram");\n    }\n\n\n    function setupMenus() {\n        var menu,\n            menuItem;\n\n        // File: New From Template\n        menu = MenuManager.getMenuItem(Commands.FILE_NEW_FROM_TEMPLATE);\n        menu.addMenuItem(UMLCommands.FILE_NEW_FROM_TEMPLATE_UML_MINIMAL);\n        menu.addMenuItem(UMLCommands.FILE_NEW_FROM_TEMPLATE_UML_CONVENTIONAL);\n        menu.addMenuItem(UMLCommands.FILE_NEW_FROM_TEMPLATE_4P1VIEWMODEL);\n        menu.addMenuItem(UMLCommands.FILE_NEW_FROM_TEMPLATE_RATIONAL);\n\n        // Model\n        menu = MenuManager.getMenu(Commands.MODEL);\n        // Model: Features\n        menuItem = MenuManager.getMenuItem(Commands.MODEL_ADD);\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_TEMPLATEPARAMETER, null, null, null, "icon-UMLTemplateParameter");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_PARAMETER, null, null, null, "icon-UMLParameter");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ENUMERATIONLITERAL, null, null, null, "icon-UMLEnumerationLiteral");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ATTRIBUTE, null, null, null, "icon-UMLAttribute");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_PORT, null, null, null, "icon-UMLPort");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_OPERATION, null, null, null, "icon-UMLOperation");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_RECEPTION, null, null, null, "icon-UMLReception");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_EXTENSIONPOINT, null, null, null, "icon-UMLExtensionPoint");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_SLOT, null, null, null, "icon-UMLSlot");\n        // Model: Packages\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_MODEL, null, null, null, "icon-UMLModel");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_SUBSYSTEM, null, null, null, "icon-UMLSubsystem");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_PACKAGE, null, null, null, "icon-UMLPackage");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_PROFILE, null, null, null, "icon-UMLProfile");\n        // Model: Classes\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_CLASS, null, null, null, "icon-UMLClass");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_INTERFACE, null, null, null, "icon-UMLInterface");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_SIGNAL, null, null, null, "icon-UMLSignal");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DATATYPE, null, null, null, "icon-UMLDataType");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_PRIMITIVETYPE, null, null, null, "icon-UMLPrimitiveType");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ENUMERATION, null, null, null, "icon-UMLEnumeration");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ARTIFACT, null, null, null, "icon-UMLArtifact");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_COMPONENT, null, null, null, "icon-UMLComponent");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_NODE, null, null, null, "icon-UMLNode");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_USECASE, null, null, null, "icon-UMLUseCase");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ACTOR, null, null, null, "icon-UMLActor");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_STEREOTYPE, null, null, null, "icon-UMLStereotype");\n        // Model: Instances\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_OBJECT, null, null, null, "icon-UMLObject");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ARTIFACTINSTANCE, null, null, null, "icon-UMLArtifactInstance");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_COMPONENTINSTANCE, null, null, null, "icon-UMLComponentInstance");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_NODEINSTANCE, null, null, null, "icon-UMLNodeInstance");\n        // Model: Behaviors\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_COLLABORATION, null, null, null, "icon-UMLCollaboration");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_INTERACTION, null, null, null, "icon-UMLInteraction");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_STATEMACHINE, null, null, null, "icon-UMLStateMachine");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ACTIVITY, null, null, null, "icon-UMLActivity");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_OPAQUEBEHAVIOR, null, null, null, "icon-UMLOpaqueBehavior");\n        // Model: States\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_STATE, null, null, null, "icon-UMLState");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_REGION, null, null, null, "icon-UMLRegion");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ENTRY_ACTIVITY, null, null, null, "icon-UMLEntryActivity");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DO_ACTIVITY, null, null, null, "icon-UMLDoActivity");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_EXIT_ACTIVITY, null, null, null, "icon-UMLExitActivity");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_TRIGGER, null, null, null, "icon-UMLEvent");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_EFFECT, null, null, null, "icon-UMLEffect");\n        // Model: Actions\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_ACTION, null, null, null, "icon-UMLAction");\n        // Model: Common\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_CONSTRAINT, null, null, null, "icon-UMLConstraint");\n        // Model: Diagrams\n        menuItem = MenuManager.getMenuItem(Commands.MODEL_ADD_DIAGRAM);\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_CLASS, null, null, null, "icon-UMLClassDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_PACKAGE, null, null, null, "icon-UMLPackageDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_OBJECT, null, null, null, "icon-UMLObjectDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_COMPOSITESTRUCTURE, null, null, null, "icon-UMLCompositeStructureDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_COMPONENT, null, null, null, "icon-UMLComponentDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_DEPLOYMENT, null, null, null, "icon-UMLDeploymentDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_USECASE, null, null, null, "icon-UMLUseCaseDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_SEQUENCE, null, null, null, "icon-UMLSequenceDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_COMMUNICATION, null, null, null, "icon-UMLCommunicationDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_STATECHART, null, null, null, "icon-UMLStatechartDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_ACTIVITY, null, null, null, "icon-UMLActivityDiagram");\n        menuItem.addMenuItem(UMLCommands.MODEL_ADD_DIAGRAM_PROFILE, null, null, null, "icon-UMLProfileDiagram");\n\n        // Model: Profiles\n        menu.addMenuDivider();\n        menuItem = menu.addMenuItem(UMLCommands.MODEL_APPLY_PROFILE);\n        menuItem.addMenuItem(UMLCommands.MODEL_APPLY_PROFILE_UML_STANDARD);\n\n        // Format\n        menu = MenuManager.getMenu(Commands.FORMAT);\n        menu.addMenuDivider();\n        menuItem = menu.addMenuItem(UMLCommands.FORMAT_STEREOTYPE);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_NONE);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_LABEL);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_DECORATION);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_DECORATION_LABEL);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_ICON);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_ICON_LABEL);\n        menu.addMenuItem(UMLCommands.FORMAT_WORD_WRAP);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_VISIBILITY);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_NAMESPACE);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_PROPERTY);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_TYPE);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_MULTIPLICITY);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_OPERATION_SIGNATURE);\n        menu.addMenuDivider();\n        menu.addMenuItem(UMLCommands.FORMAT_SUPPRESS_ATTRIBUTES);\n        menu.addMenuItem(UMLCommands.FORMAT_SUPPRESS_OPERATIONS);\n        menu.addMenuItem(UMLCommands.FORMAT_SUPPRESS_RECEPTIONS);\n        menu.addMenuItem(UMLCommands.FORMAT_SUPPRESS_LITERALS);\n    }\n\n    /**\n     * Extend ContextMenu for Diagram\n     */\n    function setupContextMenuForDiagram() {\n        var contextMenu, menu, menuItem;\n        contextMenu = ContextMenuManager.getContextMenu(DefaultMenus.contextMenus.DIAGRAM);\n\n        // Model Add\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD);\n        _addElementMenus(menu);\n\n        // Format\n        menu = contextMenu.getMenuItem(Commands.FORMAT);\n        menu.addMenuDivider();\n        menuItem = menu.addMenuItem(UMLCommands.FORMAT_STEREOTYPE);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_NONE);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_LABEL);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_DECORATION);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_DECORATION_LABEL);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_ICON);\n        menuItem.addMenuItem(UMLCommands.FORMAT_STEREOTYPE_ICON_LABEL);\n        menu.addMenuItem(UMLCommands.FORMAT_WORD_WRAP);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_VISIBILITY);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_NAMESPACE);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_PROPERTY);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_TYPE);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_MULTIPLICITY);\n        menu.addMenuItem(UMLCommands.FORMAT_SHOW_OPERATION_SIGNATURE);\n        menu.addMenuDivider();\n        menu.addMenuItem(UMLCommands.FORMAT_SUPPRESS_ATTRIBUTES);\n        menu.addMenuItem(UMLCommands.FORMAT_SUPPRESS_OPERATIONS);\n        menu.addMenuItem(UMLCommands.FORMAT_SUPPRESS_RECEPTIONS);\n        menu.addMenuItem(UMLCommands.FORMAT_SUPPRESS_LITERALS);\n    }\n\n    /**\n     * Extend ContextMenu for Explorer\n     */\n    function setupContextMenuForExplorer() {\n        var contextMenu, menu;\n        contextMenu = ContextMenuManager.getContextMenu(DefaultMenus.contextMenus.EXPLORER);\n\n        // Model Add Diagram\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD_DIAGRAM);\n        _addDiagramMenus(menu);\n\n        // Model Add\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD);\n        _addElementMenus(menu);\n    }\n\n    AppInit.htmlReady(function () {\n        setupMenus();\n        setupContextMenuForDiagram();\n        setupContextMenuForExplorer();\n    });\n\n});\n\n//# sourceURL=/uml/UMLMenus.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, staruml */\n\ndefine(\'uml/UMLPreferences\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Core\',\'uml/UML\',\'core/PreferenceManager\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit           = require("utils/AppInit"),\n        Core              = require("core/Core"),\n        UML               = require("uml/UML"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    var UMLPreferences = {\n        "uml.package": {\n            text: "Package",\n            type: "Section"\n        },\n        "uml.package.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Package.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.model": {\n            text: "Model",\n            type: "Section"\n        },\n        "uml.model.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Model.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.subsystem": {\n            text: "Subsystem",\n            type: "Section"\n        },\n        "uml.subsystem.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Subsystem.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.class": {\n            text: "Class",\n            type: "Section"\n        },\n        "uml.class.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Class.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.class.stereotypeDisplay": {\n            text: "Stereotype Display",\n            description: "Default stereotype display of Class.",\n            type: "Dropdown",\n            options: [\n                { value: UML.SD_NONE,             text: "None" },\n                { value: UML.SD_LABEL,            text: "Label" },\n                { value: UML.SD_DECORATION,       text: "Decoration" },\n                { value: UML.SD_DECORATION_LABEL, text: "Decoration with Label" },\n                { value: UML.SD_ICON,             text: "Icon" },\n                { value: UML.SD_ICON_LABEL,       text: "Icon with Label" }\n            ],\n            default: UML.SD_LABEL\n        },\n        "uml.class.suppressAttributes": {\n            text: "Suppress Attributes",\n            description: "Suppress attributes of Class.",\n            type: "Check",\n            default: false\n        },\n        "uml.class.suppressOperations": {\n            text: "Suppress Operations",\n            description: "Suppress operations of Class.",\n            type: "Check",\n            default: false\n        },\n        "uml.interface": {\n            text: "Interface",\n            type: "Section"\n        },\n        "uml.interface.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Interface.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.interface.stereotypeDisplay": {\n            text: "Stereotype Display",\n            description: "Default stereotype display of Interface.",\n            type: "Dropdown",\n            options: [\n                { value: UML.SD_NONE,             text: "None" },\n                { value: UML.SD_LABEL,            text: "Label" },\n                { value: UML.SD_DECORATION,       text: "Decoration" },\n                { value: UML.SD_DECORATION_LABEL, text: "Decoration with Label" },\n                { value: UML.SD_ICON,             text: "Icon" },\n                { value: UML.SD_ICON_LABEL,       text: "Icon with Label" }\n            ],\n            default: UML.SD_ICON\n        },\n        "uml.interface.suppressAttributes": {\n            text: "Suppress Attributes",\n            description: "Suppress attributes of Interface.",\n            type: "Check",\n            default: true\n        },\n        "uml.interface.suppressOperations": {\n            text: "Suppress Operations",\n            description: "Suppress operations of Interface.",\n            type: "Check",\n            default: true\n        },\n        "uml.signal": {\n            text: "Signal",\n            type: "Section"\n        },\n        "uml.signal.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Signal.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.datatype": {\n            text: "DataType",\n            type: "Section"\n        },\n        "uml.datatype.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of DataType.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.primitivetype": {\n            text: "Primitive Type",\n            type: "Section"\n        },\n        "uml.primitivetype.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Primitive Type.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.enumeration": {\n            text: "Enumeration",\n            type: "Section"\n        },\n        "uml.enumeration.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Enumeration.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.enumeration.stereotypeDisplay": {\n            text: "Stereotype Display",\n            description: "Default stereotype display of Enumeration.",\n            type: "Dropdown",\n            options: [\n                { value: UML.SD_NONE,             text: "None" },\n                { value: UML.SD_LABEL,            text: "Label" },\n                { value: UML.SD_DECORATION,       text: "Decoration" },\n                { value: UML.SD_DECORATION_LABEL, text: "Decoration with Label" },\n                { value: UML.SD_ICON,             text: "Icon" },\n                { value: UML.SD_ICON_LABEL,       text: "Icon with Label" }\n            ],\n            default: UML.SD_LABEL\n        },\n        "uml.enumeration.suppressAttributes": {\n            text: "Suppress Attributes",\n            description: "Suppress attributes of Enumeration.",\n            type: "Check",\n            default: true\n        },\n        "uml.enumeration.suppressOperations": {\n            text: "Suppress Operations",\n            description: "Suppress operations of Enumeration.",\n            type: "Check",\n            default: true\n        },\n        "uml.enumeration.suppressLiterals": {\n            text: "Suppress Literals",\n            description: "Suppress literals of Enumeration.",\n            type: "Check",\n            default: false\n        },\n        "uml.port": {\n            text: "Port",\n            type: "Section"\n        },\n        "uml.port.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Port.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.part": {\n            text: "Part",\n            type: "Section"\n        },\n        "uml.part.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Part.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.collaboration": {\n            text: "Collaboration",\n            type: "Section"\n        },\n        "uml.collaboration.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Collaboration.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.collaborationuse": {\n            text: "Collaboration Use",\n            type: "Section"\n        },\n        "uml.collaborationuse.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Collaboration Use.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.artifact": {\n            text: "Artifact",\n            type: "Section"\n        },\n        "uml.artifact.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Artifact.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.artifact.stereotypeDisplay": {\n            text: "Stereotype Display",\n            description: "Default stereotype display of Artifact.",\n            type: "Dropdown",\n            options: [\n                { value: UML.SD_NONE,             text: "None" },\n                { value: UML.SD_LABEL,            text: "Label" },\n                { value: UML.SD_DECORATION,       text: "Decoration" },\n                { value: UML.SD_DECORATION_LABEL, text: "Decoration with Label" },\n                { value: UML.SD_ICON,             text: "Icon" },\n                { value: UML.SD_ICON_LABEL,       text: "Icon with Label" }\n            ],\n            default: UML.SD_ICON\n        },\n        "uml.artifact.suppressAttributes": {\n            text: "Suppress Attributes",\n            description: "Suppress attributes of Artifact.",\n            type: "Check",\n            default: true\n        },\n        "uml.artifact.suppressOperations": {\n            text: "Suppress Operations",\n            description: "Suppress operations of Artifact.",\n            type: "Check",\n            default: true\n        },\n        "uml.component": {\n            text: "Component",\n            type: "Section"\n        },\n        "uml.component.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Component.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.component.stereotypeDisplay": {\n            text: "Stereotype Display",\n            description: "Default stereotype display of Component.",\n            type: "Dropdown",\n            options: [\n                { value: UML.SD_NONE,             text: "None" },\n                { value: UML.SD_LABEL,            text: "Label" },\n                { value: UML.SD_DECORATION,       text: "Decoration" },\n                { value: UML.SD_DECORATION_LABEL, text: "Decoration with Label" },\n                { value: UML.SD_ICON,             text: "Icon" },\n                { value: UML.SD_ICON_LABEL,       text: "Icon with Label" }\n            ],\n            default: UML.SD_LABEL\n        },\n        "uml.component.suppressAttributes": {\n            text: "Suppress Attributes",\n            description: "Suppress attributes of Component.",\n            type: "Check",\n            default: true\n        },\n        "uml.component.suppressOperations": {\n            text: "Suppress Operations",\n            description: "Suppress operations of Component.",\n            type: "Check",\n            default: true\n        },\n        "uml.node": {\n            text: "Node",\n            type: "Section"\n        },\n        "uml.node.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Node.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.node.stereotypeDisplay": {\n            text: "Stereotype Display",\n            description: "Default stereotype display of Node.",\n            type: "Dropdown",\n            options: [\n                { value: UML.SD_NONE,             text: "None" },\n                { value: UML.SD_LABEL,            text: "Label" },\n                { value: UML.SD_DECORATION,       text: "Decoration" },\n                { value: UML.SD_DECORATION_LABEL, text: "Decoration with Label" },\n                { value: UML.SD_ICON,             text: "Icon" },\n                { value: UML.SD_ICON_LABEL,       text: "Icon with Label" }\n            ],\n            default: UML.SD_LABEL\n        },\n        "uml.node.suppressAttributes": {\n            text: "Suppress Attributes",\n            description: "Suppress attributes of Node.",\n            type: "Check",\n            default: true\n        },\n        "uml.node.suppressOperations": {\n            text: "Suppress Operations",\n            description: "Suppress operations of Node.",\n            type: "Check",\n            default: true\n        },\n        "uml.object": {\n            text: "Object",\n            type: "Section"\n        },\n        "uml.object.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Object.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.artifactinstance": {\n            text: "Artifact Instance",\n            type: "Section"\n        },\n        "uml.artifactinstance.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Artifact Instance.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.componentinstance": {\n            text: "Component Instance",\n            type: "Section"\n        },\n        "uml.componentinstance.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Component Instance.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.nodeinstance": {\n            text: "Node Instance",\n            type: "Section"\n        },\n        "uml.nodeinstance.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Node Instance.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.usecase": {\n            text: "Use Case",\n            type: "Section"\n        },\n        "uml.usecase.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Use Case.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.usecase.stereotypeDisplay": {\n            text: "Stereotype Display",\n            description: "Default stereotype display of Use Case.",\n            type: "Dropdown",\n            options: [\n                { value: UML.SD_NONE,             text: "None" },\n                { value: UML.SD_LABEL,            text: "Label" },\n                { value: UML.SD_DECORATION,       text: "Decoration" },\n                { value: UML.SD_DECORATION_LABEL, text: "Decoration with Label" },\n                { value: UML.SD_ICON,             text: "Icon" },\n                { value: UML.SD_ICON_LABEL,       text: "Icon with Label" }\n            ],\n            default: UML.SD_LABEL\n        },\n        "uml.usecase.suppressAttributes": {\n            text: "Suppress Attributes",\n            description: "Suppress attributes of Use Case.",\n            type: "Check",\n            default: true\n        },\n        "uml.usecase.suppressOperations": {\n            text: "Suppress Operations",\n            description: "Suppress operations of Use Case.",\n            type: "Check",\n            default: true\n        },\n        "uml.actor": {\n            text: "Actor",\n            type: "Section"\n        },\n        "uml.actor.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Actor.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.actor.stereotypeDisplay": {\n            text: "Stereotype Display",\n            description: "Default stereotype display of Actor.",\n            type: "Dropdown",\n            options: [\n                { value: UML.SD_NONE,             text: "None" },\n                { value: UML.SD_LABEL,            text: "Label" },\n                { value: UML.SD_DECORATION,       text: "Decoration" },\n                { value: UML.SD_DECORATION_LABEL, text: "Decoration with Label" },\n                { value: UML.SD_ICON,             text: "Icon" },\n                { value: UML.SD_ICON_LABEL,       text: "Icon with Label" }\n            ],\n            default: UML.SD_LABEL\n        },\n        "uml.actor.suppressAttributes": {\n            text: "Suppress Attributes",\n            description: "Suppress attributes of Actor.",\n            type: "Check",\n            default: true\n        },\n        "uml.actor.suppressOperations": {\n            text: "Suppress Operations",\n            description: "Suppress operations of Actor.",\n            type: "Check",\n            default: true\n        },\n        "uml.lifeline": {\n            text: "Lifeline",\n            type: "Section"\n        },\n        "uml.lifeline.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Lifeline.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.state": {\n            text: "State",\n            type: "Section"\n        },\n        "uml.state.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of State.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.action": {\n            text: "Action",\n            type: "Section"\n        },\n        "uml.action.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Action.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.objectnode": {\n            text: "Object Node",\n            type: "Section"\n        },\n        "uml.objectnode.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Object Node.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.metaclass": {\n            text: "MetaClass",\n            type: "Section"\n        },\n        "uml.metaclass.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of MetaClass.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.stereotype": {\n            text: "Stereotype",\n            type: "Section"\n        },\n        "uml.stereotype.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Stereotype.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "uml.note": {\n            text: "Note",\n            type: "Section"\n        },\n        "uml.note.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Note.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n    };\n\n    AppInit.htmlReady(function () {\n        PreferenceManager.register("uml", "UML", UMLPreferences);\n    });\n\n});\n\n//# sourceURL=/uml/UMLPreferences.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n// TODO: Separate UMLToolbox.js\n// TODO: Separate UMLQuickEdit.js\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, type */\n\ndefine(\'uml/main\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Global\',\'utils/Helper\',\'core/Graphics\',\'core/Core\',\'file/FileUtils\',\'core/OperationBuilder\',\'core/Repository\',\'engine/SelectionManager\',\'engine/Engine\',\'engine/Factory\',\'engine/ClipboardManager\',\'engine/ProjectManager\',\'core/PreferenceManager\',\'command/Commands\',\'command/CommandManager\',\'utils/KeyEvent\',\'diagrams/DiagramManager\',\'diagrams/ToolboxView\',\'ui/QuickEdit\',\'explorer/ModelExplorerView\',\'dialogs/Dialogs\',\'ui/Toast\',\'dialogs/ElementPickerDialog\',\'dialogs/ElementListPickerDialog\',\'uml/UMLCommands\',\'uml/UML\',\'uml/UMLDiagram\',\'uml/UMLFactory\',\'uml/UMLQuickEdit\',\'uml/UMLUtils\',\'uml/UML-meta\',\'uml/UML-rules\',\'uml/UMLCommandHandlers\',\'uml/UMLToolbox\',\'uml/UMLMenus\',\'uml/UMLPreferences\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit                 = require("utils/AppInit"),\n        global                  = require("core/Global").global,\n        Helper                  = require("utils/Helper"),\n        Graphics                = require("core/Graphics"),\n        Core                    = require("core/Core"),\n        FileUtils               = require("file/FileUtils"),\n        OperationBuilder        = require("core/OperationBuilder"),\n        Repository              = require("core/Repository"),\n        SelectionManager        = require("engine/SelectionManager"),\n        Engine                  = require("engine/Engine"),\n        Factory                 = require("engine/Factory"),\n        ClipboardManager        = require("engine/ClipboardManager"),\n        ProjectManager          = require("engine/ProjectManager"),\n        PreferenceManager       = require("core/PreferenceManager"),\n        Commands                = require("command/Commands"),\n        CommandManager          = require("command/CommandManager"),\n        KeyEvent                = require("utils/KeyEvent"),\n        DiagramManager          = require("diagrams/DiagramManager"),\n        Toolbox                 = require("diagrams/ToolboxView"),\n        QuickEdit               = require("ui/QuickEdit"),\n        ModelExplorerView       = require("explorer/ModelExplorerView"),\n        Dialogs                 = require("dialogs/Dialogs"),\n        Toast                   = require("ui/Toast"),\n        ElementPickerDialog     = require("dialogs/ElementPickerDialog"),\n        ElementListPickerDialog = require("dialogs/ElementListPickerDialog"),\n        UMLCommands             = require("uml/UMLCommands"),\n        UML                     = require("uml/UML"),\n        UMLDiagram              = require("uml/UMLDiagram"),\n        UMLFactory              = require("uml/UMLFactory"),\n        UMLQuickEdit            = require("uml/UMLQuickEdit"),\n        UMLUtils                = require("uml/UMLUtils");\n\n    require("uml/UML-meta");\n    require("uml/UML-rules");\n    require("uml/UMLCommandHandlers");\n    require("uml/UMLToolbox");\n    require("uml/UMLMenus");\n    require("uml/UMLPreferences");\n\n    // --- QuickEdit Actions ---\n\n    function _ensure(elem, field, value) {\n        if (elem[field] !== value) {\n            Engine.setProperty(elem, field, value);\n        }\n    }\n\n    function _setNameExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseNameExpression(expression);\n            UMLUtils.setNameExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function _setAttributeExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseAttributeExpression(expression);\n            UMLUtils.setAttributeExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function _setOperationExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseOperationExpression(expression);\n            UMLUtils.setOperationExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function _setTemplateParameterExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseTemplateParameterExpression(expression);\n            UMLUtils.setTemplateParameterExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function _setObjectExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseObjectExpression(expression);\n            UMLUtils.setObjectExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function _setSlotExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseSlotExpression(expression);\n            UMLUtils.setSlotExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function _setLifelineExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseLifelineExpression(expression);\n            UMLUtils.setLifelineExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function _setMessageExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseMessageExpression(expression);\n            UMLUtils.setMessageExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function _setTransitionExpression(model, expression, feedback) {\n        try {\n            var ast = UMLUtils.parseTransitionExpression(expression);\n            UMLUtils.setTransitionExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function setupQuickEdit() {\n        // QuickEdit Value Changed Event Handlers\n        $(QuickEdit).on(\'change\', function (event, id, itemId, view, value, feedback) {\n            try {\n                // Items for Specific QuickEdit\n                switch (id) {\n                case UMLQuickEdit.QUICKEDIT_GENERAL:\n                case UMLQuickEdit.QUICKEDIT_PACKAGE:\n                case UMLQuickEdit.QUICKEDIT_SUBSYSTEM:\n                case UMLQuickEdit.QUICKEDIT_CLASSIFIER:\n                case UMLQuickEdit.QUICKEDIT_CLASS:\n                case UMLQuickEdit.QUICKEDIT_INTERFACE:\n                case UMLQuickEdit.QUICKEDIT_COMPONENT:\n                case UMLQuickEdit.QUICKEDIT_NODE:\n                case UMLQuickEdit.QUICKEDIT_ACTOR:\n                case UMLQuickEdit.QUICKEDIT_USECASE:\n                case UMLQuickEdit.QUICKEDIT_ENUMERATION:\n                case UMLQuickEdit.QUICKEDIT_METACLASS:\n                case UMLQuickEdit.QUICKEDIT_STEREOTYPE:\n                case UMLQuickEdit.QUICKEDIT_RECEPTION:\n                case UMLQuickEdit.QUICKEDIT_OBJECT_NODE:\n                case UMLQuickEdit.QUICKEDIT_INTERNAL_TRANSITION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT: _setNameExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_FLOATINGNODE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT: if (view instanceof Core.NodeLabelView) { _setNameExpression(view._parent.model, value, feedback); } break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ATTRIBUTE:\n                case UMLQuickEdit.QUICKEDIT_PART:\n                case UMLQuickEdit.QUICKEDIT_PORT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setAttributeExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value);  break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_OPERATION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setOperationExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value);  break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_TEMPLATE_PARAMETER:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setTemplateParameterExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value);          break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_QUALIFIER:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setAttributeExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value);  break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_LITERAL:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setNameExpression(view.model, value, feedback);     break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_EXTENSION_POINT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setNameExpression(view.model, value, feedback);     break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_OBJECT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setObjectExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value);  break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_SLOT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setSlotExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_SEQ_LIFELINE:\n                case UMLQuickEdit.QUICKEDIT_COMM_LIFELINE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setLifelineExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: UMLUtils.setVisibility(view.model, value);  break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_SEQ_MESSAGE:\n                case UMLQuickEdit.QUICKEDIT_COMM_MESSAGE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setMessageExpression(view.model, value, feedback); break;\n                    case UMLQuickEdit.ITEM_MESSAGE_SORT: Engine.setProperty(view.model, \'messageSort\', value); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_STATEINVARIANT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:\n                        Engine.setProperty(view.model, \'invariant\', value);\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_CONTINUATION:\n                case UMLQuickEdit.QUICKEDIT_COMBINEDFRAGMENT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setNameExpression(view.model, value, feedback);     break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_OPERAND:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:\n                        Engine.setProperty(view.model, \'guard\', value);\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_DIRECTED:\n                case UMLQuickEdit.QUICKEDIT_CONNECTOR:\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION:\n                case UMLQuickEdit.QUICKEDIT_UNDIRECTED:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       if (view instanceof Core.EdgeLabelView) { _setNameExpression(view._parent.model, value, feedback); } break;\n                    case UMLQuickEdit.ITEM_VISIBILITY: if (view instanceof Core.EdgeLabelView) { UMLUtils.setVisibility(view._parent.model, value); } break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_UNDIRECTED_TAIL:\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION_TAIL:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:         if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end1, \'name\', value); } break;\n                    case UMLQuickEdit.ITEM_VISIBILITY:   if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end1, \'visibility\', value); } break;\n                    case UMLQuickEdit.ITEM_AGGREGATION:  if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end1, \'aggregation\', value); } break;\n                    case UMLQuickEdit.ITEM_NAVIGABILITY: if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end1, \'navigable\', value); } break;\n                    case UMLQuickEdit.ITEM_MULTIPLICITY: if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end1, \'multiplicity\', value); } break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_UNDIRECTED_HEAD:\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION_HEAD:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:         if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end2, \'name\', value); } break;\n                    case UMLQuickEdit.ITEM_VISIBILITY:   if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end2, \'visibility\', value); } break;\n                    case UMLQuickEdit.ITEM_AGGREGATION:  if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end2, \'aggregation\', value); } break;\n                    case UMLQuickEdit.ITEM_NAVIGABILITY: if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end2, \'navigable\', value); } break;\n                    case UMLQuickEdit.ITEM_MULTIPLICITY: if (view instanceof Core.EdgeLabelView) { Engine.setProperty(view._parent.model.end2, \'multiplicity\', value); } break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_STATE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT: _setNameExpression(view.model, value, feedback); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_INTERNAL_ACTIVITY:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT: _setNameExpression(view.model, value, feedback); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_TRANSITION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:\n                        if (view instanceof Core.EdgeLabelView) {\n                            _setTransitionExpression(view._parent.model, value, feedback);\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ACTION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT: _setNameExpression(view.model, value, feedback); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ACTIVITYEDGE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT: if (view instanceof Core.EdgeLabelView) { _setNameExpression(view._parent.model, value, feedback); } break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_PARTITION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT:       _setNameExpression(view.model, value, feedback);     break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_CONSTRAINT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT: Engine.setProperty(view.model, \'specification\', value); break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_TEXTAREA:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_EDIT: Engine.setProperty(view, \'text\', value); break;\n                    }\n                    break;\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        function _newSeqLifelinePosition(diagram) {\n            var lifelines = _.filter(diagram.ownedViews, function (e) { return (e instanceof type.UMLSeqLifelineView); });\n            var rightMost = _.max(lifelines, function (e) { return e.getRight(); });\n            var options = {\n                x1: rightMost.getRight() + 20,\n                y1: rightMost.top,\n                x2: rightMost.getRight() + 20,\n                y2: rightMost.top\n            };\n            return options;\n        }\n\n        function _newSeqMessagePosition(fromLifeline, toLifeline, diagram, interval) {\n            interval = interval || 35;\n            var options2 = {\n                x1        : fromLifeline.left,\n                x2        : toLifeline.left,\n                y1        : (fromLifeline instanceof type.UMLSeqLifelineView ? fromLifeline.linePart.top + 20 : fromLifeline.top),\n                y2        : (fromLifeline instanceof type.UMLSeqLifelineView ? fromLifeline.linePart.top + 20 : fromLifeline.top),\n                tailView  : fromLifeline,\n                headView  : toLifeline,\n                tailModel : fromLifeline.model || null,\n                headModel : toLifeline.model || null\n            };\n            var messages = _.filter(diagram.ownedViews, function (e) {\n                return (e instanceof type.UMLSeqMessageView);\n            });\n            if (messages.length > 0) {\n                var bottomMost = _.max(messages, function (e) { return e.points.points[0].y; });\n                options2.y1 = bottomMost.points.points[0].y + interval;\n                options2.y2 = bottomMost.points.points[0].y + interval;\n            }\n            return options2;\n        }\n\n        // QuickEdit Button Click Event Handlers\n        $(QuickEdit).on(\'click\', function (event, id, itemId, view) {\n            var diagram      = DiagramManager.getEditor().diagram,\n                diagramOwner = diagram._parent,\n                parent,\n                nodes, nodeView, nodeView1, nodeView2, edgeView, options1, options2, _x, _y;\n            try {\n\n                // Common Items\n                switch (itemId) {\n                case UMLQuickEdit.ITEM_ADD_NOTE:\n                    nodes = QuickEdit.getBothNodes(view, type.UMLNoteLinkView);\n                    options1 = QuickEdit.getRightPosition(view, nodes);\n                    options1.x2 = options1.x1 + 100;\n                    options1.y2 = options1.y1 + 50;\n                    nodeView = Factory.createModelAndView("Note", diagramOwner, diagram, options1);\n                    options2 = {\n                        tailView  : view,\n                        headView  : nodeView\n                    };\n                    if (options2.tailView instanceof type.EdgeLabelView || options2.tailView instanceof type.NodeLabelView) {\n                        options2.tailView = options2.tailView._parent;\n                    }\n                    if (options2.headView instanceof type.EdgeLabelView || options2.headView instanceof type.NodeLabelView) {\n                        options2.headView = options2.headView._parent;\n                    }\n                    edgeView = Factory.createModelAndView("NoteLink", diagramOwner, diagram, options2);\n                    break;\n                case UMLQuickEdit.ITEM_ADD_ATTRIBUTE:\n                    _ensure(view, \'suppressAttributes\', false);\n                    QuickEdit.addInQuickEdit(view.model, \'attributes\', view, \'attributeCompartment\', type.UMLAttribute, \'Attribute\');\n                    break;\n                case UMLQuickEdit.ITEM_ADD_OPERATION:\n                    _ensure(view, \'suppressOperations\', false);\n                    QuickEdit.addInQuickEdit(view.model, \'operations\', view, \'operationCompartment\', type.UMLOperation, \'Operation\');\n                    break;\n                case UMLQuickEdit.ITEM_ADD_RECEPTION:\n                    _ensure(view, \'suppressReceptions\', false);\n                    QuickEdit.addInQuickEdit(view.model, \'receptions\', view, \'receptionCompartment\', type.UMLReception, \'Reception\');\n                    break;\n                case UMLQuickEdit.ITEM_ADD_PROVIDED_INTERFACE:\n                    if (view instanceof type.NodeLabelView) {\n                        view = view._parent;\n                    }\n                    nodes = QuickEdit.getHeadNodes(view, type.UMLInterfaceRealizationView);\n                    options1 = QuickEdit.getTopPosition(view, nodes);\n                    if (view.containerView) {\n                        if (view.getRight() > view.containerView.getRight()) {\n                            options1 = QuickEdit.getRightPosition(view, nodes);\n                        } else if (view.left < view.containerView.left) {\n                            options1 = QuickEdit.getLeftPosition(view, nodes);\n                        } else if (view.getBottom() > view.containerView.getBottom()) {\n                            options1 = QuickEdit.getBottomPosition(view, nodes);\n                        }\n                    }\n                    nodeView = Factory.createModelAndView("UMLInterface", diagramOwner, diagram, options1);\n                    if ((view.model instanceof type.UMLPort) && !(view.model.type instanceof type.UMLClassifier)) {\n                        var _portType = Factory.createModel("UMLClass", view.containerView.model._parent, "ownedElements", {\n                            modelInitializer: function (e) {\n                                e.name = view.model.name + "Type";\n                            }\n                        });\n                        Engine.setProperty(view.model, \'type\', _portType);\n                    }\n                    options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                    edgeView = Factory.createModelAndView("UMLInterfaceRealization", view.model, diagram, options2);\n                    break;\n                case UMLQuickEdit.ITEM_ADD_REQUIRED_INTERFACE:\n                    if (view instanceof type.NodeLabelView) {\n                        view = view._parent;\n                    }\n                    nodes = QuickEdit.getHeadNodes(view, type.UMLDependencyView);\n                    options1 = QuickEdit.getTopPosition(view, nodes);\n                    if (view.containerView) {\n                        if (view.getRight() > view.containerView.getRight()) {\n                            options1 = QuickEdit.getRightPosition(view, nodes);\n                        } else if (view.left < view.containerView.left) {\n                            options1 = QuickEdit.getLeftPosition(view, nodes);\n                        } else if (view.getBottom() > view.containerView.getBottom()) {\n                            options1 = QuickEdit.getBottomPosition(view, nodes);\n                        }\n                    }\n                    nodeView = Factory.createModelAndView("UMLInterface", diagramOwner, diagram, options1);\n                    options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                    edgeView = Factory.createModelAndView("UMLDependency", view.model, diagram, options2);\n                    break;\n                case UMLQuickEdit.ITEM_ADD_PORT:\n                    _x = view.left + Math.round(Math.random() * view.width);\n                    _y = view.top + Math.round(Math.random() * view.height);\n                    options1 = { x1: _x, y1: _y, x2: _x, y2: _y, containerView: view };\n                    nodeView = Factory.createModelAndView("UMLPort", view.model, diagram, options1);\n                    break;\n                case UMLQuickEdit.ITEM_ADD_PART:\n                    nodes = _.filter(view.containedViews, function (e) { return (e instanceof type.UMLPartView); });\n                    options1 = {\n                        x1: view.left + 10,\n                        y1: view.top + 30,\n                        x2: view.left + 10,\n                        y2: view.top + 30,\n                        containerView: view\n                    };\n                    if (nodes.length > 0) {\n                        var _rightMost = _.max(nodes, function (e) { return e.getRight(); });\n                        options1.x1 = _rightMost.getRight() + 10;\n                        options1.y1 = _rightMost.top;\n                        options1.x2 = _rightMost.getRight() + 10;\n                        options1.y2 = _rightMost.top;\n                    }\n                    nodeView = Factory.createModelAndView("UMLPart", view.model, diagram, options1);\n                    break;\n                case UMLQuickEdit.ITEM_SELECT_TYPE: // for Part/Port\n                    if (view.model instanceof type.UMLAttribute) {\n                        ElementPickerDialog.showDialog("Select Type", view.model.type, type.UMLClassifier).done(function (buttonId, selected) {\n                            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                Engine.setProperty(view.model, \'type\', selected);\n                            }\n                        });\n                    }\n                    break;\n                case UMLQuickEdit.ITEM_CREATE_TYPE: // for Part/Port\n                    if (view.model instanceof type.UMLAttribute) {\n                        parent = view.model._parent;\n                        if (parent) {\n                            Dialogs.showInputDialog(\n                                "Enter a type name to create"\n                            ).done(function (modal, result) {\n                                if (modal === Dialogs.DIALOG_BTN_OK && result.length > 0) {\n                                    OperationBuilder.begin("assign new class");\n                                    var model = new type.UMLClass();\n                                    model._parent = parent;\n                                    model.name = result;\n                                    OperationBuilder.insert(model);\n                                    OperationBuilder.fieldInsert(parent, \'ownedElements\', model);\n                                    OperationBuilder.fieldAssign(view.model, \'type\', model);\n                                    OperationBuilder.end();\n                                    var cmd = OperationBuilder.getOperation();\n                                    Repository.doOperation(cmd);\n                                }\n                            });\n                        }\n                    }\n                    break;\n                }\n\n                // Items for Specific QuickEdit\n                switch (id) {\n\n                // Package\n                case UMLQuickEdit.QUICKEDIT_PACKAGE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_SUBPACKAGE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLContainmentView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLPackage", view.model, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLContainment", view.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_DEPENDANT_PACKAGE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLDependencyView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLPackage", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLDependency", nodeView.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_DEPENDING_PACKAGE:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLDependencyView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLPackage", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLDependency", view.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Class\n                case UMLQuickEdit.QUICKEDIT_CLASS:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_SUPERCLASS:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLGeneralizationView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLClass", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLGeneralization", view.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_SUBCLASS:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLGeneralizationView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLClass", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLGeneralization", nodeView.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_ASSOCIATION:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLAssociationView, function (e) {\n                            return (e.tail === view && e.model.end1.aggregation === UML.AK_NONE);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLClass", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLAssociation", nodeView.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_AGGREGATION:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLAssociationView, function (e) {\n                            return (e.tail === view && e.model.end1.aggregation === UML.AK_SHARED);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLClass", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        options2.modelInitializer = function (model) {\n                            model.end1.aggregation = UML.AK_SHARED;\n                        };\n                        edgeView = Factory.createModelAndView("UMLAssociation", nodeView.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_COMPOSITION:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLAssociationView, function (e) {\n                            return (e.tail === view && e.model.end1.aggregation === UML.AK_SHARED);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLClass", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        options2.modelInitializer = function (model) {\n                            model.end1.aggregation = UML.AK_COMPOSITE;\n                        };\n                        edgeView = Factory.createModelAndView("UMLAssociation", nodeView.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Interface\n                case UMLQuickEdit.QUICKEDIT_INTERFACE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_SUPERINTERFACE:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLGeneralizationView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLInterface", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLGeneralization", view.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_SUBINTERFACE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLGeneralizationView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLInterface", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLGeneralization", view.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_REALIZING_CLASS:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLInterfaceRealizationView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLClass", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLInterfaceRealization", view.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Node\n                case UMLQuickEdit.QUICKEDIT_NODE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_COMMUNICATING_NODE:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLCommunicationPathView);\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLCommunicationPath", nodeView.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_DEPLOYED_ARTIFACT:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLDeploymentView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLArtifact", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLDeployment", view.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_DEPLOYED_COMPONENT:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLDeploymentView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLComponent", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLDeployment", view.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Actor\n                case UMLQuickEdit.QUICKEDIT_ACTOR:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_SUPERACTOR:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLGeneralizationView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLActor", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLGeneralization", view.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_SUBACTOR:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLGeneralizationView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLActor", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLGeneralization", nodeView.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_ASSOCIATED_USECASE:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLAssociationView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLUseCase", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLAssociation", nodeView.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Use Case\n                case UMLQuickEdit.QUICKEDIT_USECASE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_EXTENSIONPOINT:\n                        QuickEdit.addInQuickEdit(view.model, \'extensionPoints\', view, \'extensionPointCompartment\', type.UMLExtensionPoint, \'ExtensionPoint\');\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_ASSOCIATED_ACTOR:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLAssociationView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getLeftPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLActor", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLAssociation", nodeView.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_INCLUDED_USECASE:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLIncludeView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLUseCase", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLInclude", view.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_EXTENDED_USECASE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLExtendView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLUseCase", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLExtend", nodeView.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Enumeration\n                case UMLQuickEdit.QUICKEDIT_ENUMERATION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_LITERAL:\n                        _ensure(view, \'suppressLiterals\', false);\n                        QuickEdit.addInQuickEdit(view.model, \'literals\', view, \'enumerationLiteralCompartment\', type.UMLEnumerationLiteral, \'Literal\');\n                        break;\n                    }\n                    break;\n\n                // MetaClass\n                case UMLQuickEdit.QUICKEDIT_METACLASS:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_STEREOTYPE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLExtensionView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLStereotype", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLExtension", nodeView.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Stereotype\n                case UMLQuickEdit.QUICKEDIT_STEREOTYPE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_SUPERSTEREOTYPE:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLGeneralizationView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLStereotype", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLGeneralization", view.model, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_SUBSTEREOTYPE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLGeneralizationView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLStereotype", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLGeneralization", nodeView.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Attribute\n                case UMLQuickEdit.QUICKEDIT_ATTRIBUTE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, \'attributes\', view._parent._parent, \'attributeCompartment\', type.UMLAttribute, \'Attribute\');\n                        break;\n                    case UMLQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, \'attributes\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, \'attributes\', view);\n                        break;\n                    }\n                    break;\n\n                // Operation\n                case UMLQuickEdit.QUICKEDIT_OPERATION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, \'operations\', view._parent._parent, \'operationCompartment\', type.UMLOperation, \'Operation\');\n                        break;\n                    case UMLQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, \'operations\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, \'operations\', view);\n                        break;\n                    }\n                    break;\n\n                // Reception\n                case UMLQuickEdit.QUICKEDIT_RECEPTION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, \'receptions\', view._parent._parent, \'receptionCompartment\', type.UMLReception, \'Reception\');\n                        break;\n                    case UMLQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, \'receptions\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, \'receptions\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_SELECT_SIGNAL:\n                        ElementPickerDialog.showDialog("Select Signal", view.model.signature, type.UMLSignal).done(function (buttonId, selected) {\n                            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                Engine.setProperty(view.model, \'signal\', selected);\n                            }\n                        });\n                        break;\n                    }\n                    break;\n\n                // Port\n                case UMLQuickEdit.QUICKEDIT_PORT:\n                    if (view instanceof type.NodeLabelView) {\n                        view = view._parent;\n                    }\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_CONNECTED_PART:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLConnectorView);\n                        if (view.getRight() > view.containerView.getRight()) {\n                            options1 = QuickEdit.getLeftPosition(view, nodes);\n                        } else if (view.left < view.containerView.left) {\n                            options1 = QuickEdit.getRightPosition(view, nodes);\n                        } else if (view.getBottom() > view.containerView.getBottom()) {\n                            options1 = QuickEdit.getTopPosition(view, nodes);\n                        } else {\n                            options1 = QuickEdit.getBottomPosition(view, nodes);\n                        }\n                        options1.containerView = view.containerView;\n                        nodeView = Factory.createModelAndView("UMLPart", view.model._parent, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLConnector", view.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Part\n                case UMLQuickEdit.QUICKEDIT_PART:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_CONNECTED_PART:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLConnectorView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        options1.containerView = view.containerView;\n                        nodeView = Factory.createModelAndView("UMLPart", view.model._parent, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLConnector", view.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Parameter\n                case UMLQuickEdit.QUICKEDIT_TEMPLATE_PARAMETER:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, \'templateParameters\', view._parent._parent, \'templateParameterCompartment\', type.UMLTemplateParameter, \'TemplateParameter\');\n                        break;\n                    case UMLQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, \'templateParameters\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, \'templateParameters\', view);\n                        break;\n                    }\n                    break;\n\n                // Qualifer\n                case UMLQuickEdit.QUICKEDIT_QUALIFIER:\n                    var asso      = view._parent._parent,\n                        comp      = view._parent,\n                        end       = null,\n                        compField = null;\n                    if (asso.tailQualifiersCompartment == comp) {\n                        end = asso.model.end1;\n                        compField = "tailQualifiersCompartment";\n                    } else if (asso.headQualifiersCompartment == comp) {\n                        end = asso.model.end2;\n                        compField = "headQualifiersCompartment";\n                    }\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(end, \'qualifiers\', asso, compField, type.UMLAttribute, \'Qualifier\');\n                        break;\n                    case UMLQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(end, \'qualifiers\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(end, \'qualifiers\', view);\n                        break;\n                    }\n                    break;\n\n                // Literal\n                case UMLQuickEdit.QUICKEDIT_LITERAL:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, \'literals\', view._parent._parent, \'enumerationLiteralCompartment\', type.UMLEnumerationLiteral, \'Literal\');\n                        break;\n                    case UMLQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, \'literals\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, \'literals\', view);\n                        break;\n                    }\n                    break;\n\n                // Extension Point\n                case UMLQuickEdit.QUICKEDIT_EXTENSION_POINT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, \'extensionPoints\', view._parent._parent, \'extensionPointCompartment\', type.UMLExtensionPoint, \'ExtensionPoint\');\n                        break;\n                    case UMLQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, \'extensionPoints\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, \'extensionPoints\', view);\n                        break;\n                    }\n                    break;\n\n                // Association (Main, Tail, Head)\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION:\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION_TAIL:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_QUALIFIER:\n                        if (view instanceof Core.EdgeLabelView) {\n                            QuickEdit.addInQuickEdit(view._parent.model.end1, \'qualifiers\', view._parent, \'tailQualifiersCompartment\', type.UMLAttribute, \'Qualifier\');\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION_HEAD:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_QUALIFIER:\n                        if (view instanceof Core.EdgeLabelView) {\n                            QuickEdit.addInQuickEdit(view._parent.model.end2, \'qualifiers\', view._parent, \'headQualifiersCompartment\', type.UMLAttribute, \'Qualifier\');\n                        }\n                        break;\n                    }\n                    break;\n\n                // Object\n                case UMLQuickEdit.QUICKEDIT_OBJECT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_SLOT:\n                        QuickEdit.addInQuickEdit(view.model, \'slots\', view, \'slotCompartment\', type.UMLSlot, \'Slot\');\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_LINKED_OBJECT:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLLinkView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLObject", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLLink", nodeView.model, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Slot\n                case UMLQuickEdit.QUICKEDIT_SLOT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, \'slots\', view._parent._parent, \'slotCompartment\', type.UMLSlot, \'Slot\');\n                        break;\n                    case UMLQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, \'slots\', view);\n                        break;\n                    case UMLQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, \'slots\', view);\n                        break;\n                    }\n                    break;\n\n                // Lifeline (Sequence & Communications)\n                case UMLQuickEdit.QUICKEDIT_SEQ_LIFELINE:\n                case UMLQuickEdit.QUICKEDIT_COMM_LIFELINE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_SELECT_TYPE:\n                        if (view.model.represent instanceof type.UMLAttribute) {\n                            ElementPickerDialog.showDialog("Select Type", view.model.represent.type, type.UMLClassifier).done(function (buttonId, selected) {\n                                if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                    Engine.setProperty(view.model.represent, \'type\', selected);\n                                }\n                            });\n                        } else {\n                            Dialogs.showAlertDialog("Lifeline should represent a role. (connect \'represent\' field to an Attribute of the container Collaboration)");\n                        }\n                        break;\n                    case UMLQuickEdit.ITEM_CREATE_TYPE:\n                        if (view.model.represent instanceof type.UMLAttribute) {\n                            parent = view.model.represent._parent._parent;\n                            if (parent) {\n                                Dialogs.showInputDialog(\n                                    "Enter a type name to create"\n                                ).done(function (modal, result) {\n                                    if (modal === Dialogs.DIALOG_BTN_OK && result.length > 0) {\n                                        OperationBuilder.begin("assign new class");\n                                        var model = new type.UMLClass();\n                                        model._parent = parent;\n                                        model.name = result;\n                                        OperationBuilder.insert(model);\n                                        OperationBuilder.fieldInsert(parent, \'ownedElements\', model);\n                                        OperationBuilder.fieldAssign(view.model.represent, \'type\', model);\n                                        OperationBuilder.end();\n                                        var cmd = OperationBuilder.getOperation();\n                                        Repository.doOperation(cmd);\n                                    }\n                                });\n                            }\n                        } else {\n                            Dialogs.showAlertDialog("Lifeline should represent a role. (connect \'represent\' field to an Attribute of the container Collaboration)");\n                        }\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_MESSAGE_LIFELINE:\n                        options1 = _newSeqLifelinePosition(diagram);\n                        nodeView = Factory.createModelAndView("UMLLifeline", diagramOwner, diagram, options1);\n                        options2 = _newSeqMessagePosition(view, nodeView, diagram);\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_CREATE_MESSAGE_LIFELINE:\n                        options1 = _newSeqLifelinePosition(diagram);\n                        nodeView = Factory.createModelAndView("UMLLifeline", diagramOwner, diagram, options1);\n                        options2 = _newSeqMessagePosition(view, nodeView, diagram, 45);\n                        options2.modelInitializer = function (model) {\n                            model.messageSort = UML.MS_CREATEMESSAGE;\n                        };\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_SELF_MESSAGE:\n                        options2 = _newSeqMessagePosition(view, view, diagram);\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_MESSAGE_FROM_GATE:\n                        options1 = _newSeqMessagePosition(view, view, diagram);\n                        options1.x1 = view.left - 70;\n                        options1.x2 = view.left - 70;\n                        QuickEdit.setInside(options1);\n                        nodeView = Factory.createModelAndView("UMLGate", diagramOwner, diagram, options1);\n                        options2 = _newSeqMessagePosition(nodeView, view, diagram);\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_MESSAGE_TO_GATE:\n                        options1 = _newSeqMessagePosition(view, view, diagram);\n                        options1.x1 = view.getRight() + 70;\n                        options1.x2 = view.getRight() + 70;\n                        nodeView = Factory.createModelAndView("UMLGate", diagramOwner, diagram, options1);\n                        options2 = _newSeqMessagePosition(view, nodeView, diagram);\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_FOUND_MESSAGE:\n                        options1 = _newSeqMessagePosition(view, view, diagram);\n                        options1.x1 = view.left - 70;\n                        options1.x2 = view.left - 70;\n                        QuickEdit.setInside(options1);\n                        nodeView = Factory.createModelAndView("UMLEndpoint", diagramOwner, diagram, options1);\n                        options2 = _newSeqMessagePosition(nodeView, view, diagram);\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_LOST_MESSAGE:\n                        options1 = _newSeqMessagePosition(view, view, diagram);\n                        options1.x1 = view.getRight() + 70;\n                        options1.x2 = view.getRight() + 70;\n                        nodeView = Factory.createModelAndView("UMLEndpoint", diagramOwner, diagram, options1);\n                        options2 = _newSeqMessagePosition(view, nodeView, diagram);\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_CONNECTED_LIFELINE:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLConnectorView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLLifeline", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLConnector", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_SELF_CONNECTOR:\n                        options2 = QuickEdit.getEdgeViewOption(view, view);\n                        view = Factory.createModelAndView("UMLConnector", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_FORWARD_MESSAGE:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLConnectorView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLLifeline", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLConnector", diagramOwner, diagram, options2);\n                        options2 = {\n                            x1        : edgeView.points.points[0].x,\n                            y1        : edgeView.points.points[0].y,\n                            x2        : edgeView.points.points[0].x,\n                            y2        : edgeView.points.points[0].y,\n                            direction : "forward",\n                            connector : edgeView.model,\n                            headView  : edgeView\n                        };\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_REVERSE_MESSAGE:\n                        nodes = QuickEdit.getBothNodes(view, type.UMLConnectorView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLLifeline", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLConnector", diagramOwner, diagram, options2);\n                        options2 = {\n                            x1        : edgeView.points.points[0].x,\n                            y1        : edgeView.points.points[0].y,\n                            x2        : edgeView.points.points[0].x,\n                            y2        : edgeView.points.points[0].y,\n                            direction : "reverse",\n                            connector : edgeView.model,\n                            headView  : edgeView\n                        };\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Message (Sequence & Communications)\n                case UMLQuickEdit.QUICKEDIT_SEQ_MESSAGE:\n                case UMLQuickEdit.QUICKEDIT_COMM_MESSAGE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_SELECT_OPERATION:\n                        // TODO: THIS IMPLEMENTATION SHOULD REPLACED BY ELEMENTLISTPICKER UI\n                        if ((view.model.target.represent instanceof type.UMLAttribute) && (view.model.target.represent.type instanceof type.UMLClassifier)) {\n                            var ops = _.union(view.model.target.represent.type.getInheritedOperations(true), view.model.target.represent.type.operations);\n                            ElementListPickerDialog.showDialog(\n                                "Select Operation",\n                                ops\n                            ).done(function (buttonId, selected) {\n                                if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                    Engine.setProperty(view.model, \'signature\', selected);\n                                }\n                            });\n                        } else {\n                            Dialogs.showAlertDialog("Lifeline should have a type.");\n                        }\n                        break;\n                    case UMLQuickEdit.ITEM_CREATE_OPERATION:\n                        if ((view.model.target.represent instanceof type.UMLAttribute) && (view.model.target.represent.type instanceof type.UMLClassifier)) {\n                            parent = view.model.target.represent.type; // Type of target Lifeline\n                            Dialogs.showInputDialog(\n                                "Enter an operation name to create"\n                            ).done(function (buttonId, result) {\n                                if (buttonId === Dialogs.DIALOG_BTN_OK && result.length > 0) {\n                                    OperationBuilder.begin("assign new operation");\n                                    var model = new type.UMLOperation();\n                                    model._parent = parent;\n                                    model.name = result;\n                                    OperationBuilder.insert(model);\n                                    OperationBuilder.fieldInsert(parent, \'operations\', model);\n                                    OperationBuilder.fieldAssign(view.model, \'signature\', model);\n                                    OperationBuilder.end();\n                                    var cmd = OperationBuilder.getOperation();\n                                    Repository.doOperation(cmd);\n                                }\n                            });\n                        } else {\n                            Dialogs.showAlertDialog("Lifeline should have a type.");\n                        }\n                        break;\n                    case UMLQuickEdit.ITEM_SELECT_SIGNAL:\n                        ElementPickerDialog.showDialog("Select Signal", view.model.signature, type.UMLSignal).done(function (buttonId, selected) {\n                            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                Engine.setProperty(view.model, \'signature\', selected);\n                            }\n                        });\n                        break;\n                    case UMLQuickEdit.ITEM_CREATE_SIGNAL:\n                        parent = view.model._parent._parent._parent; // owner of Collaboration\n                        if (parent) {\n                            Dialogs.showInputDialog(\n                                "Enter a signal name to create"\n                            ).done(function (buttonId, result) {\n                                if (buttonId === Dialogs.DIALOG_BTN_OK && result.length > 0) {\n                                    OperationBuilder.begin("assign new signal");\n                                    var model = new type.UMLSignal();\n                                    model._parent = parent;\n                                    model.name = result;\n                                    OperationBuilder.insert(model);\n                                    OperationBuilder.fieldInsert(parent, \'ownedElements\', model);\n                                    OperationBuilder.fieldAssign(view.model, \'signature\', model);\n                                    OperationBuilder.end();\n                                    var cmd = OperationBuilder.getOperation();\n                                    Repository.doOperation(cmd);\n                                }\n                            });\n                        }\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_REPLY_MESSAGE:\n                        if (diagram instanceof type.UMLSequenceDiagram) {\n                            if (view instanceof type.EdgeLabelView) {\n                                view = view._parent;\n                            }\n                            if (view.head instanceof type.UMLLinePartView && view.tail instanceof type.UMLLinePartView) {\n                                options2 = {\n                                    x1        : view.head.left,\n                                    x2        : view.tail.left,\n                                    y1        : view.points.points[0].y + 20,\n                                    y2        : view.points.points[0].y + 20,\n                                    tailView  : view.head,\n                                    headView  : view.tail,\n                                    tailModel : view.head.model || null,\n                                    headModel : view.tail.model || null,\n                                    modelInitializer : function (model) {\n                                        model.messageSort = UML.MS_REPLY;\n                                    }\n                                };\n                                edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                            }\n                        } else if (diagram instanceof type.UMLCommunicationDiagram) {\n                            if (view instanceof type.NodeLabelView) {\n                                view = view._parent;\n                            }\n                            options2 = {\n                                x1        : view.hostEdge.points.points[0].x,\n                                y1        : view.hostEdge.points.points[0].y,\n                                x2        : view.hostEdge.points.points[0].x,\n                                y2        : view.hostEdge.points.points[0].y,\n                                direction : (view.model.source === view.hostEdge.tail.model ? "reverse" : "forward"),\n                                connector : view.hostEdge.model,\n                                headView  : view.hostEdge,\n                                modelInitializer : function (model) {\n                                    model.messageSort = UML.MS_REPLY;\n                                }\n                            };\n                            edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        }\n                        break;\n                    }\n                    break;\n\n                // Connector (Composite Structure & Communications)\n                case UMLQuickEdit.QUICKEDIT_CONNECTOR:\n                    if (view instanceof type.EdgeLabelView) {\n                        view = view._parent;\n                    }\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_FORWARD_MESSAGE:\n                        options2 = {\n                            x1        : view.points.points[0].x,\n                            y1        : view.points.points[0].y,\n                            x2        : view.points.points[0].x,\n                            y2        : view.points.points[0].y,\n                            direction : "forward",\n                            connector : view.model,\n                            headView  : view\n                        };\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_REVERSE_MESSAGE:\n                        options2 = {\n                            x1        : view.points.points[0].x,\n                            y1        : view.points.points[0].y,\n                            x2        : view.points.points[0].x,\n                            y2        : view.points.points[0].y,\n                            direction : "reverse",\n                            connector : view.model,\n                            headView  : view\n                        };\n                        edgeView = Factory.createModelAndView("UMLMessage", diagramOwner, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Combined Fragment\n                case UMLQuickEdit.QUICKEDIT_COMBINEDFRAGMENT:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_OPERAND:\n                        Factory.createModel("UMLInteractionOperand", view.model, \'operands\');\n                        break;\n                    }\n                    break;\n\n                // State\n                case UMLQuickEdit.QUICKEDIT_STATE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_REGION:\n                        Factory.createModel("UMLRegion", view.model, \'regions\');\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_CONNECTION_POINT_REFERENCE:\n                        _x = view.left + Math.round(Math.random() * view.width);\n                        _y = view.top + Math.round(Math.random() * view.height);\n                        options1 = { x1: _x, y1: _y, x2: _x, y2: _y, containerView: view };\n                        nodeView = Factory.createModelAndView("UMLConnectionPointReference", view.model, diagram, options1);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_ENTRY_ACTIVITY:\n                        Dialogs.showSelectRadioDialog(\n                            "Select one to create as an entry Activity",\n                            [\n                                { text: "OpaqueBehavior", value: "1", checked: true},\n                                { text: "Activity",       value: "2"},\n                                { text: "StateMachine",   value: "3"},\n                                { text: "Interaction",    value: "4"}\n                            ]\n                        ).done(function (buttonId, selected) {\n                            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                switch (selected) {\n                                case "1": QuickEdit.addInQuickEdit(view.model, \'entryActivities\', view, \'internalActivityCompartment\', type.UMLOpaqueBehavior, \'EntryActivity\'); break;\n                                case "2": QuickEdit.addInQuickEdit(view.model, \'entryActivities\', view, \'internalActivityCompartment\', type.UMLActivity,       \'EntryActivity\'); break;\n                                case "3": QuickEdit.addInQuickEdit(view.model, \'entryActivities\', view, \'internalActivityCompartment\', type.UMLStateMachine,   \'EntryActivity\'); break;\n                                case "4": QuickEdit.addInQuickEdit(view.model, \'entryActivities\', view, \'internalActivityCompartment\', type.UMLInteraction,    \'EntryActivity\'); break;\n                                }\n                            }\n                        });\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_DO_ACTIVITY:\n                        Dialogs.showSelectRadioDialog(\n                            "Select one to create as an do Activity",\n                            [\n                                { text: "OpaqueBehavior", value: "1", checked: true},\n                                { text: "Activity",       value: "2"},\n                                { text: "StateMachine",   value: "3"},\n                                { text: "Interaction",    value: "4"}\n                            ]\n                        ).done(function (buttonId, selected) {\n                            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                switch (selected) {\n                                case "1": QuickEdit.addInQuickEdit(view.model, \'doActivities\', view, \'internalActivityCompartment\', type.UMLOpaqueBehavior, \'DoActivity\'); break;\n                                case "2": QuickEdit.addInQuickEdit(view.model, \'doActivities\', view, \'internalActivityCompartment\', type.UMLActivity,       \'DoActivity\'); break;\n                                case "3": QuickEdit.addInQuickEdit(view.model, \'doActivities\', view, \'internalActivityCompartment\', type.UMLStateMachine,   \'DoActivity\'); break;\n                                case "4": QuickEdit.addInQuickEdit(view.model, \'doActivities\', view, \'internalActivityCompartment\', type.UMLInteraction,    \'DoActivity\'); break;\n                                }\n                            }\n                        });\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_EXIT_ACTIVITY:\n                        Dialogs.showSelectRadioDialog(\n                            "Select one to create as an exit Activity",\n                            [\n                                { text: "OpaqueBehavior", value: "1", checked: true},\n                                { text: "Activity",       value: "2"},\n                                { text: "StateMachine",   value: "3"},\n                                { text: "Interaction",    value: "4"}\n                            ]\n                        ).done(function (buttonId, selected) {\n                            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                switch (selected) {\n                                case "1": QuickEdit.addInQuickEdit(view.model, \'exitActivities\', view, \'internalActivityCompartment\', type.UMLOpaqueBehavior, \'ExitActivity\'); break;\n                                case "2": QuickEdit.addInQuickEdit(view.model, \'exitActivities\', view, \'internalActivityCompartment\', type.UMLActivity,       \'ExitActivity\'); break;\n                                case "3": QuickEdit.addInQuickEdit(view.model, \'exitActivities\', view, \'internalActivityCompartment\', type.UMLStateMachine,   \'ExitActivity\'); break;\n                                case "4": QuickEdit.addInQuickEdit(view.model, \'exitActivities\', view, \'internalActivityCompartment\', type.UMLInteraction,    \'ExitActivity\'); break;\n                                }\n                            }\n                        });\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_INTERNAL_TRANSITION:\n                        if (view.model.regions.length > 0) {\n                            QuickEdit.addInQuickEdit(view.model.regions[0], \'transitions\', view, \'internalTransitionCompartment\', type.UMLTransition, \'InternalTransition\', function (m) {\n                                m.source = view.model;\n                                m.target = view.model;\n                                m.kind = UML.TK_INTERNAL;\n                            });\n                        } else {\n                            Toast.info("Simple State cannot have Internal Transitions.");\n                        }\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_OUTGOING_TRANSITION:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLTransitionView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_INCOMING_TRANSITION:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLTransitionView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_INITIAL_STATE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLTransitionView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        options1.pseudostateKind = UML.PSK_INITIAL;\n                        nodeView = Factory.createModelAndView("UMLPseudostate", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_FINAL_STATE:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLTransitionView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLFinalState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_CHOICE:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLTransitionView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes, 40);\n                        options1.pseudostateKind = UML.PSK_CHOICE;\n                        nodeView = Factory.createModelAndView("UMLPseudostate", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        // left state\n                        options1 = {\n                            x1: nodeView.left - 70,\n                            y1: nodeView.getBottom() + 50,\n                            x2: nodeView.left - 70,\n                            y2: nodeView.getBottom() + 50\n                        };\n                        nodeView1 = Factory.createModelAndView("UMLState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, nodeView1);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        // right state\n                        options1 = {\n                            x1: nodeView.getRight() + 35,\n                            y1: nodeView.getBottom() + 50,\n                            x2: nodeView.getRight() + 35,\n                            y2: nodeView.getBottom() + 50\n                        };\n                        nodeView2 = Factory.createModelAndView("UMLState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, nodeView2);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_FORK:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLTransitionView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes, 40);\n                        options1.pseudostateKind = UML.PSK_FORK;\n                        nodeView = Factory.createModelAndView("UMLPseudostate", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        // left state\n                        options1 = {\n                            x1: nodeView.left - 40,\n                            y1: nodeView.getBottom() + 40,\n                            x2: nodeView.left - 40,\n                            y2: nodeView.getBottom() + 40\n                        };\n                        nodeView1 = Factory.createModelAndView("UMLState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, nodeView1);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        // right state\n                        options1 = {\n                            x1: nodeView.getRight() + 50,\n                            y1: nodeView.getBottom() + 40,\n                            x2: nodeView.getRight() + 50,\n                            y2: nodeView.getBottom() + 40\n                        };\n                        nodeView2 = Factory.createModelAndView("UMLState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, nodeView2);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_JOIN:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLTransitionView);\n                        options1 = QuickEdit.getTopPosition(view, nodes, 40);\n                        options1.pseudostateKind = UML.PSK_JOIN;\n                        nodeView = Factory.createModelAndView("UMLPseudostate", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        // left state\n                        options1 = {\n                            x1: nodeView.left - 40,\n                            y1: nodeView.top - 80,\n                            x2: nodeView.left - 40,\n                            y2: nodeView.top - 80\n                        };\n                        QuickEdit.setInside(options1);\n                        nodeView1 = Factory.createModelAndView("UMLState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView1, nodeView);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        // right state\n                        options1 = {\n                            x1: nodeView.getRight() + 50,\n                            y1: nodeView.top - 80,\n                            x2: nodeView.getRight() + 50,\n                            y2: nodeView.top - 80\n                        };\n                        QuickEdit.setInside(options1);\n                        nodeView2 = Factory.createModelAndView("UMLState", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView2, nodeView);\n                        edgeView = Factory.createModelAndView("UMLTransition", diagramOwner, diagram, options2);\n                        break;\n                    }\n                    break;\n\n                // Internal Transition\n                case UMLQuickEdit.QUICKEDIT_INTERNAL_TRANSITION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_TRIGGER:\n                        Factory.createModel("UMLEvent", view.model, \'triggers\');\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_EFFECT:\n                        Dialogs.showSelectRadioDialog(\n                            "Select one to create as an effect Behavior",\n                            [\n                                { text: "OpaqueBehavior", value: "UMLOpaqueBehavior", checked: true},\n                                { text: "Activity",       value: "UMLActivity"},\n                                { text: "StateMachine",   value: "UMLStateMachine"},\n                                { text: "Interaction",    value: "UMLInteraction"}\n                            ]\n                        ).done(function (buttonId, selected) {\n                            if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                Factory.createModel(selected, view.model, \'effects\');\n                            }\n                        });\n                        break;\n                    }\n                    break;\n\n                // Transition\n                case UMLQuickEdit.QUICKEDIT_TRANSITION:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_TRIGGER:\n                        if (view instanceof Core.EdgeLabelView) {\n                            Dialogs.showInputDialog(\n                                "Enter name of an event to create"\n                            ).done(function (modal, result) {\n                                if (modal === Dialogs.DIALOG_BTN_OK && result.length > 0) {\n                                    var options = {\n                                        modelInitializer: function (elem) {\n                                            elem.name = result;\n                                        }\n                                    };\n                                    Factory.createModel("UMLEvent", view._parent.model, \'triggers\', options);\n                                }\n                            });\n                        }\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_EFFECT:\n                        if (view instanceof Core.EdgeLabelView) {\n                            Dialogs.showSelectRadioDialog(\n                                "Select one to create as an effect Behavior",\n                                [\n                                    { text: "OpaqueBehavior", value: "UMLOpaqueBehavior", checked: true},\n                                    { text: "Activity",       value: "UMLActivity"},\n                                    { text: "StateMachine",   value: "UMLStateMachine"},\n                                    { text: "Interaction",    value: "UMLInteraction"}\n                                ]\n                            ).done(function (buttonId, selected) {\n                                if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                    Factory.createModel(selected, view._parent.model, \'effects\');\n                                }\n                            });\n                        }\n                        break;\n                    }\n                    break;\n\n                // Action & ObjectNode\n                case UMLQuickEdit.QUICKEDIT_ACTION:\n                case UMLQuickEdit.QUICKEDIT_OBJECT_NODE:\n                    switch (itemId) {\n                    case UMLQuickEdit.ITEM_ADD_TRIGGER:\n                        if (view instanceof type.UMLActionView) {\n                            Factory.createModel("UMLEvent", view.model, \'triggers\');\n                        }\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_INPUT_PIN:\n                        _x = view.left + Math.round(Math.random() * view.width);\n                        _y = view.top + Math.round(Math.random() * view.height);\n                        options1 = { x1: _x, y1: _y, x2: _x, y2: _y, containerView: view };\n                        nodeView = Factory.createModelAndView("UMLInputPin", view.model, diagram, options1);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_OUTPUT_PIN:\n                        _x = view.left + Math.round(Math.random() * view.width);\n                        _y = view.top + Math.round(Math.random() * view.height);\n                        options1 = { x1: _x, y1: _y, x2: _x, y2: _y, containerView: view };\n                        nodeView = Factory.createModelAndView("UMLOutputPin", view.model, diagram, options1);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_OUTGOING_CONTROL_FLOW:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLControlFlowView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_INCOMING_CONTROL_FLOW:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLControlFlowView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_OUTGOING_OBJECT_FLOW:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLObjectFlowView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLObjectNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLObjectFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_INCOMING_OBJECT_FLOW:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLObjectFlowView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLObjectNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLObjectFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_DECISION:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLControlFlowView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes, 40);\n                        nodeView = Factory.createModelAndView("UMLDecisionNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        // left action\n                        options1 = {\n                            x1: nodeView.left - 60,\n                            y1: nodeView.getBottom() + 40,\n                            x2: nodeView.left - 60,\n                            y2: nodeView.getBottom() + 40\n                        };\n                        nodeView1 = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, nodeView1);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        // right action\n                        options1 = {\n                            x1: nodeView.getRight(),\n                            y1: nodeView.getBottom() + 40,\n                            x2: nodeView.getRight(),\n                            y2: nodeView.getBottom() + 40\n                        };\n                        nodeView2 = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, nodeView2);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_MERGE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLControlFlowView);\n                        options1 = QuickEdit.getTopPosition(view, nodes, 40);\n                        nodeView = Factory.createModelAndView("UMLMergeNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        // left state\n                        options1 = {\n                            x1: nodeView.left - 60,\n                            y1: nodeView.top - 80,\n                            x2: nodeView.left - 60,\n                            y2: nodeView.top - 80\n                        };\n                        QuickEdit.setInside(options1);\n                        nodeView1 = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView1, nodeView);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        // right state\n                        options1 = {\n                            x1: nodeView.getRight(),\n                            y1: nodeView.top - 80,\n                            x2: nodeView.getRight(),\n                            y2: nodeView.top - 80\n                        };\n                        QuickEdit.setInside(options1);\n                        nodeView2 = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView2, nodeView);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_ACTIVITY_FORK:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLControlFlowView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes, 40);\n                        nodeView = Factory.createModelAndView("UMLForkNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        // left action\n                        options1 = {\n                            x1: nodeView.left - 50,\n                            y1: nodeView.getBottom() + 40,\n                            x2: nodeView.left - 50,\n                            y2: nodeView.getBottom() + 40\n                        };\n                        nodeView1 = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, nodeView1);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        // right action\n                        options1 = {\n                            x1: nodeView.getRight(),\n                            y1: nodeView.getBottom() + 40,\n                            x2: nodeView.getRight(),\n                            y2: nodeView.getBottom() + 40\n                        };\n                        nodeView2 = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, nodeView2);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_ACTIVITY_JOIN:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLControlFlowView);\n                        options1 = QuickEdit.getTopPosition(view, nodes, 40);\n                        nodeView = Factory.createModelAndView("UMLJoinNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        // left state\n                        options1 = {\n                            x1: nodeView.left - 50,\n                            y1: nodeView.top - 80,\n                            x2: nodeView.left - 50,\n                            y2: nodeView.top - 80\n                        };\n                        QuickEdit.setInside(options1);\n                        nodeView1 = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView1, nodeView);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        // right state\n                        options1 = {\n                            x1: nodeView.getRight(),\n                            y1: nodeView.top - 80,\n                            x2: nodeView.getRight(),\n                            y2: nodeView.top - 80\n                        };\n                        QuickEdit.setInside(options1);\n                        nodeView2 = Factory.createModelAndView("UMLAction", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView2, nodeView);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_INITIAL_NODE:\n                        nodes = QuickEdit.getTailNodes(view, type.UMLControlFlowView);\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLInitialNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        break;\n                    case UMLQuickEdit.ITEM_ADD_FINAL_NODE:\n                        nodes = QuickEdit.getHeadNodes(view, type.UMLControlFlowView);\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView("UMLActivityFinalNode", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView("UMLControlFlow", diagramOwner, diagram, options2);\n                        break;\n                    }\n                    break;\n                }\n            } catch (err) {\n                Toast.error(err);\n                console.error(err);\n            }\n        });\n\n        // QuickEdit Keydown Event Handlers\n        $(QuickEdit).on(\'keydown\', function (event, id, itemId, view, keyEvent) {\n            var v;\n            try {\n                switch (id) {\n                case UMLQuickEdit.QUICKEDIT_GENERAL:\n                    break;\n                case UMLQuickEdit.QUICKEDIT_CLASSIFIER:\n                case UMLQuickEdit.QUICKEDIT_CLASS:\n                case UMLQuickEdit.QUICKEDIT_INTERFACE:\n                case UMLQuickEdit.QUICKEDIT_ENUMERATION:\n                case UMLQuickEdit.QUICKEDIT_USECASE:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (view.templateParameterCompartment && view.templateParameterCompartment.subViews.length > 0) {\n                            v = view.templateParameterCompartment.subViews[view.templateParameterCompartment.subViews.length - 1];\n                            _.defer(function () {\n                                QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                            });\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (view.suppressLiterals !== true && view.enumerationLiteralCompartment && view.enumerationLiteralCompartment.subViews.length > 0) {\n                            v = view.enumerationLiteralCompartment.subViews[0];\n                            _.defer(function () {\n                                QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                            });\n                        } else if (view.suppressAttributes !== true && view.attributeCompartment && view.attributeCompartment.subViews.length > 0) {\n                            v = view.attributeCompartment.subViews[0];\n                            _.defer(function () {\n                                QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                            });\n                        } else if (view.suppressOperations !== true && view.operationCompartment && view.operationCompartment.subViews.length > 0) {\n                            v = view.operationCompartment.subViews[0];\n                            _.defer(function () {\n                                QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                            });\n                        } else if (view.suppressReceptions !== true && view.receptionCompartment && view.receptionCompartment.subViews.length > 0) {\n                            v = view.receptionCompartment.subViews[0];\n                            _.defer(function () {\n                                QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                            });\n                        } else if (view.extensionPointCompartment && view.extensionPointCompartment.subViews.length > 0) {\n                            v = view.extensionPointCompartment.subViews[0];\n                            _.defer(function () {\n                                QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                            });\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            if (keyEvent.shiftKey) {\n                                _ensure(view, \'suppressOperations\', false);\n                                QuickEdit.addInQuickEdit(view.model, \'operations\', view, \'operationCompartment\', type.UMLOperation, \'Operation\');\n                            } else {\n                                if (view instanceof type.UMLEnumerationView) {\n                                    _ensure(view, \'suppressLiterals\', false);\n                                    QuickEdit.addInQuickEdit(view.model, \'literals\', view, \'enumerationLiteralCompartment\', type.UMLEnumerationLiteral, \'Literal\');\n                                } else {\n                                    _ensure(view, \'suppressAttributes\', false);\n                                    QuickEdit.addInQuickEdit(view.model, \'attributes\', view, \'attributeCompartment\', type.UMLAttribute, \'Attribute\');\n                                }\n                            }\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ATTRIBUTE:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(view.model._parent, \'attributes\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === 0) {\n                                v = view._parent._parent;\n                                _.defer(function () {\n                                    QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                });\n                            } else {\n                                QuickEdit.openUpInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(view.model._parent, \'attributes\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === (view._parent.subViews.length - 1)) {\n                                v = view._parent._parent;\n                                if (v.suppressOperations !== true && v.operationCompartment && v.operationCompartment.subViews.length > 0) {\n                                    v = v.operationCompartment.subViews[0];\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                } else if (v.suppressReceptions !== true && v.receptionCompartment && v.receptionCompartment.subViews.length > 0) {\n                                    v = v.receptionCompartment.subViews[0];\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                }\n                            } else {\n                                QuickEdit.openDownInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model._parent, \'attributes\', view._parent._parent, \'attributeCompartment\', type.UMLAttribute, \'Attribute\');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_OPERATION:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(view.model._parent, \'operations\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === 0) {\n                                v = view._parent._parent;\n                                if (v.suppressAttributes !== true && v.attributeCompartment && v.attributeCompartment.subViews.length > 0) {\n                                    v = v.attributeCompartment.subViews[v.attributeCompartment.subViews.length - 1];\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                } else {\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                }\n                            } else {\n                                QuickEdit.openUpInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(view.model._parent, \'operations\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === (view._parent.subViews.length - 1)) {\n                                v = view._parent._parent;\n                                if (v.suppressReceptions !== true && v.receptionCompartment && v.receptionCompartment.subViews.length > 0) {\n                                    v = v.receptionCompartment.subViews[0];\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                }\n                            } else {\n                                QuickEdit.openDownInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model._parent, \'operations\', view._parent._parent, \'operationCompartment\', type.UMLOperation, \'Operation\');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_RECEPTION:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(view.model._parent, \'receptions\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === 0) {\n                                v = view._parent._parent;\n                                if (v.suppressOperations !== true && v.operationCompartment && v.operationCompartment.subViews.length > 0) {\n                                    v = v.operationCompartment.subViews[v.operationCompartment.subViews.length - 1];\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                } else if (v.suppressAttributes !== true && v.attributeCompartment && v.attributeCompartment.subViews.length > 0) {\n                                    v = v.attributeCompartment.subViews[v.attributeCompartment.subViews.length - 1];\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                } else {\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                }\n                            } else {\n                                QuickEdit.openUpInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(view.model._parent, \'receptions\', view);\n                        } else {\n                            QuickEdit.openDownInQuickEdit(view);\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model._parent, \'receptions\', view._parent._parent, \'receptionCompartment\', type.UMLReception, \'Reception\');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_TEMPLATE_PARAMETER:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(view.model._parent, \'templateParameters\', view);\n                        } else {\n                            QuickEdit.openUpInQuickEdit(view);\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(view.model._parent, \'templateParameters\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === (view._parent.subViews.length - 1)) {\n                                v = view._parent._parent;\n                                _.defer(function () {\n                                    QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                });\n                            } else {\n                                QuickEdit.openDownInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model._parent, \'templateParameters\', view._parent._parent, \'templateParameterCompartment\', type.UMLTemplateParameter, \'TemplateParameter\');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_QUALIFIER:\n                    var asso      = view._parent._parent,\n                        comp      = view._parent,\n                        end       = null,\n                        compField = null;\n                    if (asso.tailQualifiersCompartment == comp) {\n                        end = asso.model.end1;\n                        compField = "tailQualifiersCompartment";\n                    } else if (asso.headQualifiersCompartment == comp) {\n                        end = asso.model.end2;\n                        compField = "headQualifiersCompartment";\n                    }\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(end, \'qualifiers\', view);\n                        } else {\n                            QuickEdit.openUpInQuickEdit(view);\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(end, \'qualifiers\', view);\n                        } else {\n                            QuickEdit.openDownInQuickEdit(view);\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(end, \'qualifiers\', asso, compField, type.UMLAttribute, \'Qualifier\');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_LITERAL:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(view.model._parent, \'literals\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === 0) {\n                                v = view._parent._parent;\n                                _.defer(function () {\n                                    QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                });\n                            } else {\n                                QuickEdit.openUpInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(view.model._parent, \'literals\', view);\n                        } else {\n                            QuickEdit.openDownInQuickEdit(view);\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model._parent, \'literals\', view._parent._parent, \'enumerationLiteralCompartment\', type.UMLEnumerationLiteral, \'Literal\');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_EXTENSION_POINT:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(view.model._parent, \'extensionPoints\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === 0) {\n                                v = view._parent._parent;\n                                _.defer(function () {\n                                    QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                });\n                            } else {\n                                QuickEdit.openUpInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(view.model._parent, \'extensionPoints\', view);\n                        } else {\n                            QuickEdit.openDownInQuickEdit(view);\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model._parent, \'extensionPoints\', view._parent._parent, \'extensionPointCompartment\', type.UMLExtensionPoint, \'ExtensionPoint\');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION:\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION_TAIL:\n                    break;\n                case UMLQuickEdit.QUICKEDIT_ASSOCIATION_HEAD:\n                    break;\n                case UMLQuickEdit.QUICKEDIT_OBJECT:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (view.slotCompartment && view.slotCompartment.visible === true && view.slotCompartment.subViews.length > 0) {\n                            v = view.slotCompartment.subViews[0];\n                            _.defer(function () {\n                                QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                            });\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model, \'slots\', view, \'slotCompartment\', type.UMLSlot, \'Slot\');\n                        }\n                        break;\n                    }\n                    break;\n                case UMLQuickEdit.QUICKEDIT_SLOT:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(view.model._parent, \'slots\', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === 0) {\n                                v = view._parent._parent;\n                                _.defer(function () {\n                                    QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                });\n                            } else {\n                                QuickEdit.openUpInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(view.model._parent, \'slots\', view);\n                        } else {\n                            QuickEdit.openDownInQuickEdit(view);\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model._parent, \'slots\', view._parent._parent, \'slotCompartment\', type.UMLSlot, \'Slot\');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    function setupDiagramManager() {\n        // 다이어그램 영역에서 Views들을 이동하면 containerView를 변경할 때.\n        $(DiagramManager).on(\'containerViewChanged\', function (event, views, dx, dy, containerView) {\n            try {\n                var diagram = DiagramManager.getEditor().diagram;\n                var containerModel = null;\n                if (containerView) {\n                    containerModel = containerView.model;\n                } else {\n                    if (diagram instanceof type.UMLStatechartDiagram) {\n                        containerModel = diagram._parent.regions[0];\n                    } else {\n                        containerModel = diagram._parent;\n                    }\n                }\n                Engine.moveViewsChangingContainer(DiagramManager.getEditor(), views, dx, dy, containerView, containerModel);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on(\'workingDiagramRemove\', function (event, diagram) {\n        });\n        $(DiagramManager).on(\'edgeReconnected\', function (event, edge, points, newParticipant, isTailSide) {\n            try {\n                if (edge.canConnectTo(newParticipant, isTailSide)) {\n                    if (edge instanceof type.UMLSeqMessageView && newParticipant instanceof type.UMLSeqLifelineView) {\n                        Engine.reconnectEdge(DiagramManager.getEditor(), edge, points, newParticipant.linePart, isTailSide);\n                    } else {\n                        Engine.reconnectEdge(DiagramManager.getEditor(), edge, points, newParticipant, isTailSide);\n                    }\n                } else {\n                    if (edge.model && newParticipant.model) {\n                        Toast.error(edge.model.getDisplayClassName() + " cannot be connected to " + newParticipant.model.getDisplayClassName());\n                    } else {\n                        Toast.error("Invalid Connection");\n                    }\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    function _toSingleLine(text) {\n        return text.trim().split("\\n").join("");\n    }\n\n    function loadStereotypeIcons() {\n        var css = [];\n        var stereotypes = Repository.findAll(function (e) {\n            return (e instanceof type.UMLStereotype && e.icon && e.icon.smallIcon && e.icon.smallIcon.trim().length > 0);\n        });\n        stereotypes.forEach(function (elem) {\n            if (elem.icon.smallIcon.indexOf("\'") < 0) {\n                css.push("." + elem.getIconClass() + " { background-image: url(\'data:image/svg+xml," + _toSingleLine(elem.icon.smallIcon) + "\') !important; background-repeat: no-repeat; background-position: 0px 0px; width: 16px; height: 16px; background-clip: content-box; }");\n            } else {\n                console.error("Stereotype\'s smallIcon should not have `\'` (single quote) character.");\n            }\n        });\n        $("#uml-stereotype-small-icons").remove();\n        if (css.length > 0) {\n            return $("<style id=\'uml-stereotype-small-icons\'>").text(css.join("\\n")).appendTo("head")[0];\n        }\n    }\n\n    function setupProjectManager() {\n        $(ProjectManager).on(\'projectLoaded\', function (event, filename, project) {\n            try {\n                loadStereotypeIcons();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(ProjectManager).on(\'imported\', function (event, filename, elem) {\n            try {\n                loadStereotypeIcons();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    AppInit.htmlReady(function () {\n        setupQuickEdit();\n        setupDiagramManager();\n        setupProjectManager();\n    });\n\n});\n\n//# sourceURL=/uml/main.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, browser: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, app*/\n\ndefine(\'erd/ERD-meta\',[\'require\',\'exports\',\'module\',\'core/MetaModelManager\'],function (require, exports, module) {\n    "use strict";\n\n    var MetaModelManager  = require("core/MetaModelManager");\n\n    var metaModel = {\n        "ERDElement": {\n            "kind": "class",\n            "super": "ExtensibleModel"\n        },\n        "ERDDataModel": {\n            "kind": "class",\n            "super": "ERDElement"\n        },\n        "ERDDiagram": {\n            "kind": "class",\n            "super": "Diagram",\n            "views": [\n                "ERDEntityView",\n                "ERDRelationshipView",\n            ]\n        },\n        "ERDColumn": {\n            "kind": "class",\n            "super": "ERDElement",\n            "attributes": [\n                { "name": "type",        "kind": "prim", "type": "String",    "visible": true, "options": [ "VARCHAR", "BOOLEAN", "INTEGER", "CHAR", "BINARY", "VARBINARY", "BLOB", "TEXT", "SMALLINT", "BIGINT", "DECIMAL", "NUMERIC", "FLOAT", "DOUBLE", "BIT", "DATE", "TIME", "DATETIME", "TIMESTAMP", "GEOMETRY", "POINT", "LINESTRING", "POLYGON" ] },\n                { "name": "length",      "kind": "prim", "type": "String",    "visible": true },\n                { "name": "primaryKey",  "kind": "prim", "type": "Boolean",   "visible": true },\n                { "name": "foreignKey",  "kind": "prim", "type": "Boolean",   "visible": true },\n                { "name": "referenceTo", "kind": "ref",  "type": "ERDColumn", "visible": true },\n                { "name": "nullable",    "kind": "prim", "type": "Boolean",   "visible": true },\n                { "name": "unique",      "kind": "prim", "type": "Boolean",   "visible": true }\n            ]\n        },\n        "ERDEntity": {\n            "kind": "class",\n            "super": "ERDElement",\n            "view": "ERDEntityView",\n            "attributes": [\n                { "name": "columns", "kind": "objs", "type": "ERDColumn" }\n            ],\n            "ordering": 1200\n        },\n        "ERDRelationshipEnd": {\n            "kind": "class",\n            "super": "RelationshipEnd",\n            "attributes": [\n                { "name": "cardinality", "kind": "prim", "type": "String", "visible": true, "default": "1", "options": [ "0..1", "1", "0..*", "1..*" ] }\n            ],\n            "ordering": 1800\n        },\n        "ERDRelationship": {\n            "kind": "class",\n            "super": "UndirectedRelationship",\n            "view": "ERDRelationshipView",\n            "attributes": [\n                { "name": "identifying", "kind": "prim", "type": "Boolean", "visible": true, "default": true }\n            ],\n            "ordering": 1801\n        },\n        "ERDColumnView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "ERDColumnCompartmentView": {\n            "kind": "class",\n            "super": "NodeView"\n        },\n        "ERDEntityView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "nameLabel",         "kind": "ref", "type": "LabelView", "embedded": "subViews" },\n                { "name": "columnCompartment", "kind": "ref", "type": "ERDColumnCompartmentView", "embedded": "subViews" }\n            ]\n        },\n        "ERDRelationshipView": {\n            "kind": "class",\n            "super": "EdgeView",\n            "attributes": [\n                { "name": "nameLabel",     "kind": "ref",  "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "tailNameLabel", "kind": "ref",  "type": "EdgeLabelView", "embedded": "subViews" },\n                { "name": "headNameLabel", "kind": "ref",  "type": "EdgeLabelView", "embedded": "subViews" }\n            ]\n        }\n    };\n\n    // Register Meta-model\n    MetaModelManager.register(metaModel);\n\n});\n\n//# sourceURL=/erd/ERD-meta.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, loopfunc: true */\n/*global define, $, _, window, app, type, appshell, document */\n\ndefine(\'erd/ERD\',[\'require\',\'exports\',\'module\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Core\',\'core/MetaModelManager\',\'core/Repository\',\'core/PreferenceManager\',\'erd/ERD-meta\'],function (require, exports, module) {\n    "use strict";\n\n    var Graphics          = require("core/Graphics"),\n        Point             = require("core/Graphics").Point,\n        Rect              = require("core/Graphics").Rect,\n        Core              = require("core/Core"),\n        MetaModelManager  = require("core/MetaModelManager"),\n        Repository        = require("core/Repository"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    require("erd/ERD-meta");\n\n\n    /**\n     * ERDElement\n     * @constructor\n     */\n    function ERDElement() {\n        type.ExtensibleModel.apply(this, arguments);\n    }\n    // inherits from ExtensibleModel\n    ERDElement.prototype = Object.create(type.ExtensibleModel.prototype);\n    ERDElement.prototype.constructor = ERDElement;\n\n    ERDElement.prototype.getDisplayClassName = function () {\n        var name = this.getClassName();\n        return name.substring(3, name.length);\n    };\n\n    /**\n     * ERDDataModel\n     * @constructor\n     */\n    function ERDDataModel() {\n        ERDElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    ERDDataModel.prototype = Object.create(ERDElement.prototype);\n    ERDDataModel.prototype.constructor = ERDDataModel;\n\n    /**\n     * ERDDiagram\n     * @constructor\n     */\n    function ERDDiagram() {\n        type.Diagram.apply(this, arguments);\n    }\n    // inherits from Diagram\n    ERDDiagram.prototype = Object.create(type.Diagram.prototype);\n    ERDDiagram.prototype.constructor = ERDDiagram;\n\n    ERDDiagram.prototype.canAcceptModel = function (model) {\n        return (model instanceof type.Hyperlink) ||\n               (model instanceof type.Diagram) ||\n               (model instanceof type.ERDEntity) ||\n               (model instanceof type.ERDRelationship);\n    };\n\n\n    /**\n     * ERDColumn\n     * @constructor\n     */\n    function ERDColumn() {\n        ERDElement.apply(this, arguments);\n\n        /** @member {string} */\n        this.type = \'\';\n\n        /** @member {string} */\n        this.length = \'\';\n\n        /** @member {boolean} */\n        this.primaryKey = false;\n\n        /** @member {boolean} */\n        this.foreignKey = false;\n\n        /** @member {ERDColumn} */\n        this.referenceTo = null;\n\n        /** @member {boolean} */\n        this.nullable = false;\n\n        /** @member {boolean} */\n        this.unique = false;\n    }\n    // inherits from FCModelElement\n    ERDColumn.prototype = Object.create(ERDElement.prototype);\n    ERDColumn.prototype.constructor = ERDColumn;\n\n    ERDColumn.prototype.getKeyString = function () {\n        var _key = this.primaryKey ? "PK" : "";\n        if (this.foreignKey) {\n            _key += (_key.length > 0) ? ",FK" : "FK";\n        }\n        if (!this.primaryKey && this.nullable) {\n            _key += (_key.length > 0) ? ",N" : "N";\n        }\n        if (!this.primaryKey && this.unique) {\n            _key += (_key.length > 0) ? ",U" : "U";\n        }\n        return _key;\n    };\n\n    ERDColumn.prototype.getNameString = function () {\n        return this.name;\n    };\n\n    ERDColumn.prototype.getTypeString = function () {\n        var _type = \'\';\n        if (this.type && this.type.length > 0) {\n            _type += this.type;\n        }\n        if (this.length || (_.isString(this.length) && this.length.length > 0)) {\n            _type += "(" + this.length + ")";\n        }\n        return _type;\n    };\n\n    ERDColumn.prototype.getString = function () {\n        return this.getKeyString() + " " + this.name + ": " + this.type;\n    };\n\n    /**\n     * ERDEntity\n     * @constructor\n     */\n    function ERDEntity() {\n        ERDElement.apply(this, arguments);\n\n        /** @member {Array.<ERDColumn>} */\n        this.columns = [];\n    }\n    // inherits from FCModelElement\n    ERDEntity.prototype = Object.create(ERDElement.prototype);\n    ERDEntity.prototype.constructor = ERDEntity;\n\n    /**\n     * Get all relationships\n     * @return {Array.<ERDRelationship>}\n     */\n    ERDEntity.prototype.getRelationships = function () {\n        var rels = Repository.getRelationshipsOf(this, function (r) { return (r instanceof type.ERDRelationship); });\n        return rels;\n    };\n\n    /**\n     * Get all relationship ends linked to this element or counterpart\n     * @param {boolean} counterpart Returns whether counterpart (opposite-side) relationship ends or not.\n     * @return {Array.<ERDRelationshipEnd>}\n     */\n    ERDEntity.prototype.getRelationshipEnds = function (counterpart) {\n        var self = this,\n            rels = Repository.getRelationshipsOf(self, function (r) { return (r instanceof type.ERDRelationship); }),\n            ends = _.map(rels, function (r) {\n                if (counterpart === true) {\n                    return (r.end1.reference === self ? r.end2 : r.end1);\n                } else {\n                    return (r.end1.reference === self ? r.end1 : r.end2);\n                }\n            });\n        return ends;\n    };\n\n    /**\n     * ERDRelationshipEnd\n     * @constructor\n     */\n    function ERDRelationshipEnd() {\n        type.RelationshipEnd.apply(this, arguments);\n\n        /** @member {string} */\n        this.cardinality = \'1\';\n    }\n    // inherits from RelationshipEnd\n    ERDRelationshipEnd.prototype = Object.create(type.RelationshipEnd.prototype);\n    ERDRelationshipEnd.prototype.constructor = ERDRelationshipEnd;\n\n\n    /**\n     * ERDRelationship\n     * @constructor\n     */\n    function ERDRelationship() {\n        type.UndirectedRelationship.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.identifying = true;\n\n        /** @member {ERDRelationshipEnd} */\n        this.end1 = new ERDRelationshipEnd();\n        this.end1._parent = this;\n\n        /** @member {ERDRelationshipEnd} */\n        this.end2 = new ERDRelationshipEnd();\n        this.end2._parent = this;\n    }\n    // inherits from UndirectedRelationship\n    ERDRelationship.prototype = Object.create(type.UndirectedRelationship.prototype);\n    ERDRelationship.prototype.constructor = ERDRelationship;\n\n    /**************************************************************************\n     *                                                                        *\n     *                              VIEW ELEMENTS                             *\n     *                                                                        *\n     **************************************************************************/\n\n    var SHADOW_OFFSET = 7,\n        SHADOW_ALPHA = 0.2,\n        SHADOW_COLOR = Graphics.Color.LIGHT_GRAY;\n\n    var COMPARTMENT_ITEM_INTERVAL = 2,\n        COMPARTMENT_LEFT_PADDING = 5,\n        COMPARTMENT_RIGHT_PADDING = 5,\n        COMPARTMENT_TOP_PADDING = 5,\n        COMPARTMENT_BOTTOM_PADDING = 5;\n\n    /**\n     * ERDColumnView\n     * @constructor\n     * @extends LabelView\n     */\n    function ERDColumnView() {\n        type.LabelView.apply(this, arguments);\n        this.horizontalAlignment = Graphics.AL_LEFT;\n        this.selectable = Core.SK_YES;\n        this.sizable = Core.SZ_NONE;\n        this.movable = Core.MM_NONE;\n        this.parentStyle = true;\n\n        this._nameOffset = 0;\n        this._typeOffset = 0;\n        this._width      = 0;\n    }\n    // inherits from LabelView\n    ERDColumnView.prototype = Object.create(type.LabelView.prototype);\n    ERDColumnView.prototype.constructor = ERDColumnView;\n\n    ERDColumnView.prototype.update = function (canvas) {\n        type.LabelView.prototype.update.call(this, canvas);\n    };\n\n    ERDColumnView.prototype.size = function (canvas) {\n        type.LabelView.prototype.size.call(this, canvas);\n        this.minWidth = this._width;\n        this.height = this.minHeight;\n    };\n\n    ERDColumnView.prototype.draw = function (canvas) {\n        this.assignStyleToCanvas(canvas);\n        if (this.model) {\n            canvas.textOut(this.left, this.top, this.model.getKeyString());\n            canvas.textOut(this.left + this._nameOffset + COMPARTMENT_LEFT_PADDING, this.top, this.model.getNameString());\n            canvas.textOut(this.left + this._typeOffset + COMPARTMENT_LEFT_PADDING, this.top, this.model.getTypeString());\n        }\n        type.View.prototype.draw.call(this, canvas);\n    };\n\n    /**\n     * ERDColumnCompartmentView\n     * @constructor\n     * @extends NodeView\n     */\n    function ERDColumnCompartmentView() {\n        type.NodeView.apply(this, arguments);\n        this.selectable = Core.SK_PROPAGATE;\n        this.parentStyle = true;\n\n        this._nameOffset = 0;\n        this._typeOffset = 0;\n    }\n    // inherits from NodeView\n    ERDColumnCompartmentView.prototype = Object.create(type.NodeView.prototype);\n    ERDColumnCompartmentView.prototype.constructor = ERDColumnCompartmentView;\n\n    ERDColumnCompartmentView.prototype.update = function (canvas) {\n        if (this.model.columns) {\n            var i, len, tempViews = this.subViews;\n            this.subViews = [];\n            for (i = 0, len = this.model.columns.length; i < len; i++) {\n                var column = this.model.columns[i];\n                var columnView = _.find(tempViews, function (v) { return v.model === column; });\n                if (!columnView) {\n                    columnView = new ERDColumnView();\n                    columnView.model = column;\n                    columnView._parent = this;\n                    // Insert columnView to subViewsby bypass command.\n                    Repository.bypassInsert(this, \'subViews\', columnView);\n                } else {\n                    this.addSubView(columnView);\n                }\n                columnView.setup(canvas);\n            }\n        }\n        type.NodeView.prototype.update.call(this, canvas);\n    };\n\n\n    ERDColumnCompartmentView.prototype.size = function (canvas) {\n        var i,\n            len,\n            item,\n            _keyWidth = 0,\n            _nameWidth = 0,\n            _typeWidth = 0;\n\n        // Compute min-width of key, name, and type column\n        var _key, _name, _type;\n        for (i = 0, len = this.subViews.length; i < len; i++) {\n            item = this.subViews[i];\n            if (item.visible && item.model) {\n                _key  = canvas.textExtent(item.model.getKeyString()).x;\n                _name = canvas.textExtent(item.model.getNameString()).x;\n                _type = canvas.textExtent(item.model.getTypeString()).x;\n                _keyWidth  = Math.max(_keyWidth, _key);\n                _nameWidth = Math.max(_nameWidth, _name);\n                _typeWidth = Math.max(_typeWidth, _type);\n            }\n        }\n        this._nameOffset = _keyWidth + COMPARTMENT_RIGHT_PADDING;\n        this._typeOffset = this._nameOffset + COMPARTMENT_LEFT_PADDING + _nameWidth + COMPARTMENT_RIGHT_PADDING;\n\n        // Compute size\n        var w = 0,\n            h = 0;\n        for (i = 0, len = this.subViews.length; i < len; i++) {\n            item = this.subViews[i];\n            item._nameOffset = this._nameOffset;\n            item._typeOffset = this._typeOffset;\n            item._width      = this._typeOffset + COMPARTMENT_LEFT_PADDING + _typeWidth;\n            if (item.parentStyle) {\n                item.font.size = item._parent.font.size;\n            }\n            item.size(canvas);\n            if (item.visible) {\n                if (w < item.minWidth) {\n                    w = item.minWidth;\n                }\n                if (i > 0) {\n                    h += COMPARTMENT_ITEM_INTERVAL;\n                }\n                h += item.minHeight;\n            }\n        }\n        this.minWidth = w + COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING;\n        this.minHeight = h + COMPARTMENT_TOP_PADDING + COMPARTMENT_BOTTOM_PADDING;\n        this.sizeConstraints();\n    };\n\n    ERDColumnCompartmentView.prototype.arrange = function (canvas) {\n        var i,\n            len,\n            item,\n            _keyWidth = 0,\n            _nameWidth = 0,\n            _typeWidth = 0;\n\n        // Compute min-width of key, name, and type column\n        var _key, _name, _type;\n        for (i = 0, len = this.subViews.length; i < len; i++) {\n            item = this.subViews[i];\n            if (item.visible && item.model) {\n                _key  = canvas.textExtent(item.model.getKeyString()).x;\n                _name = canvas.textExtent(item.model.getNameString()).x;\n                _type = canvas.textExtent(item.model.getTypeString()).x;\n                _keyWidth  = Math.max(_keyWidth, _key);\n                _nameWidth = Math.max(_nameWidth, _name);\n                _typeWidth = Math.max(_typeWidth, _type);\n            }\n        }\n\n        var h = COMPARTMENT_TOP_PADDING;\n        for (i = 0, len = this.subViews.length; i < len; i++) {\n            item = this.subViews[i];\n            if (item.visible) {\n                if (i > 0) { h += COMPARTMENT_ITEM_INTERVAL; }\n                item.left = this.left + COMPARTMENT_LEFT_PADDING;\n                item.top = this.top + h;\n                item.width = this.width - COMPARTMENT_LEFT_PADDING - COMPARTMENT_RIGHT_PADDING;\n                h += item.height;\n            }\n            item.arrange(canvas);\n        }\n        h += COMPARTMENT_BOTTOM_PADDING;\n        this.height = h;\n        this.sizeConstraints();\n    };\n\n    /**\n     * ERDEntityView\n     * @constructor\n     */\n    function ERDEntityView() {\n        type.NodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("erd.entity.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n\n        /** @member {LabelView} */\n        this.nameLabel = new type.LabelView();\n        this.nameLabel.horizontalAlignment = Graphics.AL_CENTER;\n        this.nameLabel.parentStyle = true;\n        this.addSubView(this.nameLabel);\n\n        /** @member {ERDColumnCompartmentView} */\n        this.columnCompartment = new ERDColumnCompartmentView();\n        this.columnCompartment.parentStyle = true;\n        this.addSubView(this.columnCompartment);\n    }\n    // inherits from NodeView\n    ERDEntityView.prototype = Object.create(type.NodeView.prototype);\n    ERDEntityView.prototype.constructor = ERDEntityView;\n\n    ERDEntityView.prototype.update = function (canvas) {\n        // Assign this.model to columnCompartment.model by bypass command.\n        if (this.columnCompartment.model !== this.model) {\n            Repository.bypassFieldAssign(this.columnCompartment, \'model\', this.model);\n        }\n        if (this.model) {\n            this.nameLabel.text = this.model.name;\n            this.nameLabel.font.style = Graphics.FS_BOLD;\n        }\n        type.NodeView.prototype.update.call(this, canvas);\n    };\n\n    ERDEntityView.prototype.sizeObject = function (canvas) {\n        type.NodeView.prototype.sizeObject.call(this, canvas);\n        var _h = 0, _w = 0;\n        _h += COMPARTMENT_TOP_PADDING + this.nameLabel.minHeight + COMPARTMENT_BOTTOM_PADDING;\n        _h += this.columnCompartment.minHeight;\n        _w = Math.max(this.nameLabel.minWidth + COMPARTMENT_LEFT_PADDING + COMPARTMENT_RIGHT_PADDING, this.columnCompartment.minWidth);\n        this.minHeight = _h;\n        this.minWidth = _w;\n    };\n\n    ERDEntityView.prototype.arrangeObject = function (canvas) {\n        var _y = this.top + COMPARTMENT_TOP_PADDING;\n        this.nameLabel.top = _y;\n        this.nameLabel.left = this.left;\n        this.nameLabel.width = this.width;\n        this.nameLabel.height = this.nameLabel.minHeight;\n        _y += this.nameLabel.height + COMPARTMENT_BOTTOM_PADDING;\n        this.columnCompartment.top = _y;\n        this.columnCompartment.left = this.left;\n        this.columnCompartment.width = this.width;\n        type.NodeView.prototype.arrangeObject.call(this, canvas);\n    };\n\n    ERDEntityView.prototype.drawShadow = function (canvas) {\n        canvas.storeState();\n        canvas.alpha = SHADOW_ALPHA;\n        canvas.fillColor = SHADOW_COLOR;\n        canvas.fillRect(\n            this.left + SHADOW_OFFSET,\n            this.top + SHADOW_OFFSET,\n            this.getRight() + SHADOW_OFFSET,\n            this.getBottom() + SHADOW_OFFSET\n        );\n        canvas.restoreState();\n    };\n\n    ERDEntityView.prototype.drawObject = function (canvas) {\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        if (this.columnCompartment.subViews.length > 0) {\n            canvas.line(this.left, this.columnCompartment.top, this.getRight(), this.columnCompartment.top);\n            var _x1 = this.left + COMPARTMENT_LEFT_PADDING + this.columnCompartment._nameOffset,\n                _x2 = this.left + COMPARTMENT_LEFT_PADDING + this.columnCompartment._typeOffset;\n            canvas.line(_x1, this.columnCompartment.top, _x1, this.getBottom());\n            canvas.line(_x2, this.columnCompartment.top, _x2, this.getBottom());\n        }\n        type.NodeView.prototype.drawObject.call(this, canvas);\n    };\n\n    /**\n     * ERDRelationshipView\n     * @constructor\n     * @extends EdgeView\n     */\n    function ERDRelationshipView() {\n        type.EdgeView.apply(this, arguments);\n        this.lineStyle = PreferenceManager.get("erd.relationship.lineStyle", Core.LS_ROUNDRECT) || PreferenceManager.get("view.lineStyle", Core.LS_OBLIQUE);\n\n        this.headEndStyle = Core.ES_CROWFOOT_ONE;\n        this.tailEndStyle = Core.ES_CROWFOOT_ONE;\n\n        /** @member {EdgeLabelView} */\n        this.nameLabel = new type.EdgeLabelView();\n        this.nameLabel.hostEdge = this;\n        this.nameLabel.edgePosition = Core.EP_MIDDLE;\n        this.nameLabel.distance = 15;\n        this.nameLabel.alpha = Math.PI / 2;\n        this.addSubView(this.nameLabel);\n\n        /** @member {EdgeLabelView} */\n        this.tailNameLabel = new type.EdgeLabelView();\n        this.tailNameLabel.hostEdge = this;\n        this.tailNameLabel.edgePosition = Core.EP_TAIL;\n        this.tailNameLabel.alpha = Math.PI / 6;\n        this.tailNameLabel.distance = 30;\n        this.addSubView(this.tailNameLabel);\n\n        /** @member {EdgeLabelView} */\n        this.headNameLabel = new type.EdgeLabelView();\n        this.headNameLabel.hostEdge = this;\n        this.headNameLabel.edgePosition = Core.EP_HEAD;\n        this.headNameLabel.alpha = -Math.PI / 6;\n        this.headNameLabel.distance = 30;\n        this.addSubView(this.headNameLabel);\n    }\n    // inherits from EdgeView\n    ERDRelationshipView.prototype = Object.create(type.EdgeView.prototype);\n    ERDRelationshipView.prototype.constructor = ERDRelationshipView;\n\n    ERDRelationshipView.prototype.update = function (canvas) {\n        if (this.model) {\n            this.nameLabel.visible = (this.model.name.length > 0);\n            this.nameLabel.text = this.model.name;\n            if (this.model.end1) {\n                this.tailNameLabel.text = this.model.end1.name;\n                switch (this.model.end1.cardinality) {\n                case "0..1":\n                    this.tailEndStyle = Core.ES_CROWFOOT_ZERO_ONE;\n                    break;\n                case "1":\n                    this.tailEndStyle = Core.ES_CROWFOOT_ONE;\n                    break;\n                case "0..*":\n                    this.tailEndStyle = Core.ES_CROWFOOT_ZERO_MANY;\n                    break;\n                case "1..*":\n                    this.tailEndStyle = Core.ES_CROWFOOT_MANY;\n                    break;\n                default:\n                    this.tailEndStyle = Core.ES_CROWFOOT_ONE;\n                }\n            }\n            if (this.model.end2) {\n                this.headNameLabel.text = this.model.end2.name;\n                switch (this.model.end2.cardinality) {\n                case "0..1":\n                    this.headEndStyle = Core.ES_CROWFOOT_ZERO_ONE;\n                    break;\n                case "1":\n                    this.headEndStyle = Core.ES_CROWFOOT_ONE;\n                    break;\n                case "0..*":\n                    this.headEndStyle = Core.ES_CROWFOOT_ZERO_MANY;\n                    break;\n                case "1..*":\n                    this.headEndStyle = Core.ES_CROWFOOT_MANY;\n                    break;\n                default:\n                    this.headEndStyle = Core.ES_CROWFOOT_ONE;\n                }\n            }\n            if (this.model.identifying) {\n                this.lineMode = Core.LM_SOLID;\n            } else {\n                this.lineMode = Core.LM_DOT;\n            }\n        }\n        type.EdgeView.prototype.update.call(this, canvas);\n    };\n\n\n    /* ************************** Type definitions ***************************/\n\n    type.ERDElement         = ERDElement;\n    type.ERDDataModel       = ERDDataModel;\n    type.ERDDiagram         = ERDDiagram;\n    type.ERDColumn          = ERDColumn;\n    type.ERDEntity          = ERDEntity;\n    type.ERDRelationshipEnd = ERDRelationshipEnd;\n    type.ERDRelationship    = ERDRelationship;\n\n    type.ERDColumnView            = ERDColumnView;\n    type.ERDColumnCompartmentView = ERDColumnCompartmentView;\n    type.ERDEntityView            = ERDEntityView;\n    type.ERDRelationshipView      = ERDRelationshipView;\n\n});\n\n//# sourceURL=/erd/ERD.js'),eval('define(\'text!grammars/erd-column.pegjs\',[],function () { return \'/**\\n * Grammar for PEG.js\\n * ERD Column Expression Grammar\\n * e.g.) name : type(length)\\n */\\n\\nstart\\n    = _ name:identifier _ type:type?\\n    {\\n        var ast = { name: name };\\n        if (type) { ast.type = type; }\\n        return ast;\\n    }\\n\\ntype\\n    = ":" _ typeName:identifier _ typeSize:typeSize?\\n    {\\n        var ast = {};\\n        if (typeName) { ast.name = typeName; }\\n        if (typeSize) { ast.size = typeSize; }\\n        return ast;\\n    }\\n\\ntypeSize\\n    = "(" _ number:number _ ")"\\n    {\\n        return number;\\n    }\\n\\n\\nnumber\\n    = val:[0-9,.-:~]+ { return val.join(""); }\\n\\nidentifier\\n    = id:[ 0-9A-Za-z`!@$%^&*_|{};.?\\\\[\\\\]"\\\\u00a0-\\\\ufff0]+ { return id.join(""); }\\n\\n_   // whitespace\\n    = [ \\\\t\\\\n\\\\r]*\\n\';});\n\n//# sourceURL=/text!grammars/erd-column.pegjs'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, PEG */\ndefine(\'erd/ERDUtils\',[\'require\',\'exports\',\'module\',\'core/Core\',\'core/Repository\',\'core/OperationBuilder\',\'engine/Engine\',\'erd/ERD\',\'text!grammars/erd-column.pegjs\'],function (require, exports, module) {\n    "use strict";\n\n    var Core                     = require("core/Core"),\n        Repository               = require("core/Repository"),\n        OperationBuilder         = require("core/OperationBuilder"),\n        Engine                   = require("engine/Engine"),\n        ERD                      = require("erd/ERD");\n\n    var GrammarColumn            = require("text!grammars/erd-column.pegjs");\n\n    function parseColumnExpression(expr) {\n        var parser = PEG.buildParser(GrammarColumn);\n        var ast = parser.parse(expr);\n        return ast;\n    }\n\n    function getColumnExpression(elem) {\n        var str = elem.name;\n        if (elem.type && elem.type.length > 0) {\n            str = str + ": " + elem.type;\n            if (elem.length) {\n                str = str + "(" + elem.length + ")";\n            }\n        }\n        return str;\n    }\n\n    function setColumnExpression(elem, ast) {\n        var fields = {};\n        // name\n        if (ast.name) {\n            fields.name = ast.name.trim();\n        }\n        // type\n        if (ast.type) {\n            if (ast.type.name) {\n                fields.type = ast.type.name.trim();\n            } else {\n                fields.type = "";\n            }\n            if (ast.type.size) {\n                fields.length = ast.type.size;\n            } else {\n                fields.length = 0;\n            }\n        } else {\n            fields.type = "";\n            fields.length = 0;\n        }\n        Engine.setProperties(elem, fields);\n    }\n\n    exports.parseColumnExpression = parseColumnExpression;\n    exports.getColumnExpression   = getColumnExpression;\n    exports.setColumnExpression   = setColumnExpression;\n\n});\n\n//# sourceURL=/erd/ERDUtils.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, type */\n\ndefine('erd/ERDQuickEdit',['require','exports','module','ui/QuickEdit','core/Graphics','core/Core','erd/ERDUtils'],function (require, exports, module) {\n    \"use strict\";\n\n    var QuickEdit  = require(\"ui/QuickEdit\"),\n        Graphics   = require(\"core/Graphics\"),\n        Core       = require(\"core/Core\"),\n        ERDUtils   = require(\"erd/ERDUtils\");\n\n    /**\n     * @const\n     * QuickEdit IDs\n     */\n    var QUICKEDIT_ENTITY               = 'erd-entity',\n        QUICKEDIT_COLUMN               = 'erd-column',\n        QUICKEDIT_RELATIONSHIP         = 'erd-relationship',\n        QUICKEDIT_RELATIONSHIP_HEAD    = 'erd-relationship-head',\n        QUICKEDIT_RELATIONSHIP_TAIL    = 'erd-relationship-tail';\n\n    /**\n     * @const\n     * QuickEdit Item IDs\n     */\n    var ITEM_EDIT             = 'edit',\n        ITEM_ADD              = 'add',\n        ITEM_DELETE           = 'delete',\n        ITEM_MOVEUP           = 'moveup',\n        ITEM_MOVEDOWN         = 'movedown',\n        ITEM_ADD_NOTE         = 'add-note',\n        ITEM_ADD_COLUMN       = 'add-column',\n        ITEM_CARDINALITY      = 'cardinality',\n        ITEM_IDENTIFYING      = 'identifying',\n        ITEM_PRIMARY_KEY      = 'primary-key',\n        ITEM_ADD_ONE_TO_ONE   = 'add-one-to-one',\n        ITEM_ADD_ONE_TO_MANY  = 'add-one-to-many',\n        ITEM_ADD_MANY_TO_MANY = 'add-many-to-many';\n\n\n    /* Quick Edit Buttons */\n    var QE_ADD              = { type: 'button', id: ITEM_ADD,        icon: 'quick-edit-icon-add',        hint: 'Add',        key: 'Ctrl-Enter' },\n        QE_DELETE           = { type: 'button', id: ITEM_DELETE,     icon: 'quick-edit-icon-delete',     hint: 'Delete',     key: 'Ctrl-Delete' },\n        QE_MOVEUP           = { type: 'button', id: ITEM_MOVEUP,     icon: 'quick-edit-icon-moveup',     hint: 'Move Up',    key: 'Ctrl-Up' },\n        QE_MOVEDOWN         = { type: 'button', id: ITEM_MOVEDOWN,   icon: 'quick-edit-icon-movedown',   hint: 'Move Down',  key: 'Ctrl-Down' },\n        QE_ADD_NOTE         = { type: 'button', id: ITEM_ADD_NOTE,   icon: 'quick-edit-icon-add-note',   hint: 'Add Note' },\n        QE_ADD_COLUMN       = { type: 'button', id: ITEM_ADD_COLUMN, icon: 'quick-edit-icon-erd-column', hint: 'Add Column', key: 'Ctrl-Enter' },\n        QE_ADD_ONE_TO_ONE   = { type: 'button', id: ITEM_ADD_ONE_TO_ONE,   icon: 'quick-edit-icon-erd-add-one-to-one',   hint: 'Add One to One' },\n        QE_ADD_ONE_TO_MANY  = { type: 'button', id: ITEM_ADD_ONE_TO_MANY,  icon: 'quick-edit-icon-erd-add-one-to-many',  hint: 'Add One to Many' },\n        QE_ADD_MANY_TO_MANY = { type: 'button', id: ITEM_ADD_MANY_TO_MANY, icon: 'quick-edit-icon-erd-add-many-to-many', hint: 'Add Many to Many' };\n\n    /* Quick Edit Inputs */\n    var QE_EDIT_GENERAL = { type: 'input', id: ITEM_EDIT, hint: \"name\" },\n        QE_EDIT_COLUMN  = { type: 'input', id: ITEM_EDIT, hint: \"name : type (length)\" };\n\n    /* Quick Edit ComboBoxes */\n    var QE_CARDINALITY = {\n            type: 'dropdown',\n            id: ITEM_CARDINALITY,\n            hint: 'Cardinality',\n            dropdownWidth: 170,\n            items: [\n                { icon: 'quick-edit-icon-erd-zero-one',  text: 'Zero or One',  value: '0..1' },\n                { icon: 'quick-edit-icon-erd-one',       text: 'One',          value: '1' },\n                { icon: 'quick-edit-icon-erd-zero-many', text: 'Zero or Many', value: '0..*' },\n                { icon: 'quick-edit-icon-erd-many',      text: 'Many',         value: '1..*' }\n            ]\n        },\n        QE_IDENTIFYING = {\n            type: 'dropdown',\n            id: ITEM_IDENTIFYING,\n            hint: 'Identifying',\n            dropdownWidth: 180,\n            items: [\n                { icon: 'quick-edit-icon-erd-identifying',     text: 'Identifying',     value: true },\n                { icon: 'quick-edit-icon-erd-non-identifying', text: 'Non-Identifying', value: false }\n            ]\n        },\n        QE_PRIMARY_KEY = {\n            type: 'dropdown',\n            id: ITEM_PRIMARY_KEY,\n            hint: 'Primary Key',\n            dropdownWidth: 180,\n            items: [\n                { icon: 'quick-edit-icon-erd-column-key',    text: 'Primary Key', value: true },\n                { icon: 'quick-edit-icon-erd-column-nonkey', text: 'Not Key',     value: false }\n            ]\n        };\n\n\n    // QuickEdit for Entity\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.ERDEntityView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_ENTITY,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_COLUMN ],\n                rightSection2: [ QE_ADD_ONE_TO_ONE, QE_ADD_ONE_TO_MANY, QE_ADD_MANY_TO_MANY ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, view.model.name);\n        }\n    });\n\n    // QuickEdit for Column\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.ERDColumnView) && (view._parent instanceof type.ERDColumnCompartmentView);\n        },\n        popup: function (editor, position, view) {\n            var rect  = new Graphics.Rect(view._parent.left, view.top, view._parent.getRight(), view.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_COLUMN,\n                main: QE_EDIT_COLUMN,\n                leftSection: [ QE_PRIMARY_KEY ],\n                rightSection: [ QE_ADD, QE_DELETE, QE_MOVEUP, QE_MOVEDOWN ]\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, ERDUtils.getColumnExpression(view.model));\n            QuickEdit.setDropdownValue(ITEM_PRIMARY_KEY, view.model.primaryKey);\n        }\n    });\n\n    // QuickEdit for Relationship\n    QuickEdit.register({\n        when: function (view, x, y, pass) {\n            pass.position = 'center';\n            // when RelationshipView was clicked\n            if (view instanceof type.ERDRelationshipView) {\n                pass.position = QuickEdit.getPositionOfEdge(view.points, x, y);\n                return true;\n            }\n            // when Labels belongs to RelationshipView was clicked\n            if ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.ERDRelationshipView)) {\n                if (view === view._parent.headNameLabel) {\n                    pass.position = 'head';\n                } else if (view === view._parent.tailNameLabel) {\n                    pass.position = 'tail';\n                }\n                return true;\n            }\n            return false;\n        },\n        popup: function (editor, position, view, pass) {\n            var endView, rect;\n            switch (pass.position) {\n            case 'center':\n                endView = view.nameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_RELATIONSHIP,\n                    main: QE_EDIT_GENERAL,\n                    leftSection: [ QE_IDENTIFYING ],\n                    leftSection2: [ QE_ADD_NOTE ],\n                    rightSection: []\n                });\n                break;\n            case 'tail':\n                endView = view.tailNameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.tailNameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_RELATIONSHIP_TAIL,\n                    main: QE_EDIT_GENERAL,\n                    leftSection: [],\n                    rightSection: [ QE_CARDINALITY ]\n                });\n                break;\n            case 'head':\n                endView = view.headNameLabel;\n                if (view instanceof Core.EdgeLabelView) { endView = view._parent.headNameLabel; }\n                rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n                rect.add(position);\n                QuickEdit.render(editor, rect, endView, {\n                    id: QUICKEDIT_RELATIONSHIP_HEAD,\n                    main: QE_EDIT_GENERAL,\n                    leftSection: [],\n                    rightSection: [ QE_CARDINALITY ]\n                });\n                break;\n            }\n        },\n        init: function (view, pass) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            switch (pass.position) {\n            case 'center':\n                QuickEdit.setInputValue(ITEM_EDIT, model.name);\n                QuickEdit.setDropdownValue(ITEM_IDENTIFYING, model.identifying);\n                break;\n            case 'tail':\n                QuickEdit.setInputValue(ITEM_EDIT, model.end1.name);\n                QuickEdit.setDropdownValue(ITEM_CARDINALITY, model.end1.cardinality);\n                break;\n            case 'head':\n                QuickEdit.setInputValue(ITEM_EDIT, model.end2.name);\n                QuickEdit.setDropdownValue(ITEM_CARDINALITY, model.end2.cardinality);\n                break;\n            }\n        }\n    });\n\n\n    // Define public API\n    exports.ITEM_EDIT             = ITEM_EDIT;\n    exports.ITEM_ADD              = ITEM_ADD;\n    exports.ITEM_DELETE           = ITEM_DELETE;\n    exports.ITEM_MOVEUP           = ITEM_MOVEUP;\n    exports.ITEM_MOVEDOWN         = ITEM_MOVEDOWN;\n    exports.ITEM_ADD_NOTE         = ITEM_ADD_NOTE;\n    exports.ITEM_ADD_COLUMN       = ITEM_ADD_COLUMN;\n    exports.ITEM_CARDINALITY      = ITEM_CARDINALITY;\n    exports.ITEM_IDENTIFYING      = ITEM_IDENTIFYING;\n    exports.ITEM_PRIMARY_KEY      = ITEM_PRIMARY_KEY;\n    exports.ITEM_ADD_ONE_TO_ONE   = ITEM_ADD_ONE_TO_ONE;\n    exports.ITEM_ADD_ONE_TO_MANY  = ITEM_ADD_ONE_TO_MANY;\n    exports.ITEM_ADD_MANY_TO_MANY = ITEM_ADD_MANY_TO_MANY;\n\n    exports.QUICKEDIT_ENTITY            = QUICKEDIT_ENTITY;\n    exports.QUICKEDIT_COLUMN            = QUICKEDIT_COLUMN;\n    exports.QUICKEDIT_RELATIONSHIP      = QUICKEDIT_RELATIONSHIP;\n    exports.QUICKEDIT_RELATIONSHIP_HEAD = QUICKEDIT_RELATIONSHIP_HEAD;\n    exports.QUICKEDIT_RELATIONSHIP_TAIL = QUICKEDIT_RELATIONSHIP_TAIL;\n\n});\n\n//# sourceURL=/erd/ERDQuickEdit.js"),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, browser: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, _, type, meta*/\n\ndefine(\'erd/ERD-rules\',[\'require\',\'exports\',\'module\',\'core/Validator\'],function (require, exports, module) {\n    "use strict";\n\n    var Validator = require("core/Validator");\n\n    var rules = [\n        {\n            id: "ERD001",\n            message: "Primary key cannot be nullable.",\n            appliesTo: [ "ERDColumn" ],\n            exceptions: [],\n            constraint: function (elem) {\n                if (elem.primaryKey === true && elem.nullable === true) {\n                    return false;\n                }\n                return true;\n            }\n        },\n        {\n            id: "ERD002",\n            message: "No relationship with the foreign key reference.",\n            appliesTo: [ "ERDColumn" ],\n            exceptions: [],\n            constraint: function (elem) {\n                if (elem.foreignKey && elem.referenceTo) {\n                    var ends = elem._parent.getRelationshipEnds(true);\n                    return _.some(ends, function (e) {\n                        return e.reference === elem.referenceTo._parent;\n                    });\n                }\n                return true;\n            }\n        }\n    ];\n\n    Validator.addRules(rules);\n\n});\n\n//# sourceURL=/erd/ERD-rules.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define */\n\ndefine('erd/ERDCommands',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n\n    exports.FILE_NEW_FROM_TEMPLATE_DATAMODEL = 'file.newFromTemplate.DataModel';\n    exports.ERD_ADD_DIAGRAM                  = 'erd.add.diagram';\n    exports.ERD_ADD_MODEL                    = 'erd.add.model';\n    exports.ERD_ADD_ENTITY                   = 'erd.add.entity';\n    exports.ERD_ADD_COLUMN                   = 'erd.add.column';\n\n});\n\n//# sourceURL=/erd/ERDCommands.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell, Mustache */\n\ndefine(\'erd/ERDFactory\',[\'require\',\'exports\',\'module\',\'utils/Helper\',\'core/Core\',\'core/OperationBuilder\',\'core/Repository\',\'engine/Engine\',\'engine/Factory\',\'engine/ProjectManager\',\'erd/ERD\'],function (require, exports, module) {\n    "use strict";\n\n    var Helper           = require("utils/Helper"),\n        Core             = require("core/Core"),\n        OperationBuilder = require("core/OperationBuilder"),\n        Repository       = require("core/Repository"),\n        Engine           = require("engine/Engine"),\n        Factory          = require("engine/Factory"),\n        ProjectManager   = require("engine/ProjectManager");\n\n    require("erd/ERD");\n\n    // Preconditions ...........................................................\n\n    function entityLinkPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.ERDEntity) && (options.headModel instanceof type.ERDEntity),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    // Create ER diagram function ..............................................\n\n    function diagramFn(parent, options) {\n        var model,\n            diagram;\n        parent = parent || ProjectManager.getProject();\n        if (parent instanceof type.ERDDataModel) {\n            diagram = new type.ERDDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, diagram.getDisplayClassName());\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        } else {\n            model = new type.ERDDataModel();\n            model.name = Core.getNewName(parent.ownedElements, "Data Model");\n            model._parent = parent;\n            diagram = new type.ERDDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, diagram.getDisplayClassName());\n            model.ownedElements.push(diagram);\n            diagram._parent = model;\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', model);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n    // Create Diagram ..........................................................\n\n    Factory.registerDiagramFn("ERDDiagram", diagramFn);\n\n    // Create Model ............................................................\n\n    Factory.registerModelFn("ERDDataModel", Factory.defaultModelFn);\n    Factory.registerModelFn("ERDEntity",    Factory.defaultModelFn);\n    Factory.registerModelFn("ERDColumn",    Factory.defaultModelFn);\n\n    // Create Model And View ...................................................\n\n    Factory.registerModelAndViewFn("ERDEntity",       Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("ERDRelationship", Factory.defaultUndirectedRelationshipFn, { precondition: entityLinkPrecondition });\n\n    // Create View .............................................................\n\n    Factory.registerViewOfFn("ERDDiagram", Factory.defaultViewOnDiagramFn);\n\n});\n\n//# sourceURL=/erd/ERDFactory.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell */\ndefine(\'erd/ERDCommandHandlers\',[\'require\',\'exports\',\'module\',\'command/CommandManager\',\'core/Repository\',\'engine/SelectionManager\',\'engine/Factory\',\'command/Commands\',\'diagrams/DiagramManager\',\'explorer/ModelExplorerView\',\'file/FileUtils\',\'erd/ERDCommands\',\'erd/ERDFactory\'],function (require, exports, module) {\n    "use strict";\n\n    var CommandManager    = require("command/CommandManager"),\n        Repository        = require("core/Repository"),\n        SelectionManager  = require("engine/SelectionManager"),\n        Factory           = require("engine/Factory"),\n        Commands          = require("command/Commands"),\n        DiagramManager    = require("diagrams/DiagramManager"),\n        ModelExplorerView = require("explorer/ModelExplorerView"),\n        FileUtils         = require("file/FileUtils"),\n        ERDCommands       = require("erd/ERDCommands"),\n        ERDFactory        = require("erd/ERDFactory");\n\n    function _handleDataModelTemplate() {\n        var path = FileUtils.getApplicationDirectoryPath() + "templates/DataModel.mdj";\n        return CommandManager.execute(Commands.FILE_NEW, path);\n    }\n\n    function _updateCommands() {\n        var selected    = SelectionManager.getSelected(),\n            isNone      = (!selected),\n            isProject   = selected instanceof type.Project,\n            isPackage   = selected instanceof type.UMLPackage || selected instanceof type.UMLSubsystem,\n            isDataModel = selected instanceof type.ERDDataModel,\n            isEntity    = selected instanceof type.ERDEntity;\n\n        CommandManager.get(ERDCommands.ERD_ADD_DIAGRAM).setVisible(isNone || isProject || isPackage || isDataModel);\n        CommandManager.get(ERDCommands.ERD_ADD_MODEL).setVisible(isProject || isPackage);\n        CommandManager.get(ERDCommands.ERD_ADD_ENTITY).setVisible(isDataModel);\n        CommandManager.get(ERDCommands.ERD_ADD_COLUMN).setVisible(isEntity);\n    }\n\n    CommandManager.register("Data Model", ERDCommands.FILE_NEW_FROM_TEMPLATE_DATAMODEL, _handleDataModelTemplate);\n    CommandManager.register("ER Diagram", ERDCommands.ERD_ADD_DIAGRAM,                  _.partial(Factory.createDiagram, "ERDDiagram"));\n    CommandManager.register("Data Model", ERDCommands.ERD_ADD_MODEL,                    _.partial(Factory.createModel,   "ERDDataModel"));\n    CommandManager.register("Entity",     ERDCommands.ERD_ADD_ENTITY,                   _.partial(Factory.createModel,   "ERDEntity"));\n    CommandManager.register("Column",     ERDCommands.ERD_ADD_COLUMN,                   _.partial(Factory.createModel,   "ERDColumn",null, "columns"));\n\n    // Update Commands\n    $(SelectionManager).on("selectionChanged", _updateCommands);\n    $(Repository).on("operationExecuted", _updateCommands);\n\n});\n\n//# sourceURL=/erd/ERDCommandHandlers.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell */\n\ndefine(\'erd/ERDMenus\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'command/Commands\',\'menu/MenuManager\',\'menu/ContextMenuManager\',\'menu/DefaultMenus\',\'uml/UMLCommands\',\'erd/ERDCommands\',\'erd/ERDCommandHandlers\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit            = require("utils/AppInit"),\n        Commands           = require("command/Commands"),\n        MenuManager        = require("menu/MenuManager"),\n        ContextMenuManager = require("menu/ContextMenuManager"),\n        DefaultMenus       = require("menu/DefaultMenus"),\n        UMLCommands        = require("uml/UMLCommands");\n\n    var ERDCommands = require("erd/ERDCommands");\n    require("erd/ERDCommandHandlers");\n\n    function _addElementMenus(menu) {\n        menu.addMenuItem(ERDCommands.ERD_ADD_MODEL,   "icon-ERDDataModel");\n        menu.addMenuItem(ERDCommands.ERD_ADD_ENTITY,  "icon-ERDEntity");\n        menu.addMenuItem(ERDCommands.ERD_ADD_COLUMN,  "icon-ERDColumn");\n    }\n\n    /**\n     * Extend MenuBar\n     */\n    function setupMenus() {\n        var menu, menuItem;\n\n        // File: New From Template\n        menu = MenuManager.getMenuItem(Commands.FILE_NEW_FROM_TEMPLATE);\n        menu.addMenuItem(ERDCommands.FILE_NEW_FROM_TEMPLATE_DATAMODEL);\n\n        // Add Diagram\n        menuItem = MenuManager.getMenuItem(Commands.MODEL_ADD_DIAGRAM);\n        menuItem.addMenuItem(ERDCommands.ERD_ADD_DIAGRAM);\n\n        // Add Model\n        menuItem = MenuManager.getMenuItem(Commands.MODEL_ADD);\n        menuItem.addMenuItem(ERDCommands.ERD_ADD_MODEL);\n        menuItem.addMenuItem(ERDCommands.ERD_ADD_ENTITY);\n        menuItem.addMenuItem(ERDCommands.ERD_ADD_COLUMN);\n    }\n\n    /**\n     * Extend ContextMenu for Diagram\n     */\n    function setupContextMenuForDiagram() {\n        var contextMenu, menu, menuItem;\n        contextMenu = ContextMenuManager.getContextMenu(DefaultMenus.contextMenus.DIAGRAM);\n\n        // Add\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD);\n        _addElementMenus(menu);\n    }\n\n    /**\n     * Extend ContextMenu for Explorer\n     */\n    function setupContextMenuForExplorer() {\n        var contextMenu, menu;\n        contextMenu = ContextMenuManager.getContextMenu(DefaultMenus.contextMenus.EXPLORER);\n\n        // Add Diagram\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD_DIAGRAM);\n        menu.addMenuItem(ERDCommands.ERD_ADD_DIAGRAM, "icon-ERDDiagram");\n\n        // Add\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD);\n        _addElementMenus(menu);\n    }\n\n\n    AppInit.htmlReady(function () {\n        setupMenus();\n        setupContextMenuForDiagram();\n        setupContextMenuForExplorer();\n    });\n\n});\n\n//# sourceURL=/erd/ERDMenus.js'),eval("/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell, document */\n\ndefine('erd/ERDToolbox',['require','exports','module','utils/AppInit','core/Repository','engine/Factory','diagrams/DiagramManager','diagrams/ToolboxView','ui/QuickEdit','ui/Toast','erd/ERDFactory'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit        = require(\"utils/AppInit\"),\n        Repository     = require(\"core/Repository\"),\n        Factory        = require(\"engine/Factory\"),\n        DiagramManager = require(\"diagrams/DiagramManager\"),\n        Toolbox        = require(\"diagrams/ToolboxView\"),\n        QuickEdit      = require(\"ui/QuickEdit\"),\n        Toast          = require(\"ui/Toast\"),\n        ERDFactory     = require(\"erd/ERDFactory\");\n\n    /**\n     * Toolbox Groups\n     * @const\n     */\n    var TXG_ERD  = 'txg-erd';\n\n    /**\n     * Toolbox Items\n     * @const\n     */\n    var TX_ENTITY       = 'tx-erd-entity',\n        TX_ONE_TO_ONE   = 'tx-erd-one-to-one',\n        TX_ONE_TO_MANY  = 'tx-erd-one-to-many',\n        TX_MANY_TO_MANY = 'tx-erd-many-to-many';\n\n    function setupToolbox() {\n        Toolbox.addGroup(TXG_ERD, 'Entity-Relationship', [type.ERDDiagram]);\n        Toolbox.addItem(TX_ENTITY,       TXG_ERD, 'Entity',       'icon-ERDEntity',     'rect');\n        Toolbox.addItem(TX_ONE_TO_ONE,   TXG_ERD, 'One to One Relationship',   'icon-ERDOneToOne',   'line');\n        Toolbox.addItem(TX_ONE_TO_MANY,  TXG_ERD, 'One to Many Relationship',  'icon-ERDOneToMany',  'line');\n        Toolbox.addItem(TX_MANY_TO_MANY, TXG_ERD, 'Many to Many Relationship', 'icon-ERDManyToMany', 'line');\n\n        // Event Handling\n        $(Toolbox).on('elementCreated', function (event, id, editor, x1, y1, x2, y2) {\n            try {\n                var diagram   = editor.diagram,\n                    parent    = diagram._parent,\n                    model     = null,\n                    view      = null,\n                    tailView  = diagram.getViewAt(editor.canvas, x1, y1, true),\n                    headView  = diagram.getViewAt(editor.canvas, x2, y2, true),\n                    tailModel = tailView ? tailView.model : null,\n                    headModel = headView ? headView.model : null;\n\n                var options = {\n                        x1        : x1,\n                        y1        : y1,\n                        x2        : x2,\n                        y2        : y2,\n                        tailView  : diagram.getViewAt(editor.canvas, x1, y1, true),\n                        headView  : diagram.getViewAt(editor.canvas, x2, y2, true),\n                        tailModel : tailView ? tailView.model : null,\n                        headModel : headView ? headView.model : null\n                    };\n\n                switch (id) {\n                case TX_ENTITY:\n                    view = Factory.createModelAndView(\"ERDEntity\", parent, diagram, options);\n                    break;\n                case TX_ONE_TO_ONE:\n                    view = Factory.createModelAndView(\"ERDRelationship\", tailModel, diagram, options);\n                    break;\n                case TX_ONE_TO_MANY:\n                    options.modelInitializer = function (elem) {\n                        elem.end2.cardinality = \"0..*\";\n                    };\n                    view = Factory.createModelAndView(\"ERDRelationship\", tailModel, diagram, options);\n                    break;\n                case TX_MANY_TO_MANY:\n                    options.modelInitializer = function (elem) {\n                        elem.end1.cardinality = \"0..*\";\n                        elem.end2.cardinality = \"0..*\";\n                    };\n                    view = Factory.createModelAndView(\"ERDRelationship\", tailModel, diagram, options);\n                    break;\n                }\n\n\n                // Open QuickEdit for the created view.\n                if (view) {\n                    view = Repository.get(view._id);\n                    if (view instanceof type.NodeView) {\n                        _.defer(function () {\n                            QuickEdit.open(editor, view, view.left, view.top);\n                        });\n                    } else if (view instanceof type.EdgeView) {\n                        _.defer(function () {\n                            QuickEdit.open(editor, view, Math.round((x1 + x2) / 2), Math.round((y1 + y2) / 2));\n                        });\n                    }\n                }\n\n            } catch (err) {\n                if (_.isString(err)) {\n                    Toast.error(err);\n                } else {\n                    console.error(err.stack);\n                }\n            }\n        });\n    }\n\n    function setupDiagramManager() {\n        $(DiagramManager).on('currentDiagramChanged', function (event, diagram, editor) {\n            try {\n                if (diagram instanceof type.ERDDiagram) {\n                    Toolbox.expandGroup(TXG_ERD);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    AppInit.htmlReady(function () {\n        setupToolbox();\n        setupDiagramManager();\n    });\n\n});\n\n//# sourceURL=/erd/ERDToolbox.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, staruml */\n\ndefine(\'erd/ERDPreferences\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Core\',\'core/PreferenceManager\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit           = require("utils/AppInit"),\n        Core              = require("core/Core"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    var ERDPreferences = {\n        "erd.entity": {\n            text: "Entity",\n            type: "Section"\n        },\n        "erd.entity.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Subsystem.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "erd.relationship": {\n            text: "Relationship",\n            type: "Section"\n        },\n        "erd.relationship.lineStyle": {\n            text: "Line Style",\n            description: "Default line style of entity relationships.",\n            type: "Dropdown",\n            options: [\n                { value: Core.LS_RECTILINEAR, text: "Rectilinear" },\n                { value: Core.LS_OBLIQUE,     text: "Oblique" },\n                { value: Core.LS_ROUNDRECT,   text: "Rounded Rectilinear" },\n                { value: Core.LS_CURVE,       text: "Curve" }\n            ],\n            default: Core.LS_ROUNDRECT\n        }\n    };\n\n    AppInit.htmlReady(function () {\n        PreferenceManager.register("erd", "ERD", ERDPreferences);\n    });\n\n});\n\n//# sourceURL=/erd/ERDPreferences.js'),eval("/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell, document */\n\ndefine('erd/main',['require','exports','module','utils/AppInit','engine/Engine','engine/Factory','diagrams/DiagramManager','ui/QuickEdit','utils/KeyEvent','erd/ERDQuickEdit','erd/ERDUtils','erd/ERD-meta','erd/ERD-rules','erd/ERD','erd/ERDCommands','erd/ERDCommandHandlers','erd/ERDMenus','erd/ERDToolbox','erd/ERDPreferences'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit        = require(\"utils/AppInit\"),\n        Engine         = require(\"engine/Engine\"),\n        Factory        = require(\"engine/Factory\"),\n        DiagramManager = require(\"diagrams/DiagramManager\"),\n        QuickEdit      = require(\"ui/QuickEdit\"),\n        KeyEvent       = require(\"utils/KeyEvent\"),\n        ERDQuickEdit   = require(\"erd/ERDQuickEdit\"),\n        ERDUtils       = require(\"erd/ERDUtils\");\n\n    require(\"erd/ERD-meta\");\n    require(\"erd/ERD-rules\");\n    require(\"erd/ERD\");\n    require(\"erd/ERDCommands\");\n    require(\"erd/ERDCommandHandlers\");\n    require(\"erd/ERDMenus\");\n    require(\"erd/ERDToolbox\");\n    require(\"erd/ERDPreferences\");\n\n    function _setColumnExpression(model, expression, feedback) {\n        try {\n            var ast = ERDUtils.parseColumnExpression(expression);\n            ERDUtils.setColumnExpression(model, ast);\n        } catch (e) {\n            feedback.success = false;\n        }\n    }\n\n    function setupQuickEdit() {\n        // QuickEdit Value Changed Event Handlers\n        $(QuickEdit).on('change', function (event, id, itemId, view, value, feedback) {\n            try {\n                switch (id) {\n                case ERDQuickEdit.QUICKEDIT_ENTITY:\n                    switch (itemId) {\n                    case ERDQuickEdit.ITEM_EDIT:\n                        Engine.setProperty(view.model, 'name', value);\n                        break;\n                    }\n                    break;\n                case ERDQuickEdit.QUICKEDIT_COLUMN:\n                    switch (itemId) {\n                    case ERDQuickEdit.ITEM_EDIT:\n                        _setColumnExpression(view.model, value, feedback);\n                        break;\n                    case ERDQuickEdit.ITEM_PRIMARY_KEY:\n                        Engine.setProperty(view.model, 'primaryKey', value);\n                        break;\n                    }\n                    break;\n                case ERDQuickEdit.QUICKEDIT_RELATIONSHIP:\n                    switch (itemId) {\n                    case ERDQuickEdit.ITEM_EDIT:\n                        if (view instanceof type.EdgeLabelView) { Engine.setProperty(view._parent.model, 'name', value); }\n                        break;\n                    case ERDQuickEdit.ITEM_IDENTIFYING:\n                        if (view instanceof type.EdgeLabelView) { Engine.setProperty(view._parent.model, 'identifying', value); }\n                        break;\n                    }\n                    break;\n                case ERDQuickEdit.QUICKEDIT_RELATIONSHIP_TAIL:\n                    switch (itemId) {\n                    case ERDQuickEdit.ITEM_EDIT:\n                        if (view instanceof type.EdgeLabelView) { Engine.setProperty(view._parent.model.end1, 'name', value); }\n                        break;\n                    case ERDQuickEdit.ITEM_CARDINALITY:\n                        if (view instanceof type.EdgeLabelView) { Engine.setProperty(view._parent.model.end1, 'cardinality', value); }\n                        break;\n                    }\n                    break;\n                case ERDQuickEdit.QUICKEDIT_RELATIONSHIP_HEAD:\n                    switch (itemId) {\n                    case ERDQuickEdit.ITEM_EDIT:\n                        if (view instanceof type.EdgeLabelView) { Engine.setProperty(view._parent.model.end2, 'name', value); }\n                        break;\n                    case ERDQuickEdit.ITEM_CARDINALITY:\n                        if (view instanceof type.EdgeLabelView) { Engine.setProperty(view._parent.model.end2, 'cardinality', value); }\n                        break;\n                    }\n                    break;\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        // QuickEdit Button Click Event Handlers\n        $(QuickEdit).on('click', function (event, id, itemId, view) {\n            var diagram      = DiagramManager.getEditor().diagram,\n                diagramOwner = diagram._parent,\n                parent,\n                nodes, nodeView, nodeView1, nodeView2, edgeView, options1, options2, _x, _y;\n            try {\n                switch (id) {\n                case ERDQuickEdit.QUICKEDIT_ENTITY:\n                    switch (itemId) {\n                    case ERDQuickEdit.ITEM_ADD_COLUMN:\n                        QuickEdit.addInQuickEdit(view.model, 'columns', view, 'columnCompartment', type.ERDColumn, 'Column');\n                        break;\n                    case ERDQuickEdit.ITEM_ADD_ONE_TO_ONE:\n                        nodes = QuickEdit.getBothNodes(view, type.ERDRelationshipView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView(\"ERDEntity\", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView(\"ERDRelationship\", nodeView.model, diagram, options2);\n                        break;\n                    case ERDQuickEdit.ITEM_ADD_ONE_TO_MANY:\n                        nodes = QuickEdit.getBothNodes(view, type.ERDRelationshipView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView(\"ERDEntity\", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        options2.modelInitializer = function (elem) {\n                            elem.end2.cardinality = \"0..*\";\n                        };\n                        edgeView = Factory.createModelAndView(\"ERDRelationship\", nodeView.model, diagram, options2);\n                        break;\n                    case ERDQuickEdit.ITEM_ADD_MANY_TO_MANY:\n                        nodes = QuickEdit.getBothNodes(view, type.ERDRelationshipView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getRightPosition(view, nodes);\n                        nodeView = Factory.createModelAndView(\"ERDEntity\", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        options2.modelInitializer = function (elem) {\n                            elem.end1.cardinality = \"0..*\";\n                            elem.end2.cardinality = \"0..*\";\n                        };\n                        edgeView = Factory.createModelAndView(\"ERDRelationship\", nodeView.model, diagram, options2);\n                        break;\n                    }\n                    break;\n                case ERDQuickEdit.QUICKEDIT_COLUMN:\n                    switch (itemId) {\n                    case ERDQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, 'columns', view._parent._parent, 'columnCompartment', type.ERDColumn, 'Column');\n                        break;\n                    case ERDQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case ERDQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, 'columns', view);\n                        break;\n                    case ERDQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, 'columns', view);\n                        break;\n                    }\n                    break;\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        // QuickEdit Keydown Event Handlers\n        $(QuickEdit).on('keydown', function (event, id, itemId, view, keyEvent) {\n            var v;\n            try {\n                switch (id) {\n                case ERDQuickEdit.QUICKEDIT_ENTITY:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (view.columnCompartment && view.columnCompartment.subViews.length > 0) {\n                            v = view.columnCompartment.subViews[0];\n                            _.defer(function () {\n                                QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                            });\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model, 'columns', view, 'columnCompartment', type.ERDColumn, 'Column');\n                        }\n                        break;\n                    }\n                    break;\n                case ERDQuickEdit.QUICKEDIT_COLUMN:\n                    switch (keyEvent.which) {\n                    case KeyEvent.DOM_VK_UP:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveUpInQuickEdit(view.model._parent, 'columns', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === 0) {\n                                v = view._parent._parent;\n                                _.defer(function () {\n                                    QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                });\n                            } else {\n                                QuickEdit.openUpInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_DOWN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.moveDownInQuickEdit(view.model._parent, 'columns', view);\n                        } else {\n                            if (_.indexOf(view._parent.subViews, view) === (view._parent.subViews.length - 1)) {\n                                v = view._parent._parent;\n                                if (v.suppressOperations !== true && v.operationCompartment && v.operationCompartment.subViews.length > 0) {\n                                    v = v.operationCompartment.subViews[0];\n                                    _.defer(function () {\n                                        QuickEdit.open(DiagramManager.getEditor(), v, v.left, v.top);\n                                    });\n                                }\n                            } else {\n                                QuickEdit.openDownInQuickEdit(view);\n                            }\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_RETURN:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.addInQuickEdit(view.model._parent, 'columns', view._parent._parent, 'columnCompartment', type.ERDColumn, 'Column');\n                        }\n                        break;\n                    case KeyEvent.DOM_VK_BACK_SPACE:\n                    case KeyEvent.DOM_VK_DELETE:\n                        if (keyEvent.ctrlKey || keyEvent.metaKey) {\n                            QuickEdit.deleteInQuickEdit(view);\n                        }\n                        break;\n                    }\n                    break;\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    AppInit.htmlReady(function () {\n        setupQuickEdit();\n    });\n\n});\n\n//# sourceURL=/erd/main.js"),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, type */\n\ndefine('flowchart/FlowchartQuickEdit',['require','exports','module','ui/QuickEdit','core/Graphics','core/Core'],function (require, exports, module) {\n    \"use strict\";\n\n    var QuickEdit  = require(\"ui/QuickEdit\"),\n        Graphics   = require(\"core/Graphics\"),\n        Core       = require(\"core/Core\");\n\n    /**\n     * @const\n     * QuickEdit IDs\n     */\n    var QUICKEDIT_NODE = 'fc-node',\n        QUICKEDIT_EDGE = 'fc-edge';\n\n    /**\n     * @const\n     * QuickEdit Item IDs\n     */\n    var ITEM_EDIT                  = 'edit',\n        ITEM_ADD_NOTE              = 'add-note',\n        ITEM_ADD_OUTGOING_PROCESS  = 'add-outgoing-process',\n        ITEM_ADD_INCOMING_PROCESS  = 'add-incoming-process',\n        ITEM_ADD_OUTGOING_DECISION = 'add-outgoing-decision',\n        ITEM_ADD_INCOMING_DECISION = 'add-incoming-decision';\n\n\n    /* Quick Edit Buttons */\n    var QE_ADD_NOTE              = { type: 'button', id: ITEM_ADD_NOTE,              icon: 'quick-edit-icon-add-note',              hint: 'Add Note' },\n        QE_ADD_OUTGOING_PROCESS  = { type: 'button', id: ITEM_ADD_OUTGOING_PROCESS,  icon: 'quick-edit-icon-add-outgoing-process',  hint: 'Add Outgoing Process' },\n        QE_ADD_INCOMING_PROCESS  = { type: 'button', id: ITEM_ADD_INCOMING_PROCESS,  icon: 'quick-edit-icon-add-incoming-process',  hint: 'Add Incoming Process' },\n        QE_ADD_OUTGOING_DECISION = { type: 'button', id: ITEM_ADD_OUTGOING_DECISION, icon: 'quick-edit-icon-add-outgoing-decision', hint: 'Add Outgoing Decision' },\n        QE_ADD_INCOMING_DECISION = { type: 'button', id: ITEM_ADD_INCOMING_DECISION, icon: 'quick-edit-icon-add-incoming-decision', hint: 'Add Incoming Decision' };\n\n    \n    /* Quick Edit Inputs */\n    var QE_EDIT_GENERAL = { type: 'input', id: ITEM_EDIT, hint: \"name\" };\n\n\n    // QuickEdit for Nodes\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.FCGeneralNodeView);\n        },\n        popup: function (editor, position, view) {\n            var label = view.nameLabel,\n                rect  = new Graphics.Rect(view.left, label.top, view.getRight(), label.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, view, {\n                id: QUICKEDIT_NODE,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_ADD_NOTE ],\n                rightSection: [ QE_ADD_OUTGOING_PROCESS, QE_ADD_INCOMING_PROCESS ],\n                rightSection2: [ QE_ADD_OUTGOING_DECISION, QE_ADD_INCOMING_DECISION ],\n            });\n        },\n        init: function (view) {\n            QuickEdit.setInputValue(ITEM_EDIT, view.model.name);\n        }\n    });\n\n    // QuickEdit for Edges\n    QuickEdit.register({\n        when: function (view) {\n            return (view instanceof type.FCGeneralEdgeView) ||\n                   ((view instanceof Core.EdgeLabelView) && (view._parent instanceof type.FCGeneralEdgeView));\n        },\n        popup: function (editor, position, view) {\n            var endView = view.nameLabel;\n            if (view instanceof Core.EdgeLabelView) { endView = view._parent.nameLabel; }\n            var rect  = new Graphics.Rect(endView.left, endView.top, endView.left+70, endView.getBottom());\n            rect.add(position);\n            QuickEdit.render(editor, rect, endView, {\n                id: QUICKEDIT_EDGE,\n                main: QE_EDIT_GENERAL,\n                leftSection: [ QE_ADD_NOTE ]\n            });\n        },\n        init: function (view) {\n            var model = view.model;\n            if (view instanceof Core.EdgeLabelView) { model = view._parent.model; }\n            QuickEdit.setInputValue(ITEM_EDIT, model.name);\n        }\n    });\n\n\n    // Define public API\n    exports.QUICKEDIT_NODE = QUICKEDIT_NODE;\n    exports.QUICKEDIT_EDGE = QUICKEDIT_EDGE;\n    \n    exports.ITEM_EDIT                  = ITEM_EDIT;\n    exports.ITEM_ADD_NOTE              = ITEM_ADD_NOTE;\n    exports.ITEM_ADD_OUTGOING_PROCESS  = ITEM_ADD_OUTGOING_PROCESS;\n    exports.ITEM_ADD_INCOMING_PROCESS  = ITEM_ADD_INCOMING_PROCESS;\n    exports.ITEM_ADD_OUTGOING_DECISION = ITEM_ADD_OUTGOING_DECISION;\n    exports.ITEM_ADD_INCOMING_DECISION = ITEM_ADD_INCOMING_DECISION;\n\n});\n\n//# sourceURL=/flowchart/FlowchartQuickEdit.js"),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, browser: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, app*/\n\ndefine(\'flowchart/Flowchart-meta\',[\'require\',\'exports\',\'module\',\'core/MetaModelManager\'],function (require, exports, module) {\n    "use strict";\n\n    var MetaModelManager  = require("core/MetaModelManager");\n\n    var metaModel = {\n        "FCModelElement": {\n            "kind": "class",\n            "super": "ExtensibleModel"\n        },\n        "FCFlowchart": {\n            "kind": "class",\n            "super": "FCModelElement"\n        },\n        "FCFlowchartDiagram": {\n            "kind": "class",\n            "super": "Diagram"\n        },\n        "FCProcess": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCProcessView"\n        },\n        "FCTerminator": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCTerminatorView"\n        },\n        "FCDecision": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCDecisionView"\n        },\n        "FCDelay": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCDelayView"\n        },\n        "FCPredefinedProcess": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCPredefinedProcessView"\n        },\n        "FCAlternateProcess": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCAlternateProcessView"\n        },\n        "FCData": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCDataView"\n        },\n        "FCDocument": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCDocumentView"\n        },\n        "FCMultiDocument": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCMultiDocumentView"\n        },\n        "FCPreparation": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCPreparationView"\n        },\n        "FCDisplay": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCDisplayView"\n        },\n        "FCManualInput": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCManualInputView"\n        },\n        "FCManualOperation": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCManualOperationView"\n        },\n        "FCCard": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCCardView"\n        },\n        "FCPunchedTape": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCPunchedTapeView"\n        },\n        "FCConnector": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCConnectorView"\n        },\n        "FCOffPageConnector": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCOffPageConnectorView"\n        },\n        "FCOr": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCOrView"\n        },\n        "FCSummingJunction": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCSummingJunctionView"\n        },\n        "FCCollate": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCCollateView"\n        },\n        "FCSort": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCSortView"\n        },\n        "FCMerge": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCMergeView"\n        },\n        "FCExtract": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCExtractView"\n        },\n        "FCStoredData": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCStoredDataView"\n        },\n        "FCDatabase": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCDatabaseView"\n        },\n        "FCDirectAccessStorage": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCDirectAccessStorageView"\n        },\n        "FCInternalStorage": {\n            "kind": "class",\n            "super": "FCModelElement",\n            "view":  "FCInternalStorageView"\n        },\n        "FCFlow": {\n            "kind": "class",\n            "super": "DirectedRelationship",\n            "view": "FCFlowView"\n        },\n        "FCGeneralNodeView": {\n            "kind": "class",\n            "super": "NodeView",\n            "attributes": [\n                { "name": "nameLabel", "kind": "ref",  "type": "LabelView", "embedded": "subViews" },\n                { "name": "wordWrap",  "kind": "prim", "type": "Boolean", "default": true }\n            ]\n        },\n        "FCGeneralEdgeView": {\n            "kind": "class",\n            "super": "EdgeView",\n            "attributes": [\n                { "name": "nameLabel", "kind": "ref",  "type": "EdgeLabelView", "embedded": "subViews" }\n            ]\n        },\n        "FCProcessView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCTerminatorView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCDecisionView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCDelayView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCPredefinedProcessView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCAlternateProcessView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCDataView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCDocumentView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCMultiDocumentView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCPreparationView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCDisplayView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCManualInputView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCManualOperationView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCCardView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCPunchedTapeView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCConnectorView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCOffPageConnectorView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCOrView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCSummingJunctionView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCCollateView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCSortView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCMergeView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCExtractView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCStoredDataView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCDatabaseView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCDirectAccessStorageView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCInternalStorageView": {\n            "kind": "class",\n            "super": "FCGeneralNodeView"\n        },\n        "FCFlowView": {\n            "kind": "class",\n            "super": "FCGeneralEdgeView"\n        }\n    };\n\n    // Register Meta-model\n    MetaModelManager.register(metaModel);\n\n});\n\n//# sourceURL=/flowchart/Flowchart-meta.js'),eval('/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell, document */\n\ndefine(\'flowchart/Flowchart\',[\'require\',\'exports\',\'module\',\'core/Graphics\',\'core/Graphics\',\'core/Graphics\',\'core/Core\',\'core/MetaModelManager\',\'core/Repository\',\'core/PreferenceManager\',\'flowchart/Flowchart-meta\'],function (require, exports, module) {\n    "use strict";\n\n    var Graphics          = require("core/Graphics"),\n        Point             = require("core/Graphics").Point,\n        Rect              = require("core/Graphics").Rect,\n        Core              = require("core/Core"),\n        MetaModelManager  = require("core/MetaModelManager"),\n        Repository        = require("core/Repository"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    require("flowchart/Flowchart-meta");\n\n    var LEFT_PADDING   = 10,\n        RIGHT_PADDING  = 10,\n        TOP_PADDING    = 10,\n        BOTTOM_PADDING = 10;\n\n    var DECISION_MINWIDTH  = 50,\n        DECISION_MINHEIGHT = 30,\n        MERGE_MINWIDTH  = 30,\n        MERGE_MINHEIGHT = 25,\n        EXTRACT_MINWIDTH  = 30,\n        EXTRACT_MINHEIGHT = 25;\n\n    /**\n     * FCModelElement\n     * @constructor\n     */\n    function FCModelElement() {\n        type.ExtensibleModel.apply(this, arguments);\n    }\n    // inherits from ExtensibleModel\n    FCModelElement.prototype = Object.create(type.ExtensibleModel.prototype);\n    FCModelElement.prototype.constructor = FCModelElement;\n\n    FCModelElement.prototype.getDisplayClassName = function () {\n        var name = this.getClassName();\n        return name.substring(2, name.length);\n    };\n\n    /**\n     * FCFlowchart\n     * @constructor\n     */\n    function FCFlowchart() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCFlowchart.prototype = Object.create(FCModelElement.prototype);\n    FCFlowchart.prototype.constructor = FCFlowchart;\n\n    /**\n     * FCFlowchartDiagram\n     * @constructor\n     */\n    function FCFlowchartDiagram() {\n        type.Diagram.apply(this, arguments);\n    }\n    // inherits from Diagram\n    FCFlowchartDiagram.prototype = Object.create(type.Diagram.prototype);\n    FCFlowchartDiagram.prototype.constructor = FCFlowchartDiagram;\n\n    FCFlowchartDiagram.prototype.getDisplayClassName = function () {\n        return "FlowchartDiagram";\n    };\n\n\n    /**\n     * FCProcess\n     * @constructor\n     */\n    function FCProcess() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCProcess.prototype = Object.create(FCModelElement.prototype);\n    FCProcess.prototype.constructor = FCProcess;\n\n\n    /**\n     * FCTerminator\n     * @constructor\n     */\n    function FCTerminator() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCTerminator.prototype = Object.create(FCModelElement.prototype);\n    FCTerminator.prototype.constructor = FCTerminator;\n\n\n    /**\n     * FCDecision\n     * @constructor\n     */\n    function FCDecision() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCDecision.prototype = Object.create(FCModelElement.prototype);\n    FCDecision.prototype.constructor = FCDecision;\n\n\n    /**\n     * FCDelay\n     * @constructor\n     */\n    function FCDelay() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCDelay.prototype = Object.create(FCModelElement.prototype);\n    FCDelay.prototype.constructor = FCDelay;\n\n\n    /**\n     * FCPredefinedProcess\n     * @constructor\n     */\n    function FCPredefinedProcess() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCPredefinedProcess.prototype = Object.create(FCModelElement.prototype);\n    FCPredefinedProcess.prototype.constructor = FCPredefinedProcess;\n\n\n    /**\n     * FCAlternateProcess\n     * @constructor\n     */\n    function FCAlternateProcess() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCAlternateProcess.prototype = Object.create(FCModelElement.prototype);\n    FCAlternateProcess.prototype.constructor = FCAlternateProcess;\n\n\n    /**\n     * FCData\n     * @constructor\n     */\n    function FCData() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCData.prototype = Object.create(FCModelElement.prototype);\n    FCData.prototype.constructor = FCData;\n\n\n    /**\n     * FCDocument\n     * @constructor\n     */\n    function FCDocument() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCDocument.prototype = Object.create(FCModelElement.prototype);\n    FCDocument.prototype.constructor = FCDocument;\n\n\n    /**\n     * FCMultiDocument\n     * @constructor\n     */\n    function FCMultiDocument() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCMultiDocument.prototype = Object.create(FCModelElement.prototype);\n    FCMultiDocument.prototype.constructor = FCMultiDocument;\n\n\n    /**\n     * FCPreparation\n     * @constructor\n     */\n    function FCPreparation() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCPreparation.prototype = Object.create(FCModelElement.prototype);\n    FCPreparation.prototype.constructor = FCPreparation;\n\n\n    /**\n     * FCDisplay\n     * @constructor\n     */\n    function FCDisplay() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCDisplay.prototype = Object.create(FCModelElement.prototype);\n    FCDisplay.prototype.constructor = FCDisplay;\n\n\n    /**\n     * FCManualInput\n     * @constructor\n     */\n    function FCManualInput() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCManualInput.prototype = Object.create(FCModelElement.prototype);\n    FCManualInput.prototype.constructor = FCManualInput;\n\n\n    /**\n     * FCManualOperation\n     * @constructor\n     */\n    function FCManualOperation() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCManualOperation.prototype = Object.create(FCModelElement.prototype);\n    FCManualOperation.prototype.constructor = FCManualOperation;\n\n\n    /**\n     * FCCard\n     * @constructor\n     */\n    function FCCard() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCCard.prototype = Object.create(FCModelElement.prototype);\n    FCCard.prototype.constructor = FCCard;\n\n\n    /**\n     * FCPunchedTape\n     * @constructor\n     */\n    function FCPunchedTape() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCPunchedTape.prototype = Object.create(FCModelElement.prototype);\n    FCPunchedTape.prototype.constructor = FCPunchedTape;\n\n\n    /**\n     * FCConnector\n     * @constructor\n     */\n    function FCConnector() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCConnector.prototype = Object.create(FCModelElement.prototype);\n    FCConnector.prototype.constructor = FCConnector;\n\n\n    /**\n     * FCOffPageConnector\n     * @constructor\n     */\n    function FCOffPageConnector() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCOffPageConnector.prototype = Object.create(FCModelElement.prototype);\n    FCOffPageConnector.prototype.constructor = FCOffPageConnector;\n\n\n    /**\n     * FCOr\n     * @constructor\n     */\n    function FCOr() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCOr.prototype = Object.create(FCModelElement.prototype);\n    FCOr.prototype.constructor = FCOr;\n\n\n    /**\n     * FCSummingJunction\n     * @constructor\n     */\n    function FCSummingJunction() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCSummingJunction.prototype = Object.create(FCModelElement.prototype);\n    FCSummingJunction.prototype.constructor = FCSummingJunction;\n\n\n    /**\n     * FCCollate\n     * @constructor\n     */\n    function FCCollate() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCCollate.prototype = Object.create(FCModelElement.prototype);\n    FCCollate.prototype.constructor = FCCollate;\n\n\n    /**\n     * FCSort\n     * @constructor\n     */\n    function FCSort() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCSort.prototype = Object.create(FCModelElement.prototype);\n    FCSort.prototype.constructor = FCSort;\n\n\n    /**\n     * FCMerge\n     * @constructor\n     */\n    function FCMerge() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCMerge.prototype = Object.create(FCModelElement.prototype);\n    FCMerge.prototype.constructor = FCMerge;\n\n\n    /**\n     * FCExtract\n     * @constructor\n     */\n    function FCExtract() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCExtract.prototype = Object.create(FCModelElement.prototype);\n    FCExtract.prototype.constructor = FCExtract;\n\n\n    /**\n     * FCStoredData\n     * @constructor\n     */\n    function FCStoredData() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCStoredData.prototype = Object.create(FCModelElement.prototype);\n    FCStoredData.prototype.constructor = FCStoredData;\n\n\n    /**\n     * FCDatabase\n     * @constructor\n     */\n    function FCDatabase() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCDatabase.prototype = Object.create(FCModelElement.prototype);\n    FCDatabase.prototype.constructor = FCDatabase;\n\n\n    /**\n     * FCDirectAccessStorage\n     * @constructor\n     */\n    function FCDirectAccessStorage() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCDirectAccessStorage.prototype = Object.create(FCModelElement.prototype);\n    FCDirectAccessStorage.prototype.constructor = FCDirectAccessStorage;\n\n\n    /**\n     * FCInternalStorage\n     * @constructor\n     */\n    function FCInternalStorage() {\n        FCModelElement.apply(this, arguments);\n    }\n    // inherits from FCModelElement\n    FCInternalStorage.prototype = Object.create(FCModelElement.prototype);\n    FCInternalStorage.prototype.constructor = FCInternalStorage;\n\n\n    /**\n     * FCFlow\n     * @constructor\n     */\n    function FCFlow() {\n        type.DirectedRelationship.apply(this, arguments);\n    }\n    // inherits from DirectedRelationship\n    FCFlow.prototype = Object.create(type.DirectedRelationship.prototype);\n    FCFlow.prototype.constructor = FCFlow;\n\n\n    /* -------------------------- View Elements ---------------------------- */\n\n    /**\n     * FCGeneralNodeView\n     * @constructor\n     */\n    function FCGeneralNodeView() {\n        type.NodeView.apply(this, arguments);\n\n        /** @member {boolean} */\n        this.wordWrap = true;\n\n        /** @member {LabelView} */\n        this.nameLabel = new type.LabelView();\n        this.nameLabel.horizontalAlignment = Graphics.AL_CENTER;\n        this.nameLabel.parentStyle = true;\n        this.addSubView(this.nameLabel);\n    }\n    // inherits from NodeView\n    FCGeneralNodeView.prototype = Object.create(type.NodeView.prototype);\n    FCGeneralNodeView.prototype.constructor = FCGeneralNodeView;\n\n    FCGeneralNodeView.prototype.update = function (canvas) {\n        type.NodeView.prototype.update.call(this, canvas);\n        if (this.model) {\n            this.nameLabel.text = this.model.name;\n        }\n        this.nameLabel.wordWrap = this.wordWrap;\n    };\n\n    FCGeneralNodeView.prototype.sizeObject = function (canvas) {\n        type.NodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = this.nameLabel.minWidth + LEFT_PADDING + RIGHT_PADDING;\n        this.minHeight = this.nameLabel.minHeight + TOP_PADDING + BOTTOM_PADDING;\n    };\n\n    FCGeneralNodeView.prototype.arrangeObject = function (canvas) {\n        type.NodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameLabel.left = this.left + LEFT_PADDING;\n        this.nameLabel.top = this.top + TOP_PADDING;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING;\n        this.nameLabel.height = this.nameLabel.minHeight;\n        // this.nameLabel.height = this.height - TOP_PADDING - BOTTOM_PADDING;\n    };\n\n    FCGeneralNodeView.prototype.drawObject = function (canvas) {\n        type.NodeView.prototype.drawObject.call(this, canvas);\n    };\n\n    FCGeneralNodeView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * FCGeneralEdgeView\n     * @constructor\n     */\n    function FCGeneralEdgeView() {\n        type.EdgeView.apply(this, arguments);\n        this.tailEndStyle = Core.ES_FLAT;\n        this.headEndStyle = Core.ES_SOLID_ARROW;\n        this.lineMode = Core.LM_SOLID;\n\n        /** @member {EdgeLabelView} */\n        this.nameLabel = new type.EdgeLabelView();\n        this.nameLabel.hostEdge = this;\n        this.nameLabel.edgePosition = Core.EP_MIDDLE;\n        this.nameLabel.distance = 15;\n        this.nameLabel.alpha = Math.PI / 2;\n        this.addSubView(this.nameLabel);\n    }\n    // inherits from EdgeView\n    FCGeneralEdgeView.prototype = Object.create(type.EdgeView.prototype);\n    FCGeneralEdgeView.prototype.constructor = FCGeneralEdgeView;\n\n    FCGeneralEdgeView.prototype.update = function (canvas) {\n        if (this.model) {\n            // nameLabel\n            this.nameLabel.visible = (this.model.name.length > 0);\n            if (this.model.name) {\n                this.nameLabel.text = this.model.name;\n            }\n            // Enforce nameLabel.mode refers to this.model by using Bypass Command.\n            if (this.nameLabel.model !== this.model) {\n                Repository.bypassFieldAssign(this.nameLabel, \'model\', this.model);\n            }\n        }\n        type.EdgeView.prototype.update.call(this, canvas);\n    };\n\n    FCGeneralEdgeView.prototype.canConnectTo = function (view, isTail) {\n        return (view.model instanceof FCModelElement);\n    };\n\n    FCGeneralEdgeView.prototype.canDelete = function () {\n        return false;\n    };\n\n\n    /**\n     * FCProcessView\n     * @constructor\n     */\n    function FCProcessView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.process.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCProcessView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCProcessView.prototype.constructor = FCProcessView;\n\n    FCProcessView.prototype.drawObject = function (canvas) {\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n    /**\n     * FCTerminatorView\n     * @constructor\n     */\n    function FCTerminatorView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.terminator.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCTerminatorView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCTerminatorView.prototype.constructor = FCTerminatorView;\n\n    FCTerminatorView.prototype.drawObject = function (canvas) {\n        var round = Math.min(this.height, this.width) / 2;\n        canvas.fillRoundRect(this.left, this.top, this.getRight(), this.getBottom(), round);\n        canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), round);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FDecisionView\n     * @constructor\n     */\n    function FCDecisionView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.decision.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCDecisionView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCDecisionView.prototype.constructor = FCDecisionView;\n\n    FCDecisionView.prototype.sizeObject = function (canvas) {\n        FCGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = Math.max(this.nameLabel.minWidth * 2, DECISION_MINWIDTH);\n        this.minHeight = Math.max(this.nameLabel.minHeight * 2, DECISION_MINHEIGHT);\n    };\n\n    FCDecisionView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameLabel.left = this.left + (this.width / 4);\n        this.nameLabel.top = this.top + (this.height / 4);\n        this.nameLabel.width = (this.width / 2);\n        this.nameLabel.height = (this.height / 2);\n    };\n\n    FCDecisionView.prototype.drawObject = function (canvas) {\n        var x = (this.left + this.getRight()) / 2,\n            y = (this.top + this.getBottom()) / 2;\n        canvas.fillPolygon([new Point(this.left, y), new Point(x, this.top), new Point(this.getRight(), y), new Point(x, this.getBottom()), new Point(this.left, y)]);\n        canvas.polygon([new Point(this.left, y), new Point(x, this.top), new Point(this.getRight(), y), new Point(x, this.getBottom()), new Point(this.left, y)]);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCDelayView\n     * @constructor\n     */\n    function FCDelayView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.delay.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCDelayView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCDelayView.prototype.constructor = FCDelayView;\n\n    FCDelayView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameLabel.width = (this.width * 0.75);\n    };\n\n    FCDelayView.prototype.drawObject = function (canvas) {\n        var r = this.height / 2;\n        canvas.fillRect(this.left, this.top, this.getRight() - r, this.getBottom());\n        canvas.fillArc(this.getRight() - r, this.top + r, r, -(Math.PI / 2), Math.PI / 2);\n        canvas.line(this.left, this.top, this.left, this.getBottom());\n        canvas.line(this.left, this.top, this.getRight() - r, this.top);\n        canvas.line(this.left, this.top + this.height, this.getRight() - r, this.top + this.height);\n        canvas.arc(this.getRight() - r, this.top + r, r, -(Math.PI / 2), Math.PI / 2);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCPredefinedProcessView\n     * @constructor\n     */\n    function FCPredefinedProcessView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.predefined-process.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCPredefinedProcessView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCPredefinedProcessView.prototype.constructor = FCPredefinedProcessView;\n\n    FCPredefinedProcessView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameLabel.left = this.left + LEFT_PADDING + 5;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - 5;\n    };\n\n    FCPredefinedProcessView.prototype.drawObject = function (canvas) {\n        var gap = 5;\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.line(this.left + gap, this.top, this.left + gap, this.getBottom());\n        canvas.line(this.getRight() - gap, this.top, this.getRight() - gap, this.getBottom());\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCAlternateProcessView\n     * @constructor\n     */\n    function FCAlternateProcessView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.alternate-process.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCAlternateProcessView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCAlternateProcessView.prototype.constructor = FCAlternateProcessView;\n\n    FCAlternateProcessView.prototype.drawObject = function (canvas) {\n        var round = 10;\n        canvas.fillRoundRect(this.left, this.top, this.getRight(), this.getBottom(), round);\n        canvas.roundRect(this.left, this.top, this.getRight(), this.getBottom(), round);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCDataView\n     * @constructor\n     */\n    function FCDataView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.data.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCDataView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCDataView.prototype.constructor = FCDataView;\n\n    FCDataView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.width / 6);\n        this.nameLabel.left = this.left + LEFT_PADDING + g;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - (g * 2);\n    };\n\n    FCDataView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            g = Math.floor(this.width / 6),\n            pts = [\n                new Point(r.x1 + g, r.y1),\n                new Point(r.x2, r.y1),\n                new Point(r.x2 - g, r.y2),\n                new Point(r.x1, r.y2),\n                new Point(r.x1 + g, r.y1)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCDocumentView\n     * @constructor\n     */\n    function FCDocumentView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.document.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCDocumentView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCDocumentView.prototype.constructor = FCDocumentView;\n\n    FCDocumentView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var h = Math.floor(this.height / 6);\n        this.nameLabel.height = this.height - TOP_PADDING - BOTTOM_PADDING - h;\n    };\n\n    FCDocumentView.prototype.drawDocument = function (canvas, rect) {\n        var h = Math.floor(rect.getHeight() / 6),\n            g = Math.floor(rect.getWidth() / 3);\n        canvas.fillPath([[\'M\', rect.x1, rect.y1],\n                         [\'L\', rect.x2, rect.y1],\n                         [\'L\', rect.x2, rect.y2],\n                         [\'C\', rect.x2 - g, rect.y2 - (h * 2), rect.x1 + g, rect.y2 + (h * 1.5), rect.x1, rect.y2 - h],\n                         [\'L\', rect.x1, rect.y1],\n                         [\'Z\']], true);\n    };\n\n    FCDocumentView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom());\n        this.drawDocument(canvas, r);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCMultiDocumentView\n     * @constructor\n     */\n    function FCMultiDocumentView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.multidocument.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCMultiDocumentView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCMultiDocumentView.prototype.constructor = FCMultiDocumentView;\n\n    FCMultiDocumentView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = 5,\n            h = Math.floor(this.height / 6);\n        this.nameLabel.top = this.top + TOP_PADDING + (g * 2);\n        this.nameLabel.left = this.left + LEFT_PADDING + (g * 2);\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - (g * 2);\n        this.nameLabel.height = this.height - TOP_PADDING - BOTTOM_PADDING - h - (g * 2);\n    };\n\n    FCMultiDocumentView.prototype.drawObject = function (canvas) {\n        var g = 5,\n            r1 = new Rect(this.left, this.top, this.getRight() - (g * 2), this.getBottom() - (g * 2)),\n            r2 = new Rect(this.left + g, this.top + g, this.getRight() - g, this.getBottom() - g),\n            r3 = new Rect(this.left + (g * 2), this.top + (g * 2), this.getRight(), this.getBottom());\n        FCDocumentView.prototype.drawDocument(canvas, r1);\n        FCDocumentView.prototype.drawDocument(canvas, r2);\n        FCDocumentView.prototype.drawDocument(canvas, r3);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCPreparationView\n     * @constructor\n     */\n    function FCPreparationView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.preparation.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCPreparationView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCPreparationView.prototype.constructor = FCPreparationView;\n\n    FCPreparationView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.width / 6);\n        this.nameLabel.left = this.left + LEFT_PADDING + g;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - (g * 2);\n    };\n\n    FCPreparationView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            m = (r.y1 + r.y2) / 2,\n            g = Math.floor(this.width / 6),\n            pts = [\n                new Point(r.x1, m),\n                new Point(r.x1 + g, r.y1),\n                new Point(r.x2 - g, r.y1),\n                new Point(r.x2, m),\n                new Point(r.x2 - g, r.y2),\n                new Point(r.x1 + g, r.y2),\n                new Point(r.x1, m)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCDisplayView\n     * @constructor\n     */\n    function FCDisplayView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.display.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCDisplayView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCDisplayView.prototype.constructor = FCDisplayView;\n\n    FCDisplayView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.width / 6);\n        this.nameLabel.left = this.left + LEFT_PADDING + g;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - (g * 2);\n    };\n\n    FCDisplayView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            m = (r.y1 + r.y2) / 2,\n            g = Math.floor(this.width / 6);\n        canvas.fillPath([[\'M\', r.x1, m],\n                         [\'L\', r.x1 + g, r.y1],\n                         [\'L\', r.x2 - g, r.y1],\n                         [\'Q\', r.x2, r.y1, r.x2, m],\n                         [\'Q\', r.x2, r.y2, r.x2 - g, r.y2],\n                         [\'L\', r.x1 + g, r.y2],\n                         [\'L\', r.x1, m],\n                         [\'Z\']], true);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCManualInputView\n     * @constructor\n     */\n    function FCManualInputView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.manual-input.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCManualInputView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCManualInputView.prototype.constructor = FCManualInputView;\n\n    FCManualInputView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.height / 6);\n        this.nameLabel.top = this.top + TOP_PADDING + g;\n        this.nameLabel.height = this.height - TOP_PADDING - BOTTOM_PADDING - g;\n    };\n\n    FCManualInputView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            g = Math.floor(this.height / 6),\n            pts = [\n                new Point(r.x1, r.y1 + g),\n                new Point(r.x2, r.y1),\n                new Point(r.x2, r.y2),\n                new Point(r.x1, r.y2),\n                new Point(r.x1, r.y1 + g)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCManualOperationView\n     * @constructor\n     */\n    function FCManualOperationView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.manual-operation.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCManualOperationView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCManualOperationView.prototype.constructor = FCManualOperationView;\n\n    FCManualOperationView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.width / 6);\n        this.nameLabel.left = this.left + LEFT_PADDING + g;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - (g * 2);\n    };\n\n    FCManualOperationView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            g = Math.floor(this.width / 6),\n            pts = [\n                new Point(r.x1, r.y1),\n                new Point(r.x2, r.y1),\n                new Point(r.x2 - g, r.y2),\n                new Point(r.x1 + g, r.y2),\n                new Point(r.x1, r.y1)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCCardView\n     * @constructor\n     */\n    function FCCardView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.card.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCCardView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCCardView.prototype.constructor = FCCardView;\n\n    FCCardView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            g = Math.floor(this.width / 8),\n            pts = [\n                new Point(r.x1 + g, r.y1),\n                new Point(r.x2, r.y1),\n                new Point(r.x2, r.y2),\n                new Point(r.x1, r.y2),\n                new Point(r.x1, r.y1 + g),\n                new Point(r.x1 + g, r.y1)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCPunchedTapeView\n     * @constructor\n     */\n    function FCPunchedTapeView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.punched-tape.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCPunchedTapeView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCPunchedTapeView.prototype.constructor = FCPunchedTapeView;\n\n    FCPunchedTapeView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.height / 6);\n        this.nameLabel.top = this.top + TOP_PADDING + g;\n        this.nameLabel.height = this.height - TOP_PADDING - BOTTOM_PADDING - (g * 2);\n    };\n\n    FCPunchedTapeView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            h = Math.floor(r.getHeight() / 6),\n            g = Math.floor(r.getWidth() / 3);\n        canvas.fillPath([[\'M\', r.x1, r.y1],\n                         [\'C\', r.x1 + g, r.y1 + (h * 2), r.x2 - g, r.y1 - (h * 1.5), r.x2, r.y1 + h],\n                         [\'L\', r.x2, r.y2],\n                         [\'C\', r.x2 - g, r.y2 - (h * 2), r.x1 + g, r.y2 + (h * 1.5), r.x1, r.y2 - h],\n                         [\'L\', r.x1, r.y1],\n                         [\'Z\']], true);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCConnectorView\n     * @constructor\n     */\n    function FCConnectorView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.connector.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCConnectorView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCConnectorView.prototype.constructor = FCConnectorView;\n\n    FCConnectorView.prototype.sizeObject = function (canvas) {\n        FCGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = Math.max(30, this.nameLabel.minWidth);\n        this.minHeight = Math.max(30, this.nameLabel.minHeight);\n    };\n\n    FCConnectorView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameLabel.left = this.left + (this.width / 4);\n        this.nameLabel.top = this.top + (this.height / 4);\n        this.nameLabel.width = (this.width / 2);\n        this.nameLabel.height = (this.height / 2);\n    };\n\n    FCConnectorView.prototype.drawObject = function (canvas) {\n        canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCOffPageConnectorView\n     * @constructor\n     */\n    function FCOffPageConnectorView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.off-page-connector.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCOffPageConnectorView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCOffPageConnectorView.prototype.constructor = FCOffPageConnectorView;\n\n    FCOffPageConnectorView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.height / 4);\n        this.nameLabel.height = this.height - TOP_PADDING - BOTTOM_PADDING - g;\n    };\n\n    FCOffPageConnectorView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            g = Math.floor(this.height / 4),\n            pts = [\n                new Point(r.x1, r.y1),\n                new Point(r.x2, r.y1),\n                new Point(r.x2, r.y2 - g),\n                new Point((r.x1 + r.x2) / 2, r.y2),\n                new Point(r.x1, r.y2 - g),\n                new Point(r.x1, r.y1)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCOrView\n     * @constructor\n     */\n    function FCOrView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.or.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCOrView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCOrView.prototype.constructor = FCOrView;\n\n    FCOrView.prototype.sizeObject = function (canvas) {\n        type.NodeView.prototype.sizeObject.call(this, canvas);\n        this.nameLabel.visible = false;\n        this.minWidth = 30;\n        this.minHeight = 30;\n        if (this.height !== this.width) {\n            this.width  = Math.min(this.width, this.height);\n            this.height = Math.min(this.width, this.height);\n        }\n    };\n\n    FCOrView.prototype.drawObject = function (canvas) {\n        canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n        var p = Graphics.Coord.getCenter(new Rect(this.left, this.top, this.getRight(), this.getBottom()));\n        var d = Math.round(Math.sqrt(2) * this.width / 4);\n        canvas.line(p.x, this.top, p.x, this.getBottom());\n        canvas.line(this.left, p.y, this.getRight(), p.y);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCSummingJunctionView\n     * @constructor\n     */\n    function FCSummingJunctionView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.summing-junction.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCSummingJunctionView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCSummingJunctionView.prototype.constructor = FCSummingJunctionView;\n\n    FCSummingJunctionView.prototype.sizeObject = function (canvas) {\n        type.NodeView.prototype.sizeObject.call(this, canvas);\n        this.nameLabel.visible = false;\n        this.minWidth = 30;\n        this.minHeight = 30;\n        if (this.height !== this.width) {\n            this.width  = Math.min(this.width, this.height);\n            this.height = Math.min(this.width, this.height);\n        }\n    };\n\n    FCSummingJunctionView.prototype.drawObject = function (canvas) {\n        canvas.fillEllipse(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.ellipse(this.left, this.top, this.getRight(), this.getBottom());\n        var p = Graphics.Coord.getCenter(new Rect(this.left, this.top, this.getRight(), this.getBottom()));\n        var d = Math.round(Math.sqrt(2) * this.width / 4);\n        canvas.line(p.x - d, p.y - d, p.x + d, p.y + d);\n        canvas.line(p.x + d, p.y - d, p.x - d, p.y + d);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCCollateView\n     * @constructor\n     */\n    function FCCollateView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.collate.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCCollateView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCCollateView.prototype.constructor = FCCollateView;\n\n    FCCollateView.prototype.sizeObject = function (canvas) {\n        type.NodeView.prototype.sizeObject.call(this, canvas);\n        this.nameLabel.visible = false;\n        this.minWidth = 30;\n        this.minHeight = 30;\n    };\n\n    FCCollateView.prototype.drawObject = function (canvas) {\n        var r   = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            pts = [\n                new Point(r.x1, r.y1),\n                new Point(r.x2, r.y1),\n                new Point(r.x1, r.y2),\n                new Point(r.x2, r.y2),\n                new Point(r.x1, r.y1)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCSortView\n     * @constructor\n     */\n    function FCSortView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.sort.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCSortView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCSortView.prototype.constructor = FCSortView;\n\n    FCSortView.prototype.sizeObject = function (canvas) {\n        type.NodeView.prototype.sizeObject.call(this, canvas);\n        this.nameLabel.visible = false;\n        this.minWidth = 30;\n        this.minHeight = 30;\n    };\n\n    FCSortView.prototype.drawObject = function (canvas) {\n        var xm = (this.left + this.getRight()) / 2,\n            ym = (this.top + this.getBottom()) / 2,\n            pts = [\n                new Point(this.left, ym),\n                new Point(xm, this.top),\n                new Point(this.getRight(), ym),\n                new Point(xm, this.getBottom()),\n                new Point(this.left, ym)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        canvas.line(this.left, ym, this.getRight(), ym);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCMergeView\n     * @constructor\n     */\n    function FCMergeView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.merge.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCMergeView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCMergeView.prototype.constructor = FCMergeView;\n\n    FCMergeView.prototype.sizeObject = function (canvas) {\n        FCGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = Math.max(this.nameLabel.minWidth * 2, MERGE_MINWIDTH);\n        this.minHeight = Math.max(this.nameLabel.minHeight * 2, MERGE_MINHEIGHT);\n    };\n\n    FCMergeView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameLabel.left = this.left + (this.width / 4);\n        this.nameLabel.top = this.top + TOP_PADDING;\n        this.nameLabel.width = (this.width / 2);\n        this.nameLabel.height = (this.height / 2);\n    };\n\n    FCMergeView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            pts = [\n                new Point(r.x1, r.y1),\n                new Point(r.x2, r.y1),\n                new Point((r.x1 + r.x2) / 2, r.y2),\n                new Point(r.x1, r.y1)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCExtractView\n     * @constructor\n     */\n    function FCExtractView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.extract.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCExtractView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCExtractView.prototype.constructor = FCExtractView;\n\n    FCExtractView.prototype.sizeObject = function (canvas) {\n        FCGeneralNodeView.prototype.sizeObject.call(this, canvas);\n        this.minWidth = Math.max(this.nameLabel.minWidth * 2, EXTRACT_MINWIDTH);\n        this.minHeight = Math.max(this.nameLabel.minHeight * 2, EXTRACT_MINHEIGHT);\n    };\n\n    FCExtractView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        this.nameLabel.left = this.left + (this.width / 4);\n        this.nameLabel.top = this.top + (this.height / 2) - BOTTOM_PADDING;\n        this.nameLabel.width = (this.width / 2);\n        this.nameLabel.height = (this.height / 2);\n    };\n\n    FCExtractView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            pts = [\n                new Point((r.x1 + r.x2) / 2, r.y1),\n                new Point(r.x2, r.y2),\n                new Point(r.x1, r.y2),\n                new Point((r.x1 + r.x2) / 2, r.y1)\n            ];\n        canvas.fillPolygon(pts);\n        canvas.polygon(pts);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCStoredDataView\n     * @constructor\n     */\n    function FCStoredDataView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.stored-data.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCStoredDataView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCStoredDataView.prototype.constructor = FCStoredDataView;\n\n    FCStoredDataView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.width / 8);\n        this.nameLabel.left = this.left + LEFT_PADDING + g;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - (g * 2);\n    };\n\n    FCStoredDataView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            w = r.getWidth(),\n            h = r.getHeight(),\n            ym = (r.y1 + r.y2) / 2,\n            g = Math.floor(w / 8),\n            kappa = 0.5522848,\n            ox = g * kappa,       // control point offset horizontal\n            oy = (h / 2) * kappa; // control point offset vertical\n\n        canvas.fillPath([[\'M\', r.x2, r.y1],\n                         [\'C\', r.x2 - ox, r.y1, r.x2 - g, ym - oy, r.x2 - g, ym],\n                         [\'C\', r.x2 - g, ym + oy, r.x2 - ox, r.y2, r.x2, r.y2],\n                         [\'L\', r.x1 + g, r.y2],\n                         [\'C\', r.x1 + g - ox, r.y2, r.x1, ym + oy, r.x1, ym],\n                         [\'C\', r.x1, ym - oy, r.x1 + g - ox, r.y1, r.x1 + g, r.y1],\n                         [\'L\', r.x2, r.y1],\n                         [\'Z\']], true);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCDatabaseView\n     * @constructor\n     */\n    function FCDatabaseView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.database.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCDatabaseView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCDatabaseView.prototype.constructor = FCDatabaseView;\n\n    FCDatabaseView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.height / 8);\n        this.nameLabel.top = this.top + TOP_PADDING + (g * 2);\n        this.nameLabel.height = this.height - TOP_PADDING - BOTTOM_PADDING - (g * 3);\n    };\n\n    FCDatabaseView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            w = r.getWidth(),\n            h = r.getHeight(),\n            xm = (r.x1 + r.x2) / 2,\n            g = Math.floor(h / 8),\n            kappa = 0.5522848,\n            ox = (w / 2) * kappa, // control point offset horizontal\n            oy = g * kappa;       // control point offset vertical\n\n        canvas.fillPath([[\'M\', r.x1, r.y1 + g],\n                         [\'C\', r.x1, r.y1 + g - oy, xm - ox, r.y1, xm, r.y1],\n                         [\'C\', xm + ox, r.y1, r.x2, r.y1 + g - oy, r.x2, r.y1 + g],\n                         [\'L\', r.x2, r.y2 - g],\n                         [\'C\', r.x2, r.y2 - g + oy, xm + ox, r.y2, xm, r.y2],\n                         [\'C\', xm - ox, r.y2, r.x1, r.y2 - g + oy, r.x1, r.y2 - g],\n                         [\'L\', r.x1, r.y1 + g],\n                         [\'Z\']], true);\n        canvas.path([[\'M\', r.x1, r.y1 + g],\n                     [\'C\', r.x1, r.y1 + g + oy, xm - ox, r.y1 + (g * 2), xm, r.y1 + (g * 2)],\n                     [\'C\', xm + ox, r.y1 + (g * 2), r.x2, r.y1 + g + oy, r.x2, r.y1 + g]]);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCDirectAccessStorageView\n     * @constructor\n     */\n    function FCDirectAccessStorageView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.direct-access-storage.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCDirectAccessStorageView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCDirectAccessStorageView.prototype.constructor = FCDirectAccessStorageView;\n\n    FCDirectAccessStorageView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = Math.floor(this.width / 8);\n        this.nameLabel.left = this.left + LEFT_PADDING + g;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - (g * 3);\n    };\n\n    FCDirectAccessStorageView.prototype.drawObject = function (canvas) {\n        var r = new Rect(this.left, this.top, this.getRight(), this.getBottom()),\n            w = r.getWidth(),\n            h = r.getHeight(),\n            ym = (r.y1 + r.y2) / 2,\n            g = Math.floor(w / 8),\n            kappa = 0.5522848,\n            ox = g * kappa,       // control point offset horizontal\n            oy = (h / 2) * kappa; // control point offset vertical\n\n        canvas.fillPath([[\'M\', r.x2 - g, r.y1],\n                         [\'C\', r.x2 - g + ox, r.y1, r.x2, ym - oy, r.x2, ym],\n                         [\'C\', r.x2, ym + oy, r.x2 - g + ox, r.y2, r.x2 - g, r.y2],\n                         [\'L\', r.x1 + g, r.y2],\n                         [\'C\', r.x1 + g - ox, r.y2, r.x1, ym + oy, r.x1, ym],\n                         [\'C\', r.x1, ym - oy, r.x1 + g - ox, r.y1, r.x1 + g, r.y1],\n                         [\'L\', r.x2 - g, r.y1],\n                         [\'Z\']], true);\n        canvas.path([[\'M\', r.x2 - g, r.y1],\n                     [\'C\', r.x2 - g - ox, r.y1, r.x2 - (g * 2), ym - oy, r.x2 - (g * 2), ym],\n                     [\'C\', r.x2 - (g * 2), ym + oy, r.x2 - g - ox, r.y2, r.x2 - g, r.y2]]);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCInternalStorageView\n     * @constructor\n     */\n    function FCInternalStorageView() {\n        FCGeneralNodeView.apply(this, arguments);\n        this.fillColor = PreferenceManager.get("flowchart.internal-storage.fillColor", "#ffffff") || PreferenceManager.get("view.fillColor", "#ffffff");\n    }\n    // inherits from FCGeneralNodeView\n    FCInternalStorageView.prototype = Object.create(FCGeneralNodeView.prototype);\n    FCInternalStorageView.prototype.constructor = FCInternalStorageView;\n\n    FCInternalStorageView.prototype.arrangeObject = function (canvas) {\n        FCGeneralNodeView.prototype.arrangeObject.call(this, canvas);\n        var g = 10;\n        this.nameLabel.left = this.left + LEFT_PADDING + g;\n        this.nameLabel.top = this.top + TOP_PADDING + g;\n        this.nameLabel.width = this.width - LEFT_PADDING - RIGHT_PADDING - g;\n        this.nameLabel.height = this.height - TOP_PADDING - BOTTOM_PADDING - g;\n    };\n\n    FCInternalStorageView.prototype.drawObject = function (canvas) {\n        var gap = 10;\n        canvas.fillRect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.rect(this.left, this.top, this.getRight(), this.getBottom());\n        canvas.line(this.left + gap, this.top, this.left + gap, this.getBottom());\n        canvas.line(this.left, this.top + gap, this.getRight(), this.top + gap);\n        FCGeneralNodeView.prototype.drawObject.call(this, canvas);\n    };\n\n\n    /**\n     * FCFlowView\n     * @constructor\n     */\n    function FCFlowView() {\n        FCGeneralEdgeView.apply(this, arguments);\n        this.lineStyle = PreferenceManager.get("flowchart.flow.lineStyle", Core.LS_ROUNDRECT) || PreferenceManager.get("view.lineStyle", Core.LS_OBLIQUE);\n    }\n    // inherits from FCGeneralEdgeView\n    FCFlowView.prototype = Object.create(FCGeneralEdgeView.prototype);\n    FCFlowView.prototype.constructor = FCFlowView;\n\n\n    /* ************************** Type definitions ***************************/\n\n    type.FCModelElement            = FCModelElement;\n    type.FCFlowchart               = FCFlowchart;\n    type.FCFlowchartDiagram        = FCFlowchartDiagram;\n    type.FCProcess                 = FCProcess;\n    type.FCTerminator              = FCTerminator;\n    type.FCDecision                = FCDecision;\n    type.FCDelay                   = FCDelay;\n    type.FCPredefinedProcess       = FCPredefinedProcess;\n    type.FCAlternateProcess        = FCAlternateProcess;\n    type.FCData                    = FCData;\n    type.FCDocument                = FCDocument;\n    type.FCMultiDocument           = FCMultiDocument;\n    type.FCPreparation             = FCPreparation;\n    type.FCDisplay                 = FCDisplay;\n    type.FCManualInput             = FCManualInput;\n    type.FCManualOperation         = FCManualOperation;\n    type.FCCard                    = FCCard;\n    type.FCPunchedTape             = FCPunchedTape;\n    type.FCConnector               = FCConnector;\n    type.FCOffPageConnector        = FCOffPageConnector;\n    type.FCOr                      = FCOr;\n    type.FCSummingJunction         = FCSummingJunction;\n    type.FCCollate                 = FCCollate;\n    type.FCSort                    = FCSort;\n    type.FCMerge                   = FCMerge;\n    type.FCExtract                 = FCExtract;\n    type.FCStoredData              = FCStoredData;\n    type.FCDatabase                = FCDatabase;\n    type.FCDirectAccessStorage     = FCDirectAccessStorage;\n    type.FCInternalStorage         = FCInternalStorage;\n    type.FCFlow                    = FCFlow;\n\n    type.FCGeneralNodeView         = FCGeneralNodeView;\n    type.FCGeneralEdgeView         = FCGeneralEdgeView;\n    type.FCProcessView             = FCProcessView;\n    type.FCTerminatorView          = FCTerminatorView;\n    type.FCDecisionView            = FCDecisionView;\n    type.FCDelayView               = FCDelayView;\n    type.FCPredefinedProcessView   = FCPredefinedProcessView;\n    type.FCAlternateProcessView    = FCAlternateProcessView;\n    type.FCDataView                = FCDataView;\n    type.FCDocumentView            = FCDocumentView;\n    type.FCMultiDocumentView       = FCMultiDocumentView;\n    type.FCPreparationView         = FCPreparationView;\n    type.FCDisplayView             = FCDisplayView;\n    type.FCManualInputView         = FCManualInputView;\n    type.FCManualOperationView     = FCManualOperationView;\n    type.FCCardView                = FCCardView;\n    type.FCPunchedTapeView         = FCPunchedTapeView;\n    type.FCConnectorView           = FCConnectorView;\n    type.FCOffPageConnectorView    = FCOffPageConnectorView;\n    type.FCOrView                  = FCOrView;\n    type.FCSummingJunctionView     = FCSummingJunctionView;\n    type.FCCollateView             = FCCollateView;\n    type.FCSortView                = FCSortView;\n    type.FCMergeView               = FCMergeView;\n    type.FCExtractView             = FCExtractView;\n    type.FCStoredDataView          = FCStoredDataView;\n    type.FCDatabaseView            = FCDatabaseView;\n    type.FCDirectAccessStorageView = FCDirectAccessStorageView;\n    type.FCInternalStorageView     = FCInternalStorageView;\n    type.FCFlowView                = FCFlowView;\n\n});\n\n//# sourceURL=/flowchart/Flowchart.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define */\n\ndefine('flowchart/FlowchartCommands',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n\n    exports.FLOWCHART_ADD_DIAGRAM = 'flowchart.add.diagram';\n\n});\n//# sourceURL=/flowchart/FlowchartCommands.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell, Mustache */\ndefine(\'flowchart/FlowchartFactory\',[\'require\',\'exports\',\'module\',\'utils/Helper\',\'core/Core\',\'core/OperationBuilder\',\'core/Repository\',\'engine/ProjectManager\',\'engine/Engine\',\'engine/Factory\',\'flowchart/Flowchart\'],function (require, exports, module) {\n    "use strict";\n\n    var Helper           = require("utils/Helper"),\n        Core             = require("core/Core"),\n        OperationBuilder = require("core/OperationBuilder"),\n        Repository       = require("core/Repository"),\n        ProjectManager   = require("engine/ProjectManager"),\n        Engine           = require("engine/Engine"),\n        Factory          = require("engine/Factory");\n\n    require("flowchart/Flowchart");\n\n    // Preconditions ...........................................................\n\n    function flowPrecondition(options) {\n        Helper.assert(\n            (options.tailModel instanceof type.FCModelElement) && (options.headModel instanceof type.FCModelElement),\n            Mustache.render(Factory.ERR_INVALID_LINK, options.modelType)\n        );\n    }\n\n    // Create ER diagram function ..............................................\n\n    function diagramFn(parent, options) {\n        var model,\n            diagram;\n        parent = parent || ProjectManager.getProject();\n        if (parent instanceof type.FCFlowchart) {\n            diagram = new type.FCFlowchartDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, diagram.getDisplayClassName());\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', diagram);\n        } else {\n            model = new type.FCFlowchart();\n            model.name = Core.getNewName(parent.ownedElements, "Flowchart");\n            model._parent = parent;\n            diagram = new type.FCFlowchartDiagram();\n            diagram.name = Core.getNewName(parent.ownedElements, diagram.getDisplayClassName());\n            model.ownedElements.push(diagram);\n            diagram._parent = model;\n            if (options.diagramInitializer) {\n                options.diagramInitializer(diagram);\n            }\n            Engine.addModel(parent, \'ownedElements\', model);\n        }\n        if (diagram) {\n            diagram = Repository.get(diagram._id);\n        }\n        options.triggerDiagramCreated(diagram);\n        return diagram;\n    }\n\n\n    // Create Diagram ..........................................................\n\n    Factory.registerDiagramFn("FCFlowchartDiagram", diagramFn);\n\n    // Create Model And View ...................................................\n\n    Factory.registerModelAndViewFn("FCFlow",                Factory.defaultDirectedRelationshipFn, { precondition: flowPrecondition });\n    Factory.registerModelAndViewFn("FCProcess",             Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCTerminator",          Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCDecision",            Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCDelay",               Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCPredefinedProcess",   Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCAlternateProcess",    Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCData",                Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCDocument",            Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCMultiDocument",       Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCPreparation",         Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCDisplay",             Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCManualInput",         Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCManualOperation",     Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCCard",                Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCPunchedTape",         Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCConnector",           Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCOffPageConnector",    Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCOr",                  Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCSummingJunction",     Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCCollate",             Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCSort",                Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCMerge",               Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCExtract",             Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCStoredData",          Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCDatabase",            Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCDirectAccessStorage", Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("FCInternalStorage",     Factory.defaultModelAndViewFn);\n\n});\n\n//# sourceURL=/flowchart/FlowchartFactory.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell */\ndefine(\'flowchart/FlowchartCommandHandlers\',[\'require\',\'exports\',\'module\',\'command/CommandManager\',\'core/Repository\',\'engine/SelectionManager\',\'engine/Factory\',\'diagrams/DiagramManager\',\'explorer/ModelExplorerView\',\'flowchart/FlowchartCommands\',\'flowchart/FlowchartFactory\'],function (require, exports, module) {\n    "use strict";\n\n    var CommandManager    = require("command/CommandManager"),\n        Repository        = require("core/Repository"),\n        SelectionManager  = require("engine/SelectionManager"),\n        Factory           = require("engine/Factory"),\n        DiagramManager    = require("diagrams/DiagramManager"),\n        ModelExplorerView = require("explorer/ModelExplorerView"),\n        FlowchartCommands = require("flowchart/FlowchartCommands"),\n        FlowchartFactory  = require("flowchart/FlowchartFactory");\n\n    function _updateCommands() {\n        var selected = SelectionManager.getSelected(),\n            isModel  = selected instanceof type.Model;\n        CommandManager.get(FlowchartCommands.FLOWCHART_ADD_DIAGRAM).setEnabled(isModel);\n    }\n\n    CommandManager.register("Flowchart Diagram", FlowchartCommands.FLOWCHART_ADD_DIAGRAM, _.partial(Factory.createDiagram, "FCFlowchartDiagram"));\n\n    // Update Commands\n    $(SelectionManager).on("selectionChanged", _updateCommands);\n    $(Repository).on("operationExecuted", _updateCommands);\n\n});\n\n//# sourceURL=/flowchart/FlowchartCommandHandlers.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell */\ndefine('flowchart/FlowchartMenus',['require','exports','module','utils/AppInit','command/Commands','menu/MenuManager','menu/ContextMenuManager','menu/DefaultMenus','uml/UMLCommands','flowchart/FlowchartCommands','flowchart/FlowchartCommandHandlers'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit            = require(\"utils/AppInit\"),\n        Commands           = require(\"command/Commands\"),\n        MenuManager        = require(\"menu/MenuManager\"),\n        ContextMenuManager = require(\"menu/ContextMenuManager\"),\n        DefaultMenus       = require(\"menu/DefaultMenus\"),\n        UMLCommands        = require(\"uml/UMLCommands\"),\n        FlowchartCommands =  require(\"flowchart/FlowchartCommands\");\n\n    require(\"flowchart/FlowchartCommandHandlers\");\n\n    /**\n     * Extend MenuBar\n     */\n    function setupMenus() {\n        var menuItem;\n        // Add Diagram\n        menuItem = MenuManager.getMenuItem(Commands.MODEL_ADD_DIAGRAM);\n        menuItem.addMenuItem(FlowchartCommands.FLOWCHART_ADD_DIAGRAM);\n    }\n\n    /**\n     * Extend ContextMenu for Explorer\n     */\n    function setupContextMenuForExplorer() {\n        var contextMenu, menu;\n        contextMenu = ContextMenuManager.getContextMenu(DefaultMenus.contextMenus.EXPLORER);\n\n        // Add Diagram\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD_DIAGRAM);\n        menu.addMenuItem(FlowchartCommands.FLOWCHART_ADD_DIAGRAM, \"icon-FCFlowchartDiagram\");\n    }\n\n    AppInit.htmlReady(function () {\n        setupMenus();\n        setupContextMenuForExplorer();\n    });\n\n});\n\n//# sourceURL=/flowchart/FlowchartMenus.js"),eval("/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell, document */\n\ndefine('flowchart/FlowchartToolbox',['require','exports','module','utils/AppInit','core/Core','core/Repository','engine/Factory','diagrams/ToolboxView','diagrams/DiagramManager','ui/QuickEdit','ui/Toast'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit          = require(\"utils/AppInit\"),\n        Core             = require(\"core/Core\"),\n        Repository       = require(\"core/Repository\"),\n        Factory          = require(\"engine/Factory\"),\n        Toolbox          = require(\"diagrams/ToolboxView\"),\n        DiagramManager   = require(\"diagrams/DiagramManager\"),\n        QuickEdit        = require(\"ui/QuickEdit\"),\n        Toast            = require(\"ui/Toast\");\n\n    /**\n     * Toolbox Groups\n     * @const\n     */\n    var TXG_FLOWCHART  = 'txg-fc';\n\n    /**\n     * Toolbox Items\n     * @const\n     */\n    var TX_PROCESS               = 'tx-fc-process',\n        TX_TERMINATOR            = 'tx-fc-terminator',\n        TX_DECISION              = 'tx-fc-decision',\n        TX_DELAY                 = 'tx-fc-delay',\n        TX_PREDEFINED_PROCESS    = 'tx-fc-predefined-process',\n        TX_ALTERNATE_PROCESS     = 'tx-fc-alternate-process',\n        TX_DATA                  = 'tx-fc-data',\n        TX_DOCUMENT              = 'tx-fc-document',\n        TX_MULTI_DOCUMENT        = 'tx-fc-multidocument',\n        TX_PREPARATION           = 'tx-fc-preparation',\n        TX_DISPLAY               = 'tx-fc-display',\n        TX_MANUAL_INPUT          = 'tx-fc-manualinput',\n        TX_MANUAL_OPERATION      = 'tx-fc-manualoperation',\n        TX_CARD                  = 'tx-fc-card',\n        TX_PUNCHED_TAPE          = 'tx-fc-punchedtape',\n        TX_CONNECTOR             = 'tx-fc-connector',\n        TX_OFF_PAGE_CONNECTOR    = 'tx-fc-offpageconnector',\n        TX_OR                    = 'tx-fc-or',\n        TX_SUMMING_JUNCTION      = 'tx-fc-summingjunction',\n        TX_COLLATE               = 'tx-fc-collate',\n        TX_SORT                  = 'tx-fc-sort',\n        TX_MERGE                 = 'tx-fc-merge',\n        TX_EXTRACT               = 'tx-fc-extract',\n        TX_STORED_DATA           = 'tx-fc-storeddata',\n        TX_DATABASE              = 'tx-fc-database',\n        TX_DIRECT_ACCESS_STORAGE = 'tx-fc-directaccessstorage',\n        TX_INTERNAL_STORAGE      = 'tx-fc-internalstorage',\n        TX_FLOW                  = 'tx-fc-flow';\n\n    function setupToolbox() {\n        Toolbox.addGroup(TXG_FLOWCHART, 'Flowchart', [type.FCFlowchartDiagram]);\n        Toolbox.addItem(TX_FLOW,                  TXG_FLOWCHART, 'Flow',                  'icon-FCFlow',                'line');\n        Toolbox.addItem(TX_PROCESS,               TXG_FLOWCHART, 'Process',               'icon-FCProcess',             'rect');\n        Toolbox.addItem(TX_TERMINATOR,            TXG_FLOWCHART, 'Terminator',            'icon-FCTerminator',          'rect');\n        Toolbox.addItem(TX_DECISION,              TXG_FLOWCHART, 'Decision',              'icon-FCDecision',            'rect');\n        Toolbox.addItem(TX_DELAY,                 TXG_FLOWCHART, 'Delay',                 'icon-FCDelay',               'rect');\n        Toolbox.addItem(TX_PREDEFINED_PROCESS,    TXG_FLOWCHART, 'Predefined Process',    'icon-FCPredefinedProcess',   'rect');\n        Toolbox.addItem(TX_ALTERNATE_PROCESS,     TXG_FLOWCHART, 'Alternate Process',     'icon-FCAlternateProcess',    'rect');\n        Toolbox.addItem(TX_DATA,                  TXG_FLOWCHART, 'Data',                  'icon-FCData',                'rect');\n        Toolbox.addItem(TX_DOCUMENT,              TXG_FLOWCHART, 'Document',              'icon-FCDocument',            'rect');\n        Toolbox.addItem(TX_MULTI_DOCUMENT,        TXG_FLOWCHART, 'Multi-Document',        'icon-FCMultiDocument',       'rect');\n        Toolbox.addItem(TX_PREPARATION,           TXG_FLOWCHART, 'Preparation',           'icon-FCPreparation',         'rect');\n        Toolbox.addItem(TX_DISPLAY,               TXG_FLOWCHART, 'Display',               'icon-FCDisplay',             'rect');\n        Toolbox.addItem(TX_MANUAL_INPUT,          TXG_FLOWCHART, 'Manual Input',          'icon-FCManualInput',         'rect');\n        Toolbox.addItem(TX_MANUAL_OPERATION,      TXG_FLOWCHART, 'Manual Operation',      'icon-FCManualOperation',     'rect');\n        Toolbox.addItem(TX_CARD,                  TXG_FLOWCHART, 'Card',                  'icon-FCCard',                'rect');\n        Toolbox.addItem(TX_PUNCHED_TAPE,          TXG_FLOWCHART, 'Punched Tape',          'icon-FCPunchedTape',         'rect');\n        Toolbox.addItem(TX_CONNECTOR,             TXG_FLOWCHART, 'Connector',             'icon-FCConnector',           'rect');\n        Toolbox.addItem(TX_OFF_PAGE_CONNECTOR,    TXG_FLOWCHART, 'Off-Page Connector',    'icon-FCOffPageConnector',    'rect');\n        Toolbox.addItem(TX_OR,                    TXG_FLOWCHART, 'Or',                    'icon-FCOr',                  'rect');\n        Toolbox.addItem(TX_SUMMING_JUNCTION,      TXG_FLOWCHART, 'Summing Junction',      'icon-FCSummingJunction',     'rect');\n        Toolbox.addItem(TX_COLLATE,               TXG_FLOWCHART, 'Collate',               'icon-FCCollate',             'rect');\n        Toolbox.addItem(TX_SORT,                  TXG_FLOWCHART, 'Sort',                  'icon-FCSort',                'rect');\n        Toolbox.addItem(TX_MERGE,                 TXG_FLOWCHART, 'Merge',                 'icon-FCMerge',               'rect');\n        Toolbox.addItem(TX_EXTRACT,               TXG_FLOWCHART, 'Extract',               'icon-FCExtract',             'rect');\n        Toolbox.addItem(TX_STORED_DATA,           TXG_FLOWCHART, 'Stored Data',           'icon-FCStoredData',          'rect');\n        Toolbox.addItem(TX_DATABASE,              TXG_FLOWCHART, 'Database',              'icon-FCDatabase',            'rect');\n        Toolbox.addItem(TX_DIRECT_ACCESS_STORAGE, TXG_FLOWCHART, 'Direct Access Storage', 'icon-FCDirectAccessStorage', 'rect');\n        Toolbox.addItem(TX_INTERNAL_STORAGE,      TXG_FLOWCHART, 'Internal Storage',      'icon-FCInternalStorage',     'rect');\n\n        // Event Handling\n        $(Toolbox).on('elementCreated', function (event, id, editor, x1, y1, x2, y2) {\n            try {\n                var diagram   = editor.diagram,\n                    parent    = diagram._parent,\n                    model     = null,\n                    view      = null,\n                    tailView  = diagram.getViewAt(editor.canvas, x1, y1, true),\n                    headView  = diagram.getViewAt(editor.canvas, x2, y2, true),\n                    tailModel = tailView ? tailView.model : null,\n                    headModel = headView ? headView.model : null;\n\n                var options = {\n                        x1        : x1,\n                        y1        : y1,\n                        x2        : x2,\n                        y2        : y2,\n                        tailView  : diagram.getViewAt(editor.canvas, x1, y1, true),\n                        headView  : diagram.getViewAt(editor.canvas, x2, y2, true),\n                        tailModel : tailView ? tailView.model : null,\n                        headModel : headView ? headView.model : null\n                    };                \n                \n                switch (id) {\n                case TX_FLOW:\n                    view = Factory.createModelAndView(\"FCFlow\", tailModel, diagram, options);\n                    break;\n                case TX_PROCESS:\n                    view = Factory.createModelAndView(\"FCProcess\", parent, diagram, options);\n                    break;\n                case TX_TERMINATOR:\n                    view = Factory.createModelAndView(\"FCTerminator\", parent, diagram, options);\n                    break;\n                case TX_DECISION:\n                    view = Factory.createModelAndView(\"FCDecision\", parent, diagram, options);\n                    break;\n                case TX_DELAY:\n                    view = Factory.createModelAndView(\"FCDelay\", parent, diagram, options);\n                    break;\n                case TX_PREDEFINED_PROCESS:\n                    view = Factory.createModelAndView(\"FCPredefinedProcess\", parent, diagram, options);\n                    break;\n                case TX_ALTERNATE_PROCESS:\n                    view = Factory.createModelAndView(\"FCAlternateProcess\", parent, diagram, options);\n                    break;\n                case TX_DATA:\n                    view = Factory.createModelAndView(\"FCData\", parent, diagram, options);\n                    break;\n                case TX_DOCUMENT:\n                    view = Factory.createModelAndView(\"FCDocument\", parent, diagram, options);\n                    break;\n                case TX_MULTI_DOCUMENT:\n                    view = Factory.createModelAndView(\"FCMultiDocument\", parent, diagram, options);\n                    break;\n                case TX_PREPARATION:\n                    view = Factory.createModelAndView(\"FCPreparation\", parent, diagram, options);\n                    break;\n                case TX_DISPLAY:\n                    view = Factory.createModelAndView(\"FCDisplay\", parent, diagram, options);\n                    break;\n                case TX_MANUAL_INPUT:\n                    options.modelInitializer = function (model) {\n                        model.name = Core.getNewName(model._parent.ownedElements, \"Input\");\n                    };                                            \n                    view = Factory.createModelAndView(\"FCManualInput\", parent, diagram, options);\n                    break;\n                case TX_MANUAL_OPERATION:\n                    options.modelInitializer = function (model) {\n                        model.name = Core.getNewName(model._parent.ownedElements, \"Operation\");\n                    };                                            \n                    view = Factory.createModelAndView(\"FCManualOperation\", parent, diagram, options);\n                    break;\n                case TX_CARD:\n                    view = Factory.createModelAndView(\"FCCard\", parent, diagram, options);\n                    break;\n                case TX_PUNCHED_TAPE:\n                    view = Factory.createModelAndView(\"FCPunchedTape\", parent, diagram, options);\n                    break;\n                case TX_CONNECTOR:\n                    options.modelInitializer = function (model) {\n                        model.name = Core.getNewName(model._parent.ownedElements, \"C\");\n                    };                    \n                    view = Factory.createModelAndView(\"FCConnector\", parent, diagram, options);\n                    break;\n                case TX_OFF_PAGE_CONNECTOR:\n                    options.modelInitializer = function (model) {\n                        model.name = Core.getNewName(model._parent.ownedElements, \"P\");\n                    };                                            \n                    view = Factory.createModelAndView(\"FCOffPageConnector\", parent, diagram, options);\n                    break;\n                case TX_OR:\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                    };\n                    view = Factory.createModelAndView(\"FCOr\", parent, diagram, options);\n                    break;\n                case TX_SUMMING_JUNCTION:\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                    };\n                    view = Factory.createModelAndView(\"FCSummingJunction\", parent, diagram, options);\n                    break;\n                case TX_COLLATE:\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                    };\n                    view = Factory.createModelAndView(\"FCCollate\", parent, diagram, options);\n                    break;\n                case TX_SORT:\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                    };\n                    view = Factory.createModelAndView(\"FCSort\", parent, diagram, options);\n                    break;\n                case TX_MERGE:\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                    };\n                    view = Factory.createModelAndView(\"FCMerge\", parent, diagram, options);\n                    break;\n                case TX_EXTRACT:\n                    options.modelInitializer = function (model) {\n                        model.name = \"\";\n                    };\n                    view = Factory.createModelAndView(\"FCExtract\", parent, diagram, options);\n                    break;\n                case TX_STORED_DATA:\n                    view = Factory.createModelAndView(\"FCStoredData\", parent, diagram, options);\n                    break;\n                case TX_DATABASE:\n                    view = Factory.createModelAndView(\"FCDatabase\", parent, diagram, options);\n                    break;\n                case TX_DIRECT_ACCESS_STORAGE:\n                    view = Factory.createModelAndView(\"FCDirectAccessStorage\", parent, diagram, options);\n                    break;\n                case TX_INTERNAL_STORAGE:\n                    view = Factory.createModelAndView(\"FCInternalStorage\", parent, diagram, options);\n                    break;\n                }\n                \n                // Open QuickEdit for the created view.\n                if (view) {\n                    view = Repository.get(view._id);\n                    if (view instanceof type.NodeView) {\n                        _.defer(function () {\n                            QuickEdit.open(editor, view, view.left, view.top);\n                        });\n                    } else if (view instanceof type.EdgeView) {\n                        _.defer(function () {\n                            QuickEdit.open(editor, view, Math.round((x1 + x2) / 2), Math.round((y1 + y2) / 2));\n                        });\n                    }\n                }\n                                \n            } catch (err) {\n                if (_.isString(err)) {\n                    Toast.error(err);\n                } else {\n                    console.error(err.stack);\n                }\n            }\n        });\n    }\n\n    function setupDiagramManager() {\n        $(DiagramManager).on('currentDiagramChanged', function (event, diagram, editor) {\n            try {\n                if (diagram instanceof type.FCFlowchartDiagram) {\n                    Toolbox.expandGroup(TXG_FLOWCHART);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n    \n    AppInit.htmlReady(function () {\n        setupToolbox();\n        setupDiagramManager();\n    });    \n\n});\n\n//# sourceURL=/flowchart/FlowchartToolbox.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell, staruml */\n\ndefine(\'flowchart/FlowchartPreferences\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Core\',\'core/PreferenceManager\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit           = require("utils/AppInit"),\n        Core              = require("core/Core"),\n        PreferenceManager = require("core/PreferenceManager");\n\n    var FlowchartPreferences = {\n        "flowchart.flow": {\n            text: "Flow",\n            type: "Section"\n        },\n        "flowchart.flow.lineStyle": {\n            text: "Line Style",\n            description: "Default line style of flow.",\n            type: "Dropdown",\n            options: [\n                { value: Core.LS_RECTILINEAR, text: "Rectilinear" },\n                { value: Core.LS_OBLIQUE,     text: "Oblique" },\n                { value: Core.LS_ROUNDRECT,   text: "Rounded Rectilinear" },\n                { value: Core.LS_CURVE,       text: "Curve" }\n            ],\n            default: Core.LS_ROUNDRECT\n        },\n        "flowchart.process": {\n            text: "Process",\n            type: "Section"\n        },\n        "flowchart.process.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Process.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.terminator": {\n            text: "Terminator",\n            type: "Section"\n        },\n        "flowchart.terminator.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Terminator.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.decision": {\n            text: "Decision",\n            type: "Section"\n        },\n        "flowchart.decision.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Decision.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.delay": {\n            text: "Delay",\n            type: "Section"\n        },\n        "flowchart.delay.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Delay.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.predefined-process": {\n            text: "Predefined Process",\n            type: "Section"\n        },\n        "flowchart.predefined-process.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Predefined Process.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.alternate-process": {\n            text: "Alternate Process",\n            type: "Section"\n        },\n        "flowchart.alternate-process.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Alternate Process.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.data": {\n            text: "Data",\n            type: "Section"\n        },\n        "flowchart.data.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Data.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.document": {\n            text: "Document",\n            type: "Section"\n        },\n        "flowchart.document.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Document.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.multidocument": {\n            text: "Multi-Document",\n            type: "Section"\n        },\n        "flowchart.multidocument.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Multi-Document.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.preparation": {\n            text: "Preparation",\n            type: "Section"\n        },\n        "flowchart.preparation.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Preparation.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.display": {\n            text: "Display",\n            type: "Section"\n        },\n        "flowchart.display.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Display.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.manual-input": {\n            text: "Manual Input",\n            type: "Section"\n        },\n        "flowchart.manual-input.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Manual Input.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.manual-operation": {\n            text: "Manual Operation",\n            type: "Section"\n        },\n        "flowchart.manual-operation.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Manual Operation.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.card": {\n            text: "Card",\n            type: "Section"\n        },\n        "flowchart.card.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Card.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.punched-tape": {\n            text: "Punched Tape",\n            type: "Section"\n        },\n        "flowchart.punched-tape.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Punched Tape.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.connector": {\n            text: "Connector",\n            type: "Section"\n        },\n        "flowchart.connector.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Connector.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.off-page-connector": {\n            text: "Off-Page Connector",\n            type: "Section"\n        },\n        "flowchart.off-page-connector.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Off-Page Connector.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.or": {\n            text: "Or",\n            type: "Section"\n        },\n        "flowchart.or.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Or.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.summing-junction": {\n            text: "Summing Junction",\n            type: "Section"\n        },\n        "flowchart.summing-junction.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Summing Junction.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.collate": {\n            text: "Collate",\n            type: "Section"\n        },\n        "flowchart.collate.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Collate.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.sort": {\n            text: "Sort",\n            type: "Section"\n        },\n        "flowchart.sort.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Sort.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.merge": {\n            text: "Merge",\n            type: "Section"\n        },\n        "flowchart.merge.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Merge.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.extract": {\n            text: "Extract",\n            type: "Section"\n        },\n        "flowchart.extract.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Extract.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.stored-data": {\n            text: "Stored Data",\n            type: "Section"\n        },\n        "flowchart.stored-data.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Stored Data.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.database": {\n            text: "Database",\n            type: "Section"\n        },\n        "flowchart.database.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Database.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.direct-access-storage": {\n            text: "Direct Access Storage",\n            type: "Section"\n        },\n        "flowchart.direct-access-storage.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Direct Access Storage.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n        "flowchart.internal-storage": {\n            text: "Internal Storage",\n            type: "Section"\n        },\n        "flowchart.internal-storage.fillColor": {\n            text: "Fill Color",\n            description: "Fill color of Internal Storage.",\n            type: "Color",\n            default: null,\n            defaultButton: {\n                text: "Use Default"\n            }\n        },\n    };\n\n    AppInit.htmlReady(function () {\n        PreferenceManager.register("flowchart", "Flowchart", FlowchartPreferences);\n    });\n\n});\n\n//# sourceURL=/flowchart/FlowchartPreferences.js'),eval("/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell, document */\n\ndefine('flowchart/main',['require','exports','module','utils/AppInit','utils/ExtensionUtils','engine/Engine','engine/Factory','diagrams/DiagramManager','ui/QuickEdit','flowchart/FlowchartQuickEdit','flowchart/Flowchart-meta','flowchart/Flowchart','flowchart/FlowchartCommands','flowchart/FlowchartCommandHandlers','flowchart/FlowchartMenus','flowchart/FlowchartToolbox','flowchart/FlowchartPreferences'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit            = require(\"utils/AppInit\"),\n        ExtensionUtils     = require(\"utils/ExtensionUtils\"),\n        Engine             = require(\"engine/Engine\"),\n        Factory            = require(\"engine/Factory\"),\n        DiagramManager     = require(\"diagrams/DiagramManager\"),\n        QuickEdit          = require(\"ui/QuickEdit\"),\n        FlowchartQuickEdit = require(\"flowchart/FlowchartQuickEdit\");\n\n    require(\"flowchart/Flowchart-meta\");\n    require(\"flowchart/Flowchart\");\n    require(\"flowchart/FlowchartCommands\");\n    require(\"flowchart/FlowchartCommandHandlers\");\n    require(\"flowchart/FlowchartMenus\");\n    require(\"flowchart/FlowchartToolbox\");\n    require(\"flowchart/FlowchartPreferences\");\n\n    function setupQuickEdit() {\n        // QuickEdit Value Changed Event Handlers\n        $(QuickEdit).on('change', function (event, id, itemId, view, value, feedback) {\n            try {\n                switch (id) {\n                case FlowchartQuickEdit.QUICKEDIT_NODE:\n                    switch (itemId) {\n                    case FlowchartQuickEdit.ITEM_EDIT:\n                        Engine.setProperty(view.model, 'name', value);\n                        break;\n                    }\n                    break;\n                case FlowchartQuickEdit.QUICKEDIT_EDGE:\n                    switch (itemId) {\n                    case FlowchartQuickEdit.ITEM_EDIT:\n                        if (view instanceof type.EdgeLabelView) { Engine.setProperty(view._parent.model, 'name', value); }\n                        break;\n                    }\n                    break;\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        // QuickEdit Button Click Event Handlers\n        $(QuickEdit).on('click', function (event, id, itemId, view) {\n            var diagram      = DiagramManager.getEditor().diagram,\n                diagramOwner = diagram._parent,\n                parent,\n                nodes, nodeView, nodeView1, nodeView2, edgeView, options1, options2, _x, _y;\n            try {\n                switch (id) {\n                case FlowchartQuickEdit.QUICKEDIT_NODE:\n                    switch (itemId) {\n                    case FlowchartQuickEdit.ITEM_ADD_COLUMN:\n                        QuickEdit.addInQuickEdit(view.model, 'columns', view, 'columnCompartment', type.ERDColumn, 'Column');\n                        break;\n                    case FlowchartQuickEdit.ITEM_ADD_OUTGOING_PROCESS:\n                        nodes = QuickEdit.getBothNodes(view, type.FCFlowView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView(\"FCProcess\", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView(\"FCFlow\", view.model, diagram, options2);\n                        break;\n                    case FlowchartQuickEdit.ITEM_ADD_INCOMING_PROCESS:\n                        nodes = QuickEdit.getBothNodes(view, type.FCFlowView, function (e) {\n                            return (e.head === view);\n                        });\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView(\"FCProcess\", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView(\"FCFlow\", nodeView.model, diagram, options2);\n                        break;\n                    case FlowchartQuickEdit.ITEM_ADD_OUTGOING_DECISION:\n                        nodes = QuickEdit.getBothNodes(view, type.FCFlowView, function (e) {\n                            return (e.tail === view);\n                        });\n                        options1 = QuickEdit.getBottomPosition(view, nodes);\n                        nodeView = Factory.createModelAndView(\"FCDecision\", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(view, nodeView);\n                        edgeView = Factory.createModelAndView(\"FCFlow\", view.model, diagram, options2);\n                        break;\n                    case FlowchartQuickEdit.ITEM_ADD_INCOMING_DECISION:\n                        nodes = QuickEdit.getBothNodes(view, type.FCFlowView, function (e) {\n                            return (e.head === view);\n                        });\n                        options1 = QuickEdit.getTopPosition(view, nodes);\n                        nodeView = Factory.createModelAndView(\"FCDecision\", diagramOwner, diagram, options1);\n                        options2 = QuickEdit.getEdgeViewOption(nodeView, view);\n                        edgeView = Factory.createModelAndView(\"FCFlow\", nodeView.model, diagram, options2);\n                        break;\n                    }\n                    break;\n                case FlowchartQuickEdit.QUICKEDIT_EDGE:\n                    switch (itemId) {\n                    case FlowchartQuickEdit.ITEM_ADD:\n                        QuickEdit.addInQuickEdit(view.model._parent, 'columns', view._parent._parent, 'columnCompartment', type.ERDColumn, 'Column');\n                        break;\n                    case FlowchartQuickEdit.ITEM_DELETE:\n                        QuickEdit.deleteInQuickEdit(view);\n                        break;\n                    case FlowchartQuickEdit.ITEM_MOVEUP:\n                        QuickEdit.moveUpInQuickEdit(view.model._parent, 'columns', view);\n                        break;\n                    case FlowchartQuickEdit.ITEM_MOVEDOWN:\n                        QuickEdit.moveDownInQuickEdit(view.model._parent, 'columns', view);\n                        break;\n                    }\n                    break;\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    \n    AppInit.htmlReady(function () {\n        setupQuickEdit();\n    });\n    \n});\n\n//# sourceURL=/flowchart/main.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true, vars: true */\n/*global define, $, _, window, appshell, type, Mustache */\n\ndefine(\'common/CommonFactory\',[\'require\',\'exports\',\'module\',\'engine/Factory\'],function (require, exports, module) {\n    "use strict";\n\n    var Factory = require("engine/Factory");\n\n    // Common Elements\n    Factory.registerModelFn("Tag", Factory.defaultModelFn);\n\n    // Annotations\n    Factory.registerModelAndViewFn("Text",      Factory.defaultViewOnlyFn, { viewType: "UMLTextView" });\n    Factory.registerModelAndViewFn("Note",      Factory.defaultViewOnlyFn, { viewType: "UMLNoteView" });\n    Factory.registerModelAndViewFn("NoteLink",  Factory.defaultEdgeViewOnlyFn, { viewType: "UMLNoteLinkView" });\n    Factory.registerModelAndViewFn("Hyperlink", Factory.defaultModelAndViewFn);\n    Factory.registerModelAndViewFn("Rect",      Factory.defaultViewOnlyFn, { viewType: "RectangleView" });\n    Factory.registerModelAndViewFn("RoundRect", Factory.defaultViewOnlyFn, { viewType: "RoundRectView" });\n    Factory.registerModelAndViewFn("Ellipse",   Factory.defaultViewOnlyFn, { viewType: "EllipseView" });\n\n});\n\n//# sourceURL=/common/CommonFactory.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell */\ndefine('common/CommonCommands',['require','exports','module'],function (require, exports, module) {\n    \"use strict\";\n\n    exports.MODEL_ADD_TAG = 'model.add.tag';\n\n});\n\n//# sourceURL=/common/CommonCommands.js"),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell */\ndefine(\'common/CommonCommandHandlers\',[\'require\',\'exports\',\'module\',\'core/Repository\',\'engine/Factory\',\'engine/SelectionManager\',\'command/CommandManager\',\'common/CommonCommands\',\'strings\'],function (require, exports, module) {\n    "use strict";\n\n    var Repository        = require("core/Repository"),\n        Factory           = require("engine/Factory"),\n        SelectionManager  = require("engine/SelectionManager"),\n        CommandManager    = require("command/CommandManager"),\n        CommonCommands    = require("common/CommonCommands"),\n        Strings           = require("strings");\n\n    function _updateCommands() {\n        var models = SelectionManager.getSelectedModels(),\n            views = SelectionManager.getSelectedViews(),\n            selected = SelectionManager.getSelected();\n\n        var isNone = (!selected),\n            isExtensibleModel = selected instanceof type.ExtensibleModel;\n\n        CommandManager.get(CommonCommands.MODEL_ADD_TAG).setVisible(isExtensibleModel);\n    }\n\n    CommandManager.register(Strings.CMD_MODEL_ADD_TAG, CommonCommands.MODEL_ADD_TAG, _.partial(Factory.createModel, "Tag", null, "tags"));\n\n    // Update Commands\n    $(SelectionManager).on("selectionChanged", _updateCommands);\n    $(Repository).on("operationExecuted", _updateCommands);\n\n});\n\n//# sourceURL=/common/CommonCommandHandlers.js'),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, appshell */\ndefine('common/CommonMenu',['require','exports','module','utils/AppInit','command/Commands','menu/MenuManager','menu/ContextMenuManager','menu/DefaultMenus','common/CommonCommands'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit            = require(\"utils/AppInit\"),\n        Commands           = require(\"command/Commands\"),\n        MenuManager        = require(\"menu/MenuManager\"),\n        ContextMenuManager = require(\"menu/ContextMenuManager\"),\n        DefaultMenus       = require(\"menu/DefaultMenus\"),\n        CommonCommands     = require(\"common/CommonCommands\");\n\n    function setupMenus() {\n        var menuItem;\n        menuItem = MenuManager.getMenuItem(Commands.MODEL_ADD);\n        menuItem.addMenuItem(CommonCommands.MODEL_ADD_TAG, null, null, null, \"icon-Tag\");\n    }\n\n    function setupContextMenuForDiagram() {\n        var contextMenu, menu, menuItem;\n        contextMenu = ContextMenuManager.getContextMenu(DefaultMenus.contextMenus.DIAGRAM);\n\n        // Model Add\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD);\n        menu.addMenuItem(CommonCommands.MODEL_ADD_TAG, \"icon-Tag\");\n    }\n\n    function setupContextMenuForExplorer() {\n        var contextMenu, menu;\n        contextMenu = ContextMenuManager.getContextMenu(DefaultMenus.contextMenus.EXPLORER);\n\n        // Model Add\n        menu = contextMenu.getMenuItem(Commands.MODEL_ADD);\n        menu.addMenuItem(CommonCommands.MODEL_ADD_TAG, \"icon-Tag\");\n    }\n\n    AppInit.htmlReady(function () {\n        setupMenus();\n        setupContextMenuForDiagram();\n        setupContextMenuForExplorer();\n    });\n\n});\n\n//# sourceURL=/common/CommonMenu.js"),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, appshell, type */\n\ndefine('common/CommonToolbox',['require','exports','module','utils/AppInit','core/Repository','engine/Factory','diagrams/ToolboxView','dialogs/Dialogs','ui/Toast','ui/QuickEdit','uml/UML','uml/UMLDiagram','uml/UMLFactory','uml/UMLToolbox','erd/ERDToolbox','flowchart/FlowchartToolbox'],function (require, exports, module) {\n    \"use strict\";\n\n    var AppInit                 = require(\"utils/AppInit\"),\n        Repository              = require(\"core/Repository\"),\n        Factory                 = require(\"engine/Factory\"),\n        Toolbox                 = require(\"diagrams/ToolboxView\"),\n        Dialogs                 = require(\"dialogs/Dialogs\"),\n        Toast                   = require(\"ui/Toast\"),\n        QuickEdit               = require(\"ui/QuickEdit\"),\n        UML                     = require(\"uml/UML\"),\n        UMLDiagram              = require(\"uml/UMLDiagram\"),\n        UMLFactory              = require(\"uml/UMLFactory\");\n\n    // for loading priority\n    require(\"uml/UMLToolbox\");\n    require(\"erd/ERDToolbox\");\n    require(\"flowchart/FlowchartToolbox\");\n\n    /**\n     * Toolbox Groups\n     * @const\n     */\n    var TXG_ANNOTATION         = 'txg-annotation';\n\n    /**\n     * Toolbox Items\n     * @const\n     */\n    var TX_TEXT                 = 'tx-text',\n        TX_NOTE                 = 'tx-note',\n        TX_NOTELINK             = 'tx-notelink',\n        TX_HYPERLINK            = 'tx-hyperlink',\n        TX_RECT                 = 'tx-rect',\n        TX_ROUNDRECT            = 'tx-roundrect',\n        TX_ELLIPSE              = 'tx-ellipse';\n\n    /**\n     * Error Messages\n     */\n    var ERR_INVALID_PARENT = \"{{.}} cannot be placed here.\",\n        ERR_INVALID_LINK   = \"Invalid connection ({{.}})\";\n\n\n    function setupToolbox() {\n        // Annotation\n        Toolbox.addGroup(TXG_ANNOTATION, 'Annotations', []); // for all diagrams\n        Toolbox.addItem(TX_TEXT,      TXG_ANNOTATION, 'Text',              'icon-UMLText',          'rect');\n        Toolbox.addItem(TX_NOTE,      TXG_ANNOTATION, 'Note',              'icon-UMLNote',          'rect');\n        Toolbox.addItem(TX_NOTELINK,  TXG_ANNOTATION, 'Note Link',         'icon-UMLNoteLink',      'line');\n        Toolbox.addItem(TX_HYPERLINK, TXG_ANNOTATION, 'Hyperlink',         'icon-Hyperlink',        'rect');\n        Toolbox.addItem(TX_RECT,      TXG_ANNOTATION, 'Rectangle',         'icon-Rectangle',        'rect');\n        Toolbox.addItem(TX_ROUNDRECT, TXG_ANNOTATION, 'Rounded Rectangle', 'icon-RoundedRectangle', 'rect');\n        Toolbox.addItem(TX_ELLIPSE,   TXG_ANNOTATION, 'Ellipse',           'icon-Ellipse',          'rect');\n\n        // Event Handling\n        $(Toolbox).on('elementCreated', function (event, id, editor, x1, y1, x2, y2) {\n            try {\n                var diagram   = editor.diagram,\n                    parent    = diagram._parent,\n                    model     = null,\n                    view      = null,\n                    tailView  = diagram.getViewAt(editor.canvas, x1, y1, true),\n                    headView  = diagram.getViewAt(editor.canvas, x2, y2, true),\n                    tailModel = tailView ? tailView.model : null,\n                    headModel = headView ? headView.model : null,\n                    viewTypes;\n\n                var options = {\n                        x1        : x1,\n                        y1        : y1,\n                        x2        : x2,\n                        y2        : y2,\n                        tailView  : diagram.getViewAt(editor.canvas, x1, y1, true),\n                        headView  : diagram.getViewAt(editor.canvas, x2, y2, true),\n                        tailModel : tailView ? tailView.model : null,\n                        headModel : headView ? headView.model : null\n                    };\n\n                switch (id) {\n                case TX_TEXT:\n                    if ((options.x2 - options.x1) < 5) { options.x2 = options.x1 + 60; }\n                    if ((options.y2 - options.y1) < 5) { options.y2 = options.y1 + 15; }\n                    view = Factory.createModelAndView(\"Text\", parent, diagram, options);\n                    break;\n                case TX_NOTE:\n                    if ((options.x2 - options.x1) < 5) { options.x2 = options.x1 + 70; }\n                    if ((options.y2 - options.y1) < 5) { options.y2 = options.y1 + 40; }\n                    view = Factory.createModelAndView(\"Note\", parent, diagram, options);\n                    break;\n                case TX_NOTELINK:\n                    view = Factory.createModelAndView(\"NoteLink\", parent, diagram, options);\n                    break;\n                case TX_HYPERLINK:\n                    if ((options.x2 - options.x1) < 5) { options.x2 = options.x1 + 70; }\n                    if ((options.y2 - options.y1) < 5) { options.y2 = options.y1 + 20; }\n                    view = Factory.createModelAndView(\"Hyperlink\", parent, diagram, options);\n                    break;\n                case TX_RECT:\n                    if ((options.x2 - options.x1) < 5) { options.x2 = options.x1 + 50; }\n                    if ((options.y2 - options.y1) < 5) { options.y2 = options.y1 + 50; }\n                    view = Factory.createModelAndView(\"Rect\", parent, diagram, options);\n                    break;\n                case TX_ROUNDRECT:\n                    if ((options.x2 - options.x1) < 5) { options.x2 = options.x1 + 50; }\n                    if ((options.y2 - options.y1) < 5) { options.y2 = options.y1 + 50; }\n                    view = Factory.createModelAndView(\"RoundRect\", parent, diagram, options);\n                    break;\n                case TX_ELLIPSE:\n                    if ((options.x2 - options.x1) < 5) { options.x2 = options.x1 + 50; }\n                    if ((options.y2 - options.y1) < 5) { options.y2 = options.y1 + 50; }\n                    view = Factory.createModelAndView(\"Ellipse\", parent, diagram, options);\n                    break;\n                }\n\n                // Open QuickEdit for the created view.\n                if (view) {\n                    view = Repository.get(view._id);\n                    if (view instanceof type.NodeView) {\n                        _.defer(function () {\n                            QuickEdit.open(editor, view, view.left, view.top);\n                        });\n                    } else if (view instanceof type.EdgeView) {\n                        _.defer(function () {\n                            QuickEdit.open(editor, view, Math.round((x1 + x2) / 2), Math.round((y1 + y2) / 2));\n                        });\n                    }\n                }\n\n            } catch (err) {\n                if (_.isString(err)) {\n                    Toast.error(err);\n                } else {\n                    console.error(err.stack);\n                }\n            }\n        });\n    }\n\n    AppInit.htmlReady(function () {\n        setupToolbox();\n    });\n\n});\n\n//# sourceURL=/common/CommonToolbox.js"),eval("/*\n * Copyright (c) 2014 MKLab. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the \"Software\"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, app, type, appshell, document */\n\ndefine('common/main',['require','exports','module','common/CommonFactory','common/CommonCommandHandlers','common/CommonMenu','common/CommonToolbox'],function (require, exports, module) {\n    \"use strict\";\n\n    require(\"common/CommonFactory\");\n    require(\"common/CommonCommandHandlers\");\n    require(\"common/CommonMenu\");\n    require(\"common/CommonToolbox\");\n\n});\n\n//# sourceURL=/common/main.js"),eval("/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global define, $, _, window, type, app, appshell, document, localStorage, setInterval */\ndefine('Application',['require','exports','module','core/Global','utils/Initializer','utils/Helper','utils/AppInit','utils/ExtensionUtils','utils/PanelManager','core/Graphics','core/Core','core/MetaModelManager','core/PreferenceManager','preference/PreferenceDialog','core/Repository','engine/ProjectManager','core/OperationBuilder','engine/Engine','engine/SelectionManager','engine/ClipboardManager','engine/Factory','extensibility/ExtensionManager','extensibility/ExtensionManagerDialog','file/FileUtils','filesystem/FileSystem','filesystem/FileSystemError','command/CommandManager','command/Commands','utils/KeyEvent','explorer/NavigatorView','explorer/ModelExplorerView','editors/EditorsHolder','editors/StyleEditorView','editors/PropertyEditorView','editors/DocumentationEditorView','diagrams/DiagramManager','diagrams/SidebarView','diagrams/WorkingDiagramsView','diagrams/ToolboxView','ui/Toolbar','ui/StatusBar','dialogs/Dialogs','dialogs/AboutDialog','ui/QuickEdit','dialogs/ElementPickerDialog','utils/CheckUpdates','utils/Analytics','license/LicenseManager','validation/Validator','utils/FontManager','core/Core-meta','core/Core-rules','utils/ExtensionUtils','utils/NodeDomain','utils/Async','command/KeyBindingManager','menu/DefaultMenus','command/DefaultCommandHandlers','menu/RecentFiles','preference/DefaultPreferences','uml/main','erd/main','flowchart/main','common/main','core/Core','core/Core','diagrams/Editor'],function (require, exports, module) {\n    \"use strict\";\n\n    var _global                 = require(\"core/Global\").global,\n        Initializer             = require(\"utils/Initializer\"),\n        Helper                  = require(\"utils/Helper\"),\n        AppInit                 = require(\"utils/AppInit\"),\n        ExtensionUtils          = require(\"utils/ExtensionUtils\"),\n        PanelManager            = require(\"utils/PanelManager\"),\n        Graphics                = require(\"core/Graphics\"),\n        Core                    = require(\"core/Core\"),\n        MetaModelManager        = require(\"core/MetaModelManager\"),\n        PreferenceManager       = require(\"core/PreferenceManager\"),\n        PreferenceDialog        = require(\"preference/PreferenceDialog\"),\n        Repository              = require(\"core/Repository\"),\n        ProjectManager          = require(\"engine/ProjectManager\"),\n        OperationBuilder        = require(\"core/OperationBuilder\"),\n        Engine                  = require(\"engine/Engine\"),\n        SelectionManager        = require(\"engine/SelectionManager\"),\n        ClipboardManager        = require(\"engine/ClipboardManager\"),\n        Factory                 = require(\"engine/Factory\"),\n        ExtensionManager        = require(\"extensibility/ExtensionManager\"),\n        ExtensionManagerDialog  = require(\"extensibility/ExtensionManagerDialog\"),\n        FileUtils               = require(\"file/FileUtils\"),\n        FileSystem              = require(\"filesystem/FileSystem\"),\n        FileSystemError         = require(\"filesystem/FileSystemError\"),\n        CommandManager          = require(\"command/CommandManager\"),\n        Commands                = require(\"command/Commands\"),\n        KeyEvent                = require(\"utils/KeyEvent\"),\n        NavigatorView           = require(\"explorer/NavigatorView\"),\n        ModelExplorerView       = require(\"explorer/ModelExplorerView\"),\n        EditorsHolder           = require(\"editors/EditorsHolder\"),\n        StyleEditorView         = require(\"editors/StyleEditorView\"),\n        PropertyEditorView      = require(\"editors/PropertyEditorView\"),\n        DocumentationEditorView = require(\"editors/DocumentationEditorView\"),\n        DiagramManager          = require(\"diagrams/DiagramManager\"),\n        SidebarView             = require(\"diagrams/SidebarView\"),\n        WorkingDiagramsView     = require(\"diagrams/WorkingDiagramsView\"),\n        ToolboxView             = require(\"diagrams/ToolboxView\"),\n        Toolbar                 = require(\"ui/Toolbar\"),\n        StatusBar               = require(\"ui/StatusBar\"),\n        Dialogs                 = require(\"dialogs/Dialogs\"),\n        AboutDialog             = require(\"dialogs/AboutDialog\"),\n        QuickEdit               = require(\"ui/QuickEdit\"),\n        ElementPickerDialog     = require(\"dialogs/ElementPickerDialog\"),\n        CheckUpdates            = require(\"utils/CheckUpdates\"),\n        Analytics               = require(\"utils/Analytics\"),\n        LicenseManager          = require(\"license/LicenseManager\"),\n        Validator               = require(\"validation/Validator\"),\n        FontManager             = require(\"utils/FontManager\");\n\n    require(\"core/Core-meta\");\n    require(\"core/Core-rules\");\n\n    // for dependency\n    require(\"utils/ExtensionUtils\");\n    require(\"utils/NodeDomain\");\n    require(\"utils/Async\");\n\n    // for New Menu Testing\n    require(\"command/KeyBindingManager\");\n    require(\"menu/DefaultMenus\");\n    require(\"command/DefaultCommandHandlers\");\n    require(\"menu/RecentFiles\");\n    require(\"preference/DefaultPreferences\");\n\n    // load UML/main\n    require(\"uml/main\");\n\n    // load ERD/main\n    require(\"erd/main\");\n\n    // load Flowchart/main\n    require(\"flowchart/main\");\n\n    // load Common/main\n    require(\"common/main\");\n\n\n    var Model         = require(\"core/Core\").Model,\n        Diagram       = require(\"core/Core\").Diagram,\n        Editor        = require(\"diagrams/Editor\").Editor;\n\n    var APP_NAME = app.metadata.name;\n\n    var AUTO_BACKUP_INTERVAL = 1000 * 60 * 5; // 5 minutes.\n\n\n    // Preferences\n    var PREF_GENERAL   = 'staruml-general',\n        PREF_VIEWSTYLE = 'view-style',\n        PREF_FONT      = \"font\",\n        PREF_FONTSIZE  = \"font-size\",\n        PREF_FILLCOLOR = \"fill-color\",\n        PREF_LINECOLOR = \"line-color\",\n        PREF_FONTCOLOR = \"font-color\",\n        PREF_LINESTYLE = \"line-style\";\n\n    // Icons\n    var ICON_FILL_COLOR  = \"tool-icon-fill-color\",\n        ICON_LINE_COLOR  = \"tool-icon-line-color\",\n        ICON_FONT_COLOR  = \"tool-icon-font-color\",\n        ICON_AUTO_RESIZE = \"tool-icon-auto-resize\";\n\n    // Toolbar Items\n    var TB_FILL_COLOR     = 'toolbar-fill-color',\n        TB_LINE_COLOR     = 'toolbar-line-color',\n        TB_FONT_COLOR     = 'toolbar-font-color',\n        TB_FONT_FACE      = 'toolbar-font-face',\n        TB_FONT_SIZE      = 'toolbar-font-size',\n        TB_LINE_STYLE     = 'toolbar-line-style',\n        TB_AUTO_RESIZE    = 'toolbar-auto-resize';\n\n\n    function updateTitlebar() {\n        var filename = ProjectManager.getFilename(),\n            title = \"\";\n\n        if (Repository.isModified()) {\n            title += \"• \";\n        }\n\n        if (filename && filename.length > 0) {\n            filename = FileUtils.getBaseName(filename);\n            title += filename + \" — \";\n        }\n\n        title += APP_NAME;\n\n        var license = LicenseManager.getStatus();\n        if (LicenseManager.STAT_REGISTERED !== license && license) {\n            title += \" (\" + license + \")\";\n        }\n\n        $(\"title\").html(title);\n    }\n\n\n    /**\n     * Repository Setup\n     */\n    function setupRepository() {\n        $(Repository).on('created', function (event, elems) {\n            try {\n                _.forEach(elems, function (elem) {\n                    if (elem instanceof Core.Model) {\n                        ModelExplorerView.add(elem, true);\n                        ModelExplorerView.select(elem, true);\n                    }\n                });\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(Repository).on('updated', function (event, elems) {\n            try {\n                _.forEach(elems, function (elem) {\n                    if (elem instanceof Diagram) {\n                        DiagramManager.updateDiagram(elem);\n                    }\n                    if (SelectionManager.getSelected() === elem) {\n                        PropertyEditorView.show(SelectionManager.getSelectedModels());\n                        DocumentationEditorView.show(elem);\n                    }\n                    // update ModelExplorer\n                    ModelExplorerView.update(elem);\n                    // update Relations of elem in ModelExplorer\n                    var relations = Repository.getRelationshipsOf(elem);\n                    if (relations.length > 0) {\n                        _.each(relations, function (rel) {\n                            ModelExplorerView.update(rel);\n                        });\n                    }\n                });\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(Repository).on('deleted', function (event, elems) {\n            try {\n                _.forEach(elems, function (elem) {\n                    if (elem instanceof _global.type.Model) {\n                        ModelExplorerView.remove(elem);\n                    }\n                });\n                // Close deleted diagrams\n                _.forEach(DiagramManager.getWorkingDiagrams(), function (dgm) {\n                    if (!Repository.get(dgm._id)) {\n                        DiagramManager.closeDiagram(dgm);\n                    }\n                });\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(Repository).on('reordered', function (event, elem) {\n            try {\n                if (elem instanceof _global.type.Model) {\n                    ModelExplorerView.remove(elem);\n                    ModelExplorerView.add(elem, true);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(Repository).on('relocated', function (event, elem, field, oldParent, newParent) {\n            try {\n                if (elem instanceof _global.type.Model) {\n                    ModelExplorerView.remove(elem);\n                    ModelExplorerView.add(elem, true);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(Repository).on('operationExecuted', function (event, operation) {\n            try {\n                var elems = Repository.extractChanged(operation);\n                if (DiagramManager.needRepaint(elems)) {\n                    DiagramManager.repaint();\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(Repository).on('modified', function (event) {\n            try {\n                updateTitlebar();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n\n    /**\n     * Factory Setup\n     */\n    function setupFactory() {\n        $(Factory).on('diagramCreated', function (event, diagram) {\n            DiagramManager.setCurrentDiagram(diagram);\n            ModelExplorerView.select(diagram, true);\n        });\n        $(Factory).on('elementCreated', function (event, model, view) {\n        });\n    }\n\n\n    /**\n     * ProjectManager Setup\n     */\n    function setupProjectManager() {\n        $(ProjectManager).on('projectCreated', function (event, project) {\n            try {\n                DiagramManager.closeAll();\n                ModelExplorerView.rebuild();\n                updateTitlebar();\n                ToolboxView.setDiagram(null);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(ProjectManager).on('projectLoaded', function (event, filename, project) {\n            try {\n                ModelExplorerView.rebuild();\n                ModelExplorerView.expand(project);\n                updateTitlebar();\n\n                // Remember working file so as to open the working file when app is launched.\n                if (filename) {\n                    localStorage.setItem(\"__working_filename\", filename);\n                }\n\n                // Open default diagrams\n                var defaultDiagrams = Repository.findAll(function (elem) {\n                    return (elem instanceof _global.type.Diagram && elem.defaultDiagram === true);\n                });\n                _.each(defaultDiagrams, function (diagram) {\n                    DiagramManager.openDiagram(diagram);\n                });\n\n                if (defaultDiagrams.length > 0) {\n                    DiagramManager.setCurrentDiagram(defaultDiagrams[0]);\n                }\n                DiagramManager.restoreWorkingDiagrams();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(ProjectManager).on('projectSaved', function (event, filename, project) {\n            try {\n                updateTitlebar();\n                DiagramManager.saveWorkingDiagrams();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(ProjectManager).on('beforeProjectClose', function (event, filename, project) {\n            try {\n                DiagramManager.saveWorkingDiagrams();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(ProjectManager).on('projectClosed', function (event) {\n            try {\n                DiagramManager.closeAll();\n                ModelExplorerView.rebuild();\n                ToolboxView.setDiagram(null);\n                PropertyEditorView.show(null);\n                DocumentationEditorView.show(null);\n                updateTitlebar();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        $(ProjectManager).on('exported', function (event, filename, elem) {\n        });\n\n        $(ProjectManager).on('imported', function (event, filename, elem) {\n            try {\n                ModelExplorerView.add(elem, true);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n\n    /**\n     * SelectionManager Setup\n     */\n    function setupSelectionManager() {\n        $(SelectionManager).on('selectionChanged', function (event, models, views) {\n            try {\n                if (views.length > 0) {\n                    console.log(views[0]); // for TEST\n                    ModelExplorerView.deselect();\n                } else {\n                    DiagramManager.deselectAll();\n                }\n\n                // Update Property/Documentation/Tag Editor\n                if (models.length === 1) {\n                    DocumentationEditorView.show(models[0]);\n                    StatusBar.setElement(models[0]);\n                } else {\n                    DocumentationEditorView.show(null);\n                    StatusBar.setElement(null);\n                }\n\n                // Update StyleEditor\n                StyleEditorView.show(views);\n\n                // Update PropertyEditor\n                PropertyEditorView.show(models);\n\n                // Update DiagramManager\n\n                if (DiagramManager.needRepaint(_.union(models, views))) {\n                    DiagramManager.repaint();\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    function setupModelExplorer() {\n        $(ModelExplorerView).on('selected', function (event, elem) {\n            try {\n                SelectionManager.select([elem], null);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(ModelExplorerView).on('doubleClicked', function (event, elem) {\n            try {\n                if (elem instanceof Diagram) {\n                    DiagramManager.setCurrentDiagram(elem);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        // 모델 요소를 ModelExplorer 내에서 Drag 할때, Accept 여부 판단.\n        $(ModelExplorerView).on('dragOver', function (event, dragEvent) {\n            try {\n                var source = dragEvent.source,\n                    target = dragEvent.target,\n                    field  = dragEvent.source.getParentField();\n                if (target[field] && target.canContain(source)) {\n                    dragEvent.accept = true;\n                } else {\n                    dragEvent.accept = false;\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        // 모델 요소를 ModelExplorer 내에서 Drop 할때.\n        $(ModelExplorerView).on('drop', function (event, dropEvent) {\n            try {\n                var source = dropEvent.source,\n                    target = dropEvent.target,\n                    field  = dropEvent.source.getParentField();\n                if (target.canContain(source) && target[field]) {\n                    Engine.relocate(source, target, field);\n                    ModelExplorerView.select(source, true);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        // 모델 요소를 ModelExplorer에서 Diagram영역으로 Drag 할때, Accept 여부 판단.\n        $(ModelExplorerView).on('dragOverDiagram', function (event, dragEvent) {\n            try {\n                dragEvent.accept = dragEvent.diagram.canAcceptModel(dragEvent.source);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        // 모델 요소를 ModelExplorer에서 Diagram영역으로 Drop 할때.\n        $(ModelExplorerView).on('dropOnDiagram', function (event, dropEvent) {\n            try {\n                if (dropEvent.diagram.canAcceptModel(dropEvent.source)) {\n                    var editor        = DiagramManager.getEditor(),\n                        diagram       = editor.diagram,\n                        model         = dropEvent.source,\n                        p             = editor.convertPosition(dropEvent),\n                        containerView = diagram.getViewAt(editor.canvas, p.x, p.y, true);\n\n                    var options = {\n                            editor        : editor,\n                            x             : p.x,\n                            y             : p.y,\n                            containerView : containerView\n                        };\n                    Factory.createViewOf(model, diagram, options);\n                    DiagramManager.repaint();\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    function setupStyleEditor() {\n        $(StyleEditorView).on('styleChanged', function (event, views, field, value) {\n            try {\n                if (field === \"font.face\") {\n                    Engine.setFontFace(DiagramManager.getEditor(), views, value);\n                } else if (field === \"font.size\") {\n                    if (!_.isNumber(value)) {\n                        value = parseInt(value, 10);\n                    }\n                    Engine.setFontSize(DiagramManager.getEditor(), views, value);\n                } else if (field === \"fillColor\") {\n                    Engine.setFillColor(DiagramManager.getEditor(), views, value);\n                } else if (field === \"lineColor\") {\n                    Engine.setLineColor(DiagramManager.getEditor(), views, value);\n                } else if (field === \"fontColor\") {\n                    Engine.setFontColor(DiagramManager.getEditor(), views, value);\n                } else if (field === \"lineStyle\") {\n                    Engine.setLineStyle(DiagramManager.getEditor(), views, value);\n                } else if (field === \"autoResize\") {\n                    Engine.setAutoResize(DiagramManager.getEditor(), views, value);\n                } else {\n                    Engine.setElemsProperty(views, field, value);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    function setupPropertyEditor() {\n        $(PropertyEditorView).on('propertyChanged', function (event, elems, field, value) {\n            try {\n                Engine.setElemsProperty(elems, field, value);\n                DiagramManager.repaint();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    function setupDocumentationEditor() {\n        $(DocumentationEditorView).on('documentationChanged', function (event, elem, value) {\n            try {\n                Engine.setProperty(elem, 'documentation', value);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    function setupQuickEdit() {\n        $(QuickEdit).on('open', function (event, editor, view, x, y) {\n            try {\n                CommandManager.get(Commands.EDIT_DELETE_FROM_MODEL).setEnabled(false);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(QuickEdit).on('close', function (event, editor, view, x, y) {\n            try {\n                CommandManager.get(Commands.EDIT_DELETE_FROM_MODEL).setEnabled(true);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n    }\n\n    function setupDiagramManager() {\n        $(DiagramManager).on('currentDiagramChanged', function (event, diagram, editor) {\n            try {\n                if (diagram) {\n                    ToolboxView.setDiagram(diagram);\n                } else {\n                    ToolboxView.setDiagram(null);\n                }\n                QuickEdit.close();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on('workingDiagramRemove', function (event, diagram) {\n            try {\n                SelectionManager.selectViews([]);\n                QuickEdit.close();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on('mouseDown', function (event, mouseEvent) {\n            try {\n                QuickEdit.close();\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on('selectionChanged', function (event, views) {\n            try {\n                if (views && views.length > 0) {\n                    SelectionManager.selectViews(views);\n                    ModelExplorerView.select(null);\n                } else {\n                    SelectionManager.select([], []);\n                    ModelExplorerView.select(null);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on('viewDoubleClicked', function (event, view, x, y) {\n            try {\n                if (view instanceof type.HyperlinkView && view.model instanceof type.Hyperlink) {\n                    if (view.model.reference instanceof type.Model) {\n                        if (view.model.reference instanceof type.Diagram) {\n                            DiagramManager.setCurrentDiagram(view.model.reference);\n                        } else {\n                            ModelExplorerView.select(view.model.reference);\n                        }\n                    } else if (view.model.url && view.model.url.length > 0) {\n                        appshell.app.openURLInDefaultBrowser(view.model.url);\n                    }\n                } else {\n                    QuickEdit.open(DiagramManager.getEditor(), view, x, y);\n                }\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on('viewMoved', function (event, views, dx, dy) {\n            try {\n                Engine.moveViews(DiagramManager.getEditor(), views, dx, dy);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on('parasiticViewMoved', function (event, view, alpha, distance) {\n            try {\n                Engine.moveParasiticView(DiagramManager.getEditor(), view, alpha, distance);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on('nodeResized', function (event, node, left, top, right, bottom) {\n            try {\n                Engine.resizeNode(DiagramManager.getEditor(), node, left, top, right, bottom);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n        $(DiagramManager).on('edgeModified', function (event, edge, points) {\n            try {\n                Engine.modifyEdge(DiagramManager.getEditor(), edge, points);\n            } catch (err) {\n                console.error(err);\n            }\n        });\n\n        // -- Move to uml/main.js (to treat UMLSeqMessageView's reconnect specially)\n        // $(DiagramManager).on('edgeReconnected', function (event, edge, points, newParticipant, isTailSide) {\n        //     Engine.reconnectEdge(DiagramManager.getEditor(), edge, points, newParticipant, isTailSide);\n        // });\n    }\n\n    function setupKeyBindings() {\n        $(\"body\").keydown(function (e) {\n            switch (e.which) {\n            case KeyEvent.DOM_VK_PLUS:\n            case 187: // '=' Key\n                if (e.ctrlKey || e.metaKey) {\n                    CommandManager.execute(Commands.VIEW_ZOOM_IN);\n                }\n                break;\n            case KeyEvent.DOM_VK_MINUS:\n            case 189: // '-' Key\n                if (e.ctrlKey || e.metaKey) {\n                    CommandManager.execute(Commands.VIEW_ZOOM_OUT);\n                }\n                break;\n            }\n        });\n    }\n\n    function setupLicenseManager() {\n        $(LicenseManager).on(\"statusChanged\", function (event, status) {\n            updateTitlebar();\n        });\n    }\n\n    function setupAutoBackup() {\n        setInterval(function () {\n            var fullPath = appshell.app.getApplicationSupportDirectory() + \"/__backup.mdj\",\n                file = FileSystem.getFileForPath(fullPath),\n                project = ProjectManager.getProject();\n            if (file && project) {\n                var data = Repository.writeObject(project);\n                FileUtils.writeText(file, data, true)\n                    .done(function () {\n                        localStorage.setItem(\"__backup_filename\", fullPath);\n                        console.log(\"[AutoBackup] File was backed up: \" + fullPath);\n                    })\n                    .fail(function (err) {\n                        console.error(\"[AutoBackup] Failed to backup file: \", err);\n                    });\n            }\n        }, AUTO_BACKUP_INTERVAL);\n    }\n\n\n    function startAnalytics() {\n        Analytics.sendExecution();\n        $(CommandManager).on(\"beforeExecuteCommand\", function (event, commandId) {\n\n            // for Debugging\n            localStorage.setItem(\"__recent_command__\", commandId);\n\n            // Analytics.sendCommand(commandId);\n        });\n    }\n\n    function start() {\n        // Load Web Fonts\n        // loadFonts();\n\n        // Prevent Default Context Menu\n        document.oncontextmenu = function () { return false; };\n    }\n\n    AppInit.htmlReady(function () {\n        setupRepository();\n        setupProjectManager();\n        setupFactory();\n        setupSelectionManager();\n        setupModelExplorer();\n        setupStyleEditor();\n        setupPropertyEditor();\n        setupDocumentationEditor();\n        setupQuickEdit();\n        setupDiagramManager();\n        setupKeyBindings();\n        setupLicenseManager();\n        setupAutoBackup();\n        start();\n    });\n\n    AppInit.appReady(function () {\n        // startup\n        SelectionManager.deselectAll();\n        StatusBar.setZoomLevel(DiagramManager.getZoomLevel());\n        ModelExplorerView.select(ProjectManager.getProject());\n        ToolboxView.setDiagram(null);\n\n        // To compute correct initial size of diagram-editor.\n        PanelManager._notifyLayoutChange();\n\n        startAnalytics();\n        CheckUpdates.checkForUpdates().done(function (newRelease) {\n            if (PreferenceManager.get(\"checkUpdate.showUpdateDialog\") === true) {\n                CheckUpdates.showUpdateDialog(newRelease);\n            }\n        });\n        CheckUpdates.checkExtensionUpdates();\n    });\n\n});\n\n//# sourceURL=/Application.js"),eval('/*\n * Copyright (c) 2012 Adobe Systems Incorporated. All rights reserved.\n *\n * Permission is hereby granted, free of charge, to any person obtaining a\n * copy of this software and associated documentation files (the "Software"),\n * to deal in the Software without restriction, including without limitation\n * the rights to use, copy, modify, merge, publish, distribute, sublicense,\n * and/or sell copies of the Software, and to permit persons to whom the\n * Software is furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n * DEALINGS IN THE SOFTWARE.\n *\n */\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50 */\n/*global define, app */\n\ndefine(\'utils/DebugUtils\',[\'require\',\'exports\',\'module\',\'core/Global\',\'utils/Initializer\',\'file/FileUtils\',\'filesystem/FileSystem\',\'utils/Analytics\'],function (require, exports, module) {\n    "use strict";\n\n    var _global     = require("core/Global").global,\n        Initializer = require("utils/Initializer"),\n        FileUtils   = require("file/FileUtils"),\n        FileSystem  = require("filesystem/FileSystem"),\n        Analytics   = require("utils/Analytics");\n\n    var _originalConsoleErrorFun = console.error;\n\n    // Check Development Mode\n    _global.app.devMode = false;\n    var testPath = FileUtils.getApplicationDirectoryPath() + "/test",\n        dir = FileSystem.getDirectoryForPath(testPath);\n    dir.exists(function (err, exists) {\n        if (!err && exists) {\n            app.devMode = true;\n        }\n    });\n\n\n    /*\n\n    // Hijack console.error functions to collect error messages\n    console.error = function (err) {\n        _originalConsoleErrorFun.apply(console, arguments);\n        if (!app.devMode) {\n            var op  = localStorage.getItem("__staruml_recent_operation__"),\n                cmd = localStorage.getItem("__staruml_recent_command__");\n            Analytics.sendError(op, cmd, err.stack);\n        }\n    }\n\n    // Collect all errors which were not caught\n    window.onerror = function (message, url, lineNumber) {\n        if (!app.devMode) {\n            var op  = localStorage.getItem("__staruml_recent_operation__"),\n                cmd = localStorage.getItem("__staruml_recent_command__");\n            Analytics.sendError(op, cmd, null, message, url, lineNumber);\n        }\n    }\n\n    */\n\n});\n\n//# sourceURL=/utils/DebugUtils.js'),eval('/*\n * Copyright (c) 2013-2014 Minkyu Lee. All rights reserved.\n *\n * NOTICE:  All information contained herein is, and remains the\n * property of Minkyu Lee. The intellectual and technical concepts\n * contained herein are proprietary to Minkyu Lee and may be covered\n * by Republic of Korea and Foreign Patents, patents in process,\n * and are protected by trade secret or copyright law.\n * Dissemination of this information or reproduction of this material\n * is strictly forbidden unless prior written permission is obtained\n * from Minkyu Lee (niklaus.lee@gmail.com).\n *\n */\n\n// TODO: KendoUI를 써서도 동적인 UI 개발이 가능하도록 할 수 있을듯. (아래 참고)\n/*\n * (플러그인 개발자는 KendoUI를 몰라도 가능해야 한다. HTML/CSS 수준으로 UI 확장 가능하도록 해야 함.)\n *\n * 1. 플러그인은 직접 HTML 코드를 삽입한다. CSS class를 지정한다.\n * 2. htmlReady 시에, jQuery selector를 사용해서 특정 CSS class를 모두 선택하여 kendoUI 위젯화 시킨다.\n *      e.g.) $(".ui-dropdown").kendoDropdown(..)\n * 3. 코드에서 이벤트를 받거나 하고 싶으면 id 값을 이용하도록 한다.\n */\n\n\n/*jslint vars: true, plusplus: true, devel: true, nomen: true, indent: 4, maxerr: 50, regexp: true */\n/*global require, define, $, _, document, window, navigator, app, appshell, localStorage, Mustache */\n\nrequire.config({\n    paths: {\n        "text"           : "thirdparty/text/text",\n        "fileSystemImpl" : "filesystem/impls/appshell/AppshellFileSystem"\n    }\n});\n\ndefine(\'main\',[\'require\',\'exports\',\'module\',\'utils/AppInit\',\'core/Global\',\'utils/Initializer\',\'core/Core\',\'core/Repository\',\'engine/ProjectManager\',\'engine/Engine\',\'engine/SelectionManager\',\'command/CommandManager\',\'command/Commands\',\'file/FileUtils\',\'filesystem/FileSystem\',\'dialogs/Dialogs\',\'dialogs/ElementPickerDialog\',\'ui/Toast\',\'utils/ExtensionLoader\',\'utils/UrlParams\',\'utils/DragAndDrop\',\'utils/Constants\',\'utils/FontManager\',\'text!htmlContent/main-view.html\',\'text!htmlContent/windows-stylesheet.html\',\'Application\',\'utils/DebugUtils\',\'uml/UMLCommands\'],function (require, exports, module) {\n    "use strict";\n\n    var AppInit             = require("utils/AppInit"),\n        _global             = require("core/Global").global,\n        Initializer         = require("utils/Initializer"),\n        Core                = require("core/Core"),\n        Repository          = require("core/Repository"),\n        ProjectManager      = require("engine/ProjectManager"),\n        Engine              = require("engine/Engine"),\n        SelectionManager    = require("engine/SelectionManager"),\n        CommandManager      = require("command/CommandManager"),\n        Commands            = require("command/Commands"),\n        FileUtils           = require("file/FileUtils"),\n        FileSystem          = require("filesystem/FileSystem"),\n        Dialogs             = require("dialogs/Dialogs"),\n        ElementPickerDialog = require("dialogs/ElementPickerDialog"),\n        Toast               = require("ui/Toast"),\n        ExtensionLoader     = require("utils/ExtensionLoader"),\n        UrlParams           = require("utils/UrlParams").UrlParams,\n        DragAndDrop         = require("utils/DragAndDrop"),\n        Constants           = require("utils/Constants"),        \n        FontManager         = require("utils/FontManager"),\n        MainViewHTML        = require("text!htmlContent/main-view.html"),\n        WindowsStylesheet   = require("text!htmlContent/windows-stylesheet.html");\n\n    // Load modules that self-register and just need to get included in the main project\n    require("Application");\n    require("utils/DebugUtils");\n\n\n    // Local variables\n    var params = new UrlParams();\n\n    // read URL params\n    params.parse();\n\n\n    function _initTest() {\n        _global.app.test = {\n            Repository       : Repository,\n            ProjectManager   : ProjectManager,\n            Engine           : Engine,\n            SelectionManager : SelectionManager,\n            CommandManager   : CommandManager,\n            Commands         : Commands,\n            FileUtils        : FileUtils,\n            FileSystem       : FileSystem,\n            Dialogs          : Dialogs,\n            ElementPicker    : ElementPickerDialog,\n            UMLCommands      : require("uml/UMLCommands"),\n            doneLoading      : false\n        };\n\n        AppInit.appReady(function () {\n            _global.app.test.doneLoading = true;\n        });\n    }\n\n    \n    \n    /**\n     * Setup before HTML ready.\n     */\n    function _beforeHTMLReady() {\n        // Use quiet scrollbars if we are on Mac OS X\n        var macMatch = (navigator.userAgent.indexOf("Mac") > -1);\n        if (!macMatch) {\n            $("head").append(WindowsStylesheet);\n        }\n\n        // Inject main.html into <BODY> tag\n        $("body").html(Mustache.render(MainViewHTML, {}));\n\n        // Prevent clicks on any link from navigating to a different page (which could lose unsaved\n        // changes). We can\'t use a simple .on("click", "a") because of http://bugs.jquery.com/ticket/3861:\n        // jQuery hides non-left clicks from such event handlers, yet middle-clicks still cause CEF to\n        // navigate. Also, a capture handler is more reliable than bubble.\n        window.document.body.addEventListener("click", function (e) {\n            // Check parents too, in case link has inline formatting tags\n            var node = e.target, url;\n            while (node) {\n                if (node.tagName === "A") {\n                    url = node.getAttribute("href");\n                    if (url && !url.match(/^#/)) {\n                         appshell.app.openURLInDefaultBrowser(url);\n                    }\n                    e.preventDefault();\n                    break;\n                }\n                node = node.parentElement;\n            }\n        }, true);\n\n\n        // Prevent unhandled drag and drop of files into the browser from replacing\n        // the entire Brackets app. This doesn\'t prevent children from choosing to\n        // handle drops.\n        $(window.document.body)\n            .on("dragover", function (event) {\n                var dropEffect = "none";\n                if (event.originalEvent.dataTransfer.files) {\n                    event.stopPropagation();\n                    event.preventDefault();\n                    // Don\'t allow drag-and-drop of files/folders when a modal dialog is showing.\n                    if ($(".modal.instance").length === 0 && DragAndDrop.isValidDrop(event.originalEvent.dataTransfer.items)) {\n                        dropEffect = "copy";\n                    }\n                    event.originalEvent.dataTransfer.dropEffect = dropEffect;\n                }\n            })\n            .on("drop", function (event) {\n                if (event.originalEvent.dataTransfer.files) {\n                    event.stopPropagation();\n                    event.preventDefault();\n                    appshell.app.getDroppedFiles(function (err, files) {\n                        if (!err) {\n                            if (files.length > 0 && FileUtils.getFileExtension(files[0]).toLowerCase() === Constants.APP_EXT) {\n                                CommandManager.execute(Commands.FILE_OPEN, files[0]);\n                            } else {\n                                Toast.info("Can open only a model file (*." + Constants.APP_EXT + ")");\n                            }\n                        }\n                    });\n                }\n            });\n    }\n\n    function _appReady() {\n        // Signal that App is loaded\n        AppInit._dispatchReady(AppInit.APP_READY);\n\n        // Remove splash screen\n        $(".splash-screen").remove();\n    }\n\n\n    /**\n     * Setup when HTML ready.\n     */\n    function _onReady() {\n\n        // Load all extensions. This promise will complete even if one or more\n        // extensions fail to load.\n        var extensionPathOverride = params.get("extensions");  // used by unit tests\n        var extensionLoaderPromise = ExtensionLoader.init(extensionPathOverride ? extensionPathOverride.split(",") : null);\n\n        // Load the initial project after extensions have loaded\n        extensionLoaderPromise.always(function () {\n            _initTest();\n\n            // Signal application is ready\n            _appReady();\n\n            // Load Initial Project\n            if (!params.get("skipInitialProjectLoad")) {\n\n                // When application is launched by double-clicking an associated file (e.g. ".mdj")\n                appshell.app.getPendingFilesToOpen(function (err, files) {\n                    if (!err) {\n                        if (files.length > 0) {\n                            CommandManager.execute(Commands.FILE_OPEN, files[0]);\n                        } else {\n                            // if backup file exists, it means app was terminated unexpectedly, then load the backup file.\n                            var backupFilePath = localStorage.getItem("__backup_filename");\n                            if (!app.devMode && backupFilePath) {\n                                Dialogs.showConfirmDialog("App was unexpectedly terminated. Load auto backup file?")\n                                    .done(function (buttonId) {\n                                        if (buttonId === Dialogs.DIALOG_BTN_OK) {\n                                            var backupFile = FileSystem.getFileForPath(backupFilePath);\n                                            backupFile.exists(function (err, exists) {\n                                                if (!err && exists) {\n                                                    ProjectManager.loadAsTemplate(backupFilePath);\n                                                } else {\n                                                    Dialogs.showAlertDialog("Backup file not found!");\n                                                }\n                                            });\n                                        }\n                                    });\n                            } else {\n                                // if there is working file, then load working file.\n                                var filePath = localStorage.getItem("__working_filename");\n                                if (filePath) {\n                                    var file = FileSystem.getFileForPath(filePath);\n                                    file.exists(function (err, exists) {\n                                        if (!err && exists) {\n                                            CommandManager.execute(Commands.FILE_OPEN, filePath);\n                                        }\n                                    });\n                                } else {\n                                    CommandManager.execute(Commands.FILE_NEW);\n                                }\n                            }\n                        }\n                    }\n                });\n            }\n\n        });\n\n    }\n    \n    \n    _beforeHTMLReady();\n    \n    // Load fonts\n    FontManager.loadDefaultFonts().done(function () {\n        var ctx = document.getElementById(\'diagram-canvas\').getContext(\'2d\');\n        // prevent font lazyloading\n        app.config.defaultFonts.forEach(function (font) {\n            ctx.font = "12px " + font.value;\n            ctx.fillText("A", 0, 0);\n            ctx.font = "bold 12px " + font.value;\n            ctx.fillText("A", 0, 0);\n            ctx.font = "italic 12px " + font.value;\n            ctx.fillText("A", 0, 0);\n            ctx.font = "italic bold 12px " + font.value;\n            ctx.fillText("A", 0, 0);\n        });\n    });\n        \n    // Dispatch htmlReady event\n    AppInit._dispatchReady(AppInit.HTML_READY);\n    AppInit._dispatchReady(AppInit.DOM_READY);\n\n    $(window.document).ready(_onReady);\n\n});\n\n//# sourceURL=/main.js');

//# sourceMappingURL=main.js.map